(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[   3052744,      54195]
NotebookOptionsPosition[   3037664,      53718]
NotebookOutlinePosition[   3040993,      53804]
CellTagsIndexPosition[   3040793,      53795]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
 "SequentialTrade\n",
 StyleBox["Empirical Market Microstructure",
  FontSlant->"Italic"],
 " \n\t(2006, Oxford University Press)\nCompanion ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notebook\n\nJoel Hasbrouck\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"Copyright", " ", "2007"}], ",", " ", 
    RowBox[{"Joel", " ", 
     RowBox[{"Hasbrouck", ".", " ", "All"}], " ", "rights", " ", 
     RowBox[{"reserved", "."}]}]}], TraditionalForm]],
  FontSize->14,
  FontSlant->"Italic"],
 "\n",
 StyleBox["This notebook covers material related to sequential trade models \
(Chapters 5 and 6)",
  FontSize->14]
}], "Title",
 CellChangeTimes->{{3.3896255910914*^9, 3.3896255932033997`*^9}, {
   3.3897167078988237`*^9, 3.389716768500824*^9}, 3.3897219191158237`*^9, {
   3.3897220399168243`*^9, 3.3897220640418243`*^9}, {3.389735393500024*^9, 
   3.389735431148024*^9}, 3.389735530878024*^9, 3.3897355723880243`*^9, {
   3.389735664464024*^9, 3.3897356860350237`*^9}, {3.3899891508672*^9, 
   3.3899891819161997`*^9}, {3.3899914301996*^9, 3.3899914551656*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Text", "  ", "@", " ", 
  RowBox[{"Style", "[", 
   RowBox[{
    RowBox[{"\"\<Notebook evaluated \>\"", "<>", 
     RowBox[{"DateString", "[", "\"\<DateTime\>\"", "]"}]}], ",", 
    "\"\<Subtitle\>\""}], "]"}]}]], "Input",
 CellLabelAutoDelete->True,
 CellChangeTimes->{{3.3896127180972*^9, 3.3896127205282*^9}, 
   3.389716999411824*^9, {3.3897170351038237`*^9, 3.389717047196824*^9}}],

Cell[BoxData[
 InterpretationBox[Cell[BoxData[
   StyleBox["\<\"Notebook evaluated Monday 4 June 2007 20:18:09\"\>", 
    "Subtitle",
    StripOnInput->False]], "Text", "TR"],
  Text[
   Style["Notebook evaluated Monday 4 June 2007 20:18:09", 
    "Subtitle"]]]], "Output",
 CellMargins->{{18, Inherited}, {Inherited, 0}},
 CellLabelAutoDelete->True,
 CellChangeTimes->{
  3.3896127211071997`*^9, {3.3896127536242*^9, 3.3896127555652*^9}, 
   3.3896128616452*^9, 3.389623031447*^9, 3.3896232934164*^9, 
   3.389716955274824*^9, 3.3897170008718243`*^9, {3.3897170377738237`*^9, 
   3.3897170482808237`*^9}, 3.389735230721224*^9, 3.3899894613552*^9, 
   3.3899897610886*^9, 3.3899911755866003`*^9, 3.3899914898976*^9},
 Background->RGBColor[0.866667, 0.733333, 1]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Preliminaries", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{
  "SetDirectory", "[", 
   "\"\<c:/Active/Empirical Market Microstructure/Mathematica/Spring \
2007\>\"", "]"}], ";"}]], "Input",
 CellAutoOverwrite->False,
 CellLabelAutoDelete->True,
 CellChangeTimes->{{3.3881708634346*^9, 3.3881708734596*^9}, {
  3.3881710728336*^9, 3.3881711170716*^9}, {3.389524702079319*^9, 
  3.389524702676319*^9}}],

Cell[TextData[{
 "The ",
 Cell[BoxData["Trees"]],
 " package contains various routines for working with decision trees. See the \
notebook ",
 Cell[BoxData[
  RowBox[{"Trees", " ", 
   RowBox[{"Demo", ".", "nb"}]}]]],
 " for documentation and examples."
}], "Text"],

Cell[BoxData[
 StyleBox[
  RowBox[{"<<", "Trees.m"}], "Input"]], "Input",
 CellLabel->"In[159]:="],

Cell[CellGroupData[{

Cell["Other initializations", "Subsection"],

Cell[BoxData[
 RowBox[{"<<", "PlotLegends`"}]], "Input",
 CellChangeTimes->{{3.3896137404351997`*^9, 3.3896137460622*^9}},
 CellLabel->"In[160]:="],

Cell[BoxData[
 RowBox[{"<<", "Notation`"}]], "Input",
 CellChangeTimes->{{3.3895618301518*^9, 3.3895618319028*^9}, {
  3.3896128213212*^9, 3.3896128249952*^9}},
 CellLabel->"In[161]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Symbolize", "[", 
   TagBox[
    SubscriptBox["Anything_", "Rule"],
    NotationBoxTag,
    BaseStyle->"NotationTemplateStyle"], "]"}], ";", 
  RowBox[{"Symbolize", "[", 
   TagBox[
    SubscriptBox["Anything_", "Rules"],
    NotationBoxTag,
    BaseStyle->"NotationTemplateStyle"], "]"}], ";"}]], "Input",
 CellLabel->"In[162]:="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Basic one-period problem", "Section"],

Cell[CellGroupData[{

Cell["Event tree", "Subsection"],

Cell["\<\
Functional representation of tree. (If you want to see how this is \
constructed, apply the rules one at a time.)\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LabelTree", "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"V", ",", "VLo", ",", "VHi"}], "}"}], "/.", 
      RowBox[{
       RowBox[{"x", ":", 
        RowBox[{"VLo", "|", "VHi"}]}], "\[RuleDelayed]", 
       RowBox[{"{", 
        RowBox[{"x", ",", "Inf", ",", "U"}], "}"}]}]}], "/.", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"VLo", ",", "Inf", ",", "a_"}], "}"}], "\[RuleDelayed]", 
      RowBox[{"{", 
       RowBox[{"VLo", ",", 
        RowBox[{"{", 
         RowBox[{"Inf", ",", "S"}], "}"}], ",", "a"}], "}"}]}]}], "/.", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"VHi", ",", "Inf", ",", "a_"}], "}"}], "\[RuleDelayed]", 
     RowBox[{"{", 
      RowBox[{"VHi", ",", 
       RowBox[{"{", 
        RowBox[{"Inf", ",", "B"}], "}"}], ",", "a"}], "}"}]}]}], "/.", 
   RowBox[{"U", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"U", ",", "B", ",", "S"}], "}"}]}]}]}]], "Input",
 CellLabel->"In[163]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"V", ",", 
   RowBox[{"{", 
    RowBox[{"VLo", ",", 
     RowBox[{"{", 
      RowBox[{"Inf", ",", "S"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"U", ",", "B", ",", "S"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"VHi", ",", 
     RowBox[{"{", 
      RowBox[{"Inf", ",", "B"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"U", ",", "B", ",", "S"}], "}"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.3896128623742*^9, 3.3896138204648*^9, 
  3.3899914923366003`*^9},
 CellLabel->"Out[163]="]
}, Open  ]],

Cell[TextData[{
 "The ",
 Cell[BoxData["ShowTree"]],
 " command renders a tree in a more intuitive layout. In the following \
diagram, a node can branch to the right and (if there is more than one \
alternative) to the right and down."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ShowTree", "[", "LabelTree", "]"}]], "Input",
 CellLabel->"In[164]:="],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      TagBox[GridBox[{
         {"V"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"VLo"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"Inf"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"S"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"U"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"B"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"S"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"VHi"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"Inf"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"B"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"U"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"B"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"S"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.35]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {{
         Offset[0.2]}}, "RowsIndexed" -> {}}],
   Function[BoxForm`e$, 
    TableForm[
    BoxForm`e$, TableAlignments -> {Left, Top}, TableSpacing -> {0.5, 0.5}]]],
  
  DisplayForm]], "Output",
 CellChangeTimes->{3.3896128624192*^9, 3.3896138205148*^9, 
  3.3899914923966*^9},
 CellLabel->"Out[164]//DisplayForm="]
}, Open  ]],

Cell[TextData[{
 "Construct a new tree, ",
 Cell[BoxData["PrTree"]],
 ", which as the same form as ",
 Cell[BoxData["LabelTree"]],
 ", but in which all nodes are replaced by their conditional probabilities."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"PrTree", "=", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"LabelTree", "/.", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"U", ",", "B", ",", "S"}], "}"}], "\[Rule]", 
            RowBox[{"{", 
             RowBox[{"U", ",", 
              RowBox[{"1", "/", "2"}], ",", 
              RowBox[{"1", "/", "2"}]}], "}"}]}]}], "/.", 
          RowBox[{"S", "\[Rule]", "1"}]}], "/.", 
         RowBox[{"B", "\[Rule]", "1"}]}], "/.", 
        RowBox[{"Inf", "\[Rule]", "\[Mu]"}]}], "/.", 
       RowBox[{"U", "\[Rule]", 
        RowBox[{"1", "-", "\[Mu]"}]}]}], "/.", 
      RowBox[{"VLo", "\[Rule]", "\[Delta]"}]}], "/.", 
     RowBox[{"VHi", "\[Rule]", 
      RowBox[{"1", "-", "\[Delta]"}]}]}], "/.", 
    RowBox[{"V", "\[Rule]", "1"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"BasicTree", "=", 
   RowBox[{"BuildTree", "[", 
    RowBox[{"LabelTree", ",", "PrTree"}], "]"}]}], ";"}], "\n", 
 RowBox[{"ShowTree", "[", "BasicTree", "]"}]}], "Input",
 CellLabel->"In[165]:="],

Cell[BoxData[
 TagBox[
  InterpretationBox[GridBox[{
     {
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"V"},
           {"1"},
           {"1"}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"VLo"},
           {"\[Delta]"},
           {"\[Delta]"}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"Inf"},
           {"\[Mu]"},
           {
            RowBox[{"\[Delta]", " ", "\[Mu]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"S"},
           {"1"},
           {
            RowBox[{"\[Delta]", " ", "\[Mu]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]]},
     {GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"U"},
           {
            RowBox[{"1", "-", "\[Mu]"}]},
           {
            RowBox[{"\[Delta]", " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Mu]"}], ")"}]}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"B"},
           {
            FractionBox["1", "2"]},
           {
            RowBox[{
             FractionBox["1", "2"], " ", "\[Delta]", " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Mu]"}], ")"}]}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]]},
     {GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"S"},
           {
            FractionBox["1", "2"]},
           {
            RowBox[{
             FractionBox["1", "2"], " ", "\[Delta]", " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Mu]"}], ")"}]}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]]},
     {GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"VHi"},
           {
            RowBox[{"1", "-", "\[Delta]"}]},
           {
            RowBox[{"1", "-", "\[Delta]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"Inf"},
           {"\[Mu]"},
           {
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", "\[Mu]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"B"},
           {"1"},
           {
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", "\[Mu]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]]},
     {GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"U"},
           {
            RowBox[{"1", "-", "\[Mu]"}]},
           {
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Mu]"}], ")"}]}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"B"},
           {
            FractionBox["1", "2"]},
           {
            RowBox[{
             FractionBox["1", "2"], " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Mu]"}], ")"}]}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]]},
     {GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"S"},
           {
            FractionBox["1", "2"]},
           {
            RowBox[{
             FractionBox["1", "2"], " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Mu]"}], ")"}]}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Top}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.35]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {{
         Offset[0.2]}}, "RowsIndexed" -> {}}],
   TableForm[{{
      FrameBox[
       TableForm[{$CellContext`V, 1, 1}]], 
      FrameBox[
       TableForm[{$CellContext`VLo, $CellContext`\[Delta], $CellContext`\
\[Delta]}]], 
      FrameBox[
       TableForm[{$CellContext`Inf, $CellContext`\[Mu], $CellContext`\[Delta] \
$CellContext`\[Mu]}]], 
      FrameBox[
       TableForm[{$CellContext`S, 
         1, $CellContext`\[Delta] $CellContext`\[Mu]}]]}, {{" ", " ", " "}, {
      " ", " ", " "}, 
      FrameBox[
       TableForm[{$CellContext`U, 
         1 - $CellContext`\[Mu], $CellContext`\[Delta] (
          1 - $CellContext`\[Mu])}]], 
      FrameBox[
       TableForm[{$CellContext`B, 
         Rational[1, 2], 
         Rational[1, 2] $CellContext`\[Delta] (1 - $CellContext`\[Mu])}]]}, {{
      " ", " ", " "}, {" ", " ", " "}, {" ", " ", " "}, 
      FrameBox[
       TableForm[{$CellContext`S, 
         Rational[1, 2], 
         Rational[1, 2] $CellContext`\[Delta] (1 - $CellContext`\[Mu])}]]}, {{
      " ", " ", " "}, 
      FrameBox[
       TableForm[{$CellContext`VHi, 1 - $CellContext`\[Delta], 
         1 - $CellContext`\[Delta]}]], 
      FrameBox[
       TableForm[{$CellContext`Inf, $CellContext`\[Mu], (
          1 - $CellContext`\[Delta]) $CellContext`\[Mu]}]], 
      FrameBox[
       TableForm[{$CellContext`B, 
         1, (1 - $CellContext`\[Delta]) $CellContext`\[Mu]}]]}, {{
      " ", " ", " "}, {" ", " ", " "}, 
      FrameBox[
       TableForm[{$CellContext`U, 
         1 - $CellContext`\[Mu], (1 - $CellContext`\[Delta]) (
          1 - $CellContext`\[Mu])}]], 
      FrameBox[
       TableForm[{$CellContext`B, 
         Rational[1, 2], 
         Rational[1, 2] (1 - $CellContext`\[Delta]) (
          1 - $CellContext`\[Mu])}]]}, {{" ", " ", " "}, {" ", " ", " "}, {
      " ", " ", " "}, 
      FrameBox[
       TableForm[{$CellContext`S, 
         Rational[1, 2], 
         Rational[1, 2] (1 - $CellContext`\[Delta]) (
          1 - $CellContext`\[Mu])}]]}}, TableAlignments -> {Left, Top}, 
    TableSpacing -> {0.5, 0.5}]],
  DisplayForm]], "Output",
 CellChangeTimes->{3.3896128624792*^9, 3.3896138205828*^9, 
  3.3899914924786*^9},
 CellLabel->"Out[167]//DisplayForm="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Conditional probabilities", "Subsection"],

Cell["\<\
Verify that terminal probabilities (buy or sell nodes) add up to 1. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Pr", "[", 
   RowBox[{"BasicTree", ",", "S"}], "]"}], "+", 
  RowBox[{"Pr", "[", 
   RowBox[{"BasicTree", ",", "B"}], "]"}]}]], "Input",
 CellLabel->"In[168]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", "\[Mu]"}], ")"}]}], "+", 
  RowBox[{"\[Delta]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", "\[Mu]"}], ")"}]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", "\[Mu]"}], "+", 
  RowBox[{"\[Delta]", " ", "\[Mu]"}]}]], "Output",
 CellChangeTimes->{3.3896128625202*^9, 3.3896138206198*^9, 
  3.3899914925316*^9},
 CellLabel->"Out[168]="]
}, Open  ]],

Cell["... and simplify:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "Simplify"}]], "Input",
 CellLabel->"In[169]:="],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.3896128625741997`*^9, 3.3896138206618*^9, 
  3.3899914925846*^9},
 CellLabel->"Out[169]="]
}, Open  ]],

Cell[TextData[{
 "The probability of ",
 Cell[BoxData["B"]],
 " is:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Pr", "[", 
   RowBox[{"BasicTree", ",", "B"}], "]"}], "//", "Simplify"}]], "Input",
 CellLabel->"In[170]:="],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"(", 
   RowBox[{"1", "+", "\[Mu]", "-", 
    RowBox[{"2", " ", "\[Delta]", " ", "\[Mu]"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.3896128626102*^9, 3.3896138206998*^9, 
  3.3899914926396*^9},
 CellLabel->"Out[170]="]
}, Open  ]],

Cell[TextData[{
 "The probability of a ",
 Cell[BoxData["VLo"]],
 " value realization AND a ",
 Cell[BoxData["B"]],
 " is:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Pr", "[", 
   RowBox[{"BasicTree", ",", "VLo", ",", "B"}], "]"}], "//", 
  "Simplify"}]], "Input",
 CellLabel->"In[171]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox["1", "2"]}], " ", "\[Delta]", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", "1"}], "+", "\[Mu]"}], ")"}]}]], "Output",
 CellChangeTimes->{3.3896128626472*^9, 3.3896138207388*^9, 
  3.3899914926946*^9},
 CellLabel->"Out[171]="]
}, Open  ]],

Cell[TextData[{
 "So using Bayes' rule, the revised ",
 Cell[BoxData["\[Delta]"]],
 " conditional on a buy is ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"Pr", "[", 
    RowBox[{"VLo", "|", "B"}], "]"}], "="}]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Delta]B", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Pr", "[", 
     RowBox[{"BasicTree", ",", "VLo", ",", "B"}], "]"}], "/", 
    RowBox[{"Pr", "[", 
     RowBox[{"BasicTree", ",", "B"}], "]"}]}], "//", "Simplify"}]}]], "Input",\

 CellLabel->"In[172]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{"\[Delta]", "-", 
   RowBox[{"\[Delta]", " ", "\[Mu]"}]}], 
  RowBox[{"1", "+", "\[Mu]", "-", 
   RowBox[{"2", " ", "\[Delta]", " ", "\[Mu]"}]}]]], "Output",
 CellChangeTimes->{3.3896128627092*^9, 3.3896138207858*^9, 
  3.3899914927606*^9},
 CellLabel->"Out[172]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", "\[Mu]"], " ", "\[Delta]B"}], "<", "0"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"0", "<", "\[Mu]", "<", "1"}], ",", 
     RowBox[{"0", "<", "\[Delta]", "<", "1"}], ",", 
     RowBox[{"VHi", ">", "VLo"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[173]:="],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.3896128627892*^9, 3.3896138208338003`*^9, 
  3.3899914928336*^9},
 CellLabel->"Out[173]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Ask", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"\[Delta]B", " ", "VLo"}], " ", "+", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "\[Delta]B"}], ")"}], " ", "VHi"}]}], 
   "]"}]}]], "Input",
 CellLabel->"In[174]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"VLo", " ", "\[Delta]", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "\[Mu]"}], ")"}]}], "+", 
   RowBox[{"VHi", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "\[Delta]"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", "\[Mu]"}], ")"}]}]}], 
  RowBox[{
   RowBox[{"-", "1"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"2", " ", "\[Delta]"}]}], ")"}], " ", "\[Mu]"}]}]]], "Output",
 CellChangeTimes->{3.3896128628402*^9, 3.3896138208768*^9, 
  3.3899914928976*^9},
 CellLabel->"Out[174]="]
}, Open  ]],

Cell[TextData[{
 "NOTE: Equation (5.2), p. 45 should read: ",
 Cell[BoxData[
  RowBox[{"A", "=", 
   RowBox[{
    RowBox[{"E", "[", 
     RowBox[{"V", "|", "Buy"}], "]"}], "=", 
    FractionBox[
     RowBox[{
      RowBox[{
       UnderscriptBox["V", "\[HorizontalLine]"], 
       RowBox[{"(", 
        RowBox[{"1", "-", "\[Mu]"}], ")"}], "\[Delta]"}], "+", 
      RowBox[{
       OverscriptBox["V", "\[HorizontalLine]"], 
       RowBox[{"(", 
        RowBox[{"1", "+", "\[Mu]"}], ")"}], 
       RowBox[{"(", 
        RowBox[{"1", "-", "\[Delta]"}], ")"}]}]}], 
     RowBox[{"1", "+", 
      RowBox[{"\[Mu]", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"2", "\[Delta]"}]}], ")"}]}]}]]}]}]]]
}], "Text"],

Cell[TextData[{
 "The revised ",
 Cell[BoxData["\[Delta]"]],
 " conditional on an incoming sell is:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Delta]S", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Pr", "[", 
     RowBox[{"BasicTree", ",", "VLo", ",", "S"}], "]"}], "/", 
    RowBox[{"Pr", "[", 
     RowBox[{"BasicTree", ",", "S"}], "]"}]}], "//", "Simplify"}]}]], "Input",\

 CellLabel->"In[175]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{"\[Delta]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", "\[Mu]"}], ")"}]}], 
  RowBox[{"1", "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"2", " ", "\[Delta]"}]}], ")"}], " ", "\[Mu]"}]}]]], "Output",
 CellChangeTimes->{3.3896128628802*^9, 3.3896138209288*^9, 
  3.3899914929586*^9},
 CellLabel->"Out[175]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Bid", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"\[Delta]S", " ", "VLo"}], " ", "+", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "\[Delta]S"}], ")"}], " ", "VHi"}]}], 
   "]"}]}]], "Input",
 CellLabel->"In[176]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"VHi", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "\[Delta]"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "\[Mu]"}], ")"}]}], "+", 
   RowBox[{"VLo", " ", "\[Delta]", " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", "\[Mu]"}], ")"}]}]}], 
  RowBox[{"1", "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"2", " ", "\[Delta]"}]}], ")"}], " ", "\[Mu]"}]}]]], "Output",
 CellChangeTimes->{3.3896128629292*^9, 3.3896138209698*^9, 
  3.3899914930246*^9},
 CellLabel->"Out[176]="]
}, Open  ]],

Cell[TextData[{
 "NOTE: Equation (5.6), p. 46 should read: ",
 Cell[BoxData[
  RowBox[{"B", "=", 
   RowBox[{
    RowBox[{"E", "[", 
     RowBox[{"V", "|", "Sell"}], "]"}], "=", 
    FractionBox[
     RowBox[{
      RowBox[{
       UnderscriptBox["V", "\[HorizontalLine]"], 
       RowBox[{"(", 
        RowBox[{"1", "+", "\[Mu]"}], ")"}], "\[Delta]"}], "+", 
      RowBox[{
       OverscriptBox["V", "\[HorizontalLine]"], 
       RowBox[{"(", 
        RowBox[{"1", "-", "\[Mu]"}], ")"}], 
       RowBox[{"(", 
        RowBox[{"1", "-", "\[Delta]"}], ")"}]}]}], 
     RowBox[{"1", "-", 
      RowBox[{"\[Mu]", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"2", "\[Delta]"}]}], ")"}]}]}]]}]}]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Ask", "-", "Bid"}], "//", "Simplify"}]], "Input",
 CellLabel->"In[177]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{"VHi", "-", "VLo"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "1"}], "+", "\[Delta]"}], ")"}], " ", "\[Delta]", " ", 
   "\[Mu]"}], 
  RowBox[{
   RowBox[{"-", "1"}], "+", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{"2", " ", "\[Delta]"}]}], ")"}], "2"], " ", 
    SuperscriptBox["\[Mu]", "2"]}]}]]], "Output",
 CellChangeTimes->{3.3896128629832*^9, 3.3896138210288*^9, 
  3.3899914931106*^9},
 CellLabel->"Out[177]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"\[Delta]", "\[Rule]", 
   RowBox[{"1", "/", "2"}]}]}]], "Input",
 CellLabel->"In[178]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"VHi", "-", "VLo"}], ")"}], " ", "\[Mu]"}]], "Output",
 CellChangeTimes->{3.3896128630271997`*^9, 3.3896138210657997`*^9, 
  3.3899914931605997`*^9},
 CellLabel->"Out[178]="]
}, Open  ]],

Cell[TextData[{
 "Consider an attempt at manipulation by buying at the ask (the cash flow is \
",
 Cell[BoxData[
  RowBox[{"-", "Ask"}]]],
 ") and then selling at the (revised) bid.:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Profits", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"-", "Ask"}], "+", " ", 
    RowBox[{"(", 
     RowBox[{"Bid", "/.", 
      RowBox[{"\[Delta]", "\[Rule]", "\[Delta]B"}]}], ")"}]}], 
   "]"}]}]], "Input",
 CellLabel->"In[179]:="],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"2", " ", 
    RowBox[{"(", 
     RowBox[{"VHi", "-", "VLo"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "\[Delta]"}], ")"}], " ", "\[Delta]", " ", 
    "\[Mu]"}], 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", 
       RowBox[{"2", " ", "\[Delta]"}]}], ")"}], " ", "\[Mu]"}]}]]}]], "Output",\

 CellChangeTimes->{3.3896128630832*^9, 3.3896138211177998`*^9, 
  3.3899914932276*^9},
 CellLabel->"Out[179]="]
}, Open  ]],

Cell[TextData[{
 "Test whether ",
 Cell[BoxData["Profits"]],
 " can be positive, given the parameter restrictions:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"Profits", ">", "0"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"0", "<", "\[Mu]", "<", "1"}], ",", 
     RowBox[{"0", "<", "\[Delta]", "<", "1"}], ",", 
     RowBox[{"VHi", ">", "VLo"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[180]:="],

Cell[BoxData["False"], "Output",
 CellChangeTimes->{3.3896128631401997`*^9, 3.3896138211717997`*^9, 
  3.3899914933146*^9},
 CellLabel->"Out[180]="]
}, Open  ]],

Cell["\<\
There is no manipulation profit even with no informed trading in the unwind:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"-", "Ask"}], "+", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Bid", "/.", 
      RowBox[{"\[Mu]", "\[Rule]", "0"}]}], "/.", 
     RowBox[{"\[Delta]", "\[Rule]", "\[Delta]B"}]}], ")"}]}], "]"}]], "Input",\

 CellLabel->"In[181]:="],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.3896128631752*^9, 3.3896138212098*^9, 
  3.3899914933786*^9},
 CellLabel->"Out[181]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Exercise 5.1 (Different \[Mu]s for different brokers)\
\>", "Subsection",
 CellTags->"SeqTrade1"],

Cell[CellGroupData[{

Cell[BoxData["Ask"], "Input",
 CellLabel->"In[182]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"VLo", " ", "\[Delta]", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "\[Mu]"}], ")"}]}], "+", 
   RowBox[{"VHi", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "\[Delta]"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", "\[Mu]"}], ")"}]}]}], 
  RowBox[{
   RowBox[{"-", "1"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"2", " ", "\[Delta]"}]}], ")"}], " ", "\[Mu]"}]}]]], "Output",
 CellChangeTimes->{3.3896128632142*^9, 3.3896138212528*^9, 
  3.3899914934316*^9},
 CellLabel->"Out[182]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Ask", "/.", 
   RowBox[{"\[Delta]", "\[Rule]", 
    RowBox[{"1", "/", "2"}]}]}], "//", "FullSimplify"}]], "Input",
 CellLabel->"In[183]:="],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"(", 
   RowBox[{"VHi", "+", "VLo", "+", 
    RowBox[{"VHi", " ", "\[Mu]"}], "-", 
    RowBox[{"VLo", " ", "\[Mu]"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.3896128634462*^9, 3.3896138213578*^9, 
  3.3899914935326*^9},
 CellLabel->"Out[183]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["Profits", "b"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"Ask", "/.", 
       RowBox[{"\[Mu]", "\[Rule]", 
        SubscriptBox["\[Mu]", "Other"]}]}], ")"}], "-", 
     RowBox[{"(", 
      RowBox[{"Ask", "/.", 
       RowBox[{"\[Mu]", "\[Rule]", 
        SubscriptBox["\[Mu]", "b"]}]}], ")"}]}], "/.", 
    RowBox[{"\[Delta]", "\[Rule]", 
     RowBox[{"1", "/", "2"}]}]}], "//", "FullSimplify"}]}]], "Input",
 CellLabel->"In[184]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox["1", "2"]}], " ", 
  RowBox[{"(", 
   RowBox[{"VHi", "-", "VLo"}], ")"}], " ", 
  RowBox[{"(", 
   RowBox[{
    SubscriptBox["\[Mu]", "b"], "-", 
    SubscriptBox["\[Mu]", "Other"]}], ")"}]}]], "Output",
 CellChangeTimes->{3.3896128635042*^9, 3.3896138214138002`*^9, 
  3.3899914936286*^9},
 CellLabel->"Out[184]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    SubscriptBox["Profits", "b"], ">", "0"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"0", "<", 
      SubscriptBox["\[Mu]", "b"], "<", "1"}], ",", 
     RowBox[{"0", "<", 
      SubscriptBox["\[Mu]", "Other"], "<", "1"}], ",", 
     RowBox[{
      SubscriptBox["\[Mu]", "b"], "<", 
      SubscriptBox["\[Mu]", "Other"]}], ",", 
     RowBox[{"VHi", ">", "VLo"}], ",", 
     RowBox[{"0", "<", "\[Delta]", "<", "1"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[185]:="],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.3896128635472*^9, 3.3896138214508*^9, 
  3.3899914936846*^9},
 CellLabel->"Out[185]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Exercise 5.2 (Informed trading only in low state)", "Subsection",
 CellTags->"SeqTrade2"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"LabelTree", "=", 
   RowBox[{"{", 
    RowBox[{"V", ",", 
     RowBox[{"{", 
      RowBox[{"VLo", ",", 
       RowBox[{"{", 
        RowBox[{"In", ",", "S"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"Un", ",", "B", ",", "S"}], "}"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"VHi", ",", 
       RowBox[{"{", 
        RowBox[{"Un", ",", "B", ",", "S"}], "}"}]}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"LabelTree", "//", "ShowTree"}]}], "Input",
 CellLabel->"In[186]:="],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      TagBox[GridBox[{
         {"V"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"VLo"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"In"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"S"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"Un"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"B"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"S"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"VHi"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"Un"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"B"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"S"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.35]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {{
         Offset[0.2]}}, "RowsIndexed" -> {}}],
   Function[BoxForm`e$, 
    TableForm[
    BoxForm`e$, TableAlignments -> {Left, Top}, TableSpacing -> {0.5, 0.5}]]],
  
  DisplayForm]], "Output",
 CellChangeTimes->{3.3896128635931997`*^9, 3.3896138215067997`*^9, 
  3.3899914937586*^9},
 CellLabel->"Out[187]//DisplayForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"PrTree", "=", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"LabelTree", "/.", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"VLo", ",", 
              RowBox[{"{", 
               RowBox[{"In", ",", "S"}], "}"}], ",", "a_"}], "}"}], 
            "\[RuleDelayed]", 
            RowBox[{"{", 
             RowBox[{"VLo", ",", 
              RowBox[{"{", 
               RowBox[{"In", ",", "1"}], "}"}], ",", "a"}], "}"}]}]}], "/.", 
          RowBox[{
           RowBox[{"B", "|", "S"}], "\[Rule]", 
           RowBox[{"1", "/", "2"}]}]}], "/.", 
         RowBox[{"In", "\[Rule]", "\[Mu]"}]}], "/.", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"VHi", ",", 
           RowBox[{"{", 
            RowBox[{"Un", ",", "a__"}], "}"}]}], "}"}], "\[RuleDelayed]", 
         RowBox[{"{", 
          RowBox[{"VHi", ",", 
           RowBox[{"{", 
            RowBox[{"1", ",", "a"}], "}"}]}], "}"}]}]}], "/.", 
       RowBox[{"Un", "\[Rule]", 
        RowBox[{"1", "-", "\[Mu]"}]}]}], "/.", 
      RowBox[{"VLo", "\[Rule]", "\[Delta]"}]}], "/.", 
     RowBox[{"VHi", "\[Rule]", 
      RowBox[{"1", "-", "\[Delta]"}]}]}], "/.", 
    RowBox[{"V", "\[Rule]", "1"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ExTree", "=", 
   RowBox[{"BuildTree", "[", 
    RowBox[{"LabelTree", ",", "PrTree"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"ShowTree", "[", "ExTree", "]"}]}], "Input",
 CellLabel->"In[188]:="],

Cell[BoxData[
 TagBox[
  InterpretationBox[GridBox[{
     {
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"V"},
           {"1"},
           {"1"}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"VLo"},
           {"\[Delta]"},
           {"\[Delta]"}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"In"},
           {"\[Mu]"},
           {
            RowBox[{"\[Delta]", " ", "\[Mu]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"S"},
           {"1"},
           {
            RowBox[{"\[Delta]", " ", "\[Mu]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]]},
     {GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"Un"},
           {
            RowBox[{"1", "-", "\[Mu]"}]},
           {
            RowBox[{"\[Delta]", " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Mu]"}], ")"}]}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"B"},
           {
            FractionBox["1", "2"]},
           {
            RowBox[{
             FractionBox["1", "2"], " ", "\[Delta]", " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Mu]"}], ")"}]}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]]},
     {GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"S"},
           {
            FractionBox["1", "2"]},
           {
            RowBox[{
             FractionBox["1", "2"], " ", "\[Delta]", " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Mu]"}], ")"}]}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]]},
     {GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"VHi"},
           {
            RowBox[{"1", "-", "\[Delta]"}]},
           {
            RowBox[{"1", "-", "\[Delta]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"Un"},
           {"1"},
           {
            RowBox[{"1", "-", "\[Delta]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"B"},
           {
            FractionBox["1", "2"]},
           {
            FractionBox[
             RowBox[{"1", "-", "\[Delta]"}], "2"]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]]},
     {GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"S"},
           {
            FractionBox["1", "2"]},
           {
            FractionBox[
             RowBox[{"1", "-", "\[Delta]"}], "2"]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Top}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.35]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {{
         Offset[0.2]}}, "RowsIndexed" -> {}}],
   TableForm[{{
      FrameBox[
       TableForm[{$CellContext`V, 1, 1}]], 
      FrameBox[
       TableForm[{$CellContext`VLo, $CellContext`\[Delta], $CellContext`\
\[Delta]}]], 
      FrameBox[
       TableForm[{
        In, $CellContext`\[Mu], $CellContext`\[Delta] $CellContext`\[Mu]}]], 
      FrameBox[
       TableForm[{$CellContext`S, 
         1, $CellContext`\[Delta] $CellContext`\[Mu]}]]}, {{" ", " ", " "}, {
      " ", " ", " "}, 
      FrameBox[
       TableForm[{$CellContext`Un, 
         1 - $CellContext`\[Mu], $CellContext`\[Delta] (
          1 - $CellContext`\[Mu])}]], 
      FrameBox[
       TableForm[{$CellContext`B, 
         Rational[1, 2], 
         Rational[1, 2] $CellContext`\[Delta] (1 - $CellContext`\[Mu])}]]}, {{
      " ", " ", " "}, {" ", " ", " "}, {" ", " ", " "}, 
      FrameBox[
       TableForm[{$CellContext`S, 
         Rational[1, 2], 
         Rational[1, 2] $CellContext`\[Delta] (1 - $CellContext`\[Mu])}]]}, {{
      " ", " ", " "}, 
      FrameBox[
       TableForm[{$CellContext`VHi, 1 - $CellContext`\[Delta], 
         1 - $CellContext`\[Delta]}]], 
      FrameBox[
       TableForm[{$CellContext`Un, 1, 1 - $CellContext`\[Delta]}]], 
      FrameBox[
       TableForm[{$CellContext`B, 
         Rational[1, 2], Rational[1, 2] (1 - $CellContext`\[Delta])}]]}, {{
      " ", " ", " "}, {" ", " ", " "}, {" ", " ", " "}, 
      FrameBox[
       TableForm[{$CellContext`S, 
         Rational[1, 2], Rational[1, 2] (1 - $CellContext`\[Delta])}]]}}, 
    TableAlignments -> {Left, Top}, TableSpacing -> {0.5, 0.5}]],
  DisplayForm]], "Output",
 CellChangeTimes->{3.3896128636512003`*^9, 3.3896138215608*^9, 
  3.3899914938566*^9},
 CellLabel->"Out[190]//DisplayForm="]
}, Open  ]],

Cell["Verify that terminal node total probabilities add up to one:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Pr", "[", 
    RowBox[{"ExTree", ",", "B"}], "]"}], "+", 
   RowBox[{"Pr", "[", 
    RowBox[{"ExTree", ",", "S"}], "]"}]}], "//", "Simplify"}]], "Input",
 CellLabel->"In[191]:="],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.3896128636892*^9, 3.3896138215987997`*^9, 
  3.3899914939096003`*^9},
 CellLabel->"Out[191]="]
}, Open  ]],

Cell[TextData[{
 "Conditional on a buy, the revised ",
 Cell[BoxData["\[Delta]"]],
 " is:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Delta]B", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Pr", "[", 
     RowBox[{"ExTree", ",", "VLo", ",", "B"}], "]"}], "/", 
    RowBox[{"Pr", "[", 
     RowBox[{"ExTree", ",", "B"}], "]"}]}], "//", "Simplify"}]}]], "Input",
 CellLabel->"In[192]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{"\[Delta]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "1"}], "+", "\[Mu]"}], ")"}]}], 
  RowBox[{
   RowBox[{"-", "1"}], "+", 
   RowBox[{"\[Delta]", " ", "\[Mu]"}]}]]], "Output",
 CellChangeTimes->{3.3896128637402*^9, 3.3896138216338*^9, 
  3.3899914939656*^9},
 CellLabel->"Out[192]="]
}, Open  ]],

Cell["Conditional on a sell ...", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Delta]S", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Pr", "[", 
     RowBox[{"ExTree", ",", "VLo", ",", "S"}], "]"}], "/", 
    RowBox[{"Pr", "[", 
     RowBox[{"ExTree", ",", "S"}], "]"}]}], "//", "Simplify"}]}]], "Input",
 CellLabel->"In[193]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{"\[Delta]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", "\[Mu]"}], ")"}]}], 
  RowBox[{"1", "+", 
   RowBox[{"\[Delta]", " ", "\[Mu]"}]}]]], "Output",
 CellChangeTimes->{3.3896128637762003`*^9, 3.3896138216728*^9, 
  3.3899914941846*^9},
 CellLabel->"Out[193]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Ask", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"\[Delta]B", " ", "VLo"}], " ", "+", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "\[Delta]B"}], ")"}], " ", "VHi"}]}], 
   "]"}]}]], "Input",
 CellLabel->"In[194]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"VHi", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "\[Delta]"}], ")"}]}], "+", 
   RowBox[{"VLo", " ", "\[Delta]", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "\[Mu]"}], ")"}]}]}], 
  RowBox[{
   RowBox[{"-", "1"}], "+", 
   RowBox[{"\[Delta]", " ", "\[Mu]"}]}]]], "Output",
 CellChangeTimes->{3.3896128638152*^9, 3.3896138217228003`*^9, 
  3.3899914942516003`*^9},
 CellLabel->"Out[194]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Bid", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"\[Delta]S", " ", "VLo"}], " ", "+", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "\[Delta]S"}], ")"}], " ", "VHi"}]}], 
   "]"}]}]], "Input",
 CellLabel->"In[195]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{"VHi", "-", 
   RowBox[{"VHi", " ", "\[Delta]"}], "+", 
   RowBox[{"VLo", " ", "\[Delta]", " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", "\[Mu]"}], ")"}]}]}], 
  RowBox[{"1", "+", 
   RowBox[{"\[Delta]", " ", "\[Mu]"}]}]]], "Output",
 CellChangeTimes->{3.3896128638652*^9, 3.3896138217588*^9, 
  3.3899914943146*^9},
 CellLabel->"Out[195]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"\[Delta]S", "/.", 
   RowBox[{"\[Delta]", "\[Rule]", "\[Delta]B"}]}], "]"}]], "Input",
 CellLabel->"In[196]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{"\[Delta]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "1"}], "+", "\[Mu]"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", "\[Mu]"}], ")"}]}], 
  RowBox[{
   RowBox[{"-", "1"}], "+", 
   RowBox[{"\[Delta]", " ", 
    SuperscriptBox["\[Mu]", "2"]}]}]]], "Output",
 CellChangeTimes->{3.3896128639062*^9, 3.3896138217948*^9, 
  3.3899914943726*^9},
 CellLabel->"Out[196]="]
}, Open  ]],

Cell["\<\
Manipulation by buying at the ask and selling at the revised bid? Nope.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "Ask"}], " ", "+", " ", "Bid"}], "/.", 
      RowBox[{"\[Delta]", "\[Rule]", "\[Delta]B"}]}], ")"}], ">", "0"}], ",", 
   
   RowBox[{"{", 
    RowBox[{
     RowBox[{"0", "<", "\[Mu]", "<", "1"}], ",", 
     RowBox[{"0", "<", "\[Delta]", "<", "1"}], ",", 
     RowBox[{"VHi", ">", "VLo"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[197]:="],

Cell[BoxData["False"], "Output",
 CellChangeTimes->{3.3896128640671997`*^9, 3.3896138219328003`*^9, 
  3.3899914946736*^9},
 CellLabel->"Out[197]="]
}, Open  ]],

Cell["\<\
Manipulation by selling at the bid and buying at the revised ask? Nope.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Bid", " ", "-", " ", "Ask"}], "/.", 
      RowBox[{"\[Delta]", "\[Rule]", "\[Delta]S"}]}], ")"}], ">", "0"}], ",", 
   
   RowBox[{"{", 
    RowBox[{
     RowBox[{"0", "<", "\[Mu]", "<", "1"}], ",", 
     RowBox[{"0", "<", "\[Delta]", "<", "1"}], ",", 
     RowBox[{"VHi", ">", "VLo"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[198]:="],

Cell[BoxData["False"], "Output",
 CellChangeTimes->{3.3896128642162*^9, 3.3896138220748*^9, 
  3.3899914949716*^9},
 CellLabel->"Out[198]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Exercise 5.3 (Informed traders get a signal)", "Subsection",
 CellTags->"SeqTrade3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LabelTree", "=", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"V", ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"VLo", ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"In", ",", 
         RowBox[{"{", 
          RowBox[{"SLo", ",", "S"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"SHi", ",", "B"}], "}"}]}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"Un", ",", "B", ",", "S"}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"VHi", ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"In", ",", 
         RowBox[{"{", 
          RowBox[{"SLo", ",", "S"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"SHi", ",", "B"}], "}"}]}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"Un", ",", "B", ",", "S"}], "}"}]}], "}"}]}], 
    "\[IndentingNewLine]", "}"}]}], ";", 
  RowBox[{"LabelTree", "//", "ShowTree"}]}]], "Input",
 CellLabel->"In[199]:="],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      TagBox[GridBox[{
         {"V"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"VLo"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"In"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"SLo"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"S"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"SHi"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"B"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"Un"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"B"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"S"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"VHi"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"In"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"SLo"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"S"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"SHi"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"B"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"Un"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"B"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"S"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\<\" \"\>"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
          "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}, "RowsIndexed" -> {}}],
       Column]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.35]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {{
         Offset[0.2]}}, "RowsIndexed" -> {}}],
   Function[BoxForm`e$, 
    TableForm[
    BoxForm`e$, TableAlignments -> {Left, Top}, TableSpacing -> {0.5, 0.5}]]],
  
  DisplayForm]], "Output",
 CellChangeTimes->{3.3896128642782*^9, 3.3896138221238003`*^9, 
  3.3899914950886*^9},
 CellLabel->"Out[199]//DisplayForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"PrTree", "=", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"LabelTree", "/.", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"VLo", ",", 
              RowBox[{"{", 
               RowBox[{"In", ",", 
                RowBox[{"{", 
                 RowBox[{"SLo", ",", "S"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"SHi", ",", "B"}], "}"}]}], "}"}], ",", "a_"}], 
             "}"}], ":>", 
            RowBox[{"{", 
             RowBox[{"VLo", ",", 
              RowBox[{"{", 
               RowBox[{"In", ",", 
                RowBox[{"{", 
                 RowBox[{"\[Gamma]", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"1", "-", "\[Gamma]"}], ",", "1"}], "}"}]}], "}"}], 
              ",", "a"}], "}"}]}]}], "/.", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"VHi", ",", 
             RowBox[{"{", 
              RowBox[{"In", ",", 
               RowBox[{"{", 
                RowBox[{"SLo", ",", "S"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"SHi", ",", "B"}], "}"}]}], "}"}], ",", "a_"}], "}"}],
            ":>", 
           RowBox[{"{", 
            RowBox[{"VHi", ",", 
             RowBox[{"{", 
              RowBox[{"In", ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"1", "-", "\[Gamma]"}], ",", "1"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"\[Gamma]", ",", "1"}], "}"}]}], "}"}], ",", "a"}], 
            "}"}]}]}], "/.", 
         RowBox[{
          RowBox[{"B", "|", "S"}], "\[Rule]", 
          RowBox[{"1", "/", "2"}]}]}], "/.", 
        RowBox[{"In", "\[Rule]", "\[Mu]"}]}], "/.", 
       RowBox[{"Un", "\[Rule]", 
        RowBox[{"1", "-", "\[Mu]"}]}]}], "/.", 
      RowBox[{"VLo", "\[Rule]", "\[Delta]"}]}], "/.", 
     RowBox[{"VHi", "\[Rule]", 
      RowBox[{"1", "-", "\[Delta]"}]}]}], "/.", 
    RowBox[{"V", "\[Rule]", "1"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ExTree", "=", 
   RowBox[{"BuildTree", "[", 
    RowBox[{"LabelTree", ",", "PrTree"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"ShowTree", "[", "ExTree", "]"}]}], "Input",
 CellLabel->"In[200]:="],

Cell[BoxData[
 TagBox[
  InterpretationBox[GridBox[{
     {
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"V"},
           {"1"},
           {"1"}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"VLo"},
           {"\[Delta]"},
           {"\[Delta]"}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"In"},
           {"\[Mu]"},
           {
            RowBox[{"\[Delta]", " ", "\[Mu]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"SLo"},
           {"\[Gamma]"},
           {
            RowBox[{"\[Gamma]", " ", "\[Delta]", " ", "\[Mu]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"S"},
           {"1"},
           {
            RowBox[{"\[Gamma]", " ", "\[Delta]", " ", "\[Mu]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]]},
     {GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"SHi"},
           {
            RowBox[{"1", "-", "\[Gamma]"}]},
           {
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Gamma]"}], ")"}], " ", "\[Delta]", " ", 
             "\[Mu]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"B"},
           {"1"},
           {
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Gamma]"}], ")"}], " ", "\[Delta]", " ", 
             "\[Mu]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]]},
     {GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"Un"},
           {
            RowBox[{"1", "-", "\[Mu]"}]},
           {
            RowBox[{"\[Delta]", " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Mu]"}], ")"}]}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"B"},
           {
            FractionBox["1", "2"]},
           {
            RowBox[{
             FractionBox["1", "2"], " ", "\[Delta]", " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Mu]"}], ")"}]}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}]},
     {GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"S"},
           {
            FractionBox["1", "2"]},
           {
            RowBox[{
             FractionBox["1", "2"], " ", "\[Delta]", " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Mu]"}], ")"}]}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}]},
     {GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"VHi"},
           {
            RowBox[{"1", "-", "\[Delta]"}]},
           {
            RowBox[{"1", "-", "\[Delta]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"In"},
           {"\[Mu]"},
           {
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", "\[Mu]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"SLo"},
           {
            RowBox[{"1", "-", "\[Gamma]"}]},
           {
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Gamma]"}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", "\[Mu]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"S"},
           {"1"},
           {
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Gamma]"}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", "\[Mu]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]]},
     {GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"SHi"},
           {"\[Gamma]"},
           {
            RowBox[{"\[Gamma]", " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", "\[Mu]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"B"},
           {"1"},
           {
            RowBox[{"\[Gamma]", " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", "\[Mu]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]]},
     {GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"Un"},
           {
            RowBox[{"1", "-", "\[Mu]"}]},
           {
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Mu]"}], ")"}]}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"B"},
           {
            FractionBox["1", "2"]},
           {
            RowBox[{
             FractionBox["1", "2"], " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Mu]"}], ")"}]}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}]},
     {GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], 
      FrameBox[
       TagBox[
        TagBox[GridBox[{
           {"S"},
           {
            FractionBox["1", "2"]},
           {
            RowBox[{
             FractionBox["1", "2"], " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{"1", "-", "\[Mu]"}], ")"}]}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
            "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column],
        Function[BoxForm`e$, 
         TableForm[BoxForm`e$]]]], GridBox[{
        {"\<\" \"\>"},
        {"\<\" \"\>"},
        {"\<\" \"\>"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.1]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Top}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.35]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {{
         Offset[0.2]}}, "RowsIndexed" -> {}}],
   TableForm[{{
      FrameBox[
       TableForm[{$CellContext`V, 1, 1}]], 
      FrameBox[
       TableForm[{$CellContext`VLo, $CellContext`\[Delta], $CellContext`\
\[Delta]}]], 
      FrameBox[
       TableForm[{
        In, $CellContext`\[Mu], $CellContext`\[Delta] $CellContext`\[Mu]}]], 
      FrameBox[
       TableForm[{$CellContext`SLo, $CellContext`\[Gamma], $CellContext`\
\[Gamma] $CellContext`\[Delta] $CellContext`\[Mu]}]], 
      FrameBox[
       TableForm[{$CellContext`S, 
         1, $CellContext`\[Gamma] $CellContext`\[Delta] \
$CellContext`\[Mu]}]]}, {{" ", " ", " "}, {" ", " ", " "}, {" ", " ", " "}, 
      FrameBox[
       TableForm[{$CellContext`SHi, 
         1 - $CellContext`\[Gamma], (
          1 - $CellContext`\[Gamma]) $CellContext`\[Delta] \
$CellContext`\[Mu]}]], 
      FrameBox[
       TableForm[{$CellContext`B, 
         1, (1 - $CellContext`\[Gamma]) $CellContext`\[Delta] $CellContext`\
\[Mu]}]]}, {{" ", " ", " "}, {" ", " ", " "}, 
      FrameBox[
       TableForm[{$CellContext`Un, 
         1 - $CellContext`\[Mu], $CellContext`\[Delta] (
          1 - $CellContext`\[Mu])}]], 
      FrameBox[
       TableForm[{$CellContext`B, 
         Rational[1, 2], 
         Rational[1, 2] $CellContext`\[Delta] (1 - $CellContext`\[Mu])}]], {
      " ", " ", " "}}, {{" ", " ", " "}, {" ", " ", " "}, {" ", " ", " "}, 
      FrameBox[
       TableForm[{$CellContext`S, 
         Rational[1, 2], 
         Rational[1, 2] $CellContext`\[Delta] (1 - $CellContext`\[Mu])}]], {
      " ", " ", " "}}, {{" ", " ", " "}, 
      FrameBox[
       TableForm[{$CellContext`VHi, 1 - $CellContext`\[Delta], 
         1 - $CellContext`\[Delta]}]], 
      FrameBox[
       TableForm[{
        In, $CellContext`\[Mu], (
          1 - $CellContext`\[Delta]) $CellContext`\[Mu]}]], 
      FrameBox[
       TableForm[{$CellContext`SLo, 
         1 - $CellContext`\[Gamma], (1 - $CellContext`\[Gamma]) (
          1 - $CellContext`\[Delta]) $CellContext`\[Mu]}]], 
      FrameBox[
       TableForm[{$CellContext`S, 
         1, (1 - $CellContext`\[Gamma]) (
          1 - $CellContext`\[Delta]) $CellContext`\[Mu]}]]}, {{
      " ", " ", " "}, {" ", " ", " "}, {" ", " ", " "}, 
      FrameBox[
       TableForm[{$CellContext`SHi, $CellContext`\[Gamma], $CellContext`\
\[Gamma] (1 - $CellContext`\[Delta]) $CellContext`\[Mu]}]], 
      FrameBox[
       TableForm[{$CellContext`B, 
         1, $CellContext`\[Gamma] (
          1 - $CellContext`\[Delta]) $CellContext`\[Mu]}]]}, {{
      " ", " ", " "}, {" ", " ", " "}, 
      FrameBox[
       TableForm[{$CellContext`Un, 
         1 - $CellContext`\[Mu], (1 - $CellContext`\[Delta]) (
          1 - $CellContext`\[Mu])}]], 
      FrameBox[
       TableForm[{$CellContext`B, 
         Rational[1, 2], 
         Rational[1, 2] (1 - $CellContext`\[Delta]) (
          1 - $CellContext`\[Mu])}]], {" ", " ", " "}}, {{" ", " ", " "}, {
      " ", " ", " "}, {" ", " ", " "}, 
      FrameBox[
       TableForm[{$CellContext`S, 
         Rational[1, 2], 
         Rational[1, 2] (1 - $CellContext`\[Delta]) (
          1 - $CellContext`\[Mu])}]], {" ", " ", " "}}}, 
    TableAlignments -> {Left, Top}, TableSpacing -> {0.5, 0.5}]],
  DisplayForm]], "Output",
 CellChangeTimes->{3.3896128643691998`*^9, 3.3896138221878*^9, 
  3.3899914952766*^9},
 CellLabel->"Out[202]//DisplayForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Pr", "[", 
    RowBox[{"ExTree", ",", "B"}], "]"}], "+", 
   RowBox[{"Pr", "[", 
    RowBox[{"ExTree", ",", "S"}], "]"}]}], "//", "Simplify"}]], "Input",
 CellLabel->"In[203]:="],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.3896128644161997`*^9, 3.3896138222258*^9, 
  3.3899914953686*^9},
 CellLabel->"Out[203]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Delta]B", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Pr", "[", 
     RowBox[{"ExTree", ",", "VLo", ",", "B"}], "]"}], "/", 
    RowBox[{"Pr", "[", 
     RowBox[{"ExTree", ",", "B"}], "]"}]}], "//", "Simplify"}]}]], "Input",
 CellLabel->"In[204]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{"\[Delta]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "1"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{"2", " ", "\[Gamma]"}]}], ")"}], " ", "\[Mu]"}]}], ")"}]}], 
  RowBox[{
   RowBox[{"-", "1"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"2", " ", "\[Gamma]"}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"2", " ", "\[Delta]"}]}], ")"}], " ", "\[Mu]"}]}]]], "Output",
 CellChangeTimes->{3.3896128644782*^9, 3.3896138222738*^9, 
  3.3899914954996*^9},
 CellLabel->"Out[204]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Values", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Gamma]", "\[Rule]", ".1"}], ",", 
     RowBox[{"\[Mu]", "\[Rule]", ".2"}], ",", 
     RowBox[{"\[Delta]", "\[Rule]", ".3"}]}], "}"}]}], ";"}]], "Input",
 CellLabel->"In[205]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Delta]B", "/.", "Values"}]], "Input",
 CellLabel->"In[206]:="],

Cell[BoxData["0.3717948717948718`"], "Output",
 CellChangeTimes->{3.3896128645242*^9, 3.3896138223118*^9, 
  3.3899914955986*^9},
 CellLabel->"Out[206]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Ask", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"\[Delta]B", " ", "VLo"}], " ", "+", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "\[Delta]B"}], ")"}], " ", "VHi"}]}], 
   "]"}]}]], "Input",
 CellLabel->"In[207]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"VLo", " ", "\[Delta]", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         RowBox[{"2", " ", "\[Gamma]"}]}], ")"}], " ", "\[Mu]"}]}], ")"}]}], 
   "+", 
   RowBox[{"VHi", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "\[Delta]"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         RowBox[{"2", " ", "\[Gamma]"}]}], ")"}], " ", "\[Mu]"}]}], ")"}]}]}],
   
  RowBox[{
   RowBox[{"-", "1"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"2", " ", "\[Gamma]"}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"2", " ", "\[Delta]"}]}], ")"}], " ", "\[Mu]"}]}]]], "Output",
 CellChangeTimes->{3.3896128645752*^9, 3.3896138223618*^9, 
  3.3899914956856003`*^9},
 CellLabel->"Out[207]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Delta]S", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Pr", "[", 
     RowBox[{"ExTree", ",", "VLo", ",", "S"}], "]"}], "/", 
    RowBox[{"Pr", "[", 
     RowBox[{"ExTree", ",", "S"}], "]"}]}], "//", "Simplify"}]}]], "Input",
 CellLabel->"In[208]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{"\[Delta]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{"2", " ", "\[Gamma]"}]}], ")"}], " ", "\[Mu]"}]}], ")"}]}], 
  RowBox[{"1", "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"2", " ", "\[Gamma]"}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"2", " ", "\[Delta]"}]}], ")"}], " ", "\[Mu]"}]}]]], "Output",
 CellChangeTimes->{3.3896128646182003`*^9, 3.3896138224018*^9, 
  3.3899914957516003`*^9},
 CellLabel->"Out[208]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Bid1", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"\[Delta]S", " ", "VLo"}], " ", "+", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "\[Delta]S"}], ")"}], " ", "VHi"}]}], 
   "]"}]}]], "Input",
 CellLabel->"In[209]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"VHi", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "\[Delta]"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         RowBox[{"2", " ", "\[Gamma]"}]}], ")"}], " ", "\[Mu]"}]}], ")"}]}], 
   "+", 
   RowBox[{"VLo", " ", "\[Delta]", " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         RowBox[{"2", " ", "\[Gamma]"}]}], ")"}], " ", "\[Mu]"}]}], ")"}]}]}],
   
  RowBox[{"1", "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"2", " ", "\[Gamma]"}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"2", " ", "\[Delta]"}]}], ")"}], " ", "\[Mu]"}]}]]], "Output",
 CellChangeTimes->{3.3896128646712*^9, 3.3896138224538*^9, 
  3.3899914958316*^9},
 CellLabel->"Out[209]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"\[Delta]S", "/.", 
   RowBox[{"\[Delta]", "\[Rule]", "\[Delta]B"}]}], "]"}]], "Input",
 CellLabel->"In[210]:="],

Cell[BoxData["\[Delta]"], "Output",
 CellChangeTimes->{3.3896128647122*^9, 3.3896138224888*^9, 
  3.3899914958856*^9},
 CellLabel->"Out[210]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Sequences of buys and sells", "Subsection"],

Cell[TextData[{
 Cell[BoxData["\[Delta]Cond"]],
 " returns ",
 Cell[BoxData[
  SubscriptBox["\[Delta]", "k"]]],
 " as a function of ",
 Cell[BoxData[
  SubscriptBox["\[Delta]", 
   RowBox[{"k", "-", "1"}]]]],
 " and the ",
 Cell[BoxData["k"]],
 "th trade (",
 Cell[BoxData["B"]],
 " or ",
 Cell[BoxData["S"]],
 "):"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Delta]Cond", "Rule"], "=", 
   RowBox[{
    RowBox[{"\[Delta]Cond", "[", 
     RowBox[{"\[Delta]_", ",", "x_"}], "]"}], "\[RuleDelayed]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"x", "===", "B"}], ",", 
      FractionBox[
       RowBox[{"\[Delta]", "-", 
        RowBox[{"\[Delta]", " ", "\[Mu]"}]}], 
       RowBox[{"1", "+", "\[Mu]", "-", 
        RowBox[{"2", " ", "\[Delta]", " ", "\[Mu]"}]}]], ",", 
      FractionBox[
       RowBox[{"\[Delta]", " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", "\[Mu]"}], ")"}]}], 
       RowBox[{"1", "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "1"}], "+", 
           RowBox[{"2", " ", "\[Delta]"}]}], ")"}], " ", "\[Mu]"}]}]]}], 
     "]"}]}]}], ";"}]], "Input",
 CellLabel->"In[211]:="],

Cell["\<\
This can be used to obtain the revised \[Delta] for the first trade ...\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Delta]Cond", "[", 
   RowBox[{
    SubscriptBox["\[Delta]", "0"], ",", "S"}], "]"}], "/.", 
  SubscriptBox["\[Delta]Cond", "Rule"]}]], "Input",
 CellLabel->"In[212]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "+", "\[Mu]"}], ")"}], " ", 
   SubscriptBox["\[Delta]", "0"]}], 
  RowBox[{"1", "+", 
   RowBox[{"\[Mu]", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      RowBox[{"2", " ", 
       SubscriptBox["\[Delta]", "0"]}]}], ")"}]}]}]]], "Output",
 CellChangeTimes->{3.3896128647762003`*^9, 3.3896138225388002`*^9, 
  3.3899914959625998`*^9},
 CellLabel->"Out[212]="]
}, Open  ]],

Cell["... or a sequence of trades:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FoldList", "[", 
  RowBox[{"\[Delta]Cond", ",", 
   SubscriptBox["\[Delta]", "0"], ",", 
   RowBox[{"{", 
    RowBox[{"B", ",", "S"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[213]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SubscriptBox["\[Delta]", "0"], ",", 
   RowBox[{"\[Delta]Cond", "[", 
    RowBox[{
     SubscriptBox["\[Delta]", "0"], ",", "B"}], "]"}], ",", 
   RowBox[{"\[Delta]Cond", "[", 
    RowBox[{
     RowBox[{"\[Delta]Cond", "[", 
      RowBox[{
       SubscriptBox["\[Delta]", "0"], ",", "B"}], "]"}], ",", "S"}], "]"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.3896128648072*^9, 3.3896138225758*^9, 
  3.3899914960216*^9},
 CellLabel->"Out[213]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//.", 
   SubscriptBox["\[Delta]Cond", "Rule"]}], "//", "Simplify"}]], "Input",
 CellLabel->"In[214]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SubscriptBox["\[Delta]", "0"], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "\[Mu]"}], ")"}], " ", 
     SubscriptBox["\[Delta]", "0"]}], 
    RowBox[{
     RowBox[{"-", "1"}], "-", "\[Mu]", "+", 
     RowBox[{"2", " ", "\[Mu]", " ", 
      SubscriptBox["\[Delta]", "0"]}]}]], ",", 
   SubscriptBox["\[Delta]", "0"]}], "}"}]], "Output",
 CellChangeTimes->{3.3896128648431997`*^9, 3.3896138226118*^9, 
  3.3899914960926*^9},
 CellLabel->"Out[214]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", "\[Mu]"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[Delta]Cond", "[", 
        RowBox[{"\[Delta]", ",", "S"}], "]"}], "/.", 
       SubscriptBox["\[Delta]Cond", "Rule"]}], ")"}]}], ">", "0"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"0", "<", "\[Delta]", "<", "1"}], ",", 
     RowBox[{"0", "<", "\[Mu]", "<", "1"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[215]:="],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.3896128648901997`*^9, 3.3896138226588*^9, 
  3.3899914961745996`*^9},
 CellLabel->"Out[215]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", "\[Mu]"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[Delta]Cond", "[", 
        RowBox[{"\[Delta]", ",", "B"}], "]"}], "/.", 
       SubscriptBox["\[Delta]Cond", "Rule"]}], ")"}]}], "<", "0"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"0", "<", "\[Delta]", "<", "1"}], ",", 
     RowBox[{"0", "<", "\[Mu]", "<", "1"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[216]:="],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.3896128649312*^9, 3.3896138226977997`*^9, 
  3.3899914963126*^9},
 CellLabel->"Out[216]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FoldList", "[", 
  RowBox[{"\[Delta]Cond", ",", 
   SubscriptBox["\[Delta]", "0"], ",", 
   RowBox[{"{", 
    RowBox[{"B", ",", "B", ",", "S", ",", "S"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[217]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SubscriptBox["\[Delta]", "0"], ",", 
   RowBox[{"\[Delta]Cond", "[", 
    RowBox[{
     SubscriptBox["\[Delta]", "0"], ",", "B"}], "]"}], ",", 
   RowBox[{"\[Delta]Cond", "[", 
    RowBox[{
     RowBox[{"\[Delta]Cond", "[", 
      RowBox[{
       SubscriptBox["\[Delta]", "0"], ",", "B"}], "]"}], ",", "B"}], "]"}], 
   ",", 
   RowBox[{"\[Delta]Cond", "[", 
    RowBox[{
     RowBox[{"\[Delta]Cond", "[", 
      RowBox[{
       RowBox[{"\[Delta]Cond", "[", 
        RowBox[{
         SubscriptBox["\[Delta]", "0"], ",", "B"}], "]"}], ",", "B"}], "]"}], 
     ",", "S"}], "]"}], ",", 
   RowBox[{"\[Delta]Cond", "[", 
    RowBox[{
     RowBox[{"\[Delta]Cond", "[", 
      RowBox[{
       RowBox[{"\[Delta]Cond", "[", 
        RowBox[{
         RowBox[{"\[Delta]Cond", "[", 
          RowBox[{
           SubscriptBox["\[Delta]", "0"], ",", "B"}], "]"}], ",", "B"}], 
        "]"}], ",", "S"}], "]"}], ",", "S"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.3896128649662*^9, 3.3896138227307997`*^9, 
  3.3899914963836*^9},
 CellLabel->"Out[217]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//.", 
   SubscriptBox["\[Delta]Cond", "Rule"]}], "//", "Simplify"}]], "Input",
 CellLabel->"In[218]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SubscriptBox["\[Delta]", "0"], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "\[Mu]"}], ")"}], " ", 
     SubscriptBox["\[Delta]", "0"]}], 
    RowBox[{
     RowBox[{"-", "1"}], "-", "\[Mu]", "+", 
     RowBox[{"2", " ", "\[Mu]", " ", 
      SubscriptBox["\[Delta]", "0"]}]}]], ",", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "\[Mu]"}], ")"}], "2"], " ", 
     SubscriptBox["\[Delta]", "0"]}], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "+", "\[Mu]"}], ")"}], "2"], "-", 
     RowBox[{"4", " ", "\[Mu]", " ", 
      SubscriptBox["\[Delta]", "0"]}]}]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "\[Mu]"}], ")"}], " ", 
     SubscriptBox["\[Delta]", "0"]}], 
    RowBox[{
     RowBox[{"-", "1"}], "-", "\[Mu]", "+", 
     RowBox[{"2", " ", "\[Mu]", " ", 
      SubscriptBox["\[Delta]", "0"]}]}]], ",", 
   SubscriptBox["\[Delta]", "0"]}], "}"}]], "Output",
 CellChangeTimes->{3.3896128650172*^9, 3.3896138227738*^9, 
  3.3899914964756002`*^9},
 CellLabel->"Out[218]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Exercise 5.4 (Offsetting trades)", "Subsection",
 CellTags->"SeqTrade4"],

Cell["\<\
A sell followed by a buy gives ... (see the analysis of the original problem, \
above).\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FoldList", "[", 
    RowBox[{"\[Delta]Cond", ",", 
     SubscriptBox["\[Delta]", "0"], ",", 
     RowBox[{"{", 
      RowBox[{"S", ",", "B"}], "}"}]}], "]"}], "//.", 
   SubscriptBox["\[Delta]Cond", "Rule"]}], "//", "Simplify"}]], "Input",
 CellLabel->"In[219]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SubscriptBox["\[Delta]", "0"], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "+", "\[Mu]"}], ")"}], " ", 
     SubscriptBox["\[Delta]", "0"]}], 
    RowBox[{"1", "-", "\[Mu]", "+", 
     RowBox[{"2", " ", "\[Mu]", " ", 
      SubscriptBox["\[Delta]", "0"]}]}]], ",", 
   SubscriptBox["\[Delta]", "0"]}], "}"}]], "Output",
 CellChangeTimes->{3.3896128650572*^9, 3.3896138228087997`*^9, 
  3.3899914965596*^9},
 CellLabel->"Out[219]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\"Price Impact\" (section 5.6)", "Subsection"],

Cell["Consider two sells followed by a buy:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FoldList", "[", 
    RowBox[{"\[Delta]Cond", ",", 
     SubscriptBox["\[Delta]", "0"], ",", 
     RowBox[{"{", 
      RowBox[{"S", ",", "S", ",", "B"}], "}"}]}], "]"}], "//.", 
   SubscriptBox["\[Delta]Cond", "Rule"]}], "//", "Simplify"}]], "Input",
 CellLabel->"In[220]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   SubscriptBox["\[Delta]", "0"], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "+", "\[Mu]"}], ")"}], " ", 
     SubscriptBox["\[Delta]", "0"]}], 
    RowBox[{"1", "-", "\[Mu]", "+", 
     RowBox[{"2", " ", "\[Mu]", " ", 
      SubscriptBox["\[Delta]", "0"]}]}]], ",", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "+", "\[Mu]"}], ")"}], "2"], " ", 
     SubscriptBox["\[Delta]", "0"]}], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "\[Mu]"}], ")"}], "2"], "+", 
     RowBox[{"4", " ", "\[Mu]", " ", 
      SubscriptBox["\[Delta]", "0"]}]}]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "+", "\[Mu]"}], ")"}], " ", 
     SubscriptBox["\[Delta]", "0"]}], 
    RowBox[{"1", "-", "\[Mu]", "+", 
     RowBox[{"2", " ", "\[Mu]", " ", 
      SubscriptBox["\[Delta]", "0"]}]}]]}], "}"}]], "Output",
 CellChangeTimes->{3.3896128651092*^9, 3.3896138228478003`*^9, 
  3.3899914966496*^9},
 CellLabel->"Out[220]="]
}, Open  ]],

Cell[TextData[{
 "Using ",
 Cell[BoxData[
  RowBox[{"Fold", Cell[""]}]]],
 "instead of ",
 Cell[BoxData["FoldList"]],
 " returns the last element:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Delta]MM", "=", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"Fold", "[", 
      RowBox[{"\[Delta]Cond", ",", 
       SubscriptBox["\[Delta]", "0"], ",", 
       RowBox[{"{", 
        RowBox[{"S", ",", "S", ",", "B"}], "}"}]}], "]"}], "//.", 
     SubscriptBox["\[Delta]Cond", "Rule"]}], "//", "Simplify"}], 
   ")"}]}]], "Input",
 CellLabel->"In[221]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "+", "\[Mu]"}], ")"}], " ", 
   SubscriptBox["\[Delta]", "0"]}], 
  RowBox[{"1", "-", "\[Mu]", "+", 
   RowBox[{"2", " ", "\[Mu]", " ", 
    SubscriptBox["\[Delta]", "0"]}]}]]], "Output",
 CellChangeTimes->{3.3896128651442003`*^9, 3.3896138228828*^9, 
  3.3899914967176*^9},
 CellLabel->"Out[221]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"ParameterRestrictions", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"0", "<", "\[Mu]", "<", "1"}], ",", 
     RowBox[{"0", "<", 
      SubscriptBox["\[Delta]", "0"], "<", "1"}], ",", 
     RowBox[{"VHi", ">", "VLo"}]}], "}"}]}], ";"}]], "Input",
 CellLabel->"In[222]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"\[Delta]MM", ">", 
    SubscriptBox["\[Delta]", "0"]}], ",", "ParameterRestrictions"}], 
  "]"}]], "Input",
 CellLabel->"In[223]:="],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.3896128651912003`*^9, 3.3896138229288*^9, 
  3.3899914968016*^9},
 CellLabel->"Out[223]="]
}, Open  ]],

Cell["\<\
So the MM has a more pessimistic view of value than the order-informed \
trader, even after the order-informed trader has bought.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Ask", "=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"\[Delta]MM", " ", "VLo"}], " ", "+", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "\[Delta]MM"}], ")"}], " ", "VHi"}]}], 
   "]"}]}]], "Input",
 CellLabel->"In[224]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{"VHi", "-", 
   RowBox[{"VHi", " ", "\[Mu]"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"VHi", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "\[Mu]"}], ")"}]}], "+", 
      RowBox[{"VLo", " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", "\[Mu]"}], ")"}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", "0"]}]}], 
  RowBox[{"1", "-", "\[Mu]", "+", 
   RowBox[{"2", " ", "\[Mu]", " ", 
    SubscriptBox["\[Delta]", "0"]}]}]]], "Output",
 CellChangeTimes->{3.3896128652352*^9, 3.3896138229688*^9, 
  3.3899914968906*^9},
 CellLabel->"Out[224]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EV", "=", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Delta]", "0"], " ", "VLo"}], " ", "+", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", 
      SubscriptBox["\[Delta]", "0"]}], ")"}], "VHi"}]}]}]], "Input",
 CellLabel->"In[225]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"VHi", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SubscriptBox["\[Delta]", "0"]}], ")"}]}], "+", 
  RowBox[{"VLo", " ", 
   SubscriptBox["\[Delta]", "0"]}]}]], "Output",
 CellChangeTimes->{3.3896128652692003`*^9, 3.3896138230018*^9, 
  3.3899914969625998`*^9},
 CellLabel->"Out[225]="]
}, Open  ]],

Cell["The order-informed trader's expected profits are:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"EV", "-", "Ask"}], "]"}]], "Input",
 CellLabel->"In[226]:="],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"2", " ", 
    RowBox[{"(", 
     RowBox[{"VHi", "-", "VLo"}], ")"}], " ", "\[Mu]", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      SubscriptBox["\[Delta]", "0"]}], ")"}], " ", 
    SubscriptBox["\[Delta]", "0"]}], 
   RowBox[{"1", "-", "\[Mu]", "+", 
    RowBox[{"2", " ", "\[Mu]", " ", 
     SubscriptBox["\[Delta]", "0"]}]}]]}]], "Output",
 CellChangeTimes->{3.3896128653062*^9, 3.3896138230448*^9, 
  3.3899914970486*^9},
 CellLabel->"Out[226]="]
}, Open  ]],

Cell["The expected profits are positive", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"EV", "-", "Ask"}], ">", "0"}], ",", "ParameterRestrictions"}], 
  "]"}]], "Input",
 CellLabel->"In[227]:="],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.3896128653562*^9, 3.3896138231018*^9, 
  3.3899914971776*^9},
 CellLabel->"Out[227]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Ask", "/.", 
    RowBox[{
     SubscriptBox["\[Delta]", "0"], "\[Rule]", 
     RowBox[{"1", "/", "2"}]}]}], "/.", 
   RowBox[{"VLo", "\[Rule]", "0"}]}], "//", "Simplify"}]], "Input",
 CellLabel->"In[228]:="],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"(", 
   RowBox[{"VHi", "-", 
    RowBox[{"VHi", " ", "\[Mu]"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.3896128653882*^9, 3.3896138231328*^9, 
  3.3899914973366003`*^9},
 CellLabel->"Out[228]="]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The distribution of buys and sells (Chapter 6)", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PDF", "[", 
  RowBox[{
   RowBox[{"BinomialDistribution", "[", 
    RowBox[{"n", ",", "p"}], "]"}], ",", "nBuys"}], "]"}]], "Input",
 CellLabel->"In[231]:="],

Cell[BoxData[
 RowBox[{
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"1", "-", "p"}], ")"}], 
   RowBox[{"n", "-", "nBuys"}]], " ", 
  SuperscriptBox["p", "nBuys"], " ", 
  RowBox[{"Binomial", "[", 
   RowBox[{"n", ",", "nBuys"}], "]"}]}]], "Output",
 CellChangeTimes->{3.3896128661122*^9, 3.3896138681568003`*^9, 
  3.3899914976946*^9},
 CellLabel->"Out[231]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"p", ",", "a", ",", "b", ",", "n", ",", "\[Delta]"}], 
  "]"}]], "Input",
 CellLabel->"In[232]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"PN", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"\[Delta]", " ", 
       RowBox[{"PDF", "[", 
        RowBox[{
         RowBox[{"BinomialDistribution", "[", 
          RowBox[{"n", ",", "p"}], "]"}], ",", "b"}], "]"}]}], "/.", 
      "\[InvisibleSpace]", 
      RowBox[{"p", "\[Rule]", 
       FractionBox[
        RowBox[{"1", "-", "\[Mu]"}], "2"]}]}], ")"}], "+", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", 
       RowBox[{"PDF", "[", 
        RowBox[{
         RowBox[{"BinomialDistribution", "[", 
          RowBox[{"n", ",", "p"}], "]"}], ",", "b"}], "]"}]}], "/.", 
      "\[InvisibleSpace]", 
      RowBox[{"p", "\[Rule]", 
       RowBox[{
        FractionBox[
         RowBox[{"1", "-", "\[Mu]"}], "2"], "+", "\[Mu]"}]}]}], ")"}]}]}], 
  ";"}]], "Input",
 CellLabel->"In[233]:="],

Cell[CellGroupData[{

Cell[BoxData["PN"], "Input",
 CellLabel->"In[234]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"-", "b"}]], " ", "\[Delta]", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "+", 
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "\[Mu]"}], ")"}]}]}], ")"}], 
    RowBox[{
     RowBox[{"-", "b"}], "+", "n"}]], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "-", "\[Mu]"}], ")"}], "b"], " ", 
   RowBox[{"Binomial", "[", 
    RowBox[{"n", ",", "b"}], "]"}]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "+", 
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "\[Mu]"}], ")"}]}], "-", "\[Mu]"}], ")"}], 
    RowBox[{
     RowBox[{"-", "b"}], "+", "n"}]], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"1", "-", "\[Mu]"}], "2"], "+", "\[Mu]"}], ")"}], "b"], " ", 
   RowBox[{"Binomial", "[", 
    RowBox[{"n", ",", "b"}], "]"}]}]}]], "Output",
 CellChangeTimes->{3.3896128661612*^9, 3.3896138682047997`*^9, 
  3.3899914977806*^9},
 CellLabel->"Out[234]="]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"b", ",", 
       RowBox[{
        RowBox[{"PN", "/.", "\[InvisibleSpace]", 
         RowBox[{"n", "\[Rule]", "50"}]}], "/.", "\[InvisibleSpace]", 
        RowBox[{"\[Delta]", "\[Rule]", 
         FractionBox["1", "2"]}]}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"b", ",", "0", ",", "50"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"bPlot", "=", 
   RowBox[{"N", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", "/.", "\[InvisibleSpace]", 
       RowBox[{"\[Mu]", "\[Rule]", "0"}]}], ",", 
      RowBox[{"a", "/.", "\[InvisibleSpace]", 
       RowBox[{"\[Mu]", "\[Rule]", ".1"}]}], ",", 
      RowBox[{"a", "/.", "\[InvisibleSpace]", 
       RowBox[{"\[Mu]", "\[Rule]", ".5"}]}]}], "}"}], "]"}]}], 
  ";"}]}], "Input",
 CellLabel->"In[235]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"lineStyles", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"GrayLevel", "[", "0", "]"}], ",", 
       RowBox[{"Dashing", "[", 
        RowBox[{".01", "r"}], "]"}], ",", 
       RowBox[{"AbsoluteThickness", "[", 
        RowBox[{".5", "r"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"r", ",", "0", ",", "2"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.3896149201127996`*^9, 3.3896149841478*^9}, {
   3.389619683251*^9, 3.3896196876140003`*^9}, 3.389621668238*^9},
 CellLabel->"In[237]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{"bPlot", ",", 
    RowBox[{"PlotJoined", "\[Rule]", "True"}], ",", 
    RowBox[{"Frame", "\[Rule]", "True"}], ",", 
    RowBox[{"FrameTicks", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"Automatic", ",", "Automatic", ",", "None", ",", "None"}], 
      "}"}]}], ",", 
    RowBox[{"FrameLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"\"\<Number of buys in fifty trades, \!\(\*
StyleBox[\"b\",
FontSlant->\"Italic\"]\))\>\"", ",", "\"\<Probability\>\""}], "}"}]}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", "lineStyles"}], ",", 
    RowBox[{"BaseStyle", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"FontFamily", "\[Rule]", "\"\<Times\>\""}], "}"}]}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.3896129925372*^9, 3.3896129954812*^9}, {
  3.3896131106492*^9, 3.3896131441352*^9}, {3.3896133181172*^9, 
  3.3896133523952*^9}, {3.3896138753478003`*^9, 3.3896138765768003`*^9}, {
  3.3896139289568*^9, 3.3896139474818*^9}, {3.3896140570508003`*^9, 
  3.3896140871647997`*^9}, {3.3896143301668*^9, 3.3896144118338003`*^9}, {
  3.3896144772148*^9, 3.3896145197768*^9}, {3.3896145959288*^9, 
  3.3896146587208*^9}, {3.3896147296038*^9, 3.3896147541348*^9}, {
  3.3896147841768*^9, 3.3896148028398*^9}, {3.3896149088928003`*^9, 
  3.3896149128838*^9}, {3.3896149873858*^9, 3.3896149931688004`*^9}, {
  3.38961974325*^9, 3.3896197461*^9}, {3.389621564369*^9, 3.389621576855*^9}},
 
 CellLabel->"In[238]:="],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], AbsoluteThickness[0], Dashing[0], 
    LineBox[{{0., 8.881784197001252*^-16}, {1., 4.440892098500626*^-14}, {2., 
     1.0880185641326534`*^-12}, {3., 1.7408297026122455`*^-11}, {4., 
     2.0454749005693884`*^-10}, {5., 1.8818369085238373`*^-9}, {6., 
     1.411377681392878*^-8}, {7., 8.871516854469519*^-8}, {8., 
     4.768440309277366*^-7}, {9., 2.2252721443294377`*^-6}, {10., 
     9.123615791750694*^-6}, {11., 0.00003317678469727525}, {12., 
     0.00010782455026614457`}, {13., 0.0003151794546241149}, {14., 
     0.0008329742729351608}, {15., 0.001999138255044386}, {16., 
     0.004373114932909594}, {17., 0.008746229865819188}, {18., 
     0.016034754754001845`}, {19., 0.027005902743582055`}, {20., 
     0.041859149252552186`}, {21., 0.05979878464650312}, {22., 
     0.07882567067039048}, {23., 0.09596168603351884}, {24., 
     0.1079568967877087}, {25., 0.11227517265921705`}, {26., 
     0.1079568967877087}, {27., 0.09596168603351884}, {28., 
     0.07882567067039048}, {29., 0.05979878464650312}, {30., 
     0.041859149252552186`}, {31., 0.027005902743582055`}, {32., 
     0.016034754754001845`}, {33., 0.008746229865819188}, {34., 
     0.004373114932909594}, {35., 0.001999138255044386}, {36., 
     0.0008329742729351608}, {37., 0.0003151794546241149}, {38., 
     0.00010782455026614457`}, {39., 0.00003317678469727525}, {40., 
     9.123615791750694*^-6}, {41., 2.2252721443294377`*^-6}, {42., 
     4.768440309277366*^-7}, {43., 8.871516854469519*^-8}, {44., 
     1.411377681392878*^-8}, {45., 1.8818369085238373`*^-9}, {46., 
     2.0454749005693884`*^-10}, {47., 1.7408297026122455`*^-11}, {48., 
     1.0880185641326534`*^-12}, {49., 4.440892098500626*^-14}, {50., 
     8.881784197001252*^-16}}]}, 
   {GrayLevel[0], AbsoluteThickness[0.5], Dashing[0.01], 
    LineBox[{{0., 5.213429983492861*^-14}, {1., 2.13281304851899*^-12}, {2., 
     4.275459156201529*^-11}, {3., 5.597235468839238*^-10}, {4., 
     5.381367478553177*^-9}, {5., 4.051139288632485*^-8}, {6., 
     2.486327169372263*^-7}, {7., 1.2789904509983182`*^-6}, {8., 
     5.626673964199449*^-6}, {9., 0.00002149520154353981}, {10., 
     0.00007216444540780882}, {11., 0.00021496108965489704`}, {12., 
     0.0005726217599121578}, {13., 0.0013731335403419764`}, {14., 
     0.0029809572537181843`}, {15., 0.005888067454965133}, {16., 
     0.010630679535784768`}, {17., 0.01762146783863912}, {18., 
     0.02693818435777257}, {19., 0.03816211508868999}, {20., 
     0.05036966825130327}, {21., 0.06231885175185832}, {22., 
     0.07276227597020764}, {23., 0.08073357425832416}, {24., 
     0.08566756769503875}, {25., 0.08733002742456622}, {26., 
     0.08566756769503872}, {27., 0.08073357425832407}, {28., 
     0.07276227597020755}, {29., 0.06231885175185823}, {30., 
     0.05036966825130318}, {31., 0.03816211508868992}, {32., 
     0.026938184357772513`}, {33., 0.017621467838639087`}, {34., 
     0.01063067953578475}, {35., 0.005888067454965123}, {36., 
     0.0029809572537181786`}, {37., 0.0013731335403419742`}, {38., 
     0.0005726217599121568}, {39., 0.0002149610896548968}, {40., 
     0.00007216444540780874}, {41., 0.000021495201543539785`}, {42., 
     5.626673964199443*^-6}, {43., 1.2789904509983169`*^-6}, {44., 
     2.4863271693722616`*^-7}, {45., 4.051139288632483*^-8}, {46., 
     5.381367478553175*^-9}, {47., 5.597235468839235*^-10}, {48., 
     4.2754591562015287`*^-11}, {49., 2.1328130485189895`*^-12}, {50., 
     5.213429983492861*^-14}}]}, 
   {GrayLevel[0], AbsoluteThickness[1.], Dashing[0.02], 
    LineBox[{{0., 2.831608282134688*^-7}, {1., 4.719347136891147*^-6}, {2., 
     0.0000385413349512777}, {3., 0.0002055537864068144}, {4., 
     0.0008050856634266898}, {5., 0.0024689293678418485`}, {6., 
     0.006172323419604621}, {7., 0.012932487164885872`}, {8., 
     0.023170706170420524`}, {9., 0.03604332070954306}, {10., 
     0.04925920496970905}, {11., 0.05970812723601329}, {12., 
     0.06468380450570368}, {13., 0.06302524541601222}, {14., 
     0.0555222400109168}, {15., 0.0444177920200569}, {16., 
     0.032387973422268394`}, {17., 0.021591982727373706`}, {18., 
     0.013195103007855194`}, {19., 0.007407789517471755}, {20., 
     0.003827415532045243}, {21., 0.00182282574486132}, {22., 
     0.000801915041534887}, {23., 0.0003289809890465933}, {24., 
     0.00013540376073564053`}, {25., 0.00008449194669903969}, {26., 
     0.00013540376073564053`}, {27., 0.0003289809890465933}, {28., 
     0.000801915041534887}, {29., 0.00182282574486132}, {30., 
     0.003827415532045243}, {31., 0.007407789517471755}, {32., 
     0.013195103007855194`}, {33., 0.021591982727373706`}, {34., 
     0.032387973422268394`}, {35., 0.0444177920200569}, {36., 
     0.0555222400109168}, {37., 0.06302524541601222}, {38., 
     0.06468380450570368}, {39., 0.05970812723601329}, {40., 
     0.04925920496970905}, {41., 0.03604332070954306}, {42., 
     0.023170706170420524`}, {43., 0.012932487164885872`}, {44., 
     0.006172323419604621}, {45., 0.0024689293678418485`}, {46., 
     0.0008050856634266898}, {47., 0.0002055537864068144}, {48., 
     0.0000385413349512777}, {49., 4.719347136891147*^-6}, {50., 
     2.831608282134688*^-7}}]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesOrigin->{0, 0},
  BaseStyle->{FontFamily -> "Times"},
  Frame->True,
  FrameLabel->{
    FormBox[
    "\"Number of buys in fifty trades, \
\\!\\(\\*\\nStyleBox[\\\"b\\\",\\nFontSlant->\\\"Italic\\\"]\\))\"", 
     TraditionalForm], 
    FormBox["\"Probability\"", TraditionalForm]},
  FrameTicks->{Automatic, Automatic, None, None},
  ImageSize->{297.9999999999989, Automatic},
  PlotRange->{{0., 50.}, {0., 0.11227517265921705`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{
  3.3896147544798*^9, {3.3896147874428*^9, 3.3896148043848*^9}, {
   3.3896149133308*^9, 3.3896149936668*^9}, 3.3896197465299997`*^9, 
   3.389621577264*^9, 3.3899914979866*^9},
 CellLabel->"Out[238]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"legendLines", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Graphics", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"GrayLevel", "[", "0", "]"}], ",", 
        RowBox[{"Dashing", "[", 
         RowBox[{".1", "r"}], "]"}], ",", 
        RowBox[{"AbsoluteThickness", "[", 
         RowBox[{".5", "r"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", ",", "0"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"2", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}], "]"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"r", ",", "0", ",", "2"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.3896149201127996`*^9, 3.3896149841478*^9}, {
  3.389619683251*^9, 3.3896196876140003`*^9}, {3.389619814413*^9, 
  3.3896199705290003`*^9}, {3.389620788172*^9, 3.389620837962*^9}, {
  3.3896208788570004`*^9, 3.389620902436*^9}, {3.3896210739440002`*^9, 
  3.38962107709*^9}},
 CellLabel->"In[239]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"legendText", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"Style", "[", 
      RowBox[{"#", ",", "Black", ",", 
       RowBox[{"FontFamily", "\[Rule]", "\"\<Times\>\""}]}], "]"}], "&"}], 
    " ", "/@", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0.1", ",", "0.5"}], "}"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.389620914783*^9, 3.3896209204820004`*^9}, {
   3.389620988039*^9, 3.389621025764*^9}, 3.389621081079*^9, 
   3.389621619785*^9},
 CellLabel->"In[240]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"legendEntries", "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"{", 
     RowBox[{"legendLines", ",", "legendText"}], "}"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.389621033892*^9, 3.3896210839779997`*^9}},
 CellLabel->"In[241]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ShowLegend", "[", 
  RowBox[{"a", ",", 
   RowBox[{"{", 
    RowBox[{"legendEntries", ",", 
     RowBox[{"LegendPosition", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"1", ",", 
        RowBox[{"-", ".4"}]}], "}"}]}], ",", 
     RowBox[{"LegendSize", "\[Rule]", "0.6"}], ",", 
     RowBox[{"LegendShadow", "\[Rule]", "None"}], ",", 
     RowBox[{"LegendLabel", "\[Rule]", "\[Mu]"}]}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.3896199844309998`*^9, 3.389619996865*^9}, {
  3.389621094799*^9, 3.389621098097*^9}, {3.389621182734*^9, 
  3.389621187608*^9}, {3.3896212580950003`*^9, 3.389621276164*^9}, {
  3.389621319869*^9, 3.3896213638640003`*^9}, {3.3896216388050003`*^9, 
  3.3896216436809998`*^9}},
 CellLabel->"In[242]:="],

Cell[BoxData[
 GraphicsBox[{InsetBox[
    GraphicsBox[{{}, {}, 
      {GrayLevel[0], AbsoluteThickness[0], Dashing[0], 
       LineBox[{{0., 8.881784197001252*^-16}, {1., 4.440892098500626*^-14}, {
        2., 1.0880185641326534`*^-12}, {3., 1.7408297026122455`*^-11}, {4., 
        2.0454749005693884`*^-10}, {5., 1.8818369085238373`*^-9}, {6., 
        1.411377681392878*^-8}, {7., 8.871516854469519*^-8}, {8., 
        4.768440309277366*^-7}, {9., 2.2252721443294377`*^-6}, {10., 
        9.123615791750694*^-6}, {11., 0.00003317678469727525}, {12., 
        0.00010782455026614457`}, {13., 0.0003151794546241149}, {14., 
        0.0008329742729351608}, {15., 0.001999138255044386}, {16., 
        0.004373114932909594}, {17., 0.008746229865819188}, {18., 
        0.016034754754001845`}, {19., 0.027005902743582055`}, {20., 
        0.041859149252552186`}, {21., 0.05979878464650312}, {22., 
        0.07882567067039048}, {23., 0.09596168603351884}, {24., 
        0.1079568967877087}, {25., 0.11227517265921705`}, {26., 
        0.1079568967877087}, {27., 0.09596168603351884}, {28., 
        0.07882567067039048}, {29., 0.05979878464650312}, {30., 
        0.041859149252552186`}, {31., 0.027005902743582055`}, {32., 
        0.016034754754001845`}, {33., 0.008746229865819188}, {34., 
        0.004373114932909594}, {35., 0.001999138255044386}, {36., 
        0.0008329742729351608}, {37., 0.0003151794546241149}, {38., 
        0.00010782455026614457`}, {39., 0.00003317678469727525}, {40., 
        9.123615791750694*^-6}, {41., 2.2252721443294377`*^-6}, {42., 
        4.768440309277366*^-7}, {43., 8.871516854469519*^-8}, {44., 
        1.411377681392878*^-8}, {45., 1.8818369085238373`*^-9}, {46., 
        2.0454749005693884`*^-10}, {47., 1.7408297026122455`*^-11}, {48., 
        1.0880185641326534`*^-12}, {49., 4.440892098500626*^-14}, {50., 
        8.881784197001252*^-16}}]}, 
      {GrayLevel[0], AbsoluteThickness[0.5], Dashing[0.01], 
       LineBox[{{0., 5.213429983492861*^-14}, {1., 2.13281304851899*^-12}, {
        2., 4.275459156201529*^-11}, {3., 5.597235468839238*^-10}, {4., 
        5.381367478553177*^-9}, {5., 4.051139288632485*^-8}, {6., 
        2.486327169372263*^-7}, {7., 1.2789904509983182`*^-6}, {8., 
        5.626673964199449*^-6}, {9., 0.00002149520154353981}, {10., 
        0.00007216444540780882}, {11., 0.00021496108965489704`}, {12., 
        0.0005726217599121578}, {13., 0.0013731335403419764`}, {14., 
        0.0029809572537181843`}, {15., 0.005888067454965133}, {16., 
        0.010630679535784768`}, {17., 0.01762146783863912}, {18., 
        0.02693818435777257}, {19., 0.03816211508868999}, {20., 
        0.05036966825130327}, {21., 0.06231885175185832}, {22., 
        0.07276227597020764}, {23., 0.08073357425832416}, {24., 
        0.08566756769503875}, {25., 0.08733002742456622}, {26., 
        0.08566756769503872}, {27., 0.08073357425832407}, {28., 
        0.07276227597020755}, {29., 0.06231885175185823}, {30., 
        0.05036966825130318}, {31., 0.03816211508868992}, {32., 
        0.026938184357772513`}, {33., 0.017621467838639087`}, {34., 
        0.01063067953578475}, {35., 0.005888067454965123}, {36., 
        0.0029809572537181786`}, {37., 0.0013731335403419742`}, {38., 
        0.0005726217599121568}, {39., 0.0002149610896548968}, {40., 
        0.00007216444540780874}, {41., 0.000021495201543539785`}, {42., 
        5.626673964199443*^-6}, {43., 1.2789904509983169`*^-6}, {44., 
        2.4863271693722616`*^-7}, {45., 4.051139288632483*^-8}, {46., 
        5.381367478553175*^-9}, {47., 5.597235468839235*^-10}, {48., 
        4.2754591562015287`*^-11}, {49., 2.1328130485189895`*^-12}, {50., 
        5.213429983492861*^-14}}]}, 
      {GrayLevel[0], AbsoluteThickness[1.], Dashing[0.02], 
       LineBox[{{0., 2.831608282134688*^-7}, {1., 4.719347136891147*^-6}, {2.,
         0.0000385413349512777}, {3., 0.0002055537864068144}, {4., 
        0.0008050856634266898}, {5., 0.0024689293678418485`}, {6., 
        0.006172323419604621}, {7., 0.012932487164885872`}, {8., 
        0.023170706170420524`}, {9., 0.03604332070954306}, {10., 
        0.04925920496970905}, {11., 0.05970812723601329}, {12., 
        0.06468380450570368}, {13., 0.06302524541601222}, {14., 
        0.0555222400109168}, {15., 0.0444177920200569}, {16., 
        0.032387973422268394`}, {17., 0.021591982727373706`}, {18., 
        0.013195103007855194`}, {19., 0.007407789517471755}, {20., 
        0.003827415532045243}, {21., 0.00182282574486132}, {22., 
        0.000801915041534887}, {23., 0.0003289809890465933}, {24., 
        0.00013540376073564053`}, {25., 0.00008449194669903969}, {26., 
        0.00013540376073564053`}, {27., 0.0003289809890465933}, {28., 
        0.000801915041534887}, {29., 0.00182282574486132}, {30., 
        0.003827415532045243}, {31., 0.007407789517471755}, {32., 
        0.013195103007855194`}, {33., 0.021591982727373706`}, {34., 
        0.032387973422268394`}, {35., 0.0444177920200569}, {36., 
        0.0555222400109168}, {37., 0.06302524541601222}, {38., 
        0.06468380450570368}, {39., 0.05970812723601329}, {40., 
        0.04925920496970905}, {41., 0.03604332070954306}, {42., 
        0.023170706170420524`}, {43., 0.012932487164885872`}, {44., 
        0.006172323419604621}, {45., 0.0024689293678418485`}, {46., 
        0.0008050856634266898}, {47., 0.0002055537864068144}, {48., 
        0.0000385413349512777}, {49., 4.719347136891147*^-6}, {50., 
        2.831608282134688*^-7}}]}},
     AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
     Axes->True,
     AxesOrigin->{0, 0},
     BaseStyle->{FontFamily -> "Times"},
     Frame->True,
     FrameLabel->{
       FormBox[
       "\"Number of buys in fifty trades, \
\\!\\(\\*\\nStyleBox[\\\"b\\\",\\nFontSlant->\\\"Italic\\\"]\\))\"", 
        TraditionalForm], 
       FormBox["\"Probability\"", TraditionalForm]},
     FrameTicks->{Automatic, Automatic, None, None},
     PlotRange->{{0., 50.}, {0., 0.11227517265921705`}},
     PlotRangeClipping->True,
     PlotRangePadding->{
       Scaled[0.02], 
       Scaled[0.02]}], {-1, -0.6180339887498948}, {
    Left, Bottom}, {2, 1.2360679774997896`}], InsetBox[
    GraphicsBox[{{{InsetBox[
         GraphicsBox[
          {GrayLevel[0], AbsoluteThickness[1.], Dashing[0.2], 
           LineBox[{{0, 0}, {2, 0}}]}], {0.08, 0.08}, {Left, Bottom}, {1, 1}],
         InsetBox[
         StyleBox["0.5`",
          StripOnInput->False,
          FrontFaceColor->GrayLevel[0],
          BackFaceColor->GrayLevel[0],
          GraphicsColor->GrayLevel[0],
          FontFamily->"Times",
          FontColor->GrayLevel[0]], {1.21, 0.58}, {-1, 0}, Automatic, {1, 
         0}]}, {InsetBox[
         GraphicsBox[
          {GrayLevel[0], AbsoluteThickness[0.5], Dashing[0.1], 
           LineBox[{{0, 0}, {2, 0}}]}], {0.08, 1.24}, {Left, Bottom}, {1, 1}],
         InsetBox[
         StyleBox["0.1`",
          StripOnInput->False,
          FrontFaceColor->GrayLevel[0],
          BackFaceColor->GrayLevel[0],
          GraphicsColor->GrayLevel[0],
          FontFamily->"Times",
          FontColor->GrayLevel[0]], {1.21, 1.74}, {-1, 0}, Automatic, {1, 
         0}]}, {InsetBox[
         GraphicsBox[
          {GrayLevel[0], AbsoluteThickness[0], Dashing[0], 
           LineBox[{{0, 0}, {2, 0}}]}], {0.08, 2.4}, {Left, Bottom}, {1, 1}], 
        InsetBox[
         StyleBox["0",
          StripOnInput->False,
          FrontFaceColor->GrayLevel[0],
          BackFaceColor->GrayLevel[0],
          GraphicsColor->GrayLevel[0],
          FontFamily->"Times",
          FontColor->GrayLevel[0]], {1.21, 2.9}, {-1, 0}, Automatic, {1, 
         0}]}}, InsetBox["\[Mu]", {1.58, 3.9799999999999995`}, {0, 0}]},
     AspectRatio->1.392857142857143,
     PlotRange->{{-0.1, 3.26}, {-0.1, 4.58}}], {1, -0.4}, {
    Left, Bottom}, {0.4307692307692308, 0.6000000000000001}]},
  AspectRatio->Automatic,
  Background->None,
  ColorOutput->Automatic,
  ImageSize->Automatic,
  PlotRange->All]], "Output",
 CellChangeTimes->{
  3.389619997427*^9, {3.38962079345*^9, 3.3896208038780003`*^9}, 
   3.389620841184*^9, {3.3896210658640003`*^9, 3.389621087699*^9}, 
   3.389621189084*^9, {3.38962125948*^9, 3.389621276951*^9}, {
   3.389621325042*^9, 3.389621364722*^9}, 3.389621585115*^9, {
   3.3896216324709997`*^9, 3.389621644575*^9}, 3.3899914987246*^9},
 CellLabel->"Out[242]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PDF", "[", 
  RowBox[{
   RowBox[{"PoissonDistribution", "[", "\[Mu]", "]"}], ",", "n"}], 
  "]"}]], "Input",
 CellLabel->"In[243]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{
   SuperscriptBox["\[ExponentialE]", 
    RowBox[{"-", "\[Mu]"}]], " ", 
   SuperscriptBox["\[Mu]", "n"]}], 
  RowBox[{"n", "!"}]]], "Output",
 CellChangeTimes->{3.3896128664222*^9, 3.3899914987936*^9},
 CellLabel->"Out[243]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "f", "]"}], ";"}], "\n", 
 RowBox[{"fMixDefinition", "=", 
  RowBox[{"fMix", "\[RuleDelayed]", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "\[Alpha]"}], ")"}], " ", 
     RowBox[{"f", "[", 
      RowBox[{"\[Epsilon]", ",", "B"}], "]"}], " ", 
     RowBox[{"f", "[", 
      RowBox[{"\[Epsilon]", ",", "S"}], "]"}]}], "+", 
    RowBox[{"\[Alpha]", " ", "\[Delta]", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"f", "[", 
        RowBox[{
         RowBox[{"\[Epsilon]", "+", "\[Mu]"}], ",", "B"}], "]"}], " ", 
       RowBox[{"f", "[", 
        RowBox[{"\[Epsilon]", ",", "S"}], "]"}]}], ")"}]}], "+", 
    RowBox[{"\[Alpha]", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", 
     RowBox[{"f", "[", 
      RowBox[{
       RowBox[{"\[Epsilon]", "+", "\[Mu]"}], ",", "S"}], "]"}], " ", 
     RowBox[{"f", "[", 
      RowBox[{"\[Epsilon]", ",", "B"}], "]"}]}]}]}]}]}], "Input",
 CellLabel->"In[244]:="],

Cell[BoxData[
 RowBox[{"fMix", "\[RuleDelayed]", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "\[Alpha]"}], ")"}], " ", 
    RowBox[{"f", "[", 
     RowBox[{"\[Epsilon]", ",", "B"}], "]"}], " ", 
    RowBox[{"f", "[", 
     RowBox[{"\[Epsilon]", ",", "S"}], "]"}]}], "+", 
   RowBox[{"\[Alpha]", " ", "\[Delta]", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"f", "[", 
       RowBox[{
        RowBox[{"\[Epsilon]", "+", "\[Mu]"}], ",", "B"}], "]"}], " ", 
      RowBox[{"f", "[", 
       RowBox[{"\[Epsilon]", ",", "S"}], "]"}]}], ")"}]}], "+", 
   RowBox[{"\[Alpha]", " ", 
    RowBox[{"(", 
     RowBox[{"1", "-", "\[Delta]"}], ")"}], " ", 
    RowBox[{"f", "[", 
     RowBox[{
      RowBox[{"\[Epsilon]", "+", "\[Mu]"}], ",", "S"}], "]"}], " ", 
    RowBox[{"f", "[", 
     RowBox[{"\[Epsilon]", ",", "B"}], "]"}]}]}]}]], "Output",
 CellChangeTimes->{3.3896128664561996`*^9, 3.389621898863*^9, 
  3.3899914988536*^9},
 CellLabel->"Out[245]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"p1", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"n", ",", 
          RowBox[{"N", "[", 
           RowBox[{"PDF", "[", 
            RowBox[{
             RowBox[{"PoissonDistribution", "[", "10", "]"}], ",", "n"}], 
            "]"}], "]"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"n", ",", "0", ",", "20"}], "}"}]}], "]"}], "}"}], ",", 
     RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{
  3.389621747774*^9, {3.389621779368*^9, 3.3896217812209997`*^9}, 
   3.389621864998*^9},
 CellLabel->"In[246]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"p2", "=", 
   RowBox[{"Plot", "[", 
    RowBox[{
     RowBox[{"PDF", "[", 
      RowBox[{
       RowBox[{"NormalDistribution", "[", 
        RowBox[{"10", ",", 
         SqrtBox["10"]}], "]"}], ",", "x"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "20"}], "}"}], ",", 
     RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.389621785126*^9, 3.389621804918*^9}, 3.389621866954*^9},
 
 CellLabel->"In[247]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"p1", ",", "p2", ",", 
   RowBox[{"BaseStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"FontFamily", "\[Rule]", "\"\<Times\>\""}], "}"}]}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.389621812567*^9, 3.3896218464300003`*^9}},
 CellLabel->"In[248]:="],

Cell[BoxData[
 GraphicsBox[{{{}, 
    {Hue[0.67, 0.6, 0.6], 
     PointBox[{{0., 0.000045399929762484854`}, {1., 
      0.00045399929762484856`}, {2., 0.0022699964881242427`}, {3., 
      0.007566654960414142}, {4., 0.018916637401035358`}, {5., 
      0.037833274802070715`}, {6., 0.06305545800345119}, {7., 
      0.09007922571921598}, {8., 0.11259903214901998`}, {9., 
      0.1251100357211333}, {10., 0.1251100357211333}, {11., 
      0.1137363961101212}, {12., 0.09478033009176765}, {13., 
      0.07290794622443666}, {14., 0.05207710444602618}, {15., 
      0.03471806963068413}, {16., 0.021698793519177577`}, {17., 
      0.012763996187751515`}, {18., 0.007091108993195287}, {19., 
      0.0037321626279975197`}, {20., 
      0.0018660813139987598`}}]}, {}}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwt2Xc4le//AHCjkNYp7WWcY0vOOTLT+61S4URkZ+QUskeIkBlZGfmIBiUq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      "]]}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesOrigin->{0, 0},
  BaseStyle->{FontFamily -> "Times"},
  DisplayFunction:>Identity,
  PlotRange->{{0., 20.}, {0., 0.1251100357211333}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{{3.389621819335*^9, 3.389621869723*^9}, 
   3.3899914992785997`*^9},
 CellLabel->"Out[248]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Mixture aspects of Easley, Kiefer, O'Hara and Paperman (section 6.2)\
\>", "Section",
 CellTags->"c:132"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"fMix", "/.", "\[InvisibleSpace]", "fMixDefinition"}], "/.", 
  "\[InvisibleSpace]", 
  RowBox[{"\[Delta]", "\[Rule]", 
   FractionBox["1", "2"]}]}]], "Input",
 CellLabel->"In[249]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", "\[Alpha]"}], ")"}], " ", 
   RowBox[{"f", "[", 
    RowBox[{"\[Epsilon]", ",", "B"}], "]"}], " ", 
   RowBox[{"f", "[", 
    RowBox[{"\[Epsilon]", ",", "S"}], "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", "\[Alpha]", " ", 
   RowBox[{"f", "[", 
    RowBox[{"\[Epsilon]", ",", "S"}], "]"}], " ", 
   RowBox[{"f", "[", 
    RowBox[{
     RowBox[{"\[Epsilon]", "+", "\[Mu]"}], ",", "B"}], "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", "\[Alpha]", " ", 
   RowBox[{"f", "[", 
    RowBox[{"\[Epsilon]", ",", "B"}], "]"}], " ", 
   RowBox[{"f", "[", 
    RowBox[{
     RowBox[{"\[Epsilon]", "+", "\[Mu]"}], ",", "S"}], "]"}]}]}]], "Output",
 CellChangeTimes->{
  3.3896128668402*^9, {3.389621883979*^9, 3.3896219037019997`*^9}, 
   3.3899914993136*^9},
 CellLabel->"Out[249]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fMixApprox", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"fMix", "/.", "\[InvisibleSpace]", "fMixDefinition"}], "/.", 
    "\[InvisibleSpace]", 
    RowBox[{
     RowBox[{"f", "[", 
      RowBox[{"\[Lambda]_", ",", "x_"}], "]"}], "\[RuleDelayed]", 
     RowBox[{"PDF", "[", 
      RowBox[{
       RowBox[{"NormalDistribution", "[", 
        RowBox[{"\[Lambda]", ",", 
         SqrtBox["\[Lambda]"]}], "]"}], ",", "x"}], "]"}]}]}], "/.", 
   "\[InvisibleSpace]", 
   RowBox[{"\[Delta]", "\[Rule]", 
    FractionBox["1", "2"]}]}]}]], "Input",
 CellLabel->"In[250]:="],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"B", "-", "\[Epsilon]"}], ")"}], "2"], 
        RowBox[{"2", " ", "\[Epsilon]"}]]}], "-", 
      FractionBox[
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"S", "-", "\[Epsilon]"}], ")"}], "2"], 
       RowBox[{"2", " ", "\[Epsilon]"}]]}]], " ", 
    RowBox[{"(", 
     RowBox[{"1", "-", "\[Alpha]"}], ")"}]}], 
   RowBox[{"2", " ", "\[Pi]", " ", "\[Epsilon]"}]], "+", 
  FractionBox[
   RowBox[{
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"S", "-", "\[Epsilon]"}], ")"}], "2"], 
        RowBox[{"2", " ", "\[Epsilon]"}]]}], "-", 
      FractionBox[
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"B", "-", "\[Epsilon]", "-", "\[Mu]"}], ")"}], "2"], 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{"\[Epsilon]", "+", "\[Mu]"}], ")"}]}]]}]], " ", "\[Alpha]"}],
    
   RowBox[{"4", " ", "\[Pi]", " ", 
    SqrtBox["\[Epsilon]"], " ", 
    SqrtBox[
     RowBox[{"\[Epsilon]", "+", "\[Mu]"}]]}]], "+", 
  FractionBox[
   RowBox[{
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"B", "-", "\[Epsilon]"}], ")"}], "2"], 
        RowBox[{"2", " ", "\[Epsilon]"}]]}], "-", 
      FractionBox[
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"S", "-", "\[Epsilon]", "-", "\[Mu]"}], ")"}], "2"], 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{"\[Epsilon]", "+", "\[Mu]"}], ")"}]}]]}]], " ", "\[Alpha]"}],
    
   RowBox[{"4", " ", "\[Pi]", " ", 
    SqrtBox["\[Epsilon]"], " ", 
    SqrtBox[
     RowBox[{"\[Epsilon]", "+", "\[Mu]"}]]}]]}]], "Output",
 CellChangeTimes->{3.3896128668761997`*^9, 3.389621907432*^9, 
  3.3899914993986*^9},
 CellLabel->"Out[250]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TestValues", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"\[Alpha]", "\[Rule]", ".4"}], ",", 
    RowBox[{"\[Epsilon]", "\[Rule]", "10"}], ",", 
    RowBox[{"\[Mu]", "\[Rule]", "10"}]}], "}"}]}]], "Input",
 CellLabel->"In[251]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"\[Alpha]", "\[Rule]", "0.4`"}], ",", 
   RowBox[{"\[Epsilon]", "\[Rule]", "10"}], ",", 
   RowBox[{"\[Mu]", "\[Rule]", "10"}]}], "}"}]], "Output",
 CellChangeTimes->{3.3896128669152*^9, 3.3896219095299997`*^9, 
  3.3899914994835997`*^9},
 CellLabel->"Out[251]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"p1", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{
     RowBox[{"N", "[", 
      RowBox[{"Log", "[", 
       RowBox[{"fMixApprox", "/.", "\[InvisibleSpace]", "TestValues"}], "]"}],
       "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"S", ",", "0", ",", "30"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"B", ",", "0", ",", "30"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "20"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<Number of buys\>\"", ",", "\"\<Number of sells\>\"", ",", "None", 
        ",", "None"}], "}"}]}], ",", 
     RowBox[{"DisplayFunction", "\[Rule]", "Identity"}], ",", 
     RowBox[{"BaseStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"FontFamily", "\[Rule]", "\"\<Times\>\""}], "}"}]}]}], "]"}]}],
   ";"}]], "Input",
 CellChangeTimes->{{3.3896219251619997`*^9, 3.3896219538450003`*^9}, {
  3.3896220378929996`*^9, 3.389622089898*^9}, {3.389622221758*^9, 
  3.389622243935*^9}},
 CellLabel->"In[252]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"p2", "=", 
   RowBox[{"Plot3D", "[", 
    RowBox[{
     RowBox[{"N", "[", 
      RowBox[{"fMixApprox", "/.", "\[InvisibleSpace]", "TestValues"}], "]"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"S", ",", "0", ",", "30"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"B", ",", "0", ",", "30"}], "}"}], ",", 
     RowBox[{"BaseStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"FontFamily", "\[Rule]", "\"\<Times\>\""}], ",", 
        RowBox[{"FontSize", "\[Rule]", "12"}]}], "}"}]}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "50"}], ",", 
     RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
     RowBox[{"ViewPoint", "\[Rule]", 
      RowBox[{"2", " ", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0.9`", ",", "0.4`"}], "}"}]}]}], ",", 
     RowBox[{"AxesLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<Sells\>\"", ",", "\"\<Buys\>\"", ",", "None"}], "}"}]}], 
     ",", 
     RowBox[{"Ticks", "\[Rule]", "None"}], ",", 
     RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{
  3.389622009024*^9, {3.3896221296359997`*^9, 3.389622197682*^9}, {
   3.389622253207*^9, 3.3896222538190002`*^9}, {3.389622327458*^9, 
   3.389622404193*^9}, {3.3896224529960003`*^9, 3.389622524448*^9}},
 CellLabel->"In[253]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"GraphicsRow", "[", 
   RowBox[{"{", 
    RowBox[{"p1", ",", "p2"}], "}"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.389622261493*^9, 3.389622314986*^9}},
 CellLabel->"In[254]:="],

Cell[BoxData[
 GraphicsBox[{{}, {InsetBox[
     GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyMfQVYlMv3P4goqCggoBgoIiIqUssuu8SO3YWd1+66dncrdnd3YaDX7sTA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       "], {{
         {RGBColor[0.293416, 0.0574044, 0.529412], EdgeForm[None], 
          GraphicsGroupBox[{
            PolygonBox[{{3497, 4106, 216}, {3284, 3252, 970}, {5351, 4732, 
             1306}, {3283, 3251, 907}, {3398, 5111, 1016}, {3387, 3477, 
             180}, {3273, 3272, 216}, {4375, 3515, 1097}, {3241, 3275, 
             1306}, {3255, 3241, 1306}, {3419, 3497, 216}, {1097, 2564, 
             970}, {6316, 5492, 855}, {2511, 1108, 889}, {4744, 3527, 277}, {
             3261, 3234, 2750}, {4198, 4959, 907}, {3266, 3240, 2784}, {3238, 
             3283, 907}, {4959, 4199, 907}, {5493, 5495, 857}, {3232, 3257, 
             180}, {3477, 4953, 180}, {3280, 3246, 855}, {3277, 3276, 2968}, {
             3250, 3237, 889}, {3281, 3282, 3087}, {3248, 3247, 857}, {3271, 
             3238, 907}, {6318, 6319, 2564}, {6571, 6490, 2750}, {3245, 3284, 
             970}, {5492, 4163, 855}, {4260, 5783, 1016}, {6237, 5032, 
             1530}, {3236, 3263, 2751}, {4734, 5351, 1306}, {6587, 6491, 
             2750}, {3527, 4743, 277}, {2783, 2782, 277}, {6265, 6264, 
             2511}, {3249, 3265, 2782}, {6492, 6493, 2751}, {2784, 2785, 
             277}, {3267, 3268, 2835}, {4164, 5493, 857}, {4388, 4389, 
             1108}, {5783, 3398, 1016}, {5491, 4164, 857}, {6529, 6530, 
             2785}, {6527, 6526, 2783}, {5030, 6237, 1530}, {6491, 6571, 
             2750}, {4163, 4235, 855}},
             VertexColors->None], 
            PolygonBox[{{5025, 6828, 3235, 3256}, {6264, 4388, 1108, 2511}, {
             6525, 4182, 2983, 2782}, {3269, 3239, 1108, 2852}, {3282, 3280, 
             855, 3087}, {4389, 3810, 2852, 1108}, {3253, 3267, 2835, 1016}, {
             3240, 3269, 2852, 2784}, {4235, 6792, 3087, 855}, {3254, 3278, 
             2983, 1097}, {4041, 6829, 3258, 3242}, {5111, 4198, 907, 1016}, {
             5032, 4162, 2968, 1530}, {6526, 6525, 2782, 2783}, {6827, 6586, 
             3270, 3233}, {3246, 3245, 970, 855}, {5495, 5496, 889, 857}, {
             6528, 6529, 2785, 2784}, {3785, 4260, 1016, 2835}, {4182, 4375, 
             1097, 2983}, {6317, 6316, 855, 970}, {4732, 5030, 1530, 1306}, {
             6530, 4744, 277, 2785}, {4743, 6527, 2783, 277}, {6319, 6317, 
             970, 2564}, {3252, 3254, 1097, 970}, {3264, 3266, 2784, 277}, {
             3515, 6318, 2564, 1097}, {6829, 6420, 3259, 3258}, {6722, 4041, 
             3242, 3279}, {5496, 6265, 2511, 889}, {3278, 3249, 2782, 2983}, {
             3265, 3264, 277, 2782}, {3810, 6528, 2784, 2852}, {3251, 3253, 
             1016, 907}, {6832, 4108, 3244, 3274}, {3243, 3255, 1306, 1530}, {
             3239, 3250, 889, 1108}, {3237, 3248, 857, 889}, {3276, 3243, 
             1530, 2968}},
             VertexColors->None], 
            PolygonBox[{{4199, 6722, 3279, 3271, 907}, {6830, 3785, 2835, 
             3268, 3260}, {4162, 6831, 3262, 3277, 2968}, {6792, 6827, 3233, 
             3281, 3087}, {6420, 3387, 180, 3257, 3259}, {6490, 6830, 3260, 
             3261, 2750}, {4106, 6832, 3274, 3273, 216}, {6828, 5491, 857, 
             3247, 3235}, {6586, 6587, 2750, 3234, 3270}, {4108, 4734, 1306, 
             3275, 3244}, {6831, 6492, 2751, 3263, 3262}, {6493, 5025, 3256, 
             3236, 2751}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.32520934005089347`, 0.11268443208005588`, 0.5741013855775641], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVL1PU3EUvdi0hT77+mhfSx+LIaEa+RccQEYnhUhMBaIOsulggsUE/IiG
RhdZiBqNMXEwkX7gpmCiMRjxY+LDyUQdtC46aQcZOCfnN9z0/e7vfpx7fue2
78z5kXN7zGwKFoNtdZj9gP3H93Av/IFZwzPrj5v97NB9iHMetonvuwWzI3mz
gU6z775ZqdusBf9T5FZh6yjaiMw+4m4b/n7cV4pmO6i/htwacv/CP46YBuLH
EP8oZzaBcxs9h9B/IzQ7Dv8DxF+KhK8na/YuY/YZ4MuIf4vvN4h/jPh1X9gm
0aedUZ39nrCwVxN9UogLca6g/0XYaNJsDjX3Oj/nOdipOrPoOQerxNw8XWZ9
5Am4ymmzfwndkSPy0wTOL/DfRs0R2LMu4a8Dz0pa3C7huwaLpcTlgOv1HNgf
In9fXPmvUopnndcp8VYH/nKk+EHgPVVU/XvI++ALA+sy91hS77cWF1f0xVzN
ZczdhN1AzGXUm89qBvLHHqx/CHhDzo6cFub9BWsnxP3vQDWvZ4WbmMnDltMP
8dJHzO994ePdMn6/+dLTCmq/iPQ+E3jHyZyweqg9ndVbVHH/EjgXgPNoUu/J
eVfzuqN/ClZyGr0J/1dfOrkSSZfUZBk8HQ6kMfJAfslPFX2uRconFwc86YR6
ZR7xf/KlY/qI/X5BvagF1qAeqE32Zt+ZorRFXfHMXuSEPVqu9qjTB+svAsNm
WnWuRsLNOO7EsMNM7REfsfVgj8YC8c2aBU94/gDbaqQZqGVqnRzWuItuX4hl
wc3O3aKGyOcdzLRYkO65NyeK4vos7tt4l8GEsDzJCU8d8204zHznbcfVBfiX
evVW1BPfhtiId9btEfM4M3P5P3Er0HzcFeYRD/ej4XbkJH7HuTO4z3RLH6Hj
g3msQw5KDsdpxE7nxe8u9Mi75w==
             "],
             VertexColors->None], 
            PolygonBox[{{6494, 4740, 1311, 2753}, {3787, 5789, 486, 469}, {
             6572, 6573, 2837, 2836}, {5552, 3823, 2855, 1925}, {5494, 6494, 
             2753, 1874}, {4733, 5031, 5030, 4732}, {5495, 5493, 1874, 
             1875}, {5493, 4164, 2482, 1874}, {4164, 5491, 1872, 2482}, {6588,
              5829, 501, 2855}, {4334, 6419, 2680, 727}, {4726, 5350, 1782, 
             788}, {5450, 5784, 2089, 2090}, {6574, 5809, 487, 2838}, {5032, 
             6237, 2477, 854}, {6236, 6234, 854, 2477}, {4235, 4163, 2479, 
             969}, {5351, 4734, 788, 1782}, {5551, 5550, 289, 1873}, {6526, 
             6527, 1316, 888}, {6828, 5025, 1527, 1872}, {6240, 6241, 1874, 
             853}, {3515, 4375, 888, 251}, {4743, 3527, 1895, 1316}, {6586, 
             6827, 969, 2854}, {6573, 5785, 2090, 2837}, {6264, 6265, 1875, 
             252}, {3540, 5553, 1925, 289}, {3788, 6574, 2838, 470}, {6232, 
             4160, 853, 1527}, {4163, 5492, 1873, 2479}, {4041, 6722, 2991, 
             727}, {3808, 5808, 1936, 486}, {6491, 6587, 2854, 501}, {6493, 
             6492, 854, 1527}, {4959, 4198, 1599, 1477}, {6571, 6491, 501, 
             2836}, {5110, 4957, 1477, 1599}, {6490, 6571, 2836, 468}, {6420, 
             6829, 727, 2680}, {5549, 6572, 2836, 501}, {6530, 6529, 488, 
             1317}, {4958, 4631, 727, 1477}, {5492, 6316, 2563, 1873}, {6319, 
             6318, 251, 2563}, {4199, 4959, 1477, 2991}, {3549, 5564, 1937, 
             1936}, {5791, 3789, 470, 1311}, {5783, 4260, 468, 2089}, {4161, 
             6238, 1527, 854}, {4389, 4388, 252, 488}, {5788, 3786, 469, 
             487}, {5565, 5551, 1873, 1937}, {3527, 4744, 1317, 1895}},
             VertexColors->None], 
            PolygonBox[{{3498, 4367, 788, 4106, 3497}, {3399, 5110, 1599, 
             5111, 3398}, {5353, 3498, 3497, 3419, 1783}, {5031, 6235, 2477, 
             6237, 5030}, {3418, 4733, 4732, 5351, 1782}, {5782, 3399, 3398, 
             5783, 2089}, {3478, 4954, 1475, 4953, 3477}, {6419, 3478, 3477, 
             3387, 2680}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.3663682987297166, 0.18424874562826687`, 0.6319552811237561], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVE1IVGEUvZXR82ee48y85xtXRUt1hBZmu6wkiGgROZqZVAuXtdOohCBz
0aokpMCIwLLGnxxq0UIX/UhQtijMWhhEkNCibKXbzuncxce73/3uz/nOPd/b
de7C8fNbzewi1jasqzvMflaZbW4xS6XNLmXMIhzsrjY7GJitIma20my6Ugn7
EdOdmE1hP4SYCeTuQY18jVkR8TtRZzE2m89rv4mcE/huwP8S/lex9s9Ds4dZ
s2S72QfY9+Fvgf8vfLfQYxl59cDSh/2PCrNB2Cs5xZfQew6rGzGFQHmLqP8Y
WFodM7EX3B4FvnWcFRDTD3sVdg72JGoPoe4SYsZSZllgLqDXIXw78rrbgUA5
zN0L+5vn3gXeGcRcw1kRfHQmwnUqUV3WPAv7DFYZ/suIvZJXPrHVVwvfRq3Z
yYy4JV+cBTnsQ157Wty34M5rVeLwGHA+SKl+YyBMxPMFHD4CprZAvMTOyXX4
RmLtiaPkvHEuo36vWeBaCoWrB317E827HGEO6DWCuNvA+CklbJxPOdGMJlD7
cyjeqYGkRpyQb9Yf9kWb+PvQayASBs4wdry/UbM3K/6jBsyiQfqaRezTSDXI
DfVKDOzDfrTfgsMM9QFhNwXSLue+Qo3F4ohct7ke3sD3GqszEFbWoobvwfce
Oc3wz6Pnt5T44b2nIvXKNQgfsc0h/nsojZCPm67bonPB2keR9yLULCYR/xX2
vkCr4DOijqZdS8s+G/Zdg94HMtI8NfIxJ52UY82M8xoElo6Ucvh2w7Te72lw
uYH1DCklnP9Cbju4XUf/uUjvfQHnYZ381GOLYyIGap66ilCzy98g+aIWOceu
QLNgnYo66ZjYqBXG0M/3Pej4+bb+pDVfaoF4WYcz/f+fgN2T1TtkDPU+7Jrh
bPgWeZfaOt2HmKmLBdcG+ZuJxOFhnI/Fir+D75FINYYz0h/19g48jMfSC+fd
6O+o32dA/ulrcs08QY1SpDu3ejz9/H/0+Hvkf6XZ65B74iAG8ncjFIfkpsvv
Sz3yn8U61O94ThouZsU74/8Bx0vChA==
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtk0tI1lEQxce0/NL8+/j072eromWl0KLHKnpIFNEi8pGZWYuWtTPoAUEm
IS1ahPSOwF5qptSiQqGnRdkiMmuRUC2iFmWr3PY7HRcDc++cmTn3zNzF+w5u
PzAnIjZjBdj7kojBqojOwojvRRFJecSjbEThvIh1AM6URRxaGDGTHzGB3QH7
Io1ozEScIOci/mQSsYrzUuw3/hh3Tfh1cyO2gD+fGrsC6+d8mnq5BRGL8iLq
6f+E+I6Me3Thb1UPYil3Q/jDWBe5M9xdIP6aHstm+03D9U/W3CqJP0vNQb0b
MuZym/zjheZ0lfPDxPXrOE/Qv6fCePW4WxPxNnGsujiig1hxmfXIgXlK/mPs
b54578lFnExnuc6nN9hXpfQFW4t+3dRqz/r8jfMm6veCXwn+Fnq3EduN3QvH
B+BaiT7Hiq2XepeWexbicCN1DeWuzpi7ZqjZ6Q2f6HcFTC2xNdgo9U6Bny7y
jEfkJ9ZSM5QWndRozrcmk5UR26g3Hn7vFP4Q9daDrc3zbtxPHPu/I/BrQYMB
3j6IvSO/iXM/flCzh9xzqd+iN4rbx8TcxVHa/CqzVtJoJ7kF5c5VzXr4Nszq
08sbsmizsca+NJpB67aceyunOec7+eLQznkt9fvwh7FWzrtyfp/qN+Q8Q81O
mD70Olvh3dTOjCfeCe3CBvheKzFH5aqGuDVmPTtx/Ap+ODV2CTkfEv8R7apm
rt2orjBe79VsRqqsrWb0APzN1LPTPLqJHQbzmdh++IyR35K1XpN85Eupe6qX
/sRP9P+BVeV7Z1q0G5Wel/heB3+EelNF/rNvEv8p8Vue8W4/T/13tONfEnNQ
b+3A0Rr/Of011dBf1s5qvvrT4no5dS1x7iC2N2fttWOt5L4sNXe9IWE2o1lz
Fed/Moulcg==
             "],
             VertexColors->None], 
            PolygonBox[{{6231, 5332, 5331, 4160, 2475}, {5332, 6230, 2474, 
             4159, 5331}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.4075272574085399, 0.25581305917647823`, 0.6898091766699483], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VEtoE1EUvTGNybTOM20y4zSbIogKxYLoQnTbokXtQkmtIFQEoSsrVOxC
ERqt2KjgpzvxB9XgZ+UPbRWTGHUtaHVXN4pK/SxE6M5zPOPiMu/dd+695553
3yzfN7zjwCIzG4IlYVdCsw/O7GPC7FZg9gr731j358yKsFrKbH/WbGVkVkbA
T/iOtpttWGz2cqnZAPZ1YCqIe4w8yzJmU/At4GwB/tf47sG+gfVEwew0LFxi
dgbfO6g3ljZ77pt9AiaTUM2edtXdgvNe2FpgppB/M/wrwGEaGNeKusD3IXY2
b/YNfErADgMzCHym2cyDHU5qzRz30O+FSP2wl1Pg24tcFeT50iIcMVXUqsN2
ZhSXifNw35mSPuRb9cW5HznfgsNXcLiEuHfIOwd/F7BrYjzPd0XCTLaZ3Q50
Rt+bOPYqcl6D/YG/D+fbYevT0no0KQ6jkXQhr1853QfPn1ET9OURC23PZdXT
I/jvO2m1GrGrYGOe+p5w6p19+G3qZS/85dh/N9A98Y4ug+NG4BrIPwPMU94X
Nfd0Tj1nnfpn79SevMiZmnXFOnC+fjita1j3+NL8e1a1WfehE29y3goOx2Hr
0rp7zmkz7Al8M7Czad33XOx/4DQfjKXv3ywkFEcsMaW0tCfnAfTdDQ5VcNiG
fCdD1ZrGuu4LPx5otpjLAV/Mqq9GqFnhXHRj7nLQqgP+GuIOYu8lNLM3Q3Fv
wszuhv6bmlT/sycOeeSrBIrl++H7YOwgsMVI3MjlRCA+7Iv5uGa+675ynge3
eSe9j6DujVBzk0LdZKsw46HyMifv7liL7i9f0H1zfRHnk5H4MZ71qGG5oN68
WE9y5fq903+EvkOR7oZaMzdrsC/2RDy15RmxjKVOI5HW5E0tOQ/kUYrnivPy
Iv4v8U10RnoXnGP+Jzr+v42kePKMddjjX633poM=
             "],
             VertexColors->None], 
            PolygonBox[{{4621, 4622, 886, 264}, {5543, 6184, 2433, 799}, {
             3820, 4757, 1326, 1323}, {4757, 3821, 1325, 1326}, {3768, 5452, 
             1843, 2071}, {3538, 4756, 1325, 1922}, {6321, 4241, 2569, 
             2568}, {5054, 3765, 450, 1545}, {6795, 6794, 265, 3094}, {6677, 
             6676, 1921, 2942}, {4956, 4333, 423, 1476}, {4240, 6321, 2568, 
             3048}, {3489, 3406, 1770, 1083}, {3800, 5799, 2071, 480}, {6511, 
             3519, 264, 1887}, {3763, 5028, 1529, 450}, {5330, 4696, 192, 
             1769}, {5766, 5765, 192, 1843}, {5449, 4384, 178, 1841}, {4366, 
             5053, 1545, 450}, {6182, 6185, 2434, 906}, {6292, 6290, 906, 
             2534}, {4355, 5023, 1526, 1083}, {4181, 6263, 1841, 886}, {6469, 
             6468, 799, 1473}, {6259, 6260, 1933, 265}, {5333, 4357, 1083, 
             1770}, {3821, 6291, 2534, 1325}, {5508, 5509, 1887, 266}, {6187, 
             5451, 1843, 1844}, {5797, 3798, 479, 887}, {4086, 4356, 1083, 
             1526}, {4241, 5104, 1592, 2569}, {6675, 6677, 2942, 799}, {5546, 
             5545, 500, 1325}, {3396, 5454, 1844, 192}, {5798, 3799, 480, 
             804}, {6524, 6523, 887, 1592}, {4952, 3720, 423, 1474}, {6229, 
             6230, 1769, 192}, {4384, 4753, 1323, 178}, {5827, 5828, 2131, 
             500}, {3797, 5796, 804, 479}, {6183, 4036, 1473, 2433}, {3397, 
             6775, 3048, 178}, {3722, 4951, 1474, 423}, {6185, 4116, 799, 
             2434}, {5826, 5825, 178, 2131}, {5547, 3538, 1922, 1921}, {5507, 
             6793, 3094, 1843}, {3721, 4955, 1476, 423}, {5563, 5562, 266, 
             1933}, {5029, 3764, 450, 1529}},
             VertexColors->None], 
            PolygonBox[{{5024, 4698, 4697, 4086, 1526}, {4630, 4037, 1474, 
             4951, 4629}, {4627, 5277, 1732, 5276, 4626}, {4728, 5333, 1770, 
             5334, 4727}, {5052, 4731, 4730, 4109, 1544}, {4698, 5330, 1769, 
             5329, 4697}, {4731, 5054, 1545, 4110, 4730}, {4955, 4627, 4626, 
             4039, 1476}, {4036, 4630, 4629, 4950, 1473}, {4089, 4728, 4727, 
             5029, 1529}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.44868621608736303`, 0.3273773727246892, 0.7476630722161404], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVEtIlGEUveOz0ZlxnIeOtkg3vSdqH0QURFguhMAUooyk1LTQKBHSyCkt
sNJaRKRBD6VFi8pFD8qkBxSECtXCRYW0MXXTpnad07nQ4uP//nvvd+65597v
q2xoq2nNMrMMVjbW/YjZ9bjZk4DZYMxsCGt9rtk1fBfg+wV7brHZp4RZAAdq
cuSnfShltugxp/PN5oKIy5KN52n/ANt8gVka+84ys7dhs1n8H8XZFqxX8O+F
bRTrC/hMwTdcaHYYucaB8wLcypBvBN/sYvFshu9SvjAncX4E8Q34PxiFPSUe
dxD/COdLc+VnHDEj4BXG+pqj3JvyZB/EuatYP4Ly7QfeEdgH8F0Cp0P4PiiU
f0VANS26fRrfhQLVujwkO/3nkb8laXYvIL4/XQf6lnzPeJ7tyJYui77vw9la
1PANXF5Cm7YyadMBju0p5doC/3RYfCqhzZTvG0Pi9B4xJaj1dZH6UoNz1dCo
Cfhrsd/te3IZcD2rYa+KqvZenO9GzQecO3MyZldUcYxJAH8Qs/E9R/nJiRye
ou5nSdU9EZT+7Bt7O+L9ZZ62EuHMFCiWMT1BaU0c9od1M549Zhxj0iHVR/4n
wWUPsD4GpeW0a3gZfLaCz12TrrPer3Zwvh3XbHwG/jvnxp4zH/lQG+pFfOah
BtwzdsLjN4aVg/hngFcP3AJgVqPu7ZznPHHiPHEGyLkx9B8v4/zJ4V9NAfHk
TJB/HfBuxYXJmhrdfhyaHcPKX2b2pkg9Zn/pT3tMEWw7fH7i2NfH1SNqP+z3
pQv2536/arGfSSgX34OxiO7aamgwmtT9Ju/SkLg/RkxnTGcz8PcmhcmcxTHl
JUa3vy1p4GzAOhfUW3IqpveEGtFWgf0a+Fem1P8lv0PsV3+52dmk8k66/pyH
JsQ2p/S/4LrN+12c93ndjD62ptTXCtyRnUm9YYmo8jEX667z3v2GnvucWxti
k1FxG+M9dh3I/U+RYvpg60+K582EtKbOvDcZ7zP7z5niDNDe6/XOJTSLnMNx
7LeFNQs3EuofNeS7+DCut5FvYZdrfgGaXCxXP6qQf11KuCdiwiVmxu8vc12J
qX7Wzlnoj4gn9Wzwe8d5qvFZ5ZuxKqW54ttAXml/53v8DfwLC7Xftw==
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlE1IVGEUhs80/s3UjDN35todN2WLitKobbSrRVK5EAIhiDKENHOMjHDT
r5YWJDO6KvqByqJFi1LRDNKggoLIoFwUVISr2rWpXc/hdfHBd+573nPe8/Pd
hvZia88KM2vmVHA6MmblyGwpYfZ4pdlU3qwJ+xL2YpVZPGtWyYnFzWZiZndz
Zn9rZa+uJA72UNrsO8GmsZ9yn+JbRZXwO9xv58RNYo8GZuuI9we7Efsfse7l
5OsxN5N7X2i2lvsAGnalzNoCcV3PA7A8ms+Bnak26weryShfAZ8y+rfCuU9t
P9DUW2f2FbuVe3WN7uVAuf3bc3LPchZN/CzYdK1q8Zoukm9PKC2HybcTfi8a
32B/Jmc32Ik6xR6nphnsPvB34L+SZvP4X6mnvlVmb/k2gb7JvLjXiHcB/0FO
O/db9P8qvsP1yuWc/dSWRtM3tLyHP0e8JfRug38In25y7cjIngcfg1vkWx/9
a4J/EO6jtPrv/Q3xbQQfTKjHZ4l1Pqf+uk9LRjGc2xXXLFpC9d79e+Duxacz
rpypQJpci2tcH2mnfJd8Rj6bU4Hm6zM6Cd5bMFtIqkfDoXrivejgvCbWUKj7
F3yug93IaxafWNyX4M/AR8A/ghcL6qH37kNStW2KpN1rbKO2V7XieoxmfHeH
mpXP37X4jHw2rmkkUk73/5lQr1+kFNvr9d72F6TNezxOrCMZ+fo+LuD7MNR9
TUy5Tkeanefcwr24vN/+xjZgb4zk7zlL3Buy4vqbfEL+iZx203d0DHw00vtx
fDitN+j76vt+INAb9nn5TLej52ZemL8J3+18INs5v+GXAvXCYxbZ5ctpafUZ
d5LreKTdOhrX7ndx5hJ6A7Ohdt533Wfib7u0/D/xeMcixXCucwZCzdBn5zvv
/4rJtOrz/fgPQiWkTg==
             "],
             VertexColors->None]}]}, 
         {RGBColor[
          0.48984517476618633`, 0.39894168627290055`, 0.8055169677623326], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw9VM9LlFEUvWmODvbZzOd8w9NVtiyV/oASLWqjWTZBv1uYm/LHGGHkSsKF
GWVlrQyaxWCl0aJSxzRoYdm0bNGqoIiCoE1hlrXqHM4j8OKb+94799xz7vfq
OrMH+krM7ByiFFGeMOsLzUL8yMfNXiA6sL5QbfY0MFvDmcEas6wzW8beH+T/
IlpiZlP4PYCz1evMtiJyWD/C+Qww5yLtMf82pjOxCrMl3PmNM7uBsQcxV2aW
TpqN4k4R52/VmrUnVKsC/wc9N+5PO52ZB3YBcaXcrI38nXg/R9wpFf+9yO13
6ul7ldmrNHoBh4M4P+DE7SxyPcBpBU4DcnMps28Qpx7rCOeGcfcB1juwnoyL
0724ehnCnSfgPgqcLuyPOa3J8bLvpRJ4UVKcTiPf7cQxi5qNWLcDYxMwXgMP
f//1Ij413pCSzgOhtKAOQVKecc3+f8Wl+yJ6PBoJqxl3ZzeK3zz+J3G/WCLu
L+PylL2xPtfb4PM4aj1DrUVoMh/IownkroHnEvKrwOxE3aaY/CRPep0KVYP4
LfCuGfGwEhyRL/j8e+D1A7ced84Ar9/p7kVgDiE+gmcvuHd7L+jDsVBeEONr
QjjsL1OjHtnLlPdiBL1n05qvJpw977RHHuROX+gBtWfdFWBOoO4K6n5CrcPg
9zMmPXPeg+3A6XGapTZyjqTXPqwLXp/N8OImeGbWi/tYWvxngDkaSSfqPew1
Jzb9JBfOIr2n78Q+UuP9ALc8uK1hrw74N5zwF9DjSFp3LuH8cqDe6Qf3qe19
5N8Ems0tTnPM2sfB8QSiskz8qBP59zrNJXUZD+UTcRrhYcZJY9Znn6zBN4Df
J78Dfsd5/y1z1if9u3E7pbmh7nw/GmrVO+eUvVODq+gjn1Atfot8Lzj7nNnr
TnM76fE7vHdF/13n/G/6OJuS1tSZM7UrUP4DcFqd5mcGWj5GlODMTr432JsB
/mfc+4JYjclPzj17mYbOJ71WxbR0p+Z8P35UybeFQO8j+zoF7ncjzdg75Gcj
zc+hUDzI4R9zhsfO
             "],
             VertexColors->None], 
            PolygonBox[{{5132, 5131, 926, 1601}, {5767, 3771, 457, 1312}, {
             6304, 5113, 1601, 2550}, {6720, 3385, 1731, 2989}, {4328, 5265, 
             1724, 712}, {5266, 4628, 712, 1724}, {4992, 4695, 747, 1504}, {
             4373, 4374, 968, 926}, {5744, 5745, 1867, 300}, {4934, 4615, 712,
              1462}, {3825, 5832, 1312, 504}, {6219, 6220, 1870, 747}, {6288, 
             4196, 905, 1459}, {5513, 5512, 275, 1749}, {6262, 4180, 885, 
             2473}, {4927, 3536, 1917, 1459}, {4625, 4935, 1462, 712}, {6224, 
             6223, 247, 2467}, {5557, 5560, 1932, 299}, {6286, 6721, 2989, 
             905}, {4259, 5109, 1598, 300}, {6701, 6702, 2471, 248}, {5026, 
             4700, 1291, 1528}, {4702, 4088, 1528, 1291}, {5484, 3719, 2054, 
             1867}, {6254, 4156, 849, 2500}, {5743, 4171, 2499, 2054}, {4171, 
             6222, 2467, 2499}, {4157, 6221, 2467, 2469}, {5347, 4725, 786, 
             1780}, {4699, 4993, 1504, 747}, {4387, 5541, 1917, 1107}, {4234, 
             6315, 2469, 968}, {4117, 5487, 1870, 2435}, {6190, 6189, 747, 
             1870}, {5774, 5773, 300, 1867}, {6489, 3514, 248, 1870}, {4718, 
             5346, 1780, 786}, {6222, 4172, 2500, 2467}, {5559, 6304, 2550, 
             1932}, {4173, 6186, 2435, 2501}, {4194, 4387, 1107, 1598}, {3512,
              5485, 1867, 247}, {3526, 6521, 2502, 275}, {5558, 3547, 299, 
             885}, {5017, 3754, 1291, 1523}, {3537, 4927, 1459, 1919}, {4729, 
             5044, 1538, 786}, {6287, 4195, 905, 1731}, {4174, 6226, 2471, 
             2502}, {3395, 5301, 1749, 177}, {3536, 4385, 1107, 1917}, {4755, 
             4754, 300, 1107}, {6252, 6253, 2495, 849}, {4701, 4083, 1523, 
             1291}, {6251, 6250, 177, 2495}, {6226, 4173, 2501, 2471}, {5541, 
             4928, 1459, 1917}, {6227, 4158, 2473, 2471}, {5043, 4365, 786, 
             1538}, {5542, 5544, 1919, 905}},
             VertexColors->None], 
            PolygonBox[{{5349, 3496, 3495, 3417, 1781}, {3385, 4624, 4623, 
             5275, 1731}, {5018, 3753, 3752, 5017, 1523}, {3473, 4930, 1460, 
             4929, 3472}, {5265, 3473, 3472, 3376, 1724}, {3416, 4724, 4723, 
             5347, 1780}, {3496, 5043, 1538, 5042, 3495}, {4724, 5026, 1528, 
             5027, 4723}, {4624, 4933, 1462, 4935, 4623}, {3753, 4992, 1504, 
             4991, 3752}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.5310041334450094, 0.4705059998211116, 0.8633708633085246], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1lV+IlGUUxs/+m93ZcWZn59sZxrkpWQ1LiuhOw2m3rDZzcTQpBbtIrK5S
kgiCiCDC2DVdSwi6WnUVbaEg21pNL9sVsuyP3WQliBsF3pgV2UX1e3zeLl6+
9z3vOec95znPOd+SbTs37miPiKOsDla5ErGyGDHFfqIWsYd1a09ELhcx1BmR
oZQvR+yqeL9/IOIXziu424nsK87VroiP+iJ+Rj6H8yZ6Jex/xed15E+i9w/y
y+juqFi/2B9xIvMbW5EdYv952LbZjZ+2iGH2m4nlKvJV3OfQu4j+ILJ16Fxr
c6x/ovc1+yb7efbfsP+sFPFWzfs1nfYrnzO99q/YnsDfq/jNE89z1YjZov1u
6fG70j+Wj7ibXDZh/2wj4hnWdwXk6C4MOH7leqTDPicKPuu9Ddzt5nsD+RH8
D5LzG5xn2c+wxnhr6SJ/v+WtILa2XutL3mItJLn8bGS9jO49OZ/Ppdj+RqWl
eDuMySVyP1CzbZOaPFZ3/D9Si5fI94cu5ykb6bcT1299rpFshFmF9fpi6lK1
jnJexlsnOxzbQtIRTsMJ2/1F80PcOFi1vWwfzMwzcUzxru92zAV0F7FaOfs+
ULCvKWzPF53rfT3GUZh8j2xP3eeRTtup3mfJ9VrJdVvPfYs1BTZ3UKsVDfs9
SQyzmXktrMdSDC8Q1+aiufcOeqPdxmoQu6WsfQVzU3rSEZcku8mnDr+juvxf
S2GyD38rM+e7CvyvZM77PfCYL/n9+zPjJaz0pnimd58n911VY/QAOmtYM13W
03eeGo2Q3yOsSWz+4P5w2T0gTogbiicq7kn14yH0LmHbQj7U4z5RjZ/C7nDV
dqqraqBeU14bEjfkby5x7yLxfoz+OLENZea0+DxKLKvL1plk/0nVOAiT8YTz
+9Top5LfVyyKXfHo/mjBOppDQw3jpjk0nmbRdmRPsy6g9zD+7y3bdhjZXs6X
8+ZvK82Z09idqTmHUXxOpvr+m7dM+XSiOzFgPKfxMYbPdvrsNnwua7iP5e/L
Aft8u+Dai593cX9nw3zZjt0E9lfwfaLPuAtz9fFI4q1wld3V1Kenex3DWvgw
TWzTnF/D7mzihuzm09xQrdQD2muezZVcO82t4yXPLs2zZpo/mglfJC6trbvn
1e/CWHfCeR3yR+vmheaA5qnm5+3ktLzhXBWv8lVff1DzTFHPbyLfF+t+48OS
/wGa5+L3ufSu5rrmo2ajeLk6Z55cR/575rPmxvKCMVEe76Y5rFoKS2GoPIWD
uCQ/S1Kc3bz5UOZ/wZvEtrfmO+FQT3P7U3zurvl8ili2gPWZXs9U9alwvpGZ
r/Kvf8srJfvU/P4r75l+UDjVfVafRJrPiquZ6qK5vq3sHFVD9bn8C/ebbyI/
D48erzjHGvrVftdZ2PcXnJd6TRwVN27p97zTrDuGv+OLrSfuTCUd8VXxqO/+
A34kEmI=
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlUts1FUUxk8LfTGd6XT+nWb6j4kiICjRGFaCaa2IWpGGgTa8AgvBx0Kt
QoLRhLDQBQJCSyAxMS54VAMSwEiGUh5LWxLeKCsEElLQJQQ0ykL9fXxd3My9
/3PO951z7nfPTF790eK+6oj4gDWRdaAtor814nJVRHMm4uNixIJSxN6GiPIE
fjnvwD6lPuIePqvzEX8mEX9h+5LV2BxR4TyhNqITwMaWiIecR7GV+TYzjZjO
epnzLvCfZP8Ua4B9B9/ugLcNvlvwjZDY8cSYwqolvg/+Xuz7sXfXYc9GnCSf
Mvks4HwJvm6+naWWYk3EdmzvF13PZHzawT9IfPWkiMXwHWB/Cp9F2P4D8wLx
F1kNxP4L/7pCxHBi7gT/zcS3kk8X+VSBUWSfaXFt+nYarD1F90aYQ+yPs7aQ
W6HKvR1gPVHvnI41RcwFfyaxo/C9xv5V1iD5X+NbD3zryfEcuX1Pj44V/U37
rWCu4/w5OT7EfyHnMWLXFr1/npxGchGH4euE7wp8Q/RmIXiD4J0i/zL77pJ7
oW9ruIv3WFfBH+bbJ9g64Otlfx57BexrWXOrpo3wdeGzG9tNejaD2B+xzyP3
nWCMwv8t38qNEb/Av4lcTvBthH2p3lra32atSVOqtTdvLtW8Ar6TOft2wLcb
7Opm343uWFp6OjWXNLUP/0HWBnw3cX4DvNdL1q5yvAfWKJzLwFvO2gLXD9hj
kjWxpOCaVIs0MAv7M6m1Oo1+DnFfdQXfzVCNaznTaizVtLzNb0Z8p8HbiL0r
sa9i1Isj+N+uck968J1PzEF8vyPmOv6PU98ifF8C84+8ORUvjbxJrjebnJ/e
5HbOb+Mz1uA3MxuuOYm5pKEB7JUmvyX5vIjvfL79VuMc301do2rTneutfFj0
W5I+H8O2B/+/ic3hcxutj7X4LehNTkvtI5ve8N6Sa1ItivmK81t534VqukF9
h6i/o96a1NuVJh9pkfMuzj/nrFX1SG99Tt5a1pt/J7VGpc1f4etps0akjQoY
n1L7iax7rxpVe2fi2aQe6K18UbAW9WYeYGsft7fD8VniGaPZovhOzarUvVU/
p6aeWZpV/RnfxdKCtaI7UW26Q92datzK+YWs56Pe/HDWOQpbHPs071o9m+Sz
CqyVBWtbGu+h/ufg+zrjNyjt3gV/RsYa1mz9J/Fs0YztI7Yu7/mr+fdsak5x
CUNa1wzT7JLmldv9xLWLX704Oq4X9USzuD/r2tWDedgnFjyrKjXW9u95a1Ma
FXd2fF4rh2XE3mpxLqrpTuKZq1k7q9axryTGEsY37O83uXb1VP8lP+UcL7yL
WWMoVm9cs2ZK6rvQzPkfSKfu6g==
             "],
             VertexColors->None]}]}, 
         {RGBColor[0.5699543306930553, 0.5374202052772205, 0.909638565392054],
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFV1tslEUUnnZ7h13+7nbX3Z+Ckd6gPAAtrdLayHJRaGIMF4nG8qBGfRKE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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlVtslFUUhfd0egOd6d9O28yMLUZ6oRYTpKU1gqhYitjEqKhEoy8aL0+K
8iAiYDTGJxtoWokmRqAtabxEaluToniJiajxQaxFQqpg1Iq3oAYlPuq3XH3Y
yT7/Pnvtdfbtv+TeLZseLomIbUgp0lyMaELOoO++IGKmPmI950n0sXTE44WI
s7mI6lTE3OKIy5KIHdj/wN5YHrGX+7PZiFWVEcuQm2sizlUhZREVyHLutiPX
gjWIz1wm4uLqiHnw2pGd2BqRb9LGTGF7MWf/1RA8Q7y3wJvk3M55J7Guzvl8
kvNm4h2rjfgJLh/whv68Ywj7u0URf3P3L2QU2wQ+XejdOfvqWz/8B5ClcG+p
iFiHby8Y+/CdBXM38R+D33n4zaadi99z1pWTTvRVyBx4+ziXwWUZPkP49+E/
je0RONYTu4zzBOc3kQ9T/lbK/TSyFrwm7FPo12X8FuUvyzmDfMT9hjLnZixr
m3L0GefN1OgwtTkbjn0iZyxx6Cn4zXrrKHc28JYepJ94U+S7A24TtX7rLzRG
d11EUvRd5eMWcvFUjWOPgRfoU1XOpXKoXuktGks9cz/6fcgPXJ3lfBvxO+G0
F30azKfRx8BsIN4bcEgRuwQ5gb4GOc3bDlGP1kr3iGohDoqtmpQTfzXvGSf+
UfheBF533r3agqzH1pNzLVSTUd5zT+Je2MF7buDuxrzzu59vK7Ed4VwHt2Y4
bUD/J3EuZB/iLRt5w0Gw2y7ED27vIOfA3wTHrfBZlLi2qqdsv/KGP1O+M4B9
D9KY8rdnwRqG03a4zC92L/+YuLfV0wPkorTG3I+A+XmtZ0r1Vo9/C/Y4MdYR
eyk5vUqzUOVcCGME/sNIK3yvJMZa9OXKUdo5b+E9rUXPuu7ksa3kTgd4TZzf
A/tdZEnKHNPEXpExF3Fqw/fSomdZO2MN/tfnnathpAO9kBhLO+F1fPsK5nKa
+8ez3hnaFV3Iq9heQZ6o8AzeSW7G8Znn7tdp74aGonXtCPVSwo7oLXVPTYD3
EBz/LfFMlaDvyrr+yskM+hDxmiv9Ru2mkmrvFs3PC+DtKrr2p5Qz8rkn41yq
xw6h3wGnIvbv4XQe/5dz9ldMYX9JjM5Kx7gcvK9yxtJMf4rtGTgtAasN+3P1
3jmaLc3kFeC/X+9e0o65veCZ0axopk8Rry/v3KonXysYU1g1vH8E320F72Zx
GKlzj6m3lNOtnB9Fbk2bw0sZ97jqpRk4DNe3c+5d5e/5eveYeqsFvE+yjqlY
6rn9YB2oM3/19DT20XrXUjvqyaxnVLlXDQYT/zP0r9CMHyD+jcjP5c5ha+Kd
qV2pHB/DdrDOs6oe1GzexfmaCs/ocbiuSLwr1AN3Y5vh22/hOx8Tf3Ch3toh
XfgXE+8GzfAXxNqy0C/i0J54xjXbmpF51ZZvN6Wds+0510C51zf5TmadK2E8
gO+DRe+6//9xxD6ade7EQf8u7cyTC/P9H41a/eg=
             "],
             VertexColors->None]}]}, 
         {RGBColor[
          0.6002156891711842, 0.5860417880390154, 0.9103284311590973], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFV2lsVFUUvsN0ZmgfHV9nOnVm2FrCIih7WWQRpBYwCooBIxER+aNEUMQE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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlmlsVGUUhr9S2lKGTm/ndsrMQBUQNGCEAoUqS0QatkQhEDUSlyh/FGXV
hCK0SFV2BcUICgIGBZXNFkgEEjEmLmEJUjYlgmCMIYrBBVQiifq8vP1xknPu
Wb+z3q6Tpk+Y1iaEsAloC3TMhXB3NoSe+SE0JkLIQI+D3gr9AvQM6JnA4fYA
397NhBCXhzAA/J/iEHYi+zhwEH4O+WeTITyNzAF4J3GwAd3LFSHMahfC5A4h
fBWH8EA6hCFFIWzDxmR0pwBH2tvHE+ALUyEU5NlmE7ZWRCEUgRcCw+FH2JyD
r0r0s+Dd4VeBv8i3XuA5vi0A/wD5P/CdKbXvOmKYhv6r8HsoFvibS0JIlGET
/VPkYyr8J4HCPMc0GP93ZGx7JW8aAD4G/hZ4L0PnK1ZgH0ldhf5+7DWm/Xa9
4Q14Q9B5vdg5G4/uBGA3+ruAmyPnRLnoD5wlt+eA8gLHtJJYdyH/SJFjzpb6
TXrLbcR0BN1u2HgePFtIHsDz0R8D3g1oC/41Mrvy/a0H/GPQN4JngLeIr5r4
XiK+s8R/HN5nwI/gXeAH9FugbwAfBYwllknEdDThHulM7vYQzz3EsxHxrpFj
UiwVwEf0Qwk2xoNX8qYDsoVMGXQMrCNX64GF2PqVfJ2A30wOL8gW/I7oriFG
VMJE6GbsNSGTV+gcNYPPTDlX+tYIPp34zhHfIb6dQP6dCvs+i/15kWOQ7y35
jqUi595XTI/lnCPlRm+8iux/wEjoYUAb7C8npn3gp6nx0aRnQL3fGfiG2BdB
/0Rt95PTFnzvh+4F72FytDvtHlTvLaGmXcj9TUBDsXPQB7wj/ucjv5hvX8Su
uWod4TNZ7jfqbcrpAviHY+f6U3J0H7bvB8qha5FpIb6+2FyKrTO8fwfxHUW+
U6FncA3xdINfX+x52gb/EPw0/PPBvbc767epB1WbqMy2VaM84ukTuVajgd7C
c+6dQchcxP/e1v4Y186zfAn7NYWe6TO87W++VZPLb6nREPUe9DpibYKuBP+X
byOQvwV7/VLWkewVdP6CXxeblg318npkruW5p/tBH0+69sugjxD/Cvi9kJ9d
5NoMRGZ5sWtUBZ6NLLuk2LM+Outdopn/nbecZw4xeX2njSR/3dGZh2wV9l5J
+c16q3ysTbmH1buKSbuxWjpF3pEjydWInHujCf8D0p4pzZJkvi/3jtJuuoyN
23nrnKTzswP7U7C/lm9XCjwPDeCb4e+B/x30eGwdQObeAses3fZU1r2pHTcb
vA54Jt81UW9sx19tW/dIPUo1sX3J55vEMxB+U3D+tctOxt412mknO/hG6Dbo
zsxFf3DsWkp/Kbq9od8v8L6sgZ7F+y8k/GbNQhy5FzUTqu3qpH2pxv2hG5Ke
va3QvdHPS5nWTD5E/h4E7sz3zuqPvVzkXa1+WB15B2v3/obP16DblLt3f4Fe
Bd2JeBaBX0p4d69OeTdqh2vXDsr4FmjnLo98o3Sb9IYUumU530r1xMS0bcrW
UOqZRz3W8YY/C1yjRnibWvN1rdi3TDdYt1c37S7e11zu9yvn2m3zU8a146Q7
H52WhG18DF2bcy9liKkvvIac36qcdIqcE+VCPT8UvGfk27QKei++Bqe8S04X
+pbeGpmnm9oA/mXsXaSZvH5LkFmW8E2pzfpGq37bC7zLNFOaJe20HyL3gGqv
m1mP7lygBvnFCe/yGSn3u3Z6lfq51LKq8U7iu9q6L07g7wq8qSn3vmZgE71R
H7v3lV/t7g9LnHvt8JXozsHfxYR9rof/Xuu8qCafx9452jX6J3oU2eci3wrt
RN3SU7FvtW6qbn917F5UjMrl5oxrqZyuxffwEseqmLvCayp1brRzFYt6UL2n
mDakvaO0m+qKfFsry3wbtUMzkW+EboN2+kDk305bVv9EB4llJ/rDinwDNKuj
0t5N6pmN2DuW9C3QzOtWnS7xLdLN6kI8N+S8u1VT7e6fkR/bzv4/0T2r8Kzq
m2alc86108z8D16JW+U=
             "],
             VertexColors->None], 
            PolygonBox[{{4690, 5298, 1748, 5299, 4689}, {4604, 5263, 1722, 
             5262, 4603}, {4607, 4017, 1454, 4919, 4606}, {4016, 4607, 4606, 
             4918, 1453}, {5020, 4693, 4692, 4084, 1524}, {4923, 4604, 4603, 
             4018, 1456}, {4062, 4690, 4689, 4989, 1502}, {4693, 5022, 1525, 
             4085, 4692}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.6304770476493131, 0.6346633708008107, 0.9110182969261407], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFV2tslFUQvWW7K93tbvfRbXcXihTQlggNBUShJBp+aIyk1SqQaDRKJDyK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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlllsVVUUhnen21La3nvP6W17C0XqADWxlUGQwcThQWPEFsFKYqIgiREC
zmIUEGKiQBsIYkCmMlUJYhTEoJEXFRCMYMEBETHWUtSoCYOizBq/P38fVrL2
WcNe61/DPrWTHx/3WG4IoR3Kh46UhjAvG8JvxSFcmxPCsTiEBakQmhIh1EN3
pkPYGYXQWBACqmF0VQijoIa8EJb0CuFRbKdDHZw78DEI+xz0l6G7GZsGfPWr
DqG1dwgLoV3Iu2L7LoGyyEZW2de72PeHn4RNF+dZ0EXufyMZwiZ8HSbgbvgO
7Cux/ZlvS9F9D5tibE/g/xv55ttI4ilFZwqyaVnHtgZ5M/5WEd9gbKfhbyP+
GrAZCL8j13evzYTwbKFjWA2/EnqG8wxoLL6aso51G/Qq8S+BOsGuD/6PY78J
n28V2KdsJ/Kts5d9NADi4vIQvss3RjvL0MdfCtsN+FuFfpxy7tK/kfjPol9J
Pov5loSfj7yR3MZBBytDmFPu2Acj70Z2oNK87k8Q2yXiycO+N/4nZIyZsBpd
aOyWpSwThrPAYhj4/IG/H8lhD+e7iW8LsiT3Rfi7Gpt++JuNj0Z0Z8bWTUA3
Ra6RanOe+3fD3xtb/xA+N5LvUs6nevSr8dUHGgR/AQxnIa+PXW9h1Jg15sJ6
K1RO/Vq5YwDyH9DfA57zyakR3RTnu9D9CvlPYPs2+vvhm7DJcHczNs/j/3za
tm3QOc7r0WnG/jLn1fD/8G1ogXukP/leAQ3Ocw8Pwf/kUtfiFfxvALtb+dYO
fybPs6SeU69ppq6BT1cbq7nYTISfi81BzgegFdxXx3lpL2MwXbFTz7/Aeh85
PIH+QuQF+OpGfxLn/pzr8+zjIrHsi5zbVHKoRF4B1SF/B5u/yeVghXv9IvkV
47s7NlbqofHYnk66HkXQCM4LkP8Kf1I9Fbun1Evq6ank92S1Y9HMDwDLwyUh
3J8whtupxUbki0o8E/cg74J/OOH4tFvUw+pd7RjlJoyEjXL8F/0W5B8kXH/l
up8Ybst3ziuRDY3cS7sS3g2yka565jS5Diz3fTOKrLscDLbm2KYWf1/E7k/t
sJe0G8CwL/KnmYcWYmmtMl+X415Yi48hBe6JOSnXULUTxuu5qwb5+9RqL3f2
UyyltlV/aZZW9NyvmcoQ7+eRsVBPqvYxPiYk3AO13H1llWf/U+Rp+EroKWRp
9SC6LaXeHcLod3zXcGcT/sYUeffVVnvXagce5a5a7uyb6x4YQe7bkvatfTqM
et2QcazyoV5oijyb6ok47ZnTvHRAnchnx5Yrnyng9wh0FNlVfPsY2fLIvVxM
fB3U40yZZ1v98TLy0ZF3g2qg2WxM+20RHqrduLTxUQ3XENvyjOuhnXw7/B0Z
11475QXy/S/2btrM+Szn8ynHJsxrwK5vlXtfNfxT8UDPFdn/a8Q2qKefpK9d
tTv22yT/FXpbiOfmfNfgXNo1Fl6H8r2rZpYZS/XfqaRnSrN0AkoSy5qe/hEe
x8ivq9y1lM8d8B+We9aEid6KRdrnCb8ZnfBTIu8C7YSHqO3kau9C7aR1ALck
9l3SuQR/GWrP886vR34h7fjkU7vqlqzfGu0svaXHifmXAr+pp8BmJ/Lr6J83
sW/Hvkk+4IeDf4p41qe868ajf7jU38RrB78OXrNTth1VaOy2cEdIOF+9JWOy
3k16U1ZGxkjYqGfaIveQzsJsPvyqMmOjnVSgt6qnHvKpt2RTmfFWv1YkvQM0
+8rhSOQ3X73/IPJ1YFeFTlWue6CJXhqbcW7C7NsS7zT1qmayDv9tGb/92hHD
I+9I1Usz3wJ/Nu1e0Jukt+VF9B8o9Myc5Hx97H8hzax6WTOrWVVPa1d+X+p/
L+3MNnRXx66lcmoll0/KPHs12Hwd+Y3T26Z/gi+Rbc/630MzuBfbz2LPgnLU
v8WuMtvqH0NvY3PGtdEbOQ/d+zL+F1APfKT/wcizrBz+BzrDYDo=
             "],
             VertexColors->None], 
            PolygonBox[{{5039, 4714, 4713, 4101, 1536}, {4911, 4593, 4592, 
             4006, 1449}, {4593, 4914, 1450, 4007, 4592}, {4714, 5342, 1778, 
             5341, 4713}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.6607384061274421, 0.6832849535626059, 0.9117081626931841], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1l31s1dUZxw+W3nJLf+3v/n733t5epYY5x1K7oWYOXFjiYFrbOetwA+PL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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlmeMlVUQho8sd5cF9u69391y76UFO0UQA4IGEyKGpoKgYgKWBOlNCWgE
jP4QWBb0h0ovoUoiEiVRmomUXSBKB6WDgNhCBIL0qs/Le3+cfGe+mXlnzsyc
mdO431u9RtYIIaxi1WTNKQrh7VQIB2IhrIdeDP1CaQhtCkLYBf0S+5dZfeA3
zQthMPvN8RAa1QqhKzKpYuST/AesJzI30O8FXYaRJ+/hmw6hnDW8MISN0IuQ
nwp9AnoJ8gPBW14WQnvwOoO3gn0G/cvYfop/E5GdzDqG/ATsd4HfmTUT/lnk
p7Cvwp9dYN8LXqNECD9ynt45f7+CfzhuLNn4i/26TAgP1AnhDpiDsL8an6+A
1zw/hLbYmp4NoV3dEG5D57P/ln+nwBqBzk2wbxZ73xH+beix2Hya/TOsYVEI
O0tC+JN9Ch8WgP1ckekniNEZePXBewfb1/Idi0n8+y7fMfkGvK9TjpX0K8Gb
wuqM7kV8bM9+DmdYCf8sMe6E7nvEKx9/x4ExLmGf5Mt2fLwOfTNhWensIB4/
sZrA60s8KtCfjH/Tgn0QL4NPowst8w/YzcFbhi9rwehBvMaDd7XQNVJGLDYR
zzG1nPOr+NYEnd742wad+8HvBj0Q+k3wu7DvzMLdECHfAFvzOVNZnmNSDd5p
7LVEdiP6i7E1EX5j9peQOYCv9dAfxf4hzvMv9n7I2Ff5vAX+b+i3QL9/gWM/
OHIslYPjim3ctdIUugP22yac+9nop5KWEU8xX4w/5/m3l/1SdKZwnsoSx+JB
/h0iHgdZmA8LsHcD2etJx1MyU5H9gLUCvDWsD/FlKPLdClxzVeA/yr+LYG0A
f095CPtY1wtc42eSzoGwhJnA1yslrs138XcH+m3Rv4L+NvSPcrY1xKMh+ThN
fFYjvyqRu4tg7Ef2F1YjsIaj0xzZZhnXYlltcqJ8Rs6X8nMYujX05ZruEZvx
bWu5797k4FxfwWbrmHN+hLMdLXUslGPVyq2U74pq5hr5qAKvR8w2nmXfLXLv
uYW9QnxpgExFcMzOY//TlO+O+N2lG3kvnerINSVfhTkfe6vJaZKzpGr7Loak
9ZX/hdALWEeQ/4N/zcA+mXAuVePfo7uOVYP6itBvAr+yyHzlvz90g8i6irds
zUtYVjYr065Z1epxzjudWMxgNcP2R/yrInbV5T7bNHLSFLyRce+/AK9h5JwK
WzaU253Q22PO8eORfdiWo18BuzerJdhH8lyrS4qNpZo9ie8LIt9F5Ue9NZu0
L+qxOyPXiGpDPbMl9Isp16JqUr1jDfE6lO8e0greYyljy+feYDXG3hsx19OS
tGeGeu91zp8GLxM53suC77ZqXvi64yOy7snqxce5RBXwJ6d8NvUr9baruXpX
j1Nv7seZGgb3aPWGg5pZNd0jtkNvi1zvmkl78eVn1twC39GO+NeOMz2fZx96
Yasn61Lwv6uq56R7rTA/T7lGVW/nkK9Ie4ZpdmkmnUq4R6g2VCPn6Z/vJ3x2
9cQT+Ng9Nx/V/9S7Kkrdm9TDziVdY0tz+foP3c9StqWa7V/qHqXe1Amd0fBm
xX0fD8U8e5bz71bMM2g38rtYlOLdMx/G9ztFzo1yNAvelLRrU/N0JVgz0b8Q
88wZSnxHZp2LzfB/hTcM+lgd1/Am5OdC96jrmavZrx6vs+mMN7G1NfJbQDl4
GP2d/Dtd2zGbkLEPsq1/N4jdl8j8nuc7N6TUbwy9LdQjW5S4xuT/EGxUIVud
8mxQzmZjqx7nGVPoGbIW/GNx9z69UeRrB/7Vz7PPWeztiVyfGXK6iPgtTHiW
SEa1Ogj8HTHXbF30i7J+v+zH3oViz3jNE8VMb4PpCd89vRH6kv8+Za519fDd
3PVd5b4ruvOTwD5d7LukO7svck9XL9cbaxW6G/B/B/Ld8L8256+AvyXPM20A
+rNz81894RFi+wkyE/M8HwbC/zju3qEznFJ/y9W3MI5gu1Vu3qh+uiZdo8qf
+vGMuGvsUO7+l3C+VNq1oRhMYv96wvlTjxuP7LiUe7Fqci65m1fqXMsnzQ7Z
FJ747dC/L+Gzq6ekoV8t8uzR/D6IL5kS9ypE7r7V/o47FnqzrUL+tazfNqoZ
9YaxSedSPaI67jel3pKagdm03wjCVo0Mz7rGVduq6ST8fPwZVejz6S1xMe63
kd4U8nVt2rNEPqtXDCj1XVRM1yNbWeZYKGf/AxBvqNA=
             "],
             VertexColors->None], 
            PolygonBox[{{5142, 5840, 2140, 3828, 5141}, {5355, 5142, 5141, 
             5354, 1784}, {5838, 3581, 3580, 5839, 2140}, {3581, 4825, 1375, 
             4826, 3580}, {4767, 3551, 3550, 4768, 1333}, {3551, 5835, 2139, 
             5837, 3550}},
             VertexColors->None]}]}, 
         {RGBColor[0.690999764605571, 0.731906536324401, 0.9123980284602274], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1l3uQlWUdx59l9yyc3T33s2fPHloQBBTBnEpEqMFuk7haA9h0UXGMAgoQ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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3twVdUVxndCgiTknnvPfedSiFCilYRxbOsjtlMq1oIUOw526gNxNBYz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             "],
             VertexColors->None], 
            PolygonBox[{{5144, 4392, 4391, 5145, 1627}, {4440, 3871, 1372, 
             4821, 4439}, {3872, 4440, 4439, 4823, 1373}, {5581, 3570, 1948, 
             5583, 5580}, {4392, 3832, 1334, 4769, 4391}, {3574, 5590, 5589, 
             5595, 1953}, {3571, 5581, 5580, 5587, 1949}, {5590, 3573, 1952, 
             5592, 5589}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.7212611230836999, 0.7805281190861962, 0.9130878942272708], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmHuwlGUdx18OZw9n2T27++7tnLOgCAEKSGigXNJhdIhb+I8WFy0VsUiS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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3mQ13UZxz+77MnvvneXw0QOF1yIBDkymRojMJqMGkXNAjpQEljYRZxR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             "],
             VertexColors->None], 
            PolygonBox[{{3554, 5844, 2143, 5846, 3553}, {3607, 4842, 1388, 
             4843, 3606}, {4773, 3554, 3553, 4774, 1336}, {5880, 3607, 3606, 
             5881, 2173}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.7515224815618289, 0.8291497018479912, 0.9137777599943141], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHmQlsURxt+9v293v3O/cwHveCAKKgJWACsBiaukpNREOQREjIuICmKV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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtmHlw1dUVx2/295K85ffyliQEAbG4FAe0LtiyjKDYIB0YtVpkX2pBBBVh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             "],
             VertexColors->None], 
            PolygonBox[{{5863, 5359, 5358, 5861, 2156}, {5395, 5398, 1808, 
             5397, 5394}, {5359, 5579, 1946, 3566, 5358}, {5675, 5395, 5394, 
             3667, 2008}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.7798315502795794, 0.8527339251933471, 0.9063251688887227], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmXl01cUVx39ZX16S9/JekrckVQlQlCiVRVBrAbECVhEX6taKJixCFdFK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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtmHu41WMWx3/n2t7nnH09e+/f3meYkgld6CYZk8qMZJzJpXGJoU6daJR4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             "],
             VertexColors->None], 
            PolygonBox[{{3597, 5363, 5362, 5615, 1963}, {3698, 5416, 5415, 
             5708, 2032}, {5416, 3699, 2034, 5712, 5415}, {5363, 3596, 1962, 
             5611, 5362}},
             VertexColors->None]}]}, 
         {RGBColor[0.80700830406364, 0.8617966484300827, 0.8941500075474393], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFWWmUVNURftPTe0/36+7p1697YkS2ETwoS2I8gSCLgijLCYYx8YgwLAOM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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtmHeYlNUVxr+Znd2dLTOzMztlZzVU4QECypIYn4goKGABSVDWFFTa7gKC
gOQRBBRQLFHEBgo+tiBIpCQBjCmyGBAEKUalGDURBARFkBZLNBjze33njztz
zz33lHvuafdrPXz8VePCQRD0DQVBhP9V+SBYyQgBbygPgnNyQRDO8g9ueTQI
/gT8AmMxRC3KguDtqiDYzfh1yGvboN3O2MTe7tBsYr6ZsThqHmOg/Sdwn4og
eI+19sA9EuatPQeAF6bZA6/e8M8Cj4oFwVxw8xgPxYPgIGuXgjuCwjckg6Cc
tUuQ3wGacfD6C2srwJ1gfxdk7csEwVclQXA/59kF7TL2NLP3X+zZgKwA/Seg
zz72LAZ/O/h54JvFA/gT4EHIu5rRDvg84CUFfW9ifgdrx5n/O+r5BNZORb12
Nrabw9p/mL9XylmYNzGug/8/WOvAPI/+V4TM80rgAYxXmNezFmVeymiNmtNZ
u5a9L+Zs+3Xo90N0/5DRkXl79P8WeQF7ekA7k/0Rzh5mRJBdzDjKvBJ8b/Ad
0WEa+6/Gvvdhmyr2/xe4bcK0kneE+ca0bVOPzF8Cb0FeAnlTioMggy5fg784
5DtqwN7vVKMD8GZ47oNfGJrR4EYxhqWC4Crk3QsuBn099IMY14fsMyd1/zpj
yDb/OGMf0N0vLbUubZFxZ7l1agH/sTn70tesjePua4EXAE9Fx53oOgv5CXid
CdwfXL+cfVM2rmE+Gn0eZf4Y45B8Gxl7wvaxHPDptP1LezrBq5q1SbI3e6Lo
eihj35MNKoDHJm1b0cSAj4HvFLbNj8Lrg6xtIZ98G34rWHuDeX/kDedsI/K2
3SbGiYx1lG4rOH93aAfHHW+L4PkcuN3Im6Rrhz4P/L+0Zc2HZlEttIyZlUEw
g/EM+IHQT5dvsX8B9nm21rh6dHga/B0J8ypldOC+3metJ/NjnLkIWw2B/1PM
L0RmO/D7c45VxeNN6DKRcTa6dQLfJus70t28if635xyjJwrxobu6IWZddWet
OO+DjDHsvZmY/DO8JrBnb9QxfRa2eZL9mwP7wDZwrzO6Iv8FZH4P/ALwzeBn
gn8a+pE5x9qT0D/F/LaEz67z3Q9+DqNzQV/dxZl50+pO9macQ5Q7lCr6Qn85
Yz38+kBzGPtdgb//FV/uDP/32b+9Bj3ZrzDI8JNlDC7i7MBrsFcT8iqgvQya
FcDLU+Y9GPoa6GsZKdmKOzrCWV4Efwl7r484tt5lDClyjF2AbvFa26qGtZeh
XVfId5Jfwl2/lLIs+ax842TGviQfKWNezigDbgo51xdlnS+Ur99C1xyM5kH/
N+h/CzwUHo/AfyHwpdiib8K2eBR4JvOTeceacuhIeF0M/8Xijc06cp6mrOcT
yp37IvCYFXUO7M5Zzq+1b49GhxegnYS8NuDuAX4AeCs8xpbaR+Sbo6Rz1D46
mnnPgjytKZcczjm2lFP2cDfFGefCc5F/PvtXZ8z7AONT5W72V3Her9DhTOav
cp66CvvIdfCrxh4Pg18fcS2Tj8m3VNP6xZzjlNs+wEanmfeI21bTWfsSuC7u
+FH+3AP/R+KOrV7oOKbaMaVYehP8qGrXPMGvA58B/51J5/4m9I8AL0mav+5k
CmeZyqgvtc/diH/2r/XdPFJk3VVDRK8zhLBHmHEU2jXY50bwc8EfV+4EvxT6
ImzSnflp6PelHOOKlQbW4uzdz1pjsfNte+AD8nHgh4HXQt8l4b0lwB+j7+9k
k2L7u3Lr4Yxzu/LltirbTLaqAv9jZC1LWb78fxr0J7B5fYVz3jL2RxLmJ/1e
q7KNZVvVqGeSrlmCZfOVwH9gXAvtWO5zOPKHpT2XDUvBlSRtiyj8WqLbLeCf
BZ9VTCK/NuZ5I/afj21bsCcDfBE59CrmM/GpFL63Fh47qqyzdJ0LjwfQ5Ufy
7wrz7M3+Poy3wvbZPRnnEMWuYngh/jE9YVrd2Tx0+aTKueZl8Hdh70bkvVKg
b83ZWzGeD9xDHYL2GDx2cP6TwEeBT0N/HvTXlLlX+XvSvY56lgBe32Yc+8oB
A5lPZy0Jrhn+U5C/K2nffx75t4C7NeveQfhi5Y6sa4Pof4qscznvwgqfeb1y
H4ebjT4z0OfGhGuCasEA8GulS+F+hI+yd2LC89XIvDXhHK7cLR84B9736E6Q
9ThwGfMRafuOeJRji7KCPbbC4yl8/UnGanR9v8S90l70HRl1z6Tc3Jj8zgzf
5eiW8LsS4CT8R3PfA5h/H5s8FvKaeptuSfuyehzFyvlJ+6p8Vrlya6FfUnzW
oW+3Wtd29dhR+LeD5y549WVtatY2lS0TUfdKU7Keq2eahe+/Jv8DPwqa4TXO
YcpdL6pfAXch8CHgoey/iHlr9J0T8loZtivPubfTeduDrwQ/MWQdWmVdE1QL
3gDuhLyB8NwYOEc0Ysv3ONOnJc6Rqn13xN1LqwaG2dsya9peyG+TcE8peV25
n23o2wL6ISWukartrbPuDVTjDxFDHyJ/CXAH8AeZnyj0l1obij5D8n6bbGSU
I/+yuH1DPqVee0PSsHruNujzGfBO4FXInBGzjWQb1dgLss6pyqUHSx3LinHF
tmJ6DLI/R+eBwE9Af1PMd6a7Us0cBu/hjGjYPlPH/i3sbwt+DPt/k3TNUa1R
/rkrZp+SL+Whb6N+Nu3aLH8rwV5/TNl/xLMn9L0K/YL4b8aXPqqxbYcqn8Rc
U1VL9QbYCq4h5veCdFBvODXtWFUOUm90Ch4Hwu6RloJblnb90Zp6w1nY8GTU
PWIMe9yZ89tGPqJc+4OCvyvn9sP+/RmHI46xn8N7R9K9QndkzkZee9VrdGmp
+g2vyfCYVYjnOg5VH3ev83jYvfrThfhWzz4yaZ+QL8hGX3C+SfAbFfhOWgA3
Mdawd3KZ52vBv1TmtUbgaZzh+iKv/UzxhbwnQs6ZzTHHkGJHNXMg+F/FHd96
I3ZGnzNyrm3q4aPcTTdobi63zyu3/UQ9dIVzXBm0H2X8NpDP/kJvH+gnFvvN
d5z5x6x9FnZOTuLLc9POTeJ5L7a8j3E64hwVsLdlIV6Uo4prnJOVHxrQeQb6
lrDWWOaaM5/5uCrXesncnrONZBv1yPuxZ0Pc+eD3yDwrYZ+Vr6oHU+14NO3a
ovvsCv9BhfvRnS1Pu4aodqg/jtS4ZqpWjmCtIeU3o96KCeTfpt6sxjjVkPEJ
x5RiST4m3RfFfBadoRR7lta4dupN/mDcPZB6H/mYeiX1KOpN5LN62yyode3V
G0e9Rzv2PBRxD6JecTZrHYrdMyo2NuSdixQjJdinOOdcKBvfn/KbSG8h0axD
l3crXY+FT6s2Zv2Wl00GcZYvKt0PSaa+hZyqtu/rjdYF3p1z7m1l44no1i9r
28vHHsR2X0J/d4n7pS3oNyfj3lc5Xr5+KO/aKJ+fKn+uce5XT6zYkk/KFxVj
m6qcI5Qb9KbWW6kY+4yt8JtJtaBtxrlHPrAx7zez/Fs9rmpb54xjVfrp28CI
lO9S3wik68PsnxuxzgfBfZjytwqtHa30mXSWSeAvy7knFKw3083YKx/zW0gx
eYr9Vxb8T/cxgr17ks4F0k9vn+Wc55siv4FWAa9MORdqTb3hkJh7P9Vb9ZbP
VxlWfroc3Lq0a71q2Hrmr6TdmysG+7J/E/D+EufD8dVe03x3sXvb5ir3rupx
OwK/kzBOPfJz0C5hfBPxN4OLmPdM++x6A3fl/HU5fyuSD3ye9DcbfatRTVLu
GJZ3LVMOOZzwNyd9a9KdvZp2zCnWpKN6HdU01TL1PD3yfgPq7ac3WkvmLfLu
9aRTz5TfFHpLKKe0B9cu77eNesDxMb8h1JuoR2nAt7rWupfSGzOJL/TIujbK
n1X7NxS+B6kHWJd0T6f+TP3WZO7vmqzf4sqx89m7N+e3jXrc/wNYfaap
             "],
             VertexColors->None]}]}, 
         {RGBColor[0.8341850578477007, 0.8708593716668185, 0.881974846206156],
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmXt0VNUVxm+SSWYmZGYyITM3EzAKKgIKiBGogPgC5CG2IpWKT0AQMYLK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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtmHeU1NUVx9/usjtbnBlmmZnfzopIE8Gg0gKRBQsCUsQY3UgQIs1VOipN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             "],
             VertexColors->None]}]}, 
         {RGBColor[
          0.8613618116317613, 0.8799220949035541, 0.8697996848648726], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1mHmQVNUVxt8sTPd00930MNNvepJU2BTLLVOOImSAYdOoKCgjLqiJiiAY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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3twlOUVxt9sNmSTze5mk71k1zrlIoUBoZRQNA3XgK1IBQ0KiLVFIRRt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             "],
             VertexColors->None]}]}, 
         {RGBColor[0.888538565415822, 0.8889848181402897, 0.8576245235235892],
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmGuMVeUVhjfDXA7nnDlnzlzOmTPWNEbbmrTFgCAzVBBUNKTMjOUyNwaa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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlllsFFQUhm8X2tJhZjrTbQa3GBB8UIxCoQXZNYaILZalFVtMChExgRa0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             "],
             VertexColors->None]}]}, 
         {RGBColor[
          0.9157153191998827, 0.8980475413770255, 0.8454493621823058], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1lm1olWUYx2+Pus1zzs45m7qzZ1rQ576V7zPfCHzDNnVzBjqbmkPIjRbO
3qDAqLlUar6FhB8D+1YaQlBBFIQfLBdRH8o0sXRqVFSmpvT/9b/8cHHfz3Vf
9/Xyv17u54HN/Wv6cimlMdF4Uc3klGpFQxNTmpPpuz6lXgn82JzSxikp7a9J
aae+Gyek9NS4lJaUUspJfrv27zVIppzSVp3vEVUls0v8veL9LV3HxHuymlKP
aLQupWF9fyyZftldLz03Mn9XZPOO7HXlU7ql9baoQ/tjNV67Jb9AOlpFJ6Xn
oO6VdWco5/ObBcsUpeuMztvEf1GyL4gu6/t5rb3y6RftZ2s/R/S+9uu1tonf
LvlV8nu+1oel5+gEn7fmbO8D0QzxP6r1N/sNuruxanv3KZZzjSk9Jh3NTVpD
5/3iP1I1f4l486vWNU/rXNEJ7QuSrxV9WGvMXi8bt+FiSksV21nxX9X3oHT0
ye42nT+nuz/XOX7wBk/kb0t+VPLzIp6HxN8s3pa879ZIf23mb+z8FTnqKlhu
t+705L1H/jvl4ZUG47JOtFD0qPijuvegYvuk1mfEB27L8/aZXBxWcb2mmhrI
mb9MdLHOdbRS+8GcZZcHv6h8Xm/2GflcFnrOydazivdSnXnLg0+tUS/U1Fu6
N1BJ6ch4y4ENtYqNS7HnbCj8ob6P6/5a6fle+ndQB8lY4wu1fQ8LsH2pJaVN
Zd/LTzC+YIv+U/ru0X6LdLQoj6elY0Q6V5acM/KDj+gpKcYDmfO5V+tsyWwh
f8rdIVGf6GDsyWPPONfRIt0dKVoOzN7Q3eMtzvfSwA/fiQ3d5G6T/PlcmDwu
HRvUx//q7j71U3vB2NGvz1SMHdiQ287Ib73iuFt0TlsjZuJdXXC/Es+VSeq9
nHthsOq5AG5gfKvOvuAfdYlv9OsXDa5ZYhmOGMFoKPBpVVxdJWNP3PQzsZ8K
f8G5I2zjb7ty8lVmHj1HP3OPuLdWHfsscBZt1f6M1s1V6+wWJmvKjpl5wXz6
31bB+olrTsl5IkfwxsLuLPFrpzo/5Hx65H00Zg92vxWuN6a6VtYr1i7RAunu
m2h/0XNVMmPNrmv6izP42NkRe3AaibyDw906Y9EffDAEp7aoE2btl5l9eTvq
CZl7tUVfjkjmQNW6rmluXWi2PfD4vdGYFJSvgXpjzhvQGjVPDR0Ne79K9mxm
v6mLY5HHT4X/zKrr5nyzse6MXhyMPIIpeNCD3Nsdc4xaHIganlty/sgd85V5
yqxhPvXEvOqWz/slsy3n+X6kxj3XKf434i8quJ8ORn+9rL7J1xuHPzPXHHpO
R7+BIVh0Ro4WFpw/fL0q2bHMfXBF6+XM/XRB63nRioLn+KqYh7wVvBn4TP7R
Qyx/SHa47Lm7VP5sr7jumI3gw7zqFn6fVZxbZtXKgt/hwZhL4INP6AVD3hLw
Jk/7pH9tOWZ2wRhgl3nMfADfNzPPEXp0RdhGPziSG3TyroMJs5E3f3H0OMRZ
W+RxXcyNnsARPljzfjOrf1PuOsqWwWfusGdOIDMQ8xj72OUfhJnLvCWfe+Kd
q1Of3Sm6DokLXeipTPYso3b4H8ma7B/Y9IbOaU2e9fj2rnS/0+L+oI9nlux7
r+r0WrNtgiN1Ty/vCT+oW/6Jnqj6jDxtC/1F6S81Od+9YRvM6Vt6FRyoC+oD
/3+SLxcz55UZtrrsuQAefdGn7A9HP3ydud/oNfI/GP2youT3hu8TDe4HfKJ3
f8jsM/9Ap6K2eLPxkfn2dMX9gI1D8b6MRr0iQw3vVKy7qn5j2lv8v8iMv97o
2UGv9EdvESNzgfkAbjujlsFnxhTPROZhR/gxI97x4cgv/0GT4l+IN2N71N7R
qGt0/jPV/yXzY64ujNq+Kf3TS8aYtyvfZP+nlXwGf3HZfuDDf+E8YNM=
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwllElsTmEUhk9L29v2Hzr5b2+xsLYzldasCVG0qClKdaAqWkG0ph0xBUmN
iYilaWeKRIJERMSCtFiaYyoShJQO4jl5Fyf3fOe835nueb9RDZsWtaWb2Wpk
KJJImcWRsei3s8yWJcx+Rnxzzfakma0JzeqQFehHM80ulJhtzjM7NcTsRMxs
Cr6pyPXAbAGYQWyvioUdQ8yv6Llxs3r009h2gZ1Pjpsk/8SdFmKdBPM5Gxs5
VxaZ1SJHwFZR6AtqeYm8DxSjB3059+9wfyH4b9xdD74dbGWO9N2hYrutjXMf
mBp854iZTa9BSrXdot8c9NyUdJ9BBvEzkUbwDUgluU5wPkCsdvqbQOwy5Brx
y7HNwn8t32wd+kb8D9C/kG8Od/9SXyn+I9xfir8R/xb6rWaGBzKYBzPMwpcs
pE5TzhHg/wwzmwx+PvhLYC8i3dTX735qHY5UgX+EbWESHzHuotdwZ5DcA8gh
9GXgO5h9Xlx6yMxKwU5EmvE3Eb+NXjojxe7KUq7LyNmYcraGwrjPc9aQ7xn4
afRWwXkG9V6h5ybitRDvFb43yFz827BtJXcsrn9TQP4GYjWGqn266d9vYCY9
2dqBAfKeJd6ZodoB35Vj4P8F2pl3kXJ47EpydOL7zLka/WC6ev8SaTd8BsdD
YdznMbajr6OHD4F2KmKWE8hRn6YesvgX2/H3BfpHHeDbQ/X2Dtt9aq3l/Bg9
Rj0V3H0bqZZLnFeF2mHfXcf08i+HJ/Qvy7BN9lrobxy938BfRa7VoWJV4x+P
PiMpbA62HaF2zHfLay7nPDOpu845n1UptquBZjYx1I76brptL/6RCXHTd+oN
83mNLEFvpb8un11Su9PGeRJ37+Upt++41zq7RNz0ms+X6A1w7vuOHEqKM84V
fzN60Q9jO5cujh73XUiIO/uwZTDfRFz6fvLlxGVz3d+UJvKvRboDvTn9xeKU
c8k5PJ5YxSnNyv9ZM7mai/S2fAzEpbxCccs59Zv8vyK9ZV7TPO7vDIXtSNO/
9jfC3wb/558icca54jv1MF93HOtvwFHnT0Jcds4/4fw40i57zaPxxVJ6S52T
ryNxwHffOeG7/q1As/ed/4H+vUBc8n/i3FpCTdNzxbHnnCclxFW39TCPbaF2
0d+YuqR20HfPOb2Y89NI/9LfnP+ZP8y+
             "],
             VertexColors->None]}]}, 
         {RGBColor[0.941176, 0.906538, 0.834043], EdgeForm[None], 
          GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFlDFPVFEQhQfELL63+5BoeIhuYe0PYAEthEQNWOxbEgstIKtoohGibMzq
X1g0MdjZK9qa2GisTUw0iC36B6xcpRAXPV9miMXkzr577sw5c8/dk82VxnK/
md1THFD8ScwKrYuKK31mE6nZtPJvo2a9stm49r8eM9tWXNBeQ4crA2ZnhVlV
Pqz8ls4NjpgdUrwpmX0U9pOilvr+rGq0hL0+ZNbO/dzWoO9T77OwhfZO63da
Mfs+6mfm1H9buCVh2sovlZ1fqj6J4m3J61MbDnXV2FStqdTxk1pnxLOq/Lfy
m32OQyuaV/X9rqJujp0KPD3o1RWHn4nnfC/6Xd/lPtdArYZ5kFMTDdRilhcz
s/fDZq8HXAd64LAx4Oc5VxPmkTjvJM4TDHrPqcaHkuf7PMCjeb7fOcCL7+yj
Yzp0tcqurSnMrzgL/xtHFUM+LzRyjjNnUsftz4qe9ajVjb6cQQezq2j2meJd
yTn+SLxXWd9OZa490z0ePGK2YO6b4dDcDb/R94l0zwn/V98eKh/PnAf6qAEG
/zHzInzHivfA0B8cs2Ve8GaO51PnjO6d4F/EzDiLF/A93HaVr8e9bInD/JDP
485hs+djvrcgzy4qNtWnl/hZ5kAf7pL64J5GnS+DzrURPmdlPhuq92LMcXCo
hvfgwG/4LKnPtdxrPMjdQ8yd/d3wcC1x7cwALuxRB/3cB73oD98iML3Qi6Zn
oWtWtdeleS/xO+iFLta9wE/GW8Iv3DN6qN8p+zynAzMRnsd3+I85o7sVPizi
vwaeL8d8FszhqjQ2c6/LG8JncGhGLfBr4Wdq8huvF6EXXD3e8lq858fStJy7
L47LkydGfC7sdeKdwJ35kfOO2vHGX+m9zmR+r6z3c8/xclLxmaCpET7kPdaS
/2+yEzzpfzs4dIIb33nf1fDJWvCB1z/gILSg
             "],
             VertexColors->None], 
            PolygonBox[{{5639, 5643, 1980, 346}, {5641, 5639, 346, 1979}, {
             3325, 5640, 1979, 2615}, {3343, 6548, 2815, 96}, {5658, 5987, 
             2259, 2802}, {3921, 6608, 2874, 2876}, {3943, 6010, 1206, 628}, {
             3627, 6610, 2876, 347}, {5660, 5658, 2802, 1995}, {3659, 6546, 
             2813, 2815}, {3442, 3339, 2634, 1041}, {4846, 6611, 2877, 67}, {
             6822, 6826, 3227, 3219}, {5985, 5989, 2260, 594}, {3639, 6540, 
             2804, 2805}, {6620, 5659, 1995, 2886}, {5988, 4293, 1037, 
             2260}, {6009, 3942, 627, 1192}, {6609, 4846, 67, 2875}, {6542, 
             3934, 2887, 2806}, {5213, 6367, 2615, 593}, {4293, 5983, 2257, 
             1037}, {5984, 5213, 593, 2257}, {3338, 6784, 3067, 95}, {3934, 
             4531, 1192, 2887}, {3923, 6541, 2805, 2877}, {5642, 3326, 2616, 
             1980}, {5646, 5644, 347, 1982}, {3326, 5645, 1982, 2616}, {6544, 
             6545, 2269, 362}, {6608, 3920, 2875, 2874}, {6540, 3640, 2806, 
             2804}, {6825, 6544, 362, 3227}, {6621, 3442, 1041, 2888}, {6546, 
             3658, 2814, 2813}, {6380, 5223, 628, 2634}, {6547, 3342, 95, 
             2814}, {5997, 6620, 2886, 2269}, {5986, 6606, 2872, 2259}, {6607,
              5985, 594, 2872}, {4552, 3343, 96, 1206}, {6008, 6622, 2888, 
             627}, {6784, 4209, 3219, 3067}},
             VertexColors->None]}]}}, {{}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{5989, 5985, 6607, 6606, 5986, 5987, 5658, 5660, 5659, 
             6620, 5997, 5998, 6545, 6544, 6825, 6824, 6826, 6822, 6823, 4209,
              6784, 3338, 4869, 3342, 6547, 3658, 6546, 3659, 6548, 3343, 
             4552, 3660, 6010, 3943, 5223, 6380, 3339, 3442, 6621, 6622, 6008,
              3942, 6009, 3651, 4531, 3934, 6542, 3640, 6540, 3639, 6541, 
             3923, 6611, 4846, 6609, 3920, 6608, 3921, 6610, 3627, 5644, 5646,
              5645, 3326, 5642, 5643, 5639, 5641, 5640, 3325, 6367, 5213, 
             5984, 5983, 4293, 5988, 5989}],
            RowBox[{"-", "5.2`"}]],
           Annotation[#, -5.2, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{5946, 5942, 5944, 5943, 4508, 3324, 5212, 5211, 5981, 
             5980, 4292, 5982, 3919, 5976, 3918, 5656, 3637, 5383, 3638, 5657,
              3933, 4529, 3649, 4866, 3650, 6006, 6007, 6005, 6378, 6379, 
             4550, 6395, 6394, 5230, 3960, 4871, 4870, 6736, 4210, 6296, 4211,
              4880, 4879, 6407, 6406, 6405, 3978, 6642, 3671, 6459, 6644, 
             6645, 3979, 6646, 3980, 6647, 3672, 6634, 6635, 6396, 6397, 3344,
              3447, 6632, 6633, 6033, 6035, 6034, 3661, 6011, 6012, 5224, 
             6381, 3340, 3443, 6624, 6625, 6623, 6627, 6626, 3652, 3935, 6783,
              3334, 6456, 6455, 6454, 3641, 6613, 3925, 6612, 3924, 4848, 
             4849, 3922, 5214, 6368, 6369, 4844, 4845, 4202, 6293, 4201, 6727,
              4489, 5948, 3906, 5947, 5950, 5949, 3614, 4830, 3613, 4484, 
             6360, 5624, 3612, 5369, 3611, 5623, 6359, 5172, 3905, 5941, 4290,
              5945, 5946}],
            RowBox[{"-", "5.8500000000000005`"}]],
           Annotation[#, -5.8500000000000005`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{5603, 5368, 5903, 6451, 3431, 5904, 4451, 4452, 3880, 
             5176, 5177, 3301, 4485, 3316, 4486, 4487, 4832, 4831, 3615, 5953,
              3908, 5951, 3907, 5952, 4490, 4491, 4203, 5201, 5202, 5203, 
             4847, 3327, 5648, 5649, 5647, 3628, 5650, 6370, 5651, 3629, 5376,
              3630, 5652, 6372, 6733, 4207, 6371, 3926, 6734, 6735, 6373, 
             6374, 3328, 4853, 3335, 6377, 6376, 4208, 4858, 4857, 4859, 4532,
              6386, 6385, 6384, 3945, 6628, 6014, 5387, 5668, 5667, 6013, 
             3444, 6383, 6382, 5225, 3944, 4554, 4553, 3347, 6552, 6553, 6549,
              6551, 6550, 3346, 6399, 6637, 6636, 6038, 5390, 3662, 5671, 
             6037, 6036, 3448, 3345, 6398, 5685, 3673, 5403, 5405, 6650, 6651,
              6648, 3981, 6649, 6556, 6557, 6555, 6559, 6558, 6654, 4594, 
             4595, 4013, 6101, 4012, 6656, 3468, 3367, 5730, 5731, 5437, 5438,
              6659, 4028, 6657, 4027, 6658, 4646, 3390, 4942, 4943, 4214, 
             6742, 6744, 6743, 4941, 6738, 6739, 6737, 6741, 6740, 4011, 6298,
              4213, 6297, 4212, 5715, 3702, 5422, 3701, 5714, 3995, 5699, 
             3687, 6554, 5698, 6065, 6064, 6643, 4881, 6063, 3975, 6062, 3976,
              4560, 4559, 5233, 5232, 5231, 3961, 6032, 3657, 4877, 4878, 
             4549, 4548, 4868, 4530, 6001, 6002, 5220, 3941, 4526, 4525, 3333,
              3440, 5384, 5382, 5655, 5654, 3323, 3438, 6605, 3915, 5975, 
             3916, 4507, 4506, 5939, 3903, 5931, 3904, 5940, 4289, 5938, 3902,
              5170, 5199, 5200, 4482, 4483, 3878, 5173, 6352, 6353, 3430, 
             5900, 5602, 5603}],
            RowBox[{"-", "6.5`"}]],
           Annotation[#, -6.5, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           TooltipBox[LineBox[CompressedData["
1:eJwl1ElIlmEUBeDXsgGc0kzTCtSMpGg0LNOwwbCBBqJRK2wwwv4SyiaDZqKR
ihY2QVGLFkGLoDZRi1pksw00qEGbsFVBFjTX89HieL577rnnvd/wm7u8bs66
uBBCgj/pmSEMTgmhJD2EUmhPDGF/aghf8fdeIfyAn/ScZOz6Te8QfuFc9W96
XsTqNvq9rBDuQ6vr7dkhtOD++n/5JjpnEkx21h/+fHocLqft412bpOf6mLOn
qqfBFIj1CGE6rjD3WvZC/BIvwK/w5zQ74RaoNX+ad6zso+oS3C8jhGreOr1r
sq/DGvUOdS1+z7fR2V/s2gG1do3Rd+rPNv+IP47nI2yjf8IBzur3sP8sngJn
nInOhgZZe+zw1N4L4KH5B5DWM4Stes20BjmNvKfgJHSX11tvl7kiWcXq0ThF
fqX8vrhYPQau8q+wZ050jl13w1l5b+Xuo+XCXL55MMJcIRyhFUCS3JX0vTLL
6BOi9ybvlvlWfBMP9DyP639QP1e342c43m6NMvrIyIZLzn1Bn8xfDnm0XKiX
f978enwOb8A59GlmY55BVnSvkAlb7Fwpt1nOIrxZ3YueAeP48+UW4hNQatcS
2CNzmcxEnuHqYTAE6un37HofmqJ3IXMC3yp6nn4NHq8+QB+F56uzoufjeiQc
NXPXezqID0HM/W3iPey6gve23ipaDdTTNsD46Nu3Yxm+rL+Y/yIepf4W7Y4v
qGfQuzqjykwl3HGfX/QH6Z/Snymzw+8t5j6bou+WfxBfKR6Ks/kS8AB1Ie8j
7/8x1PC3ymmDqug75Xvr+ob8JP5keOcen/Au0e+kf0W/M16qLpJVjePV1+ld
cKqZNBitN9B5q/W7Jf//P/EPY7GMZA==
             "]],
            RowBox[{"-", "7.15`"}]],
           Annotation[#, -7.15, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           TooltipBox[LineBox[CompressedData["
1:eJwl1GtozmEYx/Eb29gchx0xzZCzOSVzTBlyKIfIodbGxrDNG8Y2p+SVY5FD
ORZiFkLRHEIU4QXijZAwRtgURZLPPy++z+++f9d1X9f1v5/n/2QWlM0sbRJC
GO4jpm0IQxJDqGkfwhkstH7bKoTXSSH0TwvhDf3SMYRX9CUepobwABm8uJQQ
YlHbLoR4moA/ySG0oM2xSa3vak2Tm43+vH64KT+bDkJPjDbDQNrU2WYR+sTS
GMzjH1OnWesQ8tQ4Yj1f/n01yngxOMkrEMtHjXW++Hl6LjrXQT96Cy2tEzCp
TQg71C2XV+LML8+Trddsfqre6digbgrG8Mcik5eFLbyumMabjr68ftjK64kW
7nAwTXR3u9Wu1SOBF48pek9FlvVOvZZa76LvzVKH4/JP4CTGy+mqRgZu2Nep
81XOY/qNfqS3+LcxRb98+TPpWjNVYA2qUIlZ/EXii/HZuTvOZKnbHQfFYtND
qOc3qPuJNkZ3bq56+ty+Rn5nuV2wX/53d/+Mf4p/GtXopfZL+Uv1K8a46K7U
KLQ+K3duNJP9YH45nWN/iP/AmUfRs5vhIc3wXRThuj4HeIXWlc7k6rlKnzT+
RiS5u2RM4K8Vr8A1foH8HLUv2F/EamfKURHVwVbeZs9xT26j8/fpdt42jNQv
B1XyRqqxwJyJ/NHWOc6Mop3kP/VbOmbW9eY/SjfQJ+aoFGuQV2V/iL+OHqEz
1NlMy+yXm+WH+j+xWn6d/BL+JvFSekXty1hpn6lfN2Rghf1yvJB/WHxJdNco
wjzniukkuk/dUrPspSV0qufZI3+ieK54Lp0c5Ubvnrne+t56RO+DnNlm++l9
3Wa9HQPkd5Z3V887KOWl2y8z9xZ106z/OtPojo7aN9BUXrH4MPkp0f+BeD3/
I5KjmcWGiiVZ/xaL1/uDWG+9cuz70A5icfxas11Fkb6FeGe2/OhdR3N5eTSO
XpIzwrqa5qlf3e7//9o/3Q+l0A==
             "]],
            RowBox[{"-", "7.800000000000001`"}]],
           Annotation[#, -7.800000000000001, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           TooltipBox[LineBox[CompressedData["
1:eJwl1F1MlnUYx/G/gIAGlPgWtgViGLYKcb1sLhe40iWpwXTrQJAEY8tX8CRd
lmKrRQLLSnQLp1jzhVCY0yNINhYiOrcUnXqAQSBl1MosD3Kjz70Ovs/vvn7X
9b+u676f+3lmrtlUtHFcCKHIR+fUEEanhPDjtBAGEPNwCPMmhfBzcggtqSEc
fSSEY4jlz+f/yj/Nb+Z9izbXrciTy8dEdSf5mXrOQgXvdlIIQ+YM4w9eYgrP
9eVHQ3gnOiPu1OMhul58hT8qnyT+S30y/V3cx79HnzbjsekhpPAHnWs3rwPP
8AfElXpUIVf8svN5GLVDPe8+/YzuwbDaITypz1L9lmEJXkcBRuQupIVwEYXi
lViBHPX/yHWbeQ4j9orxXNI9vwxMsONEVPISkcWbjSReMj7Uo9Ju1XQzvWSX
Ij2/Fu8UN9Ed9E9z33P+ihm7xAf41fQbetWZbe5rK/pcFzs/eXIIq+hp+TM4
hXr115y/irvyb9LL6rfb+Qc6nbfAnJeQMsP+9qtHHXrN/tR97sY+55rkD2Mv
9qEBj6vbqW4GqsXp4nftVGfuC+I3nHueLqdvmVGK1WhDK0rkcjEPa8RlKMd1
e93AYufK7TpXPgcfm5OJe3b/G8/JD6GONwdve84VWGiPV/AqUvVrsNMtz6Mf
ezyjDXp/Tue4t+t23SReFH2n6r5Hc/TO8B/o3ajvbD2y0MP/jX9XnxPOr3Xm
Jq8LQW2CHRPsW2TuDnt8gHjxYT0WOT/edaHc+/ztiBXHIcG9xKPFHi36jove
KdzQ96no/XO+Vu412sk7i0b5r3BA/SdyNciPfotY5zqLDtrzJxxSf8neM3kZ
+FL8Be7Y+RcsUV+tT5/a4+qK7XaMFtIt/Gw7V9EC8X7+oF3LxRnOldF+8Uf8
XPmV4inqc1w/i1o9u+QT1VZF77t5W2gHrx0T3FOZXS7yL+CB/RbocUSulN/L
68F9fi6/xvM4KNfv3sebE4cYBLPGov8Yv/F1enarOxd9b+a/aF42TXM+jj4R
7ar3eT16Ua6+S20m/199ZtExGtTfilScrr4z+k70L1F/k//d1P//Q/8Dy8m/
3g==
             "]],
            RowBox[{"-", "8.450000000000001`"}]],
           Annotation[#, -8.450000000000001, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{3329, 4509, 3321, 5209, 5208, 5971, 5970, 3893, 5967, 
             5968, 5205, 6365, 3320, 4499, 3310, 5188, 5187, 3892, 5920, 3603,
              4469, 3887, 5182, 3888, 5917, 3600, 4808, 3599, 4463, 6357, 
             3306, 5621, 6449, 5622, 5365, 5619, 5618, 3305, 5613, 6447, 5615,
              5362, 5611, 5610, 3304, 4459, 4461, 3854, 4457, 3853, 4798, 
             4421, 5864, 3424, 6446, 5863, 5359, 5579, 5578, 5858, 3849, 4270,
              4269, 4793, 3565, 4786, 4787, 4413, 5153, 4407, 3562, 4783, 
             3561, 4405, 6338, 5571, 3559, 5569, 3558, 5570, 6337, 4400, 3557,
              4777, 3556, 5851, 3838, 5567, 3837, 4263, 4262, 5854, 3839, 
             5843, 5853, 5852, 4442, 3298, 5167, 5146, 5886, 5885, 4281, 5890,
              5891, 5877, 5889, 5888, 4480, 3311, 5195, 5164, 3894, 5927, 
             4286, 4287, 6358, 5633, 3897, 5929, 3624, 4851, 3625, 4504, 3913,
              5974, 6537, 3632, 6539, 3634, 5653, 3931, 4516, 3644, 4865, 
             3646, 4522, 3938, 4538, 4864, 4863, 3653, 4873, 4301, 4303, 6391,
              6392, 5674, 5675, 5395, 5398, 3454, 3350, 4570, 5249, 5248, 
             4575, 3991, 4577, 4576, 6087, 5706, 5708, 5415, 5712, 5711, 6099,
              5724, 5726, 5432, 5728, 5727, 6105, 6106, 4639, 3725, 4963, 
             3726, 6116, 4047, 5282, 5283, 4670, 3740, 6124, 4068, 5306, 6123,
              6122, 4675, 6747, 6748, 6299, 6303, 6302, 4071, 6143, 6144, 
             6140, 6142, 6141, 4706, 5036}],
            RowBox[{"-", "9.1`"}]],
           Annotation[#, -9.1, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{3348, 6402, 5234, 6055, 6054, 4308, 6058, 3972, 6051, 
             6057, 6056, 4568, 3356, 5244, 5243, 6075, 6074, 4312, 4892, 3678,
              4566, 4889, 4888, 3458, 4311, 3985, 6068, 3984, 6071, 4579, 
             3363, 6718, 6719, 6272, 4188, 6274, 4185, 6271, 4187, 6270, 4186,
              4255, 4254, 5768, 3772, 5469, 5471, 5472, 3811, 6761, 4228, 
             6760, 4227, 6776, 6777, 6716, 6717, 5128, 4226, 5137, 5138, 5127,
              5126, 4376, 5801, 3507, 6791, 5800, 5475, 6566, 3773, 6800, 
             5510, 3523, 6513, 6813, 4150, 6700, 6487, 6488, 3509, 5082, 3508,
              5083, 5084, 4229, 5129, 4369, 4370, 4230, 6312, 6311, 5474, 
             4149, 6206, 4015, 6102, 5738, 5739, 5443, 5741, 5740, 6112, 4650,
              5294, 5293, 5270, 4055, 4339, 3481, 3391, 5295, 4680, 3746, 
             5007, 4349, 6134, 6135, 6131, 6133, 6132, 4710, 3412, 5339, 5319,
              4097, 6154, 4360, 6158, 6159, 6157, 6667, 6666}],
            RowBox[{"-", "9.1`"}]],
           Annotation[#, -9.1, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{3322, 5210, 4503, 3914, 4502, 4501, 4840, 3896, 4479, 
             3608, 4842, 3607, 5880, 5882, 5165, 6344, 3297, 3427, 6594, 3868,
              5876, 3870, 5879, 3583, 5849, 3835, 5147, 6334, 6335, 3421, 
             6591, 6592, 5842, 5845, 5844, 3554, 4773, 3555, 4396, 5148, 5149,
              4398, 4399, 3841, 4401, 4776, 4775}],
            RowBox[{"-", "9.75`"}]],
           Annotation[#, -9.75, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{3413, 3492, 6664, 6665, 6152, 6156, 6155, 3759, 6136, 
             6137, 5324, 6439, 6440, 3486, 4350, 4078, 4679, 5321, 5322, 4681,
              4980, 4981, 4654, 3733, 4974, 3732, 4651, 4032, 5742, 4033, 
             6113, 4331, 4944, 3718, 4620, 5260, 6175, 4114, 5473, 4113, 6501,
              6814, 4170, 6707, 6257, 6711, 6712, 6514, 3524, 6515, 6516, 
             6414, 6415, 3370, 6708, 5096, 5097, 4251, 5099, 5098, 4168, 5095,
              5501, 5500, 4169, 6247, 3688, 5470, 3689, 5700, 3541, 5810, 
             3812, 5520, 5521, 3531, 6275, 5056, 5057, 4899, 3998, 6273, 5252,
              4580, 3690, 4890, 3691, 4581, 5253, 3364, 4584, 4891, 3987, 
             4567, 3988, 4313, 3459, 6652, 3986, 6072, 6077, 6076, 3679, 6059,
              6060, 5236, 6403, 3349, 3453, 4884}],
            RowBox[{"-", "9.75`"}]],
           Annotation[#, -9.75, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{3939, 4517, 3940, 4521, 4520, 4862, 4537, 4539, 3957, 
             4876, 3654, 4543, 3956, 6022, 6025, 6024, 3667, 5394, 5397, 5399,
              6404, 4571, 3681, 4895, 4315, 4316, 6410, 5702, 6561, 3696, 
             5414, 5417, 5418, 6411, 5721, 6564, 3712, 5431, 5433, 3474, 3377,
              4634, 4635, 4043, 4336, 4042, 4961, 4642, 4969, 4970, 4669, 
             3739, 4996, 3741, 6125, 4069, 5307, 5308, 5309, 4703, 4704, 4093,
              5335, 4092, 6138, 5006, 5035, 4095}],
            RowBox[{"-", "9.75`"}]],
           Annotation[#, -9.75, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{5152, 4404, 3845, 4408, 4406, 3290, 4411, 4412, 3848, 
             4792, 3564, 4415, 6339, 5575, 5860, 5859, 3566, 5358, 5861, 5862,
              3851, 4422, 3569, 4799, 4272, 4273, 3852, 5588, 3572, 5361, 
             5865, 5866, 3855, 5596, 3575, 5364, 5868, 6448, 3425, 5869, 4427,
              4429, 3860, 4276, 5157, 5158, 4464, 3307, 5181, 4467, 3601, 
             4815, 3602, 5919, 3891, 5186, 3890, 5918, 4474, 4475, 4200, 5192,
              5193, 5194, 4838, 5207, 5206}],
            RowBox[{"-", "9.75`"}]],
           Annotation[#, -9.75, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{3313, 5196, 4478, 3898, 4843, 3606, 5881, 4477, 3871, 
             4440, 3872, 4822, 4280, 5878, 3869, 5140, 5162, 5163, 4441, 5847,
              3833, 5834, 3834, 5848, 4261, 4770, 3832, 4392, 5144, 4395, 
             5846, 3553, 4774, 3836, 4397, 4772, 4771}],
            RowBox[{"-", "10.4`"}]],
           Annotation[#, -10.4, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{3341, 4542, 6631, 6021, 6023, 5673, 5677, 5676, 6061, 
             3973, 4569, 3680, 4897, 3682, 4573, 3992, 6085, 6560, 3695, 6562,
              3697, 5705, 4008, 6097, 6563, 3711, 6565, 3713, 6104, 4025, 
             5434, 6416, 4636, 3723, 4962, 4335, 4337, 5278, 3388, 4665, 4666,
              4065, 4346, 4064, 4994, 4671, 5003, 5004, 4249, 5749, 5750, 
             3755, 6324, 3756, 6798, 6799, 6797, 5751, 5336, 6441, 3408}],
            RowBox[{"-", "10.4`"}]],
           Annotation[#, -10.4, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4099, 6153, 4100, 6165, 4361, 6162, 4098, 5323, 5340, 
             3414, 4711, 5013, 5014, 4685, 3748, 5008, 3747, 4982, 4059, 4684,
              5296, 4341, 4340, 4977, 4978, 4653, 4652, 3381, 3476, 6209, 
             4332, 4031, 4619, 5272, 3380, 6207, 6212, 5762, 6817, 5763, 5761,
              3766, 5764, 6502, 5504, 3791, 5793, 4118, 5792, 3790, 5502, 
             5503, 6713, 4178, 6517, 6518, 5100, 4177, 6773, 4237, 6770, 6772,
              6771, 3371, 5814, 5815, 5812, 3813, 5811, 3814, 4750, 4749, 
             3532, 5522, 5523, 5412, 6671, 6462, 6463, 4898, 4582, 6276, 4583,
              4903, 4902, 4318, 4319, 3999, 4587, 4000, 4901, 3692, 4893, 
             3693, 4585, 5256, 3365, 4589, 6079, 6080, 6073, 3990, 6081, 4314,
              6078, 3989, 5235, 5245}],
            RowBox[{"-", "10.4`"}]],
           Annotation[#, -10.4, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4791, 4414, 3291, 5573, 5574, 5572, 5577, 5576, 6341, 
             6340, 4418, 3567, 4797, 3568, 4420, 6342, 5584, 3570, 5581, 3571,
              5586, 6343, 5593, 3573, 5590, 3574, 5870, 3857, 5867, 3856, 
             4428, 3577, 4807, 4274, 4275, 4805, 4804, 4435, 4436, 3863, 4279,
              5160, 5161, 4468, 3309, 5185, 4471, 3604, 4819, 3605, 5923, 
             5924, 5191, 5922, 5921}],
            RowBox[{"-", "10.4`"}]],
           Annotation[#, -10.4, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{3288, 4394, 3287, 5145, 4391, 4769, 3552, 4767, 3551, 
             5835, 5836, 5833, 6169, 6788, 6168, 3420, 6444, 6443, 6332, 5355,
              5142, 5840, 5838, 3581, 4825, 3582, 4823, 4439, 4821, 4820, 
             4476, 4841}],
            RowBox[{"-", "11.05`"}]],
           Annotation[#, -11.05, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{3358, 4572, 6653, 6084, 6086, 5701, 5704, 5703, 6655, 
             6096, 6098, 5720, 5723, 5722, 6103, 4632, 4633, 4967, 4966, 3724,
              4638, 4044, 4663, 4045, 4965, 3737, 4995, 4345, 4347, 5303, 
             3400, 6200, 4131, 6328, 4253, 6326, 4252, 6327, 6330, 6329, 4090,
              5033, 4250, 6325, 6749, 4218, 5117, 4217, 5034, 6139, 4094}],
            RowBox[{"-", "11.05`"}]],
           Annotation[#, -11.05, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{3415, 3493, 6668, 6669, 6160, 6164, 6163, 3760, 5015, 
             4081, 4715, 5325, 4352, 4351, 5010, 5011, 4683, 4682, 3392, 3482,
              4976, 4057, 4660, 4058, 4657, 4655, 4947, 6211, 4152, 6210, 
             4153, 4946, 4945, 5477, 5476, 6208, 4151, 5087, 5479, 3510, 4735,
              5482, 5481, 5480, 3780, 6584, 3804, 6582, 3802, 6581, 3803, 
             6583, 3779, 5478, 5802, 5803, 3801, 5804, 4377, 4378, 4189, 5524,
              5525, 5526, 5086, 6763, 6764, 6762, 6766, 6765, 3815, 6585, 
             3816, 5813, 3775, 5413, 3774, 5769, 3372, 6278, 4190, 6277, 6280,
              6279, 4001, 6283, 5254, 5255, 3464, 4900, 4586, 4588, 4908, 
             4907, 4320, 4321, 4002, 4591, 4003, 4906, 3694, 6082, 6083, 5247,
              6409, 3357, 3460, 4894}],
            RowBox[{"-", "11.05`"}]],
           Annotation[#, -11.05, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4796, 4419, 3292, 5582, 5583, 5580, 5587, 5585, 3293, 
             5591, 5592, 5589, 5595, 5594, 3294, 4423, 4424, 4803, 4802, 3576,
              4426, 5156, 4432, 3859, 4806, 3579, 4814, 4277, 4278, 4813, 
             4812, 5064, 4127, 4816, 3866, 4470, 3867, 4473, 5367, 4472, 5190,
              5189}],
            RowBox[{"-", "11.05`"}]],
           Annotation[#, -11.05, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{3407, 3490, 5456, 5455, 4219, 5116, 5115, 4091, 6662, 
             6323, 4245, 6322, 4244, 5463, 5462, 4998, 4067, 5069, 4066, 4668,
              3738, 4999, 5000, 4664, 4662, 4964, 4637, 5280, 5279}],
            RowBox[{"-", "11.700000000000001`"}]],
           Annotation[#, -11.700000000000001`, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{3830, 4393, 3831, 4768, 3550, 5837, 3827, 6589, 3826, 
             6170, 6171, 4761, 4111, 4765, 6173, 6172, 5354, 5141, 3828, 5839,
              3580, 4826, 3873, 4438, 5166}],
            RowBox[{"-", "11.700000000000001`"}]],
           Annotation[#, -11.700000000000001`, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{3858, 4801, 4425, 3295, 4430, 4431, 4811, 4810, 3578, 
             4434, 5159, 5872, 3862, 6593, 5871, 4220, 6307, 5460, 5461, 4817,
              6475, 4818, 4112, 5366, 6199, 6805, 3426, 6450}],
            RowBox[{"-", "11.700000000000001`"}]],
           Annotation[#, -11.700000000000001`, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4102, 6161, 6167, 6166, 4362, 5038, 5039, 4714, 5342, 
             4712, 3404, 3487, 5009, 4079, 4691, 4080, 4687, 4686, 4986, 4987,
              4661, 3736, 4979, 3734, 4656, 4034, 5091, 4035, 4948, 5059, 
             5058, 5088, 5089, 4232, 5135, 4231, 5130, 5090, 3525, 5511, 4379,
              3542, 5136, 4256, 4257, 6465, 6464, 6811, 3777, 6567, 3778, 
             5770, 3776, 5085, 4371, 4372, 4746, 4745, 4598, 4599, 4382, 3543,
              4741, 4258, 6281, 6282, 5528, 6532, 3533, 4904, 5532, 5531, 
             5529, 4191, 5261, 3373, 4601, 4602, 4004, 4605, 5257, 5258, 3465,
              4905, 4590, 4911, 4593, 4914, 4913, 4322, 6094, 6095, 5246, 
             5259}],
            RowBox[{"-", "11.700000000000001`"}]],
           Annotation[#, -11.700000000000001`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{3286, 4390, 3285, 6308, 5139, 4136, 4760, 6204, 4759, 
             5055, 4764, 6196, 4766, 3829, 4763, 5143, 4762, 4437, 4824}],
            RowBox[{"-", "12.35`"}]],
           Annotation[#, -12.35, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{3500, 5067, 4128, 6752, 4223, 6751, 4222, 5065, 4126, 
             6678, 6679, 6476, 6477, 5061, 4125, 5118, 4221, 5063, 5062, 3865,
              5874, 3864, 6815, 5875, 5873, 3296, 4433, 4809, 3861}],
            RowBox[{"-", "12.35`"}]],
           Annotation[#, -12.35, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4005, 4910, 3466, 4323, 4007, 4592, 4006, 4912, 4609, 
             3374, 5263, 4604, 4923, 3707, 4909, 3706, 4920, 4017, 4607, 4016,
              6177, 6178, 5530, 4115, 6179, 6176, 4192, 5534, 5533, 5527, 
             3817, 4381, 5816, 4380, 5429, 5428, 6258, 3705, 4600, 5505, 3518,
              4742, 6504, 6503, 6578, 3792, 6580, 6579, 6802, 6801, 6510, 
             5556, 6821, 3545, 6535, 3544, 6534, 6248, 6507, 6506, 6509, 6508,
              6505, 6769, 6768, 4236, 6181, 6673, 6674, 6180, 5446, 5447, 
             3382, 5273, 4659, 3735, 4983, 4342, 4344, 5298, 4690, 4062, 4988,
              3749, 5012, 3751, 5022, 4693, 5020, 5019, 4716, 5343, 5341, 
             4713, 4101, 4363, 3494, 5345, 5344}],
            RowBox[{"-", "12.35`"}]],
           Annotation[#, -12.35, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{5060, 5075, 4141, 6689, 4142, 6686, 6688, 6687, 5074, 
             4133, 6753, 6754, 6478, 6480, 6479, 6680, 4132, 6681, 6201, 6203,
              6202, 4130, 5070, 4129, 5068, 4997, 4667, 5305, 5304}],
            RowBox[{"-", "12.35`"}]],
           Annotation[#, -12.35, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{3375, 4611, 4613, 4916, 4915, 3709, 4610, 5262, 4603, 
             4018, 4924, 4325, 4926, 3708, 4919, 4606, 4918, 4917, 6467, 6466,
              6672, 5430, 5536, 5535, 3534, 4751, 4752, 3819, 5817, 3818, 
             5818, 5823, 5822, 3383, 6774, 4238, 6320, 4239, 5102, 5101, 6520,
              6519, 5794, 3794, 5779, 3793, 6193, 4120, 6191, 4119, 6192, 
             4121, 6194, 3795, 5781, 3796, 5795, 3767, 5133, 5134, 4737, 6767,
              4736, 6225, 6213, 3393, 5297, 4658, 4060, 4343, 6214, 3483, 
             3394, 5299, 4689, 4989, 3750, 5016, 4353, 5021, 4085, 4692, 4084,
              4717, 3761, 5040, 5041, 4721, 4720, 5047}],
            RowBox[{"-", "13"}]],
           Annotation[#, -13, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{3501, 5072, 3502, 5464, 5071, 4134, 5125, 4135, 5078, 
             5079, 5124, 5123, 4144, 5077, 6482, 6481, 4247, 6781, 4145, 6331,
              5467, 5468, 5073, 5120}],
            RowBox[{"-", "13"}]],
           Annotation[#, -13, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{3506, 5066, 6486, 6485, 6484, 4147, 4368, 3505, 5076, 
             4143, 5121, 5122, 4243, 6778, 4242, 4140, 6779, 6780, 6685, 6684,
              6683, 4246, 5465, 3503, 5119, 5112}],
            RowBox[{"-", "13"}]],
           Annotation[#, -13, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4224, 6309, 4225, 6310, 4137, 6205, 4139, 6682, 4138, 
             6803, 6472, 6473, 6197, 4122, 6195, 4123, 6198, 6590, 5841, 6724,
              6725, 6726, 6723, 6333}],
            RowBox[{"-", "13"}]],
           Annotation[#, -13, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{3499, 6474, 5459, 6306, 6305, 4124, 6693, 6694, 6690, 
             6692, 6691, 3504, 6696, 6697, 6695, 6699, 6698, 4146, 6758, 6759,
              6755, 6757, 6756}],
            RowBox[{"-", "13.65`"}]],
           Annotation[#, -13.65, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4105, 4722, 5049, 5048, 4364, 5045, 3762, 4719, 5328, 
             3405, 3488, 4354, 4082, 4688, 5326, 5327, 4694, 4990, 6216, 6217,
              6215, 4155, 6218, 4061, 4985, 5486, 6228, 4154, 5092, 5489, 
             5490, 4738, 4739, 3784, 6570, 3783, 6812, 5807, 3807, 5780, 3781,
              5778, 3782, 5806, 3806, 5805, 3805, 4383, 3546, 6284, 4193, 
             5539, 5540, 3535, 5094, 6313, 5820, 5821, 5537, 6533, 5538, 5777,
              6568, 6569, 5775, 5776, 5772, 5771, 3384, 5107, 5108, 4925, 
             4019, 4608, 4020, 4326, 6285, 3471, 4921, 4922, 4614, 3710, 4936,
              4327, 4931, 4932, 4612, 5264}],
            RowBox[{"-", "13.65`"}]],
           Annotation[#, -13.65, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{3824, 5831, 3825, 5832, 3770, 5767, 3771, 5466, 6483}],
            RowBox[{"-", "14.3`"}]],
           Annotation[#, -14.3, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4022, 4930, 3473, 5265, 4328, 4023, 4615, 4934, 4933, 
             4624, 3385, 6720, 6721, 6286, 4196, 6288, 4021, 4928, 5541, 4387,
              4194, 5109, 4259, 5773, 5774, 5483, 5485, 3512, 5819, 3511, 
             6223, 6224, 6221, 4157, 6315, 4234, 6314, 4233, 4374, 4373, 5131,
              5132, 5114, 5113, 6304, 5559, 5560, 5557, 3547, 5558, 4179, 
             6261, 4180, 6262, 4158, 6227, 6702, 6701, 3513, 4984, 3514, 6489,
              5488, 6220, 6219, 4063, 4695, 4992, 3753, 5018, 4083, 4701, 
             4700, 5026, 4724, 3416, 5346, 4718, 4103, 4365, 5043, 3496, 5349,
              5348}],
            RowBox[{"-", "14.3`"}]],
           Annotation[#, -14.3, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{3417, 3495, 5042, 5044, 4729, 4104, 4725, 5347, 4723, 
             5027, 4088, 4702, 3754, 5017, 3752, 4991, 4993, 4699, 5302, 6189,
              6190, 5487, 4117, 6188, 6186, 4173, 6226, 4174, 6804, 6255, 
             6521, 3526, 5512, 5513, 5300, 5301, 3395, 6256, 6250, 6251, 6249,
              6253, 6252, 4156, 6254, 4172, 6222, 4171, 5743, 3719, 5484, 
             5745, 5744, 3548, 4754, 4755, 4386, 6289, 4385, 3536, 4927, 3537,
              5544, 5542, 4195, 6287, 5274, 5275, 4623, 4935, 4625, 4024, 
             4628, 5266, 3376, 3472, 4929}],
            RowBox[{"-", "14.950000000000001`"}]],
           Annotation[#, -14.950000000000001`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{3386, 5277, 4627, 4955, 3721, 4937, 3720, 4952, 4037, 
             4630, 4036, 6183, 6184, 5543, 4116, 6185, 6182, 4197, 6290, 6292,
              6291, 3821, 4757, 3820, 4753, 4384, 5449, 5448, 6263, 4181, 
             4622, 4621, 3520, 5093, 3519, 6511, 5506, 5509, 5508, 3522, 5562,
              5563, 5561, 6260, 6259, 6512, 3521, 6794, 6795, 6793, 5507, 
             5451, 6187, 5453, 5454, 3396, 4696, 5330, 4698, 5024, 5023, 4355,
              4357, 5333, 4728, 4089, 5028, 3763, 5046, 3765, 5054, 4731, 
             5052, 5051}],
            RowBox[{"-", "15.600000000000001`"}]],
           Annotation[#, -15.600000000000001`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4109, 4730, 4110, 5053, 4366, 5050, 3764, 5029, 4727, 
             5334, 3406, 3489, 6233, 4356, 4086, 4697, 5329, 6231, 5332, 6230,
              6229, 6239, 5765, 5766, 5452, 3768, 5799, 3800, 5790, 3799, 
             5798, 6471, 5796, 3797, 5787, 3798, 5797, 6523, 6524, 5103, 5104,
              4241, 6321, 4240, 6775, 3397, 5825, 5826, 5824, 5828, 5827, 
             3822, 5830, 5545, 5546, 4756, 3538, 5547, 6796, 5548, 6676, 6677,
              6675, 6468, 6469, 4949, 4950, 4629, 4951, 3722, 4960, 4333, 
             4956, 4039, 4626, 5276}],
            RowBox[{"-", "16.25`"}]],
           Annotation[#, -16.25, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4038, 4954, 3478, 6419, 4334, 4040, 4631, 4958, 4957, 
             5110, 3399, 5782, 5784, 5450, 5785, 6573, 6572, 5549, 5829, 6588,
              3823, 5552, 5553, 3540, 4758, 3539, 5550, 5551, 5565, 6536, 
             5564, 3549, 5808, 3808, 5514, 6522, 5789, 3787, 5786, 3786, 5788,
              3809, 5809, 6574, 3788, 6575, 3789, 5791, 3769, 4740, 6494, 
             5494, 6241, 6240, 4159, 5331, 4160, 6232, 4087, 6238, 4161, 6234,
              6236, 6235, 5031, 4733, 3418, 5350, 4726, 4107, 4367, 3498, 
             5353, 5352}],
            RowBox[{"-", "16.900000000000002`"}]],
           Annotation[#, -16.900000000000002`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{3419, 3497, 4106, 6832, 4108, 4734, 5351, 4732, 5030, 
             6237, 5032, 4162, 6831, 6492, 6493, 5025, 6828, 5491, 4164, 5493,
              5495, 5496, 6265, 6264, 4388, 4389, 3810, 6528, 6529, 6530, 
             4744, 3527, 4743, 6527, 6526, 6525, 4182, 4375, 3515, 6318, 6319,
              6317, 6316, 5492, 4163, 4235, 6792, 6827, 6586, 6587, 6491, 
             6571, 6490, 6830, 3785, 4260, 5783, 3398, 5111, 4198, 4959, 4199,
              6722, 4041, 6829, 6420, 3387, 3477, 4953}],
            RowBox[{"-", "17.55`"}]],
           Annotation[#, -17.55, "Tooltip"]& ]}, {}, {}}}],
      AspectRatio->1,
      BaseStyle->{FontFamily -> "Times"},
      DisplayFunction:>Identity,
      Frame->True,
      FrameLabel->{
        FormBox["\"Number of buys\"", TraditionalForm], 
        FormBox["\"Number of sells\"", TraditionalForm], None, None},
      PlotRange->{{0, 30}, {0, 30}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {192., -189.}, ImageScaled[{0.5, 0.5}], {360., 360.}],
     InsetBox[
     Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJxM3XVUlVnfh3FUTDBAxEYsxAZEREVCMBCxQREEu7uwuxMdY+zuxK6xO8bu
7i5sVHxnrXdf3/v56yw+ax7FE9dzzn32/u2CLbs1aJPaxsamorONTZr/br/e
O3W0yMTl/tyOWN2u2smyC/wDr6W5MXL24wD889pWGwM7rfXPs3ltuZGzP8kL
5y2e8W6Gnf5OfRoODGn1Wx617dWb+CZH/NPnH/JwxOw0gXiXCh03Pa193n/U
0AajWw3OIK9XuuM5h9Y3/FM9cise0iqzPOJqI8/PdR75Bywd3HZ6EQd5SJEh
t570f+U/+HK7XSNmZ5fP8ej1/mP3JP+9tg0y9crkLI+rXmJ4jQPf/X+U94tp
NTiX9ff+GvOyZMof/wrt3DY2/JBHHpr3/YhOb9IE9Pk7m01Iq/zyFrbFl295
kT5g66nk+t7XCshfpv3a4PoKu4D+g5PqTy9SUJ62nmvWsAtZAnLNej4gU2wh
eeK3MI/dk7MF7Nx4Z9mI2YXlh36f7V22mENA5ImLZ5MvFJEXnnfJN9nNIeDL
/eNfemZykzvPDgu1mZYtYMb3vS6vg4vJ8/2cNfLbpywB5RwSa7Qa7C6/HD2q
24L09gGXiq/sfntHcXnjOf4PWm7KENCj6rw5DT+UkO+c2mJQqYC0AdmiEw6f
KV7K+j0P1Jve3CZ1wKZeo18HtyotnxRcNtfDBin+dSYNcNo3v4z88dO2X55u
/eH/dnm3Kt7XyspLdPAJbeLz1f94rFe5a2M95D8SG75/duKTv/Pr9TWmF/GU
T6mWZ8k9vyT/tvHFYuocsvzbsLKX6sZ89N+RZmn3TLFe8pQHfwose/XBP11C
vtHHky1PM2WjW6vED/6R+WbPGTG7nPV8+7g2+fPID/4rVzts9Pf2lq8KvlZ9
cdAH/6/ekw4nX7C856kb/h7r3vtXP5Tu+o4u5eWHK5w/OLjTO/9Z4cNf98zk
I9+VK9fHIYfe+D+7+fNPmVWWHyy6ON2lna/8fdr2dXodXEFeYU3FJgVbvvAf
k/TBfdUDy/eHPX8bdPOp/7Uhnaq0GuwrP7+m88UbFR77u9k9q18gT0X5wx35
2i6e/cC/7+zmbW/vsHyGS/z1HS53/I8Xvj1gdsNK1vO/aw2bpJbX/Z03R0xt
+MHyf4532XfE+bJ/W78Ly7JOrixPqBsd023mv/47Ttbadaa4n3xixZUL59md
9E8Xcezs2OOW59xRYPDRyYf9Ix8GPAxuVUV+MXakz7Pc+/wXdtnz5c8fy7Mf
eNdsx6Nt6hR9okv8zG2LTa0qvny5U73C9w+Y8HDDzaPqFd5p6dzd10ZcVK/w
Wt0m5MgScUe9wiOjr779kv2ZeoVfPu8wbdj0D+oVXuyta1Lxb9/UKzzHw1GF
qiXaBNArvFLclfaj26YLoFf49ylBx0Z9sQ+gV/i+oNnzD2dwVK/wLPl6Lgld
nkO9wmem/XV5bbrc6hWenBCXJ+RJXvUKz1cxbNM/3/KrV/iVKdcDpmYuoF7p
z/kRlFjmP6dX+MWrsWXW/86vXuEBx87+Pdk2n3qF91u+JG2JuNzqFb58zrJt
rk2c1St87rcbu1tUzq5e4VOuVp3aLCybeoU7j3f6trOnXQC9wveG/hnyMjh9
AL3CM+Sya+Wc1zaAXuHn3//wWvs8VQC9wo8H7khwHmETQK/wxiu6hC3v8ke9
wn2SC/W7sShFvcJ37CyUqpxvinqFx03ueuzT09/qFe7U5uqBe11+q1d42T7T
/4St/aVe6XkyZGXU63E/1St8w7bn4Q3TJ6tX+K+olVVcy35Xr3CHPwG2BRy/
qlf486gt+08d/6Re4ccc2k70b/NRvcILj4mvOSXjO/UKb5PqXOTPXi/VK9xx
+8nowiWfqld4D6ct62vXeKhe4TFudTo92nhbvcIPtCr849yRq+oVPqzV2AVh
Zy6oV/iyR8/nL+p2Sr3C896rlW/x88PqFR3i/RL/HU6X+JnbeU3On39R+pJ6
hSd0DvjuNv6eeoX/dfXxntdbX6hXuE3Xu77vAj6pV/jdJVleZX72S73CrzfN
967yU9sAeqXf50nh5G15M6tX+PTavfK/GptdvcJ/N4n8fjkgt3qFnxvXYv/G
ngXUK7xTyoM3U+8VUq/w2XaF93y+WFS9wotdGJjyV3139Qp3+fXq5zfXEuoV
vmnNrwZe2UuqV3jFsrYr7jqWVK/wQtX+9C3kVkK9wkNcu+Tf1cVdvcJX+wzK
lqeAm3qFJyZ9aNlkYGH1Cnf3THVj9VFX9QpfsiWLd1KHfOoV/mZ/zLbh73Kq
V3gd9yGd7tR0Uq/kZTp/WzrWQb3CM9aqO+SQb1b1Cn/36kS85yl79Qqf+3nl
sp1t7QLoFf7Z5eXmkK2ZAugV3tPhcb3HTTIF0Ct871a7xJIOmQLoFb7dpki3
fqszBtArvPzEnyM//84QQK/wIlVuhKxOlSGAXuEjP3oU75+YLoBe4a6lXbxO
5kgbQK/wzK03BOTxTxNAr/Dj9w9lyeCfKoBe4QP6nnLxjEhRr/BpCwpnLueR
rF7hzW16D07t+VW9wl89GXVuZ+8k9Qo/cGdCqmTXd+qV/r0x4b0OFXqhXuEj
xmYIvfTjkXqFd/uQbeWBmnfVK3z5or/2li99Tb3S37u7lEv8wAvqFb3hc5z+
f8U475f43+N0iZ+5jbkQZe+74qV6hT/reiyx4IHP6hV++Vh8rR0lbALoFX4p
f+OhVaplCKBX+LnFIZ8nTHJQr/A3U+4fr90zl3qFb4tzOleinat6hfstu75i
iXdR9Qpv9CL7kyYdSqhX+OkzsfXvJ5ZVr/Chkz+/jGropV7htvO+ZRnf2Vu9
wu+Uau+y42N59Qr/fbXg5SL/+KhX+PR+v3/f2eejXuFFMiYvyfS1vHqFr263
Zk+xUd7qFR63IG3aAjO81Ct8/MdJDjNjPNQrfPu1FJ9aJUqpV3jJiltXvL5T
TL2SR9usqpS7iHqFLx2fPzpnxoLqFe47L2DxzB751Sv8bJ4bI1r65lGv8C79
I9esKpVLvcKvnJ++93XqnOoV3rV7ARePzs7qFe58amjqEFdn9Qof2HRvs3RP
cqhXuFuUY7Ock3KoV/i8hELZn/xyUq/wDu1ytkpT3Em90t/byfni5izZ1Svc
5fT+KV+2OahXeP80gQ9uumdTr/BPCz1arvjvcwG9wk9tP5CUrX/GAHqFd9/y
vl22lLQB9Ar33Hrr7r9LUwfQK3zXpfqr+v5JUa/wyjG+8RXCf6hX+KDxcyL+
DfmsXuH3mm99bnv/vXqFF64X/njJ7BfqFd7Up/eXBYseqVf4u7Zr7O/89/6N
XtEVri/RK5zPcfQK5/0Sfy5Ol/iZ2yOnZk2dc9ImgF7h9zqW7V/NPZN6hdc5
V8Sn1dLs6hXutrFbtUOt86lX+Po45zfXjhdWr/BUYz2m55hQQr3Cv11tFL2q
mKd6pd//fc6CZ5qVV6/wWfdGVPuyrKJ6hW+dvSZfx5VV1Ct889ELn4+PDlSv
8CW3otvMPxikXun3SezfvEybquoV/rjkvfafWldVr/DWfWsm5DkapF7hxfOP
LnHv70D1So/vq7TP3bP6q1f43gVZjj3uV0m90v3crlvKk5M+6hXukLn6pcfh
5dQrfGI/Z//gqh7qFd78W/axJ6aWVK/wpwtvJXa6WEy9wtO1OeZ2vnxR9Qo/
lbdp+xxTCqtXeL2UXhEeCwqpV/j2XL3W+RYupF7h/867U+rk04LqFR7um29K
xu0F1Sv9ORkuezzrUlC9wtfmi5meNdlVvcL7OX4LXl/HVb3CY3NX+D7viot6
hTe5WSVT2Ip86hVeZNbrOwOK5FGv8L4ZI++f7pxTvcIzxaXOfXauk3qFl3n5
0t7ptIN6hUfty7Ymu1sW9Qo/1L207SPHjAH0CvezW/2+YbxtAL3C7T0XLXVr
YxNAr/Dr9rP27zjxQ73Cw2ISOvZ88km9wjsuXOf06r/3b/QKj0/lM6NP62fq
Ff3guje9wrm+RK9wPsfRK5z3S/x9OF3iZ24n5X9xPUdLJ/UKfzH6he3KOS7q
FX6wSL8Nnq5u6hWeOW3R+n0ulVGv8OLLj+Sqs9FbvcLXVSrh0WFfJfUKP9+p
0dcmKwPVK3zMr8BfP5KD1Sv9+aMiL17PU0O9wsd/3jzF5WCoeoV331Xsz8m1
YeoVvqNClujgdbXVK3zbfd95T9fWVq/w0EbdP3lvDlOv8CsT0vxz9W6oeoV/
Lt5pRIfONdQr/IR/7bRXF4eoV3jJ0uVzeB8KUq9wr+Cpjfbm8Fev8EcbZm5d
GllRvdL3Kiur7XrVv7x6hVfu+aC+w1Qv9Qr//erT75NjPNQr3DFyz828/cuo
V/j1TfGz5qQqrV7hi6+dK9g7uJR6hfdp4hK4+nFJ9Qq3nTK59NrZJdUrfK3b
/tOn3UuqV/iF0JQa1XuXUK/wCQl/Hy7Xobh6hbdpGn27cVp39QpvHrg1MKG+
m3qF370z/XhobBH1Cm80407rqoGF1Cv8UIzbpb2ZXdUrPHrrp3XtduVVr/Cf
aSa3jx6cU73Cj7e889ZlfHb1Cu/as1VrrydZ1St1oFeN476TMqpX+L7MMedT
O9sG0Cv8cGSZXfX/e/9Gr/A0H3vuiuj/Tb3Cf2RY2bJuz4/qFZ3g+zh6hXPd
m17hXF+iVzif4/T+xjjvl/g9cLrEz9w2vVWnS8HvbuoVPifqjcumeR7qFe7Y
e/rRY4UqqFf4xm+NSznaB6hX+N8b0zjMdAhRr/AvVS+8uZ5SU73S778lYfjN
lHD1Cq+2wWX8pcb11Ss8aVase7lGDdUrfN6FLe2HHW2kXuEDGo06nmZ0hHqF
f0x5PWHvqAj1Cnd2cJs96WQj9Qq/MP/d2viODdUr/O2tr59SZtRXr/A9twrb
b/lWR73C9xYa9SlsVS31Ct/s+O7iwxnV1St8U7NjZw7ZBqtXuN+ZHp8nuwSq
V/jWcvN9zt+urF7hi5e6+a3411e9wleW6DHp3D0f9Qqvcun6uztHy6tX+PU0
a5yz+5RXr/BK+wZdLvDOW73CG9j/+ZVlu7d6hbdZ9+rilRbe6hVet/qNpCa3
yqlX+IOU4YdfOpRTr/DlFSrdupnOS73C/9m/v8SFAx7qlV4X20fnDZxeRr3C
Lw6ru3Jb65LqFX5k7tV1bXK5q1f45LwX7sw4V0S9wkd6bZx1ZnFB9QqfWjuv
Y3iD/OoVPj96QqHX73KqV/i43kOflS2VXb3CV6W7X+5o0SzqFd538+aEYsHp
A+iV7rcvZa4dX5AqgF7hPes2qj67bLJ6RQ9YJ6DvmYzzfRy9wrnuTa9wri/R
K5zPcfQK5/0Svx9Ol/iZ2w7jduzsbuurXuENN4ZnH7UnUL3Cd4+0dXjbs7p6
hTd1O+48OW24eoVfzfZ1yT+ODdQrPF3+sCpVN0eoV7jtpaa1hzlFqVd4p+oh
vdd9ilav8NmN5xSoliFWvcIdJiWmXu0Sp17hU9tWSqn8n9Mr3PfJ1sNOWWLV
K3zV9TZrzmaPUa/wTb3eTr7TOUq9wp8/3Jts3zpSvcJvlx/s169aQ/VKz5Pe
QX+2NqynXul+6+z0o8awMPUKP7R0aseHdWqoV3hc22d/pysSol7hjQ4cnpnv
fZB6hQ8v1yf9iHGB6hWeo13WDO79A9Qr/Hvj9EMnr/VXr/A7E8p861TVX73C
v/XoUPDW+yrqFf6kSIESg+OrqFd48N6bE/7e6qde4b+8veMvTKusXuG3Sn77
uSNrJfUKH3O7cOOXPr7qFe5RaOAV39w+6hU+rO9iZ7dz5dQrvEmJtQuvdfRU
r/CHHhVe+GQvo17hR18fGFViUHH1CnetWvTyR8+i6hX+OO/Bt7dqF1SvdL8t
qJL31M686hWe/3zT4edO51Cv8MYd2v1OvJBNvcIH2P1qeC4+o3qFX+rn5bvt
bWqtl+J1z/oleoWzToBe4XwfR69wrnvTK5zrS/QK53McvcJ5v8TvjdMlfub2
zKuQObvCa6hX+OdXu9Nuq1ZXvdK6rUexNa/6NFKv8P4lrob5lY5Sr/CK7ULs
R/nEqlf40t4jL9n0a6Fe4ZeTLj3P0q+VeoUPSkq70vZ1a/UKv5XpUNqX69uo
V7h9rp5j5v7n9Aq/5l/2Tbr3rdUrvO2r/Y5xU1qpV/hahw9fe21voV7hM27u
DN/lEade4YuGZUxd7kZT9Qqv3D7k79z7I9Urfe/R7/W6isUaqlf43K6HLg6t
XE+9wk8WXFmpZuba6hVuU2D3p1LJNdUrvMvOP/MHpKuhXulx7FW/R8irauqV
fs9yhStWbVpNvcKdA3e2XOVQTb3S89au+I7cN0LUK9xz19JqVUeFqFd4tcFF
W9/4Gaxe4bHN9hV09ApWr/APXZJaVSlaVb3CE/bmKGN3J1C9wuc7bH537Ji/
eoU/Kl/h7r3pldUr/E7dvidtgnzVK3zg9WWTln3wVq/wmgdtxw7b76le4auj
jy7cMbK0eqXn7eTxboE53dUrvPqtCS5DaxdWr/Ao30Tn6U1c1Cv8wasQl+iO
OdUrPZ+XZVndd6+DeoW/flYhx67oTOoVr2/WVWrdjHHWL+l7X+OsE6BXON/H
6Xoun8/MdW96hXN9SZ9DjPM5jl7hvF/i34PTJX7m9uTq3gcX/2ykXuEP7sXf
aeYarV5pHYTTvg+znjRXr3DbxQe811RrrV7hk8blG+DZo516hT9p9Dkg0rWj
eqX7s2JYbCHHzuoVPutbqcl1nbuoV/jR3OV7P8jRRb3C52SYmqdnzs7qFX6o
0ty3xf07qle4zZEDbwcdaade4ZttS7Wt/7a1eoX/lZx61KnULdUr/ISXXYOQ
UbHqFZ5h4e03dluj1Cu8V/jSsCEnItQrfEamywmRFxuoV/jZszOvOByqp17h
fj3rnK/7uI56ha/M9XTWpjbh6hUelJJvSMDy2uoVfvt34SMbq9ZWr/BBay5/
iPsSpl7htXe+rXlrVJh6hWf7x6PljXO11Cv8cJZ+aTwOhKpX+Iv8tTPOiayp
XuEucxOS7BZUV6/wilP+qlVyWYh6hUf2Dy6VPLiqeqX7Oce0TGlDAtUr3W/j
3pQs4uynXuH7vkztf+psBfUKL7XJM1eZM+XUK7yc5+1EW3cP9QrPcSApYv6j
4uoVnv9dyZh3UUXUKzx4at+DhdoXUK/wzomJuXaczKle4ZmzRDjW2eCgXvE6
Zr03vcJZV0mvcNYv0SucdQL0Cuf7OF1HM851b3qFc32JXuF8jqNXWrdu3i/x
78TpEj9z63o83adT41qoV/jYNWnXHLnWRr3Cy9zKFJCvfEf1Cs97dv43vzpd
1Sv8Qot7p6s/7q5e4VNubnmasWhP9Qo/nOpF7XLneqpXeMZ3raY9O9tTvcLX
LnN/0sG9p3qF7+8dlrrP1+7qFb6za0JI411d1Ss8ZrOjTf/MndQrPCQxqlK1
mu3UK7xOBdv8XrtbqVd4g1RrPd/Pa65e4RXev072d4tRr/R7Pnnlf69+E/UK
v1PcOVXiygj1Ch8wYPKXm58bqlf6/qTKo9c/fjdQr/DVrf1fnxjWQL3S75PX
6+ecoAbqFV4g3QHXaVkbqFe44xi375eO11ev8JTNNx3/qldfvcIvnn/UeteS
euoVfn/Xo+9Te9RVr/Ch9jU25KoQrl7htZIzTx67ppZ6hR9/5tUz1ZMa6hUe
V+/PxlVpqqlXuGeT0fPmFamqXuENyxQdNOyAv3qFlx+X6DdwekX1Sq+XDim9
B9z3Vq9wj8Rns2bu8lCv8HFtEgq9qFVCvVJPTtr02hBaRL3Ct/Zw6+W3x0W9
wkdXqjyjwzVn9YrXK/tQ6BXOem96hbOukl7hrF+iVzjrBOgVzvdx9Arnuje9
wrm+RK9wPsfRK60DNe+X9Hw1Tpf4mVu/K38lBJfrpF7hD468zpi9QXf1Cnec
5zS8/f2e6hUeknhv2uwMfdQrfEjWDS8bzumrXuHLR+SZcaBjvHqFj/49+2bj
/5xe4f1a9ArqtqCveoW72Dja13fto17hy9L0eeZfsJd6hZdonDnf0Bvd1Sv8
wv44xw0pndUrfMLTO3cPTW2vXunvnVHHfcbG1uoVXqXT57yhY1qoV7jnxZlL
89aIVa/w2S1uzLn4qal6hd8f/v5NnutN1Cv852qf67WXNVavcLcCNr3ypm+s
XuG7ZrUfX2lXpHqFT05ZHPZv/0j1Cv87IvNYjwKR6hX+NrJc6V8TI9QrPFOR
xJXNtjRSr/DIN367BiQ0VK/wx5fvzhhTuoF6ha+r6X/53qx66hXefuU1x3N7
wtUrfHHGctcGuNVSr/CDq7s4nvyrunqFh7/9cmZ9wWD1Ct+0fm7QoEsB6hX+
+/jXS31+VFSv8Bpfc2w49a+3eoV3il1eKCXaQ73S/Zb0094+oLh6hR/w3Pen
jUth9UrPW5f90+p1yqde8bpkfxy9wtmHQq9w1nvTK5x1lfQKZ/2Svlc2zjoB
eoXzfRy9wrnuTa9wri/RK5zPcfQK5/0S9wtOl/iZ27iw2PqXZvZSr/Cfpeal
XFnRV73CGww9+Wtllf7qFf4s0a3paJeB6hXefGDjmmdyD1KvcLdTFVcG/ef0
Cl/Zr4tTWOGB6hWeo7Vv7O2o/uoVPqNr9l29P/dVr/S41Lrlvsint3ql+2He
xO5ObXqoV3ih54tnvI3vol7hx9P3HXb0fHv1Ch8Q7dG8b9o26hV+1nVUQT/3
luoVnnfdwoWuxZurV/jo0hmiTp1spl5p/8GvA4MfR8aoV3iRyFUl0s+LVq/w
djk2pXLwi1av8MhpNS7/+tBUvcKrThhYz3dUU/UK3zipy5rNl6PUK61/yX95
8Nl/m6hXej6/7uNQt1dj9Qp3n/nc6fqpCPVK/65/u1Vs/6iheoWfmvygc8DF
+uoVnrn/zLQlm9RVr/C1u+Omnsoapl7hn958/N7vXXX1Ss/nw3fXhToGq1e6
/zccaO9dKEC9wr3Ot/C7+MFXvcKXlXXtcWVeOfUK73bi35dHqpZRr3D7t0eG
THzlpl7hJ14F+5XNUFC94vXHvl16hbM/jl7h7EOhVzjrvbWe0jjrKukVzvol
fc9tnHUC9Arn+zh6hXPdm17hXF/S+07jfI6jVzjvl7i/cLqkf4+5nZe6+4lO
HgPUK/xu+LxjaTwGq1e4TYH2/2aIGape4YcqXbWtFjBMvdK/d2+a/K/9h6lX
eEufEc/qtBiqXuEnjgb08ak3WL3Ck0blv7dvwgD1Ct/dzGfEj9h49QrvUe7d
6jb5eqtXeOKCp1MnZuihXml/0q/f4203dlav8M2tG27sF9ZevcIvrHmQy/Fn
a/UKjwmseWb8upbqFR435/HnKeVaqFf4mj3B8TWaNVev9Pt3W/l61YM49Qo/
9ubt/WY949QrPN37xPw188apV3j5+1u7xC+IVa/wvo1t4kdfbKZe4fNPb15Y
ZXmMeqU/f/LUh2myR6tX+ISk4pdOFItSr/BtNx2q236JVK/w/N89++SY3ki9
wpc4pr2QJVcD9QovN921dj3/uuoVfjKP17AF/WupV/iwJ0ln9thWV6/w0GyR
p59/D1Kv9Dp16D6w+xk/9QrvGFNo1JhlPuoVPm7gkYxv4j3VK7zz7ofjnPqU
UK/wj18u/pW0prB6xeuMeQL0CmffLr3C2R+n/QPG2YdCr3DWe9MrnHWV9Apn
/RK9wlknoO/zjPN9HL3Cue5Nr3CuL+nzpHE+x9ErnPdL3I84XeJnbkf4/d6+
O3iYeoVPWzR3xrj+w9Ur3T/Dvj6NzTtCvcKf76j/5FWeEeoVPmFu8881Bw1X
r/B7b9yqREUMU6/wX6GXWh/8a7B6pc+vn6J/Hsk0QL3C6zfoPLle+77qFf5m
yq9bz3/0VK/wLw5JGZyTuqlX+OoaV2wqZemkXul71JJx+Z5Gt1Ov8L/3VRp4
90Rr9Qpv+dmpd4EirdQrPX86jtu+0buleoWfSLcp/MCeFuoVXjbuo/ebQS3U
K/xgxLEPHrVbqFd4oUzFftqna6Fe4fvP58sxaUZz9QoPsm+9/8PNOPVKz4cO
dr225I1Vr7Q/3mZmQqsf0eoVnmvCy5aTx0apV3hEpF0l9weR6pWe549WnMxR
oJF6hW9b9zljSFx99Qo/m9q+XuYe4eoVfi7TR79Gg2qqV7j7l/Lzu/8KVq/w
wkc8krelDlSv8HUTWya9KlVRvcJtWkz7ZNOlnHqFR9s9ckrdubR6hbffv6Bk
qntF1SteT8w5oVc48wToFc6+XXqFsz+OXuHsQ6FXOOu96RXOukp6hbN+iV7h
rBOgVzjfx9ErnOve9Arn+hK9wvkcR69w3i/peWacLvEztyPGNA+POj5CvcLj
/twIK957pHqFt7k8OsvMXiPVK3xW0VuD0p0aoV7hN+P3Pfk9cbh6hS8bdG3x
xWND1Ss8NuDsl2w/BqpX+Ms6g/s9zdZPvdL9duJ13OzmvdUr/JF7Nt+koj3U
K/x003cPEzN2Ua/wjVN8nhzM2kG9wiOLNp/YO7SteqX7c+6rbe1Ht1av8POh
n6f4jGylXuHHc7Zr3ONPS/UKbxp3oHu/fS3VK3yLi/unf8e2VK90v5Wu5tbO
r6V6hf+MSsq87nAL9Qo/U+3P3zmytlCv8M09dsy2c22uXuGjG1Tafv9rM/VK
618+jlo4pEm0eoWnXfDa8/CwJuoV/ituV6MxkyPUK7zYAr+VS+c1UK/wdgWe
7PAdX1e9whs/8+7hX6mWeoWX/PtA+w7tq6lXui6btntCDbcg9Qo/t3b3l9bf
KqlXeJdyKxKO/eutXuGXQ2bHXBxZVr3C3fOmuVK5czH1itcN85foFc6cE3qF
M0+AXuHs26VXOPvj6BXOPhR6hbPem17hrKukVzjrl+gVzjoBfU9mnO/j6BXO
dW96hXN9iV7pfjOf4+gVzvsl7necLvEzt8EOT3xLuI9Sr/AlC+5X6lpslHqF
t/kY+nFM35Hqlf5dY/uNO1d6hHqFLw0LLZC9zzD1Cl/Q4VuLGT8GqVf4veED
wka9sXqF5/4rPCEkdx/1Cq++5dqIhwesXuE37i+Y6WrXVb3Cd73xebTgldUr
/M2ORy3eZ2inXuG16/5O+8OrjXqFX9qb29PBrbV6hbscdvbtNL+VeoWXvFv/
WsXIVuoVvu1wC5sKBVqpV/iXiBur01y0eqW5ZUn2mTwatlSv8LLv/A6emGP1
Cs+xcYF3zmVWr3DbaiNrNnoVq17hHQJfTljhE6Ne4fOcMn4/VyNKvcIfFQ7a
OccvUr3Cl/39eOIpr4bqFe7Vam6nxJB66hX+z9BiY0KzhqlXeHn7qMJ1nKur
V/jQvF2DZiZavcJXHHJo2v1SZfUKr1fXK+B0fHn1Cg8Z5zP3eZCHeoXv+bGp
z5AS7uoVrw/mwmkulXHmL9ErnDkn9ApnngC9wtm3q32JxtkfR69w9qHQK5z1
3vQKZ12l1qkYZ/0SvcJZJ6Dvb4zzfZyuwxrnuje9wrm+RK9wPsfpfYNx3i/x
eOB0iZ+5HdmvTePhblav8AEtvwXG9rF6hdccvrdTTCmrV7jP/tCxYb2sXuFP
bvZoVemL1Ss85P6Z5UHPrV7h6z3XLPztaPUK/x4+p93QXVavtA9ycVzB9MnW
+yv9OQtiUxe5bvUKz/R307kFP1nvr3DXsmnSpeSxeoW3Ln22Ywl7q1d4r04v
mn0aZvUKH5vQ3X9JRatX+OcOEcVTUlm9wuscblp/21arV3jd7hf6RfhYvcKd
e2398Gyw1St8Qeo/i1aNs3qFex/K91eB3Vav8IIzq1zbmtHqFV7izOUHIwtY
vcI3/tNvcLHcVq/whbGhsWWcrF5pXlH24TsPlbB6hUe/H+dd9In1/kqvo4Tv
wTk/We+v8CyT3cfHzbR6hXfdVn+r00qrV/is3/EZEiKsXuGON35n+1XM6pV+
//SBtt/SWL3idcC8SnqFMxeOXuHMX6JXOHNO6BXOPAF6hbNvV/v3+H3M/jh6
hbMPhV7hrPemVzjrKukVzvolrZ8wzjoBeoXzfRy9wrnures4xrm+RK9wPsfp
/7+N836JxwmnS/zMrd2Q2umT/ufzIF69+Zjnh//n8yA+O32l0DGHrc+D+K7p
s0/1S7I+D+rv6Xaj16J0Vq/05+T0exDTwPo8iE/J+HDlnOxWr/CoQ+mabLrT
Wb3CExq3nPj3rfbqFb5q9tdfM+ytXuFFQ6sHjQqyPg/iNctfHxxU0fo8iPu8
PTx//mbr8yA+t/0yZ/eO1udBPe4tI/NMKmd9HsRdnO0C/n1uXb/Ch999Oa9x
N+vzIF7K1rve5P3N1St82o6KyyJWxalXeJWUvsO2tLc+D+KV8n+LWXytqXqF
f2qQa0ykvfV5EF9/bXKe3MWsz4N6vF7W3eYZbH0exNdVbba/4ac66pUe96CO
TjP+CVWv8NH5M/ValdrqFR7xYXf5I1es61e4d1LgG5946/Ogvi86cT3xbifr
86D+3sGT29i+K6Ne4V2qJh57+stNveL5zhxdeqV9aWZeJb3CmQtHr3DmL9Er
nDkn9ApnngC9wtm3S69w9sdpP4lx9qFo/bRx1nvTK5x1lfQKZ/2S1gEYZ50A
vcL5Po5e4Vz3plc415foFc7nOHqF836Jxw+nS/zM7eXvrnk/tbCut8vzzPyw
boV1vR3vu8TNN8jJut6Ou2caZb/yf66349PPnj9w/IN1vR3PW3xFWPpz1vV2
vFrlke/zX++oXun7zOMLf3/Pbl1vxyfV+t7sUHfrejueutUI58MHWqpX+P2C
R7oEHGuhXuFHrrw5tbSqdb1d99u21n02pzRXr/Copc+2Dj7dXL3SvoGGF7tM
HdlcvcJn1/Jpei+7db0dL7kkx5PMD2PVK3xAjjOpp5dtpl7hp58O2vBXfut6
u/avlxkbfXB/E/UKD842P7lwKet6O95i0dnU24Y1VK80ZyjDjCwjrtVTr/DE
h2NbhYyvrV7hL+KDq0/ZW0O9wh9vLnHfOc663q77OfJZVN2MAeqV5rHNHbJ1
e1tf9Qrf/KVC4WxXvNQr/F3Reraxo0qpV/iz69lWXfe0rrfzvGa+N73CmaNL
r3DmVdIrnLlw9Apn/hK9wplzQq9w5gnQK5x9u/QKZ3+c9g8YZx8KvdJ1HbPe
m17hrKukV3rdm/VL+p7YOOsE6BXO93H0Sn++ue5Nr3CuL9ErnM9x9Arn/RKP
qx5H0yV+5ta+QcfT9het9Qx41YMlq174y1rPgD/zzj5+mr+1ngEf1uu208uc
1noGfNfOa23mrbbWM+Abbe2erC9irWfA3w9qOa3fDms9A557p/0v93BrPQM+
/32PBXMONFevcNcBSXOfbYpTr/Ba5063mbA3Vr3Cq5Vu7XmvUqx6pX3hrRzL
+Nxspl7h669UzxrWpJl6hff4GjGlSkKMeoWfPfq0dPku0eoV3rrLUrsjr6PU
K/ybc8s1N5yaqFd46njbXZd+RKhXeHLElVn3NzdUr/Dtd/Z+fV6/vnql71vO
Bq/N9yJcvcIn77VLv6hxqHqF768yM8/E7NXUK/zhrekLM3pb6xlw/9Lb+6ZN
qKxe4W5nP38Yv668eqV5IYE3/Kfv8lCv9N/ntl8a3LC4eoVX9NpebdKlQuoV
z1/OHaBXOPO96RXOHF3NAzXOvEp6pf1qZi4cvcKZv6R5NMaZc6J92saZJ0Cv
9H7B7NulVzj74+gVzj4UeoWz3pte4ayrpFc465foldaPm3UC9Arn+zhdLzbO
dW96hXN9iV7hfI6jVzjvl3i8cbrEz9r3PLHz0ps7rPWi+KIVUzLGHrXWi+rf
W2bSsmJfrPWi+Mf4TguHZ+2gXuEj50RF3s1srRfF3+fb/HuXjbVeVM+3kyNS
59oap17h3efF2914EaNe4TtGje4xIiRavcKbzAkoG9qqqXqF155mf/Hwqyj1
Cm/0ttzDgwlR6hWe9dSifktdotQrPKrxvOINejRRr/BKj1PWl4pvrF7hmeoe
v3D1v/dF9AqfkrdT6JY5jdQrPOnNzbJ9zzZQr/DSldv+SXujnnqFZxiY4eOM
JeHqlR73HBXHZJsZql7h8UWqtxizoZp6pbn+Wd3yRaevql7hpzO2KNO9axX1
Cs9dNF+fPX9VUK/wNRdLXIsd76Ve4RM2FT9Ufl4p9Ur7w3r27HB6TVH1Cu/V
ZHqn07ld1St9L2TOQ9F8eeOcO0CvcOZ70yucObr0CmdeJb3CmQtHr7Tv1sxf
olc4c07oFc48AXqFs2+XXuHsj6NXOPtQ6BXOem+tRzTOukp6hbN+iV7hrBOg
Vzjfx9Er/Z7muje90nUmc32JXulxMZ/j9D7AOO+XeB7ofjNd0r+HuTJz9jdu
2rWreoWfmxRzoOycDuoVnlT108esUW3UKz1/bC+2O3rH2o+DT3MukGNBG2s/
Dp719vq7Dgut/Tj6fS7na3GvqLUfR+f5DGvhf2B9pHqldfrRn/J3uhehXul5
uHn9k+1dItQrvOPep02D8kaoV1rfMfeoZ51djdQr/OC6bCe2ujVSr/T7hPls
zta4oXqFx9pdDspRvYF6pefJiQ0bR/2op17pdfGwytDZyXXUK/y59/Ouhw6F
qVf4nhUrX9p3rKle4dnzZKn057/3S/QKb7gka/uud4LUK/29Cz9367e9inql
ubMRjf69FeCrXuEfX/avmrZLOfUKPzQjz77tE8uoV3r+vyqx4sK0YuqV5q4d
nxPX5GZB9QpPHNg3beFxedQrno+c00SvcM5DoVc45w7QK5z53vQKZ44uvcKZ
V0mvcObC0Suc+Uv0CmfOifbPG2eeAL3C2bdLr3D2x9ErnH0o9ApnvTe9wllX
Sa90f5r1S/QKZ50AvcL5Po5e4Vz3plc415foFc7nOHqF836J5wdOl/iZ28+V
T9+73beNeoW3e37OIc2lFuoVXnBe8ZHuK5qpV3hynbRPemSKUq/0fNh6eHGp
g9Z+Z9wtruK3mX9Z+53178qRN3R5kLXfGd+0dfJf10bWV6/wHVnKFp+Wu756
hb9dXHr6znP11Cs89btJhVL3r6de4beD21d9n6qeeoXfqvWxdK8dddQrfPvv
fpmKpAlXrzSnx+upQ3BSLfVKr6MLgy4Hz6qpXuEjxzzI1jV9dfUKL936SXTF
5sHqFT4rdfHqrfYEqlf45aKBDydG+alXeKewKnev//d+iV7pcVn7vO5a/3Lq
FT73XsjeplfLqFealzbE+9ftKu7qlf6cWZf+nrCtkHqFX3Fd+W5Ro3zqFX53
T8j56Cw51Cued5wfR69wzmnSOS7GOQ+FXuGcO0CvcOZ70yucObqaU2mceZXW
vLn/d+bC0Sude2DmL9ErnDkn9ApnnoD2eRpn3y69wtkfR690PozZh0Kv9L2Z
We9Nr/TvNesqtY7NOOuX6BXOOgF6hfN9HL3Cue5tXe/7f+f6Er3C+RxHr3De
L+n1b5wu6XEyt5VyPdvSt10z9UrnE1a7dLRlYBP1Cl/f/Unl5JBG6hWe0il5
7PI29dUr/GWJBmV6frTmyeCFpm5su29ybfUK35Cn0bCBJcPUK9ytdYDz8rha
6hW+bMbSttkvhapXeGyBdw3jgkLVK/z500fH1o2oqV7hQ3pFjss2tIZ6hbt+
3rD7Ytnq6hXefkP4wBpzQ9QrfPbgdRu+nqmqXuFL+/v2y3c9UL3S8+rHiNDp
i6qoV3iHIe8T3f+qqF7h7kPKxl9fV169wnM2Pf/mbVov9QrPV2fKB+fOpdUr
/O7tsJ/X/nu/RK/wrF+7Oj8aW0i9wt+l73Tg5d/51Cv8Qua8v+xX51Cv9Poa
9aXB0FzZ1CueX5xrSa80x8icH6dzmIxzThO9wjkPRedGGOfcAXqlczXNfG/N
HTbOHF16hTOvkl7hzIWjVzjzlzQPhfkBZs4JvcKZJ6B9qsbZt0uvcPbH0Suc
fSj0Cme9N73CWVepdT/MQTfrl+gVzjoBeoXzfRy9wrnuTa9wri/RK92f5nOc
3o8a5/0Szyc9jqZLen5w3T396/sTjjVUr/BXOca6xNWsp17hbp1upwzrH6Ze
4Yc7NvEokqqmeoV3HuK3IeisNa8Pz/sqsqn/gBD1Svsab1fu1HBLsHqF5+w8
5FFgyWD1Cvfu3bxJu41V1Ss9f1zKNkvIWVW9wlM6pH6+o0GQeoVnLtnA82GT
QPUKb1eo2eYJUf7qld6fJjlkqb6psnqFN93S1ibTOV/1Cr/3+WmL5PPl1Sv8
3OSytf654qVe6TyQn/u8Tr4sq17JO5yoFj+hhHqFL7yyL+jOmqLqFf5+SPO9
W5oWVK/w/LkfzT8fmle9woOf7v4S89/7JXqF349u3qzgk6zqFR4/YMT+C1vT
q1c8jzhvl17hnGtJr/S8M+fH0Succ5roFc55KPQK59wBzRlnX6yZ702vcObo
0iuceZX0St+fm7lw9Apn/pLmXBhnzgm9wpknQK9w9u3SK83RNPvj6BXOPhR6
hbPem17hrKukVzjrl+iV7jezTkDfsxrn+zh6JTfXvekVzvUlfV4yzuc4eqXf
07xf4nmG0yV+5nah3bv3tcNrqVd4Gef8x9+ur6Ze4d09G5cvEF9VvcJjwla8
aRkXqF7pcV/zpUTx4tY8ZPzooQE3f8z2U6907nT58CV/T62sXuHHvn4+VXtr
JfUKP7KkWdZ9RyuqV/j5mEWzsi72Va/w+11u1VvkU0G9wjdcada++fjy6hW+
JFNCqhPLyqlXeKntVx63n+2pXuHVv2UMfvOhjHqF33o4c3vaHSXUK73u0lz7
u9hSN/UKd/Dy+7DzUiH1So+7Xe3y0Xtc1Ct8jtu/feYsyKVe4QUcR55q3TS7
eoXXTdNh1fG7mdUrPPZi6tnZFqZTr/AFz372b3zeRr3SvDRzDrjO0zXOebs6
r9E451rSK5zz4+gVzjlNOofDOOeh0CuccwfoFc58b3qFM0eXXun3MfMqNe/M
OHPh6BXO/CXNSTHOnBN6hTNPgF7h7NvVfjbWcZv9cfQKZx8KvcJZ702vcNZV
0iuc9Uv0CmedAL3C+T6OXuFc96ZXONeX9DnZOJ/j6BXO+yWefzhd4mduB034
Z7NdY2t+u86jmONees9UP/UK/9gife6qA33VK3x+2SKjFjyxzpvAXU+tmz62
lnXeBB7d4u8//wSVU6/wgv8cnlSzv5d6pd/T9t3jm7s91Su8a/jX600feahX
eh4+zXFi0vGy6hX+1dfr1M1epdUrfEG15/fa3yihXuGpq32oVziTu3qFv/hy
dNSIwkXVKzzNopn//b9mIfVKz0+n6JpN3rioV/jDVcttp+zMrV7hr95NSr/6
t5N6hdd8ufHyngvZ1Ct8zWbbDre6Z1Kv8LlD17T81NZWvdLr4l6DgdOepeh8
HPz2EEf3mG9fdd4Ez4sp1fIsuednzW/X88WcA06vcM7b1XmQxjnXkl7hnB9H
r3DOadK5L8Y5D4Ve4Zw7oHnc/L1mvje9wpmjS69w5lXSK5y5cPQKZ/4SvcKZ
c0KvcOYJ0Cucfbv0Cmd/nPZ1GGcfitb9G2e9N73CWVdJr3DWL9ErnHUC9Ern
PJjv4+gVznVvXd80zvUleoXzOY5e6e8175d4Xuo8HNMlfua26/hiDWtNt87H
wTuOynPj7DAv9UrP276lCn7fW1a9wiflWTrofadS6pU+dxZ/tGBZvRLqFV5i
Ss+Iqdfd1Ss83+zTGfPPK6Ze4V9PlqtWJcZNvdKf87Jkh/TJRdQr/FzjDlmb
dyisXul58qFc2g0bCqpX+FqX+82eryigXuFLN+0cvu9SXvUKX3ynmNe3NLnU
K/x7qsffM0Y5qVf6d721b/zpdTb1Sq+X7CHz5y+yU6/wZbUG3H3mmk690uNV
uPvhs21SqVf49PYbkyrfTNZ5Xjpn8o1X0XQRn9UrPGfz7PnsKr5Tr3j8vw0r
e6luzEf1Cm+8okvY8i5/dD4Ozjng9ArnvF16hXOuJb3COT+OXuGc00SvcM5D
0fkKxjl3gF7hzPemVzq/zszRpVc48yrplc4/N3Ph6BXO/CV6hTPnhF7hzBOg
V1pPZPbt0iuc/XH0Cmcfitb3G2e9N73CWVdJr3DWL9Er3Z9mnQC9wvk+jl7h
XPemVzjXl+gVzuc4eoXzfonnK06X+Jnb5xcy1Gh4vJR6hZedlmpSREwx9Upz
HQaV3BvyprB6pc+XJ86u6HnYOi8Vv3Vu0r55Ea7qlX7Pb6HRuye4qFf4tuTT
SQVW51Ov8Gv/zEj4+k8e9Qp/Fu65N2JDLvUK97p7xPlmd2f1Sq+vdH9iOzo6
qVd41qetq/yzzEG9wv0/PPyZs2cW9Qr/fflkJ+8zGdUrvNaHJTPceqVVr/Bf
2bauezg8lXoln1rx+hz7XzrfGW9f/e3Rbpm/6rxUfOHSU5Vy2X5Ur/Dv/peu
LDv2Ur3C/6meIXJyscfqFY9zyoM/BZa9+qBe4T7JhfrdWJSiXuGfXV5uDtma
Sb3COQecXuGct6vzQY1zriW9wjk/jl7hnNNEr3DOQ6FXOOcO0Cuc+d70CmeO
Lr3CmVdJr7Tex8yFo1c485folT6vmDknmk9hnHkC9Apn3y690uNi9sfRK5x9
KPQKZ7231lkaZ10lvdKcTrN+SesA+P7NrBOgVzjfx9ErnOve9Er7Zsz1JXqF
8zmOXul+M++X1FnjdImfua1bc1qPmLMF1Sv88LbX1VMPyqde4TOLrC6YNlMu
9QpvvrJd5t6VrPPo8QL53iS9rJ9dvcKbvqlp9+qqg3qFT5owuuf1wdnUKzxT
gywBaUKzqFd4XL3nt/P2s1Ov9Dw88Tx0wPgM6pXOMUtnd3D1pLTqlc6PzfQo
e73ZqdUr/EPx14Fphv7xp1f49NBnXZZ6JOs8enzIliUd3Jp/0Xn0elxKHT8y
osBH9QoPTL1o1oo3r9QrvHO2uut6HH+iXuFnmhXcUyb+vnqF25Qc9vtqy+vq
FY9nmikb3VolWr3S47yzUKpyvlav8J4Oj+s9bmL1Su+7Tw1NHeJq9QrnHHB6
pTm15rxdeoVzrqXO/zPO+XH0Sn+vOaeJXuGch0KvNM/bnDtAr3Dme2v+snHm
6GpepHHmVdIrnLlw9Apn/hK9wjXnxPQKZ54AvcLZt0uvcPbH0SucfSj0Cme9
N73CWVdJr/R8MOuX6JX+e7NOgF5prqf5Po5e4Vz31vVN41xfolc4n+PoleZa
mfdLPL9xusTP3KZdmGnCXo8c6pXu/5LFGkxYmU29wnd/6j5i3HY79UrPnwt+
uWYXz6Be4XaLHl2K25ZWvcJfRzZ8kHwgjXqFD936cv/EH6nUKzzre5crFSvZ
qFfaV9QnU5v7r3/50yu8689XK9bE/vCnV/pz/tzOef7+F396pfu5eetNxcck
qVd4/X127Qaee6de4Vs29iozqfBL9QqvUmH2rHyFn6hX+OyXsc8vrrmvXuHb
Jj/cPKLrDfUKX352/d1ewy6pV3jVNyerjR52Rr3icYv4uDb580irV3jc5K7H
Pj39rV7pHKStdoklHaxe4QOb7m2W7kkOa56M8XDffFMybrd6hXMOuObJGOe8
Xc2TMc65lponY5zz4zRPxjjnNGmejHHOQ9E8GeOcO6B5MswdMfO9NU+G57uZ
o6t5MsaZV6l5MsaZC6d5MsaZv6R5MsaZc6J5MsaZJ6B5MsbZt6t5MpwLavbH
aZ6McfahaJ6McdZ7a54M36eZdZWaJ2Oc9UuaJ2OcdQKaJ2Oc7+N0vdU41701
T4Z1B+b6kubJ8PuYz3GaJ2Oc90uaJ2OcLulxNbdrMtw81dM1gzVPxnh7m41e
vxJSW/NkjM/rMytxVKsUf82TYZ5W66els+dP9tc8GX7Pd3nmnsz/zV/zZIxf
+Jap6oPtn/01T8b4wpnhswtNSPLXPBnuz4LjfT73/KBe4VeO9Zky7swb9Qpf
+mJWJadzL9QrPU861Yn58PuJeoXXyhlb7VPPh+oV7j56+6vWA+6oV3jM82WP
Zp2/pl7h4852K/vx3UX1Ci+zM2vs/vVn1Cs8aE6nfcW+HFWvdH5RpxIz7Iod
UK94fFYFX6u+OMjqFe7U5uqBe12sXuHbbYp067c6o3qFu0U5Nss5yeqV/vsM
lz2edbF6ha9123/6tLvVK5xzwOkVznm79ArnXEudS2ec8+PoFc45TfQK5zwU
netgnHMH6BXOfG96hTNHl17hzKukV5pfYubC0SutAzLzlzTXxjhzTuiVzs0z
8wToFc6+XXqFsz+OXmm+ptmHQq9w1nvTK5x1lfQKZ/0SvcJZJ0CvcL6P0/cu
xrnureuSxrm+RK9wPsfRK5z3S7wecLrEz9wOqtWgYsNiyf70Cs//q+8F9w2f
1Cs8c9PAm/sWv1ev8JO/m2V0PvhKvcKrHLRP3a/ec/UK7zSm6pZek5+oV3r+
D26QepHzI/UKPxM+sFWZ8ffVK63TnxGTeWTe2+oVXuzktLWOb66pV/i0DOOK
DLe7rF7hYz80XDU5+3n1Ch9QPz7Hmccn1St8S44ZP4tMO6Je4WVqTopL9tyv
XuE3Puaate/uDvVK56qtf7Btacpm9YrHoeepG/4e696rV3jZPtP/hK39pV7h
5Sf+HPn5dwb1Cp+XUCj7k19O6hW+Nl/M9KzJruoVfiE0pUb13iXUK7xu9RtJ
TW6VU69wzgHX+b7GOW+XXmn/hDnXUucUGuf8OHql9TXmnCZ6hXMeCr3COXeA
XuHM96ZXOHN0NafSOPMq6ZXuBzMXjl7hzF+iVzhzTuiV9r2ZeQL0Cmffrva/
GWd/HL3C2Yei9evGWe+t9aOcf2XWVWqdk3HWL+l7feOsE9D3bcb5Pk7fBxjn
uje9wrm+RK/0+5jPcXqfx/PWvF/idYLTJT1O5rbj91vNMrx6pV7h7T5+OzHw
+GP1Cm9cpNnk6R731Sv8wJ/KecKTbqpXeJrBYV/XB15Tr/D2S698qB12Wb3C
F3WY+3Dn5gvqlV7XHQ7cnz3zrHql5+3N4S2Gup5Ur3DH5M7F49IfVa/wMkFt
Gy6OOKhe4ak+F39atPge9UqP16AdR0YkbFOv8G5T7888sXeTeqVzs/O23XE3
/Wr1Cr/a5W73sneWqFfc34crnD84uNM79QrPMmRl1OtxP9UrvEiVGyGrU1m9
wju0y9kqTXGrV3g/x2/B6+tYvcInJPx9uFyH4uoV/iBl+OGXDlav8F/e3vEX
plVWr3DOAadX+neZ83bpFc65lvQK5/w4eoVzThO9wjkPhV7hnDtAr3Dme2uO
sHHm6NIrfU9u5lVqHp9x5sJprpNx5i/RK5w5J/QKZ54AvdLrz+zbpVc4++O0
j8g4+1DoFc56b61rNM66SnqFs36JXun3MesE6BXO93H0Cue6t67rGef6Er3S
PGPzOY5e4bxf4vWD0yV+5nbRXyFlnQJuqVd4XKfzK6ZduqRe6XlYqs31g/nP
qVd6vXwaVn7zhRPqFf7Su9TqTTmPqld4vYy+MT0LHlKv8N5jHxdxdP1HvcIX
+6Z75jF0l3qFD7v/dnhE+Db1Cncf3HdAz4mb1Sv9nn4Pb6cZvFa9wp/P2lNh
ycXl6pXmWJwMdb2bdbF6hfd0b9e8RcO/1St8aMObnYOcpqtX3K+7cuX6OOTQ
G/UK37DteXjD9MnqFT7yo0fx/onp1CutN+nkfHFzluzqFR6bu8L3eVdc1Cu8
TdPo243TuqtX+PIKlW7dTOelXuG3Sn77uSNrJfUK/9AlqVWVolXVK5xzwOkV
znm79ErXG8y5ljqfj30V5vw4eqXrqeacJnqFcx6KzocwzrkDmi9vnPnemhtr
nDm6mk9qnHmV9ApnLhy9wpm/pPk4xplzQq/0eJl5Ato3a5x9u/QKZ38cvcLZ
h0KvdP+b9d70CmddJb3CWb9Er3DWCdArnO/j6BXOdW96pXXo5voSvcL5HEev
cN4v8brC6RI/c3tq284ysyudVK/w2avv1Pf4fFC90uurdf9+rXbuVq/wE1vX
RyVs2KZe4S7Zm9pucU9Ur/Dkbwl2PkfXq1d4xiqHvwT/936GXuGpX+Y4VPXm
MvUK77D1y6Y5bovVKzzN5han7a/PUa/wPzdK/1kUPkO90vlmSxtd+9Ngqnql
58ks31K2u8aqV3ham+5OmxYPV6+4/w4WXZzu0s5X6hX+K2plFdey39Ur3LW0
i9fJHGnVK9zl9P4pX7Y5qFd4k5tVMoWtyKde4c0DtwYm1HdTr/B/9u8vceGA
h3qFj7lduPFLH1/1Ck/Ym6OM3Z1A9Uq/z9yEJLsF1dUrnHPAdf60cc7b1fmm
xjnXkl7hnB9Hr3DOadL5OsY5D4Ve4Zw7QK9w5nvTK5w5uprDaJx5lZq7Z5y5
cPQKZ/6S5s4YZ84JvcKZJ0CvcPbt0iuc/XH0CmcfCr3S5ySz3pte4ayr1Lol
46xfolc46wTolR53830cvcK57k2vdD+b60v0CudzHL3Ceb/E6w2nS/q9Wef+
qFvWf923q1d45eSZWaY/Xq9e4cVONFpwauAK9UqPe/57e51/LlavNGcrdds5
NXbOU6/w8KBQhy3XZqlX+IroXp5e+6erV/i1iS9u32k/Rb3C92w9MChz2Dj1
SvNydvmmGpV9pHql58MHv9btOw9Wr/DnZU5tele9r3qFN4h0uFR2Snf1Sudg
r6nYpGDLF+oV7vAnwLaA41f1Cs/cekNAHv806pXOT0sT+OCmezb1Ci8y6/Wd
AUXyqFf43TvTj4fGFlGv8BfbR+cNnF5GvcI9Cg284pvbR73C5ztsfnfsmL96
hVec8letkstC1Cu8VnLmyWPX1FKvcM4Bp1c45+3SK5xzLemVzmUy58fRK5xz
mugVznkoOh/COOcOaC65ceZ70yucObqaT2qceZX0CmcuHL3Cmb9Er+Rmzgm9
wpknQK80H8Xs26VXOPvj6BXOPhR6hbPem17hrKvUujTjrF+iVzjrBOgVzvdx
9Arnuje9wrm+RK9wPsfRK5z3S7wOcbrEz3r+P738aFf/JeoVPmn6q7ETqs9W
r/R8a3hsT4YfCeoVbnP0Us6PcRPUK92f/gFtD5YYrV7hZds69sxXdLh6hd8K
vj08fMVA9Qr/87jk9w9p+6pXet5u//NxU7Ue6pWe56UKXmuzuKN6hfvFrKu5
olhr9Qqv2r5OVNHPzdQr7o/9Yc/fBt18ql7hz6O27D91/JN6pf1M9w9lyeCf
Sr3CPy30aLnii716hffNGHn/dOec6hXeaMad1lUDC6lX+MVhdVdua11SvcKH
9V3s7HaunHqFPypf4e696ZXVK30P3D+4VPLgquqVfv9nXj1TPamhXuHtV15z
PLcnXL3COQecXuGct0uvcM61pFc458fRK5xzmugVznko9Er3szl3gF7hzPem
VzhzdOkVzrxKeqXH3cyFo1c485foFc6cE3qFM0+AXuHs26VXOPvj6BXOPhR6
hbPem17hrKukV5orbNYv0SucdQL0Cuf7OHql54m57k2vcK4v0Sucz3H6nGCc
90u8PnG6xM/c/uh5vGm+sInqFT5zRdXnVfcOV6/ww1G3x2WN6a9e4XPivPON
ju2lXuncsOnfE1pO76pe6Xm+bNX2w5Ht1Sv8xtjIa/urt1KvcMf931PC3WPV
K3xP8yDbjL6N1SvcPyays/u0+uoVHnj6fIYqeWqpV/y7z6/pfPFGhcfqFX7M
oe1E/zYf1St8QN9TLp4RKf70Cj+1/UBStv4Z1Ss8U1zq3GfnOqlX+KEYt0t7
M7uqV/iRuVfXtcnlrl7hTUqsXXito6d6hd+p2/ekTZCveoX3yjEtU9qQQPUK
j6v3Z+OqNNXUK3xxxnLXBrjVUq/wzP1npi3ZpK56hXMOOL3COW+XXuGca0mv
cM6Po1c45zTpHCDjnIdCr3DOHaBXOPO96RXOHF16hTOvkl7hzIWjV/r9zfwl
eoUz54Re6TqxmSdAr3D27dIrnP1x2t9inH0o9ApnvTe9wllXSa9w1i/RK5x1
AvQK5/s4eoVz3VvX14xzfUmfr4zzOY5e6RxU836J1y1Ol/hZr9NhfjVGuvVW
r/As74Oav53QQb3S4+U3/suvly3UK73u4q492PWsqXqFz79a/rDXg0bqFd6y
2+CvoyLqqVd4+cCVzV06hqpXuv9jxlSuMDpYvcK9Mn1Y8Xx4FfUKn7Oocosd
L8urV/z7Hu7I13bx7AfqFV54THzNKRnfqVf4tAWFM5fzSFav8O5b3rfLlpJW
vcLLvHxp73TaQb3Co7d+WtduV171Cp+c98KdGeeKqFf6PT0qvPDJXka9wgde
XzZp2Qdv9Qr3G/emZBFnP/UK92wyet68IlXVK/zg6i6OJ/+qrl7ha3fHTT2V
NUy90lz/6a616/nXVa9wzgGnVzjn7dIrnHMtdd6hcc6Po1c45zTRK5zzUDTH
n/N+zbkD9ApnvrfmBRtnjq7me3Id18yr1Pw748yF0xwr48xfolc4c040x8E4
8wS0v9o4+3bpFc7+OO2nMs4+FK2/N856b3qFs65S6wiZ02nWL9ErnHUC9Arn
+zh6hXPdm15pzrq5vkSvcD7H0Suc90u8nnG6xM/clrhrX+mJa4x6pfNUHWIX
7W9dX73S/T/a6UqFdzXUK72O3KfVzrU2SL3Cy1ar8THdl0rqFb41IDJw6B5v
9Qrf+L32wI0byqpXev5k2Zzv7uNi6hXetNKiwXN2F1SvdM6PS/z1HS531Cu8
TapzkT97vVSv8OY2vQen9vyqXuGeW2/d/XdpavUKj9qXbU12tyzqFf4zzeT2
0YNzqlf4SK+Ns84sLqhe4UdfHxhVYlBx9QqvedB27LD9nuoVvu/L1P6nzlZQ
r/CGZYoOGnbAX73Cw99+ObO+YLB6hX968/F7v3fV1Sv8ZB6vYQv611Kv8LOp
7etl7hGuXuGcA06vcM7bpVc451rSK5zz43Qul3HOaaJXOOeh6FwK45w7QK90
/5j53vQKZ44uvcKZV0mvcObC0Stc85dMr3DmnNArnHkC9Apn3y69wtkfR69w
9qHQKz0/zXpveoWzrlLrloyzfole4awToFc438fRKz3fzHVveoVzfUnXU4zz
OY5e4bxf4nWu85xNl/R7c+5EjuHNa06tql7hwxrNHhPQxke9wt+HusS/rlhG
vcIzfT3mG/++qHqFzw+/9u1zb1f1Ch8Sb7f+3r1c6pWez4kfhjRY66he4f3n
3t6ZabmdesXvG9q1hk1Sy+vW/CvjjttPRhcu+VS9wl89GXVuZ+8k9Qrfdan+
qr5/Uvw1/8r4oe6lbR85ZrTmXzF3p+Wdty7js1vzr4xPrZ3XMbxBfmv+lXHX
qkUvf/Qsas2/Mr46+ujCHSNLW/OvjJfa5JmrzJly1vwr4+XHJfoNnF7Rmn/F
XNj1c4MGXQqw5l8Z/3n47rpQx2Br/pXxYU+SzuyxrW7NvzJ+LtNHv0aDalrz
r4w3fubdw79SLWv+lXHOAdf8K+Oct6v5V5xvac611Pwr45wfp/lXxjmnSfOv
jHMeiuZfGefcAc2/Ms58b82/Ms4cXc2/Ms68Ss2/Ms5cOM2/Mq75S8yFMc6c
E82/Ms48Ae2/Nc6+Xc2/Ms7+OM2/4s8x+1A0/8o46701/4rXhVlXqflXPB/M
+iXNvzLOOgF9L2ic7+M0/8o41711vZVz4831Jc2/Ms7nOM2/Ms77Jc2/4vc0
XdLr0NyOjPg6w69YMfUKb7nfYeS3qnnVKzzp8OHck+0d1Sv87rcj9Qceyqhe
6ZzD91+X/fUntXqFn3r9ZW5Hn19VNP/K+KDfX7b3OPO5iuZfmd/rn+Nd9h1x
vmzNv+J8DKct62vXeKhe4QfuTEiV7PpOvcIrx/jGVwj/oV7hfnar3zeMt7Xm
Xxnv2rNVa68nWa35V8bnR08o9PpdTmv+lfHHeQ++vVW7oDX/yvigyePdAnO6
W/OvOD/N83airbuHNf/KuGuHlN4D7ntb86+M/z7+9VKfHxWt+VfGR2840N67
UIA1/8p4aLbI08+/B1nzr4y7fyk/v/uvYGv+lfGSfx9o36F9NWv+lfHy9lGF
6zhXt+ZfGecccM2/4vcx5+1q/hX3jznXUvOvjHN+nOZfGeecJs2/Ms55KJp/
ZZxzBzT/yjjzvTX/yjhzdDX/yjjzKjX/yjhz4TT/yrjmLzH/yjhzTjT/yjjz
BLQf3jj7djX/isfL7I/T/Cvj7EPR/CvjrPfWul7+HLOuUusRjbN+SfOvuD5k
1glo/pVxvo/T/CvjXPfW/CteL+b6kuZf8bozn+M0/8o475c0/8o4XdL8K3N7
s9Tocqnq2KlX+j1nzFg8LO5PFc2/Mu6/Lj5Nzxafq2j+FZ8vCwSllPr7TRXN
vzJe2Ptso5X7nqhXerwutfN8GXRPveLvT6gbHdNt5r/W/CvjMW51Oj3aeNua
f2W8SEx4r0OFXqhX+KDxcyL+DfmsXuH2nouWurWxseZfGbfpVeO476SM1vwr
4+N6D31WtlR2a/6V8W8LquQ9tTOvNf+K9ba3JrgMrV3Ymn9lPMeBpIj5j4pb
86+MeyQ+mzVzl4c1/8p4ja85Npz619uaf2Xc63wLv4sffK35V8arOXQf2P2M
nzX/ynjhIx7J21IHWvOvjFdI2z2hhluQNf/K+NC8XYNmJgZZ86+MZ5nsPj5u
ZpA1/8o454Br/pVxztvV/CvjnGup+VfGOT9O86+Mc06T5l8Z5zwUzb8yzrkD
mn9lnPnemn9lnDm6mn9lnHmVmn9lnLlwmn9lXPOXmH/F//+bOSeaf2WceQKa
f8Xz3Ozb1fwr4+yP0/wr4+xD0fwr46z31vwr46yr1Pwr46xf0vwr46wT0Pwr
43wfp/lXxrnurflXxrm+pPlXxvkcp/lXxnm/pPlXxumS5l+ZW6c1PY6u6P5O
vdL6rNvXltk5PFSvcN/262bEHr6qXuHT3hdr4W/7r3qFbw7bNXtq22PqFX/P
xIorF86zO2nNvzJ+oFXhH+eOXLXmXxkfMTZD6KUfj9Qr/F7zrc9t779Xr/Dr
9rP27zjxw1/zr4zvyxxzPrWzrTX/yviqdPfLHS2axZp/ZTz/+abDz53OYc2/
Mh7lm+g8vYmLNf+K//5dyZh3UUWs+VfGx7VJKPSiVglr/pXxTrHLC6VEe1jz
r4wvK+va48q8ctb8K+MdYwqNGrPMx5p/ZXzdxJZJr0pVtOZfGT+3dveX1t8q
WfOvjK845NC0+6XK1vwr41231d/qtLKyNf/KuHdS4Buf+ErW/CvjnAOu+Vdc
BzXn7Wr+lXHOtdT8K+OcH6f5V8Y5p0nzr4xzHormX3H/m3MHNP/KOPO9Nf+K
38fM0dX8K+4HM69S86+MMxdO86+Ma/4Sc3CMM+dE8694Pph5App/ZZx9u5p/
ZZz9cZp/ZZx9KJp/ZZz13pp/xXl9Zl2l5l8ZZ/2S5l8ZZ52A5l8Z5/s4zb8y
znVvzb8yzvUlzb8yzuc4zb8yzvslzb9ifajpkuZfmdtm9zwiq2S6oF7hfSbe
339n8iH1Sh0oOGFy2m/b1Su8Sqo861+f3qBe8efl3FFg8NHJh635V8aHtRq7
IOzMBWv+lfFuH7KtPFDzrnqFF64X/njJ7BfqFR4Wk9Cx55NP6hV+OLLMrvp/
Uvw1/4rrc5s3JxQLTm/NvzLeuEO734n//f+v5l8Zf/AqxCW6Y05r/pXx4Kl9
DxZqX8Caf2X8+0mbXhtCi1jzr5iLn/TT3j6guDX/in/viX9fHqlaxpp/ZXzc
wCMZ38R7WvOvjNu0mPbJ5r/3J5p/ZbxLuRUJx/57/6b5V8br1fUKOB1f3pp/
xT6G3/EZEiLKW/OvjG89cT3xbidva/6V8c1fKhTOdsXLmn9lnHPANf/KOOft
av6Vcc611Pwr45wfp/lXxjmnSfOvuP/NeSiaf2Wccwc0/8o48701/8o4c3Q1
/8o48yo1/8o4c+E0/8q45i8xD8g4c040/wo38wQ0/4rng9m3q/lXnGNs9sdp
vxaPl9mHovlXxlnvrflXxllXqflXxlm/pPlXxlknoPlXxvk+TvOvjHPdW/Ov
jHN9SfOv+H7PfI7T/CvjvF/S/CvjdEnzr8zt1ZBGW3d/2aRe4a3qrs11w2GJ
eoV/q9e4zrGvM9Qr/ncXY0f6PMu9z5p/ZXzZo+fzF3U7Zc2/Mr580V97y5e+
pl7hTX16f1mw6JF6hXdcuM7ples79QpP87Hnroj+39Qr/MGXMteOL0hlzb8y
PsDuV8Nz8Rmt+Ves41iWZXXfvQ7W/CvjnRMTc+04mdOaf2V8aw+3Xn57XKz5
V8YPeO7708alsDX/yrj92yNDJr5ys+Zf8efvfjjOqU8Ja/6V8Wi7R06pO5e2
5l8ZvxwyO+biyLLW/CvjIeN85j4P8rDmXxl3vPE7269iHtb8K+NfBk9uY/uu
jDX/yvi7ovVsY0eVsuZf8b1xbvulwQ2LW/OvjHMOuOZfGee8Xc2/Ms65lpp/
ZZzz4zT/yjjnNGn+lXHOQ9H8K+YhmXMHNP/KOPO9Nf/KOHN0Nf/KOPMqNf+K
562ZC6f5V8aZv6T5V8aZc6L5V8aZJ6D5V8bZt6v9qMbZH6f5V8bZh6L5V8ZZ
7611z8ZZV6n5V8ZZv6T5V8ZZJ6D5V8b5Pk7zr4xz3Vvzr4xzfUnzr4zzOU7z
r4z/H1f3HVfz3z9+vCEN2tHU3nvvOq+WNDS0905KWlIaIlRGRqhQaKmUhlJG
KCQkGhRKorLSQqT5c13fXs/Xdftd/3Tz+Bx9+tw71/uc83q/3s83fr8E869W
Oj4uwe975Wv+yc9R4bzH4HiFO2f8+5vb/VPgeIX/Oee9Ca+Gj/VwvMKdf9BS
4NLn+3C8wv3eTXnBuMROOF7hPhFcvnagqh+OV7jHUWuejg38BMcr3P8yXPa3
jZ6G4xXu0baOG3OU5gzx8Qr37nhV7fpxGjhe4T72SWvdDQ8mOF7hzszixGFz
lR2OV7gf1NU7va13PRyvcC8SvHvSLkwAjle4t30z0VdiEIHjFe7TM12nfpSL
wfEK95C7+XLUgxJwvMJdmp/2pd52KThe4X7rb3XsHllpOF7hfoEerfpDKw3H
K9zDjWtbRxck4XgF1131sZX2qUjA8Qp3HdXrZke7ReF4hXuMa1bYU15hOF7h
ju8Djo9XuOP77eLjFe74vpZwvzr8e1+5fxzcx2ul4/s04eMV7vh+KPh4hTu+
7wDMAV/peL43Pl7hjufo4uMV7nheJczFw8/Dlblw+HgFHc9fWjle4Y7nnODj
Fe54ngA+XuGOr9vFxyvc8fVxcF0Q/jlXrkPBxyvc8X5vfLzCHe+rhH17Kx3v
X8LHK9zxPgF8vMIdn4+D8yUrHa974+MV7nh9CR+vcMef4+B98ErH75fw8QiO
PyvHJXjer3yl2BTeXf8s1CBTtEH4YE4PBX+9yVrybejKFdjngHsy0xib6q0i
wwSp5LGCWgOE+6ffsfz125rheIf7g52GqFqvCf4+/nv48fj74272jNMjdtMN
+DP+ytM+tfrB2SQD/Gf87zl91ry4Zv1O+Pnx9/v/f07c8ffHj8ffB/+c+M/4
7+HvL3mb71p6zgMK/prT48zG8KjI8OjT81Odr/+9X1npbXRdul26twzxn/E/
xx3/GX99NLY+QjUvBl4v8N9juMoYUrdwCR6P//n8yfimMxkN8Dj4/3VRfupd
2xvw8+O/t+Yz7+ygXST8/Pj74O8P+/FW/jn+Pvhx+Pvgfy/+M/57+Pv7Uz8/
TkW1BN8/o5LjyV3uh4a449c9rYtuSnNK3Ya2OwoMqKhY4L9LeXn3xi2Ow9Dx
4xPr1O6sS5k2/DN+lZdq73p4fHLZbUcO/jno+PFKr6UtnQroKY/Ej3BRUQnC
41tNB17M9zFAx4+Pzuaa+JC0nvJUoZ6eqlkUHp9tUS9RdYcNOlz3zxknf9xG
kKL6RJSOaq8kPD41MzHbW5oNOn58LuumBoQ2UM4GnqShQrLw+A2jmVxWivTQ
8eObfnyTzJznolBT/ed/ivD4Kq72tADaBUPc8eN9vzLetvNgpNRvblxafq8M
j58LdDC59vWHIe748a7v6qTSBakpIed3LC7fU4XHizAriPc/mjLEHeZ0dt5n
ZvdZMhT4KrGwfFEdHv/4V4LL9YlJ6Pjx0r6l/ds+Lxi+0Hw3t5yiCY/PEDU0
MtAdg44fX1vTQK1QMmu4/8Dpv8s+2vB4g7IHaQINw9Dx4z18VpVk9E0banVb
zS5TdOHxuwMVzofO9UHHj79nKONhWD9qOCZE+2dZSB8eL3dk31vtnY+h48eL
sW1EKaK98LyFfaZT3FQcje/h+Ym7w4iD9PnUJXge4v55MPl30DkueL7hfmOf
ivC282LwvMLdM1VY68p2WXj+4L45LuTM3F4ZeJ7gTsMinR1mJQrPE5iXEzPO
Ws2/Hp4PcN2keFEC1Vtm+L3jvt959GVbAhMF/x5xn1CKv2OgxkjBvy/cjQY/
RtQn0VHw7wV3i8CjEwkCy+CPe9zjVDGj7z/AGffll5V6ftbD4Izf5yp/eNby
uPAbOONu7p989MZFRgp2xt174ca62s0i4Ix7RzWVzuE4ZXDGvVHlY8MmFU1w
xv30M70jTmEa4Ix7MfNzubJbiuCMu+8PFmqZk6LgjDutckrIE3ZecMa9r7T/
aJP+enCG/64UA75imXXgjHvANUefwDUc4Ix77ofitu3CTBTsDD+P2eAhvcfL
4Iy7bNWMSp/6FDjjzw2K0+cfGx+dAWc4Hy7DMlt1iwuccZ80oLdgW5YDZ9yH
jlonZhXrgjPu3xVe0TL9MgJnuP+wutC3YQcjcMb91Yy0zUi/NjjjrrLzErUU
kyI44961prxil7k4OONewuTk2b0gAs6479j4ochYUASccQ+UaTITUtkAzriz
s6oOyomuA2fcvd4+0d50nZGCnXGPzKKSGtKaA2f8OUwUBRgcf0pFwc64y917
2WcoJQzOuPsPSSr87NQAZ9yjvNOipu+bgjPMXWh+rzc5ZwXOuN/OoKfYU1uB
M+6sUsZ5S94m4Ix74JXx6xtDtMAZd1rDqgzWc0rgjHurVHP9aXp5cMb9leHb
mupKWXDG/dtGL6nUu5LgjLteehmzRpwwOOPOHKeafJmeC5xh7uDDp4rB4aso
2Bl/rs0zk2TR1GAEZ9yj1/nSMyJpcMbdM1zdSc0XgTPuvgO3g9bK2YIz7l0W
q3aVUzmBM+4JZifvmpo5gjPuGnNefge5NoMz7pGn+YdKTxqBM+7Cx+2+3m7U
Amfc17lcXcvPqAHOuH+mO1ZWvUodnHFfzMvPzmJXAWfc52OfJdQ3SIMz7s8j
9jtpsgmCM+6CdFHj42Us4IzXCQrsXdxYdDnAGfeoi3Rb2jJUwBl37W4ejrAq
c3DGfRMTa4v9IWdwxn3x84nVpj7e4Ix7ykXNmzp2XuAM6xa79n6uWO0Ezrjf
p++uPb3VApxxrxBuLAmSMAJn2B/dJ3Q4fpshOONuvv3iwospfXDGvbjatVng
qjY4417tKn3Q7J0KOOMen+P6Vfu2BDjjbrYwq5svsx6c8brLlN+vPppAXnDG
XeRkDxtLuDY4406zZVTo7B47cMb9Hn1s9HCgDzjjzhc6RZsnFQTOuMsKlcy1
hQaCM+7FWecXFi28wBl3+5jeu5619uCM+62XU4U8H8zBGXfT20IxH+TNwBn3
mot5CbslTcEZd8UvR2+7KBqBM+6cie7b3Ye0wRmuv6G6+v29sSI44x6+l/tN
7oAgOON1LCnR4N1ujCLgjHvX2vQzJQEInOE+9sbXNQ6IOoMz7tJv1288whYE
zrg3PpxOigzYDs6w/0uvjvFPUBg4447YdH9evxwAzrhfnzEUMLvjAs643zhr
SwleZQfOuFN7GzGabbUGZ9xN263mGOitwBl33kSKdsQac3DGXT/jI9VwFQJn
3IXivQ74WamDM+4BHo/1dl8TB2e8Ltj6YGSX9WpJcMY9c/E2O5eyGTjjHjBp
spST7wnOuFc6K4UH6oWBM+6Cu36+ynaLBmfc2a7FslZdjwJn3O80n3RT3boN
nHEfnhWqZqn2BmfcVc3PMpVdcwRn3FsaNDWs3LaAM+5n6vYxpcfagzPuhX8U
9ja0bgZn3EsWcvffyDQDZ9zFFi+O8GvrgjPueRqmDtZ3ZMEZr7M+KZ25mbRD
Dpxxf3y3zqUywBKccRcasnBY7vIDZ9yr/l7fGjITCc64J5hMHmt9uwuccZ8Z
2DG+n2UXOON+cDS3ca4wApxxL00RKdzBFwDOuDcFnHl95YEbOOP+YRv1h+ev
nMEZ9/27zA8t3HQCZ9zpz13PN0nbAs64b2g8aWTZbQnOuNttU/1tZYfAGXeu
eKt+2SJlcMbr1jvyk2guxSqDM+5GAbVDLBW24Iz7+K2rj475B4Ez7o/3X13o
dtsJzrgPTJ1RXxOeCM64T5zgVW6JSwBn3MUGteJ/zEWDM+65Kg6R+d+CwRl3
m4j3z8ZHvMEZ9yqFtfZXfTzAGX7+sQLmfiZ3cMb9luwaLg11Z3DGvaunypuZ
0w6ccXfftPfMi0wTcMa90+Klgfi0Gjjj8wDCg0LjhxnUwBn3typpRiG2W8AZ
d+0zxQVFt0LAGfcg/86ahIw4cMad8e135a6GFHCG65OsGJtGz+0BZ5gT1i7Z
83o6Fpxx96yRzrubFArOuIcVVl9b+9oPnOH7b3AovnneB5xxH47i4TJi9wZn
3FXdaHuzC93AGfcbyYssU8e2gDPu4e0Joi6nNoIz7r+0csToVmuBMz6v8kWn
0HO3pgY44z5o3a8iGuIIzriXfWW8tPd4GDjj/kd1UdFQNwGccR9zoj/GX7kP
nHEP+XuFtZp5Hzjj/lzAJLE2Jh6ccW+VH14lvTccnOHfm6iRc78/AJxxZ9ij
7c1/2g+ccV+ipvJadcMXnHE/f07V4s8+T3CG/uWLgUWRIzjjPrPN0WeMzwKc
cc9NTP0YfU4HnPF5Kl8JBYuqp5rgjHtcwfHl2e9O4AzXkSg3F17MCwdn3JWY
tbQ+KSSBM+7m8p+sryykgjPuO1a9MBHbkgrOMLeg4e9EskkCOOOu70wv034l
ApxxN6W/+1HBKgiccU81NWw788ofnHEX1j/66voXP3DGve33o1MKh7zBGfd9
8yrHRPidwRn34o5vNzeMWIIz7qzs23Mbt+uBMz7v93ZiyX42XwuccQ+dZ7TR
vuEMzri/PHTkPWf1DnDGfUyncnZONRmc4T4VO7+f2Va5H5zhesRt89uM5okz
7hoPhV3fqSWCM+6cXd2fbvhHgjPuFytVnp+mCgZn2C9WN+9+NDIAnGFO6r5K
udQEf3DG/Y1lbaflPh9wxp23IHh/3XPijPs9aYuao3etwBl3s20XFhKt9MEZ
n0fdH6xz/ur/OOP+sd3p7FgjccZd/cLUy7wq4oy7n7pCe/D/OOOe+9Hv++MK
4gz74yaoTi7+Jc646+gzntigRJxx5z4untTiTpxxpw3/mp35lTyfcb+UasDk
60iccZ8sTxTY70WccTficNojZ0uccbfo5O10vUKccQ9Y/3reLos4485ZYeTm
I0ic8Xlpz7hhC4UX5LiBe1x7Tp7BNDlu4O61UyxMvZAcN3DnKfvB8EyVHDdw
N19nUGC4TI4buA+p+ntc+p/jBu57fnLWW+iR4wbuyqEjW6b3keMG7olhAac8
mIgz7o66k49PHSDHDdzVKCNVC6fJcQP+uyqlLDxmvcAZ97XPGy9wXXMCZ5iv
WVjGaB1Ijhu4+4uoj3h80wVnfJ6/a6ftKmpb8jqIu/SBO5aFKeR1EPeD579O
j1eR10HcTfXnxKJdyOsg7t36n9uO3SWvg7gnuA3NcQmQ10HcUQqrk8Ju8joI
82BUT+dzbCGvg7hbz7a2qiST10HcY5u5neR4yOsg7jF3nGtaNcnrIO7NUZ3G
nIUe4Ix7/gET6jdU5HUQ90103xnNnDaBM+7C60c3v+/UBme8b0KxUKLohRV5
X4c7bZVG6dxx8r4Od7Xh1YNB7NvAGb7PrzMtq9+S93W4JxayMWaz7QVn3NsV
F7bW/yHv6+A+G7QqXQcEd4Ez7rKDU+LFweR9He5Dl185Z4ST93W4m5cztQqG
eYMz7qN6BY9annuCM+7n5X8YnTrsCs6474tpDmx4YA/O8Hhm8VzmPDNwxj2q
2668i1ETnPE+FMu1rQtuS+RzCu7f5UoPvHe1A2fcP5jS97hyBoMz7n62Kveu
U8WCM+5ZTWfpdS2SwBl3A23mn2KURHDG/cuuI8cfbI8BZ9yvZfgnVLNsBWfc
VVOO0juKkM8puNvV+6jPcLuDM+7Dt6yNE0tdwRn32C9GcolLjuCMu7FuvolV
oA04407H8fSAcJIxOOPO/pn3xKq3quCM9/XI93qq3UcK4Azz4cZUDzSLWYMz
7q/01Ro6l/3BGeYHGBXf/cIRDc7wfXZs50t3igdnePwTnnG/sDhwxr1y4mRZ
7uVIcMb9nthH+W/nyOduuC9f7/nPO3XJ527cA5Wl3v7NcQJn3PU/hTCoOzqC
M+4ePHE9Zdb24Ix7WXPh/T1MFuCM+9iPv/MjsYbgjPsqmTKG82xK4Iz3SWm3
XQp681ganHEPubo3R7XPHJxxp3JplYzl8QVn3DWSwv5oXdgBzrjLujClbD63
E5xx35nIefphRww44/7OpVFpbO12cIb5NO615VZbfMEZ9y8xQR+408k6Eu6n
jnKohBnZgzPuDDdjpCWk7MAZdycJJrvEXVbgjHvVlR6DU+9NwBn395fveB2T
1wZn3A8ekPjiUCoNznjfWYa4Q+Dke3Fwxv0kJeKkKpcpOMOce2WOgJ/8HuCM
+7tVr1Md27aBM+5dajuW1zpHgTPulfmJwbWFkeCM+52h3x+rI7aCM+7MSgO3
v/e4gzPuNhUHeHztyLoo7mel6Qe5Ki3BGX7OrRs/mtzdBM64C94bZbqybArO
uN/LfZFh728Izrg3evMLViSogjPu+Q+bAve9FgVnvI8vaWrAK7WWrD/jLpum
EZC7ygiccX/Cv3e78V+y/oz70w03f/7uJevPuPuor0sQdw8HZ9xZI1515vJt
B2fc3V4vhvuXkfVn3I9c9u7+TeMMzrh/uVX79NlBC3DGnY/60wGrRRNwxp37
UNWRuo3G4Aw/5+REXu6kITjjvpvngm24jyY44x7DHxzgpykHzrhfo987l9HM
D854X6T3zK6m8tOC4Ix750aa387bDMAZ9+T9cdSRbxzAGXdWE4+L0wP+4Iw7
e2X2TwnHbeCMux61ToOAcgg44672zj6s8JEvOONu8kDjauXGLeCM+57Wq6fL
B03AGfdrrxlywxQNwBl3p++VFhasuuCMO818kmjNLw1wxn1m4LNVepMSOMP3
2W47WmItBs64n3tgzpVbxQnOeJ/pHbP9lGw9fnDGfcMejls5yrrgjDsaHK8U
qrQHZ9zD2Z58vfXEF5xxf/UjZMLyYDA4435b16VAIjoInHG/qhy0ZXQ3OW+F
e6pICH2QqC04465WvfYXjQEFnHE/NqndYTuiDs64q+ypia+cUQFn3LXEKzLS
GBTBGfdOLuaK5ioJcMa9MJZ/s5Q3HzjjzmLyTWBkmAmc8b7dzU7P+i/yk/OD
uCtuafbkkiDnB3H3DTK2NLIk5wdhXuyEVrL3lDc44742Bj0vuR0IzvD4bv2o
/poAcMb91fBk5+F/x2HsjHubpZOS1W4rcMZdgyrDWE5FB5xxF110uvWjXAGc
cd/CFdFCtSQFzrjLL7wZPdQhBs64H+VaPux4WACcce+pNpfLdmMHZ9zZHgkH
7qmlBmfYBy2awRTwiRuccX/1cMk+eF4TnHHnm2cJGKC2BWfcw95Ouz2P8AJn
3CMXL37gawgAZ9yXr75obL7qD864Mz+QWLPB2A2ccb+ZNsO4LE/Od+NuJZEq
7iuoAc64Uz9MZLTSlABn3EX6zz0cUhcCZ9y/yxyeSlbhBWfcd7bX1iXzcIAz
7iPP0myll+jAGXf9NRzB/uG/YZ8M3leeOFCh8tKCOOPu+rw+oDCQOOPu9Kvq
2F0NG3DGPUHgs3tEpSc4406zmB+eokyccWcsvjbuLUWccacYVtFLb3MFZ7he
J9VpfKDbHJxxV3UWr/u4VxWccVc339vySFIInOH7NKryn/nGCc6496dmSlfN
sIAz7smhI4xsAqvBGfcnoxszdrydg31fuC/2XT3T7jcGznif/oLPz7bZufXg
jPua3z9cynmIM+7xDM1Nbc83gzPuRu2GOsUWxBn3CsnkoQPf/cEZ978jcrr9
H/zAGeaKXSiUYf7lAs64v794p3megTjjHnZPu+L3oDI443509Xu239+5wRl3
J/7wQ1vMyD5G3AMETNOlwmjAGXf3j803xRjnYB8j7jRetMHOZybBGfcN0+8U
nvEOgTO+7qFfrkhmaxNxxn2V21G/kAENsi660mnyfvqoJxNn3FulP5Ycn/cg
66Ir/aeIZZxQI3HGPSzhe51kJXHGPb4xar1bPnHG3XbvCfErezaSddGV7np0
56aPl8m+L9zrMv74vA7jJOuiK/0Uz8ftByhUZF10pRfNsLi+tZ2Bfby4ow1q
GQknxsEZ968i+l+2jHwAZ9yDWIoOz+t3gzO+jqTstMdVDj/ijHuW7vTAoQPE
Ge4Dk2Mbv3c1ccZ938Un3BmHiTPcP12Wk/V1KHHGXSv8TI2mP3HGfc/IxbPv
VIgz7k0WUs7Df8zAGXe/V7r8vmKK4Iz7MdmnCpcvsIEz7iUH9Nid62ZgHzXu
tawf9vvIfwFn3E2PvymbVRsCZ9yLkIXTqfQecIZuoepl++kBOOPrcsTH9B66
PFsHzriP7Xw9undGHZxxX9R783uXmTXZh7/S/SndAxL/3ifAfvuV/rT8Vt7L
ST+yrx7f3zltoxjTuC/ZP7/SjSr3DFaecCb75Fc6Dc0hdsHNxBn3N9o/Fgaf
yJN97ytdl9tlNDyHGZzhvsxfOo2FOibIPvaVvrxpVdygUT/Zr46vs3SMfVmn
0EX2pa904ewI5f2HH5L95yu9q65jZu23enDG1zlRegKvb5riAmfcC8x4zn4S
Ic64j5+9LL0gZQXOcL/gDKZH0g/dwBn30MPGZuG6xBnu63pi5LulEXHGnV5k
6w7hcrK/DvfaHZWNwyGm4Iz7jeKRn9NucuCM+xVH0+1y02SfOe6KWd4bahI+
gzOZXxuz8yzlBTjjfqIr9m1+dQs4w/zIb5ETJ3rqwRnub0u96OciXwLO+Lqx
v7EZ+1nbOcEZd+neqD0z9GrgjPsfd23n0goLcMb9rkOSmb6HKzjjfqL+rvby
Jl9whnnhumzm2295gzPuoR/0sj7EOYIz7kdY98a6mZqAM+4XzY3v76uTAWfc
9Q15/L0bGMAZ7n8acNDkJtcwOMP9PcM4GZ+23gdn3J1OutWf76oFZ9xLYio+
0/gUgzPul9d/aNe8fBqc8XV4bLd3fZoLIPtFcXf0t7KrqyL7RXE/+SLLaeIN
2S+K+355bVOHArJfFO7TJ2gk9suY7BfFnVXDbTrtqCc441698YbTBVoHcMbd
ROjutrRhsi8Xd4purd1MhxQ44/5zx59jrjmrwRl3lrwy6kj1IXCG6xENU+uv
ud4AZ9zLvcQ3avcUgDPuiTw7FIyNToEz7m7S07kLwfvBGV/XWNjENHXqMRs4
4/5m+KykmJgyOMN961KZcjwfmoEz3FfOPleftdQRnHGXsgx8/C3fA5xxTxUL
96xkdgdnuI+QYPOFHjd7cMbdTM/97TN7BM64NwZFq/vUSIAz7pmcTDk7olaB
M+5cCXe034sPgDPuijnZRUomteAMP7+tPvtJqSxwxt2TOneqs2QfOONuEO9P
U38yCpzxdaLKzrNbRzhZwBl3hyqFnK1j8uCM+4CX7yH+fSbgjLvvl18TO/dt
AWfcA7tfvp+56wrOuE/efHLSxNUFnHGvlXnt085tC8643+kOiy8wMABn3B8M
vjjIEyYGzrhvVrKsMDGgAWfco+/SmX85+Rqc4X5bk6/8L8ZcAWe47t9zrFIv
bT84434o99PBxpwIcMZ9D+3Lv5eqvMEZX3drtbo++M1OJnDGfTGK7gJLpAw4
4763OejQTykjcIb73Wzh7eU0twNn3NU37A/Qf0j28+M+4VeiUpdB9vPjXnO1
I9zCzQqc4f41cfVOjxp1wBn3yZcnBZ9ziIAz7t9OvLNypKECZ9zpapW/eTq+
Amfc9S5xsl6xKwFn3AWkzjXvp4sDZ9ythzfQ+K3yBGfcgw6H8SoMWoAzvo55
+y/m4ai+1eCMu/qSvqDFbwlwxv2hEs3dxosG4Iz7i8njhmH81uCM+912queD
T7eAM+7LO2V6I27ZgzPuruaffuXmmYMz7sxprEs/1DTBGXdpF7ek67s3gDPu
1vQ9yV7583B9Iu5nb62XobPuBmeYP5ES6vn4TAE4437oZXptsMkOcMZdZve2
fZcsLMAZ9012QbelBDXAGV8Xfp/zmCytBy04416+LXKnVp4oOMP8/6k1T634
dMAZ99DZFj7jLnNwhvtsOywOmb2xBWfcrQw3L2SusQFn3DmtDEfW0JiCM+5P
Fzc+9upUBWfcmQ+X6iX95QFn3J+P/TZLDPgDzrjf8S82GP76HJxxn2N9nV4q
ng/OuL+R4Iw+3hUEzri/qx65uk1fG5zBZ9Mof/4zYXDG19l3qdTeHech11vh
vk+tYCTllyBZf17p70sZjx8dUifrzyvde83wz9HnJmT9eaWbNzR/7kqyJOvP
K12xgf9z18Imsv680mXHtuS+KkJk/Xmlp7PUHZexUiLrzyv9oc0yDX0rF1l/
XumRq0N3x0T/BGfcA6dtxYwrn4Az7kW7C+jM53PAGfc31w5mK5T7gjM83ndx
uLhfBpxxF2MfoswEM4Eznlsw0Hcz2cZ8Dq4fxF2Qu3y3qSkvWX9e6TXtUY9C
mVTI+jOeDxz07iyzHCLrz3hO+FHW4eYLZmT9Gc9Td8zWsw8yJevPK706weMl
i6weWX9e6ZtPSpz9e1iGrD+v9JkOdPzdTzay/rzSWU4ZbXe8S65rxp0j41Ly
3NRDcMZ9N0v0lYsJp8AZ94jz34yurXcHZ9xF+OIiObcJgTPuJgkW41vYZ+A6
ejwHYlXfKVNJZnKdJu51OtzMRQbkOk3c6++ECbpsJtdp4n7TR2Lt7s86ZP15
pStuv9f3wp5cpwlzYoMjPt5wRmT9eaWryc0ott/XIOvPK91NwbnfdJc4WX9e
6Qy9HU4SMmvI+vNK5xf/OXUq/Cs44x56PTHZsbUZnHF3uNic2il2HJxxp/G+
nEu71xGccf/65bbpm43rwBl3/UNDZnSfP4AzzP/cdfAGXeYUOON+5s1N5ns9
rGT9Gc9rfeitsiFOgqw/47mst4Zmmr3UyfrzSp9WWXfwtZ4eWX/Gc0TZZ3et
rtUh6894ruOnfN5X/Cpk/Xmlm1FH/ghcL0zWn1e6jIj97UJ3OrL+jH8eiyc1
LxaHwRl3atqntOzet8AZ5lLG7aOKic0AZ5iT5Ck+4TRjC864/7m0K+DRAxay
/oznr1Zumrkl1wXOeE5JWVooR8Phr+AMc+12/toweYyBrD+v9I+2zUWCJ4TJ
+vNKd+/R/8WwQ4msP690cWsV+hxaDbL+vNKnXnvkTXWrkfVnPAdP59sB3q2y
ZP15pR/aleVe1MND1p9XuopGmItYAhVZf8Y/pynD5Kc3g+CM+69q3zeyrvXg
jLsErTz/1nep4Iw7a82METq5iaw/r/SkqYS3rn2ryfrzSn9Ald5qqfAAnPHc
lzeVWxuvuZB5GjD/KvdLCf1ZGrL+jOfUnSm1WF7HS9af8Rw2X6sWhX+ff2H9
eaUncTHVdPYokfXnlT6VtJ46eLUiWX9e6ZtKxQ7mLomR9Wc8ny0/6BVLBwdZ
f8Zz0kaH+Op+zcI8Cpivtd916q9nHzjjrvxUw2rpRRU4416t+JNX42QSWX/G
89M+M5pqnDIm688r3e5hukkADTVZf8bz3FJ6qTa43gBnPP+k4kTQUlLzADjj
Xh+QHcZeMg9zIXAP1KzRpS3hAGfcT4VLcVyqEQFnmNsqfOhyTakUOOMez6eR
0GwnCc645+4OWXPqxgZwxt20RfPa1h9rwBnmFB0OjzI5+gOccZ95Mzii49UF
zrjfKC22Sft0GZxxF6Bu3TqvGwvOuFt623DcztEDZ9wPSCgqrOb/a4Cd4fvo
FYVNFdWA8yGbXXw5OUWU384dyllXW8FZ/Pvwy6M5dZRvsdekhmg+gXNo7VEB
m4A7lBcT3z4m/lkyhOfzynwbsUcCQf1RLGQ//4ZJp5Dkx5SpbKl1levWgXP8
3gd/NwV0UKIZBMydBrjAOWm6/flp8S7KLEuLcH4DCzi/W5kn1BV7hO6qJi04
r3G8FB3L1Evpurwv7LnOPDgzHjz8MCj5NUWl9PraDXp/wZmhYed656m3lEsl
fU7ZT2bJ+7pP3iEbA95RAh/5rM24Tc6ntHJb3NLsfU/xzT+xfElxCpz9i3nT
Tol/oAixtXldMR+B5/OHJpbqNd4fKc30O0/re76C93UDE7NUC50jlOlor1bJ
fT3gHE8/IbOT6RMlrXJXQsrNn+AsIjy85bvJZ8r9CeuPehVr4Pn8RPt1YmDy
F8own1S2ijg/OPvZdxQPNHylVHQqT0g/EAVnjtD7HY5T3ygzx9LehGaIgvO1
1Mbfz2S+U5LY80UvG/CDs//5SiGzgHFKguYBNQFqdnKdZl3Bpjt5ExTRjqsW
aasZ4Pgc/yw7SqN3ktK1m6s8TYOs1yX9MOvoTZ+iRJg3fLnfRgfnrSr3NH/I
Ep+mbMx1OI7YaeH84BiT3m+blmnKG+HWpmT3BXCWy7nOtMb7B0Wdx+iQ751p
cA4VUxZqm/tBYYrs1qos/QjOIfoSmyjqvyhNIdkJil+G/uf9xkXP+c5flMVj
cT2P6anh+MzkxBfVGD5DiToi2CvbzQ3O3h9OH4xh+k05T1f0KdeOHJ8rwlnP
KZX+phx1O9UcuV4ZnGnnDlWNmfyhrDrr7fzzlhI4O6StelA69IeSvLpR6aO5
FDhXcKT0BSTPUhS+Wmm4Um0A5z8X/o4J8f2lZC58KMwIJ59TNOV2Ug00/KUk
nbrt3/aKA5x3NU5w5TrMURICDl5QiuQA58sm22Qcp+YoZRoadp83sIHz6Ith
A7bMeUrMBqpNy+P0cL5bwtN7yzOZBYoEz3rRstFFcA788jo449ECxZwqvAYF
TIIz/to8unuSyeA7ONdkbipuyluiZEs8Fn7ktBacV/E9uBGvs0yh8R7KqL0v
Bs6ulw061HuXKX4/rprZWKuB8xbegch4HSq0rkpNJfSMLjjv8eFN7E2nQoX8
wmbh8rrk/GCJ80H1XipER6fMOCunRvY/j506niVOjZxkmLj3vZQC51mVrrNT
0dQoY8Y+x6hJmFxvFc9SbNNCjcaO+pg8axMEZ+27VlWVrDRIJcCxIY9TEJzt
Vh26weRNg84u/kwspvCBc6Tlo/shlTToT2KaU8dOTnA+foK249EcDXrLE3Dx
vSkD2cfYi/rELWhRmOlq7YyIv+CsGHB7bGiUFtnQvTDTvvEbnJ3KZ2cM1Veh
Z/p3dcUa1oNzyqQGVX7qKnTdpG/2dbYSOJdrxDDNd65CHNs6BgIeG4Lzi8Qa
LjchOnQrRyxrSoZ8HpxvGRdsDKdD3BdYWpo7yOdBMQY5mXVNdMiiObiWOdwQ
nO1sQtRimFajdlbF+fWDZH0j9nSJQZfralQpH2F1JUwWnM+8/WiuVLoaZQp7
h+zRkwbn68LCWzJnVqNtrc1oUyHZj9QR7OU5ZkKPtjhkVt/LESf76yrPBVtk
0SPzeG9KY4IgOK/92RdZOkSPWD7WWnrOcYCzvM66RDolBtRBObF25i0tPI8r
Wo8fb2lnQALDp61d8qjJfv41HWeF+BjR786hbeKHRcB5wZ6pODmEEb1z3uuu
5ahN1utyzav6GxgRG1dzi3gZWUeyHjxwQ4eOCb2gvme2Ud+OvA6K37+f48CE
nGXdXs+9swXnS6HLz2YKmNChaweYy03Iet2zGv0+hykmpFeob/P+iT44//i9
+0Ot4RrU6813zIpHHZw5DBrHWDPXoI+ZB67s9lMFZ9X9v2bC+9cgl+DC3qQR
sr/O8okK1TOZtSjt1y9NFhElcA5ljWCSjV+LQr6msbxflgTnQ06VXBmP1qLN
+78OVOUIgHP5+a+Cn7iYESWTLllugBmcpaUD1QprmVF9AWvnvs1k/Rl/FSnI
CtfhkyXXw9YPmnvb/Ps9DG/qnWY2BueiOf4tTXksKPBPZVHtAXtwfoLcPPnG
WJBaWPfjSiqyzj+Tlh0cr8OKzswv5j88QNb5N3T0RPamsyLed0NXWcTJ+RRL
TvZE9V5WZLRX9nyXDDnffUO+1NiCgw1JtNRnzcUYgPPjF63m8TpsSDd+ofDp
O11w7osesS71ZUNoNavfGj6yX3R4He2W3nQ2FBH9NW70iiY4/7wh4kJXzYYC
GvbZUf5zf6IVZ1pP5Kney4ZYfbPXdNwTB2fOZW+/gEU25M57tXT40jpw3mKW
F9ZixY6uie4Xzf9Ozg8Gf7kVORXNjmZ0PlLeNqiCc+KRN7FC59iRBPqTO9VF
zsOeUJxNsGlhR4JsutyqVOR8d1HX+r3JX9hRKmP56SEecr775k6Ng5WsHMjg
xXuxpzfJ+e4ObsfD/ZocyKtvPP+dM9n3NXwr+jiTNwdiLbF6XjS/mewr8Dp5
WucgB/rG9Gq30TNT8nmQuuZsSCUHOvplvwTtmDHZx1j8/EJODwd6W7TDodOF
7H8WNR8vejTHgWQCBQ8sdVDAWeXbmvIZEU60pCnYZh6mBc5GmbJV4hac6EjH
fv5WGgVwtle2qHOI5ERnl/6mHVQkc05O7Eq7U3uXE7253FgZqs8HzsW8JfeH
RjlR3EYjO+l+HXC+2fSgjZWZC1Gor9z2f0r25T73+fjMUJ0LvXssPHxH2Q+c
P9BQd4d7cKH14sYsR+O3gvOfEqG+vFQu5OjVuLb+ezA4r7EwHGgv50KvaPKX
xy6T/c8i3z0/zHVyoScfH817xDmCs/rxxE8ys1yof4PBcbvN1uBsqnpuzFVo
HXpfG8CaVmEBzo6vbkylb1yH9nqY7n7TS/bz+8b3zTSEr0OLLF9teFLMwDmC
//fc6Ol16JXA5Tn2cxRwTrnLRbWuaR2isK8Nzj2kBs7H/dToTIf/PX7APoTb
gsw5eV4ayVqosh7F1l9dc/Ug2c//0fI4V5freiRmRMn+q0H288P1QA77XO5e
dAHntSefCSqVrkdsY7sknrcFg7OQ+piY9/P1aOeNkZrGtRHgrNHHKJM5sx7N
LNhzpJjvAGeLBGnFJgFupE5FxSkwEwTOPhvM1cZMuFHJo+GYku/kuono5iBt
vjButF9QZKvqsS3gnBpwwMAiixtVNxeLP2uzA+es1UXG8Te50ebn7tpS6uT6
lAvlLealQ9yoxW7Tw9klcn3KVesh6156HrTInSSZc9gUnO9MLtnTKfGgPqfN
q1rddMC5I2uDi7ozD9pw5FFQgCmZc7L2jbtfVjEPopbKn2TLJPNkhJN2B7e0
86CZJt8+dq6N4KwhlBs29YMHVRzYX/hK1xucLe83RArx8SK1qO6IQ9zh4OwV
9CrWxogXBRuqVXY/jQHnGIZfCckhvMhNUviUrxm5Dii9gmNv5XFeVBvM9Xj5
Uxg459uoHOxv4EUWmi8kBdj9wLlm2vYw0yAvCghrPvM9ygWcm0/vOK5Dx4d+
POA5FiFGrrfq1Mo8HSLPhwqdLlr7eJPrrQbeVpzNceBDc9fvTzifINdbjSU/
vfAogQ8x0MZwrrpFrreaF/5aNFPAh/oXxh4L1FDAec1D+nLxJ3zo/P4vZ9v8
yJwTSyazulRufpRPt3/J6A+ZJ+N9NeBGrSE/6qY+tIfZygqcY+xS7wwF8SPK
m9DJ6iJyXVvGz0v3WTP5kdwF7nTaz1HgfD77XpthPT/StDfW2m1Prmur1Rl8
Ft7Pj3YZGzzjLibXtbUOLHTl0Qgg3TOroqjWR4Fzfwp/X7uMAOqIo5jSvQ0E
5wlR3YE5OwG0xTRgJu+IJzgvtbp+kIkXQKy7Fb6rl7mBM8u2uE+uFwVQVGL1
hU0V5PpBvrXZY+mPBNDXb02LKXpO4CxTXT/VMC6ADtSyh5Qz2YKzzpaemVGu
DWjj4yFWTS4y58RiZnqOS38D4j36/iFtNZlzkqGnRBdzeAOiYpy0bbtOrtPM
G9zMVFi7AVHKdFVZjcl1mrX7trN2vd6ALHksvppKkes08VfVmx+r4vXJdZpv
2sp5laQEEcP75c60Z+Q6zcnQx4LeNoJIw6nJVewXuU6TluWzWGasINpFNUwl
KbcTnHlr6WSa8gSRbkNgzoXMEHCWdxRXHHsgiMSiJKqX7vqCs/4fYzW+MUFk
sfOnRz41uR7W+pyftgWHELLr++3u9YRcD+tqsNcgXkcI1U0tFJTfItfDbh26
YFzqK4QadXaMtzGROSdx+++Y96YLoezWrJM/RMick3TJAWu6aiF01eGyksdj
cj1s23Zel4BFIaTnNSnzzpBcdzxOL6BowSGMgn33X95cQa47jnerqBmSEEYF
pyzvqfqR645pKnTV4nWEkbaGR43vpnhwPrrw5DrrZmE01N1ZpRxJrjvmtnHT
LvUVRiWnJgxeDqSAc8HFL7cMdwqjW7Mbclh/kuuO5afjDHrThdGgyDYzocEw
cL5uTN8cfl4YXQrSm/Oa9Adng9PZxnTVwiiAbeMGtS++4Nw6KtGad18Y1X4t
/XNLkVzfbaV13Vy9VxiFF/UUt98j13f3ZJg+bf8qjD5KUt0sUSfXd3u87bEO
WBRG7+TdghZvk+u7h+UCXsyxiSDpwJ+WPcFkzgnti30vZbRFUEleHdNLL3Id
/VFhdpcWKxFEO2m29foVch09d/SlN64+IuhENu9TGYbt4Fz4QMlzKloEGW1V
jqK0kuvo5dbdG0xPE0GpGwY73nKTeQUNwTZ+QudEUAbz/ufyqeQ6enTj3XDD
VRHkcISvMFphNzi3M4YH27SIIHfNyJnh5B3g7Oix8GX0pQjaWX61PbEwEJwH
Ko+EJX8RQWKGZ2l7PMi8gsAlvgmuBRG0uouj7GY6mVfwzfZKZCWrKDLLrQrk
oiJzTqILdH6aiIkijYnG6lUdZF7B3I/Hsf2aoojqKetmngIyryDV1HU22lIU
NUgXm5ltInNOCj/vWiyIEkVWQycy0HsyF0JOZ/VenYOiaHXZD49WVjJ/o+Hw
GZquXFEkI7RTprqfzIVAA+IHQypFUWchfVraLjIXAn99X/A3+5s6mb/hlGJy
OKdHFAUf1hDpOkbmQrzv7F6r9FkURSdtCjtVROZChIr6H380J4oCjkYYtawj
8zd+xkyze7OIocmjrmwvKslciMTWvadnRMSQRM/Ny1UaZP4GLTcbd6aGGLI+
OGjMbk3mbxwKuXhW3EIMFVYa0LRSyPwN9luKAk2eYqiUz8jGr5zM3zi75u4F
h0gx1MdJ2Z5+gczfEPHaLDK2XwxFXd6zkUOXzN8wotouyVchhrj3rPk59T9z
Tp7az5fV3hVDN9cndsj+zzwZp6LDchbdYiiCRpf3z//MORn6xVs1NCqGmBMe
rHr+P3NOtm0sV47/K4Zesh3pVfyfeTK/crTrWJnFkVu6+nLDHHHe87VNo1RY
HLF7qSudUCFzThj1XG4YqosjQ6MHh6K9iXPW0U+6vebiKG1O/QL3T+LMNxh7
J9xDHAkPsyxUehPnQiU6RBchjri+dH+uCCbO0vtO389LFUdLW4s3BXgS55pu
MTP1bHHE+jym/lMtcdYWr2trLxdHoaGDX57lEufmWGPLgDviaNjTku2TCHH+
9xnNNmtEHPVUS+7NOkSct4VOdcnMiqOc+ZHaT2eI86/bKY4taySQnf8c50Qk
cd7DzNrnKiSB4kasfpuOkeczg88FtylVCdQqEFCydydxPlWjMJC+UQIVU4cv
P7hDnDfQ3PEWcpdAabcvnOt9Rp7PZQ7WHxrCJZC1y/NDYZzEWaWkP8BmnwQq
kaPw6EQQ55u/Qz+NnpZAFRZe9vXd5LhhtGkuJLlMAtFK5J+uf0mOG21nD41x
NUkglmbXmjdbyHHDdoxnR+ULCXTh1y2hq6PkuNGnXzZlMiyB6gzvLJ88QY4b
Pse0Yvp/SyCGmYIdETTkuJGi4hzPJPjvc8NauTut5eS4wbB/dK5ARRJ9Ytxx
9lctmSdz6uXOZB0zSfQwNTI7XokcNwQlV1F1uUqimjWF3w9WJYJzadyp1JDt
ksi0OLHmQC45PuOvnPJNO358JMfnpv+cljkliX6sfp8tn0aOz+bbjZiUSiWR
aGnhBekUcnzuvNN59NEtSUSn6+ThrUKOz66svqzezyXRJUbpVdsKyJyTId/J
kzMfJJH60xv2vJfJ62DwtT1cmTOSiIPvu+wqROZ9TdCy5IgzSiHZrtDEj+7k
dXCXUz5vk4AUSqzdOJNWSF4Hly7L5zkoS6HXE4t3P86ROSeCllYFqS5SqEcp
oqNPgLwOlp5/K8YXJoWuxNmYP5Qlr4Oq49su1+6RQnsDv4RzrCPzZJoM/0pb
ZEmhzZUqdzkv7yb7+U9kVAyVSCGGBIb0l+pknkzXB27F+JtSSHn6BXNo2l5w
9lArrWHtkEJTIsc8HqbFkf3PBzTVSoek0Kv1r7nfzpL3GxG9rdcNf0khn8ZK
an4v8n7jj5STdi+9NHqnZ/f7YS+ZX5eye+RWOL80ejqe/Dtgxguc6dpjDOiU
pFGtc9NymgeZX5cpQNucZyyNUj7FavXokvd163ZkGas7S6PdWXUmxjTkfV3+
PZHW9m3SSH50v3mwEXlfd8cfPZ07IY0CFFbbh3GQ93Ub619YZxVLo/bEF/VD
iuR9XRedzwuZG9Io1VZJxdOPzAn0dJmwb2mXRt6yed/WsZJ5MqNlyS9d30uj
mkfKyUuje8A5cm6ty9QPaWTa0u0vX5AMznNWeW/SV8ug06u2WrAwkjmBB/Pl
PIX4ZNAxaR5v/X7y/pll8tZgg4IMeqaWUmsoRt4/ZyNLPxsjGdT+uaeM+SaZ
xyiY9WZ41FEGHZnwnU756wbOxcMhwckhMkhwana35gT5nCKvMfuFK0kG7TwQ
PJAXSz6n1Kelh1Uel0F0ralnA6bJnBP91+snTIpk0O04NfSEl3xO8UzU+Bn9
VAZ95PFgc39APqeMPnsYyzQog/oq777h/UbmXkYKOs4WTMugtWd+bfDxJnMv
5yKGE3ToZFGH6fvHbMpk7uWBlujFTh5ZJFGfRSVBR+ZesnLS7A2Rl0VJlJr7
TwZ3gzNcD3TvmOKSDZl7Kf6f07AOskjLUJBWbFcQOF+lr6FX2iqLju+4mVMa
4wHOGm6Uw48S/n2/tiLhDgsXcL5z5fla72OyaDow01uxnswXNVnwOj5TIIvc
vqr9cd1I5ou2bx5nz7wui9bdyb8jL0XmizpcTDot/kQW/TxlPicbSj5390+t
4W4akEW3Ixeqos6QOSfzp2QFxmjl0Lcrfcz+XfJkbvzozQup3HIoZn54p+1W
8rmbVctChE9ODv3WqBNedCSfu89lvC6qNZRDjgL+2j82k8/dYm+3SlpskUNa
gRNK+8XiwLlK7k/ZUJAcuqLqpLxDkMxx1U5Ok4vfLYc+/ImvmlAic1wfPF9X
xZoph9QZHvJ2r/MHZ2vhEuXSS3LIbnSNvHyZMzi/jFKvM6yXQ3sqZkf8h8m8
XM8HDzR62+RQr6DG+PcEMi/3I5fDjfB+OdQU5jTR5Erm5YYGf9Slm5RD6WM7
36f+IOtIPxqj7uTRyP97n3s2MZ6HrCMlMFIj9fXySHpn88gFKRlwPvef08gG
8qjsJZ9D0FYyT0Zsqaptzk4e/XzXnW2jSubJVNkaWmYFyqOWN5s0pul8wFm7
oOOZTLw88pWftY8xI+tI93942rYckUeflV8d9j5G1pE2m37vcr0oj3akOYaO
lUeDc9+ZRMepa/Iow/yN2TvlUHK++zNTX/ojeWTFpLVT/YcXOI9pn3MTeiuP
wi09i+a1yHpdzGGZgYZxeeQxejKp9MJmcJ7vv+FtQ62ABjmKw+MkyPznfQqb
PoxyKaDifTwX99RvBGfG/5z+lVZA323o2bVlyfznk53Bn7j0FdBwjFeFnx1Z
F+UV/R1SaauA2JyaLt20FgNnnVauHf27FND4aerYohNknsz99cVT0YcVkN3l
C/fojpmA8+b/nKa7oICMru5JKZF2B+fXN+/PFNQqICqf8SO1VSFkX8GaLfE6
rQpoXdtYfk9iJDj/dzn5tQISSw21lNsWQfY/V0Umh3xXQNGj6b02X8m6KP5a
dDzkyd5sV3A+bH88NYdTESW8ld9y+iZZf+YoEqRTklJE9j/FY+6nbATnc7+u
pj/SVUQXn1syFTiSeebCGw2YvG0UEXvosciMcQTOZTnPjs74KaL3qy4JXGYk
88yVv3qwZsYqor8eb3UFi+XB+Ybu2EnxQ4roSUxyani6ADi/fseY41CjiGbc
D13KiyXrz35KZ3nHHvz7ebypGA0uknnm3/dK56X2KaKurxSLSCMyT2ZXd6Mg
39h/5sN42ryjIfNkqMTNC2qXFFEgA13hPCuZZ34ktlfMgkMJOT4Q+h1aGwrO
69qCLg9JKCETNZV3q/f4g/MlnhnpeB0lNHS1ctrXlKzzy4QeqGDdrIQKmPYt
yoRsBOfa25yKpb5KyOVXb6xTPgJnHeaiGsOdSuhr6bMPRoEG4Hzvv3MzlBAl
dvJTvAOZz29e03I9/LwSErWx7bfXJvP5X1Dba9NVK6HzVkdnz8aS81YuDkO3
8u4roahp55dcdlzgHPd76V77VyUkZsMlp2lE5slQbTpmHLCohFi9dGIFBvTJ
PvNnx/yTGZRR9KX3B13cyDyZ/3NTRjXSM0dYLcg8GTtBwVEuAWVk5JEUdjE8
BJz/z1MZXclyf/tZdSs4v424urVSSRm5NvtN/Bn1+f+clREHZ7nXOyoynz+w
Rf+bicm/f6+4bwDXAXIfBOn/+v87fie3ya1a0gLn7xzPtvc7//t5lDvbO9LJ
/Saq//t7UUZLkeUpvG+UwTkm0GMyOlQZ8T6z9StolSJztv/7+1JGDqfNZCVl
yHnY+evfopj2KCN7OcXplHdrwPkIfcKvgpPKKHvgOv3F1WSejN1/f4/KSJY/
Zf10Lzlvtc6NMU6nRBl5GTx0b3Mn563e/vf3++/1/5OSelMYmSdz8Uru384b
yijGuHY6/Ae5D0LQf3/vymiU9sin1hfkPgiyC1JJIc+UEUXhU7tNkCc4T/53
Ho4ysiiQvKQZTM4P4q9b129n3DmoR/Yj/ff5o4wkdfd+ieVWAWf9ixv35fxU
RopL9TZRj+TAedn8P88rZXQo0dGgYi+5f8qDqVe0SvQqiNft3tbVF4XAOf3s
htY5NhW055rmzHIF2VdgZRyU9ohPBS0v3bVL30YDzv2nfjF4K6qgCycjdPYv
8oDzJX39pzLaKkjlx0g/zw8tcA4a3X9kxkgF/bCI3vSHn8yTkT3Wbt1ipYIS
lrfe1T1I5slManKyZDqpoBnHmx75W8k8mevv3V+4+qigEL6ZqG9byDyZhIzC
E+LbVFDP5B2VTEUyT4ai8s1+KloF5Tq2jb/xtgTnVW9VOJuSVJDnVburjp81
yfnu1N0v09P++bwNo5Z2J/s3jsq1nHE4oYJu6KxLXWsiCs62LxlchM6pIOfJ
OapVywLgzJlsxzNWpIK0OxPFJEy5yOduidw3DVf//XtjNqJV/fTgfP75+3Op
jSpIqWlz8fXIP7BPRk44YgNfuwoao9u0TryPzDmZetwwOPpSBU3ff0d16gOZ
v3E9auli7aAKajvUVfi1m8w5SeDb6Jf85Z9/rs5ZRx0yT4byIFPU4sc/T9YX
tNxHyJwTuu2vhrkWVBCXvc9caxKZc/KUa0PJEJ0qKsqa0X/9mZzvPn4nMLiS
VRUda0r1Xdy/iayLBldKxfOqIrUm8/qnhWpkPxLrry8mYqpowvDKU99yct+l
gUa9K6wKqojT8a18dzI3OF/03R/Wr6mKoo897R9kIve3CmBsly9Fqkggt2xZ
mZERnKWucUxEW6r+5/oaPQ8vsu9rzN292tBRFemYCcTvuPcdnBMrv6r0blVF
79e+EZE0Js7ISeVnQZQqul5/U/WCC3GmW4qvD09URe2uVxyZBYnz08vNsToH
VdGL+dm+gRNkzslxWwYtuuOqaFcs2/JxPuLsNGs725mrit7mLDOPMBNnvoKc
m3mFqsja4eLd11rE+b3F+4SQSlUkMt8W+3kHmXNS/ENSX71BFVG2vzP1zVIB
Zzh/lRtz5FYOHzjLmzbcbX+iilAO0zbhD8zgPPl9MSWnRxUprY3O4LpF7tdW
d8bMKOCdKsrqO3gtrnMR9jHGGWbSKH1WRRIZqkGPtpF9jHqfXz6Ym1JFz0zG
mw7Pkzkn7dqBG7NWqaGjDvy/hn6R+RsnPlTQe7OoocktuhtcWIiz0+Gfj2V4
1NDxoZehq5rI/A0+Nb3DMyJqqKu9+vGgGnF+359q1SKnhoIFCxcd+sn8jZID
T9dmaqghdCFPZdNzMn8jVIHjuStFDdWd8lCYe0Hmbyj1uh0Xt1BDbF+8k6Jb
yZyTX3sK7Ka2qKETD7IcLczJPJkbUl/ZmzzV0MPBqc0569eR/aKdyj3pwWqo
ctULsyk+WnI/oN3xpx0i1VDVaG1BUMhf2JdLK9rsJJSghhqfd3l+FST3K2x7
Ss89tl8NMYYf2ST0aRicj8TYvm7IVEOpjVbVQY494MzfOuhuU6CGvtDN3Umt
I85D4ZICfBVqyI+16L5nB5lzUrJ+x7vRejW0GHaLbdyPOIfeu36h9q4a6iz+
uJt/gMw5UQpZ9El+rIb+KE2GVV8gzjNsZiIW3f9+j0nFbpknifOtm0c/cg2o
oRHtmHqWaOKc4v+yaGhUDXkd2OdjgIiz6RqBoMpJNcTeev39o24y54ShPkAy
/q8a+ujmfyapkh2cn3lWfDahVUe1mUtV5SJ/Yf/zcbqfZazM6mgdQ46N4Zpx
cHao0g3tX6+OYg6829479xGcuV1S5UqF1RFbERN35tdX4Ny//OR7tKw6Mt6i
lEGn2grOYfZuEUyG6kiq8eibnC3EWXnuknKvuTryzmSwYN5OnGcKv0wX2Ksj
m7YtqdJvyZyTW1bKdeEe6ujkHYFzR1yIc8qvuJ06Qepo8/mY8YMmxNks/54G
XYQ6emKhZ+GoTZyZNtL/6YxXR1ry00yyP8ickxcTNjfyUtVR04Vjr+Mqyb6v
0znZu0OOqqOt7pWdPlEKZF0UDeqqZ6ujb3fVdr+iYwVn/LV5Qf5j0/ZpcH5/
MvxOe7k6yl9W2P4xdAici3Sv78mpU0dpQfIM7K4vwXnr8AIl4I46quwYnogz
agNnuaOm1Ept6qhsWWbuZ0ADON9+17P/0Vt1NGgR4T7WQObJ7E3jN8saUUei
DwOGzbvInBMzpYDV3hP/XrdL82ZGaIkz0+srbTKz6kjO8+6nnnQyT+bF3h8Z
M9QayKqsnG7kOtn3dUZG17JljQa6rK3p+aSa7Pty7963JnOdBnIVTTnwSoM4
CyU+eeYqpIFGSqOda7+ROScjYuzHxGU00IdAL8F968k8mbJnrrZTqhpoz40l
NzmWteC8PfYSW5O+BirveXi+auQrOCsJfulK36iBNHU3/LJu7Abnn4+UTjnY
aaBVZgat26RawbkxIs5RyF0DdZ9ju/K0vRGcE3nurRsL0EAKCiNDFU8ug/Oa
UJvc1DgNlO6/7XtTD5kn08mR7WazTwMlF9dsK5omc07O3H7Hx3dEA5WxJFEX
dlqCs3ugxMDoaQ0UrHPf+a0P2V8nxByeX3tBA/k/usTrPE7mnIxer/dOLtNA
SGFzEP2QDzhf8V4QsrimgR7W8++4pkLmyUTQm37gatJAWrYvB2bHyJwT9Zoj
hUOtGqhHU7isLovsy5117QmofKGBstuuOG6/wQjOTdT8EvFvNFCTcFLg+LkR
cE654v/JZFgDpalvZgsXfgTOJg5XSlnHNdDvB2dTf+y+Ds70C9Mh/b810Oyt
nYWfkkvBub1YR7aUShMZijcmBLtmg7PH78eVhlya6O+Hsp18ZWSejPBFth1M
gppIcHvfk/5nZB/jqLmrUq+UJiqRXTNbg8g+xitTF6cKVP49frzQKHzIBZwj
zn6uDdfTRMMfTMwtX/mQfV/GSjE6ZprI6AvH58qf5L54c992qdPZaqKiU/32
/p/InJN7p+7OdLpqovPGExlWVcbkPo/6qxvz/DXR61cPL4p1S4PzptHN8SHb
NRGPtua46zl6cF577IyO+q5/75eXpwPszT+AM3xeeTgp+mn+Njifei/e1H7o
n5u+2TmezBJwdsnYnpxzShP5773+RT8kG5z5VeoNA/I1kWxglLvChoPgXJFq
0jJXo4neZRcMj+uR/aKRckdSH93693ir35TFUDJPRuNlt0nWQ00kJZAhFOBD
5snMJfHReT/XRNaSqn6FXGSezD0J/0cyrzXRwAJVRkeoFzgffF6ePvNBE4mw
5V/7U+YBzpZx05taxjTR7cci4WZeW8CZVViHKXPm38/jIGc0o03mybx8vLfd
dflfV/55zvelJDjnRj0+Ks6ohRTqtT1bU+jIuigfm80UhxZCg+e/tQy+A2eh
By6sTQJaaF9W8MQO2zpwHg672JkuqYUGBJuv3X99BpxLuT6fdFDWQiWhoiES
DQfAOeyOooOQrhbaXRtX2KAbA87zLHdfNWzWQq8DVKZ48sk8meZGupxUFy30
WOBOvlGdEjgf9N3sauOnhV7uoI52oJB5MpaMZ3j5wrTQeVffDAcWMk+G9drA
29GdWmiX/p5Q8zp3cH7lLp5Xu0cLuc5y3ahd7wbO52i3eyVnaKFX0grtgSV2
ZB9jZZ2gRZYWutI7eDVRlQLO4k7z77nytBDN5NhP5xJyf9gvi8YFQyVa6CEN
k3GzMy04V14+7F9ZrYWeP5hXVxB4C84Rtt1i8Te1kOom/R8Zs5XgrDbLO2ry
QAvpCg7anBNNB+c/l/wus3ZoodoTC9q3r0SD822L8q39vVqofv3hqsJhX3C2
Oq/9LfqbFhoeUjhefJOZzDM33Vth+EsLWfIr3+zZQubJvPretp1pSQuV1T6q
Z/pD9pmfO8Oq2EuvjZarys1+fiT7zH0MXSYL2LWRjEv2kdh2F3CW+HyhJpxf
G+VM9p/+6+IMzt+Of4rSkdBGjGNHgrYXkPuWVmsrqtEpaaO/MXLlvPf0wHnn
h9hfndraKK1tJK7eRRSctQ/fuZ5nrI2Q24n3oiLU4LygShcXYq2NtjdmPito
6wXne/3W2urO2ujaVJMOL2cpOMN1hKfHrGLPJ4CzucLArfZt2ujcx1CZrDxf
cF7TK5aUE6ONRPeqKMuXWIPzeam6JcV0bXTyj3LQfT6yn9+3c+7e3AltdLDD
r+deH7k/rMRu432PzmmjPx/umolcIPeH/SZy2DirWBs5U7Rd6u3J/WGrn3bR
eldpo5oPM05yXY7kfm0xvK0yN/75y7DtGspwAGddAb+0mZZ/x8WpuciiYQtw
Xn5YZt7Sro1aTN48kdLVBueH4VMMma+00Wk/lUuXqch1QBnrtZ+6vtdGvA+n
l3xeLcL1g9b3Uo6If9VGFY5U9ZZVPeDMGtJmPfVDG9FY1fVz/i0E5x42Vpam
BW30tK3nYPnqKHDOuen8In21DnI77ewYMboZnD38L5xwYNNBZjpTh7QPaP0/
ps47nOr///9KEllJEbIlIWRknuexkxIZUZKslKSSEmlo0N5paKCiktLQ0k7a
0aBQKdFQ0dCQ6vf+fq7X/fH8/fW+rtv1vo5znk43x8vr8bjRObccN+vdom8v
lr/aJWfdQr5P5mhY8qNSM3vxhj731k+K4vtkkqXOb84Yai+eaPQjtWmzE52z
Q3G3Mb4u9uLP/ptWit4Pp3P+FzxCTd3HXizb8s5tToM/nXP5v41PmwLtxT5T
xzWeu+pH57yisG57Sbi9eOGX7tu8rT3pnEf564elx9mLG//sj+l+zZr3uzum
9PeeaS82yNGTWzdZg875Sd6x5yrz7MU/3LI0g5fyOc0cn47dDUvtxVrrS85f
u1pJ5zzhm8vEorX24tyEixt0Fu+mc9bfuVwvZdt/j3O1amy+Uhyd8xuPqka3
fHvxyFlJTd+HO9E5H/qktk/xsL245sGiZZ539OicHcWFRgWX7MUyL5hSpQzf
JyPxrvXtzFv24mQV5/A2Zb5Ppnz90IOiR/ZiK5cB92p1+RzQCocF8bLP7cXV
g44lyJzwpHMe1XjdtPqNvXiR2YEf/fV9+fXnVQqfcj/bi7dvCSiZLBrBrz9b
Bx9J+G0vVvlh3TqyxJXOefezndPtpRzEHttkVi9PseTXn5c1WUopOoh9tHYv
1JZT5defzc2+Vqo5iDfWrrxRk9hO5/yhZtaJHD0HsblMX9WdanfonI8sLEuO
M3UQV+b/2O2lvJ3OOcm421BrWwdx7Ln3UmaWkXTO+O941a1Dfi03o3P+nbrx
zG1vB7GL3dMpLrnydM4r7+g5RY13EH8prTo7edc/mmvzS57yZ/AkB/G3d14m
Dr/78+vPWscudEx3EMsqb53c+deKX3++/mvB9VQHsW91mtXn2678+nOii8uG
JQ5ikyENIZKtw/j1Z7XlXcPXOIgz42fqGI714tefL1deNd7qIB5Z2mpv+5rP
tbVOVlvanusg9r52WL6+1pRff1aO8Lx8yEH8dPBvsyFSvfn153MF0qtPOogH
Z5w9FXL4M52zU3TrjZCL/z3P0SZtCnoVdM7/5IauMLjpIJ5s3NXqW8BmOuer
J+f7tD1wEG9vH2itkBBG55wZfl2urP6//3/bn0E5y3TpnH2kFe5lNjuIXRuP
Gs/f9YPmu+tCdvppdziIt5apPepR/ovOeU+Xpl4tko7i5qt7XnaPU+PXnw+a
PiyVdxRbhd6t13az4NefA2ZtylB1FM+6Hp+weBWfH2z9fS7IV9dRbK07W9OL
ufPrz3slVdVNHMXHMi5nTfrqyq8/j/R50mTtKL7fYjKpSt6eX3/+vmFbichR
nBD743F0FJ/T7La7dmz6MEfxW3WLxlmVCvz6s5eepvdoR3HMjTGHFXZ9oHNe
1Tb5mUqYo3hggZKPy4krdM6jtpXsaohxFKefv9I8w3w9nXNv118TihIdxfNv
RMtJxQbTOde8F+umzHUUT/nrHzg7VJXOecfGrFduix3Fi7sW/Xnf0kjnbNKk
GlO3xVFc4m9c35nzjc65bfWEAQV7HMW1L2q3SS7sza8/2xa8mXnQUezj67dq
0Hdjfv35xadC0QlH8YIv/mvmj7fj15+zbKfIXnAU19yccHmIvphff7acb1Jd
4Sg2ap4zIMVCxK8/Py3/kFvlKJ6wTyq/8DOfh12bIV+cUOco1p31PHrCDj1+
/dkkKNG+yVH8RXvr/uYFfL5b7VGOhVSro3jTcL1Dx8ua6Zzr573+XPnTUZzY
4+fD0yXn6Zx3G5oez+nqJP4eqRFr7LCSzjnqXtKsODkn8ZE/+W/yV/vTORvN
OWdj3ddJ3Obw4/LiS0r8+jPmrqTD7ItrHtA5p83YMDfbyklsquGRVDWY75MR
q9c6RDk7iY0kzpvabVbg15+v6nYO9nISD6hJsngVbMCvP8dPPt/h99/3s6v3
mFplPne8VqVk/vWxTmKb8CM5S5g9v/58/ifbEO0kTt34LKLH9qH8+nOsuEv4
NCfxpi9DfgUNMufXnxWyrhinOInXvfVZr6TI57v3nrq/uH2Rk3iXutWUFC1J
fv05QtXj8konsU7/K62e517SOZvKTOi+erOTWHLS5dNu+qfonFtL9leE7HYS
G1aYGc4xX0rnfHzspyyDA//9/pHUoT3WxIdff5a0Hd52zEn8yT7qkEZtD379
uSi9Z1mZk7jQ502dy5lrdM63/8itCah0EkvbRMXatr2lc163P3CUdq2TuFpD
3btsuTS//jwqR6ml0Ul8evmP9Zle2vz688/GqtKPTuIPizf2Chllxq8/7zHZ
mPHDSdy6M8/qUqQVv/7snRTo28VZ3NzYbu6kOYRff/5yto96T2ex/5nU5uhm
PkdvvqNrTZOKs/jxdvmh6R59+fVnt+FbS7Scxd7fxPFvvf/QXojTH9aHpg90
FtuNkD7U37COznne5qfq3kOcxdpta2aNPlZC58xEuvUqTs7i/dsXzgk1X8iv
P7+J29ng4Sy2l+mafrPEnV9/Xns0vGiUs3jQ7Hu+y9Z25def7X5qp4Q6i03D
6p+a9jhL56y+IjNPMcFZ3DMgIrGl7hWd84sh96PqZjuLRbGJGdcq+L6CvXV9
DQsWOot10+oueu3g+wriloQ3z1zhLHZLlHaPUR9A52xmtr9AtMlZHPpn2qw3
Q83onL88/hgnu8tZvPei1q4kLxO+H2m+zaDqAmdxiU/GEu8/unTOaUbpLbkl
zuKZhsWHOy4p0TmLK68VJZxzFg858bT8fbfvdM6Sc+Wm2Zc7i7uM3GFSF/2I
71XTDTSXuu/83+euSWnf1x+kc15+a0db5RNn8bgRB77u80yhcx6R1FiS88pZ
rGrnsvBKh4jOWUnTJCnug7N47NYjF0wzf9Oek0fXZlpbf3cWu1hUue8/9d8T
rpZ8sji7kXe8d4ead5g/EH1/fvOawcq99Hkkq0j55gXVayL1owetFmd/pf9/
dpuqhPKpFyKV5IA096g/xC1e3rl8I++9SLr//JcZ2ZJimtf/vOOG66p20ZIF
o5dGpfcgrieOcl57S4J1eTXA2D1KnniOxwAFWxsZxvLSYzcY9CKe6z8mVMFB
maU/nHQ6I7s38baJ32q6Rvdj6MGDG+nFzg2V0aUePDi68Ohtg6Njjj40OLrb
6BmDoxON/i44usboxYKjw4u+KTi6sehxgqNzSv1I3BcpdDnROwRHRxJ9Pjp/
oXuInhzN2wmdPvTPwNGVQ68LHB009KXofSJ0u9BDAkdnCv0ecHSR0Juh+2qF
jg/6KODozqDnQfuQhE4K+hP081DoeqCXAI4OBfb70+dBoZuAffTg2PNPe+QF
jr302PcNjj3q2E8Njr3f2KdMfTlhTzX2/9KcjbBXGftq6d+XsAcY+1Xpuo6w
txb7QMGxZxX7K8GxFxT7Fun9I+yxxH5A2tMj7F3EPjtw7AnE/jVw7LXDvjA6
T2EPG/ZbgWNvGPYx0fVXYc8VfIWfA2nHrc73WfCZfEV/N/s313N0YCP5Cjzg
dcDAHRl/yVfgXpHpq07vlmHwFe0DMFb4WXxWhcFXNOd08VGNyEiHfAU+s0+E
tIx4IPmKeoy7pUZXZFmSr2ifw/qHSgoJduQr8Cq5zM37osTkK/DVf871UrHw
IF+B37hwfExR1HDyFbhLVEmDwqFR5Ctw9OvhK3D01uErcPTB4Stw9KzhK5o7
F/rL8BU9jtALhq/A0beFr8DRY4WvwNEPha9oT7PQu4SvwNFnhK/A0ROEr8DR
v4Ov6O9XQq8NvgJHXwy+AkcPC74CR78JvgJHbwi+Akcfh7opAkfPBb4CR38E
vgJHLwO+AkffAb6i74vQI4CvwLE/H74Cx753+Ir+3Qn7yeErcOzThq/Asf8Z
vgLHvmL4Chz7deErcOyDha/Asb8UvgLHvk34Chz7IeErev8I+wzhK3Ds34Ov
8PnU/MnA4UG50gy+Ak8vPBeorNFBvgJ/8zz9e8x2FQZfgYd3nu5TMlKXfAXe
6iztrfTPhHwFHtkwwOxrpQ35CjwswTrIKkJMvgK3e6CmHF/sRb4C7zq6SXvb
fD/yFXULXU/aLNELJl+BR7W6/c3eGUa+Atdu8A74VzWRfEWPc/bw9TWRMeQr
ej6b9+bmn40jX4EXvpPZs3BtPPmK9v5aXMrbnZNAvgJ/tHzli95HppGvwNGF
h6/A0TGHr+jvTkJ3G76iXrTQiYavwNE1hq/o7ypChxe+Akc3Fr6iToXQOYWv
wNHlhK/ofYWOpOArcHQP4StwdPrgK3B05eAr+r1W6KDBVzS/LnS74Ct6/wid
KfiKuhBCFwm+AkfHB76i94nQnaE+CvaaCJ0U+AocXQ/4is5H6FDAV+DoJsBX
4NjzD1+BYy89fAVOe9QFX4Fj7zd8BY491fAVOPYq095bgWMPMHwFjr218BU4
9qzCV+DYCwpf4ffmmVtUPr2c15d8BV7uXn//d00PBl+Bn15kqTN5hz75Cvzu
EQn7FXMsyFc0l7xqRNqGvQ7kK+pRhy+b8fmKO/kKPKL+XIycySjyFfgwWcXL
/suDyVfgF6WTZzZGTyBfgQ+s7eu5UimGfAVeFGyeEO0YT74CL/51clJc+3Ty
FfiNxYc7H4TOIl+Bx0RWHk3NmkO+Av8x5M9gkUMq+QrcXH7o0GazeeQr8Bb7
op8dQ9LJV+ATrc1ux/7H4StwtcIvPe4MmUe+And36tCfOSaVfAWOfj18RY8v
9Nap8409H0IfHL4CR88avgJHfxm+ouuLQi8YvgJH3xa+AkePFb6izobQD4Wv
wNG7hK/o+rHQZ6T+n8DRE4Sv6H0u9O/gK3D02uArcPTFqAslcPSw4Ctw9Jvg
K3D0huAr2v8h9HHgK/p3IfRc4Cs6T6E/Al+Bo5cBX9F9mkLfAb6ifZ9Cj4D2
4GOfn7A/H74Cx753+Aoc+8nhK7rPS9inDV/R4wv7n+ErcOwrhq+oy9R7jula
Xy3yFXWxvE8YFp9XIl+Bh2XoDD04dRD5ivbyWr4qHWZpS74C/2D2WFL2mwv5
ijpFl144tnb4kK/Aq7y7zT4gEUS+Av/zZl139wnh5Ctw9SltkjlGMeQrej7X
Ps+bHjWVfAWuNfvr4y2hM8lX4KlurWvKa2eTr8Dr2zZb90xII1+By9R+sKgq
XUC+Am8Jkl6jUbSIfAXuZdo84mBnBvmKenqzPmyeXLSYfAW+9dXEDzcOLSZf
0eP0cc4V/csgX4E/cHpTsebCIvIVeFqekswWpYXkK/ANZdukHbznka+otzNt
qnpmUAr5CnzQGNkFI7fPIl/R90vowlPXHPezCB1z+Aoc3W34ChydaPgKHF1j
+AocHV74ChzdWPiK9uwKnVP4ChxdTvgKHB1J6h1iz4rQPYSvwNHpg6/A0ZWD
r8DRQYOvwNHtgq/A0ZmCr8DRRYKvwNHxoU4M/l0L3Rn4it5XQicFvgJH1wO+
AkeHAr4CRzcBvgLHnn/4Chx76eEr6jAIe9ThK/ydYavisFKxuD/5CjxjddqW
8IHcVzS/Oyduc8dCY/IV+KY7jiuD4m3IV9SBt9Z+3xjAfQV+Lkua+XfhvgJP
9Vh/wd0jkHwFvmC37Rl7v/HkK/BB2vs6KqZEk6+o4+F4XOZHTDz5ClzpWLJi
8ckZ5Cu6379+2sfFCtxX4J/W9bO4PCeVfEXdGx+Zsqbt88lX4HG/Dioekee+
Ap/W7b6b/mjuK3qcyb8nu/zOIF9Rt+GTxPo/vzLIV+ANQyLH7RnNfUXnFtrQ
oaLJfQV+e3DnpBM/5pOvwJ3t5L/qszTyFT2fm2ofJ8bPIV+Bz0rrvena3STy
FXjRzrTYkrzp5CvaW5D4uHKr+lTyFbhjF/tSTYs48hW9H4R+PXxFXTKhtw5f
gaMPTv1pgaNnDV+Bo78MX4GjFwxfgaNvC1/R91foscJX4OiHwlfU+xJ6l/AV
nY/QZ4Sv6N+d0BOEr2hfndC/g6/o/Sn02uArOh+hLwZfgaOHBV+Bo98EX9H7
QegNwVe0F1no48BX9HqFngt8BY7+CHxFXOhlwFf0+ELfAb6ifTlf3g9Y/VuF
fEX7ippWq/gMliZfgXdVGLgl3kePfAW+V/6eSeHZweQr8MftA31f19mRr8AV
jVxz/oa7ka/AbTrGT1yqMpJ8RT2K2QvfHOoeRL6ir7thR+cf7/HkK9rTpuTw
9eT+KPIV+PlL60OHTJpMvgJf2rT1VEdeIvmK9n8/H5rypWMm+Yq6lLcHPHzy
OZl8BX5P0y2tJCmFfEU9utJfn9LdUslX9Hqv6YQ8s0ojX4HbO8ms62+eRr6i
fV1fe5/wdkwlX9HrXaAYZDY3hXxF3V1Jy6olWrPJV+BvZ69ce3VqEvkKvOjT
+sKt+6eTr8CfjTll3iI3lXxF59nw/dWRxEnkK/DQJ38SIgujyFfgVs/84/Ou
R5CvwA9bxIxumjuefEXvn8bWyhX//f4FX4HLXzXs2d81lHwFzkTF0gMnh5Cv
qJ8mdOHhK3B0zOErOmehuw1fgaMTDV+Bo2sMX9F9HEKHF74CRzcWvqIOp9A5
ha/A0eWEr8DRkaQeocDRPYSvwNHpg6/ofkChKwdf0fMUOmjwFX0fhW4XfAWO
zhR8RecpdJHgK3B0fOArcHRn4CvclxHxTuac3zgZ8hV4scrtZVGSnXS9HXxL
0kfFIxp9yVf0OF8Uuhiv1yNfgVvO2tPFSHYw+Qo8+uDHk55xQ8lX4NM3aTQU
rHchX4FfkX5QsmmSN/kK3D+p+kJYiT/5Cvxku0jT4/wY8hV440/tIwpHwslX
4AULdPOmqUeRr8C3WgZM3/k+lnwFHnZ0YM6FeVPIV+Dlpo3dBi5MIF+BOwVL
G98+mEi+Au9d9aD5dOR08hW46lqDeZfHTidfgVtMeT3686JE8hX4gCGbdiqP
TiBfgQ963mawN3YK+Qr8WFZk6hGFSeQrmkPSf2X6fnsU+Qp89tiSAz6j+fV2
cHnz+nMf/vMJfAW+cn/4g+9d+fV2cLerNoeLPEeTr8AzdOOkY/RGka/AK4YH
mfvM9SFfgZ9Z1i7zz9SbfAUenBH0sf6BF/mK7qfbff7S7x5e5Cu633zhOoOD
8z3JV+Bl3kbBjT88yFfg6NfDV+DorcNXdA5CHxy+onMQetbwFTj6y/AVOHrB
8BU4+rbwFX1fhB4rfAWOfih8BY7eJXwFjj4jfAWOniD16vD8hf4dfAWOXht8
BY6+GHwFjh4WfIX7xUKeHTfK1OpCvgLviA5wO/buC/mKusoG+akStfLkK3BJ
iwVxN3v1I1+BV/U8cGi2lwH5iv5/UXGW4nZz8hW4zlq/d+dODSVfgR/SObUv
xtCFfAV+9lFbntpLL/IVdSG2jWKx3fzIV+BDvLbJFh4LJF+Bl0VtfnLwaij5
Ctw38cWdj6/DyVfg8XlHjsk9mUi+ol5Qmk32lboo8hXNhUtfeGXmE0O+At9d
ZHlvk0Qs+YrOJ+HdltXvYshX4GnxURvHycaQr2gP2c/ycsv0KPIVeMP+x8FZ
CRPJV3QOC1ZJB+qGk6/A91fveDPLIZR8Bf42KealamYg+YrO59AStQg/P/IV
/f9nS27dWepNvgKfX35404HnbuQrcKsjct+6OjPyFbiNRJariaU9+QrcxzDD
IELLhnxFryvY4PirhUPIV/T9umh36PtzC/IVvf9XzRr2ar85+Qp84mMHjQj9
weQr8Kd2Xzqf3zQlX9H7be/rr59DTchX9P31cr2y6Lgx+QqcOZT4td81Il+B
owsPX4GjYw5fgaO7DV+BoxMNX4Gjawxf0dyV0OGFr8DRjaXeqsDROYWvaK+V
0OWEr2hfrNCRhK+o+yR0D6kjKHB0+uAr3Me6v/KKfK8Jf+n+K3BdeTODuutt
5CvwxcFNjypSZclX4DUFdavKnPqSr8D3yQaFPejUJV+BlxtdOrFJ2pR8Bd5n
zGE5DRkb8hV1jGu0V6RMFpGvqONxTjvppakH+Qq8S7iLjMekEeQr8MultjY+
oaPJV+AvJ3d5ee9xMPkKvNhMzv/whHHkK3o+/QP2ntkxgXxFeynm24VrbJpI
vqIOhruoYvPjSPIV9V6O/x67anoU+Qp8T4azbERgFPkKPNCh9cbGJZHkK/Dk
S6pBJmoTyVfgXgdky7Xiw8lX4H4nJli3q44lX4FHWxjV/soOIl+Bb1ylbBnv
4k++At82UPq5StFw8hXth+7SvMTnjxv5CvzYkx5b4wfz+6/A17Ta3R312pp8
Ba73J+jslwNm5Cv6/l5Lk/GxNSRfgVt7Lbx8fYA2+Qp8VfcXSt8/qJKvwI9n
/ZjwJL43+Yqez6BbZvt3KZGvwB1UxzQlZMuTr8APBrpPNfksS76ijoFILTK8
tAf5CvzrtB9rQrK7k6/AV/eWzZ42oxv5Cnyk+fBDbs5dyVfg79c98wnsKkG+
Ake/Hr4CR28dvgJHHxy+AkfPGr4CR38ZviIPCL1g+AocfVv4Chw9VvgKHP1Q
+Ar31w+MKKib/KaTfAV+41vqmJOfWslX4J/MU847W8mQr8C9Fjir7zXuQ74C
n+b5Mt9Vi/sK/LGo9uiRokHkK/A3UmsKj3SzJl/R40/d3Xm/zYl8BX50d07q
3AHu5Ctw99s+HT2kfchX4JuPL5LNTPYnX4Evnu21vPNMEPkK/GNLrnyd7Fjy
FXjjDDUVl17h5Cvwv10kxnc7HUG+AtdxWvX45NuJ5Cvw5kVFJhmpkeQr8NYD
aZqLx0eSr8Ct2Ovizv98CF+BJ50PPlpuG0G+Am9yzL1++V4Y+Yqe/9kRrmkF
IeQrmjtsjuthHRhIvqIO6pmkgYZGfuQr8GeTPF+5XRhGvgJXXV688rinK/kK
POhDkbe3ogP5Ctxy/tGUonZL8hX4aJXEyxJ/jchX4Lp12681WGuTr6g/fGqI
xub3vclX9HU1EpaP9pAhX4FvVHs1dQmTIF+B71vi2Cv4eLsIvgJPfFvpqn33
kwi+Ah+8Ibz/0dQ3IvgK/EXUUrczKo0i+Ir6NjmFXaZbN4jgK3CV1PN2Lwzq
RfAV+MwLUl5v1z8RwVfgUiUW78MCH4vgK/BtZ/saS414QL6iPcqRe50b390j
X1E34/Mofdeim+QrcOWsPekdbdfIV+DowsNX4OiYw1fg6G7DV+DoRFNvWODo
GsNXmPspOVraxWzfT/IVeJaeyMXZoYV8Be7y/FXiiXlS5CvwqGOBE6J7KpOv
wKONyzy0LfuTr6gf4jneKOPCAPIV+J+cnVs29LIkX9H+7yMhlzQP25GvwAe/
XXVuzGAX8hV4vzRml9jTi3wFnvfDbGFp+UjyFbj09pM73ZaNJl+Bnx3UU8XG
Oph8BT4kVLJ6S14o+Qp8x/Yh3j8WhZGvqHvw/fpGs+Xh5Cvwp8NLKocvmkC+
onNWDppvMmoC+Qp8fJGR97if48lX4JdmVLr2zhtHvqLnY/rFZeOKEPIVePJb
F5O0v4HkK/BxanMeFo7wJ1+BBxnK+qXN9iFfgWtdbJI9+I/f3057Alo/5Wxt
FZGvwLv+nqd39JsN+Qp8qMGhrGU9BpOvwE07nzYtv6tPvgL/YLyiLd2yH/kK
vC5j9cDidgXyFb0/Nd0zjeK7kq/A89sVQmpHtYvgK/p3ofhy8QTTt+Qrmhcf
1m3Oc5c68hX47OykWdvYffIVfd/je8vcKr9CvqKOnCjjxLGQ0+Qrej9nb8k3
dyshX9H7tvVx5O6kg+QrcMc9vRUP+u0jX1EffsGUsBubc8lX4B2KTzILDHaS
r+gc5uZKef3OJl+Bz1WYeXB36kbyFXjA7ksZlfpryVfgojmLJJKSs8hX4IaS
phqTnmWQr2jPgdCvh6/A0VuHrzCPOG5Ct31ZNXweB9y58OoyzVI+jwPuHb3q
U6rmP5rHAd/6cm/FVB1Z8hV1dRSHPDfR60O+AnfMLJS3mcPnccB/J99JPVHK
53HAj4QMXOrxjM/j0Lx72tipYxv4PA64U9YricZiPo8Dvq9z6+LTq/k8Dnj/
U+tdhj/g8zjgVQ+Lw+V7+5GvwE+n/1FoW8PnccB3vH3r7J3P53HAF/22XKOr
EUy+Au+XG7v4+D0+j0PnXNmvMuQgn8cBl7t3apfKMT6PA75ziVuXpxJ8Hoe+
btKl6NKr/uQrcFeHnW4+0b7kK9ovfinvynxZb/IVePHBh84bX7iRr8Avbr2f
5R8pIl+Bz1XbNSphgi35Cry9/o1PZpk5+Qq8UkX+0KViQ/IV+CqVfysCV2iS
r8Bn3S45nq6mTL4CT5/yWkZJszv5Cnzsq0tn9GU6RPAVuLi/VVbquo/kK3D3
tU8Lf1o1kK/AgwOTHx03qyJfga+rSq7deeQy+Qo8aH3oiR1VJeQr8APjDTzt
HuaSr8CNRjn1Wm+0gXxFXzespchx2WLyFbim0fZLi6XmkK/Alz/KLIl1m0a+
An9q2Hvm2qoY8hXxY0u3mB2IIF9R73THe5djfceSr8C7hu/fKrkwkHzFn6fB
p6D2UeQrcMWj7S7i9cPIV+Aub2TcbTa6kq/Ah4f7Kp/LdiRfYU76osh4nOhE
E/kKfG602Y4pHTXkK9oTcCND3+XDF/IVeITH8+WON/7RvDPte6m9aTfspAz5
Clx+zpD0/dIq5Cvwe4mLg2yVtMhX4CnZIe/szhmSr6gLJHH4wwvXweQrcO2U
8Usm+liTr8D1/+x+rWHnQL4C95s85LuPn5h8BT522MLN91e7ka/AE26n6o3Z
6Em+Am+fHDihRd2bfAW+9+77M/1fDydf0fkP9D666oIP+Qo8qu+T334bfMhX
4LZ5hTIjooeTr8CHSX2Q8QgaRr4C3yFvsFU+x4N8BS6lfGuJzjxX8hV4y5df
v18ni8hX4C/2nx+/xtSOfEV7McM1tA6lDiFfgSdpxEZNtDUhX4EHTR3VtG+E
PvkKPC9ZY6RRuDr5CvzhES+TLaG9yFfgr+8sGzXwrxT5Cvxmk2fWtNoOEXwF
3nW8ZGzw5lbyFfg7Xae3o1+/JF+B54u9gzZmPiRfgetsSbRYvOIa+Qr8+vvp
n9Y9PEG+At+XdOhN1wl7yVfgaWrTzFxdNpKvwMO6bG2r3LeIfEUd9a3NS09l
J5KvwEc09u86sVsY+QrceO7kRXu8vclX4M+OvD482cmOfEWvN+JP4946Y/IV
uK76nOm9J2uTr+jc3p5zf+rZh3zF9/DNjrp+VYF8BT6vLbU2pKY7+Yr+3V3L
dIvq2oV8Bb7EcLBZd41ftJ8B+xv0lTzFC/SqyVe073blolq7WTfIV+D/HhU5
ThzRSL4CH1Tcbllj3Ua+Ap++QcKoYWgH7WcA97x2a3BsQjfyFbiW1IyPHwsV
yFfgHp0/HXYa9yVfgScsVH26tV6LfAUeNe6G49xjBuQr8Bwb94AR5weRr8BV
UnzqBuVbkK/AK70fORt8tiJfgX8bmq0v1X0o+Qp8a1rGq5nb7clX4Iq9pm49
NdWRfEWva/KuzjQfJ/IVdeYPuYRO0HIiX4FH6lq/HvfegXwFrtO3aeSLSjvy
FfiMB34HqmRsyVfgvd70W9etdgj5CrybcWGPHUrm5CvwpUsM3wYU8P0M1DG7
Vha96Ike+Qr8mPTCjqxLGuQr8O1XvVS2FvcmX4EruL3XfN0oS74CV7quEz2/
pAv5irpePZVjIxO+i+Ar8D81hzffnthCvgLv//mZ2Z1+DeQr8BiF/BW/nR6Q
r8DzvYeMH9V8lXwFXnX8brvc+xPkK/CULn8mjjHdR74C39/35W3b/ZvIV+Ch
Az9v7YxdTL4Cd06J7Hpi/QzyFfh8yUe/9hSHk6/o+a+I72f23Jt8BT7ML+ac
kZYN+Yq+7rAmjZ13dMhX5IFeDaw9VpZ8Be6W6v1xdK92Z/iKzn95g4fUm5fO
8BX4vqJh7WdNqpzhK/CrEpnlw82uOsNX4NILqiX6h5x2hq/ANR3z49vyjzov
952tnp2dT3tgvgfftdhwuFxU/+mnRGfla+KfZ44vH7DooSjOyXAYs/5GvCxu
S+rgtw2iwVHnWhqa+N4YX6n7Hnanv4sOla9de/k23xuj2bhpxJicLmy0R078
ZR++H+aY3mK9nR+U2brZy86XXOD7YZ7uP1U0xUmdOu7g6LmjOw6O/jj6zbQH
Ueg4ozcMju4wuq3g6LeiMwqO3ih6jcSFbiP6guDoDKLTBo5eG7pitGdD6Iuh
zwSOThN6QuDoCqHLQvtVhD4LOiK0d1PoiaDHAI4uA/oBtA9H6AhgDzs49rFj
bzg49odj/zI49jBjXzB1qIS9wdi7Co79q9gTCo59odi3CI69i9gPCI49gdiz
RntyhH1r2AsGjv1g2GNF+9GFfVYGHxofrco+TntI3icfM2ro2ixKkf5kPEu2
mfiyotmpC858/e/zxu6w35XfiP9ZM+fhDeku7OjqYXvLcv4S32J4Q+d6kBwL
OvCzXWTdjfYS3HG64KBf2pc96Hl3m7a6DPHvlQ2TDVbosti3Z6e3zeR7S9rt
X7Ha0iFsb799Vxqa+N6SOZ4ufgPr7JnOvLmxl2/z/STtZRE1vVQ8WfjhqNMl
Ig3iD7osny/v48PQO6f9KkL3HH1u2vshdLrROQZH7xhdXnD0edE3BUfnFD1O
cHQ50TUER98QHT5w9PjQMwNH1wz9LXB0uNAxAkfPCN0dcPR30C8BR8cEvQ3q
qgndDXQLwNEvwJ59cOzbx75y2nMi7C3Hfm3aJyPs2caeYnDsK8ZeXdo3IuzX
xX5ScOwpxT5NcOzVxF5CcOwnxB492usi7NPD3jdw7H+bUrJK0zfqPO3HuP/p
/au0H39FujqNoz+4vSF+5dOIV46HejLZIPUZpxLaic9YqVU96IEq665+9XSK
/T++HzG8Iavkij5b0GojsTOjG83Ln3Sr+flkiznr9Jfdmx4nQ/xZ8MKxQwPt
2NwTz73CfRWIVzUOq/4s78rSVj5N1t7O92wYin9sbavyZmfKrlYoyqsQZ10O
nou85c9stLfGt33h+zQOLVmc99ghnCX5ZZxviNHg///TKa1H8iNZSuihow2G
OsRzNw6/OGTiZIbOPTh69+iFg6Mbjr41ODrX6ATTPgShF4yuLe0lEPq26IOC
oxOKniU4upboAtK5oQ8odOzA0bNDDwwcXTD0q8DRsUIHCBw9IHRraO+E0K9B
/wMcHRD0KmgPidCtwN5/cOz/x5562kMi7Kunfd/YXyHs/cZ+anDsqcaeX3Ds
+8VeWtpDIuynxX5PcOz5xD5KcOylxP5EcOxRDD73f6Hsq7TPQf+6ZkzdDAV2
0+5JWnT6W+KN6kZbLA00WPjLTUuTZL8T3yGV37zVz4iN3e9817r6H/GoL4c9
fEdYsQM2SbK/K7vRfLfy5Lv1UTdEbMBWr+K6UhniSiqXLhsUerH8Do3RZTkK
xKN/FOWXLPFn6wb/TPW9zPdCaCk5qA6RCGX3Jry6I7JW4XsJbug0nreYyOTW
39EyL+B7IZRaZhveq4hlw6+UTtdW70fcasaDxOWqCSzr654riqs1iJvsUs2U
fDODdT3kYJVir0Pczmbc0YhhKSzvqnlY20y+/8FlksUMVp7KGr71K25o4nse
5FOvdrs3JJ2hHw+Ojjw63ODocaMbTXP6Qj8a/V1wdHjRi6W9AUI3Ft1N2hsg
9DfRiQRHLxK9PdqrIHT30IcDRycOnS1w9LbQhQJHHwp9HTpnobODHgw4ujDo
aoCjr4EOBL0PhR4E9umDY68+9r+DYw889miDY5829j6D0/5nYX8ufV1hjy72
vYJj7yv2k4JjT+mp/q1Bcek3aC9B2xajPkV9+rCJ/nf31pe+I36o0uLTwKt6
7FCC4nbzgu/EV4VuvDS9rwUb3a9+eoq9BM0j9y62spyy2YHdTzuqEqotRfxs
tv6GNmMPNuL5ktP2UrLE73e56OHp5Mdui0PD1FsUiNvEP7hRJBHC8qv6Lkx/
y/cbZMgc2PTf7zrsZdcuDxLGqRDva+CqsCplEtO2btEPv8f3GySdfn30lFwi
Gx/zONnXpR/xWJFV0YNbSWzHlosVohMafN+Cv+vQuf4prLbiQD9zIy3isi/+
VS67M4+t6rx5UnGkDvGGB5XFFtMXMpM+F59nLuP7DTL6P79bq5rBJnsesEj5
xfcYPFJaWT24aDHrMWFXaNsQQ+LlmlH7Fs5azDzXZR1q2Mf3FfRI7ZH5yHoR
Q68dHN12dK/B0b9GpxkcvWb0bmleXujeos8Kjk4rOpfg6F2iywiOPiP6dnTO
QucOPTZwdNnQtQJH3wodJnD0mNCzob0TQtcG/RVwdFjQsaBzE3oW6C7Q91fo
L2B/PTj22GPfOu0TEPauY281vW+F/dXYs0z72oV9y9gLDI79wCkLr/4aFnWX
5u5n9tD0CqpXYcpTrtwNbHvP+3Vrlj2dkqXHJDuWF7e4/SDebVt48Nez5mz+
hH5p1ZkSNG+7R0PHI8HUgf2+/FHrVIIUcdVdCpcv3XVnKQZXrmQHyBIPHhT6
pOPZKPZj2ZbYFHtF4tm//+y8tmQMOzPLZmmRojKfN7//Qv/WmQj2Y592TU6G
CvHA8afkTnyIZTY1Msar2/n8/rdOf+UFXtNYUo9vqelx/YiHDtDZGOGRxErs
n99JqNMgPtvV+Y7q3jns05QbWuG+WsStg8pC9L+lMVXfULuCCB3i+zZ+cn5U
v4CVxvpO1N7O5/ez5BffM81YxIIWuK3Ifsjn92NX2OhWrclg37LtjivK8zn9
0Ezrf6UdGWzjUbP6TE9D4nu7JPy7ej6DVb1UHZxyhs/pW3y+Lz9l2UI2vUNu
TNsXPqfvfvlBpGluOkOHHhw9enTTaZ+A0E9Hf5rOU+hQo5dMjyN0k9GdpXl2
oT+LTio4eqnoTYKjO4k+Is2nC51EdObA0ZtDF40eR+ijoS8Fjs4Uekjg6CKh
KwOOvgw6KDTnLvRQ0JMAR1cC/QNwdBCwR572Kgj75LH3HBz7z7Gnm77vwr7u
eZ9v39tkUEXz5j8VLuvsLFVgxzJOfb9j/IH4vF479fY7a7CAZd2uFjT8IJ7e
/ZT5Ky8jtn9f8FLragmaM5WSspD5aWLF9HuYGPcpkyLufSm2RD5BxPZM+Xen
PVeW+PJjS+QPuHkx7bsPp1dnKhJXf9ZwWMHAn91VDVxRZ6tMfHzNx53PgkNY
T29R/e0DKnyuuevOfy37I5h36sDBZZqqfK5WQqK3ZnsMyzykvLBobT8+/xur
cuNfczwrr++syumqSdxhc7cZEn1nsG4Kb/RXJ2sRny3RKDHAZBbL3f32rGiW
DvGzP/tnK36dzcSnnzWWHuZz6wEr1fNmms1lLyofyJm/4XPrM+cNi9+Yn8rm
v6uwKdDh8+m9xlubr7NMY/27ng/XHmvI55fP7dpefSeVlf3frxcb+dz61+4v
tpgum8vGWRUcVbzL59bbdNeMu7ZsDuvwyXma2Z3Pp2/qNslbQSaZ2aUvM0mZ
a0L85Y+U4k/miaxmc1pg2zE+n57l9dTjmQX+nfJ5c/Ti0TWn+Wihb44+NDg6
0egZg6NrjC4sOPqw6JiCo2eKHiR9XaELiX4hODqG6MCBoweHbhm9f4R+GfpP
4OhAoVcEjm4Rui/g6L+gUwKOXgl6D/R9EboP6BOAo1OAffr0vhX26j/TK9VZ
mv2Q5q+rkldKHbaVZJE7/u8C90fiqbZLrDS79GKHlBfURKX/JG72zscmRKI/
e9Cyce0Ggy40dxloLKu66JER8/ONs0qS7U78tuLg332fD2F3jjrVBLTJEnfM
c/J9cdOJ+fTulWZdrUjcdeGgHVX/fU5uPDtzrWy4MnHFfT738n+PZLofwl52
VKoQv/nq+u9xcwLZhP5eVi1uqsT3Xm9M2vdhLNvpa7m0rrQfcW/b+wM0e01k
dQs0am4baxK/O4e5S9VGM7USKeOyHC3iDqXR2btWxzHTz3OcqzN1iD/Xneyh
/Tye3ZZJiPW9zOe4x9pOb29Mn8am6EWuvd7B57ijViW6XO4znck4jjktsubz
2iKXq8tnhk9nhQEjXpYmGBIfMebe8vje05nXVBdZ8wI+x61bkLdr4IJprHmJ
rVVBA5/jftz3iWrtz3i2dKdJmLY6n9deM1At3Kkujhn837c9gM9r24q0JPVn
x7Cr9/oUK642IW7d41q/B30iWeQb2ZrM63yO20d26CzrL+PZHrX2gSn2fC67
4XDR5wj3QIHzuWzl3gfGP5PwZ+i701y80HlHj5yep9AlR9cZHH1ndIjB0SNG
zxUcXVf0R8HRIUXHkebchZ4juoPg6A+i3waOjht6Y7QfQOiOodtE7zeh34TO
EDh6Q+i1gKPbgr4IfX+Fzgh6GODoYvQM3DMzWbaa5pSr9i+Kv2f/W/TreO6w
8zmfiOvdPey9rHsP9mPXrxZt9V/EV3e+zMtKUGE/Lau2tc3sQvOJWe3+2S5l
Oix50z7nqpDuxItME30Oxg9iX77PfVki6km8Olx9jY+aNTtlWuDqraxEXP/y
iQ0dSc7s+/j1m+yXKhN/L/t4rssdd2a9Nq3Z+KcK8br+zmv9Ro5gMy/F2KnH
qxJfrKU7acia0ezo51ErZJ/3Ix4Vf2nzhxlj2Cc9h/oOP03io92j2nNWhjGz
QIPBLVe1iBvMMDzy90IEO+kqfSlhhw7xPTGOHeNbI1nguM63TY/4vPOsA4dv
p+VFs29Jn3uFK/C55tZVIUr3i2LYhlXNDtVefK55WYf1LtWvMcxyX12U7yJD
4vtMmJp9YgyrPF+56vpZPu/c3SFoXLhlNEusLj8p+sbnnSecKuqiMT6SKbSe
fV5qxuea71gtKBHpR7DD0kelzSfxueYN085kFySNYyN09lkU7DEh7tfU09S0
MJi12G0P1a7l884Jw8Pyfw8dzVb6r83I7s3nmmfXmo7edGYEM56y5JDiSD7X
nCu76I9xnCeLvuz03s2NzzUfNYiIUlniwrrVWvYum8fnl8MO+x0OfGPLAmOL
jFL68TllqzKvE7fyrBh67eDotqMvDo7OODrN4Og1oysMjr4w+qzg6LSiJwqO
rii6jNTfFvqM6AjS+0foCaLHRnPTQpcN/TCaExc6YugwgaPHhG4QOPpB6NzQ
3LfQu5FZuuJaTPoTmvO1LDgp19/xlyjlzpYZNtWtxKvmqhxYZtOd2ZrMkqgv
/UV83sZzkRWPlZlaisJe38td+Nziqgludyq02ObaV17mBd35/KNOeNx8x4FM
2flUi+Lqnnx+cPWSg3MnDmEV98u9UuyViNundObdeubAunc5ui2uSJnPe75d
bCjZ4srch2xvCdHuQ/xFSZTiskPeLCNqibP3BlXiRy7tNbhT4ccubZq21l6K
zwV/uaq2JlE/iP0tD3lpnKJJXHGu2QfrwlDm9MPVSr1Fi/jwWV/H7ewSzpw3
bXGVOqJDPErJs7/V2whWX7QyPv0tnxfWF22TfDguks0rX7ipXZfPBRs+PLO/
2CaKqT9PPp8wjs8F6zQqdBaFR7Ez36c0N20yJH7Ie7z/iQeRLEQxQjH8Hp8X
3iMzsNvk3Insh1GQXbU0nwt+5uj3/Vr1BLZFPHyirwufC7795mGh/JlxzCaU
rbieyueCt1fk69z1HsMezbA+LjphQnz+oZ+vIxtHs6QVxvWlH/m88Lim9fMK
do1kvfO1pMyN+FzwqK8GSVcWeLKSc70HF0TwueAx36qTg3aK2cD/eYDPBY9I
rzDp9ncoS/nffQ6cD3BY+DZZ1ZLdyJj7KHMZnwvuVxvfZeDYgUxN8dtbN30+
//tJdPBWxAFddtroXa+yMD7/e+1528jsvn1YjxNRA1J+cf4qNHLzvKJeDF12
er1Cnx0dcXD0xNFjBkeXGf1g+rpCRxgdVnr+Qo8V3VBw9EPRXwRHhxG9QHB0
A9Fdo/eP0F9DJ4y6u0IvDF0rmrMW+lY9Smf1DW6rpfnZPftqgrbc/Cma98Xj
bnVmG/FEr9K3Vyqk2OxTn1S2BnQQT41aust8ujKzu+BTXKTYlc9RRgWW5vTW
Yid1dEavbu9OfHL5JfGwPCM2ZPG39oS6nsTHxOZVz3ttyWpmvh5REKFEnHVX
nNhT3YH13XtvV/ZDZeK1+dMCKse4ssDHp9syPfsQXzjOfe7T6mFsQ/d815Qz
qsRH3htrZ2TtxyqHrt4UZ8rnbfOCdo+YEB7IFCbPaQ7ZrUl8RtqRXcMOhbAR
2yfaeSvzeVv/mu9jx98MY+VNhuU5V3SIl7wr+HF2cASL/qv+SaWTz+F2r1Iu
PJM5kUmqKqmutuHztiOWPnftNSKS5ZlLiaUS+bytytsHbw7FRjKXYR1x6YWG
xCUNd2468Wgia4hoXd/+ks/hWt867d9vfwRbMPf12QQNPm9762P696j28Uxr
w9PGpkA+b7vyU8TnBb9C2fmD9+TC1/B52y/Rq8MHnwhiYVev2lRXmBCv1rL5
+CHVn/2uOx3u24XP2z5X3pswx9CH7fh2OPO6A5+33XlvuGxuoDuzl88/KprF
523fFdx56RLtzD4o35laF8znbV0sKm/fzbRmTrs9F2V/5Xzw3xO+M66bsFUm
lzcHrOPztqft+2TIuemx+lOOBxXN+Fxt78Ba0wfpqszUvfTC7Zucs2zZyTov
5dm8SouHmbF83rao232PNnVJdifs0Bs3ST5XW7L6b/EB3V+iqcl7lMqc+Fzt
gYfXdhS/ficq66JhmPKU8zKdedEft78WocsOjj47OuLg6ImjxwyOLjP6weDo
CKPDCo4eK7qhNEcv9EPRX6T3j9BhRC8QHN1A9O3A0bm73hwe5xn1jOZVo69P
kMs61y4qmn/p5QaDz8Q9twasFfeSZPvdJhsHtnUQL7Sx8XvTX4n5dVt+Wja8
K82pbf3zNW0vU2d3Y8eHtbhJEx8dsPrIxWwDNvympcQdYzniy759s1XQNWeN
fSRHV2cqEU+c+W5O00Fbpuf1Mf96hzJx42itJX/vMhaRUtNemtCHz8kqvPNV
W+DBdh247FXQoEr8it+waz//+rD62kPbsgP4fGvHySufgtf5M3W5LS2Z1zWJ
v3tf9meBYxALcV7onGLP51uPt3XmHjgbwnyGnvSyrtYhnpD/cO/ti+PY+1EH
pxcp8vlWj63F0SoS4Wx53O5tBt58vjW3yLlrOZvABi7adCUng8+3/p20d1hU
2ARWsW15i0qZIXGFSyFHn44OZ7HH5qusbudzr73VPwzqJg5jUreTnKXM+Xxr
SXDZv2XjQtnexrjY9Dg+36rV9nOu7acg5tY5fm17Lp9vHfvO6keI52j2SiXg
dEKdCfGy+KBPZSEj2SKzYS+bVPh8695Farvnn/BkOp7OsuG+fL5Vccqa6Vkf
xezi/+5b5vOtLLm1OSXAjh353+cHPt/6d/qBBf2eWrB/Xv93Hxfny9MCnQ8t
HMBGPeoxRns7n28Nbu2Q6PZPk+2OWBxfZ8vnWGeuuVX3XFaZtX74syD7Iefm
cjOzVM5KMTY3ZVPAdD7fWtxUkhsT90u0VuproaI8n2Pt0yPbV9Tzo+jF+oTz
tw9wvvOf2dRXUxpE5lpvqzI9+XyrrUP/byNOPRAtOBjZ7NbI+TLrkUoJOtdF
2lfHKJZp8nnVRRtiP00bdVyUOOqBfsoZzocMc/qS9bNIhF47zQUL3Xb0xen9
I3TG0WkGR68ZXWFw9IXRZ6X5U6HTip4oOLqi6F/SHLTQwSxX9T5rW/2C5kYj
dq77t2dwm6hF1vG77+XPxJ/qlJelj+0UNd1vdFZa/Zt4Un+JYf8+SrPE4dev
xBV1pXmxH2nLgu7O6s0+Fm2P9d4gTdwrJZydStViUxQTZQelyBGPe7dM4cW/
Aezrad0xUkeUiEeWLvJjpy2Y5fueB9p1+ZzpX1utCq/4oSxR43tH06Y+xB9r
7u/otZ2xwyMaRlRL8znTP6rzBmSvcGct6bd2XU/lc6Y9JJN6dzvrzYyPnGgr
/ahJfElJr7gDsqNYXMMu14IIPmd6yn7axwrZ0exhlvut2+90iL8aIHFmn3Ug
m5lr/9VNn8+Z2nw6daTb3SDW6+xgzbIwPmdaoO7iO/FAMDv6QN/DegufM1W8
l3SiuSSYjWpRm1Z035D4rm9ntQ83BbFPkgrZBjJ8ztSkakraq7GBbLWm5KUc
Vz5nuqA5eehDh9HM1ObnW5V5fM501pLY+pzkUez2yI+9Vp/kc6Z9z+88b2o0
nE2JfeUg1WpCPLNl1ouML+5M5v8uDw/kc6YffKV72Q0Ss8LsO6vaJ/I502fd
9mjul7FjXkcvn0zYwedM9XxH1fnbWbKk6HGtM6fwOdN+d0ZNzC03YlfbHkua
S/N50n6hFyd1363Neqf7qbXkc25XmaZv6K7ComRumxaI+Typ5tbCfxYyMuz4
Zg+XqGecb6hZemxO5R+RpN6lIO1UPmd66l5V2Dutz6KAYocpdX35PGnSkmdT
qzteifIdTs7PPs75shjTHr1CHom+XjffGODH50y7eTiXTzYqF7kFHCxQ/Mj5
96vbMr7MPSna+MKg7PZyPmcqcvLYrrZ6n6gxfndl5gA+T1qvdenYlSebRVY/
+zW5XeXcQeu573a9TNFDJUWFsk4+Z3qr4uHSA91niPR3LtdL2cbnSX9crRqb
rxQnQt8dHJ139MjB0SVH1xkcfWd0iMHRI0Y3Fxz93Mi9/ZZtNHhJc53aShXj
D3q9Fplkn5TtGf6FuLWay/KI859FhmHho+8YdxI3VOurV9j0R7R2neTd6x1d
aZ6rVi1q9wv3Hkzua830ggZp4gqvSoaHdSiz5UFFKlnX5YiPXPyuvjhbk0mG
icOsq5WIK0Zs6Xn3ogFzWT2o2MCbz4GuvLtYo7yrGVtwQUWiT1kf4qyXXOzW
5VbsfOtffylzPgdaEzSyW3moPfut8y6/PZfPgdZ1ttzQPMqY/eiH7U0qfA7U
80aDoq2KG0tZfN6rOpPPgW4p37D+i64nG1f7cETUHx0+Z2caGvPn3DDW8eVG
cp0tnwOVuKU4Ui13ONvW88KugOl8DrSmN5uaucuH2Rkcr7h9gM+BTpny/O2d
rT6sxqmwza3RkPhx0fl/69cNZ7ODdvYr0+RzoPNKPNuX5Q1jfaZtcLUO5nOg
czccd3Pt6slOLMuML1r7/82Blmdsi/rsygJ2z9tkcJPPgX7b6NUxaApjX07N
OJ/Tlc+Blq7YlpaiZs/WV8Y2qzjxOdDGpPGHJvpZMYt34xRXJ/M50J/jah20
9pqy+1387aSO8DnQHT6rfm5LNmBD//f7C58DDdjkMWiAsSbL3Na/vEOJz3vO
P2bb/u+QMqsx3Pq09DDnYUme4m510szomPKnmcP5vOfgUg3HceP/iOaIVnc1
f8O5YdaQmOuTP4sqbkmrtizmc6AyCSuHaTc3ilTHZJgU6PB5T6V8WdXV7x6L
JjV2sqjznBfdbfw0x6VCdCpxTqD2WD4H+mC70sFbt0+JpDs/x9V95/zn2Vl5
zekFojFZU9OzN/I50MiFJ986xW0RFai8WR9gwec9903RizMsXSL6sWfifsW7
nJes67Q7d3CmyMus/uztyXwOdPurKcYbciJE2WeC72d25/OeoZuCAxObRore
eFQ1uuVzPnJWUtP34U6izPDrcmX1fN5z8rY/g3KW6Ypq3ot1U+byec8pf/0D
Z4eqitCJB0cvHl1zcPTN0eGmeWqhx/2yTOFIz/BXNI95SXrWJqewx6Ip+hba
FR1fiMtOfzC0qOCVKPrtk9is653EvSQSjoqjWkVHqsU1Bt58fjPevbtdVuIv
kal9nzQpcz6/eZetk2uvlWQHdrzTalbh85tstVS6Sb086/0vfGLUHyXiY/sd
Lmjc04f5W3gfD5jO5ze3/f21bOlgLbZ2opWUe2Mf4o/r/eNUvQ3Y3Q39x1gH
8/nN/iuvx0S5D2I9r0kfMLjJ5zd3LH67rWKiBfNu/9yh4sTnN/utenFN8ogV
yxxQP0LqCJ/fPByw33zcDVvWaBJ1v0OJz28OjP46/GGsPctwD/k5czif3ywd
uNfDY5gj0x0/UrdlMZ/fnLF/vqeygxO7lOw6POo8n99sDBuu1KzrxCasGZpU
992QeI/23GmJXR3Z3/2mOQEWfH7z6ac/F1512LGdF3XLb0/m85umTYu9Yl1s
mdOTvp/c8vn85rk5VuKb/axYXVtP1bJ6Pr9ZNr2zeMZmc5Yq00Vs3ZfPbw6c
den1LiNj1k/ve1zRKD6/qRRUtufMCH122qFlvcFyPr95Iyk9IyFTk40JaDib
c4XPb874HPxIxU+F/T75fobsfD6/6W8y+POCZz2Zj2vMsuvqfE7z398LfpmT
u7Id915szzjFuXnZyL0np/8QtYwde0QUyOc07T00U6Zd/CByfPPoakcb53fc
Ppat+N0gWpk06knpaj6/mXHK50hM4ENR3b+bH2YO4nOarqPNs6SGlItMVrl3
Ma/gvPCfccfXqFJRmtrFPi1RfH7TzOx1w6Gb+0W399oPKpDgc5oig1OpsSFb
RBqWJ0RROzkfFD1jrFn/paL484MDtB34nObckjl5pQ5JonPeBybVVXN+ou+K
4rzGCFHPav152Ul8flNvoaWF6b4RonGRu9YFKPE5TQ/7tuV2S4aKDn1S26d4
mPOaB4uWed7RE/1O3Xjmtjef03SxezrFJVde5COtcC+zmXPXxqPG83f9cN6x
MeuV22I+v7m4a9Gf9y2Nzo5F6T3LyvicZqHPmzqXM9ecV9r91E4J5fOYpmH1
T017nHV+dG2mtfV3zl0sqtz3nypxHjgw2iqvRJ7m0U7kKlYuGinL7hVMV8yz
7Es8+cThnoeX6rIsR3OppBX9iUvItI6qOGnB8t7M/pM7g3ftfRrWZYlf2LIF
lsEpslq8U28jZ3K+/IAtC0uz+TrzFu/Ov1IbpzT2qgWzL1eZVjebd+Q/buqS
nL/OgK2UTv2Wu5534bfUn5Te3V2DpRW9s6yexHvuL+Se6g5wVWXx/qGJsiI+
V2h0atXT7NF92bjvN4pEKv9fh/1l4Sz1wt7MZ4fd+5nv+fxgY4PZ2r1n5Jmj
uNCo4BKfE5R5wZQqZSSZSZNqTN0W3j0v8Teu78z5Jno1fK1KVQjvd+u7sC2/
bFxYzvORsnklvMfNCh2GKLr6MVP77gvtl/K+tnThl3HlisGsx+KmjlxL3stu
lpm27VtJEGu+cy1Z9jnvX9cUXXja7/0odqXv3raZK3jP2m//rotSa9yY3/9+
f+F9amONBX0/V9szcZDl19wZvCt94sSZIbvG2DKLjj0W1V58jix8dQ9v+ak2
TGe30jRZLT4vpjW15mbdnSFMyX3hIdE3Pi/mo2Fx5uFoUybxrvXtzFt8LixZ
xTm8TVmPta2eMKBgD5//qn1Ru01yYW9WsmiqYtUT3v8drub9zt0olpWu2Ny1
aivv+Q7SnmV8pC6BbXw0K93eg/d5yzOmb0kxT2DTtQJ/5n7mvV25zd/6TwiP
YSP/7zafXbyf63J4/oJ9A8eyPqEyc+z38Xmi8c7XxlaM9WdSf1NOJKTxju3t
kIOB8lq+rD3v7edcfz4H5FsxOmNg7QjW5BViXm3E5332Der586jYmz3+UDFV
9i+f9yksuX5C9ocrK18/9KDoEZ/rsXIZcK9W146dtC14M/Mgn9/x8fVbNei7
MRPXGyyNK+Kd0Mo86WXLZs9jWgO6SVSF8O7n0Z55H5YWp7GOxMZUeyne67zr
/uKGksUs9uTMlfbcEt7llJjwcWVJcRyr/d/nUj7fcbjZ3LosPoLd2n8p2X4p
72Pe//2zpn5dGDvrY3E8YRyfy1h/XnP7yjHj2MG23W25lnz+QutjnktCwxi2
fbPi4GppPmfxr/iAx9dX/myFw4J42ed8zqJ60LEEmROeLPXFp0LRCT5PseCL
/5r54+1YwZyNGXFTeTfQfW/a0SVbM9iSyzP/VKrx3p/hiQ0ShlJpbGLP0Sn2
5bzr16eiZefDtOls98GtvypP8/vnk1xLPid8iWFrR/UYKrWW9/VmJyv9W6se
xRZ8mzPLPob38kbuSPq41C2SJW57U5LgyO9vb3zp5jX88QQ2QTSmNbcXv4/d
eMyWlcm3x7BRjddNq9/w+9gXmR340V/fl7Es2ymyF/j96jU3J1weoi9mir27
Lowz5V2wNHb0ys3nc9n/bit7wvtf+hlThptMTmQx//u5z+83bpJc2Vx+P5oF
/Rz1s3Ir73DVZv+Tfy0fyTx2XrSRSuRdrZvejt6BdhOZjat5kr0Hvx/Y5a3y
m6Kv45nhm11HEzT4fb/ZrXWbfo0JZn1WKXzK/czv+922JaBksmgEk7Kcb1Jd
we/vNWqeMyDFQsRmF09Pj/vAe0BJTZnVvu9i2KBOo3lxd/j9ls5mzbd9Y8KY
em72mZw8fl+lT8DuC0+GhjBZT+kflSm8s2Nr+ll20Jdg1vF+trXUKH4/ZN7G
Ov/I5gD2fm3zDHtDft+jTMvKmKm5I1mtdfCRhN/8vkeVH9atI0tc2a2n5R9y
q/j9jeH7pPILP1ux1v/tx+D3j3nmDthjGzuSvfB+kRpXxO8T0/5dkfxmmhe7
/8n3dE4G72uc27XmyZwiD3Zx44X2yhB+f9d2109ZPsWu7IjdYCspc34f188k
kwP9Ljqy3c92TreX4vdruW+TWb08xZKtzZAvTqjj92vpznoePWGHHtv7ZYCT
dSm/j4VNfeYescGSbcreMjduFd+fP2lsUeWEGWZsiVP3UzmR/P6TJ4+v7dZ/
MJDNepn8rdKO32eyrOL1nBNj9Fj0siZLKUV+P4mP1u6F2nKqLNAkKNG+id9P
8kV76/7mBT1YiPi5g/UWvge75YLV3MdSimxY08iUuKl8r3U/O9uPIdulmd2K
8ydzXPnftcWh617o6XZhA83Nvlaq8b9fb6xdeaMmsV2k9ijHQqqV//1603C9
Q8fLmkVyazbbW8/me2hH/fsc5e/1UtQ5RGpO3Aj+d7Gpp1bfya2oFn2omXUi
R4///ctcpq/qTrU7ovp5rz9X/uR//0rs8fPh6ZLzoot1I+ysg/l18mNtZfb9
eheIjiwsS44z5dfDK/N/7PZS3i7abWh6PKcrv+79PVIj1thhpSjJuNtQa1t+
vSv23HspM8tIUdS9pFlxcvy61pE/+W/yV/uLjOacs7Huy38PbXP4cXnxJSUR
PpfQ/PjVvOCLHbK0XwM/X95qXm8ep61H/x+4f9mHoGPzXej/h4cTJUydzpla
0v9PfeSZHi0aq/3occEfNe9K+BMaS48Dv20cftFGU3koPQ745bblf55ZBdPj
0BxTsN99684Eeh40x3Tk/WuHZfPo8eGZ809fPhpsa0uPT31A04jwxLFB9PjU
r8zpqnCueio9PrjW5Lr2Mclp9LzBJ938NeDEkgz6uvCDprzpQJ0qc/q64H5X
5b78WudLXxc8Y/QUw/P50fR1wXss8bt+vTCJvi51uKrfrtPOSqXXSY8/MtBM
bs9cej5033LoPclN9vr0fOh+1FEvfI/NcaXnA97zmqazTGsIPR/wd1/rJLXK
JtHzAX9gPviy8+9Eej7gzarHb9denEbnAp5V0X2D9YEYep7wQ2PP9u2XJdTp
eYIfKN19putfO3qe1MEpf5/7r96Pnif4qK2ZKS7dI+h5gl980btzyqIYep7g
pu2hV4/NjabnCb61otvd7nJhdI7gKbPlhmwbOJKePzxzXjRq9AxVVXr+4M3r
9NfvtbKl5w++K+5Z1/CvI+n5g2c17sgSJ4TR8wd/1LPTNk8qip4/dQdK53zw
7JxIz5/uv7rcvorphtDzB/ePXpo1NciLzh28tqm86cJ/P0don47gt6qnwbdV
f/Wh10V7463X7KjRtqHXRffhDP+2pmD+CHpd4Lm7urv4yY6j10X73lcGRr/o
E0mvC1z9rdj2iDJ/XeC3FS1+PL4eTK8L/OfiD2alWR70usDdTOcMWOJiRt8n
8MKRr22u3VCg10v7k/PlTk0+q0yvF3yTeFBZssQQvldC4KuOuuzwHzaMXi+4
ypVxbiU9x9DrBV975W4X18fh9Hrp78KNMmfNNcfT6wXf0nvt0ElxAfR6wX9a
BP96O8yVXi/4AsNBSnmRA+n1gl/bPTtF0l2avq/gGS3zFo9Rekk/L/DzRX3I
NkcbzZ50DvR3wJGdU8crDKJzoL2yVqcWVG1woXMAD538p6RnsR+dA/iZ0505
Nf7BdA7go0LvnyzuHUTnAP5+auSlfxoj6BzATft22bw01YHOAbzm61N3uV26
vDcjcOP20s3n1kvQOYBfXRT3RNuqmvaq097aHOXE9Lz9dD50/Tz+h7lfugSd
D/ilYJmBk49p0/nQ9fDpNr8vRtjQ+YCXrVv6LH64O50P+Kadsy1KBvjQ+YBP
7zu2fnm+N50PeOvH7xI71FzofOjvESsNf/mrW9D5gC/VnO2zJ74PnQ94Qd6t
jUzzG/UwaO/ixAdLYvfeovOh/Ydr7oyu6L6N9juDHzqdtDKjeCKdG647tSbl
zh77oE2EcwPv/mFf1CwTJTo3up5p+Vup8L4hnRvtu3u86YrScWs6N3CrrRVG
JiWOdG7ggeKWaSYGDnRu4M9/r3qYOtWSzg384f3z4m6TdOjcwNcct00suitF
5wa+SfvJkfaZr+ncwF9N6V4R0+McnRv4yi9vtu0ZsJzODTw76ePnPwv9aP8s
7S30eDrXqpcSnRvO9ZPwuQ7nBv5G+FyHcwPH5zea0xb4JuHzGM4NHJ+jcG7g
GsLnHJwbuIPweQPnRtfThJ/vODd6fOHnJs4NvFL4uYNzA4ef6b4KgcNX9Pck
gePfKe1hFDjehzg3nOto4XMvzg0cn29xbuD4vIpzoz1XwudMnBu4v/A5EOdG
19mEz2P0OUbghcLnH7rvGdcDhc8V9PsMPC38XMa5gePnF3lW4PA5zg0cHsO5
gePfKc6H9tgIn/NpX4DA8bkd50PX8YTP2zgf2hckfB7G+YDjcyn1/ASOz4F0
f7PA8fkK5wOOzyc4H9rnI/wcp/sPBI6fa7TXVeDwOc4HHL7COeCcJgm/p1DP
W+D9hd8vqDcscBnh8z/OAfyt8Dkc5wDuJ3zupa6YwPF5EucAjs9jOAdwfG7B
OYDj5zjt+RX4eeHnF84BHH7G68V54PcmvF7aQyL8XoPXC14l/H6B1wuOz/N4
vbSvQ/icTPclY5+D8DkTrxccn8fofhSB4/MJXu//6+mObRAGoiCI1kEHVIPo
xCExTbgHl0biGkjmbTpCd/uFZM+373y4+7J6cfcdde16UF+mru1brz/aPpm4
fkRdOM9XF/7Ik7duL84n1YW/8it14Udeoa59x6D7pvzbN10fJz9+1jftXDf/
c/3IzunB83n5N07eKz/+ywPlx+/8Z++n4+7vcqrjUx+3/Qnxd/3RzvVxHaq/
kHPP5fJwOXFeKuf2XeZjcuJ8Qx559WXy4PoaeXD+Lw/Oh+XBeeDed8Z5jnnl
uuqbzLt9KPUXWwcZf+bb5sV5pnn3vC6P2vkizf+tT9k6uThvNz7OV/dd2jgf
M455rjzfOFt3nd8aB+dvfm883rv3tXFeh299UV73B6KCCCQ=
       "], {{
         {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJxMnXW8VcX3hs8953KLe+rCNVAUQUVsFBELRUWwu0UUExvFDrCwsLsLBQvF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             "]], Polygon3DBox[CompressedData["
1:eJwtnXm8VdMbxs88n3PvPfucWxlTSomIlCFKRCkRERGJzCQyjxWRqQyRyBQR
0U9ERKTIkDFSUZRSIqJEin7P17P/eD9nPWetPe+9hnd43p0GDDrmwlgkErk9
F4kk9Pu/2kjk0mok8o/w7Hgk0qgciYysRCI/piOR1ZIPipHIT6q7RXXHSx4W
flP4D+1ktmSB8J/Ck1V3leRD4Z+Fb1P5REkT7e9u7e937es3yQzVf5PzvnaW
vC68JOe2u0gmCM/h3FT+XPtP1ej/QiTypLa9RNJK+xun/W1W+W9JWee/TP/9
HY1ENkm2U/l21f+sup8kD2l/b2h/a7Sv6ZK3hb8XPk/7308yTniG8ArVTZWM
FX5VeKnKkyUvav+X6f78q//eVfsdtf9R2v+v2vcvkplq/63qTlXd7pJA7Zer
zeaoz+l+1U9X/dfa1yTJG8JLhU9W210lDwi/JrxMdVMkB2v7aKBrUPk2ycvC
S9Vmq9q8r/aPqPyWyn+p7l3JNXU6N7VvnNK5STqpfUz4jpj30Vk4LnxnzP9t
q3O7reJ7s0ZSVf0K/fePznWL5HrtL6r6ptpXE0la97+F7v/TanuZ5FrVb9X+
dkr5mDtr23vVfoPq1kse1Pm9rvNbpWNNk4wXnin8u8pvSTpp+5t0P4fldayk
3hnhLdrfDtrX9pJ6nc9K7fPfqM/pQNXfqPZD1f4atW+mujE63h9pH/NL7X+j
9j9J9+ZyyXvCPwpfq3J3yTvCPwgPUbmzZLj2l9T2u+hYzSVXCm8OfOztJF+o
/R9qPzHubd4XXiN8g8o9JfOFNwhPUHmwZCedz10Vv9vrJHNVvzrnb6Gr5Abt
P6b6nVO+p7ur/YPC/6jtFsmpJbXRtQ3RtXWWZHS/W+p+P6u6KyWDtf16nV9V
21YkLXR/ftM+anUvaySXCl+o+/NRzt/TMzr+hyo3ivsZfCT8i/Bdcb+jnwn/
Ljw+7m/gaeH3hYO438EW2vdYnd9fOvafkitU/wD3j3dJ0kPHG6b/Jui/+4WH
Cv9P+DPhPbWPW3S+OW2/m861lWSccL3wQSofKOmm9hfofB9X+3u1/WHC5ws/
xjkKP6j2DdS+Y8rb7KrzeaDib32T5BMdax3H1rEGSh5S+4aq75TyNp+rfr3q
H1XdhZJPhX8THqfy2ZJJwh8I18f9zV+t41+k43+S8/c4RPvbqPvdUPtqILlW
7R9W3XC17UV74Yd4v1TuLrlI7X8P/GwCyVWqf5Dnr7qukptUn9b5teTZ8b6p
fpzqL1ddF75f4fG8Xyr3pP9R+6rad1DbAyRPqP4d1ad1bl+q/gqd7yCd78c5
95eHCJ8n/GjO3/hEtZ+rcjHuPme28Iqc70UHyZPC7wrnVF6o+hE6XkbH2zXl
c9xL9/sR4VhG/YDkOrV/RO1HqG1vySVq/0fge1MvGSacqPhbaiZ5Su3fU/va
uPvQy4XHCg9WuaPkOp3vYJ3vpzn3lzvoeHdW3JeupY9X+3tVN1Bt96bPEF6W
87PeWzJPeC3XGncfOkh4dM7n1oTj6Xw26fy21blsI1mo+k2qnxr3N7yLjr+u
7G+nJPlK9X+pfkrcfcYs4eU5vyvt6KPV/ne1L6ttneQ54SE6/y1q85bqL1b7
e1QeoLo9JW8KfweO+3t4Su0vVvu/9d9r9Bmqv5vxQm13lzyj+ktUv1n/vaH6
d1W/Kue+rIvkMl3PX4GvpZFkiOrHcH7atp1kjvDKnPuijpLz1f5Xta9T21rJ
hcLrhMsp/3eB8C+B62okz2r7edp+u7jHyI+Ff+XbjPsaLhO+n/dHdR0kbXQv
Hq743YhIdhG+v+K+YqPkUrW/j/FVbfeTnC98R87fSiO+F+G7GP9U3lVyofCo
nL+tHSXnCt+W87cRSM4RvjXnvoc+77yi5y58W/Xccx1/fMXnslXHv0D1d+b8
LW0naa36h1T/b9p9bnfhl4W7q30PyZm6H2t0Pyaq731SklX/u6v63+fV9hrJ
MuE3tc1AjUVnSL4Xfkv4LJXPlDys/vtN9d9HpNxnFPQ8v1H9aNWNkqxQ+1nC
Z0e9zSrh2cLnqXyu5Afht8sunyNZI/yO8IUqXyD5Tnhm2cc+XfKj8Jyy685n
vNbxH9Xxv9e5vyC5Uddzj96naXmPcdcJ3yX8Yt5j9s3C9wq/nPc3f6XOd3dd
f0s96xaSodrf46r7Uft6RXK56lsF7mua874L7yLclLkd74fwroHrmrG99n+3
9v9S3mPeENW3CDw2sc1Snf/rOv8BOvfTJN8Izyi73F9ynLY/W+cwUtvfq+P/
qvr3VH+J6i6WrBWeKzxY5YskV6ntg2r7idqOl1wt/JDw5yo/iuj43XT8o3Xs
oyRR4S+0/U3a9kZJTPhL4RFR/3extr8n72u/SXKN8HjhL1WeIBksfLfwSyoP
k1wkfFfe9575zCDh0cLPJz1fOUzXc7Pux41576+b8C3CNwnfInyd2j+i8ndJ
b9ND9SNVP0L/3Sb8s6733bKvdZDkGNXfpvpbVH+X6peo/rWy792pkqNUf6vq
b1b9nap/WNfXNfDc5Uj6OOE2gfve3SSXCDcP/Cx3klwjvGfgulbM6YVbBx4b
eUceFD4s8NzqCMZ7nf8kHetfHWue5Hbh54Rz9L3Ctwo/K5xI+ZkM0vndrvN7
Pu8+aaTqn1E5mvIzHKf9dwm8726S8cKHBz73HpKxwocGnlsdLrlT+EDhg1Q+
UHKTcHvhfVRuKxkhvG/gvpz/Wut+HaXve5eM55BthI8R3jPjPmIv4WOF9xVO
SfYQPlp414znRHsK9xLePeM1yFKd/1ad/w4q/6xruE14snA65XfmG+F/hBuq
/gf9t0T4X+FthX/kexR+Wniz2r4vSeh8F+h53qJneTPrB+FDAl/rYYxhav+Y
2v+Q9Dv4vM7nQbX/WF3gR5KLdX/v0P2dkveccYrqH1L9JxG3eUb4AeEPVf5A
cqna36n2/8u7/7paeLTw1Lzn5FcIjxJ+Ie8x9Vbh+3QOrwjvLvyQym+ofGjK
z/Mm4YnCf+jcZksm63jjdLx5ER/zWeGxZR/7fclwtX9C7X9V2zck9+p6D9b1
HsrcRnK3cKfAc/eDJWOEOwceG2kzTNtP0PY/a9vX6P+EnxT+XeW3JCOEnxL+
S+V3+Ua1/d7afg/eBe6ncNvAYzX/3SXcMfCxOrF/4X2E91K5jSQp/BVr5Kif
UU54MXMYle+QTNbxPtLxrtS9OJz5kPB7wgNV3pvxVnie8GUp37NJwh8In6fy
fpKnhd8XPlvldmyv+32/7v8S+hThiaqfq/IAlfeUPCb8tvCxKffvjwvPFj4+
5f53gvAc4RNTHg+eEf5Q+MKU55iPCs8SPjrl5z1feAP3S+V3WQ8J/8H7qfL7
vNPCG/neVZ7H8XT93XV/jtG96cV8W/W/8TxV9wbfvPB63geVZ0s+E/6d56Py
W5yPrm+srm8pfajw/4THCX9HHyz8pNq/q/KpKb9vTwi/I3xyyuPXc8IfszZM
eQx+Qfhz+taU78FLwl/SV6a8j2nCCxhbUr6HLwt/JXx/ys9oqvB8+t6U79n5
ur4mgXUHrzB/EW4m/LrKM5iPCO8UuO5l9AXa/gv63pTP8RXhhcLjUn6mT/D9
6Pq+0X/PMj/T9k21/Wva9lV0AGr/qeqGpfxMZqt9B81XPlT5A8nzqv+EtbPK
R0peFV4s/GjKz3SW2h9Q8bN6TzJd9YtUPz7ld+x/wp8xFqX8zgzS8XcOfC2c
w2PafozO72u1eVr1C9T+T/pn1l7Ci4U303cLf0efLbwu777vNcki4b+Fa1S/
RHizvvdP9X1cr2/jOskQ1Y/J+1tnfPtD9fNUf6XqrpBcovp78/6WGR83qv4j
1V8VdZu/hD8WvkblqyWXqv19eX/rjHfjhWfm/e3R/21S+0/U/tqot7lM9ffn
3Tcxfq5T/fuqHxL1HON34Q+EL2PfkrTuz0J0Jirfyjlo+3F5j3XjJFvVfr7q
h6tumCSj9ovQQUW9zb+q/7zsuqFco7Z/IO++/n7mV8Jj8+6bmO+sV/sP1f7y
qM9hi/BnwjdEfQ8fUfu38n729MfXCj+c91jLHPZr4S30/br/3/POCP+U97Pg
fZsnvDbvOr63OXreB1b8LfOOfSx8qPACvnXJR8KHVFz+Aqztf8l7LHuJMV54
k3BBx1vM9sK/5j228f0+pvtxROC+gTnYV6r/i/mA2n+u+te1/4f1vq3Iu394
U3iO2qzMu/95VXi86r/Pu7/4UHU/5/3u0R/OVX3Hit/Nj+mjhDsJf6byp/Rh
wgdVXPcR90P44IqPTZt5wp0rvrb5kheFH9TxluX9Pr8s/JDw8rz7s+eEHxD+
Nu/rf1/nsybva+d7Seh5NdP8YJyu72zJcF1/u8BrV8aQUcIHBV6rMWc5Ws/2
dR3/Rs0nhkv2VP1G/ddQ32IDdA4qv6H6mzJuc6Twq8JDVb5B0lP4NeFhGf/X
RfgF4StVvkKyQOf7qM43q3M6WOfTTvv/W2120L63lxyq8v8qbnu5pI3q/yx7
rcY5sF56RfXXowuQHC78ovDVKl8lOV37Xx14bvy4pJ3qH1f9uao7RzJQ9T8G
Xls9ITkiXH+xr2slA1S/KvC2j0n6C/8QeO7+iORU4ZWByw9Lumr7l7T9NRmf
w2bd/+10bdfr2o6UzFf7R3S9Gf13kPDeup6/tM22upZtJG2FN5W9NuS/P7V9
w4J1y9yfldr+SW1f0n9HCy8XfkK4WPD+vxWeIFwQPoI1r7ZvoPJFaR9vteon
qr5G/x0r/JfqGxWsOz0UnSX6Iv0X6L9T0WmpvE3Bur7DOZ5wnPlm2mPw98IJ
5ptpj8EH6Nyf0vUP0rVfKPlG+3tcx8sXvP1KtU+pvHfaffYi1T+m+lzBx/9B
9WmV26X9Da4QTjL/TXtM/1t424LXvlzfMuEY8+W0x/BvhSPCO6U9xn8nHBXe
Oe0x/Rwd72c9r0l6Vk9L9tH5PqrzPVvnepbkbNX/FLjuKd4Zrddna70e1bfw
iZ7HgXo+W7VNM5V3liRVfkbnv4+O8YWOUSf8vPB+wt8J54UnC7cXXizcQdv/
W/a2TSUHCUdYn8a8z4rqpqj9/mr/vdoH+l7bqjwv7W82o/pnVd9O/30lXBJ+
Tnhf4SXCh+j8RwgPz3v9V9D2u6vuFdXdxPkI7yb8ksrDJHHhnYXvV3mgJCnc
XPjxtN+Zrbp/jYVvU/lEnoHwjsK3qHy8JKL2OwnfqfLJkpxwK+EX0n7no8JN
hO9K+32KCTcVvlflAZKicGvh19Le5086/6d0/rUF779G9Xuq/FbaxzhR9ct0
vx7QtY2VnCz8vfBDKj8oaa37+UfZupeqpL3w5rJ1N/QpddrfXtrfu2mfwz+6
nh0Kvjd8D/20vxWB1+rsc3dtv6HsfVUkW9R++4LvHd9fK9WvL1sXhA7uWG2/
RNvfo23vlvQS/jrwWD5asp/ab1H7ndS2seR41X+r+vtUN0bSV3h54LGba9xf
7f8pW3fINr1VvzTwWMwxzhVeK/xM0u90Sde3h87vjbSfWVl4b+H3037GtcJt
hGen/UyO1vaLA5/bKEkf4e8Cj/2c095q31vtD9C3kZX8ovqn9XzqCn4f9tO5
Panv5wLVnS9pTP+r+oMKtkEt0PYv67+NelX+kCwUfkX4L5X/lOwvfBJ9mbbd
QfKl8LSy226QLBKeLrwp4m0+F35R+HeVf5M0Fz5c29dk/P7vItxVuJzx97dY
+FXsbxHvo4NwP94t1e8sOUl1b+t8L+QdFj5BeJbwBbzzwser/eXo2zLu03sL
X4q9SeXLJKeofU/9N4h3XLiPylfw/mc8Jh0jfAnfk8oXSY4VHoK9UOVLJEcL
DxZ+KOMxqZfwxcKPZHxPWwh3E65m3B/sLNyl4GfB999S+AjhhsI/8A3pfdlP
z29f9VftJbcLdwisWz8AHYPaH89Ym/E9u031BwSu259tVH8CfXXGxxyp+v0D
2yLZ5+6q71nwvcNm2lb4OMYy4YJkX+ETGXsyPqd2wn3o2zN+JrsJH0n/nPH7
0Uq4R8HPHpvoC8LjdU8/07P6VDJV+GHhzyP+r5nwYQUfi/60qfChBetC6H/3
E+7LWCi8raS5tp2m59mpYH3KrqrvXnAdNuSWqm+s/w4uWH/SlPFb7TsWfH3H
qe4y7F0Zz1mOU/1bqj+fPpDnK/ym8HkFz4Hmq/1L+m99xO9oXvfv67J1razB
D1R5utqfUfD70JH+vOJ34WLJwfTnwpeqPEQyV/ubrP9WaV8/SHow39L25xT8
fr2r+mfLrluJzkL4OeEfVV4t6cR8TO0HFnyMfesjkTOrHk8YV7oxvxM+u+D3
uzPjV8XvNuewr/ATFX/b50m2Yf6m9h0Kvn8NmJ8JH1Dw+7e98FThAwt+nm2F
H6t4rsUY20H46YqvnTnCIcwP1f7Mgo95EONpxfdisOQw4RmqP6vg+/+Bru95
/bcm4mv8Tf3RJNWXCx5/9mc+KHx6wd/PHLV/Rv+tUNvvJTOEnxD+WuXFkreE
nxL+TuVvJe2Z/2n7AQV/j6+pfkLZbRdxP4VPU92AjHVyM4Unqn6p6pZI3hB+
suzyN5KjVJ6p/Z1b8Bz4JdU/ov++UN183nHhqcLrVP5V0lG4P3OvjHV404Qf
Vf2XEW/zkfD/hNeq/LNknvCUsss/SQ4QPpm5Xcbf2GzhSWVf+3LJQcKncq8y
1hnOEn5a9csivgevCD8m/JXKCyQHCp9CX5+xTnK68OOqXxhxm0+EXyj73H+R
PKr50izNlzZrrHpfUmAtrOf5jZ7N14w/+h6+pf/Xt/An9oKibUsz1besk6wu
2hY1Q+W1ksewP9DfZvV+SD7X9tV628xexP4tXK63jf0Z4Yno35nvqO1Nknmq
D+rtA/Ic9kX01cx/VDdMsqZoW9ks1a2X/FLko9V9Vnmr5HHsIXz/anul5CHh
N4QfUPksyc9F+17MVdtNktk6Xm29bfBPCE/AHsL4obbXSH4q2pdljuo2Yk9T
+7p62/Sf4ppDX4oPVN4ieRT7DOOFtr1E8hX9mH5b6jRP4v3V9jX1tik/xhxV
uFJvm+H/4ran409yvsoPx21vKNTbprZ93PZn7InnoDuX/Kz9/6v9X6x9PxGx
vjNTb5tpXah/TNfbhluK2z8GWy+2mb6hPjxfbxv/NnHrh7P1tuFX4raXfp6z
7vtMrlfH26rjXapjPRWx/w2+Athq+sTtv4ItFl1o/1Cfnqq3jTgv/AT2KMZb
7eP6rP0fsDdeHPf9H4f9hrFddSdLFqr8p35307FOidjfBds6toR+cdtH8RVA
l3q6ZLHaL1X79mp7TsT6/1y9bfQNVP+GcKnePgvc34WhP8bQuN/PMXrXt+j9
36Rv52/6T80fd9P71T+h+aFkW+HdhU9XeUzC9smbtf3+2CZjtqeOEG6PrVFy
j/a3GXsm+n7Jx6wTdH4sezpFbJ++ifurtmdiY1RFa+3/TO17LMdkLqTtN2a8
j4aq31X1fVV3u2RH4TbCg1R+TBJV/7qdrq+Z/jtM+F9d77bCOwsfIvyRjr8s
+98nEzlQ0kDbtxToo7qRkkbCrYT7qTxKcr+O/y/+UTr2Fsl9wv9UfG/4b3u1
30Ptz1HbBxO2f+Mvc13c7/M8He/b7H/m1Mj+9J/Cq4QxsR4WsX8F/gSD4v4e
HsEeyvxLbS7K2h8H+/UVcfcX47FfMv9S3blZ+9fgj3Np3N/jw6p/k/mY6i7I
2p8M/5mr4+5PHsReqPr7VHeG5DXdn2K9fTr4np7CPsj6RnW3SJ5Gn8f8XOXb
2J/K6/XbXOd+Aucv/JPwNoz1zHkYZ4UbqNyN/lZ4pbAeS+QQbBas+4STEb+j
84XXCm/P2CP5Uvg34aYqHyeZhP6Q9VTW38QXzNP121h1x0gS2vH2Ov/megbd
EvY3mJ/zXJI+Av+0L3KeK9KnnKv93cY+9EDeQKcifCtjP2MnY77wSK5B5Vck
pwrfIPxxxnPe/sJDhT/LeM7bhe87sK/NaMnJqr+W8814fn6K8PWs31S+V3Ko
2icDtx2F/4vep/c13o5hDaDrukL4PeF7WSPQrwm/K3w3c2jhQcLvCN/FHFL4
fOE5wqOF48L9dLzrWC9lPL9freNdU/U3fKruz1BsVcL3MwenHxOeK3wPc1Th
67AlCd/HHF74PO3vduaWGV/T+cJ30KdnfIwbsUWp/VjmqIwX9NdhHzFAx7uD
+U/BY88+vEPMDwvu23aV3M78r+B3cS/JrcwPGb9V3kUymvG64G/hIMko5hMF
v9sHSO5mfVXwWHuoZKWOf3XV3/DJOv6dzDcK/lb2ldyh8/1E9Q+yJuB4wh8L
j2OOLzxA7Yer/FXGa56zhG9hrshcSHK68I3CizNeU50mPEz4i4zXTGcK30wf
l/Gaqo3ez5l63kV0G5IzVH8T30DGc/SbdfyPdPwHWFPo+ANVP0Ll7zJew/2k
8j88d73rj0kuUP2dBfelvFP3s94seO5wbNb+dV/nbJunT8d/b3HOtl76pD/U
Ni18h/b1ZsT+fF/lbBukD8FfZmHOtlH6nA9C/1LGtpFx++MtytmWTJ+yRvvb
wnuofT0csb8q/qeMrbdLfuO9ZHxR3YsR+5vhf8RYOjZufzr8PRkr72Y+o/YR
+jtsqxH7W32Zsy2aPvIkXe81BX+7rHfnhf6fjIWj4va3wV+IsXCM5Fe1jdL/
aV/PRezfsyBnWyRj7jrVx+iv8d1lzNTzmK3nMYrz0HX9rt+E6m8UfJk5vPDm
rMfWByXL6Duy7vvYx0rhDVn3jbcyhxdel3VfxT5W8C1l3bfdLHlA1/NawX0r
7/wPKm/Meq7EMxqLPqDgvvh4ydk6/5E5+34VJSeq/qqC+yrWc8tV/iXrvpR7
/p3wmqz7Yu7BUt7lrPti5jCreJeynluMZg0m/LfwaSrfI1kvnMz5XF9lzSG8
Ouuxi2f0Ne9q1mMb78AS4RVZj+3MyRYJbxLeI+J9fiO8POuxl3e6Xn1FC32r
vfWtjpBUhXcR7qXycMkOwnsKn6/yw5J2+p52Vn9/rP77Vri76vupHNO31RZ/
EOGThLeorjX6ceGThbcKt0E/pO2bavtj9N83+q+H6k9ROam69ujPVL+T6o/W
fwtUv6dwE+FewouEu6p9X5U3qdxK7Q8XPlF4o3AL/FOETxBeL9xM+Dc9rzj7
0ruW0X/rhGPC87F9J+x//ALzcT3LTZKj0a9X7ZPG/Ol31Sc4ttrn1f4Q7f84
4TUqb6f9Hyp8vPBa5kLCXYT7CK8TboIPrHBv4VXMbZL2N/8f83nte2PM61nW
z4dkrPPYS+0PVfvnEr6/HcP1bUfVdWJNpfuxo+5HT7X5VPU7Ce+vbXoIfyC8
s/AOqj+SNQn3XHWHqPxMwvf7zqJ9Kxn7ZnG9qh+g+jqd2yGSnsL9hfMqH5i0
/oH194EZ62w6h/qGzhmf86iifV3vjvma9gvX//upbn/JkdrfqdpfBt2k5Fjs
FVX71DFf3T/UB9IWnWH70N7SPmOfleOx71TtszcIHztdX2Nd31E8w4T1A9gP
OmR8jseE9qVjMl7Tdgr1Jdw7dFisN6eyXkI3i31Y+EXhj1TeGvrD4t/XA9/O
0F6FPatXxvs8LLRHHZaxzgv9WaLePv5JvSP3aPsnmdtr209j9rfFH/AYlW+M
2T8Z//jlMa9vxgg/xdpPeIHkyNAfGZ9R5uPXhf6gx6l8c+hfiz/gUSoPjdl+
hP2qq86lW8b+qfRPnVR3ccz+ufgfHqbyFTH7sxIP8G3M67Nuof2JbdHBXRH6
Jx6i+ktjHq//wB884znI3ap/grlqzPdslOr/VP161W3IOD7h+Zxt/z/F7L9L
vMA3Ma//RrOWr7gtcxjiFSazNlf9ypjjFZ5h7R3zOd4n/DRrXZUXxRx/MIm1
e8z7vFd4ImtZlefHHH/wLGvZmO8x/vpTcn7X18fsz/9czr4Nq2O217yU87OK
xR2vMS3nffE88Vd+Oednk4nbHxp/zW68KzH7mxKPwbWyPsZfnXgB9s16Fv9s
4jG4tgah/w3+5W/n/L5h78T+2VP34qiM/ZlfyflaWa9i/8Se2SXjdw79EPZR
2qIDbxfqy9pl/M2MLtoXfUzMfdp/+kfVH5exDvtP1Wfz1pXsqO/nD+G08HLh
ivB64SR9s3BJeINwir5euE74d51/A73vjfVfe+GNqs+ovFL1DYT/Fi7krato
IfyXcC5vXU0T4a3CdXn3vWzPQrScd9+8P+tHbIl597UHSvozH9b592eux5og
jB/BZ575y2nMpyuuQ8f2r+pqGY+0bduE9e3vq/6UjHVkMe2/kve1dZL8qv3V
63p2zLt929C+t0/G9/TncD69g+rbqD6ucjXve8F6dpOOl89bF9RMeLNwMW/d
Tivh3dW+Y95jJ/11C+H9817bzpLsJnxQ3mtX+utdhTvkvbafyxgnfGDea3H6
95bCB+StC5gjaSy8l/DFKj+RsH/zLczXYu4vd1H9fnmv1WcyXgjvLXypyk9J
mgrvI3x1wmPOX7reRrofTfK+/38INxTeKe/ns0X7L+Wta2ot/I9wTd66L+7P
ZrXfRu2b5n1/m2v/++a9tp/BmCzcPu+5xnTGSOG2wlckPEbtLNwu77nHNEk/
1mN6Hv0yfobYX94VPiljnWRv9OnCvTN+x4/T9mflPfb3YbwRHpj32NwLET49
77G4u+Rs7Kn1jlmgPz1G9WfkPbb3lPQWPjPvsby35Hjhs/Oea/SVHCV8Gu+T
yp0k/YQH5T12DpKcInxR3nUXJx0fMzjvsfbSpP2lYvWOSaB/v6voWJSxMY9R
J2KPrDrmgP58IPbkqmM2GJ/wJ9hF21/LHC20bzYXvgadSNL+XNF6x3ww/mAP
bSZ8NTqepO2J+CdfhU6J7bE3Vh0jQ/9/gurOzXsu15964QvzHss5XmvhTnnr
fhiPf9X5R5mbxP0Nns76VM/ndD2bMyR7qP3Beb+rzO/6sp5Wfd+Mnyn2tNnC
J2SsQ07ij5B334J+oQ/2N9X3ybhNQvX1efdd6Jvuw15YsK72aMmequ+c97vO
/BH7VbLeMSj056diL606hoXx8SS1vyDvuSj3r4/wOXnPNfuxfmS9WPG1DERY
v1ZcPpNzVvvz8p7rcj8HYL+vOoaG8aKv6s/Pe67M/ce/N15vnwHGnxrhJdg/
o/r2JHeh7y9Yl3uwZAz20YJ1l0dm7f+9WvvvzBxD7dvq/h+Ws2/9mNBfa5Xq
D2bOEPp3/iTcRfikqJ8Hc+0vdW8mJOzf9gP2L+ZAob/YGuFDhU+I2p/3R+FD
mJNE3b8xF/1E245P2J//O/wR0PlE7e++HPsPc5Ko3xfmbp+r7aPh994F3yOV
h0ka6Pxbci+4lqj7P+a276MLlDRU/a7MtVTXPvSfW4n9jTE5av+7FdiTmfNE
7W++DHsTfWLoD/c99ibmOFHP5/net9Wz+DFh/+O/dH9zOa8viWf4Fv+MnI+3
s47fTuWWUd+/HYXboLuO+n5tp+9pSqB9aF9/sn4SfkH474TXLPTHXfG94l0M
+8PDhF9R+SbWEGo/Ve03J7xNU+1/H+2/edT3v7HwXujio36eOwjviW456usn
nqO5/ts35/u3ncqt0cVHfb2ML93xRdO+7w3HH9YK81Qel/D3wlprocoTJdtr
+z3QxUd9PxsJt+Lbifr5biu8u3Ap6udLf384vmba9hbJNqrfDd171Pef8esI
1c9W3V2SetW3YC6ke90mavvQ17rfVf33uv5Lq75Rzr7gtMEfYKnqG+q/d0L7
2Zf4J+Wsr+gafq+7Jv0MsNctwD8kZ/1DRvvbJmdbMe/Y0WF/TKxibbiepH/f
BV/OhNeTjCc7C/8uXNb2O+dsu2qs7euEm+ZsS+OZdAv7y92TfoYV1TfP2TeB
Z1gr3CRnWyvvDPbCL/DPytme0Tkcnxpq+x8SXk8y/uwk/KtwVttvm7MtkG/g
0HB820H1P6s+UH2znH0dmkY9Xl2u+hOTvibsiZ/j75WzPovx7hLVH5n0N4B9
eCH20Zz1RfhPfCNcn7O+ivHgSnSZSV8j9sTFqq/krJ+gv7wC3V/Sz+DEcGwZ
oPKeSY/Plwkfn/Q9ZXy9VPjYpK+R/pGxaSC6gnD8vF54sModk/b3WKLjNdDx
3tbxqrreXXL2DeGbZHxiLDwP35mk7ZPz8XfL2T7F+H+x6o9I+h5j7/wK/5Sc
9U/YwxcJBznrmxgPGEvPxhcwnE8MET466XvOeH+d8IUqd0haH0L/zrlHuaf6
np8P3Begoyjjjyf8S8I6g7j6+ovx30xHIp8Rz6X6LwP7Et8qSan+MvxXVfcl
PoGq/4p4aNXdjs+QcF3FvvX7SNJqf3nFvh5skxAeUrGvw3x8eNS+tuK2bSUx
1V9ScR3nkMVfVPtfoXP7nv6L/l9tUqyHYtYfsJ4i9pn4zBr89dT+p4R1IneH
6+l5qhuHPYnxQtsnVI5Lcvi/Bn63Vya8Xmb9ha/ceNZY+KcGruMcxoTrL2JF
J8Ssb0B/Qqz+fzr/UH9D7Djxt6O0fU3FsQl7Se4ULlVcbiMZIxxUHJuwr+R+
4UrFsan7S3oKLwzse809fixcq/0b8zU10vWs0vVEY3QgegZqX6w4dmEPSYn1
Sdm2asbwhsI/lN12K+OH8C/CBdZ3kmbCv5atS+S/bRnPy75XMclt2n+h4n23
lowUzlccO0BMckO1vaViboNV+KipvlzxtbWX7Mh4rzbpmJ/hTsI/C2e51+H6
mfUu3ASPhutt1p/E0k6UNFH7tWqfi3mbRtr/5MB944aE56voU7DdEz9dwF9V
9asT1nkNDO2RxHbyDrRW/euqz+neZpPW57wqnEpaB4i+Ct3g19p2Eusl1U9X
fUJ18VA/OCPwtuiU0HdNCyL/GbTQMaIfeyVwW3SSrYRfE04nfYyzwvURsZ9c
UxPVv6T6f1nLMB7hHxK4r/+N9ZLa751zX8/8qoo/W+A6dHzNhF8O/K1zDg2F
nw18b9BB4g/cot59BOuBJtpf25z7ZuZPrAe7qe4ttb0zYX3ei4HPBR3qCeqP
rizY9jEs1A+ie12qusn09/jnBh4b6FO2ER5WtU6UZ5TTs7uqYt+nRXzfwldW
XF6I/6naX1+1DpPxoyJ8Q9U6TPa5vfDwqnWujKdF/K8Dz5V4xg2Eh1atc+We
1eJPHHhf9AmBjje8Yl+05ZLGob4SHTHjY63qb6jY9+5b/DeFr6+4vDRt/yX8
tYltxUerKHxtxb57+IiUhYdVvO9lknrhERX7Fq3Ef1XHu7ZqHSx9TlX1N1Vc
twJ/WNVfV7VOl2uq0fPZKWffKOZUJeHGOfvCMAc6KlzPMRZhU+ql+tMZG1X3
m+Rw4RMZW1Wez5xJ+AT0gSp/Kuki3Ie5pcofRW1PZ33OXPBEydGqH8DcR3W/
MGcSPirnWJP/SToI98w5VuQ55mA6/3d0v6/WuVwl6ar6vox1qlsgOYD4g8B1
Vyat72f9zVzzWMmRan8qY73aroza3wD9AnMz7JUdtP27gWM/OUY1XJ8zVz9a
0lPb92es1Laro7bns95n7nd8wvZ89B2829gDj1L705grqO1Pkgv1PBYw/unZ
zg/Xc/gOdOHdkRyk47+n41+nY18rGaz2i4jfUNsF6KNVPy+w7/JQ1uDa/zE5
xxpyDOyjP9M/ZmxTPlT1x7O+Vt0HkoO1/YeBt70BHbjqj8t57TRX0lG4V86x
tVzjQcJH5xz78mLYv/fWf2fm/DwP1f4+1v5u1L6GMwdRXfecY315Joer/jPV
36y6ETwv4c8Dxwbx3wFqf2TOsUPPRG1v/qrie4ON8wi1/yLw3GAkcyLh9wPH
6nKPugh/Ejg2l3PYX/vrkXPsz1Ncr/CxOcf+8I5tF+qXWFsNlPRQ/Sk5v2vL
Jd/q+q6s2keD54m9+wudz6cZ2+Cvx9+sYNsu9vB9dfzZ8Efo2AHPDH/Mgm3L
2POWa39XVe1DwvuOPehNta9JWofSVvitwHNx/rsO/7eCbdfY17EPfqbjf5Sx
T8DN+KMVvNbHfnuv8LSCfaeOkNyDf2XBvliHZ61PwFehK3295Dztbz7xNhlf
0zc6vyuqtonxfZyJvZp4NWydko3aVyZnWx3zz5H4CxZsG8Sefbbaf1pxW86x
cajvYG09OGH996fobwq2r7fX9b4d+F6Vmf9qf1cX7AuBv+if+B3kbAtkvYO/
62fER+Rs79021BeylsPe313Pr1/OfcO3vP/4FxZ87fgTdFP9STm/C4skRwif
nPO79o1kOP6OBdvy8Ye4CX/Fgu8l/gpD8ecr2BcAfwX8i67Rf+ML9ifYPtRn
spY8O2F74azA14aNaYdQH4qu4DzWiPgbFmw7x/5/A/6EBfsW4G+xY6hfQldw
oWQE/pcF6z7wdxiGv2XBuhH8NYbrfH7S/V+V8TffWdfXO+dYVvrofYWPyDn2
nD4a++IbgX0DsHmhn0a33U7HSqC/Er4057lbm5j15eib6JuWxa2/Rje6n3A6
YX0ouuI9Vf43bn3p9qwH0a3EHa/0h57fRTn3f+ir0T3vrfpowv7+G1U/OOf+
H/07+uZa1S2N2/8QffPvcfN1rA71+T/HzcfxHx8I/sU5P88Bqh+SMxdS65j9
J9HH/xo3Pwf2TfTtOe1/Ydzz/bX49+Y8fhFvtEF4UM79O/YB9OdFtf867vns
r/ir5vw+of/ejvW36v+KWz++DevnhM+5wP3J2dcVnUGPUN/G2oo5Xs9Qf8pa
ijniEaE+jbUjc7Q89zNn31p0CkeG+kjWeszxitzPnH1z0XnkOJ+cfVPRWTQN
7avMna6S7KP6w3PmvmBMaCfcNWfuCsZo7CUNWV8nfI+xd6CPpO9YEXd8zp+6
/otzHv+xf2yL/kD1f2CfEL4kZy6vVjHr39FHolsZkrB9pBHr+4SfSZPQXsvc
93JJe9V3y5kbgzkE/C2/4U+e8/eN/9L3zJcy9lm5Sni58NKM/4PP5cac393T
Y/Z3+VX1P2Xsc/VraC/h3fsk7vnxlTmvTbAh3K72G9R+XcY2wBHCvwivyXgf
8L8Mz/nd6h+zf9OPzK8y9tmBD+WGnGNXsIHCZ3J9zrF62DCxx2Ar6JjwOwVf
yjU5x84cFq5XfsF/Pef5DOujn/Hvznk+9Utof+Bbmxc3v8rVOXPbYMOEX+Xa
nGMPsdnhv3FVzlw52Eix5+Cb2Dnhbwx+l+tyjq3DBjxS1/ObrmetruWXjP2V
fq+4jA8bfD1Dc47Nwkb7U2j/4tnDP3e52n+r9l8ztmTsr4v9hL4A/rMzhC/L
ea2HDybxQ78TT5Lz+Is/2ncVb4vP0rXCK4WX4WuSsb0MW02HhL9h4i3XE5+S
83xukfDlVft0MF/DHwJ7G33Vl3H7ty2ueC6FzxX+Zz9UvG98qNaF9jr6ws/V
/gbVr1b9ioyfMeu7y3PWDeBzij/bEtYXGfto/Rzaw+gL4JuDL2dYzt9CP8mZ
wlfkvBbFh3WItv9G2y/MeB8b8LPL2ZcHfSCxq8SwEs8+q2T7zY7qv1+K2YaM
rrkWfXDUOmd00SX0t1HrpNEdFiRzItYhotstEh8XtY4X3Xgj1jtR68jRHdeg
v45ah4yuOZB0jVrnjC65Dv171DpldOMNWG9ErSNHd96Q9VTUOnT6imaSdVH3
GfS120o+j7rP5dntJFkR9TNkrGgqWRP1mIHvxC5V8xHhQ0Hf3ESyKuo+mm9j
G8knUX8jjCU7S9ZGPabg29K8ar4lfFzQlZexB0StM8eWUC/pFbVNAVtARdI9
aptAPtTfvJiwzj4T6oteUHmqpFW9dS7oWu4veT2Ifghb36sJrzdZH1JGR5wL
9T1syz73DedD3zG+Mh6G80nKyxNer7H+fDlhnXkx1DdgK+Q/+L/gJ9uoseBP
xnPhQfiTaK23L/GHer8uqri8X9q2kaqkZ9Q2Esay7SULox7Tomo/uOK2+0v2
qndMNrHYc3V9beodk00s9rvYVusdg03s9XvCe9ab8wCug3dK7su2k3wZdZ/G
WL2D5Ouox2y+/caSZVH3Aegmk5IpEeso8VVcp/PZMWufRXw/1wpvl7UPKL6l
a4QbZe1jSqzaXOFRGcesoQtPSCZHrBNfqHProfpadAFhfO2RFZfhREAXnJJM
jVgnjK/metU3ydpnk3jdo4Uracfo49u5UbhZ1j6ezHU3CbfIes67Re0vEG6r
tvuk7ev3uXA0a58/dNMRyaMR66jRjcckEyPWkaPLjUomRKzTRVeblkyLWGeL
LSAumRSxTaBNOL/D9rc44Vi9OTre7RnH7BGLNVN4RMYxWfgCrxAOsvYJxjd3
qXAxax9dfIOXcb+y9hHGVpGVzIjYZoGuPMc+I9aZ4yu8UO3TWfsMM3bvyD6j
HsPRZeclsyLWaeP7u0rt67P2Acb3+GvhXNY+yPgafymcyNrnmNiyWcIjM44x
+1LX243zRbeRdrz8ERWX4bD4Wvgori/tZ/apcBfhLLG4acebHy6cT5sD4XPh
wyqu479/hC+s+NkRc43uPyOZHrENgLXJVtWPj3iN0jZc3+F7tiTh9Q7rAXzb
+MaTwmMD23aPkKSEHwhc7p5w/BmcaHChNdFcP636cYHrekjaErui/05X/Xp9
X1s0Htwd2PfgYOZ3qt+s+jNVvwHdhfC/wucI/4EupN4clnBX7qT9/63t74J/
KeE5yT/C9wTeF3OEjcJ3BvaFOID+SdsP0PZj8M3R9n+qflTgOsZkHO/HwP+T
sE4jJnxfYFv14ZJNaj86sK8Fx9wqfG9g3xK2iav9/YHbsmYl3nzXenNs4j8A
PxJ8msuT5kyD7xE+wxUqr0yarxP+03Uq/5Y0fxp8p78kzVEE/yl8q2uT/g/+
UvhD16j8U9L8SfCZUoaTCD42+EY3JM2JdLae97e67j31PjcrOn4Bf9mRej9n
ZM1PCp/i+qS3gU8N/lDOBU6jSWG8zSi1nZU1nxb8ft8mzSEGXyh8lMuSvkb4
neCz5NrgjNqg+3UH8asJr8HgB4QvcGnS+3g69O+9XfuemTWfIHyBS5JuAx8U
/Iqrk+aIg28RfsJVSf+3QPu/RHhb7Xu7hP3rbxbeI+E13hrhEYF9YfivcWi/
fS9hm3WTUJ+L786Hkt1CffQXCdvcW4X6b3wn+K9pqI+mLTbZBqE++s2EdcQt
Q334Zwnb0FuH+nB8Kb5KWH+NfhpfH7aphPrh1xO2Oe8R6t9pi40X/TH2KHxz
aLNXnf3l6SvRwe9Wb5sRtqKxJevj0a/jy8Q5oB/G3v1OwjblbUL7wNsJ23ix
h2P/wleJNo1C/Ti+T7TB3o19HF8n7tk+of6HvgIdd7NQv49v1MfMCXS/rw3s
a9Vcsk741sC+W/tIfhEeKbxXwmvq9cK3B/Y14x1ZK3xLYF8l2qwSHh7YV2s3
5sfCtwXeFzoA4jduDFzHGn6PenPAwv36oO7H7vW2uWBreUB4hdrfENj3rSX3
kPl3YF+7xgnr064Wbprwmngp+rDAvnH8h37susDXxhqyeWhf4NrxsWhdb5sV
tqpxJc/vL1X99gmvUYm/GBr42KzB/9NnBT42a8yFwkMCv8tsA//HsfoeG6TN
6YavbYuq+THxuSVedqDqW6bN4VEb6ucnq/xc2vYF7AkTVX4qbd3ZlYH9ydGh
5UP7A3VwLhfD+M5JKj+T9trkssD+5qxRSqF+nzo4g4mduSlw/BsxNHCt7hU4
vo84v/pQnz9VbV9Mm9v6pMDxlcRZwm16XOB4TeI24SY9JnD8KHGk6dC+OkHb
PpE2t+lRgeNJiSuFS/yEwPGXxGHWhfYKrh1OzSC0P0xR+X9p83Fg76AMpwWx
MMMCx7MRE4Mu9ZrA8W7oVNGFXh84ng6dKLGgB1XNx0xMKLGaB1bN/0vMJrGD
HarmOyaGkFjQjlXzOxMTSmxrp6r5o4lxJZZ4RsVcPMQUE+vZuWp+WGI+iXU9
uGq+c2JeiRWfXjEXDzHjxHpPq5jbhphvuHkTgf3H8SOHq7ll4PhL4jDh8t0t
cHwncZ5wEe8ROF6SuElsLY8FthVjczlH+Luy9SPoSc7U+7lDYO7yF2LmVs4F
9mfFr/Vc4e/L1meh14JrOR3Y/xw/9Exo/+ZZwrGeDO3fj6XNURIL7e2PqPxo
2rH791TM9U0MP7H/91XM/QoHALHdEyqOZSfGm9jzKRVzCxGDDvfKjRXbkuBg
gctidMXc2XBaEFs+sWJuG2LMia2fXHGsPjH2xMLfXLHtlph4YuVvrZjLnJh5
YuPvqJhLmBh5uDK/CcwVAmcmXJyfBrYlwMkJF+Vvgbmb4aSEO2BSxbHxcAjA
xbkgsO8AnJzY+p4KbMvE5gfX5qLA3B5wbsI91FXbd0ybgwjuqrbCs1LmsMI3
9LCqx170oHDZDK3YFgenDVwz11Vsy4NzBu6bSyv2ZYADB26Q04X7ps0RQmz+
mcL90o7RJ5Z/asXcTMT0Y6t8IrDvBDZLuIn6qb532hxFidCfgWfLM8f293Bg
WzQ2QLhwrqjY9gknDtwx/YX7pM0hA1dSX+FeaXMmwY3UW7h72hxJcPNcXXFs
Ohw9j6p/3IbnqftxqeRh4UbCl6T8H1xLfYR7ps25BHdVd+HOaXNYwXXUS7hr
2pxHcIe1F56TMocYXEc9hbukzXnUILRv0tfByUPs6AFV84ETQ9oo9AeYprqX
047N7F713BG7B7EhPaqeW+D3RWxut6rntujBiLU5uuq5AX5gxPocU/VYjN15
t7LzFbyXNkfMGl3vKawHVW6aNl/UycKN0+ZUIr8BfOdz094G/mL44j9Q+UPJ
HmXzGVOGYwK+bviYP2YtKdm7bP5uyp+mzb8Bn/s7aXPyYFs/vmr/G2zsLYX7
VO0vhN8QsURHVT3Xws+R2JGeVc+d8EMklubIqsdW/Bjhg4Z/+aO0zwH+ePig
56R9TPiGTq34WuGIgl/rROHt0+ZMxXd216r55fGh/Vv156t+r7Q5rDay9hBu
zbWn7dvbsmo+cXx84dM6W/W7pc2RtUn4POE2ae9jnfAZrKfTHpPXC58l3Crt
bf4UPrfifcOBhS9w76r9l/Bjgo9ngOqbp82BRSxWr6rnevhdLFP9cegPVLeN
ZKnwMaxH054jrBI+Cf1D2s8YPrHjK24LByy+yMdV7X+FH9YK1Z+AfiLte7RW
+DT0A2mfA77Ix1bt74Xf13l15ie6nLl50mudE6rOJ8Cah3wV8KvP0Lavp+07
26Zqvi18aPHt3btqfnl8fOESaF91PgA4BYjV2avqfBzE7MBV0K5qfnc4C/B9
P7zqtRt6VbgL9qmafx4OA7gE2lbNNw+nAL7Yu1fNn49PNr6+u1XNl4/PL77l
ravmw8fHnFiDQ6uem2L3gZvikKr53OGoILaiS9VzT+xQ+Ca3qppfHx9luBj2
r5rfHE4GuAX2rZr/H44BuBT2qzpfApwK+JbvWTXfPT7m+ILvUTX/Pz7h6OtO
rJrvGN5jfPe7Vr32RS/PfLdf1fzH8CDDj0O+irfT5oQiHwr5Imaq/GbafE7k
E+HZwCnF+nxA1e8ivGzkMyFfx3ThV9Nej59RtS4BHjf4YMh3Qh0cX6zfT6/6
24EHjnwZ5OeYlfY5wL9DPhaODecX65W+VfMvw8OM/u2UqvlQ4UVlvn5S1fzN
8Dgznz65an5jeI7hLjii6rU6fgjo906tmu8U3lP0e6dVzU8KTyn6wP5V85HC
SwrfDflm6HvhUEO/MLDqvhJeLfJBkD8ir/aFlPm44Je7LOl3/onQv/sGrVWn
Zq2fIqaNWLYjaqzPIwaC2Icja8zfAj9Sk4xjINBv4UOP73yPGvNlwZe0Y8Yx
bPDRwA/dJmOffPRHxAgRG9S9xnxI8B1tl3FMEvxM8EU1yphjHr4Z+KJbZxyT
w/hMTByxcIfVWPfLO8W7tDXp+SQ+PPjudK3xWpJnxLOJpTw/JaaJWKZuNZ5v
EpNGLNrhNebfgj+qWcY+//DLwHfdIuOYAfhp4LNulXGMCPMzYiqJpexSY75r
uJrItUEb5nu8M7wrO9SY3wo+sNqMYxqZLxHzSKzjITXmm2LexfyLe0h8P9xd
d6s8J2O+AfSq6FdnZcxvBd9UMeOYTeKl0TOib5yZ8XyQmEdiHTvXmO8K/rEg
4xgv+Lrhp0pnHEMJvxZ8XvUZx/jBh8W8kvklzxz+b+apzFe5J/B5dy5Yn8Uz
Ym3KO8+7nkyZj4t5LvNd7il84YcUrF/jnWB+SswksZKH1ng+zDfPt75jjfnd
4BPLZRxzyXyXb55vffsar035pviWMin7L8KvDSf4lqTns3zjfNvb1Zi/lnks
89k5SfPf3543N+anSc+H+ab4lratMd/srXlz+X2QND/+HXlzbc5Pmq8d/thF
SX9j8NveljfX30dJ89PCxQ933tyk+XaZ5zLfnZX0/JA1BGuHao3ne6xhWbvW
15hPn3k28+3pSfP/Mo9mPj0z6fUAa0DWfo1qzKdPbiB8i6YmPf9lTcJapFLj
+fJ/HJDCDWvMtw4//4Kk7x/8/HAz4ks0LWk+i4sK1jfOzZgfm9wE5Crpk/J6
gjkic8NtaszX/Wre+pszU+bjfi1vfcU5Kc+fmbMxV/ulZL5huNbJhdI15fk4
OgDW/r+VPD9mzsNcZ23JfOmsM1hvdElZN8KYwVjB/YcPfEbe+orzU+a/hmsd
X9b+Kc+/mVMxl/q1ZP5uuNrJ/dQrZX5guNrJZdQzZX5wcj+Qy6l3ynzg5IYg
F03flPNXwP1ObpHuKa8vmPMw11lXMt82uSLI5dMvZf7w1/PW5wxKmY98et72
q9NTXr+gk0AX0aDGfCTw743P2GeI9SExy8QqH1xjvkz4HoenPQcn3w98cXBI
MgdmPYWOHd36qpLn/9hQsJ2sKXm9gU4GXcxPJfNjwgd7Q9o6DvjrWDexfqKP
QHfNGMPYwpya9QI2HGw3P5ac/wP+02vT1lGg+2bMZazFJgYXPjpQdJ+jS+bK
hjMbPvfXhP+oM78rnK3MWdGlMyYzFmPDgQ+WdQnrk8vT5o9lnch6cXDafLSs
c1jvDElbV8+cgLkAc1r4F1knsl6kD0TXj84frjnm4KyXsGFgu1hdMv8nfKIj
054Dws8JP+iYtO8v/KfwFT+Y9vZwH8OBDH/6WyWvr+Bohj99dslc9HDSw7f+
esnrWzia4Tt/u2Q+TPhOx6Z9f+FHhf9zRNpzGPgz4Tu9Pe05EHyZ8Jfenfaa
Af5B1omsFxkT4GNk3cn6kz4UPkrWxayPGYPIlwV/7lVp69jgVodjHf79GSVz
ycMpD9/5myVzN8PhDL/6zJK55+Ggh0/9jZL5UOFbHZX2nIlcCOj00eXfVXKu
BHTq6NJHlczfyzqU9SjngG0DGwccdswX4J9ED4I+hG8Avkr0CugXJmTMf4me
CH3RlIz5KdF7oP94NONcLeRsIX/BmJL5LdGDoA+ZmDF/Iut81vuTJaeWzZcJ
xx3zDfgy0Uuhn5qasX80ei30W9My5m+Bf5Nj4yPB+ps1CmuTn0vmi4Ffk3PF
hxR+TvQ46HMmZex/hx0Ne9qMjPlo4N/k3PBJgD8GfkfODR8b+D+wG2I/nJ4x
Pw18n+wLnw/4iuDn5NrxcYVPEr0O+h3mWOS2waaCLeXekvMDwt/Mt4JOFf7t
+oLtVbzT5MLA5oOt586S+Xfha+ZbQ4cJfzR6BvQN9Bnwz6LHQJ9BnwQfMnoM
9Bn0CeTSwSaELejukvPjwe/Mt4yOltw32Gyw1dxTMv9vtWB7Gt8E+iZsNthq
7iiZ/xc9Dvocvgn4SNGLoR9jDgP/KHou9F3MKeEHRu+C/oVvGn5g9D7of3hn
4dtFr4N+h28OPuBKwfY+vnn4guGe7p/2NwffPrlo8I2dkXS+mzvz5upmTIVf
flTeXM3MEciXBJ83sbmXSM6qc/4kyqw54feFL5hY0HMlZ9SZH/yipHUM8IXD
941vP//dHfpPdtHc/XJ4XerMD8y2rIlPqzMf+AVJ6yzgB4cPnFha/jupzny+
ZyW9ZocPGL5dYk9PSTqfEHy+pyUdU3VP6N/ZVce6Kut8O8QPnZB0zBf5c4hn
OSnpmCziW4gfInaZ/+APhh+ZWFSOST4e+Ic5FjFed4X+lZ217yFZxyfhf0xs
NcfA/xz/8sNV7pq0vzj+6JS7Ja1/w2cEX5G9S9bXkiOPfGE3FK1/xEaFbapd
ybEG2JSwJe1XcmwBNjBsX+1L1v/hQ4LvyD4l64fJwUe+sqFFxz5gg8L2tG/J
+mZ8RvAVaVuy/pocc+Trur7o/I34T/dIOgYOf2z8s4n1Pi7pfEf4Ux+TdIwa
+Rvxh+faiCEbFfqrdtC9uTDrfEj4c7MtMW7wa+OfflTSMWP4Z+MfTiw6+8R/
G/92YtE5B/zZ8Ucn9pxt4G+GH5rYa575CXXmWz4jaZ3JvaF/bncd+9qQLwz+
M9YicHLgv4sfCP4gtIHPkvjo4SpPy9qfFr8R/Ed6Zu2Pi18H/h29s+Zrg++J
tQs+gPAjwh/WL2ubLXx28IcNxN8ka34y+M9YG8EZCF8W/GvMVeE0xF8Xvwn8
J3in8O/GjwF/hnZZ+4vjB4I/yN74WIT+xXBusT18TfCT9c3apov/OX4V+Ffs
l7W/M34T+E90Df2J8ZvAf6JD6L+MHwf+HB2z9nfGzwN/D75Z7E3Y/LD15Uvm
74Q/7Wz8XbLm94P/jGf9aNb+3/i54O/SJ2v+NPxc8HfhHPFPx08Hf51eWfOh
4S/PuZ6XNb8a/tScy+Cs+dnwpW6V9T2GXxu+b2LJeebw0cM3j+0fnQH8bvjH
c+84R/jc8Lfn3g0M48Xxl+bZolOAzwp/dc4VGz18o/jL8+5MyZr/EL43zmVC
1nyH8LdxruOz5jfFn56+ZnLWfKvwv9HXTcqaf5R4dfqKiVnH4+Ofz7XzzsGX
hr8379aIrO0N9FH0TWeWbJ+gD6PvGliyfYNvkG/v9JLzfZBbZyr68ZTz+5Bb
Y67Km1K2J2ITxRaaKjm/z6q8/bO2ppzPhfwhS1P2eSLfC7kxZgivTTk/0A95
+2ttSdn+R05h8o1+U3Q+EHL5TFfdmpTtndhYsa2mS84XRG6Nmapbl3K+LPKL
rEpZp0q+AvKFJNOef5PfgHw0sbTn0+QfIf/G1yn78GAvxGaMrThTcr4a8oUs
S9mHh/wo5D/hXqCjJT8RdgPsBxtTzk+EnQZ7zfqU83WRz4RzRyeLfZQckOSf
/LrofCY/5u2/xjmSb4p8LqyVNyWd/4x8dKy10SeQr498caxt8Tkh/xn561i7
o98hHxj5YVirs/4nnxb5aljr46NHvjjyy01J2seD/Gvko2Gtjv6I/Ffkw2Ft
jg8J+bXIt8NaG58W8oeRv461MT4b2G/JUUl+50VF568ifw5r2atSzq9EfkPW
htemnA9odd7+fDwD8lmR/4u17OUp24vJaUn+58VF5+cinw9zC3xmyC9Ifj10
AfjMYH8mxzL5JpcUzS8MPyDv+vSs81OQb4p3kfUP+THIT8W7iw4f/kL44Jnb
4mP7Vp3zx5DjaUzKvoToNNFljk05Xwu5jSal/M7ge4iOEt3kYynzJxI/yNyY
mAh8F9FZoqt8OGXfRXSc6DYfTJl/Ez475rL4eGOfxicBX4RsyfbaOuF3iYcs
OncEYxxj24CS7ZPMIZg7nFVybkDGRMbCM0rOR0X+Ltbqt6ecO5kxmLH3tJL5
QIl/Ye6NzzW5khlzGWv7l2z/rej47+v4xxZtX8fnAV+HXMn5RcjvxbfLepX8
VeTXYq0/KuV8VOQHQ7cwPOX8U+QfQrdwQ8r5eMhNNiXlb4z8QOTGmZbyN0x+
EPJ98S1hwyB/CPnB+NaxgWH/Jqct+cSfKzrfCPnC+Bax2ZBfj/xS9DXYUMhn
Rb40dCM8D/JrkS8N3QzPA/s5OX3Jj/180fmPyLU1OeU+DPs7OZPJRz256HxY
5E9DNzIi5fxh5E9DNzQyZf8JctbCtz2l6Hxh5HNDN8T7RH4x8pOhm7k75Xxc
5FdDN8L7RL4x8rWhG+F95PnODzzfZt6NP9gbZfOPwENCPmPyH6PrLqbMR4Gf
NP7StVHzR+Cnjb92MWq+CfyQ8UcOoubXwO8Y/+NGUfNJ4IeNP3Yu6nha/Kzx
t24SNT8Gfrz4824Xdfwufsz4M+9ITFuN+TGI+SAHKr5rnDPnSg5P+DTw68a/
uz5qvgX4RhJR5xQgHgu/8Tkq3x3yM+L3iP/jiIj5NvGTxV/2hojjT+DLSEfN
yQ6fJX6q01QeHrF/O36j+I+OjDi+DD9Q/EFHhXwV8KFEo855CH8ofqX4l94e
MT8IfuX4lzeLmp8Ev3X811tEzdeB3zv+75xDL/i/A3MbnksMBnzUrDdi9gmH
rwK+Efa1PmI+CeKVubfr4ItQ+9rA2+KDTv7sAvPRmHMc94TPODB3HfvsKpwJ
HBtyWsx8vfDv4vt/Ssz8DfCpcK4bI+bTgN+EZ7cmYn6Q9jn7F/4bcXwufCc8
mxURx+fCT8K7sixi/hi4T/aO+p4RDwT/Ce8anPvED8FHwrsDJz3xSPCX8O7B
8V8O4715V9ZGzNcBvwvv3qqI+WeIEyBegHcCfhL4Szi3eRGvRbKBr5U1CXzl
+C1PVN2QiPlV8UvGP/nyiPl6iDe9N+r3b58wPvX+qN/PdmG88Lioc+4ST4xf
P/7946OONyZugPiBR6PuK8hv+mfMfQbxfvix488+Mep8qeRDhevy75jzoZIv
Fy5KtqHvIj/uhpj7sPZh/DLHIsfvfmF88YSoc9YeHMYrz4g6h2SnML53etQ5
aDuG8cHTos4pR/wd8dGcKzmBu4Sxs/Oivkbicwfm3L9zD/YP4xU5d3LaEZ97
Rs7jD/eM+D7ir2dGnaOO/L7k74Xrj2sgvgZ+c3Jvw9lAvl/yw/6q8rqYxzLG
NHyPyDHQIIznJebv36jjsYmbIH6CayTem7gN4jc4JmMRYxK+SJuFn6sxfxCc
NbwPp6t+28DcT9fEzHeLH/8U1V0bMT8ufviTVb4q4vho4jyI9+AeEs9NnADx
Atxz4jvxG8d/fHLU8YnEd5Grm/NlLCA/8W8xjwnkKya/MdfKf6eF8TfkquZ6
B4TxW+Tu5vyJByfuhPiTqVHHhxKXQHzCpKjjg4lLID5hStTx38SdEH/CMyYf
AH75+OcPjjhemvhy9kXOv+46n3zgWDT6jOOEy+hDYvY5fKDO+eKzKduc4X+D
vw8u7dEZ83/BDwb38K3oG8vmA6OMzg6+QPgA4fq+J+P4cuKniU1OZq1PhE+Q
Omx4g8P4J2JNM1nzycGXSOztzZJHap1veNu4be7wlcE/B7f4HRnrR+E3uy9j
m8xUtT8xMJfyPXHPBfsGLjMnnCx8bGBu5jskJ5bNf8a+sBGy1uij+tFxrzmm
CB8fmFuZ/5ir9w68LXN2+NXgI7s/43OAnww+srEZ/0c+U/IV7gDfY9z5C8kf
DVcl/02odb5T6hpL7qt1Ptpy3D4T5CcmXy9cpVXJQ7XOl9wwbh8D8ieT3xhu
S/4jny35buEWZR/31DrfbU3cPhbkvyUfLVyg/DcsjNfdJmsdxfAwXp/YXv57
oNb5kTk2Pg6ja51vsBC3Twb5B8mXDFcm/5G/iPxEcLtlJbfWOh9RKm6fEPJH
k38Zbk6e6R21zndEW3w6yF9EPiS4P9nm5lrnW47H7TNC/mXyMcPVxn831jof
YiRuHxTyI5JfF25P/ntZuJ/wfSrfL+ktXBeY+5Z3fhq6qMBc2bSZJHx0YK7v
WyUXhPwIvLvoSDqpPqr6E7Vt35jjzYmHJ7Z+a8bx6sSfU/4XG7PaxwK3PSnm
eHfi94mlj6HfCePfKaNzIX8w+YThxv0n5rki+bUZK5gzviN8lvCTcfucnhXG
x3OsfzLW7cHHz9iOju804W2Er4zZJ6a/cKPAXHv8d4pwQ/wfYvZxQTcXD3yu
6Oj6CTcIzEVIm77C1cBciJdwTcL1gcv4XJ0gXEE/G7NPVR/hIDAXJv/NET4z
cK4PruFt4YHCj8ftcztX+OzAuRyelswQ7h84N8ND+BwLXxg4t8QL+LgKXyD8
fNw+tR8Jnx841wP/vYquU/iBuH30nhDuIXwT7xL3UPjIwOWbJdOFTwnMbco2
rGV7Bq5jTfuh8HnCz8bts/uZ8KDA54IP0XzhiwLnEnlJMhPdaOBcI49IXhc+
LfC14EM8S/iMwLkvuAdvCp8euC0+x+8xPga+F/gUf8B4FzgXBufwhfDgwMea
JvlS+OLA5ZclQ2udj5p3CQ4l1tK9Ar/brKnhu4B/gLGFHLHwm8BfwNhGjtxz
ah3fCTfnMMmPob/hcWnbBFaF/obk4jsGG0/oX3ZC2jaFH0J/NeqwgawJ/ffI
Fcg+iA8kHvA8lc+VfBf6v/VI20aCvZT4QeqwwUTCeMDz095maeg/T27JbmnH
txEfRa7UsyTfh/5vR6Vtk8F/D396ck+dFNpTiYejLTaVjaH/HrkOT0vbHw//
OnJ19gjtp/gDnpG2DeTX0B+QfWFz2RT6G5KbkTb43+GfSO5BzmFt6P9HLj7u
0ZLQv49zx+b3RZ3jxzqlbcMlnoz4MXKJ8h/xWsRnXav19jUJ5/8gfosyHLLw
vxJ/dUnCnLTEYxE/Raw7/xHfRfzWdQnv46s6xx8ekraNmHhE4g/J7cl/i+sc
r3ZY2jZm4teINyQ3KP99Vuf4tQ4qH5A2fy3xVBclnJOG+CriueDe4D/y0xB/
dWXCnLMLwniIo/D/5HxqHe9D+ciE47WIv4IrgG3IZ0N81mUJc+gSP0d83IFp
nwPxYcR/wVVxQcLxc8TTkUuVNsR3EQ9GbD/7wL6OPyjPGhsWfLvEl7EtnPu/
19m/85S0bWLPC7fEvyxlncQR4Xzqvph95LuH6zG4uPjv8HD9dU/MPvJdw/UJ
XGn8d2w4Hj0SM2f5qWH//GzMHNu9w/kZ3GCPhP03/fnkmNs8rvPZXufzTMo6
qOPD/vbxmDm5B4Tz3+dj5vQ+MlwfPhAzxy3+0vhPP4XvRcr+1NtWnHuY//j+
G9OfxBzj3TNcr8Itxz4eqbO/Nds+mfJ8ezvVT4n5mPDlbh84lgBO8F7h+hWu
uYdi5vtnPUwZTvMJ2t8OFeeW5pr6hePTJNU9HTM/LuMRZTjJn1T7xmr/XMo6
mUnCO1fC3O/4nwg3rbiMTukF4d3wb0xZhzxReKeKcz//tw/hXSrOxfyiZKrw
7hXnbmabKcK7VpyLmnfgWeHmlbAtPgDC+1Wca/odfACE9624jE54hnC7inNT
v805oC+sOBf1mymPl4yfcO09Ho63jK9wwT0ZMx8x4y9lOOtfQZ9Y8bbovKeh
n6w4t/TrKesv9684dzs6+teE96n42OikXxLeo+K26ODxZ8b/mdyvvPPEKxHf
BHfBUWH8F/FRcAedIPm+1vFWJyUcQ0P+DPhy0M30jzhfEvlA+qjcLOL8RuT/
6K1yk4jzo5Hvo6/KLSLOr8b4vzjunA+zw/nCV3HnkJgbjodwnywOx1fGS7gR
Pot7fsH8CC4VtnkjHH+pgxOZfEvkB+mpY20XcT4l8pP0UnnHiPM3kS+kq12H
/8tXRb6PLpH/0v7+l8+D/CBcW+uI85ORP6Sfyq0izvdEfpHO6C4izvdFPpCO
xEtHnH+L/B8dIv+lefkvfxT5U7qr3Cji/FDkL2unsm7nf/MD5kdwRXwRd/44
5guUyaHxfjgfWBI3Zzz5aIj7Jf73PMkFqm/CfCdmH+TBws2Y36p8q+R84Z0C
c1PThnw7xH0T/31hxPFAfL9wXV+PTqnW8UGU8ak+N+wfmBvg0wxfGvxVrE2v
lVyk+p1Vf0vMxyR/DNy+47TvsyWDVN80MPc2beaF87fv4ua8/yKcz6xWeVU4
/2H+BRfPknC+xfyHuh/izjfI/IvyynC+yPwMrpjvJWeE/RO6AHzGPwnnk9Qt
D+ePzCfhFuIcyCdHvhvOlWcyI5y/wdXycdz5X8j/w73eL+L8N/BloEvknSc+
lXhVuKFOk/xY63hVysTUwW9PPOQZCeeUIz6S+Em4wPgPfg7yy+0XM4fkxHA+
CtfQq5Jjy+ZfZ62MD8yT4fx2uupeYQ4drmdejzumDr594jXPSjjHHPGZxO/C
BXVuwvkFiNekfE4YH0m8JNxjx0m+q3X8J98+MXTEWxLvCLccfcDiWsdbHpNw
TCHxl8Q/wuXFf0tqHW/JvohRJH6UeE+4qjgn4kGJ94T77hTJD7WOD6VMDN+k
cP4KdxLXdFPIh7N91jb0J8L5PPeGe8B6n/U6XE1vxp0fiHxM6IbpA8hfRP4j
dEF8g+QLIl8SuiD6APL5kF8RXQrfMPmMyIc0POI+g3w+5KfiW+GbJp8Q/GTo
jukTyHdE/iB00/R55Dci/xS6Y/o88hGRTxHdJH0G+YXoGNBN0ueQP4/8Qrxb
TH+nh+sZuHY+jDsfA+sVyuSgZD3Lehdum/dYg9R6fUuZmEz0I+hD4IJ5hzVU
rfUllImxfC1cb/Fuk1MT/Qf6ELh03mZNVWt9CGViOp+ttT6Fe0tM54iQ3wgu
OJ7JyWXnD0B3hI82vmxnVZ0rAp824kPIOUm+rVf0rv9do28L//OobQ4bheeV
zX19eNTxFuR0JP/W9KLjMciRSb6v14qOvyCnJfm+Xi2aXwVOcvj8dis6voN3
nHe7hO1XdR+VvW9sDr8Jv4//ddQ2kn+EP8N/PmqbxhbhT8vmGuc/8gUvLpuL
/GR09qr/oGyu70Ojjm8hJyf5xV7W8dep/r2yubo5xgbhD8tui80lo7YLy+Ya
P1GySfUfl80tzj2B34OcMOSCuarG8T7kIMBf+7yS443IQYC/9bklx/eQcwB/
7XNKji+Dcw2utY4l8+fAyQ4/Yaui+SPguPtU79qmguPXWGOwtqjgu1br/Mhc
a7+o47/gWMX/+/yS478G1tnf64KS+UrgbIKr6coax7PBSQcX3cElxxfBqYA/
2IUl85XA0UdumqtrHB8FR+F/+R1K5l+BQwvurCtqzL8CxyTckpfXOH6ENQZr
iwDfV53vorLvJTaZlPBXxFtEbbOZFc6PsKXdx5xM+ICKy8SsMAmA4whuo9uK
5v9hzs5c/cai453IaUB+xReLjocipwD5Ul8oOt6JHEXkY5xaNP8QnGXwVfUv
2neU+ebQlH1I/xfOL29M2ceUeCFySJAv5/ai57fMZ7GdXxfOR1mPYEtlG/hb
4Bifr1P/u2BfVObDtL0+nN8yX8XWOjSc/zJ/xjZ/tWSrtv28bG77o6PmP4Iz
D76wU4rmM4IjDz6xU4vmM2IOz9x9uHBM5S/K5uo9VvKR9t+5Ytvl4ylPiuaX
vW9sgknhBWVz8fJMiPciZy356aZxP1X+sux9YWOcEK538A24QvKx8CEV73uC
5BPhQysuP8GcV/igim25D0neE+5YcZmYoQ+EO1Vs+32EObpwh4ptsQ8whxc+
sOIyMUgfCh9ccVtikIhHI8ct+fleKjp+jTUwa99afEFV/qbsb+WUqON1WWOy
tqwp2f5BfuV3orapwHdFzgT4EVtqf4Hql5XNhfoe9aqbWTZX/15R52eG73pO
1PuA/5r8FbOj/m+F2r9Vdi6CfaLmz4YPG67Uj0N7y0p09lHbgOqFV5TNzcp/
Bwr/Wza33u9R8/HAMQa32MU1tue+Wfa5YPODTweO+A161Mmi+b/ISQF/Xoui
7TXLy74WbEwthNeVzZW4RHKA8D/0F1HbeLapNd8354rNrWmt+bC/itqm9o32
/Zrw7iq3lqwSfrvsXBH7SlYKzyr72rEZLhWeUXbbPSSrhWeX3RYb7RrhOWXn
Vjggavv062W3xQYKXxk5KeAn3KVoPjNyIsB/nxLevtZ8659FbROEXxy+cbgZ
F0ua15pvnDI2HfjSyNEBP+2uRfN9ww8O1x7XCH84/O9wQX4RNZ87/ORw73GM
xrXmF6cOG9VBwlvLflbrJR1rvXChvCFq/ilyMsDP+G/B/E9wusLlemGN+Y7g
pIOLbnCN43fh/Ibr+6CS+ZngPITrcEiN+Z/g7F+t/UWL5iMiJ8FPwrGi+ZTg
tIPL7tIa81mR42BlxAsi4oHhIId7/MCS+Zrg3IVr96Iax0Oi00KXVYcvl/6b
S3xY1D4LxFPDQQ73+AElx1OjI0I3VC6ZrwnOXLhyB9U43hiOcrjJO5TM1wVn
IlyJl9WYT4ocGL9gPiia/4scBcuFtxbMh0XOCPgvtwi31rE2lM0V+L2kvfDf
ZXOL/sw7JvxH2XXY9NZyj8vOxcE1wJdFDg34OP/R/vZX+y1lc4/yDewrvLns
fWEDbSX8e9nck98xpxFeX3YZm2BL4d/K/pawUbYV/qts7tAfo+bvIicDfJyb
dbx9VL+p7DpsnvBrwXkI1+ElatdG9RvL5jL8IWp9CPoTfFtuTplvixwVv7Fe
xXdA7f8suy02yRdD/QltiQF5KtS3MNYQ8/Ee/qzES0Xsk3FR2fzS2ALTof8p
+bh3y9on87FQP0WsPzEy5OsmH/XuWbeZiT922blIno04fzb5t+HGbSm5q+x8
2pTxEX0T/92y22IDhg8QDlb4FI8v2v8+XnF8F3FexLO0rlhXgt8i8StNKtYt
4bdEPEGq4ngo4qKIH9mxYt0UfkPEX2Qrjn8iDgr/f5JhE09FXBXxP9tVQn8y
/Nzw/y47N8nTktfxby+7jA8D+bjJz90663swB//OsnNpvCB5B3/Tssv4xHyA
v3DZuTVeiZgPD45P+B/76Hrn4t9bdu4Rnsnrob5qdMoxO/AbwgEKX+Zxav8p
/qplc0O+gf4Cf92y941Pzkf4r5adm+Q1ydv4owo/H7EN/mX8cYUfj9hmPQ1/
2rJzo/Dfq/jnCj8ZsQ/HdPy9y84lwn/kPySfIVzOzSUf4x9c9rHw6ZmF/23Z
uWc4JvE3/wSObyPOjXieVhXrAvErvbPsfIrsC59i/N/mBuY+H8rzrHP+ilTK
Mcv4z30QOL6QOEP4SuEnhUu5cdb5Kcgfge/lP/gL1zlfBWVivsmHQb4IfDnZ
J/73/wucqwjOf/zvnwmcywCO/9vK5oNtmrWP+Hht37Bi364S31jZfKkcG59n
/AM/Chy/SBwjfLLkk4Qrm33A59mmYt0hfrPEt2wIHH9IHOLYOufnIBYSnwD8
+58LnEuFHAfwNcJBDF/tCUXnx8hV7Kv6d9LxTX8GjgckLpB4qRYV687x6yNf
CPlB8DWN6L/Rdc4fQpkYcvh04deFC7su6/iqZhXrWvFDJv8K+VTwdY1L7qlz
PhbKxJhfUzZfayVrH3P4W+FXhYub/64sm6+XfeMTD78q/LlwbTeQXF823ypl
fNovK5tPtpS1Dzt8qfCtwsWdl1xSNn8qZXzeb61zvhDuxeak4/3+DhxPSVwl
/LTwy8KdzT5ZO7GGgjuDmJif9P6+U/ZcBJ+7qfinl809/kjEfAbodNDlFEvm
r4TDFv7ek4rmk8ImgC3gpqL5LtAhozseUTRfBjp1dOk3M37jL6v9v6t9z42Y
TwEbAraDkUXzGZAzj/yidwgvwH+9bC722RHzp8KBC9/vyUXHzqHPZ65ODN1X
av9y2W3x4YN/AZ0durpVRfMtYAPB9nFL0fyt5ECDL3+vovlQyZFGPobWRfPF
klMNfvG9i+ZPJScb+QL2FH4Rf/yy7xU6TfhGyclG/oA9iuZvhWMbvuF+wl/g
X6/2b0XsAwg/KJze8BWfWDR/JBzQ8Bv3Ff5a7V8t+169FzH/LDnayH/Qpmiu
XPTxt6bMmbtI7V8pm0uee4y/0jjhByLWiU4hnqHs3GcPRWxPwH6AL+g9KfNT
oRNHF74cf1PiHcpui84XPil0qOhOVxbNN4WOE93mD/gKldyP0Z/h8w1/FTp3
dO3fF81fhU4ZXfKKomMFMxVzuRIz+GZo3+Bc7k2Zj5cccOTD2F3t5xOPUHau
Ku7hjNDegO/v6NDegb0C3+M7UubXQqeNLnsZ+gfiF8oeS/CxhF8HnRS6qG9V
/4zqx6r+ftWNZcwmvqPsMvcQPhBsTtiabi3aHoJ9A99bngH8XdgEsAV8V3Ss
KfYXzoWY0/8Djg3ijw==
             "]], Polygon3DBox[CompressedData["
1:eJwl13mcz3UeB/DvmDHDIOcIMwwjVyPjPpaIyJhxzjjGMQw52iwRDSNy3yai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             "]]}]}, {}, {}, {}}, {Line3DBox[CompressedData["
1:eJwl1Edsj3Ecx/GvWTVae68arVGbAw64cOHChYsV40AJvXDhwoWL2dq1arRW
7dGWmrU5UKsRGkEEEUSsxOuJw8v7+UiT///5PU+bMXPhxAW1IqKZf56mRiTX
j/Q6Q1zPJp/i9Iib+pt+fqjanqZruUJ604jvmlU7Ygpj7NVaxmdy7Yw6EZNY
SaF9Rt/ToW5ElT1Bl1NCarOIGm1VL2IcI+2lWkw1OXZ6fZ9DLgV2oVaRmhLx
0B6pORRQp7n/S9ogYljCnqubuc1c+68OcN8zuMVz1lPFfTawkTxKGzsX3cRm
yuwtupVtlNvbdQcFXLR36i52c8neo3sppMLep/s5wGX7oBZRzBX7kB7mCFft
o3qMEq7Zx/UEJ5PnZ5/S05zhhn1Wz3GeSvtCch/Jd+emXZ58z+S7JfdvVyTf
I/ls7vAi+Vye8IB1TKc/+c7vj/O7xSZm23N0KLUZYj9w/juYTy17hDbgsWd2
3/Paq4sZzXY7TV941kXMs5foWFoy3H7t/TjGMlLs8dqed96pR96n07qCieyx
u+on72Ipi+xVOplMRtnfvL+XWUMTe6pm88s7/9z7Xql5zKLSPT9jsOvHeo9B
rgcygJ/JuSS/K2Tzw+6rfejNd7uXZpHJV7un9qA7X+xumkFXPttdtDOd+Gh3
1A6054PdTtvShvd2a21FS97aLbR58rvOG9sthtuKNGrsJuqxRyNe2Q1VwuOL
ly48qvBIwvFHte2ow5GG44u7qf//fvwDw6WTaw==
          "]], {
          Line3DBox[{4011, 4760, 4010, 4761, 4012, 4762, 4013, 3320, 4014, 
           4763, 4015, 4764, 4016, 5375, 5585, 4017, 4765, 4018, 4766, 4019, 
           5376, 5586, 4020, 4767, 4021, 4768, 4022, 5377, 5587, 4023, 4769, 
           4024, 4770, 4025, 5378, 5588, 4026, 4771, 4027, 4772, 4028, 5379, 
           5589, 4029, 4773, 4030, 4774, 4031, 5380, 5590, 4032, 4775, 4033, 
           4776, 4034, 5381, 5591, 4035, 4777, 4036, 4778, 4037, 5382, 5592, 
           4038, 4779, 4039, 4780, 4040, 5383, 5593, 4041, 4781, 4042, 4782, 
           4043, 5384, 5594, 4044, 4783, 4045, 4784, 4046, 5385, 5595, 4047, 
           4785, 4048, 4786, 4049, 5386, 5596, 4050, 4787, 4051, 4788, 4052, 
           5387, 5597, 4053, 4789, 4054, 4790, 4055, 5388, 5598, 4056, 4791, 
           4057, 4792, 4058, 4793, 4059}], 
          Line3DBox[{4061, 4794, 4060, 4795, 4062, 4796, 4063, 5480, 4797, 
           4064, 4798, 4065, 4799, 4066, 3366, 4067, 4800, 4068, 4801, 4069, 
           5389, 5599, 4070, 4802, 4071, 4803, 4072, 5390, 5600, 4073, 4804, 
           4074, 4805, 4075, 5391, 5601, 4076, 4806, 4077, 4807, 4078, 5392, 
           5602, 4079, 4808, 4080, 4809, 4081, 5393, 5603, 4082, 4810, 4083, 
           4811, 4084, 5394, 5604, 4085, 4812, 4086, 4813, 4087, 5395, 5605, 
           4088, 4814, 4089, 4815, 4090, 5396, 5606, 4091, 4816, 4092, 4817, 
           4093, 5397, 5607, 4094, 4818, 4095, 4819, 4096, 5398, 5608, 4097, 
           4820, 4098, 4821, 4099, 5399, 5609, 4100, 4822, 4101, 4823, 4102, 
           5400, 5610, 4103, 4824, 4104, 4825, 4105, 5401, 5611, 4106, 4826, 
           4107, 4827, 4108, 4828, 4109}], 
          Line3DBox[{4111, 4829, 4110, 4830, 4112, 4831, 4113, 5481, 4832, 
           4114, 4833, 4115, 4834, 4116, 5482, 4835, 4117, 4836, 4118, 4837, 
           4119, 3412, 4120, 4838, 4121, 4839, 4122, 5402, 5612, 4123, 4840, 
           4124, 4841, 4125, 5403, 5613, 4126, 4842, 4127, 4843, 4128, 5404, 
           5614, 4129, 4844, 4130, 4845, 4131, 5405, 5615, 4132, 4846, 4133, 
           4847, 4134, 5406, 5616, 4135, 4848, 4136, 4849, 4137, 5407, 5617, 
           4138, 4850, 4139, 4851, 4140, 5408, 5618, 4141, 4852, 4142, 4853, 
           4143, 5409, 5619, 4144, 4854, 4145, 4855, 4146, 5410, 5620, 4147, 
           4856, 4148, 4857, 4149, 5411, 5621, 4150, 4858, 4151, 4859, 4152, 
           5412, 5622, 4153, 4860, 4154, 4861, 4155, 5413, 5623, 4156, 4862, 
           4157, 4863, 4158, 4864, 4159}], 
          Line3DBox[{4161, 4865, 4160, 4866, 4162, 4867, 4163, 5483, 4868, 
           4164, 4869, 4165, 4870, 4166, 5484, 4871, 4167, 4872, 4168, 4873, 
           4169, 5485, 4874, 4170, 4875, 4171, 4876, 4172, 3458, 4173, 4877, 
           4174, 4878, 4175, 5414, 5624, 4176, 4879, 4177, 4880, 4178, 5415, 
           5625, 4179, 4881, 4180, 4882, 4181, 5416, 5626, 4182, 4883, 4183, 
           4884, 4184, 5417, 5627, 4185, 4885, 4186, 4886, 4187, 5418, 5628, 
           4188, 4887, 4189, 4888, 4190, 5419, 5629, 4191, 4889, 4192, 4890, 
           4193, 5420, 5630, 4194, 4891, 4195, 4892, 4196, 5421, 5631, 4197, 
           4893, 4198, 4894, 4199, 5422, 5632, 4200, 4895, 4201, 4896, 4202, 
           5423, 5633, 4203, 4897, 4204, 4898, 4205, 5424, 5634, 4206, 4899, 
           4207, 4900, 4208, 4901, 4209}], 
          Line3DBox[{4211, 4902, 4210, 4903, 4212, 4904, 4213, 5486, 4905, 
           4214, 4906, 4215, 4907, 4216, 5487, 4908, 4217, 4909, 4218, 4910, 
           4219, 5488, 4911, 4220, 4912, 4221, 4913, 4222, 5489, 4914, 4223, 
           4915, 4224, 4916, 4225, 3504, 4226, 4917, 4227, 4918, 4228, 5425, 
           5635, 4229, 4919, 4230, 4920, 4231, 5426, 5636, 4232, 4921, 4233, 
           4922, 4234, 5427, 5637, 4235, 4923, 4236, 4924, 4237, 5428, 5638, 
           4238, 4925, 4239, 4926, 4240, 5429, 5639, 4241, 4927, 4242, 4928, 
           4243, 5430, 5640, 4244, 4929, 4245, 4930, 4246, 5431, 5641, 4247, 
           4931, 4248, 4932, 4249, 5432, 5642, 4250, 4933, 4251, 4934, 4252, 
           5433, 5643, 4253, 4935, 4254, 4936, 4255, 5434, 5644, 4256, 4937, 
           4257, 4938, 4258, 4939, 4259}], 
          Line3DBox[{4261, 4940, 4260, 4941, 4262, 4942, 4263, 5490, 4943, 
           4264, 4944, 4265, 4945, 4266, 5491, 4946, 4267, 4947, 4268, 4948, 
           4269, 5492, 4949, 4270, 4950, 4271, 4951, 4272, 5493, 4952, 4273, 
           4953, 4274, 4954, 4275, 5494, 4955, 4276, 4956, 4277, 4957, 4278, 
           3550, 4279, 4958, 4280, 4959, 4281, 5435, 5645, 4282, 4960, 4283, 
           4961, 4284, 5436, 5646, 4285, 4962, 4286, 4963, 4287, 5437, 5647, 
           4288, 4964, 4289, 4965, 4290, 5438, 5648, 4291, 4966, 4292, 4967, 
           4293, 5439, 5649, 4294, 4968, 4295, 4969, 4296, 5440, 5650, 4297, 
           4970, 4298, 4971, 4299, 5441, 5651, 4300, 4972, 4301, 4973, 4302, 
           5442, 5652, 4303, 4974, 4304, 4975, 4305, 5443, 5653, 4306, 4976, 
           4307, 4977, 4308, 4978, 4309}], 
          Line3DBox[{4311, 4979, 4310, 4980, 4312, 4981, 4313, 5495, 4982, 
           4314, 4983, 4315, 4984, 4316, 5496, 4985, 4317, 4986, 4318, 4987, 
           4319, 5497, 4988, 4320, 4989, 4321, 4990, 4322, 5498, 4991, 4323, 
           4992, 4324, 4993, 4325, 5499, 4994, 4326, 4995, 4327, 4996, 4328, 
           5500, 4997, 4329, 4998, 4330, 4999, 4331, 3596, 4332, 5000, 4333, 
           5001, 4334, 5444, 5654, 4335, 5002, 4336, 5003, 4337, 5445, 5655, 
           4338, 5004, 4339, 5005, 4340, 5446, 5656, 4341, 5006, 4342, 5007, 
           4343, 5447, 5657, 4344, 5008, 4345, 5009, 4346, 5448, 5658, 4347, 
           5010, 4348, 5011, 4349, 5449, 5659, 4350, 5012, 4351, 5013, 4352, 
           5450, 5660, 4353, 5014, 4354, 5015, 4355, 5451, 5661, 4356, 5016, 
           4357, 5017, 4358, 5018, 4359}], 
          Line3DBox[{4361, 5019, 4360, 5020, 4362, 5021, 4363, 5501, 5022, 
           4364, 5023, 4365, 5024, 4366, 5502, 5025, 4367, 5026, 4368, 5027, 
           4369, 5503, 5028, 4370, 5029, 4371, 5030, 4372, 5504, 5031, 4373, 
           5032, 4374, 5033, 4375, 5505, 5034, 4376, 5035, 4377, 5036, 4378, 
           5506, 5037, 4379, 5038, 4380, 5039, 4381, 5507, 5040, 4382, 5041, 
           4383, 5042, 4384, 3642, 4385, 5043, 4386, 5044, 4387, 5452, 5662, 
           4388, 5045, 4389, 5046, 4390, 5453, 5663, 4391, 5047, 4392, 5048, 
           4393, 5454, 5664, 4394, 5049, 4395, 5050, 4396, 5455, 5665, 4397, 
           5051, 4398, 5052, 4399, 5456, 5666, 4400, 5053, 4401, 5054, 4402, 
           5457, 5667, 4403, 5055, 4404, 5056, 4405, 5458, 5668, 4406, 5057, 
           4407, 5058, 4408, 5059, 4409}], 
          Line3DBox[{4411, 5060, 4410, 5061, 4412, 5062, 4413, 5508, 5063, 
           4414, 5064, 4415, 5065, 4416, 5509, 5066, 4417, 5067, 4418, 5068, 
           4419, 5510, 5069, 4420, 5070, 4421, 5071, 4422, 5511, 5072, 4423, 
           5073, 4424, 5074, 4425, 5512, 5075, 4426, 5076, 4427, 5077, 4428, 
           5513, 5078, 4429, 5079, 4430, 5080, 4431, 5514, 5081, 4432, 5082, 
           4433, 5083, 4434, 5515, 5084, 4435, 5085, 4436, 5086, 4437, 3688, 
           4438, 5087, 4439, 5088, 4440, 5459, 5669, 4441, 5089, 4442, 5090, 
           4443, 5460, 5670, 4444, 5091, 4445, 5092, 4446, 5461, 5671, 4447, 
           5093, 4448, 5094, 4449, 5462, 5672, 4450, 5095, 4451, 5096, 4452, 
           5463, 5673, 4453, 5097, 4454, 5098, 4455, 5464, 5674, 4456, 5099, 
           4457, 5100, 4458, 5101, 4459}], 
          Line3DBox[{4461, 5102, 4460, 5103, 4462, 5104, 4463, 5516, 5105, 
           4464, 5106, 4465, 5107, 4466, 5517, 5108, 4467, 5109, 4468, 5110, 
           4469, 5518, 5111, 4470, 5112, 4471, 5113, 4472, 5519, 5114, 4473, 
           5115, 4474, 5116, 4475, 5520, 5117, 4476, 5118, 4477, 5119, 4478, 
           5521, 5120, 4479, 5121, 4480, 5122, 4481, 5522, 5123, 4482, 5124, 
           4483, 5125, 4484, 5523, 5126, 4485, 5127, 4486, 5128, 4487, 5524, 
           5129, 4488, 5130, 4489, 5131, 4490, 3734, 4491, 5132, 4492, 5133, 
           4493, 5465, 5675, 4494, 5134, 4495, 5135, 4496, 5466, 5676, 4497, 
           5136, 4498, 5137, 4499, 5467, 5677, 4500, 5138, 4501, 5139, 4502, 
           5468, 5678, 4503, 5140, 4504, 5141, 4505, 5469, 5679, 4506, 5142, 
           4507, 5143, 4508, 5144, 4509}], 
          Line3DBox[{4511, 5145, 4510, 5146, 4512, 5147, 4513, 5525, 5148, 
           4514, 5149, 4515, 5150, 4516, 5526, 5151, 4517, 5152, 4518, 5153, 
           4519, 5527, 5154, 4520, 5155, 4521, 5156, 4522, 5528, 5157, 4523, 
           5158, 4524, 5159, 4525, 5529, 5160, 4526, 5161, 4527, 5162, 4528, 
           5530, 5163, 4529, 5164, 4530, 5165, 4531, 5531, 5166, 4532, 5167, 
           4533, 5168, 4534, 5532, 5169, 4535, 5170, 4536, 5171, 4537, 5533, 
           5172, 4538, 5173, 4539, 5174, 4540, 5534, 5175, 4541, 5176, 4542, 
           5177, 4543, 3780, 4544, 5178, 4545, 5179, 4546, 5470, 5680, 4547, 
           5180, 4548, 5181, 4549, 5471, 5681, 4550, 5182, 4551, 5183, 4552, 
           5472, 5682, 4553, 5184, 4554, 5185, 4555, 5473, 5683, 4556, 5186, 
           4557, 5187, 4558, 5188, 4559}], 
          Line3DBox[{4561, 5189, 4560, 5190, 4562, 5191, 4563, 5535, 5192, 
           4564, 5193, 4565, 5194, 4566, 5536, 5195, 4567, 5196, 4568, 5197, 
           4569, 5537, 5198, 4570, 5199, 4571, 5200, 4572, 5538, 5201, 4573, 
           5202, 4574, 5203, 4575, 5539, 5204, 4576, 5205, 4577, 5206, 4578, 
           5540, 5207, 4579, 5208, 4580, 5209, 4581, 5541, 5210, 4582, 5211, 
           4583, 5212, 4584, 5542, 5213, 4585, 5214, 4586, 5215, 4587, 5543, 
           5216, 4588, 5217, 4589, 5218, 4590, 5544, 5219, 4591, 5220, 4592, 
           5221, 4593, 5545, 5222, 4594, 5223, 4595, 5224, 4596, 3826, 4597, 
           5225, 4598, 5226, 4599, 5474, 5684, 4600, 5227, 4601, 5228, 4602, 
           5475, 5685, 4603, 5229, 4604, 5230, 4605, 5476, 5686, 4606, 5231, 
           4607, 5232, 4608, 5233, 4609}], 
          Line3DBox[{4611, 5234, 4610, 5235, 4612, 5236, 4613, 5546, 5237, 
           4614, 5238, 4615, 5239, 4616, 5547, 5240, 4617, 5241, 4618, 5242, 
           4619, 5548, 5243, 4620, 5244, 4621, 5245, 4622, 5549, 5246, 4623, 
           5247, 4624, 5248, 4625, 5550, 5249, 4626, 5250, 4627, 5251, 4628, 
           5551, 5252, 4629, 5253, 4630, 5254, 4631, 5552, 5255, 4632, 5256, 
           4633, 5257, 4634, 5553, 5258, 4635, 5259, 4636, 5260, 4637, 5554, 
           5261, 4638, 5262, 4639, 5263, 4640, 5555, 5264, 4641, 5265, 4642, 
           5266, 4643, 5556, 5267, 4644, 5268, 4645, 5269, 4646, 5557, 5270, 
           4647, 5271, 4648, 5272, 4649, 3872, 4650, 5273, 4651, 5274, 4652, 
           5477, 5687, 4653, 5275, 4654, 5276, 4655, 5478, 5688, 4656, 5277, 
           4657, 5278, 4658, 5279, 4659}], 
          Line3DBox[{4661, 5280, 4660, 5281, 4662, 5282, 4663, 5558, 5283, 
           4664, 5284, 4665, 5285, 4666, 5559, 5286, 4667, 5287, 4668, 5288, 
           4669, 5560, 5289, 4670, 5290, 4671, 5291, 4672, 5561, 5292, 4673, 
           5293, 4674, 5294, 4675, 5562, 5295, 4676, 5296, 4677, 5297, 4678, 
           5563, 5298, 4679, 5299, 4680, 5300, 4681, 5564, 5301, 4682, 5302, 
           4683, 5303, 4684, 5565, 5304, 4685, 5305, 4686, 5306, 4687, 5566, 
           5307, 4688, 5308, 4689, 5309, 4690, 5567, 5310, 4691, 5311, 4692, 
           5312, 4693, 5568, 5313, 4694, 5314, 4695, 5315, 4696, 5569, 5316, 
           4697, 5317, 4698, 5318, 4699, 5570, 5319, 4700, 5320, 4701, 5321, 
           4702, 3918, 4703, 5322, 4704, 5323, 4705, 5479, 5689, 4706, 5324, 
           4707, 5325, 4708, 5326, 4709}], 
          Line3DBox[{4711, 5327, 4710, 5328, 4712, 5329, 4713, 5571, 5330, 
           4714, 5331, 4715, 5332, 4716, 5572, 5333, 4717, 5334, 4718, 5335, 
           4719, 5573, 5336, 4720, 5337, 4721, 5338, 4722, 5574, 5339, 4723, 
           5340, 4724, 5341, 4725, 5575, 5342, 4726, 5343, 4727, 5344, 4728, 
           5576, 5345, 4729, 5346, 4730, 5347, 4731, 5577, 5348, 4732, 5349, 
           4733, 5350, 4734, 5578, 5351, 4735, 5352, 4736, 5353, 4737, 5579, 
           5354, 4738, 5355, 4739, 5356, 4740, 5580, 5357, 4741, 5358, 4742, 
           5359, 4743, 5581, 5360, 4744, 5361, 4745, 5362, 4746, 5582, 5363, 
           4747, 5364, 4748, 5365, 4749, 5583, 5366, 4750, 5367, 4751, 5368, 
           4752, 5584, 5369, 4753, 5370, 4754, 5371, 4755, 3964, 4756, 5372, 
           4757, 5373, 4758, 5374, 4759}]}, {
          Line3DBox[{2525, 3275, 2526, 3290, 2555, 3305, 2570, 3320, 2585, 
           3335, 2600, 3350, 2615, 3365, 5480, 2630, 3380, 2645, 3395, 2660, 
           3410, 5481, 2675, 3425, 2690, 3440, 2705, 3455, 5483, 2720, 3470, 
           2735, 3485, 2750, 3500, 5486, 2765, 3515, 2780, 3530, 2795, 3545, 
           5490, 2810, 3560, 2825, 3575, 2840, 3590, 5495, 2855, 3605, 2870, 
           3620, 2885, 3635, 5501, 2900, 3650, 2915, 3665, 2930, 3680, 5508, 
           2945, 3695, 2960, 3710, 2975, 3725, 5516, 2990, 3740, 3005, 3755, 
           3020, 3770, 5525, 3035, 3785, 3050, 3800, 3065, 3815, 5535, 3080, 
           3830, 3095, 3845, 3110, 3860, 5546, 3125, 3875, 3140, 3890, 3155, 
           3905, 5558, 3170, 3920, 3185, 3935, 3200, 3950, 5571, 3215, 3965, 
           3230, 3980, 3245, 3995, 3260}], 
          Line3DBox[{2527, 3276, 2528, 3291, 2556, 3306, 2571, 5585, 3321, 
           2586, 3336, 2601, 3351, 2616, 3366, 2631, 3381, 2646, 3396, 2661, 
           3411, 5482, 2676, 3426, 2691, 3441, 2706, 3456, 5484, 2721, 3471, 
           2736, 3486, 2751, 3501, 5487, 2766, 3516, 2781, 3531, 2796, 3546, 
           5491, 2811, 3561, 2826, 3576, 2841, 3591, 5496, 2856, 3606, 2871, 
           3621, 2886, 3636, 5502, 2901, 3651, 2916, 3666, 2931, 3681, 5509, 
           2946, 3696, 2961, 3711, 2976, 3726, 5517, 2991, 3741, 3006, 3756, 
           3021, 3771, 5526, 3036, 3786, 3051, 3801, 3066, 3816, 5536, 3081, 
           3831, 3096, 3846, 3111, 3861, 5547, 3126, 3876, 3141, 3891, 3156, 
           3906, 5559, 3171, 3921, 3186, 3936, 3201, 3951, 5572, 3216, 3966, 
           3231, 3981, 3246, 3996, 3261}], 
          Line3DBox[{2529, 3277, 2530, 3292, 2557, 3307, 2572, 5586, 3322, 
           2587, 3337, 2602, 3352, 2617, 5599, 3367, 2632, 3382, 2647, 3397, 
           2662, 3412, 2677, 3427, 2692, 3442, 2707, 3457, 5485, 2722, 3472, 
           2737, 3487, 2752, 3502, 5488, 2767, 3517, 2782, 3532, 2797, 3547, 
           5492, 2812, 3562, 2827, 3577, 2842, 3592, 5497, 2857, 3607, 2872, 
           3622, 2887, 3637, 5503, 2902, 3652, 2917, 3667, 2932, 3682, 5510, 
           2947, 3697, 2962, 3712, 2977, 3727, 5518, 2992, 3742, 3007, 3757, 
           3022, 3772, 5527, 3037, 3787, 3052, 3802, 3067, 3817, 5537, 3082, 
           3832, 3097, 3847, 3112, 3862, 5548, 3127, 3877, 3142, 3892, 3157, 
           3907, 5560, 3172, 3922, 3187, 3937, 3202, 3952, 5573, 3217, 3967, 
           3232, 3982, 3247, 3997, 3262}], 
          Line3DBox[{2531, 3278, 2532, 3293, 2558, 3308, 2573, 5587, 3323, 
           2588, 3338, 2603, 3353, 2618, 5600, 3368, 2633, 3383, 2648, 3398, 
           2663, 5612, 3413, 2678, 3428, 2693, 3443, 2708, 3458, 2723, 3473, 
           2738, 3488, 2753, 3503, 5489, 2768, 3518, 2783, 3533, 2798, 3548, 
           5493, 2813, 3563, 2828, 3578, 2843, 3593, 5498, 2858, 3608, 2873, 
           3623, 2888, 3638, 5504, 2903, 3653, 2918, 3668, 2933, 3683, 5511, 
           2948, 3698, 2963, 3713, 2978, 3728, 5519, 2993, 3743, 3008, 3758, 
           3023, 3773, 5528, 3038, 3788, 3053, 3803, 3068, 3818, 5538, 3083, 
           3833, 3098, 3848, 3113, 3863, 5549, 3128, 3878, 3143, 3893, 3158, 
           3908, 5561, 3173, 3923, 3188, 3938, 3203, 3953, 5574, 3218, 3968, 
           3233, 3983, 3248, 3998, 3263}], 
          Line3DBox[{2533, 3279, 2534, 3294, 2559, 3309, 2574, 5588, 3324, 
           2589, 3339, 2604, 3354, 2619, 5601, 3369, 2634, 3384, 2649, 3399, 
           2664, 5613, 3414, 2679, 3429, 2694, 3444, 2709, 5624, 3459, 2724, 
           3474, 2739, 3489, 2754, 3504, 2769, 3519, 2784, 3534, 2799, 3549, 
           5494, 2814, 3564, 2829, 3579, 2844, 3594, 5499, 2859, 3609, 2874, 
           3624, 2889, 3639, 5505, 2904, 3654, 2919, 3669, 2934, 3684, 5512, 
           2949, 3699, 2964, 3714, 2979, 3729, 5520, 2994, 3744, 3009, 3759, 
           3024, 3774, 5529, 3039, 3789, 3054, 3804, 3069, 3819, 5539, 3084, 
           3834, 3099, 3849, 3114, 3864, 5550, 3129, 3879, 3144, 3894, 3159, 
           3909, 5562, 3174, 3924, 3189, 3939, 3204, 3954, 5575, 3219, 3969, 
           3234, 3984, 3249, 3999, 3264}], 
          Line3DBox[{2535, 3280, 2536, 3295, 2560, 3310, 2575, 5589, 3325, 
           2590, 3340, 2605, 3355, 2620, 5602, 3370, 2635, 3385, 2650, 3400, 
           2665, 5614, 3415, 2680, 3430, 2695, 3445, 2710, 5625, 3460, 2725, 
           3475, 2740, 3490, 2755, 5635, 3505, 2770, 3520, 2785, 3535, 2800, 
           3550, 2815, 3565, 2830, 3580, 2845, 3595, 5500, 2860, 3610, 2875, 
           3625, 2890, 3640, 5506, 2905, 3655, 2920, 3670, 2935, 3685, 5513, 
           2950, 3700, 2965, 3715, 2980, 3730, 5521, 2995, 3745, 3010, 3760, 
           3025, 3775, 5530, 3040, 3790, 3055, 3805, 3070, 3820, 5540, 3085, 
           3835, 3100, 3850, 3115, 3865, 5551, 3130, 3880, 3145, 3895, 3160, 
           3910, 5563, 3175, 3925, 3190, 3940, 3205, 3955, 5576, 3220, 3970, 
           3235, 3985, 3250, 4000, 3265}], 
          Line3DBox[{2537, 3281, 2538, 3296, 2561, 3311, 2576, 5590, 3326, 
           2591, 3341, 2606, 3356, 2621, 5603, 3371, 2636, 3386, 2651, 3401, 
           2666, 5615, 3416, 2681, 3431, 2696, 3446, 2711, 5626, 3461, 2726, 
           3476, 2741, 3491, 2756, 5636, 3506, 2771, 3521, 2786, 3536, 2801, 
           5645, 3551, 2816, 3566, 2831, 3581, 2846, 3596, 2861, 3611, 2876, 
           3626, 2891, 3641, 5507, 2906, 3656, 2921, 3671, 2936, 3686, 5514, 
           2951, 3701, 2966, 3716, 2981, 3731, 5522, 2996, 3746, 3011, 3761, 
           3026, 3776, 5531, 3041, 3791, 3056, 3806, 3071, 3821, 5541, 3086, 
           3836, 3101, 3851, 3116, 3866, 5552, 3131, 3881, 3146, 3896, 3161, 
           3911, 5564, 3176, 3926, 3191, 3941, 3206, 3956, 5577, 3221, 3971, 
           3236, 3986, 3251, 4001, 3266}], 
          Line3DBox[{2539, 3282, 2540, 3297, 2562, 3312, 2577, 5591, 3327, 
           2592, 3342, 2607, 3357, 2622, 5604, 3372, 2637, 3387, 2652, 3402, 
           2667, 5616, 3417, 2682, 3432, 2697, 3447, 2712, 5627, 3462, 2727, 
           3477, 2742, 3492, 2757, 5637, 3507, 2772, 3522, 2787, 3537, 2802, 
           5646, 3552, 2817, 3567, 2832, 3582, 2847, 5654, 3597, 2862, 3612, 
           2877, 3627, 2892, 3642, 2907, 3657, 2922, 3672, 2937, 3687, 5515, 
           2952, 3702, 2967, 3717, 2982, 3732, 5523, 2997, 3747, 3012, 3762, 
           3027, 3777, 5532, 3042, 3792, 3057, 3807, 3072, 3822, 5542, 3087, 
           3837, 3102, 3852, 3117, 3867, 5553, 3132, 3882, 3147, 3897, 3162, 
           3912, 5565, 3177, 3927, 3192, 3942, 3207, 3957, 5578, 3222, 3972, 
           3237, 3987, 3252, 4002, 3267}], 
          Line3DBox[{2541, 3283, 2542, 3298, 2563, 3313, 2578, 5592, 3328, 
           2593, 3343, 2608, 3358, 2623, 5605, 3373, 2638, 3388, 2653, 3403, 
           2668, 5617, 3418, 2683, 3433, 2698, 3448, 2713, 5628, 3463, 2728, 
           3478, 2743, 3493, 2758, 5638, 3508, 2773, 3523, 2788, 3538, 2803, 
           5647, 3553, 2818, 3568, 2833, 3583, 2848, 5655, 3598, 2863, 3613, 
           2878, 3628, 2893, 5662, 3643, 2908, 3658, 2923, 3673, 2938, 3688, 
           2953, 3703, 2968, 3718, 2983, 3733, 5524, 2998, 3748, 3013, 3763, 
           3028, 3778, 5533, 3043, 3793, 3058, 3808, 3073, 3823, 5543, 3088, 
           3838, 3103, 3853, 3118, 3868, 5554, 3133, 3883, 3148, 3898, 3163, 
           3913, 5566, 3178, 3928, 3193, 3943, 3208, 3958, 5579, 3223, 3973, 
           3238, 3988, 3253, 4003, 3268}], 
          Line3DBox[{2543, 3284, 2544, 3299, 2564, 3314, 2579, 5593, 3329, 
           2594, 3344, 2609, 3359, 2624, 5606, 3374, 2639, 3389, 2654, 3404, 
           2669, 5618, 3419, 2684, 3434, 2699, 3449, 2714, 5629, 3464, 2729, 
           3479, 2744, 3494, 2759, 5639, 3509, 2774, 3524, 2789, 3539, 2804, 
           5648, 3554, 2819, 3569, 2834, 3584, 2849, 5656, 3599, 2864, 3614, 
           2879, 3629, 2894, 5663, 3644, 2909, 3659, 2924, 3674, 2939, 5669, 
           3689, 2954, 3704, 2969, 3719, 2984, 3734, 2999, 3749, 3014, 3764, 
           3029, 3779, 5534, 3044, 3794, 3059, 3809, 3074, 3824, 5544, 3089, 
           3839, 3104, 3854, 3119, 3869, 5555, 3134, 3884, 3149, 3899, 3164, 
           3914, 5567, 3179, 3929, 3194, 3944, 3209, 3959, 5580, 3224, 3974, 
           3239, 3989, 3254, 4004, 3269}], 
          Line3DBox[{2545, 3285, 2546, 3300, 2565, 3315, 2580, 5594, 3330, 
           2595, 3345, 2610, 3360, 2625, 5607, 3375, 2640, 3390, 2655, 3405, 
           2670, 5619, 3420, 2685, 3435, 2700, 3450, 2715, 5630, 3465, 2730, 
           3480, 2745, 3495, 2760, 5640, 3510, 2775, 3525, 2790, 3540, 2805, 
           5649, 3555, 2820, 3570, 2835, 3585, 2850, 5657, 3600, 2865, 3615, 
           2880, 3630, 2895, 5664, 3645, 2910, 3660, 2925, 3675, 2940, 5670, 
           3690, 2955, 3705, 2970, 3720, 2985, 5675, 3735, 3000, 3750, 3015, 
           3765, 3030, 3780, 3045, 3795, 3060, 3810, 3075, 3825, 5545, 3090, 
           3840, 3105, 3855, 3120, 3870, 5556, 3135, 3885, 3150, 3900, 3165, 
           3915, 5568, 3180, 3930, 3195, 3945, 3210, 3960, 5581, 3225, 3975, 
           3240, 3990, 3255, 4005, 3270}], 
          Line3DBox[{2547, 3286, 2548, 3301, 2566, 3316, 2581, 5595, 3331, 
           2596, 3346, 2611, 3361, 2626, 5608, 3376, 2641, 3391, 2656, 3406, 
           2671, 5620, 3421, 2686, 3436, 2701, 3451, 2716, 5631, 3466, 2731, 
           3481, 2746, 3496, 2761, 5641, 3511, 2776, 3526, 2791, 3541, 2806, 
           5650, 3556, 2821, 3571, 2836, 3586, 2851, 5658, 3601, 2866, 3616, 
           2881, 3631, 2896, 5665, 3646, 2911, 3661, 2926, 3676, 2941, 5671, 
           3691, 2956, 3706, 2971, 3721, 2986, 5676, 3736, 3001, 3751, 3016, 
           3766, 3031, 5680, 3781, 3046, 3796, 3061, 3811, 3076, 3826, 3091, 
           3841, 3106, 3856, 3121, 3871, 5557, 3136, 3886, 3151, 3901, 3166, 
           3916, 5569, 3181, 3931, 3196, 3946, 3211, 3961, 5582, 3226, 3976, 
           3241, 3991, 3256, 4006, 3271}], 
          Line3DBox[{2549, 3287, 2550, 3302, 2567, 3317, 2582, 5596, 3332, 
           2597, 3347, 2612, 3362, 2627, 5609, 3377, 2642, 3392, 2657, 3407, 
           2672, 5621, 3422, 2687, 3437, 2702, 3452, 2717, 5632, 3467, 2732, 
           3482, 2747, 3497, 2762, 5642, 3512, 2777, 3527, 2792, 3542, 2807, 
           5651, 3557, 2822, 3572, 2837, 3587, 2852, 5659, 3602, 2867, 3617, 
           2882, 3632, 2897, 5666, 3647, 2912, 3662, 2927, 3677, 2942, 5672, 
           3692, 2957, 3707, 2972, 3722, 2987, 5677, 3737, 3002, 3752, 3017, 
           3767, 3032, 5681, 3782, 3047, 3797, 3062, 3812, 3077, 5684, 3827, 
           3092, 3842, 3107, 3857, 3122, 3872, 3137, 3887, 3152, 3902, 3167, 
           3917, 5570, 3182, 3932, 3197, 3947, 3212, 3962, 5583, 3227, 3977, 
           3242, 3992, 3257, 4007, 3272}], 
          Line3DBox[{2551, 3288, 2552, 3303, 2568, 3318, 2583, 5597, 3333, 
           2598, 3348, 2613, 3363, 2628, 5610, 3378, 2643, 3393, 2658, 3408, 
           2673, 5622, 3423, 2688, 3438, 2703, 3453, 2718, 5633, 3468, 2733, 
           3483, 2748, 3498, 2763, 5643, 3513, 2778, 3528, 2793, 3543, 2808, 
           5652, 3558, 2823, 3573, 2838, 3588, 2853, 5660, 3603, 2868, 3618, 
           2883, 3633, 2898, 5667, 3648, 2913, 3663, 2928, 3678, 2943, 5673, 
           3693, 2958, 3708, 2973, 3723, 2988, 5678, 3738, 3003, 3753, 3018, 
           3768, 3033, 5682, 3783, 3048, 3798, 3063, 3813, 3078, 5685, 3828, 
           3093, 3843, 3108, 3858, 3123, 5687, 3873, 3138, 3888, 3153, 3903, 
           3168, 3918, 3183, 3933, 3198, 3948, 3213, 3963, 5584, 3228, 3978, 
           3243, 3993, 3258, 4008, 3273}], 
          Line3DBox[{2553, 3289, 2554, 3304, 2569, 3319, 2584, 5598, 3334, 
           2599, 3349, 2614, 3364, 2629, 5611, 3379, 2644, 3394, 2659, 3409, 
           2674, 5623, 3424, 2689, 3439, 2704, 3454, 2719, 5634, 3469, 2734, 
           3484, 2749, 3499, 2764, 5644, 3514, 2779, 3529, 2794, 3544, 2809, 
           5653, 3559, 2824, 3574, 2839, 3589, 2854, 5661, 3604, 2869, 3619, 
           2884, 3634, 2899, 5668, 3649, 2914, 3664, 2929, 3679, 2944, 5674, 
           3694, 2959, 3709, 2974, 3724, 2989, 5679, 3739, 3004, 3754, 3019, 
           3769, 3034, 5683, 3784, 3049, 3799, 3064, 3814, 3079, 5686, 3829, 
           3094, 3844, 3109, 3859, 3124, 5688, 3874, 3139, 3889, 3154, 3904, 
           3169, 5689, 3919, 3184, 3934, 3199, 3949, 3214, 3964, 3229, 3979, 
           3244, 3994, 3259, 4009, 3274}]}, {}, {}}},
       VertexNormals->CompressedData["
1:eJw8nHdcT+/7x9ulpS3tvffWOHdDaWpvIpGGlQiZKYqK+CiiyCYJ0cS5U6hE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        "]],
      Axes->True,
      AxesLabel->{
        FormBox["\"Sells\"", TraditionalForm], 
        FormBox["\"Buys\"", TraditionalForm], None},
      BaseStyle->{FontFamily -> "Times", FontSize -> 12},
      BoxRatios->{1, 1, 0.4},
      DisplayFunction:>Identity,
      Method->{"RotationControl" -> "Globe"},
      PlotRange->{All, All, All},
      PlotRangePadding->{Automatic, Automatic, Automatic},
      Ticks->None,
      ViewPoint->{2, 1.8, 0.8}], {576., -189.}, ImageScaled[{0.5, 0.5}], {360., 360.},
     
     ContentSelectable->True]}, {}},
  ContentSelectable->True,
  PlotRangePadding->{6, 5}]], "Output",
 CellChangeTimes->{{3.389622274722*^9, 3.3896223041689997`*^9}, 
   3.3896223421070004`*^9, 3.3896225325880003`*^9, 3.3899915127366*^9},
 CellLabel->"Out[254]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Supplementary results", "Subsubsection"],

Cell[TextData[{
 "The \"base\" density here is a bivariate normal centered at ten buys and \
ten sells. The stretching of the density along the ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " axes reflects the higher arrival rates on information days. The canyon \
between the two lobes occurs because if an information event occurs, good and \
bad news are mutually exclusive. The density above is representative. Changes \
in the parameter values can dramatically distort the picture, as the \
following examples show."
}], "Text",
 CellChangeTimes->{{3.389622704992*^9, 3.389622705856*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"g", "[", 
    RowBox[{"\[Mu]_", ",", "\[Alpha]_", ",", "\[Epsilon]_"}], "]"}], ":=", 
   RowBox[{"Evaluate", "[", 
    RowBox[{"Log", "[", "fMixApprox", "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"gg", "[", 
    RowBox[{"\[Mu]_", ",", "\[Alpha]_", ",", "\[Epsilon]_"}], "]"}], ":=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{
     RowBox[{"g", "[", 
      RowBox[{"\[Mu]", ",", "\[Alpha]", ",", "\[Epsilon]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"S", ",", "0", ",", "50"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"B", ",", "0", ",", "50"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "30"}], ",", 
     RowBox[{"DisplayFunction", "\[Rule]", "Identity"}], ",", 
     RowBox[{"PlotLabel", "\[Rule]", 
      RowBox[{"\"\<\[Mu]=\>\"", "<>", 
       RowBox[{"ToString", "[", "\[Mu]", "]"}], "<>", "\"\< \[Alpha]=\>\"", "<>",
        
       RowBox[{"ToString", "[", 
        RowBox[{"NumberForm", "[", 
         RowBox[{"\[Alpha]", ",", 
          RowBox[{"{", 
           RowBox[{"3", ",", "2"}], "}"}]}], "]"}], "]"}], "<>", 
       "\"\< \[Epsilon]=\>\"", "<>", 
       RowBox[{"ToString", "[", "\[Epsilon]", "]"}], "<>", "\"\< PIN=\>\"", "<>",
        
       RowBox[{"ToString", "[", 
        RowBox[{"NumberForm", "[", 
         RowBox[{
          FractionBox[
           RowBox[{"\[Alpha]", " ", "\[Mu]"}], 
           RowBox[{
            RowBox[{"\[Alpha]", " ", "\[Mu]"}], "+", 
            RowBox[{"2", " ", "\[Epsilon]"}]}]], ",", 
          RowBox[{"{", 
           RowBox[{"3", ",", "2"}], "}"}]}], "]"}], "]"}]}]}], ",", 
     RowBox[{"BaseStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"FontFamily", "\[Rule]", "\"\<Times\>\""}], ",", 
        RowBox[{"FontSize", "\[Rule]", "12"}]}], "}"}]}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.389622631927*^9, 3.389622653505*^9}},
 CellLabel->"In[255]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"GraphicsGrid", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"gg", "[", 
        RowBox[{"10", ",", "0.1`", ",", "10"}], "]"}], ",", 
       RowBox[{"gg", "[", 
        RowBox[{"10", ",", "0.2`", ",", "10"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"gg", "[", 
        RowBox[{"10", ",", "0.5`", ",", "10"}], "]"}], ",", 
       RowBox[{"gg", "[", 
        RowBox[{"10", ",", "0.99`", ",", "10"}], "]"}]}], "}"}]}], "}"}], 
   "]"}], "]"}]], "Input",
 CellLabel->"In[257]:="],

Cell[BoxData[
 GraphicsBox[{{}, {{InsetBox[
      GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyMfXVglMfzd3ApFiCBQIDgFkIS4jqXy0VwghWnuBanWNHg7g7F3aV4cbeW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        "], {{{}, {}, {}, {}, {}, {}, {}, 
          {RGBColor[
           0.38706038036848905`, 0.22022668739102078`, 0.6610405050504429], 
           EdgeForm[None], 
           GraphicsGroupBox[
            PolygonBox[{{6399, 6398, 3564, 3565}, {6378, 3827, 3513, 3526}, {
             6388, 4404, 3525, 3550}, {6395, 6390, 3553, 3558}, {6387, 6386, 
             3548, 3549}, {6389, 4347, 3523, 3552}, {5283, 6384, 3544, 
             3547}, {6377, 4530, 3528, 3521}, {5280, 6371, 3511, 3542}, {4666,
              6370, 3510, 3530}, {6390, 6378, 3526, 3553}, {4530, 6373, 3514, 
             3528}, {6398, 3963, 3518, 3564}, {6385, 6392, 3555, 3545}, {5281,
              5280, 3542, 3543}, {6381, 4251, 3522, 3536}, {5120, 6383, 3539, 
             3540}, {6382, 6393, 3556, 3538}, {5965, 6395, 3558, 3560}, {6392,
              6391, 3554, 3555}, {5091, 6381, 3536, 3537}, {5596, 6377, 3521, 
             3551}, {6394, 4563, 3529, 3557}, {5282, 5283, 3547, 3546}, {6346,
              3949, 3516, 3566}, {4962, 6387, 3549, 3533}, {6384, 3950, 3517, 
             3544}, {6397, 5281, 3543, 3562}, {4961, 4962, 3533, 3532}, {4756,
              6374, 3515, 3531}, {6393, 6394, 3557, 3556}, {6379, 6380, 3535, 
             3534}, {4521, 4961, 3532, 3527}, {3950, 6385, 3545, 3517}, {6391,
              6346, 3566, 3554}, {4404, 6376, 3520, 3525}, {6373, 4756, 3531, 
             3514}, {4347, 4348, 3524, 3523}, {4251, 6389, 3552, 3522}, {6369,
              6388, 3550, 3509}, {6386, 5596, 3551, 3548}, {4348, 6375, 3519, 
             3524}, {6050, 6399, 3565, 3563}, {6374, 5282, 3546, 3515}, {6370,
              5091, 3537, 3510}, {6371, 4666, 3530, 3511}, {6396, 6397, 3562, 
             3561}, {6372, 5120, 3540, 3512}, {5964, 5965, 3560, 3559}, {5220,
              5964, 3559, 3541}, {3949, 6396, 3561, 3516}, {3963, 6382, 3538, 
             3518}, {6375, 6050, 3563, 3519}, {4563, 6372, 3512, 3529}, {6383,
              5220, 3541, 3539}, {6376, 4521, 3527, 3520}, {6380, 6369, 3509, 
             3535}},
             VertexColors->None]]}, 
          {RGBColor[
           0.41825908458067645`, 0.2744728083378958, 0.7048940580218185], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVE1IlFEUvZqjjmPfzDc/+Ba1aFebIHTMaRlURDMEURDtMgj6UVcR6cIW
RVBQbaKBqSjbhbTQNlEraVGYtGhVtLFyoaQUjlbqWOdw7uLx/s4799xz7/ft
6B86NthsZmcwtmDsaTF7mDD7gHU1a3YzY1bE2d9gtoTxHuA+3Fdw9grrW7h/
AFwv9jew/pQTx51Os2ir2VuQlvHuCMZCSnzLBXHebjN7Ca4Z8MxjfRXrU816
8xrzZJPZRZw/SojzWmx2PRZurkNviO8D796UtM0jzhfw70uarSPWCvYNvB2H
xjXsO4EppXQ2ZdLBfJgLtZBjGuNeTvkw7kfEXAu6pzfkpj/DabNDsfCzuP+B
0e3evGs3u4I8zkPLhYL29KTovu1uVx7MnTNz4Xp/ZDYaif8yztKt8A9vzibF
Qe6d0PUccWvgf4b5KLRWE8IRz7jMr+w5HoyllTpHMG96HWc8Z55/BsdUJL/p
JT3lW+K4n3afS37eA8665/sb85+guh1vlUf0rQU67+akh3p3ZaR5DtiJWDjL
m30P6oVVzOtBNd3APJCWV6w/ecm5EIQjph6prqwp8x7vkD9vcH4/q/7h2Tb3
jX5QC/W/QNzJvOpQ9H4l/lxBuol7jPsneb0fhJafQX25jLkfeTUQcwX4VYwT
SfU3e4l89GbW/TkJLd2x1hsZ9SXrQ47TzvMP2LEueTzQJj5+U7yr+vf4FFrG
MLZ7L9FvauZ3xj29+ga+r0G9U8vKC2Kom3xcVxCn3KVviHrZW/SHvbyZU61Z
z19e00vgOBDLN+LoB2PVkMdikK/UTH7GoT7WjOsSfDucVQ70shH0nmfsXZ4T
y/fE1wv6p/R6L1e8Zvz3sHfZj/8BdOO07Q==
              "],
              VertexColors->None], 
             PolygonBox[{{6049, 4422, 715, 2871}, {4421, 6049, 2871, 194}, {
              3779, 5621, 2389, 1901}, {5008, 3701, 194, 889}, {6386, 6387, 
              1789, 207}, {6346, 6391, 968, 3342}, {5081, 5082, 1900, 1073}, {
              6385, 3950, 700, 968}, {6381, 5091, 1911, 1100}, {4759, 4760, 
              1077, 435}, {6375, 4348, 1277, 2873}, {4764, 4763, 435, 1078}, {
              5622, 5620, 1078, 2389}, {6376, 4404, 1331, 1570}, {6389, 4251, 
              1100, 1277}, {4531, 3779, 1901, 1438}, {6394, 6393, 716, 
              1465}, {4137, 4696, 1331, 923}, {6397, 6396, 699, 2076}, {6050, 
              6375, 2873, 2872}, {5964, 5220, 2025, 2789}, {6374, 4756, 1620, 
              1438}, {6370, 4666, 1551, 1911}, {4756, 6373, 1437, 1620}, {
              3949, 6346, 3342, 699}, {4961, 4521, 1432, 1789}, {6372, 4563, 
              1465, 1934}, {5299, 4484, 1408, 2094}, {5282, 6374, 1438, 
              2077}, {6219, 3777, 1336, 3050}, {6384, 5283, 2077, 700}, {5119,
               5300, 2094, 717}, {5638, 5482, 2249, 2406}, {5483, 5481, 889, 
              2249}, {4403, 4694, 1570, 1331}, {6383, 5120, 1934, 717}, {5220,
               6383, 717, 2025}, {6380, 6379, 923, 1331}, {4485, 4425, 602, 
              1408}, {4530, 6377, 2362, 1437}, {6371, 5280, 2076, 2732}, {
              4425, 3828, 524, 602}, {3777, 5638, 2406, 1336}, {6395, 5965, 
              2789, 602}, {4694, 3897, 1432, 1570}, {5118, 5119, 717, 715}, {
              3778, 4531, 1438, 1900}, {5619, 6220, 3050, 1077}, {4521, 6376, 
              1570, 1432}, {4943, 3650, 207, 1432}, {3827, 6378, 602, 524}, {
              4666, 6371, 2732, 1551}, {3711, 5013, 1073, 207}, {5596, 6386, 
              207, 2362}, {6398, 6399, 2872, 716}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.44945778879286385`, 0.3287189292847708, 0.7487476109931941], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VEtrU2EQndbkphG55t48rEIXgvgAV71dKO4EJalCoE1j3IhN0aK3pJt2
7UZBQcGF1VYFce0DtVWKu7SFlv4DC+5da3RVH+dwxsXwfXe+mTNnzkxyuD07
1uk3syuwPbBvObNq1uwynCdx7wVmG3hYLJj1V8zaebOzoVm9qLfNyGwd1kXu
Any3QuUdjc3eISfN4B2+ZcR86DO7hJwpYIzAP5kXHnPHcL5F3feIqSH2DmwU
3x/LZp9gId6vBfIRc9XzmZvux71k9hmcdxB7D3WH8f4GGBPgsZQVNmsk8B8H
v+0B9biC+Lsef3HQ7EdZceeR+/KAtBgB/7lQtch90vnPoO4jr0tO5Pa1T1ij
GfXbRd6FQfVFDozlnbjkQQ670O037D6+W+ix6fiv9kovYvXA6xesgberqPui
JLxpYF+HBfvE91wkbGI0HId80pzmyD42fV43gJfCtgZUg/HkXM9rhozn3NZC
+WtZcWSPryPpS22Zz77YyzxiHxbV201gBxX128AcToE3l+1IoDzeE58l+TTh
7+T0/bOsvWFd7hnnTz4d14gxx4DXLaouZ8jZs9ch6LAdSr/djPYm9ZlQT2rC
uXFvyPs0zmYsPks4n8aKW8T5JBavKs6DqHEI9xXk/kXdB+DyDP7nsThxp7Z8
r/j+x2PG88JIXGPyIIeJQDP8r0HLue0g70tRM+MujvvekivxqSd15W7xzho1
3z3+5oYq6pf5w76H3xFbjRTXC/Ub475yb6gR45uua+K5ddd8Cv22S/pv4O6v
Ov8NcDxTUMxjaLAQq6cWzhMFzfk26iyXxI0aJ5F0/gdLSZf4
              "],
              VertexColors->None], 
             PolygonBox[{{3962, 4422, 1352, 1464}, {4346, 4780, 1637, 1276}, {
              5483, 5482, 433, 2248}, {4137, 4526, 1433, 1572}, {5633, 4248, 
              2402, 2401}, {4693, 4692, 622, 1433}, {5057, 5219, 2024, 601}, {
              5618, 5917, 2731, 1098}, {4695, 4524, 1433, 1571}, {4562, 3826, 
              1353, 1464}, {5218, 3825, 1352, 2024}, {5299, 5300, 1933, 
              477}, {5217, 3879, 601, 698}, {5619, 4234, 2387, 2386}, {4764, 
              4235, 2390, 1624}, {5622, 5621, 434, 2388}, {3778, 5082, 1899, 
              434}, {5081, 4231, 2383, 1899}, {4235, 5620, 2388, 2390}, {5634,
               3636, 193, 2402}, {4485, 4484, 477, 1354}, {6246, 3637, 1276, 
              3078}, {4942, 5557, 2318, 1780}, {5661, 6247, 3078, 1131}, {
              5117, 4645, 477, 1932}, {4645, 4646, 1131, 477}, {4780, 3828, 
              1354, 1637}, {4155, 5556, 1074, 964}, {4759, 3780, 1902, 
              1622}, {4526, 4136, 1571, 1433}, {4548, 4758, 1621, 1075}, {
              4757, 3936, 1454, 1621}, {5558, 5060, 964, 2318}, {3825, 4562, 
              1464, 1352}, {4423, 3962, 1464, 1353}, {3896, 4680, 1562, 
              622}, {4100, 5481, 2248, 2250}, {4547, 4548, 1075, 193}, {4403, 
              4696, 1572, 622}, {5118, 4423, 1353, 1933}, {4249, 5633, 2401, 
              2403}, {4233, 5618, 1098, 1074}, {4231, 5013, 1836, 2383}, {
              3780, 4763, 1624, 1902}, {4762, 4249, 2403, 1623}, {3711, 3650, 
              1780, 1836}, {4681, 3897, 622, 1562}, {5008, 4100, 2250, 
              1832}, {4421, 3701, 1832, 1352}, {5279, 3948, 698, 1454}, {4424,
               5116, 1932, 1353}, {4234, 4760, 1622, 2387}, {3786, 4761, 1623,
               2731}, {6219, 6220, 2386, 433}},
              VertexColors->None], 
             PolygonBox[{{4523, 4942, 1780, 4943, 4522}, {4121, 4523, 4522, 
              4681, 1562}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.4806564930050512, 0.3829650502316458, 0.7926011639645697], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVD1Mk1EUvaU/X0uxfN/rVwQMxiLI7k+CcWNQGhKDBkJMjEgVSIjR1U03
44AbauvgZGJCsVEBR6NFMY6uLm6Mhm4gGs/Jecbh5b13v3vPPffc+75y9c7l
2x1mtoCVxBrrNbsbmv3Im63EZns9ZlHW7FNk9hlrLW02FZjlYcsjsIWgLpzX
cG7gWzOt8zJiH8X6NuzMfhfNEhmzWezXsQ4y8iNeA/t75NlG7l8psxuI+1BU
nlvdZs9xPwKcjZLZRdiXkbMZKHcnYi/lzEaxviTEm+dnOO8i9i1y34TfOPZR
3J8g3xbqaEXiOwLbHPB3gLdbMKvGqmsM/psF4byA71XkrSWFfdbnYq0jnkeY
lU71hHKUM/LnuS8jzgOHzdol8Z0C/rSTXwP4Mzh/h/7nsV9wimkj/7dIfE5i
Hw6FuRdKL2q1CM0GcL4HzENYM4E4t1HX0Ui1TwfSktwmvR99msAZRK4l4Jzy
seTZ8D1kX+7n1DfivIE2raLwyIW6kE9XTn2mDqzvYagaqc1+SvZV+PaFwt8n
d6wHnZgF8FxxynUa9Xb0KOYa8ryMpB81YG+ow5VAM0N+1Ii8icl5pB7/ZpIY
nI0U8NJY8znNDnnQfwLfXwXyYT95Z0+fgmMC/hXcHzvNCvWo4fy1IJ8ydBiM
/8dXfOw28N8VxJHzSV3IhfjzPge1pN7Uc7ZXWMQZwn48VhzfwapTPRvAex2r
d1vA3yypT5zDSd/HA9jHI+HyG3Wl/QxiF4C1jrk64fQOmZ89/Oj7yFproWrg
W+HckBttdSf7hNeSNSbhXw9l48xxPuh/DL4/EfsnrbfA98qesoYl3xf2pOKx
+rPyY13rqOlcqLnkm6Jm7K/zPg3/n3D+zFne8fNc7dZ/im+Ptn6P+Rdwx58P

              "],
              VertexColors->None], 
             PolygonBox[{{3947, 5268, 2064, 2074}, {4679, 4212, 2355, 1561}, {
              4762, 4761, 456, 1097}, {5152, 5156, 1967, 522}, {5595, 4217, 
              2361, 2360}, {4547, 4551, 1455, 1452}, {3767, 4549, 1453, 
              1881}, {5594, 4232, 2384, 2360}, {3717, 4402, 921, 219}, {5634, 
              4248, 1097, 1274}, {4216, 5595, 2360, 3040}, {4121, 4680, 1431, 
              909}, {5062, 5063, 1881, 685}, {5058, 5056, 1876, 1877}, {5061, 
              5882, 2695, 1879}, {3786, 5917, 2385, 456}, {5155, 5555, 2316, 
              1967}, {4551, 3700, 1831, 1455}, {3695, 4640, 1534, 180}, {5057,
               3879, 2023, 1876}, {5279, 3936, 684, 2075}, {5883, 4674, 411, 
              2695}, {4960, 3717, 219, 1788}, {4250, 6239, 1966, 1099}, {6246,
               6247, 2431, 180}, {5268, 3948, 2075, 2064}, {4693, 4525, 921, 
              1569}, {3896, 4692, 1569, 1431}, {5154, 5152, 522, 1966}, {4382,
               5660, 2431, 868}, {5555, 5261, 2060, 2316}, {5558, 5557, 909, 
              1879}, {5267, 3938, 2065, 2064}, {5262, 5062, 685, 2060}, {5881,
               3765, 410, 2065}, {5217, 3947, 2074, 2023}, {4550, 3939, 2066, 
              1453}, {3766, 6210, 3040, 411}, {5117, 5116, 1130, 1537}, {4779,
               4271, 1130, 1275}, {4272, 4646, 1537, 2432}, {5218, 5219, 1876,
               523}, {4642, 4081, 868, 1130}, {4232, 5556, 2317, 2384}, {4155,
               5060, 1879, 2317}, {3939, 5267, 2064, 2066}, {4525, 4524, 219, 
              921}, {5297, 5298, 1877, 410}, {4212, 5061, 1879, 2355}, {3700, 
              3636, 1274, 1831}, {5637, 5635, 1099, 2405}, {4424, 3826, 523, 
              1130}, {4233, 5594, 2360, 2385}, {4217, 5636, 2405, 2361}, {
              4757, 4758, 1452, 684}, {5661, 4272, 2432, 2431}, {4520, 4678, 
              1561, 1431}, {3935, 4345, 1275, 2059}, {5660, 3695, 180, 
              2431}, {4956, 4958, 1431, 921}, {5056, 3935, 2059, 1876}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5118551972172386, 0.4372111711785208, 0.8364547169359453], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVE1IVFEY/fx744z25s3PG2dhaToUBS0K2pq6qCTaKERIpOnspIR+rDYu
giKN/ndSmyiIRkfSfrYmaCAR1KIfEoLQZWWbSC3snDl3cXnv3Xfuueec77t3
a99g56lyM7uEUYHxKDTbnTDbU2VWwOjDZAGAO2mz+jqzV3h/7putZsyuxMxu
Yv5E3KyI+V94NiXNNrCuWGt2sNLsdRm4wHfZF+f5rNkM1u6LCTMRMQuBa8P8
J+z9tcZsBTzZlFk/9m4E3yDWNmBtDns1prWG3H619l0D3xzwSczdwv/baf3L
4PsIsAk8C3hOAnsPeiqAr8LIR80OQ8NJz2zaxN3k+BfK5Z/eK4G9GIjvLv73
x8U1C1+doXAz2H82JfwqsD1xvQ94yoP8D4Fvg5e3Jn87ncdR7Dcc1Z774fdx
Qpi/NfJBr2lP/jjfG9U/zm/DXg+gYQw8U1i7EGhP5tsSU/1aNmlNqR7Ycx7j
KPY8A2xrVvn8gN6hrPDPwPMmUC7kYUbU/wX7XHA5HPJUt/vIcwT4c4HmesAx
hvd/0JfH95Jbm4OPxZjqkAO+GcM8ZTmfUJ7MnT3BnjFkvhFKN2uY8aTzOLQf
w1j35It46vgOnt8p6Wf2M64WzDbA/8UyeVmuVS714N+SUF91eOJnvtS77DST
m3swt5KfqPy2QnsYqCY8E99SOhd/8HyH+o5EVOdhV2viOhz/1ZhyIk+pNjHp
ZP+eDYTh/ktOJ89Nb0pnh/P0QG2d0NCF0eiphqcD5fMe+CdJ+e2OqFacbwfm
ha8sPyLXIeD3RlW3z6E0sY+2B+qlzcimK1Q+7JWiOzsN6P8d8Dweke/BuLxz
sMbETIKjkFT9++I6M6xvHeY/hMqa54n9RH30fSAp79eBnQ7VAzw3eXc26YXZ
MTf25YCbZ23JTb/r0LOGcQ3YdvCtZ7TXFDifYuyq1l3C2tPXOOZ++tJddB5K
+svVp+TkWbmRVgbUwB6gZmbX7e4W9ij7JHT3zLhbS15+k5M6R929suIrR+Ko
ddTh2Te8S4ihVq7hO88NsQV3T4w4HvY7zy37iD1KD8yN+TWn9U4e3tPkyjvN
zJBaJ9w771x6CB0HvXLtHHrgpS/9/wHgIs+b
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtkztoU2EUx49pHo1KcnOT3GZRm7YiDg4WXNUWVERcIogItjV281FQaJ06
Fh/4BJegm4KY2mJ9jWpFI1UEHXyA4OIqrU6mUerv8M9w4Hz3O+f/ON+55epY
5VTMzAaIOLFSNBsqmf1dYzaSNnuSMRsMzaKk2cGUWS/5upzZhgTFfHsemdWo
37/WbMdq+jifp2Yfd+c4l8nfBmYnOK/QM0P+J08NWJPg/86aPQx173hf4Z+g
phgXxmu4XhE3V5nNIrSH2lnuR9t4I2ANE7UOs37Oe7jfTbwHLgSjhZ7lSLl7
+ICfefDqCWn4Ap9F8vodzzcKZtWsuAJ6Fsm7Q/k9BkcDr8+o3463H3i+TP0l
ooG+Qyl57QvlxT19Av8M33Jtfvc2UBKXe9zVZbaEpml6v4F5j94Wfi76Gcwx
+KfB7+s0e4GmXs+J+ylxbgLvTlHefQZPwWrkpcVndhevQ5xbSc3EtTfpqcfk
4U1eb+ZvdZie09x9Bm9bWppfcj8H3xb4F+hpRpqRz2aKnkn4dqK50MbfjJ+P
GWl3T75LV6nPdGqnrpFfIWZi+vaA/HFGWM4xTv1ESbtUxc98Xhqc2zVPwdUV
aJZ7icWMZuiz8xkdofZ2Tlrc8zLnJnEhpZkewNtSVm/pb7qV2gqY3UnNsBiI
w7HdUxl9w+j5h/ZbeErgP05E+B9Na/cfUX8yqX/AuXzHfbedcyN5f07YzvGO
8xz1x5Pq8d3/ldUu+76MB3oTfwt/g3qonfNd851fCPRPeb1jnOU8WNK/6ZqO
gnW9oF3xGfcU5MG1+85U8P8zq9n6P1CjvyOSF/e4Pqd/0vfX5/Mf4C6RVQ==

              "],
              VertexColors->None], 
             PolygonBox[{{4691, 4518, 4517, 4133, 1568}, {4518, 4957, 1787, 
              4956, 4517}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.543053901429426, 0.4914572921253958, 0.8803082699073209], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVV1ojmEYvvHt2/fx/b7v3s+7JAkhK2FxNpS1fEtpO8ABte3AdoCyfMsB
J4qJ2fxECSWbWP62lJwJLfk5mGb+orXvSIallJ/5uS7XIwdPz/Pez/1z3dd9
3887u3FH3fbJZnYGawrWQGj2BquYMDtQarYnbtaAi6sZsxk5fX/1za7huyVi
dgrf5TGzYzhPh2y/Z3YI+osjko3C5wHIktPN8lGz0/B9B346cTdRYtYH+1Gc
P8O2Nqfvg9CfwLkM8vdps1medAtJs11YowmtZtz3w/9H2K6B/gi+fwVm72Bz
HZhaUmZp2PZAp+jiMRb1fuB8BPsi6GxPyQ/zfpJV7rVx4fWhM477auCPg6gP
8F2HGDfgfwD3/djXT1Y+fTFxNQ6d+kB3+bjyuAP/G8vMNmGNlgoHd9rWRIR3
YJI4p+/Xk8Tv1ojOy+PilHyui4t38hsF5vOI9SUm3Vsl0mduxEQ8J8HPz0Bx
7nviixxsRh0vIt+TsNkNzINZ2ZMb8kUfj6A7GzYdOFeDh31Yr4BhJnTneML/
FLKF4OfWVLPhqHgh54aavAiEvRN5H8VKAudtlzPzXefyYG4LgKcDawj2P7D3
wn8Tzl2w2xIq9xGHi3ltgLzcVzzmdyojv+yRxqj4+BdvwHHCvJjLZfi+5Ekv
AewvfGFgnN+BYg374q7G9cueqOy7Q/U0+2OuJ47osxK5vJ1mtgR7P+SPM4qZ
h/5YoF7g/dK4dAqode9UYfMS6h/WjLP1zVf9OU9BRjNVk1MPsW/IC/nmDPTA
93zacA5x34BapnGeF1MM9i17kPHZz/Tb5PhpRe3WhupP5jfhas+aP3R5seaR
UHUfg+97nvivSIiXEee71vmnDfubvdEGu0Ko/IiFtSWeMKq+24bzStxVJXTH
fqz3dMeZoIz8tJdqNtjb5IlnYmgFxuFAnJa5GjAX2jEebck178nhFfCzIy1u
ig7n395GD3xKCRvnqdvNFPurK6ce42y0OwzkN+lmljXge0Q+i+5t4nvCN+yB
L1xzUnqPehw25stY3VnNIfOlbKWzvQv5YEqxGl1s+l8FbvZCXlGiWAX3HlZA
f1FW8iH4e+ar71nnZveOHMf9spT4GUd+nwLNH9+/w77i0hfnh/rViBVmVCvO
xnJXX+4r3LkN9y8DfbNnv7u+vUBZWvUlB4WMeCCWKpfjTU//A/Z8u6e54kyt
DuWXPivdrBBzDrITWdWxyr3/5JAzlHa1OIv6niv7/08qd/PP2WA9iYc8rfbE
1XPg3JlRnDrIlmSl0wEMh0P1OP3QB98E7qw7z/T3Naa5yDuOyAk5bnU8/wEz
uAs/
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlE9I1FEQxyfTbbdc1/3trm5USNj/hLAijyUUkiKFQtFRu9ShhMKNDnYs
I00ryVMdSqmkKCWQblJREBUYalYQsZ0qSyk0y6g+w9fDwHtvZr5v5jvf91Y2
NtUdzTGzPVguVpcyexg3e7fA7F7YbDRhNoZV4XxA4E/W09iZRWat2L6k2X7s
SchsL/5/5FuR2baIWZblb2LnsIP4D4PxOWb2NDBrYT+XZzZE7Im0WZBv1oF/
nPxK9hURYTwi9jH2AV/DQrPnhWb3A8WexzaAnU7o/jFsnPUUGLXkbsK/vNjs
IvXF6KVvsVkX2BsLzAaobTt3ThB7Gswa4quxZdQzEdNdLezL4aKXmF/kH8lR
73cK5XMOSsCeAi+C7xD3NeM7S31J1gmsE7xS/L2mGpvAzoKRIXc1mJXEFpFz
jvvK4CNDfd+I6Q+Lk3ruXhEXtwOcdRP7l7O7Yc3Ea28FozqkHq5Sz0hCs1pK
zCew1rEfCSnnGPE7uOP9ErOtxDfg7yAnGhZHufhC2DTrt9S8lvhrKe2dA+d6
KqZavJ9J/FX0mKWfLPXsJrc6LS6HyK+ntrpA3F2mv1GwLsXlKydmEm5uJ8WF
c3gDXy13/Agp/ia5J7lvkHUjZ8NxadC152cp7ycQd87hMHizCWnNNXyKfWWB
8DzGtfcFvD950uAk9Wyh/u6INJMB73VK3PgMe1hfT83PPizujhOzOSIO36Q0
c5+1a3aMffH8PD2mLS3Nu9bLmGeba5Wzj/nS4Av6GWOfierM1y85a47qLF0o
TpyLNPWuL5bmXGuD6DnKPr9YWncNlNDfd3L6I+rxGeuBQG+lHStlvSrQ2jXl
3PTM4ztHNUWaqc/SMbyWW4G495p24m9Pqxd/k87dlaS05hweSOpPcGzXhM+y
LC7uHb8PrFcFqt3/hBn8X2P6a/w9XSC3M6m/wzXpb8XfsL9dfzOuxTXUNxvW
PNoTetP+ll3jXXFpzLXlM6gAe1cg7nyG/rZm5v8vf2P/Ae6ruXU=
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.5714906066251039, 0.5398885732266976, 0.9096735877515102], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVVlsjGEUvXQ67TCd1UwbEoKJkBINDygiQis0SCT2WKNVFM/iTVrx0CaS
tiq2F3ut0dSLeEGIB0vtWh6a2EpNxZImEuIc54qHL9+d+9/v3HvPPd83Izfu
XLJjoJmdxMrBupprdqjArBz78iFmy7C688y2hc1eDjK7NMBsK34fcrs3ajYs
bfYL8WuKYBealQXM9iKmA3snYp4lzZ5iVcNXGjO7DvsGkj3C/illVgl/P+yO
uFkVznyH/S0pjDDwCgqV7yhqmYxcl4HZid9dWPko/jby1iN+NOL7gF+BmAsh
sw1BxGL/HNC3fMQ/hm8cYg+k5WfdVV7n6IjZfvgHDFKevpRyzYe9IC2seWnl
IH4L7DKs87DbwuJoFezawWZ1WAvR47F82eSQvLFeYjJHrcdsgX9CQJyQv+6k
OGxCXw2wM/g+A/sk1PcG9ZfgW3FAOOyRfazH2feo9wPWPNTQGNM8iLMvAS5j
ysH8JX52GnyfU8r7FfYIxGXxrQe+GvwuB/Y72HUx5VsUEjbP96D2MV43a1rk
fmqk0+dCbsqdH9bIvTsgjraH1Tdt8kYt8XdNWJjEoZ/1L8XcsxFpkrO6O1Dz
OlwgrbbnSgutzu9wxE9P6jf77k0J8wLsH0lhcCacDfnfnKf+LgKzBBrMIK4F
/rGI7YG+1+VIs+SLXFFL1BHrYb3bvf5S5HkdFeYTxB8bIuxmxPXnC3MWOJ6I
mE2osxG5nuJMPb6fhf9+XL3Mwf4woh6pM86EvK0E5vG48Fh/reuqw3XM+CBm
HsA6OFj351RI36m9lz6XVvjOuZ/6aXWbd32l3/c18K0NiZNryDsKNS/G2WfA
vud1LoA9v0haYO38Rj857neeqQXm5RypgaxrlbNt8/l+BAcjHf+fvllnDJx0
wd8HzDLYU6LqfV1I9XEuH+GrLxLmceCcSInr39hbYop9QD7jett4ju8Nz7b6
G8Gz1BLnyplS91V56p0cVLvNOde4bheiv9PArMfZmbCvIN/boHol19QDtXnY
39IjbpMfzjaS0LyoUfbMfnnn9iSUn9hFMeFTaxmveRVyrcb6CV8a32cjPhVU
DL+TQ+qaM+z2d4J4rH8XuDqT0P3ZHZRmOjyGOuWbkEUfz5PqsTiiNyjj95z3
nfonH5udE+Ykn8x7J6F5kM9bsG8mlKcCeCuS0n/WdcDayMcR5513YmpC94J5
yCVzzYFvLlZhUPeEvTKG+iU26+e7nvE7yblVem0lUd03xn9BXy+Smg21Rk0v
9reQb9pfnnKFQX+l3wHitPt/Inn7CZyhhf/fMO6c9auo7j/vPt+AdFz2KdcY
6+Rdb4qrjwbwOjctDU5Djc3u/xQVj+SQWv/bd47yU7+sYXxc+qOfb1WFc/sH
SZ4Ygw==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtk9lL1XEQxcfUq9fc7va7t4KihSgskHpoT8KbomFBEGlQVpQthD30FL1F
24NSYHojSgglyoykqD+gIugh2hesHoQ2U28rBIHRZzo9DMz3NzNnzsyc3/Tt
+9e3TjCzBiwPm1FmtiFuVpNvto7AWMIsi+0pMLuda5aJmFWSs4NYG+9HvNPY
fd63qHmA/zZldq7E7Cbv07yDcrNEiHze6agwvDbJt6cxsyfYTvBbsBS5q8np
zFfOYvwlUdV6z+P4veDvKza7lmO2jNrl2Duw+qjvAbs6KmzHWAn3Kqy70GyI
AUuJ/ST/GLmD1L/E/018U1gzLiw164io33swFvD+RnxtWD3WBGbf4ZgF62pY
sY9YbVg5X/C/YnvBGgB/Ff0qIuLuO3iMf7BM+2oB4xO5H7CKPGFk6fcwol36
DSYn8Zn3RrE4Zug9Tv5m/GbsEviXo9q1Y44Sqyd/eKLZfDBPwXcSMx7hnVMk
rA7fcZ4wm7h1Y1y7GSpQ7sa4fK/xW97DZhbopt3Ezsc12y6+peE3Na7dvy7S
bU5E1dtv1MksK2Lape+0Hj5zeY8RH4B/P1zqAvnbQtJSdUS38XkOlOomXj+b
HheYL0SPM3DLoI9hdvknoV00864JdHOfp59vi4j38P5VKD2kiWfpeYVYX1ja
eENOQ640Mg3uI7y35urGXeRnAmE556pAnJ3rRb7dIX+U/HGwD4V063n/9e03
PxyVxl3bNcR7E5rBuTsn331tIC0555KkNO7aPss+f8TU03u5Zkeo/ZzQv+n/
TBdcXpCzG38LOa/wi5OqdQ22sav2lGb9NxP5d6Pi6ju4DtbScv3bvuMpzqVc
2vSZWtl9BTYLrEIwjxIbTmgW12x1Uhrw2w9i7fSfE9MtvMa16hydm2u2LSZM
x3JN5cOtMqJ9+E3rUrq539o13Eh+U0y385yhmHp6L9dwQO3JQNr2Gz4n/iwm
bfpO/gJx+rMf
              "],
              VertexColors->None], 
             PolygonBox[{{5775, 3690, 3689, 3619, 2561}, {4955, 3716, 3715, 
              3653, 1786}, {3690, 4624, 1519, 4623, 3689}, {3716, 6149, 2968, 
              6148, 3715}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5944288733481605, 0.5767439852100458, 0.9101965095819687], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VFtIVFEU3abOOFPM3Lkz92qlRf70IMqilzVqYFgTgiUESfQTBWE1FhQE
1U8kBFNJr7GgvvJDKSIrgiL6ys9SKuknKAt6otCD3h+txTp9bO6556yz99p7
r7Nnbetqz08ysyKsFDYRmI3DWmNmPz2zq7DtZWYtsAiAQyVme6eYdcE6gGlI
m51PmR3E5abQ7Cv+dwA7ENc58e/g7zNsEP47I2b7YQVgXkbNcsDMx3qi3Kw7
qlg3fGB9Yfch/l/crQPuI76fAt3ZmQDPSrM+k6+WmPyvi2kdx94Y7t71FW8l
1jdxdzLO2qP6kvMA1kWsTwDfAe6bYb8i4nM4plxmZMzWINaBqIy5Ma9j4LAF
+EXAfsM34eteDnW45ulugP0Q9jCieix0NcmjfnGse7E+FNM95n4dNs/x53eO
W7NGXBdL1Ic3EfmhvzrnkxybHc9OcOsBj2nl2htLaz8LjilwS0UUnzwWlMkP
1+RTD8wKWAKY8irlyRzzru/E38JeY5XwrMfvycrvD2p8zpNezkAXp2EF3F2D
7whsLdaeL37kRn97nU/e5z+5LE2anYKv3ci1Py4Mz7+A0/cK7TfhvBF2Cf+z
XV/Z0x7qNy1sG/aiOI8Cv8QXb3JmPhdK5WcrYh3PCNeLmhVhMcQcjahv1Bjz
63ea3gA/G2EzcXYHNZhWqTPWO0wpr7jjw1j3weVeWlwmgK0Hv4uIlcXeqrR6
2QQ/zwLlThzfRtG9NfpmfYhjD9nrOe7dUBtvce99IN2/SKourMlT+D7qCzcV
3KpCaY1vcXFC7+UBMD9CceX9gvP5CJhdnuqaxfkeT3qkRtlD1vO8p363uLfM
OOTMnrSE4kBt8pyc/9eU9aRGpmd0xhlTGcp3NfhXgHNukvRH7bCnzSlpiLGb
cX4F60cmfS33hRtETjcy4sBa8Z0wx1dRpwP899IP8CeBfwyeI4FqSd3lnfZ+
Y+95WjXnXOh084rzaa17p3w/9EEOc4FdHyo36q4vlPbqkopH3Dj1nhGnupTe
FTVILTQ4PXzAt9vTDNsEbG1S3G8npF1qqQb71RnNLdoJN5cLbnawd5xn5Mwz
znFqkblcrlAc1nYVOL7xdIezirOb8+pJWrphLyvxrUXsIfg660nr1OJwoFqz
ziuQy2vnhzwYkzVfBu5tvt4Xe3LT4dnbmqQ48JzvnBjyol6oDfojP/rkDOYb
JLfV8HEkoZq3luqd0U82kG5yLn7BcRgGt4mE+k7sZZc75zv7Qz/0Per8c66w
J+TzD3fQ4iY=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtIlGEUhk/qzDgjzPz/jDOaZmEEXYg0F3lJM7AsS6iEoAtuoiBKi6Ag
qDaBEEgFhWNBbcpFbiKlTddVLU2JIuhCSdAVXVRkSYuel+PiwH++95z3O9/7
vd9fve9o15ECM9tOFBGVZWbd5WazJWbDCbOmrNnblNm1YrOrxMfArDlntiJu
dpuGFvBmoqPA8YpSsypiOGYWZa2c2tf0T0fMTtHTTr6RGDLnvBma7c6Y/Y2a
jbHWS15G/pS8s9AsR/6O/uvihmM6abYT/g/wx+CvI2+m/iP1I/RfYJaLxA/y
HvAzwtNm4Ry/9h4k4gmfYQD+RvAk+En2m+L802V+FuF9nPcTfO3MvpnoIf+a
9e8E52+lt5b5zjNbBo4KemfBV4HdYo9zzDoT+KwxOJPUTzBTX8w1kdaXA6+V
5pMZvwNpf4KaCmbNsvYE7tPsmeP7YNJn01oZ+Pyca7ufPe+Dd6d8r230z4Dd
pX5wnlkX+RbyDuIAtS/pH+J7fdK1kWYP6K+m/33M+7eSL2G+38WO16NXQ+h3
30/8BP+V8bOIcwHnrYMzH3cPjKJFS8a1LWFtirwRvgTfncQjsKbAMc23Dj1a
iZq4z6xZvslfcZ/pO9/P6VlZ5GtfyD9n/e51R/KaNJNWmukF2DH4a+PO+QY8
SLuX5K8J8HEiz17LwDeAtaVdD91pIb1j4EvjfoZXOmvWZ9cdh+D50GvlsV0Z
10xaydN7+d5DXKF3dcS9rDclf8nT9WnfQ9z9EedKB+4lce4AvxF6/yLWJvge
D91PmyKupTSUdtJU3moLHZPHalLuUfGJvyHta/qW53vpf5b1s+tN665rQ59V
d65ZLoVeLw7d7R04DkX9jqtS/malpf4BI2CT9ByPeo28/ydwb2rme2n3kLyj
mrXMvrDUva43vDjlHpT39Mbz7Lc8417Vm5UWa1L+tqVJJb1TSf/XtMN5mL0e
z+mhO/vH2QYCx+SBs2l/w3q78tAovQ/n9pMH/gO2jqN/
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.6173671400712172, 0.6135993971933941, 0.9107194314124273], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1lUtsVVUUhpfifZx72/s+97YIA3Tio4mpA0qkRQeQ+BpYTdREdOBIRdpo
NFESJ1xIAFESQQJxoIBaqW2ixoliEBMHDBB8JVKjVGOiE6vRCSWa6P/l3wx2
ztpr773Wv/71OGsenb536sqIOK61QuurPGK6ETGSRVzUd1cn4tJVERcGIrZl
lt8qRbyttXBFxPhQxOO6V5Z+TG/P1yPukKF7dD5Sjjgt4xt1lkmelbylGfGk
1lQh4pT210h/SHaqyS420SGjfxCfVdt4omF/+Lq/HfGA1nIx4u5WxBqdrZK8
vqK9sB5TLF8IzwrpB6XfrTgeqRvDTt2fkv72om2DDft1rcmS7b+j7z86OywM
8wM+/1zyQ8JzX8n6XbLfkZ09ives5K0Nx0Fsr8nmndIv1SKu7tjee/J7c9d2
jsnOkcz3Z0o+x+bvuj/cMb+1dIYeLPhGzhX3Stm5S/afl+5AZhl7RzPzBn9H
kgxe3hLfD1Vziu135WdW61phndP++oRnVBie1vpUHD6mb1f+hhXTHt19sWO+
cuVwfW49PtcW7Ouk7m4RD5nsjOt8IjcX5/T9MreP2YJr5Lhy0Re25zLXCbXz
zJDPRhLP5OJn2Xtf3M1LPl8xN8THOTFxh7zfUklvUzzYvFX2Nmj9WHUeZxKv
SwVjh7d+1TL5wgY1Djb8US/EBUfUJTW7SXVTKTpG/MAZdfuw8H/c9tunhPnb
3Pj+6Mlfz35uUt5O62x/5h4BI/cX9LbYNoZh3f2pbV/4P5UwfFd3jxHjhGL6
Onc+D0k+POS6314yv9gdl69fGq61xQH3MPHdppy+qbOe5D/l52zN724sWEc9
/y39X1o7SvbHO3CSq+0J9+uqgb3CsL9g++SAvoP/fsXc8L6f+pp9P72FS3J5
OY87K65t+mNVxz3yW9m2wbO3Y17goqWYDsr3buk/FPaLerOj4pzwJS+fCPsH
NXO8Vrwd1f0DBffWdT3XEPMC3UI4H/QS+Md096OWa3qf4ns5Nxb4aRXtd0Ln
G1re9+Rrq95MYx9eu7b1kt7d0PS7Z5Wfk11zRPzkCD7PCeNMy/y80nRs2Ce+
ZpLBPtkyVubZWMszjRnwfd01DH/MMzgs6PyM9L8WPBuWUp23G55Z8AlH2KDH
X5X+X+k3625IPpObS3pmMvXYG03nm3zghx7CFzXbTnVLb9Fj5G5f03ME+8yb
iUG/4RyZecqXPXr+LRfS3IbHE/JxUHIm25W298sN/5PoT/heXffdzySvy80p
M4+aqqZZThzl5Asc+O20/R9iTg+Kq4GWOSI3i6lHwLKY8JRTrxPL5TMw8A/5
JjcfG9OsIPZR2fuv7nfjkl+ouQao/bk0K8jbUJrhl2RjOTe+1arPlR1zSf0x
g+h9+onZOZf8zKf/0WjXM4X6Xdd07fKOeb85zXxwbEoxUJfbUp3/D5B/AJ0=

              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlF2I1GUUxk+u48zs6n9n5j//2VHXwPWmDy8UygxdFUHIXEjzoiAN0RtL
3bUwSKGL1CDN0nQVpRs1yVoJNL0xRRO6UMhvwRYU1KCE2lJv3EXBfo/PXBx4
z/uc93w855x3/NKeN7uHRcQSZDjyRTViE9LHZbEQkaYRVaQ7F/F6U8S7rRFb
wH8ZZpvBLGJRPWKoJeLtYsQ48EIl4i72PUitFnGuFNELthY5AnYY+fWZiB8J
2I79dHx8U3CMHmxPE29hPmIi9ivQz4OPGkE+2F9IIs6UiY3temz+AJ9GjH3Y
fosswPd8pJ9aenPG7mCzv2ib42C3S449Bfw3fL+PXkBvJsYb4FNa/fZZ9L+J
N7st4vfmiA5yPA/WXXZtqnEVb6fXHLsFH5M4T0Z2KJ9wLYPonzW7pgOcu7jb
DJdt3D1EP5W6to3onxBvVmLsRfy/RD7HEmPKbyzc9xLzEXx/j7/D6EfAN4BP
JL86/kbXzPW8JmMXqfH5om2qvP0ys2/loN5trrrX6uHnnBdT4w/0doicvsL2
ZfQd2P+FzdHEOSmXBXnXsjUzJn/qvXra3eDnFLnsJkbXSHpB/D11cyRudNdO
vHHIwbxnYmXZHOutfFzC94XM3KqGsyXPlOoTx1fBVnOXgL9GTjOIvw0fm8il
gv4EX59yN4dzARmP7b6y+9uO/grY1+inwzP2AufOit+Koyv4H8o82zdaGr2s
OJZ6+ghsJz4Xgb9TtK/tjfjyqVy2Nvzr7hZ8HaLeCQXvkGapyN2JEZ6pAWZt
ORyP5ryBeD+DnUTm590T7dIo4g/kvFM/JZ4B9V4218hnJhzf5G3rcPdOM61Z
Vg/XY99Vce3akQ787S0bk82rvP+3zbG1783EXsabXeEcD2E/WPJsrCua+866
uVEPpvG+v9VvVUNZ+1Exl6p/C3ha8qwo5kGwB8TYSO799L+v6p3Tromjqdhf
x9/cJu98Ddss9WzLv3Z7atm7rB2fUfeMajbFgXofJc+rZuAeb/vQP9Y+YNNW
c03rGv/TY+J9UDJ36mlnZk5Uj/6o98AuZ/6b9GfI13+p/yL5fA7uLiau5bu8
ucuRw585c9ih3U7da+n6W/5JbKs/RrOpnqgXmtH7qXNWruJoJL6ysmvXDHzI
28kVz7Zm7OnfkHkW9UesgY+P6t49cTgGfgcS/x3aubdSz4RmYRAf/wMBP8ps

              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.6403054067942738, 0.6504548091767424, 0.9112423532428859], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVltoXFUU3XnMzJ1nZu6dOxkQkSoJ0tKYKlSlidr0o/lQP/rjExs7ICja
io5WRERtxWYaU1+TTgtiLUhNRS0ioj+tNj8+qhJF2uLExECtUMVH+hE/qq7l
2uDH4ZzZd+991l77cWbFlm2btnaa2SxWF9Z8t9k7abMfsNfzZnM5s/cSZuNl
s91YQWC2Bd/akN+NfbRq9lTR7EzW7NnQbLliloe8BflRnBs4xyWzHL6N4qId
OHdCfzsuOwmd7T3y34eVTpo9CvnT0P2nR+crcb4q1LfnY7MybH8Cho343Q/f
uW5hJJ6DJlkzY/YjzlMZnd/uENb3ccepDsmWctLhfh7rQ2CbBLYOx7Y3I3va
8jvP1HkMuOKKfFHGb/RH3sjZLqxMr1kWax/krYx2+vk5q9/EuBs+JrD+BtfN
tHgaxypD9iYwjOFcS8mG+qug8z3OfdgvKiAHWE3g/wucfIO1GvIaZJkI+Bzb
2awwD4O/lYit1inZ1QnJn4H+LdAfxO+d0NkRitfrIX8C6wT8H8V+W1U2lwDP
MOI9Dp2hnM7kfCt8fwlehsDbDHRPV5UTYuwOhZO2jPWg1xhj3dilnbGP4nwc
uucQ++Pw/ydwHcLvsaSwXOgVns8hm6zKNoVzEuts4n9MiU7V55BjY70ueX6b
ng/m4gA4ew3rPL6nI8VA/K8E8jXToTpbCMQbYx3y2LnqKemkYLsTOfsIssuA
fT98TsFPC3ZTgeJaAPZ5rDZsToKna6CzNxBu5or3Upcy1idjZgys2Vno3ltU
/jOB4uG9b6EXFyLh+ATYvyrI9nfI/sCqwfZlcPNxQfG+WlJeySFra8rvOpSR
X/okT/TPvmh7joi/lFM8zNMwuH8QeAaAZy6lmNjXrP05P59GjOuAe7/zdybQ
vQXYNSBfdPmiczuTFH+sE/LMWmHujgHvbKQ+GIDdFViTgfJAfphr3k9b3sve
5Xxh/zbAz51VxbISfh7AOoLzhm7lmPGSe/pbB/1F+GgkZFvGnVEkXE+WZE/b
ccgakdcy8vlcrDh4P2MlBvbB/V5L5KvlNbAJeFbHwsseY0zs6xH4OVdQLBHw
T8Tii73en1a/czYSH7F9Bp1KRXOGdofLsr2AXAeRYuE9C15vnOXv+jyfLmq+
MP/0S/+csZzd69OKkXUx5nlf5fcf8bnKGZPzfcDPb4TqSfYja4d9Rm7ZxzdU
1MusUfYMcbJ2vo1lfwx4txX1XuR9xvGuzUnNV9b/SFp88a7lrHLHHH7Xo7t5
7yPIc72qOrWi3g/GdbiofmAvsBaWY9UD71rv+F9Arq4LlXvKRlzOvnkxVO/U
vBfZL3vgZ7QiXhOhZi7riJyQI/pfA/lDBeWNfBAvOWEepj0XLwHbrVV94zv7
cF5vGHP7aVH55TuQDZWPU9D9oqS3knrUJ6ZB6O+r6g0hZ2OOMx+qVuiHbzK5
Ztw3QX4z1sXg9LdIHDE+ck6f9L8W3/tKwk9Z3eWvQ7YC+s0uvRn0wfO1Jb0T
fCMO4Hyp6wSowVSv3tI1FWElzjuA9/aq8sl3ou6x3xjqTRr0XBAv6+E++DsR
q145Gz/IqP9b/r73+/8Ryjg3voaPy3ultwF3jlTEO+fZvL/RxErumZ8J9NBd
PZqDfM+X/E3n23VPSXPpV2BYG2su/lJQ37Ke6bPtbxnrnPN0xmuGWJkjcrjZ
z+yHtvcXa2jebf+bWQnNw13+X4LzhO8T+eCbQl6ZA3I77fOXentKqmPas0b4
v4r5+hcrvFSx
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEuI1WUYxt/m3M8Z55z//8xxjkSIi5FQmmkKtPBC6cJZVAuFTIMSJxK7
KIRltOiCI84cp5EujqPgBReKShYhgpsi3QQOhZdqJCUVzBaKGC1sofV7eGbx
wvd+7/c87/V7Z63duHxDW0S8hmSRGV0RI50R5WLE4XJE//SIu7WIr0sRx5GJ
akQujWjkIhZkIp7Bfhv7ZcAf5CMG0R/uiNgJ1yn0cj2ihCzMGLOS84vIGPrj
6BX8fQHfVw9F7MZfCb3c5fM4sqABLiEOgtxDTL2cn+f9I3nj30J/M7E/+Zgk
vvlgxng7juTxdalqrOJ5Aew+vc+Y43XO6xJjxfEHvgt1v71ciPiE3AbBLENf
Qv73sT9ArsI3Ea7NUnI+UnKNUrDDyEywp8Fca5hD2GeRi1W/ke1LOF5BH6be
xaJrfpTzMWQT+Bw5v00t+5sRNyoR34Rz+w3MrqJz/A7f72JP2iNa9GB10z7E
fQ/MFnw/TX7zqdVN9Jewv9y0bSnvC+SSR8ao9S5kD/wHkH+K7lE32CfIv0S8
3XDsRz8wVT/lv7/hnqgXwmxD78Dn9aJrNIqvHU3PzjIwF7BvwN6O7x7uLmGb
RL4Ff4UctuDre3IeKLjm6m0n9TqTd3/Uu72JfSuGJ3n/Efp7Gce4mdwPcfdq
3pwN9QZ/a/B3ImffW7FPyzqGu3D/jazh/QD6CO+H0A/C/4D4Wpy3T/dZPV3C
eQX9uVJwz9vFhQxlPaOfcS52uZaa6Z/I5U7d3JPoP+K/F47hrDEb0c9Rk7kl
x6RczqJ3l5xTgt6DPlr0/7la9x/V39SMDGGrw3Fjqt4T6FEzdjYc59EXNV3r
x5DDxPczMZ0sO+de7PdT/2X5GGm4x6qvOFvoPdR0oM09zaeOSbHcRB4llhnU
Y23WnKrdmaZ7qRp21hyjYvsTWY++EPvvFefchj4v9WxtzngWNbOaVc3kKt5+
XvOsaoZV++Wd/pvqwTy4f6l6trSD5hDbpw37Usz/Yeub4ldNVPs+ZGfJPdgO
Vwu5Fv6DT4H9IXGv9McqYH+Foz/jGZpFLM+lnnfN36amd452jf7gYM07TLvr
r4pn8f2qZ08zKa7TqXelOM/VjdFbzbx6/2Hiv64ZUK3eqLmXqtk97P823EvF
/A697EOuT/1PnXeAb+V8txv8eNO7QTmrV7c6vFvUs3rdO1L7Tn9sMbEtSl07
caj3o4l7ozvtykLq3mtnflzzjtNu0476HwGO1JU=
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.6632436735173305, 0.6873102211600907, 0.9117652750733445], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVl9olWUYf9zO951zdrad831n3zmHBC3ZqrN0QoGwLiJPWRJkzhWYQUJo
wiK9U7OYmw7mXFt4Rl2old04K6HEi0EX2QwR8qZC05Do7yDaLKWcEKz6/fo9
4MXD+533fd7f8zy/58977nphx4btDWZ2DtIIuSdldiIw+3CR2c2c2RQOH4e8
UTEbh6xohh7k+dBsc0rrlyl9rw91lxj1FrMJyJK02WBiViiYHYCBQy06exR6
o9ivRmad+C6XzUqQ1bh7DPjtWKehf29Ke+/Bt9OB7vNuD2xNpWRrT8bsFH7X
GqTfCTkJ/+dziuEx4DTGZpMF+dkEO9my9N7HXlIy25U1Owpf8u5nb6hz4n2R
N/sb97uxvwzxfB3KtwXsrUxkexwYY5B/gbMb+sdxdh7+HMP5Ycgf8LEDWDX3
7Qq4/LYiji7gfC9s9yOuw21mfXnx/kTR7KFWs+vAXwK8X+kHdN6G7juQeiBf
ub4E/Vb4Pgys4UbFQd55ngbO94j3FXBxI7x9bwNs9UK+g09dwD4I/RGcLcb3
ujblnXf2QB6E/kCofM9C/6+szri/H7ZC4Fexf7wJuKHinEtJl/htge5QP41Y
tsH/PuydwN5oRjrToXJFu6wz2iPGDeCvLenbmnRn1mvvzYzqbxUwG6DXjL0z
4Gx/LJ5ZK8Ql5igwDhb1m3HXSrLL+iMv5IR5eCBWLmbAxy+QNVnxvS4Wdx8z
55As9mcCxcba6MTefYniaYEv3UXlYgj3Bguq6QB7WyPpP1JSbIyLfLzonCTA
OJu/XdezsDNswvgxEQ7jJw/0n3yv9lplnV3PqB4+CXWfd2ljwHPH+qXvrOE7
gXkk0Z2tFdUxa/gS7K9KxO816DyM/Unsv4r1tZJ8Ii/Mc4f3P/NwGZzHiPFi
3vuuWXi0OwEecmX11j5g/pToLvu9w/ui5vEwluWcG5H4rXqvcy5NeaycCZOB
5g7PdmbVx+y7OmxVysKifq9jEqfH9Sd9VhFnpCi/6fNi1M9QSfXaDh8u5FW3
7IOxjL4/g1+fRsJgvse8hrmOus5bOeXjimn90785S1gHrIEh8PBVUbNmJfDe
TZS7msdITtYEio1xBc36Js+fI18HipqxzBPzwxysAP7d8Hljg85fL0mH/m7K
y2fOmKfaNGf6I80g1nyf1zJnad3nN3toKTjZ1iqunoPdjRXNV9YiZyJrqd1z
+b/PWc0CxkEM9hxxPoKdO3D3g5xqkCvv/oNcLUAmcsrPlrTy9hts1iJxsAj3
lhbEKzngHnl4MhZ35I3vyUiinuZ7QR3y1oNY10OuAvcI1qOQCjg7i7vfgIu5
QDZ7vV6WtygW+rYPmF3Q28v3B/bnC+q3q84NZwvn/a68bC1AfxrfPwTi7Ewk
3vh2nXZ/WHdTXnucnWmfn8zfnMdFv3KxftMX+sQ+YT9xDrKnnkZ+n4HcCpW7
Gc8f32HGx/xGsB/Gyul4pFnPOc9ZMuBz6edEPUl8zlDGyvqpY+9Z8H4rp/dz
S0W9Tp9Yi/STtmiHvcmZzJ6nb4ORuCMW58GmivqLdUM86t8Pvzoi2eqO9JZU
PUbi0Bb/Fxxyzvlus2fYL12JYmNcrBvmnPM2Q/1YdbQWNl9uVY1eA0+/QwbT
qguu9J++33R/6B+/mVO+tf3OA9/gEcfnTKommkvUnfe7/wGbjjd2
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlV2IlHUUxs/OzO7s7OjM7Oy8M0OCpmnipgZm0nZRuWVJkNlG4AcZiHph
qHdpJrqisK3bqu1SkBrVRbmWYNKF4EV+dBGpUAaREpFUgt8phUKw5u/pmYsD
57zn+znnf96Jy9f1rE1FxGooAz1Wivi+HjF9bMSJdESqHJGGXkP5cxOE7nR7
xBD6d6ER7CvViDdyEc1jIl4tRIyiv9Ac0Ue8PUnEA9j83RrxFzQB/oPE/Pv5
iEeJvRX7LdjnWyI68d+MvBn5R+RH0M+CFqX8rVqMSPDfiH8337o6Ip6HbmLb
R731WkQNmku9PXybiu5L7HM5x7tN/gnk+CRsM5PYMxrxVcMl4vc28v8LLUHe
jX4hvvuRz6C7gv88+K+gd+h9ADqBfnwWTLA9jn4F/Et8Ow9e39StGwavywXH
kO8z0Cr4DD4X4VdDU5DnII9pcU3HkI+3O7dq/ohePm7g9zr6B6lvfNm1yv4p
arlKjz/Q2356XlChDugX8j+Xcm3D2L/c4hrnEOt6ybbvEbMJfif5+hvzWEHt
K6H/ct6BJ/H/UPNHfwv9PmLvgY4Qu47/XuLNLrq2m8hL4fdWrBvBZ1PBMeTb
ifwE/GAjn2b4O/4XoXnkm0K+P+G/w+azNn97hXluRT4HFnewX1gxpsJSPSr3
PuK9nXYN2rX7S65FmO3Avx96Nu2ZzUDfQ4xfyTWIflPVO6zdvSr8wOqFsmOp
58Vg8TX6L3gLd9jf0Zp7Ui/DyNvajaGw0z7pLf2RuBe9qR3whZJ3VT1/Suyf
isZ+GT47sT/Ft17Vxnv6jfgXoI3EfhPKw7dBnRnPcJB6J5I/2jyju/i+VXXt
h9GXyDWQuD5h0g8/tpFf35bjOw76B/sK8sPoZyZ+W/LXLp0k5rVm79QS6lsK
3c4bw1TJPatX7ex8cu+qu/Zr2K8v+mboVnxLzNPI0xPnehz/s8ziGPkPNrkn
3ZZzdb8V3RjNuqnuW6GZ9+F7t2as1UMr/qNl13oUOlk0BpJ1I4SlMBa2wlS7
ppuj+Qj/w8Q7BH2e940Q1toJ7YIw78b+unYu6zevtyrMVL/ebEK/G4rubT3+
29BvhyZn3FM3+qerxkI7pt3pIua0jHcoX/YN1e0Uxr3w8+l3UtYxtPtDZd8y
vYFF6IZK3j3NQLf3QMm5dYNvdPhm61ar5jUF31TdPsXcjn22w7ukmWt35vJt
JOcd0u3TzZS9bqB0neAx0GqbLPi01nzb1MND6CYnnqVspsGfAo+utDFa2e6b
/f+tTvuW6iboFuimvsjbG1f0bdINaUGfLfsWyue+uv9J+hdpRvcAgpvhww==

              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.686181940240387, 0.7241656331434388, 0.9122881969038031], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFllts1FUQxk93t9vtxd39d3fbBoKUWEEKVRMvqQ+aVH2xDd4SEy+AWq3g
iyYWjFhNStQWURJti0RAY2hrgk0sJl76ZJUXbiY+o4lQqUZJ9MEYL6EUv5/f
aXiY/M/OmTMz55tv5uyq3mfvfyYVQjgpSUuuSkLYUgmhrTqEnpoQPtTm11Uh
TEv3iSRXG8IVxRBe13ouF8J4XQgTkqOyeasxhIcLIUzqzLz2vpf0av1PJoTN
kozW/fJ5qC76zMun4u3VutIcwkct3jsv/TeJ7V9sCqGk3+cVN51yPOxHFOtE
3nE/LYUwI7lLvqfk40zJcX5Sjj9K7tDZNn3XSYaUU6YhhO3SbZNcVL5Xy35b
2ne9VO/8yeO5GufZrNxaJJPS/Vnv+5LLXNYYcMfj8r1VuGQz3p+MmNxXDuFu
yeka5zce9Udkv0ZxN6edB/l0an1O/t4W9qdUi+XK7R6d/U7nbtT6QMV7+EI3
Ij83CYc9wupL2b+i/Y6C87lBeTbnbFOqdgz8b6hx7mC4XvanZP+m7D5XLp9J
urV/QDEfiXWkdv21rt8H+t4sv+054xSEY0b6fcJmVLI2YjMe8XlNfh6Un4mU
98eiTXfEYrTKvk7Hu3AOPblxP+6A/lfV+d6y+UBN8AE398rfO83WbSm6BuDP
vcmLs/AVHbFaZfOubEaptdZPiCsp3WFBPgYr5utEvDOx0sI2JZkXftdoP58Y
y0v6LkrmtB7R2da8z4BHKmKCv1ejzz7FebLFvLuoc5tkX9TZRu1fkP/prHUv
NVlP78FHatol+8d1dqzK8QvJ5XryHZPNVvlZFM7HcubUYGJewRXuCpeG5eOs
7jyl3BoVa6NkSPpu2d9SMpcHlcuZaHNWv1+OefxbbQzxyR2pPfm0JZ4X5Fsl
/zvl8zqtv9W6U37urHWd+hrcW9QZfMH2XMV22KzUub9k/7dizKb9hT+92n+s
xTHx8VSD4w4Ip0Xl2pF2Ln1Rv6Lmsg/i8aV/sznPA7gP3juajDlnqRU8P5i4
x8Bsl2JuL5jvFeXZpPXT0n8lm1nJYa2vlJ/VBWNEHuyD5/uyf0/yR87zYFmj
sXtDumLRc/E34X174p6it3bl4nzMGHt4Sx2oGb+PM6N0fjjyGn5Pxl7oiTOK
OXok7975ouTZ+v9c1dlEZ7tkv182t+bNB+4AXmBU0n5ZsiPnu/Ylvg+8Jy/m
BvXYEGOBMXNwaQYW68xV8GqPMwcusscs6oucoSZw9oV687ZTeV6fN9c4ty72
34LiP5oYT3i6ENfPK7fpsu3g2gMl821jwXN/qcb9Mc8fSp4R4JTSflqyr94z
hjeBmTCTNRfw/3O9+Yh+Nt4ZP8zFYxXPrcNl9yR8441iNsHnqTgfuTu+8YUf
+mBZ5DmYzsRYzKGVRfcnc6QYeQRvuNOY1gPyfaHZ9aJuA7F2vBXUhroQi/vw
pjBreD+w+TjvXMmTd3K+YhvqvtQfcJfepq9ZZ7Pud2oOJ/FP3rmICTnCkfXS
P1TwfKdeK3iLK/Y1LN1Q2T3XKt2Jgv2c1Hetfu+WfjVvX8UxutSLq/LmA7P2
95LvxjtDneEF/UMO5Mn7TR/AZ3jH7IF7zIOjBWO4M/G7iJ63bSjawzvWxOJ9
4/8KfcS8/KVg/8yo9shD8trTZPv9utPBsnuvW+d6JMuznknwmz5sr/g+2FND
9qkv7/NteefToXPXNhpreLc75vYfq9E6Lg==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlNuLlWUUxtfMPsweD7P3/vb+Zu8UD+FoTToWBGJdBJY3jZgWBRlhNqZj
XRTkAc2E1BhNLVBHBWc08dRBcBSk7MbUm8n8CzIoJ0cJwS5COqr1e3zmYsF6
33V4nrXed60Hu9554e3GiOhGssimcsRG5Bz6qkxEqcQ5jfixEDGMPJBEfFWJ
6GyK2N1AXEvEJORPgidwNxm90hqxsjniL+4uFyPaiO8hdgcyHf0RpDd8dwB9
JniP5iJ+5/xGPeIzMFcTnx0TMY7zAD4FznORJ8Aei30I36F8xCdgLavbdxX2
S/h2Y8+D/Tc5p3JO4TCHIrcQMwlbQ914u6nvJfK9iPRmfPcR/u3w6Wo0xhJ8
jxPTSP5A1oGXUGMPxwHwf8F/E/YboyOeAX9c4h6pN+IwiP0M+Z9tcs/key11
LYr5lHxt4LWBXRoV0Ye+H7kc5jcI9ykj/VuLf45c/yTGPovcw3cMUiJ+KGe9
gv+6gu8uEH93xD6DfK9TT1fdtfTCcSn6cjhls67xffisRzoyjhH2d0X3Thwe
o/YN5NuA7WresR/A50TWOYqc+7HvxH4N/1P4fwPn+dS+B7xi2TWplgo+rbWI
GnKM2o8ib2HbX+WO2MfxOQ3+KWRxxn/gKfJ1Vvz2+qOv4H+OnMd5r2NIB+cV
+A8W/IY95HqVu5Xg57nrBetz7uZxbh9tbluqtoljP3pf1bnEYRqxHdSXz7vm
f4m/U/M86I2ew3c8PleI/QEpwWVG6r+uP7eE839lv416OhW82RX3TvMl7ueR
L3KuYYB8i8h3FH06OQ6BtaZofRZ8r9LfrbzZoWb3fEHVPVOvxKGO/y74Xmxw
T8+C9XXFvdcfnJl4xoWtmj5u9QxpdjST7alrEPdtYL5M7l01z7I4rS6ak7iI
w7dl7wDNvmq4Q/73yHez2f/tV/wnJu6d3vBnuKTEHx7lN9lZ8h/Q298m50Lq
eb7qXmbB64PP5JJnQX/qS2r/iRzvNjlHN/a7RdejGdLsbk+9q/T+/egHU+8W
zfjT2G+2OPdianqt7DfS22imq6lrEHfN0LKyZ06z9mbOu1B/XH9bf+Ah8UNm
47+14N03LbWuHfhHyTWqNu1Ize73RfdWM1zGN0mtq+fahZ34jM97H2nWJiTe
Zerhh/huTl2b8PfSu301/2X1RLszg/9wzjv0CP062eJZX858z0u8Y9RPYdyD
y5Mj86SZka6dpF2ku2GwrqfebfpTmh3lvJ8LvEvEP5x6ltWDi5zT1L1ST38j
362Key1OmbrfRG+xD87/A8MW1IM=
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7091202069634438, 0.7610210451267871, 0.9128111187342617], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFlltolnUcx/++27u92973fd7Tsy3Ttc3NTU3zohMZERREBzoYZMVWppDW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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwllUtsVXUQxofb3tb2Xu6555577m0I1Ja+BKKy8ZEgiQY3PuIDE1FTo7Ym
VBeVmLTUx4IgL5WoFBE0PjD4NtqSWDC6gFUBV7owrqCuNG40bkRtRX+f32KS
mf+8vpkzM6d35InN44WIGIdaoXu7IrZARzsj5lAMZhGHGhHtyN+1RayoRDya
RowVI04i765H3J9EFC/DHzoH35NH7IJ/HtoBv9SMmC5h2xIxg/9JYt7eHrGT
mCn6PHG8Kex3I5eqEXuwfZm3b5AnyXER+/6OiFn8j5P/TXR38LYH3d66+Y/A
+xC6JWgB/Sj0MLWMQMvKEQ9S4LPoBsi3BdsJciyA5VP0L6E/xdtW8vWl1s+D
5zFsXyf+GWxP8DYDP8nbfME1v49vlf58jX6SGAPE+yexr/q1HfsJaFvBNfxZ
i/gLGgHbWnwmsN1HjIGyY+7K3TP16jwxeqh3mv4N0b+DfJ+D8IehdcgfFlzr
MDZJ0TXvJdYC/esnXh9UJ1YGbcB2FjwV7K/I/W1u5O01Yh2C1pYc8yy6rfiv
pte90BnkOWoq0rvLkc8hrwL/TNhmrGof2fZBa8Ayj9wN/y35fqwak7Coxi+J
9UXmWJqBDPunG8Y2RcxrsP8M/9YO93QU3w+q/nY3Y7NI7lruWVnXYmwnoItF
YxylvtbMs3kd+sPYrsT/APxV2KyoGbOwykdYNWOaLWFW73bUnEs9vKTZRL+v
6Hn5m285w9sq+BJvy9GdSvwt9Q3Uywvkf7HsniboO3JjnQTzIPqEereRa475
r8CXoQfAPs5bDX55w/yTHd61Sm5f7Yhmsz81Fs1od8U+sv2DGEnqb6xvq/rv
xPdn8A3Ddxc968OJd0Uzfy1Yb4F+bfPMDWB/NrGv8u1E7qxatwb5PXpzDPp+
mTF/Qry86d1Rzfclvgm6BZrZZ8C3Efqtzf71xN9I8YVxtWYnN6+bcQndv6l3
VzP7VMPfXP37hRrThndOu6YeavZrFevU/w3En6p7V7Vz7+B7FPoKrKfBvL7m
G6D+tYPpJ2p7ocu9k41qG2r626hGYX8D+1eKruFK4j+Xezc1E/PI57HfXvLM
Xmi6JtWitxvw35/aVj0Wf3XNufXWQu4C9AOz/zs+t6G7FXq84Jxl3cLcvdNN
XGy6p5J1U3ULdVN1S3UTj1D7WOLbrZu1Ht31mW+RdraGfwa92+kbX0XXn3lX
/58R8h3JvSvTyEPwg7lvj2q6C/7u3PE0f4vkmyVfd4tvWr3pnqvXyrGZ2Pdk
vuXawZvA+zE+vcTaxNurTd803bIe3t4i9tu5e6EeyPbzinXyOU6upbp3TzlP
I1dzf3vNyCNdzqHYuvn6d61MPfv6h/0HjgfZhQ==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7320584736865003, 0.7978764571101354, 0.9133340405647202], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVltsVFUU3XXa23lx78yduTNFKvahg/3SRFsTMVD9EhOePghota30MQQk
LcESqdSY1ERFja1V+FESfEfq48MvQX4khsQftOVHqAlajBQ/SsTS1sS1ug7x
4+See+4+66y999r73PrOPVuevcnMLmPEMM5EZqcxjmLemjX7I2f2YJXZfNEs
njF7K2V2JG9WDswKcbOtCbPZtNmnFWY9sGsCyDD2vhmavYGx0TN7GPa/+2YP
4PvuhJ7bKs3SBTO/oLUgadaI9RPYewDPOYw2YCXAIYnh4z0DDlmM92Abw54n
gPFzhZ7b3PwwuK8Ez7/AbQXOrw2F9WiN2WMYs+C/iL0jwLhWbfakJ/+4vwrr
Q1hrwblTnjiRT29aPq6BzYvVsuFZT+P5FMYnmDfg3AvgNoxza8F3rCD+S/FJ
aS/jc8HFaAZxvZwTXk9afE7GFIP+mM79ETGbz+qs/YjhasR8DvbXMR/HfAfO
2sX4YZzHOR1478YYwN4xnPkn5pOY78K3bzz5xbhwvYyk1yJO74L3DN5HMU7B
ZgPWr3hao82XOGchL1zmYG9K+f0Ha0l8a4ZNCjjD9B82E3E9z8Dmbnw/GAmH
8fwF6x2YJ2H/UqT3BOb7MZ/EfAd8vdkXfk9c/hA/C5x1OG8t5lfxnMvr2xFP
fpdh348Y7MW4iPePPD0ZhxJiu7ZSe58Bfj5QHjbjzE0Y43HprcpT3L/A93ng
dzn8LsfhOGybM9KIH4kreTKXjAvnzG23y+8iML4K5Ecz8nwgq3PWe9INNUNe
1A+5GbS5IqO8rUYMnvcVwwx8+hDf+nDuyzg3DZspYJ7F2hDmo9DNQ8A/VlDe
tmL9cYyrwGkE/gas1eEZ4HupqFqh/ql91jj9HnSaO4RaeT3UHuqd2qVuhzzV
MXGasL7dE/fBtGqxAvxPwNdYJN/b8H1PQjXFOu1ztboMHEKMEVM+w0C+rwP/
lpw0sgDb3xx3Yux2OMfA64NQ2OwfrFuuk0drpeJ5iX0mVN5mEO/PgX8WtulI
fYR5asb8p0B6zyEeYVH7ya/NYb4Cjrf44rIdz4mitPR2UvXHXldOa844cn+H
y+m5pPYxJuwbnc6evZLzzzBvr1Ysac+6Zz5Y+68ib9PI6R3YNwi+d4XKTXVC
NsxXHbjXhKpT4pET8Xl+u+NA7HZ3Vj1wuiL1FeL8m1CdM4etLn7sZ+wR7A/M
83qXa/rAPBBnSaeO823IVSYnnBfA97VQe+7F/Osa+bLT6YfcyJNYxCFencOk
761Oi9Rrh4vlxZT28httic1zqQtqkxjv48x8UX6wBrinNfH/HUONsc9dcT2w
IatYMA4N4H5PTpq/PVCsGeePPWmH/eQQ1u731Yf7gNGPcR982IR9mzH+5v2Q
UF7pSwAuP/iql0vA3IiYH4+rPs65Golh33JfvXja1/3KuyeKC4s47B+8Q5gL
3gujLkYDcfEgB/a3fW4+5Xou++o+6H8gr3hSQ6vSikM3uCyC0/dx3e1NvrTU
ifNTkc680Qd5Z30H22WRzqgsqteQB3PbklF+id3kemvJncU5e1W2oJpnf+h1
9yb/GybcvUBtRi5u9ZHuTvqcwfNUIF978Tycl92dofLBXDCegw7zubz+V+jL
t7BPRsoT8zro/iVWFdU7WNsHwf1X2KwBt8mi7hLeI6NYeyeSXq7zfwFjHPbX
PPVUck7gexApB9Qm9c28sA/sdP8/rK1qV6e8G6fzujuIxZyxh4wDY3mN7FjH
twaqZdYXebEfMk/shcwpe0zZ4VfmdIfx/joP/qcD6YR1SK0R5wYnaoCxZ6yo
vbFI9zHvYuaLdUJtPALMLTn5yj7Afs3/DdbZkOsD/M8gf+qNPf5koD5Pv8su
DvwvYGx4D1Ir1C51suhiwjl7K/VO/FmnDfpFLkv3NPCPhqpt1jX1Mum0PeIw
eRa1VnLaZi1Rv4xDp/OZ/vKeWenLZijSHclc8r+slJPOFwL1AuqFPbvR9bz/
AHRiYOE=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlUtsVHUUxv8ynel02s6duTO3U2xBUQSaYHRBSyKmPhYSTQRBkQQFbSPF
JiABhBJRSkwwgq9oQWFDCKKuqLpxoyAbjTGiRqVupJoArWIxCgqlxcTf5zeL
k/mfOa/vPO+M7vVLn5oSQtgJ1eg3CWF/MYQveQ9nQ7gM/1BzCBfqQ1heF8Jo
IYSDcQgrMiG0NYSQbwqhEXoD/XXID/C+Afsb0yFEuRBaeLdCK1MhLCLQ07yv
j/zWfwt4by6HcKk2hHHoe/hiKYTj+B8g/nAlhFNQB7o74TuQ5cDwE++f0Xke
fHPw2TXF//2G/DDytWDpx9/DYP+G/7p4XySHEWKNQvuxbcfmW2Rp/O8Bazc6
F8htIzmcxv5ddCbRXYX/PvLrwf+ryJ/D//3IFkNbiT83cvwh5CPIdhFzNrWZ
02AsKWJ8lDGmHL4+Rn8TWPLU6G/8fxDZdxe0m9gzq/n/i34D9n3EOKl8od7I
mIW1CX4C+5qic1kNPxg5B2F/Imvb+oJ7KR9b4OsKxir+KvrzS67lZ/AfRsYk
LGug1tg9U6/GwTsT+Vz+S6O/DX4quc7H3zHkteDdhm5LbF3Jj6NfR8wNwgLF
8Pfh/y5kd4D5bD6EQ+g/mvF/0/G1B/1z6P4BldCfkbj3Pcg3UZ8VefdDPTqD
7fuRc1dPy7wnwTCM7pq0e6GZ1ayqJ29jvyDv+T6Pfkzvlzd7Nu5kAW5Hdm+1
HkfR6QPrFuhg8Ixeh//2xLOpGRbWafg/nzXmqdh3F41VPZ8Ov7vZs6AdWZx4
J2R7BP234G+G74Ufyxq7cpa9cvi95JnWLO+A3iFWQ5N3bT10lvzXwp+qc/6N
6jc+bsPfZvwtI3ZEjp013mHt9oP4u5RxDf7C/tPIupqZfeTaSYwIfwn/bcdf
f+JaCeMsfM2GVsI/gs9swTOlWXq93rb7y55N+ahHdhT/bSnnNB47pmINZjy7
V8ueNc3w5+gewSZd551fAtYd1HAIrP9kvOsvJN4F7Zx2KVudL+3UPc2+WbpV
p+FX855EZ2PKN6kFfAPV+dqL/Fp815Q8K7qB6sW+xLOnfF8C6wnkj9V65xvx
daXs3VKNX6T2u6BJ4v3IDfkK3e3E6IRfhc1JajVU8T5pZtuRnav2UznqtlyJ
XQvJfyF2MfJuyMc89Av438A7BS3FdmHJ/VAPX8H2T3T6eS+CboW/JfbsPpv2
rt4U+a2d7S+4RqrNQM5YFCNVvZfy9Vrs2yafEXjyiWujGk+Ad6zkWfzhmhAe
SHwTtOuDyGchW5L4/f/NoFaXy+6tatCB/5qKY+tm6NYPJr4dunlted9A3T7t
XJF31ORZUw0024XEt0T1V+8/UX+zxqdZCc3urWbmYuwbod14D8ol7qFsZaPb
3ZP49mmelmE/RozvyKcX+1/BdyLvXO/mv3VN1pFMO/cF7wL5HMh5x3TLRvLW
1U0rIytVfFv0zVwItpdj9+oMGN8k9t7Es6aZVG9bC74F6vHX8E9G3sXHa/0t
039665um261vrr61ulnFimdSsyhMml3NpGZRM3wMXw2Jvz3ayWfAOlHx7uoe
zEO3veR+amf0bdxa9rdZ38hpeecg7Jr5w7Fvkm6RbkIB29HIt039/A865x4o

              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7549967404095571, 0.8347318690934837, 0.9138569623951789], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFl21s3lUZxk/X9nn+ffr0ef+3z1gBOxMdETBOCyqRxRcYIQGUsA6aoXGF
QSIb9gVS2Fa2MDcIuhfHlAS2wQDHZNRPIlBgH9Yaowwl21QmI35wXVmJ4rRD
pkGvX69D/HDyP88597nPfe77uq/7frqW33n9qjkhhBUNITTqe7oWwl8rITyZ
CWGyEMIezT/U3iX5ELo1tmt9i9beK4VwsimEi5pD2KqDt0tJv/aGNSa0fiwJ
YbP2NmhvRzGE58shTEumW2uDknlGMr+Qnhcq/v0n6bu8PYQBzZs6dJ9+L2qx
LtZ69H1EY5Xml8mec7T/X8ktzlkuo7FF62+2hvCo1tZpnJG+GY31mj+m/W7t
LZBdV+V87qje+3Xp+Uw9hOON1r0i6n8sZxsPaD6WlY2y/ZTkh2T/GY1eyY/p
XTm9a4Hm32729+7gL/5jPpqGUK6GcIXWFkimpPX7NT8tnRdp3CH9T2h/j8bn
9XtI/j+I3Zr/VGvPalyu+bnxzlHpfD1jO/h9rWzYKD/8WPb16S3jui/I7p3R
J8yfUxzXys/bdNcn5IPz5IPFWeuYbrZt+OaCxP6ZanJsDzZZ3y0l68lJx0Tq
M/c0+E7sX6vxl0b7ba1kN0vufMn/utHrww0+gw7u6s/aDmzYoPcu0Rv2SNeY
fHmd5o9o3pY15sDbdt05T3q3NNqXh4vR5rxleqX6S/LZlGTGdMeg9g7oHT3S
85B0flZnNur+1+vGDbFHL+dLkp3S2ccLvvdS7c+V3D7Z16XvV0p+B/Ekrl+Q
TJI69vzuku4FBceE9/Tl7bd/a/9WyY0nXrtFY6v80CY7BqR7g2x+T34q6uzT
OpvXXW0au1sdN+7njX9o8Vpeun+bsT7eNZU478BwUfFforM79a6eon2KP9lH
jtwkF6cT+3CH7PpR6t/7O5yf5CZ+QR6dJ7RWqBpjb0h3ocN5BXZ4C29dqPUd
sn+H3nu7ZG/TmKP1zlbjG3w8KN8+oHFW79iqO3+Y+g7ygJiQC8Rsb5Nz/LjW
lur7Sa336M6lGn/LOQf5kofoK3dYJ/gCf2CMnNkSuehnefMCnEBug9eFGh9o
7VV9twW/oy++BUzyZW9TtOtlyZR0z68K5oHv6Oydie2D5wYi1xG/m3Vmns7c
GG0nvjN52wGXLNP+MY1OeEb+yUX5Q/LZQ8JBl85dLay+pPGGzk5Gf2xuMB/3
txuD18p/k4rN/sR431C3/x4uOpbEkbOn4nneAx5574B0nK7YBmwfizL4YDzm
+x2SOSGZsmRu0h2/yXqOn+FfchzZqY/4Id5Fvt8kP/2jbN7bqfm05ssajKG7
ms1d86R7ddnzb+otHy8azx/Td6hsuZGKOQu+Io8HYyzwXy76cJfsvLTqvdeq
9iM+JG7gCvnOyMvYc6H0D6eOLWc2RRnwtL3J+OnKu06AxaeVQ3trxvsXddfb
8vXuFsexM/L8Rtk4v+7zYIw8AGefKpp3sKONGpHajjTvGoMfyZtNHT4zHmMA
P5yMvsWf5PBMk+vvm4nrB/UUzoMf4b1BvftI3b/zuudV/Jg4Dxs0zguu1T+o
WHcm51qC/2+M+QZej2TtP/B2Yc68A+cs1zvmFoxn7AYH8OFC3fNw1bn2hPb3
a/5dnf1lu3kWLiGenRXHdKBiOWQ+rW9/wf6v58xN5O2zra7NF+veL5fMv9wF
1uE+cPJ2xvPRmCNd0VcHY80C5+iiL8G2ubLhmtRvnoi+xs9Tcc7Zn0tmps2+
AsuTcb0rzjdHzE/EfFkrfWtSx6Q/cgA679PaSOreh7qS0Z0Xz3EPdaLgOP5E
mFpUtjy9ED0GHFUoOH/AFfGgTuDzrGSTsn/3an+w3TmwTvNvVP3OJZqvbPd9
b8lv99eN4dfy1k8e9USO5S3vlFwvyf9dLbYDG7ql73NV+7g3MaeR+zfrTd9K
nYvvav9s2bhZ1GZ+my89zdrPpObHbGq7sfmcsmOA/w/lbRMcDCdylnvh6d35
2K/o7EtF+wA+Phg5jP19sTaSE3AxPHyD3vp+ahtYXxrz5aT8PFnzOxpl81Ml
432kZL4gti2a352aMx5ULj7Q4dr1VM35T+5v1Vq9YN3rY1+JP8m5oarzDh/B
k8SaHG3POU/JIXKJPB2Xvyc0rswZv+QhGG7WWw/XfBZ/w7noWxdrmrbD4azz
k9iVtfDn1L+fo38uuWd9Xzr+pdGXGB/LE9eXvVVz8bJY+7Aff66suPeFt+hx
7om5Nr9sboUj6U+xl3pEvwSmsfPdsjEH3ral5gJyDf3EF/+ANbgUm0d11yu6
47aM+XF9rMVn5dt7C/YZsvSeyPNe5rP8l3dfQ3xD3bgH83DxztjfMsDFbI+d
+Dx6LpGd3WXjOxPlwdsrsrdVdg8m5sqVifs6+jLm5B29y+nYn+/TvUuL7hHO
dDgfyAX+B9wV+/Dtsd8lZuhYFXXCbX9MjIcXpaMp9R69xyy+YpyJN/8v1uiu
1XXXio36fq/uugQ2+mJMh6RntPb/fv2C6CtqCXZ/5GO+vwvukw7FvKP+jxZd
w4Zj7w7GiBsy9DnPRO44EHWMRD38T7ouYgZf7or+/Kd83Ft2bfyg5ppHvVtT
Nb7BDPn3TtU5CAaxBZxg2+xc+r8v2xbXPSfmcAQ2/F4+P9rhvol8I6/IKfQ8
GvEMl8xELsJ+zmLbIeFlSveO6NytJfeynOlWHM4v+j8Q78YO7sLG6WjnEb3l
rYqxf7xqHIABOAaeg6Nm+8w2cxicOBL375X+1alj36lz8+qub2BjMGKM+kxf
S9x597kFv53/QPwn4f/IjGw4U3M+wz/8v+Xd1M+ZDtdQ8E78P4w9MDWVGvhy
0f9V+J9ymeSfbDc3Hc36DPJfle6v1cxfV+tbKXp/iXQvKro/oh5fFf8vn4p1
k3pCjGZifzsSsUK+PC+fHSu5NvJfdrBmu+DpK+uOEXhpKRozK1L3Z/Rk9HT/
KZp/MrH+gJ+81v5ec07B2UnJvI2++0rWia3DMY/+B4OxPhY=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlltslFUUhQ+9zkzbuU9nkKK2JIIxYASLIgnViIok4iUCiqgRpJAIRTrV
yK3EgCDxQpGLJIo1oIIB+mAiKiC8oBiNEgVUkBJfipiSoGBRkajfyurDTs75
995rX846+/z1M+Y/0FISQngbKUNOJUMYUxvC5mgIncjFbAjn0iEMKg1hRiSE
hkwI/yRCOMj6cEUIn2E7MRXCt/jeGWOfC2Fm0mC95SF0Yfs3GE+wrgMjxv5P
9jPxf5RvH4O9DNmOfRG89eBdAd5gdBXgHUD3E3it6J5D9uBfSYwa9EPB20g+
B2sde3hlCPNYz0VS6CqwWYztQmQF+x+JWQp2X9a1xMpdazf4b0Vd82vYrkXm
0JQz2NyYcc2q9RSyFfsJGePVEnMTeItyxn4VvB7y/QX8aRHnUGQ9Ph5CAds+
8vuuQGvyITQRq6mG+qmnBP952LdWgYV+CZKrBpue3E+s+5AOsIaDPxCsUvYP
oVtBmw+Rz+fIXeBPQCLUcgn8Ydgux6cX27NId7l9evCfSE7HyOXfAeRMvmuU
M7qrkAvU8wcyHftnwV9OLiuRevLZhr6B9W3E+BLsdezXg92WdW3q2cKCa1Du
svkL3YWs+aMz30m9Z1M+myTfZlF7PfspJeZUkVpaa/u5gk0k5R6pNzrzKPtq
fNqwHYPPRtbrkDWqlW+jsN2Q81rnsZhcDpNvS4V7uoj6x8Z9durRI+D1kN/D
/Xz8j9515Hz26ukL+BfBfDPqHjyO7rGc8z0QMbeWIscj5kQfZzqHnpTQjyvp
Xxrfu8E/Uumei+vtyNoB5nwi7hwU+11yXEG8k0n3em+ZuTkafWO5OXpDwpwX
188jO1i3gvc0eOOEnzTHxK0G/Legn4t+JPqb0b/C+mXFD/1nDl6B/UXw25DT
7If0830nNb2Yd0zFqqtyL3dh31zhnqp3tyNfVbqH41kvwX9Eib+p9hT+Ryvd
g0EFc1Zc3QFeE7qVcXOpiM9JsIv412C7Dvxx2P6adC80A8aiv04+4g/72dg2
IwMq3fNb6dXpuLk9FZ8u4k/BfgvY15LfYHSrwbwUdQ+GsS+mXPs3mjGcZTTp
u7OKb7GUOSeuDdP50o+LeZ/9vmBuf8++pcocvwf/HuJNK/UMG0i9XXy7g/37
VZ4dy9if6OfLz6zbiVdf5h7F2O9LOJZm5gJyjcd9F7eS39CUe6Daxd8+9GOJ
cRzs5+npLvDrCu7tCPA+YH1T0rE74e8o9NfHnbt6WIN+O9+qse3EZgixJ+V8
9uJ4nWY/0oX9M9g3E6+6YNsfoubO1QnrxKHd5Pdhytzvxf9E0jNcs3sBMjrp
nJSLZuAqejcJ/9ex3UAPOtm3JXxWjcRYyX513tyTje6ibKTTnSxQSxU5PcW6
D/8ycq9O+LxawIjqvSGfE+WeaXF8D+GzH368Qb+OoG/M+fx1RprFnyTsq5m8
jfN9D+nA/kr2v7M+n/VZKMbOjHuuXusOTqa2Bwu+e5vB/4JYibzXB9AP0ttY
8CzXzKogXsj5LZsf8VnOSjo3nemTrG+p9VusN7UK2xkpz/4i9os1GzJ+m3RG
x4g1Le7ZKU5eppbmnLlcLPVb/lvas0Z8LUX3acJYqjcOVn3Gd0EcXpIz58Q1
zTj1ujznXNXzqfiuJeY1xFpAD/ZguxfZVOJ/gpeItTTpt0UcV++7M7bVGWjW
6M3XW6+Z04tvFz6zWU9HfybjO6273I7PUs2qWnNdM/QSuq/j1umN01urN1mz
X2+uZsNlvu2t9IyYSq6T854152Lm/raMY+kOtOf8hshfd1RvU0PKvdN9uJda
RhbM1W6+NaHbnTa3xRFxTf8Y6r8415bxndFd0Uzbn3DP1Wu9YerdO1lzSz3U
rJ6XNrZm9mTsO/LmtmaYZlUs7X8rcVa1Tcm7FtV4Out/LL0n+gc5wv5o1rNU
MzmFbTJvLmoGNoL3Udr/Xnpj/gdNok42
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7763750371043867, 0.8515812701514754, 0.9078736836937149], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmXlwldUZxr/c7LnJTW7uqtFYtdVWxzosVSGxiFhBqRtW0TJTERUVUEig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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl2eMlWUWx5+5U+693OH2O+8VFMQNBiXRUDSCgzRRdJdqFJGogB1EZxAT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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7969751574663384, 0.858450861382541, 0.8986448482320005], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmHmQVdURxu/MY9682d6bedvFKtGgodwTBRE1KTcEiUZ0iIJo1IgbsqMm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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlnmQz3UYxz/7+9mDXb/d370aanQQUbGU0qSskEraEAljdTjW7qIpV6Uy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              "],
              VertexColors->None]}]}, 
          {RGBColor[0.81757527782829, 0.8653204526136066, 0.8894160127702861],
            EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtmHuw1VUVx3/3wbnn3nOf5x0KZCI1dlEBk8YwBMHxwSPl2WCT0CRCl/dM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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlnmQznUcx78edp/HLvvs8zy/58iZ7hxlV9EolKgppFhHqUkqV+vYNZP6
J4OGYjDVHzm61EyRJh06rBJltynJlaZDCDVEp43SqF5vb398Zr6f7+f8fs5v
hzFThkyOhRDqgWbAgLIQTrUM4ZPm4PEQumZC+CsZwrpECN8Wh9AhH8I5wBsl
IbwOrEuHUAdMQcnSohAOcz4EPAPvs8C58C4H39rEMsfQlQVvhHcNOjdHIbzA
XWgawjDuenJehkxFqXVu4/wFMAk8B8/kbAiP4dPt0D5H/lbw9qkQqmL28VP0
bQH6J6xzLeeF6Lwc/LKEeW9HZn3CMt04D4LnWngf466yPIQ+2NuFryuwmcfX
HDAQ3lr8W4quevj7wHsL9C65ED5GZg7Bi/HeN/GtkbsZyF9H/LqAXwJ8DW0C
Niqwtx0dfTlXo689tLOBGs7DuYuja0shhPPJwQzwqfA/Dv0OxQKbbchPTcq+
rEfnAc4L0HcNvnzCXS98uwq4geeXYH80/C3h6YauRdydgPdS9H2D7Lvo6wi+
CPx8eH8D7sb3McAJ7BWD1+BLLbCAengR+VH48yj66ji/rxhz3ou9YZw7I9MO
+k74B+H/fHJ+G/jfSfO+BzzP+xLoX4X9KmL2GbK12C+Nn8kZ5ybAP03tcw/8
n4uN2bwtQ0y7gM8Bn1nkmBaQzwPzsPUyeBNs1WT89nrgR84F/HlJ9QO+ldwe
gn8Utrqjbwi6DpDfxciOxHxn+DsB47D9Bza+gfcldLYAHwG+H97vgSeLLSPf
p56xpzdshH8VNq7DXk9sfAXvx9xdk3BMF8I7kpjUJRzzQbxnTd61pJy9hq+r
C45NNXfriFVdzr72UM8hPztybfUD5uBbB+gPk5/d6B+BvlJgH3i8yLW6DJ6r
E67ZVdDeyFhWMdued4+ptyTTQH6mFfx21XQ9suNTrtVZyh+0V8qc2zEtQvgd
2h/ASOiVMdfuxpxrVTVcjf0pvHcr+C/YPJF3jau25/O+adBqsj7vgGcv8dqX
9iwoheeRlGtWtaoaWIr8WPCu8F9a6t49C7x/zD2c4Hw9d1n1GndF2OrNe38k
V5sTrs322JuZcI2ugb4Ne3OKHcPjnP9Mu5ZUozuxtyPvXGgG6S3Tyzx79Cb1
zsXY/DXufKoXNXM0a9STJfAOTbl29qL/QXTfhc1T+HelZgbys4jXMWhrsNcd
/rHIH0FXmnwvwXYDd3XE+xL0DYd3GFCL7KYi98pGfCo0d8/Mxf6+lGl7Yo7F
5JxtKyYDoL3HXQ/8uwmeDch+BLRqbh2aDYvhb1HiGaFZdgx4qMQzbTH+3Ju0
L52BXrzlHvAunBvQv4fzMPS9hf7B4Lt47zvwzEX3POlD/pjmObSW6Hu23DtF
u+RfdHyZtox45wOzc55hml2K0Vf4/mLStdEM//thaz/4CPDp4O9DXwccivmN
U3lvZdazXTNWvbc0Z9vqwaOKdeTa1c7KF9xz6jXNjAOqLWRaF9nGuKRrULWn
HXWTYq2Zi66b4VkB/dvIvmlmKHZV3J1X6hjeyFtOUnf1vOU/cngy6Rmp2aid
NBj7i8pdG1/APxza8aR3h3bmc+i7MO1dk4L+Nrzn5jz7VRON8FYh82bCOaiC
1og9WvV0jZ+X88zQrFBOngJ/OufYKScHU86BYv8pd/fylgnA0bh3rnqvHT6+
WuoerIN2ijcVYt6xe8CHo2Ms9qrAf8h4Bp+evdTwndBHR+6VtcG9U5t176uH
vi73jNFsUY88AX19ZF+U04qM/yT6i+wu9i5Qz6hXtBOW8JapZdatGj7IuTf5
2lnina7YV6Lvg7hzoFm+EhtDEp7pFdA+jDy795Z4906KPLu0g7vi61H0zSr1
Du+m+MDTKmadmjXKkXKjmTMA/oHAioT/EFOwNxkYxfkCxRza/WU+b4HnInyp
znpWiecKaMtz3s3auSnw54jRUOUauA971Rnv6rYx+zIx47+bfGqb9QzX7NbM
0y4YnHKtaiesAv+ONwxNeIcrdz+l/XdTDndH3vHa7YrRBGgT065d5VyzY3XK
fz/NkBvAD6JzZcJvVuwPlLmWlYMqeH8FdhV5Z6hW9SfUX1A12xB5h2t3a2fe
CP0B+F8J1qndHoPeEPfO7UhsNhGPhc39R1mU9x9Ofzf13MTIMVZsf4a/L+fv
k/5rqadbg7fJevZqh2h3a0ZqNmqHHyx3zBXrtcC4yD2hXtCM7pTzzNOskw9P
6++bNq9mtv7S3eHfGPef+oj2F/ryRa4hzY7LgA1xzxDFRjtcu1sxUm5eTrqX
lKPLI9eoalMyPSLXgHIvG9KtP6L+hrIxPvIfSn8n9bBmtWa4Zrdm9g7o2yP/
fRTT9vRe24JnuXpcf+37IsdOb15e7pmtWa0a/Ana4ci7Vn8q5aoy495Uzv4H
SOGwvw==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8381753981902417, 0.8721900438446721, 0.8801871773085718], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmHuQ1lUZx38se3n3/r7se1lkTVDHqRgWKrNAICCcnAFiuWUNFNqI3Bpc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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlnmQ13MYxz9t7f5+u+3u7/r+jnRIJUfEEKPtoMKYId0M5ZxSYbIydBik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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8587755185521934, 0.8790596350757377, 0.8709583418468574], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmGmQlNUVhm9Pz8L0zPT0TDf9ddBUCWMwaMSNVIIpy2WopBQ1JRh/RGJc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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwllllslVUQx09ve1u4t+3tXfi+K2LCJgpIkASXYhCwJAYoaAB9oaAsFQoC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              "],
              VertexColors->None]}]}, 
          {RGBColor[0.879375638914145, 0.8859292263068033, 0.861729506385143],
            EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmnlw1dUVx38vIYnJ27I83ntqFe10gSASOwOkMy6xVq1LlULHQUEUBQlu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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3mMVVUSxk9v7/Wi973u9/rep6MSlyiySGvUbjOguIuAAiYkDg00CAoq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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8999757592760966, 0.8927988175378688, 0.8525006709234287], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmnlwldUZxr97CSS5N3cJXG6ioyyVAOIGCioI1gpBdoKSRNmCIK3jWsVd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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwllnuQjnUUx3+72PXua5/3WV7P+9YfZMaQWyi3wXYdIdcRYcOahmGmGXJt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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.9205758796380483, 0.8996684087689344, 0.8432718354617144], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNl9lvVVUUxnd76W17L4feFuy99AkwaiLGW6ZojNCWRGZChGqgDWD8A1SU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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwllElIlXEUxW8Oqd/L955D33NpCwsysEEJopnSLEtSKwfUqH1mqe1aBS0T
Gmyn+VJaZUFRWtKwiAoiGoTWRqlooRZquel3O4sL3333/M89d3prTrXVnEkx
syNYGrY7btaYa1a00ux+xOxmzGwoYXY03WwOYHrU7Cx+JrFq8KfzzLoLzEpW
mV3izYMcs+VsswuQ9fJmL+8bwM8HZr+xTLAbyNGXatYFZhL8BHZuhd40Eusm
x0f4J8Fc4X039p3c7+B7E5pdJn4IroNYFf5h7GGW2TgcjeSqAr+a97+IL2VL
k2txDtc6y2+U+F9zHfl2gP8A/iv5Rnnfh7+copq34J/E/sI1A/8Y+Mfw9RNL
pultLfHpQBznIe7IVawTfCv4beRMgl+A/wRa34bSPoIF+Wav8Sv5vp5BHHxT
XLVvpZ8H4G+DPyOinu8iVgjfPb4r0D9ILzrJtx1sFxytcLWEmlUSjk/EM8lx
Fe4SbDNcG10v2l6Cz8rXDH12jilLqGav9VWgWZQm9O0zmYF7OlQtHYFyD0T1
7RqGc7QjvhteczNv91HDEvFF7Ecoza61E5tjFvVgZgPNsBzspoS0XaSGZ9T6
vEC1+szaid2OqrdZEfWmMqZv75HnGolrtzxna0KczuUzvIvfgz+Lfwt96/HH
our9MPE6/M9R7dJMoF16lCOs75Tv8p6Ydst32mfXk6fe+gwjaA0K1Pt2/FK+
a9AznqqbWou/QM2F1PIUvhaKqiA+yvc64jfgniLf+3RpLAJfHhf2SUSz9xv1
2/Qd8N4twvciSz1siGuHfHcmyNkLXzX13EnVTvfz3ZTQLfrN1YItK9Duu8Yk
8alQ2ufxj8d1g3573/AHiRUn1KtrGbqF+rhy+U34bR6LC+s36rexM6Ze+o14
bZ7Dub3GIfh+htoF11gf6kb8NrymL2Bb4PgTaGeL4W7OVa/8P2QglCbX4j31
XnrPvFfe0/0x7aDvnnP8A7oTudI=
              "],
              VertexColors->None]}]}}, {{}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4160, 4698, 4697, 6196, 4178, 6195, 4179, 6197, 3737, 
              6301, 4180, 3901, 5971, 3902, 5972, 3796, 5708, 5709, 3569, 
              5976, 3575, 5720, 5719, 5834, 5833, 3719, 4181, 6324, 4182, 
              3983, 3984, 6289, 3576, 4532, 3579, 6290, 4283, 4584, 4583, 
              3746, 5979, 3906, 5977, 3905, 5978, 3916, 4535, 3911, 6062, 
              3991, 6061, 3990, 4585, 3747, 5865, 5866, 6155, 6154, 5801, 
              5727, 5726, 5725, 3992, 6063, 3838, 6077, 4000, 6076, 3999, 
              3663, 5807, 5870, 5869, 3750, 4725, 3749, 5868, 5867, 4191, 
              6202, 3914, 4537, 3589, 6000, 3912, 5999, 3913, 6001, 3748, 
              4723, 4724, 4310, 6331, 4309, 4536, 4184, 6352, 4185, 4183, 
              4187, 4186, 3728, 5940, 5941, 5939, 5984, 5983, 3910, 5937, 
              5938, 5936, 3800, 5982, 3909, 5981, 3908, 4282, 6329, 4281, 
              6152, 4140, 6151, 4139, 4701, 4702, 4167, 6181, 4166, 6182, 
              3574, 3656, 5797, 5798, 5931, 5930, 3797, 5130, 5832, 5015, 
              6310, 3795, 5128, 5129, 4963, 6288, 3568, 3654, 6335, 6336, 
              6294, 4159, 6176, 4160}],
             RowBox[{"-", "7"}]],
            Annotation[#, -7, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3570, 5711, 5710, 4273, 6248, 3966, 4564, 5713, 5712, 
              6052, 6051, 3833, 4964, 5301, 5302, 3967, 5162, 3834, 4965, 
              5662, 6337, 3655, 5663, 4566, 3968, 6053, 3969, 6180, 4165, 
              6179, 4164, 5722, 5721, 5932, 3798, 6344, 5835, 3720, 6300, 
              3907, 3739, 4789, 3740, 6303, 4304, 5016, 3721, 4703, 6153, 
              4141, 5542, 4142, 5543, 4284, 6183, 4168, 5837, 5838, 3724, 
              4704, 4705, 6270, 6271, 6269, 6273, 6272, 4305, 4307, 6330, 
              4306, 5246, 3586, 4826, 4825, 5135, 5134, 5678, 4706, 3726, 
              5842, 5841, 4169, 6184, 3805, 4413, 6295, 6311, 3839, 5918, 
              5919, 3787, 5244, 5245, 5243, 5809, 5808, 5810, 3664, 3585, 
              5733, 5732, 4001, 4237, 6325, 4236, 3751, 6304, 3920, 4301, 
              4192, 6305, 3752, 4726, 4727, 4312, 6274, 4311, 3667, 3590, 
              6186, 6187, 6185, 6189, 6188, 4144, 6162, 4145, 5813, 5814, 
              4341, 4340, 6192, 4172, 5847, 5848, 3729, 4707, 5544, 5141, 
              5142, 5138, 5140, 5139, 5247, 3604, 6291, 4287, 4286, 5687, 
              5688, 5684, 5686, 5685, 4721, 4710, 3732, 5024, 5023, 5562, 
              4173, 5033, 3743, 6200, 4189, 6198, 4188, 6199, 4709, 3731, 
              5022, 4308, 5862, 5863, 3742, 4783, 6255, 6002, 6003, 5849, 
              3730, 5949, 3810, 5947, 3809, 5948, 4170, 6190, 4171, 6191, 
              4018, 5748, 5749, 5750, 4589, 3592, 3670, 4991, 4990, 3846, 
              5189, 5735, 4983, 4982, 4981, 3843, 6070, 6071, 6069, 3997, 
              4586, 3996, 6075, 3998, 6072, 6074, 6073}],
             RowBox[{"-", "10.5`"}]],
            Annotation[#, -10.5, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{3738, 6302, 3915, 3904, 5242, 5124, 5125, 5123, 5127, 
              5126, 5161, 5121, 5122, 4813, 4814, 3567, 5241, 3900, 3965, 
              6316, 3964}],
             RowBox[{"-", "10.5`"}]],
            Annotation[#, -10.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3600, 4590, 6103, 5373, 5374, 5963, 5962, 3854, 6253, 
              6254, 5757, 5758, 5759, 4608, 5766, 5765, 6113, 4046, 6348, 
              5850, 5851, 5852, 4711, 4712, 4190, 4800, 4801, 4370, 3684, 
              6201, 5567, 5248, 5249, 5953, 5952, 3815, 6007, 6008, 6004, 
              6006, 6005, 3621, 6010, 6011, 6009, 6013, 6012, 3816, 4877, 
              4995, 4996, 3685, 6115, 6116, 6114, 6118, 6117, 3863, 6105, 
              6106, 5760, 5761, 3605, 3678, 5820, 5821, 5398, 5400, 5399, 
              3855, 5375, 4019, 4852, 5751, 4358, 4357, 4796, 4797, 4445, 
              4444, 3593, 5817, 5816, 6163, 5815, 5887, 5886, 3770, 4238, 
              6221, 6222, 5943, 3806, 5944, 3771, 5888, 4218, 6211, 6212, 
              5885, 3768, 4746, 3769, 5065, 5066, 4984, 5961, 5811, 5812, 
              4414, 6046, 3952, 5293, 3953, 4253, 6327, 4252, 6159, 4143, 
              6158, 6161, 6160, 4004, 6089, 4005, 5333, 3844, 5320, 5321, 
              4827, 4973, 6333, 3587, 3665, 6079, 6080, 6078, 6082, 6081, 
              3840, 6064, 6065, 5728, 5729, 3582, 5804, 5803, 5802, 6345, 
              3801, 5942, 3802, 5993, 5994, 5990, 5992, 5991, 6018, 5986, 
              5987, 5985, 5989, 5988, 3799, 5723, 5724, 3580, 5980, 3577, 
              3657, 5800, 5799, 4138, 4700, 4699, 3722, 6055, 3973, 6054, 
              3972, 4569, 6250, 4274, 6249, 6252, 6251, 3835, 5163, 5165, 
              5164, 3571, 4565, 4567}],
             RowBox[{"-", "14"}]],
            Annotation[#, -14, "Tooltip"]& ], 
           TagBox[
            TooltipBox[LineBox[{3921, 4541, 4540, 4812, 4811}],
             RowBox[{"-", "14"}]],
            Annotation[#, -14, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3970, 4570, 4568, 5715, 5714, 4815, 4275, 5664, 3971, 
              5666, 4277, 4817, 4818, 4819, 4574, 4575, 3986, 6056, 3985, 
              6057, 4581, 6058, 5311, 5312, 5166, 5168, 5167, 5315, 4428, 
              4429, 3993, 6066, 6067, 5806, 5805, 3660, 3583, 5178, 5179, 
              4974, 4975, 6085, 6086, 6083, 4002, 6084, 4432, 5739, 5738, 
              5737, 6094, 6093, 5335, 4985, 5184, 5183, 5334, 3668, 3591, 
              5736, 4829, 6090, 6092, 6091, 6318, 6317, 5920, 5092, 4254, 
              5639, 4839, 4838, 5137, 5136, 5190, 5270, 3753, 5039, 5038, 
              5250, 3922, 6312, 5040, 5874, 5873, 3923, 6028, 6165, 6164, 
              6166, 5946, 3808, 5945, 3807, 4410, 3781, 4798, 4330, 6283, 
              4332, 5871, 5872, 3754, 4728, 4708, 4600, 4601, 4809, 4808, 
              4020, 4449, 3856, 5402, 4035, 5401, 6108, 6107, 3679, 3606, 
              5195, 5196, 4997, 4999, 6121, 6122, 6119, 4048, 6120, 4463, 
              5779, 5778, 5777, 6128, 6127, 5441, 6362, 6366, 6368, 6367, 
              5204, 6363, 6364, 6341, 6340, 5440, 3691, 3622, 6334, 5003, 
              4893, 4064, 4470, 4469, 3630, 5213, 5214, 5210, 5212, 5211, 
              3628, 4628, 4892, 4891, 4890, 4293, 4626, 4625, 5672, 4290, 
              5671, 4291, 5673, 4047, 5670, 4289, 5669, 4288, 6256, 4609, 
              4876, 4875}],
             RowBox[{"-", "17.5`"}]],
            Annotation[#, -17.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3620, 4617, 6257, 6258, 5674, 4292, 5675, 4062, 5461, 
              4084, 5460, 4083, 4629, 4063, 5462, 4086, 4909, 4910, 3629, 
              4486, 3638, 5222, 5223, 5221, 5225, 5224, 3639, 4487, 4488, 
              4087, 4911, 5784, 5785, 3696, 5463, 5215, 5216, 5006, 5464, 
              5786, 6293, 5787, 6132, 6131, 5788, 5789, 4471, 5444, 4066, 
              5443, 4067, 5445, 4375, 5442, 4065, 4894, 4895, 4896, 4464, 
              5426, 5427, 5197, 3865, 4998, 3864, 5403, 5404, 4866, 5762, 
              5763, 3680, 4365, 4036, 4863, 4864, 4865, 5569, 5568, 5571, 
              4527, 4528, 4795, 4794, 3898, 6043, 3943, 6042, 3942, 6174, 
              6175, 6173, 4156, 6323, 4157, 3940, 6040, 3941, 6041, 3782, 
              4765, 4766, 4325, 6332, 4324, 3601, 4256, 6328, 4257, 4255, 
              5921, 5922, 5923, 4769, 3788, 5093, 5094, 6168, 6167, 5819, 
              5818, 6347, 4013, 6095, 5352, 5354, 5353, 3847, 4986, 3848, 
              5191, 5742, 3594, 4435, 5338, 4007, 5337, 4008, 5339, 4351, 
              5336, 4006, 4830, 4831, 4832, 4433, 5323, 4003, 5322, 5325, 
              5324, 4978, 4976, 5181, 5180, 3584, 3661, 6068, 3994, 5317, 
              3995, 4431, 4430, 5316, 5172, 5173, 5169, 5314, 5313, 6059, 
              4427, 5308, 5309, 5307, 3987, 5310, 3977, 4580, 4822, 4821, 
              4279, 5668, 3974, 5665, 4276, 4816, 5716, 3572, 4571, 4572}],
             RowBox[{"-", "21"}]],
            Annotation[#, -21, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3975, 4576, 4573, 5718, 5717, 4820, 4278, 5667, 3976, 
              4579, 3981, 5305, 3982, 5306, 3837, 4713, 4714, 4350, 3658, 
              3578, 5170, 5171, 4969, 4970, 3659, 3581, 5174, 5175, 4971, 
              4972, 3662, 5731, 5730, 4824, 5319, 5318, 3841, 4977, 3842, 
              5182, 5330, 5329, 4434, 4835, 4834, 4833, 4009, 5340, 4352, 
              5343, 4011, 5341, 4010, 5342, 4438, 4842, 4841, 4840, 5356, 
              5355, 4353, 4593, 4594, 4437, 4436, 4591, 3671, 3595, 5743, 
              6097, 6096, 6100, 6099, 6098, 6320, 6319, 5924, 5095, 4258, 
              5640, 4854, 4853, 5144, 5143, 5192, 5271, 3755, 5042, 5041, 
              5251, 6030, 6029, 5043, 5876, 5875, 3924, 6031, 4146, 5951, 
              3812, 5950, 3811, 6223, 4239, 5083, 5084, 4729, 4193, 4313, 
              6306, 3756, 5570, 5548, 5406, 6355, 5407, 5405, 4037, 4456, 
              3866, 5425, 4049, 5424, 6124, 6123, 3686, 3612, 4465, 4466, 
              4631, 4630, 4376, 5451, 5452, 5449, 4068, 5450, 4472, 4914, 
              4913, 4912, 4088, 5465, 4383, 5468, 4090, 5466, 4089, 5467, 
              4489, 3640, 5793, 5230, 3882, 5007, 3881, 5490, 5491, 5489, 
              6135, 4103, 6349, 5825, 3704, 5826, 5488, 5010, 5229, 5228, 
              5487, 3703, 6299, 5486, 5009, 5227, 5226, 5485, 3702, 4390, 
              4719, 4718, 3880, 5484, 4101, 4926, 5792, 4647, 4085, 5677, 
              4295, 5676, 4294, 6259, 4627, 4908, 4907}],
             RowBox[{"-", "24.5`"}]],
            Annotation[#, -24.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3978, 4582, 4577, 4578, 3980, 5303, 3979, 5304, 4163, 
              5559, 4411, 4412, 4302, 5017, 5018, 5019, 4715, 3723, 5836, 
              5020, 5933, 5934, 5935, 5131, 4823, 5995, 5996, 5839, 5840, 
              3725, 3917, 6019, 6020, 5958, 5960, 5959, 4339, 5997, 5998, 
              5843, 5844, 3727, 6021, 5176, 5177, 5133, 3804, 5132, 3803, 
              5560, 5561, 5021, 5845, 5846, 5327, 5328, 5326, 6088, 6087, 
              3666, 3588, 5185, 5186, 4987, 4989, 5347, 5348, 5344, 5346, 
              5345, 4441, 3597, 4845, 4844, 5360, 5359, 4354, 4597, 4598, 
              4440, 4439, 4592, 3672, 3596, 5744, 4843, 5358, 5357, 3851, 
              5377, 4023, 6104, 4022, 5546, 5547, 5545, 4147, 5642, 4259, 
              5641, 5925, 4554, 3954, 4805, 4806, 4553, 6357, 4552, 6338, 
              4992, 3675, 5752, 4994, 4993, 4855, 4021, 6226, 6227, 6224, 
              4240, 6225, 3772, 5890, 5889, 6045, 6044, 6264, 5598, 5599, 
              5067, 5891, 3773, 4731, 4732, 4220, 5597, 4219, 6213, 3681, 
              3607, 4457, 4458, 4194, 4730, 4371, 4372, 5767, 4879, 4050, 
              5428, 3870, 5447, 5448, 5446, 6130, 6129, 3692, 3623, 4473, 
              4474, 4651, 4650, 4384, 5475, 5476, 5472, 5474, 5473, 4490, 
              3642, 4934, 4933, 6137, 6136, 5493, 5011, 5232, 5231, 5492, 
              3705, 3641, 5794, 4932, 5029, 5028, 5861, 6321, 5860, 5030, 
              5566, 5565, 5564, 5159, 3832, 5160, 5234, 5233, 6027, 3736, 
              5859, 5858, 6017, 6016, 3831, 5969, 5970, 5966, 5968, 5967, 
              6026, 3735, 5857, 5856, 6015, 6014, 3829, 5157, 3830, 5158, 
              5855, 3734, 4720, 5854, 5853, 5027, 4177, 4349, 3651, 5563, 
              4174, 4929, 4928, 4927, 4102, 4649, 4648, 4931, 4930}],
             RowBox[{"-", "28"}]],
            Annotation[#, -28, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3744, 5864, 3919, 4539, 4538, 4426, 5829, 5828, 5014, 
              3889, 6024, 6025, 6022, 3918, 6023, 3733, 6194, 4175, 6193, 
              4176, 4717, 4716, 5026, 5025}],
             RowBox[{"-", "31.5`"}]],
            Annotation[#, -31.5, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{3988, 6060, 3989, 4534, 4533, 3573, 4968, 4967, 4966, 
              3836, 5974, 5975, 5973, 3903, 3718, 6178, 4161, 6177, 4162, 
              6265, 6266, 6268, 6267, 4303, 4781, 4280}],
             RowBox[{"-", "31.5`"}]],
            Annotation[#, -31.5, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4285, 4782, 4587, 3741, 5031, 5032, 6157, 6156, 5734, 
              4980, 6356, 4979, 4828, 5332, 5331, 3845, 4988, 5188, 5187, 
              5349, 4443, 3599, 4848, 4847, 5366, 5365, 4355, 4599, 3849, 
              4442, 4596, 4595, 3673, 3598, 5745, 4846, 5362, 5361, 3852, 
              5380, 4025, 5378, 4024, 4360, 4359, 5383, 4026, 5376, 5382, 
              5381, 5643, 3608, 6032, 3925, 5252, 3926, 5294, 3956, 6047, 
              3955, 6277, 6278, 6275, 4316, 6276, 4328, 5693, 4329, 5694, 
              4315, 4799, 4314, 4790, 4415, 5286, 5287, 5285, 5284, 6033, 
              3927, 6313, 5085, 3783, 5902, 6353, 4197, 6203, 5877, 5878, 
              4733, 3757, 4734, 4735, 4792, 4791, 4405, 4406, 4317, 5689, 
              4318, 5690, 4196, 5572, 4195, 5573, 4618, 4467, 3624, 4897, 
              4878, 4069, 5453, 4377, 4378, 5780, 4899, 4070, 5454, 3875, 
              5470, 5471, 5469, 6134, 6133, 3697, 3631, 4915, 4491, 3883, 
              4667, 4391, 5500, 5501, 5497, 5499, 5498, 4504, 3652, 5235, 
              5827, 5236, 5012, 3890, 5521, 5522, 5520, 6143, 4122, 6350, 
              5830, 5831, 6171, 6172, 5037, 5036, 3745, 4722, 5035, 5034}],
             RowBox[{"-", "31.5`"}]],
            Annotation[#, -31.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4012, 4588, 3669, 5741, 5740, 4837, 5351, 5350, 3850, 
              5367, 5368, 5363, 6102, 6101, 3674, 5747, 5746, 4850, 4014, 
              5369, 3853, 5386, 4028, 5384, 4027, 4362, 4361, 5388, 4030, 
              5379, 4029, 5387, 4450, 3609, 4867, 4260, 5097, 5099, 5100, 
              4602, 3789, 5096, 4416, 3814, 4770, 3813, 5194, 3858, 5193, 
              3857, 5600, 4222, 5068, 5069, 4747, 4221, 4750, 4751, 4336, 
              4749, 4748, 6215, 3613, 5769, 5768, 5905, 5904, 3784, 5903, 
              6229, 6228, 5623, 6214, 4223, 6354, 5892, 4150, 6322, 4151, 
              6169, 5550, 4881, 4051, 5429, 3871, 4632, 3872, 4468, 4636, 
              4635, 4475, 4917, 4916, 4898, 4091, 5477, 4385, 4386, 5790, 
              4919, 4092, 5478, 3884, 5496, 4105, 5494, 4104, 6138, 3706, 
              3643, 5795, 4936, 5503, 5502, 3891, 5525, 5526, 5523, 4123, 
              6144, 3712, 4945, 4944}],
             RowBox[{"-", "35"}]],
            Annotation[#, -35, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4125, 5524, 4126, 5528, 4397, 5527, 4124, 4935, 4946, 
              4947, 4505, 5509, 4107, 5495, 4108, 5510, 4392, 5506, 4106, 
              4918, 4937, 3644, 4492, 4655, 4656, 4476, 3877, 4652, 3876, 
              5456, 5457, 4902, 5781, 4380, 4379, 5455, 4071, 4880, 4900, 
              5770, 3625, 5574, 5578, 5238, 5239, 5237, 3899, 5240, 5893, 
              5072, 3945, 5273, 4158, 5272, 3944, 5070, 5071, 6230, 6231, 
              5906, 5907, 5908, 4767, 4768, 4327, 6281, 4326, 6282, 3614, 
              5650, 5645, 5646, 5644, 4261, 4772, 4771, 5927, 5926, 5647, 
              5649, 6309, 5648, 4148, 5098, 4149, 5549, 4038, 6109, 6110, 
              5410, 5409, 3859, 4603, 3860, 4451, 4868, 3610, 4452, 5390, 
              5391, 5385, 4032, 5392, 4363, 5389, 4031, 4849, 4856, 4857, 
              4446, 5371, 4016, 5364, 4017, 5372, 4356, 5370, 4015, 4836, 
              4851}],
             RowBox[{"-", "38.5`"}]],
            Annotation[#, -38.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3603, 5754, 4860, 5396, 5395, 4364, 4606, 4607, 4448, 
              4447, 4604, 3676, 3602, 5753, 4859, 5394, 5393, 3861, 5413, 
              4040, 5411, 4039, 4367, 4366, 5416, 4041, 5408, 5415, 5414, 
              5651, 3616, 6034, 3928, 5253, 3929, 5295, 3958, 6048, 3957, 
              6280, 4320, 6279, 4319, 4736, 4198, 6243, 4262, 6240, 6242, 
              6241, 4417, 5289, 3758, 6307, 5288, 6036, 6035, 6039, 6038, 
              6037, 5086, 6314, 5909, 5087, 6315, 3932, 5254, 5255, 5049, 
              3760, 4529, 3759, 5047, 5048, 5044, 5575, 5045, 5046, 4633, 
              4634, 4200, 5576, 4199, 5577, 4637, 4477, 3632, 4920, 4901, 
              4093, 5479, 4387, 4388, 5791, 4922, 4094, 5480, 3885, 5507, 
              5508, 5504, 6140, 6139, 3707, 3645, 4506, 4507, 4683, 4682, 
              4398, 5533, 5534, 5532, 6147, 6146}],
             RowBox[{"-", "42"}]],
            Annotation[#, -42, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4033, 4605, 3677, 5756, 5755, 4862, 4034, 5397, 3862, 
              4453, 4869, 4858, 4042, 5417, 4368, 5420, 4043, 5412, 5419, 
              5418, 5697, 3617, 5773, 5774, 6245, 6244, 4263, 4611, 4610, 
              5102, 5101, 4418, 3818, 4556, 3817, 5200, 5201, 5198, 3867, 
              5199, 3946, 5274, 5276, 5275, 3931, 4752, 3930, 4542, 4544, 
              4543, 3687, 3615, 4342, 4241, 5910, 5911, 5912, 4754, 3785, 
              5604, 5605, 5602, 4226, 5603, 6351, 4152, 6170, 5551, 5553, 
              5552, 4903, 4480, 4072, 4638, 3878, 4653, 4654, 4482, 4481, 
              4661, 4493, 3646, 4938, 4921, 4109, 5513, 4393, 5515, 4112, 
              5505, 4111, 4508, 3892, 5531, 4128, 5529, 4127, 6145, 3713, 
              4949, 4948}],
             RowBox[{"-", "45.5`"}]],
            Annotation[#, -45.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4131, 5530, 5536, 5535, 4399, 4685, 4686, 4510, 4509, 
              3647, 3708, 6141, 6142, 5511, 4110, 5514, 3886, 4494, 4097, 
              4483, 4098, 4662, 4389, 4658, 4659, 4479, 5579, 4478, 3626, 
              3693, 6216, 4224, 5601, 4225, 4740, 4739, 3775, 5073, 4228, 
              5897, 5898, 5899, 4755, 3776, 4753, 5896, 5895, 5894, 4227, 
              4409, 3774, 6233, 4242, 6232, 6235, 6234, 4052, 4882, 5001, 
              5002, 5771, 5772, 5000, 6339, 6361, 6360, 4555, 6359, 3959, 
              4774, 3960, 5653, 4265, 5652, 4264, 4619, 4054, 4802, 4331, 
              5696, 4333, 5430, 4053, 4773, 4810, 6286, 6287, 5704, 4337, 
              5699, 5698, 5203, 5202, 5421, 6112, 6111, 4871, 5764, 3611, 
              3682, 4612, 4454, 4455, 4615, 4614, 4369, 5422, 5423, 4861, 
              4872}],
             RowBox[{"-", "49"}]],
            Annotation[#, -49, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4044, 4613, 3683, 4874, 4873, 4462, 4045, 4460, 4459, 
              4884, 4883, 4870, 4057, 5435, 4373, 4620, 4621, 4297, 5654, 
              4296, 5679, 5681, 5680, 6297, 6296, 5822, 6261, 6260, 4056, 
              5431, 4055, 5432, 4545, 3790, 5103, 4266, 5104, 5105, 5106, 
              4775, 3791, 5928, 5145, 3820, 5954, 3819, 5290, 5291, 5088, 
              5913, 5260, 3934, 5257, 5259, 5258, 6236, 5624, 5625, 5586, 
              6205, 4208, 5587, 4206, 4741, 5880, 5055, 4207, 5585, 4201, 
              6204, 4202, 5580, 4096, 4737, 4095, 4657, 3698, 3633, 4495, 
              4496, 4670, 4669, 4394, 5517, 4116, 5512, 4115, 5516, 4511, 
              4512, 4130, 4513, 4129, 4684, 3714, 4951, 4950}],
             RowBox[{"-", "52.5`"}]],
            Annotation[#, -52.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3618, 4887, 4461, 3869, 4616, 3868, 5436, 5437, 5433, 
              6126, 6125, 3688, 4886, 4885, 5658, 4058, 5656, 5657, 5655, 
              4267, 4344, 4343, 5256, 3933, 5050, 5052, 3762, 5296, 3761, 
              5583, 5584, 5582, 4205, 5692, 4323, 5691, 4322, 4408, 4407, 
              4793, 4321, 4785, 4784, 5682, 5148, 5149, 5146, 3821, 5147, 
              4244, 5627, 4245, 5628, 4209, 5588, 6207, 6206, 3763, 4660, 
              3764, 5879, 5054, 4203, 5581, 4099, 4663, 4497, 4498, 4114, 
              4502, 4113, 4668, 3709, 3648, 5796, 4940, 5519, 5518, 3893, 
              4687, 3894, 4514, 4689, 4688}],
             RowBox[{"-", "56"}]],
            Annotation[#, -56, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4059, 4624, 3690, 5775, 4374, 4061, 5434, 4060, 5439, 
              5438, 5702, 5701, 5783, 5782, 4905, 5706, 5705, 4268, 5695, 
              5107, 5108, 4622, 3792, 4419, 4420, 4560, 4559, 3822, 5205, 
              3874, 5051, 3873, 5610, 5611, 5074, 5075, 4738, 4204, 5609, 
              4229, 5606, 5608, 5607, 5617, 3634, 4924, 4923, 5150, 5956, 
              5955, 5089, 5915, 6237, 4243, 6326, 5914, 5626, 5079, 5614, 
              5613, 4153, 5053, 4154, 5554, 4925, 4501, 4665, 4664, 3887, 
              4671, 3888, 4503, 4676, 4675, 4515, 4953, 4952, 4939, 4132, 
              4400, 6149, 3716, 4955, 4954}],
             RowBox[{"-", "59.5`"}]],
            Annotation[#, -59.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3653, 3715, 6148, 6150, 5537, 5540, 5539, 3895, 4677, 
              4120, 4519, 4941, 4396, 4395, 4673, 4118, 4500, 5589, 4499, 
              3635, 3699, 6217, 6218, 5612, 4230, 4744, 6308, 4743, 5901, 
              5900, 5080, 5616, 5615, 5683, 5277, 5278, 5076, 5077, 5078, 
              4246, 5629, 4247, 5630, 4073, 4904, 5005, 6292, 3627, 3694, 
              5004, 6342, 6365, 4557, 6358, 4558, 4777, 4776, 3961, 4561, 
              5111, 5109, 4269, 5659, 4076, 4803, 4804, 4334, 5700, 6285, 
              6284, 4074, 4807, 4075, 4639, 4338, 5707, 4335, 5703, 5208, 
              5209, 5206, 5207, 4889, 5776, 3619, 3689, 4623}],
             RowBox[{"-", "63"}]],
            Annotation[#, -63, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4134, 5538, 4135, 5541, 4401, 4690, 4691, 4518, 4957, 
              4516, 3649, 3710, 4672, 4117, 4742, 4119, 5590, 4211, 6208, 
              4210, 5591, 4214, 5064, 5884, 4745, 4213, 5593, 4215, 6209, 
              5592, 5632, 5631, 6238, 5264, 5265, 5263, 3937, 5266, 5916, 
              5090, 3951, 5292, 3823, 5957, 3824, 5151, 5929, 3794, 4778, 
              5115, 5114, 5113, 4270, 5112, 3793, 4546, 5458, 4077, 5110, 
              4078, 6262, 6263, 5824, 6298, 4788, 4300, 4786, 4298, 4641, 
              4299, 4644, 4643, 4381, 5459, 4080, 4888, 4906}],
             RowBox[{"-", "66.5`"}]],
            Annotation[#, -66.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4079, 4640, 3695, 5660, 4382, 4082, 4787, 4081, 4642, 
              4271, 4779, 6343, 5823, 4345, 3935, 5056, 5058, 5059, 5298, 
              5297, 3765, 5881, 5269, 3938, 5267, 3939, 4550, 4549, 3767, 
              5063, 5062, 5262, 5261, 5555, 5155, 5156, 5152, 5154, 5153, 
              6239, 4250, 5635, 5637, 5636, 4217, 5595, 4216, 6210, 3766, 
              4674, 5883, 5882, 5061, 4212, 4679, 4678, 4520, 4958, 4956, 
              4517, 4133, 4402, 3717, 4960, 4959}],
             RowBox[{"-", "70"}]],
            Annotation[#, -70, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3637, 6246, 6247, 5661, 4272, 4646, 4645, 5117, 5116, 
              4424, 3826, 4562, 3825, 5218, 5219, 5057, 3879, 5217, 3947, 
              5268, 3948, 5279, 3936, 4757, 4758, 4548, 4547, 4551, 3700, 
              3636, 5634, 4248, 5633, 4249, 4762, 4761, 3786, 5917, 5618, 
              4233, 5594, 4232, 5556, 4155, 5060, 5558, 5557, 4942, 4523, 
              4121, 4680, 3896, 4692, 4693, 4525, 4524, 4695}],
             RowBox[{"-", "73.5`"}]],
            Annotation[#, -73.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4136, 4526, 4137, 4696, 4403, 4694, 3897, 4681, 4522, 
              4943, 3650, 3711, 5013, 4231, 5081, 5082, 3778, 4531, 3779, 
              5621, 5622, 5620, 4235, 4764, 4763, 3780, 4759, 4760, 4234, 
              5619, 6220, 6219, 3777, 5638, 5482, 5483, 5481, 4100, 5008, 
              3701, 4421, 6049, 4422, 3962, 4423, 5118, 5119, 5300, 5299, 
              4484, 4485, 4425, 3828, 4780, 4346}],
             RowBox[{"-", "77"}]],
            Annotation[#, -77, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3827, 6378, 6390, 6395, 5965, 5964, 5220, 6383, 5120, 
              6372, 4563, 6394, 6393, 6382, 3963, 6398, 6399, 6050, 6375, 
              4348, 4347, 6389, 4251, 6381, 5091, 6370, 4666, 6371, 5280, 
              5281, 6397, 6396, 3949, 6346, 6391, 6392, 6385, 3950, 6384, 
              5283, 5282, 6374, 4756, 6373, 4530, 6377, 5596, 6386, 6387, 
              4962, 4961, 4521, 6376, 4404, 6388, 6369, 6380, 6379}],
             RowBox[{"-", "80.5`"}]],
            
            Annotation[#, -80.5, 
             "Tooltip"]& ]}, {}, {}, {}, {}, {}, {}, {}, {}, {}}}],
       AspectRatio->1,
       BaseStyle->{FontFamily -> "Times", FontSize -> 12},
       DisplayFunction:>Identity,
       Frame->True,
       PlotLabel->FormBox[
        "\"\[Mu]=10 \[Alpha]=0.10 \[Epsilon]=10 PIN=0.05\"", TraditionalForm],
       
       PlotRange->{{0, 50}, {0, 50}},
       PlotRangeClipping->True,
       PlotRangePadding->{
         Scaled[0.02], 
         Scaled[0.02]}], {192., -192.}, 
      ImageScaled[{0.5, 0.5}], {360., 360.}], InsetBox[
      GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyMfXVglMfzd3AvBCsJQYMFAsRJiNzc5S4XCkECFKcFWty1uAf30gLFimuR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        "], {{{}, {}, {}, {}, {}, {}, {}, {}, 
          {RGBColor[
           0.41451304602822947`, 0.2679594589691, 0.6996285476667962], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{3597, 3593, 207}, {1522, 1995, 689}, {3609, 3606, 
              2868}, {2868, 3380, 194}, {3599, 3596, 2451}, {3572, 3565, 
              689}, {2035, 1538, 700}, {1995, 2166, 689}, {3794, 5234, 
              1995}, {4150, 5147, 874}, {3910, 4544, 594}, {3571, 3595, 
              1397}, {4149, 5227, 874}, {3596, 3598, 2451}, {3610, 3590, 
              2409}, {1095, 2409, 688}, {3589, 3610, 2409}, {4188, 4664, 
              912}, {3567, 3611, 3380}, {3587, 3559, 2035}, {4837, 4510, 
              1397}, {3584, 3576, 953}, {4002, 4619, 700}, {2166, 1999, 
              689}, {5398, 4002, 700}, {4910, 4447, 517}, {1998, 1997, 429}, {
              4545, 3795, 429}, {3578, 3585, 1890}, {3561, 3599, 2451}, {1536,
               2868, 194}, {3600, 3603, 594}, {5149, 4291, 1062}, {3590, 3592,
               2409}, {953, 1999, 429}, {3607, 3566, 699}, {5229, 5231, 
              1062}, {1890, 874, 194}, {5232, 4673, 1522}, {3601, 3607, 
              699}, {3566, 3608, 699}, {4691, 4000, 1536}, {953, 1997, 688}, {
              3594, 3581, 1902}, {1062, 2451, 207}, {3738, 4542, 194}, {5237, 
              5236, 1998}, {4516, 3793, 1404}, {3573, 3600, 594}, {1404, 1095,
               688}, {3579, 3578, 1890}, {2827, 2107, 594}, {3575, 3584, 
              953}, {1997, 953, 429}, {5998, 5999, 2761}, {2474, 1404, 688}, {
              4664, 4189, 912}, {3605, 3604, 2827}, {1536, 1539, 699}, {3795, 
              4886, 429}, {1999, 2761, 429}, {3793, 5741, 1404}, {3612, 3567, 
              3380}, {1999, 953, 689}, {5227, 4150, 874}, {3556, 3583, 
              1397}, {4658, 3690, 207}, {5399, 5400, 2179}, {1997, 2474, 
              688}, {3577, 3602, 1538}, {3581, 3580, 1902}, {3595, 3556, 
              1397}, {3585, 3557, 1890}, {3564, 3572, 689}, {3591, 3563, 
              688}, {2868, 1536, 699}, {4834, 4835, 1658}, {3690, 3744, 
              207}, {1538, 2179, 700}, {4291, 5229, 1062}, {3846, 4910, 
              517}, {1539, 1538, 699}, {4312, 5604, 1095}},
              VertexColors->None], 
             PolygonBox[{{5400, 5398, 700, 2179}, {5604, 4149, 874, 1095}, {
              5147, 3738, 194, 874}, {3555, 3579, 1890, 194}, {4001, 5399, 
              2179, 1538}, {5338, 3910, 594, 2107}, {4693, 4001, 1538, 
              1539}, {3586, 3587, 2035, 700}, {5234, 3992, 2166, 1995}, {3992,
               5238, 1999, 2166}, {3744, 5149, 1062, 207}, {4000, 4693, 1539, 
              1536}, {3563, 3575, 953, 688}, {4293, 4516, 1404, 2474}, {4189, 
              4837, 1397, 912}, {3598, 3597, 207, 2451}, {3570, 3561, 2451, 
              1062}, {3565, 3562, 1522, 689}, {3559, 3577, 1538, 2035}, {3576,
               3564, 689, 953}, {4619, 5338, 2107, 700}, {5231, 5232, 1522, 
              1062}, {5238, 5998, 2761, 1999}, {5999, 4545, 429, 2761}, {4510,
               4834, 1658, 1397}, {5121, 4658, 207, 1902}, {4542, 4691, 1536, 
              194}, {3560, 3573, 594, 517}, {3611, 3555, 194, 3380}, {4673, 
              3794, 1995, 1522}, {5236, 5235, 1997, 1998}, {3604, 3588, 2107, 
              2827}, {3602, 3601, 699, 1538}, {3569, 3571, 1397, 1658}, {3562,
               3570, 1062, 1522}, {4886, 5237, 1998, 429}, {3608, 3609, 2868, 
              699}, {3603, 3605, 2827, 594}, {3583, 3582, 912, 1397}, {3557, 
              3568, 874, 1890}, {5741, 4312, 1095, 1404}, {3593, 3594, 1902, 
              207}, {3568, 3558, 1095, 874}, {3606, 3612, 3380, 2868}, {4544, 
              3846, 517, 594}, {3588, 3586, 700, 2107}, {3558, 3589, 2409, 
              1095}, {3592, 3591, 688, 2409}, {5235, 4293, 2474, 1997}},
              VertexColors->None], 
             PolygonBox[{{4835, 4659, 3574, 3569, 1658}, {4659, 5121, 1902, 
              3580, 3574}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.44588202472007354`, 0.3225016412509203, 0.7437214420179455], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1U71rFHEQnYu3uzkP3M3e3p4gFoEUSecHZ2EnqJ2FmphYCImFGMGcSUwa
Uwkhf4BGxUIihGiIac5CUDBG/MZClOAXKPhRiCkUktS+xxuL5Tc7v/m9eTNv
pv1k48hQi5mdxrcJ31hmllfMFkOzi61mlwOz9wWzJoLWNpuFfq67/SoyO1A0
u4OYiZrZhUz/U7nZwBazvYG+JeCdgv9banajDTberiRmg7nid8P/Hd/vQP/M
QczjuH+CuNsls3FgX4vFpVY1uwn7fovOnlR2IRRn2s/AbQfeLQBnGr53XssX
xPbF4vAX2G8q4jYOuxc5FtGI7ThnwH8KGJ8RvwffaiC8B45ZR/yPqnzMP+N8
JvFmJ3yHYX9AzNlcb4ZD1ce87N+G97CbPYsVfxQ4D9GfS4hbxtlAT68EwmM9
jAkR+6hNNRGj3Coc1jDtNbLWeln2J+CXU/leRNKMvaXGr2F34t3zSPisi//M
Scwu2B1F+efBYbake77t9DtqSO2pG/H4lnfUnHzI6yM4FGPVtQTuVzPpVMik
PfsyDPtMLn5BrDjGUHvypi6/oEkzVV0vI/lZC3NyLsh5IVQ95LwrVQ7iMzfn
gxqNwnc+U30/EXMCOk3grgf5j+Wqc38sbdhz1kNfx/9+hMLnnDIvOawA51Ai
ne/hzCqKow7cJXJmPbTJYR33g6inP5KGPMk/x0xtrSpuAzHt3CXczaIfTxNx
boLXWkXvuBMjmXgQY941qqeaXc7twVhzwBkgxqjX3uVasqahmnaMON3IuZzI
z76s+txyHwZci3OIbWTS4W6qXaLWb1HTvlQ94ZyzR5yNfp811nirJCziXEf8
tkx705urd+wbd+6r8+eO/nF86jjpu8B9HUuEw1kgLu3H8PUBa873Zs738R9q
N7JX
              "],
              VertexColors->None], 
             PolygonBox[{{5228, 5226, 1990, 1991}, {4289, 5719, 1707, 1060}, {
              4336, 5767, 1338, 1126}, {5604, 4312, 2491, 2352}, {6039, 6040, 
              2806, 2805}, {4189, 4664, 1516, 1660}, {5147, 4150, 2353, 
              1925}, {3793, 4516, 1065, 427}, {3844, 4692, 1537, 1427}, {3795,
               4545, 1430, 1996}, {5271, 3817, 471, 1428}, {4664, 4188, 1659, 
              1516}, {5741, 3793, 427, 2491}, {4292, 5230, 1992, 2472}, {5236,
               5237, 1709, 1065}, {4673, 5232, 1993, 428}, {4910, 3846, 1429, 
              1727}, {5738, 4309, 1093, 1994}, {5229, 4291, 2471, 1992}, {
              4619, 4002, 2869, 1491}, {5227, 4149, 2352, 1990}, {4692, 3845, 
              1428, 1537}, {4835, 4834, 614, 1515}, {3690, 4658, 1515, 
              1889}, {5718, 5716, 1060, 2468}, {5230, 5233, 1994, 1992}, {
              5722, 5717, 2468, 2473}, {4213, 5721, 2472, 941}, {4660, 5110, 
              1889, 1515}, {5108, 5109, 1888, 2059}, {4447, 4910, 1727, 
              1338}, {4542, 3738, 1925, 1427}, {3744, 3690, 1889, 1927}, {
              4884, 5228, 1991, 1707}, {4150, 5227, 1990, 2353}, {3909, 5337, 
              1427, 593}, {5720, 3908, 593, 2470}, {5999, 5998, 690, 1430}, {
              5400, 5399, 698, 2178}, {4886, 3795, 1996, 1709}, {4544, 3910, 
              1491, 1429}, {5234, 3794, 428, 690}, {4000, 4691, 1427, 698}, {
              4786, 4787, 1126, 471}, {4002, 5398, 2178, 2869}, {5226, 4290, 
              2470, 1990}, {3928, 4821, 1649, 614}, {6040, 5298, 2059, 
              2806}, {3689, 5722, 2473, 1888}, {4173, 4660, 1515, 1649}, {
              4831, 4830, 614, 1516}, {5665, 4214, 941, 1889}, {5231, 5229, 
              1992, 1993}, {4510, 4837, 1660, 614}, {5243, 6039, 2805, 
              1093}, {4836, 4661, 1516, 1659}, {4291, 5149, 1927, 2471}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.4772510034119176, 0.3770438235327406, 0.7878143363690949], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1Uz1Mk1EUvdrS9iOm/dp+/XBlUFZN/IkOglHZXBSwOqiBiEEbjDHo4OKg
MmgItlacJcQNFYlFDQPExMQ/FmqjJrq46CIxyuw5Odfh5r13333nnnvveZ2D
F46MbjSzEVgCNheZ3e8wa6XNthfMtsF+tpnN43w6Y5ZEcBO+ck7+Y7HZUmg2
E5j9wnod5zNJs5N5nbkfR+x6pPc9uH+TNbu7wawCX1Q024KYOnyzwJ0Hhxbe
nUdcL/zdWPfDbgcyxpZg1U1md2CT8L3D28HNwADmU2DMwQZSZjuxfod9AeZL
YCwAdwLxy1jPRcJqoK6z8F1LCG84EE4f3iVzqvcTYg9hvxf8F3BfQ1wD62qg
PKxxT0Z4rKsaiPsy9m05xwLOR+R9FOnuVFo9ZkwfuA3ApgNZ3WtkPvadb+/h
/DqlO/qYj7nIo+p9SHjckqkW+rtwToDDbKT7lYL6xRpZK+vnfhfmsLuoHO+z
iqO/yVrSmgvxao45hH4fiJX3IPDLsThRL6uuGd6RB3Geo/ZKrN5w/sWS+JfB
a6hDPNoz4k+e7A9X1nIRMaOR8q6D446C5sX+tdrVw2HPxRhiv3WNrQWqibi0
uveHGp9ynX+mnsFvDHFPkOdxJC5f4f8Gu5LSbKk/YpLLq1C5JlyXJcde9Nld
xn1nLG0xz9VIuai7imvvOO7HQnGf8dnz7TT+zta8sLi+iLW/ifVGLN3xf1BP
1FI/8I5G0gVn2ONzvIQePMxK/+OIXws1z1s+d/aw1/vNHnK21Dfr+guMP0XN
gXPk3BjT5N+JFdeN+wd51c1+sdfsMfl2Of8PiFkpSkMn4B/Jqv+L3jvWW3Ot
Usv7EPMsFEfOtuF5+c9+p/XX6Jt0/3+tcP8D2FOR4vpj5WMu6uVwqHrJlX2m
/x9LjLJ4
              "],
              VertexColors->None], 
             PolygonBox[{{4514, 4515, 1213, 1173}, {6012, 5737, 2487, 2775}, {
              5720, 4290, 1061, 2105}, {3844, 5337, 2106, 516}, {5987, 3789, 
              405, 1975}, {5071, 3733, 180, 1862}, {5699, 4281, 2450, 2448}, {
              5768, 5070, 1862, 2518}, {4445, 4446, 1125, 516}, {5740, 4311, 
              1094, 2450}, {5799, 4918, 1735, 2544}, {4786, 3817, 2034, 
              1622}, {4213, 5207, 1975, 2406}, {4928, 4927, 1173, 1735}, {
              4391, 5819, 2446, 1213}, {4884, 4279, 1045, 1061}, {5294, 5297, 
              2058, 515}, {4337, 4787, 1622, 2518}, {5204, 3908, 2105, 
              1972}, {3787, 5205, 1972, 404}, {5271, 4543, 1125, 2034}, {3689,
               5109, 1887, 193}, {5718, 5717, 193, 2467}, {6242, 6243, 2448, 
              405}, {5697, 5696, 404, 2444}, {5072, 4337, 2518, 1862}, {5296, 
              5799, 2544, 2058}, {3681, 5767, 2517, 180}, {5366, 4445, 516, 
              2144}, {4173, 4821, 1514, 896}, {4279, 5719, 2469, 1045}, {4827,
               4661, 219, 1653}, {5764, 5766, 2516, 1125}, {5738, 5233, 1063, 
              2488}, {3928, 4830, 1655, 1514}, {4278, 5208, 1975, 2443}, {
              4292, 5721, 2406, 1063}, {3751, 4826, 1653, 219}, {5207, 4214, 
              2407, 1975}, {4543, 3845, 516, 1125}, {5765, 4129, 1622, 
              2516}, {5203, 3975, 2144, 1972}, {3909, 5204, 1972, 2106}, {
              4820, 4172, 896, 1514}, {5665, 5110, 896, 2407}, {4280, 5695, 
              2444, 2446}, {4172, 5694, 2443, 896}, {4289, 5716, 2467, 
              2469}, {4129, 5768, 2518, 1622}, {5737, 4309, 2488, 2487}, {
              5108, 5298, 2007, 1887}, {4185, 4657, 1514, 1655}, {5295, 3843, 
              515, 1094}, {5243, 6012, 2775, 2007}, {4336, 5072, 1862, 2517}},
              
              VertexColors->None], 
             PolygonBox[{{4663, 4185, 1655, 4831, 4662}, {4184, 4663, 4662, 
              4827, 1653}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5086199821037617, 0.4315860058145608, 0.8319072307202442], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VTtMVFEQnV3gsY//vn37WAoN2JjYqI3GRjBaKZVIAhiNfAoLUQtjARYS
UFGjBARsBBI1Eo0RiIGwgoW1WUA0MTFS2FGR2KgxATyHcy1u7t13Z86cmTl3
tqbtyunLcTO7j5WHla0wOx6ZZfLNarHP43eEc6NndqEAO4zK0mZPys1uJMwS
WNMlZh9iZgdKdZ4A4AJ8HvjyvQ2cPqxV/K7DfizSXVNoNgf8EDbN2A8HZg+B
3waM3kKzt+SFOLlKYVbCPhPqbgb7m1DxG7F/LDObxfdUyizAeuWJy2xc3PYB
+yJsloA5Bsw/oe6IS4zHMe2+O3+D/STsryHnbXDtxj6K7wtF4s78xuG7mC98
3vNMmwZXqzPI5QdwxpLCaUbMFqx58BzFvg78nzh3JeRLvGewfZoU3gncHcE+
hu/94Pw7lM1R1K8e9brqKedsXDZ/YZv2dc663zyvoC8bvupfi9rUYS3BN4f4
7ajtInLaD55ngTmQp9i8J6fWEvWDvWjP4Iy1XIz8wWEveBp8J4CTC1Rb8voS
iBv7RF/m0QGs1570cBI2K7Dp9NR72tKXPfv/rRo2m0nVkD1ZL1Zf7qAODaHy
ysN5KtT9wUAaon6IwZyIczOhXrEe7EFnoXLk+bon+zjqEytVjp99nVmrU8Cs
D1RHaon6pmaYD+vC3uzgFynGLOInUduXwN1VZfY8EnZ5Wpqjb9bXPXOh3psc
Z8ZtdXVmLGqR2vwOm7VAXCeBNx1Kb9XA312l3rKum06TIWKlsXoS0gb5Mfdh
9OhFJPxV+HxyObMvW750S33dLZfGiNXjfGlHfvSdA+agr/kwA17TgWbDoMuL
39m3XEq9Y5wVV8+hAsVjLMbcdnHJn/GoDdpRq5wz3Qn50CbmOPOeb4Y+xOI8
KKuU1ojZ4XDiyKM3kj97sOrypXaZA3nWoH57sNYc9prjWhdo7rD+G+jPSChc
an3LzbTtjHpMbL5fciVP1iDr6kPN8sxe94NLTYXyOo/6nEtKP9TEuJtRtH3n
fHn/K5TNEPZHoXrxFRiXItn1wWYq0Kw5lJI+iDOCXi+n9J2aYu+Iwzl6z+VF
npxD5Ma5OOFm407tPfHk++9yeQ2Heg98Cy0VmhcDbla3ujfO/wPaMx/meisS
DrXZ497hUkq14HfOKvrxLb9Pqmfs1z/3qetg
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtk0tI1GEUxa/jODMm8/7POG5SWxTUxjZFmzRyJa0ywwoitF1ZQdEiWxTa
EyoyrUVZ0JMifBCKk7ZoWzo+ioLIRTtXRpuIoOx3ui4u3O8+zn2c+9V3HN99
LGRmLUgYOZI3+5wye11pVkRiNWbJnNkYQQ1xs6P4i/gLYY+ZQJ/EZmvM8rKh
r0+breD7iwxlzAYSZocqzFojZu/Qm4i5hm+SnBL+bNbsBb6A/Mf4aqm5QK0Q
sha9Crxl4ud47yK+lpo53r/BvJs0e4O/D707ZjaLf4YaU2B3gbkTvM4C9irv
WdhPsM3HvUY25z2pl3Pk7w3MmvHfBH88avaT9yVqDDL/Nmr84n2Z9/2Q7+Qe
ehyMM+SejXlv08zTHPUeh4n/hG2J+pX4e8Be5n047D13Vpu1Bz67et6BvzHv
u9SOLqBfRBbQ28rNetG/k/+h0m0J8vvp/xUU9KgmtYvE/MH3kvlvgd0f+K6K
9HuFfhM571UzbEg7Z+JKHGoW7VS71EzvE74j7WYM/DT4+8CbQH8Ofg5fX+DY
52PO1SgctEWcs/34GnnfKfOcDPktKedGnMvXhDwIecwI8aNIDKy3ZT7rwbTv
XjMP4LuN/Ij6Ducyzpm4EuZW8LdkPXcq7LvXTeuWxYFu8xHSXeE3+pH8BqTE
+CfIn074DQlLmLNglbK+e93wAfC+4T+FfgNbR8Fz/sdWOffVgXOhG3iG7yTS
FfWch9RuFGbEexBXQc53J85GwO5Ney3t8Cn+lYL/BXF8Nel/Un9ROxsmvo6e
9qz2p9uux78Y9xtfh15X439HtlZ6awucS920bn0s5Vzq5rWLzcj1ct/JJvSN
q/uRbSbjf1b3KtsgvX7lfTri8y0l/AZ1e+JoHOzted+tctp5f1ndn/AWM15D
ujCass6huNOOhui1POl/RzfxD4nLqJY=
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.5399889607956058, 0.48612818809638114`, 0.8760001250713937], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1lUtsVHUUxg/T6WPa0pm5986dzkKg7tRQF7rQxFeMC1AXBozE14KWBkl4
7DQkAhpd0C6ASKkLcKfGRBFNqtIFxEQXLhwoPkCTutCNNaGMuHJB1O+X7+/i
5t77v+d855zvfOfciakD2/ZXIuIDXX26nhqIyPsjTlYj7hmJODQU8ZvOp3Sf
XR+xU/cl3Y+m5+khP3+5LuJ0PWJ9K2JRgMdGbY/vw1nEc0XETeF+o+tpxdgi
m0rFtvgeaUTU5HteSRzU9z9kN6NvH1d8Bs4l4fVqzu0tXVm/cSbbEQ8qRk/v
u4bsg/0+ne8p/L6lGXGlsM/3sn1C8Q4MOMe5VAv5tYW7V/Z3K//79H5N76+V
ERtkv015zCn+8Zrthjp6r5ujn4S5o+6cH8ocm7h3KO6jecSyYt2p87t0rfb7
G7kSC/9diecXBx0jrzq/1cQD32eS/RHlc135XNT3S8Lr6to7YG7mq+bnMdk8
UxjnXuVwoXQtO6v2+3Gd+SIvML9VjhuFub3PdU9X3Z/fpYGrwxGf15z7Lyn/
nvK8MWicPMWFH3rU1bVWc2/RB/09Pmqe0QXf0Rg9hNftuX3rLXMKH9iARS3Y
dtPz5XQOPvfLSRPgopOzitkS3uqY8lbced3H1IsdqmEl5Ubc8+MRX4y7zx+q
ptsa1v6b4uqTzLrapFxu79jvjM7+aVq/a6r7eqodvngm/03COJdZ3zNJT3AL
53B1NfGw9P/s6PzPQfO8Ir93mu4184QNnFMbucMvPd4vzMN91l8v4e9Wn18q
XPuC7qcK48Lj5Ii1Myn8nzPP1VbFOZulHsi21XJ+he5LpfP8SN+/Kz0vC7Kv
lI7NGbPE+bx8TxbWwqeyH2maWzDoATicbej4PM+tV+pAi+iM2k/VXQP5Py4O
s9w1snPYQczFr/J7vuHdgN6KYesBrtAYfKLrtdSX+4V5OvWROVjOPAtzaXZ5
PqHz10v370LT+wvNPpJ5Z91Mc9CtuYdj4me27njsgF7aF8wseORMz5gXNH9I
2IdL+0+oph+U/+KAc8YOm82Ze0NfXpXtWsOx4Ga55vjoGn3Dw3SaYbhlXp/M
rEF0dWLUuMS5WDoW+2+h7n3D/GEHV9jyDOdfq66vdBXyHW55FxMTTR1s22Yx
9Yyc4Z5ZgX8wjiWcpYRPDszTC7ntrqVdgu+zwmzmqQf91hHaILfZtB/Q7pWa
feCS2Pj+Jb+ybb657x7zM7ORp//CA5n/B/QGnOnEFVx2E5/UiT0ziy/88cxu
WEm7pV14J/KNvKbSfj4qLv8u/M7M0Av09lnDOxcdsJs359bjezrb2DHmDZ2/
Ubq2f7V73i8938zoTPoH8P88k3SILokP/iuyf3nc+xi8d0tjTnQcg2f20620
o7Yq1kqao7ebnmHm9z9S2h/g
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtI1FEUxk/TaI7Wf2b+M6PjotB2ReaiFgUZIRSZEaGBpBVpSC1Ed5WV
9lyULTJTW6S0iTZlD7ByY4tq0WZIiazAwFYZZPZY1KKo38dxceCee8/5vvO8
5S0dde0RM9uFRJE1odnPlNmBJWazeWadJWYzWbPepWYjGN4qNlteavZqmdlc
zOx32mw4brauyKylwG1P4PM45j43A7MjgeO15zn29aSfdVcN3iY459Gv4FPJ
uTFhdjzf7DN318DvQ75iO49M4/8D2c+5e7FzxzNmXQUewyxcg9h/431qkdk4
+GPEVEksb0nwPvgX4N9OLnvgyIUeo+JpQ69F70871xv8I/jXJJxLNs/heoak
iDWNDMJfiD7G+z0we9Bz5L+2yGM6i/8ZpI73L2DGsB2Iey3lI6xtxJMudMy/
nMsSHttL+KrVC+zbsJ/g7jLnLcT4nbdW8Iex/4jeyVsrNgHcWeLvJ9cGMM/B
3YXkwJ5ADgZ+p/MGMCLgjYMxim8EOV/sdzpPYtONfgppjjrGDFwVofdGPRpQ
bxJuq5p/4G069Fg0P+85v1vQ5ZNLeY7KbSv2O0OvuWpdTz0a4H6N3rGQzwpm
rQyZZt76iCHEP5XyXqknTQmfGWGrBvW8vch4b1Rz1aIk7diqyR+4mwLnVgxD
8AXYHyrwGlSBtSrptdZMPkB/GDq3ZuY2tegPvLbzMZ/t1aHnqnke4XwXCdFT
yCTcG+Puq7sbvA2Fzq27k/SrnJhH4Zsq9N05xrx+KvIdepL1nqlXmmHNwubQ
Z00zodm5g74732eoNuE7rHhUw0b496oGUZ+ZjGqT8d3QTj0ln/VJ3w3FUEEs
V7Gf0xv6SmpfXuq7rh5o9/9lPXf9AUnsH8GZijrHL3wvxv0vaEb28V6T9Foo
Ru2GZlizqx05mvUaKHflPAfW6WLn1swdJvcq9J6Y77zO+jP0V+juUtw5xaUe
KvbetO+acpigFjsW9lcx/geNBLIf
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.5693623571995244, 0.5364690685914929, 0.9096250702197047], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNVUloVFEQ7Jksk0lM/syfzCTxpBCJ0XgRDwouuKDEBTGCHhIl5JBFRQ8G
xbgkikdBxSWKXgQFVwRBFFwQk4mJihoXUDEQVBQUweUYtyqrFQ+P//5/3dXd
1fX6j23aULc+amYnsXKwViTMqkKzPXhJF5pVYI3LM+vD80Tc7EzELBvTPgLH
Gtgugk8n7C+Vm01MmlWPwjvsx+PZn2v2KwXgUrN18LuKcwvM9gKzHX5l8Nla
ZNZRJPsI/NqKzXphexqxzgF/PnwSsP8I++vYNwLzGp7vSsyewuYu8O9hVeYp
12X5WMinH1if85TzeeBcCHU2Bb43M2a5OO/Gt/eOcwh5xdNmGwvwDX7r81XX
ZuTYUWb2zDlg/cS8jP0u7Ocin3mwTcJnFux7YsI4C5ta5BxFrMWw3QcO9mM1
wf4I4hbjbDb8WlHvnTzhFCL+YeTRDv8B+PVjFSPuIOwfh8ppaUx4DVHFaY0r
FvNizcxtcoH6x5yyeN7x73OANwPrFtpwF3iLA+E8wX5loLrJ9UiuODwXVT3E
zykUZ8yT/enxHjWCvyvg6BIw56IP9Ulpg9oh3+zJrozsaPOvTznqObXCvqdd
a8z5AWq6H1ev64DxMC7eapHnoxLFJS/X85Ur+5DjuY1BrAnIYQD77RnFZtwu
PHdmhNULnpcj1yH4d3l8cjICv6mB+nEVNd3ICPcN7IdS4r4WNg3Yv4XvNNgO
lopvaoBP5h8JxAV5+AL7ICFt1GV0xu/U9e20+OYa8lrI2VvX8ALYHA2FyTtD
TlgL9ZH1vlD79a7/1cCek5LuY8A+GKiHvX4XeB+pB76Tn4tRnRNnLPy2Qufz
csVrpfeIXFDr5LYkoXpYC+8ptUt7csQeMDfe75qU8qXNGr/P7G2l93fE+8p7
x5ibyoVDDPoRh9p8nZY+K8DxaKzdMemAuqAve7eqUN+7kde3lO7sTzyrQ+l5
eijds7/kj/j0by5Vv1nfNpzvyOj8eFJzkDOQ/enxHlEjna6T58B8gdWerzqa
PB/W2uactPqeWu0E9qeEdF0P/X5HjFOmeTApJf8hxyI3DTjvDtQf9nBhqD6x
Ps5c1siaWTs1+Tcee8S5yLnGfKi7Tu/XkkB3nnxyZrT7jOJcW+M5U1MR1wO5
Y33kgTOjpUS1k69WryuswLeE/g+cB8yR3xclNZfZ7wOBtMgz1lSQVl2TkMsw
1kuTNnt9RlWN0p5a5X+B/4cB31f5P4X5Mm/qamYo3ROD84R2tCFG9j/MDsep
Rm4zQtlwLva5De1p9ycWNDlcrhjkuMv7vrZMNff5zOhy7R0H3rFQM4D8XXQO
P4CbwYzuE/+rP1LisxnYLViDRfpfZv0+fsX5K58zI34PeQcZj7b8/zCvLf4/
+g0WSwxH
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwllEtIVHEUxo+jjo9qZu4d544GRUYbK1tEi4IeVEKZPVAXLjQIF2pltkiK
nlPRMiiIrMhNiyKwCIRIyEJQUyGqMYOK3FQg9IBoW0m/j7M48P/f8/rOd77/
rW4/2tQTM7O9WBEWVZgNBWZXis0ulphdiMyu8q0d5+syvifMboZmW+Nmd4m/
xTmRMttSaBaQM0x8VGl2aoFZQTnx5E7xbRHnP9To4t6JjVJ7DMsSuwz/FL7T
5BznfgJ7R3wd38oyZjnqd9H7WKnX7k15rHo0k9uEFTDEHWKOZM02g+kXWCap
0ZY2W0J8I/i+gvkvs80Hjlf4Z/BVp73XdiwX+IzyKaaVexv2KOY9hjk/xWYK
zA4Qv5rc7yk/i7MRzh1gzIPtBf1LwX8taTYQ8xp5Zt9JfgbfWuaZButyLEev
HvD14bsR+L0t5lxdT/rs4uwz9b5g+0rc30C/t+TPFXtOH7EHyRnH30vOB3xr
sMv4eqnfgv996GfldDJrBGf7wTPCzJNw+Y2aw3Hf2SF83dgE53HsDfEd1J8g
9z4czDJ/FfdLJY5JvWqxj+Y9tbvyjGPRDrsi15i09Zj7bWI3JJ2Lurhj+Z32
WsJ0MuuakVakCeN8L+H1u/n2j9j5tM+ifosrXIPSnjAptj9wn3I2iQ9q5vBP
srM8sQ34VxT7zD/Be4ZvL8H2qsy5lKakJXH6PHLNSCtFfGul947Ata8asch3
qN3sJn8X/epD50oa+ER8fcp9mlHcHs66VsTxqsA5F9c1C82eoKUhrJD7IPjX
J51DcacdrCR+sNJj9YZ+UDsXOXZp8qH6B679Rqyz0t+M3oo0OpfwN6q3KU3X
pr2GcvXmp4l9EHpuVblrV990loZr8OVD16520Ez+UvKbuM/GXdt7ks6H9Cnu
N4auNe1gG/E1ge9Cb3YUrYxlPDfF/Rm91gXOtTjQWx4I3ac3rbfcH7p29F7V
+3zk/yphOMf5bOTcaaf/AX7EquY=
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.5924258150652623, 0.5735256279063238, 0.9101508460226225], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNVUlIlVEUPk7v+USfvsHfZ5ssCZoosKAoiFDCBqIoKGhEcSiNBq2FbtpE
u9JykxlBk5s0tEWZWVkL06LIXrqTNKFBN0Uroazv87tmi8O9/73nfuec7wz/
gtITu44nmlkzJAlSlWO2LGT2KtmswG9WCDmMfX2K2S9IG5RH0s3iOOuGjMTM
RrPNVgTMxrF+gqzE/nSWmYe7NoCuwbv3PrMP0G8NwkjUrBZnd3HXArxOHNV7
Zul4sxF6Q37pPkswK4U/ncDpTjM7GNC7C0nSex2QbzWpZichV4E1jvMivL8N
zNXAG4E/R6E/DL0hSD6kDO/OQ4LYXwLeE7wxvK2GrWM5ir08ID3qFIOP/RFh
74Tvm8Pye9gvX+lnLWz9jCqmPwlaGde1kGKm3/S34D8+ySV9rsbZZ3y/gX4F
Yu3Nkp9bYGcrZCpF8bU4/efg6it0DiWL69EUxdgE3e+w9xTfezLNFoeFO4h1
SVj4dXj7zb0lbo3jrQn6qeDqVKq4Zwz0fxq+t0X1PbMGtb8PP5fCViLy8hNy
MyB+e4H/AuJLU855x/M+2GqEFMPWJOwvj4j3x8DoCSkPrKlhlyMPto540mdM
qzIV15DTIec3cN4fFJe7oVsZkx+0TTzmlG8KfaqHbtoJz/nR6HR2RJVX8vsI
OlMx3ZFz8kEetsP+Qrwdw/lbV//08wHieBhRXlnn5I76QcTYHJZt5plck2fi
nXGYzCHzwVzkQXd+WLkkb4yTWJN+1TprhvbfOx8+uHomDxfBweWwYpzwK3Zi
7vLEy2xuuOa7midH1FmXrr25vm71CXMCvvV7sst42RPshQBi7cDdLZ96pcr1
y0vY34TzSpxXwJ/rQdUbMWszhHsuU1yT566Qcsw83AFmF2QDargEePezpF+B
+3fAeYGYxrI1azhnyD1x6edAUDOC8yEP/ubE1LsdWBeFNKPY67z/t3exkO+P
2eL8OfTXh8QH51yf688r0GmAjX0p8mPKnTMn5Jockot6l98DnBUh1UId/K/0
NCOK3CxljlhfrEXWGGs57vLI2Dgv2cu5eDsS0Z78Md5uN3Pjzv+SDOVmdqaV
u/fsyXI3uwZgvyGqOqrw1M/sX8Z0NqS42qOaX5xZ5H3YP+dPmcNkzTIm1i3j
iKWpNmbq0O8wgfMlKFus+3HXd7mwG/M0v5nb9Ijs0HZCQHnvwf28qGy1Iyf3
snXHfwjv2ty/4qTrHdYc42SMzEef62nOjLWZ2tPPAsf5XtiNePp3sEbH3LyN
u/omzjbwMegpNs7OHxHNz2msvyOa09Ql77T/F8RbD4U=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtVEtIlGEUvWP+oyPq/A/nn5k2KRHRg4IKioQIpexJIOQirShqRIRe1iKD
WhTtCs3A0gh6btKwFj2sRF2UKUEK1UrSBCNdiauedg7HxYX73e8+zr33fF/J
4eOVx7LMbC8kG7LCM3uaMsvKMxuEYTg0m3LNDkJPwVYA/YZvts8x27TAbK7Q
rLPI7BH1iFkj9LOQTzlmnyGD8N0dNxuHf13U7BzyPQvM1uBuKGb2NWG2Djnr
ED8Gn+vwzYHtRK7ZKch35G9Fjs2InUbMTegVwDiBcxmkF7F+2uxQgdliYHyO
3FcRc99U4wfuG0PFH8B9F853EN+IWvdgW4Z8/xBTD98PiAngm4bsB7YrwFSO
uwgwzaL3EfgvRG/pIuHtgAyg1h7k+OMoZyly9yI+Cv+N+WZNuOtLSTcM+hrO
LZAeRxgXwbcUchn1jsSkv0fO7jzZniC2C9KdLVsS+mlXtY/ifhT4JjCv1TFh
+o36GeDrRz8dqDeWUA3mXgWphZ4JtSvu6CJ6iwWaRRuJgNi5QL03AGMRZ5s0
e4j7hgLNuj4pbnDm+YF2zl3TZxT1xhOqRTy1wJtJaTd3YdsC7CW++FAL/wH2
G2q35NgM8Be62i1ntBb42uBfPn++gPu4qzM5OIP6mULtbtJR7q2uuMYau+Li
IM/k13LEF4eaHTnO3Oc95WLOWfRyCTHtWeL0EPIPQsqArxyyIa4Zc7YVkHbo
X5CjIap5LYVe5epMPOTuJPBN5YiD/ajdF4qL5Eh1oBrMTU7/Qr6XkCbEvoU0
++IQuUNbtSdOkovrHeV+BVuzoxrfEuqZvZITI4gd9jUfziPf1Rvk2yMeYq+K
Cysxk8vEwNrkdCvuajzVOoP3aDi/8NQ7MX1E/e2I+emIczt8vWG+Xdo4205g
isQ0Y76lylBc4Jv6G2hmrM83yN0V+8LPnS3xxEFyj2+As73ladbEexv13/ni
Envc5ovT5DJ3mAq1I/rzTfNvacH9yVz9MbkJ9cjeaHsN32nUeBPVTFcG6pm9
0kbukaM983x5jPgHgf5C9kiu7PT0V5Iz/wFxUr7M
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.6154892729310001, 0.6105821872211546, 0.9106766218255402], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNlV+IVVUUxpd15x7v3Nude8+9c2Z8ySTqJXtJX2q0MKikjEihEGZIy6yX
SoIatSbIaxaBg39SyCIwnR4Cw0lL5qU0+jMTzVMPFfQXtRwsIqbAwrDvx7cH
eticffZZ+1vfWutb6yx68InVj18WEW9rXa71WFfEYBYxopcbGhHf90YsrkRc
p3W91oeyWarzH3r9/ndfxLe9/l7T80BPxJb5EbeUI9YKa7lwXq5HvJRH7Nfd
o/0R41rX1iLuk+1XOt8gAtv0/nzNvhqtiOPykZcibtdzkWyeFc7CIuJH+dg9
L2JUmCda9jEoux3yNyCbLmHnsrtfOBPd9nNR+xfk6892xOvydUTPS7qfiWeX
1rRsPhDmuiu8x++wzscz2z8svC/FY4XOR3X3rO7OZOb8YsV8XmtG9BeOBdu/
2rYHjyf4ZWHeqv1e7Svi3hGnC13mtaPHvjbLZlRne+b527uZ92eF+VThWMHo
SThwHJxvfGIe1PpGed4qzFrLttT00bJziM3ainlOKY6BwvzXCL8o/A28hyrG
5PvCZLNR9n+0rA/iIl/EtkD1eaBubZzW+Zmaa/GvzoYLx4M9uSa3u2S/W2uZ
OP0ivzvFdUjn66Wl9oJUN32/o2EddnR3e6oj/sgT3N4Xl2cKa2xS36a0DonD
Rp09onVc71v1nBHO5xXnnzNygg8wySPfuA/OuZJ5wf9KcViZONRKrge1+Ee1
vkffdlGXFG+15HquzG33q2zeE79luntEmn1O7yvE+7CwJ0rGOyluA0375D65
A3Of7C8J6zfxXS89H0u9cL7ivoIncdca3qPBi0mH9BC6ZE98xMk3+opv9Okh
3T1YOFfEfT7FPl2NGCs7JzeL113iP6v9zrrvYH9Kz4+0MuF+XNgOG3r0toY1
Nqn9VG7N3dS0dtEt+v057cnHgdw5wW8n9dKYfL6j+K/OzL+TakRt8Umurso9
F/A1LR/d3ebwifA/LfwO1rY0T6oN14NaUCtqBgd6azz1FzhgzNWGJ7XALz6J
nRnztdYMmpefVpozP4nvd2kGrhL/u1u229J2H6K3N2W7uXC+R8TlldyzCz+z
qf5PKs9jdfcvc5GZQp/if+Z/HNijT2xL/banJ4iFvqAXX039eKLpPqQH7xSv
G5v29Znw36h7NizJnG/mM9jnEj7645wavaW70ed3+pLeGEn5PJ30j95/T3le
nuY/ezQ/nXS/pp70JLiDsj3acL7pp8m6c/6F9vua5obumRvMOmK60Oe4mE9Z
mnvb1Suzbc+tuZnOOc9yuouuj2Xer6r7n0S86Av9oTH0uKTH8T6tWm1qON/8
C5jL6GSi6f6HJ/8o+hOO5H442TDDrmm698ZlU23Z/1DSFjb7iTF3P/O/5C41
xIZ8zWmwmnTYSfMPXe/VvT25cwxvsImLGg8l/HuVj9Vtx3wm6Z2a8t86nLic
6vcsZt79B7+HGDI=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtIFVAQhke7er160/vu2qYikqDaaJtSC4NAyqiMEsHA7LnpAUFp1CIo
WyWZKRQRWNkqw0dktimjh0buC7IH0UMiKiTQiOz7mbsYOHNm5p//zOMsajpc
eyjbzOqQANKdMruOjIbMbqDnx83WIOO5ZjeRT4V+p/Nk0KwF368Rs7GQxzxG
f6Q7wIL5ZpsTZmeKzBryzPrx35LwO50HkCvYCpJmx7G3IG85VxB/DqzlSIrz
dnz2QLIe3eaZ1cChi+PioNtKYman5rjPZfShqFlbjlkzeHfA60Wqst3/KFx7
yR/Ethu8OPh13DVwrkP+gb2Pu7ucc/G5BHYB9grw68HsBHuUuwO8/yD6ILZo
sVnjXLMYb95PbAv46wKO8Rn7Me5a8W/g7gfxpUXOR5hF1DIS99iOLOqA7xdi
dgb8DbPwuQ1eTp5zHsC2ixoMU9sifMrA+hl1LGGu4vwnTZ/C8Ma+Hv/nMed6
Ep8+9BD5ptHD2I+g16C/ynWOYfQTcHgB90r8Y5y3Rbz2qs8CsBbGHOshPdhE
bD05JwOOUY3vy0w+9WQq4TVXrTUD1TGfAfVeHJrBzyf+W2Z+foG3NNPPBnzC
cX+z3qqa1ia8ZqrVIPYlxPbx3pKw16QA29aE23pyndu2Qp9lcTyPrS3hs6sa
X1O/Cz2XOLeT+wEx5eSvIH6aWs8gy0L+Xs2qZkyzpZnV7I+kfNa1A884P0Gm
wM7nLgC3nLTX7jXxTWCNka8d/n+5uw/fnrjPZhlykfxthT5rleTvVC2irq9F
34jvhrj3VjkmI76Dqp16NoTtXtx7pxjtUiP5xwt8pzRLmnHNtmbqXdJ7qret
wD7L225l+HzHPoF9ZcRtqsF79AnwTof9DW/QSyO+q7L/Bv8sGFeZxaqA97Yr
5tzV4zHO8zPzox3S3/I05bXSH9Of9hqpNuqpuPdHvJd6Qzl4I2nvjf4YcfmQ
9NziJOzizHuUQ7u5N+VctKOtcAsn/a8RxzS24pT3R/+Hdmd11HdVO6RZmOHu
Y9hnQrO5I/MfaUZlG8bnQpb7aBc7Yt477eQote6O+q7JR7vfmfTaal//A8XQ
uYo=
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.6385527307967378, 0.6476387465359856, 0.9112023976284579], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNVU1sVFUUPnQ6r/NmpjPz5r15nQIuunGBG37CAmIICZCIlAgDTQQDg0BV
NLFoDMgCdkDlTyrUhAZSJAbQlVoSN6ILWtCEEMCFG1PSKq0BjImJCRqlfl++
k+ji5t5377nnfPc73zmva3tf480WM/saI4Mxp9NsNsZIu1k5a1bCeL/V7GZq
tiIyK+J7J9a5CoxDsw+rZvlINgHOshgTWL+cmG3H+BTOLwVmXdirYCzBejKQ
7wF8L8b8CQJvKJltivRdyMM1xiTON8Vmm2PdYeyT2FsK+4gYgWEEGH7C3g+B
7EfxPYZxd5ZZB3AOwefbuPc8fKyJZUe876bCvLxV77gK+9M1s1MYj3Jm89vM
mkWzJ7OEkxgZd0tOb6L9L7C5EcomDyzrY9mQB+L/Hvv7EOskfPY49rLzyUFu
l7fID3mj/Xn4+BhjHvzMFLV/ARCHEed3jCHcm4b9VJveyHPeJwZi5D4xXMY7
I+f8W+RoEDheQqwH8PEoFBbeDTxuHjHvl7ReFAgvMdwGT6siYW3AZqVr4ERV
Z9yf9Nwz7yeQ8+OJMG0EJ0XYHM3Ix+1E9n/VoYNIeTuK+Sv4eiWr3BQ87xcR
a6Zs9kxGedhWFA/r4O+FVHxQL2tjaYZ559vJwVPwN4ZxD/ub8aZTGJ/xLvY6
oZsvoO1dsL2P8yMZne+sy2ZZLC7Iw3uheCSHZew1En33FqSpI54j+uPbP8d7
t8FPO9ZngL2AvI+06PvVjPwPY78BHDO4/w3mRbwTSMe34Lcf80Ls/VjTnb8R
962S8DIG7zHvG2BTTYVhblVv5nu7Yb+7Ih4WItZv4Ogq9tuxv6tDtXgTts/B
5g2sR7FeU5Y2WPN8C9803zVFnTNPzAnjcp7wNTUeeP6PRcol80iNsF9QJ6OY
r6eyHXb9she0VZUbckJ8D3PK9ZjXM/0PgIMPajq7Avy1RLiIm28l5k74mY0x
ju9j8Jck0gmxj3idVmLlj7m7VxcX5OFcqJn5PQwfv1KLWXE8/b/6arr2tgLH
FHI0YaqhA3n1hmlwuTdVL+D72B9ow1xSQ/TfzKl3EM+zsB2vSResUc7kgeeM
wbvkjr2SdfrA65U9g3EZk9/U+7VQ9c71qPe966HW1ECjJF6I/2yoeIzVm3oP
xd41cBJWFO/LSNoldwlsahgvhtLykO//DNt3UsVm72RO+K6lkWqb+evFvDZV
zCLuHiqLg/XITyEWDtZ007XGemIc6pl5eOi5KCDWaeRmWSBf55xP3j/r66dx
3lNWPWcRt7X+X42+3qb+swNx71TU73cA111fL8Ccq6rWmFvyQf4vwOajVNxv
LUjXtGGtjrv29uB8qiK+l/g/jXkZrKn2qOv9eMdgVX2eZwP+/+Lb+CbuU0eL
/X/IHt+Xk46a3v+p5Sdl/RvYk1k/PCfOsVQ9i/2Kvbanoveyl1B71CD1yLcy
R3+iBzzu0Jtew9ybyN8C+D/kOPnd79ySM/LF+93QkeFOd17/B86sWWqi6drm
vS7Hxrr8zmuT+UlT5Yg9kjXQ7v8g9psJr5strv+DsP8jUT2sjtU3WZf8NzM2
42bBc1ATX6VYtX3Je2m/55E89fk/m73hn7pq4V+1TjLg
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEts1VUQxkfLbe9ty+3/0fvvrRgTFrqQDYWYCAtCwkKU9wUTH5Fni4CJ
IFHBhexEkQIqlESiMWIUZaUW4wZ1QYsSDCGwYIGB+Gg10kBCMNEEqL8v311M
cubMnJlvvpk509dvbbx4b0T0I1OQ890RfV0R87msliLeqUXcRX+4JeJJ9B1F
xJ0qd/dE/FjhLono4W47tr34VNCfziN+a42YgzyG/lAWMYZ9C/rbvF0saXe8
3cS+Rc7nyhFrkBrn8/WItZ0RM9siXsL3web7zegPZI6pWFe5uwK+WejPk3sq
BbwCljH0UbCNIHuJ9wf6Kc7jbY51NvN7xXwZ/3bsc1vscxy9RMytYLmJvo/3
fyau9S/eN9BXICm5PifGssJv5PsRcoDY+zNz14Ucob7OmmsbhtPd2N5ETpbM
8XpibUDmE+84+hj17u92rotwvLknYhMyHfsx8n2KfTuypc2cP5M7p3KJ821g
fRU8p8HyBTWdwfYT8iyxr5Rc+/2ZuRMHHbz/KjF3qmkR+ryUe/pzCfsdenEX
maQfR8O9/zZ1LZqBOZyfyO37O+8bxB4UZ+CbJEeA/TPsM1rc84/B9gkyAb6/
kfeJtwZZiv9G/Ed5P9LEK4zKPa2XfFON4XuwVnNz8SHvB8i1vDD34mgaby/n
jqd6y/ifyD1Lgc8qcl0H83fY+rj7BttE6n5I17mDN/NafXeCc5K713t4/x69
HKy5nmv09CT5+przJ/yZ5gfMAx3MOncLE8+sanmBGOOJZ1SzqR4Jy7s1xxKm
f6jnIj7jvF9d9m680eWzdqQV33XdnhXNaEGslYlzaZ7X1Y1ZWM8Ro4b9Rm77
UxVzvw8OdrW7B0fI313Ypnnayflo4d4cgM9Jcg/VjE01i9vB1LWKY+1qo+rZ
0M4K+wDvhyuu4VF8H889C78S71rqHohr8fsLsf9jRlZ3mMMlhXus3n5Q8Sxp
h7W7mqnZiXus3q6Fz0Xg+yHzWTNzI3VPFV+YV3BuIJ0l7/gQ9p8z90L+S7At
zr0rj+Dzb49nUvUJ02nwjBbexQqc9au3dWP7kpoPEev11LusmRG3K6uOJY4X
YDtV918oDMOJe6TeiCPt2kjh2Nq53l5zJq6+ZubvQ1+e+awdKHNuy5xbfB2E
v9dS/6UTZf99c1Pvov5A9f527r9TM3ABe3/hXdCfM6Xe/FNKntkhYh/O3HvV
9FbmP1vxZ7d69g/VnEvzoN3tbfZD8/A/yf/j5g==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.6616161886624756, 0.6846953058508165, 0.9117281734313757], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVltsVFUU3aXtvDqde+fO3JmpPIxD/PCrVn+wJCgWE2tVyqR+lCYaKU0w
MSoGbTUxlA9BYRAf+IEmBo2PpDEGjDEQ+4AAiVLQQozx+QeaaKyFT2mra3Xt
pB8nc++5++y99tpr7zO3bHu69tQKM7uI1Yh1NWn2b5PZpQaz6cjsVqwL2J/L
mJ3Gt4+xvzmB781mvTj4XtrsFPbHcHgRe+dbzer43VE2ewLrDM7tzZmNws8+
+FkHu3HY342zC25/As+T2FuNuB/B/0yL2R6834z3szh3DmsIMZvhL4FVxbmv
8B5mhJW4GIfYJkOzh0o62wNsA/g2yAQR50KrYu7GXi/s2/E77fvcq8F+K9aP
MK/HZm9i/Z5SHMa7q1G20+7nA+xNJIX5vO8zd3KYdWxJ4L0IPBHef0joHPOd
D8AH/N8Gn0cR8whWFfYfgqsVFWEgBxud28OxMBHP/qLZq0XFIWfEQQzk/6D7
p90fWJ14XhWplqzj4bzZW1hPwm4GcWL4GXU++EtOviiYpULVKo/9XxPK5TNg
3J8Vz40421ISZ3fgewditcLnffB3e6D6HkLMrfDVydoB+xjsP8HZszh7Dmsj
fC00KG/ur0HsPx3zsbR0Qgys5R7noQO+Z4qKR/4WAnE4BH/bK9LODewdjJX/
cfiZ8hqVEf8R5DWQVp3aXYfD4GMEqw97dfz+Asw74HMf8O+NlB/PMFdqidp/
NCH9tztG+mkPlD+fC+CtjLW2Wc/8JT/9LarN2ya/3CN+5kM7vncA43ikGL/h
/bGE8FCHedcVa2gl1ZHaZE9Qn8/ibHdRXA1m9J11n4C/yUi+vkZ+/dDZPUmd
W9u8rG3zXlgJ2ztDxe1FnKGi+mxTTv3MXu6FzU1t0vRJPN8biBP2wqLrsAs+
fkK8F+HnAHzUsQaahHFnKJw/p1R71p21mvR6VZPaZ6xh2I9gfY/368jrWkbP
dWjoQFYziFxxTjDWIrSwUFE+DcC/KdQ8YL/QP/W2sk12tBkD/lWhZtsSL03S
JrH/7fg3ZFUDcpWEfSqSFlg/aoc15bzcmRPn5J7n+My+Yf9sgN9Zai0n+yv4
fTnSvKW++lxjnCvjzsMW4ApK+kZtsjasyz94jkuacaxj5PohN5zXjLWImrwe
q68eB3+JWPl/CQyvFDUvMvC/C37WI/emlPJn7lOI0xOIV/JxxPmbRdxKpH1i
fDctzXdjf11ec472q72vd/tcYc/lCprR7OmuQLqhZmo5zSPOoO3Achk2n6c1
e9b7/KFOq95HD8NmKtT9w/1On8+0oz35ZMxR78/7Yd9dUg7UYC2SDmk74Pfd
a5FmFn2xVldbxOEZ4OrKq/498PEA1jv4/g1yWVPSjKbWyeXSvdmqHmBf0I7z
mVwwDu1oQ94441jfZ3Ka+5z5zJn7xDyB/B4siV/uHXf7U343sneW7reMNLYL
OJ+rCPdpryH5Zz2P+v1SA+YtBfE67b3OusznpAnmwLlFDshrkbnmpclDsWYl
tZ71O5F8113bvBc44zf7nH8ed99wWRxSF21t0kY/4qdD1Yu5sH+ZDzXLPqE9
dXg9s9zvfGc+jMl7o+o65zfuz/lM4PP78P9fSbOD5695L7wE/LOB5s130MCN
nHJnL/6VVz9+Clypgmo2UtbcoY8XeG/GmlFvxOph2g/i+7ai5hY5n3MMzGPO
/5OQm3p2Ode636FXQs1E6oL/Eb71/wnzBfU2a9vndwZj/Q+h8lJ3
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlV1olnUYxu+57X3e93Vu7/u8n/Oj0LMOog9PzMCPpqStchvzYBukOQcW
oShWUwj1oEw3tfw4yKAsokAiUjoQdK5wwwIpJxERRQepQaF0nJv1u7h28If/
/fzvj+u+7+u+n8VbdvRsnxMRL3OaOP9VI9a0Rhzk/kAScacUcZfzW3PE3saI
edx/r0eMtER82hDxVhrxWBsnG/F4JmJjIaIFHz25iAFOe3tEFzoz2P/E+7/o
jlQibqPfN5dYRXTx+StvxXzEo7z/UI4YRX8lwI6j+y7nVNjmVWK/VotYAdhb
2Pfg+0NsLqA7wLc8cgdyF/I28B7Fdhr5IWHHZzfv0+R3hPt9zk7wPg/eD8B6
Fp1J3helznUITJ3YTqS+qwZ3wHswda7yOY79VvB+ndjHenw9zVkElveQH8b+
DTD8AvblYDrB/eRs/pJnyKeM/T7sY559bcb+i5x9HkK+itxCbW4hX+d+lPjL
iV/jWwd4rvGtwH0FPRlCfxv1uYL8jfSR16JTyrtH3dQ6D+YnM655Vv1DfoH7
FPnNAU9j3f37WRzA966Ca7Maf88hD/F+ndo/2OTary+5t+rBKLXt4exHt4uY
L9XMGXFFmAL7joLr+wrxMrwnnJS3JchZ1ROd88Rfhs5w0ZwRV3p5r5DPVyW/
Kcbl1D1XrzeBYSn246l7pW/9YElS5yb8fcjd+D+D/1XIz3D/HJuPsT3Nt2eR
H2kz995Hfor7Wew3ZMyp+fB5Qbt7JU7f5f2jkmdFPb6E7kjRuqqpevcH/i9m
3EPNRi9yf841nqSWzTXnvpr+9eKvAZ1O3gbxsRO80/gfwP81wt5rNYfFXc2I
ZukeNn9mzakjFXNO3NK3HLrflZy7arCw4G+6K6cNxOqqmiuf0aO2qjEodi8Y
3qbeN4k51eCZEZfEQXFPnDrR6hrL/4HE3N1d92yKwzeIN8i3L3PuaQZ8zRXH
upy4F+uqnhX15CpYvy25N03gX4P9P7P5qyYVdMtV49M3zY44J65phrLobubb
WOKeTik+8jjyOey/r3onaBeIc9LdxLmU2GZr3TkpF3Fcb2MFc0E6M9TmWMVc
1PwvoHcni+byYN6zqJ2mXaaZrFPrsdTcFCfFjdGiuSmOvMnb4bJnWzXW7lmY
uteakdfxP8z5MfHOe4fYe4ru71/UZzfxbha8+zTT4tpE3VwS5/qpRx/niUbv
jL+L3mHCrp2q3qhGqo16pN3eWvJsa8er9spRuaneOXSv1L1r5HML2F8su5eq
8Vz8dRfMHfH7E+zPzM7HEs5w2T1Wb5WTdmVn1bOnnSnd+23OTTanwVqr+l+i
nv8P207a0w==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.6846796465282134, 0.7217518651656474, 0.9122539492342934], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFlktslFUYhg90pp3+tTPTufwziVBhpQtMMXFjkDQRIlgLocUYqBhKg5eV
2KJQRCXRaFoQ3bAwLjRdmGgVu6odgmhEsboQSDUBwZU2iguJCxovrdH38T2J
i5Nz/u98l/e832Vm9dC+/ieWhxA+1WrSWrgphBtafUkIFyohrC1o5UKY0fcR
7a8sC2E4G8L21hDGmqzfr7uBjHfsTsjP9sTfnNP2EPZI/7hsT7aEMKTztib7
eVzf4zqPFEPoTH13v/ZNWq/r3JDOauncJp12yfKpYx8VtpdKIdzRLH2dT8o+
l3EcYhCrS3g7M8aMrJFY/rbsvsj7vFX2L0Q8v5VD2Cg/h3W+qvuV+j4tnQfE
QUFx54Wl1u73zMnXV7pvLdl2Wt8fJo7VyBhLS9y54zwgDFu0PhYnf8rngY4Q
bpXtuozfOiF5qRrCmwVz3Sffi8KxJJ/fJI5N3B1tIfyQM7c3aiH8XrAM3z3y
M6CEbihCvL/X6P55+bpL8nPCfEnrEcXNyvZc3Xn6Tv626X0rJR+U3V5xukq4
BrXnyuaxM76FN74v/z2p33VMmP8qOEct8tlcc/4nheehnPHjuyexf3J7X8wv
tUYO4eHFqnnhvFHv/ie1zWZhmFK8JONckUs4py6JgX/evisxt5tl15eaT3iD
s4+k86XeMavVLd1lqfNKTr9NrIcOWMBEPb8jzt4tuUamlav94upY1jo7M8YJ
prlYV2uL5hduuzPWXZS8Ij6qNeM7oP3p+v/5ZIfnScW5R3guyuaEYt1ccx3y
TXzeOyrZwZrxdirOecW7O+Pa/6DuWnuq7vzhc02zeV8h2zek/1bZ57Gqa4J6
uCjbVZKvj3jZwfxHwfngjT8Lz08d7sXXJPtbd7+I++nUvUdc7sg/Pqnd91pc
J/QENZOLNb4u9gXymVhL3HdG+W69b7DmXkW3EeXU6EKcS88Kw+Gqa5b8d8W5
1CVcvam/yTHvoAZul/y49E8J45LOr+p8Leca/6Roru7MOqfU0nNV26A/JT4f
rhnviHzv1zob63Yh1gm2yLDn3ZPx7cyGFUXn7nP5+bFo/UXxvU/1/Zj0eoVh
S8V1RM6ZQeQdjOdT43xStl/HXO9QHvryjjmj+4bWomLvbfM8ZSZT+zvzfjs2
8MgcOySfz2hdls+RunNPvpfynjVLsf7nW/wWZimzDn5uKXsm4gde8EEvLxbc
/+S+Q3GLWg/K7lDWuMgf/TCXdW/Oy8dYxTF4/3zEPCu78Yr56Y41SD8+Kjzf
Z83J0Q73GP76tbem/u2gR3Y1Wz4k2R6tKcU+KFyjWpdy/g05FWu0T7ZtqeNu
Usx7K87bsDiZzRs7vLHD3ZGSe4AaZ86NdthPKnmtZHxntfcUPHvP6Fwv+b3U
K7VCH02Iw+sF84bdZ9GW2dYba4B3wNV/nCXmn9lyRjq7K+4T5OjCD7LBKIcj
/IEB/ugBdKg7+IXbrdqHi85tAsaS4zEXRyu2uaxzc8m8kh9ygG96nt8denxD
wfbYwjWzg7wxb+CbOUOtwN9E/A9Qi7UEtvWxJvl9oJapvSTmHsxXlbcrZc/1
C0XXKzbUFnVO3pl/9Aa5ui7dX8v+jQXjtfj7SKzTcQ40SXd53b1CbhuxN15W
rPGq7cjhlao5/BfFdDGM
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEuI1WUYxl8958yZc4bO7X/+/zMLnRQCWwQquAkKQSfTMcVRCDW7OGS5
0sZJZ7yLkM6MSItcWItASDAvuWocMRWsrBZmpOCtVYgYkaugYCby9/i4eOF7
v/f+PO/3zezbvHLT1IjYgOSRabWIk42IXwoRK3IRI82IUWRuW8RaHNJWxNFK
xCV815Ujvu+M+K7TtpXod5OIZsu2ZcWIUhZRR14vkYt8z5H7KvIe+e5TYwDb
aBqxhvgr+PSSe0vm8zzsg+Tahtwg301kBvmfRS5MiXiZmCF6G0S+QZ9Pvcv0
v5z49cTPwL6D3APIdGrfbo+YIP91/OdwniSmA98c/b/bETGMz9Y6+fC/hX0F
9Yc4f0a9aTnffUvvV5C1gPYb9rvY76WeVXc95FuKHKV+Me/ehlruXT1updaH
ne5NM31CvSPIzpzjp2L7gh77C+7pPLUWViN+RV8HZl+in2LG/eRfjt5FrZ8S
c9WnO+r1cXeW83xqdOM7JTPWqlHj/Gnd+cVJnt5GqHcEPnvp5xL5LzbMr2re
IvftxPuwgfjpnLuQl/LmQNz9S3+zcubwEefd3J0vGHNxnTWMlThvJ/Z0zdh0
IZ/j/yo9nwK/veC7hPOWmmc5w901zv1gsr3gmpsr3iHN8z72BVVjJGzU43/o
B6n/O7n+QNbg/2Pi3TjGTIPM/nFq24vgvY/YAxVz/QI5vqLWWWQMLNqptzP1
Tsh+h5htdWMmrDRzN/3up+Yb2E5So418JxrmRv2Iyw+QjUVj/g6xx7EfBrvx
srH/m7vVHeZgWdMcirsycofeF4JBT8k79lfiuyf7VnTv3UhP2T0+oL+l6DOx
jaOPwffXmWcZz/vtjtbNpTge4zxZ8VsQf//jeywxd8o5VDdmwup53dHb2y33
Ln60S88Qs6rk+RY1jYmwEKfXsBXxX1X2fBX0PxO/NcV0gsWjxLOoH+22aii3
dvxNYt9CzpXNyRLiF2fmUm9Mtkbq3ZHPD8xyDvtEyTsjblbX/ZeIo/XCA/1Q
wW/kMLETVc+mnX2IbXvqv0L6K/QyG/vsp/z+w/nA0/0SZ3Oq9pFNf4r+ml2p
d0UYVqnXW/O82tmfM2MiLB7i85F2mZ7vF/3HDrB712v+29Sjdnmyal/V7K/5
jQgf/XF7iN/bMNfiaDwz5+JaGGgXNJNm0U60gVeh5b9anIyQfzh1rPIPN70T
2gX1JK4mEr81zRCZ/6wnfx35X0O/WPNbFkZXqbe46d3Vjj8GMyfc4A==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7077431043939513, 0.7588084244804784, 0.9127797250372112], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1lkts1FUUxk/LMO3MtDOdmc5MDaKYoFEwJD4WwELFGB+IPDQqJuKjKiSA
EC0gFpVXS4REDboRF26MiUZTDAZqcCMS68JoFTQgJOVRICq6UQNaE/H78V0X
N3P/5577nXO+87hzRffKe1c0R8RhrXFa/bmID1sixmciNrRFjOn71aaISZIN
jo+4SkrXNiJ+1+rNR2wuR0zoiJgr+fo230H/llLEZ7WIeQIfaPHZmOS76xF7
6sY9UYjIyc4y6VyQ7q3CuUvyb/LWB+edasTsou33SKdX62BrxJ2dEUNF+3qT
1jU525or+T1aH0n2tPT6hXVccR3VvSNaU6S3s8VxEuNS6czIWAe77Pc22f4r
yWd8JS7iOCx/DmnN0f6c/N+ns4W6c6XON2s/qH2vznbIl2HdHc4bB+xd2YgD
GdsAC+xvtU7AbbP3v4q3FfJzgvjcLW4fF1evST4zY77gCu7Plcz/KclOFnyO
H/gDzpL2iMXtvsP5krRnIYd3eGAP/kPadwvvK/Hwcda4YA4rJz1dEaMFc3dU
NbB4nPHA5S7xrU8xEt/MxOfGnO3A83XyOXdJRJ90diiuDsXVrVjmS+d+rUPh
/YKc5dN1PrNujG7xcUzxfqFczVZuP5BP2Yy53pzifSr5RQ3A/WDif75083Xj
Ts26huF2eav9hM+HhV3u9P6Jgv3Anz/l81bV2H26s0Y+9MjveeNd77uqrnly
eSDF+5Zka6XXEJcHW8wjZ98J54DW9cK+Q3beLtm/9ZWIlyrJr2bHQyxwhC61
ia/Y3BK2vyr5QA99kvpokfDG1ez/OuGN6PvqrHsV36dL/rrubddapu/npLta
+xuRS7e55l4Bd0HBtqhVYsB/4oTvneF+m5N1Ti8oliNF98uzcKz9G9KpCO+R
huOhvqgz6gDZoiQvSz9XN9fkAvtwOCBuz3Ra/9GG841t8pBpOJ6M8Jtqvkfv
jOSda/K5POE8o5r9WljHUv2fSPW8jTzK7inp/aJ81SquaXpxYeJ/kzi5rdN1
NCPNCGJcLfurGu7pTcLf2OUZsEWY50vmm9wtTT7sk3yo6vm0U7oDXZ4T1Bo5
I199wutvOIZLpftj4nOWfP+BehJmWfcqWj2ydVayYt39OlJwzzEnX6h5PjIb
p0pntGhO+N3S6T01Bl/smRPsifekbG3ocvwLy46f2OlrYh1Lcwxb+NYk3Xkd
7hX86ki+keslaZ5sE86bRfclNUcczBZktYbl/88gammKbL5Yca0yyzmnl5nj
6IBJrw8knzekmcP384r7p7LrltrMppqtC2+oYrvEw1ymPnljLs67ZnP8c+KZ
Ongy6/2gOP6yaG7pCXgFv02+t2s9IC469Xuk6rlDjpHRIw+WzSl8XqbziVX3
0zlhnte6Oe+aHE3z83P5uL9i28wOztcK56w4WaXYvm+1fX7p2Zcl26o1qu/J
urey6Bjxf1HW/hwv2Cb1S49SW+Tucv1Oqno2wTt8wRU8TU5c0fOc0y8TW+wT
/nTX/b6Sk7+FMUbsLe7LNWX35rvCeKxhXOY1d/GZebk/zUx66/bUX6d1b3XN
c3lOesvhcK9wPq04HnJ7d9H899VcK9TJLP3uqbhO2ut+V8gpumdSPVBTU5M+
uvwvQf897bMV2+Ldo0aJnRn5b83f+ATH9PU/VfcedfNXyXOEGcJb/VvJc5r6
Wpd3jd0gnD3qi2ltfq/er7gu6aNp6X8Fb/gf6X8U93rT3dPMj1b7zJu5veS5
ggyb6NBvF+/I5oy630veyt6khw7/ffrS/x9mBTVBPTD719WcF+YjZ9T5Jsn6
a75PLk8nH+j7kXT3P2r4bN8=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlV2M3GMUxs9+zc7stp35z+z/PxOCuqhgSygXuCBc+KhlbRtViQaDXvha
dre1dgXb3a2PC2JdIeGikZCwK21qpW4UrQthUUnpJv3QD1F1RVpWon5Pn7k4
mff83/c87znPed4z59f7Vz3eHBH9WCv2XxpxfSliuCXikbaIFvxW7NE8Pgd/
XhKxoRox1xGxoyniPtb3YjmCe3IR+SziDuL7CvxiT9QivsU/2BmxHztejBgA
bw94ffg3dkXchM2C/QbWB/7axOvXyWcZ5+uc38X5K/k2wvppbHMYY5LYCe7f
Tz6H2iM+5K5b+TbNepz728rUA0YvseupqUQ+l3B+lPOD3H+iEnEWfh3/7nbn
9jvnf8wb/37qqWMfsfcq9a4ht7uwtdS7gJ8D/ycwetifCWNNJeZOmO+w9yZW
Zf8cbDXYp7njFnKrk9OhkmtQ7jv5NoN/M/4H+JMF57ocG+lwzrex14PNtDun
3WAvYn8Ne9fibwD7BHecTa2z5LAbPt8tuzdnMDnbTcxckzlTb87F35pzj46Q
+xb8pZy9AIyU2B3YOvbnwfsKvCfh8ABnXwHjGriZBqMAH2Pk+xn370ydizg/
QC5vNepXD18g96Rm7kf4ViB+FfF3NvTSkZkjcSMNfVwz5+L60kURP5B/gP8w
WCvA+BN/E/7BvDlqZr2Cb5OsH8t7fTk20eFv71PLe2X3SppdyV2f4z/YqO8L
1tWy9a9vWmfY1+Fvk+BPYFc1+84U7K6qtfWQ4qn1S6wN7NtbrMXt1LRQsCZH
iX2moX/pebpmzUqry6jv07I1Kzxx/gmxs5m5FsYx8O4p+i2K8wXu2kJP9pLf
evD34c+Df3HBGEuIXZyZT+l7jLOj4F8I9jAxf4C1kfO9bdbMLuIvI5+LCu7h
POfP49s2zu/BP5UZQ7HXwedYzRqWdk/Sj2P06tfEWHqjG1n/wx1HqXd1m7Vw
MnPsmTdR8R3ClgYH2RvI/PaPEDNMbk+l5krnT4H1V9VY0ucUseOJuVZPxf3V
mbWoHjxPfptqzk1vdh2xMzVzPdtqLZ3u8iyTplT7y4l7KQ6GEs8szSpxtLdi
jsSN3vw3Jc84zTa9ie/wB/B/0VvChrjvisTa1cz8jdjjFWtNelLvmjj/QKd7
2I//b8W9kuZe0qxLXeth8ltcdc2qVW9+X2oNC089f5t1OXVtmqmbu/zm9Nb0
5sfZ+7tobqXfElwlmd+eNNJd9AzQfdK4evkceunOuaeD5Lo98Sw83NmYRSXP
Ks2kleR2NPHs0Qx5ltjlRfvCGEo909VPzdilFXMmrqQxcTdXMrY4lHbGG+9b
GmpiPVX029Z/0g0lz3DxpZnaW2r8BxXM6Wuc/b7qWaEZoFxeTM2lcvofFyD2
lw==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.730806562259689, 0.7958649837953092, 0.9133055008401288], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFlm1o1VUcx8+8e75u997d+793k0CCfJGWMQdlgoRB4hKdGM58KNPshUTO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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlUtoXHUUxk9eM07GeOfOzJ2ZaEFcuDCVgmShFkTjoiSW2iLY2Da1tuJC
xEnVpBKkom1S6bIFcSstWKUq9mGEFKVFcKHSGrXBihpN8F031URSWuPv83Nx
4J4533l95/zP3LRj+IFma0Q0kXbko0rEmSyiXIhY3xYx3Ii4fF3EUkvEt8WI
K9iuIoP5iO04NrF/WrLtG2SiHjGObCTYVC5ioRZxN/7PEruGzxfE/xI52xHx
KPGzcsRv6A9hex9MT4I/8X+4JuJdMGk1Yojf+tF3gxnF9mNKHrDn+e0VYu9P
HUv+8+grK849D34b+buIMcb3W/R0CX01mMfBJvxW6I64jXrfw7+3K+JzvqeR
1+ntCeJ/R+7nqHEttSRgJvC9iH1D0TV8jb5cNXa6M2IW/A5qPAa+BH4ZbMDR
OnLP8HkgcU7lUo5u7I8R4+Mwp/uq7kG1zyHH6eUG+E3xX4V+BfxVpK9gnxPY
y/g8n7f9MvoS8hK5d1JDF9hr657lNvRO+t9AvH3o8+gH4O4gMoa+iXo2U/sm
ZBXfH9DTr8Q6j6zNe0YNaj8B/iz78DI9jBB7FPmM3qfbzfVx7Ps7zPmHFfeg
2u5CVtL/bvjsYT6TYNZQzxT1dOO7AsmDzSGHWvzbEPHmiHcS7BT+7fi+hqzL
eYdaqLUdaVLrG9R8B/HurDnXq/CziG0hc2/irMQsysgo8ztCzz8R/23yjxe8
Y1sT75x2bQA9T6xczbuuGco3bXg2ivFPZg4Vvx97K/rBxLVop8RdX8k2cThE
rE/QD9Pf0YJnuzf1bDTje9GXM89GM2nLXJNqUY9/wvUi+s3YBsm/l1pn8TmN
7yzxb0m889p1cTaB/57MWL2ph8FvqTn3KfCXiHdr3VxsBFOp+03pLW1hpkfg
ein1/ojze4g/yW+b/+f/AtgV9Hgs5x24D/1wxbPV+3gRrvYgfxV9E06m3iHt
zoPoFeppI+f9nd7hr6j1QmauxOnqmmeqWWpGFWIN0+8Y8XYVXUsfNZ3rcE1N
fC8mvg3nkBc0K/ALRdfwM7YnM9v0hkeo5xmkl1zX0+M76qXiXdTNGwH7e2Kb
3vQfiXPIfwY5Rf6psvlSDaczcyRudEMH9JZT37Ib856dOBfXmuFO8MOZ8yvm
f28jMV5vRNzfjr6m1TPoBzuQ+XbpjR7l+83M70uYp1PfKHHfDf4RbNsz317l
WA8XLQ3fBt2AGd2Pumctzuf4nq/77etmZ2CrDe+KONfb3Fr1W9Qb1S7rP0C3
XzutXdBN1C3UTszxPV71bupmPEUtvySuTT1qdw9VzI12WLugGWu22gnNblDz
6fAMe8B+X/Lu6MZMgu1NfXuE+ZtdWaz59upma7a7Ms9K/f8LJAH0OA==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7538700201254268, 0.8329215431101401, 0.9138312766430465], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFV1lslFUUvu1MZ2s72z8bxZCIMYqKBARKCBYF2RpTIiBCKWuKIWAZKYlt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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtllls1VUQxg/t7b1tkdu79N97ASOBahTUEBfAIIGE2rLEQARFaAsCqQmC
pYIbMZKo8AAIUQNaMIJSCCAgQtsIohFEiDH6gKhEfECsCxhRI4u0CuLvy9eH
SWbOzJn9zJz+s5smzS8IIWwGYsDUXAjTgLbSEFqBd7Mh/JMMobA4hFsS4FEI
L6dDmFsUQk1JCPXIvpgJYXmPENqRr4P/EPAJ8slCdML7Gh3jubsL/Zfg/Rv5
7tPwX0H3C+i7t9D848iegH8T/AbODkAfBA4XWaYK+VPI74a+Fp3rUiFszYdw
3TUhnIG/rSKEt4Fq7DcT0CDudnGnE//ej4dwGX/XJ+3/zJ4hHAFvQ74Le4s4
y5aHkAGGSRdwhvtX8GdMt78zwa8vs/3D2PgV/BHOPgc/BlRCf49/eWRvJ7Hn
0P0SOgqgh5Ofc8heydn2U5yNwNbdwCZi2QwUIh+IZzb8pfDPg7cAbxDLMWJ8
C/zNvGPLQn+F7o+BEQnnWLE9DMwqdYzPgY8CRkKPh76EbCfQkLD+pfB+w5+J
sofpCN1NKce+iLOL+LMu61pKx2LkhycdSw36foF3GpiScDxLiGUpcAr6fvmI
7kzOtWxAvnfOOVUu5eOj5ONncpaDnox8itxtwocVccdci3wKmWU93JMjsX0C
me3oirAxClt1OffeFuixyPYrM7+desyAX553LH3R9yT92Bf+FvgDsLEEX4ag
szb47DS8s+ibUOQaDiYXy9P2/UZ6oJ18tALvxdwzh7hfivwDpa5BIfpHoWNC
gWNekHaOlVvF+Be+xDhbxv2j1K9nzm9AvS8dVfg7A/070T0UmW0px6jYFnP2
E7rndPebajYF/Ed0jAHvhf5e6N6Qd298g87V4GuAOaW2ORnZ/dgbQj63lzq3
6ci9pxzfgO0ydJzHViMwSL4AF8Afhz9A9U2Zt5I7H4BXw18D/RqwKmOf5etd
5DeHrkk529IbVe72AAUx53AH+O/cWdjDNiuRHwDsgdcKJMH7YXM5tiLuJ7K2
KVsb4+591Ui1Uf7/jvzG1ItVnHVFnjnqt+oSz67z0KNL/D6aub82615S/i5E
7mndl0wN/n2I/mbwV4Gd5PKdvGejYhhIPTrQ2Qd8WIFzsT9lWeVkEvw4ZxPB
O7C3O+uZqll6a8K9fA82W4vds5pVrWnr0sxamPaM0WxRT2rWnIH+stg9MD3y
zNX7PAS9AFsnU35LtyHfF1t9gH3oujPmWPZkbVsxzePu3MizSvo0W+sj69KM
q4Duoj6r4+6BanhVkeeZfNYsbII+2i3/GHhj5Lejs9rIParcHiz225wWGVe8
UyPXQPwDnDVh78HIuObVXvIxmPq/rnwRQz3x1VX4LQ8BFqj/035fykkbunZy
drXYM1yzeEPWu0I+FWbdY+qt9cR0ld5swUZTkWduosIzWLNRNask9s7IvaP6
1ePL9HL3nmq+A97QMu8n2WxH96mUd80KzkaX+03rLceAw5rF5ca1M+/j7nH1
f9w7S7tiXOTZpRx/pFxE7m3NFOV+DNDWzf9B+yttWyeRaaQ3P0u51ss4m673
ljaumfIntiqxvyrhGavZPivyLlMPdeS8M5R7zZQZkWsiWjv9iYx3gGZ/f+7X
wJ+XNi2fFMvatPOpmPR2NyY9i/SGn83YZ/k6MO7cz9fOSvjOzRWeOZo1qrl0
xaAnlJivWfFHxrteM0O5b4EeH3cNmoj/i5R7Q3+WrfC2ZDxfjxRZdliZcd1p
Trkn1Avq8f9yntGazZpBzyM7Oe23VIu+cdAXI8+W7zgrqvBOVb/IR+1y3ZGs
dvq3ke9IVn+afRnvCO2GOuw1EHt5xnzt0Dy6egN3xPxmxdubsaxkOvFlZcaz
VfE+w/2e2Dxb6B2mWmjHabepJh1J51y51k7+lHjn5f3XkY1F0Bvznq270HE5
8sxSr+vP05j0GxCtP8M46Jasd7venHpJO0G7QD31P8TYX64=
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7754755959605135, 0.8512813304964518, 0.9082766325201515], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmXmQVdURxu8szGPezNvmbUNIgqIRRYxgtFIuJQgYoxQEISYaFSMQQFwQ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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3mw1mMUx5/eu7zvvd3e5b7v7/29XFPKlkJCyNJyyTJGWmyhKCEtKlEY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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7961881464654493, 0.8581884141843953, 0.8989974284551325], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmHmU1WUZx38zl2HuMnPvnbnbQFASwzojmxsgmnBOR7NkE0+26OGIoijo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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3lw1dUVx2/eI3lJHnlL3nv5PRhpwbIFQmRTDJuEmY5WJQkUR9qOjuNU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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8169006969703851, 0.8650954978723389, 0.8897182243901136], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHuwllUVxl/O5bucD777DSNLrmYe5SZ3mHAccOKikRRdBopG0LE4MJUy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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3mQz3UYxz+W3+7vt8vv2N9J0mBRyhlC1lTTtE2FkkrHICZqEpoOimnp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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.837613247475321, 0.8720025815602823, 0.8804390203250947], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHmMVdUdxy9v5s32Zt4y7933Rg1aFi0WhQYFEaW2oLIYa0PaGmEoqVVB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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwllleMlFUUx+/OziwzzOx8076ZWQm9CCIldBYQS4RdiCVEJFIkGhFWBMEH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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8583257979802568, 0.8789096652482259, 0.8711598162600758], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHuQ1WUZx397O+dwztk9e5bdPWfRrAUmDIabAquAjonC4iQ0pWSYNaml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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwll2twldUVhndu53w5iSfnwne+E8eqBAYQRVBJQkl01KKArdWxXGzRcVRE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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8790383484851926, 0.8858167489361694, 0.8618806121950567], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmXl01cUVx3/hhRDz3ktelpf3AC0IBcGCIrhVQUUgoMbSIpt6tIoc61KX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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwll2dsVWUYx99ypZTecmdvz6kitGIZCg4EUUGGUirLKCJxRMA60QgkJiqI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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8997508989901284, 0.8927238326241129, 0.8526014081300378], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmgtwVdUVhs+9ITeY+0rCNfcmJCRRXlp0alVai0KK2hltnYLYUqgPxFrH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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwll2dslWUUx58W7i30lve25dL7qoVSoQKKxImiLBkmEuPAUcWYQDEOVBqR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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.9204634494950642, 0.8996309163120564, 0.8433222040650189], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmFtwleUVhr+9gSTuvf+9k/BrNnasxOChNxWUkhOOWgIyFFpDRdJanHq4
sCWWSu1hQKAdDoaZ4khTD+SinmrroTO1U3RGrRWpgNi7akHbC1rGthIOCmUc
bSzp+7hehl58+df//d9hrXet9a61037zisXfKqaUHiikNEHPUf25QxP/lryr
nNKrGu/q/blKSs9qHJb8gcYKra9r9GQp/U6jW2OhvvdrfFF7d+t9l0ZeiO8v
es1FeUoXatzbmNLf6zqvmtI9kue0pjSjGns5m31fkvzjUkrzJU/UmKr952sc
ktyrZ+Mk6VuMOxf73v4mrZUNN6bQk2+cN1tr/9qQ0smJKW3Nwo6zC6EX+iEf
nRB7sGtE927T+LbkV2o6e3Lctdg2su6nwqZfe4f1fFO2/FnjNn1boLkvaPxE
520sh7xBz7k6q6AxHbz17W86Y4LerynE8zLNv6eRJF+QAq+i5Kslr5f8juRx
yV2SO7PY/1+d2+f5YxqnNNo1jljmrlPlWHdA6+cV4swZKfyDPldJfvOclD4t
/M+TzbeXwg6ZnMZk05805J60xHiB1SLwrkRsDJbCZuzlPM69MsX3RfYL9rPm
sORzNfejSWFPt2MDHc7T/XdW466F3ntdIexK1nlMsXKt1k9NgTtyl9f0GDfw
Bnf0YY47mT/sNe3ey74jjtU9Wdy1Vs81xKrk1/TcmwUOfN/teD6q917fhZ57
PH+z4uSdeuTPMs3dqHFCOow0RF4d0tin9V/T/DTt/aPl1/Wco9FXCduZv8Fr
HqxE/JOPo8qb23TH6mL4/Ij9+wMp+H2Nj3XPznLc+0o5zllmHXhHnl2Iee5E
vrsUzw7HXq/lUY1OY9Vn/Q41RK7y5Ds+fEDn7tH5q/X9rkrE9XLpuF1+XFOM
eMgcY/iq374reh6fsqbivCBXwTQ3B+zKzuzF9/gMXYinTvu2274gXhY63vAh
/ttRjDN2OyZXVULX03m3ynJDFvLlzp0u207eoRt77/Le2/U+LQ/+usc6jTrv
sA9ewMaK8529YET+kacHC+FXfEpeLs/Cn+A/7/8wmWt5dGKsudQ5PtdYMcc3
9sKLA4XgPXJ4SOe+rDsXNUVczExxd5/3wttwKJx2fzl0Hp4Q+l9rTiPuxhqC
e59vS+lW+fUZeFG6j2isLMbZYz6f/ewDP+bYz/xnFCeb4R/n117n2HzjMNNx
vs0yuPANX8Dx6AkHflYcdVYeMVZoCq7H3h82p/Qp+WJTOWzHbvz+uPR945yQ
iRVihnhp0hkdZwd/whObzBXE2xLXHWrFTteLT7AxPrvNE/gU3lri2D5mvoIP
4U84kTPh3sEs/JyawkdTChFH2EHsUWcGfC9kM94anLFZmN+p94Ol8Fe3cQCP
Hsvoj13wxogxJL6IjUHHzJDOOap8/Lgxal2POZwaPOL15OM816DjKXIDedDc
c7lzttv5CC7gAyYPC//PtwZfbSlFjJAj2yuBD3c9pDWPNAcX7XGvAI6zqlH7
ydlNxYhXYmCadH4kj5x/Q749tx62HrdO6AOXwqm8Uw+7jAnyrCywzWwbOuPv
Ts9jN/avdL1fYfz7zSG8vyAdvqkYmlKO2G+cErYRk12uv481R1ziy8YseISe
BfyOG7dhamMWeDAHrnAseqAn7zN11+c0hkqh7yzbQo2EO8ZdA+EIOAHevcLc
O8v9wNXmt2uMz1U670qN+3XmP4Xx9a1Ryy4UlhfXI65PmVOmm2fwJ/lLzpBH
1Lq/aO0ttcCKgZ/w0Vc1f309+kT0ppaQ46d5ud09A3aD7dPy5xN55MFL8vsd
tTgLroIjuK/b9RU+5515zrxE6xe2xPtsY9vhWr/A/VWv8bnAPIc92EKvt9Vc
1y08utoiTqnTBx3fqRK5CkdX3E+BCTnEPHn0kXw9JHvHy3FPh31JLR20TtiK
r7GXnKi3RF5Q56m7yPiDHgjfIt+nMbkQvc+Yey3uXZ6d6dt4jpu3tzp/fyEs
f6kx3hj5iL3oiR/wx/sp+uuf5VGnyP9NpahV9AN3e826LDhrsjE/YB9krl/E
25Oy44mWqPn4r2DuhbeS5VOunfj9hO59Mo919AHoeMj1lhyBH8gTZGKVZ6Pn
8eUc90LJvR0+xQ/j9sXFOv+71aizL+meHW1Rp+hNOl2P8XGj8xE+7nZcsq+j
LfYecA+EL8EGnNh7g/0FX10inww1R52k9+J3ApwBL4z4NwLcsLQt+KEkff5Q
i7hb0RrvyAPm+il+jlje615ljXmCb8QPv73ILWJjqXT4TTVk4v91c8k/NP8N
94TYuNa9K2tPtMb6ba7VMx2H+9z//V77vlKPmrUxi1gkJpfZdu65SWvezSNX
6D25j7velvxWLX5r8btstXmUPICTyan3df/T1Ygx6iP2wPnE+6u2C+yJRe6i
d0VmPTaAB3GIHc/Y9i/r/iX1WLtTeO/KA0NqJHxOPjP3lOfhpwFzFH0x+/Ap
/uD3LHkEPyPjR57PWSZm/1WKuH1c521pi/fr6lHPOOdX1eBH8MXuUds+R2v7
2sLHYPBoc9h1ViVyjd7gt9WwAT1XysbvaExV/L5cC9/gF3537qgFd4HZMfcY
+G+feYfnftdCOOCkeZz1YA3mb+met9uiZuDrDVl8m+88JAexfbsxeaoa/kPn
/b4L7lraGn0V9eGieuQheUQdusJctZP4z6NO8nuIGCJ+iPO1/j213joQb/ul
1/O12LtZ6//THHzM2nVeD8cT3+TvUectOUtsP+gehrwEvw/cg63zHvic++DP
za5V7L9Pd01qCZ/AH7+uBofcoguLeXAJHAI+zJO/dfcJ6L7evynoJV+oRT9A
vcvdbxBvy/xbjH4BPKjR39P5P8+jRyZ32p0/1PGi+y54FkzBhxyDQ+HYNf5N
Awb8T+OkOf90XqFPr/n7E3xKwSnEM/3pMfdd6+wL8Jlrjp3hGBh2r9Ljefic
szfYXnrXVf4dxNnDPh+fL2gJO1m7MTtT4zY63rAVm8mp8y2DKf3po3nUKGzl
O/bepPO+3hK2nt7DPL6hRsD1W93TkiOP5VEPJrqffc81esB7uWvMtRCepz5s
cb/6oeJu+uToa7u8jjX/A0+pfwg=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwllFuIjVEYhj+z52DY/j177z/7n3HIOCuFHKKIMldiYhzGoRxTToNBKBQX
M/ZckBxmai6EcZgZiiJFZIyU5MohRJELzEihyHk8n/fi+1trfWu97/t9/7tW
6cpNFRtzzGwxkUsUJ/mkzRrzzTpiZl0psxuBWWdMay2h2YaE2ZzuZt2I/uT6
RmbZnmb9epidKjKby7yd+SegShjvZa2WeQ0xPmPWGiiXZf9JcolisyW9zL7A
MZb8YDgOFnCG/Gq4dqLnN9yz4Mshdw/Msjj6WFvEeDFxF+z3FPKQ8aNIXEi3
Z5xfT+whlwv+fHLzInG5xktomQZnPVzpbmZ32NtB/ir4HzizkHFlpJxzDIH/
Ame2kDuExlOhNLvW38wXQNqXeu71+t9GW04/ZzN4gdbJeWan2d9E5IL3i7Xq
hGr02mrB7APXUnpyOKaeTQU7BKMkLo3nOHuGeMe4C77n5J8Srzg7hf2VjO+g
rwr8S+xZBfbNQNiu2bV8B789X5q+Rvrn/q+fxvWvV3KmOkf//B1cdWC+AesN
mO3krjMvhmsz8z9gPQCjgrPfOPOE8duUal/D2kD2Po40rgIjSorTuQazZyL5
axlpPUA0w/clkLeMaINvU0r/opOYwfnbCY0L6Oc1xnXgWVw9qEHPuIy8VYfG
teTbwPtILbty5N3nCXnBPezeO1EkLe7BGPPOUHtLmT/qjf7e8nYba63kzofS
Vkz9Y8C+D385/OUsfw6E4Wc7yC8Dezj5erQcgfM63O2htL8Gbx3z6eht7KGe
uncqU+JyD22jkUNDec3/gXu/XyBv+x1w7lsJed81HGfvqrTuht+hYeSHEgPY
fxT8EYyfZeSVhp7y2jHWCuPynHvnJfPdcXloJPMVSXl3FBgHMqrRa3MPzkyq
597rEXnq9RbEDchXz5uT4nSufeRHp9UT74V7sAy8y4F66T0Y5Pef+FUgzXlJ
eci94xqznB0SqRbv6Q7wtqd1l/yN2E/uB3u6yP31NyGUh9277mmvrYl8Q0w1
uncnBPp37mF/y4YHqsXftK0pedy97TUVgtU91L9zDL+bLUnV5nf0CmfPhrqb
/ub8LJIm1+IePUkum9Fb4TX6W+N3yu+SvznZSD33XnsNFxlPyuhu+JvxD6ND
zUY=
              "],
              VertexColors->None]}]}}, {{}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{6171, 4224, 6170, 6355, 6354, 6395, 3700, 3613, 6344, 
              5122, 4963, 5278, 3819, 6369, 5156, 5939, 5282, 3821, 6024, 
              5281, 5924, 5923, 3701, 3623, 6182, 6183, 6181, 4232, 6184, 
              4191, 4912, 4344, 4944, 4448, 4449, 3628, 6280, 4345, 6279, 
              4346, 6281, 4025, 6103, 6105, 6104, 6185, 4520, 4521, 4372, 
              5951, 5952, 3760, 4850, 3761, 6231, 6232, 6230, 6234, 6233, 
              4030, 4031, 6346, 3633, 4675, 3638, 6348, 4349, 4723, 4722, 
              3776, 3945, 6373, 3944, 5354, 6403, 3825, 6029, 5285, 5287, 
              5286, 3764, 3947, 4056, 4057, 3948, 5289, 3826, 5288, 5291, 
              5290, 5317, 6082, 3763, 5956, 5955, 3946, 6074, 4034, 6377, 
              4035, 5130, 4719, 4032, 4942, 4033, 4426, 5315, 5314, 4026, 
              5411, 5128, 5129, 5678, 4259, 6364, 3779, 5975, 6407, 3940, 
              6073, 5944, 5945, 3759, 6080, 5309, 5283, 5284, 4982, 4983, 
              4984, 3941, 3953, 3773, 3939, 6362, 3756, 5157, 5158, 4980, 
              6025, 5351, 5352, 3937, 6372, 3938, 3772, 4863, 4864, 4377, 
              6389, 4376, 4674, 4256, 6415, 4257, 4255, 6228, 6229, 6227, 
              4258, 3755, 4848, 3754, 5941, 5940, 4229, 4428, 3624, 6172, 
              3614, 5844, 5843, 4005, 4695, 6428, 4694, 4838, 6357, 6356, 
              5922, 4357, 6293, 4190, 6171}],
             RowBox[{"-", "7"}]],
            Annotation[#, -7, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3615, 5846, 5845, 4338, 4697, 4696, 5848, 5847, 5302, 
              3850, 5301, 3851, 5303, 5849, 5305, 3852, 5304, 3853, 6276, 
              6277, 6275, 4339, 4698, 4006, 6099, 4007, 5850, 5851, 3616, 
              6173, 3625, 5860, 5859, 5943, 5942, 3757, 4378, 6390, 4379, 
              4342, 4343, 6345, 3626, 6175, 4230, 6174, 6177, 6176, 4193, 
              6150, 4192, 6396, 5926, 5925, 3703, 3629, 6187, 6188, 6186, 
              6190, 6189, 4194, 4839, 4195, 6379, 4196, 4347, 4913, 3774, 
              4380, 6312, 6028, 3824, 6027, 3823, 6295, 6296, 6294, 4358, 
              4840, 4197, 6201, 4235, 6200, 6359, 6358, 4522, 3705, 3634, 
              6347, 4348, 4915, 4914, 5245, 5244, 4518, 3805, 4943, 3804, 
              4517, 3639, 6349, 3827, 3796, 4843, 4217, 6382, 4218, 4678, 
              4677, 3752, 5361, 5362, 5360, 5359, 5152, 3929, 3964, 6375, 
              3963, 3961, 6374, 3962, 3828, 6030, 6032, 6031, 3710, 3643, 
              6205, 6206, 6204, 4238, 6207, 4201, 6422, 4841, 4202, 6153, 
              6155, 6154, 5658, 6036, 3831, 6371, 5164, 5165, 4385, 3778, 
              4921, 3777, 4254, 6384, 4253, 4842, 4206, 6212, 6213, 6211, 
              6226, 6225, 4252, 3719, 3653, 6351, 4416, 6338, 4415, 4866, 
              4205, 6209, 6210, 6208, 4239, 6313, 4382, 5805, 5807, 5806, 
              4381, 4384, 4383, 3765, 6034, 6035, 6033, 6203, 6202, 4237, 
              6117, 6118, 6116, 4064, 6119, 4055, 4726, 5876, 5875, 6283, 
              6282, 3869, 6053, 3868, 6406, 5428, 4043, 5316, 3863, 6052, 
              3862, 6405, 5413, 4028, 6106, 4717, 4718, 4029, 6107, 6109, 
              6108}],
             RowBox[{"-", "10.5`"}]],
            Annotation[#, -10.5, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{3771, 6363, 3951, 3936, 5275, 3818, 5274, 5277, 5276, 
              5300, 5272, 5273, 4960, 4961, 4962, 3935, 4004, 6376, 4003}],
             RowBox[{"-", "10.5`"}]],
            Annotation[#, -10.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3617, 5853, 5852, 4340, 6278, 4008, 4699, 5854, 4964, 
              4341, 5769, 4009, 5401, 4011, 4966, 4967, 4968, 4231, 6178, 
              6180, 6179, 4018, 5404, 4017, 5405, 4706, 4713, 4714, 4369, 
              5946, 5947, 5948, 4849, 5950, 5949, 6191, 4233, 5972, 5973, 
              4865, 4250, 4950, 4511, 4512, 4370, 6388, 4371, 4234, 6192, 
              6194, 6193, 4715, 4554, 3635, 5867, 5866, 6111, 6110, 4451, 
              5417, 5418, 5414, 5416, 5415, 4559, 3640, 5869, 5319, 3865, 
              5318, 3864, 5430, 5431, 5429, 6112, 4044, 6410, 5927, 6360, 
              6380, 4198, 6412, 6013, 6014, 3806, 4932, 4933, 4405, 6329, 
              6331, 6330, 6332, 3644, 4397, 6391, 4398, 4396, 4400, 6392, 
              4399, 3790, 6366, 3980, 4219, 6094, 6095, 5988, 3791, 6071, 
              6072, 6068, 6070, 6069, 6237, 6235, 3648, 5881, 5880, 4065, 
              4464, 3714, 5885, 5884, 6120, 5475, 5883, 6350, 5882, 3878, 
              4957, 3879, 4577, 4740, 4739, 4585, 3662, 5032, 5031, 5530, 
              5529, 4474, 5534, 5535, 5531, 5533, 5532, 4594, 3671, 6217, 
              6218, 6216, 4244, 4856, 4855, 5965, 5964, 5963, 4243, 6215, 
              4355, 6214, 4242, 5960, 5961, 5962, 4854, 5959, 5958, 5957, 
              4241, 4442, 3670, 4917, 4352, 4437, 4436, 4916, 4240, 6309, 
              6310, 6308, 4373, 5527, 4096, 5526, 4097, 5528, 4080, 5781, 
              4351, 5780, 4350, 4737, 4079, 6128, 4081, 6125, 6127, 6126}],
             RowBox[{"-", "14"}]],
            Annotation[#, -14, "Tooltip"]& ], 
           TagBox[
            TooltipBox[LineBox[{3960, 4679, 4676, 4959, 4958}],
             RowBox[{"-", "14"}]],
            Annotation[#, -14, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3661, 4738, 6284, 6285, 5782, 5784, 5783, 4098, 5554, 
              5555, 5551, 5553, 5552, 4746, 6288, 4353, 5788, 4354, 4767, 
              4766, 5904, 5903, 5331, 3898, 5330, 3899, 5332, 5905, 5334, 
              3900, 5333, 5336, 5335, 3677, 4607, 5558, 4114, 5557, 4115, 
              5559, 4480, 5556, 4113, 5049, 5050, 5051, 4595, 5538, 4100, 
              5537, 4101, 5539, 4475, 5536, 4099, 5033, 5034, 3663, 4586, 
              5504, 5320, 6423, 5321, 6059, 6058, 3880, 5476, 4066, 5020, 
              5659, 6157, 6158, 6156, 4203, 6236, 4260, 6314, 6315, 4412, 
              4923, 4922, 4868, 3797, 4523, 4524, 4417, 4951, 4431, 4432, 
              3711, 3966, 6086, 6088, 6087, 3965, 5173, 5174, 5175, 3981, 
              6055, 3870, 6054, 6057, 6056, 3829, 4934, 3830, 4525, 5246, 
              3807, 4898, 4899, 4314, 6264, 4313, 6265, 3645, 5835, 5434, 
              5435, 5433, 4046, 5436, 4456, 5432, 4045, 4996, 4997, 4998, 
              4560, 5421, 4037, 5420, 4038, 5422, 4452, 5419, 4036, 4988, 
              4989, 4990, 4555, 4716, 4552, 4553, 4949, 4948, 3860, 4551, 
              4022, 5307, 3857, 6046, 3856, 6404, 5406, 4021, 6100, 4710, 
              4711, 4019, 4978, 5861, 5862, 4705, 3620, 4971, 4970, 5775, 
              5774, 4010, 5770, 5771, 4965, 5855, 3618, 4700, 4701}],
             RowBox[{"-", "17.5`"}]],
            Annotation[#, -17.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3669, 4745, 6286, 6287, 5785, 5787, 5786, 4112, 5576, 
              5577, 5573, 5575, 5574, 4768, 6134, 4131, 5578, 4132, 4621, 
              4620, 3682, 5341, 5342, 5340, 3912, 5343, 5911, 5912, 5345, 
              5346, 5344, 5348, 5347, 3683, 4623, 5583, 5584, 5582, 4135, 
              5585, 4485, 4486, 4133, 5579, 4134, 5581, 3901, 5562, 4118, 
              5052, 5896, 5897, 3728, 6133, 4116, 5560, 4117, 4597, 4596, 
              5541, 4102, 5540, 5543, 5542, 5136, 5134, 5323, 5322, 3654, 
              3720, 6129, 4082, 5502, 4083, 5679, 5680, 6239, 6238, 4261, 
              6416, 5683, 4263, 5682, 5685, 5684, 3930, 5153, 5369, 5370, 
              5372, 5371, 3753, 4685, 4686, 4221, 6169, 4220, 4844, 3798, 
              6000, 6001, 6038, 6037, 5886, 6333, 6334, 6015, 6016, 6017, 
              4935, 4936, 4407, 6335, 4406, 6336, 3808, 4728, 6152, 4199, 
              6151, 4200, 6300, 6301, 5132, 5929, 5928, 6419, 4361, 6299, 
              5792, 6297, 4359, 6298, 4360, 4901, 4900, 4460, 5450, 5451, 
              5008, 5009, 5010, 4564, 4565, 4049, 5000, 5870, 5871, 3707, 
              6113, 4047, 5437, 4048, 5438, 3866, 5424, 4040, 4994, 5868, 
              4454, 4453, 5423, 4039, 4991, 4992, 3636, 4556, 5412, 5311, 
              5312, 5310, 5410, 5409, 6102, 4024, 5308, 3858, 6047, 6048, 
              5408, 5407, 6101, 4548, 4549, 4020, 4979, 5863, 5864, 4709, 
              3622, 4974, 4973, 5779, 5778, 4012, 5772, 5773, 4969, 5856, 
              3619, 4702, 4703}],
             RowBox[{"-", "21"}]],
            Annotation[#, -21, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3676, 4765, 6289, 6290, 5789, 5791, 5790, 4130, 5606, 
              4152, 5605, 4151, 5339, 3911, 5148, 5607, 6361, 3739, 5608, 
              4622, 6138, 6139, 5609, 4155, 6223, 6224, 6220, 6222, 6221, 
              3692, 5676, 5677, 5611, 4156, 5610, 4157, 4493, 3740, 4801, 
              4624, 4625, 4861, 4860, 4494, 5612, 4158, 5088, 5089, 5090, 
              4626, 5588, 4138, 5580, 4139, 4487, 3734, 6135, 4136, 5586, 
              4137, 4609, 4608, 5564, 4120, 5561, 4121, 5565, 4481, 5563, 
              4119, 5053, 5054, 3672, 4598, 5544, 5325, 5326, 5135, 3886, 
              5324, 4084, 5503, 4085, 5505, 5661, 5660, 5681, 4262, 5808, 
              4388, 5827, 4413, 4925, 4924, 4869, 3799, 4526, 4527, 4418, 
              4952, 4433, 4434, 3715, 6090, 3968, 6089, 6092, 6091, 3967, 
              5176, 5177, 5178, 3982, 5373, 4570, 4571, 4528, 3832, 4727, 
              5823, 5742, 5743, 5247, 6018, 6019, 4902, 4903, 4315, 5744, 
              6267, 6266, 3649, 4572, 5454, 5455, 5453, 4059, 5456, 4461, 
              5452, 4058, 4999, 5011, 5012, 4566, 4567, 4051, 5004, 5872, 
              5873, 3708, 4457, 4050, 4993, 5001, 5002, 4561, 5426, 4041, 
              5425, 4042, 5427, 4455, 4851, 4852, 4558, 4557, 3632, 3704, 
              4450, 4027, 4985, 4986, 4987, 5671, 5670, 4519, 6196, 6197, 
              6195, 6199, 6198, 4023, 4981, 5865, 3630, 4550, 3627, 3702, 
              5124, 5123, 3854, 5306, 5858, 4977, 4016, 5402, 4013, 5776, 
              5777, 4972, 5857, 3621, 4704, 4707}],
             RowBox[{"-", "24.5`"}]],
            Annotation[#, -24.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            TooltipBox[LineBox[CompressedData["
1:eJwV0ltMz2EYwPG3hpg5JjGnwuZfDomwchHduDA64KrNhVEYlXKYzakuMKfc
ZE7NITdICBtzTA6bG3K+cNjERVS4MJuL5vO7+Pb9PYf3fZ/37Z+6oryoLC6E
0MufpkEhrB4awkpUJIZQjlFJIVyWnzg8hJrBIYznap7A8SND6BwYwu7kEPbg
9DAbyZ3iOG5Sq7D2jtpdnMQgcT0vGhJCh74S8ST+gU5Uiu+pV/F9TuTe8n2Q
gAx7Lra2VD4m7kI3Nopb9CfxGW7AWbSo7eNu676MCCFNnI5m8R/3zFObj8ly
B9xrQ3R3vHRGtp5D3uAgXts311uslc/jt+J5/Ibn83v5I/rqsB8f5Avly+SX
8UdxES/FEnyK9rF/vd6PvlPVtzo3hUvUS/HdXB1ol+vvPdOjOaM3dI91Zu3i
XeauRg1i9kqL9lPbpvaBt/NO7MAC513nTOdd4xl8g5/oS7auyt6VWCO3GiW+
V6HW+S94h74e5xwWF3Op3FwzF0dvYH3MfjnibMzx+5iNAmc2ql1EQjSXnv1y
8dbGoUn+grueR1b0v7N2Fv/Vd853A+Kx3JqXetswxT5v9O/V846niV/xVH7L
ifq/6fuKduRGbyVfq/83zxN38y/8xHNzppolBePwNHoTHNX3TG2mu2ahx763
zFGo55q4nzlP+O7LzVEOV1AgvspbeIa+TExHmbjVDA8xXT0DY+Xz5R/73syP
eBMXcD4WY734Nifw5SjHjdHb4RJacdNco7nN3GP4OI4hZv6fcgP9fgagzp0W
io9wmloO/4v+Z/yAr+OzN/uE/yZkk6s=
              "]],
             RowBox[{"-", "28"}]],
            Annotation[#, -28, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3691, 5918, 6292, 6291, 4356, 6385, 6142, 4174, 6378, 
              4386, 4387, 4867, 4247, 4857, 4374, 5804, 5674, 5675, 5166, 
              5167, 3767, 4859, 3766}],
             RowBox[{"-", "31.5`"}]],
            Annotation[#, -31.5, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{3775, 5974, 4251, 5131, 3706, 3637, 4995, 4563, 3867, 
              4725, 4459, 5446, 5447, 5443, 5445, 5444, 4569, 3646, 5016, 
              5015, 5464, 5463, 4463, 5470, 5471, 5467, 5469, 5468, 4576, 
              3651, 5022, 4575, 3875, 4730, 3874, 5482, 5483, 5480, 4068, 
              6123, 3716, 4465, 4070, 5477, 4069, 5484, 4318, 5745, 3656, 
              4435, 3969, 5179, 5180, 5181, 5393, 5392, 5978, 5977, 5976, 
              6317, 6316, 4265, 5184, 5809, 5810, 5812, 5811, 3781, 6251, 
              4295, 6250, 4296, 4889, 4888, 3800, 4890, 4891, 4298, 6252, 
              4297, 6253, 4529, 5725, 5726, 5187, 5186, 5185, 4266, 5182, 
              3780, 4747, 5980, 5979, 5183, 4264, 4749, 4748, 4600, 5056, 
              5055, 5037, 4122, 4482, 3729, 5899, 5898, 5058, 5567, 5566, 
              3902, 4789, 3903, 4611, 4792, 4791, 4628, 3684, 5095, 5094, 
              5620, 5619, 4496, 5626, 5627, 5623, 5625, 5624, 4643, 3693, 
              5112, 5111, 5637, 5636, 4504, 4822, 3921, 6421, 5349, 5801, 
              5800, 4364, 6303, 6304, 6302, 4365, 3745, 5151, 4366, 6305, 
              4216, 6168, 4215}],
             RowBox[{"-", "31.5`"}]],
            Annotation[#, -31.5, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4911, 4920, 3770, 5971, 5970, 4375, 6311, 4249, 4846, 
              4845, 5803, 4367, 5669, 4228, 5154, 5155, 4847, 4227, 6307, 
              4368, 6306, 6387, 6386}],
             RowBox[{"-", "31.5`"}]],
            Annotation[#, -31.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3694, 3746, 6143, 6144, 5635, 5639, 5638, 3922, 5628, 
              5629, 5096, 5913, 3685, 3741, 6140, 6141, 5618, 5622, 5621, 
              3913, 5592, 5593, 5078, 5906, 5907, 3735, 4489, 4143, 5057, 
              5075, 5076, 4612, 4774, 4775, 4601, 3895, 4770, 3894, 5547, 
              5548, 5043, 5138, 5139, 5891, 6163, 4207, 6165, 6164, 5214, 
              5723, 5724, 4299, 6254, 4300, 6255, 3834, 5039, 5040, 5041, 
              4284, 5702, 4282, 5701, 4283, 4871, 4870, 5213, 5212, 4578, 
              3881, 5727, 5133, 4530, 3833, 4906, 4320, 4532, 6424, 4531, 
              3809, 4732, 3810, 5252, 5251, 4319, 5026, 3657, 4579, 5489, 
              4072, 5481, 4073, 4466, 3717, 6124, 4071, 5485, 5488, 5487, 
              3876, 5472, 5473, 5017, 5877, 3647, 3712, 6114, 6115, 5462, 
              5466, 5465, 3871, 5448, 5449, 5007, 5874, 3642, 3709, 4724}],
             RowBox[{"-", "35"}]],
            Annotation[#, -35, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4062, 4731, 3713, 5879, 5878, 5019, 4063, 5474, 3877, 
              5492, 4075, 5490, 4074, 4468, 4467, 5494, 4077, 5486, 4076, 
              5493, 4581, 3658, 5027, 4580, 3883, 4733, 3882, 5508, 5509, 
              5506, 6131, 4086, 6130, 5932, 5933, 6160, 6162, 6161, 5255, 
              5254, 3811, 4907, 6021, 6020, 5253, 4321, 6273, 6274, 6272, 
              4322, 4439, 4438, 6326, 4403, 6004, 6005, 3801, 4892, 3802, 
              5375, 5376, 6098, 6097, 3985, 6409, 5937, 5938, 5378, 3986, 
              5377, 5380, 5379, 3933, 5137, 3932, 5350, 4270, 5690, 5688, 
              5689, 5060, 5059, 5042, 4123, 4483, 3730, 5901, 5900, 5062, 
              5569, 5568, 3904, 4790, 3905, 4613, 4795, 4794, 4629, 5098, 
              5097, 5077, 4161, 5630, 4497, 4498, 5914, 5100, 4162, 5631, 
              3923, 5642, 4176, 5640, 4175, 6145, 3747, 5114, 5113}],
             RowBox[{"-", "38.5`"}]],
            Annotation[#, -38.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4178, 5641, 5649, 5648, 4505, 5645, 5646, 5099, 5115, 
              3695, 4644, 4806, 4630, 4631, 4803, 4802, 3914, 5594, 5595, 
              5082, 5908, 5909, 3736, 4490, 4144, 5061, 5079, 5080, 4614, 
              4776, 4777, 4267, 5194, 5983, 5984, 4773, 3782, 5192, 4271, 
              5196, 5197, 5198, 5730, 5729, 4533, 6259, 4303, 6258, 4304, 
              4896, 4895, 3803, 4893, 4894, 4302, 6256, 4301, 6257, 3783, 
              5815, 5816, 5814, 5813, 5195, 4269, 6318, 6319, 5981, 5982, 
              5394, 3998, 5191, 5190, 5188, 3970, 4440, 3665, 5746, 5747, 
              5514, 4088, 5507, 4089, 4470, 5892, 3721, 6132, 4087, 5510, 
              5513, 5512, 3884, 4734, 4735, 4583, 4582, 3659, 4584, 5497, 
              5498, 5491, 5500, 5499, 4469, 5495, 5496, 5018, 5023}],
             RowBox[{"-", "42"}]],
            Annotation[#, -42, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3696, 3748, 6146, 6147, 5643, 4177, 5647, 3924, 5633, 
              4164, 5104, 5915, 4500, 4499, 5632, 4163, 5081, 5101, 5102, 
              4632, 4797, 4798, 4618, 3907, 4793, 3906, 4778, 4124, 4617, 
              5065, 5662, 4210, 5193, 4209, 5710, 4287, 5220, 5728, 6260, 
              6261, 6006, 6007, 6008, 5239, 5240, 5063, 5064, 3673, 5713, 
              5704, 5705, 5703, 5707, 5706, 4268, 4872, 5216, 5215, 4285, 
              5708, 3887, 5189, 3888, 5327, 3835, 4689, 3836, 4534, 5256, 
              3812, 4741, 5258, 5257, 5831, 4323, 6340, 6341, 5893, 5894, 
              5895, 5837, 5838, 5518, 4092, 5511, 4093, 5519, 4471, 4472, 
              4090, 5515, 4091, 5517, 3885, 5501, 4078, 5025, 5887, 3652, 
              3718, 4736}],
             RowBox[{"-", "45.5`"}]],
            Annotation[#, -45.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4180, 5644, 4181, 5655, 4506, 5652, 5653, 5103, 5116, 
              3697, 4645, 4811, 4636, 4637, 4805, 4804, 3915, 4799, 4147, 
              4635, 5083, 4492, 4491, 4796, 4145, 4616, 4615, 3674, 3731, 
              6244, 6245, 5709, 4286, 4878, 6367, 4877, 5993, 5992, 5221, 
              5712, 5711, 5802, 5381, 5382, 5217, 5218, 5219, 4305, 5731, 
              4306, 5732, 4104, 5044, 5141, 6352, 3666, 3724, 5140, 6397, 
              6398, 6427, 6426, 4688, 6425, 4408, 4937, 3999, 4690, 5260, 
              5259, 4324, 5748, 4106, 4945, 4946, 4420, 5836, 4419, 6339, 
              4105, 4953, 4954, 4751, 4750, 5840, 4421, 5328, 3889, 4742, 
              4743, 4589, 4587, 3668, 4590, 5521, 5522, 5516, 5524, 5523, 
              4473, 5520, 4094, 5024, 5028}],
             RowBox[{"-", "49"}]],
            Annotation[#, -49, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3698, 3749, 6148, 6149, 5650, 4179, 5654, 3925, 4646, 
              4167, 4638, 4168, 4812, 4501, 4808, 4809, 4634, 4633, 3679, 
              3737, 6136, 6137, 5596, 4146, 5598, 5692, 6240, 4273, 4875, 
              5201, 5202, 4879, 4880, 4276, 6241, 4275, 6417, 5733, 4307, 
              5365, 3973, 5364, 3974, 5388, 3994, 5387, 3993, 4535, 3837, 
              5751, 4327, 5261, 5262, 3813, 4874, 4926, 5829, 5830, 5828, 
              4414, 4939, 4938, 4409, 6394, 4410, 4325, 5749, 4326, 5750, 
              4208, 4940, 4941, 4441, 3667, 3725, 5795, 5793, 5794, 5752, 
              4362, 4758, 4757, 4477, 4756, 4107, 4588, 5045, 4591, 3890, 
              5525, 4095, 5030, 5888, 3660, 3722, 4744}],
             RowBox[{"-", "52.5`"}]],
            Annotation[#, -52.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4183, 5651, 5657, 5656, 4507, 4823, 3926, 4648, 5105, 
              3686, 3742, 4807, 4165, 4641, 4166, 4640, 4639, 5599, 4148, 
              5597, 5601, 6365, 5600, 5663, 5691, 3785, 5986, 6321, 6320, 
              4389, 6420, 5985, 4274, 4876, 5242, 5241, 5735, 5734, 3838, 
              5292, 3839, 5293, 4211, 5363, 3972, 5199, 5200, 4680, 3971, 
              4873, 4272, 4513, 3784, 4687, 4602, 4537, 6393, 4538, 4753, 
              4752, 6337, 4411, 5824, 5826, 5825, 4328, 5753, 4329, 5754, 
              4108, 5755, 5046, 5047, 3726, 4754, 4755, 4592, 3891, 4762, 
              4478, 5549, 5550, 5029, 5048}],
             RowBox[{"-", "56"}]],
            Annotation[#, -56, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4109, 4760, 3727, 4479, 4111, 4593, 4110, 4761, 4423, 
              4955, 5067, 5066, 4427, 5841, 4330, 5832, 5833, 5263, 3815, 
              4759, 3814, 4539, 4540, 4331, 4908, 3840, 4536, 5142, 5736, 
              3896, 4603, 5222, 3792, 4681, 4682, 3988, 5383, 3987, 5384, 
              3991, 4444, 4443, 4931, 3841, 5714, 5715, 6247, 6246, 4288, 
              6418, 5995, 5996, 5994, 4395, 6322, 4394, 5664, 6167, 4212, 
              6166, 5820, 5146, 5145, 5085, 5603, 5602, 3916, 4810, 3917, 
              4642, 4818, 4817, 4649, 5118, 5117, 4647, 4182, 4508, 3750, 
              5120, 5119}],
             RowBox[{"-", "59.5`"}]],
            Annotation[#, -59.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3675, 4604, 4606, 4764, 4763, 3897, 5329, 4424, 5842, 
              4780, 4781, 4425, 4956, 4125, 4779, 4422, 5570, 5571, 5572, 
              4947, 4126, 5760, 5761, 5264, 5265, 4909, 4332, 5759, 4333, 
              5756, 5758, 5757, 5762, 3680, 6327, 6328, 5821, 4404, 5822, 
              4308, 4897, 6011, 6262, 6263, 6010, 6368, 6009, 5224, 5223, 
              3989, 5385, 4223, 5386, 3990, 5225, 5997, 4670, 3934, 4930, 
              4393, 4669, 5934, 4668, 5698, 5693, 3687, 5910, 5106, 5084, 
              4169, 5634, 4502, 4503, 5107, 4652, 4171, 4819, 3927, 4824, 
              4825, 4654, 4653, 4832}],
             RowBox[{"-", "63"}]],
            Annotation[#, -63, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4186, 4655, 4187, 4833, 4509, 4828, 4829, 4651, 4650, 
              3688, 3743, 4813, 4814, 4883, 4882, 4170, 4816, 5206, 5700, 
              4277, 4881, 5209, 5210, 4671, 4672, 3979, 6093, 3978, 6408, 
              5391, 3997, 5368, 3976, 5367, 3977, 5390, 3996, 5389, 3995, 
              4541, 3842, 5763, 4334, 5269, 5270, 3816, 4885, 5817, 5396, 
              5397, 5266, 5267, 5268, 4683, 4684, 4127, 5068, 5143, 5144, 
              5902, 3732, 5798, 6399, 6429, 5796, 6431, 5797, 6433, 6434, 
              6432, 6430, 4363, 4785, 4784, 4484, 4783, 4128, 4605, 5069}],
             RowBox[{"-", "66.5`"}]],
            Annotation[#, -66.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3699, 3751, 4826, 4184, 4663, 4185, 4657, 4656, 4820, 
              4172, 5694, 4278, 5208, 5987, 3789, 4815, 3788, 6242, 6243, 
              5699, 4281, 5740, 4311, 5739, 4310, 5295, 3843, 5294, 5297, 
              5296, 5799, 4918, 4919, 4928, 4927, 4514, 4515, 4390, 5818, 
              4391, 5819, 4280, 5695, 5697, 5696, 3786, 5395, 3787, 5205, 
              5203, 3975, 5366, 4445, 4446, 4335, 5764, 5766, 5765, 4129, 
              5768, 5070, 5071, 3733, 4782}],
             RowBox[{"-", "70"}]],
            Annotation[#, -70, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3681, 5767, 4336, 5072, 4337, 4787, 4786, 3817, 5271, 
              4543, 3845, 4692, 3844, 5337, 3909, 5204, 3908, 5720, 4290, 
              5226, 5228, 4884, 4279, 5719, 4289, 5716, 5718, 5717, 5722, 
              3689, 5109, 5108, 5298, 6040, 6039, 5243, 6012, 5737, 4309, 
              5738, 5233, 5230, 4292, 5721, 4213, 5207, 4214, 5665, 5110, 
              4660, 4173, 4821, 3928, 4830, 4831, 4662, 4827, 4661, 4836}],
             RowBox[{"-", "73.5`"}]],
            Annotation[#, -73.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4188, 4664, 4189, 4837, 4510, 4834, 4835, 4659, 5121, 
              4658, 3690, 3744, 5149, 4291, 5229, 5231, 5232, 4673, 3794, 
              5234, 3992, 5238, 5998, 5999, 4545, 3795, 4886, 5237, 5236, 
              5235, 4293, 4516, 3793, 5741, 4312, 5604, 4149, 5227, 4150, 
              5147, 3738, 4542, 4691, 4000, 4693, 4001, 5399, 5400, 5398, 
              4002, 4619, 5338, 3910, 4544, 3846, 4910, 4447}],
             RowBox[{"-", "77"}]],
            
            Annotation[#, -77, 
             "Tooltip"]& ]}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}}],
       AspectRatio->1,
       BaseStyle->{FontFamily -> "Times", FontSize -> 12},
       DisplayFunction:>Identity,
       Frame->True,
       PlotLabel->FormBox[
        "\"\[Mu]=10 \[Alpha]=0.20 \[Epsilon]=10 PIN=0.09\"", TraditionalForm],
       
       PlotRange->{{0, 50}, {0, 50}},
       PlotRangeClipping->True,
       PlotRangePadding->{
         Scaled[0.02], 
         Scaled[0.02]}], {576., -192.}, 
      ImageScaled[{0.5, 0.5}], {360., 360.}]}, {InsetBox[
      GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJycfXdg1MUTb4CEGoQEEggESOiBENIrye1driRAEEIRBJGOVEGKoFKlF+mg
KIJ0pCPSS0C69CadUJUuLQKhvK+X++x+Zy73872XP1jmdr9bZmdnZmdmdwPb
fZreMa+bm9t7gW5u/6b8r0rq1MqpUz0Fhf0lfGlDj4sbelRh+cES3vhv9sUw
Vj5Kwnns/8ZJuKr9+zq0Pm/B6jNK2F58YbKEL9rrt0jYXjzaJmHHVwxW48w9
9ZcwxkvzgyWM8dLyURLGeAFjvKQ+bby0PqOEMV7AGC9gjJfOW6pwNY+5j1Ol
dLz+bLz+bLz+bLz+bLz+bLz+bLz+bLz+bLz+bLz+bLz+bLzOdJr7PKpx8hTj
BYzx0nJREsZ4AWO8pD5tvLQ+o4QxXsAYL2CMl667VOFqHeZOp/5O46L5KqXz
G8zmN5jNbzCb32A2v8FsfoPZ/Aaz+Q1m8xvM5teZz+S+Dv2d5o2PCzBPMV7A
GC9gjJfUp42X1mOUMMYLGOMFjPFSvpkqXPHR3PmMvxNd8nnj46LlVUrpOYrR
cxSj5yhGz1GMnqMYPUcxeo6S43UlJ3Lno/5O647TJZ83Pi7APMV4SX3aeGl9
RgljvIAxXsAYL5V7qcKVHMxdTvg78RW+7jhd8nnj4wLMU4yX1meUMMYLGOMF
jPFSuZ4qXMn53OWgP6OzKk58ha87Tpd83vi4SH26lPArDSb8SoMJv9Jgwq+k
3pLKYKXH5C7n/Z3kAuebnK/wdcfpks8bHxetT6WUXxkZvzIyfmWU43Wlp+Wu
x/g7yT0uFzjf5HyFrztOl3ze+LgA8xTjBYzxAsZ4Kaz00Nz1NH8nuc7lHpcL
nG9yvsLXHadLPm98XIB5ivECxvioXOZ6p7+TnsLlOJdzXA5wPsn5CF9nnA75
PPFxAHYjf1xP5nok17O4HsLlNJdjnM9zPsj5BF9HnM5QPlSmde35ETLtaR9n
jEynaz9XPhgv003/1t8ySab2cllCpvZmMkwy/bfaymPNMs0Z/2sDUmAScM74
fWS5nPEEStieXTZF0O9TnepFPTxFfa5+52lO/UGyHGDSLw2PpN8ajHzgFfmA
kQ88Ix8w8oF35ANGPuYB+YBJ/VmC1p8lZD7w+b/x7s/wVsUlnngKfADG+AFj
vIAxPsAYDykPOkM7Gp2RfoD+tHyMj8876Z9cr0Fsvar+YhzoL2D0FzD6Cxjt
czridMLxxL/ndMbpiOOV95vTIaczPg98nJxOOR3yeeN44XTM6ZTPM8cjp3NO
x5wueD6nC8Ccr7g5/vi64HTE55XPU+58CvswH9kOXT+qXTf2R9envxP/4uuU
0zXVi0MlTOhEg8m8azCZRw0m84LyWYK1lypcjZPzEb7uKJ1XYXRbhdFhFUZX
VRidVGH8r4qcdy6X+TqCnKb4CGb4CGb4UHLdFd75OFHe1bxwPKAdV/PG8ZT7
OPyd8MbH5WreOZ5zx4O/E95dzQOX45RuUpl8UXjleCPw1PsGpP9+vWFiisvf
6f5P6UW5/s7tXTq+yu1AlB9Dv1Ht8t95/+nqd/4d9eTgNZSm2jy70uugX/J8
/I55d6UHQh/l+fgddOBKb4T+yvPxO+jElZ4JfZfnEz1YoytXein0Y56P36Gn
ojxS/E70aV0+fv+3WGq61Unvxe/2edP2Ezwfv+eMJ0SmOfWHy5TPI1KyvtOj
ZcrnDSl+t8//zTiZ8nlCit8//bed/oky5fOCFL/b6x2hUj4PZL8CvpymUo53
pPjd3o6vSjmekeJ3ezuZFplOs9cTKfELmKfApx0PZWOdYJ4Cn5f/bX9lghPM
U+DTPj/C4ATzFPi046GL0QnmKfBpx0N4shPMU+DTjr5ssxPMU+CT6pVhEk/A
L6E7je9QORQl8UTwi/Wv1QsYdAw84TvQHeFjWcpuBzzhO9Ad0T8hl2AP1PBE
8Au6y1Iw4Ssangh+QXcZCiZ8BHgDfjX9D/US+6Zuf0t/95f5VB/0ZHqo8lPy
/R7VH8OYfhklYWoH96F40unJqA8wt8PmSLEguv65Hi39Dj6sX1FS/lL7bCiz
z4ayfYrzuKhepvgp6BX18/0Tb5f7B0C/pJwGEzmn219x+zromdCtjn9wOzTf
X3E8A8Z3oH/8zvkNfsf88P0ZtwdjfeB3zo+43ZTLXawX5HP+hN/Bz7ncxfpB
PudX+N3eXLSNynlNv+JyGOsy93XkTNecbmm+h0xz2vNjcBBdh1LffG2gev5r
A9XjXxuonv7aQPXw1wbwG8AXHXqjq34ROazREfACGPUBRn34nvLDQFkeMMrn
jKsS0/+DJP3Z6zlYg81TEOWnGoz6UJ7jEeuF6slCrjvK70Mp39Vggr9c5gXy
HnoC8E/0buhB2A9kmCSM9gCjPcCgI1d0gPGhH0SvhT0H+rAGE/mlwWgPMNpz
RWdcbmO+0A+0h3y0BxjtAUZ7rugY80r0HuA9F70A/IbQkdY+8tE+YLTP7U18
3jheXekZ4FfoB8Gvlk/wq7PLcjrkdMLnkeMZMNdjwP/QL7Tvap1jnfJ1x9cF
p1tOV3ze+Ty40qNS/v3PHqvsN+djnA9xPsP5BF/HfJ3xdcDplNMNmedc9Drw
U84XOd/jfIvmRzitQ75OON1yuuLzzPVKwld0egXXA7gc53KZy9mc+fBi9go/
an/xDpAwxk310PsGV/Xgd5KP+rSUyJVcfs/BZzUJY/yon+xTdfPG+8Prp/ZD
1Q5PUT9g4BMwwYdWb878hUt7AJUzIWyfFEL3oRpM9/Uhcn4Boz20z+0O/6/t
Ejzq2nXVD473/8IT5oHjiY+D6jk+dN60lPv9eD3cfo180h/44SemONmPMU4q
n0IYXwii+MnlO663Yfw536c66WFcz0I7BD/7sN4uGZDid6QYN5lXnX0E/QXf
4PYQYhcEH9HmH3yE7NtHCKd9B7d3cHsffgcfQj3EbqTbdwBGyu1/+B18jKxL
X7PTPgQwUvBZIg+03yHPUI89X9tXoB37vPSwONlTiPzNdLYXopwrOx63u3K7
HdFTRzjb7YBfV3Y8Yqetp1K6TpVdD/gFzO18+N1ef0mVAm/AO8ZP9Kxc7ID4
3V7/5WSZ2ttp6Pp34BvzArkOfNvrX2yRqX3c2Q77YLjDPjDRYf/b49jvrzPk
zMedxJz8I0kytdc/3yFXtzvkTqhRpvZ2uznk7CgHHl+q1E5fkRbZf3t9u1WK
fFe/U30ynMpDHYx1T/R0bb0TepJyJ4nSo0YvxA46QjjZ+0k99Rx05p4o6QP0
CJjb+wl9ljTJegg9dFH2OW7vJ/R4OVnWA3og6zzT4mTvJ/S32CLrsdfvZ6Xw
Mhtd75oeBpjKJw+qB7jxuHS1b0Y+9VMhjiOM6ldyH8vjgF8baBys2vehfho3
6tiHSv8vtz94Mr+840/n96Pfe1I+qv1hfsh45feebJ/C4+heG+j3nmyfwOPS
uL3EU84X1UuNue/bnOL0fOi+ySnOzYfuU3T+c6pvwq5bReofoAtCv1o+6uN0
Arog86DzuxM9NUvFydLxBbPxBLP+s/MDOnsJ6BB2GOg5+J3old7KPkPjRMKY
3VHt03mcJ99H87hIvo/lcYV830i+n5gi+4v1y9cZ1hWx38p4HMH0vSS5Don9
VGd3QD7Gx+0CyMf4+L4c+Rgf3wcTu7dGf3wf6opPoByPk+b6H+QD0et0ccbc
rsLjcrndg8excrsDjwPl+3weV8n32ZwP8vhz7lfg9jFu5+dx1CiP8XE7Do/L
5XYUHtfK7RY8TpTbDfh5AW435uuK28eo3yfKye5J9HhdXAD1ewiqT+jirFEe
4+d2I5TH+Lmdhvo/Up3sJITP6OQUlzOYb26v4/H1HF/czszj7bl9kcffc3sj
j8fn9kfupwEMvYXHceN7sj50di0eR8ztSvw8ApUTSu4ScaOTk1yucD5N8jX8
gS6of07ZR5FP1rXOXop8si509lMqN5ReCXwSPpSl6BR6G2DgE99j/4d5AIzy
wC/KA7+kPzo9h9gDdPH1XM+gfN/H6fwAl+NcDtJ1Fup0ngD5wD/ygX/kA//I
B/6RD/xTuazswZwPcL0d80HwoeGR1KfT+4A/Mk/aH9fDqFz0cTqfwPUeridQ
PhTqdF4B+cAf8oE/5AN/yAf+kA/8Ub1FrWPgD/mEL+js2cgH3SEf+EQ+8M7x
B7sRxs/1BsA0Lk6db0A+kfO68w7Ip/sRdf4B+dRPrc5DkPqzlFwg61hnL0c+
5WMqHpHoufI8FuJaPaQdmMr9QLoP1vLRPvFjwJ6l+z6n/SD5vb1f8I/CrqXB
lK8EUTuJzo8GPfyiQ58FjPp5/BP0N/ST94vKpVAnvyTpt9Zf7pflfkTAsJ8g
xXhy9RNzvx/8b/NVCn6NfIyH4w0wqQf+s24qJb9nK7sKUvI77IA6vNvtHs3i
ZYrxE/tdQoLMhx2J2JO2C5nCTsTtRYDtw4Odq6GyN9B94iXH/uW59HOAron+
pH1nr3+ZTeZjfNi/Ur819IRwmQ/6Akzt3p7SzoR8Sode1P6s9RPzj3w6z16S
XxB7PexG2nrDeHLoQMVpcPs/X8/0d6PTeiZ4RDyZ1h7y8T3l04GyPyhP+W6Q
zMc6IHjMCpX52K8CRj7wTPVQLxr3CP1Y6qFqHjAujgfAKP+/5yVQzgvx1+nm
hcYXBUo8u8IrtRMGOOEZMOYXeOF0DZjIB8S5LFPzROVvgNO8UboKYvvSak7z
+F/zxtdd7uv4vsHOd5oofw+19we4pGvuh4PeQ+ykODcv/TMhzC8XKscPesB3
XI4ReaHVw+ul+8tKbP6qMPi59Ge56heXe9TeFOLSTkXtTqES79x/xvuBcjn4
dOBlhDpnwO3wrvxugMEXsW6Rz+P4Uc6Vn47Uc1PF9fM4Zh5ny+OUefwA90vx
uH/u/+N+Ku4XJP4mnd8K5cFPeFwj8ePp/FjcTwi9j+zzdXG/RC/TxTkSv57O
r8X9hlP1emSGykd56Ak8DtL+nZ9VymtXfsTKDn2dtKOLK+Z+SDv997DQehcr
PxFg0AePg+fnDri+SOg5M0qmJA5AR1/8XAKPpyfnRD6Lkynmm59T4PH1hG7c
E2UKvRL6E7ebQG+E/gQYehXmCfOwwaEXAq+AgVfohZjXnPmIlCn8ifAf2vG5
LEam8C9CH7TfwRVQxwmGfgj9EeMlejXu9zii8AIY/jOsD2IXCTXKfMDwlwEv
HIb/C+MHDLyQ/D1W6hcfb3WiRx6HSOISdHyM8APdeQLQFYl3gj0bdPFZnMQj
YKKX6M4bEDqrJ6h+rq1ntAO64ucPSJxESRPV533Nsl3QGT+PgHkCHom+n2mR
/QB++fkEwqegZ4BPQE76Knse1YuV3ZTaBV8bsD6IXXCZshvQe2Z8aByppmcR
ucXvF9PpnbnrAUo/4f4krmdQ/1iqzIe+Ruxamr5CzqngPOZN5S/433jxd9Kr
iF6mwxvXu4h+CzsD5KfUxxVM+NvEWLnugTfwC6QYH6cHwNBbAV/4t543Vsnn
oPeQ/aOmd+aMP4/UQzFO6Kcoz/V8rsejPxym+quX1JPIvk/KreeyXdAh6qHx
8A674HWb7PcGxz4AMLVH+bF59Jf9JPQo/U5BNC4qF7oifh8tH/zZPt7+teS6
ofYY53XD1xWvl5//JfEWGr5yXxepMiXlMh1+wvAIKccIDHuA7lwY13+J/xR8
d0+0lHOA8R3Bm+7cI2CiX9xJdJKXXE4CBh/ONX5HZ2/h8Tvc3sLtMfgd8g/6
APg5kXfeSg5jPaMdwMR/M1/ps1iXqBdyFXKA2KnGW2m8EPyI6wxO+gPGi3zI
PVJ+hANvAw058vkn1T+uX3D9CnIQMPiAvf8jHXy0q4qzAz4JH7qs7HSQX3Z2
0MBhb41g8XZ+VhoXtdhC46s0Ofru379Sjvif10r/5vozytvbu+Lwq/ym9Bey
Hm4q+gZ95eC5tuT3PG6X75/Jvgnxx7q4J8qHouh+MjNK6p+AuV+fyJPP4qQ+
Cpj472E3uOnM57hdG/yGj4fGTSg7OegW9E39rAF0P6jDC+GndxIl/wQ/5Ptb
zn+AF8CEr0B/0MHE77/OIPFF5AP02BHKz4t1gfKQr+AbRE6lKRj4sKe9bJTP
aOMj+ATf1M039h2AQR/gk3y+iT7xWZyTPYHYR6T9NkzWn9P/cFreV+nvaI/o
UdDfER+ojQP6CIkj7WWT/ITIYafzBp5S3qMcjTPyYnE5LO4c8V4T1TlAmq/u
zST6plucEwy5R9YF+IS3oPLNW8VHgr6oXVvZQUhcQD1B44f4ec2SJurn5ecx
LyfTfN1+Dr9jPuh+IdXJ7kDkG/zVy5R/FTDxT2sw2b9o80b4gUbngME3AIMf
EHmTFUrtwlmhlJ9rMPfbYH1jfaA+wKgPMOoDjPoAg154fVhPqA8w6gOM+gCj
PnousbLUc9AO+C3qB4z6AaN+wKif+wldxXmhPsCoDzDq435JjleOF8Dcr0jm
u4uRnrOS9VWmeoGODvg88nngeOPnENCeK7ridMHnlc8DxyPGAfj/dx1wuuZ0
yemK0wGfR453V3EvnO453XI643TC55XjPVe/Z1aATPm+hO8riB1Rp/+TfRf0
esTpNzRT+exnpXq97hwCtWNwv40/43v/zRftP5dVfg56Ps5ZjqEcYBLXGa70
PX7ei98fQ843AF86+wP1y6r9E+ET9ZT84X57HseGeYA8Qjm6zpS9jMdpYd4g
f/Ad1hHRnzJVHBGxp/mq/RqXd6iHrAPY5ZapuBlCx7q49P9XvYKsH10cMtY3
6IrOtwezC9w3QE/gfAT5wAfyiZ6l8RmMG3SG74kdBvjU8SXif4S/lfMlec7O
g56Tk/6iENkf4rfzNUsY7XMYfI/aYcNl/YDJeGGnkP6ecEYv4UwfVzDxz8Ge
BXsU+o9zc75mah/JtDjBRA6CH0Bf1PBJz+M7n+Mh+xyNr0E/IHbGm3HU7wV7
N+zR2N9lWiRM5A7sWvCD62Bqt/Vwmgdut+fnjEj5UKMcB2BiF9H6i/KgV/K9
1n8OE36k9ZfsK5fZnPzTnG5c7fOI30vnR3B1TgpyBeOC3gKY4B/t47wg4sKA
f9QHfgS/J+JcdOuI0/1/0S2nG1d0RPhNLue6uJ+EyFHdeTfAhE/qzgcT/UrH
F/g6pvtPsxOdk3vOcqEjsn/P5ZwZ99vY0/FKP8D+HjDySRxZkxTql92nYGKv
lvfVOOI+dPog8jmfBh/mfJfof7nwTc73/otPcT7him+4Wof/ReeACd3mcm6P
+8HQf86nOV/9Lz72X3ziv9Yh0fN0dh30n+cTfddX6RmgM8Dcj0f8mTq7rb2d
+c5+XehdyOd+XehV4Ev/5ee119uQnT9drOzPOfRWXvpXSPyMd3H5O8oR+2P/
WjIldjrtO15PzrpIlXID5TFO4j+vp/R/bvcm496uzp8Cf9yuTcY9SumXhM/z
uIFwpb+S+nT+N9QDmNQzXuVzO7W9XT/mB8g20/nKdD4/jHIknqeZugeHyOMM
hQ/AfDzkd18zxctYxf/t7ac780v0h9gjs5X9EjDmCfoO2id+0ssq7gMw6T/8
NvMVnyfwYiWfgF8itzMttN5eNorf6zY5H9Rv5/BTwp4v48mElAOgC+TTeD0P
yg/dHPdN684dE/wj3jdb+f2pXFEw9+8DJnqDtq+h9jvHvRzpVeX+PGd8YdL/
AXzBf0/q91ZyiMTR7UuR30N+gi/ge3LOZZk6D0n4bC73AIKuYZ8m9lXd/Vs0
fs/kBEOO2PlyDyUnQd8cpv6UVCd7Oz/XROTQWKW32Lsb7Sy3+LkKns/PYXD9
nfq9Hfu5LkZqN9DZB/k9ARyG3QD0ys8Bcb2Anw9EfzBeLofJOZFMC20v3Urr
xzkV3fcoD3xQf7naJwMfZBw6OqHnIATVN3UwP98H/AIfgLFe8T3Gz/UM5NP1
o84FoTxguu9PdaqPn7Mh/Smr9HPgh+jfiK/OVud2Ob74uuF0zvPt1S5MpvOo
uyeCf4/y5BypLt4T+cTPnovdCeMAfyf4f2mS8orDkA+A0R65D61bMuWfkB/a
fKA/0PMwPswP6sN8oDxgMh86vkrWvW7fjHVG1kEucWkoT/icbt0ROtLdw8HX
Mb4Hvgme05RdD+Mmcktn14O9jdu5ME8kLsmlXHOWYxSuwvZXoS7lDOid29s5
3+cw+kvieHT7DSJHwC9065Tb6/k6Bwx8czri9n1SXncPN+pDfzEf3I9P+JKv
2em8oSu7KvGX6PgwYHLueaJrvgQY40N5V+uEf491iPHge9Ah/57wUZ1dmJ8j
RMrPG/HzRTw/p/4g6cewj+tmENP/glx+R+Q/4nd19xbZi4Ua6T4h1PncE+oj
cKiKn0aKepCCD5JzBc3iqV4r4wxvOeLTXrLzLvflPa38fBnxi6Sp94Oo/TqQ
rZtASre681cUn+rcW+7tqnNnPM6RxNH1UnGHlR37gv/qJ/rhqp88DtMezxQD
P8MtA/GTaXp5Dj1XlPsl0BHoisQhyv1eRSmvctp7nBP/eUPp4Zgn6Olcb89Z
pw7/1Y0UWQ/8VNjPAc7h7ypuhZw/QVwVv5dLd44A/ce+H+Ml+6sj6twJP4fJ
7QR8H8y/s8+nLq4N64LQge5cJ7crEL7ZRa0nV/VgPoi96KXab/P9OLdLgE8R
fr/bdT253jumu0cN65vbLUjc3nhlx0C+nU8uctQ3ziLnk+/3eXwSv1cd+fb6
Rjr41gIVX5izPEy0P9kqftCe313B6BfiA+1wlNpHIF4H8wV6IXbXxcoPQexa
iy3U7uZnlTDwQuwovWzUjnFdwVhnwCvy3Rx/WNf2dg6WkfTL436xTu35K2s6
xdMSedLLRu1Z1xUMOQ+Y2MM1Opvy73+uWSWe7elb8MXDBjLfWj/Qb4wDfAP9
Br8gccJxDnp446Cj5QomcQK4V7GFgy/0dfC//Cq+FbC9/gvqXkTMO+gL6xfj
wvoicacaXYCe7OV6KpjoW/lVPC7ibe2/hwjZHlnfOxxxMGGqfdCDPR3jsB+9
Spbtge7saSOLjGcl/RlnofwU+FtYifJTDcZ8E73upYnaA7T1TOKJtXmwt1PG
8S6nj7KfYT1g/nLoXMUj8HhBwuekfoV7Vf2Y3K5G9QkZn3+fnqfV2du4PY37
0/m+l9vbsE9BeaxP9J/7eVAe32McVO4zv4/uvjnYxfk+hvArbf1jXQLmfiK6
3w6n/hkNJnqgBqO/5Jyu7h0kjJe3h++hN/P2sL4JPlC+h8WpfZRHe8gHfnh7
xM+l4ZnYKTSYx9Hw+86IXwnv9+j8pMjHfgQw8vn9Z3y83K/GxweYxHHp7kND
PsZL7Y9ubvz+DuTDrkrPx6k4CxJXrPPn8/vRuB2H20sBY7z8PrT/6VfE/Xao
D/dwSDubF/VTa+uS2/fAV8h8S3uhHz23gXM98t7cUKpX9ohywo+rc12Ej7on
0rhu90SnuCvgj9thuB8b5YE/5JN4DnkveiqtD/Qh9zNebHweks4pvzVSeV1P
4ZPwDcmP1T034LPAJ2AeZwP88fvTeLwFj0cg9sxuyh8EmMdLc3szYLpPU/ep
EbvWRGWn4PtS7rcBjHnFeiR8A/GFOvxQv06UE75I3JQuDomcz9TKk3Xvq+iG
44/HQ6C/ZD3r9FfABH/Y78BPq/N78/FzucdhzBPoh8rlYAkTP5IOX4DpuRl1
3xnHH9oj9yL5mqlerPM7IJ/Eu+ruO+P4RD7mnZbncXfKT0fPYaj73ADzuDLi
V8pU/kgiR7yVHwH9Rz6he529F/lkHnXxGsjHPNL6vFz68TAewOTcLvw+Uk6r
+8WQT+1gqZSP6+yryCd8ySmuKpXGq+riOQhdyzhgvCPvx+AgCdNzmp5O9ZD9
nVshdm+iD2vfj9FNINUfND0d9ZNzZftSZBw97yfh97rzFlhnlE8FU30M778h
fkV3XxmhW3kfZSjjn2E0Hk0X30z8jogDmKjuJaP3vSo8Yzxc3+HvtRE5otUL
OY39Czlf3cVI6Q7xut5KPwBM2hEGp3ohl/j9aFz/IPJP9x3og8s3Iv+y1X3p
oCNC17p753g9JK5Et79HedAdpwPAvB58z+9ZI/YjeX471YmOSfya7l5cYldf
YJKpvTrdO27wB9G4ese59YUlJH5IHGLZFCcYcXd2fF2HPzaA3ZNYiZ0DU++K
Y1+K7+31l60uYdhnSPks9X481Yu8pJ8I+fzeBPouUG1px6T3jan7cbm+R/GQ
x8nuBBh2aX5vlSs8Au+kvLczHgHbG8uoSPGu4RH5GB+dl1Ba34hQalfT3XMA
+uDxa2gXMOrhv5O4F6knhrL4tVDG5yGHlH0cfI/eyxZDz7nrzs9z/kXOq+5R
5+dBF4Q/HklyuseQ2Md178TxewtJ3JXO/k70SsTnrVPvwPB7e4h9K0vB4Ef2
7oey94ZD1b2QhH519wC5Kkdgfq9ihoLBv+zrQXdfEGDk4zvU91/lSL1+7N6n
hs73CQAmeqPunifS/4bqfIu9H92VvR31AQadQK4SOstW9xmRcgmKzggdTYyV
Kcphvvk9JTweE/tHzA+/h4DYYV8yusgwOc0n2X+4epdmlPJbkvkYq2AiF3c7
xzXCzooU80TiNNOd72VAPok3xD1hmRanfPxOzoUuU+fTyb1tAx3rKpHd96Dz
/5J9Xn7Hff6fsDjhEcr+jHzIBeDRjp/6yh4NvJE4hHDmh3rp/G418Gpvz0fh
k8TbIq7gikpz8KLi/HLw6LDLvQyn8VgaXYM/EXpuqeKHiZ8PdA361J0nhL0G
/IXEo2YJSudp6h5yYrfWnUfk9E3uCclQMPKJvQXvISHOe7eCkY/5Qj7mmfCD
+ezcQ0nlPyNxPX7KX0X4lC6+F3QAmPivFqv5JPXhnajXFjn/RM/Q+bswDqrH
KT8A1fPy0Djy6FIStrcv44I8Zf3Ent5ExZ278jNwvQz51C7rx/Y3Pmw/quyO
5Htpt/Sh8Vk6u2RO+SA6Tzo7G/JpfKLSC6GXcLsvtwvT91JDJT6Jfx73Y7on
0v2Otr6I/RN+ogR1jxVgV/dcoT2yPnA/yQi17qi9OIiuw3rCSe4QPIY6vztH
5M2YZMnvANvx19XkpCeAv2E9QC4Teh5nkffZAAb9g8/hd/vn2QpGfWQdNlL3
26Ach8l9Njo6hz5N7eTYbzn8y3jnB/4x3T2qiJtx0//hvlMtn95LfF++g4yi
ZL+E86+wn+v8d+gfiUfU/uh9MfcN9H44dc8r2cfI+CsvZkdUMNbjBv0+BOdh
m+Dcbhlmj3e8+yXthArm9hjAxA6EfQ3igTUY33MY6x39A0zuZ1tZk+LZO9Rp
HpGPeeTzhnzMG58nrCty3yXiy3X+J8wbnyfgnegH15W/19W8UHt2gBPeXM0L
sS/8D7wCBt3Q/Zsf29eXcqILCqt5sg//ZpDTPAHmcRnEb6S714vEb+G8oJyn
Uk77cozDzj8mKpisW2nHvGTAOMi+WKuX6C1afs53qRLPwBPOewGGvSQHr1FO
+2bMA9k345wJ1m/Z6nQ+pX0ogLaD+cQ+W/uOrjvYfdS9gtwugfGS/bTufArm
EXik9o/XLvEOuU7vM0yVeEb8HOJlACPeD+XwHX5HOUI3uvcsQVeQa1yfBYz5
5ecNSVzU9CS574CcI+cSwxk83/k9V7SLFL9D/+f6K2CiZ8LvgPiuSyaZ4ncS
n8PPL/Jzc/x+P927rcgHXyBwQ+d68B05z8bfhdDZY/h9hWR/p/udxC/q7t8m
/CDU+X1S4t/V/U7sMVnOehGZB915UmJfTnO+ZxnzSfY/aey+bdhhXhhlSuL3
dOdToV+B7xM62O3i3m7dvBD50VDNL34n874rWaYoj/ahtyEfv5N6d6l4Q8D4
DvRH9MJMRZ/Q06C3Qc8jdJepfkc5pNDLIa8RV4X4Lm6Xs6f3HPdLV0yS8WaI
hwMdEDtdlvrdPs76Dv25hErxu73cmGRpn5/q0FcJHsc6v3OL30l85iJVntxj
ht8bKRjfkfi2tHCnOGPwPW4fIOu2nrIHYH1w/ol1CZjcT1iSxeOnsfduR7F7
63eruFzAZJ+3S+1HCD/rpvgdoctMlW9PdXG6gMk6AN3tSpZxsdif2L/rZaPz
tMxG/UZl+X0HVZifJ0jqgdCPqL8m2EmPhFzl5xjpfdQeTnFB9F4i5ffg8YA8
n+z7M1R8IOaB53M/PvpL4xbUu3AkTiTd+f4eohens/t5eik/BpGnurgbwMiH
3kH4ji7ugviBxjrHIyKfjs9fwphnsj/g+0ZdXCboAPgm8coZzvcDgU54HAj6
i3zaP/XOG+9PTjvq3hce54X9Etlv6u5V4XFMxM8D/qi7t4TYT/i5PfgfeHyj
typP4ibGOt93iPIYL6mvrIq3xPix73YV54f9IPDB49yQT+Pu1LuiPK6Lw/ie
rEfdvS48zgnlaZyVeleSx/nwe/uInSLTQtvXxQ0Bv+RekbEq3pTe06fqw/fA
P783D+UxH9zuwfkcsXtAbsDfAnu+zr+HfKJ36d7nJXY+XdwT9CZS3le9Awn8
8XwSpzNW1QcY7YF/IJ/eWxMj8wEDXzwf7QF/qA8wj/Pmdk/+3i+/14jfS0n0
8zQTtYPq4sIA8/L8fD+PI+P3RpLz/JkWp/b4O5y8Pv7OJLn3Mt3qFP/J3xfm
93nxuHBiv/EOdeIL5D4AHd8AnfN7l0n8jO7daUJfOpj4U3RxjMAHv9eSnAtP
V/GwGD/yCf2McNYLqL0M/S0l9zuQK6AbIrd0coDIacgF3XvDRI7o4swJX9S9
P0z4KNaZLi6d81USl4h1pSuPfHI+OZd1AH4BvZbeSyro+v2/0KOoPHbIQU3v
QDvQMwC7OtfA5SjyAZP2dXKVvtNlcpILwB/pjy7OH/h0xUfpvX7OfITTdWWH
3oz6wR82OPR3cl9Qhoo3JnqD7lwJie8b6+pcy2sG++d+LkX3PjGVq5VYe1Wc
4gKJHMF9wWPVORLk83MlJI5Kd+4DsCs5DRjzT8ajO1dC5LIrPUF3buF/yjld
+6ify1EiZ3Tfozzhy7nIBXKvyViz0z7BTf+nO7eEfJS3J9E2qvcvcz63Q9ab
Tq/k53qQz88BYf74fcK8PpQH/vk5GFfnYoBfV3oFPxfhSi5yOcHPUfDyaB/r
l5cn/csStP86uy6hR925KK4nIp/Qv06vz5nwVIk/onfqzs240pt4e6iP62Vc
jyDnUsryc0cqXpP0RyeHAeeqh8j3a5KcypP+6eQO+B34EPgo4UOI87mp3kEm
eky2ej+Zxx3bm8V7agkK5n5rsk5fqnMW9n6OVOdKwe+JfqC7N9RORw2S6blT
3b2a/B44pPyef2KvaaTOgZLzoo1UnBnkDPzc3N8N+W1H3yWVb++/7t1Dbne1
l+9ulvY/pPg9Rxwx/7WMOy3E3tN1+L3wfgb8UXhPAnoa1yOu22Q8POoDfeB7
4j/tZZP0QurvX0vWlzN/YdTOs4yd+2yi/IeUf5WS9cJvxvsJGH4x0q7Wb9Tn
qp8ET13CXMTDOvTePdGUjt3Yu42fOb/LBLsyf8eQx5WSc4SI+9DFiZC4LHf1
vguPF+HtELuFzo+FcuAbZB/VRfm9Kjv0eVd+Mew3SPxISXbvg7QDOfvBsJ55
3Bdgco6yJLvHfJS6r4HH+RM9/P/Cb0bi+BaqOC/CF1+rOFIex0XiIXXvnNpT
3T2JRP/xVX4Psv5/c46D4XFExP89VcX5EHr4XaUkDlkXF8zHj/nB70Q/HKPi
cMAX7eOJsjrFzfK4WNKvliruMtfzJNp6sQ+rvvJ/gS7Qb/BX5Nv7gXP0Otj+
3S4VP4n5Jfx9VzL1l+xS5/kJ/YD+M5k9D/GnU+s4xU+CTxD/Mfwu2xVM7L0J
6v1Ngq9PhPRHkThed+VPIfeyaPNI7j3T5hP5wCcZV4byx5D44Cwh/Srwf4Ge
efwluQ8E9+bq4jF5efATwKAD1I9+wc8Cfxk5J5Om7p8jdJyu4jER54B6SZzJ
5WRqL9SdiwZM46deG0g/deeAyb6yi3o/FPOBfMJnnO475f6R1wb67kRtto5q
y3Ku+knsNbp6XbVL7FjL1HteJK69njrX3evf724nyBTyFnSH+0/AR8h9SvJe
Ezc3eq/TYcf7o270vgHvAEaf6vwR1xOgF6A/kMuA+fuR/B4Wcv/+4igZTw19
gJ9D4fHSxI/wkyMuc426F5OcE7mg8qE34jv4j7k+SfxuEWaqj45V/lLIF+SD
f+I78D8u5yCfSNy1q3eU7xlkPwGDjxO9PJd3kSFP7NPc00r18SVWWR/i+eGP
B1/EPoDEizZwjotAPpEH0KerOuRyYauUcxg/YJQHDPkO/oZ8cs+TNk7QP5Hn
03EOQb27QPxF4JN4Z267glEfkYO9nO9dJ/7y6+r+TsDgryiPfHu/99okv0X9
KG/vZmMbjS+aoGD0n9hpQxz8Yny0/J7so+T7QZ4y7iuHKxRicXWFWDybI45z
arCsB35BYlfF+0CIs3RTfhH6jqOyc6N+Gj+u7bAd/BvlMc+AoTdhf4V9CDn3
l+0cB0DG4a38v4QPafsW0DOJk5R2FMQJ+EiY+v3h7/V0stMDb5ALgPm5e8jT
nO/CqHxNC6dyXMM7HW8Us0tEyfLEn6W7Rxflyf3VujgH2Cn4O06kfC73qAHG
+PA92qN04enkF8Y8UX+zoHYp+X62p5NfHXIN9QGm92OqeCHwPcD29IVR1s/9
8KBT1A8Y+AGM9gBDj6B07unkHwDdk/fOeqj7acEfXcVF0PxgCaN/WDc5eCwu
+RAZr4yfKSRT0r6MA1Z8hL8zy99ppe8MVmb+dvWOK7Hzd0uW5YkfDu/A4pyn
tl75PTD29YZ7FLV8/u4RuX9Dm1dKH2HUb4hz/VJeq/sG+HrF90Sv0/Zx5NzW
RPX+On2PQjjlk3M+Wr95P6k/OUy2w+ke+KD5YdR+qH3P3w0lcXz3DFQP0dE1
5ot8j7iZLmoe0A+sN/STnGOcr+KK6HpR90SgPrRr/72MTdYLOqLrJ9jJX4R8
0DPmkdCpbl7Bh/i82/E6OV6m2E+Qd1UqJtH4R925YuirSKFPwG5mT+R9A8/Z
OfjX8r7WnHG8ZPfC5pFx50ip/d1PwpBzJG5RK0/OfaMe6c/1kjC+J+9+4BwX
9sHa9/ZmYffSxcPje6pPVZT0j/UIGH4fe4p3CrX9G4nr0cpT+QK/bKQsj/Zy
5Emoy3YAk31ieITLdug7qLVlu67wDpjgOZd5QHs0zkHBaA/fu5on4sfX5gnz
Rs4v5DJvoENXdAo6BF3yfSW/l4Kus0DKXxGPt0+dhwCcU07BwAPqIXoV7O64
f+d/1EP0MvnubJikD8Boj9x7mhbO9BHX3wEm+g3udf5Znc8gejjuPb6h7rOm
cYWhTnRMzk3p4raJvKmYJPkX2bcfSaJx1wtMdJ/SXd3ja0cj7gmIssp8O7xI
/Y59oH38Y1XcF6H/PXgvLErCZN+RrvIJH4Wef4/dt3rPIPku9rlIYf/IWd/q
/DHkgx1u5HzOHjCxM//G7j/Q2XPs/Szn4Iv52f47vzo3b8dbfRf78a4maY9F
Cvsr7A/28lfUPQKwS5L17c7etU90+DOWM3vqVPUeEuQZYGK3yHA+P07sTK9U
PDk5J4p7DuQ56DzUXyTf93ptoOVwHtuTyjUtn/BTDQZfIvXpzm2j3+ALxH8B
GPgaESrbJ/dHg19OVOewAZN9O86l6ugaMMqRfaq856k28xOp89Boh8tlzA/6
R/RmrX9cvyP3FYcIai+76HxfMOgRMNnXj2H3NYxj5/MbqXMmyAeM+YF84PPI
/ZwE1vQF8FPAhJ+GV6L3BerOSUJOgN54PDfwDjsl4ee6c8pY99D/CZ/W3bcD
PsHv2SF+C7wL2tdhF20SK/vHzyXTOPKKclzQh4m80+EJeAGecvCgzmXa4d5Y
P27Uz6KtHxJHifh7GXdZjdmvqlJ/pKYH5ODRKMvTOMJqcj3ie8D4HnKX2POz
IymMeCptfun572AqF/vXonqWBpNzpBmu28E6dtUOYNT7P9vFfQQSz9Vc4tkV
Xsn5H628Kzy7witgfs4W5an+5Cfpkuj98j0OHwkjn9opHfGBeHdWV47XQ9dT
KRmPRPT5nx3ngz4LovxTK4d1z/Uu6FVknJr+ReYtK1TOI9/vgw6JvpUWTuP9
5HkN9U4FYJQj+wi8S9/Ecf9HrxjJn8m9PPI+fnVuFnZx6BWA+TlbfEf0/f/x
nZ2OXqq4BMAk7sonmd7vg3coMpT+Aj3SPk/hydT+nsHuvbjk+jvID36OD3oQ
9FDAGA/8Q4Q/eQsnOYhxACZ2xxDlT8J3aM/e/13J1C8yzlnuEX+/r4oPI3Hu
kQ4/RreIHL6wO1LC/Fwkxo8U8hYwkSugt2UxlB6jFYx80C/a4fgjcQn1mP4I
f8XvdSQ+AYPPkPsHSjL98oWR6iU4lxii/OfgZ9CD8R30TFKPr8oHXoh+HWWl
+nRPBXO/APQEqncpPZHKaWW35nbm3O28noxPOuy6briHoZpTu9xvQ+19tdk9
aKE03mhEKJW/Ors3sTvo/EDQQwlf8xZOdnhuN+d2b36PC/RrlOd2bCpfld/J
vq7mq/hJIscSEpzs9Gif3JOu4Z3ed1mc3UNYXPJLwISPpoXLfHv18C/hnNXl
ZPo94vmkP7g429c67PWhKh/7WdAD6Ar0QOK9NP0feCF2NtzPhfi9UOW3Qn9c
+ZUAo33iL8G7qjLOSPnBMF/AO/QlezpW+bXsic5vBRjrgPvRuN+J+5GoX++S
gcr/w9LugvEARj730xG7ZVo4s2NGOfnZAHO/WK5+LrmfqCT1KsDYJ8NODb6b
w5/VO1uA0W+sX7IvGKHuSwaMcefQn/KPkP0N3vnyVf4NwMAP3+fwfQ1Zn03U
Pgd0Rd4Xq2+UdhR8h/VD9k8ajHLQX0B34HvQI8j9ItnqvjroCfZyI41OcczA
E7nHJUvdv8rxCjwCrzwuleDhpsOPmlfZi9BfyCce15xDPwFMDgRQPyPuxYGf
Q97768H4uiezUxaX4yF8COeU4S+U53495Xog9ha8r6TB5Jyu9j3l+5503ww+
5xtJ24f9RONrRJ/FepF2j0pSvmG9EDuVtj5c4Q0w8IZ9EMVjALsPshrVezQY
dIx9E/HD4d4g3T1TkE+AsZ7RPmC0j/VC+YOCIf8wfmrvrEjjZ7X+EnmPe64Q
76P1HzDZf+dCRwTGvYi50JUrOqD+lOIu6QL9dDUP6CeZBw2vVL4HUPuxhmey
f9bwTPQkDa+u6M4VndF9RgCT9wFy3tA+j8sjcapae67m3dU8U706wCXdoT1C
Z7g3Kxc6AX6J3NHWh53fXXc+NwH/LWBX8wCYxu+6nhfEQZD4dg0mdhNNL4Fc
sOPpTi2ndQk9Gt8R/86RJGr/aanewSP6nG6d5OhVDnlYL1Sm+N2VXYGcD4I+
mYudAfwX8tmVXYHbHci51F4xUg5DLvHzGFhPiMsk5yPGO+zbi6Pk+3lISbwh
zjFcjqBpD2UPJ+03i6fnQ+BXxn1D5ZQ/g/jb0U+cC2lRR+5XuR+EnNvHOYC8
CXJ/SuRxYqLcT9vTXeodKuCZ9NM3kp3HVe9RkvswpL88Ro7Lnr5R7wwCJucR
lqtzMqAPEt+MuKyKSXI+gAfA2EejHOiWxLO/UPc1TXXo6Tn0a5R4gH+QnFPU
1iHiK0ncu6Z/0zi9+wbwE6r3OuvBVG4rPRgwie+/ruxloG/yHiLeA9T0+5x2
8gn4rRFPze2grvQeAmP/loseRPzpmh6DdtAu3/eDnwJvgN0cfxSP6l1gfG+v
/i389LdYPMktmY/viZya6k/3e4i/wrkLxHUjjknblxF61fKJHwV+AY0P8zhM
Ik/h701T8eHAL/iNvT8vw+W6AMz9MNyOxe1cqBf5WI+YH/LONOI1ysXLfpC4
n/HRTnzFFR8BjH0H9Ht7u9Mdcn2AAx6j7ncj+hfuN/vIcT6nirL7kXvedivY
vj6PO/bZJywyfsqejlPvBhL8llP3mUHuEXzAbrxHwcAvn1fql4iR6xVyyE5G
E1Q8M2Byv9syBWO9kHsgsF6k/9hf5kMfQT74Ts53fvTefq0c1ctCnOwm+B75
RG/qX4vaMRF/CD8s4nvxrpGb87qCPgB5PNWhR4POsP9FebJPXq7ONdI47WAa
J491Ks8Jhsl8tEf5QrC0W+J7qnd5sPPrat+E8WI+aFwjv4c3SNrL7M3eDKLv
xt1U+egHPQcXRuM/9sVKeQW/NbFL4NznQXWfJsaLuCL0H3IJ/ST7AvmuZ6Dk
h9BnyXtduvd+IBfAnwGjHjJv0P+lvzpY9gO/Q0/H7xzGd8Q/mMt3ZJ+WFi77
Q+gJ54hyGSf5XVuH4KdkHUp7daCUQ9Dv8R3qwXdcb8J8oV/En7pP+c+wHonf
7J7yc5H7/nAfKt53xjkT3btPdJ7UuwGws9L7+IOY3VT5J6mf4LW8P46Mw1e9
m8LtKODHkCvQd3LmTcVB2OXDRBVPin0SiQuUdO3H9DF/Gt+CuPNcyhH7gy4l
6wJxCbBPId4T929nqXugib9Rgwn/0r1jwON6UB/fv5N4Q9hBsJ+U76uq+DkS
9+mt9msk7hTzonsnifr1Y6ReCnok8d6691NAf/b58lH3MhD/me7dAeJXeqXO
IdN1dZ+tq0v03TLI7Sx1LyGRq6FGyieyVFx+Dh5NLumUnn8MZfEmNSTfIPwJ
dhHYydBv2E9y+Q7lSBzDTXXe314sU70DiP0A9jc8jp34092V3kr9T4fluyCE
fuV9lP40HkaTh7AToH8k3kXeb+J8zpL7bUEPJO5gu3rfHDDoD35q0B+JD5br
15PxgUJsn8XfQfdkcqOQHB/93Rkv5Nw37OFllR0C/Bf7VcwjiXtzV/td6Nso
j/7kjNcm28XvVI56OvkBoY9gHUHfAN+APkL0NnkvXzh7tydcyh8y3nsG2t8M
dY8wvuP+S+gvlB97ML1Bxf9DrlD9xIues5f31leReME6IXwB8QmIM45Wcemg
O+QDJnyzpeIz9P6MUOZnVXHMxE6Ed/Iy1TlE6FHYT9uTt8reSPx3urhy6IlI
0R5g8CnslwGT/bjWD/o+RRSVA5FRlN/r/D6EbuFX2xcr5TjolJzn1sWxUv9F
cVkecpDcA6HxD3v/8c6nBoPesX+h74WH0nsyRrD5xbzyuG23VLre96n3DqB/
0Diw1/K+WNA94Qt5E6TctM9Lb3XOAO2AboEnXj9g4qfT6B39AYz6c/hdSQnn
9M9X+v0gX2g8apAsb8fzZ0HU/yXtPwom618Xt0X0TV1cVg7Zqbgo6AWAyXrS
2UWJPbeiin+C3INcIOsd9uPIKGmvtbeHe+tfsPslXqh9MuaF2Md1cdn0vFlx
p/NoPL4CdEv9Sep8mb25UAUDL+APVM99buDyDHKB+Js0OU3iKTWY3kdcReKZ
yDtfdV4Z+CFxlL7svPJl5V8DHZF7xqHHuCdKOUT8MZq+Q+YBehDkFvx0l5Pp
fu6Ium+V23G43kr8Db1i6Hr0U/Z/Mq8XBJUvN5VfgNvvyb1Nvyt7PexrsPOR
8ymwh2P/pouPpvuw51I/Q3/A/3hcFIm/x3tgY9T9B3Z+MdDZ3pYjBxzxfeuV
fY2sR539jOhnWv/At7ndl6SavkHfVwugdIJ9lbyHpiLzT1SSMN+HYdyQj0gJ
X5OwH/UfSf4bJfluDj4gn4Opf7Z/LdlPtAsY9Ezftwlk8TYVJZ9Fe1TPLUX3
Q97sPdAQlfLfQQ/0nT4V74H9C9on95NBXz2oYOJPWq7mgcZpBVK9Wu5zisv2
CZ5GqHNnND5DlaN+TOdygEGPmF8Sp6bxURqnFizplchtb0H5RYjaX5B9OPyM
IwTlM5A7b6Kd8EL0Gl28BvgnwZPcb5Ri54l8mD5cSY6H6Gk6fZbzSY4vrocS
PXixs75H/KZxcUy/wztPYWw/HS77CZivQ8CQN+BrJI4AfKcse6fleozEC9Ev
de9ckvh3nENwU/tVYqfW5Ay55wF2Qd26xvon58k0mN6nHsDOWwfS9Z5LXDbo
DPSO73P4eG35Pd/ngq6oHlLbyd5DzvGGOPsXqbysJucL88th8EvMH41/Cab3
DmN+4ZfR7cvs0xRtE7tCj/xwbri6j/ML30MZ57raxIvt2QWf/mQVa2r+sbPN
dvV+ytWfGhy6VxHn31PF5RKbTw3eYhZLh349/5+RKeLrmMeDVj01i2HNV/70
8/wU4XX4+78H3jbLelBvRrsCeW43dH4/CL/b+7vMJornud30vSYW8fTID5/t
7Jsi4+7QD/SriuW9Hu8Xtcp05qVV3w1vbBXfdL8xbt8Vi5iRp33k/BI28cvk
XSX/nmwRfR8XLFXrmVUU9E1f1miORexrkZx3ntbve/PPVL2jjWPcx5V/fk/r
d4G9lf/yWaDsn/6fl1tf4m+zuD1lY4L/6RTxw7upT8YeM4umZ6b7fhhnEaET
8j4p/qdNxM2umbJfa6fYixduv2vtjj9k2ZuitfPPotfhlZ+pfqGfWeEzwntr
35X5qtoHW7R6etS6PHXsGZt4t/L8hUZGdY8q8IB+oF/wn//5zuvg/dUW0WJT
11+OX1TnTvE7uY94sSpXpKSh4rXG6t0czCvmeUap1vsONrKK3kd//2noVXWO
9YOM1S1TNBi/o1yOfmcVJV+83+xIPpXi9+R8KyLzzbKJCTOOD5rRUqO/bren
WLvZ6P0KvWzi1zuxBw50s4g3PY21X1hsolvdFz+c+soiSv7RfuleLV2cvSL+
rkXRMega9z95HHqxd9x3FtlP9Nt87Z/5NcrZctKJ6h6sJdc/rDFuH+jbzc39
XNeutUd7SD55OiQo78NHRw2NSvvOW1+hjOQHWH/Nox6eKx9wScbRDx74/R/T
v1Fx5nWNq3cVqnLYYHTf3OF8NxWHPWfo9IjYSiYJ24svTBae0dOGz3Tfb8C4
4g/NTBvx4p5hzdu+jxsUU/ewRNT9fcDd8yoOoNezBr4+kYWZHzNMXEuauu6m
j5fkA507hI+ZO9ZLzPpweuhvTdV9tGhvrWXVnPgXxcXXVT8c8HZyisyHHH29
ele/ml4BosPO+9PPvkmQv6f1KD1k99hkCQ9oe29r8etWCfstv3u/0roUCe/2
jPBL6qzw+F73vCd6TjRKGOOyeId882p9sJQnH1U40XTU/WDxxT3D9TE3DXKc
oumdK4X2qrj0mJGVDEMS1X1KoDO/rV/U6BZWS9b3W3/jX3NbqnvY/f6u/Dbf
p+FiR/sad6bFqvdUeh43jLz3LJLGy2jze/jzgIkDvNQ9/PnudGi8VNvnjD70
/NZPzS2yPPD7KmbY7eXjouQ6u7Guftnuw6NE6qsJU4NfFKJ2Qe3v2etZ5+4U
rSPxsivu7as2DdR5h6nn7h1fu07Vb1074I895jgxwRiW8e96oPWlims1H6T3
SIoT6Z5bZ312yPm+pYkFTSWTvBLFvE2hm2JDsM/V5OXQiIybS9W9N3cGnc4/
u3eshJFuHVvIWPKJul/G/NwteP7jOrK/S1f9vWfInDoS34/arBldxJwoSmYc
ehiVxyLreVs860LTkjYJZ7ZcfrnpFX6/k79YW6bykvOlhfBsdK3J6vz+sl3Q
mdfFq58s6yREvmfmbfVXhsn8sIdJPf7oEyvhgk3rvWvV3SBhpGLXuObxKUL2
/03oz57ulYXsfz/vAX0qdNXa/2DbpdqjCrE4ATe3p6e33I2bqNZZ+GW/0pOv
q/59dfbS2loV1DstH0SsbvlTPXXvavclzxb/6mkSiT+2nvzBBqOsH+0/sO7u
4ffMKOf/jzwx7W55meR8p/1y0H9SMZP4Z/2hGpHTUpzulwr8ceDF8fuSJZ0/
Lr/zU09Ps+hjOfjOlidKtuPZp3/zsVdjJYz+Rnh99a7N6mTR5XHG9nX32H1U
2jzN23ysb40jyaJ+laybg9cbZL7PBNuTxv2NEka/PijVqer6VcmiXJm/9j5o
rMaJ9Nv9azLr7E6W43166k2XfReT5XgvTG61KuCCwn/k3eWh029aJf7T/AZ3
v/erug/r/Rn/LH92Vd3/M6/8pVJXB1kk3/gt0nt42nyLuLe2wovWoo66L3vT
RfeEbAO7Z84ohpUq83eZRRax5Nh427rpRnoftja+KeXK3sjYZBHXwk7WGdVT
jQPj+3Njr6XbhljE9gaTDQ28lXxDWmvHz7dv91X9LX1rwfkXg2yyvwHhZ69O
mmOT81noyIzni5PVu0XFv+19qm1rm1g9qWnbf+IVXwjv0TFkY5ZBwhhP0Ng8
z97MtYmSX88osnuqUeanTv+876nuCu/o/4iDhvbjUmzitq3+wi3byoql1vyV
g/8sLC4Malq5xvBg8bLs3EpHGnmKmatO+WfsSxAL+s657RadX9w+MLziBU1e
bAxo1DsyvYD8DvW0SFyf+PH4ZFGh/YqVdwtWEe9lHhH7b1hFm4MHq3UKrCzr
le0U+/O9PMuMIvRkxoiZB2qKJhm/HhvuLoRvq4pFr+0MEy1vfmnxOm4U7z70
P71+T5g4Xbrt42tdk8XRL5oUKPpJmGhQd133JZet4st6J9d7BIUJj7BG16fO
jRWZf+7Ynlo0SrQovK9ixmCDuBdS7Neb+yPF7aqR5VtMMYr4oAo/TzwfKSpn
Zll+i9P4c7r3mdLfRorzK7s/z0y3iHMLuxUrEhsl60G9y08eKdx/pVHcOXhz
SfGFsWJmbKV6S5sni+/rv8kX/Ems+LtQ+YrjO1hEzcVtJi59ECvWfdzmZNYZ
q8avGx+MTooVsZa9xZtpemufceMCV1hjJV6B5+ur4gaYI5NFi+Hbhq9cmyD+
EEOyEuMtosJHv374Qc06Ym7D8D/flbeJsRFjS01tV0f82mHWxSd/pogp3rsu
hmgw8AY8YtzAw/WbHRp3nx0ran4+9oe3OwyiT8CUq7tTLOLLjt0a/XzBIAZF
zJj9IMgmon7q1ailhxCHV77Kv/KIpo/HrPgyKt0gAk416N3qSYqw+F+bsUir
/8fjZyvMm50iRkecHHOyhUHOI+YV84Z5BN4xD8Aj8Bo98rpHXGaC2Dn+6yUd
hxtF9QPNej3T9JWZC4peOZZhFMUCyg2uXNAmWqwz7t+wwijq9LoeX+Vkinh6
efvjqdr3oDvQIegEdIN5xrxj3jCPwDvmIS1VuM/vZRCHeqyrOrtusigYu77B
aq0fs1u9yTyl9euL4u7ulS+miN/mV0/bMCRZ3Lqy6cneKSkipPflhObRyZKO
QFegC9AJ5hXzjHnAvDw8fLB9+xlG4e3basyKaha57rAOsa6wzrAOsC5Ad6BD
0A3oCPOMecc8Y96BZ+C9497Gie9r4xz73u2233rZxI3+XdocC08WBTKyR5Te
a5V0DToHXYJOQTegI9AN6AjziHkFXoFn4BV47tTxZZch0+JlmjRiz8FXS00y
LfV1tfhTcYEyPX1xTZ2DTaqLm1NqfLFwVEUxPfGvhKwlMcISsnR+vWUlxO3l
GyYu+C5BlPzYLePT+ADx4udJPlkbk8T5XT+0mZFdQqRNqLX/ijVFBO6+1DzE
u6SsB/Wu6bm936A+1WQ6p8D3rVu3ixCPv/LqW2NebeH5sGHN3x5FiL0Xfh8R
+k1t8d7kR7UKvU0Uz8/WjznYpbaYW+bQ4z9KpIis+Z/OLpcvVHRMW1npeoEU
sW/sw7pt4kJlPagX9aBejAPjwjgwLowD40K76AfGhXGiH+gX+oF+wQ629J/N
fQbuKivl+czzHVPLa+3m67rvfJ9fq0s53ufR7fodHig7LOT9sE0bO595oepB
vRP83SeW7Kzefcd3qKfF0OmVR29W93AtmfRkS0gBdV8Xfke5RlsL3lq4Xp0L
hX1+R707NUxv1fvqPTz8wvrUeyB/R7mPz1c/v2V5qEDaMq3I0rnr1T1cXV8c
DB9aLobZ1WJkubdNzD9+UC9JXCiybuB7m+PF2GJrhp/8Ikn8Nrzvvp3L4sXq
ar7LHm6KF6d6LT0SU0/9jnKXPR7OvnhWvcec9NWWVn2SkqQeh99RrvPCur92
WmASBWK6b+pbwihM7kGB2afUPUCbtl0ZWnWgWVzo3NXr7TFNfz364y7/4ybR
tNnPxwtov+frdShfmx6IjzCJA25/Tev7s4nqz9rvKNf5YK3lzypbxfsLVxYZ
uMEsnvT4o9WFdLMYMTXg0duy6h6u49bxUb1mmsWa2Y17P5ykuz9s39Owt2Vj
mP8hRnxUqmRYzY5RMpVxgt17xg/NF61Sx/jLHC/XOb56HZni99tVOyaXPK/2
QYh7mtTi0woFCggx8MG1jQULsHertfEZK2/79U0RleJ39HtE7YPbo1eo973D
19beWWiO+h3lWlzt/MmphgoPsHsc3hL0QaAGd797blKg7vd2jeuV8t0dI9PE
L8sufbI1Qfzc7KTP0Sdx4np8gfCPfQyioe+OufH1E8WIv9qemhNuEBNGnQm/
NkiIbstS8rT60Cgqlaq86WA/IR4/9O0bpu23l7V6UmNuc5M4nhFsW3vdLNNi
QQuO3/VLEqUDHjVuPUC9bx3YObHf7YA4EVxmdXKpq0KUXtL89ooX8RJPQw5O
GZ7VUL1fmXiijsdjbd/vnX3J7ZvPzOLh2rqXzJlC9Bszd+yNEeo+MeDFWLRV
ozXnjaJ57/a1n1WxisjqJwovn6beKyi9a3vLxSe1felPM3p/dNwifAfVvxpb
wCx+meq+qt4jm5iU+tnYvB+p9+x+XfvPhVeF84hTh9ILFy7B7h3T0oCrPT9Y
kLe4WNZnlsj7Wtlrph5bcLtody/pZ8A67u1ddleTar6i1qNTaaN/VveCRe/p
uq3jKhWHNMwUd23FCRvzZwhxdkCBKU2uBIn3D0+blRYM+1KQxMPODcNWDn4W
JDK/Hzd442MVT4r9W+sJr1cf2BUqAnqNftXp03jJ38AnR232L13g1zBxO108
/iKutlw/SG/82S1oesUI2e81LUfNG7YqVpT3ObKy3z51Xxna++7CzL+OvIkR
Nbstrnf4lLqP7OP7fx0tUTeB+QWECNpcPS5hqvJ/YJw+dfJ+u3uIeoflxc9X
hixcZZR0hfIY99gj40O6xav4x8YTyw0//LGR0ksXo/jzzB+F7h5U++xjFf6u
dT7KKMrvXHp4X0W1Du30o+kna86V2FlxtNqnRlzc7zPoukmcbDMgq0eGRdIt
8pO+/PRl81rqfayAJX3GXByg9nk3rp74sdtKs1i223T18XgbPUelyY8Pa/Wf
dHm5ut+g4RqP+FHtU8TMWoUndxwRKbZW7909qMYDw5nidfY3HxMtZsy9ta38
z48N9e7ue7ZkdYxo3XLKL0M9nhvcb3eZ/VhDSFG/5EbbwvOIhxXdMj/5LUkc
eG6dOtM7y5C9/c/Bv/W1itBVN86/Mb0wXL/w2fuF5qSIvv3Lfj9pgLvoGt98
8rYbJjH+xvLmJ98vKdp+sffT4c3Nol3B8Y/2P/EVxgpevz/sbhXPPT5fMvbb
kiLDNrqPrWeKCB59OGOEe2lRaE+z1p2+TBHjZ2786+upfmLzg1Ij3ttiFmVS
5z3aWypQPK95YPJH1y2i3/V8+f2/rySeTYpoO2epWQxIb7bHf3h1MeHG5MH1
R6SIpn+cKf/lghqi//DU5R0Hp4iqWefTu+SvKT4NTe71zyMhqj/9OWiYe6hY
HfeqR90fTcJ9YNE82+bUFjtXB33ZuYJZZO9YfH/OoBDxsu3IzFFRVjH3+d+x
Y9fWFgsv15z5JEvbZwd8bMw6ECrxCjxfuJU/2/1SouhWuOHEoCoRokPF/bvu
nxMiOsDyeP0nEeJg/7OTX36jzfuQE11HmiJEnfQxQSvLm8UPD+Z/vuBYuIj+
9WXRp9WsomfXp2Vvvx8hwovWKfcwX4ownrjlM61fhJw3zCPmDfM4eNyQr95+
niB8Zx/0qdooVrgtb5xgHWESn44s9k+DVTGiywN/D592ZlG2R+vG2/6KFgda
dbeU32oTkzefG1enf7Scd9BBs4rnCy3/J0mMmT79RJGsBHF36Ifdnmw2ic5V
335SaHu8+KRNSt5PT5vEb+03TbTOixeDjFteBDUzi66ta35cu328GNroTuy5
elZRP+qfLaZt8eLTF383/2a4VbzdOz5o9ifxokn2zLkfb7SJY4aM9gXzxItd
b9v/cfG2TQzOG9soJTxB0h3oEHgFno+P27Irs1+M+GDHH+0+8U8S98ptunNv
qNZ+m80dX9dNEvPK/XivcCmzOP3xyC7xnyeKbNutEvUWmjW5vOCXgCpJYvqq
6UOsRazi29jM7p8cSRRv17Za9PufFuH+g/XXT0INImSr+/DD121iacGQtJp3
EyXdgI4wr5jnN5+v/W6bW6yoNnHzH3enCLFyffVxNf6JF+1K7O99TpNjX8a+
eBdcK1ncKbG6R8EO2n61SWbByf/ao0tnr7z8kVEsPXYzbFM+q3jSttCXdZcK
UaNfpUlNwqzik9VV8h0vZRQjMg374i/YRJ0GZQct/VWIrEfdu887aRMhLaps
WmowynWHdQi6Bp2D7kCHoAvQCeYV84x5xTwfjsnuemV7oih8q9iHsb1N4taP
G744qNFH187e5U9qsHHze+EVb5jF6ul7577xSxYnft+/3CPCIgrdW1R77SuT
WDVr0dqNL23iRdChV6lafw7a8tQ9MCZFPKxyofuidJPkE+AbWPfgA1jnWPdY
p1i3WEdYV6Bz0D3oUtKpg05AN6AT0A3oBHQzud/pKUU0vWL9jk4bmmh6RrUL
y+8lbxHifJUmJbftNYuoo2GmRausYsGyv1YPvWUWrct+FBT5wirm3Zx/9vf9
Zsk3wUfBB8EXwdfA58B3wIfAd8CHwCfAN7DOsO6wzrDuQOege9A56B50BzoE
3YEOe3zz2+tUTU+a8kfTwGma3rTW0KdK/7LJok+hCu9Oav2BHIBcAF8Hnwdf
B58HnwbfBp8G3wbfAx8EnwLfAt8AHwHfAB/BusU6xrrBOsK6wTqa+E/7jvk0
ukzuaPDvrdHpg+Tmnc+9bxIL1kaUty63SToF3X6e/nT8ylZm8f33I9dkPbSJ
6T4bo9oeNYv1X/UoEXbWJk7N63BvtaY3d9q7ctBpTY/ucWdWYmpgosj3pHzo
1BkGkSyaxD2ulyjOdJjzu4+mV4f4Ts2Y1VGIVfMulqkwxigOTH6+a72mRxev
VKaITdOrPzt5LXmqpkdv6ZB/YmWNb2y7HjAu6mWycO9nObX/slmML5j//d+s
ZnFqdo1vm9a0iMTNy8tH5tfkaom//e7+dtMQ17fjnKOd1f3v+B3lxuWpP8t9
UyXx+y9nVqx8U0GEt7z2/NH0SqKo9/snFucNkL+jHH5HuQXNToxut6W6GFLU
0N6vehUR1L7VyAF7Alg8ToB4/TT7Xv42lWQ5fDe9WdnnHRpXFylt0m83KlNV
pvj94Q8bK7RpqfFZj3wHvKdWFfdLh/09q7RJLG+/J+PjdlVFiU3Xgv+elyIm
jBmxff+4CuL5jK3lDyzW4L0Hw77pHyAqLFzTb2hWlNiz8ceS77xri729AhtU
vhMjTvbpfOtCVi3RZ3J7T+OKFFG839Vl67ZVFRuLLv0hTfv+UvS8z6K/qya/
Qz34DvWgX+gn+oV+ol+yn45+oZ9oF/1Au7IfDn26zIk9Xhu7PDT8eDzwr8M3
1L2W+J3Ey2q/oxzmAfOCeZDzYokpv6mpaqfpIWMpv8bjDMjHPvjR3UW2Hv1C
xI73Kz+b3TpGxo3hd3KurqUqB7oDHd72DO/VP8JL6rFoF/2YN+HDtrtPpoh8
azanH/vxrAEpfkc8dv7aB8Iv5/3dMHJCVOaLerEi4tqis30nhYsfPQwrar2N
EX/Oav985/xwGRcTcXxl2TLtI8TSz28/HLQxQfxdqEhs8VYqzqmIuc/OuMEx
ouv6rg3X/GQQv1gnfWeyxouvsty6HBxnFLNjbBlTIhJFRLFRK+tq63LtmtCF
1VokitIf/XzorLYP/nFGkXxjw9V7GLfvFG1o1dZ79OWQqz2rmMT9uoVnXhJG
EeZW/6N9I03izx7VSkb+pPYZ2HcgbRAfPHxTPpXi99JNOpTarcnXpDlZCUVG
mYTh3qT8mVfMomNkxZpBYSbRPiN/bMLdZOEXcv3hUo9kUW3zkch2iWZxueqy
e8eaq30K9i1IA7eeGXrtokmm+L3SgFOlhpy2imnPC8x9mN8sDDeOzArdYRUd
g9tsG1jcLDafe7Wt1y6L+NR6ts+VS2YxauStcg0LW0Wdpxmnt1dR/rryDQa3
+cvNou2/NhiqDHR+H/irUunTrB+quNiGm/qtNXVVv6Mc9nWjm6VPbTAnUdxZ
0XbuwGVJcv+H31EOv6Mc7Akf7JhmrX9Vwd+tzXes13yV4necW27Rx7zg3kaj
6Hyh1Z0rC8wSP3Jf1rnRxvXlzKJ6yvkWrWfp3hl2/C7tT47fUe6n5H/yJiZp
43zV4crT35Nlit+BjwIV3o58EG0RDY7kj+z9uVXio16rJ+3d3aPEpphlJ67G
R4hrE09nN7oXKV5X/+5qB03vC966vPvpWkaZVuvz4fjGVqM4P2FR3xV+KsXv
r9M6XglNtYg0886ubRPMYs3Cp8VuV7WICS3b3vthvVmm+B3r/tlIjz59joaL
94Zd89w8SN2TsGZj9OIrc+NErXfRUeeORAq/i3WXTiyi6cc9Pn1cYY2676SL
x2N39/nRotOD4MVFgxLF6iGNHs09EqOtt+Fd+7bS5OOa6uW//ChB7tubf36y
wfyFdeQ8TVx0xtp+fqLw2dwuPTVaCI+giKGpF+qIE02ufDM+ySTuJVzrtCzQ
IOezbfEJn60vr+kFM6+vurNB3Ufb6FVC67B8JrHzzJPfPeO1eqwmU2pmssje
vH37hrlG0fLNloO1QsxiiDXtYf+hOvvSJ00a/dnYJMSR9FVbv1XvA+75fPtw
7+PJou2J0z9FVzSJh73vlH981iK8t36ZHB+ULIp/MfNYuc0Wau/U5tvcVJw/
NT1Z2qeSTr/5qoS2jidfPb8+O9AqLvR++2bi7WQR+Tiu1exZNtE8/vDQG3fM
ov7ZMYW/umwTvSqUrb23tkVs2/cg/N84ZNAL7ObEPq3xefB92JWulupY+tu2
eN8pSHzZOXqUOeQnw+mOBXw8tqi4HNi/J+6ou2F8l+Ji2V8PZjQaECf5/ldT
3sXu0/StzqO/3Pqo71MD5utYjQJ3hiX8Zah7tFrehvf+3ad8ezWl+j+G+TdD
ap7Zqe47RXnD0ZhqteOKizpP6nSe3Mss7kwrPerowneGTdeKzRnfSd3rMG/3
s8SD+81yfLtLfT5133CL7M/5mY++ihtuk3CRS8VLjCtuld8bBzQu/Mtn6pwc
+Nf0sLMdM8f7iEuJb9undbfIfMz/wnUXm7WoVFac+WbGxQM9bTK/7suu3p2W
VxWpNa8M2TFH4b1omdHTSs+sLvEKPAP/8GsYStxq9O++CHYe4Bl4B93EH/gp
+9jxUPHrtJk9h91Ilvmv62zrfq2kRcIDLpV/vnyYmcYPa/1Pf/Fuyfx9YaLc
8kK106rYZPnpcY0tlTwfyftbYZfr33dtS9uBCFF/YLX7ta7j/a4IKf8+PflH
VJMekWLjmCXJQbWTpJ0QfNlav3Dp7S2ipN0tavrsjZN7Rcr5fhhRIW3r/Cg5
vm1DH4Zln4gQCy/1TDq0NVm2tyVh3dmsDSZZP+xsowu/vLrq4yiRZsq3oVIB
myy/vfmZGzGpVlke66veNw0nbtP6M7DW1WdbT6bI8hsccaatf70zdeusKNFv
4PR9FyJfsXcMXxtA9ziXFDbofL+Eo7FyPKNK+AvzBypeq4+pQNZg91jZ3xMb
dj4fXTlO1Ptje8cTHyq++rzR8lXNa6t7TTFf06bU8d4xVN1v63lpXI+IMTHi
RMGp7SLb2OT3GJ/ts2Zz0gJwvjVVpK35enWBYTFyfMe+a9rlWIM4aecl9zEs
VPOM+Tp58MDUZ9PU+dIrs/Y/KhOTIMq22+Q7290sv8f4Rs3Ms+1W5TpiQdLJ
Lh7hFvr+qzaeakfHHx6areJqlq0tETD1dLzY/0VkSu0kmyx/e13lAYveqLgj
jC+oh0/TeoEqjqXnZvfz14/Gi/STTzaOv2ej7zFr4+2z/eriByl1JF8CnwJd
gk6B9xX7Oq5/tzFJlK1f7XzPgDiZvyCmiFebb6Ml3S+fMSH8Y594CUv8XDl3
KilQxT3lXTmi0sWRSWJIL6P3rGkWeq5Cw8fuxVfO/dXXIOGPrnYKnFlPSPw0
bbzw03nHksRl/6fPj0y2ye/ve4ZmPfJR99gCPxW+/eWp+ZpBwnP7ZT2bO0RI
fN3cee7N3UNJ4q+7P3Xe4Kbi64CvHcHRE0LzCMmnwbfBx4m+pK2r0K+af7ak
olGuY+QDnzOG/LRsXhOjXBfIf3KjwuKHs6MlnR3c9qJEUEMl59eO/uTVB6Xi
VbzeF+c7vBeXpPyNDjzADwA9LWV+iSpVrhnFCL/HHQZ2ssj6gM9PW3sPylvC
KO4WM/xyv4tN5gNfHcofejhhrFHCrysu/+zn/CaJv8rh+0u4eRklvjYPPJH9
2ywFTzj9ef2OJU1SbkGOga/7dG3+4873FN8GHwcfwzrCegQfBD5Ph3pE9L6q
+Ahfd4CxjrBOq3xRc17qTYPM/3NahU9KDVN+SevMlS1X/ZUo6fjskZJNslYr
uoZeEZSxZOGmocnC5uPRP/w3jf72rf1naUSyOPh10pnNO6yyPOLQp1X57bv8
81Xc2IJvquYPe20SB+oNPmP7LkW237Nv6eM719hoexo+vTzWLIjureBRm+5W
bHdFxTOe+bD6Z221fhX8btbASQVesndiXxuoXuAv2kTseRM3S8lxrDt5Xp7I
/Spi5ebHqRl7lNxEeS53waeRP8V7w9A1PjGSD4BPg++DT51qUmJfxduKL+L7
id/f3mf+PF7pJY75BF91xTcAY32A7xzr90O7FeeVHjuke1R2p6eC9k9bB1g3
T7rebvRBDxON189Qfi6uZ0HfeV5rXMGl1xW+L75v8/feZpX4hd6E9TBsbrd8
+cao+7oDT7aZd36ZVeSfFZi/w/Da7H3SUKm3QI5jnX6xsuq9XeUiZHmsH+gF
WD+Z3ft3Cu6k5CS+x/xArkLOIB/4h5xyxWcB/9gxO1/naYnsvivFx8HHFnQw
ZvdZ4ZrvAF4dX6DX0fGKj2G+wMewbuZ9t3/UCm0dftMzvfHuDmpfsbzs5btp
JxU8LqBDh/VjTbI+zG/xuScvHNxrFROr3zv82mKW+aAb+Ckx36ez1k/ZsF7Z
aWC3wXzWS7nfrsowdf+5OarohLPBSs/Cul47r3DhsiUiJJ/A/EFPw/w1KV3B
60+L0mO4nOdyTMKdb3/95TeJsn7g/70FZ9d3/9o1HwdsHDn2801jlRx1xZcA
hy2ad+fmaCU3gF/wvXbDlzVubzaz/gSKGn6Da6z0yC9T/B5QtO27zPoe4mLn
wJOtfqgkfD+f+cCU4iHqtJttfrSykugYc+jLHXcSZPkCg9u+nJS/jsjKl1g0
80lFsaPbxNgeI5NlvuXjKPff2qlz4UPPbIo/YUkWlZ6uinP/soZofS58St2O
CaKk9b2UTSJSfod24IeEX3LWZ2s29philOk/szvdW3/ULE41Or2vWx2j/B79
QLvoB/xm8KPBbwY/2qNBfYNfNVUp/FDwS8EPBb9U85FVVz4OVin8NvDjwG8D
P86S767FvRmfIIauXnO3Xv540fTtw2upfgmiSPdD326MjBepV+JHbb+v7T9n
LG3b9wuTWNT1xYfdU80yhT22y9kWS7w2vjRcnTk7ueyU8uKXlcNuL5xeUBiG
HHTbOzRAWCbVrb+4WgHRJLZ00pCKKh7sHxF+5/BVm6g56F1qnXMFRd+J/QYe
DA0TPy7KHP3r0+Lin1/7px9ZEyou/HEhNeq34mrfcqC1T5CXl5j6ZEveyAnh
4m3M/l2Jnl5i0boH+1Y8CxeVF1T+fktBL9H3UuzNiWsjRfg3/p8eiPMSxUMa
/+jZMlKsmJbvz6KVvETIlqCrh03RYtqyQYdb+3mLenWnvIpqniDyZa7b+83R
8uLbwOI9a7URch5DvD64cbCbEFtmfpj5S4HK4rujq0Y17KrJj6/TDOMTKokt
e66UbDDLJna9Tb/3yzd+4urnK0c3HWsTHWZXXD46T2mJF+AJeAGezuUP+fh3
j+r0XoPoamKRpf9HF98LFAMt1g9r7E0QUTs8ej0rUE1s//vJ1wGbEkTt93ft
WFWvmvjZuHFSQBl1z9WBLdnfp3dOFjU8zk+0flVefLa206BGlUzi8RKxq1qD
yqLrnKJ3Pw40iWHBC7+OflpRTNzwqsv9K1b5/SHb2Ra+p20iuEKLzi0aVhZR
H/ez80nQcY89Pj53Z2tyapHv6Pfy1hTfvn9k4Jp5NvHz8RahTZbUkPOIecU8
Yl6LVJxrCjhkE23Otjt6MjZMzivmGfOGecS8YR4xT5g34AV4Al6AJ4wL8zrm
x0UL7v8RIB5v+7hLUGM1r5hnzCvmGd8Dz8Aj8Ao8Aq8n+o1flzJGWw/dO35w
smOypHPQPdYB1gXoBnQEugEdoX3MU97OH31/u18FkV0ob8FXO9Q8Yd4wT5i3
AUXLDn69vKooVjq0waXf1bxhHjFvmEfMC+YJ6xz9Bt6AR9iBYBdCf0HXoGPQ
9bvueY6GdFb3lXdIv/juWAV1Xzn4OPi6LOc4lxedeuRw9/m1RMaDF+OG3EgS
W7v4FC+4I8TpPvKXPx2otndikvwe5/hwv9Hy46WLd+ipUvwOvwv8MPC7wA8z
LqRhwn33CPF9GdOdVkWjpR8Gfhn4YeCXgR8Gfplbc1/dfjQ6ThS+M6FIXJlE
6ZeBnwb2fcuL90+cfxUvRg/+osHoNepcHn4n786PEGJKen7R5HGikx8Av6Mc
/D7wA8HvAz/QKh/D2cdttfk9vGLt0mLq3o/LLdvcevbMJFIq79uzSVsHF1KK
fXD5hvK3wI+AcvgOfhz4deDngd+n8tAF95qFq/jY0cW/KTjJ+99zvFvHzmhm
FqZpgcerFzCL0xu7tks7bhHBHm/Gtn+SLExnN9/43F35DeBHgJ/FvgwyLdLP
A78P/Dzw+yCOD3F9SVNappWfFkffN70ZJ+P6RkQIW7qIk2nLQ2umbWyi7l3H
vCCO7/InVd+63VQp9LE2tws0eWJU+3jg8YfMtU+z5hpFVvKsH2tpeiv2oRjn
mM1joh91ihJlrpw4UCI0RrSaUyig0LxY8WhX+rpaXeLFkJg7d/P8GisOf5jQ
enLteNHr41ot5pvqiLwj+t/bVyRJ1Nx3PnBx1QSx/W7F9Yce1RHz3ac07eid
INpk9/x5dq1E6ZeLS1lxuc8Og7g75EnQzesmMf/BkcxGb42i2LsfZ7ZMtAif
uk8nPjidLDqt2j1lQahFvDpZpPKB82o/3Ker37WBQp3TrlHsSlT++RZx43zd
rXW7W4XxU1uXt3vVfm/ItbRFB/pHCv8SA4otn6Tuh181PD24apFE0eHvWjem
3Y6S+J38e/HNZ7vFidDjL7IfXawjRJEdve/NjhNdnh3v8OuBRJFdusw/5xvF
iYdnt1Ur1F/bL5Ztmn/o/jhx53DyKq+lWvFytwdPmxMnLi9u+su+VkJcmZk4
/qyGnzZhlsKNNbq/vdw/7lFoolj2ycebB3mZxKDM8G6/ZwixKcz2V0Ffk6jb
bPIXtXYKUehB1xfjSpjFrQXnThmHCTHpRaMeAyuYRVC/sanPvhSiVK3XqxoW
s4jljSzGft+qfWvF+WEFFmnr7fMFr4Y/7GSWeP9o5KIp5/YliweeQzZunmMS
t9da64f/lSwGnJ10pdkUk+g1Y/0XI09ZRP36FZvVMZpEw7U/GYrnsYoJHoe/
XxxsEu9PKj++8e9WcWZzn++eW03i5s3iv0Tu1uRAoQ8tLeqZxISo2oEeS2zi
k/s/vN5aUtsf104efH6VTQw7vm1YVS+zmNGtWfmWM1LE+qpHa8QUM4uLz04F
+oxPET39yq2P08rf/2NLU9v9YfT+em8hBoS2/j+MvXlYTt33P16KSnNpINKA
VJrn8V73fCsRhVQyR5GElCkqUkQpEUVEJVMqlCFNpMhUKNJMSUUZQhl+x/ux
9vl6PtdzXb/+OZ37nL3POXuv4bXWXmtt9y3Fw+BpQ4DXG18GsZuR3lnLbGc3
HB0GAcGtCt0HmOS6Q+mv7gVPaDsL8xrQrj74/FffkzUYF6xB5DzKfZTzKPfR
X90yWXXTzltTwKmzyThitj5krgtULdWj6Ep49wHdIv1/1eX9zvj3e+N7VnRu
vtudog33pO61jHWm7RBil2z/nC2zHSB16tZQXwU6b9hl0+Lo/rXWUG87FH4z
ngF6t0XDmDnWxO5AO2TKBu+wWyl82P0lzDqbQdkHezSyX5/nw+aLfnYjrdgg
m9VXe6D5IwOPU1a+s1dkPmd87e/4xgsShuXWXd5Hlnf+XY8m4SVjoM8kYvkH
IYITEDcgDkBcgDgacTXiaMTViJMRN3ObTn5cHDWczBfSAdKFbnEz51k/H07x
6n4U3nvCMJRb/Mtd/o/fXGg0ybP/O95bg8wzzhPOG85Tr+g55+ZKfSIv6hIe
vXdW04Nl3YqFHqFYT5muC4PPNWlvl9mto/e3f566rzt1aYPKZlvin4nrlkyd
70bTDdJRbT1HR7PEAr5q1jn7K5iRdWH0o/xj79LrxHsa/OMrj9lAzoQfl+T7
LWH6zB8lhu62oCu+MvV1qSXYBIUrr/Oyhe3f5x0fLLaECYWWg3eb6H0ucN0Y
31fpTWDDYRd7uDfbOUHMj5bbKMdRbqMc1/lwa+TMPAC9lC99HjUO0FIS26H4
FGCPnOQW/2sOcLsxOvrFF0dwdRrKrawBKE6SnJWixQSDoLLiDRtZMLDQU9HJ
jgNL7eXDJ65lwbMjk4riOljA2dCot9KB0t+r68zHFXHg3NR96V8VeXDZrNRp
5nYOqcOBegPl2Rf1Bvuz9Qx44z9ByoTNBPfDmvB6D5P4Cf5LT+B5xo3NRxku
XFBINpVUO8UDr/JRJXtKGLBNKuvVEwUHiNZoLrxxgQljuZOdW+cCxFhP1Kza
RNnN4QVchV42TB9n42W9gsYx/yO7lWxIbFWsAUq+dd28O/eVBxvMZr4/voH6
voan4frVfC4Ijx/lqnTtz3uYcyHg5n79zNdc0L9i9ahmIheCBd6jl43kgUHE
2+tJnfT3RxT1LZ/8mMYfKxK187++5EDeRMH33aVcYEa65h18wYH228y1eXe4
pB5PV+0Cpal7af058IshlFDCInTY+1aX9UvYCh4Flo1qCQYwFn1w/ZW5JSy/
5rtg2hWA7q/SX5+8s4DNy0d+367IB7Onr/PeybDhdO7xmK3pfKjQVX41350F
3yu1C90LaD/Y/+Ypmw9Be6YvlLrDgnz9kz+UF/GB59oTZ/eW3oeUv9LALe2w
JXTvfJI7dwEf5i3aM95sAwtsK7/Vj+/BdU95UsfFQk/iaM/YsSSO/+/6WRYk
78KrgpPt9tSO4LB/rzuxfyb3Hp4JhD/ufQ/a7dnDIHWVMC4J22HdugiWR7/b
d8b/WVdZfHfeNO8aJqmX9O91ANG8sqQ6SQ7By59lrw/WPmWSetLoJ0a/cU+R
wGNiAZfERzxVKLi/IZJN6g1inBC2Qz8j+h0jty2YzE3n/6vu/1TiN6vQOZ2x
eRaX0BfqRcyjifR12nWsVY3MI8pVzKsxkO7XDbvPg13imQ++NomC6fBXEQm7
KZx8aOi+sLEKrJg6XW54JmVX8vMq/D6NhjPWLy59buPBra8uR0T1tWHf8q8P
X+TwIHtj8nneBj3wzTpTcymNB2MfGa864q0P7k6rlGtaOHDmQP2UU6XGkHMv
tz+Nor8TK1LHlngYg4eyfEFRhCPYy1g3+PIsoDHqipInJU+0soUuDtO0gBW1
YgeEizkw8P7EtGlyFP4zM330tIAHtr5Lj58NNId5reb3WdECiNtYF2wfZg7S
D1Z87lsmAOOBWT4X682hzvaTmLcqkzJsTNJfbrQC6XuvVWLvsyAmKOLox5lW
cPllujmH4s8rzv55Yz5aQVP2iuQibx60py6ZfcvZCg7W3lktZSoAg6y4RtWl
VnBEuuHMJN8JsPOTQCpcxh6k22+G7c2dALc1bsfck7SHcw0TN350NIeBAKey
nUvs4MPp2dbzW8wgzHSRRdgce1Apmmc5awobvjxoKjh11Q6kGtNbeu5SuD3p
u+HlaXbAP8woOUx9X8Otk1W3E+0g9tSc9yfH82F2gkRc4hdKX52eYb40SgC7
D5vslU+zg/dX3T2FNQUwosjuufMTW+hOzD9T3y2Am99b7df125JxxXEurS1Z
nepmBZK+rwsMQx3hhYyhwukODghYGVfWKzMgfkfGmBsMLrjUrLReUuAIpxxl
Div93t/XozZtRKIjqEq8N5dL58FihZWTRXcy4KvG7OMSIAD7Jw9iMk5R99eX
VFS6UHjwxjzdl8UMMu44DwFR/tVvXtqCZ+yd3UVCTLhhohmbO8EetJe/t8ky
YIJDiROnO5+ix1mBe7d6MCE0dVaM43welBmG1DwSpuzRHp+6xEEe6N6VEdU2
Y8Ldw3sXPAsXQLiYnzJnPhMuN+p76BoJIMBZTPbQSCaMix9WlVIkAFfnS6cY
NkwQe2w6XWfYBDg8rFXMgJKnSGdId0gXSCc4TzhvDQoJU9tTHCm72avKtoUF
kTHFNh+uMGDJjjyxRwMU/z9XuG/BE8CEVVbKlZR8THifaBxJvV+A3KqHWxzZ
hA+QL5Cukc6RDpEukS6QTnCecN5wnnDeuqpbzv60Y4K08rpH/Acc4HWIGy6o
Z4LIOTU1s1gO4XPke+Rz5Hvkc+R75HPke+Rz5Hvkc+R75Gvkc+RT5FvkK+Qz
pHOke6RzpHukO6RDpDukQ6QTpBukE6QbpBOkm1ARZ9PqdhakTmuco7SB0s+f
4otOmLGB0Tnxok05j8gRlCsoR1CuoBxAuYB8iHyJfIh8iXyIfIl8gnyDfIJ8
g3SMdI10jHSNdIx0jXSGdId0hnRnIMcw2byCBbXvSx7sUGFDll78nmWbWHBv
ndbFCz2U/ZgkNWnBe1OQbD7mdDrcEowmMcW7HpsCXA5e9mGCFUxpe+fcROHS
mfrha3QonHqyQN+pm8KlyldyVkV42RK/BuYx3lnWzn5VYE/qdYseajnmmgAQ
O7DLPXICA6byrRaepnDllTqh14cpnFku0TpxGYUrZxvqFcZQOJPLv2g0h8JZ
dSu/lW6i+PGouOOOjxSONDLYpaxK4UqsS4Z6DvUa6jmsC4N4AtehZreeKI25
agTyltN+VDoIiD2IdcJwnQ7rhCEOQdyCOGbEK77XeN0/+Z2mZnRe+p86WVin
C3/HI+axb87X1ssztif+vg7Z+HUOTxj/qisOJA8d8V32jOepQasAbpV6ay71
ZPxrPyK6jlPr5W3rXivTuBBxtucDJ/vofXTcCOatz/Pft6Otj/V3/IcpG6L2
X6p43Eifi0UtvK+4mAmpq/WMljhyyP2IS+y6E74cEKPzRzGvHXEN1jV6/p2x
8XMJl/T7Jl6yJDedQ/Am+rGS0o/EXkilzwv79pvNkaPjRX8UB5+aO5E+17nU
L7M8jwU1VQmXv7TReB5x1s091cc71DkkT/7feAvrDt26624stppPnuu/r3qr
0CEe8csg/t62PEc/dAN9npRf2RbzkkPo0X9ly3KbNvp8ldrPp+5bOBB/izvz
+BoB6Q/x8K22/WOjN9Pn+14/CVzbSudf/133biypU4c4Gtv9u+4c0j3yAdI9
8oHy59s111/qQ034bpszuc8Yjw1ke18p6UL/o9//ZDDu9Gzp3HZkCqRc61Rd
HSQFbZu2bLrrbw8aWp0jZxeXM7TCNBQSbtkD64RGobCQJMwLjPwVJMYH1ZFX
RhdcqmNkpHIjpj7lws4exQL10SNg//MYoY5zXHg7Q/KVsv0weJ43pBTWwgQl
jXj9Cr+x8FZr6kgDbS6MuRWgtu6oHCzTv5SisZILRl4nGkXnKcHbXK7WHTU+
zGk91bCXLQMP34zkfM3gwq6rp5sdHo0BXbGdRjEPeZTBo6HcUD0eWGOTXZxq
eSAZtNBS10WDfCd+N34nfvfOIMHw4uc82OKjs3Vy2yTwk85otX7Kg747hQ+D
N+qQccBx8f5+MZxjxIGfXR98q74ZAF957mrzjVx4WmG1hlEyBdwzzi+I7+CC
yiQFg+8iJvDTNmHL3cUcaIapbZqTTSFFN+LwylguWLuJmtWpmoKmxOufszfw
Qb3K6vliT1MQkq2PbN1C0eMM/ZwzreYweetKkahTfCh5und95zZzGN77Yfd5
SxaEXk6NtJxgCZ/9OClPgEXZi9GW2cLWsMDmzuZdtWwQH54PJ+5ZgF5FTvfK
l1wIfdo/pOFsCef4TSvHF3NhYZN6s+F8axBKv7nwQDsXruZ6hB7Usga7vsVO
mQ58CG25fWDaZgsy70gHp5waU5vsTOHdS+GONQtt4FSLQ0vaPXOweWY0pJxu
A2axUjrmOebw9ltn44mRttD2IU3sNYVXxO1NpxiOtAflhSsfqViz4Oar8Wbu
V21gVKubsO0hFrxw8PixZ6YNxA4tzR/vwoYxX6p8GWNs4cnxruK991iQzazd
pqZiAxeXfBdSbWDDg/ScQMkAGzjCyOpTauDCNvl1ejdf2EDFG9GO22I8KCoZ
s3VEkg08eW95fSyFe5a4TDXI+mwDg+1tTaZePDhncWJ9rbwNxMczSxyn8oHv
r5RRMcEG5vxKL1tO2V/LVmnaOxy3g2lX1GfI2/Jh1gvV2fEPbUF1yu6q0yME
0H37/dTiFTQfIF+IyNeFLnxlAS4nh0o/iDvAw9C1o02qrSCqw+XRMR0H2C5e
4NWow4TGOw0JQUWOYM04NHcYkwkrrRlT8s46glvWmuOy4pT9srW+f5QwwJeq
lR9HsZkwqtFPct1vv4nWEnf2aTYs85e1O3vYHrgze3dNo+Re2fupNt9CHICR
MH+rjBIXwidcHxx7xgEiZ22wvHmGAxIZPau2PLWHKi/x2tpTXBiWeEcxUsgB
9h3fWPCmkZLXzI+rH9sxwGSK6asZm7mw1GNpZdoGR1gb5B18wZ4HHW2Fb72/
MWDzFe2yAxF8YP1aef+6rgMcteTkTg7gw9sGibIvBQyQun3krY4nHwbg5MUR
6Y6w6nBRj5C8ABrWr/d8KQYQEjf5u1wBH6qf3qyt92KAyMYGRU9zS7DRqyo5
EkyZD53a0R5vLEC23OvQ7qsAltWzXaYoWENM6aID/dsARoeM95e+YwO5ounP
H1L36wkduVN/xAY8G7okArMp/eTVf19lsT0kLPUtK2sCMG6c0jjH2w7WTRR1
avMCsOsZozqDwvnZdx7v1BNlgpCn6GDfNjY49UxmR3Ioe0W20yc7iwsFJyRP
200D2BX30DdhPRfKhs1u+/Ue4Cc/VWTqLS7Er3qdFODChGmr5nx4O4YHzfFp
O2RPMeGl8FyBXSQfDgSa/jrtAxA9c4Vq+lH+P0cK50k+nuYiTNGb1O+jBBNi
5tYdKHvP/+eYQ8tFlJPI58j3yOfI98hHyFfIR8hXyDfIR8hXyGcqSlt9zSi6
yKl9VNf2nQm7+8dxEjUd4YG+bZ/nVybI9aUPCxhgQLthsXfvaRZMV32+cIYm
NW/sF3t8HKn+UiJ/bD9tDzO6znpvOMyGkJyxaqONHWCyGXeoU4YN/mlr1kdL
O4KW/vustK1sePd9u6xrPAe866226t9hQdWhgSwHCy4o6qn1P/zGAm7U95tX
GgT/HFOo7534udUyTgBvfx9nsEBe7c6AHYWjFX4fhVhEb6AeQb2BegTlNMpt
lMMol1FOotxEuYJyBvkM+Q75DPkO+Qr5DPkO+VC9c5SiTTgAyzjLKkKRA3dC
45jH1Sl1Varp23WRDc8c3DUKNwO0Ge06eHk8Bx5Mc5aLO8YE0/xL79e0suES
m5Vm+IIJcYM5m17v4kBW4sNix+1MiHRySPlswAGZuQnncrWZ8FZaKHrFYzZc
HlwZP9oGQPOZ1s9PORyo6X52cccnAMadveoLqfEosuCO4yQD6Mp/3z1elguW
uoejhH15cErU9dPLVA58Kr7wWUyID46zah2P3OLAotAUgYMbD47tWlpQtooL
m6SXbzxH2XFSphclHw/nQkRFXr7IKR4odQ9xJ7twYZFfsjDTUgAOGiMav5zk
ANdO45r1XQEMnR5ak3mbQ3AB4gTEBYgTEEcgrkA9j3ofcQDiAtT7iANQ7yMO
QL2Oeh71Oup51Nuox1Fvox5HPY16G/Uo6lXUo6hXUY+iXkW9hHoK9RLqKdRD
qJdQT6HeQjmNchvlMMpllNsox1FuoxxHOYZyDeUYyjWUWyjHUK6hnLvYUSil
wWTBEyNnE79aLpz49Y25rYcJtwWxvpVhFJ6+3ziqUZ8FgV1xFc+FeaCZo6F1
LIQN35ft++E+kQe8duXTk3TZoHVkR2TjMy4o16S3zqlnQX+0XJF0FBeCt1lm
5fNYkFPltTXpHg+O1rHE3Z1ZIJH/LdiojAdhjHahLEoOdY85sjoikAf11Ton
ZlJygXu6ScZAgcZNiKMQNyGOQtyEOApxDeIc1PuIA1Cvo55HHIC4AHEA4gLU
g6gXUQ+iXkS9h3oQ9R7qQdSTqDdRb6AeQb2AegL1COoV1COoV9565Uur/WLC
8ap9t6Kp/iq2DSmMd2fBaa/rnjeo93+q8rLSh8GG1tm/emVS+fBMr4/fGc4G
zoisy2VfKRwffyRXyIqyB1fOHP/yPGVnL9XJfNLFgj5Gk+LFhXwid1EOo5xG
uY1ymsjtd8m3C0ZRcspa/61QFh90p63eeOguG1yShI61ePOhrC/olJg8Bxxl
93l9vcCHY2u1eEuNuLBRN15TrZEP8V/2ypZ1cWDVhaG3oYf4cORu/Z5Eyn4M
Ductd3Llw+a31T6SspTczJr7NipJABoTo5LMqeftFXIQkdsjANWOtIcH7rFh
Z+kchzUsAZgxSz6We3LgxNiQRWmPqfnNaG6t5TrC96WjbLdQ88RVFH/5cC5A
7rubU89eYMKRaVOX61N2cXtTwMWiUia0SFyPtfzJhMmpzU8r2yi+YF7/EvGK
CWLBee1mFJ3LXh8cIbjIAg+X8a4vDrBh9pmeupdP2PDwWMa0VQ5cEOpU+1n+
nA2rympWphlzwSSydd6IM2zYqDTiZPI+DqxT8dnlZ8eBp5sqSidP5ZK4CIyT
2JZWfMLZdRxY3juTMaPkDsM6XMLnk4QGSD2Pk9+bHc94P/67/qzXArB3erR0
koMMuQ/b4X3YLig84OPgOBsYPi1AkvFgCjni79gP9ov7QrjmzU8T/SCAFDHu
htp1jvDqsF/xMlUbcLsi/kznIRP6flp8bmU5wrKS7ns73gGMNbwT8UECwGbE
BeEta9mwYAZMun0LYJ6rj6mWFxtsnrxdMOwBgNbzaTPHd1Pje7Xuy9p5TCgZ
d0heL4ay80fkbR9Mo8Y5R5QZZ8mFnJvPgpt2sCBAgnHvwU02CNWEpEupsqHo
yB3Z96Ec+FajmBXGouORNAoEku3eLHLE37NcPPxkZfiQ/fv4jg2xwm52cuJ8
yHV6vWAeRSdvLrZP8H3Fgw/Dd19x9eNA6sNX7pfW8qBMNWTjj1tsONcWMtso
hAtLXN2kjiZwIDq+bWrjTB4wfhYJOQdwwaHStTDVlpJr78Zqj1Gk88IL5O4N
XXzAgUonH+5+GXpfcvRvvMs88kHrMQdqFTLDDozg0fuW/2mH/eDv2A/Ja/Ox
qUrytYWGxh0Gei22ZF8eMxeJbdFyTLKPC/ph2O+/r1KSoNdhNziNO6h9kAVx
R59EaNf98RdF0b/jffj7X/dRuAj9MuiHQb8Myf9yOPj5XRUd558ZfWPRlVMc
sl+Btd/NS/s/cuDthS7NxLk2sKXW9FJBizkYNnteGtUMkOn6Ivc6hfMj1IsU
DbMpXPrw2Jp3FE61jd3bfPglm/o9b/KUE0zIeTDQ/JzCRbUqMaLaE5jAbAo3
Uh1kQ0p999ljikzo2Wix/g2Fq+TL7ySbvGCS958ROmb8lxgmqKxzaWZResr+
RrNILKW3br3ZOC+P0mtdJlJhQ5Se+yixTqF6kMIl0bmWFTG0f2r1HDWzH0Ic
iIC9rM2UnMqMGb/8FiW3Lq+eN9f/HR8KVs1XDprMBax7j/ETaxPHbHdcDuB1
SnrVR5YYDL4uMxlKZsPCox7lhoYFDJGWJ0dm61B0P2Ho1d1aMcC669jeccqE
pSwKdxpkLa4ZN66DcV7vm3RyIRu6ywR1PTMlCd2kay/osB/VyMC64dj+veMy
rf2xPBL3gv6uNw9+LWT0iRD/VumetGXuYcNInXf0Uy37mul8M4TOe+nYsvBk
Vg2FpzxU5CMljEFIrvCJzmTM3zOG2Q7LVh6Uo+uZYh1y9M/ieMpqJ+b5jzaB
n1GVFScpucO6wFuq+d4IZu4JSkyeJYCE4rcHP602gJXqEzoSXQXQfqbw25Mo
A1In/K98YOr9YwsGrFfnmsDroEnh8z/ReTN4/ZPYkfWH15mTuuLoH8Z9FNDP
a+6t55oX6gjdVcPFNNytiL8V6+7PmzkyTorCR3/tGzQA4LffMm/cUYwXofSp
zJsbxUvo84ysOOXCK7b/py498kvU8dY3X3ysQFw9/kZbJ5f0j9evDmttK8yz
BdnbitknCgXkOu7j8Ff9wVmWZN0dnz8nsOnI0k0OYPpmIqOnnEXi7JB/T/tz
7UMp+W64aeZRXxUOuY7+VawjH5c6q6X8MJ/4u3F8RXKSHK8n0effCuYctBqN
cUFTSd12vH6q0j/M4SOd5yImd9A1odEWPo559yFpq4D0j3yDfDT21AjX0PvW
xD8+e3zHwu8+dF1Ovahrnqn5tn/vY/XKhsQZTF99Zc5HPTa5jnrDL2yLzmN3
RzI/WPe/136kkMEL7t/7nFP0mzbnV81GG/r8/cDW6F/CdD7sa32FrhHP4P/U
3W/hq6e0vuT97d+nxuPp3MCqRFtH0t9U0Q7XKUZ0HlZoxUzNvZccyXhi3fvG
5PYpSecF/6e/plEHLp4JoM+PvJuRSw0zGe9T3zfLKNU5wp6QAdXaaj5pj3II
5RLKKZRbSOdI91qxRhk6r+m6p1sDdkt3n3Ag50hHT/Zwxnub0vtqIF/hugau
c+D44z4LBulDJ6bv5pLrKO+W+eULnzkB5DxxjsNo+cn0vgW4j4Frk7ZbaTT/
73o11HhUn4gqnTqXPn/FbnW9UANkHwR8P5yPPDl/S24JfX5MzTt9hx2t77JM
T6atF2PCWoXYgbhEHnkeXg/bKFjivI0FsRM1TlkU8Ug8Cc4n7lOA+xb8Vd+T
er+QM6c7457ReV3sc+rfjJWZ8PzEzsvpAYK/x4e6PrRBzj4hlgWcGZ8nBOTw
yfNQb6AeQb2CeqZ3MLq57rA14Yv/kmP/lit4jvONcgb3ecF1JeTH8xeezTu/
mQ17yidp5xbQuAXjcvhtqib3r7DIvivYHvkW+Rj5Ft8X9zn563k76LgfpEtc
N8N1sf9aN8P7cd0M18n+ijej6C/O/2fCt1428Bcp1k2bxCX4COnROdut2esn
CwbChsa3Mflk/9YJ5cNKFm2g9PWmNSmqr1gwrubRqZ3lPNAvcTslVcqCXaH8
TzeDef8cZ1Pfr33SpZqyo6/+PqazCb6K3DwrK+EaHV/mMa/LW9qMPp96Zsvm
O9J0/BHug3F2wceO3MMC8r1k/WpN2klxVxY5Z85+fnxcBR1PJjO8QIJnwybn
94c1MUsU6fWyZJZL0vjTdB5hT+g9lUIhNswfO93vNGUn4/PwupP4xOE7BRzY
EJa1P26n4O99ZhVonGE1jPdEeKM9GXfUWyh3kf5Qj+F6LO7TM9ukwldHiZYb
KKexPe7rg3II+0vS0hMks2g5/1964NsrUclITzo/GflSvsC9Jc6XAyqHBwNe
uNF1spEvwzJk9iQkc8i+PPh8pN9w9vKpy6y5RM79lxzEfXWwPfaPcWzIZ7hu
i+uy/7Vui+uy+H3/XrfF9U3c5wv3/UK6x7yikPXxV+Ot6To2iEsRhyKdIg5F
OlLttIrU+zSOXEcc+jcupfft+jcORRyIuBBxIPa/b2v7XfZ8a9I/4jCU24iz
EJchDsPruG8Vfi/KtyGLo7kv6ngEB/0XTkK9jv0hDsDr7We1F62n7G98v6Pd
jzSMnFjkHPUOtkd6Qz2EegevIz2gHkK991960Vs7Zp9wOi1HRlzSqJf8f+TK
e4U6iSvS9Do6WyD9vPEj/bz8xKSKanvO3/npFF/38fY5vb7Ag9Pl0n6r5en8
7cQxhU/jS+jvQ3pVTk/baUXJjQt7fi3sc6Xl0C3x5wM//h+5hPsqEdz153kY
h4l8h3EBuO7/X3EBuO7/X3EBiFtRjiFuRRyLOAuvI85C3LUx9nzVoadMImdR
r+P9OJ+o51Gv43WcT9TzqPdw3yxZXmHGKmtarsc0n1vaq0DHKUyaF1I4LYu+
jnIa+0c5jXIb5TRex/FFuY37dqGewfez1F858Hk/rUf+S89ceKLt3JpD949y
COuC/b1P1lQynxiXgXEX/xWXEaj2abgTRxv+3j97OPkd5y9axXGJyhENqO87
Wr3Pnq7jjtePhIvN2BKkA4vK70ZNlqLOHUS6xKJ0STu8D9thP9gOr68IuH40
uYUF0YJZH46lOpJ1ZFxXxnVkXFfWkdx0/xIlt384qM49MINJ1j1xHRTX4XBd
DteBcF0In8sZo97e98UGTlvHvAv0+sDAfY9r1tc+eF/TxRhqytuYajrEwDyI
+M3R5Y3aijBUUCO48YDOWyVyecT+7g1KulC15Nj/+BS/G/lE8tmOyTwJPdIP
9vv1ubCaYekAo9NHhLd0GR/4V/2shRt7GYY2I6aPsOdD/Oey1KLnTxj3Ok/Y
Xeunn4vvgc/9S14I0e+Bz8X3GKP/9lnKWmGwNRefeM35OeP0wcFto8cPh/dS
lS/nZJ1kqJ9ZNO7aeAHYGIlEGm3rZqR0d7ceD9WCh58/r3H9Ts8zzjsr9Oe9
LdO0IPNhQ+T0aDbo6gQpuLO1YXtByHVWLBscC5fWvKzrYXCuCHK7ftJ+DfRz
oF8D/RzoB0G/CPpZ0O+C/WC/+V8OaRfZmgFvbOvlVhELuP6mX3MGxwy0Hmy+
urzLHMSTem6+kjeDdmfPIynFFpCxpFt20Qa6jiHm4U2bE77l+l5b4BYNt9XW
toL61unV/B5rUJovu1ljlBV4hmV9uKZlA4opvzSP9FnC+5kBP1yP2RA/MfqN
MQ8R8xJ9TxdFeCQ5gNGWPbLV6+wJnnm7b2ZIiwEDQgt9XhbHA9kv2bLu1IYn
Ro7ge+D8LaXJAOefFWYdo/q7zUssPeMNxM+Jfk/sB/tFPzX6rVH+jOq+IO1K
2Z8HznwR6Lr/kU/G9PWWQP+NrLMAPFOXzbpjWGS/9eFP7vAelwBoBN9tDKH0
1GDBp0j9YoAViWf7Ryj/Xz8ryusLLaZ3A8azwb2wvXpiCJPgfXyPv+w/Y/q9
0I+OfnX0o6NfPSZojcZNSQ7pD+Wv8fiYwF5KzzftvfpmsSiH+HFjzy1Y9ego
C26+vvWytIINEkXMGqf9LAhJ/sqY9IZN/LYoRzV+pS65PIlN/ND4HHwu+unR
b49+XtTv6OdFuwT98OiXRz87+t3RL49+evTLo5/eczBuT2YCB07GXnpiHcIl
ebGYJ4t5sZgnu5Y1WJqobwvPZ+jt+lloTfbvTkiV1AhStgHlGzlZj/cz4cPa
cZHROSwQuaDx6WwEE4Ji6hbtek7J4efD6my2MSE1MM5Ftp1F7DrUQ2jXoZ13
cvSdUetYbFB0H5U+GMqB+fetA82nUvO2I9Opm8L7dosflKVQdlK2w07jEEoe
i8SNcFNR5EKdzvLoNFsezNfdvr2WsrttWnq9dpvRfqHtmT2ftzrQ+bqM3jvb
gtYAqD9wrHQopPdFlrJ1qBi52IF8595Y0TXvw7jgx17rnNnDhLiBtZM1tHjA
Xzq9ee84Fgzb9bN7kwIfks7yw9JSWDBDY2felPl80K/7sJdCwnBdfVVKFIWT
NJKFwfgsG74uKou3+cyHiw+P8rP62LC9eZQV1RasXPp6h1G4QVKu3HngsQCO
mks1jfGivr8+43VolgDkReSjFwf+oYs2PhReaFdOSS5nvOC0jp3dbgtSlSaD
UbpjCT/nXa817KgxIHod97k6/+zh8IwOO6K3hv2+rGEPLuVyOT8i7aD6pfpX
3kI+RDgNXrhE8YdLRX2h2Dc+NL/VdJ4SQeNA9Rb78L7VFB0mlh+JdOSTPLGe
sglMXgWP4Bzcj3mMk4ZmnyobxNySJo25zYN2CZFhj6nr+N74HbgvFr5348zd
F6y+mhI/7Lldn+VOadtAuVupt5WiFYzMz3paTtkLs7jV6nMkrCCh+d6DRyU2
MIy7tdztjSXZz/7ExgOcgam28K05z2BNiD28fnbtPG+OHZQ/GeZhOYEBXTra
40ISAJw/ZghejWJA+MWSX5NPAxQ9GYgMf+MIYdPWjt7QBYRu/vKv7aRww1Dn
G+3HLMjYcWvWAnt6HQfpCukstMxfpmE4FzqOam654s2DinMbKt9K0PsxSTRE
ijJGMIi8/F976rx8k+mnxzcAFk/4IV0bzIDZDfFnTQ2o873nti9bwwAxnZ0D
lNoC219WbQ+PM4m8OrQl891gGRNu3LZW5FhQ/Lf8tDrnAWXPJM9tva7Jghz3
gwaNM//gP9c/dZ2mU+d+P45/ncsBT8n7AbIaXLJ/96/ffypccPEzsf9gwIOO
bfecnlF29GKFeT+vN3DhuHz+iQ8/OXDbk1thF8QlfLT8vWvYsB+WkGnXPyE/
0Oaf7/thSfL5Ii77jjgsa0/svcifG496XDcH/8n7lKqvMmBfkIatUrUlXGFp
ncg+xoBryQeqTeotITpG9OXPKXwI2TCkLSzChhDZKVnPPPlQesK1b8UvJojE
VP5QrrCBawMnL+1otYaw+JGp+ncZhC6Qnq3bR2pP2s0Cw+hIn2QHPhm/yIGD
Z636J4PXoELN6fijjGpLcZeqsTaw9aWM+6XbgwxHn3u9I25S85B88zPD/yVj
P9fCZ2wIC2pTLqx9Y93GSNr9wkNWggPXGLUGWcP6GCmle127FHkQe4s97K1R
M0NRtj3n3kYuaLE6ky7IDjFEH+/PqHvAga/hYn0qVuKkjh36BVBubtMpMDiT
KAVYV03oz9/ffoEehkL3npacVRR9KW3JexesBBnjB4S6EzmUnaCXo2Q/Fvx2
xLesXMCH76uPc8x0R5G6bLhehTgQ/QbIlx1/6rjjOjeue3/mv58u4sYE1tC6
7DfDDCHx2bsv+yj7dd1JF9GdCw3ATc1kcooeH1r3S5vt/2gER7acvu+8kaIr
5e5KN0MKt+UuLr865Aj+v4+bTSHsnIWt/jwm1Ms+nyF93BQadrJWhAqzYfXG
hbt8fEwh5/G6YQmmXEj5Yf/6p5URxO39xYPpXBLXjfOLdePCth86e3U2n1x/
tX7Ol7djqPm/vcQ7d5UZGLrIB36L4sG94OfODzimMCm/njf6lQAOfX367BZ1
XTCyxeeJlwAaMl+NsPM2hc33++eePPO7rlV7Du+pJVw8ZdrlI8aEhVOX1RTt
s4DBiT3FBbpc2LVGU+HAWDOogBe8yzPQb2QKXiF3b7cu5cJq7zUsk4lWMGZO
TGFRIwd6Rr/T6VKm6RPr2P3SOH+5bw6ftMf5wDpp365+rLQ6JoCFM479clpn
CeVNir0fJghgILmJX7rdguwzhutz6BfHupuKg5tv14xiEXm7Kjdq87NjTPh5
Nk1tYIYtHLfbuCJ9FBN8S3Z83mtnA+1ZbSWZbmw4OcJsvmiSJUxyEmxu2c4m
7bEuJT4P9QL6xdGvhnUR79pedaqo55L2uaNLmKtnc4HxttcnMdkWPmaEi3Q3
c6B8WFBb7Hkb0FHuzDR9ROkTV5WWGD9LUBa52ZLWQNcpwbp++HykZ/RTrT1w
1m3TOQG5f0Pms/biIwIIDxqYd+qnDRyrGzzaZCSAmlrHB62TbUDDcvBcdIkJ
/OhZHs24QO+ThvumfbNLPBEsZ0rk2HCfSvWPCyygdZTlYLaLI5wzTy9gFJlB
dLds4sAZeh80XG9E/xjmxeD8YN1TFbvzouUyrP/j98dzHE+sE+l3oUIv4QmX
XMe6kn/Vw6XkCfqB0c9Gv7/1itmDPHiztunsojYHcPYXxNse4kHwnp9D+mMd
wHiXSKVutoDc36emrWV4WwDJGZpDVa8d4H6Cque65QJwS5Ra6a7nAGNHd3ms
3iEHB9eYcVOqHUA8bLJM3RMhUMs6/U2tlkHqYqCfGf0J8pyRERQbgJtmut+8
dypwPuRQht8O3Kd6NIxZ5yF0MJcBp6uOzmjn6ULc9Y1fhScAXHHZXxXTawyr
78mHdG2j69NiXWpOh9qTE9foOilvMupemGqYw6ealQGjXYHMx6KRixM7ggGa
XMffCt5pDzrtbGlxf7qODda10bWPXFdbaU/yrXHfOvwexF2Y54T+Jayzin5x
fB+cH/SzozzAOpJbtrz1FJ7GJ/eHnDE48L6fwhm/j7kAIz0M7yQl80Dq99GD
rlOIdQ8HMq+9EykSwLKIIO/mfICUVydCZFYIYGSJx+suH4Deoeu/ZoYKARNE
WzYV0nVJkC5xfjb6rPt1fgSlL0cXZBxePRZG5wSMKlYGsBIyPBa6TQX2LI0M
szjMJPM1QUf87c5cJrxZGZTuztSF3MqQ5rJuJrx2eBj56YkxqLzZ6Ftuz4II
/aDaR8sswO2a8HepGywYunF2a72SOciJjH8sN4qOG8A4ApwP82mGi5SPAcxa
XBkecIq2kw+fm/DpFd8erplUdd4/woLQVGc/Szc7yM72Tt+gQeeJYd5Y3qj0
zmBLNnBtujceaabXAbBOKq4jYP84P1iXU2iM26jtbD65jnU9cfxQHqFfGucH
60piHUq8H/1u6OdU6pF0CGyQhcfLQo8F7GEBWLtFfpwkAjL9ip6X57DJ/PTV
x0XcXE/ZI0aXP16vMYEDw69cvLeDAw0rvpWO8jGGCVr1KqF32YQ/sK76Eoet
9S9FLYl9bTxtfZXwZ3OwD5wmcXcaZZd25Jfuum4G2+xXDZe+yIY7yYn1Oy5b
w+e+hJ8dSzjQIalyinXBCnaecX/FbWFDceSJ9QHx9qS/ZeMvxCV/sYM82Tp+
tR0HLIPv3JWXtYMj15a8jr7DJnzDLmLKslazQObbvJdJ6vS+HEpeI7u2n2fA
V3H7thQX6n1mMnRSWx1BsGNOeHYnvW8j8hkesY6WZOjPMd2UHb3J7XJ0xUQW
WAhbH48z4ZP6BTgfWLcT/a74fJyP/79+YcRpuE8BxhkhnyMuQ5yGuA1xHO5D
gHSI/LRofbzzjiQufJRqehwlOgWilFJ/5C+j60vjPgKor3AfgI2z5bjOnhbE
71Gsm+FlsdQCDg67sDTahQd5s994ujubQ1zyjFcqpVyiR7Eu/Sq7NKV8DRvS
/tjinQ9OaNqT8yqfc2ZR0+1hwG38ueXGPBDRVA1eHmoHihZPp3nc5oL1YWnz
va0Mcv/wa1fPl22j8yixTjp+L6nv8SeOBPmivJRZV0fRc3r45fHnY+j1ZE7Z
AYmHI2i/DtptGAeB+3Ri/4hPMC/z0BS3UINZstBrO6fZaRKPxFn9bxhauIRf
cF+Bn1sK2WocS3I9WDkivWSqNfSHjhWfXsKHsV5FJo0LrICjdm2GtzNdlx7r
2qs+X2UrXGhP2mMdfJQrqD8wbgj5A+u+Y5wLtsc68die1Hv6EzfzX3EYeI51
0bE9jh/GgWz0bsu/dIBevww7NZRQK0THaSp6WeX1/6DXo3B830m4RbAqeGSf
U7yOfIB5p2iXoJ2CcWlk/aie1dEfbE3OcV8D5FvUDxjnheON+wJg3BS2x30D
sD2OL9bF/684IDzHOvrYHscb44ow7gTvx7rxeD/KJ6x7/v83LgDPsY469ofj
jXEJuG8x2pOoH159l1mmcUyMyBMp/TN7RWuk/5zrws71K8v0DMSJn7s+3e+B
0MlhEBmsveTmKQHpx3XZ7uycawLQOGZdxHYQhx8PSvxiygSgcGchQ4MrAboa
0jMyBlhk3dlIvOaoOZuus/5eWv2iuS4bZk/y8fYuUAVNz88FJYZsGB3auZQ9
djQY5WlcqCnmkHUL3Rs2IxYQOaAJCzYNufne4kDvrVNF6yu1oP6ndAe3gqIn
F89Yk40T4FtkgeaSSwIoq3BezGZMgPZhWasu5gugOCr64KOnE8h34nfbtjLy
ZL+Kw9DhA977OnQJ/UdlGu0I3mLwd91+Ss7ieCswg5ukefqwjVknp4L53FR/
TbMtrjTf5JJz9qxYZa2jHLh9RUp+YIYuoRed3rnhM2aYkjrZKLdRn3RPA8ea
MyaEvgcDNwac0bcgR/x90qHZz9arUvJ978frOWusybzhPIYW3o1ccXI4nFFe
nCmUZUf2gUY+wTqfp5MCxbUd7MFQclO4vI01uLlLZXres4M3bx5kv1/KgFtp
9jsza+zI+Iz9MFd4uYsD2Zca+eh/16v0IOS5+f6VXQzSv/ieOA/lzQC+HloL
pzk4kHHAfvE5796O1Og8xIRrsZFqO48yQEmC4+ZE4VJOleEdXhSD7HP9V7wX
9bymjoXv31TQ7bAfbIf9IP91KfruKr7ChLTb8T03n//x36fTdSixf/R7Vi4P
GxxbTceRvE97Grwumz4fd5XlLD2Rpvv/4gN8X78E2zARBkVXT+zFNcy5kJjz
MXjInw2fV1uVJ1twQWJ/pf1yf9q/+mJe0tX8aLqO93/xCdLdf9ElPgefi8/B
53IzDt769pkH+dtdTJ9O4cL4bQ/qf8ryQeRuubOTFpf8jvfh73gfygmUG4ZL
zqyd9tWG6DW5mNa7/uX0fRXelq/lTn1hYN2JOTkPBzx+fGSskPvxcJNAEQ78
qNS45dbHSC25/0WmexTIXFhqPvBqFKlTcTTunXLeFAE5Z3arlzqUod7UgJnG
NrpqFXzQDp21XTtKAyq+yWZU3qPkuGrJykZXTfDVi3JrreUTPyX6CcR3R9a3
dRqCi1BPv3ol7rejA/6G6l6Xb/HhwTDJ/eoZOlBqlao64RofOPn5HcI3JgOr
UWnf+xd8QveO45aeCQkYRfi1rDF0o+IeBRKXh++dv8NnvbuhI6GXTZnnypdL
UPauhm/K4yhlwi8Yr3PvhOy8K3sdQNpCbl22tjGxD7G/s5zH62W0maB9rd9W
YmgU4Vscd5yHbS/XOHd5iYL1B8FK5jW67iTWocR1CFyXuOP6XW9CeDfj0O1S
p7zxlB3Z+KmhvaWLgfEHWG/x7snSwsjv/P8zT7f0pyTlXlWAkJ0ZppZWAjJP
OG84DzgvmUNlZuN/aIHd7hNzhalz3sNDq75s0YIB+8JfUY/peflrnqhxx3lo
8PIave24MfSePx/L6KPs6okVj4dbDTFUhw2GlTV0kTqRWDcS60Tid8xPmhlS
kCuAm53+k9as1AKj0bGS5c0CqFTeJF0+yxicRDbtL3kkAOG2M/HeycYwJ3+J
y7pVWsCulGV+zaXvw3Z4H7Zr71ype0DLjOBMrLOCdVdInRTG4P1+H0uYo+14
YVqKHZx7WTozf6klJHocitcQo65vbdHixluS+7Ad0o2hYfwKt+W24N48eseZ
L47QKbJE+eF2W/I73hduxWgwq3EAt5I32xzzAN6Oy17bctUBTK45dHs8pexV
bR2zIT9HUleuQj1sRPdxAA/2nWRZfxYcvRyscmscvT6xuunAm4EnlNxNGnbr
/GImXBUqjoUgJiQVTFnx5Sddz8Vt77iI6gVMOHttoHJtJPPv+iyUXL7dsPVq
RBeQdtgP9ovPwXbYD7bDfgZtTu2WXsuCWJ/esaaUPdr8gLm8fi4L4qSCR4hs
40DW3Z3v9zT/wVeuHHiWat375DAbct0yK/fo0PalevHp6gotNtEPHLlp2+6V
sMh92A5/x/twneVCzzxn1esccj4m45DYdzcONC5tUoqZw4HLxf5bUhTo3/+r
3cQ478LzVRQ+ys9wqlnLAcfNgd88DOh6Lxt0tcaXRnBgZkvcYMhxLlm3evwo
YMvrOC688u+/Nm3Pn3VHyl4az10/fhzLCvRdLuoGL7aANT5Hls59bQUHsqbE
HLeyIEf8vSYzVM1ltB0MvB8JB1daE9xRXjhPqyDCGpxP3Y7mZ9j+Q997rSG8
reQ6h7pv/mHDQ0OqdqBuIprTtN6RrFNinQysmxFpce9w3kog9X8+BqjpLH/O
gp0JS6/uiaDjCGp7X5U+2cOEzUVG7+clsGDu+qN35TSo+xrL2cbnWMTex9/x
Ptut1WaB8RyoPqFUkUjh8AMewh5LWzkwaavT2+jd9DrfCt4ERQ97NshLaX9b
fJBe7++PvX3CdpBNjnsCT0x7Qh2LtSK/F32jj/j7beOkDwV8HpzoHPZg9VMu
HE4s1DggzIP1TpO+O8XR64RRcz8XtPw/64bz16cfKn/EJfdhO+wH+72Q5l39
rZ6uj4hH/P2A+uy06Barf+JYsq0IH6/1X9a7ScUB5HUFXz4mMWBB3uoNnRoO
sN3qqNNOfwZUVZUmqhg5kPXnr03PXvQMZxC+6qxyuGPxO96E8WX93GhqXkI2
+C5YBpAeqVEVtIMJldV5vdd8Aa7N4zZv59P1I3196s98ACZc7Pv+cQeFn9i9
1Z9PmzHhcNXx19JOLDg+ySNjryeb+GOQj77YT/y6ms8i8RY3NKYEtkmwITH2
u0T2SLrepAjjzIiC4WxwefxgxtS3bFK/tHWzmgOw0Y61AJVIpfgdUmYg5fds
V5C0Jai2XE+tEzWDX4c8NEw1Lf85CpsRPfuC2+E1lGsF3wWzJ9RQ8nLo9zHB
ivgBrwRy5iU72oGd3jjHhTkAG9xtt5So24HFw29LPPYAfC+z93yaZg/Wdvcc
xKdQODbGb8d7HXvYayIiuGnCBLUlI+K3aNN+NNn+ExOXKNH7mx4ucuzmv3QA
nQlujMY1LPCfaGGd/80RpgXeOWoeT11v2mfn1+IIH7q+O24vYoH9Fs1ohQiA
raatt67PYUHFreEy09UA+kI2NKz4TLVPt0oouUrh8W+BEgWDLNj9sVlh9WUG
1K1KETp5mwXrJsYI3ocBNETEHRRQ9tfErWv8Cq4D9Gxcy9tXyoFTnS4/hm4A
rOkfEPax5MDVSDXnvkEmSNTlhmjGcgCur328pp4JsWrhfkvrOf8cC5lknjLP
OvfLjGOBvqn+L5mvXHh6/4lQfxYLNN98m+Q5wAWRRck/3qmxITYbBpR0eP8c
xeh9jRq9M1V0llHzPKuyKpHNI3RSaqTOj7XikefM/GayYtVpLvEXIX9kKG9V
DVGh5WqPsYjaWIpexKukQ/miPPBVyP6c+IEN+q/u3Jz6jQtiKRZHhL6wAYb/
ylYv+Z0Pk7hJj5K/wuszrTyf8oGvHiZv/ZADi9Kiu+Z38v853uEQPIS4t3v+
dYMdB38xomsnZfuOkoHA4PvPAkxUYFL76b0NUnKw8WTxwhZXFTAbprgkuk8W
NgzX2/50uACM7p+Oq+6WhnaRd7se6Qhg5g2e4dxQedL/tMSrE7vfjoIi4yOn
e1qtoeejqI1ohxLBoYhLCzoeN2hGMGDyssL40C8qxI47XmhcaG04lqy7oN8A
cSniVMQNiFNLP0vqOuxzIDgV+QVxKq6r4Tobrivhulre2qJstRYlkJRfVe3b
bg3xzXrS3K3G/xx/2MOL3CLhvtnGcG3tnnhzMwe4UC+/9q2fPdRonjpodsMG
crwerQx9Y0f6Rb0SEnZkSWeNDaxz7Xx2foUd3GiT1AAXum4f1vFD3IRxNBg3
g/2gHjL3H7rxdhVAsZv0pr2RTCIHw6cq6xyLZ8Oo1QuultmwgG/7NtU1mA3O
b30+17JZkG5xPX8zh8YNQ5pgKBLGhX28mBIdHQ4M8nnTGjO40LZaJ+45RY/D
v0+Sc+3lQJHfsDtFyVyI+7aUHf2R1teo5+c1L19R68qDXXGjN5XsoOM5BKPX
gLklZRfucv5yopULa7O9B16lcOG7iWfIwFsu9JoqRF89yYVtc73GNL3kEjpC
uuoUbElePEIHvgp5GB8xGgGB6a+vLNyqA801k4Wqd40g8ww1+rZfA94y4i6a
TOrNN4C0DQOjYUwbY4NXuuvQXUNgB2hHBx5+zTjMjlwm/cMYqtnD9UdWdzNG
ZdyZ9/K9MXRsPZvpm9fNwPwDqfBteXUvehmLWtJd2cWmYDX7nXX+ph7GOdFB
t6ISU8jIn5lxjDpf972ubt4+c9DnBjzrt+4l8bcxYdrLW5o+Mgwj3xwM06Xs
uR2vV91/2sfQOj9FP6vaGgzV5tdd9/7GQDtsDFeSNzfmA6N7pYxU+3d7MMkV
b96Y+YJRnSnzmn3CAaIbk21suZ0Ms6sD41ooPd7s9cI6cUgOJhSNUTskywGR
hTec45UUoHNMl4ZzCgceDu0/vFV7HBySV9xSWskF0fFFusf8x8P1B9f3XnzI
BVuZk71RbhpgayAuE9jGBTM+N3z6BW2YUfuLE35JQOygoHMBV/onjgCJpoEr
1eL0uOM8CIa/mfXhJgeu2HQ19nImgfvHEzYpFB4oPu3oLpDWgZuXKqdo1FA4
UPzANc/VOuDz+kfpw51cGB03c/drkynQcM7izfUcLvyoG282SlwPYp4d2Tcy
QAAtd3j9h2SnwOTgFbO+mvyxkxToecN5xHnDeWR6bNh5JbiHzCPOC84TjjvO
w8+Afbou+wFmXBEb+CJsDcKXfJfAfYBGdbt0q0or2FXectTpLB8ezx7e+XIG
i4w7zgOOO84DjjvOA44LjhPOA84LzgPOC84DzguOI44rjiOOK44jjiuOI44r
ys0ESaMfxRsEECjTHXb5ljK8kfw5om6xgPiDmh+WHZwQQY87zkPHmfzLNxQc
oGyR5v51JbQctoxaFn0vp5WB/kLLyZE/xFOHgdSFK32rdqE/RJjch+36pc2C
9k9+xuCqbpMuMVKDY8/mfDzo/ZzhH2ARWJU0hqzr4jqvSlB3zMmTtJ3e6PNR
/uZ+AVjEFNdH5I2FMqs7P1rPCWCsf7BBqY8m6FdLmx15ZAFzktV8zTWNSPwA
2uk58zpnWV4UwGSW+9VLXybCyFafixEpAuDNtTv3qHIyeQ4+F/vF52A/2C+2
w37Qb4R+JPQboR8J/Q7ohzjTK1H0hZqHRUKxD3LKlWHUwbcL9PZawkGxWAPj
x4awzuCovNQEK8ipkgmUCjeEYvNyY8u0X4wwI6l1M4Ip+17Ufmn++q//1Alw
p+cB5wXnAedFW2qn94ZZSoD3Tx3t+mXNC9pv4nxIbub6IQE8VStcMT+wmxFq
ev+WfJcAjkd998kXfcXA3/E+/B3vi5P1rB7+jAEjlN7fVjezhfdtHte2nGNC
8+3c9d9UHUhdVdTbqM+wzurK7NUlw2Mc4bJdokRyAAPudV+U0tzgCIr3bjxI
OsQAu66Yi/N3ssHsw5SirSYAcyfv0OleyIZ5v49A54EerPSYpOMO8PZY4uYz
fDZ5DupJ1JviJ6eIJjC5sDt87L4EMRYkmt9et8aIC0m/j7I03ke92X+5U3Ub
dby/avYnixoWpKjUGwz5sUFtjkjM2lIW+R3vw9/xPsXREg+GXaP03sMN2Vf5
tH0UNf3c52OdHMjs8In78IIDEUer/bNXcMD8xYfbEXs5ILd/4QkbNQ58YNx7
8SKLQ+xCtANrEkoVT27gwJlO55Klo7jEf4DtsJ856+oryqh+N93w75Zs4JD7
sB3uQ4j7EtY6HD0cYG4LYx+Vtv2i7GWsa4J2NtY1wTonaJ85S2opM5c7gMIk
88o3FB7zKvSQ37zOkeQNLPlxHC5/cCD3YTvR0MgQW3MmsWP+937GTNh+fKwh
L4qy7zaEPPU5RP1eIjGYJcEkcawWLu5VTx4BnT/8px9sh/34Ph2tOtGCBczY
XR8bypiwKutT5mUpFvEDoV2I+wzcqFo7XTySRegG8wTQn4n2H+YNKHqX1X+b
wSJ2EvqhCuui9l+bRP+O9+HveJ/IjF2Jx/uZ5NhvNTVUuJcN995dcSvcxIa5
Kr6TLl2g/UBoV+A+B2hXCH9TX2i7grJP3Nfrh8rS+2wgvfw7b+Cw+4fpbFH6
iL/vUtl2c2w2mxyRXljDZ1z+Us4mR/z9rGxf5oxcNhhnbTVfvoXCjZnDt7pc
Y8P6Ua1m15dxoLNgzekb22jciPSP+yrM3RUalf+a3ucDcSHiRMSV6pKru273
c8kRf5fSbQotDuaSI/p9DgcO6ZlwrUC7PTpyaw+F628y+j20af8D+hWd3yjf
2itG0c+5V58HuI6gOdJnSUspE+pDbMZMbwSo1LZ2/17AhHtC8o6TW2m6yl6j
rzJyCn3E32fW3WZMcaHkTd6W82Kj6CP+ftDu/ZaeA5TdmH3XzPMiC05WbGOf
c+dCpazCis+WNJ8j3yOf4/zgODm8MVu9+QKHHPF3a/Fnar5pHLCobZkZPJcL
7yZbPhc6yYGMg/Lxq6dzYcXSmVODl9iAes0UoXZHU9hz5nlxbLwNhMyeBOuM
TKl537tescge5qS1+J7so+t2lM8McGvLtwXVj2Mvz9QB+LrY41oGZadYSKU6
GZgxid2F4yDLGa5ptdGB0Pm8hvUbgg9SeuB4889FbRxwNhUVfMpkke9yNNgk
1G9P2f2MwVdzftPP76MeC76/XN7dROEcTmCseZATG25aG4jce0LZw/dmTpt/
mE3oaW/VwqsvzrKh6LyWaUABD8SAv/nXZsq+MTZUUM6h81r/R4Z7ePCpxbQ6
fyIHZh4KuTE2ksJdaw31vl5lw12/hnMVR7jQIyI0MusiGy6W3ImcyOfDt1k9
1QWSXPCDL3Zie/mQV6O28/BMim73TIlj1/DJ+OO6hMGs9SMmX+SA0aOkwfYP
fPKdSdMf3jwTPZGsi+A6MK4L47pvmZTZaMflUsSuTJ5291dq/08GrqviOiuu
q04djE2Y8lUCHFzO8YaqHCAsc7jbu31yZP5wHQjXfdBuD1u73LBXGusJ9TB0
4ZjhpecA63+oKRctlYHuqbWlPlpMcj/6P1sObP/Eje1nTDWvEw3LYJH2GDeG
cWR4/3SGxtriuQrAeGS8/+oVFulP89jGhj0V9D7rSP9uqzMObX0pDMvc0oZ5
5nHIeKH/5bj9kfpu0Q8MxedXgjvteeT5GFeDcTZ4/1rO57Df3x+56MYlqak8
8nzMa8Q8R8xrxDxHt6hGh+eHmDDY733Sc7cK0RtSM1vdc0aMJXGHBHf/0RO4
DoZxVHh997yuTJt1HHLuMjpsVfdlel+ZGUlfzn5qpveF/8dfyIUg08tFIr4T
iZ2Ldi/auWj3Yl4B5hng+gRfNXbBYKcRxD0xWRzEYZJ4dZyfx7465+ZQ+HNL
0PZNm2pY5DrGfWEcGM5P3naj2r51ejCsrfugTCS9znxc/aVK81bu3/EL1PiP
Kjafr37NEEJyn0Z0ePNI/2hno92Ndjba3Xd4uXVi2wFWcB7trug1ht7XH7dZ
vQXiB0D97yt3vD7mlTGh92PWaSsjWkyh6uLd1RUJWL/ElMSBYlxo8/ibadLR
LNIf6msJRd2RvWeMyfj1Lhg/s9beDFJ7vV9d4TFJfxjniHGPGAeJcZEod9LC
Uk4536DH+2eQgSC10xQ2X9/LnvaERfrD9f2bnS0bq/2o7z1XtDv/EYesM+J4
Di257f37+5qTmBfNfHh/t6fkT0Gpw6Y0QzOYtoSZ2eMoINfRL4F+Cowjx7hy
lOMy25rgnJcZiVvGOOY3H9olVS4CqduA4z/bI2Tb1SbaX41x0BhXi3G2GHeL
cbhH2Q/DRnuySH84/gHuseHhV+k4UIwLxThRjBvF8a2eUHd3SqspaY9xphhn
iHGHGIeIcYk3tqrP29HPofe5/jP+qz8GzV201wwMTv0MvyXGI9eRH/U3+2zp
FzGHfvXiQCkpDllPxffBOEjkFwNhlytTIs1h3N1ZBjZRHJLXi9ejFzuan8y2
hNsfsj2+3eCQuACM5z258OXVXzPMyfMxLvDBsC8Rh87zSH9oB2oHHTp3c7U5
XH9uJpt5j0/6wzwjzDtCvYJ6pvCl3r4yNcY/xwZruKccU9h91ZHEaZL83lUt
Uzu/WkPg2guvxT2YJF8c5Z94i1JIsjYdp4txuxjHi3G9mbmJLfptLNIe7bmh
ORaFjApL+BUamlZowiHXcXx3KJWd8l9gRebLRNhyk0ctfR3jSK1PXTt45j6H
+KHJeuSanuynWrYwf0XG25WFHKJnMY4S4yoxzhLjLreyk/U0VvDI++D8PO23
OG8abkni5g4qzgp0P2oJBi/37UvlC8j9OD96VVsWhh22BLnkoNpFPnwyvtge
4xCRn09aKg/fl2gFJ0/Eb0qq55PvwTgEfH/0I6JfEf2I6FdEvkY+hUSrRmlN
CocGCjcvdbcjcewY135/sbJHoCrz7/0bf+cpRkXf0IqwBe8zsvLzLrDIdWLv
3BFaUzGZjrvGOGyMy8Y4bZyvW6p5D1UpfDnXLMfrhDMdd4/9YZw3xvli3C/G
AWNcMNLDrbeXrSfI2cGuA7M4E9k88n7IPxHRfnu9qO81k9/ya2EO++/4tP8n
7hjpxZibWPL8GPW+y9T9r1J2zl/1I1roOFvv04FrKzPoOg4ory3yItrHhjjA
hn0bVLZS5zj+17qKFK/oCMj7IX0sS3tsZK1Mn782lUlpT7Cj6458CbIxSXEA
m+r0y2fNBaQ/i4bsBM+tpmD5+/iD1oOoF1EPoF5EvYd6FfUoynWUeyj3Uc7j
9aD7L1N071qCyw6nkJYCBpEznvvmh/nMA0L39sv4ae0ljmAd3HRFQdiK+IMe
ZpcN6v/Oc09VGXOizZHE3WMcvmPLoQVSQRzy/jg/dUGfQusv0eebzJJSVlLj
e/zqw/V69+k4f7z+X3Hh9Z/Karbe5JD3EXxaxzZK5ZHnIX9rXg45FjPZEdiP
/sGP+P2Lehs/GQYLyP04X531SyXUrem6Mh2HtF+pxQGk/YlLwvaY54l5n4hj
EZdvHQ2PHusqQ8ea9r6s7RROH/njkdtwQ7iw57HqMTcmwXWIUxBXIc7DPEfM
e0Qcg/cjPSCuQXpAXIQ4CPkQ8xAxLxHzFDFvEekF9TrqcWyPeX6Y94d5gJgX
iPSDeYNIP6hnUK+gHEK5h3lsyEcot1BO4f14HeUWyil8v91zvibFrOWR8cH5
ty7yWyV+jz4Pv1Z5oHsGE2R41s+uugrI/Tj/M/zS3U8+ps+7pVpjBXOYxA5C
HIh2CNpFmOeLeb9oZ+H9SB9od2FcM8Y5Y1wzxjmjXXHI84Bx+Ww2LHEc5iTu
N5roWcT92D/SD9oBmBeLebKIW/F+pB/EsYhz/43L8BzzOjHPU6Rrqdvp0ZbE
v4Xzj3mfOP+IExAX4PvjdcQJiAv+rVfwHPkU5QrKjf+SK6a5RsUSRzl0ns8f
Ox7jck6pv7YcNOUSf9seo197q2/TdbEwnnbg6Dz1w9R8YR425mVj3jXqLbR7
Ue+g3Yt2MOZ1Y5435n1jHvi/7UyMW8c4doxbxzh2jBPHuHG0M9DuwDxozIse
fX2T3koTA+LPI/LmT5402i34PZh3jHnImJeMecqIi/F7EcciTkZcjNeRPhAn
Yx405kWP7pvwUyTQ9O98Joo+MU8a5RPifsT5+L54HXE/4nzU85gnjHnDmEeM
ecWIyxB3Is7E90d5hbgTvwfzlBHH4vsg7sD2iCsQh5B9q//k5SLuwfYo7xAH
Ie7B70G9h3oQ/SSIW1A+od/k/6vru+OpfP//aRDJFkk2ZW8ODud1FqehohSJ
VEhJSRq0pERKlEppKaUkq6JhRMmupElSQkWSFWX1PZXXdR69f7/PX9fjvu9r
3a9rvebzQr0Lfsf9CPUwWez0M1Y/xEn8DZYfkPsbt4hx+Rinj3H7GMePegKs
H+cX6g0wjh7j6jHOHuPuUS7G8jj+KCdjfRi3j/MN5SjMj3HxKHeRuMrRuHCM
Ex+w2P0xNcqM/C+OJ8aN4/6EcgPKCVgfygnI92McNMZFY5w0xk3j+COfiXwl
/i/ygchnYpwuxu1iHC/G9SIfgnwNlsc4YMIHjfJFyAdh/4ndZjQuFc9BPEfx
nPxf52gGVcMlw90YjJ1DIw4t5eEiIE4C6jGQj8LxQb0G4hAgLgHiFCBuAY4H
ysEoF+O5jHH4GJePcfoYt49yF7aP6xvlMFx/KIc1XZ8zdW2Y2eg+PJO0j7gA
OB9QLkQ5EPuDce0Y545x7xgHj3w/9gfHC+WARp2vTv62lqR97B/ar3E/QLkD
5Yr/JXdg3DbGcWNcN8Z5I1+L/cH5gHwuzh/kk5EvxvpxviAfhXzS/+Kj3sl6
y51YPp7oR/F8RL06xrmgPlLH6jnFxoUJisWCjtXxWlB/2sQhi04h68h/b3vO
ez46HN58U93grRVMS3U0Km3g+bHjeX9W8AJfK51O7i3De8zwHi28VwvtAHIf
X5aXafJSfN+3oN/BejkbTlU7sloF2dAp17Xv2Bw2uPde7g2Q4r3HfPge8+E9
UXhvFGvdhMW5cxkkRfvkDqXVYm+CRvmz337lhqt7ZFi8FN8HfJmZ7B7Mgljp
HNcLmixwvv7r+YgvCx5E5zDqDHjvMR++x3xp56upwY32sHqL/PYRESE4VFud
FHwE4zqmgGd/JDN0hz0EKK93rK2YTPg25EPVnlXk7XlmD8eHfIysitUhVjFl
QuJZezJ+m41KTE0j7SFn+770CFFdKJHcNj1grT2EayZNPO2rByeLFLRW+9qD
8AtmufkxPWJHSNkYD2OGeP3AfmE/sF9r5daKZJQowDYP1rysDF4/sF8bp84t
zBfQhNs2u2f7POH1C/uJ/cB+YT+wX4jbiDiOsZXid176WZJ16319xXCAkSU4
5269M6RlC7esxB4cdbQEjcklhgFlNsSvAv0sdobXekaftoGkx37hLzuswL1+
R/oXWVuyrtHePuVR2/6zr6yInyL6LX6y/Xg+N8ES9p+urlSq5+EjoR8j+i2i
/yG2g+06hE4R4q+jwp3mF5M1zAF0Z9yssE2nQukFqc3xm3k4gYgbGPhyy5pd
3//fOD58Rj8R9BvJ3S9El+6mEvkdcS8qhzZELWmnQfxdR0avJYDWu4TncmMZ
0DbJt6VdA0C2O15eyoqHE4jyHPoLIG6g4885XvfCaWQehokbl11bwPMjQT8T
9DtBPxP0O0G/ZPRThqIoFysOL64E/U5WVLjOWVpDh8iku8UKqgw4e+nVMt1q
JsEB/McfeB8PFzDQUilwzP8nzg+f0W8F/VjQbwX9WBBXAnEmruh0f7tqyQAZ
wRT3fbE8v3b0c0e/dpRf8tl8gt+72X/TFSzYtLPrWeMdJggmWfkI7GEDZV8F
f/hHJnT0XLrdrWIHJ0oz31PvM/+173P/C/2e0c8Z7bRt/ksufg5kgctMCcni
cjYc0lwk9jKE+9+1KZ1a2Ty/VPRTRb9U9FNFnBTETSG4KKM4KThvPmp6M6Vr
AWIlrZRZ04DMg3/iKv14frEox1Ulit65GMUEy+0PNUMbufLft8dTGqKZYN3+
acHwaxaM2zJz1e97hhyE1mX+vncI407+0X9G8+JQcB1h3AmeK3jOSE+kLz0d
QufFRZQq3LJxocOrCsEjC9vpMOdpXOvgfDqMpa1PXD3E8z/xniFkLNhHh3hO
3MpJjxlwfebhlr0CDJgeEn6VpsWEm9TPhlua6ZBvIbYgXoQJH0LPb3nfQQc9
j+CXjpOZADts4vcXMkHvKaOqS4YJzbaKe5YV8eIVFmS32rJfc+mQN2vr2UMs
6NTSNBC5yoQ4A/NLVz/YkXzl/lPb7PuosFtdaNmsDjNy3itMVM1PXsbDIYrY
GTI3ItOS8Acq1tMyxudaQqBM5iefEgAnAVmblD2WhB9YsHJvZniLFVlPDYs+
bnDj7nND5snfV3LXn+EbtuXu7zZQEP96xGGYDiZnDt4UekmFKYYuV2P66dD3
ufGJlpotHKRGagqqMEAuWNxBdAzvHoKj4lee592wJs9PdNpMckRt4MmzfkPx
nUx42Ll732pRW5BSUkxw/43DUfSgkjWPdw/Cl7dJtlm3baEm9/qZ78kMYhdB
f6LibuFtWfe4ZD7OWiicxd1HHK4bFCkCXLrceHs1d125y+Svf+UOoLmx7ZW3
HxvuxJusieoFKC05WOyayYIlX14s2snd58a/hfqK79xzu1b31xdZgMTabtc7
qqy/6Rg6qN54de/7Ttbf1INOzvVy6iL5zEBevMNBTWtfk2M8fUQj7Uehoz4D
CjRzcx6stwPH2eNmfv7BwymfTTUp5PQy4L7VmZ9gb0fmXQRXxs57xYHv44bS
4ju4+2XWyp6DMRw4f1B+uW8yEy7teHAzj8EBygUR+8ZK7vqa9eD0wkgunzfy
OKI3hUlwmRCnCXGYkI/D9bHvzluz9q8DNMQJQtwgxAXC/HiOrD29ftPBe/00
xKlB3BrU24gszqs32CdEcGuwfMNHgYZpMqgnG6LhfOhe7LqW5cUHq7Wm1Ip4
sMh33N8CXljMPv2zi+b37P4mzcm4rg1Iv/E/sF/YT8TtQxw/rBf7ge1iP/A7
9gPbxX7gegoLuK6pLEqBL21f4yS45xc768ixERcKVEzvGZy+mYc7hetpw3fl
4+e5fPCgo9iD+FJrkiKfjHzzsOusi3LldnC0YuTrxEQmeIuL14S02cHzLyW0
RYeZ4LU/vuDKOLSfK0Pd5OwsRUkG6JUHVabwqcAFqW0P28UZ8Oi9+DL2Z2XC
ZyHfhXwV8lmf5IVT84O14d65FMUN563+jl+zFjzbs26W/ycraCqwWf9ZWIf4
IR6PPPd53BkdGGuso9OZQgML2x8Dg8K6sHX8o6PJe2jwfKjnQR9XPtQbK0IV
UjIhcawY14pxrBjXiudEbdEJp9UVNqAZdsDocT3AuMDuKcLZNvDY54aq+RoG
UBlrYiXOA7znq5EO5D6/Xit0YTARyPq7r6HQqcx9r1rx0adXhQV32/cF3d/G
gOO0TQGt3PX6+hR9uGsxA8bpQsMBLxa8/5B2h7GCAWafjCR3dTOgUu7x19Bp
LHIu4DkRe5kl+7xl1C/bkA52jnKGbXwADs8MV1hNoMOQmoj7xD4a1Jb17M+T
o8Pi/bUTBotpELJXI7FyGZ3s27iPa6UNyz1axCZ+UU8sNqf8vnfWsGz/ot/3
0JatLOukurMhSd68qifEDia73rlC5bBhW2VM66dMOxiy0R5QW809b1LHZx2W
ZkFw8M32pKDReTvA5Sf6BL9xdNh/x3GIDQaSDV+sNX7je/olf55o91dv0sAC
G0md0+V5bJhbcmQF/yMWaO/gs73dxrun70egDOucgB08kr+7d0YJi9SH+1ns
8ND54wfY0CoY7B5cxc3nvcK19ywbblT1n8zlnrvpOtI7hA7pk/VyRzZBcru6
OaQ9ksup4+7TF/QWZds3m4G2x4SVfcV28KGyaYoX6rcD7EGwPj+6xp93j4H/
Qn1a/3wGdCQ+kKpLs4el20N2ZxczIEQw81TcgD18/uAjtF6CSfB1T555UDnv
KgOi5Vl5HVy5YvXTrPhLWbz6g2PpxT8/MKBG2dnE6JQ9FI7t9eh5xOVnJx9r
OB5h/zd9xYATNbraSkuA6BVwPT9N/dI2a7k5ZMi05nhpA1h9oM0MDTOHu7td
tlpTueetyonzaQHmENa5K2rOgyDa877ru+W2jodJ8hFxcsdnjMpJfHzox3DU
cgFbTeQbDe3Sm4KPltSZDtCQftUhWid/zCykIT8Z9rglJvrHU5rY+IDy7iEm
0QsiXzjhZHxwjOBPmm/hzlm/5wN+9zQpHraMZxE7BeoJUW+YdqdrZmExi8j7
veOe9cdYM+GZ1Jx3tA5t2H3Ob+zYSNRD6oJm06zH2h1iUOGX03DSwQB0pGL9
ItpFid4P9+HwT22fvfcak312JCIs6G2vEajUeCbWptgRfBTUU3WN5C3rCzWC
LU3jUqyuceCSacuEJhMjMNzuEnhZlU78P7D+noDj1YESZoRv76cx8qc1mgHL
bNLBl7ockv/YrvMpiQvp/9r/ueUvTs3em/eFQsrvvr+u1zLCEhbKKUl8YnNI
ftSLOKxsXnQ0kAKbg7Lc7MtMCN+1dVeDgXUjBYokxgY/emhJytkednNQjOPh
biOuLv5v+cXssCXvqaT+e8yDelc7qXDVK/MLf78okV/X17wyW+hvSvhu1Kfe
c0h2Ot5L4+k50yzOKrjzntf73189A3h6neXfXB76+tHIc97kQ9MF7IHI8xea
9XVecNepv5VyQlznBLKO0V7RIZQoLmvA3T+PJtyKDTAl8hj2x2ggqNOgkg41
5WVHeuOsyXfUA+lrqOi5FdAhZeBbc84eLv/3viRS/QGdtH+7UezMAR8u/yN6
Ky3ykjCRo6zKzg8+qTbk4SyN9md9y6cc33gm5IV2GA0+NSHfsT/Dg/WtIQyu
PMkQ7Ns5jvLv/Vncc7owkT27K44JDfGl3+QtrMl37O/PEtXL0ly+tKHh9TNb
FR7uI/Idfs58cDqM197gpSWVC7nyF5Zvjt50oP0M736gifXiUlHidnC3qXjf
WWtBYh9Av0N8FqmP8jeJtCD6buyvcs/7D4GtdsCq8Ag2PyoBb9b7ON+n2RMc
KOS3RL8UzVfdIUBSbN9RbnLiTSX5/+BETQEPpcRLn0WoJD4tvJcjsluUCgmT
3lzV9FGHh8oP91dOpMKkpoKd0VnqEJzOFzQmmknynxzTKKjHfRZ8ajx3+hh1
ghOEehzkG5CPQL4B+QjE+cH8Gy9V6ezKpsM59t6CBk8dgmOE4070iialkdvY
RmRd8IeaFDZfMSK4RJgfcZQQVwlxkBAXKeSR3FSzZhqRK5EfMM1WH6k9TyHt
YLtrFywX09Exgnp9uR0CH3jzGr87M3dU3g2jg1fvC4em99Zknwv5QvsQ2czD
icJ5FMZw6VowRCPtauy33vzkGS/FeYb5M3b31b+dxQTZDWUBWhtoZF8WXTvm
6bpoHs4V7nNIR6Trn2GPZMJ1KBM9WESHqLXXgmVmsMFHTd656hgdktgW/Ruc
uXLnsV2bV++mE1ws3MfxXMBzIuyC0O0fEvbg4bolLmMXEyQqrX28De1Bcuei
gqYNPD2xSM9tz4gkNsEZQ9wxxBlD3DHE+ULcL8T5QtwvxNHCfQzPCTw3MA4G
9WSPJj3ZuO0rL8W4FuRDV/XtbHQt4aX0ZclWJnYcolfEuC6MC2uPb5x9djYH
3Mc82mN+sI+GODuIu5N8JndNjZMUSZGPuNGi36OmJQmd7XYlr9eYc9dtms01
G0mojn5w+dBPczC5eOVxYrUkPJNbdvdxqgXMWrm91SpDiugPpdfeNLi/VQ56
EuDLvR+W4PY2ZsvMDDnoS7Af/wTocOruJ0V5RSkSh4ZxaYizg7g7iLODuDto
79n+sX/g610DqNJ3Kc/vN4YbpkE5k7MNyP7mmS77SGa9AQypv3nUamEK0+I4
hQphBgSHB3F5dA+8Fr+Rb0DqxXqwXqyH2KFH68F6kU5IN6QT0g3phHRDOiHd
EOcJy2PcH8YBYtwfxgEiXZHOSFekM9IV6YzzAucJzgOcF4hLhThViEuFOFWI
M4W4U4gzhbhT/FPLVl7I4BAcsDNTLDw8To7a5fi0yHvMpzlhpbDjdC1o2Sqg
eCCelw/L/d0HjGDOGgvHFn1DGHfNXXrieEN4LSxemvrZCLJnS188V2oAOfVb
0vYdNib5sJz8G89OvjwOnLnMd23gZjbN40ivt14d+q/xUnyPcUG4nyIuEeIU
HZbPsk+R4u6XheM1jZMYcNRHIn0/9/lPyn1u7L4d8OPlaOrDgne9S+4aq9kR
uS++81kZv4IdjK+KMn/rxIJJZknaMTksSKFQDjao28Ez/0bxqWmsv6kN754Q
GaNLUrNDWeQegeD5w3J+B83A5sCeXFkDxP80I/sC6jX908c8GRECyIHpcoc7
AEaeX7GNW8JLUR+FuDR/6DaFF7ePOC+o9zE2f5qhn88mKeK2II7LuQVFtDMf
bSHVdqXQd3cbiM9R6DD8ZEvSvqRvEil8TJJWf5stvYzL70e6T6+ssmPBHtdL
hc1vWWBdGrrF6ScTfsW2veFvZEFcnEeO6g8m0eceF8w3PXrIEtquaouukreB
kuUnWqdFWsK9XYcn7T4L//qH/pbXHyfNz9hGBeot00sXNzDAoJ7Ff6vABjZ1
p/hV5zFgX9+K41Y7bGB4nnxD+joGOZdR/577wW/lrwnoD8eAd95h0sq7efL8
EzNlyOykQ3Bj7dnmJiZ8yosVKZRnAOJoIK7GXEu2V1LsT1qRVpnduKkSYFGz
cYWF1xAN7zMQ9lzpK2IgAy2TnzIfnJYguBaKN3e8KFolARmxSgrPtBAXQ5zY
VdNdf67K2iMOnzYe8T8hw7sfAXE7EMcDcTsQx8MqQC/imjqA0+m1UdEzteBz
+OHH7HY63Pd87eHJ1CI4F4h7gTgXiHtBe+J+btZNaTi36t3DspkUUHC6E+qg
J0PWz4+ZunOFLpkBo35+/Kn9vHmK8/b1iVxrna2GZFxjhE87J+sA3FygH/Lt
DED3/RfWvpoAYjbaxfEpAEo3hVxssujwUqVFNHUpA2QXe41Jv8W7j+nPeHH5
mR8vLeTEbBmEX9DZoTv0O37szzyOYsPeNtN5W1RZMGWNtvRnczbhX93ynXPq
zvOeEdcEcU6Qzkh3pDPS/Z1ojt/XzbiPTQHz86e2d15HvwRV0PfQ+JXO5QOG
etl3s0TUYL+DUr8pl0+4taRk3Q9XNZ6dfZ366sZ+LegZiekpOsQheq6HX4y3
WYfx4vTb/IP4JUStwVM90373cSOYoXM1uXW2NYz5IBuWGGxE5OibV5eVCFWa
wNtqxWqxtT0ElwL5PrR7I/6IoEP1k4blFDBiBp24GGj9N3WiQPjBu5W//XtQ
j4/1yx26uEygx4bw1Ygbgf+NdMD/RjrgfyMdEAcAcQEQBwBxATxeDd7ZsYkD
BZTHTv4nf9EYAU5Ph3dxYMa46TJvf/EBlZbNLvCzhLIbjO4MBVlYI7TVu22j
JXjWKa1ZZCgLxy3yg487ot/xdNhrliqusQKgV2tBvl+/BlCy1AbNqAC3lt29
NMdlBnybL5E+hgVwZOB6XNHwdJC68WHMbz9SLO/8bIF/4xQG9LNexInYaMLE
EKMMiUE63Hx8WTFg9wyYnJk98f54BrQ2KxXpvp9Ozi08x/DcwnMspbNXVEbI
DJrHm2TOdTOACU+1vxq3mcJyy+Vxm1YYEHmqIWrqhLtlejBBcNoiy1MUmGzt
svWlmx7wr0xY7b+fArdMxKM46/RIPVgv1oP1Ii4C4iRgvdgO1ovtYL3YDtIZ
6Y50RrojnZDuSGekO9IZ6Y50RrpjeaQ70hnpjnRGuiOdke6IC4E4EYjzgLgP
lcEhewWOycAZzx9rS6oQr0UWlpykebvv/0TD95gP32M+9Ac817P0Sl+8AVzS
VjuaxOVjZjx58eE6l6+pK80PHxvDgTlzm88te6dF8mE5XOe47nGd47pHHIaf
UXzP99RxICg7W21eeeNffNImDmivSMtuKcV8VTQct55rV1cPJxjDvU0vnfJu
UP6mG41BvKumxWCJKZR6pJ8+s8kEkqzqE9wmmZL+/LUbmMI2L9lp+WdMYMUL
a7/5HArIMWVVC1ZawPwLac1vNlJg4Ntmr5z5Frx7oUbvicJ7ofCeKMRtQBwH
xG1AHIfnFysNLmrQINPNcqLbDZ79M+tBbW/mBVto3+bSH+4yen/SUVuSD8th
PiyH+bAcx3TklJsyV44rUW352gzETuXqoiDWtgbAteWtfKkYE3bpB9DEufOx
u0Io6JQaE9q+rFw1lcsfNdwscFDilp8Tuyl34CMDOO8TjeSNRuPt6xnkPebD
95gP76XCe6rENVevTlrBgmT7QIeKzywYkW7yuLjRFrLDCmQkY2zIuYfnIJ57
eA5eiPLMeRMMIO4jq34LePgGrjIdA0OWAEEaTw8LnxhdN/oACRzJmPtCdNj8
a9ea4Hjg8g/+3xp1ufIW/y3XzAggcfkYp49x+Rinj7hMiNP033Pyz3wbNIUL
C3MHvGxMoGG2FG0k1RQCApY+VXU2gYm+VbkNx1kkxbhzjEPHuHOMQ7dlphU+
YVhA9ao+p95KY8JfTohzfnEvxxg0VdR+9QjQ4Z1PTKxIvA3YVFv2qOjToc+K
GREfYkP03Y0RTE5dJRUaN0V8G6cMUEOxuiHfSoVi1ZSGtZsBjGbs/xmXTgVt
50brIU8AYbmz+j75VDjpOeL8+16Qhy1s2xRPGuGPp7vefX+dO68p80OeeLyn
g8UB6tntdgDt3x99ePPbf0LYeJo6d3zUBiLHqeoxYCSz791rbYD+pxTRW7os
KDp14UrOCB1ilnUMWU1iwZrJS193CfP8PPa1sBOsBRkwvPLK/fhoFtj3VU3U
dmLAF7r56ZcsOyj9kbQlmsGCH5r3+vb42wHjThNtsjGXrpTTDyQD7WCKRJ+n
uSGL6LVRzx09gSFtK2FD9Giol0U9LblHZFRvinwU6jFxvFHv52LW8VpRuZ6G
epudwadeHT3UTkM9EaZqItHLFwkaEj3q2F5W3pw0I6KH/a8eFJ+Zfhlvo1Kp
sOomfczzfBOiJ/6vnva/etJ/4hv92bCRXf7Lnt+M6DXxO+pN/6uXxOdZ9Iwi
IY0qGsoPxZuStM7VcOdXyXDhlNPWEHZL42NNNx04arXb23fy9KzIR2H7q7sK
869/4elRCY4bdaODfAITKhaemSCuQ4M0q4b1PYVMsMo4eN1GkAYL9zXYlLqI
wilZ1YMb6wACJt8O3yEnAYvt3my0FKPDkYPJKj9niBM/COQ/HUKc5wb34n07
fHz0cXe8av14dhgc5+QJFP8EWxlwrRNTnrCeTuSLw0uiF70KpJPxA+fWBqGH
PDvIf+0W/7VD4DO2fyb0qAlFjWf3If5WqiPTMrz1YF3fkp21vQxQvdawLClL
D7wN7x0LbGOQ9i3C1Wi7bHh2l3/t3e3EToRx5n/a38+LM8e4c7SjFKzUbo2j
sImdBe0maO+IqPject6FTewo+B3tOmiXwHlxkG5U+NuuiHYQ/I5y44h4X52z
tP3/tGuQ++BH9RKop8BzHM/19WVrIhUqufTwMmMdCucAvzGwJkkbQ87CXOlV
G3h2HeQfyH0TIrnxgRU8uwx+x/69d0t969zwv+0q+FwqHBr6dOZ4wtcHG92m
3fIcD3ucQ3s3r9Eg9yrjPct4jzLeq4z3KOO9yncXpNoZ5HJ4+u3faQkHGl9o
/5IoNoRrn2ja7QUc+NJT9nHoI09f/lPMOW6xuim4MZc8+dlsBVXjKc7z/E3B
O2L5pCEnKkxc6Oz5rskEdkRLqPFNpEKbb1nsG35TIhehnIRyEcpJKOeg3INy
Dso9OK9rw3+t9eqzhgXfXg8pmdN5fltnyt03L6GDfBdffNIYKlzW/u6zYiyV
+PsoP615x+GWK2hJGPTvt4Vykaahyg5rKP2divL8DMMVr33fuo8Gz+Nei4WN
MIg+3V3JufqnKhNOfpfdfjWOBp8+XYhpnsgm+ENIR6Qr0hHpinREuuJ/9sSe
j6drWIFz/AMDrSlMSPWlVr/0YYBV0P6LZtJM2CuQ/rXUl+e3kVpddsMqRRBs
Eus/N6epwIN2X6k6NQHSfnWFbcmdr+LwNaFMtyzZCGJUYw+9miAB3w6veX58
pSHEzArpNz8uTvSqXguc88UlJUBCYi49Q9QY5r4Xj14lKgGR4e9MHq43gXSr
0uWW3PK4b/fUCqRPNpEAx30ihTnOZrA/ddFM/20SMBR5J6/ilQl0HIn4fIBb
/u3eQa99fkDm6xv+sZvsVgMobfoUFcGnASsrVZRGvAAW7P/oUeCqSv4L/zNv
jdq6TRUTYLLaV/8roEj+E/97UL5yVmSeNZFLtuiylR06rUHsgoJ3U7cGKF4q
qBqzkUnqez+4hOPmxQTBd68YdaqKIHm7WKg3lAlvxi290ztHBUJF30jM2MYE
e5fb/N23lEFAIPhAvyqD9F/1yb4jvtznbtmaWzsK1eHNMrmcBCUGFJQbzmmI
UwXqled/7uPD/mjPnFSvH8GExW8qYlhNGjDfNa3AfjcTovtrmVkpmmDe0Joh
GcCEHAOZU/z6M0Dqx+LHBwO565XV6tI1Mp2MI44rjiMZ19FxxHHFccFxwnHB
ccJxwXHCfiIdkU5VKdP2nuXKezs3OC31/6YAhnOjUr9O4/Jl3isbH3xXhEfr
orpUKQAvBLyltWaowJEf+5SLFvPGGccdxxnHHccZxx3bQ7+kH9TTqbmfFcDo
Wp4g9SMdVmkqMDQUlcB0W95i6yE68VNCvyX0U0K/JWwfxw3HCccNxwnHDdvH
eYLzAOcFzgOcF0gvHGccNxxHHDccR/SzR7979ItHP3mkM/YD6Yx0Rzoj3ZHO
SHfJM3PKpfpH4x8360Eoo1WnrskWbnWniaom6ZF1gOvi4qOa87bR+pA+bbHu
oiCeHxz6xeG90bhf4rzFeYzzFuex15RLe9kGRqAQEWp9Y5Yh5H7KTRc0NoLh
cKUJPjMNyXvMh+8xH8rFyO+hXIxyMsrFKCejXIxyMsrFKCejXIxyMsrFKCej
3wTe841+wOiXj374aO/zq3XKcIwyBxV7fZ97l7jrZyM06x0yB8oLR5h60QwO
+phHij03ISnBFSnpMRqZakHsAmgneHd8QXNeMoX4wwkVsI8uLaDAlpND/BQZ
KyituldWNY9K2o+XiO8sCaDCL8nr591P2MAG0zEqE1dQ4Z3YnUMfb9uA09pH
S+4qWRN9rWlD1QYPYWtom31lRhXbBu4Fmjv1uLL/puVM8O6R8bPMtQKv3+k2
U9KPnnozz+gEU3D3iFIJ+Gn5N601J3asrwnisp0N5lB+2yxl833q3/SCJemn
5vW05Kn2luBUmHLKVMsWfNOMploaW8IPfWbPG0Vb8BEe8LxlZglB5iNM81Ab
UOmYNbfD1xKaopL0lnAZMJu22xdnG9iCrI6Q7C8uu7vls/mW2Bm20Kvac1js
JBOeTP9+ZG4KFSprTMVXLmXCpqWvBrd/pILvhX0fH3Plw+gvchPlHgOcsrcP
2FPAhMLY2qNViQzC17WvUykxn8WANjgnmf2Z+/w7pTBAxDwu7Pi4Uhr6YyA/
i35RuJ8g34V8FvJdblV6xm5aABsyzQuPLKNAa60So24dDR7ERk87nEeBqK3r
KwbfnqcN/GC222qOByWDy/MlTMtpV9+3VKavGwuO1Rq17/U50L/l2JvIW520
ozMCG7vMOCDpdjzSaFo3DfdB3BdxH8R9Ec9tPMfx3MZzHM9tPMexHWwX28F2
0c8M/c6WFRVOHyvEATerXpeypg/kHpeeBSXWB1630mKyYyVfiqF9u4XmGDx8
8uyQPdGT/REfRuzhejx/jKplA+3gyWtD/XLoJ6QAhS88DzNSLEGiWjg7Zaou
HGthX7n9O54G/cNG1+uVDr8SH19dKODzf32qbxTXJk0HStKDn3hd0YWn1ze+
mDSbRvZB3Bdx3dVEXXzX0mQMLg+vRPuHmYPJ9smr1TuNIUTGJN3mhDlUb+9K
WNhuDFe/Kzy9qW4BxS0nT2vWGEOzeNWtKU3mkLDz8rqIBmOiB0G9COpBUC+C
6+nji51FukXmsPhd7T6jdCuY5JBtcy7FnKxTXLe4TnHdIh/ZIeOo/TOMAa12
l44+TR31a1rDAI8NX1XNz9PhyljJtWXhDJC1WKv4dOnovU8mv+/P3KQh6sOE
jmcKv6wUWMAS6t31Ow7mTzqDBe7ZeoK29aM48w/YEOssYTnwnAXqIUsqFV+y
YfEzy/evtO2hq6DN924wG5h2w4+1pez/pnt49zK7xfkk5v8whsitxvXD80bj
YX8aw+DDdiHxWVaE7n/GaRp3vzk+z1el2hz8x1o0Nclz5ZuGr/PvSXDp51l5
VOCBLYmzxHPndLPqYKixBRRNYixdG24PZ4ozLNRWMkDdR8dtyoopsE6mtW7b
okmg1Rv0kOHAhMHO7dO2350KD2dT5umE0qG7Za6cV4cuqNbIWF0Mp8FrWlen
hIoxmWc477K9L1A2SVJhuL9kt+ArS9gwbfNgpKsRTG/b8OUck+fnvlddoexZ
jR3hQ5EvRb8d9OP5/vnSksJCGqQu2i2hf5q7HwwPeHncYELxTV+QyaLDxg1J
VedezSD7yuZT5S7mh3l81ywJcV99BuJ7S4FB+k27t5nTCd+BKb7HcljPgkpq
V8xGKxjnbbTXaZE8yHtWZs/IC6cN597Tvy2rTN5jPj0+nYob8dxzVz1o6Wuw
AJezDV+d82jQdn96aIi3LSyZ1xqno0kF8fziWSeX25J+HcvUjZTklwJrP12B
oNdWYBYQtWrVTgoIKf664mCsCq7f9DaoZfFwyodm/+QXG1IiKb7f1nfEcXGw
MkyaWt+0jaFKUnx/tOuw01S6BYxPOBz6db0+SfH9VuHK8XF8FHAqojTOPGQB
yTNf8nuMpZF59IdeAjSYvr/0ZEWFLdzO1TknJMQkeuc/crUMl6+736yY1TF6
/68vwMm7WzcFzQV4Vk1LN+9lEr0j+kn/CjV7C1z+E+mKdDZ8VTDJ1XsUL8SQ
DuL9WYkHUqeDjNP7Pfk+KiTF92g3x31hSPiCWMECnh9W19bZr6jcfW2ctdaz
+9x9jmLiQj9w8CUNcaNRPyHQfCRYQWssSIg4ZrhKKENnT/r85HUaxC8f/fRv
yJwfY8URIPMI9+HzNcWGj5SEwWHmS8kVP6eN6o2EiP7Gu+viGk1BYSL/T6uh
bEu+Nx42WopM1YphEjtvvMOpDQK7bcAhd/OMIAVDwh+oqEf4DUcbwJSsyewc
OzOyX/ZpqsQprDCDhX5jczZx99PBrycpG+eawXW3Td/e77eAc1dvuMrpCsMi
uRjxfYJj4ZGxp1j6CA1OeBcqLt2hDYGrqoalHtFh58RJ6fkx2vDm0iTBE4FU
UKMYz9d+ZQiZfroNpqswjlsWhl7Mepmr2EUj/jxqyRGPPDngr+S5sFF4kPbJ
qGFovTNXbr+TaFPuIwNyCns/wCLu+Td+TbLONhmY93XDSS0hgJDDjqLCiTQ4
eGuaQp0Ol99Zd6A/9o4NnNoudf5yC4CQS4XonmsACjmaASwjXVjhtaVfUJ5J
5inO28ZTFwaAYQ55TuIv+5+YwZ5pcq8+T+DOv2XB0BdCgbeD9ufKv42H/pGn
q7oHVSCvVUf50T19uHXQ4HC4KeptlCETwgeOX+UQOwTaJaTLJgz0x9tBlVlz
pvcNHt7pfebhrCxpNiy96H1lcBsHKG7P9lxvZcHbe683p+ZzINsrYGpwJYuU
w3pkDzjb1ZjaET00pvj+z3xJsYenlIWyX9ls0g62e+H4Yrf5Y+1Iullf0ly/
zQ6EFriVOcWx4XNIpvthbj9Eihlhddx+FVG25DxMYkHLbqE3ouYc8DNyidtT
wYLpPZ0jiQUcGK84S+Ti73MqqI7awT23ij/KxQG3HsP7T/gUuPWyODa/ktls
0q8leptj3qYinudMGLLYnp/Bsf8XfyTAHsT4C9ZcdmdD+xrXBb+sueXsxIvq
AtnA4fvM/5Kb1qkbHR3gvtcruPrxYxD73/trD9iBz7Yp13bdZsP/AfL70Bc=

        "], {{
          {RGBColor[0.293416, 0.0574044, 0.529412], EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{8347, 8346, 3900}, {4342, 4341, 2005}, {4337, 4338, 
              1750}, {7080, 4950, 1750}, {4950, 5922, 1750}, {4360, 4327, 
              3900}, {5429, 6319, 2038}, {6281, 5330, 2005}, {5330, 7681, 
              2005}, {4343, 4344, 2038}},
              VertexColors->None], 
             PolygonBox[{{6171, 8450, 4329, 4340}, {6168, 8447, 4324, 4339}, {
              8456, 8452, 4330, 4346}, {7038, 8444, 4320, 4350}, {8461, 8453, 
              4334, 4357}, {4468, 8459, 4355, 4312}, {8452, 6171, 4340, 
              4330}, {8445, 7039, 4351, 4321}, {6773, 5331, 4326, 4348}, {
              8444, 6770, 4347, 4320}, {4478, 8460, 4356, 4313}, {8457, 8458, 
              4354, 4353}, {8455, 4478, 4313, 4345}, {5777, 4692, 4316, 
              4332}, {8459, 8457, 4353, 4355}, {8458, 7079, 4352, 4354}, {
              8460, 8461, 4357, 4356}, {7079, 8446, 4322, 4352}, {8440, 4951, 
              4323, 4315}, {8450, 6168, 4339, 4329}, {8447, 5844, 4333, 
              4324}, {5861, 7892, 4359, 4336}, {8453, 8454, 4335, 4334}, {
              8448, 7038, 4350, 4325}, {8442, 5777, 4332, 4318}, {8443, 5861, 
              4336, 4319}, {8441, 5635, 4331, 4317}, {7892, 6775, 4349, 
              4359}, {7039, 8448, 4325, 4351}, {8449, 7680, 4358, 4328}, {
              6775, 8445, 4321, 4349}, {5844, 8455, 4345, 4333}, {4951, 8441, 
              4317, 4323}, {5635, 8442, 4318, 4331}, {6770, 6773, 4348, 
              4347}, {5331, 8462, 4361, 4326}, {8462, 8439, 4314, 4361}},
              VertexColors->None], 
             PolygonBox[{{8446, 7080, 1750, 4338, 4322}, {8439, 6281, 2005, 
              4341, 4314}, {8454, 5429, 2038, 4344, 4335}, {6319, 8443, 4319, 
              4343, 2038}, {7680, 8347, 3900, 4327, 4358}, {8346, 4468, 4312, 
              4360, 3900}, {5922, 8440, 4315, 4337, 1750}, {7681, 8449, 4328, 
              4342, 2005}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.3088473851110933, 0.08423541123113866, 0.5511026785547418], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{4951, 8440, 789}, {2653, 2393, 428}, {7080, 8446, 
              2694}, {6771, 7891, 2390}, {5888, 4631, 426}, {7039, 8445, 
              2653}, {2394, 2390, 428}, {3408, 2394, 428}, {2393, 2395, 
              428}, {7575, 6652, 2280}, {6172, 6170, 1909}, {8452, 8456, 
              1007}, {8447, 6168, 1907}, {2392, 2390, 426}, {6290, 5477, 
              525}, {7559, 6871, 427}, {8443, 6319, 1701}, {6655, 6654, 
              2281}, {7939, 6875, 2395}, {5631, 6832, 788}, {8053, 8052, 
              2394}, {3162, 3160, 1340}, {6654, 7667, 2281}, {3160, 1701, 
              1340}, {2694, 2693, 788}, {2281, 1340, 207}, {8444, 7038, 
              2652}, {2987, 2653, 428}, {2390, 2652, 428}, {3174, 2280, 
              194}, {6872, 6774, 2393}, {4949, 5631, 788}, {7079, 8458, 
              2693}, {2695, 1655, 789}, {6875, 6777, 2395}, {7668, 7670, 
              3160}, {6652, 6653, 2280}, {8459, 4468, 194}, {1701, 3160, 
              427}, {6145, 5845, 1690}, {8457, 8459, 194}, {4631, 5773, 
              426}, {6776, 7037, 2394}, {3407, 1701, 427}, {8347, 7680, 
              3174}, {8439, 8462, 1223}, {5633, 5634, 1556}, {2395, 3408, 
              428}, {7037, 8053, 2394}, {6874, 7939, 2395}, {4813, 6145, 
              1690}, {5777, 8442, 1656}, {5861, 8443, 1701}, {8455, 5844, 
              1690}, {5627, 4949, 788}, {7669, 4632, 427}, {6281, 8439, 
              1223}, {8460, 4478, 207}, {5845, 6562, 1690}, {8440, 5922, 
              789}, {1222, 1223, 426}, {4468, 8346, 194}, {8449, 7681, 
              3175}, {8462, 5331, 1223}, {4630, 5888, 426}, {5634, 6291, 
              1556}, {7671, 5430, 3162}, {1656, 1556, 525}, {2652, 2987, 
              428}, {7082, 7081, 2695}, {5775, 5329, 1222}, {5429, 8454, 
              1340}, {4632, 7559, 427}, {8453, 8461, 207}, {2393, 3407, 
              427}, {3174, 3175, 1222}, {5170, 5850, 1007}, {1223, 2392, 
              426}, {2280, 3174, 1222}, {8461, 8460, 207}},
              VertexColors->None], 
             PolygonBox[{{4467, 6655, 2281, 207}, {6871, 6872, 2393, 427}, {
              4950, 7080, 2694, 788}, {6777, 7893, 3408, 2395}, {6833, 7082, 
              2695, 789}, {4692, 5777, 1656, 525}, {5850, 6172, 1909, 1007}, {
              5626, 7078, 2693, 194}, {6775, 7892, 3407, 2393}, {8052, 6771, 
              2390, 2394}, {8441, 4951, 789, 1655}, {8442, 5635, 1556, 
              1656}, {6773, 6770, 2390, 2392}, {7893, 6776, 2394, 3408}, {
              6167, 4813, 1690, 1907}, {7081, 5776, 1655, 2695}, {6562, 4467, 
              207, 1690}, {7681, 5330, 1222, 3175}, {8346, 8347, 3174, 194}, {
              8445, 6775, 2393, 2653}, {8446, 7079, 2693, 2694}, {7670, 7669, 
              427, 3160}, {5844, 8447, 1907, 1690}, {5635, 8441, 1655, 
              1556}, {8448, 7039, 2653, 2987}, {6291, 6290, 525, 1556}, {7892,
               5861, 1701, 3407}, {8454, 8453, 207, 1340}, {4478, 8455, 1690, 
              207}, {5329, 7575, 2280, 1222}, {5430, 7668, 3160, 3162}, {6770,
               8444, 2652, 2390}, {7680, 8449, 3175, 3174}, {5331, 6773, 2392,
               1223}, {5922, 4950, 788, 789}, {6319, 5429, 1340, 1701}, {6171,
               8452, 1007, 1909}, {5428, 7671, 3162, 1340}, {5776, 5633, 1556,
               1655}, {7038, 8448, 2987, 2652}, {6832, 6833, 789, 788}, {6774,
               6874, 2395, 2393}, {7078, 5627, 788, 2693}, {8458, 8457, 194, 
              2693}, {7891, 4630, 426, 2390}, {5773, 5775, 1222, 426}, {6653, 
              5626, 194, 2280}, {5330, 6281, 1223, 1222}, {7667, 5428, 1340, 
              2281}},
              VertexColors->None], 
             PolygonBox[{{8451, 6167, 1907, 6168, 8450}, {6170, 8451, 8450, 
              6171, 1909}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.3358230865394901, 0.13113887082225756`, 0.5890202902441432], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtUz1rk2EUPWk1qZoIefPxJm9MpLODg8WtdbZQhzpIqIg6BKcKVoIf2fyY
KooODk5+oCj4BWIdTHRwEXRQ0ZYi+AssYqstLVbP8TzDw3Pfm3PPc+85N4NH
j49P9gE4yNPPM7IR+JgBnm0AhkrAYh04kAWWeO/m9zB/P8Lvw1ljhG9mndcR
tsD8TAwsFIFvjCerwPcQr2TMr7reJuBcDtjKBrqMl7aYs5sGxphfYP5UAnxu
AD8YvybmfMAvBw69eYfn0GZjxKG8eKZYe3I78DPnsxg4H9SAp/xtmnwoA48Z
F9Ku3Zdz/VioEb5J7onALw7Fyl9h7X1yfUkBo7x3FZx/k3Ff0uEM82dr5lDP
K6HvcgR8avgd9a/ZNNe2CjBP3htpz/oq5KW39NJcGHCN8tJY+krbmLX9g0BH
+tTtn/y4zvgDcX1Z69wLnBeJf1mxH6vEjJT8zhrjPSX7rPe7Ab/G3F3Okk/b
k9OJNZnKA7fYw70BayotpclN5nbWnV9l7Z+qa+W5ZhCn9Nb+SJP9kf1Q/TTv
8cj4FmtP5D2z3heX8jPkbjF/O/jfDB49ZN06MS3GOxg/j93PRMDIoxznngs6
t3nepo2XD3pHPBcq1kj6dHkPl9z336pn1lwNep6KvbOdmr8Va7+ehB2bLfs9
vTXH3GxiTed5txNrPMS6Yuw+1FcU8O8j+9oJe7UcdmuUfq7H3jHde4uO5X8S
8PK7Hea6TM5LZX/3gqf//yPEviiaO1X2Tksr6fcocm2emHeROX9R8991e3aV
+Ws8XzP2QLf2EMzVyXmM3/8AvW2kXg==
              "],
              VertexColors->None], 
             PolygonBox[{{5774, 4916, 1737, 1654}, {5631, 4949, 1749, 1555}, {
              4631, 5888, 1725, 2391}, {5329, 5775, 1654, 3074}, {7081, 7082, 
              2444, 470}, {4782, 5772, 1553, 614}, {5908, 4781, 614, 1737}, {
              5438, 6873, 2494, 3168}, {6774, 6872, 2493, 2495}, {6772, 5774, 
              1654, 2391}, {7667, 6654, 1043, 3159}, {7670, 7668, 1341, 
              3161}, {4632, 7669, 3161, 3406}, {6875, 7939, 3460, 2496}, {
              6145, 4813, 636, 1896}, {6873, 7938, 3459, 2494}, {5193, 6259, 
              1988, 1043}, {7559, 4632, 3406, 3060}, {7891, 6771, 3570, 
              3405}, {6832, 5631, 1555, 2444}, {6653, 6652, 614, 1553}, {5437,
               5478, 1444, 2048}, {4949, 5627, 1553, 1749}, {4690, 5921, 1749,
               1553}, {5888, 4630, 3405, 1725}, {6831, 4651, 470, 1555}, {
              5634, 5633, 470, 1247}, {5439, 7674, 3168, 3169}, {8052, 8053, 
              3571, 3570}, {4812, 6144, 1896, 636}, {4915, 7036, 1725, 750}, {
              6331, 7675, 3169, 1204}, {5850, 5170, 1908, 1691}, {6332, 6331, 
              1204, 2002}, {6070, 6071, 1247, 470}, {6320, 6321, 2040, 
              2039}, {6291, 5634, 1247, 2012}, {5921, 4691, 1555, 1749}, {
              5357, 7607, 1443, 1247}, {5477, 6290, 2012, 1443}, {5304, 6320, 
              2039, 1988}, {7037, 6776, 752, 2651}, {6162, 6161, 636, 1691}, {
              4466, 7040, 2654, 1444}, {7040, 7034, 2649, 2654}, {7471, 5194, 
              1043, 2215}, {6321, 6277, 2002, 2040}, {5430, 7671, 3159, 
              1341}, {7938, 6333, 2048, 3459}, {6169, 5847, 1691, 1908}, {
              7939, 6874, 2495, 3460}, {8053, 7037, 2651, 3571}, {6871, 7559, 
              3060, 2493}, {5773, 4631, 2391, 1654}, {6652, 7575, 3074, 
              614}, {7035, 6986, 750, 2649}, {5889, 6772, 2391, 1725}, {6170, 
              6172, 1691, 636}, {6654, 6655, 2215, 1043}, {6777, 6875, 2496, 
              752}},
              VertexColors->None], 
             PolygonBox[{{5846, 6561, 2215, 6562, 5845}, {5149, 5846, 5845, 
              6145, 1896}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.3627987879678869, 0.17804233041337644`, 0.6269379019335445], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtU0toU1EUPG3zsUlc5CUvLx9fmy61ai3qsn432hLbJCDSpS11p1Krgh9w
oxtFXJqd0XW1arF+2kVx428j1ooI4kYQVASXmlhnmLu4vMs958yZmXNe39ET
teOdZjaF04XzNzR7ljdbi5t5EbNbSbNlvLdyZhO+WQrJl0pm02m9X8T3Jc4n
3KtRs451ZofwLWfMRpA3mEAt6pq4ezGzMcQqOMsdZtsSiq8Acz9ybpbN6lHV
N6PCf4GaGZxfyBkGxvmS6s6VxIM55LvoOL8Dz/mCWSMm7t/WKz+NuIdzEjln
mRuYHcP9FO4Hs2ZXcJ/1zLKB+oXgHwnEbQGe/EROb8psCfmdvWaj4LfTaWnA
p99RcSXPqtNAjR/Ap5CXP+T1Niduk9D7HZifUduHPjVP3lJ/1dU+wPsPl7MG
Dm2cCXDYhO+BrOYzU4Tngfoa+rzxhE//bieVcwPvX6FnKC5P6Ad9I1bF6ah7
4kEOp2OaFe/0qe28oqZRx208IRz6Qz8+FhUbBq8B8LuDPlvw3RqKx6ses8NF
9SIW41+65AF1UuPzuOL0s4bciCdfFxB/FIj/iq8Y37kv+3x5Ri2TKXG7DL33
88KhTxtD+cDd445U3H7xjT4MAONfqN57cH9f1Az24t4oC/d6Tl7T52ZSPrL+
IXZ/FtzmTfyudkvLKvLDrDRuBnZ/qLoNeDuT1/sU8Id8+Uo9xKaf1NQKpSsa
SDPjq+QFTxa7tUetwM0a/0Hb18y474NuNpzJLl9z4X7TX3pyDbV3Pc2Zde2c
av+g525fPrEH/2X6MAe+j7OqJdaY2wH+90m3282CNFNXxc2DOU+A+RSnnNL8
6TV9vgDO2zPaI/Zpun9mGnive8S1HxzvFeUpZ1B3fY+k5Ts955zn3Kx3ZDQn
+vgf0vevYA==
              "],
              VertexColors->None], 
             PolygonBox[{{4812, 6161, 1904, 1689}, {7035, 7034, 193, 2608}, {
              5304, 6259, 1926, 1177}, {5892, 5893, 1522, 724}, {5885, 5884, 
              524, 1554}, {5771, 5628, 1554, 1652}, {7676, 5440, 1361, 
              1363}, {7677, 5441, 3172, 3171}, {5907, 5771, 1652, 1736}, {
              4895, 8040, 3560, 2976}, {4690, 5772, 1653, 524}, {6262, 6263, 
              1895, 992}, {5630, 4781, 1652, 1554}, {8040, 4896, 2609, 
              3560}, {6278, 6277, 1177, 1357}, {7883, 7882, 1333, 1987}, {
              5437, 6333, 1359, 1358}, {7471, 6561, 992, 2979}, {4915, 6986, 
              2608, 2650}, {5475, 5476, 1246, 524}, {4612, 5907, 1736, 
              1522}, {4894, 7036, 2650, 724}, {7678, 7679, 3171, 1333}, {4689,
               6334, 2049, 2492}, {6332, 6278, 1357, 2047}, {5356, 6288, 1853,
               1246}, {6070, 4651, 2443, 1853}, {6831, 5632, 1246, 2443}, {
              6869, 6870, 2492, 1361}, {5474, 7469, 2978, 1441}, {7470, 7466, 
              1042, 2978}, {8345, 5443, 1363, 3172}, {5167, 5843, 1689, 
              1904}, {5149, 6144, 1689, 992}, {4454, 7607, 3103, 180}, {6142, 
              5194, 2979, 1895}, {7467, 4895, 2976, 2977}, {6987, 4893, 724, 
              2609}, {5358, 6071, 1853, 3104}, {4466, 5478, 1358, 193}, {5439,
               7675, 2047, 1360}, {5908, 4916, 751, 1652}, {5093, 7609, 3104, 
              1853}, {5632, 4691, 524, 1246}, {6334, 5474, 1441, 2049}, {6140,
               5148, 992, 1689}, {7608, 4440, 180, 3103}, {7465, 7468, 2977, 
              1042}, {6141, 6188, 1926, 1895}, {5193, 6142, 1895, 1926}, {
              6158, 5849, 5848, 6157}, {6188, 6258, 1987, 1926}, {5889, 4894, 
              724, 751}, {5438, 7674, 1360, 1359}, {4782, 5630, 1554, 1653}},
              VertexColors->None], 
             PolygonBox[{{5849, 5167, 1904, 6162, 5848}, {6511, 7608, 3103, 
              5357, 6510}, {6155, 6158, 6157, 5847, 219}, {7610, 6511, 6510, 
              5358, 3104}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.38977448939628373`, 0.22494579000449533`, 0.6648555136229459], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVD1MU2EUvZa2Tyhtsa+lpY9n0U1FdDKpZSgyOoEkim4gxqmA8V8XNxMV
ECclQeNsHIyiFKNuEHEjaNTJoJsuRhfbqud4XuJw8977vnvPPffce9+2kfHB
SsjMKrAm2FTObBr2wTEbaDWbaDMbjJhFsmZh2Muo2VKz2Qju5sNm5YxZ3df3
bj7xHQNgManYO/AZaxUWcTpdswfAWYPPJO73J3W+L6IY+h8HxijsfkQYkwGH
NuSuh4Rfx/21drPZTfDzhEvM+ZhZDc91WGcafsh1ArV0o6YG3m/Bf7zDrAqu
VzebnSyYFVz5P8T9J7wXHZ3t8nVOzErC7BVi78bEge8LwHkKewycHuRaBZ9v
EPEHcCIp5eqFVs+jwikBc9oTPxrveO7A981Ws/642Qbwizh/h/M/yP/bl37U
ifpQ8+G82dG8NKsh59gWabUHHN4HHNgj6kWtXgN7FXYA+KVmPcmNefri0pAc
mZ95+6Ht7S74It8cdPsM3gvA/I66lpBjDhwuQrcLsF+IqeH8dE46XwbfvRlx
W0HO6/Apx9VHzgD7uwifLwHmW9SxnhdfzoUX9JH1lpL/+8571j4E38W0cMi9
HPCPIraQUl8Yy5mhXvS95Mv/HydHOL3I42TVyzO4f5KW9oybCGITqCkGqziq
45CrfH2or+GLG7+vtIhDHdo3YFXwfYG4gZT0J/9Ki3JRA/aAfLgr9KPPYdS1
3ZX+xEwkhHsDeacK+p7xxJs41bT6wV50uZpjzpKP94Oe+skaWDPr5XyQF7Wd
Qc6viP+IupYzml3ODHE53+Sz1q554vmGqzlgv9j/n0HendCgx1fMMfA/n5O+
3Es3K024E486hMP+hLLqEeeG8889WHY0O8TsBt4OXzt3E3XMetKOeOGUMM8h
z9mc4k7ltNucZ+4ouRC/Fuwt35uy2j2eHcGuPAt2n3M6HOwR/2vUjPrznv8I
+nD272VUO2tjL6nnX0Zbvng=
              "],
              VertexColors->None], 
             PolygonBox[{{5629, 7073, 2689, 723}, {7470, 7469, 523, 2975}, {
              7073, 5628, 402, 2689}, {6741, 6740, 1178, 2363}, {4689, 6870, 
              2491, 523}, {6260, 5306, 1179, 1894}, {7881, 7880, 3392, 
              3393}, {7465, 5192, 3694, 2974}, {6143, 6263, 1991, 1179}, {
              7058, 7056, 764, 2670}, {5625, 7057, 2670, 1552}, {6869, 5440, 
              3170, 2491}, {7547, 7548, 1334, 1178}, {6746, 6747, 1992, 
              1334}, {7678, 5423, 3153, 3173}, {6156, 5575, 1005, 1902}, {
              5355, 5476, 1442, 2010}, {4898, 7541, 3043, 727}, {5093, 6288, 
              2011, 950}, {6738, 5588, 727, 2361}, {7676, 5442, 1182, 3170}, {
              7075, 7074, 723, 2689}, {5192, 7466, 2975, 3694}, {5885, 5629, 
              723, 1722}, {8345, 5441, 1362, 3899}, {5890, 5891, 763, 448}, {
              5849, 6158, 1902, 1005}, {8078, 7075, 2689, 3598}, {4891, 5883, 
              1442, 723}, {5887, 4615, 3395, 1724}, {6289, 5091, 950, 2011}, {
              4947, 8077, 3598, 3392}, {5301, 6739, 2361, 3043}, {4612, 6738, 
              2361, 402}, {6141, 6143, 1179, 403}, {5536, 6509, 2184, 950}, {
              6987, 4896, 725, 1724}, {6262, 5148, 1894, 1991}, {6133, 6132, 
              403, 1179}, {7467, 7468, 2974, 725}, {4613, 6741, 2363, 403}, {
              5588, 5893, 1727, 727}, {6739, 7881, 3393, 2361}, {8057, 4614, 
              2364, 3578}, {5892, 4893, 1724, 1727}, {7884, 6743, 727, 
              3395}, {5475, 5884, 1722, 1442}, {7055, 8058, 3578, 763}, {6264,
               5625, 1552, 1992}, {7677, 7679, 3173, 1362}, {5423, 6742, 2363,
               3153}, {7610, 7609, 950, 2184}, {5895, 5894, 448, 764}, {4614, 
              5887, 1724, 2364}, {6742, 7882, 3394, 2363}, {4453, 6286, 2010, 
              1442}, {7883, 6258, 403, 3394}, {5442, 8344, 3898, 1182}, {8344,
               5443, 3899, 3898}, {6581, 5842, 5841, 6580}},
              VertexColors->None], 
             PolygonBox[{{6287, 6067, 6066, 5355, 2010}, {6067, 6289, 2011, 
              5356, 6066}, {6578, 6581, 6580, 5843, 1005}, {5842, 6139, 1894, 
              6140, 5841}, {4532, 6062, 1849, 6061, 4531}, {4554, 6156, 1902, 
              6155, 4553}, {6583, 4554, 4553, 4477, 2228}, {6509, 4532, 4531, 
              4440, 2184}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.4167501908246805, 0.27184924959561424`, 0.7027731253123473], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVM9LlFEUvTo600xazXwzfuMMM2a7WhURhKX2B0SJFkFtQmiRP5LAKPsJ
FkSRmlEUgRXlRiGJ1HQKJIJ+SRRUWrQpUiRbRItcTEJ1DudBi8v7vvvuO/fc
e+57lY1t9QcKzewULACbjJqNlZvNF5tdLTLbEDGbKjA7mTE7Afu+FFZq1gT/
RxxcnjRbAWsLme2FPYW9Bk59CTAQ2weMQzi3yhPWHHxVYbNr8LdgfRw0+w2c
bWmz7bD1iOkF3rBvtimkPfqmsU6WmR2MCn8Eayt4vgXPB3Gzjowwb5Trn99f
kHMa/hEUVhsWJ/J5A5x1iPkB/4ynfMx1LqyaWS/jyJU4rGPO1TIK/Puw4SVm
DSn8x3XmfFjneTZfKs7kvgcxu2FbsPcpizh85+HfiHyPgv+51cCu4Ow41mrY
ZXx/APcF5JpFrqMVZj3oTzNw9+O7Pi0dzoaFEw0oR63DeQhe7Unp0YvYfEL+
NTGzlb54zjsdyfMCYupiyr8LHF9lxWPAVx/J80xE+ZiLWla7enNBcRyGLneA
0w1+nRHpcyshjbrBZV9UMay/2Wn6F1oEY6r3JmLv+S4eOF1p8duKtdJTfBD7
IV9zcx2YRTgzE1It9HEmLyL+Eqw1ojnqcfpOBDVzrHcANa6OqR7ONPtPffuS
wiXmC/TgZVb6DqbUO9bLWnNOu7q0dGVNnz31i37i9QfV8y70o8FT32ZR71RM
ezVOL+Yd8jXTrJ3430rUK9bU5u5aJ/KczmpvEThpTzyI15AWJvtKLuznO3Au
QO4ccvwEvh/XHeUsHEmKG2eWupIbtc+5+RtFfBi5NiN+kZqU6d5xfqs85aCN
u1nlDHY6nNtpzQH/J9z9nnaatzjdQzFpz7Osr9XVyFnoqJC/HfvNmKHRgM4R
i2f5Bjxx70AA3Ap97ZHviC/Ox3F3jmV0d4nd72aDevLeUtOdWItjeo9ojSHd
d2rPeWH/2V/eVWq6I6UzjC1HLw9ndKZ0mfrI2qkXZ5E108c95uIc552fXMiJ
/U9Ah2cJvbclSb2//P6D2gd99fWrp16wD2NxvXHk+TyhM4xfG9e7xjeNuciB
9Q6B792U3qdfwFvwxZlz8T4lvf8BjWDydg==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtIlGEUhk+jjjlqoTPj78zfaNaqTRQRhNplHVFiQreNCC0qy02R2mVh
QRSpFUVBSRFtspJCJzMhIkgjooWVtYtyiHQREbkQIXoOr4sD//e95/Ke95zv
r2lpazwSMbOtWD72MW32ANtYZDaCLQ/MBjhf4PvTIrO6uFkedy+iZq0xs/ZK
s33gW8CvgQcJs2PcjRWaNWMr8N+1kG89/mNJs6XgbWDTBWbHM2bDxNygeAt3
R8Hmyf8eLq85d1SbdWKHiL1K/mXkuxuadXEehcMffJ8n5Ov1CjnX49MG/hK8
ie/71D9H/bMxYZeIPxyTTzff7eSfKzU7yPlhWjHue56YHrBurCxPNXtDaeC9
O6fV1B5Pqhfv6VFaOTzWNbuC/+VQ2njNNfiepOfpYrPGEtXeDr4uJg6DKbO3
Fep/cDG5AvXsvW4g/058V3L+wtDucfeqzOx3oN495g75c+D1nCfg1IuePdgg
WB13N/meius7h+0nfp56I1H5ZH1+Gc3qOjMJ4+Lgtd3Hax+A80ypOORT70mg
fFPOD/wieOkSaTYEdjspLs6poNzsb6BZ16LPmSqzWwv8fpZod1IJ4b5DJ+Ay
wvlHsTSbRZ+uKvnm0Gc3WucqpMUm8q3FN1mpefyi/gRYNiUtXZ8CuGQDcfH+
v8F3IBC/LP4N8PuQ1u4/wzqo34n15WtHa72/UNgc/fcT+5W7oTzNqBGsJ9Qu
uUY+C9fca3nNx/h/j6uW14xwrqZma5HelGv3pkrau4bv+C4s1+77G/K32Z/W
W/TzqYw08N5nijUL79Hr+UxWEfuvQtz8DUzi/zkj7ZzzDrhuwyYj2kHXuimt
2bjmzi0a6C05xxq4N4Tq3WP24rsH21ykf8AwWj9NSXufgWO+877r7jOa0Jtt
Xpiva3s6I+6u8WwgH8d8B3w3+iq1W74j/wEA/LXR
              "],
              VertexColors->None], 
             PolygonBox[{{6160, 5836, 5835, 5166, 1903}, {5836, 6579, 2227, 
              6578, 5835}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.44372589225307735`, 0.31875270918673315`, 0.7406907370017486], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVFtLlFEU3eqoM85M5Xxzn8YLPfUSZToD/oOodLRAhZ6qFylSK6mUIhVK
6PIWvWgjlWV2wagIfOmlgm4UNNRDYVRe+gNB9FCt1Trgw+Y739n7rLPX2nuf
xr19nYfKzWwYVgE7lTUbgv0Imq2pMwvDBkP6X4RN+Mzaa8yaYN8R3x02a8b6
SZnZTMJswTNrqTa7njGbho3BdzttVoKNB8ymKrUXwWW7EXsH+x7WSzhTBp/B
bqXl4/43fOeA2wp/LGm2JWp2GOsisDvhW4eY4/VmFzLK53Uce4i7XAU/7snB
Rsr1zbv1wZRZV63ZWb/ZFcQWYcvATAF7EOuFCmHXJ4RfAHajJ57zMbNkVDGV
WN9HzF9o8g57l6DbU5/0O+E0LFWJK3EmgT2RFNcNwNuVUU4riLtaJW0fpMSB
+a93WpPXJsQfQXzBcepw+6xDwel/JiDtS2XKtcntv4wLl5ijqNd8YLUWXNdA
k2LE7By47CkXv7m4OP7Bd2NEmL3AO+Di5915cmMus0HlmwPvfFb/jxLSiPpM
4v5+aD7tV83vurp70CMKG0DeediK64OPwPhArJA05b3kMptWPzH/e/g2JKRx
EH1aA+tH/LWE6sFz5DQVEa/3iC9ElD/x6Od6FOd+pnQv7192OQSQ1wvEDyPm
N3DeuLp0A+dkVvUqgt9Dp9uriM4wnnUeq1OtOTcrbnbIfykobcmBOlLDx8j3
WFY+3s0c/s9CUD1AbdlrPqdnC2Kbs/JTo3z16jm/X2fJkfNJ3boazPbXqmaf
kNdnWE9Y/H4llDPPDFUK/wv6bRb7OeBtw53PksLdiXWoTnHbY9Kdmu9Ab7Zn
Vvuxx927Fueex5wmPvU5Z5Cz8tbViPXbF5YO1IM9yrr01mtOOCNHgb3Z00yf
BmYpIpwOT28B56sto7eAc0G8caftDOp1M6354Fs2EFL+5xHf6qmvv6K23+LK
jzOUc3M0nRBnxntujrm/GFcfkBP3yYE+xhGfGvLb59b8Uif2SRG9NgW7AT0v
8h3LaKa3QpO2qGrd4WpOLsxrJKnc6G/Kyse3KuHeIurI2lK3f5R0xzM=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlM1L1HEQxsdc23Vdy3bdXd3NVY8FvWDagv9BaOnaoYJO5SWC1N5fKNIg
IqprEOkupqVG5aGXY9DboYgObQUVVqTZsQ5BdKg+D/M7DHx/OzPPzDzzzLbu
Gujbt8TMurAQ9j5n1ps1a4+aba81e5kyKzaaXY+YXV5q1odvT7NZHv9GbLjB
7DQ2DMg4/tVxs5mM2blqs93k/yL3XRPxMbOKKrOTvE9h32vMrlJwXQIcvmf4
LoB3jPf9erP5Go/J870BWxnymI/0N542e1hhtgP88xnvWb3upebjFWY/8f/A
HvFeD34h6732YSX6u4OvM2y2k55zvKfAOEtuGcxnSbMk8+zHfwL/IBiHm7yf
STj4TfyftHMh/1F8X1I++yIxV8gdxRbIN+atgKvX4EeJHaHHA/TyIee999LP
GPw8YN6FAH8N/W4mpg1fD9aSdk7FZZkak/hKWLzS7Az+Nup3YNM1ztFd4qvB
H4j5fOLqGvYk5Jz1M89E1nNLVY5VoGZdpWMq9x7x/0KO0d7kPakX1Rgi/3nK
55cm6ph1eYNzod8i5HfB4WDM86fB+qQdw0cHFsVfjHtvinkL/mf8+bBrZIb4
vynfhXJm+S4mPV4766HfLdhXc45u4i9RfzHsGDHwl2GHwDrOfFuJfRN3fUqz
tfi6wRuKeYxyx7BvYceYpfZoo+9iThwTP5Jz7WqnF8G7FOhJPbTSe3ewL91M
c9o1pf7FqWYTBx1Bf0VqbaJ+JOI15xI+Qz7g5wZamQripclQ0jUlLWknt/Df
zrgWEuC/QjvbWnyX2vER+Gys996lKWlXGpU2peEJsCfTzo0wyxnXqLSpm9Wt
6CYLwX33wt0F3XzUNfy0wTGUqxlW4Z/OeC+aWbc7FvCnemX8B7N+e9qBuI8E
+tQOpPW1Cb8FcdiZ8JvWLaumdjef8v8K5aeo/yLuetON/gfjWqVM
              "],
              VertexColors->None], 
             PolygonBox[{{6560, 4547, 4546, 4465, 2214}, {5840, 6166, 1906, 
              5169, 5839}, {6063, 5749, 5748, 5090, 1850}, {6129, 5562, 990, 
              5143, 6128}, {6164, 5840, 5839, 5168, 1905}, {5749, 6508, 2183, 
              6507, 5748}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.47070159368147413`, 0.365656168777852, 0.77860834869115], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVEtIlGEUvT7G0XFUZsZxXs5YtIuearWQliW9KNNwH7WIbKx8Q5RBSVjt
25RDOjrmojQSihZBGFmERa16Ei0iK0IsElvUOZwPWnz8/3//+zj3nHu/lQfb
96fzzewCTgHOrrBZccqsz2PWkzCrDZl9h0N73Ow4zoESs+EVZjvht63CbC3+
1yfNvsDHU4wYxN1GoocBs0zM7BO+67yyT8E+XGQ2h+/JQrMW5Gr1Kf/zKrPK
qNkV/G9C3Q3IO5+vGmm/2aBHsYyhfU9CtVl3M+qvd/7XkCMTFYZe+PQlVGMB
ePfBpwL2QMRsI96/wb8NubeWCl+u2myRmNFHHuwNsM8AZxew3UTMEdSv9co2
iTxZ2EYi6o895EqFYRi26xH59pTLPp5ntozc54D1c7EOOXgAzptL1Bu/0+C4
MijbEJ4+aPEV79vBQ4fjobFC3+TlpF9P+vyJqQZzF4GD1ZVmR4HhRFz6UTv2
PObw9AJLd1IYdoOfMqf7GsT5EH8MsTnEpdDLS+DrqEE9nIUy4SAeajoLfuKI
+VAgjrpQP4ucnX5xQ59Fjzgld6xPHNTuB/j5i+8p0/xQD2rxC3iycenF/+YX
5xOYq4GI8lMn/qOd2jKG/tTwp9PxEXCdT4rbj1Xqmf1O4z3hMDfDtyUm3OSW
nJIH7gF54Tv1fJuSpqzJ3MT8BDztDf/no9NxwrmYiap/YmcccbZid+Yq1Xcn
ZvNVUNqTD3JETuq94oIzPIo8YxHZ7qHOQkQ7Rb7ZA/GfAiePcV6b5jQTlA5D
6OmZ87kE+xw4WuWV5scD4uFdoXKz1m/43A0r/wTe34dk3wHbUFS6DCQ1o5xP
zjXnnrPEfepOqI/L8J2NiodN8K9NSnPW4hxRoxfIPwlsF/Gd8gsD/3N3WZt1
+/F+Nqr7YhS7OVatnZxGvjtJxXEWyRvrkn/uBfWiJkUp6dIW0w6Tk6t4v+Xq
UudGt0f3g8JEewZ9ZHHO4N7Ikf+4sPHuYb/ccdZpdLV49/DOmncacsapY1VU
9xrvNNoa3KymHU5i427VuP1axnMJ57BXOvMOpRbl6MOfUn/9wHTaJw6528xN
3XlPL4WFrw7cbElKH3I8GFNfjGNPnFvW63d5qA3vKerYFFL/9HvDmglhHwf3
N6p1V47AdiigPE+D6pM4OB/8T93byFlI/a4L6b4mJ/8AKMr5bQ==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtI1GEUxa/pOI6m5jz8z6gzGu0iItRqES1dVBSlhutei8jM1DQ3ZVAt
tJZFm1Sw8VGbNAqKFm2MLMqgVpVFuYhelBjYY1G/y3Fxme/73/udc+5rVu8/
1ti2wsx2Y3lYttLsGjbF+XSh2RDnPTGdT2Gb02absM88Gi8ya4wpxn19WDfn
zmqzhWKzr8Q0JMxCGbPWlfwWmM0RP15lNs95MtesMm62FHAPmb3jXh0I07Fe
5Jsl8d9LyL8vbNYBdixqthgSxyB8C/hLc6WhCK4d3Hvxd8D5qMwsP2l2hLev
yOky2kfh34L2HPyzcHWXmx2Gaw0xB4lvqTH7XqIajFaY3eFNhti5PHE7hr91
Db/gWsImcsy+ke9winqklItrriV2Fzn0gDcG5we43mMPwJ4E7ynnP+R8KKw3
l4h/TryB/4/4dvRcBW8gX2+2wxUmxzb8DaWEcd7Jty7unVgcffNgjhDfFIGT
XFsrVGvP+Tr3i9RjivtW7mfhexiXlo/0JILvCRhXeH80LKwE9+aIMMeC5Z4S
Wx9W7ZrQd6JANbxZrp55r1zzT3w/Eqql93wMLeMV6q3X02t3nPveiGo4A/9j
rD8kjbe9X4G4vAe/Oc+Uq1Zes2bwb1CjrElDO1irAmE5pms/yYy0FCqHtz5/
gbhvgVEP/sa0evOJ/vUS21OpXvqbu+hbDJSP99NneaJKtfOZfoaWdXHVymvm
uQ4u74PnfABtf1OqrfdviNwGk+LuCamXIeK/RNTTFuJbU6ql5+S5lAXaJc/J
tdyPqrauqT8lDsf2b9Ngb0sIuw5/tkIz7Pn6jqyNS7Nr9Z3Ih3+kRrPkNS/m
XpLRLPkO+SyOwFkb1kwWZrRjvluuOQ5fLKndcswN1LerUrPi/xHnqO35tGbR
Z8xzvRBo1/z9AOehqM7+zXelLq3/Ft+Z4ahi3Oc18VnPBsrNZ/4NWl5j03nS
2An++ph66d/OJLVzzj2LfzauHfPd8h1/CX5fUj7X/B9ogL+l
              "],
              VertexColors->None], 
             PolygonBox[{{6154, 4810, 4809, 6153, 1901}, {4810, 6137, 1893, 
              6136, 4809}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.4976772951098709, 0.4125596283689709, 0.8165259603805514], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNVU1IlFEUvZqOjko543yO48yk7qRcF6XbCtKM/CNwlQZJpOMPlNGu1FKs
XVaLLKjUamNagtUiCCUNzGgV1KqFmgWVmkoUncN5UIvHvO+++3PuPffeKWpI
VLUkm9kVnC04/nSzojSzzlSzq4Vm13B2bzPri5ld98xeQ94fNkuNSmc8xawU
vyNJZok8s9GQ2QvYt+A9gVORYbYXdhtxs3LonYHsLE4d5L8iZo0BsxuIOQ+f
v3OgA9vlZNkt4HcO8v3wGUiRX8bjfZvP7Ch0/ND5Cf938822QpbIRmycXcA8
CTxleBuDfg10J/A+nCS8lBHzILB8g06WT3nSjm/Exzv90yffiH8l0+xHpnCs
Ac98mvzFssze4/4B8suI+yCm7zBy2shV7FHkew75Pkr/F4MYGIf3MVdL+hxz
+b51OZ+Hzwt5emtGrE6cXuge8ZtVwrYeOG/huzpD9XuMmF9Ru37E9YJmn3IV
axY+ZnCygKEWutM+1fk03t8A6200wR1w7tuuOCPI4yPkZch5CDW+h9ONmOvw
HQUvx4HnIu6XwuqbTfC8B/X8kybfxEL/7BW+038x8HRB/x3wHHK8UKcE/uaA
Yxk6B0Linlw3QjbnsDFvYmfuHZBXBsUd+a11PcO+qHO9MeHeFlx9iIdy1qnC
1aoJ+Y7HxeFN+Fv1VM9sxG/NVx4TceXP3EuApT2qum9CZxY2A9A5BruGgOKc
BNd1uPek6z0jTzppqOtBT/Xog+2Am6lT8D0cE5+Ucd4ojyPuS3wv4v4Q/moi
klf5pct+JgfkuS1Z8fyuPqwTe5B39tGa69Vx1G3J9cZMrnIgts6oeoV90oO4
3XH1ey9iFsc1p9UR9Qq5YC04c+SRGNPzhJO9Qmz1rv5VDucK4kx6wrnOneBp
jjkzTany8wU6z4LSITd8J1/sJ9aV944CYIiK17Z8+aIeeRsMKi73Sl9EmFNR
832eYnAOQi5Hzv1qRP0zgPxygsL53dNeoDwQVh+wBzgPrC25OxzU3mH+A4Xa
cdwP3D3ESpzceaX/yblHyh0XpW53st93hNTznO+nbsa7EfNzSLWhvxZnOxQT
VuIkt8yJOieAYcnpM9fhsN64W2hLPMyvvUC23EHknHEZh/a0fR4UT6x/G2rQ
GtWM+oDrVUD8sne4X9g/rEez20X8T2B/sz7T8NkV164cgv6TuPgl5pjT505N
uF095anXF11daMfZ3wk/U24n3I+JS+b1FxJb+BA=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtIlWEQhqfU4yUpzsXzH8/xpO6k3NQmItcFWVpqErTKWthFjxqVUatQ
u2DLrBZdoFIrCC8kWC3aKFpUWqug1hXWosyyguh5mbN4Yf5v5puZ9535/sqW
TEP7SjOrA7kgkzTrBGtCZgs4RkvNpkrMPuWZjRWYTcTNkjGzA/lm/cQcT5m1
gx1FZo1gJmx2JuyxunOHXFURs7crzFYT3xeYPcD/Al8FOV5FzS4G/n0rxyyN
XR/x2DegntrPE2bF5OsuNtvC9++02T/u1nLnfpnZPdCK/Y6zuojn0N1iMIRv
EAxRf3ehWTO1p8lXSL4+ME/sU+50wrWW+zewj8X97j7OXtNfPzHd2f6a0KOx
1G31/Jhck2AM8ebyXZtZ7g+EXKMZ7J+B2zrrpfdpYr6vMlsES/DZFfV+xVm9
nSO+Mt97TMVcc2l9kLOv+L4E3q/OYvTbjsbj1B+G4yPyT4DeQtf8D73+BdfI
Fc71Wt9KnJ9qbuA7TL5p7D3M7wPfR5OuVQ39FdDPenpYyPF5bsR/knk3E7sX
bCbXfnAzz2dSg325wmzTGtdkHXdDCb+rHJP4fwXORRw+428lvozZvie+jdpH
svUbQE/aOah3adaR9B3Vbqrnu0nXUNqJ8w9iF0t9duJ4CPtwqc/6Aj0U0Ut1
lo/mIe360j4LaRha6zunXWsD27FvV7itHZhiFiPkGy3wHJrteHY+mnEb2nSC
nWjzkR568PcGzk3z6MJ3PeL91RU51+qo2+L8kP7OJnyXlkAVvW2LOX/NcAS7
B/88vmegJexvUG9PMxii1mDgtfQmzmv34r6v2qmr8HkZ9ben/V2Ou+bSegIO
W8mfSXhu1TyR8h3RbjSBK2gxUOFvTzPuKue9lLtPb2w87ZpLa81Usxgu810S
5ydwnwv8rWn+4iLO4ipOHdQLot6LNDwd9jekWUnz5RLfQe2e/lGZlGsurfUP
OoXdnfK3oB2dJfelhP8b9A/5D9Jot5o=
              "],
              VertexColors->None], 
             PolygonBox[{{5810, 6537, 2201, 6538, 5809}, {5117, 5810, 5809, 
              6097, 1867}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5246529965382678, 0.45946308796008983`, 0.8544435720699527], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVDtPFGEUvSyPXQSCLDs7+2Dw0Ujio7KQaGKlBlwQKawABQsBhQWJNmih
gQjE2ABqVLShMlFBaHxEgwmPGJCNlv4AMYEYNUEtiJ7jmYTiZr755nznnnvu
/WZHS7q+M2BmlxHZiCMRRNJsa47ZQJnZIGJXgdl8ntk5xArAC1Gz3hKzJeCf
YL3omg3h26xjNhk3WwVRR8Lsd1T4NuxNeWb9ITMP2NqwWQj7X5DrGt4/I1c/
nn2IbUGzuZjZKfCPgz9QbJZVrLO14PwFzgzODgH7odRsJ/A/kPc+NBdBw/5c
sxnsvckCH3gmkGM4R99/OsJUA9OWbxYMiNsrNOsBf3u+9nn2LPIMIEcD8Pfi
qpM1Uu9KRJqrcpWP+KvYHwkLT176Nwn9k8B2xaSpAxrSiONbzG5ib7FU9a7g
bBi4BqwboaWpUJztcXnH2lu3m32NyI8M8K/C0tOdFC85v+fK72XEDVc+0sM9
yHM+oW/N8KDSkQcx7Nck5OcE8syixlFwlqNXZYhb6HsKuI2gam2Oqk7W+DGh
WWGdaeQ5inU/cK158pj9fYf3asT7bD3HCrQ+BmyFp/fdWIdw/kJQmugdfXuB
vZcx5V5GziVXHBFoXoeOaeSohOZ6xKNs1Z/xa6eOt0H1/XShPCX366A8+o8J
qef0h3qnfc114DsZ17cUatznqMcp+F2X2Oxtxtc5gL0RT3OTxszO+PeC+XmO
Gq648o6+cf9wkb7R+1rff55v9PtuyDVfvomr8nmuw4+nMa0HofF5VH7uhYeu
p3t3G3NyB3EAczKF7+uu9KR8HvIPe6qHtTAnc9Ofx9De68oTzmVfvuZ8DNiL
Sd1/zmbK5+GaefjO+erwZ4w5+Y9g3ks411IirxbA883VjLWxloh6wr05f/Y6
kzpDfDcwUxHNEevu82vvAfZhWLPmYh66kurpibD+L+zjmqe7x3tHfzkf5GQP
a/za/wBz0NGMENfu333OyLz/r6PHG6XymTPD/wVng3ofxKX5jKN/Gb1aAvau
o5n8hOffEvFVIOd4Qp6Ooj+HHPm2Cr1riCbkfxbVv4b3+h858sqs
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtk7tPVGEQxUfYZVEkyu6ydxe8iDaSYKwsRE2slIhvCysExYJHzPIqsVI2
ssTYAGoh2lBZKGZpfCQGE4IxgEZL/4HFYOEjiIlEfyezxeTe75uZM2fOzLen
q/9itszMWrEIdrzeLJ0w+8jlTi5+p8xSnD+U+d06585as0r+5yrM+onPYs/J
PbXN7FHc7Bf+anw9WDaNg7tj1WZvYmb5OrMxrHerWR/2MANGjdkyYVPEh6HZ
Avlr5WZ3qxxrmvzuCscM8DeE7lNMIQm3wKwN7LfYT+KTCeem+Dliz1NjuBLs
7WZH8G9wd5n/TmyF3FYwpqscY4H+mjlfjzmfUfxN1B+F627uZuC+DsaXiN/9
g3ucmEViv0fNRvi/gbVznqGno8S+QINNck/jX2zgC8bBqGui3KGU9ycMaXul
hC+Nl/E3k5+LOb+lhPekXsTnK1xzgf/r7iWxt7CTUa+5AXZQml8RawH7T+i9
t0U99iY2EfEc9XYu7rXFYYT+ZjOuhWYkLpqpZilOQ8x+f8Kx1cMqfLobzbbs
MCvDVvBdS7kee4lP4n/GeZxzB+fD1LtH/CFiO+DURb2rtc5NO9UE1zz135f7
jJ6S/7nWtX5N/qZ2NXA8xQykfce0W/Mx5yKNpI04FcDraXRuOXbiAFjfQo/V
TFbBKqS813bu/oI/mPbdEv4rtFkLnLtqFsG+k3b9pZdm+aO0/9LjDPGf6vwt
xdDoRNJ3rq2k3wNix3eZ7aO3x+CfJX445W9HM7hA709qfJe0w5P484FrKU1j
1H4X9/loZyfpZSL0WnpjA8xnsN7fpnZAb7k37drpnKd2S8Zri8M8tYqB9y4N
LnHuy7hW4jCG73bg/6p/Hy2nGn23NUO9Lc1Us9Qbm6Xf/tL+aseWAtdUWkq/
/2W/oS0=
              "],
              VertexColors->None], 
             PolygonBox[{{5138, 5813, 5812, 6121, 1884}, {5572, 6151, 1900, 
              6152, 5571}, {7452, 5572, 5571, 7453, 2958}, {6025, 5527, 5526, 
              6026, 1826}, {5813, 5140, 1886, 6125, 5812}, {5527, 7304, 2855, 
              7306, 5526}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5516286979666646, 0.5063665475512087, 0.892361183759354], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVEtIlGEUvaPWOM5YOM77obYWDTOIglbZwx5mEbS0ghxfjBatVDJoodJT
1CIQF9WqoKBS27gIQ82sKDcFERFFlpVQUBBRncP5Fpf/++5377nnvv41R9r3
Z3PMrAeSC7GgWTZmdsFj1pcw64dM5ZkdLDQbX22W8ZkFVpl1F5h1QWbLzB6V
6a0c99EVZk8BuFBqFknD32/WiHMvcJrgG4NuBjE6YVOTNNuSlN8X6JcgewNm
p6Nmg2GzOWCdwfsyzhfBpwr+63zi9gJ4e4AzgfudkNk0fIbB8xBsG8Li8afY
bB76Wq/ZsZjuPGfT8qH9M7zfdedW6Nsgr8F5G+Juh1SC227EmilR7CT4TQJj
APbd8O2CJHEPAftHWLl/BUYlzvWwDQF7LAJMFDcGm63ArCgQjzd+xSVWNfie
96i+jFntU77kMpSnO/0mYfMSvmnIKDArEOdTWv41RWZXIZfQxk1xs83gfDjf
bG2Bar8Bsgu57EwIe8LpiHk7Iq7k2Yh+fsDZ0NMT+cIgZhtqmIOaD3hU458R
1bkE8aOu10HU41+RchmHzRLu72DTCd005BVw7kP/OyX92Eqz9T71tzUl3oxH
XbXTZ9yZPMvB4UlUMzAILi3wOZAj7OelwucM1QVUk6PQxVH395jfdtiMAP96
vvJucph9wLgcFk4x8r7p+sW+BV1NemEz7GayFj3ckVRd38Im19WE8x53dVgE
xwdFypG5shY8kwf7y5y4UxnHgfPOGMRnD5ujqvkNnPfFpW9B7p/Tmj/OFueR
szEI3RDkL+L2w28oqFy+A/N4RL1gfnyjnrXh7lI/C1tvTPtIHfeaM8xZG/GL
G/efcZgjbR+GZU8c/gvodw/x54s1k+M+caEfMSZcjg3g8tjZ1OE7klQ8cuTc
kA9rd86vPjJOKq1Yc7A/CZuUV3lXudz5ryp3/6srqFMW9bqG/iYCmi3OD/lM
Of5h4CzHlSPryBnh/rUjTkdaO8l/XkdMPk2uT+RPPOLSnrvBM/EjwPwV0ftZ
+C2GtKP1cdWF+faA+6mo9tYT1H+NvgvI8VZImN/wHsG5wav5JG9y5s5xzngm
X9adtWL8pOPTHBMPcuBcfiwU9wR0mVLdW2OaIea9Edz6U9q1/3Pz1wA=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlElI1WEUxa9aOTwrfIPv+Qa1dWiUQbRolVaWz4miZQPNmhq0ykZapNgk
alCKm1oVFBUNmxYRpWYD5aKEaBGVlpVQULvqdzlvceH7/vd+59xz7n1v0fb2
5rZsM2sg5hBj5WY7ysx255vdXWi2LmEWD5lNzTdbUsB3cruIPfn6tpZ8Q9ys
insl+UbOdUHecl9BzEaoB9PAOp9lVgLWmoRqP/K+I2XWRgxA/j5g9jls1hIz
e5Brlig0GwHrR9RsC/dDNNpcYraS2JZndjBP3FlBYXsP07w/QX2K+rfESCnc
5M+RX0a+Fa794PfD9w6+JPdSojsg/Fsp9eDcceJTWN/87BqmuG9F0/Bcszfg
d4B1Myy8R8R6+mlJCXtpvrzcWSat7um+JPmk+t8I3yW03CimPsfsMt8eFpm1
E1c4f4DjD7l6MJfz9s48pMA1HVXO30yib7Rcs9oMR5raGB5XFMgP19pK9Gap
xyb4Golh3j4FoxItdXF504jG6+DvjYmrl/pnYJ2OyUt/s5rZbSqS/sVwPKeX
l1HNzz2Oor0CjDMBeTpOrj2l2daCMct9NCivfaZfyH0jGuCuAvMJ+cdRYTnm
bbAOxPTW77P0Xp0Q94tsef89pd5dXw7YTXHtnmt279KZ/XQPPVeb0C57je+K
z9Rn6TszgB9jaE7kyp9TcXns3vrOdzO7rqRm5zN0L3pieu+e+PlYZv/821HO
fUHVJ7kf910ulpdeU0LvsZT8cL9WwVUN5gUT/hC9RkLSepieZ6hNc68vlGfj
nE9G1atj1lBfg56hgHZgkPtPNJ3Fi84C/Tb7wPgb0G+0h92sDwm7cIHZb3oL
Z/hczwzYobB2zXduEP/vR3T2Hf0VEUdnZh4+uw1x7b7PsB+urrh+G855DfyL
Jdp1x/zq+xLR7P03/w/tExHtnvfsWr1H7801u7bX4N3Ll8Z+9ByJylv3uJPz
1SL54TNMB/Wf5LP2N6/AmiiT15N4/B/NFrVV
              "],
              VertexColors->None], 
             PolygonBox[{{5831, 6555, 2212, 6556, 5830}, {5730, 5066, 1820, 
              6013, 5729}, {5142, 5831, 5830, 6127, 1887}, {5065, 5730, 5729, 
              6012, 1819}, {4462, 5806, 5805, 6554, 2211}, {5806, 6091, 1865, 
              6092, 5805}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5746901820036648, 0.5450294005687265, 0.9097465282153145], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VEtIlGEUvZaNrxmtmfnn959xHNtYi1q0SNu0KnqbGNWi6CVlomZREW7K
MuwNQS+ICrIgAnsRSVZQFJWaLrJWBRFYCvaEWpi06RzO1+Lj//773ce59557
p9ZuX9E8wczO40zEqUmYPYqZncBPV57ZgYhZCAobIN/vm60JmX2ZZDYM+ZsJ
+o64e00SPqJmc2C3HvqnotKvnCy9Ifgfiuif9wUpswCxBmHbDttDOHWwvQ+b
1dAbg7wuY7Y1oxi0m4PzB/I+2J31zC4B50HY1BJDltmniHQYp9thJbbeMrN3
pbJlTsyNsahLG8rp40GRWRvk/dAfKNM/sVCXeKqBcSFwz85TjFS28DcVm23D
eZxjdjgwux1H/nj7WGD2AacXOKsgm5WWTQNq+QA5FEF/QY70qLNpCuQ4T1Cf
zci7r0z4eBiPsWg/5Oo5H7pzkde1sFkUPtaG1DvmVenqcC5buIhncUp9Yo8a
gOVeXO/zIf8J3B9zwQV8O9G/o/B1FBiOZVST5cB7HPcI/O8pVJ+IZTf87MIZ
QvwfyKsHfT+Dt1XANg9+rwLndMR5Cp+jsB2APOwLvwefF6EzI99sI96bfOVA
HJQNoqen4W+fkyehvyilnpZDnvKlMw55e0I13JJR7Viz6kB+adsK3ePFwnMT
8ucJ4WSsz2HJ+U7cvO9DXVtL9Tbs+EeeMG9ygb3wkNfvEvQYcf/iO47Tgjr0
wHciLjm50+X4cwW1/+HqHMP7V2Bah97cgP6RQDUvCEmfczcNOXYklSN70Jon
nrMHvFP+PlszQDl7dcT160IgHMRQm9DMcF725kuf/u+ib8Nx+WAtGY8+68C7
n56wcC7anP9b5DbOnVztiZOe+PYwpt3BeyN8LkmKb+xJe6C+NEO+I615aEed
DuGUF6gPf0rUiyr4+ZpWfhFfM0OOvYa/+kCYydmdafG2xdeu4Z7hfB9OqkfF
eA9wjsF/RZ74T/1+6N9KaL7IZcZjrPp8zTR7VOiLo+TnQdz7oVOaozfqsO/P
PHGafO7Fvc/TruG3yNd9JK76srbsZ4HjDevCmWdNKGPsMbdLace4WVHtGsbb
nhZuYv6G+yjOsrDmrSOkvLg7B93+bCxWbMb9jRx/ZTSvl/E+Uire0L4qrJ5y
zlgD9p3+WC/6ZD9YP+Y4k/mlZfPfnrbceU1p1XeZe6Ov7xnND2eHu4Z+ObPc
48Nu376KiZfkJHvS7PbeJcx3dUw8ve5ptlmrbk+cJp9foB4vfdWRebKO5AZ3
TGcgzOQNZ5X9It+pRx1i6XZ4GsG7hhLtzgrYLQ+E52FUM8l5fItcBnFWooZL
k4rNuP8Aw0UfWA==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtI1FEUxo+PpnJ84Ix/Z/4zzYxtWtWiRRJRq8JephDRoohIyERNo6Lc
lGWY9oDAHhAkZEEEEUGbtEVBi7JskboLpLAytFLIjdmm3+FrceCee17fd865
d2VD++62fDOrRQqR4oRZCZJGmUQviJl9rjQbiJitX27WHZr1IH3YhwvMenNm
d1Nmndi6kNaMWUtG9hdLzX5w/onUFpMX/2uB2cas2QP0q+hX0mZ51FhHbFOR
2TS+a/DZVayYbdi3p1V7DKC91D5UbvYSbJfAtBnbFuTTMrP73L2v4j4nLENl
ZvVx7sF3Af0M+Q3bB/QIuc6XmL2G69MK4qNmNeD9DraRKsU2YJ9CXwW+0Txx
DMFXiUyD/TIxWWzd5GvEthf/GWzNSbDmicMM+Z/HhPUA+fM5tybVm2pi3lJr
GFkAzxfi5+AXgHl1kTgdAW8qLu5T2E+TrwPZFxHfEWwHK6VnyXmHXtTF1WvP
cQI84zHNbxK8N+ltzwo4ce4vUG+cs3P1Hh3F/2Gg2U7g00avb4fC7z0PyX04
J6yjYKqkd7PguYF9gphW5xvXbPdzN0+tv0hHVPZz2DsTwu6YHdsi+Acjwtjj
vcyJq/dkE9g2hMrnOzOA/V5KvfGZ/Mb3D/HPiD1VavaIc7xCvXbMc4F2zHcr
it5Erv60dsF3YAFsQYWwOYehcvXAuTtfr92CTywqDJ1ZzcDxfwXPcXxPZtTb
W+z8N3I1VqnWx0LtQktSZ9+JN/BNZrQ7Xq+Z3Ndj6pXXuIheHWo2PqPRlHbW
d9V37An5mzOq5W9sB/adKe2u68ewtWeE3Tkskm8wED+fwRi5XgXa3z3eL2o9
DoXF72Y5/8qJm/e4DK6lCf0FM0u0++M5xbr/WWzv4sLrHHyWw4F8fab+d/jO
+677fH1W/mb8rfjMtjKLmrR2yf+AOurXh8rlOddSbz6n2fqf1EdsV0K77vvj
b7ft/7z8Df8DLie5cg==
              "],
              VertexColors->None], 
             PolygonBox[{{4515, 5994, 1807, 5993, 4514}, {6018, 5524, 5523, 
              6019, 1822}, {7740, 4515, 4514, 4420, 3238}, {4551, 8146, 3668, 
              8145, 4550}, {5559, 6122, 1885, 6123, 5558}, {5524, 7301, 2854, 
              7303, 5523}, {6575, 4551, 4550, 4474, 2224}, {7417, 5559, 5558, 
              7418, 2936}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5945235645120444, 0.5768961275623107, 0.9101986682488525], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVV1IlFEQnVar1dXV3XVdv3V3rYcCH3upoEAhhCRJqF4jjCA1LSnB6CWt
LKWgkqKIAvuxogwtgkqCohSVItReCsmKLKE/SoPKCDqHc6GHYe83d37OzJw7
u3DzjnXbfWZ2BpIG6U6aVXhmZ/GRHTPLgvTPM/Pm45wu/VLcH0yYbfLLdmO+
9PXwvRcyezHXbDZi1g79MPQTsOvOMmvA7wF3voBc1/JlR5urUfnT1x8025Zj
9gx5C5B/TUTnIwGzKdx/hP3PHNkwzh9gmYHswv0y4FmRErb0ArO5kFpgf41c
E5BOxBmKqE5izkSu0lzlpJTkKmZfGHmQu8EnHW2CacL+zuGnjv7UX0TdK5F3
APjGiswGQ8Jah/v6oHrXCownUOd6n+xCwHs4TT25naH+HAL+VsgpnKthX5OQ
3TCwXImqP2/w/R3S7FOP/7gePgfmXUnpHyD/IHA8Ndm/hfTi/Bv209Dfn2O2
HTHrISH0pxd+7/LMXgLnmYAwNwPTDGyX4K4LusfkAqQM9vvw2xFWLbUx1c+a
vgWUj7ka4bczqe9AUL3KAs6MoL557oybtRVJV55pdgm28XTVx1mxlkdR9XTK
cYNcoO9knnAT8xfcf/XU76/o3cA8xRmGftpxg/Eqss3eI2YY2MdS+iaPGJcz
fZSuWid9wut3OIlrdaZi8v6ys5nNV9+JsypbddCe3ChxvKoFxh5gHYfPfpwf
56lPQ1H507cYumBS3Oasxl0PR4F5xNXb72pi3lLY74mJV5UZwk09uVTiuLob
900xvV3OlPNkrz55elfsYwdyHof8xf1i1PEhpTqeLEC/U4rTTh4tUD+IhffE
Q74fjWru5AtzEE8qLJ6RY61xffPMeBfjikken/aLt6yLM2NtW4CrEr4nUXsN
bENh1bcK/OoMSc8YlRHFoQ9nzZluK0SvC8Ujvt07Ub1fzoo52J+WDNXAPowg
jhXp7VWiJ2shO/3iSlNAb4cxficU5zPuv3jaZ+eQp9pTv7kPuBeoDyJvTkyY
opjRr4Ri9QFvHfS58xW/MaCYD1FrfUi5NuD3Vkhz5zwWBTRXYhyNC2e54yDr
fYb4R5OyIW7uPPYnB/ofCeXIDUjPd0rsZan/+5jYuXNuQH8d0uPX3urx9Na6
XH7O7gLyn4+LQ5zJsaTmwn3GXcD4vGtxHCO3Rh1vWd8DVyN5zd3GGg/kq2ae
twLv3YjwkrN7U+Jtp5st30ghcsaT+h8gx4mfb7Yas6gp1PskXvaNmNkn6jj3
V+DvR0//G9xzxQXCSU6Rc+wb/dgP+pLPdW4HpuC33NOuroJ9myfuVgDvzaT4
9A9LlxKY
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlElI1VEUxk9m+fTps6fP93qTzxZNu9rUok0QQtEg1KpF4yZ9NpgKRdBE
aiNNEEUUVDRjWdGiaVEOqCSRrYrIkiZokCaoiKDf6XNx+N//vd855zvfOfeO
W7lu4docM1uM5WK/U2Z/sIagWR0HbRVmH+Nmj0ab1RabFcbM+sJmQwGzTva6
M2a9GeH78F8SNTtUYrYI3xMjzZLgJ/C/osjsMfjqhFkNVpVv9gbMUvAbwcTz
hJ8fMSuKKfZ5Yv4pJT7/T0fx5TyLb23SbE6B2Vv8L5SZdXBemSdMIm32gL33
rPfh/w1u37FzrO+NMGullvtRcW2jhingq4k3m3hnwdRQezmYveQK8x/iPJlW
rA/s1SRVg3N3n8MlqsG5e83LWVfhX0/sI+Q4hG83NYzH/6/HiEtT1/In1gvX
Hmxbjur5DfZimWrxmuaC/4QNEK8OmzwWPYl5nFjPaNh+sNUpce3CZxL6tQ/X
n8V/PdhO9r5y/gpbQKxp2LKAOP7Ct3NYjw2c36aey1Fp7f13LXPHKpbH3ITW
PaWqt5D8fdTyEAvxXxAy21putqVcZ97TIXKdQa9t9Hs1563EvlOi2J6jn7OW
hGq7CaYS383Y60LV7GfNCfXOMc71Sly9c84zwM4KS2uv6QB878IvnCc9DlD/
4bS0b4dTG75X4+ql1+O9cY09tvfIZ3cVmowJqp+74uLsXI+B+c5ZWURa+czf
JveKsM68Z6fhOpEY2/NV8zv4nWIvP6Q7sIdZ3J2RVkFsV0Z7vnbM6phmyOsZ
Qw3V5H+ckDZNAfV+ZkT6+gzcIPcO9r6wvmbS/nNGWjtmgFgl8JlH7lPcqQaw
jWnNgs9cR0Qz4v8eowo9TiaVy3tahX9tTFyc003wWfDPwQ5iB1k/ikhbv9O3
8J8+PF+uoWvhb4DffdfkOvg3Ec2u77WgZ3NKWrvPmphm2nvnPfyR0kz6vDQG
NRuXqPmMaUampvVm+Fvhd9xn/WVUtfrM16fFeXB4/otjunN+17rAzIVvir3+
0Xrj2sDuhP/RgGrMJvVG+Nvgb47nflGhu+sc/C0MxRTL38Rv8C2OiOv/+SB+
NqZ6/E15Uq43zPN7Tzz3vFJp4RxayN0UVS+dwz9V1Nxk
              "],
              VertexColors->None], 
             PolygonBox[{{6149, 5826, 5825, 5160, 1898}, {6084, 5766, 5765, 
              5113, 1861}, {5826, 6572, 2223, 6571, 5825}, {5766, 6534, 2199, 
              6533, 5765}, {5714, 5998, 1809, 5051, 5713}, {5995, 5714, 5713, 
              5050, 1808}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.614356947020424, 0.6087628545558949, 0.9106508082823905], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNVFlsjGEUvcp0pp0uZqazz7T1gIgXiYTgRUJDWkNTXsQSabXTEhXeSLQE
JcReS9KoJZag1Fr1IvbWS9t4EklJGiGxvAhP1nOcL+Hhy//Nnfude+69595x
tetrmnPM7BjOaJx41CyBM5hrlk2ZHQqaLcX9TtisqMTsFZwWhMxS8PHjfn2M
2TWcYpyV8FkcMDvpke+FiPxPJcw6cVryzHrwNgP7HthHfGbbfXr7BrYnJcLa
D5x1wM+AWGPSbEoaWKPMfLnixbhz4dsZ1dsFwP3uNzsHn6642SNgHYHfgNds
pkcch8rMCoGzE34thWbVeHMe/iOIsTBP+Mu8ercJtgcRYfE3OY04nh3Ibwni
7kKtlsMnG1V9svCdmFY+a5Dri1LFKIN9Gu4Z2HtjqhHr0zFavGtyxP2H489Y
5EtumTz50L4DsdqAtRGx7iPmFfSlHm9v5cqPPs2w18CvGrYj5WY/EbcVuT5F
rXoQuxu4E8BxEu7bkOt04PWG1IvLeHcPmLtz1fMVEeV1DD6XIrI/Rw37cCqL
zOagDrOAf74Ad+DXxVSfenC4DUy/V7FZa8YdRv5X4Lse33r0dGxI3NnbNUnV
fwPuj50GWoHXElMPWa93yNWDcwlxt5QKa8Tlz9zfI26gRD2sBd7kkP7fg1wW
paSlUuTSlFIvzrCHEdVzbVI9Y7+qU+oZfZLgU+Q0Q77fIor3CT08GFTvyPHq
fzwr88WVfVgbVS+Gy8WbnPlfRb641SHu6qR+N+B7wGFWIX5fSBqhNunLHMmP
9WQd2P/TUWlgE7Cf4rw0vTuU1lvaBstkJzZ5Ez8GHvECzdHvIv0eAv7sEumD
2uDhf7R/BJ/ucvWQOS1yGuM8J7zyYU8qXF84f56YZofz1+30zHmlVqiTNs57
VPUfA9+HYc3ddsS/GdC8fYZtfFD41A1zIn/ulaNx7ZZ+5Ls5Kh7t4Hi4XPF4
WCvGnQqcLTFx+QWOv3G2FmrfUHPUG/dZVUg1uYvvvrDmIoj4ryPCeQv/G2nV
7lNcfSIn1uZDXPXJUDvgv8GnmWt3fDgLW13vlgGvKqh9xlknL+JzD3DeiUnu
rc7/ODBP4JwFZgN0mk3Krxh8vqRUB+6GNqdn1qwuqrpRb9QP56sxpd3KWZgR
0C7jHmsEZlNSWruD3BvwtsCrPX3X7YeE08SQ28PsB3tBvVfk/5vPdy4WZ2av
X9rjXvma0g45jPodSCtX6tnj/Ff5NHPk0xXUjqO2n+P7HRzme1VfxmX8Ptj7
cebB/hjfi2H5c8ePdTPblFBuzIt6/ZsTfAZQt4Np7dxifJ8FpT1qMJHW/Q9V
D/58
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtIlGEUhk9ecmomp7n+8884aquQlkEX27RIinRy0DZhQejoaKVSO4Nu
WBhGZEq1iK5kUVnaxXIZ2EVbmLSKoAILgsoWUZtu9pzOLA6c7zvfec97znn/
f0ljZ11HnojUYwVYU0wkg41w8GNxR+RrRMSbL/JivsiaUpGVrkgv50GfvT2A
TRVZTntA5FFUZIC3lz3klIlMY0e8Iq/EsIaTIjGfYbaViLRiKUikeJ8ifyv5
W8jfzbmKerPU68S/Sc5bYu+w29QanCcyBNZECJ7UjxIv5lwRFjnmtZqbwTvM
+z3gdYOxArzV3PWI1dtGrMa1WlrzOf0OBUWa4bMBzOvEn3G3Hv8o8WXUak6I
3MOf4U01b13HZuPBHuOPO5avNePwSSSNSxd3I/gf4Of6bAbHmUdlqc1SZxoj
9zN83ngM75P65da73v2k/o2ocVFO35mdH7zX+XbXB34ATleYzW9qriK/1LVe
t2NZ8B+ErJb2u5fzWvIPFdkMs3GRFqxmATlYLfjpktxu4H+f3DT10+R6i2w3
X7Cz+bajP/TyFzu4CF1g58Hqcgx7P3hN+Bfi5k/DcQJtjEaM/8ZikVNgnXat
F73roVbWtd3pPL/BZbLM3mq8n9kMlFutYTRxgv6nwtb7L2yc/DOuaVE1+SNq
PWpvusM5uD5Ev4V5xnmM/rIlNhudUZZdt2B1eTaTNvxW7CrcN3F+SX4waFpM
c15K/XVh+3aU8wx+Kqcv1afupi1uubqjXWA1YVUL6Qkb5303GDMe03ggbD0o
9wZmWB0yjag29PvZB/cnYeu9slDkbsA0/782Vki8IGbaU4xasDKO7U532I7f
4Ziv+9XdjlLDl4u3EOsNWkzvFsPNnzQ81XQfsYao8dEZ1QesJ+3lHHwqqP0e
fnPFpjHVgs5EZ6GauEV8ecS46z9E347k/g+aswOsnQmbrc78IvmXHNOCfmNP
qf/Rsdkop2tgTTr27Wr8JFj9SdOCavQOfk3IZqffoP6LZiOmRa2pWhiLGXfV
hP4b9B+jfHWHGbg0Juzb1539A1JftfU=
              "],
              VertexColors->None], 
             PolygonBox[{{5770, 6090, 1864, 5115, 5769}, {6552, 4544, 4543, 
              4461, 2210}, {4513, 5989, 1803, 5988, 4512}, {7738, 4513, 4512, 
              4419, 3237}, {6088, 5770, 5769, 5114, 1863}, {4544, 8136, 3658, 
              8135, 4543}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.6341903295288036, 0.640629581549479, 0.9111029483159286], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1lVuI1VUUxleOc2xmrPHcr3O0Jx/qoaYxqIcu3o4VjpLEBI00zeXYQyQh
CEES6ZQpTER3sJqmF5Uyg7ImMpMskxExeimohyxIKJIorQez+n18u4fN/p+9
1/rWt9b+1jpXjW6+66F5EbGH1cEabETsLEYc7Yxo1yM2sVrd7FXOSxHvYDTQ
G7GMtQaboYzvfwTkxmzEDc2INxZGnFoQcZ41NT/i03LEXnx3YtvN2bauiK85
3w/mJ5w/m3E8+T0K/taeiDP8/pbvf8C7xGrxu4MYPfjNwHMcjNWcfXRZRD/7
qk7HOrEYrKz9bylELK/bR/5rkv088NdlHEt5XFzgO+G1ks298HomF3EPdmNg
PFh33NlKxAesv/FZUSNH7E7C67dO12CONUO+r7Ouw2Yjtfy+4HzFQ7bC+ZXz
g33UI+M77fO5+y5PrSuu8wh5zLKOYP8IeK8R7xtsvyTH06wNVxpvZbcxz2Jz
POd6trC9I+eaTXA+A85ncCwQ8wixf1K+3fadS/yPZZyDeD+cMYcd5Le96t9T
+N6eMNdz/h5cd3W4dqtS3a4H+0DFbz+Gls6k3Fen9xPPOfjsxn9LxnXdkPX5
WuL8UHCO0kl5ob8PEPftPv9+AOxp/K/l/q2csYRzDLxswZoRR+WveL9zPlx0
bcVButH5ODhjrA/B+YX7P7DrAmcCPlfnXZtJMHawJvD5i1z+bFib4qj3ELfd
xGqXrach9idS79zP9y7uBjnPgndbzW+1FN/DxDoLz6+wvZR17Q8VrV3pVpwm
U+/ojfVWsl+E77mGc3yp6pz1rdr3p/pLy+o9nendHi/57T7Hd1vFdy9y/gJr
5nL3ystVx70VjtcUjSW93NS0ZoTxSjnhNG0nmzb7ppo1s5396T7XV7FPJf5H
i9accthLHW4uue+q4N2Z87toNkxWPR+kwaGkw/Xkuq7hfv0Y+6eSZlSb2YQ/
m/N8kW8v+MMV61m8LpSMM8rZ8aZ9fobPyGK/tfqgo2JuI+JT8nt9wff+kvtu
mH0ttXg+rNkTSW8Daf5pXtThPlB1L6hmU0XX7Rx7rezeHkVX43X3m2qwL+86
TIN9X9n4b1Knd/PWvrAvpnfUvPm36ZnzvuYD9lekOfr//JDuNLulPc0b9Z7O
pbsVSXvT7GNl4z9WcQ9IaxPYtlm91HkZeQyWrGfVQnHE7VDePa8cVSfpW/ea
NU/2WJ/STrtibYhrK83UlfBZSm57etxL0oi0qtl2uOw662360wxX/LtTT2lG
qo7KU7U7mWas6iu+4rM86zqKp3prX9V8NCdeTbNCb64+17tvrLj31HfPLSGH
JX5P9YN0JD76X7rQcH7yF3fNw174LOpzzqdr/o/RXNf/RFea4f8BllsJOg==

              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlFlo1FcUxo9JTExGk8xMZv4zmUyiD0WoL9YN+hpFoSVVqmKJcSExiU+K
CIWCVtBgqjRV1FbBpcYHl7og1CXiguBWWoq+KbQvIiiIFqm2PtSlv48vDwfu
uWc/33fvpO61n6+piIhupArpK0T0IpfHREwbG7GR8/RcxKuaiN+RK0nEVWR9
dcQ1/C9mIzryEZ+h12OvxH9yU8STyojHxM/mfAj/qdgGkH9aIhq5+wP7LmL6
S9RsjnhEE/PqsJOrkZy/VvjudmvErVb3MkL8cnoZocYbznO525EhvuTYJeQ7
U6bfYsQDhknGU5vcPyJ1Nb77mdyny7bp7hX1Gkbr/YZswXczsqqWPSDt5E5n
bUsRX0/v5+nhIbU7ie/ivAy5j60Kn7RmTzybZvyE/u42O7YSe0/BO9EuLiGL
8N2fjrhOvnfEHET/sGC7dtaD/gE5Nta630H6HebuI85nyX8B31tNxua/Gsd2
F7wr5ejnvLrgXPLfNTFiDzKzIWIGMpld9Bdt25+K2Eat1YmxFKbfYR8qu3ft
5C29prDfwPaCnt+DzTtk3lhjco5d9WK/ib6NnKvYXw8yDFfmgNE39P8an/Ya
c6gP3/NZ+86vNvcGmr17cbCDeh2jeO4hxxR8u0vONReZgW1m0bHKIa7NafIs
mvl7bEepOYh9eFxEG/oCOFiBfQP+nzLvUuxfVBu/k+jXEve3jrvj1FuSGB/5
9xFbKrpWAzme5cxBca+22rvpzFvXjhamzVlxVTs4TuyJovmhN/KQXk+Wja04
NJ/82zOeRz3+hf4a+TJlPgk7YS6sheHdNvBP+60trHfvpzLerWb4ltq9Le5V
mHQRu6xg7MT5AXpZmfZb1o7eguXuieaGMF1MvmNZz66ezpHvcWLu6w38wvlO
4njVlG0k41zy0VvuLRkr1fu3xTkUq5n68e/KG9sJ+B9A/zNrPmr/K9BXJuai
fPRWPkaOjDcmg+Q6mvXbkS7uambNKg7vzBgjYSOMxQW9YfWmHvV3Xci6tmoM
5fynKVY5TmD/oWjuaMZZreaQ9qEe9Ne0p92bODoBLO+1GQvhe5pdf40MVfkP
Ffe35D2b8LlNfw3EbE2ZE/vIt7dorLUjYfsVcmacMX5J7N95c1UzNTNvMbFe
h95Ydg+qrZyHE3Na9fRniJtPR/chjupveZ5zrP6Yn6h9PedaqqneNxX8V2mG
/wEvjNyd
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.6540237120371832, 0.6724963085430632, 0.9115550883494666], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1lF1o1XUYxx+33Dnb2bJzjudl55xNu6nOWlSUaBCGYpDbghIiCOfmyU2D
yG666so0nZsugiwMvaysWOAsC8tIYkJ00U1URC96YVlQ9DojyPp8/f66ePj/
fs//efk+3+d5fte2dm56rC0iDiHtyO5yxKVixFkut/VGvNEXMZqNWEQ30Wv9
cCOiwXkY/SXOPyOP5yJexmZfKeJJbC5UIk4T69mOiHX1iLuQ67qw4f9LyOdL
ibUi4qfkuwn7XYWIDdiXyFlEpnLGMlV23hs7I25CDizBrhZxd8338WrE5XzE
QfQnuN+MnOL8OjHOLyded8Q8ujNX2UY5f015b0n20q8WfnAuthnrETB+xHmC
OJPIF9g3sf2M7yyxtqJr9fm+je8cuZ5Bf4Za/iLWhgyxM/aT/vLVEYVu45/v
8HkG/R2qMeVdl0ncUO+7yV++53oivkU60Q902VY1bqOOafj8EJ8L/RFHa7YZ
Rbe55Bp2wM/2quPdutT/p/G9H5x3ls15nX5ONtzTp4ixu2a+DufMqXL1YbMG
nkey9lPP5KsaFFcxPyDna9hdJMdgp3umel9FdwyZw3eCPAewG2nzLHzacM2q
YybVMoLuXmQJ+T8By1DBOHYRYwwMh4izEf09NfdwFNt58UjerZxbSA++U/gN
NZxrFb63I1uy7vOaonnUfFyxbfcszP8/D+R5EdnLXgxm3PsZ9PvxLeDb7HIf
B1ONmvG1ac5V9w1d5k0xB9K5iN903fc85/V12wlHM9koz0DK1UyxpM/B9z95
8zzOdw+1PJ81X9pR4Vdt6qEwv5UxPvVnfd77LLvhbsfXDH9HzDfhfCf5H8Xm
feRjfA8yz7PI3znv2d66d039007L5p2C/eX7UNn4hO169qCdeXuE/E/j90PJ
/KjnOyruheZ9deK/CJ6TGeOcI+eJ1MdlfP8ou4YWb8XDyHl24HDNs35lF8C0
tsdcaUf0Xzu7wP+z/f6nmhZWGHMLXANV22s3jnekHUc3iZwCx0LB75fq1Hz/
22E+H0R3DEyBzXG+Le6dnMf5jiHZjN/FVf3uwSvYPFCxv+KKd3G+nRq/LHqm
TqadVO06D3X7rrdBdYu3H0vmV9z+wiy/XTDn2k/FVUzNrHol3WlyvlfxW6I3
41ziRO/xc72uS2+K8osHzZ/2XDM2Cgebq54dvalfl70Le2rmSPnE39GKfcc4
b6kauzAX05smHLPpvdUO3VcxhxuL7p96N7Ey4pvlfhfzcFJA9uXMzQsV49Ns
LjZcr3rdTHP7FTZHKn4fNVP7C54rvelP5Nzra4j9Z8P336jj97L7rdziRXx+
T86Lfeb9P3ki+aU=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtsVHUUxk+Lfc1QceZ2Ou10LsWNCpagiUTYFKhEaakETHBFoTO0BRIj
blyxgir2IRAT0KgQdsojJVgCGoEokJIYFybGqAsVWImYGEWsiYni7+NjcZJz
73l95zvn/B+u7nzh5dqIGEIeQEZLEa8hnzZEXOLHutaIpe0Rmxsj9mIfRt+N
HOL76pyIj/P4FSNW47+TRM8R28u/6aaIJcgnuYhB5G38v6uLaEkjPi9E3CR2
LBsxW474G3kl6/ij5NreFnGefAeo/xu2h1oiXs3aZxX5c0nEY5mIx8n/LN8J
34syrpcnf13q/G/gfwK9Qo5m9Ja5EX3oayXg6afe1fkRVepN1kR0N0es7KAG
8gj5vsC+gtpfUeMMud8lx1F66UEGiL9CP3vgYrTVXIiTO+jP4N+F/3Q9GOh1
HfVqiN2Fzzv0twL7Quxd9LiXWq8jb1J/Mf8+I/44OafI9SV8r8d/g/itN//j
cFvhu6nB+E/R3xQyNseYZjrB0mmuFP9X2Tmli8MMuVOkv9F4puhvW2os4rsH
bKtLxiJ+ny87Rr7qYSl8PZ0zFnH4Nf0NwN9Z4ieJH6PWRfCtajAnVeIHysan
GVzA/ig1d2B/i55WUmtmvrlfRL1jzLK71f0GPkewV+jnOvYmclwndoCah+s8
z5/4Pg7+PPp/D3r2hdS7pR04CPaXcu5f9WrBOpO3Lgy/kGt4gWM1gz+p/WvB
s/8InxGwD5fNtfrdTPzP5O+j3jniR9CHUnP3PTX/wPf3sndZM7iJ7T3m+2Sd
Y5R7kH6uNbvGU+DraL+fu9G3s4+Yf7K+Ie2Sdl6z0E7tx7an5NuUz5qSb0C7
/wSypegd1W42gu9C0Teo29NMbsDX1vv3UABPhX4GkX30vp+co2D5oGB/3eyx
ds9MfGknzzCfk6ljNdOugmeo2a3lX0+Hb0a3ohvVrlfw/zbrnX8fPMOpuVoC
nh/J3VsydmE4UvQbID7v3ST2D6l5t947oV2oIjeavRNbO32z4lL/xO0tME3U
mONt9PZD4lp6UyaxnU7MjTiqpuZAvQvjbvh/sejdU70q+lje89fNbSz6TdT+
C9NImzlQ73ozJ8CynPyztX6T/s35RnQbwrQJ/Xard0s7rFlop7XLmsllzSvv
/dGbuoxc4+RcmHHOTdSbl/ht0c315f3mav56o6axDZXdm+bzDfps4lvUTK9R
a0ubdzeZ69vRP+m6oXziN0D7ppr9Ob+hwie83R3+J11vpHa7N/Ftasf/B5/N
2GE=
              "],
              VertexColors->None], 
             PolygonBox[{{5567, 6146, 1897, 6147, 5566}, {5708, 5041, 1801, 
              5985, 5707}, {5974, 5510, 5509, 5975, 1792}, {4447, 5762, 5761, 
              6530, 2197}, {5510, 7244, 2817, 7246, 5509}, {5112, 5800, 5799, 
              6082, 1860}, {5800, 6531, 2198, 6532, 5799}, {7443, 5567, 5566, 
              7444, 2952}, {5040, 5708, 5707, 5984, 1800}, {5762, 6043, 1839, 
              6044, 5761}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.6738570945455628, 0.7043630355366474, 0.9120072283830046], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1lVlo3FUUxk+nzZ4GmslkssxkFNSiD1oEUV9MExeapdJWEMQ0k0w2Ffen
NAht0iapG7iBS3Cjphu1tfGh6Ev0ofqiFcVaW9BWLLhSK7WSBwV/X7+/D4e5
/3vPPfc73/nOmctLj2x6OBURh7Hl2MDKiC1VEceWRRyviTiFPbciotgUcboh
4hvW9+Uj7s/7bH99xIfZiGGCDOb4Zv9F4gy2RvRjHdURk80RI9hUZcTexoil
dMSnPLbIvY+wWysi8pxXY+34zJR7Tz7beGMKGyf+SX5PYAfZvwoslXmfXaiN
OA/udt4axpZY/wCGRwsRjxV89kSVc1NefeDKg/uDZc61mOz/2BZxts3fv3Pv
V+yBuohpfO8EZw8YFjMR6bzxLJZFvM3bE+ApEaePdSrlHOZYj4Mhg29Z3nkM
Eu/1FmO5g9912Of4X0z5vu72roq4pdUcnofPP7HHubuWvSHurya/3WDY3mx+
HsT/JdZv8eab8HZtleu1lb3nsW/xeQjOh5rN2xTrIncOEf8T4h3FushxqMoY
xEmpYKzCqVrofcXZVOc8TnB3HlyjSU1/Ir9+9PEFd6+Aq7kW8/lbwTyKwy/Z
P4DPVnLMZu0nn33c/T7R1Rx8Vi83D1uqXRvFPFVpLPKfJd4MVkHMTnz+qnWt
bySnG9pcD+lUcRVTuZ0pM/530N3anOslzj9LOV/56U29Vdtg7uQvvF/VG/Pd
YJ5kfXvKHB+pMM/i7GISZxD/Ut5n26nRjow18hq1+5n9CfB2g2FHwXmm09aB
NPAs2M9dZi2rD9bnrCP11t5691dFud8SVmHWt9bqMcUQTulffaDc1IvSk7R0
Mm0dHEz41fvK9zbe72yxv3g+U2t/6Uyca0/6rU80LI1d0hf+fZyt53uWWJvB
W8Jq8LmOXNa0uQeOJ/PjmRWOfTqJfwg8+wv+3oDvRmyh1vW8kNRU2tKetHoU
DlJN7rv+nHtS/dgO9pezjv8eMZ/OGE8PeU8XrClxI9yXMHO3mOBUv+pcfKi/
Z5OY3fC0LufcdrJXyno9DAejec8g6U8aFY/jYN+X5BI19pXGFsDyR876G8wb
q3CqH8SFuB1j/90Gz7bd1H0PdoD9Ndx7lXzO8cZItWssTvSe3lXtJom3rcW9
Kz/pRfq8nhgdq1yfrpy1KB2+kbEOpAHpeqDVnHQR47tGc3QN58VW95b41rwT
5/dmzKn4Ul9OJBi64OYV7ixx9xgcXNngOdHR6pm1OsE+kmj0ZnyfYn+M78Os
extdr43E2YD9Q5zRZI5Lk5rlI0nu/893raeJMaOZqNhwOJe1TtQT2lOfaE/z
TvsD+BTz1uNZ9leC8x76tCvt/pxIcHYm8f9m/2u424z/C/A53+hZqPkxn3Z9
R+jXXzJ+U7WXpqRtzWb9z2k+S8vtScxh7i5wtw6/9zPWh7Qxk/F8Ue168OnO
WUfKX3c1W9Sj9Wnv9eZ8R/7iZizhdqDJM0jz52PiHak35ifJ76a0ed3F712c
/Vvu/8CdTY6vXlOumjOaN7tyXqt3FU+6lcbVt8pVs+rqtPNTPspdvaN6qg9V
0/8A/qExaw==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlUtonGUUho9NNTGZDMz8mUwmc9NFFV1oEaRxY6haSFILNa7ENJNMJqmK
91VShCalbWK9oBaUNN41CqUqrYvSjVVodaMVQa0GLwVbvKFWvJCFis/LO4sD
3/d/5/Ke91z+S+v3Dd+7JiLuQdYiR7MR50sRD3ZE7L0oYojzAt+2cA6+rSYR
7+X99kFLxHJ3xHHuN7b6253liK/49gTOVuSjGLGpN+Ivgky0I9WIOvLQxRGX
c/8d/78gX7Q45tOZiEH8reJrmpBZ/GWQ05xneX+H97uRF9sizlyIP96ez0W8
if1p3ndzfhmM/2B/nPfHKxGPIX+kIv5EPiXWMjoN8GxD/5uuiMlLIlrR/xzM
8+Q6AP55/G1Gp47+OLJf6aM/Sy47kTr4J5DRnohaj20/viDiHNhP4mMa29vg
pAa+O8rm4jNkC75uRjbx3k8Odd7GkUexP4r+f+Q21uPzVfgfINYg8gL4PsJm
K/4PZJybclyL7WXksKvD/LRxP9GMr2+j8H8lfNwA1xuRJwsRTxVcK3E+yfnv
xFzsAs829EeQY+TS326uXsu51uJsAv1GwXjF+a3gmc06n3/xWSO30ZK5E19f
dzsn5aIclok1WDL+Hdhv5G2o2/01Qvw5fA3n7Us+r0D/fvrlfKdreDjjmqvW
b1GT18FyIuP+OIT9I2Ddl3N81XAA+wPwN0P8GfLp4+12ZIn4I/irFd0j6g1x
9AyxVxLnphzr3A8n9iWMi5w7Cq7dr+jPgv1g1tyII3F/Nu/aqwZ9YCs39dXP
R8R1ydykkTHijxddm51r3FvCKGzqsSlst4LvSMr1WaA35nrde6rRLRVzIi6k
0y9fRc/WSpt7+ZqCuVUO87xtAMMqfDbQeaDqmqvW4ngJ38/22vYU/q8v2qfu
moESvjaXvAtUP+n+jI+70tapgWWuYKzC9IrmreS7OHoO/XNgHuv0DjjLOZ93
rFqna/VdxWfVTL21vmSu1Q+qbaNk7lTjn4g9g/6ZlDG8m3dNVAvtpAF0X02M
VfU7iP5Ik/9vsUnopVzZs6N6/cD5+7JneUe7e2E79y873BPXVsyB/L2E/fqK
d5Zqu5TybIyVPevqn0XNb2Jfyke77n0wDKY9f/vgNkk829vRmcJ2smzu1OML
vF+X+G0S+ZH89xS9S7Wz9lbd4+rt1rR78+qKsajn1nV5Ryi/4bR3x4aMsWsG
tas/ybr3tLN/A1+qy9i0X9UrwqDY6pk9xBviW3uLZ+gNdBvNeRRH+4n/ds62
2tGq9WJz/lTzLLYP428Kfx8S82TVO1364qQvcY+qN6Wjf8dNvdbVP0Szfqrq
XDTzuznPV527MKkXpiuurXrif8R6/TM=
              "],
              VertexColors->None], 
             PolygonBox[{{5822, 6547, 2208, 6548, 5821}, {5693, 5027, 1789, 
              5970, 5692}, {5133, 5822, 5821, 6117, 1881}, {5026, 5693, 5692, 
              5969, 1788}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.6936904770539424, 0.7362297625302315, 0.9124593684165427], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFlVtsVFUUhje0M9N2poXOtDN0OnMGHzQRlUtN1BgVtDV0iokoakyEFsUa
DVLoG0ikbSzggxpFUYuIQi0QfNDGeL/AA08mGKvyYCTxAnhDRSAxMRj1+/1P
4sPO2Wedtdf617/+tc8F96y9dWB6COEDVg3r+XQID9WHcLQ2hMfKIZxoC+HL
uhCe5vlRnj1OayohDLCW4NedC2EL+zrsPew3s082hfAV+0H8X8O+qzWEVcSa
SoQw1hzCPnw2k2sbMZ9ivUT8fxpC+JPVBZilUQj3tfns3JTPaZ9ocuxtnN2U
dw7Zm7IhnOF997QQUvh34FMH/gy1XM5+S43rOBnX0guGt1kfEqe3FEJfyb4r
wXgvPiP4NGdCGEqZhwls6/F/Ffsh3heB82twbqT+I0n7TPL9lshY32g1X+Lq
FTB2E/9vzvyE7UfWMeIUCmCKfP538s6khj3gP4H9MPsnsb/L/iznF4LjC9bt
jc5/P/59rJfBP4t4bawqMR/m+ySYXmfdCB9HWkI4iP8O4owVzGV/0f1T734F
03vgHahxfMWWvzCpnynWOzOIj218mjF+Ghmnvtc22UdY70y5XtkScY/UT9Wt
XquGdUlzfo79i+yvTFhTjS0+ewM8bcpaA9vB+VzR8bvBPhTbh3ku470zaZ1u
jLX6BLj62o31bp4rWZ0NjqfzOqtaqvXu3ZvYPyOfxF+EvyvoXw8cflvwN9nP
wf37vC8Cbz0+C/C5Dp9uMG/F5yYwnKYPZ1i3ZdwfaUM8Sivi5ST455eMW5j/
473ROHRmV8yDdLcq1p70IZv6m4Gb05wfTLs3ii2d67s4lI/0Uld2Ps3SM3Xm
+a2s61EtB3lO5G0/gH04735MsT+Wd6yuhHOod4c1p5oZbN9MN071XbjFo/Sw
Gh09WHE9Z8nfge7vyDi/8Crv0VhP6qNia/4U/1niz4+sv2vJvweuN9R4Pn4r
mwPdGZorzdQPxF6cc5wdRd870uBF8FNbNs+ae82/5r1acA7FH027ju+x92Lf
W7bfioLftRcfhwrmRPEubHFMnT9FT9fU+G5c3WTdXUpPLmP9wrflaG1Fu3uv
HsummbpmprmS3jZQ688V30s7wf9C0XE+qZhr4ZtDvEtKzrec54GK7680nIzl
HHOY/bLIHCvnYMV5vwP78YLn8Cq4Giq6nyM8H53l+VZvpDlpR5jUS2lJvZVd
/dUMqAb14o/Z4Cv6TllA3sV5c6u4iqd+7oerfWXfl+rhaKvPzgP/3JLvGGlW
+ZTrrlbj64p1pbtKWprD2d72/216Cs/nJdeseh+PPOeacWlhe3w/jOesIeXV
f2BG1rnUL93h4rwejA2sR9LmZjjmZ6Ls2lRXR5tnW3OdlEZYI/jv1H2AvT/p
2F3xv0Y903fp5FTFfVVPNcP98WxWwT+R8//tZvqzm/35pDnXPaheLGn0vSAe
lFt3ue7MDPnTZetX36uxz/rIelJd0qPq1F61aq+Z+4vv5yP/E3Qf617WvGjW
Hyh73tdWPMOa3608Ryv2nQDzOOvqtLlWL5RXOhBW7aVr6Vvf9U8YKbtfU+w/
nm0/4UnF/Iu/njbzsp88C0v+P8yLrG9pW32fjGvszlp/etc9Lr6Eo4c41zfH
/xRidMZx1hHz4px1djzyHaF6/wUNjj66
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlW1o1XUUx4+77t67691wu+5e/+7+700ooSy1BUUP1GpG7i6isiLI3CqM
opzunT2u5dReFJVEuUrL2tQssAcoMsJeBxqzejEIinRF+VQKQRjV58vXFwfu
73+ev+d7zl14/7rbh5oiYhiZjcxJI77sjJhpjhibE3G4GvEt8ko+QobH6xHH
kMdbIh5pi5hA93Q5Yl8m4irsj9YijiB53gezEd/wu60jYuesiD58PmuP+LwS
cV2OPOQ7eV7EC9gkJL9mbsTuJGIX8kvGNTSw/433D+R/G/uryXUxOY8XInqJ
f3BexPPUPI3+ADHOEmsF+T4i11OtxEO3aH7ECPnaixHP8v6nZt2H2Pxed0/q
JY//GL8bpYgs72ZkMbn+wn8t9Rwj53voN9Rs243+MLk+QH+G+Duo540FEefz
7YkWY7QI/6XICXyHiPEk2F5Udaz99LeMWEtr1un9IL1OVo2lMD1NLSP0vD5r
zPr4PR+bPvq9mW8D9DOIjJ7D52Pif4JMZzyvRmIMpVuD/Sxsi6lnK4w30U8/
+q3hni9vN8bCtkHMV5nVmx32XUz+5ehuLBtrzfTCkmeo2f1Ivkuq7ln9aEYv
k38kMX/eQl4Dn23I9/jnsEnQjZNzU7in28Di1pp7nwKPAWKtQi6jtiI176Xe
feTcnPG3FegaZfv/S75x6t3ead8lOc+iUjEXNZO8+ABnZvB/iW8neV+K/RXN
5phq25OaK6pR2NyDfnmzMTpVNcbCdph61pJ7umRshNHOkmek2ZxFRlNzTlx7
B7y2SV/xrMWpQuqZaBYbNRNib6TH2W3O96L4gLyfd80r8X2XGI9lPKMx2au+
Jn97hvpHke9yrnkD9X9Rdq+K8WjdO6ldPMEO7Cfe3g7zqwefr3gfqHg39U1Y
igPqT5gq1s+87845Zi51japtlPpbwfZoxTpxcKALvlPvT03ecd2WbGpb7fcE
8T7tMJ6yF5Y7zs1POcX9MyXXpnr+xPeP1FjewYyOVBxDvqrpa7i4hv776a+/
1dwrzvOsxMERcl1ZNlbCbDN4bKl713WzdvN7D/J3wZyS7YLEuyCfm3gPVs1F
cfCBxBzQ7LWDk/zempjrwlC3o4X3tXnP+zneKTXc2eoZ6fcqMOop+FsPsSdK
5rJquJ75dSfmk+oZR7es6tlrx4aodV3ds1TPS6q+MeKXbk4v7xuq7lXxhrFd
X/euKqe404f+v4JttlDPLRVzVzlXwpXV7e5F+O0C+8nUvedyvn1T+GzP+gYO
dvkm6hb2EvNhbB9KfX9k002+U6lnd1fRu6ybqluqmIeY3VTNs9MN126eTm2r
HZ1JvCPaDd38YfB+veJd182YLBlD9aqeLmD2h+rmmji6mvru63Jt4qT+G+6t
uBf9R2j3fu30bdIO/g+dTAKs
              "],
              VertexColors->None], 
             PolygonBox[{{6075, 4777, 4776, 6074, 1856}, {7264, 4742, 4741, 
              7262, 2828}, {4742, 5967, 1787, 5966, 4741}, {4777, 7332, 2874, 
              7331, 4776}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.713523859562322, 0.7680964895238158, 0.9129115084500807], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVltsVFUUPZ1hmGmnLZ3OzJ2507lTFAMo+FYsGk07LY+OolFSoUUa34kJ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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlmto1XUYx5/dtzOdbvOcs7Ods2kXstTKLiZK5abOubQosWml0BUi2+hF
ngW5LIqgehHJ1GlEmqbWm6jMrr4pCyxtqzAiHMwmWYTmumEq9fnyPS8e+D+/
33P9Ppfff8o9Pbd1F0fEs1Ap9HouYnU64juYCeMiarMRz9VzVhKxtyziWHPE
Eu4PFXE2PmJ+KuJEMqITI4vLI6onRbyYiXilMqKnOuJJ7A2isxjZU/Cnka+p
i+iAfwv5fxoipnG/GV9XorOrJWKgNmI/vqbXROS5Wwsd5X4YWsP9Q9DRYtto
J76nkpY/nyDmOvuQ7c6qiF74OmR+4O4GdJZhaz3xHVA+2FsB/wn+DpL7h/AX
EX8JMY9ibxSZfdytrrXskTLn1tVsWeX4EVj8AT6fIdtTbiza8flfkTEZhB+C
hvk+ALUSy6vI50sc8x78vQkGYxXUAMyPY380Y90y7C1C9uekv3vJoQX5L8n/
MbAcKTG2E/HXOd4YX479K6AThXy7weJl7u/gewvyU9AP8nsc/VvB9118r8D+
ALH8Swzb8HcLOd0kf8gfQn4hNjrhz6DTwV0f9lpVO+hTvn/LutZ5fMwDq/aU
9e+sNNbfcn9XtTHvIva5TREXEt/3xJvhviFjX8LjKmIZI75l4DsbvjPjHBR7
N/YW8T0zY9vyce9k+xR/nnwON0b8zdkXE8GHeJbi7zr85fD3Ff5KyG9Ji7+F
YQJfVdCRMCYziHU6lA9jmMJXOuN8VbPP0V8LBolSx3SCu18yjk1nM4mlKuNe
fhAb87F1Y9a6Z7HXje+HoZOaB6gC3+U5Y6d+kuzulkLvo3M3tvqgDZXusb3g
PYL/Lmr1dLlnoafFtjQTvfjfWZg/9fSClGdCs6Aafpx0D6j26nH1wgXw66qd
/w7uDxPDymr3+9dZxyDfOpuDvQr467Hfhr1KvvtrjZfOpiF/Wda9JwyeqHPP
qFckPxv9TfXmhWkN37e3uBc042PwH6TtWz4vxtZGzn5NuB47iPe+ya61dtTV
1LaY+ybuGqHjxL+3wb3cio13+H4Dmsf3Gc76Gz1TmqU8mFya9Y5QvPIxDf23
GyyrmXy+3jtKuWiHKHZh0Fbof/XWLOjRIvdYcc4zoXoK03XIbiafGRXeD4/A
/1hv/oFyY7ed/C+pMYbavasK+1X5zSW+RVnvNs2IersD+0XF7nHZ6kvZtmxe
QyzXNhmLPDZO5tyj6k3N1GjSGAmb97n/CV/H0u6VZ8rdW+/V+Vs9dhr9UznP
4nL0q/keV9gfylG9fD+Y7pvgnl6Z9M7WrtZO0axMBb/1FY6hv941VS2V81T4
RNpvh3bkVnyvKcy7dpaw60kZK+W3Af6ltO90dhA8hpo964pBb0k+6bdKb8qm
Rtdcte6FZoF3Y8b1E94Jen97yrOkfLeR2/ac5bXDN6I70uy3TmfaXXOavMu1
w9RbexqMpXpsF7q7c/alGmpXtmU9++p37cYF8OcS3l8Ls66xaquz11KOSbGo
Bmfxfa7Zb6dqrrfhhZx3sd6Iv8BvKOt50X5dVeueUz9oZ+ht+D3n2umN+Cbr
HaJ8tZMH4f+st67ObqaepfD7iz1TVZMck2JZTkwDYL8lbexVA/0LLC38Lyjf
lQ1+0/SWqeZ6W4fT1lWPaXfvzPnfQjv7f/KZMjs=
              "],
              VertexColors->None]}]}, 
          {RGBColor[0.7333572420707016, 0.7999632165174, 0.9133636484836187], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNl2uM1OUVxt+9zezszF5mZ2dndmd2ZrG7ASRhMXIpsPQS5ZJwaWIU2F0s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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlnmQDmQcxx+7dteerN33WPvu+25Z42osuWWZcjaDZoxzKec/DaESlaxK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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7531906245790811, 0.8318299435109842, 0.9138157885171568], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmH2w1FUZx8+9e1937957d5f7trvsLhcQeVcLRAR8SUFzEqVSFC7pnUnN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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3uQ1WUZx1/2cnb37JWzu+fsOXv2/M7CcjUuMgpL7JqW7GrNiFLJdUmZ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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7719646257490242, 0.8501105154840344, 0.9098495440286037], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHuw1VUVx/e93Hs555577uOccx/nfXn64mmGgPgKQaxwClQUp/IBPsi0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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3uQ1WUZx9/dPZy9nL2dc/bsntueg7C73ORiIBdB0AFJK5wClUUmExoD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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7897763493543901, 0.8560502506962413, 0.9018699078150665], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1mH1s1eUVxx+kL7f9Fbbb3tvXe9vKgC4M8AVKHZppZagYh7YUprLMdCa0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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3lsVFUUxi+0U9oO1Ew705l2XjsVZQkWRKWguEJFxKBIWVwjosZiEBCD
GsBERYkbsqgRkKK4gqKFCGhEhRYR4woooFDc0AR3iSaiEZff5zd/nOSed+49
+/aOvmpa09TOIYQ9QCGwJhVCLBnCpHgIn8VC+CQK4VNgCHgzFxtrQugEfXtB
CBv51plzB/SrOQ8vC6E+HUJfYBVvs9yfXRlCC/QG6FOK4M/524oQhsbNc0Mi
hMuA9V1D2MWbHtUh9ASWdglhDAot4/7jwEDurkbmDPQrBN9RGsI67n+fCeFQ
NoTN4KOgx6D9wrdLC3xnC3gb0A4+Gh4v8P415G3D1tHw3w+tEBua47YhWxtC
b/TvBn0uOvUB/xV+l/F+Mfi6cuzBpo/AJ2NfP3j1Bh+IbYOK8QH+mQe/zfDa
BHTn/iLsubEkhHc7hfAk8l9H5gVx27uW87/4Y0LeH+9zdxc8GpC1mjfL4d0e
Wfc7kLEefdYBTYX20YSqEJ6APijvn6XcfxQYW2gb1yP/K+jX5t8PwJYb0G89
+KIutrUP8EWRbd4JvbrWtiexpwR9SoEF3C1G31/w9XR4nlBkn6eQVY0P7uZ+
d769GdlG2daEjHd5+0G1bfkD+1+G9wZgnGILzzOw9RPun4fsddx/jPfLwLfH
HPMRddCADzjfyftWfHUBb7aBz8XeMngfVW1bSnhzCrQrsa8V+otAO+fb+PYD
55+A0zlPS1rWS8AznEfx7W3O7wBT4T8YfA3nAcoZzhcCczr5Tm90W1Tnu/2Q
9zn6fhY5F5Q/47m7Ep4fxq3zrdC2wrM751rgS/w7C3wTvjsa/x6A/kXkfFKM
RvD+XOkTnJN3QJsT+e1oZNzOuRmfPxbMc7ZyG365PP+v4F+Hfx/oav79ie2E
csfmG2AovOcmXbuvAoeJ3WD8V1diGxfDe0mla+MtZPSB3r3S8a6H3728HYyM
/THnWFfk31Lu/N+FTcmEc0Lyqrh/f9I1oVp4k/7QhG6zq1yPqvka8L0Z+1o5
NAxeY/Dv99xtRafSyDWvWv8Y/gcr3ENUK43cuRv+Z8P/mZhrIoHsZu4s5m6u
yLU6D3ktxfbxOOSNBd7jPBN6gPYv8ieBvwKPntB61LqXrAFvTbnHqbfJ5oMZ
x1C+7QVcg+8O5ngPfTXy7+PuOejzbMw5rl5wWsa5rp6wm/OOjGsrXezYbKGm
Wkrz9cO5HehX4G+dsXUF/I8tc09biX9fBj+zzDmwG1sXpF0vzxc4NyZV+iy6
cm16yrYo505E9lR0OhndVsfsq515feSztgrnlHJJMgap90buXQ3c2ah8Q/4w
aG3ImI/sN3KOxQbkVeDPvzK+ew5wErqcCMwI9nF/6MdXuXaL0OHytGta+Djs
34rv/0y41o8h/ueDj6x1b5ONw3g7ImHbJnJ/Ie+XlPvcq4tzdQDfvityzq5C
lw+x/wiyDkF/RL0b+kjOt0LvwBctxOtLbN8hn3A38O1C6PuQ16bZkvUsk480
G/fn551m5JyEc165rpm3FHk9U5Z1APg07ZpSLYnn39D/yXhWNBd4Vh3KuLYU
33G8HZ+yLn/igzPR7awa55p65Fps3QvP0Xn9NSuWZCxLM+MmdOlf51pTD5ml
Xq18gN8QbLgZvJH3v/G2AdjH2+uR9xy6R8iYmHQPVe9UD9yH7R1Z5/vxnR3L
ZJVjq5iq10xLOZfUc1QL3wLXlbkmpij/4XcY+iB0KADfmvNuoXp8GvzUasda
PV29XzFVLBXvRvQ9kLLu8kcd8ieXWxfVaBq8I2Pd65Vj3L0LnlvK3BO6qp9A
b4XfcGS8Qewur3LuagZrF3mo0r1AO8kC8CP5/FVOaHfYGnmWqWdF3M0CpwbX
kHJpIz7fG3NO3Y6tW5D/M/d/1Pzj3A0dDnPeWOjZqxmk2aMZfF+Ne4h6h+7c
j257kHdF3D3qKfB94BPj3rkUy6urbYtimqm0D2T7wELn+vaUZ5Nyfjnxaam1
7zXDx2Y8EzUL5RPNxrakddWMVG/ehrwxeXv3YN/utHc/2dgXXg8mHPulxLtE
+0vW+9FH+Gtz5B4h/46CR03knFeuH4C+kLfHJfK7VMy+rq81L/lcs6SWN1+X
eqZo1ygC31nqen4WfFNk3tonVQvPp1yrqom58J6Ssi8Uo8dzlilZ6qEzoUWR
dVG/1+5WHFl3yesSueeqFiUzx7k6sv7SaXnknVT9SDutdsNukWeffDA55Zmr
Wauaeo9YDS337vMaOvXFv/XAxdh6CTC9yj1UvfNe8IXk3xM56yaZDWnnvHJd
PW1x1jWhWpiBzGHE756kd2XNnO9426PGvVU1qFyfCSwodky0K1xU6V6rnUG6
aOZq1kqn8fSO95PWVTtMB7xKU979fgf2g8dT3rVU02eh35C0e4lyYiH2NlZY
N+28lyb8D6BYSYfdkXuYepdyfCzymupcG5r5bTn/E2h/1IzRLrcnZdna6ZRr
8ci7gPy9N3JPFz/VyIrIOSJ52onVC7TzaNdRT1Bt9quy71Wj16HrwznvCpq5
l8Brfrl3Lc2kyQnv4NqdtUO8FXmnFq4aOTnrnv1/b0KnU7QvV/isnUq9Vjmn
eKrn/gcpn9+n
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.807588072959756, 0.8619899859084482, 0.8938902716015292], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHuQ1WUZx19Y9uzlLGy7Z89l9/zOepBcccFKYBuTtNRpJgXTCJBb2QUt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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlmlslFUUhi+ddjrT6UI7nem083UTBWTRyGIQBIXEREAUaym7omJDWygQ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              "],
              VertexColors->None]}]}, 
          {RGBColor[0.8253997965651219, 0.867929721120655, 0.885910635387992],
            EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1mHuM1NUVxy/sssw+2J2d3R13Z2YfNNqipq1S/ilJDW8XsKZVKw+L2rVN
lFawIiA++oI2lKZACaJgMcKWtEYlEWJZo2CkKKvyXI2FBUlay1JQQdoEBdqm
3w/f4x83vzv3nnvueX7PuTO8a+5NcwanlLYMSqlM34OZlN6qTulnmr9XmdKv
alK6WvMdWt+uMVrzgdqUjmv8XmduEO0UjTvFZHN9SlPbUto3JKWPG1J6sZTS
ZO2Vaa2zKaUHtL4ym9Jh0a7X2c3af15jjGjqRHN90NRrntNYqLu/VxBvjYmS
5bsdKa1rTWmI1vdKlnmS4UnxOVVjGV6SbJWXpZTReK1Cckuh4fq2675J2r9e
o0c0DfmUGvPee0j3T2hJabb4NZWn9PWqlPpF//f2lLp07yTde7LG5zm7THQT
NZ8lmocz1g/dt4jP1pLprhHf6bp7muj/0ZzS5dLlctF+KD6dIWe2mNJ53Tug
9QdqrctW6bJWPPpku2tF16v7CzmvH5EcS2ts/9u19lqL9aqsS2mU1hfIbtUZ
22yp5pNky126+37NB8TvWfHdJpmmSJZZOvuM5iNFO0Hn54tmq+YfVZpmhGiu
1FijtXJ9f6Hzb8kuB3TuomR6XfesylpufPlGxmuTk211VnyO6/eYMu+x/rFk
aZEMV2Wsx46IJXjMDz7N2n9Qco/W/HTw2CaaOfL7cslwTDK8Lv/v0rhSPM5U
2n74ZbfW3tC4SusrJGOHbHSf9leI53LulT0bsvbto4Nto/HSvUsy3ipfTCva
9/u1Nk97F8odq7+LeMU/zIm3Tvl3l/gtLrP9NmbN6zPR7NW5QVrfkPU6fuHc
quDzdrXPkl/dWvtD1vbHDtBwD7J1x/pRybRHPO8Sz8N1XidHiC9iihg7kPV9
3MX36az1mi273aNRrnMfFSMWdaZL+38qdw4SO/vj7KuNKT3S7D302RB8tmt9
t9Y3ab1P9IOk//3JduIubPVTfKdRrLbdoMVWY+SHx4o+A+3+oG8S7WUl60ec
XCG5vkIeaX9r1thCjH5V/KZp/bB4tOcdS/+Wf/fp9yH593zBcUn8YX9s1TvI
8u5otBzzFDvvFX0PeT4yeDYrtuuFWYtrnBPvNvjOTyqcN9gc360Mv/dVGlPI
u8aQk5h6pco5Py3wYXy145a9LRFz4OWW0GtUxnnwA9H01zkG8C05xjrx/x/J
O1e2GzXYeo+ot+4/FratVI58S+v3aT+VfPcVkW/QlNdYB2zxL/K1xdi9ucr5
OCbidnzk/tMdxgt0PgMu5mxD4hc6YuOP0n16nXGX74w64xJ4NkG6bQ9fYb9n
JcN5rV2MfBgQ7cw6xwzxMzP4vCDZT8nmE3RupHzxNY1z4jlMut9T77y+fKiw
UGfOlTs3pse9xP+WIbYP8t7eYZkPy+d3dvhuZCAmWD9UZ13AvHG6c3nBefq+
7jykM/vKXUOoJejyWZxH/i7x/6TgeohN2eMsNmWN+ki9+aTGNQTbICc2OxWY
D8+F8sOivOesUQ/ArxXSdV6T9Tkt3RdJvu3y1f80X9xgXQ9Kp+5G48ce4dxe
jWtqnFPkIXnK9/6Y850XMfy8+L/cbr/06dw7rc77HVp7td3+Q3bwD/m3aa1H
Y0yt8xVacvZI2I+YqZJsd9c7zl+SH083uBegJm6u8V0NQy0jvsOP1Bjqyz7d
v7/VfqXm0x+AaV9Wzk5ttH+flJ1m17tWglnYAF8f0r1/LTqW5+jcZnwt/p2a
T27z3WP1HafxTI1znX6EfCf3WVufHJP0EfgITAVzkYH6cUy+/EA835Qs3xb/
E7XuefgSn/gd7KF+YZuqOtvnrpyxib0LkvG/Gr2S866sz3P2cywAe2u0/0LO
9ZK+gdp5PGrpkeC/RzRnC46xs9GLkBeVWu/OuQYelC3v7bAc10nmD+S77+uO
ubLB4Cbb50Wt7ZCPrqs1xlDbwZ8NOrdRY2ydMYMvsbNMvjjW5rzIyQ8tmi+p
MSZsiHq3utr30CPxGxsSG/0Vpv0gYgaMG6g0znUH1mUk/8ac63xn5B2+/rNi
qafkXOzLuB+lVtIDbu9wH9gYuMteU9a/kWFixmegXy99fy59vyD5ljQ4l4ih
S32AYrG7zLwmVBrbwSSwCYxaM0y6DfMeuFEZGNgonvNz3qfXWhq5Rl1vinrK
Gc6Cb1Mi7+kPR7W5vlJb90qWzwr2723SdaZGFbYU708L9v3MwA/4nNHa6YLr
z+wK5xD954TYR/414vFEyRhM/36s1nYo111lRdt/HfLLFm3a26i4LGRcy+jx
mRNjX5Kvl9W7lhE39FxgyIMZ5zC5TF9FjQELt4l3j8Zq0e2UPcqHui8i7vdW
Ocaoww9H/3CL8vpmRoXjhHNfFP0NiuHnGnyWvri2aGw8GXmKX8C/zg5j4G7R
vNHiPpK6SUyRv7/NuUZSH7H1pvAL9kW+JWXGDHID/T4Vj+qi8+7zGjw6cpQ+
ibN/a3IfQw+zXbQ7NNaK7hb6moL7SWxOPDJf1+S3ELhDP3kx+rcZkuupessB
xoyIHLlNMszUeCTjXq0h7Ix9yUnO4nP6c/w+TbQ31tsn3EHNBd9+KfsNKtnu
xMoTVe5XuGttzMHvI5GP1ChqKjF0k/zzbNifOrSvyngLHfmLTRa0uIYR0w/K
hwvzjkfqwGN51wJsBt6Bdew/VzINOdEX7zZwva7ofMe//6wzzoCRnMX29AT0
UugLDr0fWMTbC36sH40YIn7Q9Vi8+ZbIV2Pz7tOxzY8qXI+2BlbQg/0mY/7I
TD8OLflCj3Ao8v1x8VlUsm051xRn6XFL1Y4xerGflDzfKfs9qnlHtWn4Umtu
bnX9vlS7y93z0pPwtqPmUe8q4178syreHfgGHBgILCD+TkQfdVZ+2JCzHdbp
zk0568m7qD/qF3kzEPTkQlf4BdqH8qa/tc11iNoCrh+I98O5ausLrlIziqHL
bsX7mwXLMzbnvg2e35BPr5ZM02TTAel0vNk96QV9zze75+VtN+4yxxL2nhhY
TQ7tyzl3LhTc69PH4m9igx5sW8k9ATED1vbHfwib4p3GPF9y/4pt3232O583
PnlJvYA/OUYfgC75vOsruAdevh3vsuF599b01eTioozrRVvebwn653v1fZsa
XOX3K/0fdeAO+XlWh3tH7gZnwVjwtTvm8wIXyGv0OxE6gn/kITn4PH1fyXi1
VPHz61b7ZL5s/ECL45e8PBy5uTzj3os4nxQ18mDUw9WBTTvF4y+txp394Wvi
vCn8TWxvjTcL9ro7aznpbabq3DdbnSsnImeJq+9o7aZW41+fZH+n2TlYIdsM
LRqzoD0Z9MTc8pATneFFL92rczc2ua5SQ4gP7HBt9JTUUGoG9iDXXml3zaPe
3aGYaWh3PSTmTwaeEFO3tNrvP+Q/DMmzIeoUcQov/MIaMbBANKfApSrXqBkl
3/lUu3t0cBFbYl/siW2w0cWoiXvi7U8sEKPEAz46Eu+sLYFtxC69UH+s91T5
Tt6/1NujUXP5v4r+jn4PGt6Z1GWwuTfwmXdCT5yl1+mNmnsg3trkMnn4ctDU
K3ZyLc69x0vGOPAN3O0K+fg/jlpPzeEN+mHRd13yVYV15I1L7wZ+goXYCb/Q
W/SEnAtlz4+Lvvv/xCL1qg==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl1ls1VUQxk836F0s9N6W3tu7tCgoS+LKiyYqIGhZRGWxgOKCmAgKEgUp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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8432115201704877, 0.873869456332862, 0.8779309991744547], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHmM1dUVx68Mw6y8Yd6bgXnztlFcAEvbxMaK2CpTKiitMhZZpkaWqkCt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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlnlslGUQxl+3Ld1tl+Juu7DLnqUoLQQ1okgBTVuRckQUhaKgHBVjEJEI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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8610232437758536, 0.8798091915450689, 0.8699513629609176], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmAmQVNUVht/QPTPdDTNMT8/AzPRMd7OJyqIGXFgEYhBQCUZkEjWZgQIT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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwll3lsVdUWxrcWO1Fvue29pbd3OhTKbAFREVooIFIEHyi2Go0KgkZFGRQ0
EREQTcQ88QnIe8WJxKC8B/jEOSoo1oCCFEecFRCcZ3GMQ/x9fH+sZO+z17zX
/tY63abNnjzryBDCu1An6ON0CAegcQUhPF4KsZ5eGUId+34wPJIP4b54CNuP
CmF4LIRN7B+EliJb2TmE9VEIjZwvY39+eQht2RCOYT+dfUFZCOdUhPBlTQgd
JSG8DD1bHcJW6OuiEHqhcy7yA7riQ2EIMeTHsm+GGo4OYSH278SXLqkQxhSH
8D3+/YZ/byasezv7Ieg/lAthJvwvltnXcdB/y+yzzk6EZ02JeXqjezHy49G9
Ch8Wo/9HdLaj6wWomPNLOR/M+SbiOzYTQhXxtiH7GTpaWbdA7SRxCvyj8e1q
eGrgXUnOypCfgfxxyE+Fnkb3U1AO/lXwf4BsI+d/wbuD81rku0NH4MsP7IeT
m/7kqwJbg48gfvivw8fnkY3IfwfnU9kvZx8nJ12ItWvOaz6FGPYroDXwHo++
Ital0H9ijmkJvhyK2/ep0Nmc7UnYl5e4g7exdwH6fsa/s5GZhq0VfHun0DUx
FN638j67Bp93cN9/JOz75/jcmVzvRr6ZfN2B/7XJEO7BxgjudgC0hvU5UOtR
/nYHshvxZxf7S9FxJrm8BR2TsVdETlPk4iHsDcPeGM6v5PzRvGtxBf7MwL8d
adfqQPaXIHsw7Vrug46KKu6V/Rrlj/2LrLenzatvY/DvJHJ6L7F0IofL0X8b
1Mp9rYWnlbsZSzwPUE9Xc96M/lF8u4T1g8Q7HF++4ds6+H/Bh31J51i5Ddi7
Al1zoImcV3X2W1uNTDbmN7ce/5ci8xzxF3I+L+MYFVuK/TpyVUd+WlF3Bvna
gOz/oSHIL4PnUfRNQUe3AtfEGchOhsZhrwD5E8jfkBo9RmzwrTuxHCDeU8RP
TPejvxb9zbBMKnfu52ZsW3ewoNI1qNpbwbfvua8tWb/l/YoX+3uh/uh+GPs9
ecsr4U/AL7PCgpakbQsTFrH+B/bXk+/NxNyd/a34UI/tgdC3ep/4+Be6T+kS
wq/o/hlqwd9tpfb9U/ZPlDqGrzHyFYQ7hzHmee6mIeu3O4jzEXnfme5qCzU5
H93X5nwXdyLzP+KbSbwJ9gvhPwhvz7zf7ip8fjbrmBRLHfl6hv0PCa97QIX4
X49PS4nnRPaHEubRWU+oIXKNq7ZfJSffsb83YWybQnwDkC1GRydycTM6hgqv
oCUFrolF8PaKXFvCrBz5LGd/D7q+YL+b9aac38Yz5cZS1bxqXZj6b+K5nfjq
ie989rtSvnPddb7Y2Leo0lgqDOwvPI/s66nofC3lGlTt1cI/KGNMFBZeQ05m
of+XtO+mHv1JcrekwlggzLwZf3vGXWsbiac+sg3p3oqNEmxvY/9IuTH1yqQx
WVi8Cf7e6KvL+y7WoW8GsV6Wc63pTfyWM+YIa+Zx/ja5nxe5lxQhP5r8HRPZ
1g3k982c70C5fwIfdmZdA7r7et0X8f6Z8Ho68faAd2HCveJxdMTZD2O/oMgY
O5Jc90GmFd5F+HMq5x3YOBlf9nF+LoVzHrST8zZ83IzshMhnwsjl5OYt5F/i
fA7yu4i/A+qL79fhX0lkzBXWCsNbhE1QS7ExZQ7ns6UP3qH4/Gql35Dezn0F
zu38GseqHPdlfSv62qmHOHf0K742VTh3c6Hd+NIBNSH7O/Qad/t62rWiN34D
vIMz7sXCmIuw92TavU1vXr22V953pZ67ttqYKCxUz98P70dp19KxyI/mfBT+
LCkyBvfF9lkZY7/ewFUZ17BqVz22X41jkO/qkY3Yasr7bQuDR1S7h6p3xsjh
pITfmN6W3ujD8M5CZxJdDfj7APuN0AkxY6qwdGvWb1eYujrpHqHeoB41ivN/
RcYqYWoT+RuRMxarh3fnrJvyT74fQ99p3G9t5PV8dDQh344/hax3EtMk1j/C
kyeWDchP4P2O7+pauhYdB/HlQNZr1cQnab85vTVh4EDqb0CVa2c+PP9U/8y4
V6mHdYk8E2gWEGaot0+DtpW5x39IbMmu9mUaOtZVuwZVe9/iTxP3MRK6uNg5
O53zHkmfbYQuI79Ho39ZkWeeWnxtSLh29EZG5jwjaTbSTLaQ+O/K+20L09tT
xmxhtWYk5WZkpWcR5eh9ePcKj2OemdQrNFNollDPKE+5h6p3aka8iVxenPMs
qJ72Yd49TL1LOj5Cfj+0mvqI80Z+Ip8zK1yLmrHa8sYsYZV69P340y3pWVUz
i2YR1ZBqRzNJM7Zmk7+24BmhMWUMEXZcCP0U9x3rbjXTThT2Rn5r8kFYUVxh
bBBmXFjlmlGtnEcOtuD75RnP2upRT2H7ybzftjB9EPKLk+6ld2N/bNw6JKs7
U21X17g3qMbfzRojhY2N8F+O/C1x370wqgRdRdCwMvek8Vn3OPU2YYZmn0+g
94o9Ay1Afm3cvfTwN+7+nax1qyb2p/yPoH8DYVYBuvZkjcXq8Ssznrk0a/VH
/8Rq17RqWTNCP2T7QBPYb4kZK/pGXgszlPvhKedad6C3dFq1a1NvSv8OypFy
o/rYjO3vEp5dNNPO4+yqnGelw/8w5Gpf5NoQJvdOucYkK4xXbt7g/JVy5+jG
uHuyerHe1J7ImCwsFs/12FqRcWyKYS++7aHmNpQaY99Ie2bVrKpvwo66yL1G
GPI3ESPNig==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8788349673812195, 0.8857489267572758, 0.8619717267473803], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmguUldV1x7+ZOzMMw713hjsPmTsz914GqICCIlqVp4gMrmqjUWtCBjBA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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3lwleUVxl+yAAnJDbnZ77259yMBCRXoCIKyJRQEO4oiYFtkLHS0IC5l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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8966466909865853, 0.8916886619694827, 0.8539920905338431], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmgmU1NWVxv/VK93pqu5qqrcquqqMgKBCZHcZV8IMMZMRyKiQMUoSNCYq
iBMB0+ACuIFEiBrMiMgWBQFRczRENMoWcyYeNTriikpMRFDm4CiKkuB8P77L
Mef06/f+b7nvvfvuu/e799UR3580ZmJFkiSLK5NEf8nWuiS5uiFJCvqoaUqS
WqU9qSTZqHyT0keqb0+7vk1578Yk6aX0dxGZFn2mKuWVfq76v6TcD1rjRX+R
8ruVRqv+MbVXK+1MPIY5jlK5vdL1VSrn6pOkQvWfdkuSlbWar1p91fBmWf3z
SbJU6/1xtelCsynmYs33K1+ltF+05tU5f031dyhvVDpOa95T5fX0Vrkp6r+l
eW8VnRUac4zqr1bdAH2nlS7VXB9ozCLVpzWuocltrLcm9tItyuzjZ6LxWczf
X/36KV2lNcwUnYOqf0/lurTLYxLnX0R9fdrluljLMqVsyvlypW+r/1N58wI+
7Io26lkfdbtVd2JzkmzOe0xjKUkyJe+vSXlSSJL5Kv9F7eeoX5vG7dOYsVrf
kcp3F7Vu1V+kPntU/kDpR2nzZUWs5xS1b9H4vSnLRU3Ixmsdkqe86Y3QGocr
vZrxWaaazIO1ytcoJU2u46yb1L9vleUBeVsbbU3BN/puUvkIreUn3ZPkDPW5
XnRnK7Wq+Wy1j1O/3vSP8ozE53eF6gpKvbW2r2vdB1ReKRlr09j+6tNL9T2V
RnfzHXi6zjx5qzNJPs8lyQ7J4F36HpXy3k/Wnk5RekHjj1f+S/X5YZX3+YrS
dmhq/pdVvlL1l2sNk5ROC7lbFLL3Q9W1qZxF9kRjsGg9rTEjG11/k/p0Ki82
mfdbM24/tsp1tHHWc1T3B6VhojNf+ZmcidLClNfD/tu0nv2V3sPdmusMzbUt
Y1lGppHPD5X+R3WTROcBycg48eqZOu+B+pLaN6vvFqUVFb4DjB+deO+vZ3xP
mfPVKH9L83y9bJ7M1hr+N2WZH1NyG/X9gg5ruFZ1A5X6qP5E5Qu11qXi/53i
SU/1GaM9fLNde2y3blirMZODZk3aZdbTFOeO/oEPV0Q9+oG7+W7i/PMo99Fa
D6j8dsqy9mza8lYXNBuVvmhwn7bE+d8O90+7TDs6oDZ0Qrbe9NEHyCS8+17K
MrA4dNes4Cftv9P4DuVvqP5i0Wxvsoxcmfbd5E71iHNHDkfFN2XkpT1k5vRu
PkvO8QuNW1rhb3RzPmRviHg7VKm92jIEnRHq/6LqPstZRy5o9Ho6Kyzfm+Pc
39P9HivZaFe5WGkZgSeDNHZA2XJ6nYT6QclQO7q9RXehbBl8RhXnd/e41eq3
RmkOKqkmSeaKX3nVv6u5/lplu7Cr1uPYK7yEp5wX/GPt8LBD9NeLfg99H6s5
+xVsQx7XOnsULEt9lPctePx9Sq2a96RKzzdH3y3ay631npc1c07/UmF98rb0
wNsd/h6itU+R7H5Y4b6s9VP1P0X1XSXbRGScdmTzZ2nPd7S+z2yy3kHnMJb5
PlN5tMZ9W+lF9V1V7z691L9Xjb8p/ylt2sgk+pL9oRu5Y9hH5HyQ9vuv7f7u
0F5f0RndiI2usj7iHpfU/nzWurYkvt1SdlsPlXdon0WVF0rHPqc+ByWPN6r9
5rJtfofaLlPqmfJdvqzOe+XecxdZyxT1bRCtrPq3Vvv8DskpcqZyXvk9kq/l
7V5XV9CE9s7gJ+deTrnup4nHMo66ZyXbOzXvJaq/SvvL9fD3I+gSrfl0jd+n
vpX13ntefCgU/L1R+7pLcx8vXq5VeY1SR9r6hX1z5l0xV1fivBDlC0W3U+Xz
1Sev/W3v9HePtNfH2rhPv2i3XMLfnbEX9pSttpy8ozU+Wvb3kTrzoc3GS2CQ
k6uMT+7ReiZg86o870Uqp6J8Ydg17sJnjV9ijcN45/EG59gUMMulcUfWqv2/
KowX2MPyCtv62U3WQegYbOZFQR8d0yPsyyXVpgO+QA9dGHwYrruzo86yzT3a
onKrym9LDnfk/f1X5e/mbVOHqc9rdZbnfap7pezv8aJ1rmhOT7xnyr1Snmds
rOer4tU74tUy+K1yQenutDHHIblW/+2iuVpnvanO9uJU1R9fYXoTgj5nj90G
l12QsuxxvuBJ9okOfEfrOrbZvDwgeVmp7w9F67uSl/OK1tPgp4fqfV7gA9rh
CTihNfTV4sC96NJxIV+9Y07m7hn7Zv/st6B5RmStD8drnguUNjVYnjhLzvGg
9vhF3ny/NNZM/Qjx4xOt+Wmta5TK1dJHz6eNNdgbeKCf2h/OW2Y+rXQ9+z2q
xniKc1yiOY8tGYcmddb76Py9une36jut/DTR+mWd9epvRW9gs+3gVI2r1bwv
aexXBMrqlfKi/TXV91dao/o/aX+/y1vvlVQ3IMaCL14K7DEwbUwCvng56jlj
5mRuxlaJV4N1NlNrbXfXd/P9Qo6RZ2TmzVrfM+7fuer7uea7TrRuUf1R6E2l
LWnbt+3oVdH8gfp9jbuuezxI5Snq+5Bk6tcF6+unNNdZoj9U9L+rvd4qmj3E
u3NE92ylg6ofnnIZPf/NJrch+4zFX/o8sc80LbAue5oaflB36bNbRHe86suy
OcN0Hv/WzX2viv7IXUfG92+ADO9CreGylOc5N+bClryQtg0aqn28VHJb97zp
QnNjg2myri2hBzjrn2rPt2UsIxdpbI348nN99xJPeipdXWvcPSuw96zA4UfC
t0p/g5+/o7EHNe8N0X5D9JmStt7hrFap7hzsYuDWmwLHnp/DCBjb0ga+5+6g
O5tbfK6cw6g4iwUZl8G/t4tn/5HzGU8L3xSes7/ZsccNjfZFZ1d4LPuDDjQW
RJlxnA1j3+qw/3Ag1nhm1NP/rJgXfpFz1+DfgqgDrzOG/ujpXIV90oFNxsrg
00qdy66i79nkwO6jA89Dn/k+kgx2dTcGnh8yi7xxjmMzlg3aVqh8XuL8V1FG
HgY1WSbui3rsDfd+ScZ3/0rxrE38vTfa742xfTXuaGxBYkzYN8q/Ea1H0+bJ
Kq19iGTjNsnGZNHJic4yjd+a9tgtcdb3Z2zL0HmcK2d6msbdkbPdK2uPO6UT
6uq9PvaAjp2h9rJoPqzvZzKmhT5/SPmDSnMT3zkwFvcO+usy1qvI16qYl3bu
JHcE3nMG+CDo49WBRdcFzRFVbh8UfahbF3PRtjroc7Y3h9wuzXjfxBOGB28p
I/fXxL2YqfzazJd1lCXSh85mYfgz4Kq14ddMV3tX9KHvdVHGV70myjMy7ke5
K8ropVnhW8wIOldH/dni5/6Sv19UOiFtvftAYHL2N0Dtv9K5ZlX/tso7lL6X
8TrhC2vlXPuGj7Aq7irnwr6Xxt6XRxlZA9ueGfzHv18bcQB0F+fCGbIv9gCW
wG7ODL8JzDlLabPKe+u9looq1xEjABsfr31tK/k+4H9kWo3noYMOQQbmxnlx
L5ekfbc4OzAYcSLO7azQi7RhB2/Hv+tm3UUZHYIeviNiSknENPAbwVprInaB
/8SdwacDj4BLuFPYUeq5R/h68xvtu9VIDzxe9vea4A90iGnhu+L74AdDB2yD
/f9xYIANGvtE3nYSG4nuwqbMVt/rG60zwHRgE+z+loz7YMdppx+2C5uMXccu
v6dyv5KxI2dDH3QOsa8hKcfrtqlPq3g9q8J+8IHwheEtPif6EH08qsU6Gb27
Icajfx8Lf3lk8ABe4SsUAouD528Mffe6eLOmYJ8NfECcbqLqH2g03w/zHIyG
TOKHNEV87828MSL4EEyxLfDGO6r7c95xULDW4sBsjTEWPQRW/SRvvIoO5zzQ
43+UTbhBmGaTzjMT60lHbPCx2CP4nLNjHL74ukb74+QPNhq7ZkQnrXRG4KxH
ullGPq72PK+qz9xq84vyYTnbFdj+gcD34Kh1QfMx0XtEa360m+MI+AXYnXnV
xjjw82Ld77Vlx5fAUBMajaPQa+jErtCH6APqqkWvSulD9f1ji31I/MfrRKO1
xdj3CMlLT6WlmvNj5aN09qXw6aHPfXwqbR4iz/ig5aDDvr/R6L2/UHY8grO4
sOhvym2avzXv+7hK+LIcvuZNal9Utn6YqXxW2W347QcCeyPvW8IWE2c9GLFW
YiJgZWQV3TAn9DmxDPpzR1aUzVP4ic4hBs5c5L2jfHu1fQDO5TtF+xL4EZwX
dcRp8eMZg746WfZ8Ttl16BX0C/tCV50T+gpejAwdQg5/8O+WR1wHPw49uC/K
4DN8RcYS/7i27PXjT10Yfgr4ryN8eXwXaFGPj7c79APnjh3F3q2MMrodXxme
opOJmSInc8Jf7ghfmHhqS97txKZGxpoL2l8+7bgxsrAn5IF71hx37Q3JQkbp
K9Xm/dbQ1fDj7MCKScgj8lnutD/J3VyOf1b0XcRuEQfkXWC2eNDS4njEPPW9
RWline8KdwY+ptU+tex1MP99xVhDxO6wQzfX26chXnRt1voIXXR32XSh+Vza
8ZpNYcNfiBgi/hhtyAD5s1E+pOcb/f1s9GEsuGNgYLbXNdc3So5jE8PalPbd
qZH8TC95vifztgHof8aC+cAtYLDfp21rXhKdfy45ngn+IWZzRcQBkD3sGvte
GXvPxBsI84LtWiOOTbwf3o5PnGejD7ToD3Zlnm1Zz0VstjbtMr5lXZThC+Ux
cR9p4x0EXDk+7uzkGE9/9r0x9s5cTTEvdZvTtkHgzb6BOYkN4UsRLxqdsz5C
FxELB5sj51219nmoA7MfxvL4YRUFj8df2xi+BPoY/YUe7mqwPjhctzHW0BU4
CwxD3VOx5p/IVmZLtt3Y4f2xhu0tji0SNyS9FL4b+YtRBvunCsb42EP2hw5n
v1vjfPH/KdNOnHJo0CQ2ia5BzzwkWT0v5xjV7/PWp+icYerzQNF3FNw0v+S7
+bzmfK7gOOZ25W8UjH0+1l6Gl8x37B6ygi1GTl/L+lyI7deEPMwtO56IrkSG
eBPhXJC7qaH3mJuYGjrkw6JjLtjyHG9eWeurvoGpsO/okkmhT4gV4VOCdZ9s
sJ2jD/SpRz/S9/Loz5xToh46hfSXMcZ8lHnbag8dglwPiL0MjPLEkE32iQ/e
2M06oiXeCunHuwZxp/5RHimm7sl67CKdw6ai8fek0I3wZLf4v0tpZY3fRvrF
vP2CDnNB+4SgSRzx/4qOJU5W/nqHbcJxqt9btO3ADk0IHED8gBgSWBfsQiyc
uNxJyGbJvh2+xjXhp7yh9d5f9P0H02EvsZWNgZfBV2D7YUHn4YJjPeBQfPVU
2rjiA63nYLNx0OVFr5V1oluQCTAy7yngR/qDv8GRzAWNJOiAVejTFvgZfAO2
4Q2Eb3Ak+6Qeu0Ncc1zEUfEdmBNbybvmxTGWWDBnDXYib4vy2Ih30p87zVsF
9xo5QUaIUU/NOQZN/JlYVVvoRvAwawZL8x57e8FvsmAj9gXG+Ejjiu2OZU2L
8YxtLfq9hLcSYvrTon5ed8eJ0EuH32DgA5gWW3Zl6PSp0R+8y7s0Mknsn/vP
3ceOzWhw7HZe2vcbObi5wTFByqdKBgdmvccneAsOucUeEhsEAyBTxBQPxZ0l
O//ZbP3GOxS8hm8LSo6jEUNjTmLDzEv87Lzu5mtX+H74sMSQiCURX+pUxfVl
445K0dwQ8dwltY7BH46Rk3P3W7TOXXmPvzPnN1rWyv0YEvF23prmxHvTD1Q/
oWiZBvvdFHijT8QI4S3v1/iY2Cbq8GNYG9j5R0Xj5zuKjj0dFW+JfWIs8Rl8
SWJKu/M+12LcO3gNn/Ej0anDI36Djsev3Jn3fjojXotd4i5w7r1jDbzTENNk
vicafM+R+SP176saO1/fr0ofvNJhmeVsqcMfXxa/IeANkd9W7Cv6NwTYur7x
flEbOhydw7sAe6U/semNsS909JPhc3Fn2S8YGzz/RKyHmMDwiDMTd2QsMSt0
JvLJ1ed3KLwVIAfIEPoAfbhf92BoyXp7UMl6BB0yPrACGPVo1S8uer28859Y
Mu7fo/IJKq9PO1baN+Klf2u2f4vf9L7ygap/EAxUNI7Eng1W3RClX6v8T+pz
XMm2gHdV6lgDunJ9rOFArKd35L+JevQCGwSvPq5zWFewDsOP56yJCaCPeS+E
P8j3ufEGxDlz3sgSZ8ddRWa4r/OiP74UGAMcwm8BloVuIV8a5SVRRv7PbfY7
LnTRdQ+GjSOmBZ4HW3+MXWq2P1DUHfyzcPbwBscAiAUQK8A/BQMxL/GbMyOG
Ay7BxyRWwLvcke3mAe+uvUK37NfdrCj6beNO5eWSdSW4k3OHJ8QhV0SsEv8R
e4JM0n5ayAZxgtODt9/POY7P3LPCdhFz6wxdyX1ZWvQ7B1iTWNfq8G+JUS2J
OFVeNHqU3f4H7ftl7eGYBuvcQ3GXxHYgHxiPt1B4Q9yOt8FCi3UT+/s05z2C
O8GMxDqg91bZNGkjXxT3YlrEnq4Jfcj6U+p7Qc6xK94koAUd3t6RI2QDjI9f
Dg7kjZj4GnrrE9ZQNO7hDQYfFZnn9z3EGog5g62a88ZXxF4Zh65jXegD1oav
3Rr+7SXxHgh2PUXyfFK8W2F/8fewuwvCjwd7oJvQUcgyupg3V2zHMO1rWs4x
XWLD+K7Y6BOztj3YndfU51Wl94m1SY5+UTQtcO6WwLrosLtCL8GDvcGHGTqH
q1tsM9gze6cv/jvxRPwO3vcGZ+0H45fPiljibfxmIGsfrVV5ZdmY8hje61vM
H859dsQk0X20I/v311hH4FcfemP8hzflrvCT8feJLeJzYWMXhp3l7m+I98q3
VHdC2f1mRvyTNacibkodMaR1EVsCmy2JO05sY2b053c0N5TNd3i8JGIEp4uf
b3b6m3sHDuNuEWMYGee3N2JR8JMYPTaYNaJr7wl9Ozvii9z39tDp4I3pDS6j
/vCxwQTgAX47hk5Hn18QcQRk7zSt8dSyY9LzlC/O+Q3wFAl4/1bH63hX523v
3XiT5/0VfIYuRD+iDzt4e2z9UteQg0vhO7EP+Hyp6H+R82/3wNnEGdkDMTjW
yVzEv++NGDhnyZmyD35/NDjv3z28n/U7HDZla9iu1vAHZ0W8jt8FEROEN7xb
E8ujDf8P3xO7XhbNI9rj/U1zrs8bx/L7l67wT/Ohc+At8oqcLQ+55n7eFfMR
J0Q/8Ia5r2SdQWwCeUVW3xVvdirdW+O1kDOG3y7Rl7G8r4JNwaXgPvyWiaHX
qeOsieGgl8Hk7AdazM9bycOhV3nTRkegH8Bx/NYELFdSeXrObzidKnfl/P7Q
rvKUnN+piA8xBzpuS+AycMWn2tO/5/zu0ZB1bA4/nPPbF3cQGQZ/gD3+rv7j
cv7N4Hqt5bcdbkOnYL+wY/yG6L1s/N4kbayEnse2QRsf4QBvCzm/4Rz6TVOL
zwd9/HHsvUX1V+T87sHvXP67xdj9/wGMX4Lt
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtmGmUlNURhu/MMMyA0z100z1LD9PfxypEQER2I2GR449sLAmbuESNy4ki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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.9144584145919512, 0.8976283971816896, 0.8460124543203058], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmXmQlNUVxb/ZYNQZuqfpxgGc7p4RrCwaEHAjikBcEhPRRARB0BRjkj/E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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlllsVWUQx+de2kqht5d7ey5cWjj3lOVBBWmhLCHaVqNG4wsuxK1EA+iL
yqIISIIRMWJioLKUlmgoqCAIxYVFNG4YY1yikCBVKC2giWIoSUEoTSngb/L3
YZJvzsw3M9/Mf2ZO+cy5982Jm9l6KA+qSplti8ymJ83mwI8tMxsDbUBpN7Q1
NNsCNXNOJ8ze4fwulOS8C3oxY7YUmoWx2VD7cLNS+NoisxroatqsE9qD7j5s
rAnMDuHvB/zNQH9Szqwnaza+v9ledO6EP0JM/Tl/D3WjPwx+BLFN6WfWAF+L
/tLr5PN9YjmOfFjMrA8bs5DHsb+y2Kwe6oU/QUy3EksX+ndxtxP9uZy3YqPD
ZXwbEVPMB5E1YrOp0OwiPq9x/2nsvTaAOLkzkfiqcsrVx8TXmlLMHutAKMPb
X0G/nPdl8s2OIb8b+Y8J5exn+A7ysZpcnEE+G/sH0f8O/S5sHkKeI57FxNYD
30wsG6HGuHLy2VDyj/496Bfz3qPY+gp+Y1I5Crn7L/zCQWbnydcRfE8KdK4f
iA62RuKjGt163nuC+8eha6aY6pDPgNbxfszYNs7vQYG/DUrzvn0IluNvZVyy
Yr6VJKVzGlt9aWGnBb4H/9PxP4Lcpbwm8DNKdB4EXYV/BPlo5K/z5roS6biM
z3Y950aoFH5bQrm4Iaez52QRsSTwP4BcruH+TcjGQTuQ3879nei3hMLeprhs
VQ4x+6tANh2Lq3EUiwmTXosnAmHNazKSfLaTr/KYMDcNX48iX+FvwecF7N0L
vwT+BegxfD0O/cN7LmDjVe7+RM0O9FfOHZsz0V9VLIzuoNfaS8mfCWPjyd2n
8OsLhcEFCMpD1SqL/mHsnUSnOKaY+2FvcEq1cpud6J6F3uDueXxeJL4J+Ovq
pxi9lyv59kFCPd3GxZdSynUn791bKow5ttzG18jb8FeBfHNSWDsGPzUmzA0h
H7/xrZVzQUy9/C02thSqpy9yjoYLe6fQeQBHl/C/Kl+Y2EQ8m0P1ThPxLESe
jVTLauTj0T2TVu368oW149DlfGHuWfQzkXIzFf1u/P0OfzKpnD+M/EpOtXKM
7cLX5Jxmzdtx1XId3/qKVNNWZFMC1c570N+WzaiX/Y2ey+FZzQLP6WL4yZFm
321QCbqt8IeTyllZRjPRZ6HPII/Nc+K58Bi91pW8r7FQNfe7QUaz0W10EVsF
8gZkTyU0S9pSmsU+U3wWFDlvmgnL4V+OhP0c/KW0Zo7Pmhbee5rzfuQrkNXF
1Atn+DYuoZ64H9834mM7/FruVKN7C/RMnnSOev0dw0n5/JxzQamw1lskbPSk
5csxUkd+PkLnOfRr4NuRfQg/D/4O+DE5YcBr7z6/DLVjfLdUQGNzmik+S7yn
PVc5vjUn/s+Z5yerWX2lSLVsCHX2mnrv7sXfsqR6eD+xrg0Vq+c0QDY/EHZ6
8zULHfOOdZ+JQzg/HwiLjjnfHV+Eis13yCXO3aF2z3y+lRNLlFUvVCKPiLUp
1Kz0mAuw92Qgmc+INzkfQL4gTzn23qlOy5b30NmUdpbvqgkJzd4/U8qlz2Cv
5dCMsOw13VmmnvZe/oSYHsL+5Zxm6yjsf4Ovt/g2L08+fTdPyGmX+Y72WdRR
qrs+k3wXnAuFPd8JE4lle6hd4Du3Bl+1kWJzm1XYzou0e8/RH6PQH51Vb/iO
9VosCoQ9r8ke+F95T9zna1JY8R3ou88x41j5JaXd6Jhx3d18W5LUnT/SsuF3
fSYOJhfLImHfd7DrnkoLa36nCPnCSLX2nVHB229Oa7b4TPTd6W/w2H2H9iJ/
MFDufGb5v9LlQP8S/s80LdAO8NnvPfE3tqIy9Y7/Q/0HTlJgiQ==
              "],
              VertexColors->None]}]}, 
          {RGBColor[0.941176, 0.906538, 0.834043], EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{8371, 4564, 659}, {4558, 8368, 652}, {5396, 8406, 
              1299}, {688, 659, 49}, {324, 4002, 49}, {4557, 4855, 238}, {659,
               829, 49}, {651, 4003, 324}, {4828, 4831, 651}, {4830, 4557, 
              238}, {4004, 4003, 651}, {4002, 4003, 238}, {1072, 652, 256}, {
              4003, 4002, 324}, {8384, 4828, 651}, {4002, 256, 49}, {4826, 
              8383, 650}, {256, 652, 49}, {652, 688, 49}, {8372, 4834, 659}, {
              5221, 8399, 1072}, {4833, 8371, 659}, {4564, 8372, 659}, {658, 
              1118, 324}, {8368, 4829, 652}, {4827, 8384, 651}, {5265, 8400, 
              1118}, {4003, 4004, 238}, {1299, 651, 324}, {8400, 5266, 
              1118}, {1765, 1072, 256}, {4380, 5366, 1256}, {4012, 4011, 
              238}, {8399, 5220, 1072}, {1118, 1950, 324}, {4832, 4563, 
              658}, {829, 658, 49}, {4002, 4011, 256}, {4011, 4012, 650}, {
              1256, 1765, 256}, {4856, 4826, 650}, {658, 324, 49}, {4011, 650,
               256}, {1950, 1299, 324}, {650, 1256, 256}, {8406, 5395, 
              1299}, {8383, 4825, 650}, {4011, 4002, 238}},
              VertexColors->None], 
             PolygonBox[{{4978, 4832, 658, 829}, {4581, 4833, 659, 688}, {
              4834, 4978, 829, 659}, {5366, 4975, 1765, 1256}, {5220, 4558, 
              652, 1072}, {4975, 5221, 1072, 1765}, {5264, 5396, 1299, 
              1950}, {4563, 5265, 1118, 658}, {4831, 8385, 4004, 651}, {8386, 
              4856, 650, 4012}, {5395, 4827, 651, 1299}, {4855, 8386, 4012, 
              238}, {8385, 4830, 238, 4004}, {5266, 5264, 1950, 1118}, {4829, 
              4581, 688, 652}, {4825, 4380, 1256, 650}},
              VertexColors->None]}]}}, {{}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{8368, 4829, 4581, 4833, 8371, 4564, 8372, 4834, 4978, 
              4832, 4563, 5265, 8400, 5266, 5264, 5396, 8406, 5395, 4827, 
              8384, 4828, 4831, 8385, 4830, 4557, 4855, 8386, 4856, 4826, 
              8383, 4825, 4380, 5366, 4975, 5221, 8399, 5220, 4558, 8368}],
             RowBox[{"-", "5"}]],
            Annotation[#, -5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            TooltipBox[LineBox[CompressedData["
1:eJwl1ElsTlEUwPFrXJWEzpq2XwcLtJS2WBg3ws48tkglLAghpSyMFVsLaggx
xFRaRIIURYoVCxZaQ7UWFppoE1MHpeX3YvHv/55zzzvvvPve16yyLQs3Dwgh
JPjTmRnCgoQQfvJ8ro2F8B7vsHdECHG4aH0/NYQ7o0K4jY6MENrxBUPF9faP
WNdZF4wM4av1N5SiBJV6JGGQujL3GMjreDAX8Q/OSAmh2Pq1OSZzI2emhxBD
p75v1XTxc3O8wCH9spCeFEIOZ7tPFsrjQ0hUu02PeN7KCdEzJYawg+N4Ow/j
Ci63n8I7OIm3czI32/+A4biibzWuoticH9zvmvUI83RxN2Z47mfiHvuTzT4F
N+RvIlWPOj3bXHfe/meucJ87ci3WOXLTxbO4iht4t2v24Jh4irqdnMq7eLT9
XMxGq/MZpa6F0/gj96r5jerofPVvd0Yf7RWZqRiFaDdvfzSzmj6+bs5a1ETv
VTxJzTQu4Im4JH8ZzfrNE7+PvgvxBTzCw+ibkJtrpg5+LJ6l90ycNssT8Xf5
Bj4lni4/A/XR/eQnJodwXN8xGIsz8mfRaG+OuCk6O/E55InHYZFniqU5T3PG
Ofs863z8Fj/V+w+3IV/tBIxHrj5fnMcB+5XYjyR9DnIip+CNdTJne5YYsrDB
zEPk1/NQTtdnrVnWYDWmyjdF750z1b+yf0tdibgTVXKf5O7KjdT7pGtOIBZ9
59EZm/OeZ5jE9/m4vZfqC1x3zLoKR7FKr/nRnPb6XLdC/IdXcj8vjd49L+df
vIx7OVl9Ny8Wd/ES7uFSv5fDXMJxzuW689unttmctdYP/NZazdOCGvFKdZvs
/XX9Ru7ndDMWJvz/X/IP37mo4A==
              "]],
             RowBox[{"-", "7.5`"}]],
            Annotation[#, -7.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4580, 8375, 4858, 5866, 5865, 5589, 6938, 6939, 6937, 
              4824, 8005, 4854, 8004, 4853, 6661, 6660, 6659, 7913, 7914, 
              4654, 6835, 6584, 6585, 7771, 7770, 8419, 4652, 7912, 4653, 
              6834, 6952, 6953, 4851, 8002, 4852, 8003, 4822, 6934, 7964, 
              7963, 4363, 5862, 5863, 4953, 8397, 4952}],
             RowBox[{"-", "10"}]],
            Annotation[#, -10, "Tooltip"]& ], 
           TagBox[
            TooltipBox[LineBox[CompressedData["
1:eJwV02loz3EcwPHPVihs7LAxu5zliWIztmVyLOUIU45kylGUmIgYxQPnttDc
Gcpj90M3kUy5Hrhtc8woTxw5yvH6P3j3/n4/n9/38/ke/3+/hSurViRFxB/U
ZEQ87BuxinOyIvpgf0HEfbEH+JMT8Q9/cUD8pdgrBHqaL0+PSDKutf4rdxRG
rBZLk1vDyWL1chm5EQ2cyevFn/aJeIaDaRHDMUbfMh6ZH1EonqFfOl6LZXIv
DBQfhHKxCq7kvvrlIge1mRF5vIELeRPn80Yu4Im+L7Wvz/qP4nKUYYL5YPVa
0JogL6INub0itlqXx9v4oDplfJhH8SEu5RZ1s+wvG7fx2xlu8XO1XmCVeQ3a
9GrFEffQXTwFqRjq7BXZEePUalJzLB/jBi7Uu54LuBl3cbJ3xHH5KVykTzEm
GU/GCfHxvq/EBExFlZ4zMQNd9F7HJbyWR/ArNUe7gzfOnG9cp149GtBk/VHs
wE5sRx12YbS1tWpU8EbuzNO4E0/nIe6l0fvvw021yvS4weWcKt8Dy9zLKbWW
cjuXyn0z/o7rifcXu8Yt/ES9O/b4wxmT5X/yAWcukRuBX+aP+RGWyC/GIrxL
5NX9YlzCA/QdiH54a96fG60N31XzP/MFnGQ+n4t5Nw/jIgzH5sTvzrku2dtl
XMRVXMFD+zwvdwHnsFe9PUj1zmfNt8i/tP5MYr33n2sPp42r3ds958myz0/q
zBHv4Nn8MfEfEP/As8zfcTveI1W9N/w2cU9oQytSxLvq2Q3z1G5WO1ON/xZb
kaE=
              "]],
             RowBox[{"-", "10"}]],
            Annotation[#, -10, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4362, 7699, 5359, 8303, 8304, 8405, 8404, 5383, 5385, 
              5384, 6191, 5199, 6190, 5382, 7498, 5262, 6698, 6699, 6700, 
              6205, 6206, 5392, 8314, 5391}],
             RowBox[{"-", "12.5`"}]],
            Annotation[#, -12.5, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4403, 5677, 8099, 7217, 7218, 7958, 7957, 4738, 7625, 
              5372, 6433, 6434, 6435, 5976, 7743, 7742, 8107, 8110, 8109, 
              8108, 6680, 8398, 7856, 6681, 8113, 8114, 7278, 6463, 6464, 
              4422, 5732, 4432, 8226, 8227, 8225, 8229, 8228, 5246, 8232, 
              5248, 8230, 5247, 8231, 4433, 7997, 4846, 7996, 7999, 7998, 
              4679, 6465, 6630, 6631, 7804, 4516, 7802, 7803, 7279, 7281, 
              7280, 4747, 7252, 7253, 6439, 6625, 8362, 4412, 4508, 7796, 
              7797, 7249, 7251, 7250, 4739, 7219, 5019, 6416, 7729, 5504, 
              5503, 6221, 6222, 5680, 5679, 4396, 4497, 6615, 6614, 6750, 
              8416, 8418, 8417, 4618, 5319, 8270, 8271, 7928, 4671, 7929, 
              4619, 7886, 8433, 5308, 8267, 8268, 7885, 4616, 6265, 4617, 
              6748, 6749, 6613, 7788, 4492, 5598, 5909, 4927, 6338, 4928, 
              8281, 5333, 8280, 5332, 8157, 5180, 8156, 8159, 8158, 4988, 
              8091, 4989, 7157, 4717, 7137, 7138, 6370, 6610, 8359, 4389, 
              4488, 7786, 7787, 7134, 7136, 7135, 4711, 7122, 7123, 6366, 
              6604, 8358, 4386, 4487, 7783, 7782, 8421, 4663, 7923, 4664, 
              7975, 7976, 7973, 4836, 7974, 5268, 8204, 5226, 8203, 5227, 
              8205, 5224, 7641, 7642, 7640, 8308, 8307, 5935, 5650, 7111, 
              7112, 7110, 4970, 7113, 7834, 7094, 4963, 7093, 4962, 8082, 
              5924, 7612, 5360, 7611, 5361, 7613, 4698, 6891, 6893, 6892, 
              4367, 5645, 5923}],
             RowBox[{"-", "12.5`"}]],
            Annotation[#, -12.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4368, 7703, 6349, 4960, 7092, 4959, 7616, 5363, 7614, 
              5362, 7615, 5926, 8083, 4965, 7097, 7099, 7098, 4701, 6592, 
              6594, 6595, 7483, 8357, 4377, 8200, 5225, 8199, 8202, 8201, 
              5177, 8151, 8152, 7777, 7778, 4484, 4382, 8207, 8208, 8206, 
              8210, 8209, 5178, 7458, 5179, 7457, 7711, 6606, 6605, 6367, 
              7125, 7124, 4712, 7140, 7141, 7139, 8089, 8088, 4489, 4390, 
              7715, 6371, 7143, 7142, 4718, 7159, 7160, 7158, 4990, 8092, 
              4493, 5490, 4992, 7156, 4991, 7161, 5334, 7576, 4397, 5460, 
              4872, 6709, 6710, 6711, 4929, 8335, 5411, 8334, 5410, 8350, 
              8351, 8282, 8283, 6310, 5409, 6335, 6336, 6309, 6308, 5600, 
              7042, 4595, 8377, 7041, 8021, 8020, 4873, 8389, 6778, 4633, 
              7894, 8432, 5282, 8262, 8263, 7875, 4597, 6223, 4596, 6224, 
              6225, 5412, 6311, 5580, 5581, 5413, 7652, 7651, 6712, 5281, 
              5963, 5962, 5696, 6441, 6440, 6415, 5034, 5512, 4509, 7737, 
              7736, 6442, 7255, 7254, 4748, 7283, 7284, 7282, 8116, 8115, 
              4517, 4424, 7744, 6466, 7286, 7285, 4758, 7316, 7317, 8122, 
              8121, 5073, 8431, 7808, 7809, 6642, 5379, 7637, 5186, 8237, 
              8238, 8236, 8252, 8251, 5274, 4524, 5529, 5444, 6211, 6210, 
              5185, 8234, 8235, 8233, 5249, 7314, 7315, 7313, 7312, 6640, 
              4757, 7307, 7308, 6485, 7750, 7751, 5999, 6461, 6460, 6459, 
              5373, 7626, 5033, 7277, 5053, 7276, 8112, 8111}],
             RowBox[{"-", "15"}]],
            Annotation[#, -15, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4369, 7704, 6350, 7096, 7095, 4961, 7620, 7621, 7617, 
              7619, 7618, 5929, 7100, 6894, 6895, 6593, 4702, 7115, 4973, 
              6360, 7708, 6896, 4706, 6596, 6598, 6599, 7710, 6903, 4708, 
              6607, 6609, 7127, 7128, 7126, 4979, 5662, 4713, 7146, 4982, 
              7144, 4981, 8090, 4490, 4391, 7716, 6372, 7148, 7147, 4719, 
              7164, 7165, 7162, 4993, 8093, 4494, 7721, 7720, 6382, 4994, 
              7684, 5448, 7683, 5447, 7695, 7696, 7698, 7697, 5454, 6344, 
              6339, 6340, 5446, 8352, 8354, 8353, 5951, 7577, 4640, 6799, 
              6798, 4930, 7061, 4672, 5601, 7789, 7954, 4725, 7956, 7955, 
              6751, 8381, 6752, 6753, 6266, 6267, 5310, 6343, 5309, 6268, 
              5311, 7732, 7731, 7730, 4673, 8273, 5321, 8272, 5320, 7561, 
              7560, 6754, 8162, 8163, 5182, 7459, 6620, 6418, 6417, 5697, 
              4740, 5978, 5513, 7258, 7259, 7256, 5037, 7257, 5715, 5716, 
              5055, 7287, 5054, 8117, 4518, 7746, 7745, 6468, 7290, 7289, 
              4759, 7319, 7320, 7318, 8124, 8123, 4525, 7753, 7752, 6489, 
              5074, 7321, 4772, 5753, 6519, 6518, 8127, 8126, 7348, 6648, 
              4771, 6921, 5100, 7347, 7346, 6646, 4770, 6920, 5097, 6514, 
              5098, 7345, 4769, 6641, 6919, 6918, 4441, 6029, 4430, 6487, 
              6486, 7629, 7628, 5052, 7310, 7311, 7309, 8120, 8119}],
             RowBox[{"-", "17.5`"}]],
            Annotation[#, -17.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4431, 6000, 8297, 8298, 7627, 7631, 7630, 5072, 7341, 
              7342, 7339, 5094, 7340, 5751, 5752, 5099, 6515, 6516, 6517, 
              5781, 4456, 6922, 6923, 6647, 4785, 6924, 7766, 4457, 6925, 
              6926, 6649, 6928, 7812, 6927, 4458, 5787, 7351, 7352, 7350, 
              5102, 7353, 5537, 7349, 5101, 6488, 6520, 5754, 4773, 7322, 
              5076, 6493, 7754, 7755, 4526, 5530, 5075, 6467, 6490, 6491, 
              5733, 7293, 5058, 7288, 7295, 7294, 5520, 6001, 4749, 5717, 
              6443, 4413, 4510, 5977, 5035, 6226, 5036, 7500, 7501, 7499, 
              8264, 5283, 8401, 7507, 5286, 7506, 7509, 7508, 4814, 6657, 
              6996, 6997, 6999, 6998, 4555, 5896, 5897, 5196, 8177, 5195, 
              6189, 4634, 6856, 8361, 6857, 6855, 7578, 8284, 5336, 7901, 
              7902, 6282, 5335, 8288, 5337, 8285, 8287, 8286, 4641, 5871, 
              7190, 7191, 7189, 5006, 8302, 5378, 7791, 8366, 7790, 8434, 
              5377, 8301, 7633, 7635, 7634, 5376, 7632, 6915, 6916, 5954, 
              5496, 7192, 7193, 6381, 6393, 6394, 5670, 7169, 4997, 7163, 
              4998, 5491, 4495, 8094, 4995, 7166, 4996, 7168, 4720, 5665, 
              4984, 7145, 4985, 7150, 5488, 7149, 4983, 6373, 6374, 4392, 
              5663, 7129, 6906, 6907, 6608, 6905, 6904, 7117, 7118, 6899, 
              4707, 6597, 6898, 6897, 7116, 5654, 7114, 4971, 6359, 4972, 
              5652, 5651, 6356, 6355, 6354, 7102, 7101, 4964, 7622, 7623, 
              6351, 7705, 4370, 5925, 5927}],
             RowBox[{"-", "20"}]],
            Annotation[#, -20, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4966, 5930, 5928, 7707, 7706, 6353, 4967, 5649, 5648, 
              8084, 4969, 7105, 7107, 7106, 4703, 5936, 5483, 5484, 6362, 
              6363, 5655, 5656, 5228, 5485, 4485, 4383, 5659, 5660, 5230, 
              6194, 5486, 5487, 7712, 6369, 7131, 7130, 4714, 5938, 4715, 
              5664, 5941, 5940, 5666, 4394, 6384, 6383, 7173, 7172, 5492, 
              7178, 7179, 7167, 7177, 7176, 5672, 4398, 6398, 6397, 7197, 
              7196, 5497, 5955, 4726, 5671, 5953, 5952, 4498, 7725, 7724, 
              6396, 5007, 7580, 5339, 7579, 5338, 5462, 5461, 6968, 4874, 
              6713, 6714, 4599, 7062, 4598, 7504, 7505, 7503, 5285, 8337, 
              5416, 8336, 5415, 5583, 5582, 6312, 5414, 6297, 6296, 7636, 
              6860, 6861, 6858, 4674, 6859, 5323, 8277, 5324, 8278, 5287, 
              7510, 8266, 8265, 4600, 5979, 4601, 7876, 6715, 5284, 7502, 
              5038, 5980, 5718, 5719, 5057, 7291, 5056, 8118, 4519, 4425, 
              6469, 5734, 4760, 6032, 5531, 7326, 7327, 7323, 7325, 7324, 
              5755, 4442, 6521, 6492, 7357, 7356, 5538, 7363, 7364, 7360, 
              7362, 7361, 5789, 4459, 6540, 5788, 4787, 6072, 4786, 7389, 
              7390, 7387, 5122, 5550, 5549, 6202, 5257, 5786, 5785, 6106, 
              4539, 5548, 5254, 5784, 5783, 6104, 5120, 5547, 5546, 6105, 
              4784, 7381, 7382, 6539, 7764, 7765, 5778, 5779, 5096, 7343, 
              5095, 8125, 6030, 6513, 6512}],
             RowBox[{"-", "22.5`"}]],
            Annotation[#, -22.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5119, 7344, 7384, 7383, 4783, 6101, 6102, 5545, 8363, 
              4538, 6098, 5782, 7385, 7769, 7386, 6563, 5121, 7491, 5253, 
              7490, 5252, 5480, 5479, 7492, 5255, 6691, 7859, 7860, 6201, 
              6694, 6693, 6692, 5256, 7493, 5481, 7494, 5258, 6695, 7861, 
              7862, 6203, 6204, 5260, 7495, 5259, 8240, 4469, 7496, 7497, 
              7395, 5124, 7388, 5125, 5551, 4540, 8132, 5123, 7391, 7394, 
              7393, 4788, 7365, 7366, 6523, 7762, 4443, 4533, 8128, 8129, 
              7354, 7359, 7358, 4774, 7328, 7329, 6495, 7756, 7757, 4527, 
              6031, 5077, 5737, 5078, 5736, 5735, 7296, 5060, 7292, 7298, 
              7297, 5521, 6002, 4750, 5721, 6447, 6448, 6627, 7461, 8169, 
              8168, 8167, 6756, 7562, 8379, 4621, 5322, 8274, 8276, 8275, 
              4677, 6444, 6445, 6446, 5314, 7552, 5312, 7551, 5313, 6228, 
              6227, 4620, 6755, 5681, 4730, 7563, 6621, 5602, 4675, 7063, 
              4931, 6800, 6801, 4642, 7581, 7460, 7222, 7223, 6395, 6419, 
              6420, 5682, 5683, 5009, 5685, 6399, 6400, 4499, 8097, 5008, 
              7194, 7199, 7198, 4727, 7180, 7181, 6386, 7722, 4395, 4496, 
              8095, 8096, 7170, 7175, 7174, 4721, 7151, 7152, 6378, 7717, 
              7718, 4491, 5489, 4986, 6368, 6375, 6376, 7488, 7487, 5591, 
              7486, 5231, 7485, 5232, 6196, 6195, 6671, 6670, 6669, 5387, 
              7644, 4660, 7484, 5229, 6667, 7836, 7837, 6193, 6666, 6665, 
              6664, 5386, 7643, 4659, 5590, 5216, 7481, 5217, 7482, 4974, 
              6361, 7709, 4378, 5653, 4372, 4482, 5482, 5933, 5932, 4699, 
              7103, 7104, 6352, 6357}],
             RowBox[{"-", "25"}]],
            Annotation[#, -25, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            TooltipBox[LineBox[CompressedData["
1:eJwV01toDmAcx/Fnww5OYwdmL7PzsJgY21zIMpvDFdJmslAO7YKyuGByrBFT
trlwyAVTLuXCsRxCUUTIIcqsbUJsYy6U5PNefN/f8/8/v///ef7P+77ZG7at
2BoTQkjysXlsCO3jQng5PoRXKE0PYS7+pYTwRhzJCCEDB1JDKM4KoWJSCEMn
hxCHYfhgLxs56NRrViSEGLXJ9HZyCGPpHRrnjGFYoKZPnCffT3/rmW99I5Pf
eqY+6TwTkIHxaLZ3GPHqK8QD6grU/KSF9Ja9m1hmbynuyqfI36OpdBVdZKYq
JOox6JwE+sl935uxi0517hS0mLNjQgg75a7x9Ex0rlwnTaC99HJaCN00TryL
twXHcY7/iXy/+W/RAfqUXuB/TPvE9XxN2BNV/nbnn0SD9Wn3PGF9ih7kP4Ql
fAvNVolL7lTn/Vv1u69XrZoq+9Wola/BWtQjzdzP9Omy95yuEReqWU0LaB09
pudRfPceeXIN3i4/2tfeG/mIumZvtse7NGE3Sp1dg/nuso+/in8vPYD9qBS3
8n1U34lx5snVp02uHT08vehGo/3t6LA+q+4iLTFzOcqwW12l+iZaYtY0s8xx
7lzrbPttaiK0nebQXe6VRVvFA+4dwx+LqfxJ8oflh9AjdAzdwT+aNou/8P/C
IHL5y507D42+4zHufd493zl7ttxb+lbtO5RF/zNYYf2Xb5O7jjLHRjqCjsRi
VOO6Hrne9qu9bPqF5tAs/R675yMM6jGcNxEJmC6+pm4yXy//JNpDM2lE3XLn
rhfH866jn32XRfa+0T/ir/SM+he8V6yn2YuP9sQP+T5ctS7mLfO7WWneWHGr
/AxzbfE2D9Q8xGtvk6JPkvcpsvcfj/aU0Q==
              "]],
             RowBox[{"-", "27.5`"}]],
            Annotation[#, -27.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4585, 6215, 4586, 6705, 6706, 8175, 8176, 8174, 7818, 
              7817, 8140, 5152, 8141, 7420, 7423, 7422, 4802, 6108, 6109, 
              5817, 5816, 4471, 5818, 7400, 7401, 7399, 7403, 7402, 5553, 
              6111, 6112, 5792, 5791, 4444, 4534, 8130, 8131, 7367, 7371, 
              7370, 4775, 7330, 5080, 6498, 7758, 7759, 4528, 5532, 5079, 
              6472, 6496, 4434, 5739, 6005, 7516, 5291, 7515, 5292, 6004, 
              6003, 6717, 6716, 7513, 5290, 6232, 6231, 4603, 5851, 4604, 
              6718, 6973, 6972, 8031, 8030, 8029, 6782, 7896, 8390, 8391, 
              8026, 8027, 8028, 8024, 8025, 7043, 8378, 4602, 7044, 5603, 
              8289, 8290, 7582, 5340, 7583, 5288, 6230, 5417, 7653, 5418, 
              7654, 4933, 8061, 8062, 7065, 7064, 4876, 6622, 4875, 6969, 
              4415, 7584, 7229, 7230, 7221, 5023, 5505, 7587, 4504, 8102, 
              5021, 7226, 5022, 7228, 4733, 7203, 7204, 6404, 7726, 4399, 
              4500, 5957, 5674, 5675, 5959, 5958, 5499, 7207, 7208, 6405, 
              6406, 4400, 5676, 5949, 5668, 5669, 5948, 5947, 4722, 7153, 
              7154, 6380, 6611, 6612, 7719, 8153, 8155, 8154, 6703, 6702, 
              4582, 5942, 6295}],
             RowBox[{"-", "30"}]],
            Annotation[#, -30, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4864, 5870, 5869, 5636, 7814, 7813, 6656, 4797, 6960, 
              6961, 6958, 4863, 6959, 7858, 7857, 8313, 8312, 5251, 6686, 
              6687, 6688, 6200, 6685, 6684}],
             RowBox[{"-", "30"}]],
            Annotation[#, -30, "Tooltip"]& ], 
           TagBox[
            TooltipBox[
             LineBox[{5364, 6292, 5393, 8318, 8319, 8315, 8317, 8316, 6207, 
              5202, 8305, 8306, 7823, 7825, 7824, 8425, 4823, 7965, 7966, 
              6936, 6935, 4700, 6589, 6590, 6591, 4373, 5864, 6365, 6364}],
             RowBox[{"-", "30"}]],
            Annotation[#, -30, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4590, 7872, 4868, 8014, 4866, 6963, 6965, 6964, 4593, 
              6217, 4592}],
             RowBox[{"-", "32.5`"}]],
            Annotation[#, -32.5, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{5150, 8138, 5151, 8139, 4867, 8012, 8013, 6967, 6966, 
              4591, 6212, 6306}],
             RowBox[{"-", "32.5`"}]],
            Annotation[#, -32.5, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{5155, 7421, 5156, 7427, 5563, 5564, 5153, 7424, 5154, 
              7426, 4803, 7404, 5131, 6544, 7767, 7768, 4541, 6110, 5129, 
              5795, 5130, 5794, 5793, 7376, 5108, 7368, 5109, 7377, 5540, 
              7374, 7375, 6497, 6526, 4445, 5757, 6040, 5743, 5744, 6035, 
              6034, 4763, 5740, 5061, 6235, 5062, 6006, 6184, 6183, 6233, 
              5289, 6315, 5419, 6322, 6323, 6314, 6313, 6234, 4636, 5604, 
              5605, 6325, 6324, 5464, 5465, 7799, 7798, 8428, 4878, 8022, 
              8023, 6971, 6970, 4877, 6229, 5584, 5585, 4904, 5898, 5698, 
              5699, 5608, 4678, 5872, 5466, 7585, 7586, 6806, 7905, 7906, 
              5964, 5965, 5341, 7588, 8292, 8291, 4416, 5702, 7236, 7237, 
              7227, 7239, 7238, 5506, 7233, 5024, 6403, 6422, 6423, 5687, 
              5688, 5015, 5690, 6407, 6408, 4501, 8098, 5013, 7205, 5014, 
              7209, 4728, 7186, 5003, 6391, 7723, 5495, 5494, 7185, 5002, 
              6379, 6389}],
             RowBox[{"-", "32.5`"}]],
            Annotation[#, -32.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4401, 6409, 6390, 7211, 7210, 5500, 7214, 7215, 7206, 
              7213, 7212, 5691, 4406, 6426, 5689, 4735, 5960, 4734, 7234, 
              7235, 7231, 8104, 8103, 4505, 6425, 6424, 5705, 5025, 5967, 
              4742, 7264, 5703, 6449, 7589, 7590, 5982, 5981, 6182, 7591, 
              7592, 6808, 6807, 6805, 4935, 5607, 8067, 5606, 6629, 6628, 
              7564, 5325, 5701, 5700, 4623, 5873, 4622, 7887, 8044, 8043, 
              8047, 8046, 8045, 6757, 8394, 7889, 6862, 8438, 4680, 7931, 
              7932, 7554, 7553, 7888, 4624, 7664, 7665, 7663, 5426, 7666, 
              5425, 7662, 6632, 6633, 4426, 6474, 5742, 4764, 6038, 5533, 
              6041, 6042, 5745, 5083, 5758, 7332, 4777, 6075, 5107, 5759, 
              5106, 6073, 4535, 4446, 7763, 6528, 7379, 7378, 4789, 6113, 
              4790, 5796, 6115, 6114, 5819, 4472, 6565, 6543, 7431, 7430, 
              5565, 7435, 7436, 7425, 7434, 7433}],
             RowBox[{"-", "35"}]],
            Annotation[#, -35, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5016, 5961, 4502, 7728, 7727, 6411, 5017, 7216, 4736, 
              5971, 5027, 5693, 5026, 5508, 5507, 7241, 5029, 7232, 5028, 
              7240, 5706, 4417, 6450, 5704, 4743, 5966, 4741, 7262, 7263, 
              7260, 8106, 5039, 8105, 7800, 7801, 8170, 8172, 8171, 6812, 
              6811, 4644, 5911, 7908, 7907, 6810, 4936, 8070, 8071, 8068, 
              4938, 8069, 4427, 5467, 5432, 6783, 6784, 6785, 6280, 4637, 
              7004, 7005, 8049, 8048, 4905, 8430, 7821, 7822, 7007, 4906, 
              7006, 7009, 7008, 4818, 6316, 4817, 5852, 7529, 7528, 7519, 
              7520, 6500, 6499, 5741, 5081, 5534, 7523, 4529, 7761, 7760, 
              6502, 7333, 7331, 4776, 6074, 4778, 5760, 6080, 6079, 5797, 
              6546, 6545, 6527, 5132, 7407, 5554, 5555, 6547, 5822, 5133, 
              6116, 4804, 7432, 5158, 7428, 5157, 8142, 4548, 6567, 6566}],
             RowBox[{"-", "37.5`"}]],
            Annotation[#, -37.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4408, 6429, 6410, 7245, 7244, 5510, 5974, 4737, 5970, 
              5692, 5969, 5968, 4506, 4407, 7733, 6428, 7243, 7242, 4744, 
              5986, 5041, 5708, 5040, 5515, 5514, 7268, 5042, 7261, 7266, 
              7265, 7594, 4428, 6974, 6975, 6977, 6976, 8035, 8034, 8033, 
              6813, 7909, 6809, 4937, 7066, 7656, 7655, 6238, 6239, 5343, 
              7593, 5342, 8293, 5609, 7045, 7046, 6725, 6724, 6722, 4882, 
              7049, 4920, 7047, 4919, 7048, 4921, 7050, 4883, 6726, 6728, 
              6729, 5853, 4606, 6240, 6241, 7522, 7521, 6719, 6039, 5082, 
              7527, 5293, 7524, 7526, 7525, 6043, 5762, 4447, 6529, 6501, 
              5110, 7380, 5541, 5542, 6531, 5800, 5112, 6081, 4791, 7408, 
              7409, 7405, 8134, 8133, 4542, 4460, 6548, 5821, 6117, 4805, 
              6146, 5567, 7443, 7445, 7429, 7442, 7441}],
             RowBox[{"-", "40"}]],
            Annotation[#, -40, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5030, 5511, 7246, 5509, 5975, 5032, 5694, 5031, 5973, 
              5710, 6453, 6452, 6427, 7270, 7269, 5516, 5992, 4745, 5985, 
              5707, 5984, 5983, 4511, 4418, 6451, 7598, 7597, 5043, 7595, 
              5345, 7267, 5344, 5469, 5468, 5874, 5875, 5611, 8032, 5610, 
              5722, 5899, 4605, 5586, 6237, 6236, 4879, 5876, 6723, 6721, 
              4881, 6978, 5183, 6863, 4682, 6727, 4681, 7565, 7566, 6786, 
              6787, 6788, 6245, 6246, 5420, 6720, 7877, 7878, 6242, 6186, 
              6045, 6047, 6248, 6247, 5084, 6044, 5761, 6530, 5763, 5111, 
              5768, 6077, 6076, 4536, 4448, 6532, 5799, 6082, 4792, 6118, 
              5556, 7414, 7415, 7406, 5136, 7413, 5823, 6569, 6568, 5820, 
              5159, 6147, 5566, 7444, 5568, 6570}],
             RowBox[{"-", "42.5`"}]],
            Annotation[#, -42.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4473, 4549, 8143, 8144, 7437, 7440, 7439, 4806, 6119, 
              5137, 5827, 6551, 6552, 4544, 8136, 5557, 5134, 5798, 6549, 
              6550, 5801, 6087, 6088, 5770, 6090, 4780, 6078, 4779, 6048, 
              6049, 5767, 6503, 7463, 5188, 6046, 5187, 6243, 6244, 6271, 
              6270, 6317, 5427, 6269, 5317, 6326, 8340, 8338, 8339, 7933, 
              7934, 7935, 6327, 6328, 5470, 4435, 7020, 7011, 7012, 7010, 
              4907, 7013, 4880, 5877, 6759, 6758, 7015, 7014, 4751, 5612, 
              4752, 5723, 4683, 5913, 5914, 5614, 7599, 5613, 4645, 5987, 
              6815, 6814, 6009, 5346, 6477, 4429, 5724, 5991, 5046, 5709, 
              5047, 5517, 7738, 4513, 5989, 5044, 5712, 5045, 5990, 4746, 
              7247, 7248, 6431, 7734, 4409, 4507, 5972}],
             RowBox[{"-", "45"}]],
            Annotation[#, -45, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5161, 7438, 7451, 7450, 5569, 6148, 6149, 5826, 6572, 
              5824, 4461, 4543, 8135, 8137, 7410, 5135, 7412, 4793, 5802, 
              5114, 5769, 5115, 6089, 5543, 6083, 6084, 5766, 6534, 5764, 
              4436, 4530, 7555, 5315, 7537, 5316, 5855, 5854, 4626, 7890, 
              6762, 7019, 7018, 7472, 7016, 7017, 6760, 6761, 4625, 7568, 
              5326, 7567, 7570, 7569, 5063, 6475, 6636, 6637, 7749, 7805, 
              4521, 8413, 6634, 8367, 6635, 5912, 4939, 5916, 4940, 7601, 
              5348, 7600, 5347, 6185, 7462, 5184, 6010, 5064, 7596, 6476, 
              5725, 4753, 7271, 5048, 6455, 7739, 4419, 4512, 5988, 5711, 
              5995, 5714, 5998, 5997, 5518, 7272, 7273, 6430, 6456}],
             RowBox[{"-", "47.5`"}]],
            Annotation[#, -47.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5049, 5994, 4515, 7740, 5519, 5051, 5713, 5050, 5996, 
              5728, 6479, 6478, 6454, 7302, 7301, 5524, 6018, 4754, 5726, 
              6008, 6007, 7807, 7806, 8173, 6638, 6818, 6817, 4647, 5915, 
              4646, 7910, 6816, 8072, 8074, 8073, 8396, 8395, 7911, 6819, 
              4943, 7069, 6504, 6864, 7936, 6865, 6790, 6789, 4638, 5881, 
              8178, 8179, 7473, 5197, 5901, 5900, 4556, 5902, 5903, 5577, 
              5576, 5882, 4819, 7539, 7540, 7538, 5300, 7530, 5295, 6535, 
              6533, 5765, 5113, 5544, 7533, 4537, 4449, 6536, 5804, 4794, 
              6122, 5559, 7417, 7419, 7411, 5141, 7416, 5828, 6573, 6571, 
              5825, 5160, 5570, 8146, 4551, 6575, 6574}],
             RowBox[{"-", "50"}]],
            Annotation[#, -50, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4474, 4550, 8145, 8147, 7446, 7449, 7448, 4807, 6126, 
              5142, 5831, 6555, 5560, 7418, 5558, 6123, 5139, 5803, 6553, 
              4462, 5806, 6091, 7535, 7536, 7534, 5298, 6086, 6085, 6731, 
              6730, 7531, 7532, 6253, 6252, 4610, 5856, 4611, 6737, 6985, 
              6984, 4890, 8039, 8038, 6791, 7897, 8392, 8393, 8036, 8037, 
              4889, 6982, 6983, 6735, 4609, 7051, 5615, 8294, 5349, 7602, 
              5350, 6251, 6250, 7658, 5421, 7657, 7660, 7659, 4941, 8075, 
              4942, 7068, 4885, 6639, 6980, 6979, 4437, 6283, 6015, 6016, 
              5727, 5067, 6019, 5523, 7303, 5525, 5065, 5730, 5066, 6014, 
              4755, 7274, 7275, 6458, 7741, 4420, 4514, 5993}],
             RowBox[{"-", "52.5`"}]],
            Annotation[#, -52.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5165, 7447, 7454, 7452, 5572, 6151, 4808, 6127, 5830, 
              6556, 4463, 4545, 6120, 5138, 5813, 5140, 5807, 6554, 5805, 
              6092, 5116, 6257, 5299, 6095, 6093, 6187, 6254, 4608, 7879, 
              6732, 5422, 7661, 5297, 6256, 6793, 6792, 7572, 7571, 4684, 
              6736, 4685, 6866, 5189, 6981, 4888, 6733, 6734, 5879, 4886, 
              6249, 5294, 5587, 4607, 5906, 5746, 5618, 7067, 5619, 4884, 
              5878, 5471, 5472, 5351, 6017, 5352, 6284, 5068, 7604, 6480, 
              6481, 4522, 6011, 6012, 5729, 6013, 4756, 6025, 5527, 7304, 
              7305, 6457, 6482}],
             RowBox[{"-", "55"}]],
            Annotation[#, -55, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5069, 5528, 7306, 5526, 6026, 5070, 5731, 6024, 6023, 
              6342, 4438, 6506, 6505, 7603, 8296, 5353, 8295, 5354, 6021, 
              6020, 4648, 6820, 5622, 4687, 5917, 4686, 5747, 4766, 5620, 
              4765, 7026, 4908, 6763, 6765, 5880, 4887, 7024, 7025, 7021, 
              7023, 7022, 7029, 4450, 5473, 5433, 7673, 5435, 7672, 5434, 
              8342, 8343, 8341, 5436, 6329, 6330, 6275, 6274, 6318, 6276, 
              5296, 6255, 5190, 6094, 5191, 7464, 6537, 5810, 5117, 6096, 
              4795, 6124, 6125, 5812, 6121, 5811, 6134, 5832, 4475, 6576, 
              5829, 5163, 6152, 5571, 7453, 5573, 6577}],
             RowBox[{"-", "57.5`"}]],
            Annotation[#, -57.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4476, 4552, 6150, 5162, 5838, 5164, 6154, 4810, 6137, 
              5833, 5146, 5814, 5147, 6135, 5561, 6131, 4796, 6097, 5809, 
              6538, 4451, 6651, 6650, 7556, 5318, 7557, 7558, 6766, 4628, 
              5859, 4629, 7032, 7033, 7031, 4914, 5624, 5623, 6769, 6767, 
              6768, 5904, 5905, 4913, 7030, 8051, 8050, 4627, 7574, 7334, 
              7335, 6764, 5085, 7336, 6643, 6644, 5616, 7070, 5617, 4944, 
              5621, 6821, 6822, 7606, 7605, 6054, 6055, 5445, 7685, 7337, 
              7338, 6285, 6345, 6346, 5455, 7686, 8355, 5450, 7687, 6917, 
              6028, 5071, 5750, 6483, 6484, 4523, 6022}],
             RowBox[{"-", "60"}]],
            Annotation[#, -60, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4439, 6508, 5749, 6063, 4768, 6027, 4767, 7690, 7691, 
              5451, 7688, 5452, 7693, 7694, 7692, 5453, 6053, 6052, 6341, 
              5449, 6303, 6302, 6304, 6059, 5086, 6056, 6823, 6824, 5918, 
              5919, 4946, 7071, 4945, 7072, 7077, 7076, 4452, 8279, 5327, 
              7573, 5328, 6051, 6050, 7900, 7899, 7027, 4910, 6989, 4909, 
              7476, 7477, 7474, 5198, 7475, 7479, 7478, 4911, 6993, 4912, 
              7028, 4820, 6272, 6273, 5858, 5857, 7544, 7542, 4464, 6557, 
              5808, 5144, 5562, 6129, 4547, 6560, 6559, 5837, 6138, 6136, 
              4809, 6153, 4811, 6166, 5840, 6164, 6163}],
             RowBox[{"-", "62.5`"}]],
            Annotation[#, -62.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5168, 5839, 5169, 6165, 5574, 6159, 6160, 5836, 6579, 
              5834, 4465, 4546, 6128, 5143, 6261, 5145, 7543, 5303, 6558, 
              5302, 7545, 7546, 7550, 7549, 6744, 5860, 4821, 6995, 4899, 
              8041, 6994, 7054, 7053, 8056, 6990, 6991, 6988, 4897, 6992, 
              7898, 6794, 4922, 7052, 4688, 6867, 7937, 6868, 6827, 6826, 
              4650, 5920, 6830, 6829, 6828, 4948, 6825, 4649, 5886, 6058, 
              5087, 6057, 5088, 7638, 7639, 6645, 7810, 7811, 6300, 6301, 
              6298, 5380, 6065, 5381, 6069, 7689, 6068, 5535, 6064, 5090, 
              5748, 6507}],
             RowBox[{"-", "65"}]],
            Annotation[#, -65, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5089, 6062, 4532, 6509, 5536, 5092, 6299, 5091, 6289, 
              6067, 6287, 6286, 4453, 5883, 4891, 6060, 4892, 7074, 7075, 
              8078, 8077, 4947, 8076, 7880, 7881, 6739, 5301, 7541, 4898, 
              6743, 7884, 4615, 5887, 4614, 8057, 8058, 7055, 4923, 5891, 
              5890, 4639, 5894, 5895, 4924, 7056, 7058, 7057, 5625, 6264, 
              6747, 6746, 6745, 5424, 7548, 7547, 5305, 6740, 6741, 4613, 
              6132, 6133, 5307, 6130, 5306, 6260, 6139, 5842, 6581, 6578, 
              5835, 5166, 5575, 6156, 4554, 6583, 6582}],
             RowBox[{"-", "67.5`"}]],
            Annotation[#, -67.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4477, 4553, 6155, 6158, 5849, 5167, 5843, 6580, 5841, 
              6140, 5148, 6262, 6263, 6143, 6141, 6188, 6258, 7883, 7882, 
              6742, 5423, 7678, 7679, 7677, 5441, 8345, 5443, 8344, 5442, 
              7676, 5440, 6869, 6870, 4689, 6334, 5474, 7469, 7470, 7466, 
              5192, 7465, 7468, 7467, 4895, 8040, 4896, 6987, 4893, 5892, 
              5893, 5588, 6738, 4612, 5907, 5771, 5628, 7073, 5629, 5885, 
              5884, 5475, 5476, 5355, 6066, 5356, 6288, 5093, 7609, 7610, 
              6511, 7608, 4440, 4531, 6061}],
             RowBox[{"-", "70"}]],
            Annotation[#, -70, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            TooltipBox[
             LineBox[{4454, 7607, 5357, 6510, 5358, 6071, 6070, 4651, 6831, 
              5632, 4691, 5921, 4690, 5772, 4782, 5630, 4781, 5908, 4916, 
              5774, 6772, 5889, 4894, 7036, 4915, 6986, 7035, 7034, 7040, 
              4466, 5478, 5437, 6333, 7938, 6873, 5438, 7674, 5439, 7675, 
              6331, 6332, 6278, 6277, 6321, 6320, 5304, 6259, 5193, 6142, 
              5194, 7471, 6561, 5846, 5149, 6144, 4812, 6161, 6162, 5848, 
              6157, 5847, 6169}],
             RowBox[{"-", "72.5`"}]],
            Annotation[#, -72.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5170, 5850, 6172, 6170, 8451, 6167, 4813, 6145, 5845, 
              6562, 4467, 6655, 6654, 7667, 5428, 7671, 5430, 7668, 7670, 
              7669, 4632, 7559, 6871, 6872, 6774, 6874, 7939, 6875, 6777, 
              7893, 6776, 7037, 8053, 8052, 6771, 7891, 4630, 5888, 4631, 
              5773, 5775, 5329, 7575, 6652, 6653, 5626, 7078, 5627, 4949, 
              5631, 6832, 6833, 7082, 7081, 5776, 5633, 5634, 6291, 6290, 
              5477}],
             RowBox[{"-", "75"}]],
            Annotation[#, -75, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4692, 5777, 8442, 5635, 8441, 4951, 8440, 5922, 4950, 
              7080, 8446, 7079, 8458, 8457, 8459, 4468, 8346, 8347, 7680, 
              8449, 7681, 5330, 6281, 8439, 8462, 5331, 6773, 6770, 8444, 
              7038, 8448, 7039, 8445, 6775, 7892, 5861, 8443, 6319, 5429, 
              8454, 8453, 8461, 8460, 4478, 8455, 5844, 8447, 6168, 8450, 
              6171, 8452, 8456}],
             RowBox[{"-", "77.5`"}]],
            Annotation[#, -77.5, "Tooltip"]& ]}, {}, {}}}],
       AspectRatio->1,
       BaseStyle->{FontFamily -> "Times", FontSize -> 12},
       DisplayFunction:>Identity,
       Frame->True,
       PlotLabel->FormBox[
        "\"\[Mu]=10 \[Alpha]=0.50 \[Epsilon]=10 PIN=0.20\"", TraditionalForm],
       
       PlotRange->{{0, 50}, {0, 50}},
       PlotRangeClipping->True,
       PlotRangePadding->{
         Scaled[0.02], 
         Scaled[0.02]}], {192., -576.}, 
      ImageScaled[{0.5, 0.5}], {360., 360.}], InsetBox[
      GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyUvXVgVUfzPxy0UIKFJFigCRAgEEKEuNy9mgQI1hYvFFooDsWKu0OKeykO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        "], {{
          {RGBColor[0.293416, 0.0574044, 0.529412], EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{7468, 7469, 3064}, {5034, 6589, 924}, {6853, 8266, 
              4012}, {4252, 4226, 962}, {4237, 4230, 721}, {4265, 4266, 
              4012}, {4243, 4267, 1287}, {4263, 4262, 3579}, {4269, 4268, 
              4116}, {5312, 7559, 1287}, {4791, 6943, 696}, {6971, 4816, 
              721}, {4238, 4242, 924}, {4116, 4117, 1287}, {7936, 4553, 
              1511}, {6972, 6973, 2648}, {4245, 4244, 1511}, {6250, 5384, 
              495}, {5769, 5081, 962}, {5033, 6700, 924}, {7440, 5205, 
              1148}, {7558, 8291, 4117}, {4614, 6250, 495}, {5826, 4791, 
              696}, {4555, 5825, 408}, {4254, 4255, 2406}, {6700, 5034, 
              924}, {5534, 4555, 408}, {4231, 4233, 495}, {7945, 4814, 
              3579}, {4229, 4239, 408}, {6943, 4790, 696}, {4554, 7442, 
              2406}, {4249, 4243, 1287}, {5694, 5695, 1652}, {4230, 4258, 
              721}, {7946, 5694, 1652}, {5204, 7440, 1148}, {7559, 6703, 
              1287}, {4234, 4229, 408}, {5080, 5769, 962}, {5205, 5535, 
              1148}, {4224, 4238, 924}},
              VertexColors->None], 
             PolygonBox[{{4816, 7946, 1652, 721}, {4267, 4269, 4116, 1287}, {
              8309, 8311, 4228, 4225}, {5825, 5826, 696, 408}, {6704, 8312, 
              4248, 4253}, {4239, 4240, 1148, 408}, {4790, 7936, 1511, 696}, {
              8291, 5312, 1287, 4117}, {8314, 8313, 4250, 4251}, {8315, 4815, 
              4236, 4257}, {8313, 6854, 4256, 4250}, {4258, 4259, 2648, 
              721}, {6973, 6971, 721, 2648}, {7318, 8314, 4251, 4261}, {4232, 
              4237, 721, 1652}, {4235, 4234, 408, 696}, {4553, 7468, 3064, 
              1511}, {4244, 4235, 696, 1511}, {5535, 5534, 408, 1148}, {8290, 
              7558, 4117, 4116}, {7442, 5204, 1148, 2406}, {8310, 5525, 4246, 
              4227}, {4240, 4254, 2406, 1148}, {6854, 8316, 4264, 4256}, {
              4241, 4245, 1511, 3064}},
              VertexColors->None], 
             PolygonBox[{{8311, 8290, 4116, 4268, 4228}, {8316, 6853, 4012, 
              4266, 4264}, {7469, 7261, 4260, 4241, 3064}, {8266, 8309, 4225, 
              4265, 4012}, {6703, 6704, 4253, 4249, 1287}, {6589, 8310, 4227, 
              4224, 924}, {8312, 4554, 2406, 4255, 4248}, {5081, 7318, 4261, 
              4252, 962}, {4815, 6972, 2648, 4259, 4236}, {7261, 5033, 924, 
              4242, 4260}, {4814, 8315, 4257, 4263, 3579}, {5533, 4614, 495, 
              4233, 4247}, {5695, 5533, 4247, 4232, 1652}, {5525, 7945, 3579, 
              4262, 4246}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.31434728798751604`, 0.09379825700195585, 0.5588334573873578], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtU01sTGEUPTNlpu91Kpk3v30zo2NNsfRTumBlCPETCbHQEmxKIn5KY2Hh
byNdaGOBjY2VjbKSmCZIu2hnSzJVicSqYmElEs5x3uLm++5995577rnf2zB6
8fB4GsAJWg8tGwFTVeBnCnhaAp7RshngYQBkeM6l7K/0OadRA2q0bSHQZewL
bYbxOw1gtAnsywFnWdu71vHpOvCpAJzvcyxI4geZt7DOGKq5zPMzCZ2MyS12
f31bTvqqfze5T7DPDdppDtJDbn97gQO8z5WBtoyxVs64wlSvbsJTHFpJ/F0R
GGva38vvY4FxzgS+T6fMV/Wq1Vz7c8YIB4EOtfvTD1wlxtsB83hP7pP0VwPz
eEO82bRnXU5meMW637Hj5wLrq7xh6rrEb520+7SSXrOMvSw7/yjvMyXz3Mn8
XbTjoXcYlqxPnrtor3e/K9pJ0/rOcd57Dc8kjh9i89zKc0vs+bTDa8keF/LA
9aZnP8U9DrF2D3FuMraY8BRf+bpvJsZQbP2ekMtI2W/oFms/Jr0C6vao5L0d
yllb8ZFW0kI6HCl6BvHfxHM+713sCJ0nHW5XPY9qL5HD94rfg3agXUqf/xpH
iW7MjRp+r9qt5hDmi4Jnkz/O3Mm6cYQnX/evzJmv+M1I+2OR8TcSP64Z7xs5
TlT8vzxgn/sNz6d5tF/tZYl6dGiL9B+XXKP8gZr3Kszn7H9h0PuSRsqVtpnI
mkrPEc7eLvptDBNvTeTcVfb/QZvK+l3qXQg/YF3AXneZvz20jtLkV7996aB/
up7815p1peB5de6u+g7yeh257h/jRpkq
              "],
              VertexColors->None], 
             PolygonBox[{{5825, 4555, 2407, 1729}, {5384, 6250, 2037, 1415}, {
              4816, 6971, 2647, 3580}, {6700, 5033, 2883, 2403}, {7319, 4720, 
              609, 2920}, {7440, 5204, 3039, 3037}, {8134, 5203, 3038, 
              3786}, {6701, 6699, 2403, 2404}, {7469, 7468, 406, 2883}, {4693,
               5692, 1539, 589}, {5202, 8134, 3786, 3151}, {5313, 6274, 1289, 
              1288}, {5532, 5249, 2037, 1542}, {7560, 7562, 3152, 1289}, {
              7873, 4615, 496, 3038}, {4814, 7945, 1539, 720}, {5386, 5387, 
              695, 496}, {4553, 7936, 3572, 406}, {7561, 5202, 3151, 3152}, {
              5525, 8310, 2303, 1539}, {6938, 6937, 589, 2403}, {6589, 5034, 
              2884, 2303}, {7559, 5312, 3147, 3148}, {5694, 7946, 3580, 
              450}, {4790, 6943, 2617, 3572}, {8312, 6704, 2405, 407}, {7442, 
              4554, 407, 3039}, {8314, 7318, 2920, 609}, {5806, 6702, 2404, 
              406}, {6746, 5532, 1542, 2447}, {7292, 7294, 2541, 609}, {5387, 
              6941, 2616, 695}, {5807, 6942, 2617, 693}, {5695, 5694, 450, 
              1542}, {5530, 6970, 2647, 720}, {5840, 5530, 720, 1742}, {6936, 
              4788, 693, 695}, {4555, 5534, 1543, 2407}, {6250, 4614, 1542, 
              2037}, {6855, 7332, 2931, 2541}, {4612, 5839, 1742, 1539}, {
              8311, 8309, 2931, 3146}, {6009, 6747, 2447, 450}, {6942, 5806, 
              406, 2617}, {6206, 6207, 1288, 986}, {6703, 7559, 3148, 2405}, {
              8041, 5089, 986, 2931}, {8291, 7558, 3146, 3147}, {6854, 8313, 
              609, 2541}, {5535, 5205, 3040, 1543}, {6943, 4791, 2618, 
              2617}, {6973, 6972, 720, 2647}, {5205, 7440, 3037, 3040}, {4791,
               5826, 1729, 2618}, {6940, 6936, 695, 2616}, {5249, 5385, 1415, 
              2037}, {5034, 6700, 2403, 2884}, {8266, 6853, 2541, 2931}, {
              6970, 5529, 450, 2647}},
              VertexColors->None], 
             PolygonBox[{{6126, 5770, 5769, 5080, 1937}, {5770, 7319, 2920, 
              5081, 5769}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.3415984358930775, 0.14118064362841068`, 0.5971382424220034], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1U0FsTFEUvZg/o7+DGPP/n/kz8+miRiLEpiRVJBY20i7atIuy0YSkG6Qs
UJFq0hgTQWgimhGVWEkQydR+KBYWJCpiSWjYdCVIauGcnNfFzXv/vXfPPefc
+ztGTvWfXG1mJxBrEK83m/XnzcZTZs9xMZc2m8E6WTBr4dxfa7bVM5tCtPB+
Nmf2LNSbvqzujyE3xHpvnVkV73px3ukJsy0wuwycJvYfMvh276eB/2mj2eNV
ZknJ7Gmo2k9CfXOfQq4XqDYxGoFwisgbR3wGnwPA2Y9YAPZtrLcQEWrcwHoT
EWD/C3ce8JZQy5B3FNHEdhT3E3ndP0LddKA3GWhsJdJzF2fdJeV3481exFha
OohNnds86abeRkY6qZHeLLj9QdR6tQnewfQjsdn5CP7hrgfn+wrK+Yj7Hzi/
nhEufVzxlvjsA3V/jaT9CrjMO8xrWA9VhEl9Y5E0/sO6HIkXfVzMqo/zyH2J
eAEtv6H1ATR/x91EQb1hX2qearPuUFa+cn8hUj3WqgNjCd/TwPmCs1HM08B6
8aJmctsZq6/s6a4NwhrMao7OBpqlNucl9Q7kNZf0kzm7fb05vsXsLe7OoS9N
8C2gL12+vCEeufFtyb0n3ynnG2dnONb8PCybvYnFjVjkxHzqZy796QG3QdQ4
k9bdkMPvSOQX336D7loo7fSRc8F/6BLwLyKG29XDPtfHGXB4H2qWWJe49Jk6
yk7LdvDaEQunjn7WKsLgP3q1Ik/IscvxrCCvPVCvOGsNN29VN5fE/wO+fxE/
UfN+Ih9Zf2V2ienl9A9w/smr6nJ53+n2y7H+f3pbc76SA70gNnmdxv2dsjxZ
dJ7yDTn6judsIh7kUAT/GLHHl39cqZ0YdeftJLQfLqp373CWyokn/0t6Sj//
A993oSE=
              "],
              VertexColors->None], 
             PolygonBox[{{7928, 6908, 2587, 3561}, {5324, 6291, 2071, 3158}, {
              7561, 7562, 3149, 1147}, {7494, 5250, 1204, 1870}, {7441, 5203, 
              1147, 1319}, {4769, 7927, 3561, 3301}, {6475, 4352, 180, 
              1204}, {4567, 7840, 3448, 429}, {8041, 7332, 945, 3667}, {5061, 
              8040, 3667, 945}, {5248, 5531, 1541, 2036}, {5314, 6274, 2058, 
              3150}, {4789, 6906, 2587, 2614}, {6273, 6272, 429, 1317}, {7714,
               7713, 3301, 3302}, {5323, 8227, 3888, 3448}, {6290, 5380, 1413,
               2071}, {6746, 6747, 1870, 1204}, {5313, 6207, 2004, 2058}, {
              6940, 6939, 193, 2613}, {6938, 6699, 694, 2615}, {7873, 7441, 
              1319, 3485}, {4692, 6937, 2615, 1650}, {6908, 6907, 666, 
              2587}, {5381, 7714, 3302, 1413}, {6289, 4615, 3485, 2070}, {
              5827, 5690, 1650, 1730}, {5693, 4613, 494, 1651}, {6906, 4788, 
              2613, 2587}, {5531, 6745, 2446, 1541}, {7293, 5767, 945, 
              2906}, {6126, 6125, 217, 1684}, {6119, 4720, 1684, 1934}, {6701,
               6702, 1717, 694}, {6114, 6116, 1684, 217}, {5807, 4789, 2614, 
              1717}, {5492, 5493, 694, 666}, {6745, 6010, 1870, 2446}, {4612, 
              5692, 1651, 494}, {7560, 5314, 3150, 3149}, {5386, 6289, 2070, 
              1416}, {5768, 5077, 1934, 1684}, {6206, 5088, 1948, 2004}, {
              5840, 5839, 494, 1541}, {6855, 7294, 2906, 945}, {8227, 5324, 
              3158, 3888}, {5382, 6249, 2036, 494}, {6009, 4580, 2446, 
              1870}, {5307, 6287, 2069, 2057}, {6139, 6204, 2003, 1948}, {
              6939, 6941, 1416, 193}, {6205, 5307, 2057, 2003}, {4580, 5529, 
              1541, 2446}, {6288, 6273, 1317, 2069}, {4361, 5385, 1204, 
              180}, {5493, 5828, 1730, 694}},
              VertexColors->None], 
             PolygonBox[{{5528, 5693, 1651, 4693, 5527}, {6120, 4719, 4718, 
              6119, 1934}, {4719, 7293, 2906, 7292, 4718}, {6090, 6140, 1948, 
              5088, 6089}, {8040, 6090, 6089, 5089, 3667}, {5691, 5528, 5527, 
              4692, 1650}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.36884958379863897`, 0.18856303025486554`, 0.6354430274566492], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVDtMVFEQHWC/uAvs/+0u+0ErsdIOlMY1a8TEws4WE4ONGFglooTFQoif
gK6FCcFoYonRBJMVNcZEjUYt/YRWLWisNS4xnrPnFpM3d+7MnLlnZt7A2MTx
M51mdhbSBfmcMKunzV7h4IXMQpBBv9mJnNlKymzdZ1aJyDYGvQHbbUhnwKzV
bxaEXzVsNpc0O5+RP33/+eRfw90mcv/qwF2P7HXo857Zsie8N8A/h5w/EDcJ
/62oYsLIHYIchi3bJ0zm4dkHfQ76Hr/qJmauT+dTPul5d97tVzxjq1mzvWWz
K7AtAHcWb2ggVzClO9oZx3jGXQM/B3G30KUctDP/J7x3V1k+zDfuKfZCVPUQ
91C33k/cKN7xLGY2ivMIcO+k9Aa+KetybsPnT9GsH3EvkH++gBzgPon89YL6
sI7Yr8iz1iHOao63mYx6Rv6Zk9jEfRA3+4s3rqLpU2Hxy1hyTH0F9oqnvjKO
PBKH9RhwjoKTRyH1o+Zil2G7CdkMaXZuZDU/9OuNiKvLqL9a0PkLOFx0M3Ya
WIGUuCIH+9PioQe1D+VVZxPnJwHpQ8hjBc0SbeSGNbNvE3H1rpUTd7wjF+15
RJ07YItAjuC8xN6m5V/pVh+Y5x74/hZT32aD8uV8vgSfGzHp5KnicBnHOtq8
RdSfNleoMVFWncPAeZ1WDdOeZoicsC7Wx3e9Q/73kEvAnEENFyHHUMN4yext
QrXUPe0Jd+Qh8vXmhD0A351F9YT94j33inPxOK0eXU0Im7jc43pSs0GOOROs
YRv4HzLCIpdbGfHDOlkL69+X1x5y5zj7WbcXnIXpqLCmELcIvFHk+Qn9O+Q6
dB9wb3maw4annWHsAeitjLjlDn109pMp/Y84J6t4392iMJbg/zypN5K/ppuN
37i/H1cveD/iyYfc+FPi52lJM8S+TLp/0Zr7bwy6fwVj+G+ijz+u/eRMkhv2
g/w08d0oyW+Ys5rTPv0Hhmeviw==
              "],
              VertexColors->None], 
             PolygonBox[{{7839, 7840, 3449, 1323}, {6713, 6712, 2418, 2419}, {
              4567, 6292, 2072, 3449}, {6288, 6287, 1278, 2056}, {5178, 6091, 
              1917, 2006}, {7418, 7419, 2374, 669}, {5004, 6010, 1203, 903}, {
              5323, 8293, 4123, 1321}, {5181, 6211, 1538, 1126}, {5691, 5690, 
              383, 1540}, {7926, 7713, 985, 3560}, {6006, 5002, 903, 1203}, {
              6961, 6962, 2375, 383}, {6088, 5178, 2006, 1916}, {8293, 7839, 
              1323, 4123}, {7803, 6961, 383, 3406}, {5808, 6714, 2419, 428}, {
              6541, 6543, 1683, 960}, {5442, 6474, 2223, 903}, {6663, 7802, 
              3406, 2374}, {7838, 5811, 428, 3447}, {7552, 5308, 3140, 
              3139}, {6715, 7837, 3447, 2420}, {6292, 6272, 2056, 2072}, {
              7553, 5181, 1126, 3140}, {5061, 6210, 2006, 1917}, {4537, 8119, 
              3763, 384}, {5248, 6249, 1414, 1203}, {5768, 6116, 1932, 960}, {
              6248, 5247, 1203, 1414}, {7928, 7927, 667, 2588}, {4768, 6907, 
              2588, 1716}, {6671, 6670, 669, 1716}, {4767, 5383, 1414, 
              1714}, {5802, 4613, 1540, 1714}, {5381, 5380, 493, 985}, {4797, 
              6669, 2378, 2627}, {4798, 6715, 2420, 2628}, {7494, 5004, 903, 
              3087}, {6290, 6291, 1321, 493}, {6210, 5060, 1916, 2006}, {8121,
               8120, 1122, 3763}, {6668, 6078, 384, 2377}, {6091, 6667, 2377, 
              1917}, {4769, 7926, 3560, 667}, {5524, 4798, 2628, 1538}, {6120,
               5077, 960, 1683}, {6115, 5475, 960, 1932}, {6712, 4797, 2627, 
              2418}, {5060, 5767, 1683, 1916}, {5766, 6087, 1916, 1683}, {
              6139, 6140, 1917, 384}, {6205, 6204, 384, 1278}, {5526, 4767, 
              1714, 1540}, {8211, 8212, 3139, 1122}, {5492, 5809, 1719, 
              1507}, {6664, 5526, 1540, 2375}, {6475, 5250, 3087, 2223}, {
              5809, 4768, 1716, 1719}, {4538, 5805, 1716, 2378}, {5827, 5828, 
              1507, 383}, {5382, 5802, 1714, 1414}},
              VertexColors->None], 
             PolygonBox[{{4445, 6003, 1866, 6002, 4444}, {6545, 4476, 4475, 
              4377, 2265}, {4476, 6115, 1932, 6114, 4475}, {6474, 4445, 4444, 
              4352, 2223}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.39610073170420046`, 0.23594541688132037`, 0.6737478124912948], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVFtLlFEU3Y6X8ZKXcRxn/MxLoT+gy8NkRVoPSRmokKURgRIEmWJmSQVm
hFEgUkQQmEFRD5UQauNM6YhFGIT4ZPYu9VCPvhlpa7FO9HD4zrfPPmuvvfbe
Z1t7d3OXz8xuY6VixfPMprE208zOF5ht+M0asKbzzS7gP4bv4RL8Y41nmu0q
NduN1ZRt1lmgO/RfyDF7C5xP+Fbh/H2hWR18dmK/A6sR+69Bs1vFZvMI3BEy
Ww2bLaebTcI2Vm6WyDD7mavvGEimAiOKuykZijXg4vVWml2q1H+uZzYLvyT8
a4rMPgOzI00xr5Urbk8AmLAvmvhNgWd9ltk6zp8XKhbz7HS5DOMsjphLPulC
fW6mmLUA62iW7BcRv7tSfH4Buz2kXO6HxJucG7J09yww9oJnEvY5n2xxZ4+C
c6zCbAL/RcALYm0Hl9psYQwi7jI49fuVF3WOu3rFcS9RoXxiWIm8/+cJh/8U
Mf8g9ijifglLI+LURRAjIuwozvd40nAR/ic85dgGHq35qnsMXHqQzyzsG8j5
FDSdNGkfdXcPAW8/8FeAWZ2je49gr0J969E/Q6j7E/g/DMl+F/1wOSzNqf+a
X5jsu07XewPAOxfRWQHiBDzpGnO5sp7k0h/R3QHEKgio1ptbzfYVSQfyGiwT
tzxg5HuqJWtBPNZ3AVyGinU3if0WT7rX4PumWBokXO0Y+whyOohcZtOVU6an
vFrwfQX/Sfgv+vSlnbbXzk69iMe6cCZWgpqL74j7A2sYcV+C7xXsj2HvD0k7
4ozD/11AvcE6MQ/u54ExFxbnNacptZ1w/UGtmtBfXZWa7/6wYhB/BLkMY31D
zabZVwHNCHvnWaF4cr6nHA57lBqwD2eg7Y0y7evAsy8sTWZwb9NTzzBn9gT5
k8uE689/M0+fhONJO+eP87bkdOI5OVC3A9nK905QvcW+Ook4qwHN7umAakMO
Lwo155xxYn4AhzPAvwqb39OePUHu7IGRYt2hv69MPcf+IE/OLu3XgX+vRPNO
Hakze6+6VDmTa4+7wz37scf1JOvS6vxZh76IMMiDbyr7he/Eg5DiPS7X28F9
inuTmHtvWHkyR76z1GTUaZN0WjFXYhL7N2ZhHetjjnIgHvPlP3WhD3m1OW6N
7t6iw6x1mvPNZv7MvRm8jnvyyfH0vvBt+Qs66PhV
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlMtLlGEUxo+jOerkjI7OjH5jOV38A7IWGkFaUIEFKWTWJlCCoIuVeaEW
Zgu7oV03QbnosksILa+pWIRtpE1m+2hTy1YWaL/D4+LA+37vc57znOc977el
tb3pQsjMGogc4m6JWXeZ2WrY7Ahxk/3BcrPhPLOqiM7u8O1wWJgfxWYjSbOJ
XLOBfLM9pWahTZyzPg1he8ZsqFTYs0UU4KyW/buo2QJ8NWmzxbjZF0RkwTFI
rV74FoF+p+YB9uGE2ZOQNHjuRKXZ26g4rm02u0qcotbRArM35H5D0yXqj8G3
Fpg9h/8p+TPEZMpsmmjLUY0+9LyHc439MhqPg28J1Itr6ofvXrm0OH4S7lny
e8AeAtPFupf8P2F5si8hD927mQ3g0TpeKaxrdm2jcE7mSmMt/Uep18C6Gf0v
0Lqdb/UF0rwjIw+89xMxs1b4f1JzAP4l+F+BX6H/X4X4DP4R5/vL1MuNLLyD
+1+F2ceIPLpOv6Vwbo2p52OcNxOj6Fkkfwq+hwndRR0axqldDf5kbN1/sLsD
eTsLfltaOY7dS0zRX2Ggsz7qnylTD67dPepGe31C3rhHL+PywHv3GRp0v5Pq
xfX77PSkhPUZugLX5YxmqZfogr8jo/U5oiSjGfPZ8h4L0LJK/58iuoM89vmB
evWZ6iT/IjnnyR0Hv8zd/U7J2/lsnXkN53ZMD+vGjLCjlOoEe5uc/mz19Dqp
N+NvxT29xbojpTPH+F1VpTWLfmcxtAwn5ZXPQHEgD7x3f0PTcXns3vpMDJH/
jJgLaYa8txo8Gomqx8/UWkhpVn3GN8IXCbT2mX2c0Iw4v3P8rdAduPc+Iw+4
7/vl6t09GGN21yr0lr2Ge1ed1ltzD3exblr3s5FvK2B3prX+ENHsrBbJK5+h
uZTuxO/C9XzFn7aEavlM+9usx+OlsDT7v8U5ncs11nE2X6Jcx/wHuqOz3Q==

              "],
              VertexColors->None], 
             PolygonBox[{{5761, 6542, 2264, 6541, 5760}, {6118, 5761, 5760, 
              5076, 1933}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.42335187960976195`, 0.2833278035077752, 0.7120525975259404], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVE1IlFEUvaOWOirNfKMz44wzjlSLNiJWKtQiXaQSFIlm0CYsN4Y/mJoV
FP0jiVbLFgWFC9PKWhSlkIIEUbYqrI2BUZC2LqLUzuk8aHGZN+/de+459+cr
aels6EgzswuwdFhlzKwC1plt9iVi1lJgdhgOe6NmvZ5ZK85DYbMgfG6uNyv1
myVgi4gtducxn9kA4i/mmU3jvACcY55wHiP2c9BsBvdl8F3KVWxtodmVkNkl
kKjKN1stgn+G2WjS7F5SWC+Bcx7x15H3arZyEP8tcAdx/o7zaMJsHNYB3BK/
OJLzWqbZR+B9gL3KARbsKM418FkGdgq/a8j5pFhv94GxAB5dwEni7WeeeAbi
Zu+D0kitSad91mGS8wRiPyG2G7FTsN9Zyt3jqY7kQ17Mz/P+lFl/VByboLEW
fjW434pcv4pUq3ycLS6e7EMortjbyDMSla5zqM3XiOrD/v3IUU2m0tQbcqaO
Gsd5Dv6LQdWwEf3sQmwdOAwVCos4O4A/GBbXTch51xPWJHyfwfrg3wqfLQnl
Yz9LHT7zpNx5Gb7z6O81+G8AzoQnP+rjTFCjHxxeRFTDG5i75564t+Punas5
/3e6XhO7zOGfDYs3OT9CnBfXWzlq+7RQPajH71rA7AHOf5DXl1DfmnFXDt4r
4HYQHJpjmif2c9bNyXBYc8++VUeln9oPwLcppnm8g7xrMc0E55d6qGUjuEw6
LWPIudv1lzsxG9FerNugWZvP0B6Mh7ULK0XiSh6bOQ9J1ZW+Hc6/D3y6UuI/
B8wq+DXBZ59fO8LaduO9B3YmIN/eqPzZ88sh9f0IcvaiRg+Rt9HFc7c4Xx0R
zVglsCvieh9AXC/u92RqJ9gn9mg6JCziVKOPJyLSS91N7sx6kws5pKEmO/O1
l9Q34kkjc5MD+a8G1Ev28VCBchCfO8Md435RQ32m6taA2m+LS38/NJ2Evc7S
u8/tYz+43MpXDTlL/F5wp9o9fbPYl3r33SBmOvK+8cSrzuXh7rQFNEPUwm9J
iZv74yn1hver4BOM65tB7fRnPYlNTcSagaZdUemiD2eC3Pgd4X/6k29nSvec
UfaZu8Bd4Zxxxk4B43RUOpmHNWIsuW+Piz/v29wb9+nfbPrUk/Lc//j0Zf2/
oR5LsGH4/wWNWOwO
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtk01I1GEQxmdXa2013F133S9XVyioS0QfKtQhPWTRITKtwItIXorWst2y
goKoIBL6OHYJigJdI7sUVFCCCCF2CqpLQVGH9FxgrPYbHg/Df953Zp55Zt7n
3zo43FMImtlerBp7HTPbkDVrCZu9IbCS4cvd3/VmzdztjnPXZDZba/Yei5G7
qdnsd53ZVuLBhFkV9oHaxYDZRM5sNGn2CfACOYfBGyf/Gng315EL9r2Eenn+
aMqsP8qZ3JWQ2WPiS+QvkN8F/jf6nwRzG/ERbLLRrIy9o/Z6ldlbek03mD3D
H6hW7lKL2Zp61UyANb+a7xwmiZexrqD4vSC3I65an7GB+eJYnt5D5BTzZmex
CtwuR8wicL0C3t21ZnM1Zhfg/4D6z9R/8ZnJPZTXLCci4vKUfqfrxOkXfH9i
A0Fh+KxPmLk1rJl9t/X03xLWjm8w2xny94O3DyvgT+W0C+8ZwN9F/+P4M/Av
pcTBezvnKPuPZBS/T7+dYLdjvWG92Vdqx3PahXP0t9+Y1e5dA+eTelOP+85K
nDsT8g+AP4i/wN2dkGbaQ/9O7FxIb/KKWH9Cvt8Nw20kr126RkppeqS1f8/5
zn57M5rV36wNLnNwOhgW56PE5jn34Y+hp46sNOVa8rtu/GJS3Jyj17oGXXuO
8YhzhXMuLM37boI5adt3tDknTTven1pxqzRpt87xJefutLT3j/f/CN92MIbB
L1MzhjaKMWnH8Qx+z2P6V1xTUc6RrHKdww787dh0QBxn4D6blDZ8n66N2416
O9eIc38Y07/p9VP4y6vzuGb8X1qOiJv/U33Eelbnv4VdBWs0Le16j0spadi1
G6iR1i6m5Lvm/F87klGtv8cP8I9ltHvn3IY/FNeufAeL1J+Kibtr4j8UZaAY

              "],
              VertexColors->None], 
             PolygonBox[{{6004, 5669, 5668, 5001, 1867}, {6521, 4466, 4465, 
              4368, 2251}, {6122, 5765, 5764, 5078, 1935}, {5669, 6473, 2222, 
              6472, 5668}, {4466, 6074, 1910, 6073, 4465}, {5765, 6124, 1936, 
              5079, 5764}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.45060302751532344`, 0.33071019013423003`, 0.7503573825605862], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVEtIlFEUPjqaOlo64zx0pn9KSLEXEqUuWpY9NkIaEQo9BspF6WSYjwSh
0QKlFkVCtWkVZWWNLlqERViYTUkFLYpAwaBFtI0gKPq+vgstDvfec8895zvf
OedWJVMtXblmdg7ig+wLmDVHzW5hf6rIrKnA7Gqe2Ui+1mno95SapUvMhvHg
hGe2FDJrx31HxOx5uVkG+t+rzfoqzbKFzhbSlq+12+3pu3el2Tj8JhHrMPc5
Zo0xsw8Bxed9nV82R3D+4pPNftgsw2YMulDcrByyBXYLK8zeUJDUdayLiNGP
twOw/YS8PgL/Z6zDwHoF9z3Y7w4iPvYbgNUgB4GtBvebK6Vn3mmfck8jv6qY
zj14dxF212CTxfoK5yz2jcBSD2khHujvJoTpNri6A+lE/nvhuzoi/6WwrU0o
tyR0hcD6DLHeI9YS8I0ih10Fwkds87h/GhEP64GlFnK0SHkSC9/SXzAunxOI
uRgS5zexDlQoz7NYu+F/xKd388V6ux02OZ7OrNG4y38ndE2QVSXCQyzsgSFg
mXV1Zz2Y63Se4gy6WN8R57zjvB+5pyOyP4ZYDTHXY/DRUSGf9M13xDYEm1RU
Ph8i/gNPPZRwfNCG5+Ui5fgV8rNYuGfw7gmkj/zh3UZPd8OI9ahSed2D7j4k
VaI+Trv+ZH3YH6zRKPgcS6hHT4P/Abx9m6+YKWefgY+uqLiZwX1zSDUhH+SM
+Kkr9aTnfJxxM5J0vT2Fvp1CH1lc+86guCNvzJe9w5xzw6i9p7m8HFZPsB/I
8TdnP4n7X2uUE4W4yO0E8miNaUZYtw5Xixvg/l1IcdhPnDf63OGpBsT/EnFq
8PaQw0ubDPy0cRbDmhHOHe+Jn+s2v/Zpxy8xHADGk4hdD0y+sPJhLpyZrJvf
ioDmljO71a+6MlarX7aMxX6ZdD3DXPkfMQb7+kK5euNxVDVjvTi7rD3rvmkt
uArpP5uFTVtYNj/K9A/yD+T/wdnod/8PcRBDL2znAsLE3uWc0Gc1/F+KiAPm
QqzEyb+JOvLA+T7uZpzcJB1X7It1McWiX/YvY80B2wvIIGK0h5UPcb5GbzTE
xRXryTP3ZdAF4v//zTqH+d//4mpKjhccz1mHlTj/wE8mKPu/NgvZyQ==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtIlGEUho/X0jR1xuZ3fp2hoCytiKi0aFlaQbTQilDoItQqL0GZJbQY
LVBqUSRUm1ZRVpYatZKQIqtBu0BBERgYBNk62nR7Tq+Lw/99/znnfc/1W9LS
3tCWaWY7kWxkMmKWtcisN8esMd9sQ7n++bkB+VxqNpwwq59n1ozNSs4bK8wG
cP65wOw7+vv8O1Zg1oOcjZpdKzNLzgczy2wUrE7wv+K7Drw6bLcgo/gvxH4b
+j0BvOC35JqluW9NSDfJfRb8Me4p7tuLzB6grw6xzTNbD54RbymylvMIiT0D
60yMmOHuBnOceJrhfw3/Se7dxHa6TLEZMdaDdwf8dvBnwDzBfRf6aWyvwn8X
3SDSlCObzjjc4I/jnsb/BrbXS4VVjUyU8E2oNp7zEPpVixV7D5zT2A4lVEvH
rOJ8mH+Xs9WDHrDfEnMf/JfgfwRfE5ijUKTwX839RzG8pnp1kO9u6tGfp5r+
oje/KxSbx9ifNLtI/gMZZgcKuXNOw5HOFeYbuG8SQ2uBevYOrlYw6+DvAPMh
2Hvn+uP1uYXtJvD35cjnPFibwXhBvlfAbAtUE6+Fz4xjvQezs1CYr2Lqsfd2
Kle97EsqNu/py4hsXOcxzuL/DflgqkdGQjl6bs45ja41Ip3XzHt3PKJYvIcz
cE/FxOc1iparx95bn5mP+H8K5O8cf6ntjrhq6zU+xbkrrnnyGarBv7ZcuzHJ
vNUQf22oWnkPPddi9GvylfPyUDvgs38QmxJ0f/g3nCmbC8T2HJ/9eYqpif6M
BZpV5xvGtwifL1nyiXAeiWjW/d+9hHrkvfEdXBGK07kOIU+ZpSfBHBb2laF2
yHfHOc9FNUM+O74TR7lXxpS/93wp9stCxdaFzWN0VaGwfaZ8N/0N8N33HfXZ
6I1q1n1G2gO9Cf/fgmy9HamoZtnfkCPkPhYXtvc0Bf5EoN31eG8zG4NJ1dp7
6G+F76zn6/H9A18bsrM=
              "],
              VertexColors->None], 
             PolygonBox[{{6077, 5732, 5731, 5056, 1911}, {6111, 4716, 4715, 
              6110, 1929}, {4716, 6085, 1915, 6084, 4715}, {5732, 6518, 2250, 
              6517, 5731}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.47785417542088493`, 0.37809257676068486`, 0.7886621675952318], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVE1IlFEUvU7pZDMOOTrfp6OfmJqW1UJX5S5UKghSF0VaRGISNeVOSLAg
Fc00+pEWtQysTUpaLczUigo1wlZlVtKyok3RJig6h/MWj/e++94999xz7/02
tnY0nQmZ2XmsNVjzMbOViNkHrOZss0f5Zn/WmQ1i7/bMrmSYLePuH2zv4fDC
N0vFZd8TlT99D8D+MGl2KN3scsLsC7Bm08wGMmHDuxrY/6YLexcIlMKnJVNn
+pPDd7zfnwe/XLMMvD2cqTfXYb8EPkNY7+A/i9hHYZ+AbwVibsaaDMmHnEbw
/oHDJeZ4gB2Yp3HfCYw25NUPjGlwbgbXg3h7FftgEQSBfQgcLCHfbcVm9/Bd
hbgLyPsNfBdDymUxQ+dK4L/MFk9qMBZIB/IjHnGGgVcfV77UgPnzXA3sRnDr
Wmu2mmN2zZfO5M4cmEtWgVmd870IjpVJ3RVjjxVIo7pA/MjnCL4/RoRZCr5p
cfFl/swjhf0zYlXlKVY34j+G/88w9Ef8E7zD+Sbu+3DeHdb6EVNNx4D3i3mG
pDt35n4Sb6OI1wHMSbwpdzw7YRuA7jOw12KvzxeH3xvMtuN8C5y24O2Mp9qy
X8iFPcM6U0Nq2YK3twvNpvC+AZxjgfomFFfNqPMN8D/u6ruAfTCh3HvBbSd8
yyLSkvWgntSuJCL+F/BmR6G+2UesAfWnjpwBakVexCC3TmBMxKXBKfjWJJQ7
tRmPS5dznmaG85JArTys6vXSMisqPVlP6kWtiFfh+vktMHs86caZW3U4SznS
lPYV2Jax+sOKOeop7ifo0+SrD9n704HqTw3uBsqBtm9uLtjX865/OH89geZp
1mlPTPYO80+5XuJOfP4bptz/oRHcG5Kae79AOTPfr7mqPX3Ym8uuP9mr3Knz
NO7bs4VHf2pMfeaQx1Oss2HN7hNf9W0D3xFf81sLvvWBNGXPdvjqWfYja8O6
MA5zYiz+n/b5wmccvuWZPTVXpL66g/NSru6PJaQ7NW9H3PsubjnwUw6fs8u5
og7so62Beon9y9qyP18h7rAnDnuj6jNyIh7zISbnYzRPWj2D/Tkwu8LqCdaD
tegDdm8gjEXgvfb0P9oE7cuS0rXV0/+OWpXCVpLUP/U/U8HdsQ==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtk0tI1HEQx79ruWo+Yv3v/vfVlrolPeygR2+VUkHk41CoRSRkEJa3IEEP
aWQLWZF0qGNkXdqgokOZitFDt0PdetK5okvHoOgzjIdhf7Pfme/MfGf+jYMj
vWfKJHVga7EnaWkikPLV0p9KaXGjVFYv/YxIn/lvIuQ3KV2skObXSDVZqQhe
hOQhdiUh7c5JtTVSCX8ZbIWcv+Xu74S/GX+E3Fvwv4lJ18hRVDpWJW3NSNuw
BfxHxL/cJHXHvfZynTTOe88q/y/8Tvjzq/y7iL8Hdhdrx99Mzgy93rB+4TsM
HiG+RHwp6jmfwI4nfBabaT25B6nxkdwv2Bixi8ScA7tKztvQZ7RZjKOf9xL4
aIXzCz+ScC1Ms29g7wPnt/wfcLc0SK3Mftp6zHhNq3WU+UeSPqPNthfOAvpf
wh6XuUbNxL9Gs5mI/9dOrZa0a2ma7oBrO3YdvKnad9lIzkCV77SJd8OqfwQ7
mZImA9fqN/Xq2Gdt1rW0mPspjzHMeu4mdxr+PvSfAm8Fa8NGOZ4P1O+K+85s
V/uImQ99x7Zb22mY9ZsxfdrWSZfBt2R8dpvpDn4+473ZDHZL08y4EPGd396A
poHXeko/z8C/U3O43Hs8gH49Ge/NcibAL2D7yY0S00NsV8rf1nMv8acC166P
/zqJ7cj5fq2e3cIc+v6r9Jt4h38z5buxG5sCK6R9t7bjF/AN1fst2E0swT0e
Om43VIkWg6Hfo/U/DH4o6bWth8mca2jaWc8FZu+P+bdo92q3lcy6dqah7fZ8
4Lu2fc3S29e485kmz+GeS3o/VrMI/4Oc357FDMW8J+vFZnzF+2zCvz3Tfzb0
ndjbvvETvAfSHm98dgtjcdfebuI/U+OXmA==
              "],
              VertexColors->None], 
             PolygonBox[{{5736, 6083, 1914, 5059, 5735}, {6081, 5736, 5735, 
              5058, 1913}, {6444, 4434, 4433, 4341, 2203}, {4434, 5967, 1841, 
              5966, 4433}, {4474, 6106, 1927, 6105, 4473}, {6540, 4474, 4473, 
              4376, 2263}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5051053233264464, 0.4254749633871397, 0.8269669526298774], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNVEtIlGEUvWrOyDSTjTP/P6PmOKNLrewJtaqo3BQ+CgraaJBZCy0fi0IJ
scDKjOix0Ra9pLfYpiDIiWiRukppEYm6itpFbdPO4XxCi8v3/fe7j3PPvffP
HG9vbMs1s8uQPMhwyqwibdYVMdsSNKsqMHsIfQBnHPI0x2wybjaU1Hezb/Y4
Bj84v4H+QlK2qXydtJ8pNIvhXg7d4YTZIchWxJ6EBKE/ssqsH29FzmYKMXOK
zH7C96pntqHUbAn6MQAdC5jdxnmmxOwspAL6d8j/CjE7EO8mzhuQJhc76rC0
R81eQD+NWjYCY01S+WvDZvNBYbiD91uQZnx3A0NhVPYHoTuJHDWMCX1VsVlb
nrh57vhpAJYDCdm0wuY97gMBvT9zNlnoJyBZ6JsDqok1DpWZ1cN/HBiOgvsR
1L4AXD2wnUHeCeQadnwR5zfEnmP8oLBPOT37sC2pXqTJTYk4+45e+iEzDzKH
Ho0mhX0I3O5B7mXg+BXUSTwd8LuWVM8249yUVI4WnD1x2dbh3hBX/r3AEvJV
I3NUQcZxb0KN60PCcBc1/SkRjnr4fSw3OwfMlfDbVyQ+XuIMl8o/gnNNqfyJ
gzPEXvPddzbUzTo955X3zoj4Ze6VeakOaWYYpy9fM5ZKiWvyfBq8NSY078Tv
O/ssdL2+ZqARs1CNvr92/ozDGnM8Fq35vQfb5bg4XOQMQj8L/V/oluLyuQ7d
tC+un+Ds8JTrEe6jvvD+Xgt+o+KzLqHczNsCfj64uap2/aQva/Ac55/i2g1y
MlykHLT5Cr/zUeHhXLe72d6B/PUJxWdO9pJ5K0uEiXjIKblm3dz57oj2mns5
6GmOWdtiWBi4cyv/gF2Yr92QMN6Ogb8TvuaTmDmXxL09qb3lzrKH7CXx5wJ/
Fn0aiejfkHbc805d2vW90+Ehri53n0dPH6wDv6s1u8NuR8aQp8UXh+Sy1d1P
xbTb3Gv2eacnTsjZYLF4a/bE46yrdcHVSxzEQGzcf9bKXc7k641zztqIhbVz
5zgHy86PNsRMvvaHhfMSbPrL9N3v6mauH+jvW+j7wnpLufi9cf1HmPdzTP8O
/jeCrg+MmUlrPsgtdRnnmyjVHDNPHd4vlknP+LUOD/eVb9SzRzNu1/hv6/uv
3pSr5Qo4G4B8KdCM3Pc1J2mXlzb/AGD/1vQ=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtIVHEUxo9WWpNKzuPeGc1p1F0W2RNqVai1KUZt2WYmyIrAaMpFkIRY
ZKlRgW3GFj1o02NwVRHktE0XQdGqMldRu7A2Qdrv48ziwD3/c853Xt+5rcfP
9g9Wm1kWWY1MJsy6o2a5GrPlNWbnmswKSGytWRt6X9xsR9JsrtZsATkamu0l
5gW2XegNLWa9+FwELMPbM+wlZAy8efDfx8zy+H/ENob/Y7B6UmZPwe9AevE9
jc/gKsccx/4SvBpsjchsQB2B11dGeqJes2rNo0+nzYrIhXqzIeRm0t/0HSf+
FNjZ0LGVo5z2mlXrND7L5NrE22Kd2QhvVeC3B479s8rsOrUOBd7LJ/A6wX9F
TC3f7+jnL9jbkv5dRLrxX9pg9gj/J6of32zSZ3eIHB+oZzH0ecxSzzT5/pD/
O7V8wz7a4jHyPYg83Gh2DMwivoX1zILdtCMl6iuxyDtgLcTcliPHBLmqo95b
lPwFZnU+4b5TyCWwb6GvkP8X/pbwGPmO8raCfSDpNvW/D/uZmO9aM8yhfw59
l+phO75fY967OHKS7/30UEftneiv+R4GcwZ+jPCWpfYjoduurTProrcfcbdp
Xlfwv9rivYtTU/jmA+8/T0yi2Sxs9lpnKP926DtQfvWvXsuBc0U9HwBrc8rz
q6Z5bHOB96YZTFL/v7jvXpyfAWsg6vwtgDeA75uYf+ttGP1GyrkgTvTST1+T
96Mcb4kvh26Tf1vGrDXjtySOalbaWa7Clwh4feg7a50zJ9Dvht6rev4NdjHq
uwsi3A/2EvqWiHPgAfr9Sj+6icviYty5rxvqB+tL3G9NOZbAe058B/GJiHNZ
N6dbE6eFLR/ZlCNo9plr1orRLMVx7Ur97iZ2T7LSD/UuNnKDFX5pv/q31BO/
NeI3PEHseMq5L/7otscr9yx7A76ZJt+dYvSv6Krk0z9Dt5pO+63oZg+HnlO5
tANxWxyTrhp0W/eiPjvd2H/N9bML
              "],
              VertexColors->None], 
             PolygonBox[{{5751, 6537, 2262, 6536, 5750}, {4709, 6037, 1889, 
              6036, 4708}, {6066, 4709, 4708, 6065, 1906}, {5969, 5645, 5644, 
              4970, 1843}, {6109, 5751, 5750, 5072, 1928}, {5645, 5972, 1844, 
              4971, 5644}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5323564712320079, 0.47285735001359447`, 0.865271737664523], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVElIlVEY/dTXew6J+Xy+/zk9U6PCFopaZEIlFRQmlbrIFmqhRkGDQ9gg
BC0ysayoJMui2jQPtigtGpbVokU2bGpRrdpEkBmIVudwLrS4/Pe/9/vOPef7
zr15W3bV7Iw1syMYcRjtmWZtGDcTzKqSzaoTzQ7EmPV6ZmUhs3qf2a+o2VDQ
bACJr7B2MGJWMMOsNKQ4xrRg/z7mh/1md/G9jbE7IIxjLiY+bBbNMKtBTB/2
juPbCMwGn/A24jsBDpXg8hjrmxF/xmFumm32IiiO+Vh/FhbPaZ/ymbsPa289
4XakQxvGF+DuwXovzu3H+ij4FIP/FOILsVaZYzYYb7YMZ65NFuYdxFVjfhYc
ppB/I0FnjKTijKj2iMscxv+cZbYy4z8O95jLmjL3W4ywqxw+9ZdAfwViysFl
DLjXsb4U+ko9rcflmsXmqh7kcMonnGvQMhpUfXzo24xM1WASeScxuhHTFVW9
eB5j87MUP+yXNnJbh7wTqM89n/LfBKSxHmtP0tA/mGMIOP3gNwfn96Wr7uTf
F1EdWcPtwH/oqY/nMF+B+elYnT3p/BSAjvhc/Y+ElM/c/cB77/p1ISif0WOL
sL8Q4zU4TdM/EWl/Cl6tYXGrQ14y5rV+aaNnBly/xpNUrxLkTmTLGy3I7QTO
DuT6ob0iIs11M6WvwdVgys3Zz0LX06KI6svavvOkmXofhdQD1o8YBS6XHqQ2
6opCUy7GB5z7HVyKkTMCfnPhu+aQeLIP3SH14ihyH6TqrEPA+xHQejM4tET0
nwTsNZ60r/bkG3qmzNMdo79OYF6K+MaAzqMv2ffnYd0NepS+uxqW985H1W/q
+J0tbdTFe9MV1t1hHdk3cqNH6l0vuP/JUwx70ZPiMMFrPFOe4733O27lOL/W
+XwVvs3AWhBQ3tcU1Yq93eA08g3g+8J3gHv0AvHpD+qrd+/HlHsHeOfYV/aC
36IU6foMnLE0vT3Vnu4Aa7sNa+s93UlisyfEJx/qoa/YqyUR7bF2w+4ekS/j
GNMKnMXp8lgssC8GFROTKS+SWzvq0BSWlnnwwPwc+bUoS/eHd4f86UNyoXf+
hKT7MvKuYNzyy8s8j2ddwtr0LPmlx9WP/FsR8zek9Ty8B/mzVQv+E4NnjUD3
Xtff5eCy1WF2gGcnxkv4/yNimtLled7vQceTd469pXZ6oS1DfmDfEt3d/AfL
cOez
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtk0lI1XEQx8envueSVOZ7/+e+FC0IKVmReSihQ4tCqAc7aaFGh0qNSEkI
OgTlUmFEZkVFkLbYcsgHQXXMDh2M6lLQcuoSHcQgtPoM33cYmPnNzHdmvjO/
8gNHm46EzKwBSUOu5Zq1lpl9X2p2GUd6gVkYmY2YLRJQE5htRJrCZnX4n6K/
WGH2MNXsDG/N2C1Ic9LvWCUlZh9ThTkG/s2Y2el0s0v4u9AfEd8dEcb0crM7
vE2RX59pFkE/BP6GiGpOoO+PqVYbb8PkDiEDKWb76O8i+mgg3d+G42a99H8f
rB/Yr6kfKTX7k2O2BwmVqqbXmseuzFdNr3U1wywDf3WhWWOWcgysPuZpQN9G
zlJyjxM/QvwM8e/obwD7PPYg/X2ml09IW0gx1XnC8NxEtrBOlKgXx9xNbCcY
h+HicZqwZrHPRYTp3CSo2ZgjjjrwZcfEdyUxPfTfmy9+nM8tUbN12GMZwtwK
H7XIJFyMgv+Beu+T/Tl+BbOmloqL57zNF5n9LtLuW5FJao3nqvYTpAasmjzx
7fu4i38iplyv0cOs3ci9TO3gFPEJ4heIX8kN9DN7CP+VkGY6S79ris3m4GYB
fzXxVXHt0nPW4ltdrN49ZlNcN+G13yJ7maU2qtvzmxlCX5/cn9/bX2ov5ukW
/CZ7o5rJZ/lGvWdwm1KgWb1mPbW2F4s75/AL+P3J/fpOpvzWouLab7gP37F8
3YLv+yX2q5iwT6boNjfH1atz9o9ebuF/ENafaEe/EGge32E72F8D3ZLvqIL/
NA5nq4it4j/VxcWZc+V/1G+1qlCz+sw32NXPIt2ac5wF/o5At+I3M8es13P1
132nztVgVL16fpj4uWX6C/4nbsfEqeN7zyPUf5OnXXpPA+hdcc3i9zuDXV6m
XluWmB1kV42BfM7pNHoiEFc+3yK1Osj/FRHGLnw7A+3T++2M6494vsf4bXkN
x/Yb+w/taqql
              "],
              VertexColors->None], 
             PolygonBox[{{6516, 4464, 4463, 4367, 2249}, {4432, 5957, 1835, 
              5956, 4431}, {4464, 6062, 1904, 6061, 4463}, {6441, 4432, 4431, 
              4339, 2201}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5596076191375694, 0.5202397366400494, 0.9035765226991688], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVVlsjGEUvVrt3xrUtGO2zkxbscea2vfUvkSQkCgNabW1xVqxhRd7QogH
SyyRVJhGUFRJheFNpBFiS1+0DxJvQouSkDjH+R6+/N/c7y7nnrtMUfnmJZvS
zOwaTjpOdaFZZ9Qs3M3sS8zscq5ZFPdvuN8vMGvymU3PNpuBU9bVrLW72Uec
Wth+72WWGTQ7g/tT6D3ySedSwOx3gfTS8f4332xxplkVYv1CrAj8T8Gph+ws
wEzCPQn/TbgfjpgdwXmXBafQHQFfS+HzBnQ7sxS3zBOuZZDHoBvHOY63Yzlm
tyF7AT9psN2dMCuB3zqcw9m6lwJPKfQPwt+4sNkY+H8FfzNdjvu7mM0Jmc3F
GQr5Veg/8Jvtg/w4ZCdCil8ZN7uHGG8RLwUsd3qarewqDqZlC1vSxaTta9jU
Q+c05A2wmw8/wz1xkxEUP2XOnrbMtdXxTFzTHbYusG30KwZ5JGfkkDle6KE8
pwLbnoC4YIw/nnAaaptKSI84WHPajgSW+XmgG3pJYLkeVG0yESsjKm7Gg6sJ
YeXxBD4WRhVrPWLdjQgvMZJL4jzW22xiWPcr8Hc+VzU8BJ2bmarRyZDqwVqQ
G/omh8XAXgzbZod9mMO/HD5vweZlhtkKT+8ppFXjSXYLDV2OPBYA2wbctwPX
rqDk1cB5MSA/xEos1ElBf5XT2YZvA2uMt0GwHYd4jfA7D7LKsHCcwn11UHHZ
F2kFqm8b7uXQeYL7rwLNAPu/Ild5Msda2MVjyn8UZMU4MfjcDXkb7h/gsx3v
Y2H72Kd+/pyj/p/sas1YX2OaT87A3oDqzVqzJsmE6nIW39FR2TzPUnxi9oCz
0q/7O8R8T2ye5vJcQj31O6pZ5Z27gD3MvJjPcNeTrNUPn+pFXjkP5HZTtt5O
u5kd7InnbeBza0RYPPDzPFc9MNntAvZhk/vN+xZgrImorp+g3+pX7zBP8kI8
7AXqUacBOiMDmh0f+FyDum5MVz92+NSHFZCtiwsPa8lciHkEZjAL9gsR22A7
ICLbn1HtR85JS099mRcxEAvxc06YM+VrUP+/vcRPJ+rzM6a9WIWYa+OK1w/f
vjjtPtX4mduZ6YjfJ1/5l7h9xRyLICvM145kXzMH4t+JWu1KqDdiCfUc63Mo
oj5mD/dHnIFx5V8T1Bvl34HrR0x7i/ykXB1Lugsv8bwBt82IdcBTDW+6eeGX
PDH3DmdP24fQf4Czw9Peq3O7jzVNuh1YmadZIvePXRzWa2BE/xPcgTPxvsiv
uXvo1/6lbSPk90OadXJOvtkn1BviqV7ck8yBfskLe5cckut6t5+5p9tdvow3
1NkyJmee/vje4nY1a0a8rOlRxK/N0+7hXLIPyQPrwP8qxpwFndkhYaoIaz7/
7+F89Rm5I0esA3cO9zj/T5a5vqYPYh4Q1w5l39Y7zMTDmetw+J+5/2bm+w+N
GxvO
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlG9o1WUUx882d6+b07q7c7977+69U8kUUafO0tIktFUiCglKGpJMp6ai
aIs0qDfZP1DwVf4JIhjSRCqrOW2i03ciKsaU6I32QvCdiDqnoOjn23cvDjzn
ec6f7/mec57x7duXb6uMiLXICKRUjGjPRmypivglFTG5FDEJ+aIi4v6oiAlN
EX/VRyyojXgDeT0Xsb4hYpC3v9MRL2E70Bzx75iIe9wNEu8+8jt6P3p1gTzI
WzURbcjiTMTMJKKA7/fk30zs6+h70PelnKu7MeJ4yjkX8Xa43timYDOT3PeI
f4bYKyngBO/vj424Uh3Ryfs88KUKzqUahjgPIgcoOldrrA8LPgvzR+DfjAyN
NAfK1ULNy2qd8xrxd4PnH7B+Sfyb4O9q9NtX5PgjD4dl7uscYy1vO7m7wLkf
n6cvRvzE3THsu9Bng/87Yn5ArMvISPCkkYuVzrkD31XY70k5xnp604Z9L/Vt
Bd92sAcYfxgdcZT8S3ibM9bvLcQ7jr6u3v470FcTL50xFt2pNxNL7tX/PQL7
WeTrGsdsxX8B/u9hW0ybu8dIvtYcPssYg3J3I5fB19no3qmHj6llA/Fb4ONG
nbkpln0WRx9i25u4V79STwfxvh3mQxjHNTmHYs8n5yTwT84bj+qvIvdcenwV
+0XYfMPbroz7cx2+OtE/zju2ZmIWfFfg8ya2p6m5lPcMavb2Yn8y8QwK/yfE
/BPbnoK5vMZ89fDen3U82cwg3h04LIBtBe996EPo54i9sM678XLJu6Md6ab2
/8h3tsIc/0z9A+jbajzjm7DdWHIucVYJf1XN3rXP8XkFLOObzEUf/L9K/AdF
75Y42E38UxnbLiTmGbCuGt4H9X9f4pnTrK1BerHdm/gsTi5Q++pxEbdf8Ay2
w+065LcRxixuBxLPvjj+EfvKgnOJg8+IfavZuyCfFPWV8+ZWHGv3xZG40R+g
XdWMCpswataFWbrmcxf6qcS9EOcniN3T7F3vo+aHRfdUvRTnR/C9W/Sbapqd
892a4f3qyvpOZ83YoXr/KfpLtJPvcI5Gz9ZU7jr0v+W8q9PR54J3Q9bzoB1T
717L+e9TDx8VPZOaxfNgaOVtf+JZvpT2br6dOLb85zTYR7b6w8T9p2XPknqg
v6G1wb7Cuxh91DC+aeidxO/IGtsT5CC+B8qeDe2osMpGb8KsXX43sa92qJpY
S7PebenPAXDL2eU=
              "],
              VertexColors->None], 
             PolygonBox[{{5068, 5755, 5754, 6100, 1923}, {4941, 5641, 5640, 
              5940, 1820}, {5755, 5069, 1924, 6102, 5754}, {5641, 6428, 2188, 
              6430, 5640}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5807590522187177, 0.5547803863232177, 0.9098848797638419], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFlUts1FUUxi+02Ham7fQ505nOv1MSFV1QKBFLTNT6iKhYMJpUoD6gjSK1
D4omEl0WI4mJiYkG6pJCLAlKrcUEU8pCjZr4WLQYFhI1ceHCRB6iEon1+/Hd
xMXN/3HPPec73/nOuSv7Rx8bWR5COKNVplUohJBoHdfPezMh3KPVUR3CriSE
57XW6v2JFX6eLQ/hqxtC2F8VwullIQxqf7dWp/ZeqPb562e1VlXYviobQk9j
CNvKvIf/n3W+Jh3CuJ6D+l/M2w6b+TZh03q3JoS9WsOymVKso7kQnpLNa4q/
oz2EgaYQxrR/uRjCFa15+Rsifp38K6+1sl8m+0dlP6nnj/XGDPbxiP/BUgj5
omOAC/y9ynVYzzV19rdN5z6V3bR87hfOnL4XK51Dt/bvEG+PKL8LzSFsFf5Z
cflRwfsrW0NYr/chvZ8vtz+wrZPtOuH7ULHO1TrPi8K/oUU1kX2fbB9SnCRv
G7B0RDzLM//XBMzdkU+4OZYzP/C9IvL5sf49E3lbks92YfpVud8nDu/Xekvn
jzdYB2jgB/E6UzAPF8TNUfmaKDOuv4vGdptwTsb/aWH5XbV+X/8LwrvUbM4X
FHef4i3Iz8PymZZOzirHxbSf22X/gdaEcj8sTrpl/0niuJ+J71RiW/bRApjZ
Bx82cLqnxtq4nDJP5Au+xeh/VPZ7tNYL48FK+8JPSbn0yM/LsvlJGKekt/Ea
n+M8tT6i/DZknSP5sQ/eLuW3Kee6U7O9EQP6wx/1PdDmGHwf1PuhNmOlhtQy
J7x3ivM10c+g6vClYj2uvT91bqFkPxf1vbk+8hN7k9zRKf3G+2jBdSWvouIk
Wt8r7i2qxSqtQ5XuE3QG3tDgM9h/qxzHFPe0cvxG76/qfa7M8W9MjCFV7RpT
399q/U3vMAdS8X8q7Z6n3+mtqmr3NbzBGTar6xwfrrpazBH8jGSMj719qsld
ic//Ia3d3uQ6UrfejHlAj+iPuhRl06qVTnsGMVPInf7FLz6ZQ/QK2OjR1rz7
lDpgRy3AcT7WDrwzGWOm3/hPX6CLubS1gW/O8p+z81Fv1Go6b5w9ivWG6vu5
Yo8l1iI6vJmezluPXeL7Wp19oiN6kVnE3KKX6CO4GYmx6IOJqMMrquFfBePf
KT6+aHBNDijms/WOCxdwQi06c54p9ElfzB1+/tE8uKZ1KWXtzOR9Dn11xDmz
Rbn0Fe2zLON6w+f1mFnHpR/JCV1hS/7YUzcws7ekOP+2ul+Hoz166FdNE52p
jncCTzTG/lCc/+eYY82eJ6lYZ/JCg8THHo31RZ3BK7Mbbsm1L/bdgLjqb7E2
maPlGeeC9r9utP5/kf5fabLe6Y/vGt0jxCV/YlPnze2u9UtZ989cvOOY0fCG
jqgVNju1f7jFmLcqj5PKp1c1fVt5v6O1i1nX5ruNmqO1U1FvZ3R2VvYvVjh/
bImFtm+tdKwB1WI6+pxUnCMt3ttY8j0H3/g8UWu/U7Ltb7A9s2d3u+dPhTjb
rtxO6ewl4eps8rmbEt+1aJ25/nreWLboXE+7cyU+ueETzoeibp/M+v7mDsKO
OPTXc/J5teQavKn995Tn08opyH5jzvqnbqtj7bh/uprci/R8Kmr7btnuaDQf
s/J3suQ8H8h5XjNjrxbcM/j5D8BkPgQ=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlG1o1mUUxo97advz+LQ59flv/+dtgjb6kG2CbQilUiQ050JhpoXmhluu
FzWDXvyoliAUQrEp9WWTWJCa8wUEXz5kH4RA8CVEiAr60IfAXG8TqvW7uPxw
+N//c9/3da5znXPuRQM7179RFRGDWA2WK0XsxpbPjfgex2LWS7BL2Yif2E+L
EXcWRAyzn8HXk0a8w38D//saIs5WIu5x5jx7Jx8Gd37E2tR7OXwftEa8j12s
jrhRG1FbiPiyGatyvM+JtSqJ6GfvVTDfA3slvgzrX8FrBfsR7o/WR7wO5rfg
30iMdQHMZazX4nuB9Sl8e7k/2BJxty4iC8ZQW8R9+CSZiMeazPU37Ei1OXdx
fpr/C6xfhM8mYvUtjPgGrPPE7AG7MzG2YjwFt234RqqtwSnOdxCjv9Ece1Nr
pPz3Y6sT56jcNoJxNe8z2rv6UMQU90fY72T/O+J3wudMxVqKk7RfCUZ7nWsw
TKy/UtdmaZNj3+M7HubQDl4Q43mwx+Dzd+o7OisNzoJ1DjvAfj81mEmtkbSZ
nBPxEXuDzd7bQswM3J6Dwy243MSy6hX+N9fY9zJaTKD3o/Xm+Dj3OxLHXwHG
9nkRdfOspTQd4OyP/O/LuUaT5Yh1cDhRY99l/oP4v8DlKP+fUJudra71MPG2
UpuPi16rBz6Dywx6VZP7LTCOtVhzaS1Oursfu1lvjH/ovxlsE+tp9HiFfIaw
Gu53oO/9ijGFJd9/nF220PX/PeO7swWvhaHYxbJrJw7jxC/CL5t1jxSKniHN
jnztaP1ks7VZxflaavVvwVyEt6PkGdLsqCd+QKufK56Fi2jSQ25nVP8GY2jW
Bjg/N+ueKxFrIu+z8u0qeeY0a5rxJ8itu8W9rhrugM+7+I6z3kAPXAPr9oP5
0sweQLu38tZOGursthb3ru78Sbw/isbSm7EOvD5sqtE9uZW7l8HcU+eem0XL
6Sb3pnqkBH6x1blLY/VWF/hvP+A3yP918r9N/rtz7pVLZfeGemY7+X3KmSri
LSW/CWIdTNxrmnG9JXoT9BboTfkarCsV9/JX8DvE2WPzfVZ3SuAfKTuW3qjT
7L+UN3fl8AX/k4nnQ75nyHVNxW/TYTj0LvCbKr2VwwC1/jDxLOnO6dScxVU1
nErdE+oFvYHKZbjkXlROa7j7bOJaS6+7aN2VN3e9X2Nlz7Bmdxecl7MeLXv9
WoO1O8r/mzlrqPXTbeYqXy/rkTbXXjXrzjuGsKVJG72zqGAszeT/eWjc6w==

              "],
              VertexColors->None], 
             PolygonBox[{{5031, 5723, 5722, 6031, 1886}, {4375, 5747, 5746, 
              6533, 2260}, {4338, 5636, 5635, 6440, 2200}, {5636, 7163, 2802, 
              7165, 5635}, {5723, 6489, 2233, 6490, 5722}, {5747, 6067, 1907, 
              6068, 5746}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.600794951623894, 0.5869725016190281, 0.9103416365588664], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVktslGUUvUAfM9OhnT6YR2f+tixoCwmxaZTWREGqUQKWJoApbQcQhBFo
pSUQgpoIQcQEjAEjBLoAS0sLBBZIYmTBCjstcYGyQB6JcelKxIDyUPAczl18
+b/5/u/ee+65595/Zq7rX7ZlqpldxpqGtaLQrDFkthzPnulm+VKzHQVm53Fp
VsTsqylmH1aZbaoz+6PY7Bfsh5JmDbD5K2b2bIbZYfjZifMl1WbnYFuchn2F
fOTxXITzq9hn42YXE2Y/4n42Y9aN1VZiNoA1gbiP4HNtUvEYizGasDYBaDSA
f5zfwN3RSuCG7SDOO8Nm9Y4zV2t2MiP8GcR8CmwdRcAEDIVp3RsvVaztBcq5
wf0/wvtW3HsROPfAZyiqXI7VmAXw1YWzYWB7t1L3R4FnIXJpBM5mxHmalu11
YHgPOA/BdjHsfi2XvybcfQErhftvw0cz9mnucTdVJ8yv4zlYI1tinHSc9Snc
QV5h5P4dcP0NzEdCyi2Es43Ak+QdrP0478fzMO5PYL8aNb2GOIfAz0uIVRLo
9+fA/ATYHhfKX9j9VMMunZEuFuN+JBCG7rB80U8Wz1W+Z52L0qr1P3g+xvoz
It9z0/L/wPFy31yq99RMQVwa4p7ay7rPNnC4qFJ3x8LyzfpSm7xHPKx7N/YH
cb4XnH1Wo98t8HkX+LcWicNx19WTtOIxVgbvTyale+ZJHbCmQ4H0TU0sBIdt
WGfJM7jqSklvH8N2ICVuI/BZgjUH+WyF7XSsmyXit8dzmYf4R8uEczm4ejku
bD1eF+ZCHLOLhSPvdWdPbMPdRLn6ZRfq/glWL7HCz0fAsRq8VEMz+2rEHTH2
xIXzdpVqyfyIgVphfb/G2ZoZsr2HZxls3sS7m9DjNXC+Dzg2xFQz1ot1oz5o
25KUhoj7B9ToSq34pT21TP2zhzIJ7Sdxvj6m3qde2LvkhPVpdR5YX/LHGkdi
4or6PwUuR7A2RzUDdnnuu6PqOe7vAMsw7ryD/RB8vl8r3kbD0itjPUN9Hrom
czHlRlzs3ZGkfHUCSxfsPy1SfPY663Uc5+Pl2rOu9EkdTvoMYSz6/tf9N6EW
F1Lii3UYq1QtGHO9x/0dmL8PVMsgoRqQv9cSminEMwK7A/idLda8pA6oAc4R
ziTySa21VwkX53SnY+Pcopaez2HoIoQ1EdVsbvf5fAK232BtwPnelObFhM8T
auWg9zg1yrn0KmragNx+C2t+8UnOt8Dff+Xim/aMQ5ycJa2l0kwH7JZiXSwT
B+SCuiLHnc4z3+Wcn5+Qbz/y/QL709ifwfoAMVvg736Z8DGPq3Hlwhn/SoWw
90WlD2qDuiFW4uQ3gXOWufAduec5tc447AV+HwYCfSOO43mrSv0YxnkIazbO
R4DlEmrTC7sVMeGedM3cd93yedRxEvM854E5XU8o3mbw0V4nH7zPe7SlrvI+
84mTWuBcuheTtqirvOuOubwFvb9RJ3vW59tq1Ybx2UvkeC3wzg+kCeYwy3uN
/c1vCfubNcl5z/MedcHcf05o1pCfPuf1ed3jqhNz4fy46xpmn/b6LOV3jN9C
5sVv1BGfIV/Cbl2Fei3nHDIue2vMNcwc+rxGCxKaZZxj/E9Avqn/lXH1Lf20
Otf0f8P7n73/P7KPNWA=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlFuIlWUUhpflOPtUzbhnZs+//71npgtNAykGYQw0axQdTB1Io8Ym81h4
aJyBCFGwyCMqoqKBXpij4wEz6EBR3paNESR1UYYi3lU3aRclnn1e3rlY/N/3
f9/3rnet9a715LJ1L/c9EhFvYaOxziTiK+xWJuIMtqMloqM54mJtxOLHIubX
R/xbF/E/Z18XIsakEVlsYi7iU4AOViIOYItZD3Pn7VawsfcA789HzGyLmIHt
Y9/xeMR3nPXU29959tvxV6hG/MbdN/C3jvUAdon9/lERp4oRe0oRm8dEHH40
oqUccZ/3p7IRi7CXGiL6+Hcya4z7cHuA3YDfZDjlwPq+1b7ehcN8uHxGvP/h
/8snIro4m91qblOwi/gbaIrYjb9t5OBH1sNj+YJ1drRzNVhxbIphIeterDNv
flfhNg2fGXL1IZyWw28JPnN1zkEX5534a8fXQfI/j/PWsmNRzL8Ta9+I/634
/5X9LyXnV/+Ogn0EW1ATsRIfa8H+YYTfCnyc4G4v738Cew3vd7JfwPkAb3vZ
P8N+Gvtr+EvYL2N9mn/v1DrHnaw/rhp7Iv96iO1k0bHpXLnPkd+nc67BjcaI
uZyn3J3FeZ6z2tTa0J0/iG8QvKcy5twOfnuja6M3Q5zNKZrLanz+yf1v+fc+
62czzt0hNNKTdQ6fI7YXE2tVMR3n7omq374Cfhe5nFN27pWTJHHOleudvEk5
2wLeIrT2Kne+YH8Fn2sLzolqoR6Q9lWTbS3WqLTZi70J90sla0M1kZbv1pmP
9JAh9k/I6bice6QM3ln243P218f9oZJroxo1k99xcPxIb6l/f9U9IO3rzXX8
/Q3+rozzuxUuR8jB3lGOIeV9BZsgLdaYS0eTcyNO/WDX8ma44P4corePY6uo
5wQ4ZFrMSVx0J696NLhXNAMUi3Im7oppA742Voyl+Cel1qC0d7vGvfNz0dqV
XrtY3xzpR2l+CudDRWtRHFfx9gL/LrD+h38luGxJjC9OS7g7veTYxPkuWHdS
11OYg83WjLQijamW54jhg4JrKuzV+FhYZx/zWHe3uff1L2FdbnNtNbNug30r
tT6FnybWuHKvGjwP9voGzwL1h2bD0mZzl96k3b8431SwhsXlhZJzLU4V1tWS
8eTjc/DHY9m8Z+wDfJUTa1Ux3qv3zJMWNeOmjrVmpJVuMF5n3d3o/tQM0mzT
zNas1oxLKp5xmm3ycY58vtbkXlbPn+dupWwsaeoYsSwtOjZpStrc3ujaSqPV
sjUtPagnNes0M3RfM+8b9pcb3IvKwUO5X91w
              "],
              VertexColors->None], 
             PolygonBox[{{7258, 5454, 5453, 7259, 2882}, {5049, 5726, 5725, 
              6053, 1899}, {5726, 5050, 1901, 6057, 5725}, {5454, 6026, 1884, 
              6027, 5453}, {4938, 5634, 5633, 5936, 1817}, {5634, 6424, 2186, 
              6426, 5633}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.6208308510290702, 0.6191646169148387, 0.9107983933538908], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFlUts1FUUxi+VmQ4z08JMO4//vFoXWl8lYWNiBBOaqAUJxim2YIwLJfjC
ERdsNCG2Pqq4MlFMMAKNKNGyYEMiD7urJibGCAnVCC6pJMZHMBEIGH8f3yUs
Tu7933vuOd855zvnf+vTL7c7XSGEWeQWZHJZCFM9IZxcEsL6vhAONkOY4GI/
60jF52tyITyAbODh3u4QXmF/Bp0D6D/e8N1z+RCeRz7A5m7ebSxZfw12fi3c
tLMWmUNnhvOf+0PYxps9rRA+RDrgqNfAVgzhS9622F/j7SJvPy6HcAX98+yf
Yn8M37NgyNZD+C/qVMF2JGX8Y0XjEIZLvLuM7OKumvI6A4bfODsGjhf4/hx7
WxrGV01C6Crbzi/ofNV0bMf77Ft+93F2ANnK+Q/4+Y67Sfzfzf5h9mX2V8nB
NWRjGv/gvIL8ncU+flaD774ux3R/03Epr+1e5/bBtHXH2dcGQ0iQoeXGIbzK
83ryM1pzDSuc7VwawqPIT9gsEUNbcYHnELIdPN+nvAq/cAujdKbBeFtiPOPE
dxL8h9HJg2uauqTQGQJLMeu69GZCWJqxz3vRf12xoz/b5VhlZxEMM1Vj3Qr2
b/td31R8J5wJazvvmqzDxmLJ8f4I3lPIFPjuyPpe9ZKf4ap9rSQXY3m/Hx9g
3zC/5ENnh5Y4Lxuyzq14ppgVr2JQLO+zrsLnPRXzZhQeJYn3iytCeKjgPDfI
8Tz7zdg7h42zeePRKjvaK5eyL56LX+eizjfUefugcXTI1QK+znBeSzkPwqkc
tWNNZ4jjaGIOiNfqAXH7Yo9zIZsvioPYeS/t3OhcOsIyFHUmE+tJR734RMO9
8EXa8St29Vmm7m/lbCzWYiFnLqpGFez8XjIfhbcWebYz1k9+9TaJsXRjL1c3
pl3UanfVXLmT9S5knjz9y/1on2udRJt6q9mie/kV14VD3NNsEIdU25fKjk1x
bS67J8Xnd7A9jSzPWE/8EC7V8PaKcyuOq/fEzyn2byCnM44lFTmpuPdGDKvh
/7rYX1ty9qm+UH1GYo2exN6myD3xQ/lVHxzEZ6fgnhOmG7NHHC8njj3N/TMF
z8NtcPjZAfe/Zo9msebPWXL1R8z/XNr9JftvtjwrVTvVR3rSkc9My36XcZau
mxMjA+a3uP21erxse5r94q54exjMEyW/vYiNf5DzPX4v/olXmpfqbXFWc+6T
pt/X0J0YdE8opk0Djkv/EP0DZF+z5O04T673cZwbn3H/adP/EPXKcN39slYz
qWyd5Eb+sPNY2bmbiz0ojOL+fs53lMztPUV/ay9cHxWt+xq1erXhenaDZUfi
mtbBuQo8WWI50ec5qPMOOX6k3xy5jM6livmhOdYbZ9l8/L7eLynzXXxWP6n2
wt3kXQPJobsP22+1bnJFq3D2gKUXmco4RwuxB1fybrji/9u75O1Uwfb/qjlO
vRXHL5TMc/VTEvv0AnPsT86nOT/a754UvqsF/1M1G/8HdH8HqA==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtVEtIlWEQHR/Xe9Wb1n2/tZW9FGoRRRYoBFkSdDUf4SrCMsx0EUFBoJVa
rYIoCDI1U0qDNkWRtbOCiF6gRY+lFkElBWlpdA7nX3z8//fPzJkzZ2b+5XsP
p9syzawJJxtnMWjWi7MRH3fm4Bkyq/GZVePemWV2OmV2Cucm7jdgX1xm1hYz
O4LgmQyzW/B1x81W5Jn9XGKWj/e8uGz8VgXfEcR3wdaZazYQMZsKmKVdZgdd
sq0Om4Xc8nkCW3GxWVkh8uA+B9tKxEzAfhnnKPK/wLdOvE+C/134d8Ne4DFz
4YyD/0JQtTzC+QX8WXDwgcs08K4lzd4jpsVrdgCnNmE27DeryDfbjPMa+BdT
yj2CGjqA9TAkrNFMcXuK+BqvOKZh20U7uIxCr23I9Zj65aneDXifhx6zuG8F
Rine/yLHdIZ8iNUL/oUeYVKbAXBs9kqjNai1CvzqgH0b925wi4FDCXK7cP8H
fgnkbMgVv074tiJmMkv9a0R99Q4/1tgeVE30PY9TCt+1QWFHoOkf8PsdVz5y
3o34tDMPWxCfCKsm1pIP/1DULBzVnTH3kX8IPs9djh64VxYJn/Ya4PX7hUW9
y32qgXw4g9/wvs4vbGJWArsiqlpY0xC0uZ5U7+px3wO8poSwyHEkrJ4wd6tb
Wvc4803Nx4E95vAbg0819B5EfAfip3DmcY9D44/APwGfAuYnBvTrQo82IXd5
Ur0l3ndgF0Zlo88n+G6PyT4Oey1qrysSP/bgHfD7EL/Pqx6vimiGObuc8bO4
n4lo9rLxLVosjuTGnt9D7OeAdoczXI/8w2HVyppy4tpJ7mIJ+hcEt5mlZhdM
83QF8W+d/ePOfUH8oKMnfbgrP2KaDe4MbV+D0po+nK35sHaR/aB2zagvXSAN
n0HfQ9D/HHrXBZ+X8G0LaVe5sz2obS4gbTnzUfCLRIXPGWGusrBmkd88qCc3
rlr4D7rE2lLa5Q9ezeb6kOaD/ffA5k6pt9RjAv3oC+nfxR3ZgdzepHaH/zD2
OpYSFnt+DPW0+MS/MV+70+DME/vXj9irSfWWPeS/c8HZZ87YfmgxkJA21ORk
VP8s1v/Go3/VnYB6y38WZyuOevPcmp9RvD/wiz85tCP+eEJc+I3cXoWlLe3/
AfK2zHE=
              "],
              VertexColors->None], 
             PolygonBox[{{5744, 5067, 1922, 6098, 5743}, {5618, 6413, 2178, 
              6415, 5617}, {4914, 5618, 5617, 5917, 1803}, {5066, 5744, 5743, 
              6096, 1921}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.6408667504342465, 0.6513567322106493, 0.9112551501489152], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtlU1slGUUhW8FOoW22HY6803npxSNUSgQlagLcUFaMBhNpLZg3AiIxgSl
Nf7gTosJFBe2gooxMTY1KSAkbkSN4NKIulCJCmo0UiNqFEV0QSmoz8l5F2++
97tzv3PPPfdnFm4a7Nt6WUQc4Mzi9NRHZLmI5RinLo/YwdnNfXpOxJ+NEWOz
I4otfv+pLuKd9ojHOyNmsF/qiPi5GrFibsRHOfv0Yx/l3DLXONcVIpZmxh8o
RrzOfTsxb5sHBv7bwOzDvg/7MPareL7QRlz8R/j2UsF+y/EZIN5eSP/H82V8
9uBzI/Y+7kN824T9B7gON5BTi3lX+HYd9huK9qnxXk08j5FLV5d91uQjlmTW
4kfOFvhP1jnGHbPN8xR+b9Tb9gi8hgq2vQ3O4lLEBfwqaDPN+xfwfIL74VZy
bbZGd/L7k3XWWL8Lv5dYa5N9EL+WBnN8AO2P8zyC3zeViG85X6HZRrA7a9Z4
P1zy86zzOPzvqlof/aYYwv+UnLa1Gkua1FLuOtJBnM6WI/4qG2sC7F/Jqxfs
kZI1kj5Po2FROPCbBHM9PgPEPcE3h/h9NXFzTRFPcT+YsKWH8C+2upaqYzff
7uu0zx747q4614XEvaLmnlNNPkxxj5DXQb7Zik6fL4j4jHNiPn1HLgU4jTRY
S/Wm8j2JTl9zvoTX7cR8LW9N1AvKX3mrT4vp/ig5rsZnAJ9uvnsx9d7fcNzb
Zm2n4PgB9zfh1kzMcwXnez3f/ZHu0lu6y1+c7msy5knyeLXmd2kzWW98xby2
YJ9feJ4ueBbXkV8/ZyU5fg+f7yrWeBoNHsrcLyvAu7lmvVWnfJrfTDXi7Gpw
TaTBKvB7iXVNZr+byKMjzeNFYh4turevxtaDXxs+L5H78Vb3YzkzP3Erd7q/
pfci8jpV9Wws5b45cx9rFgeqnjXleybnWp+penfMpJ64kHp+Cbbfq7YNweVj
OLw3y3tHc6ia3kuNNpSMJf3UR9JQWhxL86P66xv5i5NqpbzmoGU9Z7rZHJe1
218zqZ01k/phLGGoR0ppTlaiwfwOazmec32Fr/7TfhtNcYStfbYB7P6SeVdS
jwlTs/VPp+frt4JnTHXTHKwpW59K5niKdRicHWmfqObDeed6HvvOkjGXZcYS
znPoN8rZ3Gje2iGav134Plvyu/hqX6g/lbNyV/+fBntBzfm9gv5nW1w3zYF2
sXTWDGn/ao60RxRHNbqy5pyV7xjxn+fc32ju2oniX1dyDOE/DMYnqb6HwD7K
fRD7W+T9GFrc2ux9qv2q2pXSzErLnfhP4H9PzvOn2dPcqf49qQfOld2Xqr/2
93jqH+3s/Wlv/9vh/y3tY9VOdVMs7chnOpzfTNqP6kn1mnykw/tF731xVB89
mHpesz+VdrpmYnHiMMHvG9u8JxVf+YmDctB/nexri/7/07u02J7sq/LeTdoP
0kyzId36i/4fld8W8LtLzn8Rz/OZZ6qLumxqd0+vJ6e7Oe/SA/8Dqu8cBg==

              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEts1FUUxk8pnenb0pnpf6YzbQETgrQlEl8LYEEACUQSEaZNXPCyIgmB
svC1UooJr4Ug72iIBBIwQMJCSYzgkvDYqA3yCGAiBBUjQtQF5envyzeLk5x7
z+s7537njls++MaaURExgIxGNuQixnVE3G2IGKmJ+KIt4tGYiIfoh1IRO1sj
PufuY87bCOxJIqYQ8wJ6ko6YmYn4k/MsfFs59xQjuov2nVcfMZ7cY5Fto11j
bnvENM7pxoij3P1ViriDPER/HXmvM+J9ZF6T734n99xMpRY1mgsRTcir1RHH
wLQXfP+Qcz6+O6i5Gts6ZLg24jt8Jufdo3o7R45+9GP0MIhexr8LLMuyEVuJ
38/dHGp1J8ZWJn4i+iut7jeTrvSasa6ezxA7MDYi3xJRpMZ5bD8QM1zj+pOw
36C/4w3GeJv4CYlnpXjF7qLfjhrnOMWsv0d+5fxBVcQ97pbQw52032Ap+nMV
fLrbTu7FOddbQY1O+vmpK+JSs3t6HluWeWyqdT/7yP0N7zuH+X5F/UXke5H5
raqLuEm+H4n9Gvv0Oucok38/Pe0hdi3+p5lFDT4jTe7pXbA/LXj2ynkK3z5q
rE95xp9gW4+crDbGHPplZvJWozGtBd+X+B9JuWfpu1uNTXcLqd9HvRnNxiBs
6kF2YZTvPfBn6n23Af8dJc9CNf8D37/I9lH2eUy9RznPVhxPdZmz4qp6+oh5
nADfA2ZXwn4B2y/IpXpz7iVqr2L+M9OeR5l++grWhU+zrMqbS5rpdWJ7c+aq
ctwndgSZhf1DMOwBfz8cuPGMMU7l/R5k/dbakSHm2Uu+s2nHFIhtT7wL4uQk
8SHvXMLci21yYm7JfmKMd0Dc1079DZZrYLpY6Ue125nPhWpjGKH2feRWle/a
Ct5B1dvMex0Az0JmMMT5TXK+Q60n+EyrM4YU9RZU7N/W2re/YF13h/E/lHj3
dF6ZOIdiF2jfebuerP8C5TyI7UDFX5w6gr6owq811O+mt96sY/Um2o3ZGXNd
OyIunszYVzHLmPfGxNiVc1+Hd17+4uQw+DeTswW8g/qD0BvBP8R5drV3dXen
bdrZLXlzRlxRzHKwPNvhXdKf9xuz+6PFs9MMtauf0uNAg/dhiNyr2/xX6M+a
ApZiYr6o3jrOL7eZX+pP3NAfrb9ZHNlI7U15/z16s89K3gFx/+0Gc+8y7/1z
vXOKS8qpXOLU1aI5Li6IE1eK7lm1FVNmP17Lmdvawa0l/7HCqx7+BxcV4Js=

              "],
              VertexColors->None], 
             PolygonBox[{{4374, 5740, 5739, 6529, 2258}, {4423, 5927, 1810, 
              5926, 4422}, {6488, 4452, 4451, 4358, 2232}, {4452, 8021, 3649, 
              8020, 4451}, {5613, 7130, 2774, 7132, 5612}, {6420, 4423, 4422, 
              4327, 2184}, {4328, 5613, 5612, 6423, 2185}, {5740, 6058, 1902, 
              6059, 5739}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.6609026498394228, 0.6835488475064598, 0.9117119069439397], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVltsVFUU3YXpTMtMCzP3zmWGuTND1cQC00IU4yOxabBGxUewhdIfiy1E
TG3hQ0UxBupPjQnBKIaOGGPEKLEReVQtyocfaoMmNtRH8BGM1gIVWkVUfmrU
tVzn42TO3Wfvs9fee+19pq57a+uWOWZ2Emsu1trAbGih2WfY/54zW+WZteHg
L+wvY/02z+wSfv/AmsG+Befn07CDzoGoZNRfVzTblDT7uNLsUKXsqPMD7L7C
+gLfjbA9xW/sm+PSLUPnl9BsuW/WGjG7GlhuS+qOBvg5Df1GAP58vtlbtdIZ
xvlJ+NteYdabNVuDGPqBxcP+DGxWYd+VNzu8yGwc+qchWwHfLfA1HUO8kB2H
7QS+f8L6EbE/Db9dKd3zNrD586TDGO/E/sQc2e2s1n4S95wCxt2QNUHWXy39
ZcBbyskmkpB8EvJu4LkMzEsgO4r7DyKWeyLKC/PBGHsRR5BULfZh/wLwvA7/
i4Ati1UNn4NR+abfAcTVmDF7Avc3IH/LsB/F2Qew3ZFUXHtrhI+5uhvyHuTo
zbmKr8lh7gCe0Sqz+4Dh5UD5pf4O6O7EGsfZJGo0XBTuppj8HDLl8JWY8ngN
8n0tVg/u3QxsxwPlqAwMgzWSN9Yq5gHYVgDvSugH0CkhLxWocT1+/4HPzYi3
GfduxO+Tnnw+VaP6EfMl2JVTimM7sB3zFMcRyJfCJg29R5GTm/KyiUG2GDp/
R8XTjqz4dwPOt/mq1Sz0/Jg4E7jaMoffkftpYV7uiU/k0q9x1YN6i3HPFXnJ
3nNycpVnzAHvYXysP2NcAjz3Z+RvAPifw9oAu3eSqgHz/xB0rguU529y6gfy
ZBp6uyLSeSCtXNMH++x712vJmLjL3mRPBAvVF+yzqVDYrkf+6p18N2SdaXGA
nG1yvO1zdaO/zoxyynyOzVdP3o7z2QXIZ1I42aNtEXH7Q2CpQ74+Ap525Hsd
1kiV+tvPqsdpz1yRw1vAL8uIT8T9Je7YA9HBuHqPfUf7rVH5IucCl9sNsLu4
QPoj0B+OCwfvJ07GS2zcU74SeW3O6i5yk33CGHk+5nTugKzO7a9EbbsR+8W4
OLs2I9/3pnQX73kM5zfmhZUzqT3UPLlQq7kx5O4cc3mry6uHGfs+2G7MK+ZN
+C376om70pqt5Cp7+ligeKeQ1zdC9d96xNEWaJay//pcD74KnTWh5KzBw1Xi
D3tj1vG8BD/nQuWL9X8+FAc4D0YTmkVnIDsbam59ijufwb4rLr7yl/rvB8JE
PNsK4ElBsR6Oq36sHfNIP+QGZZyz7OUh9819JWL9JCX9cewLefG9CziL2M+4
3mKfMW+sGWvHXiBnuSdv97qZwz1zMejqy9+yk/cjb31ZxcpZc9Rhoz/6Ib9Y
b/piH+9yZ5STFyOOY9W4I5XVezeB3Cz1FTffnJKbFdPEjnUkoVrt91QXzvek
4/t+9OJrWD34bofdiKdZTe6wl5kT8onnlHOOcH6Q87d6min87gCG9aHeWM7R
B90s5bt9iyecfPP53rC+fFc4A1ocHmLhnS/6erc4szpRixOwXV0rHjGOx6FT
iTpHC+LKnlCzjLwgl1/yxeebYdvgafaX3Pzj/X/mxHvq/D8TE+Ikv0tOZ9DX
W045Y6t3Ohfg53yoWTuFvj/r5jNnF98Jcok8YZ+1ur7jnpyZgd10qPoxx3wX
mU/2JfVYU76r7yalz3nU5/gzC/m3OeWAcsrIB/biOcf/RFZ61CG/ng3FsZ+R
h4mi5gHn34q0ZiBzypx9Dfy96N2rsuIn/0dEi/ovwfeHcTLGf3H+SEF8/w/P
6WpQ
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJw1lVls1GUUxW/pRmlBOjOd6XSmHesShZZqJEYlsWkQNS4xim2pDwJFYkil
8FCtQjTUBzG+4IKGuiQEjBCNyqaCGuODWtEnKxpcgoFaAaVVRMUHcPkdz/hw
M9/3v9u5597vTvOy1QtXTYmItUgZsjcVMZnntzqi+ZyIm+oi1mci7qiM+LA8
YmdDxC6kjvsoDnPTEZuT1q+uiNhdiPgR/4XoXp0R0VMbcRf+7eh3EP+ZRERL
jrjTIrZjX4F9ObJ/ir89ju8G5E7uI1Mj/spG/I104H8LMS8nXzf3vVMd73nw
Lm+MaK2JOID+PM7NyNqSiJ+p4WtyfYW0EW8Anz58VyJD5H65NKIffN11Pn9K
vFnEO0V9KbC8Lg6o7Wy9fT/gWyu2Hfisx3YBPtuaIhajnwDfw9MjTlPLppS5
mQWmo/iXoE/D3ZFSx5oA364ax/wT+2dTti2B75MzI56jxpfAt5EchUbHkO8k
eC7MmENxV0vOw9jXZB3rAexfRN+Vdj196A+B9YW07ddVEQv8x+uMXTXo3IBP
VaW/rSPWID5fohsl5k/0YoB8m/B/i/zz0FVif7bC+eaA7QD8fg43bfT7NOcq
YnQS6xe+zUf/e869uZ1vK4i1hpg91e7x0wnXpFpU85vEH4TTG6a7h8vg5txG
9/Ik0sS5N2UuJpDJRmMSFnF6Abmv5FsZ5/ESY++C8/FK15BAn0TmVxjjNeC7
iPw7yn3vxX5rxtxpRgbwvRabFPczyHDKPVOvNHOXwN8fOdeqGEvRn99orBuo
eTb3I9T7GvedyFLiLal3bxVT2O4mxsFyY5StZlCzJ59+ZvE9+BiGjz7691je
HIq7XvRPcd+ILCnyKduVDa5dPurlYvDeOMM9ncC2lfh7qv3G0+iHsn5rmv+j
6MeFF/vd6B/h/EnSuZQjCzdXJMyfZuYhdD/kbas3ql1xW8K7QDthf9Ix5CsM
i/LeKdolHXwL6h8FQz9YN4tffP8BT3uV3+DsjDkWt9o5entXFzxrmt95cH0/
9bxS5jd4X5N3mHaXZqiz3j1RL/QGtatuLuYXvnd5G+8gh8M9OJb3ztGuEUe3
albzxjrMfFzf4Devtz4ExovBtyDp2REnT2L7RN5cq+YOYp2Z6V0lTvZhu6rI
t97AcWxP5P22tDMPMktzE95XX0wzN5c1uPfi6Jj8C+ZqrNRvvTPt3unNf4Tv
WN6zpp16XdIzrNkVf93otiRdi+a7osk7UP4jzHNafCe9C1SPsJY3WSfMgylz
oNr1xvT225LGqh2wDyxvp41NnLbWe0Y0Gx8je8DekjK/mrEe4i/KGo96ol11
ipiTxf+Hrqz/Y8Sddv5YwTtNu+wE/g8mnUOx9R/zGfrRgnfDmhLn0n+S/ouU
cyv2j2Zcu3KK6/dz3u3i/NI6cyauNHPLsZ1DjJFi/BXc21LOp5p+w/fXnHeb
MH/L+ZucZ/O/nZhyjPZi/YfQfff/viTfvXD7Rq2xauZbqOWejOdFOa9ivr8v
uFbN+L/YLRu5
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.6809385492445991, 0.7157409628022704, 0.9121686637389641], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFlnto1WUYx1/dfUfc5Zyzne2cMxWEQC3B0Godt5GUSlZu4SUIkWlSogus
tgJt/pFZmgSR5chA/yi3EmtdaOrWRbMLBHmhG+EWld20dTeo/ujz5ftGf7z8
3vO8z/s83+f7XN4zrfPOjq6JIYTPWEWsxakQbsiEcFdxCDeWhDBjUggdHByb
wqoJ4RyyL/g9wKVHJ4RQVh9CM/IDyJfUhVDCfgQ7rVUhtLAu4f5CbJblQ9hU
7PPOqPMgd69Ps7DVXht/T7TPmdHvn/jdw/3T3L10ks+038y3HZ/t6PxUDWZs
7sPmaEUI8/G7lLNd6C3l/Blwzpwawmt539uIbmlTCB+gcyaL3WQIC9A7DP4h
1nCpca3IGc9s9jPANof9afQLYDpb4e/Vcb8mEcIp7B0uMgbJpP9Eqblay/qd
u9fga7zSHHRMMQ+KSVwrrhFwbWwEM/cH+b7IGmO/tZy4So2zL2VepP8k+4vs
Q5XtXqg0bxnw1rPKy0J4G1+LMs5XZUMIP8NDL/bGwdOaNEfKk2yI03p8Zlhd
2L+cvHySte9ClfkTt39h71N434Sdc/C0BTu92BnmrLXM+++RvwS2V+Hm8XL7
eQw3XyP/hjWIvCVpHDq7n/00+N/Kfhb2N0ebqhvVXT8c7uLeIznzPafOd6R/
G7Guq7dv2eiN8hLwvgv2IxMcj/KhXCwkvm64G4i1tCXGJd0i9geQ93DeDFel
/H4Hm8fh4gTnCTgcq3ZOCrHGxdvfyNo4Gyg3D/oKg/plJNbBPjDuZ90Bzny5
8658bSOm98A8xH4vvs7WGPOvWf/W/v2k9aSzCGzleffp6dir6gXV4OqEddRz
16XMn3pRfaFa2o6Nh3LWW552P6gXziP7jvVKwpwUYj3sQLaT1ZlwHs4n3Ftj
sddU50fAeLTOcar3C7EHl2mW1JrznoR7S3lchfzZJvPfBWdDxPYcmNeC+amU
e121MzPl+mlmtSbcL+fLnKf+Yvsfjb32C1z9lnUP7Mfe8pzvXInN6fjYDd97
ZZ/VqZwjSxP7SeSz0a8jL2XYPgi2GVPNp+pH+f/PV0vV/3VSEuVPc7cDX4WI
U1/Nja/Af1Xeesq9cqUe7MP+hynnfxU63TXO27V8H0i7R+Yjb+P388iT4Kxl
bUd/B36XJ8zVPfjszvm3bPQ22E4F34tZ97hi1ixR/21o9HzRb8Uv3hRLK3eT
xL4tOFc/5Jyv+/jeXmufqgn1j+T66rfm7QXi+4d45oHnlsngSHjeKgfqIdW2
ZqGwKEeaN4ca3O+HGj1Pdd6G/NaMOVec0hcevQPjcaap3vU+icd1adeW8Kj/
WmJtN2BzZ537XFhUc6o3LfWh+P8j65kuPOpn1ZPyqJrujvpd8P9t0jEpvmlJ
x6icrY7zekHKs1J8HIfDCvAtA8OSjGeufKn3+uNbqfgOxhj1Zv6Y97upHCk/
ynsOWT7vHsvV+W0Tz+JLOuL2bvxOrze/ys/DOedI/aS5pp5az3lb2nPoZKwb
zYQ92O7LO4Y56J6qsU3V2ZlKvy9ruLuyxnePVRqjYv8I3j7OWk/3QsZ31dMl
Fea3O84GydUjwqjZJUy7Y4yq61lp+5ybdzzSG612LSrvr+NnJBv9F7ueZEdz
8csp3q9IeG6pF9rj+ylu74WPnpzPj2JjmPUW92/m3k059+gGanA96wSY3sj6
ndMbd1nS/wnEg3ri87Tx6H3Tm6T3SLWlPKunxc2b2J6rep7sWaV8aW69HN++
lfhcGv2+0OC66YozuTXOk8P4PJK1rRNxtomrxRn/f1IdydZgtH8u/r9RbQm3
4lOOlAfNYOVCPSpM0tf71hnfONkeiPYVh/pJsQzHe7JZ0uS5LG7nkaMr8n4X
CxGzeu5fnNtoVQ==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlWto1XUYx59231mU29k5Z2fnf9oEI5hZsdLKMldRGGltCy+9SGwzK1sL
CrcCdQaFlUkQXVwW1IuyFUylwm2u28wuEHSjehE5yGbWLO0mUW/6fP3uxQ/+
v9/zPN/n9n2e/+yuezp7SyKin1PGubsx4kLOU1URL1dHHE4ikmzEi8j21kRc
Wx8x0RQxVR5xLwYVZ0WUc14rjfiUt+UZdHMRd1b6PliMOFGIuIHvNafjJ7GO
ZCvB60b+HJhzkbVwHkX+STpiBLzHKiIe534L+l+g341+Ef3j4G3H9zT3i7nP
5+ziXor9Cb4PzYp4mHgHyOEzsHcU7bvIfSV4qzgLsb2exDvJ5aYm39/G5giy
Kc5AqfOdBGsS/4uIZQv2d+Ujlmate5D7pWD346MC/HJOD7b3J85NNYmGiF7w
bz6Dep4WkaM2c8F7nXyqqNFJZF1gdHCPM/EL/mbOaKlzPlBH/mBMo3uQGI5S
m1H8j/Pdj482dC/ivgX9IeL5B92avGO9mrdbwX623tjnEN8YtTuOznxk76Ui
9iE/gnwTso2cvsQ5KPYV4E/U0v+ia62ezwG7p9HceBofexrtQ9iTvC0B+7oG
c+MJ8m0j19vIeTHxj2N/JfEPNrgXnfjYx33FDB/EGeX6ATl/VOGc78DfJs7n
VeZENb6WEO994C/H30ny+ZvTWuJ+tzRT32bn0oH+g+SyPTF3VNMG4s1zngG/
t8y9+Br7r1LuyTZqeaDW2NIZIL71OceuGr9Bv35MzI23xMesZ0R3caYF3b2N
roVi+qbgnqgX8rEM7N15+9ZMqXcddeai8BeS3wbye7XMnHqI+D7MuvdPor8Z
+zHxj3s5PtqxbZ3pvzA2Em/3TLyKZz+y28EYKrFNBvv1Wc+WajpOfHs4C9B9
n/hGsvYhbHFMs/kX8t9SntFfie9Y0fq78JHF11TG81ZJv15JzFFxs6/GXPtX
fKgy55qRVeZsq/v54P+cuJbDzMgO7mvBn4evL6nBTr53U89DxNPF27bEPVUv
u7A5F98XpD1L6t8vyKY5byJbW+JcNyTmsnKeje67Bc+zctBu2Yp8TY3n7ZHE
nBPX9DYP/UVNxlYP25G/xNviGu8M1XZdxr5O1RjsszPObyLl2qTB21rlGo1g
+0DiWE7NGN/tM/tEmD/g63CTd8kY8/Md9pmcayt77dK2tLmtnbqM2i5t8Kxp
Jo4mrqlqqRp8jO7qJuemnDoTc0ZcuRz5jdxfSPtb+azLmdPijmZW/4LWRu8u
/ROuQvd3YjiWMidWwZX/Znn3iHPD5HpNrbnaXupZXlD0btZMK9eevHenchbX
VxP/UJk5Ly6+U3Dt1B/t+ivSxtLO70P/kqJ3j2y0u54vmhvaYT+lzUFxTztX
vRgteNepJ3X4rs57V6gnc3LeSdpF+oeIa8Oc76vNOe3+/QXPhjhzGbVMzdhL
Z2e9/2H6d4mz36J7Xtpckb52+2C9uawd/yfyPwrurWr4P6YYD1M=
              "],
              VertexColors->None], 
             PolygonBox[{{5710, 6507, 2245, 6506, 5709}, {6049, 5710, 5709, 
              5047, 1897}, {5909, 5598, 5597, 4911, 1799}, {5598, 5912, 1800, 
              4912, 5597}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.7009744486497753, 0.747933078098081, 0.9126254205339885], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVn9o1GUc/ujO2+5unXe73d33dt/vbkEJNiUKsqIYS910pmmbRoRbKCQ6
xWW/DCokUIpMSVfRSM0KEuePCbYyswK3JhV0u8z+yKlIqdv6Q8vAYf/0PDxv
9MfL9733/fx4ns+v925d2dW6frKZ/YhVhtVeMBtKmbXEzb7Im92eNpsGga6Q
2RtY2yeZveqbbcVaHjO7CpkgMBuLmi3MmK3NmW0Km9VhvzJr9iT2T+HbCDuN
sHP/FLNEOWzDzgKctWAN4GwLvs01ZhPY18N/NXTCkHswrO9M6N5ZYTaO/WrI
XPXMjsN3J85uevrN/RD8boatG5CZAQ4zsWaD2N04X1Jlth72yqC3H/t9sHkG
+/q07PObzWq/7hazkXJxJqdHMuI1b6pwrIH9o5XyQfuzEINB2DwF3Xuwvxdr
Mu4PgdNBrJGI2Qr8LiJOi6HbFlFMGc8+3L8f6LwT/jeB/2z47q+UDvn+hngH
wNABDBvg/wjON5ruKEPdZcjHIO6WA8PcarMpgXxMB687sHoh8zX4HsE6Dhzz
IROBzE/Qewg+b8sqN1HwfaAgbozjUFi82hHjAcexFJMe8Y/HxZU5ZX5G48LV
BzuLkmZ/A9Oz8HURNj8BnvvwDeBvOuTeg/9iteqCMaUeuYxGlZ+7sLYA1x7I
Pw7bZ7FCuH8adurrwKtOOdmN+8WI0TvIxWr4fd1XLKjDvC2FnSb4WwLM3yFu
XyIGF9KK4Vxg7EFsd5js0j73xyBTgN1vouJFjMRPG60x2WReGSPiZH5WZZUj
D9/zadUv48e6ZtyeB67OjDCsw/4FX/ZYXx9mhbUdem/6ymMK2HZk1J8fAc9U
4OkDngGcb80o/scSZm9hjzDY27Vmw0nV717YW5YWTn43+tr3w0aXLw7diFEX
vqcRtw9wttfX7xuI4bmUzsmZ3Jnfxrj6lXFmbbGOKTOB2vglr9pkjh6LKI7X
cNaDHP8RVf+1+urBpRHpEn8R/j7G2bZJysO7Lhf9FZKlrxbYn+/9b/ucqwP2
8YEa1cpO7EN52RqOSZb55CzjrPrZFMtuF89d4NeQk33ed8QUc8Zyp4vnKchs
9nXHWVByNc+5+FVes5E4rsSEubdK84V4RpCHZFZxYy1+W6m8s/dORKXLGTMG
3Q0ulj+4XDyBHvk0pZqtxfdoXjoNnKn4Pc31GeuS+eG84KzljC1DDYRq5S8N
+bOuDpuA5zD4Xq8QZsaQfumTPUq/CxHjRZ7uxpm3QLOG3EecPLmxP5mvhpTe
AM7/k8DcjF4ejKu3yZP91Ybz1oJqh3NiwtUS5yVnCvuCNcCcMV+M5aiL53nU
9rVAs5bz45lq8e1185eztwNnuUA6v5Yr/7QzD7abPZ3t8TUTWbPER3/E/5qv
fmPeOS+bnAx7NJ1Uvrb5evNYI5xRnP+cUzeTerfI55Wc5g7zHMupH9gLvCNn
cmzDeSipuibGouP7Es5fxiri/C9guJzQ7K/Kam6SL+M95mK+ALYf9pSPf4Az
jhz8GVbcmFfm6EVf84Wzhbk87WrsMux9HohHj6e5z5n/PXI4I6V48o0quXeK
M/JCQnksuR6gHb7bj/qaBZxzrDPWWBF2hvOy8RzuP0vpXeLbfACc1sBOIaP/
B6zXHHrXq9E8OFyjt5B+Wd8drp7ZH1ccfj+jt5B3v4PHpUB2dwXiRl6MLWuU
sSLvktMt5fUWsnbIjxjJl/8RSm7P3hiPap6ztzi3/vvPwneOdbsCXOdUCRtn
Mbmxl6k36nTZT92up/jennBv7qWE3hvWNt8N+qO8D/wnk8LM94a1w/8hs+Dn
ek4yjSnNU2L6F/BAYn0=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlV1slFUQhof+s1uh26+73253twvRkFhKDBhRIikV0paipNpWY0yl0sSf
IqFqQLxQ8QJ/AmLivyAiQqKJIDXBoiBqQrGNelEKaowU9UK0lERBTBC88Xmd
7+Ik55yZeWfOzDtzZvb2d6wpMrNnWCWsRYHZH1mziZjZJKsvY1ZcZ/ZlpdnR
CrMwNEuzFpaZzcGwtcasLW12stzsWNzsPmT3spqQrUDnaMLsFuSnkN8x1ayh
2mwX+ANgz0KnC/xlSbOhUrMD4Len/E77DdgvTTuGbOXjzbzZdlYf+u2s/pzZ
PWB2gdWB/yXsG/E/q9xj6OE9dSmPRXfXYzufNYKsiDdtxHcz+BfwdxG8a9n/
i/559PuUE96+mruvzHPwDtg7Q/en+OrBH096PkrBH8H2Mvr/gDXM3Vl8LeC8
HvsPsX8S/XnorEF2k3KGbBlvHGe/+gqzHYHnQNh3cnecXO2r9beOkbPHyc06
bL4Fb5SYP0L/QXJwK29vnmY2M+U+hN0Dxij12Qp+FecFYBzB9kiBnKK7ZYrZ
IfAPZr0WqyocazDwXApzEfo/VZntJ/b3sS9VPaP4VKPehN9prxpewtdk3t/6
MHcHAueQuHM7eH+ClYvq8TJ3c9n3hR6vcqJc3UB8V0/znDUTfxl4myBnP+t1
9Fvx8SP6D3GeR70/If4vYs7HG7FdyJqDfQPrtZTbSPdF8OqTXjPVSu/5mXji
GY9V/i4lvIaqnfjQgO5Y1nUXF5v9gP7ltOdK/EhTm5DVCT+7WEuIdwC8v5F/
w3vXcr4u71ybJJ6a0GNWrGXEsJF838+5W/2C/lOcRwLfd8f9LeKAaq83BaHX
5P9aYD8d3XNRPOqpJLInonyLs/UzzPYiv4Zz63SzQuA10nuVs46C11C1awK/
EdtO1iDyV4ud27MDj1UcX45tG286Efee3p/1nhZ/DoP3GefPWYei/pZuBe8f
i7vNefZ/sQbJxwP4PIzup1nPvWx60E8j/x3dM6wWzuWcR+PO2ansh4j3Bbh7
PO4yzSDNHunI19Ial8nnOc6xjHNBHFYuusXRcs/JI+j+Qg4mkL8LR04mPcel
0XsHiK0u8LcpvhZsX+K9u8ucg6q9aq5aiwPDgddQtWsD8wK+P0BnHNlc8tEE
/irwF0fz6WPimz3Da6OZthnbTTnvb2H8Bt+2obOy0nMmrqzNeS+JM5oFdxfc
l2w6kL0VeO3UT49yXpdzLqif17N/LJoXmmGanU+HPmt0fh7Zlih+4d/GfnuE
J46UqBewea/Ic5AhttNV3lsT2PSCtTKa/5o/quWKGpeppurlV+p81qmnrwq9
R9QbmlHfg/9d1vtDHFWtmvB/NuY124r9G3mfZbq7Oe2cEVc0Q/eQ6/nVnmvF
uJleztR6b4ozqrU4KO6p5qfA3pHz2XKi2P++Ru7OxPwP3InsYsZl/fi4kv3b
Od9rfuXByuV9lspmCN/DKZ8d+mP0d9xV8NmsGuzL+B+rv1V/TjtYnQXvPeW4
JOEzUbVTD6q3fwV/T6X3eIJ8VYfO1YNT/G9TTIpFM1WztJDy3Gum5gPnsLir
mXqM/ddZn2Wj3D2H/2dzXmvNjA2B/yGyF0fFvdGs64qDp/P+R2r2KablCZ/p
8q+Zt7fWZ6jeL863iPsJj1V/4n9j8S1t
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7210103480549516, 0.7801251933938915, 0.913082177329013], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFV2lslFUUfe0Ms3TaTmfrzHS2GiCUglFkVTaJgLE2EpbEZEpFGgmp0AJt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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlnto1XUYxl+32Zxn7nbOdnY8ZzuLZl6jJFPJW5EaahBeQJhaJIRIOXVb
f1S6IpWSiNJEjGzqsrJSQ/NW0tXILt6iINQkuhikWTDFWVr2eXj844Xv73zf
y/O+7/O+33P93IXTmgsiYh1ShEzNRMxDDvSKeKMwoqwhYnsq4jyXF5CedRHj
+T6ciDiEfF8R8V11xPDrIhZhMw7bS+jsL43YwndRNmJ1VcRzBGnsHfFCTcQ+
bCZx9zI2rfiqSqNXHNEDm0+x/wQpLonohazHfhOyHdt+4HkiF3EuH9FdZkwN
+Msj28G+lPsZYFmAzlSwfd0z4jK+riDPonsPUlQZMT1jbLKZVosu8mqPiGe4
T4N9dsrni9iXJyM6s85P8aegOxnZjP5KdCb2jVhBzNc4t4O3A/0V5NOO/lr0
C7F9q8q534AMB9uInONPpiY/4+sQ9WgCy83YLOZuvu6xnU4Od2H7AP7GUJ85
3O/BXyPxXgfbh/ibm3YNlLt0lpDb48hevo+hs4N8L2acexuYr6Dbv8qxupDj
xHqHeo7vQw7Eq8TfD5XORT35Ke8eqTe/U/P52C/KGdtK7IvqybHeeFXvWeSz
OOV6LOX7QWI/hhwh3z3odHFeRfwKsA8jRoZ635dyLZegvxHfG5B56I7g/iXi
jQdPF/abibeV/PdmnfswdN6m/m/irwv8J9G/Ct42fP6Iv5LyiBvxfSljX1sS
Pi8C74Q+/m1tjWPI9yZkJ7HW8d3JeWSJuZPF35mEObQ7awyK/QEYTtCLXVmf
R6l/+B6Vdq0Vo6LBM6TZyYFnIfcD0W/t4ZrPxv4p8p2TcL80Sx3cb+ttzo1J
Owdh70c/VoEtCcbD1GM99805z6xmVZyfhP3kanNP/NoFH5vy7o10Pud7Tt6x
1I9lOfdIZ2HYSexz9PBWvveDYROxL6NfDPZvNWPUoh3ZCZZZ5LCAczNSSK1O
YnML+O6tcmxhaMRXeYNz/xibAWAbhn7ApzsK3dvTFe69eizutyIzEubUEPB+
BaZDYDkmQX8NNTyS8AyW8T2lxvOmnfUvWK8gJ4qMOVVpzoqr4qBmZ0fWuWmG
7sf/2LRnS5zbyF0DOp09vXNact4pqp8wzSb2S/gcROwh9K+VXFqQJr4f5X4r
fNzW19z+hd8OcP4MebLYmN7H33s51666xFwYhNxWYk5o951J2pd2YB3nkXlj
VU9W13iHane+gqwBy7GMfTVi30S/jubN3aeJ9x+5XU37rB33Yo05JO50IGfF
7TrzWTES9Kq0wbkNwKY/8b+8Vv/BzP9CsO9Omkuq+d/Yzqz0LtXOP06teoGn
nVrfqR2E7VFkedhHAXworLVv1ewId4eRIQW+f5dzfdK5iv/LiXcwaX6qhyly
GZy0b83bIOINrLa9fhsJ9gX13mUP0f+JxLq71rOtN+g82IZXeVeoR+q1dqJ2
oXqex/e+rH2rJqf47r62P5rx+Rtc7SLnXaXOSbPSXeu3T5iF5ZusuSpMpyq8
I+RPNRla5RoJq+5vonab8T+0yG9AUa3fQL19nXDmI85tlZ49zZje3o6csegN
zoJ9dNq7QRzWLtSbo7dGO/FCxjtXuYpjf8GNs8hB4l0q8y7uTnlXayfrbf0z
7zu9sdL9A/mi1Daj8TUm41h6c0pTnnHNtnaCZnEZeGYWmx+d3D9S51y0k8u5
G5D07OuNWJ72ztCukM1pdH+t81urfmzI+j+AvrUTW1J+k/QWqQd90V1W7bda
O3os2Nbjf1TCb+yRtHes9p1msBVutCDPF3jHl6F7ucaxGgvN5XHVzk2cnpDy
fx7919HOmZq7tmMLHENv/z+VnnXlM5bz7XnPivixB/9tOXNX++zhenNU3BQG
7ULZSFc78X8njFNV
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7410462474601278, 0.812317308689702, 0.9135389341240374], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmGlw1tUZxW8Ws73Jm4Ss75IEZbUzgAWLEFA7xbFYK1u1TgVq6dSpqBAU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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl2lw1tUVxm/eJGTfSMKbd8n7RjGA7dDaSFlClE5XrBZarJ2pgaqdVkGW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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7610821468653042, 0.8445094239855127, 0.9139956909190619], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmXmQVdURxi8zLPNm5jG8ebO9Zd6gbBrNojADBo3RLJWwxAguYGQRjBis
YBQR0KglLolgWVZKBWNcQCAyiBsxxoiI7Ahq3BFkESRiRQ3uBhfy/fia8o8z
98y5fc/p7tPn66/PO2zchcMmlSVJMl1/yvU8s2uSjFCbr/9f6ZYk/TNJMkXj
A/TsUZ8kCzolycZCkvRR/yjJpFNJktJzQ4ckOSEtwcokuVL/H67J5kq2pP7x
Gv9acleoP4A51VZr7FjNubvZcs8Uk2SOWlrztNYkyXFqr+ubFZKpyyfJTI3v
Vf+Bote6pcHvGF+p72/Rej/X/G9J97kdk+Qxja/JyS7NX6V53pHMtKok2SO5
Hpp7gWTHqx1Tbpt6qv9pU5K8Kt0urEiSvL7ZJrnfa3yNxvbVJcklkmvSuqOy
SXKevntJ87+o1iTZsVrn45Yk6V2tNSU/TW2Q2nN6/yfpmZXMkFqtV7SedXqO
jnkWar0BVR6/XGN99a5F/w8MPwzRuv26J0mulCT/km3P6P+F+m6Y5Gdr/uVa
52KNLZb+e6TnBPVryqzvJvnhzhp/d4K+mZezPbybqrW+o7UKWmuf5n4lZ7vz
VV5/SeI+708kPsq8znrNeUDyX6kNk/wCPX+iPbpXa5yk54/U1pRZd2wYI598
P/oHbanx//Q3Spej5Z+RyTdy6E/MTFP/CM3fpn5PPS/TnLfK/1sz3t9y+aRM
LS25hZJZHbHYoLGnJTNLMvXqN6r1rHFj7zto/gnS+byS97dOMTOn0josqvJ7
YuNnivFzMo6vkdrbCzL2/Skpn48bNf/2GsfJMq2/MnQmzvur30tyl6uf0nOX
fLNK8pUp+xEfnid/L816L8/RPjyo/t3l/n9Qlfe3Vf7pp7ZC83WRfGe1KxQv
i7VOe419znOR2jz1R6a9J+j2NOdJNnyk/hT2XK1NbbTs3pHzmZ9YZr8ekO0A
AfpWSD6vuW6QrR+oP1pzl1Vafqfmr+qs2CmzzdjeSzI71K/U+KUan0w8qvVV
+6DC/rw5sez2OFN7tf7bakdK1y91doblfIZvl93jJJPR+LguSXKq2l3l9uGY
GvtxuOL84lrvEXj1hcbP0HNZhfVG/qwmxyX+2V9jGfZshXzyqtYqaL51BccT
sYRNI8vsK+wYof6H6r8m2ZX6pij53erPyXtfz6mxnuAVfmrpaF9VSK9Nje53
lF0TJD9X8k9ljFng1WTF+QrpenYn2zRW7ZPEPqaPrei7P+wiZj6vtsyx0mVg
zpgxsLvjGH8eq2ffkuUGB/YcLz9s72zMe08y9XrfoLZZMpOlTy/196pfrRh5
qmh7/yn5beX2VVa6X11yvD+k+O8lbGnX/jyQdbwSq5nwRbnsH9TJ6z6v/lfg
MGdO899eaRl8tVf+Wa29eVfzj9Cc44uOdc4NZ56z00nv36/zecEPZ0Y+miWd
002Oz6/1zX16N0M+Ob3G/Xuly3b5Zpva4Smf3a+afX4f1Drrcs5Xv1B/jPS4
QrZep+eokmMbDEYWHP6BfNKvaNz7t77bkzMW3aiYuaZgvCBm78g6bqfJzkn6
dl1iDJrcyXlqT8b2YAs5Z26czc81fpH2b17KY/Ni/DTZMTxswSbspP8ctiiu
5nVwvuxR7ty6Tjo+JB3O1zk5T/1RtcaNqyQ7Ovp3p32GsfEPmqe6yed9vL5Z
GufrVo03a7y35t9X4bgjVvHTxKJ9NVsyJcn06eA9QQZ9wNoZJePtNslt7ez9
nZi33shgH9iEjXOk2+xGn+9r9N2J3f1ueo05BPzh6QqvuUC6dSgZK8HFao2d
pfcfa56p1Y4dxhP5YGnRsXYu+5zxHKPVr8wYG2q0p1nFVZ+OnqMl5Xmu0vsV
YKv+X6i9bVdbqnnWar66kD9MsXqg1rkCH8zJ2A+c1zFxZjlrzMF5Ix8tiLyV
l26LC+YatTof2W72FfhxTmAd+JFLGTeI3cPUjujgfMU3xA64fsqh+En5O/I7
8VMbc7ZK528XnbcP6h02YustVe7vlK1v5qwb3CDdzfyAs9qm/7eknBfgTeSp
+WmfG2IGDHi4xThAHF2f8Tj7X95sHciz2Iju4PSOorEaboGN5AuwFkwFb4m7
2zKOPTCjf+BGrdZaU7Te6AQvhHMu7ur1ySngVceUMevKKmMlOq+sNReBh5Dz
SinvF3O0BffYpvfb8+as4H1jyhi4odIcEr40ttFYAA4sky5Pqt1UZcznyb7B
c1pjTnTvF33wm/3iHLFXYCT7tV/6nJExv9qgfViv1lXve+oc9Opu/gMHbe3s
mHlCa3at81qLIu+DexOCS4Hzu3LOT+SmGsku1zc3V9nWAaEPZ2BWlXPk2pz5
MXuM3chi+/3a68XBXcAN/EnegUesKnq/yOP52Edy+6LYC/I9eZV9HBD7yLrr
4exF5/Z2zdktZQwHy2dm3Ie3w1/YrxlVlsFXTaEfaz2sOfZn/X6J+h9kvefI
wneRh8PgS7AXv17e7P85a6xPDhoqvy5U/9fl1nFAxNuz2utNRZ874mth2DWk
m+WGypbjJDNEOj+qfqNk2+rdh1sRR/CiT6gDpNtG/f8bxdcFeWP4+zoj75XM
oV6WbpvzPg/wKGwkBj8Lzogd4MikSufuq+sd0/hnvfz2Tq3Hf6o9vShkfqk5
T4FTVphH9wsuDX+cHHwMW+AHvBsmTBpe8Brk2w1556klmuP+nLnbRq1bnXU9
BwflzHJ2l6fN5dD9nrRjhfzyruz7T8l8kBqIWgi/NSmul2uN3vBF6f7HgvMX
efSu4N7Uf7MK3qO/EMvUh9KhTOvPLFiuKrAJzOFb8B75eXE28A/cgzqMdTn3
U4PXUXOCg2D/fNl1b73zyeDQE+7yinR/We2ptGP50uCBzDctbBkU9R085xFy
b9F5apdi7f0m52Z4zYTgNi/Iry+qfZpy/fRB2LBJY8/m/X9bYAg41xZcHb3v
lN1H1RoLxgYP5zz2rjDfAqNGS7f1ld/wL55gAnE/Kfg8eqWz1u24qMmor1dr
/jUFfw/mgX3w4R3S678lr0cO6VDp3PGm5tkuO8fG+nzHWmAAuMx52aJvt+Zd
81MHfB1+5DwwPzqTbzZGH950dPBhzhC2wwF4Tok+um5OGf8vqTbfQf7IFt8j
wIkOrQeHbI37A+QvjvhnvG/UJuhD3ci9ADUFfO3T8Nfjlc7/3Amg29nB676S
/x4pGscL6j8gv/2j0vcHj4d8/9CZPTwhOA01DvMhy10Kvjs8+Bt1+OeBdf1b
fM/CfGvjW2yH38LZwfxVGn86bS5KzQJ/Hxk1HHmaczE/uEcq7mfmRx++Rk4i
H4Hv4DxY00P7/GLJ+MNZnpt2jTo86g/44GmydU+tud3AqN+JIeKQ+BsUdfH+
qLMaA9fA7XrFwuKc52HN9lh3tdZcU/J6cBF0xg7q+IUxdrXwdobaGV3MB3nC
t3sLT2Y3W4cl2otROfNY7ga2tTiG39ceHM79i9Z9VLK7hT/Tq322wa8RgWPL
os/6C8NX7ZEHySvgKfUW+Zpc0x7j4D5zc4cD1yImyR2Xx/ecO3jJE53N33Yq
drYVfVYurXYcE1uv5V03wv2oTUeWjD254IX4kDsn7oxYa6fsWCp7plS71kzF
t/0C48m/7ZpzUT54k9bf19lnPy35rmoduhgb2wJnuLtojXsGMJfzgB/gp2As
MY+9Pbo4b95QMBYQy3OFoz1LrpuxCb3wA/6YHDZSZ4yLWuN2rf/nRtcfcJHx
MY6/7w/fvlbyHQFn50jZ93mDz9cdWrcq8hHzXxJr/U/v9zfYziOk16+y1pNz
NCDuG58hv+XM1Xtrzs8afAapw87Pu0bZIh+/UXTuv6zZd44H6zidg7eDo8CX
4c3g/wNa669Nxq9rpct1UQfiU87nR8GF2oJjgDVgDnn2Pe3jU83+hvk4Q8wJ
zk2qMHY9qffLmz33mGbfMx7KRTyRpw6i/gSjthddq8NfuHM85GPeXxu6HdPs
OCAGwEH2jRy3ljxQMEbPCo7Lt1vlsy055078zp6Tr9lX7jFY99mia29ql3F6
js/5HXdV2ALuwYHAFHCA3Nun5PzLnWXP4L3kGcbINV9qD6fXO/b4jtqYWrKB
3BJ3j+DT8Bh/oeQ7DrCSOpgamPqXM4+PqBPXpR0TcIC/c/fT4PMJv5sbHA9e
Ay5wr8K+w03BE+rx04N73yTdxhR9P1XJXVOd95a1Tw8buW84rmQMHip9rmz0
3UL/yA99gpP0jz7r7IkYmxL35FeG/IDog/fg/8i4G1wZdybLSr5TIIaxb3XY
+Il021TwHuyWrbsaXKcigy84X8y9Nupu9phv0Yd1VsVaK6NP3oEDghPgzO9k
944Wnw34AzbAwXpLn1dL1mFV3I0emmdF9PmdgfqNfAFPmhox/Lr8dlbJ+MS5
ujQwhN8cujX6XIAH36o1JnCvD4eFL1N31kbt2QY34N6qq3nM6LhnY5/hiOw1
3BZMIa4HS/boqJ3JvWvDP9Q4zEcN2zvukRmD83EfTIxRgz4Rd2eLCsasnpG7
iXvigfubt7KO1aaS7/ThQQMl+0W965LBwdGHBtYyBo5ND97OOLmhb3AYcvKp
Befl1UX7Dr9xx7Y27tlmcweZM65ybg7E2QHjmB/Ohm/g8PiHe+6ykm1jT8EJ
eBr3jp81OxfCrW6qMVbdHTUAnIG6dlCLc+WPte5v6zwHe4xvuLtgPngDc5Jn
4FxgLNwYXrw+8iY5E3lqx41RP3LOpjb7rH2YMl/gG+qZwXHn+ahi428Njq17
5IP2Jo8/I/1/WHIOB7PvC9yGXy8pOE9+JN1vrTOuEkvsNXca1N/FJudj4os7
F/R7SfMd39148b0W3x/B9ajj+gfPOTvrnMQ+onNb8BxqrBnNrrPABn4fAh+4
jxwdd5LkKOol8tS59b77Jg75veXCOt9vL457QnCJe9SVcU7JJ+QV/NxX6xzb
bDl+R7qtzrE3rmgsA8cO1utRG1bGvrKn/E5yfa3xkDjN5Bw31J1gCbXn1qLt
xMaTa31vwp0J569vnEHWXhy/WWzX2BstxrvHtOa6JteyxDHP94IHwkUO8rEa
f4uNM2X7bTnr1k9ztLY4Dvmt8rstzjVgTL/gQpPjNxRwkmdr9LkL5I6vT+Rc
dAQnqXcYg1dMk8/ezjgm8dGTFb733pzzbwPF4K3zY0+558tlfW/zsvR8Ke9Y
oG4gn5HLDv1mMy+4ALgFH3ie+9K8Y2Si1jyzxXh4kuLr5O4+b9SaD2eNPf8H
nUXwqA==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3tw1NUVx282ENgkm2SzSTb7yG6AJGhLpYBgVTpW2+kUWmulTgU6hRZU
LB2hLWqgSlvw0Qodx3GQh7aUgKBA0WK02MpLHgIJqDxNeSWAFBnBgopaRrSf
b7/948zc+7vnnvM9557Xr9e4ySMmRUIIn0LdoJ3ZEHZBvaMhXFYQwsVECG2p
ECrYzywJ4UBFCHurQxhcFMLkniE8HQ/hSJqLxSF8NRbCHtZvQhL6EXf6wJtF
xt85X9o9hEXsX0degrM0CrO1IWSgRnTl2fetCuEo+lega2VZCD+Cdwy0Fl0L
C9GVNY/OlsNzmLMHK0MoQH4vYawLYR3nBeg6z52jYDkMNUeMcS54O7iThbcJ
nUfy8EAnkbeG/ZqaEG4H41L4X8HGk/Au4c5QbLuMO+vQ/0rWWB9F/9lcCIvh
L4G3C54O1j24Mx3d96ByB7rboCh3NyO/Iolt7A+y/yI6utC3Bp9+E/5DyJiO
Le3cj3P+SdS21NaDtdw2jUX3qoR9cXMP+3of9HHUPr8G+dci40Zk9eF8APwD
oWXIH8Z5J+uurH03nv318NaC4Q7kXcd+ArLWyibktUDPZWyTbHkZ/03HF3+r
sW+6cR4H69vgfyxY5wZkb4QynL2PvfN4j2eR+U/2Y0tDWA/vTvbn2f8Se4ZU
G7OwvsS+mf0z3J8KvtPsdyO/HB88xj4pPJw9D8VY7yhwLP4HPOeKHJOT4B1N
PC3hfRZDFznbAU+fQvNUobsaOoc/Zuk+60XYdFXMPhyE7VHuX8CeT8B7rspv
ore4GzwrwbMbno5Sy9jCfmDOvMXwzGC9IW7ZN+nNOe9ER6TYMXAq5ZxRrlzO
/gnk/4T3jeGXN8A7F39twefb4R9T6tzYy53aqHMkzt21WcfeI+C5Gd53wHcm
4viZxfkO+Mvg7wn/M5xnEn67VugK/HFjpWNhDj55GP7X4C+B//PgWD8I3s+i
jvlW7r4EpmlgWQZ/f3Lly3nLUkyM4TyZ9dsphvqxbuZbBnzpEttWi0/e7GYb
dyG7HZoYcQzsSfmN9bayMcXdq/KuFQOQd0ixA90bMaYyYuUAPmvC9wXEzGb0
bYFynH2AvYO4exq8kzhvJKc3oXN32rVoCjJfZ70QG4dg2/tR5+4U9u+UOofn
sz4EnjxnfZF3rMY5Kt8egY4iux2Zo7D9x+g4Bu9T0Bb4G6Ct2DoLHYXgbUHH
MO7exvmr8C7FnsPg+zn4T5W55uyDfy+0Ht4NMcficfgbo47J8wnHmGLr1/jz
t+Adhk+XUw9mgmEV75tO2FeK/2PIfgGMd2PPvaW2tStlbLL5r/Cu5nwKZ3fi
vwn47g5oBHiHlzj3RiBzUrFzcBr6ziBjJ74o5A0Hcven2LcEeZ9x/jhnC3iT
8T1c0zvzzkHlnmwUli7295Qa0xnWzxHT/fDHpO6uHatSrtWqIWXoK4f6cv9D
dH4dXd+AWthvBeNofDUv7rfRm3+N/ey4Y1096uWsc0S5MRt896H/V3WuFcoJ
+W5GtWXJhx3Ifkv1iPMU518gN/ZU258N8DyFv76LzOlFzper0fe7uHNFNWgW
vmqrdjwsxGeF6OpW59o5u8Cxe4Y7nTHHsHrRezn3CvUkxc4foRM9HUNN6HsN
mTuQ1V7s2tUEplzENSyG7uE1xqOYegDeSMK65xe71yrnlZ/quVeCfXDWuaj6
vg3fv4u8XxS753xftYZvZyN+8yKwXKh1/qiHzeNt70+614yDTmDb8Tpjaenu
WjkYn44td80cU+mYV6wrpmYmXXPkux/0MPYZSa9lwyCw9efbKu7mwbg16xjQ
289RDeHshkpjV325Iuuepb1qzE3gvSbl/tYKXQLbp3XWtwl8MeJtftLYW4Wf
/YucN5e6pkUrPZNoFlEMxPDfnzLOJdkY4a0Kcq59bcjrRNdm9ot4v2kR9+rt
YFpQ4p7dzrpN/anA3+TrE9hwtMg+16zyNPpKizyzfM5+ZsZvV1VmW54Ew1dK
bNOGOueMcuUh9U/2Z6u8fhB6MmkbZZty8DTybsm4XulN30bWXeTkrcheDKYv
ge2+hH2tmKji7rasa4Fy7jj7dXWWPRW6jretIWZHYNsobD6pfoA9/0D+SGQe
5u4h6Hb8swx5rawvJVwrFmDTCrAszziXFJOrORuVty8VI4NZD8m7VnegYxP3
X826lmtGG87Zt/PGqhj5kPUHeffKJvA9j7xxWcfnn5F3QL0Z2gbW0eiciz/n
pTxrqCYfZF1Z5dhSjRfvWzn3Yt15l3UL71Nc5Jzdw35/zrV5JPeT8ke9c7uh
3LlRQk6u62kMbapP+Ksc3r9wZyi810L3R1zj+7OemPZb3ML9OWAZW2dbLuGD
2fjqD/+PB8XQemwrrfYspRloPPgnom8bvtjY07V8XMpr1XTF5pace49itFG9
Rf0n5h63HN1juNNQ6hn0OG/5G+QVRR2jL9Z4xtNspzdW7OjN9daKoa1gW0TO
Dy30jPJ79o9k3AtVo1bDfzDrWNAMr17/Rtq9Vj1/I1imYs/JQuucBpap0PYC
zyA1Ofc49TbNWKotxeg73s01RrXieynnuuJhLv67kHet1IyhWtwO9S50TY7U
e6bULBkrd23/qNb/LqrxP0v7TfQW6mkl2Hp53rVNMv6HNe7ZXZi3Y+u2jP8V
VKOVq9+qdq1RzrZiy66k/53U856tdU9WL1bPu4R9ibRnQ+WUapFyTrmmmqTZ
ZX+FZyvV4PWcVdc71vQP8G/uP1Tp2qOZUm+pGFLs6E1/CJ6rk57HlJOqLSHh
XFRM9cOWXnnXGvlnFPKeiPtfSDN7o/416j2rT0BGY9wzlGYn9aiP0Xexwr5X
fdrPW+xLuXemo461f6X8r6SYW4H9K2stSzFzA7KvzDvXHy33bLE6639LzRg7
wdue8b+R7Nesp5lYs7BmvmbWhfV+S8VE75R7hHrD45xPhn9QnWcT/QPo3+P6
euvSP8hEdL2QcO1R//kO+t+r8F357GF8ey5nW5Sz89POaeWyZrJbwT4y71lJ
OXiK+6uyns0Uw/p3vK3a/1b6h2wB3+KU/x1UExqQ1SfnWqIa05d1U861Sj3h
LvAPqPNsq5nnSMr/uPq31T/5neB5IOdarH+04rhnHs06+kf+L1vxRX8=
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7792336222864549, 0.8525345321399957, 0.9065930400938198], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmGuwlVUZx9dhnxvnvvc+e5+92fucs7kcTAeUm4x5YAqOwDkgBWgDdkFD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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3mQ1NURxx87yy7szLDMDHPt/nZmOBZLishljHFDJYALeKAsmoBGltMs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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7972272198101821, 0.8585349174577739, 0.8985319245278398], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1l3lw3VUVx2/W9/qShuS9/PL2hhYdmEK0iVMXtNgGWkSTdHGYLtS2QFKq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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlnlsFEQUxqct7dZtrd1u9+geXUENVUQBo6ICFg9AoVwJcsqh5VAQTLz/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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8152208173339094, 0.8645353027755522, 0.8904708089618598], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHtslWcdxx+BttDrzjk9pef0nL4UkFuVQVwyYQJlFBAY4JYJY1xcJEMY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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlmls1FUUxZ+0tNAOS2c6XWalVFkVJWpUFC3QYtiRGFlFoxGjIAQjJkBU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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8332144148576366, 0.8705356880933305, 0.8824096933958798], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHuQ1WUZx19YdlnO7p7dc3b3LLt7zp4F0mTiViloKCCCk2blgHIRqYnL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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3ls1FUQx19LKdstXbq73W27R7dQAVFAVBAVlbNG8cILL9CIEi4Fr8Rb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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8512080123813638, 0.8765360734111087, 0.8743485778298998], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHtw1NUVx38kJGw2m+xmkw1JNrubAIa2ttopIcIooDM+cLDVgoB0dOyA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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl32QlWUZxh/OOXs45+yyH2fPOcvue74WFZ0sacJFmXQZ+0ccrHSXXcBq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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8692016099050911, 0.882536458728887, 0.8662874622639198], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmXuYndMVxr/JnMzlZM6ZMzNnZk7mzJwzKJVQrZISJR48T5EgN7cGIRJ1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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtmHl01NUVxx/ZmEyYSTKZJEMmmRmotkorHo8swhHoco5Y9LDJVqAuQEsF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              "],
              VertexColors->None]}]}, 
          {RGBColor[0.8871952074288183, 0.8885368440466652, 0.85822634669794],
            EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmnuU1dV1x3/DzJ0Xd+6de2eA4c7MvcMf0VaCSs0qqHlIUprmbexqupqo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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtmHlw1dUVx28eCdlelvcSsryXvF/QGanggqUzCHSxrSzaVuky7dSCAmOn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              "],
              VertexColors->None]}]}, 
          {RGBColor[0.9051888049525455, 0.8945372293644435, 0.85016523113196],
            EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmFtwldUVx/dJghWSc75zkhCSQM6hgq0WhCJIZ4oiwXBRW2H6oDgmAW3H
lxbQqaVeQC5KC4gz2mKnqK29j04N6gztgz74AiqXFGGwlPAAtFYTEgi2HQQ7
4/T/Y/0Z+rDPt87+9rf2uv7X2vuL96381oqqlFJJP9V6zs+lVNRYLHpcKaWO
sSmNFv33ckqP1Kf0Xa37oDWltaIfEL19ZEoP6rkixdp1rbF+i94/6vV3XpnS
z7RuhPh26f17Gl/Wmo1ZSk9mwWeq3j0zMubXa9+lWjMseoH2HdcYe9wserbG
ctGP14ccfPteFnMdWv+XtpQOaDxXl9Lz4vdzjdHifUY8720NOX5cSWmzxppC
SrU1KY3UWJ3i/+pC/H+7Jt4x/4zmfplP6Vr9PyxeNXrW63lrLvij+3zp/rIU
bxL9aUNKh5pT+rw63m/3mmHJ8M9y/D8umTdI5k7xqGpJ6V961yc6JzpprJLN
WiRjo/S8QuvmFFOarH1n6nUm3kWNni+k9LS+K4hu0rczNT7T+xs0bhJ9IYVN
bhR9Xs9beOf5GRpPZOFvfF0r/nXaZ7b+N2gswFeaH5AOVdJ/QHSNntWF0HF2
MfY7Cy/zn+41IzROi35R9q6phN8HsnjH/GAKPlensDF6YPOT4lmU7o9p/rZc
2AC9KthNm17FfDn055urNP8f2eermt+kGBloCvm3av7fY8LvvxE9Qeu/kQve
R4vB/64rIy6xD7Fzl2l8eKQ+dFwjerXGJyO0n+S/R+OY1vWKxxTx7dSagvie
0tw3bXN4om+//p/TmKbxO8m2SjF5Qjq2S/65GrWKp6n6X5WPOLmiOeKemB9W
7J6tC7qAvhq7tWYU35XDT5u0dlY5vn9Dz1P6v88xOSofuj9aiH0O5kK26nzE
wFv4QnSv5pbr/xg9v6fnPs1fI97N+t+XBZ+lKZ615sl38EHH3eSV5PyS1kyW
DNeVQ2743Wae+BAamzfquVCj2zSyrjB/6KWW5fZc5N1U88cO+7PAAPy4UPvc
Wo7Y+XYh7HVJlmGvZ898PmQo5IMmlrJ8/IdeUoy9sEF/deT3pFzE7OZS6LhW
fnulJexCDt6scbY6nuTAsOgXtPZr4yM/a6vCr7P07ZgsYgfM4D9+JlaJx1+X
Qhdy5VR15MWuXOQbuVbSfBXYpLnnCuFn+P5X2LKjHO+GRK+sxDen60J+9LhX
6x8cEXwbbHN0fLsYOYMMbyF7Y6zbVIncw0e/1dzUcsTq0XzgIL7Yqvktlfjf
Vgq8I0e+oxw/WwqsfbMYuUqebisED+yDHOyDj97RmmtKkS99su1f9X1e7+/P
xbNLzw+1/5HmyEnWflYf68nvl0qR49Uan2puLrhj+np4Ou+gLzgnifkd+cBB
6PPGRL5N8tfGfNj+j9r31XL8p6a8m8Ve4PRuy0h+w7PDPF/TqDH2HsgH/lLT
wNYux3rRsc379/ORp2/4W3IQWXaYT4/8+Gpd1Jcfyt7DDRFv8JinsUh83sxi
Pbrgc2oTegzpcV0ucLVfa6aYPpOCJk8HS4EXyJCZJzISt8TozeSjnu2m2Xe+
5Wd9h8Ydxr0O14habKkxh72y+PYO85lrmQct28QUNsAW1DJy5qDpPVngApgw
Xj4/IDn/rHd/yscTHDuWxX/oD/2uZPkarQt2fd32JP8bHP9HNPe3fMTz3cXI
/z3GPGovuLdO+84vB9ZQk8YYi6jJva7LYOEoy3DQPkX+sfp2fzn2Rq6dlg3M
PGxfgzdHLAO9zOr6qD+P6fmPcqzrsq3RhR6K3os+6iY9bywHv4c1f1L0IdHz
9OwoB7bhq53G7W7rDr+rioFDxNVXtHZdKTBjsX28yH455Pg8hW/srwHTE4yT
YCQ5Tr8GRoO34NHeXODVH2Sn37dEXaRWkC9g6bosYh2/0Iecca/SWYx8Jh+X
OLePpeiz2gohM/ja3Bjxcs55Tm0tFgIrRzr+mG9Pkatd5tNLD1kfOTecxTes
z2lU+Tvwg3359lLsLjKvWq8hH3KmN+UjZ9AZG9AHkV+LvH6xZYH/NM/VeX6R
/cs+yEPOZJJpVCH4g89zLAN78o58AwPBhoJ9usQ8242B+Gy6fQWNv+hXyL3R
1pF1rAErZpg+b5qesM/xebEHcL70XfKV1xBLC5xbF/M4F/F/wWvwKf3PJMcM
tnrF9qJGTzUWPVsImp7/lGUDt7HlTPPpLrj/rA69iMWJ9gP2mmbZL+pjnDtt
X0yx3Fcbe4b8LfSg519XbLxWDhnptRsq0W/X2h/4YiiL/8yfs27XWz9ocuFp
xeaC8VEfsdlR5zi+wn93OFah8SO4DWZPcrwvsR9bG+OcAybUef6Y4yjnWKIP
oJcAr89b71ucR13FWL9Mz/VZnC8mFMNn+OudzH5K4UPmyS9qCn0hGEXd4zxA
7euzru2OJXpgbLcyC9+DFXuzwFOwlN6Ub6mDYB0YSyzR/9Jb0BMgF1gADjzh
M+BFDC+GDuAAazZkgXvYbdg5gq06rSP/yR1sy9r15glGzPm/Pg2a+ESWuy0P
38ETHLi0jl4FHVi3/FK+pJAHHMNG2JmzC3Zocu7M9boL9sMMf9tu23XbL/iC
XqLbNHbGH9jqI/VBdyp+Tmbh6z5jQJdt0ulcr3WcUAPwF3LNcmyAFZ/Zx/i3
WbHcWonYJQYmugbQI59xz8y5jF4BnXhPb3zYuTLd+TKoHuTacmA/8TLBfHbY
Jr3Gvxucd9iR3pg5zvr58Zdz/Hrj0ua6iCP473JeM8/+yEG/vq0u8ouY5/+E
4uV+nt4aGeY7hkrGe+jFxqWVti21mL6BuniPdWVv6h9ne3IRHAKPkPknhaCR
+evlOM+Tb/D7wDHOeYV+gzr7eCX6cmogNY+c52y4VvPrNB7yvQJnkjHGV3KD
OESeHscSNfNXrcHjkULUPmogZx9yjL6Ifq3o3gAZkXXY9wzIxBltp/sAbEKP
lrle7DMfzh1TytFfEvvgQIPx/EQWdxPk3Tx/S71aaH2XOzePG1tut07L7SN0
bPK+Bde49dLppy2BHwu9F/JzJuD8RhxuqYuYIA44Mz1s3RsrcafDfc7Fe5pC
7LdB809UYh31/yGfl9ifNZxl6OH22274Df+BV7N9jpvpOjPbNLbZ4/X9jkfi
FX5rzHOtaWRh/X7bk/uwrb4T40x3n891n8jOlfroVQ+1xZmNWvax5jfUR439
qClyjPz6gXRaVYnv8Uu9fQev3a5t9Kv7vC99KGd1elGex0zv9DzxCeYha7Ox
ep9p7lDWWhd65h3uUc8Yw652zk40FlDXjrgfGFD/2FO+3PPtNf68KNmfr0Rc
TjRWEBedioHu1ugZiQv6GHKk3jQ6ottu9wbUxl3uH15qjXMjdbN3dNy/8C3f
9ZjP0nKcwye7Z+zxGYpYqvG9HbbpMw4Qw9TIdb6XGjAuPOt8AgfarRt6nTG2
gXHE/gnf34GdfHvadYH/2A1sftP4DDaD0fQSy1rDFtiB/qTf39IXIwu15H3F
ycG20J/zE30gteDllrgrxP6v+D6Angpdd9gOz4+LMznn8Y/Fo78t3g0ac+kD
OS9u9JmROgpuLnbPsNQ2oUahH330/pa4ZySvOQdfOq9Tm+53f8KZaYHPTbwn
D7qM6+xLDIHn5DH9Engwb2xgAv3dFuc+91ubGwNjucPbPi4wbaVxFzynHxvM
oj8j7rABPuJ+9xe+46UX487ibd/poCuxtEv22K3xlGvLNssDP2hqDbI8ZVv1
NMYdEDmHHls8P+T8IE+4nzvWFrbBfstcr6m7Q5aTwR7wX+4769m2YbftyP6s
oc/Ev9z74OOHlE/fr8S9Tq4x7r+4+5rrfgB8/pHef94Qd37H3QuB4e3udYhj
+rJ3XRPxz5O+o2AOeTqM/ycd2xvF88lK1KI6+5k4oYejbpAnnY4Z9L3g3qPD
uTPK+I/P0e0K77nR9fod37Gw/kHt80Al+n1sMGQ7NEnfND7uk6g/4DO1hhhd
6l4RrNnmvKXXeKEStft/90/ugA==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlUtsVHUUxs8MU3y0nekdhtoOM/dWN6IiulYh8cHC14ZE3KgoLI1CfSDI
q61W24IJMWIsgqgxRsQBXOhCFyYKC4WKbbAIbEqE8jB1qHGlCfH35XNxMv9z
z+s7z7lx1Zrlz+cj4gWoACVZxG/liJYiPDTAezrlW3vEaehMnV+ofU7EEegB
ZPdDRWQl6G7ed0FXkeXhh5OIJfAF3i25iD34H0os24n/NfC90FO8L81xrH5i
nmp3TGF5GfmqojEdJ/Y3HRE3gXVnG7iRvQQ9g+wy9j8gPwxtQzYCrUU2iY98
0TEmuiK+w/7OMMbLYFneE/FrKeIo+NZl9ilfinkQPD9C+8AyBM1DVskcey6F
u4L96mrEe9dF7IImeW8h3tq8a6Rc3qoQs+Cc3sd2N/Q2vm8Gw8mya65ad8LP
4G+MeF+iOwaeZQsiHgZfA3xnro1YjPzTbvDxfgL9U+SSg18HvxG+SH7LUvei
E/uZeRG3wTfBu5oYH2HfAM9C8D2CvAP9gN+KrHFNxCvgmUL/F+zHoVt43wrN
YH+RfFaS39PQDnJtYno7srldfl9B50Vy+RZMvS3uyT6wfQadzjmmZuVz4t1Q
8swcoVc1MM0P9+w88j4w9KI7Soyr4L+Xb63ovlp0LotSx1JO8rUZ49kW+zwJ
lkloBfV4HLoP3e3yn7OPBvyh1L38iW938F4sf+Q2RY/GwTNR9yweRv4zsh7w
fI3+V9AB+EuJbfPoH4efBcN4zjpfqHZ12w7Cb8d2Kf0bLHkmjiEf5lsB+SHk
38PvT637AXSeIZjudK0145vQPZc6l+ew30vsasXvpfj4Pf1/ZnifwL6P9yg9
2FF0z7ZhP4vOOWQJOv/A15F3IH8U/w8ieyh171sLthUmYZGPs7z/SLwbE3wL
cvmk4lq9g3yY91DFWNrA1Kx45jRrmgHNbj3xbGuG78HfxrKx6iZo1taX7Vsz
11XxDGr2FhFjBDy5irFphzrwPU19G+RWJ0YN2ceJZ1kzcxR/C8qu7UGoVvY3
vbVT6oVqolqoJ5r9N4mxuegd0C24PnUuugnq1RK+DZTcs3/ht+rmoJtAY8z2
sW7vmnawD9mWzLMhHdVaNtJVzTcxqwN8W1/0zH7ILu2tuhfa6deQ9eseFKyz
B9nuqm/nu+zDX+R6IfUt1k69nrkmqsUG3Th0n6z6tmhHtQt/JrbVTuhWNRPv
lm6WYg1mtlXMKu/uzLetlW9DYC+mvvUDec/6G5l3UTP/GLWZxudZ6jMF7ULW
WfFs6MaN1vhW823UDvah21/1LdPO/43/rsyxdJN1a/Ufof8G3dwVPcag2Iqh
3l+ADrR5BnTbdfN163Xjm9RiZerZ0U5vKBujsD2Lj2H9/2S+ler5RWzH5vuW
ymdW9s5r17Uz6v1I2baagf8AJ1oKww==
              "],
              VertexColors->None]}]}}, {{}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{6785, 6547, 6981, 8267, 4381, 6982, 5544, 4823, 7953, 
              4824, 8053, 8054, 8050, 8052, 8051, 4283, 5359, 5101, 7723, 
              7724, 7725, 6150, 4481, 6751, 6752, 6748, 6750, 6749, 6856, 
              8205, 8091, 8092, 8090, 5142, 8192, 5284, 7542, 5283, 7727, 
              7726, 8055, 5103, 7775, 4507, 8247, 5144, 4727, 7904, 4728, 
              7905, 4587, 7857, 4586, 8274, 7690, 7691, 5257, 5256, 5865, 
              4511, 5146, 7743, 7742, 4732, 7908, 6359, 6358, 4590, 6753, 
              6813, 6812, 7940, 4489, 7741, 7740, 4731, 7907, 4589, 7885, 
              4636, 7883, 4635, 7884, 4745, 8253, 4746, 4730, 7906, 4729, 
              8277, 6886, 8278, 4744, 7694, 7695, 7643, 7642, 6870, 4734, 
              6869, 4735, 4748, 8254, 4747, 7921, 4639, 7886, 4640, 7887, 
              5361, 8070, 5118, 7745, 7746, 4492, 6131, 7321, 7322, 6756, 
              4592, 6755, 4591, 7909, 4736, 7750, 7751, 5148, 4514, 6179, 
              4513, 5293, 7551, 6569, 6568, 6402, 4594, 7911, 4738, 7910, 
              4737, 5261, 8263, 5260, 8074, 5121, 7752, 7753, 7754, 6159, 
              6160, 5120, 8071, 8073, 8072, 4321, 6416, 4313, 6401, 6400, 
              6605, 6604, 4493, 6158, 7749, 7748, 7747, 5290, 5501, 5500, 
              8066, 8067, 8065, 8069, 8068, 4880, 7654, 7655, 7656, 5581, 
              4301, 4398, 6566, 6565, 6816, 6815, 4294, 4392, 4858, 7965, 
              4859, 8063, 8064, 8062, 5115, 7854, 4585, 7853, 7856, 7855, 
              7881, 8093, 4488, 7739, 7738, 5111, 6354, 4287, 4386, 4852, 
              7960, 7962, 7961, 4628, 7955, 7956, 7630, 7631, 4282, 4383, 
              5861, 5862, 5099, 7721, 7722, 4480, 8089, 7874, 7851, 7852, 
              7632, 7633, 7634, 5100, 8047, 8049, 8048, 4822, 7619, 7620, 
              7621, 4380, 6980, 6784, 6785}],
             RowBox[{"-", "7.5`"}]],
            Annotation[#, -7.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            TooltipBox[LineBox[CompressedData["
1:eJwl1ElsllUUxvGDkcFoawfoV6m0VBMIGkACMamwgoXI1EaJJIBMK8EoagTL
pAktSkIH7A5KgE4GsImFIuNChoQWnHBAaYPWRDv4LYwWrGyg/t64+Pe559xz
nnvufb+0eN3GF98YERGT/bmREzG3IOIHOo+2jos4mxXx1YSI5eKVmDoxYgXN
sbeGrsUrWI1VaJe/nh/xDb7GIK88ueLsiJ/4/Ii8oogUOgojriClppPm07rx
EUO0wP4Gfq9hn1w99qNQfqPcmzgobkQDnpCvlqtBk7gFzXhK/mk0yS81+1Te
1501jZaapQz3zHkfwxiTingIs3IjevTMpL/SYXry8YiXeAy5y3ke/9I/aBoD
qODZ56x+9GLv2IgqOpIutL8INXqynVNLl4gXJ7lkdgygNulDqXwZ9qrLUZ+L
0Xw+sjeK1tG0b3Pb2/5uvl78hn70od6s/7jHEB5Q/5n4NE6hW99svrn8x+KC
+hJ+T7rjRevp7nGDb7b4oPwhvKpuPT7Rd4fnMdos35K8LQqdcZVeS74rlok7
6VY+23BZXKO/NrlT8lbu8yWPMlotrsIevKzudfWt9j61nu+92+gEHObXiAYc
sT/JrEdp+jHfAQO+dz8G1U6297l7zEl+H3xL+DyHE9btWCd/09lN+qbJd3iT
KbxmWm/nvwNt3uquN/jbff/Ccr+Vt/i+jS693ejUd8686byILXoG1ZfT23QU
r9F4EB+baSStdMYG9d+Ji/VPxAG5NWZai2cwI3kfuRHqd9L16r9VX682RYvo
fvln7beKN7vnz3p68Av67H2h5qj5j+FPs5wRn8Yp3FLfZd6bmO9OFTyepwvw
AjLtf+h9MuhuekWu3T13WX+AnahEBU7Iv0ffxzbswHYcl99Ct2IzyvEuLvG6
iE3Wj/B/hz5MH/VmWcjA9+JMWuAeWeP+/7/0H3wWugw=
              "]],
             RowBox[{"-", "10"}]],
            Annotation[#, -10, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4817, 6977, 4818, 5543, 5542, 6322, 6321, 6320, 7948, 
              7947, 6976, 6546, 4618, 6771, 6975, 6974, 4378, 4271, 7616, 
              6793, 4624, 6548, 6549, 7000, 7001, 6998, 4844, 6999, 5553, 
              5554, 4851, 6353, 7640}],
             RowBox[{"-", "12.5`"}]],
            Annotation[#, -12.5, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4918, 7137, 4919, 7138, 4898, 5898, 5258, 7171, 5259, 
              7505, 4917, 7170, 4943, 7168, 4942, 7169, 5291, 7548, 8287, 
              5123, 8075, 5124, 7509, 7510, 7508, 5265, 5372, 5371, 8083, 
              5128, 7761, 7762, 7763, 6164, 4497, 7765, 7764, 8085, 8084, 
              8169, 8170, 8086, 5129, 7766, 7767, 7768, 6165, 6166, 5131, 
              8087, 5130, 8088, 4343, 5646, 7175, 7176, 7174, 7178, 7177, 
              5427, 7173, 4949, 6431, 6432, 4331, 5620, 5920, 5921, 5600, 
              4661, 6135, 4660, 7106, 7107, 6405, 7662, 7663, 4412, 5414, 
              4900, 6403, 6404, 4315, 7374, 8105, 7898, 7899, 7897, 4721, 
              7900, 4539, 7806, 4776, 7932, 8044, 8042, 8043, 7931, 4775, 
              7804, 7805, 8223, 5318, 7825, 7826, 6276, 5316, 8222, 5317, 
              8219, 8221, 8220, 4306, 8150, 8151, 8149, 5222, 8148, 5223, 
              6231, 6230, 4568, 6718, 6717, 8262, 5082, 8031, 7697, 7696, 
              8284, 4867, 7971, 7055, 7057, 7056, 4641, 6130, 4642, 5568, 
              6371, 4302, 5569, 7030, 7031, 7029, 4860, 7032, 5401, 7027, 
              7028, 6363, 6364, 4297, 5563, 5867, 8059, 5108, 8058, 5109, 
              8202, 5289, 8201, 5288, 5484, 5483, 6300, 5145, 6178, 7780, 
              4509, 7778, 7779, 7735, 7736, 4485, 6155, 7734, 7733, 7732, 
              5287, 8200, 4847, 5863, 5853, 5854, 4845, 6348, 7339, 7340, 
              5104, 6334, 6333, 6332, 4832, 6984, 4827, 7499, 5252, 6325, 
              7624, 5847, 4825, 6983, 4826, 6323, 7623}],
             RowBox[{"-", "12.5`"}]],
            Annotation[#, -12.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4272, 5541, 4270, 6319, 5326, 5842, 5841, 6127, 5845, 
              5846, 5325, 5844, 5843, 5547, 5860}],
             RowBox[{"-", "15"}]],
            Annotation[#, -15, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4946, 7172, 4947, 5919, 5918, 7506, 5262, 7200, 5263, 
              7507, 4944, 7198, 4974, 6446, 4973, 5941, 4948, 7203, 4978, 
              6450, 6451, 4342, 5673, 4354, 6839, 6840, 6838, 6842, 6841, 
              4355, 5675, 5676, 4979, 6452, 6579, 6580, 7667, 4435, 5975, 
              8272, 8301, 5647, 8299, 5648, 5978, 5977, 5435, 7204, 7205, 
              6453, 6454, 4344, 5649, 7183, 7184, 7182, 7186, 7185, 5428, 
              5942, 5943, 5622, 5621, 4322, 4418, 7998, 7999, 7144, 4920, 
              5602, 5601, 5900, 4901, 7375, 7376, 6637, 7786, 4528, 5899, 
              4527, 8107, 8108, 8106, 5152, 8140, 5209, 8139, 5208, 7861, 
              4593, 7860, 7863, 7862, 8176, 6256, 6257, 5331, 6301, 5486, 
              5487, 5297, 8209, 5298, 8210, 5151, 8102, 8104, 8103, 4525, 
              6952, 4526, 6636, 6635, 6890, 6889, 5363, 5364, 5224, 7470, 
              5225, 7471, 4868, 7567, 7568, 6372, 4303, 4399, 5871, 5570, 
              7972, 5571, 5875, 5874, 5405, 7058, 7059, 6373, 6374, 4304, 
              5572, 7037, 7038, 7036, 7040, 7039, 5402, 5868, 5869, 5565, 
              5564, 4291, 4390, 7693, 7692, 8283, 4855, 7964, 4856, 5559, 
              5558, 7017, 6798, 6799, 6795, 7016, 7015, 7957, 5550, 7011, 
              7012, 7010, 4846, 7013, 4836, 5852, 6338, 6337, 5254, 7501, 
              4831, 7500, 5253, 6331, 7625, 7626, 5850, 5851, 4829, 6324, 
              6329}],
             RowBox[{"-", "15"}]],
            Annotation[#, -15, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4518, 6296, 4519, 6882, 6883, 6880, 4743, 6881, 6885, 
              6884, 4517, 6295, 5480}],
             RowBox[{"-", "17.5`"}]],
            Annotation[#, -17.5, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4976, 7201, 4977, 7202, 4945, 7199, 4975, 6447, 6448, 
              6449, 5671, 7675, 7674, 6476, 7227, 7226, 4681, 6013, 5444, 
              5445, 5007, 7231, 6843, 6844, 6584, 7234, 7712, 4447, 7235, 
              6845, 6846, 6586, 7236, 8261, 8286, 8285, 5009, 8016, 5010, 
              7238, 4682, 7206, 7207, 6455, 7668, 4345, 4436, 5976, 5651, 
              5652, 5981, 5980, 5436, 5979, 4671, 5650, 6433, 4332, 4426, 
              8003, 8004, 7179, 4950, 7181, 4665, 5623, 4922, 7145, 4923, 
              7148, 5422, 5922, 5923, 5604, 7377, 5603, 4316, 4413, 6570, 
              5182, 7807, 7808, 4541, 6182, 4542, 8127, 8128, 8126, 5186, 
              8129, 5184, 6213, 6674, 6212, 5183, 8125, 5185, 8122, 8124, 
              8123, 4540, 8141, 7981, 7982, 7980, 4882, 6567, 4404, 5502, 
              5829, 4800, 6232, 4801, 8153, 8154, 8152, 5226, 5885, 4884, 
              6303, 5332, 7569, 7570, 7076, 4883, 6302, 6313, 8232, 8233, 
              7606, 5348, 7572, 7571, 6823, 8256, 6822, 7974, 7975, 7973, 
              6376, 7647, 4305, 4400, 5872, 5873, 5575, 4643, 5878, 5406, 
              5876, 5877, 5574, 5573, 4298, 4394, 7967, 7968, 7033, 7035, 
              7034, 4637, 7020, 4857, 6356, 7641, 6560, 6559, 6557, 6804, 
              6803, 4288, 4387, 6554, 6552, 6797, 6796, 7637, 7636, 5396, 
              5395, 5864, 4625, 6991, 6992, 6990, 4839, 7954, 5546, 6986, 
              4834, 6336, 4835, 6987, 4830, 6985, 4833, 6330, 6335}],
             RowBox[{"-", "17.5`"}]],
            Annotation[#, -17.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4276, 7627, 6339, 6989, 6988, 4623, 6251, 6252, 5393, 
              4382, 4278, 5549, 4284, 7635, 6349, 4848, 7014, 4627, 5552, 
              4849, 6800, 4629, 6553, 4630, 6801, 4853, 6805, 4633, 6558, 
              4634, 6806, 7022, 7021, 5566, 6366, 6365, 6355, 4861, 5403, 
              4395, 7645, 7644, 6368, 4862, 7041, 4644, 7062, 4870, 7060, 
              4869, 7976, 4401, 7649, 7648, 6378, 4871, 7063, 4648, 5582, 
              6385, 6375, 4885, 7077, 5409, 5890, 4886, 7515, 5267, 8177, 
              8178, 7517, 7516, 7698, 4405, 4307, 5365, 6294, 6293, 5083, 
              5888, 5887, 6720, 6719, 8155, 5227, 7842, 7843, 4570, 6233, 
              4569, 6757, 6758, 7866, 7865, 4595, 7864, 8281, 4792, 7937, 
              7827, 6895, 4752, 6892, 6894, 6893, 8142, 7443, 7444, 7383, 
              8110, 5156, 7384, 5155, 6183, 7787, 6641, 7382, 7381, 5153, 
              8109, 5154, 7378, 4921, 7146, 8001, 8000, 4419, 4323, 5624, 
              5625, 4951, 5944, 5429, 7190, 4954, 7180, 4953, 5653, 4672, 
              7210, 4981, 7208, 4980, 8009, 4437, 7670, 7669, 6457, 4982, 
              7211, 4683, 7241, 5012, 7239, 5011, 8017, 4448, 5446, 5014, 
              7237, 5013, 7242, 5702, 4364, 7680, 6850, 4699, 6587, 4698, 
              6849, 7679, 6848, 4697, 6585, 4696, 6847, 7678, 5696, 4694, 
              7233, 5008, 7232, 8015, 8014, 5674, 6011, 4446, 5443, 6255, 
              6254, 4680, 7229, 7230, 7228, 8013, 8012}],
             RowBox[{"-", "20"}]],
            Annotation[#, -20, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4353, 5672, 7511, 7512, 7264, 5266, 7513, 5005, 7262, 
              5036, 6492, 5035, 5697, 4695, 6169, 6170, 5456, 4455, 6043, 
              5698, 5699, 5136, 5457, 4456, 6044, 5700, 5701, 5137, 5458, 
              4457, 6045, 6046, 5703, 4700, 6047, 5459, 7268, 7269, 6495, 
              6496, 6497, 5704, 7244, 5016, 7240, 5017, 7245, 5447, 7243, 
              5015, 6456, 6477, 6478, 5677, 7215, 4984, 7209, 4985, 7216, 
              5437, 5982, 5983, 5655, 5654, 4333, 4427, 8005, 8006, 7187, 
              4952, 7189, 4666, 5626, 4924, 7147, 4925, 7149, 7325, 7324, 
              7379, 7380, 5300, 7563, 5299, 6269, 6181, 4557, 6705, 5503, 
              4596, 6277, 5366, 8036, 8037, 8033, 5084, 8032, 8035, 8034, 
              4750, 7922, 4751, 6891, 4749, 6638, 6639, 6640, 6914, 6913, 
              5590, 5591, 5506, 4597, 5886, 7564, 7565, 7472, 5228, 6722, 
              7844, 5891, 4887, 7573, 5333, 6386, 8296, 4308, 4406, 5889, 
              5583, 7985, 5584, 5893, 5892, 5410, 7080, 4888, 6377, 6387, 
              6388, 5585, 7065, 4873, 7061, 4874, 7066, 5407, 7064, 4872, 
              6367, 6379, 6380, 5576, 7045, 4865, 7044, 7047, 7046, 5404, 
              5870, 4638, 5567, 6357, 4292, 4391, 5400, 5113, 5561, 5560, 
              4289, 4388, 5399, 5110, 5557, 5556, 4285, 4384, 5397, 6152, 
              6151, 4626, 5551, 6346, 6345, 4842, 6997, 4838, 7502, 5255, 
              6341, 7628, 4277, 5545, 5855}],
             RowBox[{"-", "22.5`"}]],
            Annotation[#, -22.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5038, 7265, 5039, 7266, 5006, 7263, 5037, 6493, 6494, 
              4362, 7352, 4369, 7356, 7357, 7355, 5134, 6172, 6171, 6611, 
              6610, 6609, 5135, 7358, 5388, 7359, 7360, 6614, 7769, 4501, 
              6173, 4502, 6616, 6615, 7362, 7361, 5389, 7363, 7364, 6617, 
              7770, 4503, 6174, 4504, 6619, 6618, 5138, 7365, 5390, 5391, 
              5139, 7366, 5140, 7367, 5040, 7601, 6498, 6499, 4458, 8023, 
              8024, 7267, 7271, 7270, 4701, 6015, 4702, 5705, 6500, 4365, 
              5706, 6017, 6018, 5678, 4685, 6014, 4684, 7217, 7218, 6459, 
              7671, 4346, 4438, 8010, 8011, 7212, 4983, 7214, 4673, 5656, 
              4957, 7188, 4958, 7191, 5430, 5946, 4667, 5627, 6417, 4324, 
              6573, 6572, 7554, 5309, 8216, 5310, 8213, 8215, 8214, 4545, 
              7810, 7427, 7428, 6759, 7868, 4600, 8305, 6760, 7811, 8255, 
              6678, 7934, 7935, 7933, 6675, 7809, 4543, 5780, 4544, 6676, 
              6677, 6571, 7889, 7701, 7702, 5504, 8265, 5505, 4802, 6721, 
              6723, 6724, 5229, 7473, 5901, 5902, 5334, 7574, 7108, 7109, 
              6304, 6314, 6315, 5349, 8228, 8229, 8230, 7575, 6824, 8257, 
              8258, 7988, 7989, 7990, 7986, 7987, 7657, 4309, 4407, 7983, 
              7984, 7078, 7082, 7081, 4649, 5880, 4650, 5586, 6390, 4310, 
              5587, 5881, 5882, 5577, 4646, 5879, 4645, 7048, 7049, 6369, 
              7646, 4299, 4396, 7969, 4863, 7042, 4864, 7347, 5117, 7346, 
              5116, 5497, 4588, 7345, 5114, 6602, 7744, 4491, 6157, 4490, 
              6601, 6600, 7547, 7546, 4584, 7342, 7343, 6599, 7737, 4487, 
              6156, 4486, 6598, 6597, 7545, 7544, 4583, 7341, 5105, 6596, 
              7730, 7731, 6153, 8245, 6154, 5285, 7543, 8195, 8194, 5495, 
              7335, 6994, 6995, 6344, 4841, 6996, 4837, 6993, 4840, 6340, 
              6342}],
             RowBox[{"-", "25"}]],
            Annotation[#, -25, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            TooltipBox[LineBox[CompressedData["
1:eJwV00lszVEYxuGPxJSSmGnNQ5WISAwxxkIisRCElXmIuVHCRkJjYVhgJWgp
YowpxNDqSonZva2Zls6tIkFJ6I7Ecxe/+55vOO/5zvnnDlq5aV5Oq4hI95PW
JWJHr4g85KNT94hj9CgOqj1PjzjVI6JD14hSufa0PCOiAm/V3uFX/4i98mvo
WkwZGDEVb6zf4h1eYwiG4jS/b/Q7PvCpxEecdV4Tv88oQab6Vrkz+n9YN6NK
Xw2qcU7ti76vKFerwHsMQxbO2ncOu81WpacaNSjr6y4oc59b/SLq3PkmracX
e0dcwDV8F9+V/0FL6B00WBfSRlpEb3Vzb56tnNca2WZaMCDihvw6Z9ykG+gy
M8ymBeJ5+v/a+w8r9M804zH5yfJzMBeje0aM1b9dfhQdj3GYpne5uafzy9Az
WG9z54hZfCbJL9I/3OyLaaX+KnzEDTNlmvk6rRFnWVfTOtRiC7/59q3ivxrF
vG8jl28hLUITz432f6L/nPVcbYPvMNq3zqbrMcadxmIT/xni895kId8cca29
m2lVagZUo5LHYd5P+kQ8xWMM49+idkRvuv3j+PemK+hhud9qf7DcOctwSG4p
veS8y+hstix3SPIqRYvz23qndqiQvybXxxkXaF86RG6iOSbwTBdfkc+gV+lI
73KCdhI382lIfXuM0T9Cf5p8gXpHmuSTwD5xa/EuegD7ke+NE0jiGZ6m1ubJ
5f/L/I98wzPi08hzh4T75tNGcUPqP+i8N6n/HF7iNV7hvn2Zasf1FKAedTiZ
+u+qv0ASZShFif489SPYJ37knId4gFq5pXQJ9pjrZyqn54F8gj5Ds1nbeIcm
/tvc9Z7aTusG37AR3+x7ol7snDpxPf4DjxfCbA==
              "]],
             RowBox[{"-", "27.5`"}]],
            Annotation[#, -27.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4370, 7689, 8172, 8171, 8175, 8174, 8173, 7295, 8028, 
              8027, 5294, 8206, 8208, 8207, 6180, 5132, 6167, 7550, 7549, 
              7353, 7354, 6607, 6608, 4500, 6168, 4499}],
             RowBox[{"-", "30"}]],
            Annotation[#, -30, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{5090, 7333, 5091, 7533, 7534, 6591, 4468, 5281, 8187, 
              8189, 8188, 5280, 7532, 6851, 6852, 6094, 5468, 7298, 7299, 
              6501, 6522, 4372, 5737, 7280, 7281, 7273, 7283, 7282, 5460, 
              7277, 7278, 6481, 6503, 4366, 5707, 6019, 6020, 5680, 4687, 
              6016, 4686, 7222, 7223, 6463, 7672, 7673, 4439, 5439, 4986, 
              5657, 6460, 6461, 5659, 5948, 5949, 5158, 6648, 7792, 7793, 
              5947, 4529, 6646, 5160, 6650, 6651, 6652, 7451, 7450, 5507, 
              7447, 5212, 7446, 5213, 6226, 6225, 4559, 6142, 6143, 5211, 
              7445, 5210, 8143, 4530, 5784, 7795, 7794, 6649, 4755, 7788, 
              8248, 7789, 7385, 8111, 7790, 7791, 6953, 6954, 6645, 6644, 
              6642, 4753, 5367, 7703, 8268, 6235, 5233, 5903, 4904, 6258, 
              4905, 5416, 7476, 4414, 7994, 4903, 7113, 7118, 7117, 4656, 
              5894, 4657, 5593, 6408, 4317, 5594, 7092, 7093, 7084, 7095, 
              7094, 5412, 7089, 7090, 6381, 6391, 4311, 5588, 7070, 7071, 
              7068, 7073, 7072, 5408, 5884, 4647, 5580, 6370, 4300, 4397, 
              6564, 5147, 6628, 7781}],
             RowBox[{"-", "30"}]],
            Annotation[#, -30, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{6253, 6268, 4505, 7772, 7771, 8204, 8203, 5141, 6147, 
              6146, 7538, 7539, 7336, 5098, 6593, 6594, 6148, 5097, 7541, 
              5282, 7540, 8191, 8190}],
             RowBox[{"-", "30"}]],
            Annotation[#, -30, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            TooltipBox[LineBox[{4280, 5776, 5778}],
             RowBox[{"-", "32.5`"}]],
            Annotation[#, -32.5, "Tooltip"]& ], 
           TagBox[
            TooltipBox[LineBox[{4371, 5777, 5779}],
             RowBox[{"-", "32.5`"}]],
            Annotation[#, -32.5, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4878, 5883, 4403, 7653, 7652, 6384, 4879, 7074, 4652, 
              7091, 4894, 7087, 4893, 7992, 4409, 7659, 7658, 6393, 4895, 
              7096, 4658, 5908, 4907, 5596, 4906, 5418, 5417, 7121, 4908, 
              7114, 7120, 7119, 7578, 4326, 7666, 7665, 8160, 5234, 8161, 
              5235, 5906, 5905, 4572, 6726, 5509, 4602, 5831, 4601, 6828, 
              6829, 6643, 4662, 6827, 4777, 6915, 4778, 6916, 4754, 6214, 
              6215, 5783, 5782, 5785, 4420, 4325, 7449, 5214, 7448, 5215, 
              6219, 6218, 4560, 7828, 7430, 5189, 7429, 5188, 7326, 5085, 
              6647, 7328, 7327, 6435, 5662, 4959, 5950, 4675, 5985, 5986, 
              5664, 5663, 5993, 5681, 4356, 6483, 6462, 5020, 7249, 5449, 
              5450, 6484, 5711, 5022, 6021, 4704, 7279, 5045, 7275, 5044, 
              8026, 4460, 7684, 7683, 6505, 5046, 7284, 4711, 7300, 5063, 
              7296, 5062, 8029, 4469, 6524, 6523}],
             RowBox[{"-", "32.5`"}]],
            Annotation[#, -32.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4312, 6394, 6383, 7098, 7097, 5413, 7101, 7102, 7088, 
              7100, 7099, 5599, 4319, 6411, 6392, 7126, 7125, 5419, 5911, 
              5912, 5598, 5909, 5595, 5907, 4415, 4318, 7664, 6410, 7995, 
              7996, 7122, 6830, 6831, 7579, 7580, 5350, 7607, 8235, 8234, 
              6316, 6236, 4928, 7152, 7577, 7576, 5327, 6297, 4929, 5924, 
              5236, 7477, 7479, 7478, 4804, 6237, 4803, 5830, 5508, 4421, 
              6574, 4927, 7150, 4926, 7151, 7453, 7452, 4546, 8130, 5190, 
              7431, 5191, 6217, 6216, 4548, 6220, 6221, 5193, 7432, 5192, 
              7433, 4547, 5771, 6681, 6680, 6679, 5187, 6575, 4429, 4335, 
              5660, 7395, 5661, 5990, 5989, 5440, 5994, 4991, 5665, 4990, 
              5682, 4688, 7250, 5021, 7247, 8019, 8018, 4450, 4357, 5708, 
              6507, 5710, 6049, 6048, 5461, 7287, 7288, 7276, 7286, 7285, 
              5738, 4373, 6525, 6504, 7304, 7303, 5469, 7308, 7309, 7297, 
              7307, 7306}],
             RowBox[{"-", "35"}]],
            Annotation[#, -35, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4896, 5895, 4410, 7661, 7660, 6396, 4897, 7103, 4659, 
              7127, 4910, 7123, 4909, 7997, 4416, 5420, 4912, 5597, 4911, 
              5910, 5605, 6419, 6418, 6409, 7158, 7157, 5423, 5930, 6833, 
              6832, 7521, 5271, 8182, 7522, 5273, 8183, 5272, 7524, 5275, 
              7523, 5274, 7156, 4930, 7153, 7155, 7154, 5788, 4573, 8162, 
              8163, 7481, 5239, 7482, 5237, 6238, 7846, 6727, 5238, 7480, 
              6436, 6764, 7869, 6765, 6707, 6706, 4561, 6144, 8046, 5095, 
              7334, 5096, 5817, 5816, 4478, 5818, 5819, 5479, 5478, 6229, 
              4723, 7402, 7403, 7400, 5167, 7401, 8114, 5163, 8115, 7396, 
              7398, 7397, 4989, 6188, 4988, 5987, 4440, 4347, 6464, 5684, 
              4689, 6022, 5451, 7255, 7256, 7248, 5025, 7254, 5712, 6508, 
              6506, 5709, 5047, 5462, 4461, 7686, 7685, 6510, 5048, 7289, 
              4712, 7305, 5065, 7301, 5064, 8030, 4470, 6527, 6526}],
             RowBox[{"-", "37.5`"}]],
            Annotation[#, -37.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4320, 6412, 6395, 7129, 7128, 5421, 7133, 7134, 7124, 
              7131, 7130, 5613, 4328, 5606, 5608, 5914, 5913, 4663, 5928, 
              4932, 5610, 4931, 5927, 4423, 6420, 5424, 4934, 6259, 4933, 
              5929, 6240, 6239, 4336, 5786, 4756, 5925, 4757, 6955, 6956, 
              7942, 7941, 4805, 7394, 5162, 7393, 5161, 6898, 4760, 6655, 
              7796, 4532, 5789, 4531, 8144, 8145, 7454, 5217, 7455, 5094, 
              6145, 6708, 6228, 5216, 7459, 5218, 7456, 7458, 7457, 5510, 
              6190, 4533, 7798, 7797, 5303, 7405, 7404, 5164, 7399, 6653, 
              6654, 5991, 5992, 5166, 5988, 5165, 6187, 5995, 5685, 6486, 
              6485, 5683, 5023, 5452, 8021, 4452, 6488, 6487, 5715, 5026, 
              6023, 4705, 6050, 6051, 5717, 5716, 6058, 5740, 4374, 6528, 
              6509, 7311, 7310, 5470, 7314, 7315, 7302, 7313, 7312}],
             RowBox[{"-", "40"}]],
            Annotation[#, -40, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4913, 5915, 4417, 6414, 6413, 5618, 4914, 5614, 7132, 
              5612, 6423, 6422, 5607, 4937, 5932, 5425, 5933, 5934, 5611, 
              7586, 5609, 5926, 4422, 4327, 6421, 5336, 7581, 4935, 5931, 
              7847, 6729, 7484, 7483, 5370, 5787, 4604, 5512, 5629, 5628, 
              5820, 5821, 5489, 5488, 5791, 4758, 6896, 6897, 7924, 7923, 
              4759, 8279, 7706, 7707, 5369, 5368, 6279, 6280, 5320, 8224, 
              5319, 8292, 7829, 7830, 4562, 6281, 6282, 6270, 5301, 6278, 
              5302, 6186, 6185, 6136, 6137, 5996, 4993, 6189, 4992, 5686, 
              4690, 7253, 5024, 7251, 8022, 8020, 4451, 4358, 5713, 5714, 
              6055, 6054, 5463, 6060, 5052, 5718, 5051, 6059, 5739, 6529, 
              5741, 5067, 5744, 5066, 6095, 4471, 6531, 6530}],
             RowBox[{"-", "42.5`"}]],
            Annotation[#, -42.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4329, 5615, 6415, 5617, 5917, 5916, 4664, 7161, 7162, 
              7159, 4936, 8002, 4424, 6425, 6424, 5634, 4938, 7588, 7589, 
              7587, 5339, 7608, 5352, 7609, 5351, 6305, 5335, 7585, 5337, 
              7582, 7584, 7583, 5954, 7485, 4574, 6730, 6728, 4806, 8300, 
              6957, 4603, 5511, 6437, 7192, 4668, 5630, 6684, 4549, 5790, 
              7813, 7812, 6683, 4779, 6920, 6921, 6918, 4781, 6919, 6923, 
              6922, 4606, 6686, 4605, 6766, 5195, 7436, 7815, 7814, 8218, 
              8217, 5196, 7330, 8038, 8039, 7329, 6581, 6466, 6465, 5689, 
              4691, 6026, 5454, 7258, 7260, 7252, 5030, 7257, 5719, 5720, 
              5050, 5726, 5049, 6052, 4462, 7688, 7687, 6512, 7291, 7290, 
              4713, 6097, 6098, 5743, 6096, 5742, 6103}],
             RowBox[{"-", "45"}]],
            Annotation[#, -45, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5070, 5745, 5071, 6104, 5471, 7316, 7317, 6511, 6532, 
              4375, 5747, 6067, 5724, 6053, 5725, 6057, 6056, 4706, 6030, 
              5031, 5723, 6489, 5455, 7259, 5453, 6027, 5028, 5688, 7407, 
              5687, 4348, 4441, 8131, 8132, 7434, 5194, 7435, 5173, 6199, 
              7816, 6687, 5197, 7437, 7537, 7535, 7536, 6917, 4780, 6682, 
              6685, 7833, 8288, 5219, 8146, 7834, 8259, 8260, 8007, 8008, 
              7193, 6438, 6577, 8240, 4337, 4430, 6576, 8270, 8271, 8269, 
              8298, 5832, 8297, 5241, 6241, 4807, 5833, 6733, 6731, 5240, 
              7486, 4961, 6306, 7196, 7195, 7194, 5338, 5951, 4960, 6307, 
              6308, 5953, 7590, 5952, 7610, 5340, 6834, 8231, 4669, 5935, 
              5936, 5633, 6426, 5631, 4338, 5636, 7163, 7164, 7160, 7167, 
              7166, 5426, 5938, 4940, 5616, 6427}],
             RowBox[{"-", "47.5`"}]],
            Annotation[#, -47.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4939, 5937, 4425, 6429, 6428, 5641, 4941, 5637, 7165, 
              5635, 6440, 6439, 5632, 5959, 5958, 5431, 5963, 6836, 6835, 
              7525, 5276, 7527, 7528, 7526, 8185, 5277, 8184, 7708, 7709, 
              6578, 7530, 7529, 4963, 6732, 4962, 7197, 5795, 4575, 8164, 
              5242, 7487, 5243, 6243, 6242, 4576, 7848, 6734, 7870, 7871, 
              4608, 6767, 4607, 6944, 6945, 6709, 7831, 7832, 6904, 8294, 
              6905, 6901, 6903, 8295, 6902, 8147, 7460, 7461, 7411, 7413, 
              7412, 5174, 6200, 6659, 7416, 5175, 7410, 5169, 8116, 8117, 
              7409, 7408, 5029, 6196, 5027, 6024, 4453, 4359, 6490, 5722, 
              6031, 4707, 6064, 5464, 6069, 6070, 5727, 5055, 6068, 5746, 
              6533, 5748, 5069, 5755, 5068, 6099, 4472, 6535, 6534}],
             RowBox[{"-", "50"}]],
            Annotation[#, -50, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4340, 5638, 6430, 5640, 5940, 5939, 4670, 5960, 5961, 
              5643, 4964, 5957, 4432, 6441, 5432, 4966, 6260, 4965, 5962, 
              6245, 6244, 4349, 5792, 4761, 5955, 4762, 6959, 6960, 7944, 
              7943, 4808, 8282, 7799, 7800, 6656, 5168, 7406, 4766, 6658, 
              7801, 4536, 5796, 4535, 7938, 7939, 6946, 4793, 5800, 5799, 
              4563, 6201, 6202, 5220, 7462, 7464, 7463, 5513, 6203, 6662, 
              6661, 6660, 5306, 7415, 7414, 5170, 6657, 4534, 6028, 6029, 
              5172, 6025, 5171, 6195, 6032, 5728, 6514, 6513, 5721, 5053, 
              5465, 6062, 4464, 6516, 6515, 5752, 6072, 6071, 4714, 6101, 
              6102, 5754, 6100, 5753, 6112}],
             RowBox[{"-", "52.5`"}]],
            Annotation[#, -52.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5074, 5756, 5075, 6113, 5472, 6108, 6109, 5751, 6537, 
              5749, 4367, 4463, 6061, 6063, 5734, 5054, 6066, 4709, 6037, 
              5729, 5032, 6197, 6198, 6035, 6033, 6138, 6191, 6192, 5305, 
              6283, 5304, 6271, 5322, 6286, 7836, 7835, 8226, 8225, 5321, 
              6768, 6769, 6770, 6285, 5373, 5374, 7711, 7710, 8280, 4764, 
              7925, 4765, 6900, 4763, 5801, 5490, 5491, 5824, 5823, 5666, 
              5515, 6958, 5516, 5794, 5793, 5375, 6246, 5244, 5520, 6735, 
              5965, 4967, 7488, 7489, 6442, 6468, 4339, 4431, 5956, 5642, 
              5969, 5645, 5972, 5971, 5433, 5968, 4969, 5639, 6443}],
             RowBox[{"-", "55"}]],
            Annotation[#, -55, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4968, 5967, 4434, 6444, 5434, 4971, 5644, 4970, 5970, 
              5341, 6310, 6469, 6467, 5353, 7611, 5245, 7566, 6736, 6737, 
              5964, 4577, 5518, 7490, 5519, 5836, 5835, 4609, 5667, 4677, 
              5517, 4676, 6925, 6926, 6688, 6690, 6691, 5797, 5798, 4783, 
              6899, 4782, 6924, 6934, 6933, 6935, 5377, 5376, 6284, 4610, 
              7872, 7438, 5200, 8133, 5199, 7556, 7557, 7555, 5311, 6223, 
              5201, 6194, 6193, 5086, 6034, 5087, 7331, 6491, 5733, 6038, 
              6036, 4708, 6065, 4710, 6083, 5736, 6081, 6080, 5757, 6538, 
              6536, 5750, 5072, 5473, 6106, 4474, 6540, 6539}],
             RowBox[{"-", "57.5`"}]],
            Annotation[#, -57.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4376, 4473, 6105, 6107, 5763, 5073, 6111, 4716, 6085, 
              5758, 5058, 5735, 5059, 6082, 5466, 6076, 6077, 5732, 6518, 
              5730, 4360, 4454, 6588, 5198, 7439, 6692, 6693, 5773, 4551, 
              6694, 4787, 6698, 7817, 7818, 5521, 4552, 5822, 6697, 6696, 
              6695, 4786, 5494, 4550, 7466, 7467, 7224, 4994, 6689, 4995, 
              6582, 4442, 5514, 5834, 4809, 5838, 4810, 7492, 7493, 7491, 
              5246, 5999, 4997, 6298, 5328, 7591, 7592, 7225, 4996, 6247, 
              6317, 8236, 5354, 7612, 5355, 7599, 7598, 6837, 5974, 4972, 
              5670, 6445, 4341, 4433, 5966}],
             RowBox[{"-", "60"}]],
            Annotation[#, -60, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4351, 6473, 5669, 6004, 4679, 5973, 4678, 7595, 7596, 
              7593, 5342, 7597, 8186, 7531, 5343, 7600, 5998, 6309, 5279, 
              6265, 6267, 6266, 6001, 4998, 6000, 6740, 4578, 5837, 7850, 
              7849, 6739, 4811, 6966, 6967, 6965, 4812, 5379, 5378, 7465, 
              5221, 6710, 6711, 4564, 5997, 4565, 6928, 6929, 6910, 4784, 
              6927, 7720, 6930, 4785, 6912, 6932, 6931, 4725, 6222, 4724, 
              5772, 5179, 7425, 7420, 7421, 6519, 6517, 5731, 5056, 5467, 
              6074, 4466, 6521, 6520, 5762, 6086, 6084, 4715, 6110, 4717, 
              6124, 5765, 6122, 6121}],
             RowBox[{"-", "62.5`"}]],
            Annotation[#, -62.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5078, 5764, 5079, 6123, 5474, 6117, 6118, 5761, 6542, 
              5759, 4368, 4465, 6073, 6075, 6209, 6208, 5057, 6079, 6666, 
              7422, 5176, 8118, 7423, 5180, 7426, 6672, 6673, 5774, 5775, 
              4774, 7930, 4773, 7929, 6949, 4796, 6911, 4770, 6909, 4771, 
              6948, 4795, 6947, 4794, 5522, 4611, 6968, 4813, 6743, 6744, 
              4579, 5810, 7417, 6963, 6964, 6738, 6741, 6742, 5803, 5804, 
              4999, 6471, 6583, 8242, 4350, 4443, 6264, 8273, 8302, 6261, 
              8304, 6262, 8307, 8308, 8306, 8303, 5278, 6008, 7594, 6007, 
              5441, 6005, 5001, 5668, 6472}],
             RowBox[{"-", "65"}]],
            Annotation[#, -65, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5000, 6003, 4445, 6474, 5442, 5003, 6263, 5002, 6006, 
              5247, 6248, 8241, 6470, 5383, 4767, 5526, 6664, 6665, 6962, 
              6961, 7803, 7802, 6663, 7419, 7418, 4772, 6969, 6670, 6671, 
              5805, 4538, 6669, 4797, 6712, 6713, 6714, 5808, 4566, 5811, 
              7838, 7837, 6715, 4798, 5524, 5523, 6211, 5181, 7553, 5308, 
              7552, 8212, 8211, 7424, 5177, 8120, 8121, 8119, 4537, 6078, 
              6668, 6667, 6091, 5178, 6088, 6087, 5766, 6543, 6541, 5760, 
              5076, 5475, 6115, 4476, 6545, 6544}],
             RowBox[{"-", "67.5`"}]],
            Annotation[#, -67.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4377, 4475, 6114, 6116, 5768, 5077, 6120, 4719, 7293, 
              5767, 5060, 6210, 5061, 8040, 6090, 6140, 6139, 6204, 6205, 
              5307, 6287, 6288, 6273, 6272, 6292, 4567, 7840, 7839, 8293, 
              5323, 8227, 5324, 6291, 6290, 5380, 5381, 7714, 7713, 7926, 
              4769, 7927, 7928, 6908, 6907, 4768, 5809, 5492, 5493, 5828, 
              5827, 5690, 5691, 5528, 5693, 4613, 5802, 5382, 6249, 5248, 
              5531, 6745, 6010, 5004, 7494, 5250, 6475, 4352, 4444, 6002}],
             RowBox[{"-", "70"}]],
            Annotation[#, -70, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4361, 5385, 5249, 5532, 6746, 6747, 6009, 4580, 5529, 
              6970, 5530, 5840, 5839, 4612, 5692, 4693, 5527, 4692, 6937, 
              6938, 6699, 6701, 6702, 5806, 6942, 5807, 4789, 6906, 4788, 
              6936, 6940, 6939, 6941, 5387, 5386, 6289, 4615, 7873, 7441, 
              5203, 8134, 5202, 7561, 7562, 7560, 5314, 6274, 5313, 6207, 
              6206, 5088, 6089, 5089, 8041, 7332, 6855, 7294, 7292, 4718, 
              6119, 4720, 7319, 5770, 6126, 6125}],
             RowBox[{"-", "72.5`"}]],
            Annotation[#, -72.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5080, 5769, 5081, 7318, 8314, 8313, 6854, 8316, 6853, 
              8266, 8309, 8311, 8290, 7558, 8291, 5312, 7559, 6703, 6704, 
              8312, 4554, 7442, 5204, 7440, 5205, 5535, 5534, 4555, 5825, 
              5826, 4791, 6943, 4790, 7936, 4553, 7468, 7469, 7261, 5033, 
              6700, 5034, 6589, 8310, 5525, 7945, 4814, 8315, 4815, 6972, 
              6973, 6971, 4816, 7946, 5694, 5695, 5533, 4614, 6250, 5384}],
             RowBox[{"-", "75"}]],
            Annotation[#, -75, "Tooltip"]& ]}, {}, {}}}],
       AspectRatio->1,
       BaseStyle->{FontFamily -> "Times", FontSize -> 12},
       DisplayFunction:>Identity,
       Frame->True,
       PlotLabel->FormBox[
        "\"\[Mu]=10 \[Alpha]=0.99 \[Epsilon]=10 PIN=0.33\"", TraditionalForm],
       
       PlotRange->{{0, 50}, {0, 50}},
       PlotRangeClipping->True,
       PlotRangePadding->{
         Scaled[0.02], 
         Scaled[0.02]}], {576., -576.}, 
      ImageScaled[{0.5, 0.5}], {360., 360.}]}}, {}},
  ContentSelectable->True,
  PlotRangePadding->{6, 5}]], "Output",
 CellChangeTimes->{3.3896128846252003`*^9, 3.389622669479*^9, 
  3.3899915356096*^9},
 CellLabel->"Out[257]="]
}, Open  ]],

Cell[TextData[{
 "When the characteristics of a distribution are strongly dependent on \
parameter values, different sample distributions will imply different \
parameter values, i.e., the data are likely to informative in estimating the \
parameters with precision.\nPIN, however, is a derived, summary quantity. \
Suppose that we investigate a set of distributions where ",
 Cell[BoxData[
  FormBox[
   StyleBox["PIN",
    FontSlant->"Italic"], TraditionalForm]]],
 StyleBox[" ",
  FontSlant->"Italic"],
 "is held constant at 0.10."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Alpha]Rule", "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Solve", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"PIN", "\[Equal]", 
         FractionBox[
          RowBox[{"\[Alpha]", " ", "\[Mu]"}], 
          RowBox[{
           RowBox[{"\[Alpha]", " ", "\[Mu]"}], "+", 
           RowBox[{"2", " ", "\[Epsilon]"}]}]]}], "/.", "\[InvisibleSpace]", 
        RowBox[{"PIN", "\[Rule]", ".1"}]}], "/.", "\[InvisibleSpace]", 
       RowBox[{"\[Epsilon]", "\[Rule]", "10"}]}], ",", "\[Alpha]"}], "]"}], 
    "]"}]}], ";"}]], "Input",
 CellLabel->"In[258]:="],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"ggg", "[", "\[Mu]_", "]"}], ":=", 
   RowBox[{"gg", "[", 
    RowBox[{"\[Mu]", ",", 
     FractionBox["2.2222222`", "\[Mu]"], ",", "10"}], "]"}]}], ";"}], "\n", 
 RowBox[{"Show", "[", 
  RowBox[{"GraphicsGrid", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"ggg", "[", "10", "]"}], ",", 
       RowBox[{"ggg", "[", "9", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"ggg", "[", "8", "]"}], ",", 
       RowBox[{"ggg", "[", "7", "]"}]}], "}"}]}], "}"}], "]"}], 
  "]"}]}], "Input",
 CellLabel->"In[259]:="],

Cell[BoxData[
 GraphicsBox[{{}, {{InsetBox[
      GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyMfXdAlMfzN4ldiYoKiqAiVhTx6E24veOOg1iDRGOPvfdujBW7xq6xxW5i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        "], {{
          {RGBColor[0.293416, 0.0574044, 0.529412], EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{4393, 4402, 617}, {4394, 4419, 2350}, {5924, 5923, 
              1729}, {8461, 8463, 4099}, {6413, 8013, 2068}, {6414, 6413, 
              2068}, {8462, 4524, 207}, {4882, 5762, 617}, {5045, 6936, 
              777}, {7864, 7905, 3349}, {6873, 6875, 750}, {6779, 7512, 
              2350}, {1322, 2695, 750}, {7142, 7143, 2695}, {4417, 4396, 
              3348}, {4424, 4423, 4099}, {5043, 7171, 2727}, {4400, 4392, 
              194}, {3348, 3349, 194}, {7904, 7902, 194}, {4407, 4408, 
              1205}, {8156, 8155, 3670}, {4398, 4415, 2726}, {4603, 7904, 
              194}, {5044, 5922, 777}, {6936, 5044, 777}, {8337, 5246, 
              3711}, {7512, 6777, 2350}, {7863, 7864, 3349}, {8012, 6878, 
              1205}, {4524, 4611, 207}, {3127, 1322, 1205}, {3230, 2695, 
              1322}, {3127, 3230, 1322}, {4404, 4403, 750}, {3670, 2727, 
              2726}, {7632, 7631, 3127}, {7656, 5471, 1233}, {4410, 4393, 
              617}, {5470, 7656, 1233}, {4413, 4394, 2350}, {6876, 5470, 
              1233}, {4422, 4421, 207}, {4611, 5556, 207}, {4405, 4422, 207}},
              
              VertexColors->None], 
             PolygonBox[{{4395, 4407, 1205, 1322}, {7143, 6873, 750, 2695}, {
              4411, 4410, 617, 1729}, {5762, 5924, 1729, 617}, {8463, 8462, 
              207, 4099}, {4415, 4400, 194, 2726}, {7903, 8156, 3670, 2726}, {
              4408, 4406, 2068, 1205}, {4420, 4397, 1233, 3711}, {4397, 4404, 
              750, 1233}, {4416, 4398, 2726, 2727}, {4403, 4395, 1322, 750}, {
              5471, 8337, 3711, 1233}, {4392, 4417, 3348, 194}, {4402, 4399, 
              777, 617}, {5246, 7862, 3348, 3711}, {7902, 7903, 2726, 194}, {
              8155, 5043, 2727, 3670}, {4421, 4424, 4099, 207}, {7905, 4603, 
              194, 3349}, {6875, 6876, 1233, 750}, {7862, 7863, 3349, 3348}, {
              8520, 7020, 4414, 4412}, {7171, 5045, 777, 2727}, {8013, 8012, 
              1205, 2068}, {6878, 7632, 3127, 1205}, {5922, 4882, 617, 777}, {
              4399, 4416, 2727, 777}, {4396, 4420, 3711, 3348}, {7747, 7142, 
              2695, 3230}, {7631, 7747, 3230, 3127}},
              VertexColors->None], 
             PolygonBox[{{8522, 6779, 2350, 4419, 4418}, {7020, 8461, 4099, 
              4423, 4414}, {6777, 8520, 4412, 4413, 2350}, {5557, 6414, 2068, 
              4406, 4409}, {5556, 5557, 4409, 4405, 207}, {5923, 4768, 4401, 
              4411, 1729}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.3172616279841959, 0.09886550719494477, 0.5629299148091719], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{4688, 5919, 427}, {6414, 5557, 1370}, {8012, 8013, 
              3520}, {8246, 7535, 3024}, {5757, 7009, 1612}, {7142, 7747, 
              429}, {2692, 2435, 429}, {2437, 2435, 427}, {3955, 3669, 
              3398}, {6873, 7143, 2435}, {7482, 7021, 2585}, {8337, 5471, 
              3842}, {5022, 7145, 751}, {7632, 6878, 2439}, {8156, 7903, 
              3398}, {2435, 3519, 427}, {4603, 7905, 3400}, {1614, 2090, 
              520}, {5497, 6441, 2090}, {8462, 8463, 4100}, {7863, 7862, 
              972}, {7146, 5017, 2692}, {3398, 3669, 776}, {5303, 5992, 
              1011}, {5020, 7144, 751}, {5043, 8155, 776}, {7009, 4765, 
              1612}, {7145, 5020, 751}, {4915, 7482, 2585}, {7747, 7631, 
              429}, {2439, 2527, 429}, {5042, 5758, 776}, {3399, 4255, 
              1612}, {3024, 4100, 2585}, {4689, 7652, 428}, {3669, 3955, 
              1612}, {3842, 3150, 972}, {6032, 4688, 427}, {3151, 2437, 
              427}, {2527, 751, 429}, {3150, 3151, 427}, {7862, 5246, 972}, {
              2526, 2439, 428}, {6352, 4689, 428}, {7903, 7902, 3398}, {7761, 
              7762, 3246}, {3245, 3246, 1370}, {3399, 3955, 3398}, {8010, 
              8011, 3519}, {4770, 7147, 2527}, {2439, 3520, 428}, {2487, 2729,
               776}, {8461, 7020, 2585}, {7513, 7511, 3004}, {2954, 2279, 
              972}, {4882, 5922, 1728}, {5557, 5556, 1370}, {5924, 5762, 
              1614}, {4765, 6054, 1612}, {5760, 5761, 1614}, {5045, 7171, 
              2729}, {6935, 4719, 2487}, {6779, 8522, 1011}, {4255, 3399, 
              2279}, {3955, 3399, 1612}, {4768, 5923, 520}, {7443, 7442, 
              2954}, {6876, 6875, 2437}, {3399, 3400, 2279}, {7533, 8246, 
              3024}, {6968, 4769, 2526}, {7653, 6968, 2526}, {7144, 5021, 
              751}, {6691, 8474, 2279}, {751, 2692, 429}},
              VertexColors->None], 
             PolygonBox[{{5922, 5044, 2728, 1728}, {5044, 6936, 2487, 2728}, {
              7902, 7904, 3399, 3398}, {5246, 8337, 3842, 972}, {8155, 8156, 
              3398, 776}, {8011, 6032, 427, 3519}, {4524, 8462, 4100, 3024}, {
              4719, 7170, 2728, 2487}, {5762, 4882, 1728, 1614}, {5470, 6876, 
              2437, 3151}, {6778, 8521, 8520, 6777}, {7904, 4603, 3400, 
              3399}, {5919, 7655, 3150, 427}, {7655, 5920, 972, 3150}, {7652, 
              7653, 2526, 428}, {7656, 5470, 3151, 3150}, {5921, 5760, 1614, 
              1728}, {7171, 5043, 776, 2729}, {6936, 5045, 2729, 2487}, {5923,
               5924, 1614, 520}, {7905, 7864, 2279, 3400}, {7170, 5921, 1728, 
              2728}, {5920, 7443, 2954, 972}, {8474, 8498, 4255, 2279}, {7147,
               5022, 751, 2527}, {5556, 4611, 3245, 1370}, {8013, 6413, 428, 
              3520}, {7864, 7863, 972, 2279}, {6877, 4770, 2527, 2439}, {6441,
               5622, 520, 2090}, {5992, 7513, 3004, 1011}, {5471, 7656, 3150, 
              3842}, {8463, 8461, 2585, 4100}, {7143, 7142, 429, 2435}, {7442,
               6691, 2279, 2954}, {4769, 6877, 2439, 2526}, {5017, 6874, 2435,
               2692}, {7631, 7632, 2439, 429}, {6875, 6873, 2435, 2437}, {
              6878, 8012, 3520, 2439}, {5761, 5497, 2090, 1614}, {7512, 6779, 
              1011, 3004}, {6353, 6352, 428, 1370}, {8154, 5042, 776, 3669}, {
              6874, 8010, 3519, 2435}, {6054, 8154, 3669, 1612}, {4611, 4524, 
              3024, 3245}, {6413, 6414, 1370, 428}, {7762, 6353, 1370, 
              3246}, {5758, 6935, 2487, 776}, {8498, 5757, 1612, 4255}, {7535,
               7760, 3245, 3024}, {5021, 7146, 2692, 751}, {7021, 7533, 3024, 
              2585}, {7760, 7761, 3246, 3245}},
              VertexColors->None], 
             PolygonBox[{{8521, 4915, 2585, 7020, 8520}, {7511, 6778, 6777, 
              7512, 3004}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.3439521008406231, 0.14527303132966346`, 0.6004466032357112], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{6313, 4914, 638}, {5497, 5761, 1269}, {3187, 1502, 
              1269}, {5019, 7113, 749}, {2696, 1793, 748}, {5989, 6310, 
              1763}, {2696, 3658, 193}, {7532, 5447, 1204}, {6213, 5498, 
              1269}, {4767, 5759, 1613}, {7481, 5280, 2983}, {8498, 8474, 
              4127}, {6933, 6934, 2486}, {5042, 8154, 1812}, {1615, 763, 
              521}, {5763, 8033, 1615}, {3658, 2697, 193}, {1613, 2486, 
              471}, {4602, 6033, 193}, {8032, 6893, 763}, {6877, 4769, 521}, {
              1793, 1804, 748}, {1231, 1615, 521}, {7146, 5021, 2698}, {5447, 
              7606, 1204}, {7761, 7760, 1060}, {3544, 1615, 1231}, {3148, 
              2438, 1231}, {2697, 763, 193}, {2438, 2019, 1204}, {5018, 7007, 
              749}, {8244, 8245, 3743}, {8034, 4700, 3544}, {2575, 4127, 
              616}, {5757, 8498, 4127}, {7653, 7652, 3148}, {7760, 7535, 
              1060}, {1727, 749, 616}, {7699, 7698, 3187}, {5622, 6441, 
              1502}, {4915, 8521, 638}, {5760, 5921, 471}, {7606, 5448, 
              1204}, {8010, 6874, 748}, {1204, 2019, 1060}, {2436, 1795, 
              749}, {1804, 1795, 748}, {1231, 2438, 1204}, {2697, 2699, 
              521}, {1727, 2436, 749}, {6352, 6353, 2019}, {5498, 6659, 
              1269}, {6353, 7762, 2019}, {6030, 4994, 1793}, {4510, 4602, 
              193}, {4881, 6843, 616}, {5921, 7170, 471}, {4770, 6877, 521}, {
              6310, 5990, 1763}, {8154, 6054, 1812}, {6893, 5029, 763}, {5029,
               5619, 763}, {6034, 6030, 1793}, {7513, 5992, 1763}, {5920, 
              7655, 1727}, {6848, 5332, 1060}, {6032, 8011, 1795}, {2698, 
              2696, 748}, {763, 2697, 521}, {7170, 4719, 471}, {7010, 7008, 
              2575}, {7655, 5919, 1727}, {6935, 5758, 1613}, {5022, 7147, 
              2699}, {5990, 6314, 1763}, {7443, 5920, 1727}, {6778, 7511, 
              638}, {1793, 2696, 193}, {7113, 5018, 749}, {8521, 6778, 638}, {
              4996, 7141, 1795}, {6874, 5017, 748}, {6691, 7442, 616}},
              VertexColors->None], 
             PolygonBox[{{6934, 6212, 471, 2486}, {6441, 5497, 1269, 1502}, {
              5759, 6933, 2486, 1613}, {5919, 4688, 2436, 1727}, {4914, 7481, 
              2983, 638}, {6319, 5989, 1763, 1989}, {7141, 5019, 749, 1795}, {
              6659, 7699, 3187, 1269}, {6212, 6213, 1269, 471}, {7511, 7513, 
              1763, 638}, {6968, 7653, 3148, 1231}, {7652, 4689, 2438, 
              3148}, {4769, 6968, 1231, 521}, {7022, 7534, 7533, 7021}, {7007,
               4881, 616, 749}, {4700, 5763, 1615, 3544}, {4689, 6352, 2019, 
              2438}, {7762, 7761, 1060, 2019}, {5992, 5303, 1989, 1763}, {
              5761, 5760, 471, 1269}, {8011, 8010, 748, 1795}, {5020, 7145, 
              2697, 3658}, {6314, 6313, 638, 1763}, {7630, 8034, 3544, 
              1231}, {8033, 8032, 763, 1615}, {6843, 7010, 2575, 616}, {7009, 
              5757, 4127, 2575}, {6045, 4996, 1795, 1804}, {5021, 7144, 2696, 
              2698}, {7698, 4512, 1502, 3187}, {4688, 6032, 1795, 2436}, {
              7535, 8246, 3743, 1060}, {8474, 6691, 616, 4127}, {7144, 5020, 
              3658, 2696}, {5332, 7532, 1204, 1060}, {7147, 4770, 521, 
              2699}, {7145, 5022, 2699, 2697}, {8245, 6848, 1060, 3743}, {
              7442, 7443, 1727, 616}, {5448, 7630, 1231, 1204}, {4719, 6935, 
              1613, 471}, {5619, 4510, 193, 763}, {6053, 4767, 1613, 1812}, {
              4994, 6045, 1804, 1793}, {6033, 6034, 1793, 193}, {5017, 7146, 
              2698, 748}, {7482, 4915, 638, 2983}, {5758, 5042, 1812, 1613}},
              VertexColors->None], 
             PolygonBox[{{7008, 4766, 4765, 7009, 2575}, {4766, 6053, 1812, 
              6054, 4765}, {7534, 8244, 3743, 8246, 7533}, {5280, 7022, 7021, 
              7482, 2983}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.37064257369705034`, 0.19168055546438215`, 0.6379632916622505], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{6034, 6033, 1796}, {5434, 7610, 3104}, {1059, 1500, 
              518}, {3125, 3023, 405}, {5989, 6319, 219}, {3126, 3102, 450}, {
              6313, 6314, 1986}, {3102, 3104, 450}, {3098, 2409, 994}, {4992, 
              7108, 2663}, {4997, 6850, 725}, {3023, 2409, 405}, {3102, 3125, 
              405}, {1500, 2714, 518}, {2406, 2663, 519}, {4996, 6045, 724}, {
              3104, 3545, 450}, {4881, 7007, 2574}, {2406, 2574, 404}, {6309, 
              5299, 1984}, {7601, 5431, 3098}, {5496, 6209, 1268}, {7112, 
              7115, 2666}, {7531, 7529, 3022}, {6440, 6439, 2089}, {6212, 
              6934, 1929}, {4767, 6053, 519}, {6851, 6031, 725}, {6045, 4994, 
              724}, {4679, 6279, 405}, {2668, 2666, 724}, {1796, 1059, 725}, {
              7532, 5332, 3023}, {7116, 4998, 2668}, {5986, 6290, 1762}, {
              5621, 5496, 1268}, {4537, 4618, 219}, {6657, 6658, 2260}, {7630,
               5448, 3126}, {4510, 5619, 1500}, {1796, 2348, 1059}, {2693, 
              2666, 404}, {4591, 6203, 180}, {7481, 4914, 1762}, {3545, 2453, 
              450}, {6933, 5759, 1268}, {4763, 6853, 518}, {7167, 4677, 
              404}, {7534, 7022, 994}, {6853, 4764, 518}, {6844, 6842, 
              2406}, {7114, 4676, 404}, {7022, 5280, 994}, {8032, 8033, 
              3543}, {6213, 6212, 1929}, {2453, 3543, 450}, {8244, 7534, 
              994}, {4678, 8315, 405}, {2453, 3149, 518}, {6736, 5986, 
              1762}, {4512, 7698, 180}, {5763, 4700, 450}, {2089, 1929, 
              1268}, {4766, 7008, 519}, {8316, 7607, 3102}, {2574, 2693, 
              404}, {6848, 8245, 2409}, {5019, 7141, 2694}, {6279, 4678, 
              405}, {2666, 2694, 724}, {7699, 6659, 2260}, {5988, 6736, 
              1762}, {2714, 2453, 518}, {8034, 7630, 3126}, {5469, 7654, 
              3149}, {4676, 7167, 404}, {7529, 7530, 3022}, {6849, 7996, 
              2409}, {6053, 4766, 519}, {2348, 1500, 1059}, {7010, 6843, 
              2406}, {6850, 6851, 725}},
              VertexColors->None], 
             PolygonBox[{{6033, 4602, 2348, 1796}, {6842, 4992, 2663, 2406}, {
              7108, 5620, 519, 2663}, {4677, 6844, 2406, 404}, {6893, 8032, 
              3543, 2453}, {7008, 7010, 2406, 519}, {8245, 8244, 994, 2409}, {
              8033, 5763, 450, 3543}, {7610, 8035, 3545, 3104}, {6658, 4591, 
              180, 2260}, {7700, 6657, 2260, 3188}, {7530, 7109, 1059, 
              3022}, {7007, 5018, 2693, 2574}, {7115, 7114, 404, 2666}, {4618,
               6309, 1984, 219}, {7113, 5019, 2694, 2666}, {5619, 5029, 2714, 
              1500}, {7141, 4996, 724, 2694}, {4764, 6967, 1059, 518}, {5280, 
              7481, 1762, 994}, {6843, 4881, 2574, 2406}, {6934, 6933, 1268, 
              1929}, {5447, 7532, 3023, 3125}, {5431, 6849, 2409, 3098}, {
              6290, 5279, 994, 1762}, {5332, 6848, 2409, 3023}, {4995, 7116, 
              2668, 724}, {5620, 5621, 1268, 519}, {5498, 6213, 1929, 3188}, {
              5301, 5988, 1762, 1986}, {5018, 7113, 2666, 2693}, {5029, 6893, 
              2453, 2714}, {6031, 4995, 724, 725}, {5448, 7606, 3102, 3126}, {
              4998, 7112, 2666, 2668}, {5759, 4767, 519, 1268}, {7109, 4997, 
              725, 1059}, {6030, 6034, 1796, 725}, {4700, 8034, 3126, 450}, {
              6967, 7531, 3022, 1059}, {4994, 6030, 725, 724}, {6659, 5498, 
              3188, 2260}, {7607, 5434, 3104, 3102}, {7654, 4763, 518, 
              3149}, {7606, 5447, 3125, 3102}, {5218, 7700, 3188, 1929}, {
              6892, 5469, 3149, 2453}, {7698, 7699, 2260, 180}, {6209, 6440, 
              2089, 1268}, {4914, 6313, 1986, 1762}, {5279, 7601, 3098, 
              994}, {8315, 8316, 3102, 405}, {4602, 4510, 1500, 2348}, {8035, 
              6892, 2453, 3545}, {7996, 4679, 405, 2409}, {6310, 5989, 219, 
              1984}, {6439, 5218, 1929, 2089}},
              VertexColors->None], 
             PolygonBox[{{5991, 5301, 1986, 6314, 5990}, {5299, 5991, 5990, 
              6310, 1984}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.39733304655347756`, 0.23808807959910086`, 0.6754799800887898], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVF2ITVEUXvdi7jjNGbk/c889x5kz8sCjpLmZNEjy8yI/b/Iy1DC3GQ8e
zQP5S5TiDg+UmLklTEMMYxJhZojBlCLlvipRUuSFxvf17YfdXmfttdf6vm+t
fRZ3HdjelzazPVhzsL6HsBOzmblmoxkzv9FsEPZAwaxcMmvH9zLs6wKzkzhP
wU6XFHc8ZzYZme3zzI7hvL2guy/hO4Gzjyh0fr5ZD9bTlFnF2Rdgv0HMC9Tt
xd10Rmf0X8X9iqe7rxDzOtJ3H84/Y11EzCxqbc7KJiZivA8yX5uBBTE15NjW
YnYdcUsaxI0+xh8GrnFwfo/8HcD7INZZBXsZteq4fy9vtqto9hM5u2AvLSk/
uez3xIF4qo5XN/ZBTzmnyCtS3G/k/4VVn2fW2Wq21mk4ipzdLv8O4Bx2ODeB
059IOtTg87LyU9dTgbTNtwoT8VC3IafVzmbZvMt85M+71Lfq8LLm3bzq7i0K
B+074H4lr/4PwF/FasjIdy6Qfxq4juZUcz18y5HfR/5H0HJDVhpshT220KwR
dgdiziDPLGIOYt/i+nUJ+5dYvaw7XNSQsadd/AdfWtNPTjXHsRM5V7i6T1Dr
cah+rwmkL7WdwkxNJuLMPnE2mYe69rdIn90FzT3nYgScbJH697fN7B/WuwW4
g/Oym2dyZW3WPYL4VYF6PZGo3+z1GDg9xEqaxG3GzVsqVM/YL2LlOeftGu6M
+Jp3YiVn4qT2N2PpP+TeBjFT+7NFfT9D3eeJNFoJjDdi5SHvb6HyUUfOHXG+
jfTeep2WzE09OZsTkeL6wftQm94Q38HlrPrD990TS0e+OfaSvIpN4sC6wzi/
FctHHtSP2D4Bz+2cZpP/mPFYuvzAjKRD6bkROVczzhcu5iM2alNx+f8Dcl60
7w==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtU12ITWEUXedi3DPNIeNe9zjX7Y4U8yyZzCgjP8mL3zcJkzIzN+NVmQfK
kDxIDA+UGLfkb2byOxNqZq4hhFKkzKuSB3nwRtdarfOw6+xv7732Wnvvs7Tr
yM6+DIBO2mza9gQ4WAB+zwKyIfAgB1RjYFmD3/aVgb8twIf5wOcIeEV/mnaR
uYcbgbY8MFoCHs0FhgjY2AyMLQDC0BiTxB9YCHQzt5dvU4nf9F2h/SP2JPEu
hO5xllyusX5mDlBn/QhrgyXATdbvnsf+RaCW1vfwrUb/OHM+ZVyj2BZyuBQ4
p5fcemgTgTl85/decv5Irt8Y76DWX+wXsdcANayh3x67tzDHifeMdii0v2Ix
0Eo7zdzHnM8x8t/Mft2M/yC/W6x9mPPspH+U3AapqYH5Vfa/T/95ineD/lvy
f037QuwK/U3EqhcdU85V+i+Z38fvmdCxjXwbjpyzg7Fdi9xLMz/P/gH5RVnv
5An19reYW5Wat7J2OsW7zJm0MfdAzlpWs+Zn4p1r19qxdrFcmrOe723i3aMV
moDrxMswdjL27tVT2oapcSjlD+7uRLofvb1g/Cnry02e2d2SZyItwpS2N0XP
Qhql/UqzdyXOugXNVLPUTWQYDxLz1w47yWUwby3a0VF+/8mbuzRI+/uib1d4
d9h/Vd5atKNx+u15z0o3Auqrx56V3lZy1vtzvhX1021VSt6NbmyKs6uVfZu6
6XMF37xuPcOaU+T+ruhv7fwMsTuYPxJ5hv3E/1r2reuGx0rmoN6a2R7ibUv8
r+rGtJt1sbVK3/rYmltT/WuJNZEYWz206w2xuWtm/wF0JZeK
              "],
              VertexColors->None], 
             PolygonBox[{{7995, 6280, 6279, 4679, 3504}, {6280, 6847, 2408, 
              4678, 6279}, {5987, 6289, 1973, 6290, 5986}, {4536, 5987, 5986, 
              6736, 2312}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.4240235194099048, 0.28449560373381955`, 0.7129966685153292], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVEtIlFEUPo46f+r4mnGe6swUJUGrIinTaRQ3UYvotTJRa5u6aBE9NpGE
0qboYWokRItI80Fj2KKiGnTnppDKjQQF0Qts1Yv6Pr67uNzz3/+c73znnO/e
9cf6D/b5zGwIqxArWGvWVWOWw8d0vdkMVjxgNutpv1Nkdjpt1hiW/RD/m2Df
hT0dMnuSMPMj9ncEYLDnYDfEzcar4Yuj7fCdQEwEWHn8X0S+EyVmx6Nm8/BZ
gk9rhVkb1hsQOwScA/BbK9R5FusXzvsR88Uvn0ZgbIBPMewW/N8CLsuws0lw
AtebAf2vTMrHwOcxOATWmW2CfQqcfiBmCvwn68V/Alx8QccHcSPw2VwMrLC+
aXci7yvgB0rVL56xxnSlcg5jjbmd59x53gC/4bD48ft7nXAZfw17S1I+xB30
ZO8Jql/sVaZCdbLG9phZT438FvB/b1C9eZsyO4fe7fOLAzkxfyYp3sSugl0D
ew7+u4DThDUAnAfowX2sMPxPYr+K2RTjfBD/m1PSAvehmOz9yNsBn1XkWqoV
D3LIw+cl1m5wnQNOc0gzu4y4o86/DRxuhMUxB5+dTktT8JnEygN/HD2+jVVe
KayMq5096K4ye2/SBW1q40pMeuLseNbjzhlHm3HPEqqH/AvQ/xlw8wqliY8V
4kAd0Zf47PGOuPq8gtrWyqUl5m11Wr0AvFH4TcPnLPufkt91cJl1+GOwy4LC
JLesw+eciEvMF4j7mxZfzuYDMJYLgEPtxaSHbuTZFtH9WoH9KSG+vGeXotIb
e8n7yVoWgLmY0r3iXKl36jwHfPNk//RJX+ztAPL44qqFvMmDHDi/3hJh8m04
k9a78Ad7V0p95v45IZt3qzqpfNQWc/GucA7dbhYjpcrL/JwxOdF/HvhPE3pn
GNdbptgO9K8TOV6jL49Qe09U7wNjGMtaqFf+Jyb5cq7kTN21J6S9jX71n7Pr
A15/Sr3mrKgJ1s2eEos9+RfT20Gs8/CxOvFiLRn3LnmecIl5GPM5EtF3C2Iz
WBc9nd2K6pxzZ/3kyR4zJzVAjYxEpY0caryH2OeeuNCXfL4i/7c69YVzYW7m
zcb05lIDfJ+KQrrvfJv5xrE/W0O6h6tOd+RBXPaQ+uCs+R6NubdrNCqdsR+8
H+wB50LdvXPn7CVrImfWQUzWUhDXneR9JHbE4f8HVHXe7A==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtI1GEUxa/j6KTNjKnzdJxmAougVZGGlaa1sVoE5U4iap26cNPDTSRR
VKt85IOCCILKRmkE2xSV6SpXIT02ERRBL7BF9KJ+h+viwv9+937nnnvu/f7r
jvUe7AmYWQcWxPbVmPUlzT6Xm/VWmG1Lmw3FzTaUmZ0PmZ3ImxWzZjdJ/hA1
Gya3M2HWQH6I+I5aP9O3MJpTZmc4M/wBrHWt2S7O7nM0Ejbrod7lpMeL1LN6
swL55aV+dptaW6mfIDeOXcv7HeVGqsxO4Z/EbsAnTXw+YzaXow7clmisH24/
OButNNsPn4kY+FnPnQa/GeyZrHNVT3f4HoTPUonX/AqfGu4X4dO92mxPHTnV
ZovQe8udKWqPgHGV3Dx8nlL7CdYSdQ5NxAbot0Ct6CqzT9x/RryN2C/43SK2
udaxHpMzQe1x7J25pjvRqwULV/oMStPew/QK3iVyt6dcW3HcQr+t+OfwZ/CP
Er+HP4e/jD8L9wN1Xk89BNDy7Ao/+W0pn5Fmo3qj8G/HH8ffiB+E6+6U74I4
dYFfjDl3Yd5Fv8msz0ozHCU+hm0Kek8fqd2Enm+Y30/6v5j0HsRdO3EErG/1
XktnDfS7Hvse9JlUwWUNNsP9Mu4/B2sx47v6Ev8h+I/qXB/txBR8X+V89iHw
XoPfB7+ykHPSrIeSHhN+gFoladdCmmhW81h71GemXf1S77ug/RyL+Uw0C+UP
x30ntAvSrBqu1/O+q+ohBtce8N5HvIcr4HUxgxcR5/SX3D953w3t0IWU75Dw
1ZO0O8395YhruJBzDdS7OB7OucbSVph7wf4HRpj73eQXwB5cec/a2UlihxK+
G9qRfu43Zlxb1XiAVrNZ11IYjXH/B+jt60xvq4Maxyv8jbWA9zvhu6cd1L9E
HFRbb2wh429Ub1N3enM+Y2kvTf4Du7elSg==
              "],
              VertexColors->None], 
             PolygonBox[{{5714, 6311, 1985, 6312, 5713}, {5681, 6204, 1926, 
              6206, 5680}, {6210, 5681, 5680, 6211, 1928}, {6316, 5714, 5713, 
              6317, 1988}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.450713992266332, 0.3309031278685382, 0.7505133569418685], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVEtIlGEUvVlpM/PrqDPT/48z/4xDhdCmB/QiW/SQiqhAeiwypdYx9oSy
WpTQLsoSQzMLCiI0Da19u4I2ZUyWtOhF9o7a9Vh0DueDFh/zzf/dx7nnnnsL
e9ubi2VmdglnOk5txuxb1mw0YtYYmC2abdYKg2kFszKcUtwsWmG2AO/zZ5gd
SZlV5czaPfjVmB2ugy/ivEOctzidUbNhvL/Eqaw0uwKbXzj3YbN9ltlpxBqa
aVaN9zjOfsTZkDebQJxPyDsMDHt8s9flZg2w/4mcP3AuIP+LmNlH2LTifSCQ
Df3fO7tB5FmLOL/x/yreFybMTsL+HO4DSbO7yB0Dvs2oaRAYHnvCw/stnFO4
N8wU3gehMLPmWIWw0ZY+xL8NPPUAR2eZcJEjYiN/awJx+Bf3r2nELxe/5GwM
pB9B3bsywtIPXNNh8wrf2xDvmqur1xMm4tkNm2WB8qfREx8nrEaMWrOL8Lk/
zWweYsxNizNyVap03wPhIIaDnt7JZwb3xZ7wX0JP79QrVxExB3AvwTf0ZEeb
fk+YaEPbnpTu1AX5IU/sA3tOPRB7r6u3CbaPQuXL4gSeuJ0Ab6ej8mX/WBOx
dcN+BDny6NPWuDhnjyaR6zzOk1mqj9rsikhzjMU4PZ782KMIYkTrZbPOlxaZ
96fDSpwbwc1ZcBSWK0+P6+9NfGueLf2sQr8ac3prRS92pFVXl8NCnsl9e178
UkfUOuPsxPenKfGxFD3/UKcaums0P8RAnvocV5ynoKDv1NfKhPgnr8TMfNTC
8Zz0cAB+iQr16zLs+331YwViXMd9CpxeBQc1Odm+BoY3mf+885e6/YNaVwXS
yXrczzhODsBv3OGnFj5UicND6OckuHgO3o8i/pKUensP31ekpB3ysy8mzKyv
GFONnN02N+O3Ud8NXzUwdtZpg79TVdLKQ8QcDZWfHEw63o/nxSn5JA7qnr04
6GuncGaZk7PHvNQXe09u56SVm3mLsE1m9FYC/8/qxMlYRL7k9jt2ZFtS8S7j
t8nXzBLHeZd3kycM9CX3ywvivw/3Xl87IgauPmfFC7W9JS5dcU/8dnNKbXe5
mMQ85eZiHL4tSemrA30ppTQXjPfFxVyCOk7kxVGn6zF5+Ir3s75qoa6HAml7
NeL0pjTf1P9Wh4da4b5g3pFQe4r1bnaYmZd7q899P+X2OnfCMeDsqFf/trgZ
pv1bxGlJiSfuyBkJ7cm7oTTEnnMvtrjdmHD6JgbOze68uOY8UZfUJGvizmJd
xEY/xuxE/LJaaTjq9hP7wlmnvvmd8cdcfxclNPPk5B/U0wip
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlDlMFFAQhodLjl2OZXfZ5dhlVyWWHgkJKMQjUhijhQEtBIzWBhSDRtBC
SGzFIyhHxMLGQMAAWpNYaGIlBFG0EDEixCNaauM3+SkmmffevJl/5v/fS5/r
PNGRaWbNWDb2IWI2HDMryDV7HzC7GTcbYe9Tlllii9lMwqwuavatyKwyaFaL
fzVltsa6ivVD/DHsIv5ihtkCflul2fFis2OYVZj9KzMbJ9dCntn+pNl9cuzI
MZvAhvAPsDfE/VGshdhmrB+QG1h9VBi8dgKrS5tdZu834NdKzO6C/RLn78A+
R/1Bzg4mlctrXGFdU45P7S729oXNniWUy2tMpYTJsVSD9wy9l4J/Ot+sg5xv
wb9K/FHi14nPDqtH762ddYTYEu5f4PwPmEKsu6nZGVSOJvBNcj8e1EzbwNJG
jT7m3QKm71XEU+MN/vmAYg9zZ850pxE+GuKq5TNcrxBnztUd8k8T30S9PUHN
qBcsPdh4jmpsJb4WTMvEfqVGcVJ77jvmXfRzi/yzxF6nRmtEGB3bjVxx84D8
w0FxlFlK7pDZ3zzp4y9cDYbEreN7Cp6phHp1/K6FRnIMBqWJce+F/laIfUH+
AmYZSKn322AcjUkDzn2A8wD1RmLyd3K+Sr4u8i8R318gbL+qxJVjjKGP+rS0
kcDCleLEuZjBluj9S6Xueo0N1u3VaLdQ83Stzyakbdd8J2cd1eLWNbQcEQce
65rrSaln79XfRCu5t2/yFaDGPPnmU5rlEzh5TezesLTnnL1k/WpT386597ZC
jr4C9XiW3p9HNCuf2WfOFiukRY85hH+kbJML8A0T2x/X2jnw3jKYx2KxevS3
sw1+w7l6Q669SfgpLJQGW6Oaiffnms/kbn5K3HiOOOvytGY7Dd7HMc3YZ+s5
J6g9GRdW11MRudvBNJOlNzEUE6fOpf85P+HuR5X+DufoJNhOlWtWfuca+Hux
gXxpeBF83UnN1jXegH8vqrfrGntE7bG4ajsGn3VXTFz7zJ2bmri06hx99Psh
9eIzCCX1Z/hf4X/aALG7w8rnb6QNLKdL9dc4pv/yLb/o
              "],
              VertexColors->None], 
             PolygonBox[{{5981, 6722, 2298, 6723, 5980}, {5278, 5981, 5980, 
              6288, 1972}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.47740446512275925`, 0.37731065200325686`, 0.7880300453684078], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVE1IlFEUvWo68+XMZKPjjDPO6FSG0cZoUy3Dn4xQaFERo0S1ywosY7SQ
NKhFf5soKqwWBWUhaYs2U1AE/ZtFmkYk/YFpi9pFi+oczls8vvt93333nnvO
eS+9fe/GPflmdhKrAOt0zKyu1Gy6yCzsM/MtNCvEj/tBsz7PbDDPbDxu9jlh
dgjvzSGzppD++ZHrYdUi/oT/nUnlH8Z7v9u7L2JWnDKr8ZvtCZgdQY+f88xK
kX81qngOvUfwvJGn3EUVyme9LwnVmUa8EvGkJ4zsvQ772qvMZoFvFPtTJcLD
OTyXw7gOfZd76tGJnmtK1bMB2EaS+v4aOe+wHhRqngDmHwOGEvTcHTYb9tT/
fFgYOtCrO6S+r92+QayekL4PF+jbeEDfp9Erm9L7vaD6c96vqPkuLs4WJsQL
OSmuNptfrXrstQsri/hFQNyMoH4L9m2JSrse1D6Y0gwZ/J9wmBjzyfxucDUZ
lzajiM+VSs9u14P1/6LnP6yxBWavUKPTJ/yn4JMTWHfwXo0aaaxvqD1TIt7J
8y3UCyXkH/JFvTlLryf85GpXsXglPwfAyaq02VK/9FkRkBbM7XD5xMT363mq
Qw8ybivDqtBcfe47fVIDjEtiwk2dpjxxSz9zJuZ2UYeIdDsG/lrCqjmKfa1h
6UJPZSLCwFk4B+fKloO3sLw36Wozn17zOb8NoM4Q8OUQNyN/LWp9KlAOvcW9
+4FhX0oc02vLPPEyBD82ANN4kebudfMS18ekelH/jPPAHDj4gfXQr/2sQ26/
IHcirv1PET9Lit8ZYEgD3zWc/a1ReYj+WQ0domnpOQRcXXHVakPOpTLhr48K
H7FtRs9NWK1+cco7gRosrpCu1HTK3QPkiH4cd568y3sD87w39flRqV5PUPtx
UufVgwfz0/Jmv9OP+rIf75DjhfL+8ZjwtyO+GVNN4sgFxVs7sIdS2jOLPt8r
5cFG+L+pSnnU/3e58omV55M8UzfOz9krgCWWltepV73T7DJ6XsG65xMHrMf+
O1O6+6jVG8zyBvEG+O8MZv+Q0v4lOC+NTuu8tO4y+u0scv6USyN67W9CnuF+
nn/qPoC51jts7MHv5IR8Zd390xXRnULv8s57ntS9w7uRc9CL1ImepbdZ46Wr
wzOUSegcbQfeHVg5/HuLmq8qdSeQ05zLp9a/gtKq1nmZmHlGM2XCfAGz9FQp
bxvqPYpoL3tSV8YXkXsUvW+j9n9BYvP4
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwllEtIFFAUho+a6eTMZOo44/ia6UXRxiDIloWmRhC4KCiTsHaRkSWoRfQg
Wki1CaJCW1RQUqG11aAIevnK8lGEYhZUWtSugqzv8C8O3HvPOf/5z+smGw7W
NqaaWS2yADlUYvY6YjYQNOtHjsTNvi8xC4bMVgXMRtG1YTOO7hVyv9jsGVLm
ZwCOolsfM3uULpvKqNnVPLPuDHwXmk3i34rNGLq6TLPfYHcgnzivAb8T33p8
3sGlF5+hIjhgP4R9F5ib0SXisne8LdzP4/MA2+k0s0AOeDH5OsbSAsX0WCsy
FXuK+2hQHLajr4ffaey3of9CvL+8DXPen2UWwr48Jn0j9nPoPsP3JtiPsTmM
fhP+TRni6LYNvPUG5VOWa3aNtz7OU/C9Dt8NSTCyzXJ4i3IuQEqy9ZYkt5a4
auE57gVrD9KeLsxzYNXkK1fPuZPYF2LC9rc67iMJs62L6Qf3P9h+KxBX5/y0
WDV3fO9ZLKkee2+dQx21mcHmOPdB+nkG7OW5qrX3cL7QbCM+GWBU8/YL/Ezi
tYTNbqXo/Ju3njS99ZfSM6SX+akKa5YuIhUhzdQMeB+RE8Trwv8J8feVqFee
s8e6Q/z0NMV8AbehmGZtbVC1el6ss+c0zr2aeH0hYbaA9Sai2fGep4KXklRu
rfB5j/4esjogTv/gP4D/Se7D1PAy87QOfm+5TyC7ODeXaPYd8wP36UJxP4a+
Bt/5hHwfgreIcyChWG1IGNtspCegnKqwPxtV7bxGd8l1Gf1amSn9LPOYlZCv
13SSXKeKxeU2Pjei8nHbH+Q0ERcHj30qoN2qiKp/Pk++m7vzNBu+o3WFyslz
8Rn23d9RoF3wP8B3oSVfvfad6MZ3Z1Tz5jN9KVc99t7WhNX7r0XaHed7IEc7
5LvjOb+Ee2VE/ZsFYwy+o3HVzmduJKI/yP8en+Ew5+aIcvN98t42ReXrPfbe
XCHGz5B6VJ7UjvlueQ3bsG8tVS3cxns7WCqu3mOfpcGYaumc5opUc+fuM9GO
riNPu+U5/wfEvM7v
              "],
              VertexColors->None], 
             PolygonBox[{{5186, 5885, 5884, 6181, 1906}, {5294, 5984, 5983, 
              6302, 1980}, {6284, 5704, 5703, 6285, 1971}, {5704, 6277, 1968, 
              6278, 5703}, {5984, 5296, 1981, 6304, 5983}, {5885, 6635, 2242, 
              6637, 5884}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5040949379791865, 0.4237181761379756, 0.8255467337949471], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVE1LlGEUvc3YjGPO+DHOjDPjOKMG/QEXLdXFOAlBFAhFgWiLFllBLlSy
SIqKok2L7MMiCgKJUoM2ERS0sZwxVKKVRgVqFtoqk9TO4TzQ4uW97/Pcj3PP
Pfet6zy5/4THzAbxePH0RMzuZMxWisyyIbPOcrMxnK8EZS/D+WK1WS4KX9iz
8O+tNSuUmv2CHYDt9ci3y/mXlpkdL1XOliqz0/C5j2IBv9lfn9kCfF7h/qXz
eY7nC+7q8d6NWkH4z24z68N7DjVebzebj+ibNnOzxnSx2TD8A5WIRd5GnM8A
92SRsORCwjOZNrsZVl3enQuoJu/ZM8+zafnRXvfpnDywRiioupvgaU9aOb+C
j6W42T2vuFrFU+R44Ddj1zyyeX4QGA8jdhrfbT69eb+MHBtR8ZOp1jzIST4F
PCn1dClm1poWd7eRpzGpnHzfqpTdDp9xcL2KPAPI0xDWvIZwPxjTjMhrd6W4
bcVZP/w+IucF2L1h9T2CZwD8LMK/gPupas3mGN4zVZrRFjBvxsUN+WQ/xDCK
uG9B5f9ZYXY5Jh4KwJ5Pi2/2Sk2w385azZv+Z3C2Cfs9zovB81pUWB7ibF9U
892bMHuBvD7UmQA3vQnhaYkpnrGHYI/CXvGqH2qOM90Zlo7JyTDur8Ie90vX
19x5B2KfulhiGXA587CzEWmGuelHH87jfIm48iJ/P3Id8UgjnxLSCfveyqh3
zoE7w1kQe53DTz2TS2qMuqWGsk6b625fcog96+aVK5WGyf0z4GlKCM8u1G+A
j9+nel0uD+dJrMQ5BJy/o7I7wH9XrfaQ+Thn9kjNTbq9pK6pYfbyCLGrEen2
KOLeRrTH85jFXEq4/kQ1P84uA1zZmDjkHtfH1SP3if8P2tTXu5Rqj1VJx9Tw
jNsT4iD3j1PCcxx8dqels0Vo4Udc/d7NSJfk8UONuOD5m6R6Zr/UkD+g/L24
34hrXtRXe0r314GnLSae5xH7OakZc1dy7vxGhfRNbS8kxAs56QEnU6h9YId2
4UGJZv0E9811+j7lFy7+N8qRO5SUbx966k//55v5mJ8Yl2qEs6lK/x3Gjrh5
8vxKTDtMrZAzxnIfv9doHvSdgN3s9i4Pu+BwEtdmUthKgL/M4fkHX0b7IQ==

              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtk8tLFHAQx6fdbbfVXc1ddd3Vfaj9B/UHWERKkEFBkBSUdeigFelhNbbo
1IPoEqE9togOQUQPO3SIoKCL5W6hEp1SIsge4N6SCOszfD0MzMxvXt/5zq9z
8MSe4wEz246EkOsJs83tZjMYIR6GsTdiP4maza8z241dbTP7EjE7i29rs9mH
DrNI2OxO0KyEfYz3LvLfYU9h33Ob9xr2zpRZGd8U+R/xDVDvdCt+9Lm4WTRn
VpdTvyD9L5Hbm1Ks5/RkzHYhYd6Xqfc2azadVT+fKU/8YfJfxDTvGewSMhpQ
je6k2baUZnPfHLNUeH8WUo1+5jnP+19iS/TYR+3HzerlGEaoPYrsrRden30/
ObNxYfiNHaTHQWovId3U7iH/dlA72kHtTUn19hlqLWZXm8yesvsVfL15YpBv
Ac0wTr3JhHbhNf8VzFaRn+h9DcJaoMdySJjrsUdaZA9hz2S1Y9/tFuxxat9I
iNsavvdwN098NSZM15jlVE52A++NcD9Ezte4OPmVpnda2B3PcF479117TJnZ
biGzG8xijWYHHEtG2HxHR6g9mFO8z7uaVk/v5TmP2FV57V5852+Y7dAany+R
IvoCvlfrNWMnsUtNmuUk/H8Hzw/kQVj7ciyVDmHz+Cr6dIfmd18D+B7Cx906
4btMvSttuhXn6AJ8XUT+hMXRJPpYUtxEI9IXqHGuTr5F9Nft4t59q+gTa/x5
j2JSNTzXa37O6k/5X+pjvn52NZASdr/xCeKreXHtN3Gf+EpWXPrNdhJfyOiW
/H4/oY/lxa3fyEqrbtpv2Xfif3shq17+x28WxJlz5TcTJfa57zMqDrrSuim/
Jd+5Yym2areOyW/taE7ceP5/5pml/A==
              "],
              VertexColors->None], 
             PolygonBox[{{5977, 6282, 1970, 6283, 5976}, {4534, 5977, 5976, 
              6734, 2310}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5307854108356137, 0.4701257002726943, 0.8630634222214866], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtlEtIlHEUxa/P0VESx9FRx3mIC1dhGUS0cnIRBS2KEKwoUumpSW7K0mhT
lJuKSAQz6AGhBIGULQytVr3GgoKgFmYRRguJNmFS9Dvcb3H5vvl/955z7rn3
P/UdvTuO5ZpZN5FHPI2bfSY6wjwTZrsrzbL5ZvuqzTZFzK7kmP2oMRsqJ7fI
bJFnXtrsDrUPa82miDcAtpN7uspslvxoqdnWUse5GDO7QPwtNNsAfob8ZXAW
eP+S8LwRak9Ru4WcdnKn4MiC35c0u1buXI3FZh8LzOrzHfdSsXM9rzN7RWwv
QS+1DRVm78E5gP6DxFgILuJGyHtpJScT85wivq/A3Yb+P3iQ4f0yOWP0N0qU
lMEF7t6w8w7zvYm8XPKzaL9LRND/Fv5M1GwnucsRxxXmC87nAm2b4Ryq9r6+
rfI66VmN3vXgDJA/yPczxD20HuI5GHXd8ncFvXM8R6u9RvnzaCzEo0/4cg78
8zHPE7ZmIH/m0dtf4bXqQR5K/1V8/Vnpvx/Ver1qNYPxQvdhkvP+YC5nw74j
Olef6kk9yqMeuJ4w0w/FziH8ATifwd1JzS94vta4TmF0BjjKlbfqL4/831XO
XURPIeIIuNNgPI57XpizYuIo5y/r3Gtp0G41Btyq6cWzJd6bKtwveZWt833S
LvVzNoy3EyH3Sfskr7SbM0nfT3m4LphRH/oLko7ZnTIri/u7fB2Jubc3wbxF
zIB5mOeDWp+dPNiY8p53ofd1wrlOMLvmSt+FkbDvc27Qi/qQJ7qLa+KuT7uj
u6H9GQ/mIQ91P/7VeF/yRR4vBfcjG8xiG1pagnt3Pe37rbnp93SJ+5MGf3/S
vUvx3pPyd+2H9AlH3/VN54twfq/xuWu22kVxL8KzJ+X88uZksHst+N1A7XFq
WyP+/yJvJ9CfE/G+mul1bdz3KD/td1t7pr3sCjuO/mPkozzsCnZS51nw3kX9
rmtuC8F9b4n43ZZXt8mZJGc25By62/L6PmdtCffiPy7Hwbo=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtVEtIlGEUPY6PeZLMOOo4OjrlwqWpENEmH1AYBBUhBNHCpOxhUkSlKERE
1KoiKKFsk1BJEETZwkhb9hgLCoJapEHYpqg2GZJ1DudfXPi+ufc799xz7j+r
ewd3HAkB6GaUMB5mga4UcLkIuBkGzlYDFxh/y4DXLPxRCUxUAGMx4EMpMJIG
Opl/x/w46//VADfyQLwcmI0Ao6ydrQV6Wd/H+JYEriT9tikKHCdePgMM8u1A
AniVAwqMFM8z5FBMrBLG9jj7kt/5amPorTi9rANeMHpCrtlPrBZixiPmE60H
fvL+kf0OEbOpCnjD+p1xvxkl/5WM6zXvHHOFOvcSXiP5jzJ/L+wZw8TbmDbf
g8QbZm4zORVYP8nfJnhv5psRYs/w/oSzP8pauz0xczuQcS/1PMHaXwG/JeIn
iL9YY+2kRynvkXr3+k6DVmqssfLiOMx5iolxl2+7GV+ZW0w6p5p+ajld694F
zjPP8yfGXMieLNPrNeRzNGw9fhNvjHinY8YsZ22s3tqpv7wcYX5vzO9vE38T
+bcl7Nkd3tt4Twf3Kc7+mLEc7M81atcV7Is0PVXhHVNOnBoz9khcxGkbz7cy
1lKanqv2TmmXhLmWfK5yhhBzBfI7w/xYylpIzw0NQDvvl4rM+SlnmUzarz8R
7/Ii+V2Meqc7yO95nb3XjrRWeibN8mUVUMb3xyqthTxrZv9WxvuoOVzPe6fl
nb4B7Z44iYvqpVWB8yyErdkA+R1uMJ52qqXWnskrYWo39Y2NB/vSQax9Gd+l
obx6Fnxf8qyIsw5lvIuaUbuqnRIf4Xcw/yBrfaTJbvbelfK3Jg0+s/d8zlpu
SdjLhZzP8rSTtSfz1kL/EUNV3uGlYP/WsXZ9ztrKr56cPdR5mjNNUfutWefF
r53n+2nn9NtbnvuD70Ma/Qfm269W
              "],
              VertexColors->None], 
             PolygonBox[{{5975, 6308, 1983, 5298, 5974}, {5270, 5961, 5960, 
              6266, 1962}, {5860, 6634, 2241, 6633, 5859}, {6306, 5975, 5974, 
              5297, 1982}, {5961, 5271, 1964, 6270, 5960}, {6178, 5860, 5859, 
              5184, 1905}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5574758836920409, 0.5165332244074129, 0.9005801106480258], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVUtIlFEY/dRpnHHG1Bl1Rn+dmhatAs2gJ6YWGO5CihZRmWYpqJFEYu8I
omVkaoJpVEQkFUT2wNJaBUXRg4Q2ZmIZLXu5SKNzOHdx+e/c77vnnnu+c79J
1u+vbUs1s3MYaRjrk2bVGL4cs5BndjpmlorAZLHZFEZNGDm5ZhP5Zv1YvxM0
++dHPuab3Pd8ipmXbbYAGE3IT8nSvi6sVwCvD/s/IW8WB7/BGMX6QAbwfDpr
HXL6sXcMfAIRs9642UfMm4HRAqyrmEcxDwbMLmPPRcS3Ys8Ezh8uNHsM3g3g
dQvn9CM2km6WhfxBd8ZvxH9iXMgQR5/jeQS5RzGGkP8Va2sSOiuItWXA2on1
mQKzMNYvgedSaFCEmB/nBrC2ICGcGez9VqD17/hOI28A88/Qb0WeOFMzciKf
E1GzZ4vM3kGLVcDYlBSvIayHPWlag/v1UPc03Zt6EudFkdlLjNoQdMiVvtR2
Hrw68bsvXZy6wuL8Gvp9COqsjcjtdZhbYtrPvQ+g4X2MPUFp1J2hmja435x3
Aj8FtSnyC4+4rONf8J3zpHUl4mtxr9uZqi+/vO8p7K3O117fYvgO4+BCs7qg
8uiBL4i/L5Buu7KFT87cP+hynuKcY9Dobao8W+58W4lvBYYf8w7gdEfkTWp/
o1i68WwvU/e9ibzRhH7zDN6HXj0QU224dhI5kxg9JowyV8ceYJeCxzj2lCD3
Wlw+PJsjD9E/TxA/EpXXlyD+N6J7sa70Afkz/stTznJg70PeAPA7oM0G5O8G
/l6sVcX0Lu6iPlURaUbvHHc8p6FZO2KjAfH1xcWZdxp392oC/+We6lWGb4mn
+BfsnXLvmvWgfznPRLwlordOfekleor60qd86z/A+RA4vEqV7+ri8t5VrI15
euvP4dNyxDaH1D94HvFbs/SuyZkxfonDms+6/kDsznyts+ZnYtKNb4uaUs8W
1LAY8zacuxp3XIlxPSSN6BfurXF9gzrQ+3xX9P+IJ114Fv1Ov9DnvEN9ULVg
bbe5PkMt2deIQ570KLmyR1EP6kOPsZcQ/yFq8gijkT6BVnNx8WC9+a547uGo
vMI11jvq3nhzXJ4gB/bFWdcbWQfele96B3JKXQ61bHV9kj0sL6l5O+L+hDgS
Y3tcOOxnFbnC4Tfd9Tfeg7ncuxH1v1coLf54qivvyx5B/tSK2lxx/qev+VZZ
o4Cn/xLWIBFXncj7aFRvmDnsK8Th3Ydz1PfZ8/lGGCc+9Wp0HuabnvdUJ2LR
q+T8H/1p9IU=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlElI1HEUx9/oOEuOmU464/zHSTt0CqwuRWVpQdFNpE5Rmu1gK4G0EEUU
4THLknaDoKSC9jAUuhRU0EJBF7MNo2MrlEaf13cOD37Le9/3fd/3fr/ati3N
mwvMrAkLYx+qzL5WmD3lcCCG5cwaM2ZvC82CErNQudnlMq2zEc5rFOO+7aVm
i2vN5mFhfIqw42mzE9gQvm/AP5gyuwPemrjZkZBZJCcMj92YMPsWmPUmzY6N
MzuKzZ5k9oCzU5C7Ss5OML+Q7wx43eBNnWj2qVL7lVGzyeT+if8v+LdMMBtj
3QdeGP6L8UmwL8GuxXVWx7qbml6zf4UNw+UwOS6AvQlOC7hrgX8P2Ku4f5w1
e4SdIra52Gw++WbmVGsEewm3fWXi5px6iT2fVu2uwW/yNZSrljPUt567ddig
+1Pj3qRqcO4vqKGedRf96EpI02Xod4Wa+6PCO0lsXVJrP2vkfiE2VCjMaeRb
TnyoVDVOZz8jUK0DIfV2VrXZxWL1uB1uncT/jUijQXx3gf+sQJp6bzoqpaX3
6AC+HUnNSwHxFfCtxIIJ0vA9vk+y0so1u0G+S+RLJqSn5+plX5pQzpmsl6bE
xTm9Z/8OO4c2S/DJUu+8icKK4D8X3zkp1eo1N5ZLY9e2Na5cddQfjylnlF7F
sA2sT+P/mX6NVGnteKPExgLVchaMrWBvS6kXXehVROz2Cs2qYzj3P4F8vYY/
5I+lNYve/+0ZzZTPkr+RnehxPSNurtmhlN6Ev4Xn7GeA3VEjbbxmn/09Sd35
GxgF+y7xa+Oq6V5GHJ2bn33H/0egt+M9W0S+3cT0RfVeq1lPqVStm6OarVw6
v05oVoarpbXPTCFcRsrkv2O8ettQq1n3HvfQi/3p/F+AZq1p9dj5+5u5D5f+
QLO2mrPl3K9Ia9b/zyf5xvLvy89uE3srI1//I/ythWuU23teT76P+f+mqVhv
cyGat+Xz+Sw+zOrO9b0J1sZ8PvfZk9bM+Ky4Jt3g7czz97N/Pci4Lw==
              "],
              VertexColors->None], 
             PolygonBox[{{6300, 4910, 4909, 6299, 1979}, {4910, 7479, 2982, 
              7478, 4909}, {7372, 4858, 4857, 7370, 2898}, {4858, 6153, 1886, 
              6152, 4857}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5787795075031905, 0.5515998087795413, 0.909839752241409], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVUloU1EUvR3ThLQ2TZv8JG3StOpCRV1YqrRVVwrO2ApCFavSLoyzC0Xr
vNGCgnUeilIH0FpR3NiFdCmiC6044YBjFoK4cJ7Aczxv8cj977177rnn3vuS
XrZm/upcMzuDlYfVGTBbgvUadrtn1hQ1GyrEGQ57sP8gx+xw2OxFwmwbvg1n
14rMuvLktxXrJu5sjJj14t43gLcHdJeYL+F3BPvb8T2nxOyo2z+EvecJ+X/D
b01aZ89gHwyL10T85sfMdgHzN/Z7q8TnL3iOrzALgEcMdmO5WcZn9gl73SGz
pwXCanexIsCuwKosNasCz6uVysPn6Zt2FmfdQbMDWF8R6wt5w/8QvocDrxar
Bnw2wGdDrnhNCotbm8uNsRbjvBXrqE9aLHX61IHrhCJhnUceF7BCQelCnsSZ
FhT3HNgNqEOjqwV/Oz3Zc8G3Bb5p2O/AoQm5n0QtDiL3JuRYU6A880LKJets
5sH8ksN0h1p2RaUn69jmtNrnFw9y+IAYp/PFbTNyHUzo3lLXM8wrUGw2wq+8
OgLy4f21wB6I67vb8aCG5EKb3NhffX5xIK/bQXHj7y1n7/eLE2NRT9rktrAM
NfUch4T6gD0wBH1Gl6s/m0sUg728Jml2r0LY5ErO1G0P+Lyq1vd3YOYCZwHw
e8E/g++hHJ2vj+vOReh2HXYPdFkLzPvAvAPM94g7xsX1A+MXfHtc7jWuFptw
7gH3dp64rvSJw4BP9fwI+zVwCjz5tqK+EU+6sVYry6QV87nidChC7n8S0pEz
tC2s3udstaY0X+yvDtdjLcCpRw4vCpXrjzLly3mdVaweaIiIK3n2+4VNHThv
7GvuE2NsWDjFiBVMiNNMaNMc1X41zpe7eckil73QYRAYPwuFS8yRiFXlNJ+M
mJWwV/k0Z/Sn79S0Zph9Tc3q3Bwx3pmAdGBN2jzVhT3Gu+z5O67v/89vyNUD
9mXEmheV5kn4prDeAXdUuXqIdSwNqv+JORnxp2PVhlTDyrh0mI27JzzVj/i1
rtbropoB9j+5cvbJ4Qb8/njCvYt36E1Etd6CvWkR5bsd9g6sftcf1Ita8d1K
ufeKtep0M8s8ZsSVSwY9uTqp+XmC+j9OaQ6Snt4I5s66nfJUo1vA3BhWjKzT
jDy5tzuq/SNu5jl/dajjopjyuYQ7D1Pqx3rEnJJW7v3QcIHTlvNKP87pTsQ8
jrMrPvVoQ4l6k7NCLagD3z/isabUP+RqMK5Cs8dakCPfHeq6Kqm5Zf/w7X+U
EgZ7gzicl3PwO1slvB248yCm/7AJMeXDXMilw80K/8c+J8Sb/cY4rMMxvuue
5pUz8bZYdenD/b8J9WLG1Yz7K8Alk9K9LOrzPq73jO83+4Y98w86LAWT
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtolVcUhbeJ6X00icab3Pvn3twbr6LQB9SBj0i07SgFlRaaBqRRfKGD
RmPVgfhMGydaqNBWjVRDxUfBRwIdFHSgIh2IOKmK1lIsWo2DgnTQVlpU8Fus
DDb/Of/ZZ++111l7V1dv/LC/JiKWYxOx/6ZElJKIDamIHg762GdLfNnfnBDx
fTmivxDxHc5N9RFHmyI62d98JeJROmJRMWIxdo/9k9qI59x9ht3i7tlMxKus
0yWvR7A08X8kRh3rKncuc3YiF/F+Y8TKbMT6SkQFPNPqIq6SL8/6U/5dZz2d
fx+3RPyF/cq6i39t5C5jw+C7Rv4LrM8XjfdP4u8pGINy/89+YTNx27w+gv9p
cmfG8Q2T/zDnHxVczwgcjE6N2Ej+a+SqTIpoqUbMZ39AuScTD99K0VyIkx3E
u0i8FcRahX0L/nPN5kb19nN3Q8Vcq76+9ojjBXP9sCHiPtx8hv1AvMD/KPdH
xu8L0w1q7+D+1/Xm5BD5fiLfWnINYOXEHCq2cujtOvAZrDEn9/Ad43wJuXbj
34L/Y/Dvp/6LYHiTXBn+DZPri1rX+hg82QbX/AbnqcTYdL6Pswd5+18G861W
4ud9phrega93q+bqK+7fod4z4Kmt9Rt2g28gMdeqbyfrHdiWGvP5N3j/xQ6B
9Sn/evA/zPmFlDlZAJ6iak5Zs9vYv2CfTVsPj4rWjLTyJTV+Q+5L4++zM2vu
DubMhTi8Ar4B9utYdzba9wD7XVnf6eV86RRzpX8rE+dULmlwKLHG9dbC2AUX
n/NvNOWa/ilZI9KGalrGWWfe94fwWcN+beLa5d/Gembeb6kePUjsrXlzMQGb
RW3LE99VzXPhf7aM9Zy0udE/rcWRtCNNS8vSkLTwW8lcSBPS7vace1H13qbe
u1h3o99sUL2ZMzbVtAR+P8i7XvX0c87fajEX0pu0sxebkbGGFqKFLmxak3t8
s3qn1b0on3ZivQafvSlrbHfOb6K3GNJ8oP4GMI5mPGPUu5+0u3fUw3q7TQX3
vt5wrGgN6O2l8RPMs1Nlz7LJ2B/kG2u1dqXhOvC/3mz9av8eWOdVrN3pYK7X
rCDHsawxLONub6tnkzj/mdr7xvtPM0Oz7WnJ2DXjNKtOlp1bHImLt6uOLU7m
4/97yb2snv2l3TNKs0lvMDHxjBA34uhFyZqQFoTpJZREz1g=
              "],
              VertexColors->None], 
             PolygonBox[{{4600, 6236, 1943, 6235, 4599}, {6272, 5952, 5951, 
              5272, 1965}, {5952, 6274, 1966, 5273, 5951}, {6687, 4600, 4599, 
              4508, 2276}, {5855, 6630, 2239, 6629, 5854}, {6171, 5855, 5854, 
              5182, 1901}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5984031810115944, 0.5831295917623144, 0.9102871115556601], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtlW9o1VUcxr+2P3eby7Xd3bvfvdu9N3vV+vcqxNRIKCg3szYLQheUEaSj
mWlQ+sIICnppaZQIgTWnIf55qZCC0xHXoF1XKZRJ1IaDrF76IqrPw3NeHH7n
fs85z/f7PN/nnLt8y/bRiTsi4nNGE2NrLuK5tojJloh6Z8RwV8QM32eKEevL
EX+y6XhvxMEs4ix717F+lL2fEe/ojxjvYb09otQX8U8p4utW4wijoyPiezCu
MPaw5z/2zIK7jz1DXc41BVaF9QHGb9TR4LsezHfY/8HdEe8z/l4WsakQ8Ttr
9eaI/fmIc+Seh8gTxDvJe5r4KcaDrY6vJde71DzH70VGF9jHWd9Wi2jn7BLq
HwH32WWeT7D/SCXiJnsfAP9tzv6c6hTn8y3O+XHe+OIRmbmMV/ldMJcmYvei
1ya02s23Cm4d/JV55xD+Ls4ezbsBI+QcZZxZGrEAvwLnX+PsJLEvGKfCdY4y
DjM/MRBxkrGG/V+B8WPNa+IxkvZMq39L3dO72LMic1+eJP408cvN7tE3na75
RnfEv9R2gLNVang01SxdhKs6/yq4B/WksbCEcwz8qzXvnUeH62g0SHxDm+tQ
DdJevKX/WjBmy57vI+8f3eY4zblPwFpA2zep9yO0O4IOl+A6M2A+F/iuJv5p
k787yp7LN+InHHlA/ZlaYiz5QvOdZedWXvlXvpeH1Z89vearnrxVNF/1QPyE
/15mP8lLj4MxXXGevcQ3Ev827FHpovhT6Q7JM/LxYuIuLcRDee/JG1eYI2A8
VrbfBqmlRg0fsqeP9YyxLWfdVYv6e50atlaN/zDeX8E42GSfDqU7JR6t/eZy
i7PdOc9V11Dys/q0kLSSHpPpXl+jnz/U7KcN6W0QvvKUM89X9dnf0k13YCxn
/9xHrCXz79v0tgavXtZPcI9XVq39T/32ivI3t5u3NHyBe7+ZvI07rb++6oFy
rul1Xr0Z9eRbaXoz9X04cRe/N8AuZub0ZcXvizzyKrFXGIeI/0INj+Tdm1+l
d7frH8tct2reTW0PddpDq4teU1zv0Hx6i/SW6A7I/6+Td6Lqe6U7pzdRvhTv
RsHc9VaqF+rJJWqbqThHW8nekn+uMN+re6E7hOcvDvjeyl+Xk791Xr6QJron
c0lDxdQzxQ/kfa/UY3E5nfqtt02ek+4L5LoBt0PMm6v2nDD0+2SbNdF6f+bY
MLyHyu6b3r9bPe6/tFDP5NEcOXcUXONi0lF5t7B/VdEazpK3UXIO6bIu+fZ8
msuH82iws2jPP49OGxln0WEz58ZK/t/Sf8tc+n9ZTuylks+qB/X0H5Qj7/34
50XqOwzv8Zp7IB0H0/sgfrd7zFEaS29xOUYN0WNtL1b8fyMdXy75Hqre/WDv
4vxUzrjygu7a9qp9L34Nvt8V/Db8D8LWFII=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtsjGEUhk91Ov+0SnWmnf7Tmem0Vu4rEndLgroUiVCCENqGohYuCzYk
lqoEsXIr0kQtSURiSgSJjnuidYm0IXHbsRA8J28XJ/m+/5zznve853x/w5b2
pl2jzKwZi2B/a8z+YSeiZqe4b0qZbcbulpidLTZ7lTPrrDS7gK9prNmDrFkf
dpPkaeVmf4g9gb8X/x0wBtPgJMyGKTKRmBfk70viBGsF+YPk5jNmjaPNFpK/
pxoedWatgdk34vdXmc0hvgqsM3wrB3801l2kml+p1VFr1hNRzV3ktmHd8H0I
Xj947dyvluhbntxnWG/M7DgcXsLnekK9rMSi8A3Squ0cc2DXYY+KxWGI8zC2
Ay6PItLiR7XOrklHUj16b35fz3kAvMf4C2Poj9yhEf3OgJnhHtZIq09wKoC1
G74tgTi3xYnFX8m9p9RsPucuNOnm/pycPPodqzf7Sb1earSSu456ZWXMhv4v
4r+UFbbr08h5DXYbvb9T/wHaN6Q02+V8OwrWEmpOjQozwBdLCds13w7+/Yxi
8+BfAetyVtifyUmG9Ez86ph6eFotTs5lUQWca4XpWIfo5wjxh7Enpn7m0lsq
lP7n4DQbPTeGmr3rf5JZvU1rl4bYqVXosQTMSKkwXOuuhHyu+WJ8K4gZiCqm
nXMj/R3k/IVvBbi8r9Su34hpFr5zXs9nMoHzBzic57wuEJdxCfk8ZhT3X3Hp
7f4bGe2Y79ZcNJpM/rOk3tOGQFj9KdVyzH74FbADpdrhFrQq1Esrf3OzqBUh
viImvk1ZzdBndwv81+zXm5x2t6hYXHbWafed0++4ODo3r7ea3OVJ5fsOdFZp
p1z7K4F2p3gk3vOd+yRimgP1cBythlPCcs1c2x78twNpHJKbZ+bLYnrDPsuy
tHbXZ7o0rhl4L97zafp7l9bd/xGe6zvmu+UYZ8mfkhC21/Dd9H+G/yt8R0vw
f0yKi3P8hj8aSmvna9z3hurN92MBWl+rHNkV/IfgviXUbvkb72N+9zJ6m/5P
cu1nJrTbPoMZ3KentJuXmc9WcmfXKPd8oN26n9W/0HdsHr5toXwe08asWnOa
lb/pDnzjE9Lf33RzSjUc299QNf61cf07/J+To9fejHbLd8z/hS9q9Zb8Tf0H
D0HX0A==
              "],
              VertexColors->None], 
             PolygonBox[{{4897, 6244, 1948, 6243, 4896}, {6260, 4897, 4896, 
              6259, 1959}, {6150, 5662, 5661, 6151, 1885}, {5710, 6297, 1978, 
              6298, 5709}, {5662, 7342, 2876, 7344, 5661}, {7507, 5710, 5709, 
              7508, 3002}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.6180268545199983, 0.6146593747450876, 0.9107344708699112], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1lU1sVVUUhTdt33v9uRX63uO1fX+0CUESSxwxaaIGESeWmiCmLU2IEmsj
A7FC1MSYyMAiQxATQQf+BLBYYEKMow5sSxiY0IcMwGh0gDOMM3SA4rdYx8HO
uffcfdbee+21zx0+cGjP620RcQZrx87lIkZ7Ij7kZXljxMBAxHH2dmQRU6yX
OyKOVCJuborYlWetRvQMRpQ6I7Z0RVxnHcLn+3rECjYG1uZOY9xlf6URcRXb
Bt5of8Qr4LfA6QSjgPXhu8b6W8U5vEEOnwz57AhYZ/k+TsI72P8cnIvsv9Pl
9dw6417rcJ7CHi4Zv3t9xExmnBvgLw8aq1CLyGMXwMjj/0fR+Krn8cw4b5FL
Dp9nwVll7ya2lPP3bclnhDinOf8uZ/Ng57ANYKzCwWzT9au+H6uu8UVy341d
oaZF9m6UzdtMw7Wpnl94fo58zlLXQl/EU/j9zdmRzBiXOhzjXt7PWh/L/PwM
OW/nfI29KvydB3/+kYjb4PwJ78fodQd5zRFrNnMvN8PBXvL/HZ+5qt8PEv8+
/i1y2Jq5x8pT/dlado9a+K5V3YdfOduizgfUu4Xv6+BkqhBxqmgexeH/PROO
eNee8BfJ5xjnu3oxMLqxIvUdBvtun2PfA+dC1XlKg4crzv0oOb6HfUFd+4k5
yv4sPk+yd5I8fm63Fu+0+eyuijkSPy8T5yVsmlgbSj6vs2+yRtE+C2A8D5/f
gvEp9Szl7fMEuXxNbn9x9gprAf9b4RqvJU3WyGHvgM+s9bomfZtgb33JOIfw
GSva/wQ49wc9S/M83+lzXd+hwRMl1yDtP510It2NV6w9aWpP0pV4XElaFd/q
nziX7wsN++/MPNvS8Gdotb/sOfun37OneZlHP8sNa+ttYn2Mz0LBfGm2l9Js
S+Pqo2buSJpr5SHOFKsFzmtNz5C0Nd7jXghX2pXOp6hxEuvqjtjHerDpO0l8
iTflKS7r/eazWfHcamY1z2291vaXnPuq6ffz1PVo0qrm6YMhxzpVtj6kDeUk
rhRL91mW7jTtrSYOH+D7LzZZ8CypNvX6I3CqVeOcLLlP6tEZ4pwe8t0zx/5P
NdcpzmYSpvykS+WzDz1PFK0R+enbQ05yzkW1i4/rG52n7qHpxL/40hyIzxqc
1Cuewd1Fa0u6Es8j6X74Bk4W677rVxv2k496Mpb6Ij/FlvYmiHlbmuP8dvie
pub3C9bjRKr3gO6/sufjYt15KAf1bWfS2HLd95rqEreKr1p0nymWYkrvR9Ms
T6V/j57V88k0R5o36Uuci0txKg0O07erdb9rfn7oseZfJeZsw+/tA9aW4l6q
W0MP/ymZ61Ms3dn6/x1P2tM9L37037hct9Z0b+r+kub/A6wi+/4=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtIFHAQxsfX+mhLd13ddXc1BStBo5NBRlFancLICrU6lGBWh8I0iqTH
Ja1j6aGsi0UPzeoSHT3kgw6BrnnQKIKom10t6Pkbvj0M/B/zzXzzzfz/VR1n
W85kmtlRLBvbGzZLxc0u5ps9zjDbGjF7k+R8lVkzlhEzW8anGVB7rtliuVlr
iVlj0GwndoD9IewVvn34zIPvZP+c4JXYJ3yPl5kdyTNbwL+tRDEc24T9jpr9
wZbxTZF/M/mKis26iHWNfFHiTYF/BL5uldYPKswyV+usp9TsW8isGv67A/iD
z+a+K6h4zv1+UlivYRf5p8i/kftN2J1Ks2GsoFAcHoJdIt6AKUcr+Hxyhsk1
x74L7Hhctb0j5i/uTpRrPYvPX2r5DP5fDrXC/zt4i0k7z78P39piafUaTAv7
/diiScMa6s3C/zH4G1lmV4k1n67fOe3FdzQirPsf464Kq86TxnG4bcN+sP4I
fox1e4IaCswOkrOgTD1w7b2mp3DpLZV2RXAsZF8fk/begw3kep5UPc5nivUk
9ihDM3Id7fqx2qDO5sm3gBUTux3MJHybyFcXlM9T6tmCRqmAONZzPxgR1wRn
M+ynyzWL3qMGuHXA516azxyxdxCvJqiZfRKRJq5F/xrFPo7/cK5yVLC/FNF+
gv0V9gE0KCL3C3p4F+6Z9LwzqBlwbCIqX8fMkiuFz3RQPfFYDWn+ft+Gf2tM
3BzzhdjjxMtnVibw747rTflb8hm9zfpcTLV6zRdZ16XnYZR442j7JCmtG8Cc
JNazpNZTcMhlnuKVqtVnpi+iGj23a7SEFv1oPGJ6o8EyzbzPuvekl3xDab29
hstRaeJaOGaQ+CNRzYLPRA74bvitBKSZz+6FmLg65yT9WRfRrPuMeK/zwITy
1PP3cc2Ez4Kfee962K/PVw9nqG0au5mjGfZeuIaunfckFtOb9rfsPinu5sqk
j+d7CfZFUm/L38j5Us20898D50BCZ74eI+d2cu8ulf4/8TfqHQ1p7WenKvRn
+V/lPR8IqQeuvff0DfoeXqu/4C2aNZbozNf+53Wzz00ol/9BbQlp6lr6G/zA
/jb8vmZK0xz2p8Oqzzkuk+9WSH+Ja9TK/UpYb9cx3puhsHy9xv8XxMcf
              "],
              VertexColors->None], 
             PolygonBox[{{5914, 6651, 2255, 6652, 5913}, {5211, 5914, 5913, 
              6198, 1920}, {5157, 5841, 5840, 6143, 1880}, {5841, 5158, 1881, 
              6144, 5840}, {5970, 6718, 2296, 6719, 5969}, {5269, 5970, 5969, 
              6264, 1961}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.6376505280284023, 0.6461891577278608, 0.9111818301841624], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VVlsjFEUPqNGZ6bVZqbT+Tvzzz+tCKJFJAhiSyQiElssmS5eCCUe7LtY
nnghoSpBvSBotbXGgwQPIqEEIRJay0PjRYonS8T6fb7r4ea/c+6553zn+865
M2jZ2gVr+plZC1YB1qKY2cswNjBezJhdwurCPlpq1lhs1hkx603jOGd2K6LF
M9rXlpv9Dsz6+pvt9sx2YX0eYPYI/o/T8hmaNHuaMtuPZJcRew98vsAnhHiG
tRw5YlhLojiHz9u42a9Ks2Ycf/fNpiTMDoTMbmA/HvZ64D2ZNfsA+yHEKUf8
68i1DXdHV5hNhf0g/HuAqw74uoDtG/wGI36A/cMi5aIPayFG5mJ9x6tku4t4
HuJuR8x8oVlbVDWWFCvezZBij/D1O4/9S+S7h/2fkL706Yd7r5C7D3Eacf4j
rRpXA2PYV9wobPmE/McBf2kZ9rhzAfln4s4H+HfkpAGxDYE96/g8DB5SDie5
qC7S/hc4/ukJ+3H4nMjq7Dy+HVgTsL8Y0R3GqQOWC9CmLiou7zg+ZyP/+aTw
d+J8QqW4uwP7dNRcDT4i0GNHTBrV4bsT6z76pwl19Pj6HYe/j/u9yHkCHB+r
Ug+xDxqc/yrUuD8uLcjzDMf1xjJpT93JcW25+H2F/etAfqdz0oy1T0GejMs1
B/X/CKuWSUWqew5yHQHftb56fiE434Qam+D7Ffa8r5pYS72rqxOcDUsqRjti
tzmuWsh/Sjl2uDv0nwjuW1HLbaTogO/KQLyx16kJe+kBYi2OqSerwENBVFqz
T1+EZe9y/cTe8BAz5ak32rPii1zVV0oPajEDWKYi35sC4Ys4PFvA90jw9q7E
bEBas8o5bUtKY+o7H9/ZKWHhXb4L5OcK+LjqC2uDq5F6ER9tfCsyiF+Y0F3q
fD0n/lcg5rOkZm5yQvNMPEdgG4ezs7CPAf9jsfaAlwfA9rZK79HwYt3jnLJm
nnN2WN8CTzW2gof3CWn6qVz3efe5m3Vyxzo5/8RZg16aF2gml1ZIJ2q0Gfst
WK2Fep/2ujeKd/hlvcx5NKm87MFYWn1IvH8C5WXNrGkD/POIV4vVjJizgPla
XO8BOZ4biKuPiNnn8HPW25PqAer//y1ZD8zhtDBQ44HF6odpCb135KeVvZmR
jutgC8qEge9NxhNvfC9DFZp5csk3l/YW4PrqevU3eumcex+a4F/jC8MpTzPD
fuB5m/Nh7x/Lqv+bE/KjDzVhbaxrH2J3B6rJ98Q7c7EOakz8xFsaCPPWCulE
jagj54RacjEG8bCmhqTwf3Y6kZ9RwDvS171u5D2TEzbGorbU7gn7P673jxhK
HJ/Uj3NJ/OyXf/OHfWWZOCU2/o/x/eWbyVka6PD/BTCtDqA=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlElszVEUxs/r/NpH0+G9f/ve6yCEqGFDowklhIWFIVSealdSlVhQU00x
bdiQKJWYFoaIVluzsChbU6qJSGiLhdhV2FALlN/J9xY3ufeec77zne+ceydt
3LZma4aZNbOyWPeLzFYFZiM5Zhlhs7ao2V/OqVyz1ohZbYXZ0pjZDoKuF5h1
J826WCs413F+XW72CYxOsPryzOJxszOl8v3M+W+V2XiVfN3nXsLsFj4viF+X
bzZG7Noys9Pke4q9hXx3E7K99Hz4tpWYNXBuhN9tzndYP3OE0UBsIlCsY3xn
PwTm72w4cj4Llz1lqu9jptkusB6B3wheE+sD+YZZLxBjGfVeoLYO1j58a8g/
SO3nk9qfAqMdrLpAWF3g92LrSao2r3EzWK2sO9ib4fsFLsdY7+AzOazcO+Gw
IV8ceuC3Gv9qsL8SkyBfH3jzC6RxIb4pcnaSqx+fOrRcVGx2MiT8GFzmYD+c
tqewbcCnhlr6Q+rt7jJxdQ2GyJWix8+yxGkx+03cvSLfdGIWEt9eDe+JyjEr
oZ54L96S71ql2dVK6eMaHwmkqWvpPZlH7gVgnAipxgHqGWRGbjELx4mZRr03
ktLKa8wGvx2fUfh0g3+oSJq71h4zA/vshHL/A3McLiHyt0QUswT+l7mbEFHN
N5mNMfLnpeflaKAeuPa/wHxI7MUizYvP53H2vdw9yVMPr4A1G8yJEWHORbuZ
nPMj0vgf9m1R5X6frdn8FtXeZ/QP+bYUqxZ/QzHqvZiUVq7ZKuK7uRvNVD+y
0SaH9YP9c/p9A1u8WvPgPufZX2CFC8VxJfFd6Xj3MbhPKVUu16SLeurjqs1r
dG28x9PT9YSoZ2qpeuEaHwykkWvjHArxL6pQvL/5emrpiUtL7+l89svj+iuc
Qz7cw+WaNZ8p72UnMZYhjXaTKwg0mz7DrmVfXLPrmrrWIxV6e675G/x703af
Sf97lsX03pxPgP1SUtxdz3Pso9ztTc+jv82qEvl6zHp/6wnxd07nqvXH+d/m
mrp2mZU6u4ajxZpZ18ffXzKmN+l7v3PfgXL9dR6Ti/8DNM8Mqx8d5H6c/l/2
s2ZFNVOuvb8pt53G50D6/Q9z3l6i3vvdTLcXa9b9T0olhOlYPtNN8K0tk57+
x/wHvrLPWw==
              "],
              VertexColors->None], 
             PolygonBox[{{5917, 5237, 1940, 6232, 5916}, {5236, 5917, 5916, 
              6228, 1938}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.6572742015368062, 0.6777189407106339, 0.9116291894984135], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVU1sVGUUvTOFdsZpaeen82bm9c10IhsoNDFtKAjKggLBoBIJaDBgLC5E
QgtCi1Zi3EBijJZaE37aQIj8pkIFLBgJQVaAbaXAQqPghoULksYfILENrefk
fMbFzfve977v3nPPPfe+YkvbK61hM7sCK4E1F82Wwopxs66U2baM2bEys18D
s/pqs95psIjZrJjZclxchf1+nPsW799nza7CvsT3rlI998BnG3zdgr2H9V++
2Z+wnqfMNleavQPrx7kTUbM3y82m4LMa/vbXKMbOAu7BxirgB/GnEG8jzo15
ZimcWwlsxah80c+rOHs/Z/YQfn4I6/lZyKwppu/X8P7iDJxD/EdYT8DPOGxt
mfwST4/pudmt63B3psv3H+Q3Gzj2g4cTyGkKNoQzI5XydwmxzmGvNGH2E/ZP
A8scnD+A8zOB1/LgBET/AQ66caYbON4A/tPIrwt3ZyfNGpBjJ3zVVSseY31S
Loy8O4b9rUnlsiLQHZ5vR61e9oQnFFcNDVaSV2ze7UO8fsTaB5/fAENjUVys
xt3HOJtE3bJV4oK51yL+JuAcBbZdiP9VhXJpBYYngWr2G+79iHOT081yaWD2
tA7HhYP+UojzLCyHdSQiX/QzBNyjUfknJuqAGliEmE0FYe2BFu4nxP9iPOcX
hOMkcnoJGAZj4p55sOY3gGEYeHYj372I/wA2gFyeTukb98PI/QPg/LtUXJ7K
ic8LvjRE3fyMe+MFaZgYh5yWWspVD/JJvMRN/Fvgrwm+fofP7Xhe9HVnncuJ
/l+PKnfWdH1UeOlzC+K35qTlDVXqD+bE+gz6yi1ba7YM+Q7ExMf7NeJkBe4t
hD2OqL7Ew3i/4OxdWDOw3oFfq5TGiG84rDV1ezuqd2rnvNMze/1IoH6/h72G
vPRwE/x9GBcnvEusxEm8XNNPO3gO4Osa+Hk7IQ2RnzZo7FN8O1omzRMX/RMX
18RDP+xP+uxFfumU+o99whnAOuwAtx954nYkK32zFtQWtUacVZ5qz7qzj8kr
93km7DQZR36VsI85uwqaTYzfDv9XCsqFfL/gOD8a0ezjHGN/0Ae1wLodzytG
yPVci/secutZvrggD9Qc5xH1cBI5noItwPsE+BnPqD+o2fqM8HPeHAqkGdZk
rpvDrBtrwJkzDf5LfOmxA5xVI//r1D98f1Gj+XUGOM/50l0n9i5klQ/n5ZGk
NP811gOwDVHNG/Yxub2JsyNpzfVJcPMEthV1KuDefMQaLRUm6ps46xGnzhfG
ELiZDMQD685zPLPvv/4H/teAuS9QbxzG82zu/7lIvVIb7K3vfL33gp9B+E1W
aGbxyXw70CPPJ4SD/5y7nuq1DlwuSOvsgRrpmBpudpjon/8w1rnT/TvIE7nl
3HomL+00Atsi12sTBfFLbud54p0YWH/OB+ZLHumHPc7+YL2oq374eC6h2fDI
xWOsz7E3x1e/Msagw8N67nT+Wbe+vGrHGbbLzTH6ve1yYU06MqoL/z+NGc1b
9uWOlHrzPOK8m1TsBnxPe9Ib68q6c05yjg67mdmNXA+6fImbs4Zzhv+6TK3+
d+TvYkzrJeBtYVF8sOeJhbOFvTzqOKHm+M415yV7m5pYAzxrYT1lyumOy4vc
X86LG8Zm3uwjavNBQvr0PM0g5sh60wfXZ+H/rUA88znX1/pf6H9R5g==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlF1o1mUYxu9Nc5truXf7+3783/e/7S1PbGkfk9bQ8kBDDAspsjQqsoMo
0EXlRyLliUJEyVqQOjEic+mqpbb8iMg6MZrZqoOi1NMOQjESoQ1nv4trBzc8
9/Pcz31f93V/VNf2Pry+NiKeQ6Yj2woRryNv10SM8nCiHPFyq8/vYfBtKeL3
XMQObD+sj/gOfQQZ4Lwf6ZsdMcH/VXURb94YsbDqO52Df8uyiENJxLHGiGGk
Gf1u7L+fFvEG+gfEukzMvpkRV4g5nEZ8hjzZEPEU0tUWkeAzbY7ox+XN+Bol
/hCxu/n/Ne+DxBnl7Sp3t+FrNfFjVsSv/P8PfVEL9k0Re4nRRfwFmW23EO/F
YkQ7GMZmRHxEDnvx14u/zeEcD1QiJtuxuyliO5g3Yr+h6Hj6cxL/G/h/Bl9/
odeg1yL/cr7O3SbwrsG+dYZzHuHtKPIDb4+B52d8f1H2+UFinAPbn8gA3C+F
w6HEHIq748h8fN2O7OZ9GAzT+LuefJ/BdlD5wu0tibFO3kA+/O3kfRf2W8lv
Er0X/Vns1yIvgf3LsrkXprP5iFMl13qncoCLXuLtr5viGH0AaW1yTvfB7R1F
+36C+EfwdRj5qsY1HIHPcurc9edaZo7EjTAfxdfy1PWQ/3F8PZ7YX329c50o
+qycF3DuKjp36a9g+8gUv6eJEcSraXNuO9A3w++txBsDyyXij8NPmjc36tlr
vF/IWVeN87wNUPNOuF4GxgfAuzxzr6gGr2H7U8ncaCb6yH9Tu32rhx/CdkXe
9iPY54lXKBibMC8k148TvynneWCbj/zSYM4OgP18zr1ebTB3ucyzIg6/IdYp
pH+me+4w7z3onxL/6Qb3/h+Ze0cz0N3umdYsawb+Kbvn1Gvy0Yn+TotnRRju
ka/U3H7S5FlblxqLZu5CB30yNX8vzHLtF7fYVjV7nvP0sntxjHySxD0lvlbU
mZuVmXtZHJ1BPwTfPY3u2b/5f7FgW/0ZB88EMqfRfK8iXm6Kz35s7m3xjGu2
1xCzm7ediWdZ8/kuvvsrrqV8nAX7jyX3hmqu3phbNlb1iHw3F1wrxVjE7rlL
PUVNanPmel3Bsy7OxcUlOD9da07ubPMf2Wpn3c95SdW7sIq8lRijsGmmtEtn
cze30ftmD1h3V5yr8Cr20qr/CsOrvG2p2FY57YPL9zPXXjOuXf556t2pnZ7H
d1/FvtTTRepX6nAv/0aMXRX3vHwJwzxy2djhWdYO2lrwztWu1U7rwfbRornX
zB0k1lBq7sXB9cw9ot7QjjlY8Q5VbVXj/wFWbe69
              "],
              VertexColors->None], 
             PolygonBox[{{6257, 4894, 4893, 6256, 1957}, {4894, 7440, 2953, 
              7439, 4893}, {4842, 6126, 1868, 6125, 4841}, {7338, 4842, 4841, 
              7336, 2873}},
              VertexColors->None]}]}, 
          {RGBColor[0.67689787504521, 0.7092487236934071, 0.9120765488126646],
            EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVmloXFUY/TJp0kxmms4kL/PmzT4WXCG1qVtb0UIsViuCsY0hGAiCgksT
FGzqglBUKLVNTNUKIoo4iTGCNLGKRLStKIiahPyRLhRqQdqJIvrDNp3QjOd4
Lvjj8u6771vOd77lvuIjA539ITM7g1WLdVvK7G6sMl5iCbNdGbOLdWaLWbPL
WL1RsxVps8ebzcbDZr/gWx/OyivM+pvMKnjfVat3yp7H0zx8g53HcH7nStl7
FPu+Oukv15i9hPPpesgATMw3i/t6fyBp1tSic8rSLuXbimZdrWazq8wuQXYw
Z/Yhvk+s1tlm+N602uHDakMsF4AhCptLeeDMC+8MbHSlpdsAnQjWO/BRwPsr
wPgyVjPwTwWKa6bBLAyZT7H/EroP5mS/FJb8HLDdgPN3s5ILGvR9GucL4GwC
GIaAYQfi2pBQXDvB90+B9EueckD+D4GTrxwnLYjR8/U+Cz0PcoOQqcBPwRPO
VtjY0SqfeZz14L0TNkrwfyWGMzwP0DfO7qNf2NuQV7z7gX9mlXASL9+5T8FX
xZf8Gti87Ev/YKNkmAvaXnb2j0flk/sRYNmEPOWwvxV6B6B/ihwhvmHk9FxI
61r4P4syOYnvJ9J6Z41UoqqZgTj4zOn9efD3HFYsYnYEsvN56bIW1yDuBeie
TMsW7ZxDTPNh1c1DODsbKHfES9z7YPMf6M3VCxvz83NCOXqdNsDvaZyPhWWb
+TXYryKGUZx5iG89sO2HnWOBcsO8fAGZZxBjd6Nq8mOsX0PyS1nyMwr7d8D+
AuSnYpLpatRz3Ml/XzD7rqBapuyYJ/lt8LMV73ngfGGl6ppxsX5/q1dOo1HV
NPfM82RK+/eh915W378lXsS8Fzp/IEc/JlQj5I4YyG0/8libVvzsy66kepM4
GRt7jVyyzphr9t+Ew38/5NfB1ynIXwXsL/qq84cblXvaP4P8joHDOujNwtei
L1ucK71uttD3ecc/+aMse/QY4jqKtRG93J3XXKD/C+i1313d3hxRXZJz4piM
qL9ebRJ/3AfgOFnQGWPgPKDuTcC+HqsEnY+A82vgTMF+FbktBZpj9FdtEg98
zrn50wouD2Y0E0PgryatOjoBO8cDnf+Fsw9a1E+X4Gdnq+Ik98wBOUzVyyfr
kDVedlwMgc9kSnXL+cdZRty0kQ8ks4zvS1nNpw7guTEre0u++pOx3oOze7Pi
hfXB+tkT+n92kLfPgPPztOrzcEzxcT6QW+aB86cD3G/BKiC+6yG7O6/eY74K
zs4T8Hs6Jx/k74jjcEtec4E18d8cDMsmfZWdvxx4ejqpXr0uLR+0X3XzZ5+r
lXk3l0ouj4zxT1/zl31ddr1Gmxd9zUrGytr/wdX/0UC9sdfJEwdj35pSTbOO
OIP4nTOKub2SVX45F4iftUp/10Q0E3ZndKdytrWDg9uLwtztOCWfzC9nFuPg
LN+e0zwn1jk3n/nk2bSbveSbdji/aYvnrFdiZM08C96mga+3UbxvblB+e8Dl
oKd7hvdzjadYiGtdTjZZh5Mx9SlnFPfjbp6VHSfMGft5j3v2uj3vq86k7izm
p+pm3yewPRzXXcq+pTz1tmU119iT3cBSSWqm8X7bHqgeGCtjZk4Pg5+OQPFw
PpA71vwt8BvLqk7IKzkhzsVAvumXcUw5/K8h9hZ3n1KPdUbddvjv8fRvwvuZ
84u1yfp90heXo9AtZdQH/Ef4xv0nPOWrpsnNEOzNpNTLfM66Pe/PPvf/w/iK
7h5fC79tSc113j8jnvK4EXpjcd1Xb2c0XzhDhj3dr5wD7Vn1OXv8rqLqjHlk
nV4dEbalpHzQPvuS2Nmbb8DmmxnJMb9TLte8Y/9OqwYGsR9pFua3IHsoo/pe
m9A/Fu+hAdiuxoWBuawJ6876FxWEiZM=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlXto1nUUxs82tza31t7t3fbe9w6hK1ReMppBgV2s/KeVc44ECZJuOgpy
lhZEBVG6TUuDsiSayz8iWhcJo1QoKNSJ/4RZgSmRaUQF6faufPs8Pe8fX/h+
f+f2nHOec37d9w/2rq2OiI2cWZzxRMR1nRFH6iL+ro8odUUc41upNmJtc8Rj
qYiRZMTxGuss6I74OBtxAtuJloi+9ojRYkQZ3cmLsclErOE8j34RH/MLET/i
rxbZpqaIP7H9i3PfbPsYL9hGutLpJtauHJgaI57Dx3Q64ipiNGB7GHw3dEQM
gGn7RRGrSeQe7L8i/qJLIlbhYz34hzgXqiKONkTcCt5bOC9hXwTHDvT34HM1
vkt8O8T9G3xuJrf38D9E/Hbq8VmdY2xpjfgDvFtn2+e92H/a7lhp9H9A9yh+
z/G+Cf3byWUJ5zj2vxLj3zz5oX+GYo+D5zS5LSfG+8ia6mw7mDAW+bgR2wXY
jJH/98KI/xlOAdlS8u0hl2S3a1mm/huRbVD/qlzza7EdrvQrg/8vqc0L+P+2
1jWaj/zKNuNTjAF0W7Hfy/sZ8H9eMAbFztCPB8n/iqxreZL39diuQf918nkW
PMeQDfPtJ+6X8+2TrDnyPzeo2W3ESwtDs+vxluJTj3rwN3GW06s92PSjexj/
V9PbswnXZgN46oj/Ie8q4nfxbSGxHiX+yWrHf5z3R1nHko8y2C9wxtDfJRv6
W0i7Pr+Ar418X8uZ30M15uJ+MPQ0m5MriN9W6f824o+iO9PlWViHfhX263Pu
lTh0Z945KrcJ6nkQLm1Dfhn3YXxs5d6atK1i1hBvFmd3gzmh3lVnzQ1h3I7+
yzljk4/FaeesXG+Gb6eoxyvIL0X2Yp17eUfesdXTcWLdlXetztQ498i6FvJf
ppbvJtxb8eMA8v1pxxO+FeTanja3j6Azt2DOiWuaoUMdnhnNivIbAkskXQvh
SRXdM8VXz+eCZTRpPglfit70ZLxrxNFJ7v1d3h3iUBLd3zsrvpEfhCvnePfS
iwNN5sYoZ7LeHBF3S2m/xeGHiF3kPafeNR6QDB9jtfYxhf55zju1/tZHbU63
erb0TbvhKXJqaXROJ3hPpb17NLOv5swJ9VM9+If8pjgrwbaK8yb3h/H/M1ga
eX9NvZ5OuF7aMV/ga1/atZKPGfRLee86cXYRtV7c7V2lmS/Bx5mUZ0Hxles1
Hc5dOcu3ODdc2V/KXTOrWVUN3oYfp6jxpgbvsHXUd2Fl34vf2v3ivO76NoHu
zrxnUzM+zf183rOjHHdw76/sL+1kzfJ0pT6aacXSTGuWFXMLuvPy5po4ME28
TIfrvYyYI+B/I29b+egFy90p73bNoHaXaqzaaoftzXoHaR5WKh96Ueb9QI1n
8hH8f9DuWVEPRpB9l/Vu0s44i/y3TvdCGPaB9Yk2+9IOeDLnGVN/xIFmZPNS
3o3CJKzqsXorzPp3zEk6F/kcy5mz4qr+Yfo3aqY0S/pH9uFrWSU/8VX/GvVE
9Ve+S7T7k94nmiFxvaWyn9XvzcgGU66n5uc/nQ0m/w==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.696521548553614, 0.7407785066761803, 0.9125239081269159], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNVm1olWUYvre1ztGzs+18vr5n7zk7E/oUMo1GBbF0UYJW5Ma+GAoFffgx
PzBnQX8sEba1HaWSkkWbm0H+2HT9KNK00SqaSFGsAmFpsA5k2I/QaaldV9cj
9OPhfd7nvT+u+76v+37eume2rO0qNbPzWGVY3QvMrpWYjWDVV5kNVpq1lJt9
lzS7L2/2Ht5XZcwexzLIHoyYDWN9g9cZPHctlC71KNsK3bZaszOQ3wn5DyrM
VkbN3of8c3GzOQ/fID/qmx3ImRXCZoux1kFu7haz09CbxtoB3XfD+v4idAvA
sywr3+2LzO6HnRM4fwRndyXMXkZQd2MfYL+tVGf3ZHW+GLbCwHActh6C7UVY
57BfeatZJ/zuN/nvdBg2B7CBFYKvNsh1YJ1AXBPAfBCY3wlLPx4Vhn5guxe+
hiCfRM7uyCpPE5D9KCe5QxGdEdtwWDa6obsUsg8Cz194r0JMyxDbJ8B1BTms
gt3NyOkc3ouozSHkrRe43saqg63P8TznK67VsHMY8hMR1ZE5Z+6nY2Y/eMrh
sZTZk8DThliuQv5vrPWQaQ7pWUTsmyBbn5DPU/heiIsftPthRPl8KVAuGMN/
9S4XNmKkHvnDfas7f7Na7zyvBsaTadgA+S7D1zxWU0hnsaTO14SFaS/kV+MZ
AseGsZ8Hnn+yqhfrQb3dZbJNH/SXD6tmPKc+7ZBvrGG7q2MRMbXUyC7feU77
XcAwEBcvziJX63x9W4KalCAnnaWq+5Cr43bgjQL/VJn4TwyM8Qx0O6DbRJ+Q
uQS7Y5CZAjeakP8xl3Pmm9im8f1V1GkP4voDugX4+hX2+5Hn88DzWKm4SVzk
J2W/Tkv+rUD63F9Gbq5kJVuJp+94Tj9zzhf7I5oVfuavyeVqOc72JcXTHtgc
cBwbKlc9mMvXgP91rIuV4kijr3pvhexZ4FwBm6PUg5166D4Mmyuy4s1VyMY9
5eHTqPTIjT9r1W8h54t+iOcrxPQbbPZB7kiFfN6OZ2lC9WAtDkAvAV93Om4e
w5oxzQ7qDZeIU0nHt+ZAc4czhT13PKe+m4Td7yvEhV7UsK9as4tPvn+J3PW4
PW12L9Ts49xrgF4eslOQmQGn3gC+WWAbgd9ViH08ov5hPSnP+fd8XLW4Dgw3
MqrPtrTmI8+Jafxmry2QP+quB96NKXGcffmoL9v98PlTjeTi8Lsfcd4GfS+j
mcJ5shw5q0xoXw7ddF65Itd+rBE+5qzHxci+Y08wbzfjZ4xPw85arH34fh08
TOR1F3AW0Ce5yjp3JFXrhKd5zTlJ39RjvzyQVv9wT+40BeLPNOIbCcQFw/du
7Csj6pnxCmHgXGuMaq6TG+QFucT7ibkjHs77RnfvcD5uiIgzvDcOx4Rn0tWc
cXJe1rsZxTuRNplb8oA1YE4KiLUlpXwMcsZnZZ+86PkfT5hH4uT3gZhkGt08
5n5pWv3A/PCuWOLuC84jYiIe8ugVT1wiv8Zc7O21wkFfHwNLc055aXOzjb1D
XlxICT+5xfuV/U/uBL74wzm6J6ZZyhmzI1Dt2AecC7TF71/kJZPy5Zt+c9jX
+rLLXopGVLtLnurH2lVF1NM8/xZ2LtbqvjsaE//IvVbUtxioNrvwLElKry+h
XmLsozHxjDXlrE1lxJm9btbvdrOXtWN+WSvqsV70T+7QL+9RzjPOyVnXV+yp
G/C1M1AMY66XGe9pxFH0VAvO9UH3b2M1ip+x/4z9QEL/QNeAazIlXBs88YO1
3oLzrRlxlRwIO97yzjzieMtZyRnLvM3D9i/uTn8hpvubd/cTOd3f1ONddCoQ
n8dxttETb5+CTENKdtnPs27mPAssR1PCvwnP7WnxgbHyTmG8/C+YcPbJ3wbH
N/YC5zVxennNdM7ak8D4ma9/mV43L5lz9g11KM+7NOvLB2d0nZsPXTFxgnVh
7rpdLS4gjt89zZ5/AfFNhJ8=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlW9o1XUUxo+zuat3u267/3a3e3fvhEIKZi4I6UWmUgmzIrfmFFGoF5XN
qaFOfdkfghpOhZKKItdmkC9c6osiKx1ZYbqIwCUGJYENMuxF6SzMPk9PLw58
v79zznPO9zl/fm2Pb1rZXxURu5BbkCOFiMZMxK3JiNcTEd80R9xfjvi4LmJg
dkQmFzHZEPEitttmRswvRezBfhj7N5ETxYiTyHF81wL8LPZbkIkZEZcIcBa8
3Wmw5uAPXk82or0tYrA+4gv0C8F7OB9xOoz5GPpswbpTyFxivQx+G7qNqYir
YFelHevwTPsuJ0bMNsb+1oinyPdpznvI6UF0i5A/ErZ5sjHiBveXqp3jCNij
yN3gH+BbBrw08gnmp8DfSD7XyHFtbcQ6ZCe6M/hvBWsY/7PoB4j5HbpxZBm5
j/GtB6xLsyIOoZsumJ8EnA4RazfyQJXfdFO5t0TMQz/Fewvcj+LTiO1Fvr2G
7atF8y/7zyvkhRwm1j6+dcJHJ/kdRbcTzDs5L1MNOB9QjcDqh4+BcH33gZXN
Ob5qvivvN+ktP/LtAvYbuL+D/VJy6Mc+qEEK2xpkO/d6/JdgW8e9nfOCnGuh
mpR4a1/WtV/K+0e430zbVvUYwH8853yE2dNiDsVdDT1yjPh/Nph/cVAm9tW8
a/McMX7lPNVo286aiPXg/4D/arDXIGnsX/m/X+YjP6FfxbcNSddgHfcz2A9X
26fI/QL3Xs4VYRQcU7G6EtYphrBlM038iax1XcSfKpoDvV0x2vDtxn5Fwj11
hVn6vWzu9OaVTe4x9dZe/Ltb3fPq9TE4eCbvGoh79VAX5w+z1qnm38PXeWKc
S3qmThP7ct5caD6HGtxz6jXVcAv5bS66XspB3C7KOb44Vu9ey/st6mH1xv6i
uVOPTDR7hjW725G9Ge8Azf67yBruHbzpo1nugc3YrgDzPbDmYX8Q/b3Yv590
j6bohUeb/Hb5yLaDfn4rZR+dv8Xn7ZS/fZXzG/U2zYCwRtEfSRqzj3j9Ofda
hXgLwV6dcS7C1+66XvbukP9d4D8Bxqpqx1Avfka9hhOuz7GCe1a9+gbyAdw/
RI691fZpx3dByftE83ad83TJs6gdcRzb0QbvItlo1/WWzZ123hJs7yvZ9z9O
wH8En0O1jjGU9g7W7t0xx7O2g3rMTbqGOfLPV+x7jhiX8f8N+RndDWa4wtv/
An89eN017kV901k9OUWt6zPmUzOgXd+X967Szm/Svs54d2j/lAvegdp92lF/
g/1Ps3eVYnxa8D9Eu0UcqjfPtzh39ah27wjfxpLmYxLdYNq7RjlPllxD1W5x
rXdXR8m9pR12EexNze4lze8d6G4vWaceXExuz8PHlZQ5/KXRmMIa5E0voDvJ
/bZa2ywveWeKf+U0rvlu9ezrn9AF191F1198z8h4B2p2VANhnSiZa2Gmwc9U
nItq/jW5fplzbP0jtuK7rWh+NE91+KZK5lZvOEiv3NPsu+Yxyznf7N7VjP4L
enoT/w==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7161452220620179, 0.7723082896589535, 0.912971267441167], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVllsVVUU3ZQ+2r6p7Rv7Xt/tKxjDKERBVEhogUK0hRRRVETahCIOQVqk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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlnto1mUUx4+b07m929re7Z3v6/t754rwMi9FlKnkJTVKK8u0MkFBzaLU
ea1V9I8JhncIkijRFElNl8soyGtRWihRUKbgHzoqDZLU3EBS6/Pluz8OPM/v
Oed77uf8Gmc3T1lYFBHvQN2hjt4R7bmINb0iSkojZqUjtvaJGFMW8SC0IB9x
uSFiXmXEnOKIbrw3IdPSM+KjHhH39I34qTYiD9j35REZ7iU1EQXeViIzLBMx
Gf6N8NfwrSmJ6AedBmsz/HvBng3m5xjVxPsB9B2BJvK2CZ7DnGfD/125v31W
iNgPHSs23mHwL9RHPAL+SvzZxPkY39bwVsq3uWDvQ8dA7l+g43w24iswJ3Xh
jwZ7O/ZfKLbNc+BvbbDvsmkeviwGbzxv24jREmRvwjMX/lV8K0N+aa3t+YF7
J7zTub+J7p3cl8HfXmPeFDrHwz8xY12t3LcQ65ngjUbfKGgA5zHw7OHtDDyD
OK8B70yx7w11Ec9lHKszxHcl8X4L+pj490tFbEDXQt4nYfuNbhHv4u92aAO5
fRaMduJzvt5v48CYydtZMA+WREyBp43znMS53AleC7zzsekb8L6FLpPLocRn
a0XEG2CMhfeBxPGTzUWcU9Cqcsckj+zAxLlWfJTL96DuKec0S+1Vcn+7i/9O
9M/A/kHwFmHPOuRPEaNlxOYVaBvnD6F53VyjVcTmOvZcx/5lyB/En0NZx1v3
F4jHH/jwMPnYTv5qwa5GZmJPy4xE94TEubgI/7jE33TeqxrJ2Wf5+ijyd3Ou
Aq81nGPVWnPGsVTNJVnnQPFtxP4LnI/zfq7IOTmadw+pd1SDwqpHx7lSY/7J
PZ2xbbJxF77tbnBtXOT9a+JzM2dfvky5F4vSrjX15C3OS9HRUeIanQDvQznX
jnJ0APtaqt0rv1Mzj9NLn4K5G6xd0H3wDss5V+pRzYbz0OpenhEvwzurYN2S
n65a5tvpSvfYLbB3QFnqI8t9H/raoBU9PD/WE4/+5C+L7L/E6AS8L0Kd4F3B
5tFgTQZ/T8o1XZG4hmSPamQTbzvAm15smfGcN4J5CX1jieFLyF9LnO/fkB9E
7NqJ0avo2gzPAN5W19o32beW8+DEZ/XY/jrHRLGQDc9wfrpgrLaUdR1N3GvS
OQT8wdCWCvfEEvDuz3g2aeYpdlPBHJVyDE8hu67GvuubZmEt/bui0jZVIt/B
txkp+9CZOGeKtb6pd0Yiv7vCPXQU2SPQtBJ/G4gtTQ329QNoRI1nomahekq9
Wt/Xs1n10Eyt3Eg7N6oZxXYq3zrLHOOp2HMt43pVfe3nfXnes0z1VZ54xstf
9fxBYjU851iqBt4vWEa817F3Cbzz69wbmqnpvq4ZxV8x2Igtl3j/BVtew4Yn
8p5hml3/cM+AnerqP/XM3/j6Y7Vt145q4/582rNeM1Wz7jg6RlW5Jxch35zz
7NfOkuzCatsmjCvITsm7FuV/mfZX3rr1rZXaXZD2LFQNP8Xb2RrXt2L2V51r
UrUoH37tY5/ky3LoBLpPQld7eCYchn9awbUlGzVrr/b27NPMVW8+VvAsVo9+
wv2uRvb3bc75k9rPadsqG7Rr1vPtjnLPQO3iNvz7r9Q7Zx1vrVnPxtvh2cB9
dd53yQj73gb/C0jHz/XuUfk+oqdn09q8ZTWj1FsjwRte6h5TL2hHaTepJzoy
rqGWrn2oWaEaVW1qZhwCa0St86Eau4i+k3XeRdrpmt3awdq96od81jNI/aCd
1R/9A3obWzWhXbE27VhrZyzO+x9G9d0Lm4ck3rGah1vLXatDE59Vs8rlorx5
Fc/X672DtHuUs/8BykpUFA==
              "],
              VertexColors->None], 
             PolygonBox[{{5902, 6678, 2272, 6677, 5901}, {5824, 6122, 1866, 
              5137, 5823}, {6224, 5902, 5901, 5234, 1936}, {6119, 5824, 5823, 
              5136, 1865}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.7357688955704219, 0.8038380726417267, 0.9134186267554181], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1mHuQ1WUZx9+9we6e5Sy7Z/fssmfPnkOJXHchJsRYGtOARCqJy8pNnEEX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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlnuQzWUYx989dtk9Zx12z9n7Oc5psllyKSFRI+UuEaWsaKasSy5blPzR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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7553925690788258, 0.8353678556244998, 0.9138659860696692], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1mHmQVNUVxp+z79Mz07N1T890K8NgFKNYpWKwooaCUYwRsFziUm7MgDua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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3ts1eUZx196Wno5BXouPW3PpT0FSmGKCEYdDrPpOkFgi5Z5C+gc2lJ0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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7737537884429666, 0.8507071535685359, 0.9090480007450445], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHuQ1WUZx19hYfecs3vYPXfO2bOXJNCkKSVYSAUUA1SqgcrJNFEGVBbR
gEjJQHG5CqgVUDghIDjTYJZZfwgo0IwzmdZUal5DYMALF2EduYPW99P3nemP
3/x+570873P5Pt/neU/7pLsm3NkjhPDWeSH01PtAXQgf6nlB32cbQzinp5QI
oSsXwkXlEP6pRelCCJ3NIYzQxl/XhNCq+ZLeUzR+q56O3iF8T/OnsyHcrPXL
Ja9HyutTDSEkNf+U5A/R+12ND9Xet/X+in5XNX5G+6Y0e88FtZ6/UHJO6/wz
evbWWDf0GtHb85y59zyvH5zwWFrfZxq95pJePo81DXH8M+kzOxPCj6o++9Ok
xpLWc3ZTCLvaQugrWd+VPu3tIUyX7mvrPHan9DmJfc3eU4q+QvZG7T2n58/Y
mPS+lM7NSPcTfUN4WTIOy5+jW0L4QOtHyGcj9RSlw2jNP5cPYZTWvFEK4U09
B7XmJtlzRO8vSc57jT6Pswb18vy1mt+t7w9Tjt1S6TVBsq4hlpL7d+nbKbuG
a+1jvRyXYymf+aLWX6Tz3292bHrKH0P0e7LWX9caQlZySpKzoGydOG+x1iTy
1mmXxv6tp1vfXZLdqrV34R/Nn8pbrz/VWMYOve+t2n5sR8bSquWMqvEYvrqk
GEIfzd1S4/NzUQfOybT6rHrp2KDnk16O59kY68lpx3J2MPaO1Ydwm77H68xd
krNZuj2vM4dLfj/NrVQcVrV43cY+1hU9x1ZCaNazU/G+Xb55rmCZSem2pskY
5swzMRY3aGynnms0fkOMB7FYKBnXCCsXJh2fAzG/wDEYfFn2npbOp9I+l5iN
ir78VGNbhJ+vSrcFGu/W+/pgm7ETfRg71Mvjg4XTVRXj7nXh9w09E2RTh3Qc
Vm/biA9jW2NOkCvkxRjZmCB/pdND8s8KPZ+kjMch7cYkHDAw8sBPNH9cPr1a
++eA66zz9LfywTLN7ZX8qSnjHSxt0ZpEzrmzWGct0bNWa4ZJ32l69sR8JsdG
9XA+nY2+JW7HYuzwcVuz/TxOsXi6yRh+VTL+pWembP+2MPKdVts7Cb6ocz5+
Vu/85YyfSbduvb8s2+/T9y5w28O5gX/xLfu6496JNbapXms+rvP6bRp/Rzq8
XXRu/UU63iLbN9cZL3c3Wc7n9b1bc6sl+0bJ2SM5Kck5rn0t8sl47Z2lta/q
mR+cY++nnNfE7QnFa6b0HlhwrpKn/N7Uxxw8T++hsjsjW/YrXl0Vx7RT77UV
28ha5KzXd0FrL0uaT1ZrvlXx7dDvlRVjCPyMlPxBSdv7Tsk8BV+8oDhn5f+t
mlum71e0Z4a+ZyT8JnaHNT5Kaw7pd6e+N2vN1/U9sc34wI+P9/HYOukwv2q+
2B3HieE4PW8qL5+s2CdDhIVL62N8JfOnFes3qdlcQMxq9O5ZcD5swNYo6xda
+8uKfTSnj+2DZ+DmjviNP0Yk7JMrErYfv91c42/8cFBrupPmkwnyxeUJx/OI
9Lm22XMPKD+6yubmU4rtfbLtPcXy46T3k1+cMzyeBRbgd/BAfDbFWPeXzNsr
jhX5uzHGblbC33AF+QlGwfj3pc82rS1IzmjFK5c15+NjfIo/B5bMX3DXDOHx
G4rFVO1fL7zd22DeIj+JJXF8tsHfxKisWtnY6nXEApmMw1sLGy0/n7S+2FWM
v/Hbdu29SrZ0R92xAVvA/aPS+1s9zAVwH7wH787NmHvh6Jmxx5im+c469wnk
BX5BDrhAJ/R5STi6XraNr7O/D0Wf4xv24h90Zz3YeFt4eadgPa6s9T7q1ynF
8KSetTp7kd6PVByzd6XXmrL9s7lg3oeLVte53oAN+AU/sgb/gMMN0u1w0mvo
McDhPXBz9Am2wFfPSOaUJvP5YzrnrznbfreMvqfqnmlX0vUAu8j9ueS/5IyX
jg9mzKPYSo7iK86gB4ITntD8oHbz7ZkGx+RowvyWj7HbIow8XDCmqMvM7zvP
7+74DS/uiTmyWGsXFZwT+6TvXOndUePcYg/5Rf0bGvcOkPwLy+5N1kjnMRVz
1zjxw22SM7S3Y7M+4jYt3B7LGCf/82X0JzxCfoKxFSnLA/N7tHaCZPZIOla7
M47FLOGi2GzfTCq4/6P3o289UHIfCY+zjz4JPoaD4J+c5oNyapPWvCanThDO
LgODsrdJc1NqnQeLGr2+ver6RG0iCHfmXGfQl9qD316pN+a29bIs6jy4oQfH
ns7ICWAdnJOLrAHn9EDkPT3S/cLYz1M+C7/TB1OLySV6CXqM4ynvYf3+lDkJ
jMGV++M3Na2xwXWN+rM/1iDiemPkKDAyMfqc3uTSqA89DLaD9y8UfWegT3ha
/vt9i+WeyLlvoGfAxmWxX4Wr8lXbiy/oTfA/Nq1MuS7Dd8gm7nPoGXPG5s56
+5F9GcUhq+fWWuuyMOqDHnAbZ53UvicLtgssMQ/GGhTPB6v/53vexIh7xMje
sVdLOtfAKnxBzMEkXI88eKAsGZ+rWnd8ciZtPMDD8Cm++qbOWt7muTHyzVg9
j8uGs2nvwdaLhdutLc4z8ohcJZeOZ5wPnLsj4X2cj72n41kDld9f1LNUY03i
7OVlYwWOBRPwG7zGN9jgrjO57N7oxaI5CDws0djisusVfRz3N2rHNOXQxGb3
2cRqR73jtUxrHyy7ftIX4zv8dqXs2F517OiJtsRe+g869xk958unyzW/KOfc
oJdcUzCGz8X7Dve96Zr/j86dpzVzU37TW2azrnngp13zZeX+dq1fENexZofk
v1R27XlaPny26FpAX/Bo7A1mix9+2Nd1prvgeom9tdo7IxflpS2THBmns45m
XdvZPy/y8M6IR3yyUWetLDl3Ltb6fVnzd4fi09Tu+9yHJfdt8DO1GKyCWeoD
dQL+7NSaaXrW1fp+Ob3kvrQj3o/hec4Gn+QX+Yos5GR1TqbdfvyAu2HOvTl1
lf6R+vJS1n08spAxL+Kc+8LWiJmnNP+bvq51jG2L49QZ7vDoME8+zsCrCfdZ
c2KN4/7QEPFADMA4eBicdm6Qgydl0+V563SV9Pwo65o5Me1aTm2dLNlvFY1b
7o1wETULDA2Jd32wQq9PHV4IrrhzpuxP7pisoQYOi/cO8ogcIn/htKWx14Q7
r6v3Xe2oZHRX/XtVyTUPXlyisVTeesDNM+L/A/xXsb7ova/JjrlV8wd5MzXm
zjLZckfB/FKK9xJq98MV4wbMwEnpvPmImrywxXWZGFMnqafoyTn4ZGis2Yy/
mPddi7x/rOL8JDc580TZ5+KPeyLGHlAuztdzhJ5O83V5Y/Ei+erVqn3CXGvE
TUFj/crmcu43Rwu+43wgzjmUddyo5z+uWi/uLdzhyCn69NdLrj/HtfdwwbW1
X9UcCn9uk5ytrcYIGEJPYgeG0B1709L3jpzzEuw8V7Vup7L+H4r73AKNJfPu
s+m3wQLfbT3tS3hgnfy6vsW4/lvOvRd5wX82NVXz3/aIFXDCnen5jLl5qnw7
IOuel56VdeROodlcg/xb4l2UnO2SvAf0HEyZh34X1+C/YTn7kLsWdZ2afoXG
z6+6Di7RfKXqurhbe/cU3UPQI5xLu4ckLxkjN98tGgdgAM7fUm/enyJbb20x
d+N3aibn0vvsjHzOfy5nYz0aJH9cUvbdF34Ay2BwgMb6l/2/Bv3xTTEf4aKr
Iw4vkA5/bHN/dp/0z1Vd+7GfeMBV3KXgqxtjf3Io1lR0m11r3LxWcS2BN/oL
OwNKnqvPOw/JwZt1TqN+/yPtOLwc78Vglns5MeW/L/ohYkLfxf8s5C57jsU+
Cv8fi2tWCgNjm91zk3+PxN4YjqPX6oi5vyXWd2ylx8Ve+kvm4UZ4b2uDe7kN
LY4B/uc/y474X8z6vj6PsxbI3vvLxj58f3HR+KFPWx37ohU666E2x/4H8m1v
HdSluV/JB19rcb4Q89Z4t/oo5/4Jjhil3Lqq1TpR07piXVuqNS1Vn0E8iStc
RP/BPrjivy3mDEc=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3lw1GcZx19CCEk2CcnuZrNHNkfLWelMgZbQsXIFC6U4mlg7drRcAi3Q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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7913771791331807, 0.8565840847718479, 0.901152737508724], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1mH2Q1VUZxw/sy2X3clm49+7d3bu/e3dRayZDiHJMKMjUYipFIXAsDVKQ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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlnuQFXIUx3/tau+2t23tvXvv3b3de9vU+EOJvEMxIuNRlDIe5dHLI4op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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8090005698233947, 0.8624610159751599, 0.8932574742724037], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1mHuQlWUdx5/dhbNnz97P2T1nz9mX3dPQ4Ax3MxdJoNtg2SQo5JAhNtPC
GAVaLGHDViZoMzq7NozakKSGkAM12Y3hMsMIgrMsRBoamwVikNaAXGTBUNLq
++H79Mc773Oe9/f8Lt/f9Tkf+srdc+6qDCH0VoRQpffhphDeSIXwfW2+nwnh
kp7pWp9Nh/CcCHaI5rf63l8dwu4qr3fH/cUtIbybmI5z/9bTo/2d2RD+1BZC
WXym5UP4q9ZLtB5bCmGVzvxR9H3i1yheu7SeLx6367ms89eKfpbou0V/b8p7
C0RzMad3YhmTCiHcLz4va394pPVG54q6EP5bG8LDI0JY1BrCI+UQjml9c00I
syWvT+ujoj2lMzfobLdoTo4K4Yj2V9Rar23S/74W68rv7nbZ2hFCk849oP02
0a+pNT68l0ruDOl8V5vXZ7Q/mLJu6Dg97/WEgvnC81+yo6JgWY2S8/UWY1oU
7Sit79X6oOT9Qc+Nonkx499jxP96nT0lLOaK5u1m84IP+t9Ta5yRsSKup0Re
8HlLup/W8wXhtFc+eqFgnh8Ih1dypgWbycJqptabZfexYgiPav1n0b9UMM6v
YodwuUq43SdeyyTrGfE/Wum9Y1pvEAaLtL8JPsJkptZz9P3HDYqJ+hAK8smw
eD6deP+waL4pmvWiP6G9vyc+f0TvmyVvs/avr3ecEWPETm+t4+cycZH4N/H4
tRbHZE+tdYPnIu2dTywDWXfrvVV8Tsn2xbJ9V6V5/S7l2EM/9vYrT/boPSDZ
W7TuF/1VwmWV7D4S97EXOewhi719kX6oyTl2oFF66uz3hOfV5Iu+na80zZfl
x0e0v1p6/VTxsLNTvhBOC3Tudj1dWvfL7r6MzywUn/ai4/Y22XQh5xy5U+cX
6pEaYZtonlf8z5c/d+l7o+i26vwg2Kct96t6b9OePoWDolkpnmPF5yad3Vs2
Hfw41679KjHuyDi/dou+KfF5cPogYrVetowpGNtNkjW3xvTXyKanZFv/CMc3
7x8Gxxt2k48VaefrgQrn+tlRzve5Ovs3nR2WjNXS8f6iY/XWGteuvZWOjy3R
d9SP2pzX7ypOP6LzcyVnqnC8rsH+BFvW8Dwo3/y+0evXRhhXdEjJjjcSx0S1
1umCZeCHKZEP8rETG/dJx4dyzgPqCn6C12Np1zJ8NKTYOdXpWF4u3b7VbkxP
y5bX6+yLC5Lzl7zxuSieE+SLx/V9jexep2dOleVxjrqdku4zUsYOfx2rcwwQ
Y69FW3ok59qc6VdJh9Udpqtvcx6SgwPi/YT2N6V9Bt3x6Trp8EzOPl4pOVOq
rcMa6diSWOexWh/XokbfG6TrWT3jtH9GOCxvds5ix8RoCzWSHKVmLoh1nNpy
QPvn0u4v4I5f8SM8wQhZa8XvV9KzURiulW6PlxyD/bGnUEt58531i6J/ssMy
6oXZ2BrH566Y5+hGfRqRMg4zda450lXLro8RiyOtL3Ufnakx1+S9d0L8f97h
89Q2fAvPYWH2VvQ1NiMLPA8RSzrbVel9aiLfztTYTmy8RTx3NhvD4w3ubeTS
ONm4OOUagpPpbfS12RnnA7lAnX41azznJa7drHekXYuJq9acext9jX6BvfQR
bGYPfPA3eUMsUC/r4z415dBIxxc20x/ADZ9TP9D52ynnNDY+K1snJt4jbniz
30u85X0GfC/Uua+dzDhfWC9PuRfRv4hv4hy56+Xzp0vGBOymRcz3ydZN4tkj
uhOycUJiPeil8ILPQupth+VRR5am3AfPSp9zel5IW2ZPpB8Uz/GJz3++zTNK
d+x19ADqP/28rmwMh9LuMWCFHf/M2JZp4jE98e/z0uEzbY7zRr2b9CyUbw+1
u7/CG7/jY/xLP+U7WDPjVMT9wbS/0WcPZR1bXdFvFTGP72lwXkJD3L0caS5k
nf/4jpyHLzyH9f2jnZ6ZwIaYI97oaYOxr11q8G/s3yOcs7JrR8Y9ZUfsFx9W
/btYcK0nZvdmPBMiqy/2gSuxW+31aNH/qGy9XxIOh0aZP/WG+CB25se4Qwd8
Dw7UwtktnhWoF8QIfRI6eg56oMN35dsH5fdlaeOxP236jdr7Gfkre26oNZ7g
XBJ9d9Y5S/8+Gn1NX0QPdLijwT2MeKG3b8wbK+aB3gbPRd+IM8f6mGfUF+Lk
k53uGfCdp3hYl3fukFvMaNRuauHZWA+J6x8UHZfQ3hLrA/LAF2yxaUusm/iz
KuIDT/iByUDKcypz6ZfEY1ez47xOutSXPAet0PudxPVguzAoJc7vJyX3qbzl
MT/+p9W5w7z/Xqt1oMaAJXWmt849iXigD18dezEzEnMNeNJ7mXPov9fFuYdv
POxD01ByP6YnrGg3duDG9wXx7HNlxxDxky16jsFX8KfHUxvPieZ0u3v4Hq2X
NRubB3OOOeLtVtn6xcSz5jsl92DqNLLghT5DcQ3P8dJrXMG4btV7p3iVFbMj
tc7kfGeaLH6TEt8/Rus9lPNcz4y2tdXxtEP72xPHIP2qNu+e9Yr4/Cbv3ntR
3+9s8Wz7hPTqyroHgOuzrcZ2Zayz5PgDwmFD3nm8Oe9aRh37hPh8PHGt/UnR
NZq6jJxfR1kbxX9q1n3lc/LvjW2+t72tcwtbHNf4a3+cnV4X/eWyf/9D+rzZ
6fy4TfvzSu5tJ3OeY9CPO8fqOMtx5+iJtZ75iN5J3+wUDt9pdRw9Jt6Plj3v
HIuxTH3bIH0Gi447ZrSb2pwD1AZqKDVmlvQdLroGMrORE8xCA/Guxl2pVXw+
3e4+TP3YE+dz7gdr44wxRvr0tfre82biuYdawtzU2+zaVk2sltwHfiGZS5o9
A85IPE9Qy4hdago6UP/+PxcTUwdijJ3u9OxCDBxl3i+6h3NH3FY0/SThMb7s
GeKC9vqa3RNCtJ0aWQ1mZfeB0XnPK/Tth7Q3q9W1nF4PltS696VnMedZDF24
v6BnvegvJb5bbo+1nrymf1NTqUXcg9tynpGwZSjefQqq5fmc6x99AHpqxUT5
673E8+DSONdQT8h9agQ+mJJ13pKzzaJ9Pmf5n21zjpFfd8hvI3OeUeCNTvBH
d3QgT/HplbyvdI/oj/cp7j298e6D/7dlHQPMjmBA7T0u3uMSx+dWff9Uu+dR
Yulw5M+dYGq8F4Df5ILP0ycHYq9kxge3K3eujGMfe/mPpJBzHHKPJy6XxHrY
FXn+UjI3lDzrEbNXMNP+/wDTq5aL
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3ds1VUYho/QPSj33vbe3kG9aoKJTINSkKFRcStKVUAciYCAoCDFEdGo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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8266239605136088, 0.8683379471784719, 0.8853622110360831], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHuQ1WUZx9+9nb0ddvfs2T27Z/d3zi5OlpYCAatIa3/ENN64LDgMWQzG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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlmtslFUQhg/dXrbdXtjddtvd7rbFCApSaEQIIJioMSBSKBisKOHSclFE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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8442473512038228, 0.8742148783817839, 0.8774669477997626], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1mH2Q1WUVx5+F3XvvvsLu3r3L3b177yJpSVYW0oTNBPiugFrpAk5OC1Yo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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlnuQ1WMYx9/a3bPb2e2kc7Zz9uz+9nfOFlKuUSFGqIRKrhXDaAuxKYQR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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8618707418940369, 0.8800918095850959, 0.8695716845634421], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmQ2QlVUZx9/9ZPfu3rvLcnfZu3v33gvy4QrILmHjZJkoaqggpSgmjAEq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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3mM1dUVx+/MmxmGN/PeMMN7M++9efN+vwGVKoWBKTRGKiJSKaCIgogR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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8794941325842509, 0.8859687407884079, 0.8616764213271216], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmguUlWW5x7+Z2cPMMMzMnj37MjN7mL1JRQxC8d4CBFPskJ2VDQwqZiGK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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl2tw1OUVxt9kN4QkJiGbbLKbDZuNBXSgVRCEOoBAEVtkOtqkOB2rQgIh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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8971175232744649, 0.8918456719917199, 0.8537811580908012], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmguQlNWVx7+emWYemenp7pme6Zke6EYeEtG4ImZ3QUHAB48AM/jCBwyD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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3uQ1WUZx9+9srvsnmV/Z8+ec/bsnt9ZLhuERKhNtSAKpLIQy7KGLiiw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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.9147409139646789, 0.8977226031950319, 0.8458858948544807], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmWmQVdURx+8sDxiEuW/evMesvDc4YLQSBSWWoCARURYj7hFBNMu3hM0I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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwllmds1WUUxt8W2nqh897etreX3v8tm8QQRowCCohhCghIBIpFxC/GwUig
RsQw4oA6EoY46gpThokMEWqggBSioH4ikKhRlhAHCgkjgOP38Hw4ue+573nP
Ps/5186YNWFmbghhDdQW6psK4YuqEL4r4HxHCN8mQmipDKEH/J1tkOkQQmlt
CAWlIbSPhbAxCqEiHkL3/BA6F4cwuyyEj/ivA+dn0Zfh7nw2hJdKQhiQE8Kj
6LsOfxC+CcMvJ0P4EX46/L048DW2uvHmzaIQ+mDzBPI/o2+KdPPfUuTbYf8c
8st5/xm+buJ9Cv4w8iW8XY98R+S74tMGzoug9bn+r4X3kzMh7CgM4XMoifw5
3i/gfSH+beb8CZTMs87N+DOWnAwnF/P4byR3I6AhxDYXfzPo7oyORnxbBj1c
jS70n0R3O/LzDveTiKETtsugLbzdBb+5XQjV6B+jfMM35NnGYM73QwPxpQs2
/kHXv9D3xHYf9naXh7CNmI/AX6UejdzVk/OByF7A5mnsnYJ6EO/vhY5la9a2
FFMW+bXwcfgi+Evou1xu3YPRMYa7sdAsYpmJD9uJLYW+Z7hPobOuhtpyv4xY
9rYPIUE+86nHT+h7Cply+AvcvwLfG74pjQ+Re2ML79dkXWPVVj7sxfep5Kw0
5hiUm/68ORVzjv7A3z8zrm1P9H2K/q7oW41/fXm/B9+3V9nWNfKRyfo/nRux
+UvkHCk36qFW7E2HjhF7ecy9sYMYq3PdI6rd4xXozncNa9CXzjrXivkA9weh
nrKF/H7OLVAH/KkRqfcj98ZS+HWc10bO3TroK2Sby+1bZ+5/w/dOccveIMbJ
FY5RsWmmjnNXSz4O4+staDm6VkFFvF+qepC7FmQuxZzDY+iPIvfiNnQMI59P
orMfulYgPxfZGfRcPb22G/lB3M+ttG/LoAbuZ/P+1yLHtAQ+wv6hmH0YxnkK
b/LojcvwKfj9cZ/3QhvwvTt8Hffv5drWhGrPgmxOJPet+NhF8029FyH/Ijo6
Uus3kH8b20MSnpVSaDX8isixFsMvQt9CaB66GqDnsPVW5LtmbKzC1lZ0rqd2
o+BXRo5ZsSpnDyRsQ7rj0HxsPw+dzrF96X4h7lhkQ7mQDelWTkYhewy+F7JH
6aFWcncY2pnnmbtKrzQi8yW5b4YeUe7ReQ+6ulCD01nXSLWZA9VxngI9nWNM
yvJ2PD72JB+tMc9SFDd2aqZeSxtjha2S+ZteGo/+kei/QP+M5u35pH2rR+Zo
3DOo2ZPPwuI1SddamCwsnsqbmhxjcgHx5kfG2jnQ0LRnXrOuHnuiwhglbLqL
eG4ljSHCjjN59uVm0mf5dCVrn+TLnhL34r4yx66eFParJ9QL2gE/ZL1ztGvq
4b8ht0crjVXjkDlbZcwR1lwpMHb/hf0BbY3hH+B7U+TdUlXsWX8/8lkz/7H2
ATm5SLw1xcauOPr4uY1hsVr3pHrhTImxXTOq2RTG50XOkXIjjJyG/tczxlrt
hG7c9YkbO4VRu6u8U7VLheHabcvp0eEx77gGYhua8C4Tpqh3HhRGF7qHeqm/
q93L6hHt3mZsEvLtHaxd+GrSu0s7cQS6Fqc8G121Y/AnHTlXwvAzSWOWsOpI
nmf5GjlsKfFMn5QudF4scU+PSbiGqp1mWrJ1CWOJ3kj2RNK10ZuFcc+Ael85
vo6tDDJD8bU/9s/i+KCsbfeDP5QxJgoLhQk7eb84cm8Jk9Wbo9PGPvXoLvJ5
AB3/tXHNb/L2RsbfDvpm0d3xKs+CZLR7l6SdC+3g3tpV5d4lTfj0IbbGJdwL
aWhB2pig+guT7s76jWS1E0ZljHHCNtVI2Lg47lkVRs7n/G5kbBGmXEP+asax
amfvTxoThAXa+Zsq/Y2hbwvVtH/K3yj6NhFmKHd74t41yuGEMs+IZkM9uzDp
nlWv6ptB2Depwt8+wsAV9M406EbMPfdQwjErVn2j7cN2edy10TfeRs4Tq/1t
px2st49Ve1dKx//JPGB+
              "],
              VertexColors->None]}]}, 
          {RGBColor[0.941176, 0.906538, 0.834043], EdgeForm[None], 
           GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1lM1vTVEUxffrF63eqtd301eS+97rmMRXDJqIiffmGv+AFpNGTfAPIMSf
gQkjMUANSH0knXRoxJAw0YeBtEpj/axtcHL3OWd/rL32OrezcHl+eSAirmoN
ap2ZifjWiOgORWyMRCzpe0cO79sR8/siDslpsYg4rzVQi3g4EdGTTyF7z6R9
N5RnQfcXCp+/a0asKuf6rojPylPVI04OR3QU+1U1ZuVfk19X64jsvnw/ye+0
8r6uIt5obQ46Z5E+7QnHE3t0NGJNufvhe7CTb7t0P+xHhW1s0v5gBOuO7F/K
vV25n17NNcj/YMJ9ctYu7L+RsXM1263COMh5Q/b1wncnRo29lrH0dTiMr5f4
LyUmYi8mV/DJdyFtuIEnfI6pj2dNn5P7oPo9Lk7eCvtZzWW3/B7rfqVhLqrC
+UP+V6YjTk05X0Pcv5h27/QQiQdfMLHfUuw5rU4ubPxXFFt1vEcHcME5OOiR
fqkbyc9i9tLL2XJGrZ/CtzzkendLa4I+4e/RXvPKzNALOdHOk6b36IB8cEkO
Zo/9Sjysas0W5mZ82LjYb2UfxDMbclKjynlNCcPHprVOfWbKPVxWbfNJH4s5
F+7QEvEtcfKlbSzUYN7Mmvr9EdtdcX9t2vvnDWuaPnb0/VN5/uhrPDUJV2An
dn3Kbw++eWfdfGtjmsPt0nW4460yj7XkCE7gFK7hGY18aFsnYB9N/D+E53vD
ublD9/A7l7NkZsyllVzxLsu6NUi+m6XjqMmX2Ful8YENPfOPADM8NUpz9XLa
vYGdXvGlX/KyZ0bgpC76n2lJ33VjgadOapIe6Q8ftLlZWZ+zOfN/s5g0v8xm
pON5M+t66f1Szm48feCRM/qFm99lcp/v+f+7oW4/7X5q+2nTbxWdHBDm/S3/
G4p8H/jc09n9lnVGjuXM8xdtnpqN
              "],
              VertexColors->None], 
             PolygonBox[{{4786, 8436, 4016, 3459}, {8100, 4950, 3617, 3616}, {
              6373, 4639, 1134, 2038}, {6783, 5359, 1092, 2358}, {4629, 6081, 
              1836, 1092}, {5512, 4933, 1775, 1292}, {4434, 6005, 1774, 
              813}, {5350, 5362, 1094, 3754}, {7933, 7932, 3429, 3430}, {5628,
               5627, 1093, 1094}, {6326, 6783, 2358, 1997}, {7962, 7961, 3459,
               3460}, {6081, 5511, 1292, 1836}, {8436, 8099, 3616, 4016}, {
              6372, 6371, 314, 1131}, {6780, 6322, 1992, 2354}, {6006, 4435, 
              48, 1775}, {7928, 4628, 254, 3425}, {8066, 4793, 3425, 3577}, {
              6078, 6079, 813, 254}, {6371, 7963, 3460, 314}, {7873, 8066, 
              3577, 3362}, {5509, 5349, 3753, 1288}, {6328, 7934, 3430, 
              258}, {4950, 6004, 1773, 3617}, {4927, 5538, 1341, 1773}, {6005,
               5394, 1131, 1774}, {7931, 6327, 258, 3428}, {4435, 7874, 3362, 
              48}, {5396, 6780, 2354, 1134}, {5360, 6357, 1997, 1093}, {7876, 
              7877, 1288, 1016}, {7929, 7930, 3428, 3427}, {4638, 6373, 2038, 
              1341}, {5349, 7929, 3427, 3753}, {8222, 5306, 1016, 1992}, {
              7932, 5350, 3754, 3429}},
              VertexColors->None]}]}}, {{}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{6326, 5313, 6357, 5360, 6481, 5627, 5628, 5361, 5362, 
              5350, 7932, 7933, 7934, 6328, 6451, 6327, 7931, 7930, 7929, 
              5349, 5509, 8412, 4436, 7877, 7876, 7875, 5306, 8222, 5305, 
              6322, 6780, 5396, 8424, 4639, 6373, 4638, 8423, 5538, 4927, 
              6004, 4950, 8100, 8099, 8436, 4786, 7961, 7962, 7963, 6371, 
              6372, 5395, 8454, 5394, 6005, 4434, 8411, 5069, 6079, 6078, 
              4628, 7928, 8478, 4793, 8066, 7873, 7874, 4435, 6006, 4933, 
              5512, 8458, 5511, 6081, 4629, 8421, 5359, 6783, 6326}],
             RowBox[{"-", "5"}]],
            Annotation[#, -5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4775, 6740, 7172, 8464, 4538, 7173, 5769, 6057, 6058, 
              6478, 6477, 5047, 8448, 5568, 5718, 6457, 6458, 5567, 7768, 
              7770, 7769, 6444, 6443, 4656, 6805, 6804, 5548, 5719, 4623, 
              8101, 8102, 8064, 4787, 8063, 4788, 8065, 4728, 5731, 4928, 
              4929, 4640, 6445, 5503, 8457, 5504, 7037, 7035, 7036, 7034, 
              7033, 6524, 4730, 8089, 4931, 8087, 4930, 8088, 4951, 7042, 
              4936, 6529, 6528, 6527, 4731, 6937, 4800, 6982, 4953, 8439, 
              4954, 4935, 8090, 4934, 5736, 4542, 8418, 7884, 7967, 7966, 
              4644, 6378, 4643, 7965, 7964, 5398, 8265, 8264, 4446, 7820, 
              7819, 8162, 8161, 5631, 6085, 6086, 5590, 5592, 8460, 5591, 
              6090, 6091, 5513, 7712, 8355, 8354, 6089, 4647, 6801, 6802, 
              6750, 6751, 4549, 4448, 8092, 8093, 8091, 4942, 4956, 8440, 
              4955, 6988, 4821, 6945, 4738, 6943, 4737, 6944, 4945, 7051, 
              4464, 8095, 4943, 8094, 4944, 8096, 4736, 8050, 4735, 8438, 
              7050, 7755, 7753, 7754, 5542, 4648, 6461, 4649, 4941, 8437, 
              4940, 5607, 5606, 6986, 4812, 8071, 4811, 7067, 8104, 8103, 
              4630, 5720, 5549, 7974, 7975, 4658, 6465, 4657, 8269, 8270, 
              7941, 7943, 7942, 8483, 5079, 8159, 8160, 7818, 4445, 5797, 
              4440, 4541, 6747, 6746, 4799, 8514, 6981, 7514, 8220, 8221, 
              8219, 5304, 6321, 7927, 7926, 7925, 5348, 8255, 5602, 5603, 
              7872, 7871, 8475, 4726, 8048, 4727, 8085, 8086, 8082, 4926, 
              8081, 8084, 8083, 4724, 6498, 6738, 6739, 7804, 5729, 4723, 
              8247, 8497, 5341, 7915, 7916, 7917, 6320, 8361, 8362, 8360, 
              8364, 8363, 5046, 6970, 4775}],
             RowBox[{"-", "7.5`"}]],
            Annotation[#, -7.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4432, 8410, 5502, 6075, 6074, 7960, 7959, 7958, 8292, 
              8291, 7920, 7919, 4925, 8080, 4721, 6055, 4722, 5728, 4774, 
              5727, 4720, 6366, 5600, 8079, 4924, 7913, 7914, 8289, 8290, 
              7955, 7956, 7957, 6368, 6369, 5536, 8375, 5535}],
             RowBox[{"-", "10"}]],
            Annotation[#, -10, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4441, 5792, 8158, 7195, 7196, 8069, 8068, 4804, 6985, 
              5080, 7204, 8480, 4809, 8070, 4810, 7219, 7220, 6552, 7826, 
              6987, 4820, 6752, 6753, 6754, 7832, 7833, 6101, 6102, 5123, 
              7291, 5122, 7292, 5370, 7552, 8477, 4743, 8054, 4744, 6946, 
              7570, 5399, 8428, 4651, 6379, 4650, 5373, 8422, 4631, 6341, 
              5323, 6380, 5400, 6483, 5655, 5656, 5401, 8295, 5374, 8273, 
              5375, 6343, 6342, 6381, 6382, 5543, 6803, 7969, 7970, 6449, 
              5514, 8356, 5515, 8357, 4946, 6948, 4745, 6947, 6950, 6949, 
              7519, 6338, 6339, 5372, 8271, 5371, 8272, 4457, 4556, 7891, 
              7890, 8075, 8074, 4739, 6566, 6567, 6568, 7077, 7076, 7080, 
              7024, 7025, 7023, 4916, 7026, 7977, 7976, 4961, 8107, 5335, 
              4960, 8430, 4661, 5726, 4690, 6479, 6480, 5725, 5724, 4449, 
              5562, 8459, 5563, 5561, 8036, 8037, 4701, 6469, 4702, 8038, 
              8039, 7885, 8419, 4546, 5737, 8267, 5363, 8266, 5364, 6337, 
              6336, 7720, 6941, 6942, 6535, 6536, 6537, 4937, 5540, 8426, 
              4645, 6460, 4646, 8427, 5541, 6332, 6331, 6329, 6330, 5352, 
              8256, 5351, 8257, 4437, 7880, 7879, 7878, 5308, 8223, 5307, 
              6323, 4624, 8420, 5344, 5505, 6374, 4641, 8425, 5397, 7921, 
              7814, 8476, 4729, 8049, 7540, 7541, 5343, 6503, 6502, 6501, 
              5050, 6062, 6061, 7703, 5499, 7702, 7701, 6741, 4779, 6977, 
              5048, 7174, 7176, 7175, 4778, 6975, 4777, 6976, 7805, 6974, 
              4776, 6971, 6973, 6972, 4425, 5768, 6056}],
             RowBox[{"-", "10"}]],
            Annotation[#, -10, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4426, 7806, 6499, 5049, 6060, 6059, 7808, 7807, 6500, 
              7178, 7177, 4780, 6742, 7179, 8417, 4539, 7180, 5773, 6068, 
              6069, 5531, 7922, 7923, 7924, 8409, 8408, 4625, 8377, 8378, 
              8376, 5539, 8379, 5506, 6448, 5507, 8352, 5508, 8353, 5345, 
              8253, 5346, 8254, 5309, 8224, 5310, 7881, 7882, 7883, 4438, 
              8259, 5353, 8258, 5354, 8260, 5314, 6333, 5315, 8228, 8230, 
              8229, 7515, 6530, 5072, 7197, 4805, 7205, 5082, 8163, 5081, 
              8484, 7886, 7887, 6749, 6748, 6539, 5083, 8399, 8400, 8398, 
              5575, 8404, 5594, 8405, 5593, 6488, 5573, 8397, 5574, 8394, 
              8396, 8395, 6092, 8338, 4703, 8041, 8040, 8147, 8146, 4734, 
              8073, 4814, 8072, 4813, 8136, 4999, 7997, 7998, 7075, 4966, 
              8114, 8115, 7079, 8489, 7078, 7888, 4550, 4450, 8328, 5449, 
              8327, 5450, 6416, 6415, 4691, 5451, 8000, 7999, 5435, 8317, 
              5319, 8234, 5320, 8235, 6569, 5816, 4829, 6109, 5649, 7300, 
              7301, 7298, 5127, 7299, 5828, 5829, 5145, 8188, 5144, 8485, 
              7897, 7898, 6760, 6759, 6597, 5146, 7326, 4848, 7345, 5167, 
              8191, 5166, 7733, 7734, 7732, 5522, 6344, 5328, 8278, 5380, 
              8276, 5379, 8277, 5403, 5665, 5664, 6485, 5402, 6383, 5327, 
              8275, 5378, 8274, 5377, 5545, 8429, 4652, 6464, 5544, 8381, 
              5547, 8380, 5546, 4632, 6361, 7946, 7945, 7944, 5376, 6612, 
              4481, 5844, 4473, 4570, 6758, 6757, 4836, 7294, 7295, 7293, 
              5124, 8177, 6103, 6104, 5125, 7296, 8179, 8178}],
             RowBox[{"-", "12.5`"}]],
            Annotation[#, -12.5, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4636, 6367, 4637, 6798, 6797, 7569, 7568, 7748, 6354, 
              6355, 5340, 8369, 5530, 7910, 7911, 7912, 6456, 6442}],
             RowBox[{"-", "12.5`"}]],
            Annotation[#, -12.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4474, 5827, 6128, 6129, 5843, 4846, 6156, 4847, 7714, 
              7715, 6613, 6614, 4482, 6154, 4491, 5862, 5863, 6463, 6462, 
              4860, 8076, 7383, 5190, 7382, 5189, 8279, 8280, 7947, 7948, 
              7949, 6345, 7572, 5404, 7571, 5405, 7573, 4653, 7971, 8487, 
              5381, 8281, 7950, 7951, 6347, 7738, 7739, 7385, 5193, 6640, 
              6641, 5865, 4861, 7349, 5170, 6615, 7847, 7848, 4577, 8192, 
              5168, 7346, 5169, 7348, 4849, 5846, 5147, 7325, 5148, 7327, 
              6763, 6761, 6996, 6995, 4465, 4562, 8180, 8181, 7297, 5126, 
              7574, 5409, 8300, 5408, 8298, 8299, 8302, 8301, 7984, 6385, 
              5410, 8306, 8307, 8303, 8305, 8304, 8329, 8118, 8119, 8117, 
              8121, 8120, 4692, 8014, 8144, 8143, 4740, 8051, 8053, 8052, 
              4704, 6422, 4705, 8435, 5472, 6894, 6895, 6093, 6094, 5318, 
              6473, 6474, 5608, 4451, 4551, 7889, 7723, 7724, 7721, 8365, 
              5519, 7722, 5473, 6423, 5637, 7221, 5093, 6538, 6553, 5804, 
              4815, 7208, 5086, 6541, 7821, 7822, 4547, 8164, 5084, 7206, 
              5085, 7207, 4806, 5798, 5073, 7198, 5074, 7199, 7517, 7516, 
              6334, 7940, 7939, 7938, 5355, 8263, 5510, 8227, 5312, 8225, 
              5311, 8226, 6324, 7937, 7936, 7935, 8262, 8261, 6520, 6519, 
              5064, 7192, 4789, 6446, 6447, 5780, 5779, 6071, 6067, 7705, 
              7706, 7704, 7708, 7707, 4782, 6065, 4781, 5771, 6504, 4427, 
              5770, 6063}],
             RowBox[{"-", "15"}]],
            Annotation[#, -15, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4483, 5845, 8358, 8359, 7713, 7717, 7716, 5165, 7380, 
              7381, 7377, 7379, 7378, 5864, 8197, 5191, 7384, 5192, 5887, 
              5886, 6665, 6664, 5888, 4870, 6346, 4871, 8078, 8077, 8393, 
              8392, 6666, 5890, 4872, 6348, 4873, 5891, 6667, 4503, 5892, 
              7390, 5196, 7389, 5197, 7391, 5672, 5673, 5194, 7386, 5195, 
              7388, 4862, 5866, 5172, 7347, 5173, 7351, 5666, 7350, 5171, 
              6616, 6617, 6618, 5847, 7328, 7329, 6997, 4838, 6762, 4837, 
              7302, 5128, 6584, 7520, 8240, 8241, 8453, 8452, 5321, 8486, 
              7611, 5436, 8318, 8320, 8319, 4694, 6417, 4693, 8016, 8015, 
              5452, 5609, 4458, 4557, 7892, 7893, 7081, 8441, 4967, 8116, 
              4968, 8138, 8139, 8137, 5000, 6990, 4822, 6989, 6992, 6991, 
              4741, 6425, 4742, 5738, 6896, 4706, 6424, 6899, 6898, 6897, 
              5474, 6570, 4459, 5811, 7224, 5095, 7223, 5096, 7225, 5638, 
              7222, 5094, 6540, 6554, 5805, 4816, 7210, 5088, 6546, 7823, 
              5633, 5632, 7209, 5087, 6542, 6543, 6544, 5799, 6082, 6083, 
              5794, 4802, 6335, 4801, 5793, 5071, 5788, 4795, 6325, 4794, 
              5786, 5065, 6076, 5782, 5783, 5063, 6518, 7815, 7816, 5781, 
              4429, 6509, 6508, 7182, 7181, 5051, 7709, 7710, 6505, 7809, 
              4428, 5772, 6064}],
             RowBox[{"-", "17.5`"}]],
            Annotation[#, -17.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5052, 6070, 6066, 7811, 7810, 6507, 5500, 7711, 5054, 
              7186, 5058, 6512, 7812, 6978, 4790, 6743, 6745, 4540, 4433, 
              5787, 4439, 7817, 6525, 5070, 5791, 5790, 6080, 5795, 6533, 
              6532, 6531, 5075, 5629, 4543, 4442, 6534, 5800, 4807, 6087, 
              5634, 7215, 7216, 7213, 5090, 7214, 5806, 6556, 6555, 6545, 
              5097, 7226, 5639, 7230, 5098, 7227, 7229, 7228, 5813, 4460, 
              6571, 5812, 4824, 6095, 4823, 7255, 7256, 7253, 8171, 5106, 
              8170, 7894, 7895, 8236, 8239, 8238, 8237, 6900, 6901, 4707, 
              6426, 6427, 5476, 7657, 5475, 7658, 7660, 7659, 4466, 8386, 
              5558, 8385, 5559, 8387, 5453, 6418, 8018, 8017, 5454, 8330, 
              7633, 7119, 5001, 7120, 5336, 7121, 5002, 6855, 8001, 7030, 
              4917, 7027, 7029, 7028, 7576, 7575, 4475, 7837, 6598, 6583, 
              5149, 5657, 4571, 7845, 7844, 6600, 7331, 7330, 4850, 6157, 
              4851, 5848, 6160, 6159, 5867, 4492, 6643, 6642, 5198, 7393, 
              5674, 7397, 5200, 7387, 5199, 7396, 5894, 6670, 6669, 6668, 
              7419, 7418, 5684, 6219, 4874, 5893, 6216, 6215, 4593, 5683, 
              5226, 7416, 5225, 7417, 5934, 4514, 5931, 5932, 5224, 7415, 
              8205, 8204, 5889, 7414, 4592, 7901, 7413, 6774, 4869, 7006, 
              5219, 6660, 5220, 7408, 5187, 7719, 5517, 7410, 5516, 7718, 
              6155, 6639, 6638}],
             RowBox[{"-", "20"}]],
            Annotation[#, -20, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5222, 7411, 5223, 7412, 5188, 7409, 5221, 6661, 6662, 
              6663, 5926, 4513, 7865, 7011, 4885, 6775, 4886, 7447, 5250, 
              6698, 5249, 5933, 4887, 6362, 5692, 5693, 5252, 5935, 4888, 
              6254, 5694, 7448, 5253, 6702, 6703, 5936, 4889, 7420, 5227, 
              6671, 7855, 4504, 4594, 6217, 6218, 5897, 4875, 6222, 5685, 
              6220, 6221, 5896, 5895, 4493, 4585, 8198, 8199, 7392, 7395, 
              7394, 4863, 7352, 7353, 6622, 7849, 7850, 4578, 5667, 5174, 
              6599, 6619, 6620, 5849, 6131, 6132, 5412, 6815, 7987, 7988, 
              6130, 4662, 6813, 6812, 6818, 6817, 4664, 5993, 4665, 6820, 
              6819, 7085, 4971, 8443, 6880, 8019, 8442, 6879, 8125, 8126, 
              8124, 6816, 7148, 8431, 4663, 7149, 5739, 8340, 8341, 8339, 
              5477, 8342, 5411, 8382, 5550, 7985, 7986, 7160, 5030, 6811, 
              6810, 6808, 4969, 5610, 4468, 7661, 7662, 7261, 5108, 7254, 
              5109, 5643, 7838, 4558, 8172, 5107, 7257, 7260, 7259, 4825, 
              7231, 7232, 6557, 7827, 4452, 4552, 6096, 5808, 5809, 6099, 
              6098, 5640, 6097, 4817, 5807, 6547, 4447, 4548, 8165, 8166, 
              7211, 5089, 7212, 4808, 5801, 5077, 7202, 5078, 7203, 5630, 
              6084, 4803, 5796, 6526, 5626, 5625, 6356, 4796, 5789, 6523, 
              6522, 5068, 7194, 4792, 6744, 4791, 6979, 7189, 7188, 5777, 
              7184, 5056, 6511, 5057, 7185, 5053, 7183, 5055, 6506, 6510}],
             RowBox[{"-", "22.5`"}]],
            Annotation[#, -22.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4430, 6513, 5774, 4783, 6072, 4784, 7190, 7191, 6515, 
              6516, 6517, 5784, 5785, 5067, 6521, 5066, 7193, 7544, 7543, 
              5732, 8370, 8371, 7749, 5532, 7750, 5356, 7547, 5358, 7545, 
              5357, 7546, 5735, 8268, 5365, 7548, 5366, 7549, 5076, 6938, 
              7201, 7200, 4544, 4443, 5802, 5803, 5367, 6358, 5635, 5636, 
              7824, 6549, 5091, 7217, 4818, 7235, 5100, 7233, 5099, 8167, 
              4553, 7829, 7828, 6559, 5101, 7236, 4826, 7264, 5111, 7262, 
              5110, 5645, 5644, 7266, 5113, 7258, 5112, 7265, 7782, 7781, 
              7841, 7840, 7839, 8407, 8406, 5478, 7774, 6903, 6904, 6105, 
              4708, 6902, 5740, 4747, 6047, 4746, 6993, 4831, 6809, 4830, 
              7122, 5003, 6856, 8002, 7082, 4970, 8122, 8123, 7084, 8490, 
              7083, 7896, 4563, 4467, 8332, 5455, 8331, 5456, 6420, 6419, 
              8022, 8021, 8020, 8334, 8333, 7634, 6857, 5438, 7613, 5324, 
              6814, 5325, 7521, 6601, 5852, 5150, 6133, 4852, 6158, 4853, 
              5853, 6163, 6162, 5868, 6645, 6644, 6621, 5201, 7398, 5675, 
              5676, 7853, 6647, 5202, 7399, 4876, 7423, 5229, 7421, 5228, 
              8206, 4595, 7857, 7856, 6673, 5230, 7424, 4890, 7451, 5256, 
              7449, 5255, 5696, 5695, 6363, 5391, 5938, 5937, 6253, 4604, 
              4515, 7866, 6705, 5254, 7564, 5390, 7562, 5389, 7563, 4526, 
              8287, 5387, 7560, 5388, 7561, 5385, 7559, 5386, 7556, 7558, 
              7557, 4525, 7554, 7555, 6701, 6700, 6699, 5251, 5930, 5929, 
              7444, 5247, 6692, 7446, 7445, 4884, 6214, 4883, 5925, 6251, 
              6250}],
             RowBox[{"-", "25"}]],
            Annotation[#, -25, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            TooltipBox[LineBox[CompressedData["
1:eJwV0VtoznEcx/GvHEdpz/PMaWMzZg+ZkoV24cIsW7ixXHAhwpwmpxo3TjM7
OI6aw5SlkBtKklq2uUDh1kRjo1DcGC7YXMhrF+/n8/99ft/v9/f5/Z78TXsq
dw+LiJF+nqQi2idFNE+MqM+NOE+f8fZPiXhKU1Mj+rIiNvFrkhEHpkV8sj6c
HXEESV6n/hStUrMV3yZHfMXnCRF1OREncExtC12YH3GRrjU73/cl33m0U39d
XkQXLeN90VtPV6ir1XvZ9xVcRSvKUYGpOKwnhx6i5e5QgeN66nACb2R5i0G5
j8g/QI/S53Lfl7cqEXHdfVtQ4b7lyOCvov3OL6Xf6Ur6kH9X3R204S+/ij+e
v5NWYwN2YDs6+AfNb6eP8M6ZPfgoa6XML9zzgLwfhjy8tF5n7lgZ+6yfWzea
04STqLQ3zl6//D8w6H5/cV3tbVmW2+/Qs8ibtvEW0wRSSGKz2m55atA7PWKL
dbueDDO7zavU28xfTYvUz8NcvLL3St7X+Gm/m5bK3cV/jJfmLPP/leG3t/6l
5g9dYk6H/ePWtXjv3EPoGXoLLHZuQl2B71n4IEs/LeFn8cOdCmkaP/k/UOR7
rLu1mpN07m89I5wxHIvsj6b79P/zXnvpKOti9cVqFyATJ/m5/KXqZ9ICZGMG
8rFK/T3zN8qcOfQmvATtdfZ7JN0zgTNmnbN3FmW+5+OW/7bYm5V4o2rsxDW0
4bT9U0PnY427XeDdVL+Dpq230UK6na6nTeoa0YDzvGbcUN8iw0U0yt+AHN4u
62yaKVe9+jHy1sr1hw5gFNL8QszCMW8w2z0fuGOazuHNxn91npd0
              "]],
             RowBox[{"-", "27.5`"}]],
            Annotation[#, -27.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            TooltipBox[
             LineBox[{4527, 4612, 7908, 7906, 7907, 7058, 8098, 4949, 7059, 
              7973, 7972, 8297, 8296, 5333, 7536, 5334, 7537, 5281, 8213, 
              5282, 7484, 4904, 7461, 7462, 6709, 7867, 4516, 4605, 8209, 
              8210, 7452, 7456, 7455, 4891, 7432, 7433, 6676, 7858, 4506, 
              4596, 8207, 8208, 7425, 5231, 7429, 4877, 5899, 5205, 5874, 
              5206, 6189, 5677, 6186, 5203, 5870, 5869, 4484, 4579, 8193, 
              8194, 7354, 5176, 7356, 7578, 8308, 5414, 6388, 6823, 6824, 
              6392, 6393, 5417, 8309, 5416, 8488, 7637, 5459, 7088, 4974, 
              7087, 4975, 7151, 5024, 7150, 5023, 5741, 4748, 7665, 5480, 
              6907, 6908, 4709, 6387, 6466, 7772, 7773, 7771, 5569, 6472, 
              6471, 8390, 5565, 8343, 5479, 8183, 8184, 6340, 5322, 6475, 
              6476, 5611, 4470, 4565, 7728, 7725, 7726, 7666, 5520, 7727, 
              5133, 6117, 5650, 6116, 5132, 5818, 6586, 5821, 4833, 7276, 
              5115, 6575, 7834, 4461, 4559, 8173, 8174, 7267, 7271, 7270, 
              4827, 7245, 7246, 6562, 7830, 4454, 4554, 8168, 8169, 7237, 
              7242, 7241, 4819, 7218, 5092, 6551, 7518, 8233, 5317, 8451, 
              5316, 8231, 8232, 8294, 8293, 7968, 7049, 4939, 7046, 7048, 
              7047, 4444, 4545, 6007}],
             RowBox[{"-", "30"}]],
            Annotation[#, -30, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{5518, 6450, 6294, 4655, 7053, 7054, 7052, 4947, 8097, 
              5764, 5765, 4901, 7012, 7014, 7013, 7068, 4633, 8283, 8284, 
              8282, 8286, 8285, 5383, 8374, 5534, 8373, 5533}],
             RowBox[{"-", "30"}]],
            Annotation[#, -30, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{6799, 6800, 6375, 4642, 7040, 7041, 7038, 4932, 7039, 
              4431, 5601, 4785, 8061, 8062, 7032, 7031, 7918, 4620, 8249, 
              8250, 8248, 8252, 8251, 5062, 8157, 8456, 5501, 8495, 8455}],
             RowBox[{"-", "30"}]],
            Annotation[#, -30, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4455, 7825, 6563, 6550, 7248, 7247, 5642, 7250, 5105, 
              7238, 5104, 7249, 5815, 6577, 6576, 6561, 5116, 7279, 5647, 
              7283, 5118, 7268, 5117, 7282, 5825, 6589, 6588, 6574, 7309, 
              7308, 5651, 6121, 6122, 5824, 6119, 5822, 6115, 4566, 4472, 
              6587, 7670, 7669, 5134, 7668, 5482, 7667, 5481, 7766, 7767, 
              7765, 5566, 8391, 6113, 6114, 5744, 8182, 5743, 5830, 6039, 
              4666, 5723, 5413, 6386, 4972, 6008, 6822, 6821, 4973, 7086, 
              5326, 6952, 4750, 6951, 4749, 7638, 5460, 6881, 6882, 6389, 
              5415, 7989, 8496, 5551, 8383, 8384, 7990, 4667, 7577, 7522, 
              7358, 8432, 7359, 7355, 5178, 7357, 5875, 5876, 5204, 5877, 
              6185, 6184, 4586, 4494, 5900, 6678, 5902, 6224, 6223, 5687, 
              7435, 7436, 7426, 5235, 7434, 5940, 6711, 6710, 6675, 5258, 
              7465, 5698, 7469, 5260, 7453, 5259, 7468, 5965, 6726, 6725, 
              6708, 5283, 7487, 5706, 7492, 7493, 7483, 7491, 7490}],
             RowBox[{"-", "32.5`"}]],
            Annotation[#, -32.5, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4621, 5778, 4622, 8105, 8106, 7071, 4958, 8113, 4965, 
              8111, 4964, 8112, 4659, 5721, 5406, 7978, 7979, 7980, 6370, 
              6459, 5537}],
             RowBox[{"-", "32.5`"}]],
            Annotation[#, -32.5, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{6291, 5963, 4654, 7073, 7074, 7072, 4959, 8110, 4963, 
              8108, 4962, 8109, 4660, 5722, 5407, 7981, 7982, 7983, 6384, 
              6807, 6806}],
             RowBox[{"-", "32.5`"}]],
            Annotation[#, -32.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4529, 4613, 8214, 8215, 7485, 7489, 7488, 4905, 7470, 
              7471, 6713, 7868, 4517, 4606, 8211, 8212, 7463, 7467, 7466, 
              4892, 7437, 7438, 6681, 7859, 7860, 4597, 5688, 5234, 5901, 
              6677, 6679, 5903, 6193, 5209, 5878, 4866, 6187, 4865, 7360, 
              7361, 6625, 6767, 6768, 7756, 8242, 5329, 8243, 7523, 5553, 
              7757, 5554, 7758, 5555, 7759, 5439, 8323, 8324, 7617, 7616, 
              4752, 6468, 5612, 5613, 5008, 7127, 5004, 7126, 5005, 6010, 
              6009, 6860, 6859, 5831, 4839, 7639, 6764, 5742, 4751, 6429, 
              5484, 5746, 5745, 4710, 6118, 4711, 6909, 5570, 7775, 5483, 
              6428, 5595, 6495, 5596, 6494, 5580, 6120, 5136, 5823, 5137, 
              5652, 4567, 8185, 5135, 7306, 7311, 7310, 4834, 7284, 7285, 
              6579, 7835, 4462, 4560, 8175, 8176, 7277, 7281, 7280, 4828, 
              7251, 7252, 6565, 7831, 4456, 4555, 6100}],
             RowBox[{"-", "35"}]],
            Annotation[#, -35, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5285, 7486, 5286, 7499, 5707, 7496, 5284, 6712, 6727, 
              6728, 5966, 7473, 5264, 7464, 5265, 7474, 5699, 7472, 5262, 
              6680, 6714, 4518, 5941, 6233, 5907, 5908, 6226, 6225, 4878, 
              6194, 5210, 5906, 6650, 5679, 5678, 7400, 5207, 6624, 6648, 
              7851, 4495, 7579, 7581, 5994, 7899, 5995, 5552, 6467, 4918, 
              5996, 8005, 6863, 5007, 7129, 5337, 7128, 5006, 6861, 6862, 
              8024, 8494, 5462, 8336, 8025, 6421, 5461, 7764, 5560, 7763, 
              8389, 8388, 4485, 7676, 7672, 7673, 7671, 5485, 6431, 6430, 
              6912, 6911, 6910, 7675, 7674, 5152, 6484, 5153, 7333, 7332, 
              5579, 6134, 5151, 6490, 5582, 6136, 6135, 7799, 5581, 6998, 
              4840, 6123, 6124, 5834, 5832, 4477, 5835, 7317, 7318, 7307, 
              5139, 7319, 5653, 7314, 5138, 6578, 6590, 6591, 5826, 7287, 
              5120, 7278, 5121, 7288, 5648, 7286, 5119, 6564, 6580}],
             RowBox[{"-", "37.5`"}]],
            Annotation[#, -37.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4530, 4614, 8216, 8217, 7494, 7498, 7497, 4906, 7475, 
              7476, 6716, 7869, 4519, 4607, 6255, 5261, 5943, 5263, 6257, 
              4894, 7440, 5942, 5238, 5909, 5239, 6234, 5689, 6229, 6230, 
              5905, 5904, 4496, 4587, 6190, 6191, 6395, 6394, 5208, 7580, 
              5419, 6649, 5418, 7582, 7583, 6833, 7993, 5997, 4919, 7098, 
              4981, 8127, 8128, 7154, 7153, 8145, 7092, 8500, 7093, 7091, 
              4978, 7094, 8023, 6883, 5025, 7152, 4753, 8055, 8056, 6954, 
              6953, 4712, 6432, 4713, 6916, 6915, 5486, 7677, 6913, 6914, 
              6011, 6012, 5154, 6603, 6765, 6766, 7846, 4573, 7731, 8468, 
              8506, 7729, 8508, 7730, 8516, 8517, 8515, 8507, 5521, 6139, 
              6138, 5658, 6137, 5155, 5833, 6604, 5836, 7338, 4842, 6126, 
              5140, 5838, 6592, 6593, 4568, 8186, 8187, 7312, 7316, 7315, 
              4835, 7289, 7290, 6582, 7836, 4463, 4561, 6108}],
             RowBox[{"-", "40"}]],
            Annotation[#, -40, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5290, 7495, 5291, 7504, 5708, 7502, 5288, 6715, 6729, 
              4531, 5967, 6261, 6262, 5944, 4895, 6256, 4893, 7439, 7441, 
              6683, 7861, 4507, 4598, 6227, 5236, 5917, 5237, 5911, 5910, 
              6195, 6196, 5420, 6829, 7992, 4671, 6192, 4670, 8310, 5421, 
              7585, 5422, 7643, 7644, 7642, 5465, 6956, 4754, 6955, 6958, 
              6957, 7524, 7089, 7090, 6830, 6831, 6832, 6015, 6016, 4980, 
              7095, 4979, 7097, 7991, 4668, 7161, 4669, 6827, 6825, 4976, 
              5614, 6602, 8414, 7679, 5487, 7678, 7681, 7680, 5156, 7683, 
              6605, 6606, 4574, 8189, 8190, 7334, 7337, 7336, 4841, 6125, 
              4843, 5837, 6607, 4478, 5839, 7321, 7322, 7313, 5142, 7323, 
              5654, 7320, 5141, 6581, 6594}],
             RowBox[{"-", "42.5`"}]],
            Annotation[#, -42.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4532, 4615, 8218, 5287, 7500, 5289, 7503, 4907, 6263, 
              5269, 5970, 6718, 5701, 5700, 7477, 5267, 6682, 6717, 4520, 
              5945, 6241, 5915, 6228, 5916, 6232, 6231, 4879, 6197, 5211, 
              5914, 6651, 7526, 7527, 6828, 5330, 7525, 5441, 7584, 5442, 
              7618, 8026, 4695, 6409, 6410, 5464, 7640, 5463, 7641, 4486, 
              4580, 6017, 6013, 6014, 6406, 6405, 4977, 7130, 5010, 7096, 
              5009, 6999, 4854, 6826, 7001, 7000, 4755, 6049, 4756, 5750, 
              6917, 4714, 6140, 6141, 5488, 8344, 8346, 8345, 7682, 6626, 
              7852, 4487, 7784, 7339, 7340, 7335, 5159, 7341, 5659, 5660, 
              5157, 5841, 5158, 6145, 4844, 7324, 5143, 6596, 7842, 7843, 
              4569, 6127}],
             RowBox[{"-", "45"}]],
            Annotation[#, -45, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4479, 6610, 6595, 7343, 7342, 5662, 6150, 4845, 6144, 
              5840, 6143, 6142, 4575, 6609, 6608, 5856, 5160, 6147, 7002, 
              7786, 5583, 8401, 7785, 5597, 6497, 6496, 6433, 7366, 7365, 
              7783, 5571, 6168, 6167, 7684, 5489, 6919, 6918, 5749, 5032, 
              5748, 8148, 5747, 4581, 6769, 5179, 7362, 7364, 7363, 7645, 
              4683, 8325, 8326, 7619, 5443, 6408, 6407, 6867, 6866, 6868, 
              6411, 6412, 5444, 7620, 7622, 7621, 4685, 5999, 4684, 6865, 
              6864, 5440, 6771, 4588, 4497, 6652, 5913, 6198, 4880, 6238, 
              5690, 6242, 5243, 5918, 5242, 5946, 6244, 4897, 6260, 5268, 
              5950, 5266, 6258, 4608, 4521, 6719, 5969, 6264, 4908, 6297, 
              5710, 7507, 7509, 7501, 7506, 7505}],
             RowBox[{"-", "47.5`"}]],
            Annotation[#, -47.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5161, 5663, 7344, 5661, 6151, 5163, 5842, 5162, 6149, 
              5857, 4489, 6630, 5855, 6171, 4856, 6146, 4855, 7789, 7790, 
              7787, 5584, 7791, 8366, 7736, 5585, 7792, 6166, 6491, 6492, 
              6453, 6452, 6454, 6170, 5180, 6169, 6922, 4715, 6050, 8043, 
              8042, 6921, 5033, 8151, 8152, 8149, 5034, 8150, 4498, 5615, 
              5466, 6884, 6885, 6886, 6165, 4696, 8027, 7132, 5012, 7102, 
              5011, 7131, 7909, 7133, 5013, 7107, 5014, 7134, 4921, 6404, 
              4920, 5998, 7590, 7589, 7587, 7588, 6685, 6684, 5912, 5240, 
              5691, 6236, 4600, 6687, 6686, 5949, 6245, 6243, 4896, 6259, 
              4898, 6274, 5952, 6272, 6271, 5971, 6731, 6730, 5968, 5292, 
              6298, 5709, 7508, 5711, 6732}],
             RowBox[{"-", "50"}]],
            Annotation[#, -50, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4533, 4616, 6295, 6296, 5973, 5293, 6300, 4910, 7479, 
              5972, 5272, 5951, 5273, 6273, 5702, 6267, 6268, 5948, 5947, 
              4508, 4599, 6235, 6237, 5424, 6397, 5241, 6240, 6835, 7593, 
              5423, 6396, 6840, 6841, 6000, 6001, 4991, 8129, 4990, 8482, 
              7158, 5027, 7103, 4986, 7101, 4987, 7156, 7157, 7155, 5026, 
              5751, 4757, 7165, 5035, 6924, 6925, 4716, 6026, 7586, 7163, 
              7164, 6920, 6923, 6019, 4983, 7367, 5181, 6628, 6770, 8416, 
              4488, 4582, 7737, 8469, 8470, 8512, 8511, 7735, 8519, 8510, 
              8518, 8509, 5523, 6174, 7788, 6173, 5670, 6172, 5182, 5854, 
              6629, 5858, 7372, 4858, 6153, 5164, 5861, 6611, 4480, 4576, 
              6148}],
             RowBox[{"-", "52.5`"}]],
            Annotation[#, -52.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4490, 6634, 5860, 6178, 4859, 6152, 4857, 7370, 7371, 
              7368, 8196, 8195, 4583, 6632, 6631, 7692, 5183, 7687, 7688, 
              7685, 5490, 7686, 8415, 6627, 5618, 4982, 5753, 6834, 4673, 
              7162, 4672, 7994, 7106, 4988, 7104, 4989, 6022, 6021, 6839, 
              6838, 6837, 7100, 7099, 7528, 6961, 6962, 6959, 4758, 6960, 
              5467, 7648, 7650, 7649, 5427, 7592, 5426, 8311, 4674, 6239, 
              4675, 6836, 6249, 5425, 6247, 6246, 5953, 5955, 5271, 5961, 
              5270, 6265, 4609, 4522, 7870, 6721, 7480, 7478, 4909, 6299, 
              4911, 6308, 5975, 6306, 6305}],
             RowBox[{"-", "55"}]],
            Annotation[#, -55, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5297, 5974, 5298, 6307, 5712, 7510, 5295, 6720, 6733, 
              4534, 5977, 6282, 5959, 6266, 5960, 6270, 6269, 4899, 5954, 
              5244, 6398, 5245, 6248, 5331, 6349, 5446, 7591, 7629, 7628, 
              4698, 5755, 4697, 8028, 6887, 5028, 5616, 4499, 4589, 6024, 
              6025, 6020, 4984, 6040, 4985, 7135, 7136, 7105, 5016, 5879, 
              4867, 5754, 4760, 6018, 5617, 7689, 7690, 6927, 8044, 8045, 
              6175, 7777, 6176, 5492, 8347, 8349, 8348, 6655, 6654, 4501, 
              7794, 7373, 7374, 7369, 5185, 7376, 5671, 6179, 5184, 5859, 
              6633}],
             RowBox[{"-", "57.5`"}]],
            Annotation[#, -57.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4535, 4617, 6301, 5294, 5984, 5296, 5978, 6734, 5976, 
              6283, 5277, 5962, 6286, 6284, 5704, 6277, 4900, 5958, 6688, 
              6689, 6776, 7625, 7626, 6351, 6350, 4687, 7646, 7647, 6963, 
              4761, 6964, 4762, 8058, 8057, 6872, 8009, 8446, 8447, 8141, 
              8142, 5015, 8140, 8006, 8007, 6023, 4686, 5880, 8200, 5881, 
              7402, 7401, 6653, 5752, 4759, 7166, 8513, 5036, 6926, 6928, 
              4717, 7693, 6200, 7778, 7779, 7776, 5572, 7780, 5491, 7691, 
              6434, 5598, 7801, 5599, 7800, 5588, 7375, 7796, 7795, 5186, 
              5885, 6635, 6636, 4584, 6177}],
             RowBox[{"-", "60"}]],
            Annotation[#, -60, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4502, 5882, 6637, 5884, 6181, 6180, 7004, 7003, 7797, 
              5589, 7802, 7803, 6199, 5213, 6493, 5212, 8402, 5586, 7793, 
              5587, 6487, 6486, 5214, 7694, 5493, 6929, 6931, 6052, 5037, 
              6435, 5494, 8504, 6051, 8505, 8471, 8473, 8472, 6772, 4590, 
              4500, 7900, 7854, 6773, 7404, 7403, 8203, 8202, 8201, 8450, 
              8449, 8030, 6890, 5468, 7651, 6870, 6869, 7138, 7137, 7538, 
              7139, 7140, 6871, 8008, 6002, 4922, 7627, 5445, 7602, 7624, 
              7623, 4601, 4509, 5956, 7598, 5957, 5275, 6278, 5703, 6285, 
              5705, 6722, 5981, 5278, 6287, 4912, 6303, 6304, 5983, 6302, 
              5982, 6315}],
             RowBox[{"-", "62.5`"}]],
            Annotation[#, -62.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5302, 5985, 6318, 6316, 5714, 6311, 4913, 6288, 5980, 
              6723, 4523, 4610, 6275, 5274, 6403, 5276, 7599, 7600, 8314, 
              8313, 5429, 8312, 7604, 5432, 7603, 5433, 7605, 4923, 6038, 
              5716, 5717, 6044, 6043, 4619, 6041, 6042, 5339, 6889, 5338, 
              7539, 6035, 4699, 8029, 6888, 6966, 8059, 8060, 6965, 6690, 
              7695, 7696, 6932, 8046, 8047, 6436, 6437, 5495, 7697, 8351, 
              8350, 4718, 6029, 7405, 7406, 6930, 5215, 7407, 5528, 7744, 
              5529, 7745, 5526, 8368, 5527, 7743, 8367, 5524, 7740, 7741, 
              7005, 8403, 4868, 6210, 5681, 6204, 6205, 5883, 6656}],
             RowBox[{"-", "65"}]],
            Annotation[#, -65, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5216, 5682, 6206, 5680, 6211, 5217, 6455, 5525, 6208, 
              7742, 6207, 6438, 4511, 6027, 6028, 6202, 6201, 4993, 7168, 
              5039, 6845, 5038, 7596, 7597, 7594, 5428, 7595, 6036, 6037, 
              5041, 7169, 5040, 8153, 5756, 7159, 4680, 6852, 7111, 7110, 
              8132, 8131, 8130, 8445, 8444, 8031, 6891, 8133, 8134, 8135, 
              7117, 7118, 6854, 4682, 6003, 4681, 6399, 6400, 7609, 7608, 
              6846, 6847, 6280, 7995, 6281, 5430, 6276, 6402, 6401, 6289, 
              5987, 4536, 6735, 5979, 5300, 6312, 5713, 6317, 5715, 6737}],
             RowBox[{"-", "67.5`"}]],
            Annotation[#, -67.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4537, 4618, 6309, 5299, 5991, 5301, 5988, 6736, 5986, 
              6290, 5279, 7601, 5431, 6849, 7996, 4679, 6279, 4678, 8315, 
              8316, 7607, 5434, 7610, 8035, 6892, 5469, 7654, 4763, 6853, 
              4764, 6967, 7531, 7529, 7530, 7109, 4997, 6850, 6851, 6031, 
              4995, 7116, 4998, 7112, 7115, 7114, 4676, 7167, 4677, 6844, 
              6842, 4992, 7108, 5620, 5621, 5496, 6209, 6440, 6439, 5218, 
              7700, 6657, 6658, 4591, 6203}],
             RowBox[{"-", "70"}]],
            Annotation[#, -70, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4512, 7698, 7699, 6659, 5498, 6213, 6212, 6934, 6933, 
              5759, 4767, 6053, 4766, 7008, 7010, 6843, 4881, 7007, 5018, 
              7113, 5019, 7141, 4996, 6045, 4994, 6030, 6034, 6033, 4602, 
              4510, 5619, 5029, 6893, 8032, 8033, 5763, 4700, 8034, 7630, 
              5448, 7606, 5447, 7532, 5332, 6848, 8245, 8244, 7534, 7022, 
              5280, 7481, 4914, 6313, 6314, 5990, 6310, 5989, 6319}],
             RowBox[{"-", "72.5`"}]],
            Annotation[#, -72.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{5303, 5992, 7513, 7511, 6778, 8521, 4915, 7482, 7021, 
              7533, 8246, 7535, 7760, 7761, 7762, 6353, 6352, 4689, 7652, 
              7653, 6968, 4769, 6877, 4770, 7147, 5022, 7145, 5020, 7144, 
              5021, 7146, 5017, 6874, 8010, 8011, 6032, 4688, 5919, 7655, 
              5920, 7443, 7442, 6691, 8474, 8498, 5757, 7009, 4765, 6054, 
              8154, 5042, 5758, 6935, 4719, 7170, 5921, 5760, 5761, 5497, 
              6441, 5622}],
             RowBox[{"-", "75"}]],
            Annotation[#, -75, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4768, 5923, 5924, 5762, 4882, 5922, 5044, 6936, 5045, 
              7171, 5043, 8155, 8156, 7903, 7902, 7904, 4603, 7905, 7864, 
              7863, 7862, 5246, 8337, 5471, 7656, 5470, 6876, 6875, 6873, 
              7143, 7142, 7747, 7631, 7632, 6878, 8012, 8013, 6413, 6414, 
              5557, 5556, 4611, 4524, 8462, 8463, 8461, 7020, 8520, 6777, 
              7512, 6779, 8522}],
             RowBox[{"-", "77.5`"}]],
            Annotation[#, -77.5, "Tooltip"]& ]}, {}, {}}}],
       AspectRatio->1,
       BaseStyle->{FontFamily -> "Times", FontSize -> 12},
       DisplayFunction:>Identity,
       Frame->True,
       PlotLabel->FormBox[
        "\"\[Mu]=10 \[Alpha]=0.22 \[Epsilon]=10 PIN=0.10\"", TraditionalForm],
       
       PlotRange->{{0, 50}, {0, 50}},
       PlotRangeClipping->True,
       PlotRangePadding->{
         Scaled[0.02], 
         Scaled[0.02]}], {192., -192.}, 
      ImageScaled[{0.5, 0.5}], {360., 360.}], InsetBox[
      GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJycfXVgVUcTbyjuBAkEggQIIRAgDiG21xOkeCluLSW4FXf3YgVaimuKQ4sV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        "], {{
          {RGBColor[0.293416, 0.0574044, 0.529412], EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{2298, 688, 647}, {5812, 5811, 2298}, {5813, 5814, 
              2299}, {979, 2299, 647}, {4294, 5809, 647}, {5810, 4294, 647}, {
              647, 688, 225}, {979, 647, 225}, {2298, 3046, 688}, {2635, 2299,
               979}},
              VertexColors->None], 
             PolygonBox[{{5811, 4111, 3046, 2298}, {6219, 5813, 2299, 2635}, {
              5809, 5812, 2298, 647}, {5814, 5810, 647, 2299}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.3173751637361717, 0.09906291520559765, 0.5630895030433356], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{3615, 3745, 1267}, {6280, 6281, 2698}, {4110, 5253, 
              373}, {1092, 2297, 378}, {2296, 3160, 373}, {1095, 2698, 
              1092}, {4626, 5258, 1092}, {6279, 4626, 1092}, {4111, 5811, 
              374}, {5257, 4110, 373}, {3160, 2297, 373}, {6915, 6594, 
              3035}, {2297, 1092, 373}, {3745, 3035, 1267}, {3615, 1267, 
              374}, {1091, 2296, 373}, {3035, 3745, 1091}, {6594, 6593, 
              3035}, {5812, 5809, 2296}, {5813, 6219, 378}, {5254, 4625, 
              1091}, {4625, 6914, 1091}, {2296, 3615, 374}, {5810, 5814, 
              2297}, {3615, 2296, 1091}, {2705, 2698, 1095}, {3745, 3615, 
              1091}, {1095, 1092, 378}},
              VertexColors->None], 
             PolygonBox[{{5811, 5812, 2296, 374}, {5258, 5257, 373, 1092}, {
              6914, 6915, 3035, 1091}, {5253, 5254, 1091, 373}, {5809, 4294, 
              3160, 2296}, {6285, 6280, 2698, 2705}, {6281, 6279, 1092, 
              2698}, {6593, 4094, 1267, 3035}, {5814, 5813, 378, 2297}, {4294,
               5810, 2297, 3160}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.3446476205445848, 0.1464823522593226, 0.6014242403187413], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{1794, 2699, 1076}, {2695, 2696, 1074}, {3361, 2696, 
              2695}, {1743, 2695, 1074}, {3348, 2694, 1076}, {4551, 5189, 
              1743}, {1793, 2688, 1076}, {2695, 1743, 310}, {2109, 1794, 
              1090}, {6268, 6269, 2688}, {6263, 7100, 2685}, {2686, 1743, 
              1074}, {7100, 6264, 2685}, {1791, 2109, 1090}, {5254, 5253, 
              1791}, {6279, 6281, 2697}, {6915, 6914, 3361}, {2697, 1793, 
              1076}, {5256, 4615, 1793}, {3360, 2685, 1090}, {1791, 1090, 
              1074}, {4094, 6593, 310}, {2694, 3347, 1076}, {6266, 7099, 
              2686}, {3347, 2694, 1090}, {2696, 1791, 1074}, {7099, 6265, 
              2686}, {2100, 1793, 311}, {2694, 3360, 1090}, {1090, 2685, 
              1074}, {6278, 6277, 2694}, {3347, 1794, 1076}, {5257, 5258, 
              1794}, {6280, 6285, 311}, {2699, 2697, 1076}, {2688, 3348, 
              1076}, {1794, 3347, 1090}, {1793, 2697, 311}, {1743, 1754, 
              310}},
              VertexColors->None], 
             PolygonBox[{{6277, 6913, 3360, 2694}, {5189, 4569, 1754, 1743}, {
              6269, 6267, 3348, 2688}, {4626, 6279, 2697, 2699}, {6265, 4551, 
              1743, 2686}, {5253, 4110, 2109, 1791}, {4110, 5257, 1794, 
              2109}, {6914, 4625, 2696, 3361}, {4625, 5254, 1791, 2696}, {
              6267, 6278, 2694, 3348}, {6593, 6594, 2695, 310}, {5258, 4626, 
              2699, 1794}, {6281, 6280, 311, 2697}, {4615, 6268, 2688, 
              1793}, {6262, 6266, 2686, 1074}, {6264, 6262, 1074, 2685}, {
              6913, 6263, 2685, 3360}, {5615, 5256, 1793, 2100}, {6594, 6915, 
              3361, 2695}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.371920077352998, 0.19390178931304755`, 0.6397589775941471], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{3305, 2610, 961}, {4074, 6937, 224}, {2006, 3485, 
              978}, {6268, 4615, 1075}, {4066, 6565, 210}, {3988, 4066, 
              210}, {3305, 3630, 2610}, {4674, 6341, 2770}, {3859, 3606, 
              2862}, {6938, 6340, 2769}, {4587, 5301, 1030}, {3485, 3486, 
              210}, {6431, 7094, 2863}, {3453, 3305, 2862}, {6262, 6264, 
              2684}, {5256, 5615, 1268}, {7005, 6431, 2863}, {2687, 1158, 
              234}, {1259, 3605, 234}, {2863, 3607, 1030}, {6267, 6269, 
              2687}, {3859, 3818, 2684}, {3605, 1259, 1030}, {3820, 3819, 
              3605}, {6913, 6277, 234}, {4569, 5189, 978}, {5685, 4675, 
              1158}, {3820, 3606, 2684}, {3607, 3606, 1030}, {3606, 3605, 
              1030}, {3605, 3819, 234}, {6646, 5685, 1158}, {3606, 3607, 
              2862}, {4001, 4074, 224}, {6984, 6985, 2610}, {3486, 3485, 
              2006}, {3819, 3820, 2684}, {6263, 6913, 234}, {1158, 2687, 
              1075}, {4551, 6265, 961}, {2862, 3305, 961}, {6266, 6262, 
              2684}, {3606, 3820, 3605}, {3818, 2862, 961}, {6342, 4741, 
              1259}, {3607, 2863, 2862}, {2769, 1268, 224}, {3606, 3859, 
              2684}, {6278, 6267, 2687}, {2610, 2006, 961}, {7074, 7075, 
              3630}, {1268, 2769, 1075}, {2769, 2770, 1075}, {2006, 978, 
              961}, {5255, 6646, 1158}, {3818, 3859, 2862}, {2863, 3453, 
              2862}, {6985, 6405, 2610}, {7094, 4771, 2863}, {5515, 5583, 
              2006}, {4741, 5232, 1259}, {5301, 7004, 1030}, {1158, 1259, 
              234}},
              VertexColors->None], 
             PolygonBox[{{7003, 6862, 3305, 3453}, {4615, 5256, 1268, 1075}, {
              7099, 6266, 2684, 3818}, {6341, 4614, 1075, 2770}, {7075, 6984, 
              2610, 3630}, {7004, 7005, 2863, 1030}, {7100, 6263, 234, 
              3819}, {6277, 6278, 2687, 234}, {6405, 5515, 2006, 2610}, {4771,
               7003, 3453, 2863}, {6862, 7074, 3630, 3305}, {6937, 6938, 2769,
               224}, {6269, 6268, 1075, 2687}, {5583, 7019, 3486, 2006}, {
              5189, 4551, 961, 978}, {7019, 3988, 210, 3486}, {6265, 7099, 
              3818, 961}, {5232, 4587, 1030, 1259}, {6340, 4674, 2770, 
              2769}, {4614, 5255, 1158, 1075}, {4675, 6342, 1259, 1158}, {
              6264, 7100, 3819, 2684}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.3991925341614111, 0.2413212263667725, 0.6780937148695528], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{5762, 4203, 2242}, {5883, 5762, 2242}, {7019, 5583, 
              2067}, {2242, 2243, 2167}, {3087, 1657, 443}, {6342, 4675, 
              2771}, {3988, 7019, 2067}, {2168, 3087, 443}, {6938, 6937, 
              3379}, {1315, 1156, 441}, {2005, 2067, 1248}, {3379, 1482, 
              1157}, {4614, 6341, 1792}, {1657, 1156, 443}, {1657, 2768, 
              1156}, {2066, 2974, 2005}, {3086, 1792, 442}, {7005, 7004, 
              3454}, {4204, 6696, 3148}, {7074, 6862, 1314}, {5070, 4136, 
              441}, {2242, 2167, 442}, {5041, 4137, 442}, {6404, 6403, 
              2830}, {1156, 2164, 441}, {5683, 5684, 2164}, {3760, 3427, 
              1314}, {1156, 1775, 443}, {4587, 5232, 1775}, {2167, 2168, 
              443}, {6695, 5686, 2168}, {2831, 3428, 1248}, {2243, 3148, 
              2168}, {6984, 7075, 3427}, {2771, 2167, 443}, {2768, 2164, 
              1156}, {7003, 4771, 1314}, {5084, 4370, 1657}, {4922, 4199, 
              1520}, {1792, 1157, 442}, {3760, 3631, 3426}, {4674, 6340, 
              1157}, {2974, 2067, 2005}, {2830, 2005, 1248}, {3427, 3426, 
              1248}, {3631, 3761, 1315}, {1825, 1775, 1156}, {3631, 1315, 
              441}, {4813, 4814, 1420}, {3428, 3427, 1248}, {1482, 1826, 
              1157}, {2167, 3086, 442}, {2243, 2168, 2167}, {3454, 1825, 
              1315}, {6862, 7003, 1314}, {3427, 3760, 3426}, {5515, 6405, 
              2831}, {5255, 4614, 1792}, {4136, 5023, 441}, {2067, 2831, 
              1248}, {2066, 2005, 1520}, {3631, 3760, 1314}, {3009, 2974, 
              2066}, {3761, 3631, 1314}, {2841, 2771, 443}, {5302, 5303, 
              1826}, {1315, 1825, 1156}, {4137, 5882, 442}, {1775, 2841, 
              443}, {3426, 3631, 441}},
              VertexColors->None], 
             PolygonBox[{{4814, 4881, 1482, 1420}, {4074, 4001, 1420, 1482}, {
              5882, 5883, 2242, 442}, {6341, 4674, 1157, 1792}, {6339, 5683, 
              2164, 2768}, {4370, 6339, 2768, 1657}, {5684, 5070, 441, 
              2164}, {5303, 5042, 1157, 1826}, {5232, 4741, 2841, 1775}, {
              5042, 5041, 442, 1157}, {5301, 4587, 1775, 1825}, {5024, 6404, 
              2830, 1248}, {5514, 4922, 1520, 2005}, {5685, 6646, 3086, 
              2167}, {4881, 5302, 1826, 1482}, {4771, 7094, 3761, 1314}, {
              6646, 5255, 1792, 3086}, {7004, 5301, 1825, 3454}, {5686, 6647, 
              3087, 2168}, {6340, 6938, 3379, 1157}, {5023, 6983, 3426, 
              441}, {6985, 6984, 3427, 3428}, {6647, 5084, 1657, 3087}, {4203,
               5763, 2243, 2242}, {6983, 5024, 1248, 3426}, {7075, 7074, 1314,
               3427}, {6565, 4066, 2974, 3009}, {6937, 4074, 1482, 3379}, {
              5583, 5515, 2831, 2067}, {6405, 6985, 3428, 2831}, {4675, 5685, 
              2167, 2771}, {4741, 6342, 2771, 2841}, {7094, 6431, 1315, 
              3761}, {4066, 3988, 2067, 2974}, {6696, 6695, 2168, 3148}, {
              6431, 7005, 3454, 1315}, {5763, 4204, 3148, 2243}, {6403, 5514, 
              2005, 2830}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.4264649909698242, 0.2887406634204974, 0.7164284521449585], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{741, 1617, 646}, {4205, 5281, 543}, {4372, 5869, 
              741}, {1616, 741, 646}, {4158, 6667, 3110}, {1527, 3110, 751}, {
              5039, 5870, 1616}, {2358, 3111, 752}, {751, 3146, 542}, {4881, 
              4814, 646}, {5041, 5042, 1617}, {6647, 5686, 742}, {2358, 752, 
              741}, {2166, 1617, 741}, {1645, 1656, 739}, {2165, 1647, 740}, {
              5866, 4371, 740}, {5072, 5068, 1645}, {739, 742, 209}, {3146, 
              3147, 542}, {5084, 6647, 742}, {5021, 5022, 1605}, {5302, 4881, 
              646}, {2368, 2166, 752}, {4922, 5514, 635}, {3111, 1527, 752}, {
              752, 1527, 542}, {4200, 5066, 537}, {5070, 5684, 1647}, {5281, 
              4206, 543}, {4377, 4808, 751}, {742, 3146, 209}, {1645, 739, 
              209}, {4065, 5071, 209}, {6339, 4370, 739}, {3146, 751, 209}, {
              3147, 3146, 742}, {6404, 5024, 1606}, {5066, 4809, 537}, {740, 
              1605, 635}, {1527, 751, 542}, {5068, 4353, 1645}, {1606, 2165, 
              740}, {4204, 5763, 542}, {7097, 5868, 2355}, {5883, 5882, 
              2368}, {5872, 5871, 2358}, {5040, 5039, 1616}, {5867, 7097, 
              2355}, {5024, 6983, 1606}, {3986, 4065, 209}, {1647, 2355, 
              740}, {5042, 5303, 1617}, {5683, 6339, 739}, {1606, 740, 635}, {
              6983, 5023, 1606}, {5700, 4377, 751}, {6756, 5021, 1605}, {5514,
               6403, 635}, {1656, 1647, 739}, {5763, 4203, 542}, {752, 2166, 
              741}, {6695, 6696, 3147}},
              VertexColors->None], 
             PolygonBox[{{4206, 4933, 646, 543}, {5762, 5883, 2368, 752}, {
              4353, 5083, 1656, 1645}, {4203, 5762, 752, 542}, {5684, 5683, 
              739, 1647}, {4808, 3986, 209, 751}, {5022, 4924, 635, 1605}, {
              4137, 5041, 1617, 2166}, {6403, 6404, 1606, 635}, {5023, 4136, 
              2165, 1606}, {5870, 4372, 741, 1616}, {6667, 5700, 751, 3110}, {
              5868, 5866, 740, 2355}, {6696, 4204, 542, 3147}, {5686, 6695, 
              3147, 742}, {4159, 4930, 1527, 3111}, {4933, 5040, 1616, 646}, {
              5871, 4159, 3111, 2358}, {4199, 4922, 635, 537}, {4371, 6756, 
              1605, 740}, {4354, 5867, 2355, 1647}, {4370, 5084, 742, 739}, {
              4924, 4200, 537, 635}, {5083, 4354, 1647, 1656}, {4136, 5070, 
              1647, 2165}, {5071, 5072, 1645, 209}, {5882, 4137, 2166, 
              2368}, {4930, 4158, 3110, 1527}, {5303, 5302, 646, 1617}, {4814,
               4813, 543, 646}, {5869, 5872, 2358, 741}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.4537374477782373, 0.3361601004742224, 0.7547631894203644], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFU81OU2EQPVRKWvDC7b310p9A4gO40EiiAUoEfABcEN0aFy5M9SkKLNW9
Gp7AhQ9hFGiLiRtjBJf+VXcaXXhOzxgXX765c2fOmTkz3/nb9290SwC2ec7w
7FWA/UmgQWeP9moVOKB9PQe6KfCrDAxngQHPUQK8yoDHc8Au/b8JsFN27oh3
p2r7FnG6U8aZrAHvm8ylvUa8UQG8mQD6xHt4FrhUNa7iFfOAd61i7A3GTrfI
xe+sTT/PhcQx+i98ff+ZAV5MA58Y/6Uw5h3m5TVz3GN/l+l7Rt46fd8Ys1R1
X/3E8bNztoXziPbLzL2t565bNX9n3tfCWLtRo/qdp6/Bc5O+rTqwkJrreW4d
peEi/29l1rqgfTFzXW3aC4VtxQlDueKQT7yf+b/Eul+X3LO0kn1I7I9Nf5/y
bhXWUNqqj6fM/UD/SfO/Rrrl15w1J+GtM+9a4fkPEuswxkn8rTzxv8vNe8x7
mT0eE2eF9hOeH2XrtRY7MKRvJWKkk/T6Z/eCd8wTXJ3QVLmrqTkUL53aoY/2
Yif2TrVMhCaqZRjx2lPNVTPtxYzk3yTGTMv7Lk7FKfdtA7iSe3/FsR+aq7Zh
7Lxs7YcwtU/SVxo2ibmYWkf1vRf1D4jXz72nd1NrJH06qfdJ+JqV3oZ6F+co
3tF2xbHCH7+JxHWqxqtR5wbvTZ6ftKfmgXOF81SX6tS8xvscOghftYlXsfWI
144txbv4CyVMlmc=
              "],
              VertexColors->None], 
             PolygonBox[{{5068, 5072, 1648, 716}, {6666, 4157, 466, 2184}, {
              5697, 4375, 2365, 2182}, {5656, 5201, 416, 2136}, {5870, 5039, 
              416, 2357}, {6667, 4158, 467, 2185}, {4656, 4934, 1529, 1812}, {
              5893, 5894, 2134, 415}, {4934, 5655, 2136, 1529}, {4375, 5657, 
              2137, 2365}, {5021, 6756, 3217, 415}, {5869, 4372, 2357, 
              2356}, {4371, 5866, 2354, 3217}, {6626, 6625, 718, 1646}, {6622,
               6930, 3373, 2740}, {4921, 4376, 2366, 1519}, {4809, 5066, 1643,
               1416}, {4126, 5069, 1646, 2137}, {4923, 4352, 1643, 1521}, {
              5039, 5040, 1529, 416}, {5022, 5021, 415, 1521}, {4689, 6358, 
              1519, 1186}, {6624, 5893, 415, 3068}, {5071, 4065, 2065, 
              1648}, {7041, 6622, 2740, 3536}, {6929, 6623, 3068, 3373}, {
              4156, 5698, 2182, 466}, {5868, 7097, 3792, 2354}, {4808, 4377, 
              2367, 1415}, {7097, 5867, 715, 3792}, {5066, 4200, 1521, 
              1643}, {4376, 5699, 2184, 2366}, {4377, 5700, 2185, 2367}, {
              4352, 4810, 1416, 1643}, {5652, 4923, 1521, 2134}, {4353, 5068, 
              716, 715}, {4125, 5654, 1132, 416}, {5661, 5662, 1186, 1132}, {
              5871, 5872, 2356, 467}, {6621, 7041, 3536, 718}, {5281, 4205, 
              1528, 1812}, {4206, 5281, 1812, 1529}, {4932, 4656, 1812, 
              1528}, {4065, 3986, 1415, 2065}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.4810099045866505, 0.3835795375279474, 0.7930979266957702], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VD1PFFEUvSw6DuxGdmbg7eyskhjDlmpnjJDgRyykEaMxcTUYMQrGaPAX
LIWht7WRDRQEC6OVNJDYoRs7xA9AjY2Ktmg2Gs/JeVu8vNl37zv3nHPv2wPX
752/mzOzW1idWO8d9sisicMX3WaXQrPJXWbHE7NexFZxXsJ+Br9v43ykiLNe
s+8dZvWC2f3A7E1O8SOxcoixHghnumK2UTZr5VEDOf24P4u7Ldy5Ewi/utds
DncmkP8KOa2SYj8RmwrELdkjTOK/9fsW+Nd8/UaH8n7khE+8zrzqkOdgl9k2
vmuhdJIbay/ChJem2MEuYTzqMzsUK4fYvzz+Z+j4gvUXuJtlaaMu8l/AGsD9
y6Hi5PAvL368uwiPrzrVWw6Vx5wlYHx0qn0D+zjWPGJ17NXMbCWUBw9CceTZ
tNP54x6zbz3yYQdnN4vCpgfLHv8Pzn876W8ivunEifEVz+Mr/LgSimdbA/t7
Dn1ei+S3RZoJenaSeMCaQd471D+ayMO11Gw0Fl6AHu7GGgGeQ/7+ouZiFJip
Uy/Xgfk01TfniD7T7zHEzyK/4WeCvWQfV2PxIAf2tuHPyY/zwZmgd3Oefz/r
OmnjLDX9bLBn7BexHxbkPb2NK+prO171OdTxqSwtJdQ/4YTHOsxhLfb2SaT+
0mvOHLVQ10BB2oaR4yL1ZwPfQaZv9pZ52/5NEKP9LtgP4hP7dZ9iH5zmhrzJ
k3yZdzFTv1l/KVG/2esZP0PknALnWCI+fK+n/bs+5RTjOf2q+Xmgx+REnw/H
eht8F1vwI6p4f3DvQia9Q6jZnUkz9VEzNdI75hLnmtP/CHWxDt8Ja/HNPUuF
X3bympyph32iP88R3+d0b8p71vQ+133Ofz7ouLc=
              "],
              VertexColors->None], 
             PolygonBox[{{4566, 5280, 1811, 1751}, {5761, 5729, 2138, 2240}, {
              4374, 5879, 2239, 747}, {5322, 5732, 2213, 490}, {3987, 4810, 
              714, 195}, {6929, 6930, 3067, 1128}, {5652, 5653, 763, 714}, {
              6666, 5699, 750, 3109}, {6627, 5050, 1627, 3070}, {6322, 6323, 
              2139, 1129}, {6624, 6623, 1128, 2379}, {5653, 5894, 2379, 
              763}, {5280, 4568, 1752, 1811}, {4920, 6358, 2788, 418}, {4125, 
              5201, 1752, 2135}, {5661, 5659, 2139, 2140}, {4126, 5657, 749, 
              417}, {4198, 5760, 2240, 2239}, {5697, 5698, 2183, 749}, {5074, 
              4157, 3109, 1650}, {5660, 6627, 3070, 2139}, {6757, 5851, 2339, 
              3218}, {4932, 4931, 223, 1130}, {4170, 5067, 1644, 2210}, {5656,
               5655, 1130, 1752}, {4921, 4920, 418, 750}, {4657, 5654, 2135, 
              2745}, {4713, 5727, 2210, 2819}, {4355, 5853, 747, 717}, {4156, 
              5073, 1649, 2183}, {5731, 5730, 2212, 2213}, {6740, 6741, 2364, 
              719}, {4155, 5074, 1650, 2181}, {4689, 5662, 2140, 2788}, {5880,
               6757, 3218, 2364}, {6139, 4523, 939, 1644}, {4073, 5200, 1751, 
              223}, {5730, 4713, 2819, 2212}, {5658, 6625, 3069, 2138}, {5073,
               4155, 2181, 1649}, {5854, 4357, 719, 1627}, {4356, 5658, 2138, 
              2341}, {6626, 5069, 417, 3069}, {5729, 5728, 490, 2138}, {6621, 
              4356, 2341, 3067}, {6320, 4655, 1129, 1752}, {5580, 4057, 195, 
              939}, {5852, 5850, 717, 2339}, {5659, 4657, 2745, 2139}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5082823613950637, 0.4309989745816723, 0.8314326639711759], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVE1vjGEUvZ3OeN952zHzzsfTmdamY6KxGrWwEUsLXTXCgg2qLSIx8Q80
aRoL/oCNkVpIJCqEKRvGrlhQ4mMG8RGJsbBSrVhwjvMsnsx973Puufc5994Z
nWrsP5Mws7M4/ThfIrPaoNnzPrPXsdn5ktkzXDSLZq9i2V1nNjNs1gP+fgEx
FbPTSbN7sH/C/gTCdmi2DzxDsB/hbmdg9gC/t8tmt3CWkedU3mzK6Z4cW5x4
Wrg7tEn2NHyPi4rn2QXOdlI1bg9U5+602XyoXJPgrFZ0R1/Vv6UDnnc4VWAP
ovYDOHO462bgz+jdLdR/taLc9PHuIzhrI2Zbcfbg++Vm+cmzCjubVeybWBpR
H+aup5R3HtiFlL6fgH8iB93g34FagmHxU5fxQH7yv8DpZKTNca9P3SmGeGKv
hMJvQMvZvHzsyYZTX8bxG8fqBbW+E0lPfqeSwpCj6Xm+A/vN6Y75WBP9mUiY
RkLvaHudzwE7m5O9GMrPvjwsqgfUgHqxT9TnGPBzTljyDUbi7znlZt6jTjhi
qG0tLZ0voieXofPXQDPFXMRcQ57RirRlTzq+X+w5Y6l/CtwfwDmWFo4zQf91
+P8MqY5J1NwD/0pK+mwD9oL/HfN2Dr3LFoTh70TZ26E0oj4zmJHpEfX8JGr7
DUzfgOZpbyCeJvyfi/qmnokB6UZNiCXPpZLezPf+8DnoZ59Xiuo192DRa/4e
7+uP9Wb2ueV7vQ6/i7UfS7Bv4KyF4r0baVaZ/28kHmrOXlEf7sFyQbVz79p+
Juqo4WZZ8dSSmnIGuAddv0fsA/vJXvB+wWNyseaY/l/gWC8r91vEPi3pnayb
OanVEubrsNPbyc0cnAfuB/eEds3vIWvu+v1kfu42e8ceMm7V4/kO5uRbOEf/
ZzqpehnHveB/GPeHs3DE6X+KO7GGek/kFUsd2ZuGnxNiGUsO1kf7H4ayyQw=

              "],
              VertexColors->None], 
             PolygonBox[{{6321, 5047, 1625, 2743}, {4171, 5728, 2211, 3126}, {
              4073, 4000, 2022, 1481}, {4170, 5727, 1218, 489}, {4566, 5200, 
              1481, 976}, {5532, 4880, 1481, 2022}, {4654, 5531, 2021, 
              2741}, {4369, 5865, 1029, 737}, {5512, 5530, 2021, 976}, {5048, 
              4768, 1861, 1625}, {4919, 4057, 2063, 1518}, {6139, 5067, 489, 
              2572}, {5346, 4300, 652, 1861}, {5726, 5724, 2208, 2209}, {4301,
               5050, 1131, 652}, {5722, 5807, 2293, 623}, {5880, 5881, 2342, 
              233}, {4198, 5879, 2363, 535}, {4300, 5049, 737, 652}, {5806, 
              4195, 1518, 2293}, {4524, 5725, 2208, 2573}, {6218, 4567, 976, 
              1481}, {6434, 4776, 1327, 1833}, {6662, 5853, 2340, 3106}, {
              6322, 4654, 2741, 2744}, {3999, 6319, 2741, 2021}, {6664, 6948, 
              3389, 2787}, {5531, 4655, 2742, 2021}, {5696, 6662, 3106, 
              2180}, {6393, 4271, 623, 2817}, {7098, 5851, 233, 3795}, {5322, 
              4171, 3126, 1842}, {6947, 6665, 3108, 3389}, {6694, 5760, 535, 
              3145}, {6663, 7044, 3548, 736}, {3985, 6394, 2818, 1414}, {5852,
               7098, 3795, 2338}, {5881, 6741, 3201, 2342}, {4355, 5850, 2338,
               2340}, {6391, 6389, 1215, 2815}, {6433, 7013, 3460, 1327}, {
              7044, 6664, 2787, 3548}, {5321, 5726, 2209, 1841}, {6740, 4357, 
              2343, 3201}, {5854, 4301, 652, 2343}, {5731, 5732, 1842, 
              1218}, {4711, 6392, 1841, 1215}, {5864, 4368, 736, 1029}, {4374,
               5696, 2180, 2363}, {5660, 6323, 2744, 1131}, {6394, 6390, 2815,
               2818}, {5724, 4712, 2817, 2208}, {5580, 4524, 2573, 2063}, {
              5761, 6694, 3145, 2211}, {6320, 4568, 976, 2742}, {7012, 3985, 
              1414, 3460}, {6665, 5312, 1833, 3108}, {6319, 6321, 2743, 
              2741}, {5725, 4523, 2572, 2208}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5355548182034767, 0.4784184116353972, 0.8697674012465816], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VE1LlGEUvaXpfDnz6jvzMh99YGOEqZBW/8Gkla2yCFKHWplO22hlUIsi
W0aW7gIzadPYQh2MyKCiwIi2row2fdgnEZ3DuS0u7/M877333HvOfZ724fOD
Y9vNrAprgF1sMetOmu3ZYTZSNMu0mt1ImY3HzMZgj7aZPQnMOvNmlxH4ITJr
hU9v3OwgbDhhtgifHuQ44HkYfx1WRfwE7EJMPuvNZgMJ5VlJCpv+3b7uxPcM
sNayZh9x/qzN7FoG/livAv8SsJeQ6x3wB/LKVQvNHmL9Gr20OCaxeLbg513I
m8S/Gs6/FMy+FrRnbSk/Z42MZ+wn/P8MS2DfF6om1tNPHlDbJnzup83m0+Lp
N3CmUOcqsNbgvz8Qb1XnkDmXnQdi8v9kKJ8lP6t5znHnnPxOeOwC/H8BYwv7
eiNyNaqG/3uu+9BjOqYaWOvfhNaW1JoxkyWzZmhcRl+3cmZXctKEHAw1qYY7
yPEH/jloFKDG2/B506BeWd9so75zvn6B2jayiuee5zP41pPSk/wTb979eT4N
C4AzA30XQ81AAE03I83UnGORB2p4N6M8U6j/JiyeVp3sn71XkOdIqB4r+N9e
EB/fEPsjr/6pLeOoCXkiR+SH/C+7BieBfyJS7pfo+1VOdcaK4o59fM9Lb8ae
A+6pSBoQgzlZA+d0NFDuY1ndH848Z4Ezxb7IF+8LOedMHQ3UC/HID3VhrbxH
jOsC5lXXi5inHfctztdhGwn5JtyfPfWnNHMV5N5X1L4D375I5+ToUFJ6jcDn
QSTd0/6P59RwxXXkuu7aDaLm47D3TZr90ZI4ORzqnrCXe+Bnb0EYC8g95NzW
ERcvah44a7zfxGVPz7Pqa9qxiPs0EI/kkJyRO85MR0naUBe+Hys+c+SCPbD+
XcDfXdAbM4t6dhb0HpAjzhrvF3N2uRac07K/f49DzTfxGN/j7xvfvF6PZR/l
knr5CR3OtunO/QMaUL2g
              "],
              VertexColors->None], 
             PolygonBox[{{4997, 6357, 2786, 1184}, {6641, 5064, 438, 3082}, {
              4879, 4880, 977, 645}, {6356, 5579, 2062, 2786}, {4917, 4916, 
              488, 1214}, {6693, 6692, 2163, 3144}, {4196, 5759, 2238, 
              2237}, {7012, 7013, 2866, 194}, {5759, 4197, 3144, 2238}, {4586,
               5865, 2353, 2658}, {6541, 5530, 2004, 2973}, {6564, 5047, 208, 
              3007}, {5203, 5038, 977, 1753}, {3999, 6541, 2973, 208}, {5682, 
              6645, 3085, 2163}, {5513, 5581, 2064, 2004}, {4996, 7093, 3755, 
              438}, {5681, 6644, 3084, 2161}, {4711, 6389, 2814, 2816}, {4710,
               6392, 2816, 1214}, {5579, 4914, 1515, 2062}, {5048, 6564, 3007,
               1311}, {6430, 4770, 2861, 2860}, {5320, 4709, 1214, 1515}, {
              6426, 6429, 2860, 2857}, {6663, 5680, 2160, 3107}, {4369, 5681, 
              2161, 2353}, {5038, 4293, 645, 977}, {5722, 4271, 2292, 2206}, {
              5721, 4708, 2812, 2206}, {6428, 6642, 3083, 2859}, {5582, 6426, 
              2857, 2064}, {6644, 5049, 1626, 3084}, {4708, 4807, 1413, 
              2812}, {6218, 4879, 645, 2634}, {6393, 4712, 1216, 2292}, {6434,
               4688, 1184, 2867}, {5512, 4567, 2634, 2004}, {6433, 4776, 2867,
               2866}, {4688, 5312, 1185, 1184}, {4367, 6753, 3214, 3216}, {
              5346, 4768, 1311, 1626}, {5806, 5807, 2206, 533}, {5680, 5679, 
              2158, 2160}, {5864, 4586, 2658, 2352}, {6427, 4769, 2859, 
              2858}, {6753, 6640, 3082, 3214}, {6643, 6337, 439, 3083}, {4770,
               6427, 2858, 2861}, {5679, 4368, 2352, 2158}, {7093, 4997, 1184,
               3755}, {4169, 5723, 2206, 488}, {5321, 4710, 1214, 1216}, {
              6947, 6948, 3107, 1185}, {6754, 6755, 3216, 2162}, {6337, 6338, 
              2237, 439}, {6645, 7057, 3593, 3085}, {7058, 6754, 2162, 
              3593}, {6391, 6390, 194, 2814}, {6185, 6187, 2004, 645}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5628272750118898, 0.5258378486891222, 0.9081021385219874], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VDtoU2EUPtqkrW21t7m3l9zEV0EhbYfWqItrIaIigk7iZNKCWezg4FQL
RgeHUnQTtGIXF+2kk0Ss2IeI2sH6GlrwhQiCWl2Eit/ndzoc7p/z+M53Xukq
Dx89vd7MzkAaIEOB2WhsFkP5F+9jObN6q1lv1qwbsqvFrJY2ewDdNPx78O7H
u9t1ddevwPcXZBX+XxvNyimzP0gwmTHbmphth38Z2FfyZuNtZp8is8cZYd9o
NruPmDpipkJwgO0IflchS+A1BwmazAaahHsCfA8EitsPrBLkPXyGgDeP2H74
LeP7GbKwzmwL8ucS8SFGCVJA3lnkmkVMhNh8Iq7keQhxd0Jx2A1Zhp1NI8bz
TuFchP1eXnZyZ93k35tWj2p4F+HztFOxH4D5MVLub2lxZN455BzMKHYH+rMz
p5oY9zKr2JsZ8fvfw5S+5PAI9mnIixb1gDPhLJibHDhT2rqczzXkvx6pl/Rn
HvrQ1uXY+9DXVE59I17JfdhL2qgf2SQ9c11GD34n6j9zsn761DZid1q1K5Qe
17Nfw83qaSWnneA+0N7tnJ+h9kuQhw2qbyVRHbc4n0S49K15vcRf0/2EfXOi
egrOh/UN+MxZ40yH2UQk/F73IzfubeA7tpjVjMn3Hb5vIWcbNfMLri84Nv0n
2s2+QJbAZybQHnN/WNs59GusTZzWZngSdbd3aL/ZT/qwh4vQHQa3Re9T1fdv
Kq/75L0RizGsne9tG/xOEFdBj/pQwyp8n4S6HX4Tf08mwiJOBd9yXrP6nogT
cVhn0fO+wq29iZWDHMf91mgjN2JWkfdUpP7xpmf8rnugu9qh+2AcuRK/5PyJ
w9/EJg73qBhL/zpWDdTvAfe9oe7mdqj5cXYL6HcBOe6aen8wUv95T8dj3RTn
POv+zDPme3seOLVQdfL/bzDQ7vX5ffI252H/kajO0XblYy72jljEYU9GfL7s
CfF4s+VA/y/E+QcrZ8Pn
              "],
              VertexColors->None], 
             PolygonBox[{{6388, 5479, 1213, 2813}, {5682, 6692, 3143, 738}, {
              4764, 5347, 1862, 1859}, {5348, 5344, 1320, 1862}, {6739, 5846, 
              2335, 3005}, {4168, 4916, 1517, 2205}, {5320, 4193, 532, 
              1840}, {4915, 4194, 532, 622}, {4169, 5161, 1723, 2207}, {5037, 
              4565, 1749, 1615}, {4367, 6755, 3215, 734}, {5721, 5723, 2207, 
              1213}, {6661, 7011, 3459, 2864}, {4917, 4709, 1840, 1517}, {
              6641, 6640, 734, 1641}, {5277, 4764, 1859, 1808}, {5081, 4994, 
              1586, 1655}, {4505, 6387, 2813, 1723}, {4995, 4915, 622, 
              1586}, {5720, 5162, 1723, 2205}, {4889, 4890, 735, 711}, {6660, 
              6661, 2864, 464}, {6186, 5036, 960, 2609}, {4196, 5758, 1155, 
              534}, {4193, 4914, 622, 532}, {4774, 5349, 1411, 1321}, {6693, 
              4197, 534, 3143}, {5224, 5276, 1808, 1770}, {4550, 5188, 1018, 
              960}, {5582, 5581, 960, 1018}, {6356, 6357, 1587, 622}, {5196, 
              4293, 1615, 1749}, {5161, 4168, 2205, 1723}, {4707, 4918, 1517, 
              1839}, {5188, 5225, 1770, 1018}, {4135, 5064, 1641, 2159}, {
              4806, 5319, 1839, 532}, {4805, 6563, 3006, 1411}, {6430, 6429, 
              1018, 1312}, {5203, 5202, 222, 1615}, {5513, 6187, 2609, 960}, {
              7058, 7057, 738, 3215}, {5848, 5847, 711, 2335}, {5479, 3967, 
              180, 1213}, {7010, 4775, 2865, 3459}, {4996, 4135, 2159, 
              1587}, {5194, 5195, 1615, 222}, {5758, 6338, 2767, 1155}, {3981,
               4807, 1213, 180}, {6428, 4769, 1312, 1313}, {6643, 6642, 1313, 
              2767}, {5344, 5343, 464, 1320}, {4890, 5082, 1655, 735}, {4918, 
              5720, 2205, 1517}, {6563, 6562, 3005, 3006}, {6432, 4774, 1321, 
              2865}},
              VertexColors->None], 
             PolygonBox[{{5197, 4292, 4291, 5196, 1749}, {4292, 6186, 2609, 
              6185, 4291}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5831419112629008, 0.5586089777696386, 0.9099392016107432], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VE1IlFEUvTozzmg6znzfa5zRFuqmMG0TBEEQVJAWGVSLluIEmgvdtxFS
WrVPaaYgwxKyhbqKapVWC83fSgsSzI3UqkjKoM7xPBeX73v33Xfvueee9xo6
+y71lprZACwCe4tFR9TsbonZw6xZQ9psrMLscGD2GOtr2JuImT2Lm93BgVZ8
T8Fa4AthXdjPw+pSZms5s+19Zgs4dxHn78M/mTSbgh1B7Hlndg62XGZ2Hd8z
sLkyxUzAmmPyzaTlT4ZmVbA3Ma2JpQCcZ+OKDUuVu75S+UdRsz6n/csJ4WI8
9xt8DPESe95j2/OdRt0x4J5FnTBj9i8lbp6gfhP87fB3w+rLgQf+dtSah/8e
zvZXI8d+cURM7PtdiWqSL9a4iTyroXgexHoHHD+PiuthxiHvD+QbTovnV6h/
wanuUo3Zr0Bz+B1RDebfdOqZ/SbQy42k9pvhO+nE1xBwFYFvI659xhE/cfOf
eZZR8xbi5iLiJvD8NNahB9iJKrO/wDONmBmczcMXqxWvAbhKZ8Q311N+jivw
3a5Wr58xkwM4/wfa2IqJH/bA+sRBPK8590DxzDHp80z4f85oy6/5z/pRj2Ev
jjk5K3JH3mrBdw42j/21UGv+d2HvW0y9HIVvINR6BTyMh+KB+n3phLXHz556
WET+Jdh6heba6u9CI85GMKc2zLEzpbzMSW5Gs+KHM6aPGmhC7qsZaWAE+4cC
3cMOr4ndGPjGs5rFKmIP+pgCvkWnOfEu7d6PiPIdC5WTs9pJqUdq82dW+mT/
7IP5eX+DuPjk/UrH/X1PimvySb33V2pG7znnWq2px22vSb4T1BzPPsV/MVBu
4mVvxEy9fy8XtuPA1eukPc7/SkL5+SXP/KfmuOY+7wd5JX8Jr3Ge5dvQllX/
j1Crs05aYB+z/r2g7qkF8vMB+D9m1BNxEA/vGmfC3ORqw4lfcvsAuh3Jqd46
/F+d3r4XTj0QQ5/TW0CcC5j/Yo3yFuDvdnp/qA+eY84vTrm45lzpYwznlg+8
HlBzOlBdcvkpJz45E+qOmiTeNo+ZnPHd5VypyRb/pnG2W/5/0Oue8dQXz1MD
1EOP1zbvJ8+w1hz6GPJv0X9gotuK
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtk81L1HEQxifddjU33d3fr33tsOtJrEvXiAI7lEUdsj8gdoMsQu9dhIpO
3XUpV4gopPTQUapTrnrI1rdKCwq0iKhLCdJ7n+HxMDAz35ln5pmZb6k8eGag
ycx6kAjS6DAbQqZiZo92mm2EZutICr0HXz/6RWQ+anZ7h9lH9Heh3o4jexNm
Yx6P/rPN7CZY37Jmv3aZXQdjsmB2J2f2o9nsSjt+4lfSeqvTSJG3+ymz/djH
wKgmzbrBe0xzw+QcQT8cqvZ5fE/RnyAvsMew/4G3Ct459DnwStTL4zvbYnZo
t9mo527nO6dP9FdDNqg1i30wIU7OpRQ368ub/cU3C9ZQXLVT4Pe1CONAYLYI
v/fw+0z+EvpCVr24bxouz5CgSZzr8KmBkYxphpvEfs+Km8/ca/mMfDZe8xr4
V5Fb1PoC/gT6SfKXyZ0n50QoDt67+xYz+DPC+9qq2S9kpPsOvPY0PTyPqocH
4B0NZZ9GukLdgO9+Cru6x6w9EL7P6DXcX6Y1iyKYJbAuM6Mq9jj4N4gfQdZj
6u8N+3yb0y0EYHQxi7tZxfqOJtAns3qrYA9QfxCpRbTTBm+dBe3OfT7LD6Fu
w2da5q2CFOO6mXvEd6eE5TuK5LVT36XHnCJ3JanenK/fqu/Qd+c3+4fYV2lx
64+KWyfxva3iuAX2cFKz8RlVsGdC3a/fqM/2YSBsn+kMegHMrTbhzQWq4fql
qG7TY9z2G62DV8e+EBVmL3yWt/v1HeQCYXhuA3sVfS2Q7jfiueWEbsUxRju0
Y+/Hd7LGLoK0/ob31Mxt/E7o7znHEWotZcTNb2ic+vtS+mvO5z9pXqe6
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.6031934776423803, 0.5908262655337725, 0.9103963155645268], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VTtMVFEQHfnI8tBlYR/wWFCyWIAaCwtbKNZPsNDExMTGQqP4h85YqIX4
KaRTAQXUQkBFMJEGYiCRxsQsf8E/hKioMVLYCNFEz/HcLW7e3blzz8w9c2Y2
frB+T12amd3CSsfaETHbW2i2sMIsno19yOwL9s+jZjGctWO/FvblNPkkw/DL
NTuQYfYKPovYfwVOYpXu0r8Z9oYCsyEE6Ma+HT5j8OkAXqlvdghYFZlYWTjL
EMZ0VDiDOB/ydbYGcbOB2Qf7/gDneWYZ8H/kKYda7DOKzNbBvhO+I9iPY33H
Pgy8m8hhDDn8WGnWCt90rCmct8B/ET77YK90OWxHbvNRvXEpR2e3sX8GewVi
X0xTbGIwzyncq/Fkn8G9by63tpB4yIf9JWw1gfwmEPdnoLc88YR/GP4tyPFy
gfCWwOFAjt7MHEbC8uE36fY+Yr3GSmYqHt/GnM6ihqcLFJf+o2Hxs5yjPfG2
AL/V1bc8V370ieLu22LlPYvcFh3mZsS544uHbbi7FetCSBhN2Lfgeyxmdjwm
2wTeN4U158mP/uS9CvjVhbKVg5OTMemJ58RlzmUpbPh/8PSlliqBd8XxQxt1
RjtzTbi7R4B3NKacqKt2V6OnsN1AvH7HJd+c0m+508+cqwXvTCPH3xHprs+T
zuhDPhpCwp4MxAs5IbdnVsuH/fBipTRPnr2wOKkCf9W+zrJwbzhPPsOIU+NL
P7vw3Y01A59rJWbXS3Sfvuwl+r9Bbn8j6tFe5NATuBzx7Q6kqXv4VuabPfSk
66TjgnpiD1PbrCvfQX7+vylLv9nr8RJxG41KZ6n7Seff5anmrF3C6YH8s7bn
XH3JJTklP8Se94SxiXkF0vCJmGrGelEL8Txh0nfO+deDjzpf3JMn9iG5SumD
7yD3fa5GHYFiEL8ZeL+wbwJvd4ulA2rgPfbvilU3Ysw6nHK8ezyQbQG+p3z1
FftmxPUOf5c5PecXqq6sKedOm+t3arLN9RfvMk6qB4lBTjgnOMvYuyURzVly
zrnYGdFsHCvSrOSenN/3hMm68je5Yh26nP0q3tSI1ZYjXcedzjeCgweI1esp
18eetMSeLo0oH2q0xeVciz47j7sDqFvC1+zm3E7Fo36oJc4ozjHmRT3yLmcn
/0vow9nKGcz8yQf7gT49nnIhJjH6HQ7nYmOu9p/B/8eYZiHz7XE5s//5Pt6l
7hKuFhvwxs5AvToK3ibd/P+DXNKLpPtPMeESkzF5zlicDYNOG5fw9vX56mNq
jj1BbVDPrB11RV4Zhz7/ACuAFnI=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtI1GEUxa/O5Mz4Kmf+zvyd6cHoQntsg9rUIgpskRBYbYsKK9JV0aI2
2WORu0KENIoemqlBbnShQdCmLF9p7xeFCJHLHhTU73JmceH7/vd8955z7p3J
H2jb3VpsZo1ElPizwuxxtVkibrYqYdafMnvEt3OAJiNmfaHZ+qTZUKnZIBFm
zS7nzEorzTaWm90lvy0Aa8Lc4V6VNttP8QHuvdxj1ByP6L33+kv91Qn13MR5
C/iznNdQbxD8uqTeDhNnyNXRc3u5ME/BP6RfQ8xsX4lZhtwOvi0UmXWR38x5
jHw9+XbuHTVmn5aLX3eZWRO1p+nxkdqfnS/4E/DrhN9BNF9B2xFqXo1LY1Mg
T9yLefr95P2PpLD+pqvK7HnG7Bt6lojWQBy89xL4CLlaML+LpXkXb9sC5T7Q
/yJ+XKoRt9mYvEsST0rkoWtN0X9imTS/BdtDjVMV1OTNK3IviUUT5h+9U7xv
pn+efjfAXydGwI6WycvT3qNYnn7JCuO5X9z3pjVjn213XN4chuNoTHrfgM3R
o6dIPdyL2Yy0uydz4OeJaFScFtG+QL3vJfo2xvlZwS/n1JLVDH12nfTckNTO
+a7cJ9ZyH2EmE5Xy4yRcorzfmZCGevKv09LaB/5eqB3x3XjA/XigHfPdcg7u
3VCo3XIPp6g1mRF351QB98rCvP034LvTW+DjO7SSWu9q5P21InG7Tf5QVByn
Qs3YZ/ueGA+0c75rvpNbA2l0bT7juip57l67pgZyN6nRS76fOIo3x7LyxnfS
c67ZtTqmA35zaf12h+E3S34mFDff8Qv49YIejaXar3pyDaHOrm8G7dMZvXX9
ebC3QvX2nW0M5YljvUY/94FQ3rrmZnrvSeu37L9BP3/Napf8Wzv9z1drdv4f
kMev2py88R3qgX9Xtfi4v/8B2iCw9g==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.6232450440218599, 0.6230435532979064, 0.9108534295183104], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVE1IVGEUvY026mOwGd+8cWZcRFipUeTCRYsMahEtiiAywZVKWWYWRLrS
BCUKbKdmySxqocXYIqMiqDDMjYo/CJULpaDoB0JrlbnqnM4HLT7e9+53f849
92db46UTF0NmdgMnB6eyxKwpbvY9bDacNHuO/1HP7GRgVoOTzTMrD+s7DP2W
fLNQrtnTTWYzkE3jPIa8DDo/CswG4LS7UG+UD0H/Kt5ewKY0bdaZMHsJ2Uyx
WX8MJ0ffyUD3nkLZD0R0OmC7DsAHYLcvajaHuHXAdydfPmlDDJQ3w38/8A/C
riusuLS95Zv1bjF7DTxp2LWHlcMq8j0dCHMZ/JfjdEI+Bd2SuPKthW4zbFbg
pxz6ZTiViD8P/AvFynkHcO1KSO9PRPh+54jHxTzFLQa2czhVEcWg396QcJDT
5ZBy7XKYjyD+AnhZxP+FfOmQcx+5jMdlu7jZLJMr2yeQ7YxKh+8zUekcwj0M
mwnoVgDjM1+xabvsyf4LcH1DrA1gyyD3rzgrJh57XC3WkHdjkTh/5CkH+mfO
c574oS5tNkLCO+sJ292U2b2UOGlz3FPeBO7bfPVXX8z1hKsnv+wfcn0U7+P4
z+L9iq9eY48wHmOxBssR1ZFY+UZb9mC30xl0/URuZ+HnVVo2rBVrxn76DNmp
hO4fwcHNlHibx32vL59J6NQm5JP5/eORPRZTPxHDQ3yzSfHUCl8FEdVlDrL5
pHhnb1Y5PD8h+4Wz4YmnT2lxdTsi3OuuN4ifGFahs5aS374SYSKeafBZXaTa
0D/9sV6Ho+o/xmvwVANivuyLU/J5P6mcmS/nrDX+f9a4G4iHe+Ks2xUPYDsY
qDb8r3OxGL/MzdEH6GyNCXsJ7tuj4mvM02xwLs4EwkEMrBd3A+v/HrIlX7PK
egy5ec/zhZU4uasqiuSrBfmdT4uza/DZE2j2OAejSfVqBraTMfXSfuRRjTPl
+pFx2MOdrr+JrRccXw+kz9mo98TJCPwcjCv3LO7HcT+G+5uYYlCfPlvdzHL3
LLn9Q99jzv876L71VQ9ySb/0mYuatAfio9XtW2KbiCoeY2Xc7iMe9mNpTD1A
jA1u7qhT7+7cI7TjXHcktIvJNfu9xvV8reOAXO2BbHdCu4d7ZSyuXDivI1HN
7Jzr7xUXl/GIh/mkHe6/pXL+iQ==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlDtMVGEQhUfAuwshYfEuy+5iokYNoKIUlkbFwkcimhiJlYkSFRHRRqlA
G4kFdjx8xMIGo2ABRm3AqCsNKA8xKokQ41sSA1qpVH4nQzHJf/+Z/8yZMzN3
Ve3ZA2eyzKway8H642YVCbPTUbOHS8wOJ81mC82O5pkdwT6nzb5i//LN/mSb
rcT3i5gFfBcCs9V85xSbvYhwB3D5MrPWIrNJvnuJacXfFJqVEduH/ebtPDaQ
4xi9nO9hfZzbeP+9wGw3nCaJnYFfD74KMPvzPOYxXDrB7ITLNJxKiR3k7v0i
vzfkmgMjTT3nA+f+Bbse9RqG4DaO/z7YHdle69vQY1VzFXjPuXuCby93VeTe
iD6NvB9Tzbw9UeT1dIPxktrbCx3rJjW94nwn9LevsTpyN2DXyP2X+jaAtR5r
iLrmx8A6jpUGXpNqG1isR9+PwFobM6sn9hAx2+FTwt3kUrNR+EwQf5fv6sA1
awKrGfzBqPekjHN5wt+2wO9HofdYvV0gx3zK7EHce5/Ldw2xu8h3I+qalRL7
LOb1VPKmLu6aS+vZwH174l6rYjbB5QPWHLjG4iZN5BfH5cR2x7xezYO0nUt5
bml8itrrsav5rpmwMzF/K4zL0j7pXNSj2zHHFJZ60gLeNmwk4hoE5N+Cfxhf
Bs3OhT6TiheHFZw/cXcxcM6anXd8NwU+Q9LiY+izLk3EfZye/8z191P44thM
lr/RrB+EY0/E+dTSr8oS10ozfzLtM6FZ2Ex9E2CNFvssC1Oz3LU435qBGrCG
C3zWhKlcYei7opyalVsp74fi91PrvrjvjjRr5Pwt7b0eo8eXwOvi/VPzHVVt
O2PORTVOkWukyGdFMesSvgOafc3gENx2xF1L6X+F3G0pP+tO2m9NeC71QLvb
nfR/gXZ4DbnG+J7O8xnuQJv2Ev93qOfjSd8p7ZJi1Bv9c8RNPdLuaAc0+9qh
DL5I6LnV4/9qIcCf
              "],
              VertexColors->None], 
             PolygonBox[{{5016, 5508, 2002, 5507, 5015}, {5181, 5016, 5015, 
              4546, 1737}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.6432966104013393, 0.6552608410620403, 0.911310543472094], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVEtLVVEY/crr8XrR9HjuSyW9QWFPFRoEva0wq1kaVDQoS4wGNYkGRWoX
+wWBZU4jrQQzJUqCwJS4atpjIjmIBPMBaY3MWWux9mCz99n72+tb3/rWPpsa
b5y+vt7MWjGyMDpDZm0Yg+vMPmabzeaYfcLBxUKz6sBsEXuZiFl+2Owf9u8U
mE1i9OPuFOaRmFkYZ2sRxc4BZyFbe5eRqLrUrLHI7FS+2X3f7CYw7wHnMPZ2
xLFG7rue7q4ifiVp9jcpvB6cd2MsA6sL9/ZFzY4h9gtwnuF7AjhleWblOarj
CTh7ceUdA6/pAvHuwLoL65+I24bzrRhXw8pdhr1WzK+APRAVFvmTE/kQl3vU
qAz7Kbf+AA41iB9H3CDmlWKdTYL7ZvAYgA6j4DgcSCvWutPV+wtzPu7PAP8k
dHmDe0MhcWoOqxerqP+tp/2pXLPebOnGc/aDvSD3a2HxagnpjPvDuHcU4zbw
94LbUKHWHeDSHtPZFuy9DpSL96cj4lkL/scxaiOq+3tE9ZYA51KR+HD+ndQ6
g5gLjg/9w3vsSw3Ov/ny0jmszycVl1VitqdQfetJoI8Yk6hvwlNvyfOrr/u8
S1/QC+wF6yVX9isCDed85aW3GEPv7QdPLxAOdaWO5NmAum857y0h30KufMJ5
MVf9mnHfXL9H/kNF0rUSuWqwTofkJ9bMvKWBfMyePgCXiYSw6P0lh38WOAcd
zkbca/LEv89TT9uAUwfO8wXSrw7xeYHW1H7a6U9vpd1bJUdyJX4F7jZH5ZUN
vrzFs1lglPvS7XGg90N9Wcf2uLDIdyopzlV4p5Wl8mOr8zr9QH+lnO4dwHtX
olzkkXb/jSZoeyUmvVkrtWO9HJ0uJoO98aTqWUZMd0L1vwT3vqi0eQSOB6Ly
J9883yFrpAcehtXf3Th/7svj7Cu5cL8eNTXEFUeOFU4T1kLN2dNdOO+Pykdr
8OB8XGf8l3z25Vt+Uz/G8z836t4v3+tTp20vuL9I6F1mEHMmplpYG3WiNnxn
rIe1HEHOEV8e/wMNUq4v7ZhPxOSBsaT+rcxFP5E3OSwXy39p975bnJ7EYI+J
Q4x6h/MDe1WBtKPf+c+l53sC/TdZ439njuaM
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtk0tI1GEUxW8+ZqZBmxln/uPokBb0plJoEVRSVFgUbcqgIijsYbSoTW2C
Hka7lsWI4a5IM8HMiHIVKsSoOUEbqUUPEDVorFW563c5s7hw73cf59zHt7r9
6tErZWbWilQgI3GzbSmz0ZDZDRzFjFkuYfYW+2/U7FrSrL/WbHq5WWe52Tfs
s+QsVJrdJKYHe4z4SfT9SHuN2SI1lsg9XG0WSpv1IcWI2Tnqn4mrhufOU6MD
7IvI7DJhzsTM7iP5sFk1OWuIzxLfCdlhYjZTawvSENbbLNgnwLuF/g9+W7PU
Jv8N/kPgP4b7AFIo8f9I/EHw3oM9hf0cXxHOA5WKyeLLx2QPwud4YNYWKNff
tsPnJTGN1G+oMutNqyfvxXvshuv1QPz3kf8U/5O4ev+CPPRZgTkP1k9kHb5X
1FtF/GXyd6LvSGkXPs8W9F0l2+ttot5az2EWd+n5k/eTUT8f4PisVj07X9/Z
HPErEsK+QP5r+G3k7VJENcYDcXIuEd4K1JrOaNYLvPUn1JNj+41Uobfib40K
cwz7BfxWwv82fCbxTSBD3NZMVL0+SKiW9+yzOYn/dFQzOh/o5vzWfGd/0H+7
XaYbOoDeGWhXjjkM1hH29TmqHfjtNGd1a35D6+mtBX8efwf9nCI/mtZtOOYv
7KasbmOkQreWK/XnfLrA6ksK7zt2E3ohptn4jt+RX16v3XeTvwf8R0ntyv/A
aFIz8Vn4DJzLRKBbdk4b0uLo3HwHc9TuQX6ENc/FOuZRp936TXiv+Yxm6T33
Ms97zPNO6R6bwTpGva6I/uTuGnF0bv4n6uBXn1KvPp/xhP6s851ChlK6Qb89
/1NtaeV4rNdsJP5rUr37TkLoe1PK9Zv03eQCzcpnNohvKa5b85v4DwwKrB0=

              "],
              VertexColors->None], 
             PolygonBox[{{3965, 4979, 4978, 5477, 1976}, {3995, 5031, 5030, 
              5527, 2017}, {5031, 5184, 1739, 5185, 5030}, {4979, 6056, 2510, 
              6058, 4978}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.6633481767808189, 0.6874781288261742, 0.9117676574258776], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VUtsjFEUPqaPebQznelMpzPTlU0TC21RJEo8o+KZYIEuKipW6ELsSDut
DbEgoqpNSKrpiyDEQmPRqorOQ6LoIyxs6KpWCFa+r9+1uLn/3HvOud/5znfO
LD/RevCsx8zuYhVgTSXM9pWbdRea7cL3+wj2gNm5qNnFMrN+2BzF+fkKsxE4
ZHHXhNW9zOwXfqeLzMII2Am/RcRpw1kO9xmsPGx6EGdLzOwt4vjjZtWwacZb
ZxB7EqsL54dwXge75zi/gb0Nb71AnFXwW401XiyMT2PCeRK/s0GzCby7Eb4N
WG0+s+Nhsy/wj+B8JfYarAXgWwO/Udxtg1/aJ9zEXOU36yiS/Snced1dT6Hu
iX8qBK6Qyy185/BmEJhr/HpvBuevPLK5Wqpz4iK+SZyHwMkCsNX6lUNfQBxm
Qjoj/nXANQp8OcS/nDS7gtVbYjYI7L1l4u0J7teHhYfc7/eJ/weVZvewMoi1
G5y1wrfPa7YX93t8wlwL+3c46/QIP/FNOszMhbmxHhlX00vAPIiYecTswf03
rM+mGhIT8bQUiy/mXluFukMfv+A/hn1zueo/ByzBuN4iFnJNnon9Z4ne+gCb
j1g/SmT7Nyz7GZzNJmXXjLPBuHzziJ/D+oT4aeAcAM4c4u7EWWlUOTIfxmPu
fHfR8fAY+Ceiwt4Orm5GpQdyQF7oRy13u9rX493Tri5/8P0opnqtCCo+7QPA
dSyhnNgzWwPCcBuxRyKqdR38EmHly9qQ1yaP+M46zlm/nDtPwT6J9RDY5lLg
PaG4B3zSFTlnLy69USD9sm/ID3srFFc9PeDGF5cW+pxmaF8Ev7GI+mvGxWVM
6peaWNKMXxqlnp8BQzvsZ3F/BFi+RqRBYmoMSBvTMfU/e78gJR1QA2ngbExI
f5OweY1V4xVm7sTd4nIit9thOxYWh/RhH1Bv7OHZgOrCmVRTpbnEPDgvmMt3
5H6/UhxSm+xjxmVPs678fb1SfcI6ML+syzGEeMEq1Zbcsf/Za3yv231ngC2L
NR9wfRRSLw1ValYST8b1Eu35Jm3pT70yB8bdhJp4UppDpfhOhl0NEGM+qRoP
e4WRs5E19Lo6UheH49JGPigc7BfmST9ySFw8/z+jmDc1TB/yzRqRS77P/CbA
24Uy1aSrQjUj95yj7D3qinM6EhMexs67fumIqPaMyT6jhqgf9hXnHfVGPZY5
TXJG/A7InlwsuP8OzpjZlLS0AfEasN4UKw531pdzbtjNujvAdc3Vmv2xw/VI
NfbxiGZeP2wG4sLFPl4bFg+cMdNJ8UYs5IV4OAemnB44j+udPXXPN1lf1pu6
Ic9DiF0c14wh5xmnB/Ld5npq6f/RpzicPS/d/PkHJTkdZQ==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlMlLlWEUxo/j1et0J73TKoKgRWpFBdmAFWkoBdmictFIu/IPKHCoTdGi
FpUoFIg4FRVFiyIoK8P0XiEtBwpalZtsVWG7fg/nLg687/ec4XnOec+35nTH
4Qv5ZtaGFWJvomYT2LliszPY05jZEyybZ9aLQ1fYbCBuNlVqtlBu9gjsccx9
35IoUGNWhLWUmLViC0mzevBe4v+UmXVwP19ldptaAwGz1RBxEbOeIrM0OV9w
347/JPn2YK8SZrvAp4PUxDaBreU+wHkcvB6uJ4kJU/s5/C6RewKf2oDn3B3x
HIrNYK/xXce9jvPlfM99lHt70Gu8JzYe81zK0Qg2i+YmsEasAayfmi8LnGNX
2mwOTffAshVmt8A2xpyb+JfRi/mkaz9IP0bp3Qg2aN7Di/CdosYS8d3w/cl5
ssrxv3y7j++DXL/b4Tub9JqqNcF9C3p2UKOT3Bl6vJAyW0w516/kaCLf/oRr
/YieVvTuxL9bs8FnJeI1VauzwLX3hP2sHvzAt5J7HfVrsbmw90T5m7GZKtcg
7gFy1sKtLu35pOEbWFGuXzPcP8M/Dz134PqbnlxLuo+wfu7DYL8irjVLvXHq
9VQ7d83sOLWPYWMF3k9hV5MeK59yajUnnJs0nyJXCk0PmcUq3/aB7c31Qxo6
ie+q9lmJYxvnA9X+NkcDvgvSKG3aiRvwuxl3XByH6E8FHK+zCyslri0/7m9f
GofBR3L7oDd6hHNbjfdWnFYSzlHc1LNtzLMg5W9VO9cCVo7/IeLn+baE1sWk
z/4s8Qn8S8FPFPqOiYtmplmJU23UZ65ZLzOjfu1erp/qeUON11BuvaEq7iGs
r9BzVjDLT0mf1Xre3Bj5N8e8X5qBfLNo+BL0GP0buvHJlPqb1a5qBzQr7ewQ
+N2Q904+xWCDIe+derTM/V/I/yV6c/qX6B+hf4PwaWp9SPi/SDuT4bwh6trE
4Tu1g7l62mntal/UtWq+mu2VsL8tzVC79izl/zK913zOW0O+S9oR7XZl2rVr
x5Ng76Lee83kP2w2wp0=
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.6833997431602984, 0.719695416590308, 0.9122247713796612], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1lkts1FUUxk8rfc1MO52ZduY/LcbwaBdoG5VoTMCEgkYSxGAXbnxgcKHB
AAvRjQqlbpTEV6QQTSgU29JEXZpgkJciISBqAC2udGHEtMWNCXHp9/O7Xdzc
O/ee53e+c/6zbNuu4Z2NEXFK6w6tC00Rv7dEXNaP4zrvy0fs1PlYFnGpM+Kp
5ojBLr2VIk7rfLMc8W7Nst/p/o1ixKeyc1tGt+n9G+0Nev+5HvFYW8T2SsR7
+n2X3o5oP6p1Ufe7ZW9td8QO+ZruiDixxDb+0l7L2+a38vVvp22O9EZclc1/
9PaQ4v1c9v6U7EyLbe+RnUcVc77inIgFP2/qfqgQsU7rI8k/LpuDkjmk8xnZ
zMtml2zuUB676s7/Rk/ErNbf0jmu2JbKzh8NEeOKfW/JNid1ntK6rLd++T2r
ODfJ58HWiNUFyw8rv1eKjvGi9gXJfabYRnV/TnZOyc687oar1gOvpxMOKxXn
Ub09p99Ptnr/pMFyW7SKkulQ/NcSJtzxht8ve+wPX+dVo62yNaAY+rTuLDh3
3ogd3MYV+3Sq0Zhim6o4F+K9Vbbsg5Jp7LH+Dfn8Vet28sv+sfy+qpzerpk/
6A/knONy2VmR+Xebcs1ka7P0jkl2pOQYmuTzN8V8Tr7er5lncGyD9B7R2pAz
F8cSDtjn7pJ0R2RrAez1tkayP5bMZ2QPJfm18vtAp2XPy1e90zHDC3LgPNxq
rJGfbzZfQms653if0bm/yXZvhn3uafVdp+wXq7ZxTRhe1arr/JMwH5LcmTAP
1xdcO+pJPvjCB76IYUR6e7utR1zrkjzvnBdjHkr3y1ST5T22+4tqklcMzy4x
Z7ckPpxsc22py4IwvlZx/PQVXKbXauLJhS73FTV8rWR5OAOG5Py8arKq173K
Gms1V+EWv/GV6W5rzjndI9m7tTa2u4+bqva5qt04g+eU3g/rbS7hiD668Jqe
h9uT2p/QerHdNlqSna+kd6BifMEaXfJ6QfjNZcanseZ5RG/dUu7XU+7sYMG5
qPjPNhu3+4TBWMX1mm42B5lRE7LxVtnziZnAzKJX4ez+unkLlryDJ3Ht7nZs
8ITcyAuO8pvzS+2ePZW8/U/kjAnYTKcz9VxRdmz75Wu26NiyvOck/gbk6/vM
XMIeODEbqcN88oUPaktszDVmJL1zWHqd1Ek6OeUxWXUtiIU7YsD//V2OgZnU
VzZ/R0vmDXbR+SLN5H0l9zZ9PdPhGsJHegU+9af5DF7U8aRsfK3V3+KeYIdv
79Q9K5kh5Ape5NGnGFZqDeZsqy/J85s6IP9w1bMe7maJW9Sa2s2kmX6vfM7W
jT3fNnhAjv/Py17zmu/J+sx4nU4cAX9qBI/BAczg8Vya/ejRC+Q9k3gCX/DN
/RW+Id3Gv0P+2+uu50ySQZ66LU3fLGQPJPmXFfP2Lr9fR7fX8rXEI+p7MM0t
ZsVq9UhDj219qJp8UPM82KQc11Qty2wDA2o9WvZ3cZGX7NT0SuYZx3yj76kf
vU/tK0nmh5LnL7OXnkeOuQAv+GYjQw8V6ubtRObvHBguYoTNcd0fyczBE+ke
3Pi/sDH9Z9jc67kA11+vuufp90LVuZHXf/PiMAE=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEts1FUUxg/ttJ2Wmc5M+2/nP9MaodIuwDYqahAxAREhEFS6YAMqYYNB
gYXoRh4triBRMVAMGEAEWhJgSQLio5VneAStSnHnCgwPNyYsXPn78nVxknvv
Ofd853znMX3tpv6NNRGxDskgY6WInjRiWX3ED7URt7gv4n65LuI6902ViKVt
ERs5D06NuFSI2FDweQSb7dgfKUf8jbOt2FSKEbn2iH3c90+JeApfnal99zZF
FNA1IyuyEf3IB0nE8/zZxvkhMfwK1j18vottytth/h5EZuXB4e0EukOpz6PY
d1UjPsFfgaRezUW82RGxoyUiIb73+JMn/kHuZe4p8lsrAkYF3/fCsfSgH25y
TA85HyOf640RJ8nvG86Hys7tKm9zsZ8G5gKw9pBjH/56kQPk+hX3LXBzAR8/
E89R/O8Fa6jN/F7Dxxi2h8v2tZa358j/WeQnbN/m/13+H0D+anANxon/P/jp
xP5f4n8Z/HnIQNYcf8ffF9E/QTzd/GlAl0Ue1ziGj+Hri7JrpRrKVx79W1n7
LMDXEfh8B18z4Wui6hiE/U/O3IlTcSkO52N7p2qd8rnB/WbqWoqD42C9Tr7b
as1hKXHPCH+YfNv5e5G3XmKdnbNtP3KiwX/+JN+Lre6dx+AN42+wZN2T/P+M
+2buG2rt84Wie0i9I04+BGt9q23l8yV057BZwP0KMqXqGqp2neDvVn2Ltj9P
vOuxn1207gFvm4ltV8H1kQ/Nwpdl24rP77FvAW8Ifz3gfUps47zN4Xwa+1Hu
dejH6ox/Dd/nE9uurHevv58YSz2vWVBNVUvNRDf8f5vaVjVS7e5UzI1qeB/d
A2RfdnIGVI+Sa6Ma/oHt78gq/i5u9Cx2I31NnskRbL8GrwDej2D0Uus1YJ7N
uEaajUYwl2c9I/3trqFqJ8wazq+0u/fVgxNw1ZfYlzjbBfbOirHEgXqzpuxY
1KNT0S0Hc13eM/sG50be7mb8liaeSc2i+Bwg3jNV39UTs7BfBGdDxLIE+/E2
7xDtDs34DHLNTPKvnGdi/3SHbZXjcXRdqXeTZlSzPb3qXaIZH8X3QvQLJ/Wv
pZ4B8as31f5RwbtEPaDa7yDGU/Xugcvq/8S7QG+fk/tHJfeOei5PLM0dnm3V
aAh/A8T/S4P5S8F/JvGuUw1UK+1A3VWzHFw1V7zbtIOVq3pevaac97S6B3VX
/4rbJuR+1hw/anHPiltx2tXinaBdsDpjbrWztKvE8cGiayJ7+ZjA1+2Kd5di
2t5mTGEph2KHfcqXdvj/rmvaFg==
              "],
              VertexColors->None]}]}, 
          {RGBColor[0.703451309539778, 0.751912704354442, 0.9126818853334447],
            EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1Vl1sVFUQntJt1+7S3e12u/ferjG0WJQWsS0PhlqMIRbig5U0+qChSoj4
gBFjKBApovBgDf4biBosUGhD0QhIsAX5aYyYqqQhSNAUE000Bvx7oyltH/T7
8k0fTs6558yZ+Wbmmzm3Zu0LHRvmmNlvGMUY9ZHZxgqz4TKzG3PN7k6YjWF/
JGM2LzTbCeF80uw7nG/DhS9iZqNY/15ktjcw2467W4t1Puoy1LEQ4yXc3YTz
tyF3GmdfYpzEeAz7u6HnmzLdbag0uwNyE9j/EHY3BZIJC8CA78slZh9jrzfQ
nfch/x7Gg6Vme6qgC+PXuFl33qy22uzMbWYzObOhrNkT2A+x7oWeJ6GzG3bH
MZ8A/o/SZn9g/Ax/h+O6N4T5CvRdgf6N0N9TbvYaxtPAUAPd8zBedjnOF6Fn
AezWQP8z+G7HelVevnwFHW0ZxaoppRgTA2PAeOyB3S74/Rb8OoXvD+Ka6fvj
sL0O+q5hPRwTvoOQvwxsP2AUYe8++FcHW1tiwhpA/j/E8x3o21yheDMul+LK
6S742oW73+L7AtaHq4XhNDAuzSlfMWC+M9S6D3peqVBuT5Xo3hHo34q7YxjH
cfdZzOcxerCuhZ7zoXCvZ/zS0t8Xl2+UYQwOxBUH5qcxUo4+g/35GeVlGnan
MJIJ+T3kvi/B+fqcYv4n7NzAuAmZNHhSC139WLdh7xIwt2F9EOs67A8kpPuT
nPQP4nwv1oPF4h/1U2c98DZgrCkRphcxdxeLF7M5Jxd3O4e2I5avR4pvXygc
xHAMNhdgfzVkO0s0Uw/jsi2mnE6npP8pjOOQn8xJrgd+vxmII6ytkSphvJoQ
92mbvGcemKNxryXiJ//Im1k+kAs8fwS4kpH8qkoK03XI3A7M80PVawswnA1V
H9RHm9TZiv2jOeH+HOuFkTBPpqSHfs36ydpnrdAv1gs5RT4R57uI6y9p8XAK
d2+lJDuTUixYHyvg60Agfkx47sn9HdCxFmdNpeoT7V5fzP2npepJK8o1s9bI
G/KVMcqD4x156RmKaSaXeH/I1/2w+U9WnDwC/TcD3S3yPDMOjSn1gmxSHCFX
GINlwLUuEr97Eb/9GAcSyjFjRr61YO8BjFbsj5TKD3KP/ZKYGTf6sdpjOAws
90aK4z7GI5BsC/wo8v7DPNPGrjniJfNFX65XqEcQ+6GEuEAZxof+TjhnyA1i
64KeiyXSlUWcz+b0zV7S5Ry6H3jOhfKHeWtw33l2zXtpM76XpJR3zowT1+TU
0qx4VQaf2gs6I7ZF5eohK8uFj7njvNKxdjpfGZMU7s6N9D2bb/pCDhETech5
0tedzlHynHGkHPenPI+shWWh3jm+cX9j/Vco3u0E3kcLursY8fgpLT6xlzcU
hJMxoD3qId6HHTP7A2uVNcReQC7QR+LOuy87oL88kn7a4TfXY8jdcmBYTl7n
VJOMG/sL94izo0p1whphj2v1/edyyj3zzlyxbpkvnj+U0BtH3hEPuccc9Pi7
9mO1OEpZ9tGM9zHW+oms6p09lDkjl7Y4t+gXc9jva8abdUFfFkFHfUGxWRyp
d1APcb+a1LvwPPY3RPpmv2QfZA28Eeg/gO/g11V652jz37Q4Sn7uC9XH6Qv/
NdgT2WdGWZuVqoX9gd5g5oscmHZ+cp7xNd9z1hVr6kJO/YI9gv2A/w3sCUXe
T1k3/O+4FcheXageyv7JNzCR0xu3OVBfYE9gjXR5rfLNOeyYB6vV60/62zri
PSEPmbsi9QP+m1ytVu6aHTP75Bj2zlXof435ZF4Z/0SkGmNu+c5SljGvCfVW
kUdHM+qnTd43aYecrIXMQKX6M+00e3y6ndPjnnf2HXKgyfs29ZNHjV4LzY6R
cTsDX77PSJ61MuUylJ/y/s+4MJ+UWYOY31NQLf0PxeeGog==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlElszXEQx6elr+0rfa9v6f+99qKllto5iSUiBAdCw4EIUsuBINIilBAO
xB6pOFDaamOLEkFbpT2o2FJiP0g4ONhvdhefyddhkpnfb77zm5nvzK+kcl3F
2kwzq0R6I2cKzEYkzNqyzM72MhtebLYkZnady5l9zWqTZtuQx9lmb5Ffgdk5
MOfwnRwyG4dvCswT8DVgytGXxoSdjqxNm3VHzI7y1o48s1tgi1Nmg8NmH/qY
zSk0m4tcA/udxPrin48k8X3P2Xlyu5BQ7CsZZlHi14GvB9/kgj3L3+T9Ufn4
8HY5+NZcs8WcrQK7GskA/xB8PdgB+Awjt9Nh6Q0pxfKz6dS6Jq1c66m3gtw+
c/+d+1Zi9FDLbXw2oV+mpmbebsJnLG+PyVfvuhPy9R52gh3P2Vb0FTlmT8E+
Q6pCyslr/4jPt7B60AL2YkK1OuYN7/UUqVc9vPcJ32XgW7OV00hyHZFWreuR
2rhZe9Rsc6Y4OoX/yZS49Z5VFahGr817NAG9mrM29GOcbUc/Ccd3se8jdVHV
6LUtJGY/7HvEexEW5+PRH4GZij0BmV0sTpyLGD2sIv+K/+/5DGwg9sZAb80j
3iTwE5GdmcJPQc+Ly56CfQDf9zH5eo77sfcFsq9il8XEqXPZjHSSy5G4ZnNI
WLmewP9OrnJu5u5oUrPs/XyO/ZUcH1JLNZgX9PplkXJ5zX0eve2invKIZqyO
WMeRLb0U8wdc/UUWEK/Ge8zdKeRdhnq4kXz2BMr1hucL3wOR5TmaIeeyBrsj
R5y67wYw7bnCHMReFVfv/OxxgXru+U1D+qGXpLSf3jPfLe+B1+471h+7NKVa
3fZd7ilQbx3zJaIeeO0BOTyA38aUermXeM283xSIe+fwWVwY9/UZvoZdGtVs
z8f+ALbQdzakmd7l+xVolgqZh5vcdSQU23v+M6Eeeu8WZYn7aSnV5vWUYw+N
629oNHGxM6a/wTkZ5bse0+w7ByuwV3K+2zRjf7kbSfyukP6434HOXM8La3fD
ac2q73B1Ujvku3MvW7N3KBB3PnMeq4V6R4cU8wyzcrZIXDlHpeglyLYccZqB
Pjaqv8fPBlBb2f//z3f0Fbnujegtz2ko9Q0r1t/pMzQjqR31Whp8Psj1cER8
On8lUXHgvfeZqsR3S0R/k+c4CP9LMfXWOeni/m5EvfQZ/kZtfwL1wmvukxZn
zpX/wf8ATn/ihg==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7235028759192574, 0.7841299921185758, 0.9131389992872284], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVTdsk0EY/SDYcWxDXOPfNi1IgEQnCwmBAKJJLBSJAdGi0EUVvSyMNAni
MFGEQKGLUKTQEwIJjqkbiImBzsAEhDbAe3onIfl057uvvfeVv7xu/Zx13c2s
CasIqyFgNq3ErBcuT/jNTvnMFuDhs2c2u8zsMN73p8wOYF2H3Mqo2RqcF0Lm
Mc67kmYFnJdDrnexWR/ov4KtDshe6WaWL9F5Hu529ND7Sfg9g70Z96uh2wo7
Wfi7jfN02NtEHzhn/PJJmQLe+0dksyth9gNrLvyNxp1l8AubjcTdn5jZWvjK
4q0e/nZi3cDdqLTZpJDZCtgenhTGMfhfifWmmzh47xMPlyE/OK038lMRlsxe
4N6XUtzkYFVUPFSFZGs+9JsiioMxdATNHgaFO4+9E+sq7FT01BuxFIJ6Y5xb
YXML1mnYr/VLhroXYe8CVgaYdpaa1SH+ZpwrwMlTxPAIMWzwyf9hE3byQfw5
55t+l8LGLeh8gu1+2PtiDcVbo1/xE2M5ecYahvsJ2INxYPCJs2rHWyv2u1jd
A6oPckRd2qAtxpGLK6/MKXktCqgG2qH3DBj8Adke58n+Xty9xGpF/O3wnQ0r
dvppcb5WQn4VVl+/6oj1Rt4uAVdTQv9Ze3ynL+pnncwa2G4vFT/MCbkkt42I
sQucF+POF9TO3DHevKvbtWnVJWvyJWrtBdYXxLcUsWxKqU6H4jwkLqzsoW0h
1UZbUFi/43wUssdT6ocOxLIurT7oxPleUj7foufCUeE4j/0n5IOwMTUpf/S1
HPLL0vJRD5kjwN4A+zPwVhNUPz4PyzZ7pwn1PDmpvLAWcq4vxkF/bEi4ngHT
a9ht84lbvrNP2WNVrrYpWx0ShxND6ifW3R32rqe88T91KH8ONs9mFAv1Jjp5
9uivlHjOuJnBXIwAppFpyTEXxM16WAxcaZevnsBaF1cMu+PCT+ycVbPcvBqE
fSDWkoB0dzgsm+HzbUz5ob1a91aJu5uIf3sP6XBnv9yD7Rju7xRJb2NKui3w
+yCmWcHcsheZX86wFlfz5K/e+W1jL8QVC+uaNckcPImqh9m/212v0m8ZMH7y
1FO/4fNbSnwwzzmHlzOCtcAaY68y94xzWUJ9zh5nvZMvckVcrEXGyRzn/Yrz
L/ieEVOs5GN4iWrvHfh7X6b/zBHnO/P0J6H5yxyTgw+eeGDv5jPKV21Gvco+
bMZeHhGuARH953k89Go81esh6O5BHEdCiuW+i4e9xP6j/Ubo9fGEkzOLGIj3
tF91zhpeFFFuKM/+5FwpOLy0TXnyQq7JQ7vrBf7vRMxfHc9jgK8yIY4ox/w2
FP2vD94dS+mbxBlOP6+LlUd+D3Pum8jvBuch81jtap+56HAcExtnVsHN/17w
uTEqHS+LvovpW1GNvdXT94PfBOaJfqvgZ0pSPc472uQ9a+1bRPVGewPTsjkE
+7WY+j4CX6UJ8fcRtmeWaY4fjGsWM48NSfFOzv8B3LEcMQ==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlUtslWUQhocee0rPOXJOz/U/p1ophCo3L920hYIaLwsXYkhYEBRJCSDK
RQXUuCC68poIhYXXGA2gqBQwARRoC6G0aMSdxhULtYgLV4LXhT5v3i4m+b5/
Zt6ZeWfm+zsHti7f0hQRjyPXIaOViJ1tEeOtEanpERs5b69FfNgc8Ugq4l70
9yGNdMQT3Ac4v1iK2MP5WEvExXzE83mf0/i/h++7yJFpEWNg3lKPWIv9JuwX
ZSN6ihHnKrY9k4lYmvAduYB+KffRQsSSxOcHiDlMPhcb2JLPSWy+QHcKOYf+
OQp4lVh9Fed2jHhd5Yh1yBV8r/FtGN0o8T+g1iZi5sBbA14H+h8g4iC214i5
hnudGpZVIyrou3PUON2+Z5BH08b4m3j/IM3E34z/AnSr8Z/BeWWTa/ke/99y
rmkEbtYjz2bN6Q583yyYz/3k2835M3JoJ3Y7Plvga3PdfB3Afhv2j7W5F/K5
EfxR7qdS5udz+JxbN7f9yOvYv1QzH8fBv0TsXcjbWXP4FOeRirlQPevwXVIx
vnIUFz/DwcJWc6Lc7lC+OeeoXhb4tj/tHC8T/5WauVe+u9EPltyrEfAfAuvB
qrncixwC64Wp+RF/43A10TCfstcsvlPz7GgmxdVrNdei+ifB+qnq3ilHYXfg
c3+rY3zD+U44Ok8uXxHjasE9Vm/Vw0zZPVEv3uc+SO17KuZS/RJ3H5PPtzlz
eA+6TdjvJbdV+PyOfh8xWjLOWbMyA8x9ac/MEfjogtMeuOxD/qi5ZtUqn07O
XeS8llyPsiOLsP8U/wb5nc94t4am7MXRELp+bC6gu6HF2Ie592YdYza2h8rW
rQezg9pTJfd6Pj6Dbd4J7YLwniafDXx7OGUOJqntLeRSizmbmXgHxccC/Pu5
f91mbqQ/Tux/wVuB/ZPNnp1nap5lzdDtddtId3fWuQ+DMZ5xDbehP1G07i7k
u4ZzELZ25mzRM6rZ3M1+93E/jf8Yvt3Xu9fzqW+y2T3fWPIOave00/myZ1Kz
qBm9Sm4H8clM9evPsnuk3qiGk4l3ULunN+Uoulvrzm0xsg3feXXvluZ9Dueh
omdBPdfuyUe22sFPyu6JeqGefgl+b9Fvld6kgZLfRNlqpnZxf6Pk2dfOzMQ3
aXetekNvxrYK5o/T3POP4Gss7/o0o0Xwb0rca70JVzj/mnjX5CPuFxN/IuMe
aDZ6wBtPe0b+qnlnFV8czeM8kfe/QBy/TPydFWPrDQni/1d3b/QmXAb/l8S7
pzdwhPOsgmdbMeXbmXiWhNFbds2qVTnMqfqfoH+BdkK91k5oF9Tz/wGq7uVU

              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7435544422987369, 0.8163472798827097, 0.913596113241012], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VVtsVFUU3aUdpjPObedxZ+69Y9Ag0QhpjJjwaNTUtB0NJkICH0SNiNRX
1QgVY4hOtETFBwkl5WFrCUTLIyboh8EUxFIhEdKmMRH/TPhRKF/+2AeJP7qW
a/txcs/dZ5/9WHvtfZZu277x9UVm9hNWI1ZTBV+sx5vN9kRmN4pmJ1JmF8pm
fa1m72F/PGN2FGs9Ln632OxR6B5sMOuIzaYLZo9kzToDs81NuAedUzDcAb1d
2PfAxjmsXfA1gf85nE3i7nXIfqma/cY4SmZ3xNJvCc3KkJ+BrScQV4j9HOQD
0L+GNQ39vWmzLz2el+D/jUj/89BfDluHYD/AWZwzO4x9rVn7KewH82ZPRbpb
hzzKKZ41oeJgDAP47i8ph6FWxco4T0Pngbzs3IKNv4HVWEo50R/tTCCeJchl
BTAZwH5ZKMz+hP7JgrDZHikOxvAOYnisWXdfgO49WGehv6Sg/Jn7vYnZNsST
QP4D5CXYXwb7NxH7XVlhQjv1nGyxhu9HquNV4LcS+muyqvG6ZmHCeNe5332w
fcJjOwL9VyPVO0KMZ0rSZ41pYzfuXUdc83nVehVsX8hLTnu0S/1Z6PanVMev
cb4Am/cvVrwrPOZPIJtBrJtwZwP8rsd6GzmMA+8R7PdAZzd0+hDbK43KbcJx
vphVXVmLv7D/NiNf5FpjKL6RRxsqkjP3akYxboU8B5t74Wv/7WbtFXHhMHLJ
4X8Y9sYQ6xF8n8TdIciXFrUn94fdN2t/JS3cyNNfvV7kWuJ82wj5AvJ/Nyee
sJ+I0c+497DjOQGMx9Fvz8HnFOJ6KNbZV8hjznGepW2s9oz68mJRtvpLwpfY
DuPe57Hingqky15g3Tqy4jBrsgW644vEp1Gs3kbVoTVULehnwX19APw/jITv
Fvhpg7+ZlO7ze6BBuTJn5kW/k4FqNQv9qxXp8rzm3GuoiFvcs4fmI2FJbGs+
WwLgU0hpppCPO5yT5A+5zz7l/2hGPGGNGSN5MgoMtwLPk9ifBx5nY/Glim+9
LOyvpfUlD+mHdWMub8JPb0Ez78eSasO6kONJrHq9jPOdkXwzvraMatQH2Ys4
+yIjblFOm+QNZwDzShxv4ky9Q2ntOW/Whpo5T8POfWXNOfKGfc3+rkHWjXUM
8mHoDoXCif292nv8OPzPRsqdtt+6TfiQKw8GqhHv84zyyG2zp/5AXW5UFPc3
qMvdmDurcf818K0Lfo+mlW/B604usfeIPXur7vOF84s5M19iR96zb3tKmmXE
YCfsfQy7l9Pi2ZRz5gDkg2XVh7bqHht51+vcY95Jotw5h9gnxL/N68HY8qH8
0ddIrFnMGtzEfibWfPoHNqwq/jLmlrTinobsfEHzdS1wOAf9S1nN/u9j9TV1
xzzP33PiLzEnf8m1gz6jLvme2CfeI6wnc2a+PYjh+UR1upXXPGLO7EvOb3KF
3Bx3fnZyTpQkZ58RN/b4qrLOKGedWDf2wsq83i3GTHs8o5x98FGiHuCbM1gQ
Bl2B3nLa5LfL3/U7kXcxVv9xRvf7DFiOeJ8BRt2Bask5RBzaYf/TSDzbUVB8
jG2kSNKJQ3xz+C5yfl4B5perwpLcfzYR/+m/5jH8904G4hLn9Gexz+JAXKDO
PsgHsJpaMF/SssG3oNPnEvPq9ry4py2ekT+TnjtlnOvsRcbGefn/zGO/0DZ7
5l84lD+l
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlUtsVHUUxk+naMswfcy003vnEjRISMRgoiYCFUxNoC5YgOJKCe+K1PAs
hBAhAlFQNKERpVhKMNqCOxcGQ3m0gCaQYGMiGBYmbERgbwvs1N/HN4uT3P+c
c75zznceM33tlmWbcxGxCZmEPF+OeK8Y8d3kiM/rIv6tRByeiq4xIilEdLVG
vIsMPxnxVU3Ez6WIkZaIlbz3PRFxLrXNY10+osI7Q34Daz+BjqP7Avyz2Dfy
W08ScQeMJTnHHAXrMrIG/SV+O4B+P/J+bcSP6DfyvQH/Qb5PIsF7Gu8JbIvE
/7QpYi/+L+B/FfwH6AfRn8b/LPqZ1NNTNN6cKRHr+X6Fmo9i+y1488hvLrIL
vA4wesC7S/7/5F3fO+AtT8yNcl5LrAbsB7GtIAXw7rRFzBZ/8DWM77ySubiC
TDS7RtU2iqxE149/A3idDRG/EG8jcpL3KeKtJrftZdeiGpaCfR/McbDO0LCb
vMfBHMmZg23EP5jYVzV+xvch5K2ca3zI91CVD3Gk2j5OzK1qFHczW90f6bej
6+K3g2HOe5mFN7OI3fWeiTGwXgJjLr6vIavgdx357wpzNJ36BtD/BNbb5PsH
+T4i3z34Z+Dlwe5I7av5WIz9f2BU4G4+fGwl/pbEXKvfK9A/Sw4L0S1CPkK3
lRy6a13z7+R2I3M+f5JDN/gbkC+Zx3vo29s805pl5RDYvkg+12scU7V2wvc3
da55ftk7oVr6xB+2b+CzB9sPC57lfdV5E2c58GuQPvA6wb+I7yiyl9gT5Hgb
bsbwv4/vGPjj5H+N918F92QG3Af+r9ebs0X4tlfnUzldIF4Rvp7JG+MMsSdh
v7jeMcWNZkqzJI7Uq2WZc1XPxMXqzLMpTp4C6+kqnmZ+mPoOY/N9rd9HiL27
7P29jYyUXZNq0Y6Ki3Hit082Jzf4XspvHxQ8k03U09zq3dAMPkT/ADlaY59j
qTkTV/157+I6+rNzimsWF32pdeLk8a1pdi91cxakniHNzqt5z+KvcNSR9wx0
F30zNDtDyCxsZ0317CiHW/TjEHKtzvdBt6Gx1bul/alt88xqVsWxbtWOxFia
yQq5DpR8e7TD6s1FMGbkzelC6vmh5FujnR7SLJbNpfpfwv4S9s9V7aeltpFO
v7Wk7ol6IczzvC+kviWa2TXYHms2t9qB4+i+Tr1rJ7CZw/fLqfkRH/dS32zd
Mt0Q+W5LfPuEoV5fL5pL9Vy3TTdJu6UbN0BvT2TmU/uxJPONFJ56fqToG6/b
rv+IT+Cnt8n1CO80+keJudaM3YXbv9t8C3Uzr2buifTaiS78R5ocSzOh29Sf
ujbt25WSb4ZuhXZMvext8e1WT/8HjRLwnw==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7635066239679181, 0.8472899972614739, 0.9136387212470479], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1Vl1MlmUYfvj7Pr4f+Pj+3vf9PtaWKYVbB7aSCW2etelJEhEaGChaW61E
bG0wp83KrK0gqMlfWK3ZUZsUbkLpqqWWrQ5S04NCIMGyYlqGHum6rq7bg2fP
897P/X9f9/28S9o7GrYVOud+xyrC+jPu3PES507iIxNy7v2Ic80494ed+wTn
BjB3JJ17yXduDN+N4Gkudu510L/POzeVcO4LyG8Hz8u+ZD7A93umpwrnOHi/
LXBuHLRDpnMf5H7COgYfxqFvzvjqss49BV0tkP035dwp6sRdU1SybzvtY6an
N6b7Q07+94JeD9n9Fc497YvnCvaP8f0szlUZ515Mi/8Y7G/Nyd+1kK2CjUnI
Pg8fNmEdRg6ygXNeoLvFlHyivU7oHIDOIKT4T2CfgGwT6Ot9+TEK+pvY14H+
KGiNvr6fw77NV446sG/3lfcsapHBOgc/TyC3f4HuYLcUa3VIuZqAzXSg7/5S
5x6C3Cxyez9oD2CtAu8yz7kYcngeeopxPpxW3UZxN4LVB3/mykVjHRkP6cwJ
sUAdxANzUmvnHuirz0r2Es6z0HurSDa4X4Lsbsh+Wqw63gEe58u/lfCrJpCu
NfB5OCoM9UVUM9ZrELQDRj+KuoRRp71QEcFeinUAfFdwH0BmpkB7zs7XkJMz
nr7Hw9K1AXpugH4W9K8K5WdJueJ9y2pDuyuhuyaj+tEGsUDs3c47c/5zWLlg
Hp5BTDVZ9QftDJutoUDYYg6J5cuFysPWhOJhLB8iJ4uQD0FmfVSYvlqkHFXH
VMcx6Lo3rLheAe98IMxPl4HX+PeAvtdXH5B+oUy5OQ3M/JhXT68BTjdXAgvQ
tSWnvmJc/GZPsh9HwJPLybYX1x3pq8D7JHx+B98v4L47obxegC8tKfm4Ii6b
9HNPmWpPvautD5i3eeQ7VK78N8CvR7B2Qe5O+LUWeuogdwT1OVWhupA3bPwb
cX8xUMzExM1S8RCzc8bThb0b649C6eV5J/Y++D+QUN3o199R4ZL+UJY4mIDd
SU96Wc/2nGrK+3nTPwv7zSnVg3jcmFYdN8H/tkrF3Yr76krNsLujmgWcIcTF
oPleCDsDgbBPXHxp85DzhVjgXGJtOEuon3Nlc1rfzFlvpeJrhY/tiKszpnhY
B2JmOe6Xp4W7SFTYpU4Hu0szwifzwnyxTx+Mqs+/KVLfs//Z1yGLvcTWnPXL
PzH5zliIA+oj/2RGOCNuTsbVO5z5xA37mmf29E2Tpf4uqxf9X4gqhnGbGYzl
nqh6jTlk7r/OKP+DyN9wIH+nsc8Ekmd/7EioR7oMb4zxVfRHZ1JvThT7XZ7m
Kmcssc789yBvt3Lyh/dLjYd9+UNc/XUNtPq8asb36nhe2A6bLcoSo9PWg5zZ
V21ucz4zPvZIE/x8LFDf90LPPtgdiuktei2hudECuWZfc5HzaUdMM+px0Db4
mltDId3xTfmONMT2W0Qz4P8ZAfp9cdmesXnCM3N7A/wfJYURyhB/xAnju16h
GLutRsRLEv5+nrR3DTJTpp+zl2fmvLpMWCc+WIeH86IfRO1aKzUT+KZRH+vS
ltVMIY7YK8wBaz1lc4z6OccZA/3n/C6yGT4Cf5ak9I7ttLeSPrO/F6zHWQPq
CSw+/h8wt1vgW1tOOSXWLlsPcgZl7c2lD4yT/hNzK8yHXeb/7fzwm3Vn3z+R
En5r+fYG6i327WhYvbyQ03vM94e0fqPvD9Sf7CXONPYYe4Q2MobPkPUh83bO
6DyfQY7P8v8A30eTqhNrtAh917HqDA9TlodfPb3ZfKeJ91pP/vek9T/Bmc06
7bZ/sIvgbQxUC84U6mO+1sHmaU866UvW/BzKaA4yD7RBX+kncUF8MMbPPOWC
eWBvjdm/YAA7fqDZ+S70j+Q1S/Kg5QLNgzfS+l9k38VtdhDPBZ5mK7HHefZL
WDONOTtvPvwHAPSN9A==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlVlslWUQhocutKeLbU9P+//nNCYSRCEhRqKVFhMvTEzkQiEIBSm0FNDE
rYteGZUEFZdEuqBCaylKCFyZACmJVMUlAgajF6LRC3FpqTFuoS6FxIjxefNy
Mck3/8w38847880/b3PPqu6iiNiGlCDTacT3yO8VEX8gt+QidtZHHC+OODU3
4gds3yFHSuyzOR+xtSZiB3d3lUUktRFzGiPuLI/4szLiX+yXkcaqiAZkD3er
miIWVttnAt+3kBcBcRm9g1xtScRAJiJHviL05pxzv18acaiOvNiD3BmkkXg5
ZAJ8C9D7sG/Hfpj748hazmuQlcWOuRff443OtU/1oS9BTmNvRf+IXPWFiBvB
uph6htFHkHHqnZxDTvDchH8LvifBcy4bcQ/xdxJ7DP0d8n9aMFZxdh7f83yb
4TzDnZUNEZ3IOLGH+PYC3D2Gfgz9MGV1Ef929DH0lJqLwVqCrOb8IRx9Seyl
xPyYu7+BsbfGPVOvtpWa293o60rM8Rn68Tb3vy4yxxvBuwif6VLX9ADYH0zc
u1XYp4k3jJwrc32DYBlCvikzRwfA15Ca6+VgWs3dxdhXFJuDbmo9BMbPwNaG
z1JsLcgr3F+Pzwbub0Taiz0zU8TakDU3F5D3wNucuheK/yb6J+T4idgPg6+V
WgbBX3ZVxC/o+dR35Hs9dxYRexafZRnXmM+7h+rdQuYxiDWa9fkOfCbxrQDz
V8T6D0zfUvsAMlLpHj7B/Vn8j3Ie4NvyrGdas7ys2rVkiL9vrmuaJf4/WWOd
C54usB5EX8D9a+FgPr4Dde69evQF+XPwtYR4efDs4P6zyMvhGdZsnsX+a5X5
F9cd5O+rNOdTxH4E/9fx3U+OXs4PJeb6Db7dzd0VSJpxzGP1fqN6m5qpTcQ7
UGesmsFBzl1X5lGczyP+MBhGsY8h9ZzfrfNs36b+o7chays84+rNpVrPqnqk
WW1Fbq3wzKrWu6h5V7lr3lPjmVL/NGOa/eYGz6PewFZq3ZI3t6pP2NQT9UIY
7wdLX+K3Ih9xO5oaqzjuTjyD4kMczSf3NU3unXr+PLGebPD+UM/XpJ5hza5q
+hv/v5AP8P0ZOUO8/iv900wkqXeUZuU6/M/Kv9a9FOfaVQfxL6/wzrrY6JiK
pRlV7kLq2RUG+fbXm1vdERe9fDuaMSfr9VZrvRuH+HZfjXemdL0xzd6lxLtR
M6jd/RJyItyzGWwXEtuER7szTY1d+Z/B9mPWu0A5Hyf+q9iHK7yz2rFNpt6N
+gdoV+oN6+1qZ3bC5w0N3rePwmcHXG9Ctlf7DUwQb0uNbdqX+jdM5Pw29I84
SbxTBWMTZu2SHuo/kvFOWZd45+ruYMa7/d7EZ+UUVu147XZh1qxdXee3rplr
Sb3Dtds0k58XvPOk640NgLW/ybvlqSpz3ZP4LYlj1daemHvle414I6l3t97I
09h215o/YS7P+Y1qdrUjRsl3osb6aeIX58yJ6hnBflF3C86tmets8o7XbheH
6s1zif9tyqG3vrfgWJrxSmqdavQu086/OfUO1+7QP+t/1uQbrg==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7815142914979039, 0.8532950745514307, 0.9055713023308727], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFl3us12Mcx5/O/fx+5/b93b7fc9rIZTGxMl3U3DKbbKLounTUWVFMRS7D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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwllW1M1XUUx49c8V4uIvd/7+X+7+VuJcRqk5U2kakrzdYL3HoAQ8BCQKYV
tqRpztkWc7zQZgRkWS2xqDR70WqtTcRVLwLW0HRt0gt7kSGN9UCmPaxlrfU5
fV+c7Zz/eT6/8z3/is7uDTsKzOwANB/qSJk9EpodKTILF5htKzVbWWb2YtRs
D/rN8DuhceRjEbNc0iyRMDtXaLaeAJP4/s63ToLOEqMT/7ac2UHsu/APsA3R
12F7Hp9o2uxjbIbhj5Lv3ozZPdhcmmc2w7eHsf0mazYXN7sCzcC38u1D/Dci
zwZml/GpJlcbcox4RVCKWPdRw3psF+fNtsTMBvFJIR8kRh+2lSVmzdTbFKq3
fmIsRRdjBmPkrsVmvNzsK+o7iTy90OxRbDugTRHNqILci6E24rdDF4m/Bn0P
ulfxqUMeTSv3AFTP7PoDzeYEM/kI/bvos/DN0Dlm9QX0UzE9Q2uJVYnPuohi
XkWeCzX7Aup7j1n9EopfSs9vUEsr3y6Te5Z6rlH7/Lx6vUqPryC/QLyvyXUW
eYr+LkAX4VcTI0P/ZVnpqpC/hb8EHTK9wTtp1ey1htASYlVDi3ze1HQbtsuy
mt0E8kr4VVnF3sd7bIRvghqRH0O/m17byzSLZ+m3H34AqiPeOPrX6Wc4o14a
8G8h9y30M4T9+8yzkHoWpMX7zjyEfjqrXfmAb38Tfxe0lrdLQQ3Mqpn530/s
JDkOMI8+3vtO+AliXvfZEK+XWGvI+TO+p8n/L3wM/8PEz6S0u7uYaS22K6A9
BYqxn3g9Kb2FY+Qu8j1AvrsjmlElthVp4eNm5Cr4J1PiT0J7yf80dIfLEe1q
mpzPx7Wzy3jb6zn10k9/z/l7oR9A1w/V4rudnEeLtKOOra3Q7mJhrDMnG9f5
txbf5VC7X0dN0xlhyrG0pEi714RPV7F2MMPsPyfnlNdSolmdz2lXfWYhumxW
u+MzOYvuAjY96H7A5ibiV0IdMWGyEduZQFj2nZjE/kxOtt9De0u1I74bw+xg
H/ZxZrB9nnrO+ntAg3HdFMfiCuRP4sJkA/U347Mprh17G9/X6HesSG8wgu5w
QrvlPsuxrynTbrTQb1VGO+Bv30m876h1DjmK/Bv6RmI9GGp2fuOOpXUT/BZk
ouo1HmiW3vMQ8zuSl++fxOjFtzsQFv6/t8hbA+X299qB/ESoW+Y3eEteN8Vv
SS/zb0M+nhS/DyoOVLPX6jdxLC1MOBb8ZvoulSd1q32n6v3+BcLuUKFmNV2u
2nxmi5BPBdrFT9G3wG8IZesY8lvzGTmqo7o5p9EnsrIdQT+JfoR6/okJQ/Xo
14W6jY4Jx+ZT0P5iYfQKuuFA981v7pfluil+S35cqFvsmHWsur4M7LxEPztL
9A96JtCb+1v7GzsWHWOOLcek31bHsGPXb2wpsUcD1eo3pwZ5eVa3ym/krfCT
gW6R/xNG6eVURrvoPV2j3jfRH48Is4+X6h/qb+k3ymezOaF/mc+oC/uXE9I7
Rtt5v4683s5v7I3wf+WE7VUlws4UtDoqDDn2bshL5xhsTekf6v9O/+c5Fvwm
+C1wTIS+H2nd6ttL9G/+AzoU0f6fQFeT1K76P6i7VD5u62/yFr39GmrW3qPP
2jHq2PSZ1yZ1E/wWOCYGsT8TataeY5vvdqhd9pvj/66Jcv1L/Y3/AxKoIaY=

              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7995219590278897, 0.8593001518413876, 0.8975038834146974], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmH2Q1VUZx8/Cvezeu7t377173xeV13aqsWgCBUEEtXRyQVB5KWDGSANE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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3uQlXMYx39bu9vuOVvnsuec95zdqJBcp2bkslkq92E3FdWSFbklpShs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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8175296265578755, 0.8653052291313446, 0.8894364644985222], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmXuU1VUVx38zzOveedx753UfAyGKpmXyUANFxVamKAwEMqipGGgmaCIm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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3uQ1mUVxx/2wrvv+y69776773XZQdDUmkm5SKCI1AwjJLsgtEAmQoCl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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8355372940878614, 0.8713103064213015, 0.881369045582347], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1mHuwlWUVxl/YZ+99zj7nsO/uCzhmOmYTKFcP1mRNWQhlR7GDTk5eUgSs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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl1lsVVUUhje9HbjtlTucW87trYg4m4AoICUmDjFKBVSUihKJCAoWnCrK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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8535449616178472, 0.8773153837112584, 0.8733016266661717], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1mH2QlWUZxh92Yc8eztk9756vdzkgVjM1NXzMqkkfNjCKsjJMiFAhWILN
6EQiFtQSAfFVtLpfiGMgYI19rKEILWjSuiigJM1UM31ogGLY+BdNiKNSfyh0
/fZ6+OOZ87zP+zz3c39c93Xf7/no1++bu6wuhLB9RAj1+l2fhvADjT3ZEJbr
xV4tPqv1raNDuHVkCLM1upIQnqmEcKPeNzSEcJX2TNX8fZ15LhfC77T/cb3v
1ziXCaGi9UG979EdW0d5/xQ9d2pszofQq/X/lEKY3BLC/ZL3He1/WPu+oH2P
6Pfmet81usFrN0W9/tto3V7U+Uyd5ZSbpYPu3K73H9f97bLlR3r+TTWETWNC
uKAzA4mfmY9u9O9vJaegM126I6fft6XP+ZKf90qvS6nv7NI4I/32aP9ruu/6
Rtu1o0ky9e6g1i/o/c/q7Yc/631F8wma5+WbHZr3az4o3XZIj/nSu72m+xL7
CP3fjvqc0p1PVi3ztOa/1p5vaM8/NN9dte2van5C4x3dOSA9Lxatx1qtXc8d
sn2ibBjS+KnW76vZNuw6qve7o8xj0v+lJseXvQdGeX+TdB7Q8ybNrygLG2P8
/JjOfUTPi3V2k2K4UeMt6bNG+q9udly36uyEeO870uvF1PtZe136dtc7piuy
jiv+WJPaJye0/9rUuNrWYBy0a89d9cZmn+56S3uWtnhtp/TaN1a26855rSGs
09nuEcYGuhEj9GLOGvb+vsl7fpi3Dei/QHLuKIRwr+Rsk4ztGkey9iF2LJE+
5xPHjHi1RxvA5McavHel1q7TWKn53ZL5J8XzrOIwTvOv6NyCmu+bKv9Pq1j/
d+XL2xPj+E3d+b+i78LWp6K92LZB8xXBOcGcvJig/RNT+7FXNg5J9hMjnUPf
zlqXQb1/LrXf92r9VNZYJgY8g+cOzXePsv+xFQyAjWd17mDqM8gi54nLFtmx
fazvmytMzdPI6N1i7avX78xgjBB74n623vbh/5VRFnIek4wvac/ukcYuubRC
689rbV5c/0D+69E4NcL8kYk6fFJ6fSq1neQiOdlFzOXLK1LrgbydUSYxJ/Zz
IpfBafDG2oztQqfeeNeKiOcfR2xMlrxJqflpq2x/UOOQ3h3XnpebnT8nmoyx
o5I5X/5YXDAG4CAwznxIcayIT/ON5sOOjG1q0/hWxn5rl7x+ycppT5Pu/LRw
Plf3rhM+NxScW5eEnYsV2/1L+emQRgN4qDPvwjmjtG9RxpzWKjnTJOfOyK/X
RX0m5m0vfsMe4nkZX4N5+2G1zj2cGiO8H4x74HzswRZ0b4vcvkH+31/xHFuH
Yl34m3TMaX2s9hXlv1lVYxXfgXVygdgci1yEb4/FPN2bt7+4Y77ONrY4d4kx
a/gNX2ZiTcrq/e3a15k3p46OvEo8O2NMiQmxIX/f19pAzlgbzLm+cOZq/J8a
s2di3oDn2VHvYf5JzOnkznHNv1pzfaMWnc+aP/8pGc/o3eek3xTp1py65qwR
TlaXzDHTFd9fSM6/da6k9w9UzR9bWs1r+GFQ8T9cdD59ouI6hw3wMnWPu9pj
7YQfZpC7Rdt4SPMhjdPS6cOSeZY8Iv4fRMx3KGf/WjSORmgeyubg8Vo7UHLO
IK+ubJldIx0z8ggOntdirlorW56vWafvae8rOtNWZ95Ed3jgBb0/rPFG3hgi
V6l39BGsUUtnyd5+jc/HeIFdcIusdzNeJ793xRwHW7xjvT/WZXqZe+Wnf8n2
wzrzNd25sGYsDNfnOufLZ6V7Q8VrC6V/n/Ysky5N9c4jbR+uDXA+Pi4ljg1x
eS9jfcAheASr4H9S3jkGN71ZNj7ARkb3zCr5rupo60+uMLAHf5Db9E/Ur17d
82CrezHyj/wkB89J3sya9z0qvJypOpeJT0/iGG2MNQ5egtupwdQX7KdWd8Y9
8Bd3PSXd9lRsE7pjA9w7IfoX387X+9vYl7GcNbHuD/Nns/MaPZmDK3BAPoBP
ajH7ycGhjNfJTTCLv7HrDdl0uua8BGONGeOsW/bNKDheJyPnYtf4snsU8oC6
QV2hdsA3s0vWm3yFq+Ga8VpfUDD391WNM+x6Uutbqo7lJPnvYNny4bc7co4v
tfGhnPu8RQVjBb3hquOxFsAf8Ah9Gnmwo8U+WNVoW7AJrGUi3uBW+jVqPbX0
Yqyn9A70kdj7Te1dUrHPqA3ge3nsGx+KvSP2oQOYRPeJifWHV34euYWY9MYY
nRSmrqo4bnA764Oxp1sdYzQgX76emu83x3Xy9wbJPlI2NhZKRod8sY8Yy96N
NXM2eTMn59yZKj9NG+Nn+ll6UHqM4/o9mdqmKyXvnop7ZXDaHfsufI6/2b9S
Z8up/fJC0f0TvcUfG82p9FovkeupObuq31LFMZye+hsGbuxrdV6RU9OajHPy
FI7eFmvWuAbHhrgg72iU+Wre/dLlvulXGfd+fPf0VF2DZ8hva6XfnZKdyB8t
Nd/B91N31GFV1XlLztKXU7NzkW/gnVzs0ag79Glr5L/HE8eaegXXkl/bxrqm
EtPTsc/Dd/u1vqvFnEpNXp+Yl6g/17bYX2uFtU0Fy79Ja+NSc1iQb/YnxjK1
+stFy98c+2g4H77vqVkPuLU76nl/yd8k8O8enf1DxZxGnRmIOk8qu/5R+/Ah
/qOGTkldd6m5N+j3xtR8R4+9MtZfcMYzfumJPAUOn6gaT2Bpo2xaP8a5SlzZ
g5+/K5sKqfFCbwKuifVrqbHO818K/ga7XI8uxD3wDbxDr0Ut7Izfnkt156Mt
xiiD/WCV/h0MgZ+dev/31D3BMsn/YsW4eSX1txzfceQ7eoHhW0ruZelxFmXd
W8M/2Izt+GGX3v8k9R3EtpI6vvR5nKEWoiv2w+3EhzgtifVzUZQJV3E/fNVZ
8ncv37zrFaMDiXnmGq3lJf/WrL+J98U9xLg/1u5tsb/FV9ek7oOp1/u1t77V
tQ4efS/WyiOp8Qdmbq65v6E/uKtk27CL2sg56vuu2M9zB7WumDimvTq3ueQ6
slE6P514/5ys9R0XY9cQ/4sgRuQ5esJ5PZH3llXcK+Ajem7Ok4Ptkl1MLYec
mRNl0tM9ULIsuPVs7L0vf2OADeoQ/DIcg2ZzLfi/VLSu6Emv15G43/uM5pMT
1wriQ5zaoi2FyAnwPbxPTcEm7kA+vtkR+4dM7M/QAZ6enjhH+J1Z9Xz4WyH2
S7fVXPOoF3xbT43fp9SljpibHxaNieXx/5+7s+63qeHog//vUc4t1ViX81l+
wX+fzm0pu/6Rn8gAb6sT94jYCO/zXwl9wtXS4VzJPDhevr8y9X18H08sG8OD
8ZuSmHKuKZ4Fj3xHgUlwDb7p5+Fi9vE/zMuJezJyeFXi3hdftMnu71fNK0/L
Jwdq5vv/A0QSk7M=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlmds1VUYxg8Uentpa2/v7e2fXlb0A1ELCBFwBkSUgkSERkZBGSYYUIaz
hFFWUUAKxRKGgAMHpEDZAYQiQwMxURNRRBBwhE8QoUbBLwL+Hh8/vMl5z3ve
vc7tz00tn9I8hLAdaAHcFYXQLh3CQ1khvBwPYRZ4FbA2FsJwHtYkQlhfHEIZ
9MfAB3B+AvgH+tlmIRRmQng04XPnvBCatw5hV2EIxa1CSANLoQ1Lmf9x+CvQ
NRLYDv8S7k4nQ7gzsu6VLUNY0yaE1cBmjGvMs23tgaqYbXwTWS+AN4Lvxf6h
4KXwvwp/q+wQOnG+G3g4y3c/cj4N7OPtH+CpohD6IX9cfggbuduNfYuQsY33
r2BfJ/Bq6GegH8OnC/CeByZAuw7/Mc5Hga3w/gI+A94D4Oc4l6I/l/O9+D8E
fDDwJbb+xN01znk5+Mh5DzH7Ad8q0bkHfYuRsUP2NndsL0C/lucYjy0IoSMy
3uD8PDr3R46hYncWmQfBPwVqsmzDVeT1h/8S+AHk5UErQ37bYJuWQasDDiG/
v3IKvW/k3M6D3gPbp0Jfgi8F0I+j61Tk2LXC/svYMxz6AvgvInMh7zsQ07W8
rQaayOe+yLZtI58beb8JGVew/0He9wPvW4Ic8FLwCdDWIaNJvnD3DLKfBRYi
fxE2VfN+Cm+OxBzzJP7dAu+DLUvITyX4BvBL0L/mbiS8ceQtyrOMU9jTGnvG
Ib8r8s5w3kx+y24jV7yvQP4Q4pOd7Rp7m/ptD88Q6q+O+h2F/I7Ql4DH4L/J
+QbQi3MMnnfR9TMyH+D8Pv6vBz+MDRfQfTAXeZyXZ1zLivmRjGtKtaQ3R9F/
EX3j4a2Vf9BSkX1RzpoTq9nQ85GVB3yAr69j01c59nko/OUp21IJvqDEPaje
W53rXugCvBZ3T+zl/RjFFNo75Gc+9jSA13Hehj2l5PIk+G7wRmR2jtyj6k3J
uAf/8rkr55wrmzOuKdWSZPaA1hOYFneMJpXYR/k2B/oOYj+Lu6vYv4IcjC5w
D6t3J0PvhP7OQCfwifCv5+06YAe0nUBv/H8EGNTCMWugX+ZAr4P2Hvhv6P41
Mq9qZlPCNazalc6VvJ/HXU/o3bKc2/u4m57jHFcja1mBZ8t17k7AuzPhs2Rc
STkG8j0fmJlyzBQr2TwIfGbCtdYGeoHiUeheUs6eJpblQG/mSz36BvJ+Ou9L
eZ/Oci1PA//m//wuR/Yq3g/mbQ54b2RFyBwTd00Xcb6/0Lzi+Qhf1iU8u6ep
f5Vr4BN461s616oZ1Ypy/iT6pqT8Vm/WFjpHys0a7kakPTM1K7fKHuTXFruX
/4x5N1SUOHea8XM0a+E/AX8N/dmE7KnY3zLXM6ULttUW2RfFfIZiX+xeXsrd
Us41xZ4NmhllyOsXeXdoBjZGnjGaLefxf3fGPso3zdRQ5BpXba/Oti1zNa/j
tmk39pxIeLaox7fjb6Bf/s7xDsgBH4bM6jzPuJPo+i7pXCpH9QnXnGpNO0G9
Mh9YkeueWYWumYWe7W2zXau1rT1LVLNZ4L2I12Dej8Gfm8i+kfTu0Q44xPmz
pGsjm7u3Uu5p9bJmTPdC7xjtFvXgXGiT0p49siENrRh4KeYa34KtW4u9e/bj
anftp8i7RD06m/NkZDbEHaMsbL2V9C7SDh+V8czXrNdMbcv76oxnRVd01iN7
c7FnqXbsZOxdrR2JrDuwd03kmlR96U612wt9Y+Ou4ZvaDcQ0HfdMr0D2iIx3
r3Kg2dxe9sY9oztE7mH1ru5k+/iUYy0f1CuzyfnxfPeMak8xV6xVg0PBz+Dj
spj/GOcznjGaLdq56s1Exn8Z9eiHac9c9aJ2VjfOXdOuVcmQrlTCtS6dl7Hl
d2Ax9qgcZyO7qsS9tDPbu/pQ5NrVzp6Y9g7U7tOf5nNoX0T+W4wlByloA+Ef
neucViW9E7QL9IfRLvu2wLnVTjsC/XDSu0p/AO2O9kXWrR1yDtm72ng36U8i
2Q3o+CtmHcrNU9ifle0czSvwH0d/G83YpfrvRf4raCZXFjjnyrX+dL1LHAP5
rhrXX+rjpGtZf6p26uVC1450aPZsSNhX1ciuhGeuZq16cgD6mhL+y6hGtkLb
kvbskc36WzYrcq9r/vSB9mKJd43+sN9H/qP89zeB/1++43vq
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8715526291478329, 0.8833204610012152, 0.8652342077499966], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmGuQ1WUdxx/2wp7Drnv2dva/52xRJpOxgNaLeuFIlggR1ggLcUthpgls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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlmtM1WUcxx8PGOfASc6Fcw7nb1KWzbyvF/XCpS5RSsuJ4AQtZWupWzo1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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8895602966778188, 0.8893255382911721, 0.8571667888338214], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHuwlWUVxj/OZR82+9z2Puew2d/pxgwYlDnTDCB3cOKicjnIxDGBw+0A
lglownAzNQg8QAIDjEEkdxRSksuBbLSB/knK/kAsoAJ1xkhpKtPCpkLr+fms
pj/e/X37fdf7vutd61nPWu/Xc86iyQvLkiT5R5ckKddzrn6eziZJizrLmpJk
YSFJzlYkSX1Nkizqpv9q6/JJcrSYJI9Ldqbag2odmjtR84bXJsmFco99Q+2w
+jdp7ma1vnp/IpMkPdRfUDtXnSTrc0myU/0/TLWm2mvq25Bz//fVv0x9D6sd
Un9rXZI8rzZH/eu1z+qqJGnX+wcavyGdVku3izrHKvU/znkkW6bWT7K7dZaD
er+msSG13mOGzvh3vfdR/1ta53Wd84jG10q3/Tmf5SbJNOhZUFuk/oUVll+g
50nJdKotVv/9DUmypM46o9eq0G2exm/V3Ll6ntb4UyXZQP3Panyd+rbrvUPz
B9daH/RCP84/WGe6s8H2WtwsG8nmr8rGkyuTpErzO3TGNRobpP87tM578T5d
6wzQGv3VlkhuVo8kGVVn+w/VmnWyxacke1B7HMi5/2WdZ7V8IxMmn5TPNzZ6
D854oMY+wnZT9P+8ZN+RXsN0lm9rzj7J/kHvTTGOj5FBH87JGVeq/+Gcz/7x
2Wq9Js8hcfZ35eMrGeOnhK019xb118q/NWqf03hP8JlzP7jaH/q8JH2u1SdJ
m+a/0V2665x9ZKt+kpsqmVb0l8ytsuMayRxS3zNqY9VfoX0mVFrfc1nrjA3B
2vaS8Xala5IM1PjQMttsRK33bpT8C+pb18XxkcsZR4nmdSn5/xbJfKfKMTVP
+3xP+z8qkXP63yHZ01281qaIkXHSsapkn/5UvtpTb/t8QWeqEgay0mOUfLSn
2fPxMxhHv5F63lZr/2In/mOrXpJ7OxM+qbZN+pfZ5z+rsN9Zg3gBq3Pk0/ZG
zwH7+cD/w9r3Q+HpcoVlibEpmvvXRuMGzPyn0vuiw1/U/67apCrbG93QC53L
Izb3S/6RnLmnf/gLv8yvsk3w7y+rLdde7jnEFL7AxyO7Ot5Zmz3Yl7XZA92I
gf5hH54Dop847huxfD7r2FoTvLU2uOt6rWXgCbiPcfzSqjNNaTReB4ZO6HOP
MNYm+QcS88eI4MPpOWNubGB+dOAffD0ZGKNvTPTDwdiDeME2b3f1/xWyxweS
WV5j7mEO/IGe9HMW9L0e7zzRfb3sc6aL3+E64pq9WIO1rofM16TLfSXHKrZ8
OnTG3nAZZxxacm4gL/yxwVwADwwUHvvmjak27CafvRWxvDFjzieG6b8YuqAT
+xJT+4OLdih+t3d3zH1WeO9s8r7gABk4eZuwMEjrDdF7nbjhdskf1HhG+69q
dmy9L//0lm6vS/Z9Pf/WYD1Y54nQ5zWNjai0DpeE64tqV+Xn4dEPxxK7D+Uc
s2CGHIANsXOfwAZ+WVljm4O3gaHbe9pzTMn7kRP6VDsvYIM+YYeFqbkJXrpL
sutS23qr3reUzNP4CJ3A1ZjAT2fkln6RXxaLJ3Y12T7wzPzQf6fWba1y3x3a
57raoMRxuilj/2yodU49Xm1ZOAp58F2oMIaryu3nTLnzObFJjMJHa6J/eOw7
rNJ8AW9QI5AbsBv2uSfredgfDHVobpoYvyvChi+KS96T/8rKnF85a5ve/y3/
fD7vOUvk851qp4STos59tMm236vF9qj9XGfpVKutNTb6CRf91e6qsQ1PRu3x
Tck+mpoXx2q9J5s9BxwjCz+y558avC+1SFnwXibOgP5L5asXUudwaoD1sS86
bAjb0uhvjxpmQ8hMy5kjiDXyAfkEjJHPJ0R+v6rz3dC+N3c1dqbHOle17+9L
ti9xS66dIltN6GYZYrBT83YVjJ9Jer9T8m2Su13PO0reG96dFjoc1fqzc9aF
WGvPOd4myQZfqXM+vaSxWTlzF8/Z8Q7nEc9gBjthL/B5t94nlbwWsQIm8PXm
SutCbj8da3L+ep13Ivwsueek+9QG42pvxtgCV9j/5chfYJB3+o5pz4XC2QK1
XYFH4hc9qAlao95bG3x/Q1j7UG2q9jooHx5InXMmaM+WOsdLz8hP2Ie8SN1A
PgXvwyPW8Bc5gvpheOyLzcEvPA2Gsc/wqB8+0NpHZZNXcuYEeBPfzlffvWqX
pOOH+L3Jea69m2OP+Lm73jEA/olT1iBW4ZlTkRPBwcqopVvjnZxyv+w5oNn2
WlbwfuxFDh0ZOZr/M6O2J44vx38w1Ra4ghvJydQ2jVqvQe0W6rJG14X4+N46
177syxr7Ml4H28wLv4Dda8GrcDQcAT+AI+pCsESugA/g6lPyT6PWP5yxnntj
TdZHV2pO6se2rOufR1JjF7//JnUdTJ3Qrcl1NjX2iK7GNHY7oryzNu96kHxA
PBOnjF8MmdElz2ful3W+54ITsOuCgm27ReusyJvDsBM1DLaCe+FddNumdX6b
Ot8uk+xH4rhnMvYBmANv8O7ZwPbzqe8b+JQ7BLKc97as9SMGwRA4QNcvas1u
0qMl69ptZSF8qTW2CUPdNf+pat/f4LHF+hlbMhes1Psj9Y6Zf8reRwq+fzyg
8cOp4/BZPZdrj1e1xg7J7kuNO9blzPiXGvBfUQceT33f465Hjp0csfM79b2Z
N656y4ZvxHtH6jqAGoC4QU9yHDjFjtiQPN8rcj3jLSHDHmAe7Fdr//FNxt81
nWO89jgk+R9o/SOp76M3FE8fdbctq9V3U7PjkjkPBueQPydEDsW3b6T2b0uV
14Ov4bSe4d8RwQ/g9jNa5ydZyxPD1CLgHFsuzdue3y1ZD3Sglvl0vesZ/Izf
tsc9nTMRU6M098/1vodcizU/rgFy5rvWiN9CxAX7gVn076q9VvXw3RneosaF
D+ER8noa9zX8Cd5OCuMvyXZXo64rxDp7Ig7hCu5BcCvrEBPLU8cFupALwNKj
0vdYk/kUvbhjc0eZH5wAN8yq8X2ctTgvNTBnvqz1xuWd3zjz1eAG/H4hdOYc
i+Isp4q+R3GHWh3fMMinfKOAP5A5o/Hd9ebt6ZKflve9Dew/lFrnjogT1t8Z
9RG43Sr5bUX7tUIYa8t7/D49RzbbNsuFl8fqfKegLqFvVrl5dHpwPngnBsB8
Tus8WDKvUaNtzPz/TlwKmy+IuxY8lo1ciG5V8U5OZA/2wi9wz9zg6mmR7zkX
+YS8Qu3N/fLmvNd7XX6+UnD+/FKz44c4+JX6DtcZ4zOEnZk9PP+rJeuNzvD0
a4H5QqN5Dfw+l5ojwBPfkDry9sOLsv+x1P4l/1+KGuiE+h9LjZMLWedrcuhS
zV1e9D1prdZLi663iX/2ZX/qkVlRw1BbXo97E/XHiWrXM8TV6ai1TmiNk0Xv
83XptaJo/gJjZ0If6u7Bcefi/j264Ds4OPrf3fEXdf5ew32SepY9H4h6iRzD
OsyhxmJv9B1XZZ7AJ9ShxCz5fkTkcepGahl0Xhd3OXSeXPKdYV7UW4WoRfHJ
7PDLRI23lGyPZSXXqdSoxD3fzogJci+2YE1sjA3AKvuxL7a9lHU/9ofjDkVd
WojYhVuoTamN+L9XHLavu+O2ID/mm4xfzkoemBxcOSi+vXxC42tS+4b71q6o
DbD1kUrv+2zUXMTpOwXzOHr8OmsZZJuLvg9wF+gV36G2h/3hVHLioqzrWu5p
b2rP3aX4nhc1+qC4y+A7MEwdsTlyTXvUPeg2JPAAR8LtiwOfMxvNBfAAHMYY
du0smguw7XhhrHfRdSr3re7xXWi0sDOq2fz34ybHBnEBfyyNucc171jR5+Z7
Q2d8c6AmbYnvWfAjvuOOAI74VgV20AN90BObXQqfIrswfM03P/L5zvh2Rx6F
/3eVHHvEHZg/H99MqPnAEHUc887FXOzCO/FOrbGy6HPuT/2tgfvKED2HSf+J
OetwPO7IK1J//yVnkCtORL6A78siJvlGgC/x47jAFHEEp3J+1jkZ90B8uqzB
36/hHPQmRsES9zBkqPFekS7vFmwbsHgy5rIHmME+P0qdV2bEPetYxOcusKR2
Vv+XaXxrfdTG5Pxqcwnxxvd08tBoyY5JPQbXvt1o3H8r9b0ULvgv0v4JFQ==

              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwllnls1FUQxx+17bLdttu9ut3fGpUIRBCvBFQEosaWq7VFYotAC4VGISAt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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.9075679642078046, 0.895330615581129, 0.8490993699176461], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmXmQVdURxu+bNwvDm+W9WX3vWaGiYoERqEQEBEIQRSOMrG7AaDlIBHFB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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwllnuQ1WMYx9/dc3b3bHu2PXs7u+d3onHZKGUG6TIluiFtqbaLrpRLNKm2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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.9255756317377904, 0.901335692871086, 0.8410319510014709], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmguQleV5x7+znLN7yNnvnLO3E/esriAsKIW0YPASQcM1qAEB0YCNgCIS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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwllnmw1WMYx99Tp9vNvafzOyvn3CYtoimhEqX13ohJC5VRKRMmpe3W1W6M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              "],
              VertexColors->None]}]}, 
          {RGBColor[0.941176, 0.906538, 0.834043], EdgeForm[None], 
           GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmG2MnFUVx5/Z7Xb2ZZ6Z2Zll6sz0FYzFokK7i0awsFvwQ99Wg0Tt1kWp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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtsVWUUhTe3LRR7L/dwWi6550paRpLwEgwaKRhFHECCIYWEdwxtIMZC
eXaiMY4YgKYQiDpCcdaAJKIYnmEKRCC8WmpRJhSqESRpGPAq9VtZDvbJ/5+9
99qv9e/JrVtaOnIR0YZUI4UJEUuTiKO1EZ+Miajl3oc0cX87H9FVihhAnx8V
cZ1/Hdw/TXyWz/YsYivyalXED9gX04juYsSuVyLerfZ5CTI0xv/24/85/r34
9iCDnL+sRAwXIg6S2A70Lfz7Gd0x5BR4D8dHjIDdWRNxH9232PyBrpGcmsH+
m/gv0X9QF7EsMYZ8f0Guc88h3WDvR/o5z8Z+PfZtede2h/jPCq7xSgPYxOuj
njnk/BDbB0gTvcph01Qf8Yh7Ded+8nmA/Y7Uth8iN8F7kbn2r4n3G/pLyGXu
74P/Orpx9Lcd24+IfzU1hny7wHtKPdvI/ydy/w7/p8Q7rH/oD4yO2FuO2Icc
qXVOg8qPfIfBeoH8xb2fe0+V//0K/nTsF+HfXuNe9CLf5NyTFmz/TBxrIT49
5NaLrMN+CPxFxL6WOreZ3A+i20i/dhdsM69iG+lWFlybOCUuqcZz6AeoYQT8
SdidJJcTyA1mdRPpo/ZbyFp6M0KPZpB/JO7VGvLfhP/mimsjjdjNZ3K9e99O
vC3oluPPrzjE/Zn6i03GuayZcr4K3kTO74DfReyOim0PEf+L1P903sC/Ifrx
T+ZZVyMLyPks88iN9Qx3gbWzZG6Jg+Lee0VzUxwUd/7N7CsOjcb+jcxcU42z
iDW7bK6qJ3cSz0C9/z7nWbcgU6s88+n4Xkvci9VIIXWNqu0JNR8Fq7PB5wtg
TuN+PPWs9cY2k+umkmcpziyn9r2pe92I/RHsfyzb93ydub0t81sWx5ux35l6
tiuQ5+jquM+vdo/PlLwztCtGjfVbFaaw9GYvoG8oO1Ye/THOszJzXxxMwBrm
PgmsRuQtzqfL7sXH2NytN4fEHc34CZ8FxHiN/kxB8sQ+UPIuWQP+HPxvJ94l
6qG4Js6IK+LcLewfF70LtON+x3bheGO1Eu/wBGMKSztLs19cdG3igGJHxbkq
h87UPVJvVhW8u97MnLt2mN7K3OL/3EW/FbyLJfdiLjafMbtTZb+FMvzYXrKN
dNoB3dpN9X77esPq/VcN5rJm0Ew+8yuOrR2sXaodqd2onTqi3ZgZWz3WrruX
eDdp5+ltiGPilt6Idsfiit+Cdoh2pXqk3mhn/gcToN+t
              "],
              VertexColors->None]}]}}, {{}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{6888, 4588, 6887, 7021, 7020, 7079, 4002, 3898, 7014, 
              5533, 5377, 5590, 5589, 4082, 5740, 5741, 5739, 6544, 6543, 
              4004, 3904, 6892, 6893, 6891, 4594, 6894, 4572, 5326, 4723, 
              5356, 4816, 4817, 3913, 6972, 4724, 6971, 4725, 6973, 4411, 
              6768, 6770, 6769, 6895, 4896, 4897, 4745, 6583, 6584, 6585, 
              5243, 4088, 6910, 6911, 6909, 4624, 6912, 4418, 5112, 4419, 
              4420, 4313, 6706, 4314, 6707, 4102, 5335, 5336, 4752, 6419, 
              4751, 6996, 6420, 5832, 5833, 5744, 5746, 5745, 4089, 4317, 
              4434, 4318, 7032, 4090, 6586, 6711, 6710, 4178, 5747, 4179, 
              5748, 4229, 5783, 4326, 7051, 4327, 4316, 6708, 4315, 6709, 
              3922, 4010, 5113, 7024, 5114, 7080, 4779, 6445, 4789, 6443, 
              4788, 6444, 4221, 5778, 5779, 5388, 5546, 7017, 3917, 4008, 
              4623, 6908, 4308, 6703, 4309, 4325, 7050, 4324, 5774, 4218, 
              5742, 4176, 5389, 5390, 6704, 6705, 6577, 4087, 7031, 4310, 
              4098, 4306, 7029, 4084, 5591, 5592, 5384, 5819, 7016, 3914, 
              5820, 3906, 7015, 4721, 5324, 5323, 4096, 6702, 4305, 6701, 
              4304, 4620, 4109, 5260, 4619, 6907, 4622, 6906, 4621, 4083, 
              5237, 6573, 6572, 6571, 4589, 4793, 3905, 6889, 3899, 6450, 
              6449, 4390, 5092, 7102, 5091, 5204, 7023, 7022, 6542, 4735, 
              6986, 4570, 6888}],
             RowBox[{"-", "7"}]],
            Annotation[#, -7, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4095, 7034, 4322, 4303, 5736, 4172, 5735, 5738, 5737, 
              5764, 5733, 5734, 5374, 5375, 5376, 4302, 4389, 7061, 4388}],
             RowBox[{"-", "10.5`"}]],
            Annotation[#, -10.5, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4416, 6775, 4417, 5111, 5110, 6773, 6774, 5913, 7084, 
              4227, 6697, 4228, 5782, 4425, 5924, 7085, 4233, 6698, 4234, 
              6974, 6975, 6487, 6488, 5119, 4433, 6809, 4442, 6807, 4441, 
              6808, 4600, 6898, 6685, 6686, 6684, 4184, 6687, 6997, 4753, 
              7037, 4104, 5337, 4103, 4602, 7033, 4091, 5216, 4616, 7069, 
              4617, 4105, 5620, 4603, 6250, 5602, 5603, 5217, 5259, 6422, 
              4754, 6421, 4755, 5328, 5327, 4729, 6977, 4730, 6978, 4319, 
              5751, 4185, 5750, 4186, 5752, 4577, 6222, 5212, 5213, 4601, 
              6899, 6901, 6900, 3932, 4019, 6553, 6554, 6683, 6682, 4182, 
              4331, 7052, 4330, 4636, 6607, 6608, 5261, 4635, 6920, 6921, 
              6917, 6919, 6918, 6939, 6720, 6721, 6719, 6723, 6722, 4138, 
              4373, 4180, 7045, 4181, 4160, 5355, 4161, 4892, 5701, 5702, 
              5115, 5116, 4727, 7018, 3923, 4011, 4899, 7025, 7026, 6897, 
              4597, 5211, 5210, 6406, 4736, 5743, 4177, 5396, 5397, 5398, 
              4311, 4748, 7036, 4100, 5333, 5334, 4750, 6994, 4749, 6995, 
              5209, 5208, 6580, 6579, 6578, 4595, 6896, 4726, 4574, 7066, 
              4573, 5207, 4085, 7030, 4591, 4722, 5325, 4097, 7035, 4747, 
              6575, 6574, 7083, 4174, 6681, 6465, 6466, 3907, 6890, 4590, 
              6457, 6456, 6455, 4392, 6765, 4391, 5095, 4715, 6961, 6963, 
              6962, 4210, 5768, 4209, 5769, 6454, 5767, 4208, 5765, 4207, 
              5766, 6453, 3900, 5093, 5094, 4714, 6451, 6452}],
             RowBox[{"-", "10.5`"}]],
            Annotation[#, -10.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            TooltipBox[
             LineBox[{3901, 6459, 6458, 6965, 6964, 4393, 6968, 4717, 6966, 
              4716, 6967, 5097, 6766, 4396, 5895, 4397, 5896, 6576, 4086, 
              5205, 5206, 4571, 6545, 6546, 6547, 3908, 6240, 6241, 6239, 
              6238, 5385, 4409, 5107, 5106, 5108, 4940, 3918, 5392, 5391, 
              4412, 6873, 6874, 6548, 6549, 6473, 6472, 6471, 6772, 6771, 
              4222, 6779, 6780, 6776, 6778, 6777, 4012, 3924, 7078, 5552, 
              5399, 5915, 5914, 4230, 5925, 4428, 6788, 4427, 6876, 4576, 
              6875, 4575, 6949, 6950, 6668, 6669, 6670, 5885, 5886, 5884, 
              6551, 6550, 6552, 4015, 3926, 6479, 6478, 4426, 4677, 7072, 
              4676, 4127, 7042, 4358, 4740, 6931, 6932, 6628, 6629, 6630, 
              5282, 5283, 4767, 7002, 4766, 4020, 3933, 4953, 4954, 4780, 
              5351, 4829, 4830, 6496, 5423, 4443, 5980, 4243, 6008, 6009, 
              6007, 6558, 6557, 4031, 3943, 6506, 6505, 6819, 6818, 4249, 
              6828, 6829, 6826, 4473, 6827, 7027, 6559, 6880, 6879, 4474, 
              6034, 6036, 6035, 4971, 3956, 6980, 6981, 6399, 6401, 6400, 
              4605, 6903, 4604, 6902, 6274, 6408, 6407, 6987, 6988, 4841, 
              4840, 5357, 4618, 5219, 5218, 4092, 5605, 5604, 7065, 4471, 
              6825, 6512, 6513, 5131, 6504, 6503, 6502, 4728, 6976, 4458, 
              6816, 6817, 6813, 6815, 6814}],
             RowBox[{"-", "14"}]],
            Annotation[#, -14, "Tooltip"]& ], 
           TagBox[
            TooltipBox[LineBox[{4329, 5052, 5051, 5373, 5372}],
             RowBox[{"-", "14"}]],
            Annotation[#, -14, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4394, 5098, 5096, 6461, 6460, 5378, 4718, 6395, 4395, 
              5100, 4400, 5897, 4401, 5898, 4212, 5534, 5535, 4005, 3909, 
              5105, 6470, 6469, 5386, 4410, 4938, 4937, 5109, 4941, 3919, 
              5394, 5393, 4413, 4820, 4009, 3920, 6474, 5780, 4223, 5553, 
              5555, 6784, 6785, 6781, 6783, 6782, 4944, 3928, 6482, 6481, 
              6793, 6792, 5927, 5556, 5785, 5784, 5926, 4016, 3927, 6480, 
              5404, 6789, 6791, 6790, 5703, 7062, 6671, 5704, 6360, 6359, 
              5412, 5411, 4183, 5749, 5787, 5786, 5855, 4359, 5629, 5628, 
              5627, 4332, 6725, 6726, 6724, 4333, 4021, 4796, 4795, 5361, 
              5362, 4901, 4900, 4139, 5352, 5358, 5359, 5370, 5369, 4781, 
              5360, 4637, 5262, 5214, 5215, 5124, 4445, 5368, 4444, 4959, 
              4244, 6011, 4459, 6010, 6821, 6820, 4032, 3944, 5793, 5794, 
              5561, 5563, 6833, 6834, 6830, 6832, 6831, 4972, 3957, 6520, 
              6519, 6840, 6839, 6070, 5569, 4256, 5797, 6066, 6065, 4043, 
              4847, 4490, 6067, 6069, 6068, 4985, 3968, 4983, 4984, 4489, 
              6064, 4488, 6838, 5140, 6063, 4042, 7028, 6062, 5568, 4255, 
              6061, 4486, 5462, 6518, 5138, 4472, 6398, 4732, 6397, 4731, 
              6979, 5132, 5452, 5451}],
             RowBox[{"-", "17.5`"}]],
            Annotation[#, -17.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4398, 5101, 5099, 6463, 6462, 5379, 4719, 5104, 5103, 
              6767, 4405, 5902, 4406, 5903, 4213, 5536, 5538, 5539, 6467, 
              3910, 5772, 5773, 5542, 5543, 4006, 3915, 5775, 5776, 5547, 
              5548, 5549, 6475, 5395, 5910, 5909, 4224, 5554, 4225, 5781, 
              5919, 5918, 4945, 3929, 5406, 5405, 5929, 5928, 4823, 5933, 
              5934, 5930, 5932, 5931, 4949, 3934, 6489, 5788, 4236, 5557, 
              4235, 5955, 5956, 5954, 6797, 4435, 7087, 6555, 6556, 6877, 
              6878, 5706, 5705, 4163, 5313, 4162, 6673, 6672, 4690, 4692, 
              7073, 4691, 4798, 4797, 4772, 7043, 4140, 5304, 4141, 6744, 
              6745, 6742, 4360, 6743, 4075, 6747, 4361, 6746, 6749, 6748, 
              4296, 5345, 4295, 5043, 4638, 6289, 6286, 6287, 5440, 5439, 
              5438, 4460, 4836, 4033, 6508, 6507, 5441, 6013, 6012, 4250, 
              5562, 4251, 5795, 6040, 6039, 4973, 3958, 5466, 5465, 6072, 
              6071, 4848, 6076, 6077, 6073, 6075, 6074, 4986, 3969, 6527, 
              5805, 4265, 5570, 4264, 6109, 6110, 6848, 6847, 4508, 7090, 
              6108, 7089, 6107, 5577, 5804, 5803, 6106, 4049, 7082, 6105, 
              5576, 5802, 5801, 6103, 6104, 6102, 6101, 5574, 4263, 6524, 
              7088, 4506, 6846, 6525, 6526, 5149, 5150, 4734, 6402, 4733, 
              6982, 5139, 5464, 5463}],
             RowBox[{"-", "21"}]],
            Annotation[#, -21, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4402, 5102, 4399, 5901, 4404, 5899, 4403, 5900, 4935, 
              5906, 5907, 5770, 4214, 5537, 4215, 5908, 4408, 5382, 5383, 
              4939, 4217, 5238, 4818, 4819, 5387, 4942, 4219, 5240, 4821, 
              5912, 4415, 5911, 4414, 4943, 4226, 5917, 4421, 5916, 6787, 
              6786, 4013, 3925, 4946, 4947, 5118, 5117, 4824, 5941, 5942, 
              5938, 5940, 5939, 4951, 3936, 5414, 5413, 5959, 5958, 4826, 
              5122, 4237, 4950, 5121, 5120, 4022, 3935, 6490, 6799, 6798, 
              6802, 6801, 6800, 7064, 7063, 6674, 5707, 4693, 6361, 5425, 
              5424, 5754, 5753, 5789, 5856, 4114, 5631, 5630, 4334, 6729, 
              4336, 6727, 4335, 6728, 4026, 4800, 4799, 5363, 4187, 4902, 
              5687, 4142, 5263, 5264, 4758, 7038, 4115, 6288, 6227, 6015, 
              7095, 6016, 6014, 4461, 4965, 4252, 6038, 4475, 6037, 6836, 
              6835, 4038, 3950, 5453, 4974, 4257, 5152, 4849, 6083, 6084, 
              6080, 6082, 6081, 4989, 3971, 5486, 5485, 6114, 6113, 4856, 
              5164, 5165, 4988, 4987, 5163, 4050, 3970, 6528, 5484, 4509, 
              6111, 4277, 5002, 6532, 6146, 4530, 6147, 4863, 5250, 4276, 
              5001, 4529, 4862, 4861, 5248, 4275, 5000, 4526, 6143, 4527, 
              6144, 4274, 5575, 4273, 5808, 6531, 3982, 4998, 5482, 5481, 
              5480, 4507, 6100, 4487, 5151, 5483}],
             RowBox[{"-", "24.5`"}]],
            Annotation[#, -24.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4525, 6140, 6142, 6141, 4272, 5244, 5245, 4860, 4058, 
              5172, 4999, 6856, 6857, 6145, 4528, 6256, 4608, 6255, 4607, 
              6904, 3989, 4811, 4609, 5607, 6589, 6590, 5249, 5610, 5609, 
              5608, 4610, 6257, 4812, 6258, 4611, 5611, 6591, 6592, 5251, 
              5252, 4613, 6259, 4612, 6905, 3990, 6260, 6261, 6152, 4532, 
              6151, 4533, 4864, 4059, 6858, 4531, 6148, 6150, 6149, 4278, 
              6117, 6118, 5487, 6529, 3972, 4051, 6849, 6850, 6112, 6116, 
              6115, 4266, 6085, 6086, 5467, 6521, 3959, 4044, 6841, 6842, 
              6078, 4491, 6079, 4258, 6041, 4476, 5455, 6514, 4843, 4842, 
              5265, 5266, 4967, 4966, 3945, 4034, 6933, 4658, 6324, 4659, 
              5268, 5267, 6635, 6634, 5663, 6326, 6325, 6992, 6750, 6751, 
              6631, 6632, 6633, 4678, 6940, 4679, 6941, 4446, 5426, 5559, 
              5560, 6497, 4027, 5558, 7081, 5085, 7101, 5086, 5365, 5364, 
              4378, 6363, 4695, 6362, 4694, 6224, 4578, 6223, 6226, 6225, 
              4447, 6810, 4448, 5982, 4240, 5962, 4436, 5415, 6491, 3937, 
              4023, 6803, 6804, 5957, 5961, 5960, 4238, 5943, 5944, 5407, 
              6483, 3930, 4017, 6794, 6795, 5935, 5937, 5936, 4231, 5920, 
              5921, 5403, 6476, 6477, 4014, 4822, 4422, 5400, 5401, 5402, 
              6249, 6248, 4898, 6247, 4598, 6246, 4599, 5242, 5241, 5600, 
              5599, 5598, 4744, 6418, 4175, 6245, 4596, 5597, 6581, 6582, 
              5239, 5596, 5595, 5594, 4743, 4895, 4894, 6243, 4592, 6242, 
              4593, 6244, 4407, 5381, 6468, 3911, 4936, 3902, 4003, 4815, 
              5234, 5233, 4211, 5904, 5905, 5380, 6464}],
             RowBox[{"-", "28"}]],
            Annotation[#, -28, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            TooltipBox[LineBox[CompressedData["
1:eJwV0UlszVEUx/ErYkFJa6jWFEMlIprYdjRsEPoIMUZLWiIxtSQWjYiaWoRG
DK1hQSToICEhoVE1tbQxtmFBFJUm5qkVdGH6vMX3/f7nnN859577RuYVzMnv
FkKI9TMxNoTUISGk4XdiCOv6hRA3KIS+iEWJeLbaV74n8SEM6x9CJC6EDQNC
qFGrGxjCY/EztWbfmYNDaBfX623AN/kS/YVye9RPiU9jmnnXxTdwDbdwE935
OtQ7UYkqVPBmoUH9NurRiDvowX/bOd/5djpnk3i//DnxeczWd8U9n8q/Fm+2
YxG2YguyEEFSgjv1DWEUnUrn2nOXnnTxTHEqjdC99rtg5jxnzsVIMzPVMrCP
PxL1YwbKectQw5/Dm42x/PPMmaW+nz+X5omX0lza5q6vsFy8TJwT1agHre65
yqzpZs5Aiz07zPhHh/KukBtCF3qHRZiPI2YtoFU0yfnVdDQ9TN/7Dz+gFc8x
xs6dZg0ws0hPh++ImVnolCuVS5Eb77vLjIviEeb0ojFokRuoVmZWPC2nh/ES
L5BgfiKO6kngPUYT6XB6wG6HcBDxvAV8hXbN9l4zsddejd6gk/c7NupbIl8q
3yR/1nnv6QnxW3qc1rv3EzOKsY1/Pf9HtQ+4zB/jrN7oiUv89/hbeUt4i1HI
f1m+j3od/0l3Ko3eBU3yyfLZWIzt2IEvap/xCa/1TNYzCdXmVSHZGVfld3uv
rugcud7ih971ER4ggycdE/W1u08lTzPvL/xEDP89vvu4izTeVEzgr+A9g3Hi
Wr5dzvmhp1HuDt7JTYjewXct2sQp4pV0indfRTPNeaPnj7M3yxXhr31X228t
1iAf/wGlLqdx
              "]],
             RowBox[{"-", "31.5`"}]],
            Annotation[#, -31.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3931, 6486, 5410, 5951, 5950, 4825, 5953, 4432, 5946, 
              4431, 5952, 4952, 5419, 5418, 5408, 4438, 5968, 4827, 5970, 
              4440, 5964, 4439, 5969, 4955, 5428, 5427, 5416, 4452, 5989, 
              4833, 5992, 4453, 5984, 5991, 5990, 4960, 3947, 6951, 4696, 
              6365, 4697, 5126, 5125, 4164, 5708, 4904, 4189, 5314, 4188, 
              5791, 5792, 5790, 4245, 6327, 4661, 5664, 5665, 5284, 4660, 
              5287, 5288, 4787, 5286, 5285, 6935, 3951, 6516, 6515, 6651, 
              6650, 4144, 6649, 6943, 6942, 6343, 6934, 4662, 7092, 6636, 
              4580, 7067, 7068, 6882, 6881, 6231, 5457, 4477, 6042, 4259, 
              5153, 5154, 4977, 4976, 5156, 4990, 3974, 5490, 5469, 4512, 
              6126, 4857, 6130, 4514, 6120, 4513, 6129, 5003, 5498, 5497, 
              5488, 4536, 6158, 4865, 6160, 4538, 6154, 4537, 6159, 5026, 
              5519, 5518, 5495, 4553, 6189, 4872, 6191, 6192, 6190, 6864, 
              6863}],
             RowBox[{"-", "35"}]],
            Annotation[#, -35, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4080, 5235, 4081, 6916, 4627, 6282, 4628, 4634, 7071, 
              4633, 4112, 4885, 4756, 6601, 6602, 6603, 5331, 5332, 4746}],
             RowBox[{"-", "35"}]],
            Annotation[#, -35, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{5329, 5191, 4106, 6284, 4629, 6283, 4630, 4632, 7070, 
              4631, 4113, 4886, 4757, 6604, 6605, 6606, 5338, 5626, 5625}],
             RowBox[{"-", "35"}]],
            Annotation[#, -35, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3938, 6494, 5420, 5972, 5971, 4828, 5976, 5977, 5973, 
              5975, 5974, 4958, 3941, 5432, 5431, 5998, 5997, 4834, 5128, 
              5129, 4957, 4956, 5127, 4028, 3940, 6498, 5430, 5994, 5993, 
              4246, 6018, 4463, 6822, 4462, 6229, 6230, 6228, 4579, 6371, 
              4700, 6370, 6675, 5315, 4698, 6369, 4699, 6366, 6368, 6367, 
              6372, 3952, 7007, 7008, 7006, 4773, 7009, 4680, 5305, 6653, 
              6652, 4681, 6944, 5667, 5666, 4362, 5857, 4582, 5858, 4363, 
              5668, 6637, 5818, 4297, 5815, 5817, 5816, 6300, 6293, 3961, 
              6517, 5471, 5456, 4494, 6089, 4852, 4853, 6523, 5473, 4496, 
              6090, 4268, 6127, 6128, 6124, 6853, 6852, 4053, 3975, 5004, 
              5005, 5174, 5173, 4866, 6167, 6168, 6164, 6166, 6165, 5027, 
              3991, 5521, 5520, 6196, 6195, 4873, 6200, 6201, 6199, 6868, 
              6867}],
             RowBox[{"-", "38.5`"}]],
            Annotation[#, -38.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3992, 4068, 6865, 6866, 6193, 6198, 6197, 4286, 6169, 
              6170, 5500, 6535, 3983, 4061, 6860, 4539, 6161, 4540, 6163, 
              4280, 5006, 4517, 6125, 4518, 6135, 4858, 6133, 4515, 5472, 
              5491, 3976, 4991, 5157, 6297, 6298, 6296, 4641, 6299, 4495, 
              5155, 5632, 6294, 6924, 6295, 5634, 5633, 4119, 5044, 5636, 
              5635, 5844, 4343, 7056, 5689, 6654, 7054, 7055, 6737, 6738, 
              4342, 6736, 5875, 5876, 5878, 5877, 4905, 6953, 4701, 6952, 
              4702, 5273, 5272, 6425, 4763, 5888, 4381, 5709, 6676, 5843, 
              4339, 5842, 6735, 6734, 3954, 6373, 6020, 6021, 6017, 4465, 
              6022, 4837, 6019, 4464, 5429, 5442, 4961, 4247, 6001, 4454, 
              5434, 6499, 6500, 4029, 6811, 6812, 5995, 6000, 5999, 4242, 
              5978, 5979, 5422, 6495, 3939, 4025, 5123}],
             RowBox[{"-", "42"}]],
            Annotation[#, -42, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4555, 6194, 4556, 6207, 4874, 6204, 4554, 5499, 5522, 
              5523, 5028, 6173, 4543, 6162, 6175, 6174, 4867, 5175, 5176, 
              5008, 5007, 3977, 4054, 6854, 6855, 6131, 4516, 6134, 4269, 
              4992, 4497, 5274, 4498, 5158, 5221, 5220, 6329, 4664, 6328, 
              6331, 6330, 4146, 5292, 4145, 6656, 6655, 4682, 4801, 3953, 
              5566, 5564, 5565, 5055, 5056, 5054, 4340, 5289, 4341, 5860, 
              5861, 5859, 4364, 6752, 4968, 4969, 4908, 4190, 5053, 4802, 
              6374, 6375, 5711, 6677, 5134, 4466, 6437, 4782, 5443, 3948, 
              4035, 5133, 4962, 6440, 4963, 5136, 5135, 4838, 6025, 6026, 
              5433, 5444, 5445, 4964, 6003, 4456, 5996, 4457, 6004, 4835, 
              6002, 4455, 5421, 5435}],
             RowBox[{"-", "45.5`"}]],
            Annotation[#, -45.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3993, 4069, 6869, 6870, 6202, 6206, 6205, 4287, 6176, 
              6177, 5502, 6536, 3984, 4062, 6861, 4541, 6171, 4542, 5010, 
              5009, 6136, 4521, 6132, 6138, 6137, 4859, 5167, 4270, 4993, 
              5474, 3962, 4045, 5571, 4663, 5669, 5670, 4129, 5045, 4130, 
              6334, 4667, 6333, 4668, 5294, 5293, 4131, 5290, 5291, 4666, 
              6332, 4665, 6936, 4128, 6344, 6044, 6045, 6043, 4478, 5567, 
              4039, 4906, 4907, 4382, 5710, 5712, 5713, 6377, 6376, 5142, 
              6439, 4783, 6438, 4784, 5367, 5366, 5371, 4790, 6448, 4791, 
              6447, 6446, 4785, 6441, 4786, 6442, 4467, 5447, 6509, 6510, 
              4036, 6823, 6824, 6023, 6028, 6027, 4248, 6005, 6006, 5437, 
              6501, 3942, 4030, 5130}],
             RowBox[{"-", "49"}]],
            Annotation[#, -49, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4558, 6203, 4559, 6213, 4875, 6210, 4557, 5501, 5524, 
              5525, 5029, 6178, 6179, 6172, 6181, 6180, 4868, 5177, 4281, 
              5012, 5492, 3978, 4055, 5166, 4519, 5275, 4520, 6302, 6303, 
              6301, 6928, 4642, 6927, 6305, 4644, 6304, 4645, 6306, 4298, 
              5307, 5308, 4882, 4077, 5076, 4076, 5075, 5587, 5586, 5585, 
              4583, 5227, 5226, 4147, 6657, 5690, 6688, 6689, 6690, 5755, 
              5475, 6378, 6379, 5714, 6678, 5316, 4703, 6957, 4704, 6954, 
              6956, 6955, 4165, 5057, 6047, 6048, 6046, 4479, 6049, 4739, 
              6412, 6414, 6413, 4737, 6991, 4738, 7103, 6411, 6989, 7076, 
              6990, 6409, 6410, 5796, 7077, 4253, 5144, 4844, 6052, 6053, 
              5446, 5458, 3955, 4970, 6030, 4469, 6024, 4470, 6031, 4839, 
              6029, 4468, 5436, 5448}],
             RowBox[{"-", "52.5`"}]],
            Annotation[#, -52.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3994, 4070, 6871, 6872, 6208, 6212, 6211, 4288, 6182, 
              4545, 5506, 6537, 6538, 4063, 4869, 4544, 5011, 5503, 5504, 
              5013, 5169, 5170, 4643, 5643, 6614, 6615, 5168, 4120, 5641, 
              4646, 5646, 5647, 5648, 6349, 6348, 4909, 4910, 4684, 5693, 
              6658, 4150, 5306, 4148, 5228, 4149, 5692, 5691, 4683, 5644, 
              4121, 5060, 6617, 6616, 5645, 4346, 6610, 7040, 6611, 6926, 
              6925, 6612, 6613, 5889, 5890, 5640, 5639, 5637, 4344, 4803, 
              3964, 5317, 5318, 5143, 4482, 5330, 4483, 4845, 6380, 4040, 
              6837, 4480, 6050, 4481, 6054, 4254, 6032, 6033, 5450, 6511, 
              3949, 4037, 5137}],
             RowBox[{"-", "56"}]],
            Annotation[#, -56, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4562, 6209, 4563, 6216, 4876, 6214, 6215, 5505, 5526, 
              3995, 5031, 5184, 5018, 5019, 5179, 5178, 4282, 5171, 4522, 
              5017, 5493, 6233, 6234, 5642, 4581, 6232, 4670, 6335, 4671, 
              6336, 6659, 4151, 5298, 5299, 4685, 6345, 6347, 6346, 3963, 
              4046, 5061, 5058, 5059, 5296, 5295, 4345, 5863, 4366, 5862, 
              4365, 5798, 4260, 5638, 5800, 5799, 4191, 5088, 4192, 4912, 
              5715, 5716, 5145, 5146, 4705, 6381, 6959, 6958, 3965, 4979, 
              6056, 6057, 6051, 6060, 6059, 4846, 6055, 4484, 5449, 5459}],
             RowBox[{"-", "59.5`"}]],
            Annotation[#, -59.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3996, 4071, 5192, 4560, 5033, 4561, 5032, 5527, 5030, 
              5185, 4547, 5020, 4548, 5186, 4870, 5180, 5181, 5016, 5508, 
              5014, 3979, 4056, 5578, 4669, 5671, 5672, 5673, 5046, 4133, 
              5674, 4673, 5678, 6638, 6639, 5300, 4134, 5297, 5677, 5676, 
              5675, 4672, 4891, 4132, 6350, 6351, 6092, 4499, 6091, 4500, 
              5572, 4047, 4911, 5087, 4383, 5090, 4384, 6384, 6385, 6383, 
              4706, 6960, 5222, 5223, 5159, 6844, 4502, 6382, 5476, 5477, 
              4978, 6058, 4980, 4485, 4982, 5460, 5461, 4041, 5147}],
             RowBox[{"-", "63"}]],
            Annotation[#, -63, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3966, 5478, 4981, 4262, 5148, 4261, 6095, 6096, 6093, 
              6845, 4501, 6843, 6560, 6561, 6883, 6885, 6884, 5718, 5717, 
              4167, 5089, 4166, 6679, 6760, 6759, 6763, 6762, 6761, 7060, 
              7059, 6680, 5719, 4387, 5892, 5494, 5756, 6691, 5757, 5695, 
              5694, 4152, 5229, 6886, 4584, 6237, 4585, 5078, 5077, 4078, 
              5079, 5080, 4884, 4883, 5311, 4299, 6315, 6316, 6314, 4653, 
              6309, 7096, 4649, 5509, 5507, 5015, 4546, 4871, 6311, 4064, 
              6540, 6539, 5511, 6184, 6183, 4289, 5193, 4290, 5034, 5199, 
              5198}],
             RowBox[{"-", "66.5`"}]],
            Annotation[#, -66.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4503, 4855, 4854, 6099, 4504, 6094, 6098, 6097, 6386, 
              3980, 5845, 4347, 5573, 4348, 5891, 4386, 6764, 4385, 6308, 
              4648, 6307, 4647, 5849, 4351, 5651, 6618, 4123, 5063, 4122, 
              6945, 6946, 6352, 4686, 5231, 5230, 4153, 5309, 5310, 4687, 
              6353, 6355, 6354, 4913, 5279, 4124, 6620, 6619, 4765, 6318, 
              6317, 4650, 6310, 5649, 5650, 5182, 5183, 4652, 6312, 4651, 
              6313, 5187, 5035, 3997, 5528, 5510, 4564, 6217, 4877, 4878, 
              5529}],
             RowBox[{"-", "70"}]],
            Annotation[#, -70, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3998, 4072, 5194, 5195, 5037, 4565, 5197, 4292, 6186, 
              5036, 4549, 5278, 4550, 5188, 5225, 5224, 5276, 5277, 4764, 
              5347, 5348, 5344, 5343, 5350, 4154, 6660, 6661, 7011, 7010, 
              4775, 6432, 4774, 5349, 4804, 4805, 6563, 6562, 7086, 4350, 
              6739, 5846, 5848, 5847, 4349, 5065, 4889, 4890, 5082, 5081, 
              4994, 4995, 4915, 4194, 5062, 4806, 5319, 4707, 4918, 5720, 
              5162, 4505, 6387, 6388, 5479, 3967, 4048, 5160}],
             RowBox[{"-", "73.5`"}]],
            Annotation[#, -73.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3981, 4807, 4708, 5721, 5723, 4169, 5161, 4168, 4916, 
              4917, 4709, 5320, 4193, 4914, 5579, 6356, 6357, 4997, 7093, 
              4996, 4135, 5064, 6641, 6640, 6753, 4367, 6755, 6754, 7058, 
              7057, 6645, 5682, 6692, 6693, 4197, 5759, 4196, 5758, 6338, 
              6337, 6643, 6642, 6428, 4769, 6427, 4770, 6430, 6429, 6426, 
              5582, 5581, 5513, 6187, 6185, 4291, 5196, 4293, 5038, 5203, 
              5202}],
             RowBox[{"-", "77"}]],
            Annotation[#, -77, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4195, 5806, 5807, 5722, 4271, 6393, 4712, 5724, 5726, 
              5321, 4710, 6392, 4711, 6389, 6391, 6390, 6394, 3985, 7012, 
              7013, 6433, 4776, 6434, 4688, 5312, 6665, 6947, 6948, 6664, 
              7044, 6663, 5680, 5679, 4368, 5864, 4586, 5865, 4369, 5681, 
              6644, 5049, 4300, 5346, 4768, 5048, 6564, 5047, 6321, 6319, 
              3999, 6541, 5530, 5512, 4567, 6218, 4879, 4880, 5532}],
             RowBox[{"-", "80.5`"}]],
            Annotation[#, -80.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4000, 4073, 5200, 4566, 5280, 4568, 6320, 4655, 5531, 
              4654, 6322, 6323, 5660, 6627, 5050, 4301, 5854, 4357, 6740, 
              6741, 5881, 5880, 6757, 5851, 7098, 5852, 5850, 4355, 5853, 
              6662, 5696, 4374, 5879, 4198, 5760, 6694, 5761, 5729, 5728, 
              4171, 5322, 5732, 5731, 5730, 4713, 5727, 4170, 5067, 6139, 
              4523, 5725, 4524, 5580, 4057, 4919}],
             RowBox[{"-", "84"}]],
            Annotation[#, -84, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3987, 4810, 4352, 4923, 5652, 5653, 5894, 5893, 6624, 
              6623, 6929, 6930, 6622, 7041, 6621, 4356, 5658, 6625, 6626, 
              5069, 4126, 5657, 4375, 5697, 5698, 4156, 5073, 4155, 5074, 
              4157, 6666, 5699, 4376, 4921, 4920, 6358, 4689, 5662, 5661, 
              5659, 4657, 5654, 4125, 5201, 5656, 5655, 4934, 4656, 4932, 
              4931}],
             RowBox[{"-", "87.5`"}]],
            Annotation[#, -87.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4205, 5281, 4206, 4933, 5040, 5039, 5870, 4372, 5869, 
              5872, 5871, 4159, 4930, 4158, 6667, 5700, 4377, 4808, 3986, 
              4065, 5071, 5072, 5068, 4353, 5083, 4354, 5867, 7097, 5868, 
              5866, 4371, 6756, 5021, 5022, 4924, 4200, 5066, 4809}],
             RowBox[{"-", "91"}]],
            Annotation[#, -91, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4199, 4922, 5514, 6403, 6404, 5024, 6983, 5023, 4136, 
              5070, 5684, 5683, 6339, 4370, 5084, 6647, 5686, 6695, 6696, 
              4204, 5763, 4203, 5762, 5883, 5882, 4137, 5041, 5042, 5303, 
              5302, 4881, 4814, 4813}],
             RowBox[{"-", "94.5`"}]],
            Annotation[#, -94.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4001, 4074, 6937, 6938, 6340, 4674, 6341, 4614, 5255, 
              6646, 5685, 4675, 6342, 4741, 5232, 4587, 5301, 7004, 7005, 
              6431, 7094, 4771, 7003, 6862, 7074, 7075, 6984, 6985, 6405, 
              5515, 5583, 7019, 3988, 4066, 6565}],
             RowBox[{"-", "98"}]],
            Annotation[#, -98, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4569, 5189, 4551, 6265, 7099, 6266, 6262, 6264, 7100, 
              6263, 6913, 6277, 6278, 6267, 6269, 6268, 4615, 5256, 5615}],
             RowBox[{"-", "101.5`"}]],
            Annotation[#, -101.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4094, 6593, 6594, 6915, 6914, 4625, 5254, 5253, 4110, 
              5257, 5258, 4626, 6279, 6281, 6280, 6285}],
             RowBox[{"-", "105"}]],
            Annotation[#, -105, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4111, 5811, 5812, 5809, 4294, 5810, 5814, 5813, 6219}],
             RowBox[{"-", "108.5`"}]],
            Annotation[#, -108.5, "Tooltip"]& ]}, {}, {}}}],
       AspectRatio->1,
       BaseStyle->{FontFamily -> "Times", FontSize -> 12},
       DisplayFunction:>Identity,
       Frame->True,
       PlotLabel->FormBox[
        "\"\[Mu]=9 \[Alpha]=0.25 \[Epsilon]=10 PIN=0.10\"", TraditionalForm],
       PlotRange->{{0, 50}, {0, 50}},
       PlotRangeClipping->True,
       PlotRangePadding->{
         Scaled[0.02], 
         Scaled[0.02]}], {576., -192.}, 
      ImageScaled[{0.5, 0.5}], {360., 360.}]}, {InsetBox[
      GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyEXWd4VcXTD0hT6dICoRNqgHQIaXN7QjMEpIM0pfem8KcpIKFKL1KUjhQp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        "], {{
          {RGBColor[0.293416, 0.0574044, 0.529412], EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{6765, 6766, 3183}, {3313, 1464, 967}, {6915, 6914, 
              3313}, {6623, 4173, 677}, {6912, 6913, 3312}, {967, 1464, 
              225}, {1464, 3183, 677}, {1464, 677, 225}},
              VertexColors->None], 
             PolygonBox[{{4911, 6765, 3183, 1464}, {6911, 6915, 3313, 967}, {
              6766, 6623, 677, 3183}, {6913, 6911, 967, 3312}, {6914, 4911, 
              1464, 3313}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.316754065115743, 0.09798299244167236, 0.5622164737961536], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{3020, 3801, 1072}, {3801, 3020, 1248}, {2674, 2672, 
              1074}, {3310, 3311, 364}, {1073, 3310, 364}, {3728, 3310, 
              359}, {4678, 6967, 1072}, {3660, 1248, 360}, {3729, 3728, 
              3036}, {6623, 6766, 3037}, {5267, 4172, 359}, {3037, 3729, 
              3036}, {6765, 4911, 645}, {3311, 3310, 645}, {4172, 5264, 
              359}, {3310, 3728, 645}, {3728, 3729, 645}, {4911, 6914, 645}, {
              3036, 3728, 359}, {6911, 6913, 3311}, {3660, 3036, 1072}, {1074,
               1073, 364}, {6297, 4679, 1073}, {6968, 6610, 3020}, {3036, 
              3660, 360}, {3801, 3660, 1072}, {4679, 5268, 1073}, {5265, 4678,
               1072}, {6298, 6299, 2672}, {3037, 3036, 360}, {3660, 3801, 
              1248}, {6915, 6911, 3311}, {6610, 6609, 3020}, {1072, 3036, 
              359}, {3310, 1073, 359}, {1074, 2672, 1073}, {3729, 3037, 645}},
              
              VertexColors->None], 
             PolygonBox[{{6967, 6968, 3020, 1072}, {4173, 6623, 3037, 360}, {
              5268, 5267, 359, 1073}, {6914, 6915, 3311, 645}, {6299, 6297, 
              1073, 2672}, {6913, 6912, 364, 3311}, {6766, 6765, 645, 3037}, {
              6609, 4160, 1248, 3020}, {5264, 5265, 1072, 359}, {6300, 6298, 
              2672, 2674}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.34403350194109844`, 0.14541456585980173`, 0.6005610223297655], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{3374, 1765, 1061}, {1764, 2667, 1061}, {2669, 2670, 
              1059}, {2670, 1762, 1059}, {1071, 2664, 1059}, {6419, 4674, 
              1764}, {7189, 6289, 2664}, {3375, 2668, 1061}, {4790, 6367, 
              1249}, {1765, 3374, 1071}, {6297, 6299, 2671}, {2673, 2671, 
              1061}, {1249, 1764, 307}, {1764, 2671, 307}, {1716, 2669, 
              1059}, {2074, 1765, 1071}, {3381, 2664, 1071}, {4160, 6609, 
              306}, {6296, 6295, 2668}, {1762, 2074, 1071}, {2669, 1716, 
              306}, {1765, 2673, 1061}, {5529, 5544, 966}, {4602, 6253, 
              1716}, {6298, 6300, 307}, {6367, 4791, 1249}, {2668, 3374, 
              1061}, {5267, 5268, 1765}, {7188, 6290, 2665}, {6288, 7189, 
              2664}, {2668, 3381, 1071}, {4623, 5529, 966}, {6291, 7188, 
              2665}, {5265, 5264, 1762}, {1762, 1071, 1059}, {1716, 966, 
              306}, {6968, 6967, 3382}, {2665, 1716, 1059}, {3374, 2668, 
              1071}, {3382, 2670, 2669}, {2671, 1764, 1061}, {2667, 3375, 
              1061}, {6293, 6294, 2667}},
              VertexColors->None], 
             PolygonBox[{{4791, 6419, 1764, 1249}, {6290, 4602, 1716, 2665}, {
              6292, 6296, 2668, 3375}, {6289, 6287, 1059, 2664}, {4679, 6297, 
              2671, 2673}, {6253, 4623, 966, 1716}, {6610, 6968, 3382, 
              2669}, {5268, 4679, 2673, 1765}, {5264, 4172, 2074, 1762}, {
              6966, 6288, 2664, 3381}, {6967, 4678, 2670, 3382}, {6295, 6966, 
              3381, 2668}, {6294, 6292, 3375, 2667}, {6609, 6610, 2669, 
              306}, {6287, 6291, 2665, 1059}, {6299, 6298, 307, 2671}, {4674, 
              6293, 2667, 1764}, {4678, 5265, 1762, 2670}, {4172, 5267, 1765, 
              2074}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.37131293876645394`, 0.19284613927793112`, 0.6389055708633774], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{7039, 6407, 2785}, {3911, 3649, 2820}, {3650, 2821, 
              2820}, {3857, 2820, 949}, {7037, 7038, 2604}, {4132, 7000, 
              224}, {1980, 2626, 949}, {2604, 1980, 949}, {6287, 6289, 
              2663}, {6291, 6287, 2663}, {6296, 6292, 2666}, {4734, 6368, 
              2748}, {2747, 2800, 224}, {4602, 6290, 949}, {6407, 6408, 
              2785}, {6419, 4791, 2801}, {5266, 6661, 1145}, {2934, 1994, 
              1980}, {3858, 3859, 2663}, {3911, 3857, 2663}, {2821, 3650, 
              1013}, {3649, 3648, 1013}, {1236, 3648, 234}, {3648, 3858, 
              234}, {4050, 4132, 224}, {3857, 3911, 2820}, {3649, 3911, 
              2663}, {3649, 3650, 2820}, {3859, 3649, 2663}, {7001, 6366, 
              2747}, {3304, 2604, 949}, {3649, 3859, 3648}, {7088, 6435, 
              2821}, {3512, 3304, 2820}, {6369, 4785, 1236}, {3650, 3649, 
              1013}, {3304, 3676, 2604}, {6288, 6966, 234}, {2821, 3512, 
              2820}, {4125, 6566, 210}, {2604, 2785, 1980}, {2934, 2969, 
              210}, {1994, 2934, 210}, {6567, 6568, 2969}, {2747, 2748, 
              1060}, {2801, 2747, 1060}, {1145, 1236, 234}, {6966, 6295, 
              234}, {6435, 7183, 2821}, {4785, 5245, 1236}, {4623, 6253, 
              2626}, {3648, 1236, 1013}, {5672, 4735, 1145}, {7183, 4818, 
              2821}, {6661, 5672, 1145}, {3859, 3858, 3648}, {6293, 4674, 
              1060}, {5310, 7087, 1013}, {2820, 3304, 949}, {1145, 2666, 
              1060}, {6292, 6294, 2666}, {2666, 1145, 234}, {7154, 7155, 
              3676}, {4643, 5310, 1013}},
              VertexColors->None], 
             PolygonBox[{{6408, 5598, 1980, 2785}, {7000, 7001, 2747, 224}, {
              6295, 6296, 2666, 234}, {5529, 4623, 2626, 1980}, {7188, 6291, 
              2663, 3857}, {6294, 6293, 1060, 2666}, {7189, 6288, 234, 
              3858}, {7155, 7037, 2604, 3676}, {6541, 6567, 2969, 2934}, {
              6290, 7188, 3857, 949}, {4673, 5266, 1145, 1060}, {6367, 4790, 
              2800, 2747}, {6366, 4734, 2748, 2747}, {4791, 6367, 2747, 
              2801}, {6368, 4673, 1060, 2748}, {5598, 6541, 2934, 1980}, {
              6253, 4602, 949, 2626}, {5245, 4643, 1013, 1236}, {4735, 6369, 
              1236, 1145}, {4674, 6419, 2801, 1060}, {5544, 5529, 1980, 
              1994}, {6289, 7189, 3858, 2663}, {7086, 6906, 3304, 3512}, {
              6906, 7154, 3676, 3304}, {4818, 7086, 3512, 2821}, {7087, 7088, 
              2821, 1013}, {6568, 4125, 210, 2969}, {7038, 7039, 2785, 2604}},
              
              VertexColors->None]}]}, 
          {RGBColor[
           0.39859237559180927`, 0.24027771269606032`, 0.6772501193969891], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{1297, 1798, 1143}, {3513, 1798, 1297}, {7039, 7038, 
              3458}, {1588, 2784, 1223}, {6567, 6541, 2043}, {3073, 1637, 
              429}, {3679, 1297, 427}, {1144, 1602, 428}, {2968, 2933, 
              2042}, {1979, 2043, 1223}, {5598, 6408, 2043}, {3456, 1588, 
              1223}, {4643, 5245, 1746}, {2933, 2043, 1979}, {5038, 4356, 
              1588}, {3456, 3679, 427}, {1143, 1746, 429}, {2749, 2123, 
              429}, {2204, 2123, 428}, {1798, 1746, 1143}, {7037, 7155, 
              3457}, {1637, 1143, 429}, {3810, 3457, 1296}, {3820, 3679, 
              1296}, {3457, 3810, 3456}, {2205, 3143, 2124}, {3458, 3457, 
              1223}, {2204, 2205, 2123}, {4367, 5063, 1602}, {6369, 4735, 
              2749}, {3810, 3679, 3456}, {3679, 3810, 1296}, {5105, 4420, 
              1637}, {1799, 1602, 1144}, {7088, 7087, 3513}, {3457, 3456, 
              1223}, {6721, 5673, 2124}, {4258, 6722, 3143}, {6541, 5598, 
              2043}, {1746, 2797, 429}, {3679, 3820, 1297}, {4197, 5867, 
              428}, {1588, 3456, 427}, {2205, 2124, 2123}, {5670, 5038, 
              1588}, {2123, 2124, 429}, {4734, 6366, 1144}, {3072, 1763, 
              428}, {4673, 6368, 1763}, {1763, 1144, 428}, {3415, 1463, 
              1144}, {1463, 1799, 1144}, {2784, 1979, 1223}, {2123, 3072, 
              428}, {2797, 2749, 429}, {7086, 4818, 1296}, {5063, 5671, 
              1602}, {4946, 4251, 1499}, {2746, 2120, 1143}, {5759, 4257, 
              2204}, {2042, 1979, 1499}, {5092, 4196, 427}, {7001, 7000, 
              3415}, {5266, 4673, 1763}, {5868, 5759, 2204}, {1143, 2120, 
              427}, {4125, 6568, 2933}, {2042, 2933, 1979}, {1637, 2746, 
              1143}, {4855, 4856, 1399}, {6906, 7086, 1296}, {5668, 5669, 
              2120}, {2124, 3073, 429}, {7154, 6906, 1296}, {1297, 1143, 
              427}},
              VertexColors->None], 
             PolygonBox[{{6722, 6721, 2124, 3143}, {4818, 7183, 3820, 1296}, {
              7087, 5310, 1798, 3513}, {5760, 4258, 3143, 2205}, {5311, 4367, 
              1602, 1799}, {6568, 6567, 2043, 2933}, {4735, 5672, 2123, 
              2749}, {4785, 6369, 2749, 2797}, {4257, 5760, 2205, 2204}, {
              7000, 4132, 1463, 3415}, {4132, 4050, 1399, 1463}, {6407, 7039, 
              3458, 1223}, {6408, 6407, 1223, 2043}, {5528, 4946, 1499, 
              1979}, {5673, 6662, 3073, 2124}, {5310, 4643, 1746, 1798}, {
              7155, 7154, 1296, 3457}, {6406, 5528, 1979, 2784}, {7183, 6435, 
              1297, 3820}, {4856, 4910, 1463, 1399}, {4356, 6406, 2784, 
              1588}, {6365, 5668, 2120, 2746}, {6366, 7001, 3415, 1144}, {
              6566, 4125, 2933, 2968}, {5672, 6661, 3072, 2123}, {5671, 4197, 
              428, 1602}, {6435, 7088, 3513, 1297}, {7038, 7037, 3457, 
              3458}, {5867, 5868, 2204, 428}, {6368, 4734, 1144, 1763}, {4420,
               6365, 2746, 1637}, {4196, 5670, 1588, 427}, {6662, 5105, 1637, 
              3073}, {4910, 5311, 1799, 1463}, {5669, 5092, 427, 2120}, {6661,
               5266, 1763, 3072}, {5245, 4785, 2797, 1746}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.42587181241716465`, 0.2877092861141895, 0.7155946679306008], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{2310, 732, 725}, {724, 1587, 633}, {6406, 4356, 
              633}, {1636, 1629, 723}, {5856, 7185, 2307}, {5088, 4852, 
              524}, {1629, 2307, 724}, {5036, 5037, 1587}, {723, 726, 209}, {
              5668, 6365, 723}, {1601, 725, 644}, {5094, 5090, 1627}, {4910, 
              4856, 644}, {6721, 6722, 3142}, {5061, 5859, 1601}, {5685, 4426,
               731}, {4252, 5088, 524}, {1627, 723, 209}, {6662, 5673, 726}, {
              3141, 731, 209}, {6796, 5036, 1587}, {4124, 5093, 209}, {5670, 
              4196, 2121}, {5062, 5061, 1601}, {2121, 1629, 724}, {1503, 731, 
              529}, {3094, 1503, 732}, {2317, 2122, 732}, {5092, 5669, 
              1629}, {4422, 5858, 725}, {5295, 4260, 530}, {732, 2122, 725}, {
              5528, 6406, 633}, {5760, 4257, 529}, {5855, 4421, 724}, {732, 
              1503, 529}, {5861, 5860, 2310}, {4946, 5528, 633}, {1503, 3093, 
              731}, {4039, 4124, 209}, {2310, 3094, 732}, {4212, 6681, 
              3093}, {1627, 1636, 723}, {5090, 4408, 1627}, {5105, 6662, 
              726}, {3141, 3142, 529}, {7185, 5857, 2307}, {4197, 5671, 
              2122}, {5311, 4910, 644}, {5868, 5867, 2317}, {731, 3141, 
              529}, {4426, 4851, 731}, {4259, 5295, 530}, {3142, 3141, 726}, {
              4258, 5760, 529}, {4367, 5311, 644}, {6365, 4420, 723}, {726, 
              3141, 209}},
              VertexColors->None], 
             PolygonBox[{{4421, 6796, 1587, 724}, {6722, 4258, 529, 3142}, {
              5093, 5094, 1627, 209}, {4851, 4039, 209, 731}, {5859, 4422, 
              725, 1601}, {5857, 5855, 724, 2307}, {5860, 4213, 3094, 2310}, {
              5858, 5861, 2310, 725}, {5037, 4947, 633, 1587}, {4257, 5759, 
              732, 529}, {4196, 5092, 1629, 2121}, {4856, 4855, 530, 644}, {
              5104, 4409, 1629, 1636}, {5038, 5670, 2121, 724}, {5673, 6721, 
              3142, 726}, {4251, 4946, 633, 524}, {6681, 5685, 731, 3093}, {
              4949, 4212, 3093, 1503}, {5669, 5668, 723, 1629}, {4408, 5104, 
              1636, 1627}, {4260, 4951, 644, 530}, {4213, 4949, 1503, 3094}, {
              5867, 4197, 2122, 2317}, {5671, 5063, 725, 2122}, {5063, 4367, 
              644, 725}, {5759, 5868, 2317, 732}, {4420, 5105, 726, 723}, {
              4947, 4252, 524, 633}, {4356, 5038, 724, 633}, {4951, 5062, 
              1601, 644}, {4409, 5856, 2307, 1629}},
              VertexColors->None]}]}, 
          {RGBColor[0.45315124924252, 0.3351408595323187, 0.7539392164642125],
            EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtlDtvU0EQhSc3vjaOE3xtR1fYoeEXUPEKEh2PmndiMBKihiYRjz9AA0Ix
ChUlLf8AEA2PAv4BUmJCkBAGBaUhKQLn09lidde7Z86cOTPrQzfvnL+dRURf
a1Jr0I5YnI24Uon4dSDibhmxui/ifUdnWqPc9zfaxlzV3VDfhkhu6WxG+L5+
Py8iLgj/RfhnwvyYNGZT91+1KvWIB/reL30/34zItTKdLylmum78sni+CfNu
IuJMK2JD+1x3/2YiasLv6bvVFV7nbzPzcj8U/pjwp6T1lc4fT5t/ZcJ60Q0n
ediT6+JUxHbSCg91gKHuF4VrX9ZaqJpzTTlHpTWDHSb8YXFcE+aNMI96Eadb
zt+r2w80NBS3WBiLx/eSzwPF7dSsgS9esD8hji3peJ25ZvipF9/w4smUaz3a
sjbykI892irJEzyrJt8K5XzYtm68QSO1/NX5bun4XeVZz91rYohFw5q0fGza
j0/Cfi7tHTxoQs+ljvuHJ2g/3nLsdelcKKznpM7GXdeyXrpOMHniWEmaqTFL
fY/9vqMP1EmNeI0nxO6U7j3n9Iwa0M9sNtN8/lbO6sGInw3NlTjHDXPiP37D
tyHNf7rmZb7Qis4j2nd7nptR8oZa0IU+MPhUS/N8bi7i7JzzwE1PmbcPut9u
+zdx8wmPFnRRe730u0IzOugBNX6Xts2O9aJ7nPB9aXzaczzv72V6g/R4Nc05
dYMDw9tj7tHFb3oOD+97kP4HuKO3eLJUuK/09DL8s54P9O+l2v8DpQqalw==

              "],
              VertexColors->None], 
             PolygonBox[{{7185, 5856, 701, 3846}, {5753, 6271, 2644, 2198}, {
              5858, 4422, 2309, 2308}, {4426, 5685, 2136, 2316}, {6983, 6639, 
              3052, 3398}, {4852, 5088, 1625, 1397}, {4743, 6381, 2197, 
              1170}, {5061, 5062, 1505, 402}, {4124, 4039, 1396, 2041}, {5859,
               5061, 402, 2309}, {5036, 6796, 3213, 401}, {5647, 5648, 1170, 
              1115}, {6681, 4212, 453, 2136}, {5837, 5642, 702, 2291}, {6636, 
              7122, 3584, 704}, {6271, 5836, 2291, 2644}, {5642, 5643, 2095, 
              702}, {6639, 6638, 3051, 3052}, {4851, 4426, 2316, 1396}, {5214,
               5213, 402, 1111}, {5293, 4702, 1111, 1504}, {6803, 5880, 2330, 
              3051}, {5093, 4124, 2041, 1630}, {5882, 5881, 700, 2330}, {4185,
               5641, 1115, 402}, {4421, 5855, 2306, 3213}, {5037, 5036, 401, 
              1500}, {4408, 5090, 702, 701}, {5860, 5861, 2308, 453}, {5750, 
              5754, 2198, 523}, {5088, 4252, 1500, 1625}, {7122, 6637, 2703, 
              3584}, {6641, 6640, 704, 1628}, {5295, 4259, 1504, 1786}, {6637,
               6984, 3398, 2703}, {5857, 7185, 3846, 2306}, {4260, 5295, 1786,
               1505}, {5752, 5750, 523, 2197}, {4406, 5087, 1397, 700}, {4186,
               5091, 1628, 2095}, {5090, 5094, 1630, 702}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.4804306860678755, 0.38257243295044807`, 0.7922837649978244], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VDtMFFEUvQtmXWZ0mR2XyeyMjaKJDWT9xMJC7RS1clYDm0ACNNpoA0Yt
pLSG3aixstLKwlYFC03ExB92EpXCxqwsakFEY6LneN4WL/PmzbnnnnPvfbNj
4uLpC11mdh6rG2sZL3MFs0bO7Lln9qloVt9k1vLNPmJ/Bvv9kdlQSdi9wK7h
WzOn55+89rPArePbE2CyomJ/4n2p16w/MFtBrqvguYTVxHmtqFyMe4OzcLvi
x0Kz+2WzDfC+70NOrPXNwjOO2qjhBrSexf5RbHYiBY+nnJnT/AIct8B1D/sZ
YK4gx7Occtacx44OnmdO7wfsq3i2fdXkNeIOleSLfH9jcS4AfwCr1aV37hm7
VjH7VpGG73j+qIhrErEnU+Wl17eR/I7jfD5WHYfwfTR0vvKqAeu2E9+/QMOd
buXJXK4WOG4H6l8dWN8TZhX4r7H6+TBWjVifu3nhiGG+/zUATxsa36XqOevD
OPaFvqYK8tgPjl2xcky5fDxvYD9dUM0yVxN6H84rH3ORe7Ui/gHg5vPyOIIe
+aG88mzW9fRUKg/UcTgBX6Qc7C852KeGpzkgfl9Be3oZdjmfmnBV19NFTzhi
lsDXl2iea24GOQOM46wxdhqYo4n8taHleiiOc9B2LVANqGtPIm3k4IyQZ3es
2SUX65T2qFacnZtuJj3EvsIaLOhOXHb3YmWb2cGSfHCOjqWqJ7mJJf8M8i+W
5etIohpRw/hWzRS1VF1fiOFdGXC1+gz+eqDZHnT3lfvjqeaA9f0FLb8j6fYj
aSV2FDnHyuoV+7zger2B7w1oaG4xe1DWTHO+HsPvRKhezbledP4z1Eqd7AP/
L+wFOVmbVuef5GJeRrrD9E5NzENM5w6zpwEwvZF6TE/LzlcPzkYC+f8HpG7C
YA==
              "],
              VertexColors->None], 
             PolygonBox[{{4410, 5841, 730, 703}, {4619, 5294, 1785, 1723}, {
              4708, 5641, 2093, 2710}, {7174, 6638, 1109, 3774}, {4759, 7016, 
              2164, 1191}, {6642, 5297, 1788, 3054}, {5214, 4703, 1785, 
              1725}, {5882, 5880, 3220, 2331}, {5320, 5321, 1191, 476}, {4131,
               5209, 1723, 223}, {6333, 4707, 1114, 1788}, {6696, 4223, 476, 
              2165}, {4443, 7174, 3774, 746}, {4186, 5643, 2094, 403}, {5182, 
              4407, 1626, 1702}, {5293, 4950, 223, 1784}, {6803, 4443, 746, 
              3220}, {6641, 5091, 403, 3053}, {5644, 6640, 3053, 2096}, {4584,
               6181, 2037, 928}, {7010, 5839, 2293, 3425}, {4411, 5644, 2096, 
              2295}, {5753, 5754, 2196, 452}, {5840, 5838, 703, 2293}, {4743, 
              5648, 2098, 2760}, {6379, 7010, 3425, 2759}, {4703, 5210, 1723, 
              1785}, {6987, 6988, 2759, 1114}, {5294, 4620, 1725, 1785}, {
              4620, 6330, 2705, 1725}, {4406, 5182, 1702, 1624}, {4185, 5213, 
              1725, 2093}, {5751, 6381, 2760, 404}, {5210, 4702, 1784, 
              1723}, {4250, 5749, 2195, 2194}, {5837, 5836, 452, 2094}, {5647,
               5645, 2097, 2098}, {4407, 5881, 2331, 1626}, {6720, 6719, 2165,
               2195}, {6986, 6985, 1110, 2705}, {6636, 4411, 2295, 3050}, {
              4706, 5646, 2097, 2707}, {6180, 6178, 928, 2571}, {6983, 6984, 
              3050, 1109}, {5594, 4943, 195, 2037}, {5646, 6642, 3054, 
              2097}, {5752, 5751, 404, 2196}, {4425, 5866, 2194, 730}, {4222, 
              5089, 1626, 2164}, {4701, 6332, 2707, 1110}, {5089, 6179, 2571, 
              1626}, {4040, 5087, 1624, 195}, {5645, 4708, 2710, 2097}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5077101228932309, 0.4300040063685775, 0.8306283135314363], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNVD1PFFEUveLKDsN+MTtMxh0SA2i0sTPRSKHZNVHUxGixy6okgI2FSqWN
Qf6CMRCMvTYmBgu3MKiJyCqNQih2/QiJnUilQWwsPMfzCovJvNx373nnnnve
65+YvHCjw8ym8O3EN9Jj9jsym+w0ux2YfYzNGr7ZWNYszJg1sV7Huoz147RZ
PWX2DLER/NM5s6G82ZZnNo31Yawfuv3Gfzmd+FZxqJdXPjErOHdPpPh8CC4l
s1vIv4nYg4KZ36H9fpdzrmj2qRf8dpkl2K8mZivgP4D9qUC1NU8cF3eYvUJ+
M6+ctZQ4bJjOZl/MqYPDfeRtAL+K2sDFV4F5FHvbiL0ERoDzXqO2gbOXgb8E
4XzEu/LCZR1zWbsIvFygnCN5aUJtWLsAjd8gfgp6zoLHjC+tqCl5Pof2mUA5
P3cDJ9IeZ0OOn4EfQ6fNRD18x/4weHZ3iQ+15iy8nNbUbR8wvyEn44srOVMH
/n03u3vYOwlOd/GfQ69LmIeH2gmcsxkrn/1xfuxxHPGxRB5poyaX1twPgc9W
SXO4iv059HItq9gvF6cWj1LSk9pkXe2HSOfxLPZWc3qy95avHM626uL0kef4
e85TjLNHzpu9U8snofQ8C+wzibRnrNvpfAnry6Fmw3nQR+RA/agjcelN+ox7
1Io6cV59ffBin+ZMDfZn1Bs/9knOLXj2T6J+v8IbdwqaI/F955/tSF7ccHeE
Wv+7U8g9UBLue+SsRIrzTkQl9XixR/WsfYs+mqE0nUd/65G0eBrKT/RSNpBH
6c8KcE9kdNeWkfMuVB+sp+7kX3V3ijnHcObxkmrokR+BfFLHerAgjxOz7HT4
EosHOYwj52BROdRxzelMX/r4rgOnBp3ase7EMNanE/mSeBXHk3eRmlLPafzb
veI3gLq9sbDol7bzzGAs3ak5NeI+16wpO0xy5ZqYfDP4vnC+Q+5O0QO8Y9SC
+byXE4H0ZC8z7q2jdsTiuQs4dzSQX4jNN5T45EZtyYF1s66WvMiPOvDOtBzP
uvMc+VwB3otYNbxP/bHuyTb4ni+qlu8j/5zLX//kzBs=
              "],
              VertexColors->None], 
             PolygonBox[{{6696, 5717, 1190, 3111}, {6361, 4704, 1112, 2742}, {
              5840, 7186, 3849, 2292}, {6718, 6719, 3140, 3139}, {5320, 4223, 
              3111, 1806}, {6333, 4705, 1112, 2709}, {6399, 4760, 2776, 
              2775}, {5593, 6181, 2572, 621}, {4250, 5866, 2315, 522}, {6677, 
              5841, 2294, 3090}, {6998, 6999, 2741, 208}, {4705, 5297, 1113, 
              1112}, {4222, 7016, 3436, 475}, {4909, 5211, 1724, 1462}, {6679,
               7009, 3424, 2758}, {5318, 4440, 1647, 1805}, {4733, 6364, 1235,
               1142}, {5212, 5056, 964, 1724}, {6379, 6380, 2708, 233}, {4760,
               5321, 1806, 2776}, {6720, 5749, 522, 3140}, {5056, 6329, 2704, 
              964}, {5879, 4442, 745, 1647}, {5016, 4342, 621, 745}, {5113, 
              5319, 1805, 3895}, {7164, 7165, 3711, 194}, {7107, 4030, 194, 
              3550}, {4584, 6178, 2570, 2572}, {6987, 4707, 2709, 3400}, {
              4131, 4049, 1993, 1462}, {5717, 6718, 3139, 1190}, {4706, 6332, 
              2706, 1113}, {7195, 5113, 3895, 3896}, {6986, 6331, 2704, 
              3399}, {4622, 5217, 1462, 965}, {7125, 6679, 2758, 3592}, {5592,
               7107, 3550, 1963}, {6380, 6988, 3400, 2708}, {6177, 5503, 1963,
               2135}, {6360, 4731, 2742, 2741}, {5594, 5593, 621, 1497}, {
              6678, 7125, 3592, 721}, {4410, 5838, 2292, 2294}, {4619, 5209, 
              1462, 964}, {6329, 5055, 208, 2704}, {6331, 6330, 964, 2704}, {
              4759, 6399, 2775, 3436}, {4945, 4247, 520, 621}, {7008, 6680, 
              3092, 3424}, {7165, 7196, 3896, 3711}, {5683, 6677, 3090, 
              2134}, {7186, 5839, 233, 3849}, {4701, 6985, 3399, 2706}, {7147,
               5684, 2135, 3092}, {5854, 4419, 721, 1235}, {4704, 5296, 1142, 
              1112}, {4425, 5683, 2134, 2315}, {6180, 6179, 475, 2570}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5349895597185863, 0.4774355797867067, 0.868972862065048], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtlE1I1FEUxa+m44yNOjmTjo6LxDYt2rksJEuFICqdsaggrbW2kFZlCe2q
hRGordqWFLS1hRZBgfvC7MMihaBW2aefnR/3Lh7/93/vvnPPPfe813rhUu9w
uZmNaezQ6K0z69N4U2lWrDFb32k2UmHWmTRb0/xBmVljs9nrglk2bfZC34Xd
Zj+rzB5rbtrbUNxSrVlJY1xny1Jm36vNpnW22CB8jRHhvcqZ/cuaDSXM9mht
UvM2zQ8Io03/PYr5nTGb0vperW81mW03OY/1hHN7KMySePbXOM/Du8w+NnjO
0bxZXpx6lXtOGDP679C5GsUcV+4bKrxW8/6Yr6ieZY37EuJm2vNQN5y+pL12
/sGmli7Nt4LDYqVzXCxzXhuxztrdtK+Xqa5yjU3tTVe5ZuSaqXYsMCek5aTG
U+2Na+1O0vfQein0PKWa+jX6oj9oQe3PVOO5nNd4td7skWIuVzhHYuADl3zK
+bS0iItixqJX84GPBsyp+Yz0+SM9m1J+ju9XaTUg/Ld5x72lmjoVd1vxyWqv
jbrwD9rDYT5ygHlMPEczzqE76Vqj66rODYU2+Kuh2eseDw3gj67UDOYVcVjI
e+14AD3+6uxpcXmZc56NwrmW93i07Aks8rfnnMN77Y/Wex/IT1/gsK35vlrn
iEfwDT7BR2iNznj2V8Y9ckIxz2P9vOYDGrMJz3so7bnB6oxev4N7g9d3TxgH
mz0OjcBDQ+4Gd5JauJfMh8KP6Ikni+F/tOZbjLtA3fiA2j+I18WMx0/qezLr
nuUe8kUH8It13ge0LAVOMd4E8uKVYszpA7nAXw3NeDeORI3kgu9wwvnDrxQ9
gjt+Jna2zt8RdH+Sc2/hq1Lsg8M7UWjxvGuar2cddzj0IW9X9BdtwUpVhw/r
nDcceYdm4i3C+xvhf/RORjx97w6cs+KzP+t9hy9+hBPYK/FWcF864u4cVexE
xvnk1c+Bgq9f151uzfu5T1r7XHBvDQq/VvXOKX+7vPCt4FjL4QPwqQlO8OmP
vqAld2Ez3pm+6Bt58dRUxn3FneR+Uc9gwd9uavhBTp0ZqfS7ST54/gfdv+Ix

              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtk8tvzXEQxefWRW9dt7f38XN7f00oNhZ2XZJGvRIJqhXvhda6tSgrKRIr
j0VF0tZ/gJB0y4JYqP+AqkdISCRs0Ja01focp4tJvvObmTNnZs6vvf98z2BD
ROzCslh7ErER62qM2INFPSKD/V0XsYh1lyNGSxH5tRHTqyMO4c+SX8tFfAUo
qRtDteV8xAXed4vg877Nt8lKxEtsYI3zu1oirlNzg/gtbBy8k3y7w3srOZup
X0V8id57qT+WRqTg/aF/73rqiN2su3Yf8S3kZ/GXyd+Pvx28Pvo9A+sa/XYS
21F3TJze1CIuMc8Qwy+Qs1D2TOI2iD9Lr03kNDaBjd8PVqHFWML8Bp8aeCPU
dxYipqoRV7Ac+TNwHOP9vmq+T7DD1HdjnSt8zlLfl3pXwpiD/ynwW3PeqXZ7
ED7TGfu9iW+g3Q8xQwO9p+F3P+MbLbfSj/g9/Hn84ZJ3qN09htNkM7zL7i8+
5zRf4tlUcwZupyvmphzdZqnVWLrRFL2Op969diat7IZvJmfNHAF7rujbieN4
0RrQ7Uewy+ziADmviD2g/2v8EjkD4P/Cfwj2o9Rau5j1LT/ibyv4ph1wO5p4
du1Es7+tGVs7aOc9S82XvG8grWhmzaob/GAfLyrW7xAcnkvPVcc17yj582Xf
XhyFNYN9zhtT2pxb0bs0qtsO07OnyfiLqWfSLFcLnu1ps7WgGdV7rOjdiMNv
sE60WG//b079htSzC/NTao1L29qBuHQk1po46d+YqFi7+kfStoi2NvfWP6Z/
9WfJs6rfBO93NWvhe5O1rZvoFtL4h8Q5iumm/wAoBpc/
              "],
              VertexColors->None], 
             PolygonBox[{{5060, 5218, 1727, 4622, 5059}, {5216, 5060, 5059, 
              4621, 1726}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5622689965439417, 0.5248671532048359, 0.9073174105986597], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNVEtIVGEUPo56x7fjzHhn7p0WZhGtK1pVSu1KTbJah+4qih7bHltzm1Mt
okVB70UWTI8hMFpUEImLDEWpTUTlA2EWFT2+z+8ILQ733P+8v/NYO3h837GE
mZ0HVYNytWYx6HSN2UhsdgE00oT/0Oxyyqweyj0ZsxnQFPQ2JGWzF/q78Xa2
3ewodHbVmR0MzCYS0tnoev1Z6IIeQnYC/8VGs3HEfYv/SiA/82mzhbTkV1KK
zbhfYfctK1/z+J6C7EWV2dNmsyT8BI2K1xkoh4lW8NCZM8miWunvh78tsL8G
vXrIN4F/ZMq9Iy/bG8jjalLv1HvfoLeaNbBtMxuFvx/43qEtgDsM2yPtshnE
20tg0R0ox0ysuDPwPQ0qwVe5RbhOItYw5Dvh6x34Mux6YL8Dtl2Qdzep9s2Q
z0H2BPxAg/AiVrN4+xsJ61uwu50VjsSBb9S/Dr0PqD0J/jfiT7appj7Y/spI
xj6Mei+etSg2efpi/xiLc5FzDD9n1QPyvfBRCeVnFhhcAn2pFo5THreIfvZn
hGGuYJYHJdC3abwtpqWTr1XPGXcbMNkeq/7XoepnLT+hX0xJfw/45VAxiM3W
//Chb+ZAjIebhC39dzmereAfN6guzgt9MF/29lwsLEhLdcrnLmrtA415X4jR
Su+cpw5nmn3jPLA/zIM5sNfUWc2DPpkn55FzybzuA497oFKzenOyTn2bAjbr
I813FfKyWLkvotYF0FCgXW31HeR/p/ef9Q14jcSh7DkwLmMSh4vwX8kpFv1E
/s5ae71e5sLeUof7UUkJq1KgGed+UY81s17WWnZ8Wtq0D9TjThxynnkQB+JB
DJKOwwHIk2n5jvwWccbGgM2Dgnadu8g94yyx/z1+i87kNX/MgXaR3zFiSiyI
Ucm/fF9Oay5X9hq4rguV8yt834TKbyktjFj7zXbt5+pu5n1WeSM/FYRxETrj
Gc1AzuMQf/oaXt2pULu3gmmkm8JZ434MhMqHPRn1uM8zis24Y44x/XM3uvyG
fM/qbnJ/OuBjKKUbzHuTiFX3x4LqnPTbQOw5h38i3SD+/wNoIttx
              "],
              VertexColors->None], 
             PolygonBox[{{4757, 6397, 1695, 1187}, {5355, 5351, 1303, 1832}, {
              5527, 5526, 948, 1000}, {5289, 4816, 1830, 1781}, {5015, 5664, 
              2115, 720}, {5834, 5833, 697, 2288}, {6676, 7090, 3514, 2822}, {
              5014, 4939, 1495, 1570}, {4248, 5747, 1140, 521}, {5083, 5082, 
              519, 1495}, {5115, 5114, 519, 1496}, {4246, 4938, 620, 519}, {
              4221, 4941, 1496, 2162}, {6675, 6676, 2822, 450}, {4341, 5015, 
              720, 620}, {6658, 6657, 1295, 2743}, {5058, 4616, 1720, 1600}, {
              4846, 6564, 2966, 1393}, {6783, 5832, 2288, 2965}, {6656, 7085, 
              3511, 1295}, {6564, 6563, 2965, 2966}, {4562, 6398, 2774, 
              1695}, {4918, 4919, 720, 697}, {4616, 5054, 1598, 1720}, {4365, 
              5206, 1720, 1598}, {7089, 4823, 2823, 3514}, {5238, 5288, 1781, 
              1741}, {6398, 5482, 1949, 2774}, {6436, 4822, 1304, 2823}, {
              5057, 5200, 1715, 948}, {5290, 5291, 1714, 948}, {4031, 4850, 
              1188, 180}, {6717, 4249, 521, 3136}, {5351, 5350, 450, 1303}, {
              4847, 4848, 1187, 519}, {4615, 5058, 1600, 1719}, {5199, 4365, 
              1598, 1715}, {4195, 5085, 1622, 2115}, {5196, 4601, 948, 
              1598}, {5667, 6716, 3136, 722}, {4418, 6795, 3211, 719}, {6936, 
              6269, 1000, 3330}, {5102, 5014, 1570, 1635}, {6270, 6935, 3330, 
              2643}, {5747, 6363, 2743, 1140}, {4822, 5356, 1393, 1304}, {
              6655, 6654, 719, 1622}, {4732, 6270, 2643, 2744}, {4919, 5103, 
              1635, 720}, {7137, 7136, 722, 3211}, {5206, 4366, 1600, 1720}, {
              5216, 5215, 222, 1600}, {4816, 5354, 1832, 1830}, {5715, 5716, 
              1695, 1188}, {5172, 4221, 2162, 1695}, {7084, 4732, 2744, 
              3511}, {5197, 5238, 1741, 1714}, {4130, 5205, 1719, 222}, {5483,
               4105, 180, 1949}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5827365793354216, 0.5579577221469878, 0.909929961291236], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVE1IVFEY/cxxxnnZ5Lx33zg1tigSSlsV1bptJVRoSkRCuzIKXLQpqkVG
P2QQ1K5FFFFZkVYgCoEWabUogymoZaGp6RSG0qLoHM8FFx/vvnu/e+455/vu
XX3w2J6jS8zsFKIcsSthlq4w6yozm3Rm70P9DyJakmYDSM5WmdWnzFZhrgPR
hPk+zLuVZhGiGut3YrN3y82Gkbcxi3GsnOGc2SvETKXZJ2D3FMwKwChlMJfR
mLjEr63QHP+bEvqvS4nPt7TZTXwvgfQYeI47rX0E5toVZuuZh/FAZPYCObcw
dxsxjvEz7NsWmA2VL+JSd3ve7Ehea3347sf+LoxHwf88cC4ifx349GeE04wz
11QLbw4e9CB2A2faayF2DJ65tLRzrpRZ1FfnNZ4NlHcZns/WmP1GfMV/K/A3
RML/Bz4fstJLf0veq2KgXM4TK+21fMa+L5FqQy+Ym1wi/uQ+WCY/ZkL9b0Xu
lsjXGZomsDa21OwPztzppI0eHIrlw3WMZyJxPhyLHzn8BF4pVF+kUOckoh61
bkOdDyDalknv1Spp5vrjvHLIkd6R5xTObyUPcOgkR+R0QddxcGl06s9SJN7k
fNIpjzlF/L/EGVcSWmPPkmcDzmiGjn78fwfeuVA5zOeX59JX+ssa0SviMX9H
JM3EYe9SH2v6FzjXQtXhEcYPEZ0Yd4PPfae+YN2nfb14Pn0lJ3rA3qAPN5Lq
Z3o7AYzJvDSwNuTFeu1FLWZz2lP0XjV53gtay+QvdZL3KHhNwsfA5xZ9DzQm
VANqoe/sF/o54Nc4T9/nk9ob+PvHPGrjXuYNQl9Htc7lnVi4G8CYR//O1eiO
ci9xuJ/vR7N/Q3oLei/o9V2nupIDe30kpTvbCw/2hbpTr6H7DeIX+P5wuufs
a55/wolDJdY3Z4XPu9td0P3lmdxHbnwnpvxbEaOem5x8f45xZag3YQQ4p8Ht
DLilgd8eK6cF6w/y0v42p3+On2DuKeJCoHO4j2cNATPlMVkP+kOefFPvOd1V
6s/6fmPUeq9534hH7fRgu9OYXhZ9DjEbvK4F7pE/K5DvrBffvyCUn+z3Iade
+Q+oF9bz
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtU01sTGEUPWU606liZt6b19eODWGhrFrsbSRVFUR/bDSxVCFhScKCBaIS
qaWFSPy1hCJpKiQtoVhQ0UgQm0qZVqcobSQkznG6uMn3ve/cc8+9577lew7s
2L8AwCZGgrG7ALRmgDUVQMdioJQDphhJgj4vBB4sBf7EwJtK4GA50JAFnkXA
d+Jbkgzez9UyqoDjxDyJnCNsiZhf1UB3zvljaWCCXF8YZ1n8Mb8Va4AK5txn
vfFFwC5i+/nexbdtxJQCYJoxSL4zZcDHwBpUuy1p7R8C39XDBPkmGUVyVTJn
Q+AcYcVxI7ZmaT3BaAyBFOvXMbeL/CuZ+5YaCsSuTgGHyD9FTB3PWfb4nlxb
GP3UuzPp2cxUuzf12E58L/FzSWt4zVp/iRnl22nOc5rc7wLnakYzrF2fNV9E
jpd54Gnk2enb+ZxnKH5xqPYPvufTfi/xbY7xKW3+W7FnqNmpJ2lpZM5ouTV1
kn9v3tiL/NZL/M3Ys2hO2MvWrLXI08Qy4FKNd2FgibmKsb0T5zXyXw09uzTv
W3nuYcyy1vaEvWnL2Ft51MN9G8nZG3lcT2w6Yy0X+N7E+x3yn+Tbbd6PUs86
6hlgr8eo5zl7H4581g7qTRqlTRjNpinwrmhGDeQbIebUfL/a7a+hvf2/46zV
UfDudyXs9ThjVcqed3NWl+UJz0PkmOWs84G5NPNJYidC966cu+Tri927etgX
m0O5G9UTa/UxMlXesZ/V9lB+yOO15O6Mjb3H9xT1toQ+i2Nz6J3VvFVjPXt7
Edk7/UNBrXdWu6oazaE9EVY5QzwPKsr8D+rf/JbzLmp/VzD3Ff/f4ZRrXif2
d9ZeyNMr7OVhYC2aSSVz2+f1PeK3wzwfCT1L1fgHB5av8g==
              "],
              VertexColors->None], 
             PolygonBox[{{5053, 5195, 1713, 5196, 5052}, {4047, 5053, 5052, 
              5543, 1991}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.6027932776424729, 0.5901832554907557, 0.9103871922371617], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNVTtMVFEQHVmQZV2zv7dv37KVJhoLEgFjB3YWQKKR7xoTjbaKgrESTAwW
KEgQ1MJGOwu/BBSFhAKMWlgAnSQWVgirYmSJWFh4jueSWNzct3PnzpyZc+bu
rjMXms+XmNk9rBBWfdTsEFYZjN9SZvviZiuwT2/HGVZfiXzqsAZLza4GZruS
Zhu4/GWH2WH43Npm9gz2J1h9EbPSGGLgrAX+exBzM2E2HpJtcqfsHcjzCWfD
uLtWjhg4m0DeoYhiNsNnGLbPFbL7sN3HOg08TchzN202hXstnlk0IZzcGwN9
9yD+b8T/iPhT2KdTqoexiZvxiYffxP8EcfJxfTPuJdjnkfdnxqwQV94juDO2
XbVMIE+TJx/WfA1rCXcHs2Y3s7p/HH0ag18/zkYrzdqAbxlxOsO6Q/8B7Ncj
isnYA+77Ieq742p8BOyjadkbkLPRk+97H/nT6t8MfOZjwkNuFnBexF4D+3es
qTKz58ByIqna55Bn0fmEEWcVmKuAqzusnXogJ8OOF/JPO/naRE/WMuKmBDk7
YuoN93xM/BZx/ifQ3RvI358Sh8RFHTHvbEQ4+H0qBy4C2UIuDmNTI+1x5f0K
jIWseKPtZE72VdxrSIm7btRyJS38664W9pm9PBeon2+R4w3WEHD8CIsT6oQa
afWkk1nsc1gV5VpvnX8aPr8y0kMtcl3EakWMSLl6TJ/dSc0JeTiG82Xg7ApL
R/s93f2akh99yINfpj748PWwbKdmsZDSPFK3xMwaV6ClhYS4+YD4BxOqlX2d
df2kvjpdbewN9c27G/DvgH+mQjGJm5g5o9QZNUYsFc5e7Yk/ckcddzh+ufM3
eWd9rQnV2J6Q1pmX8/FPl6Y3hu8Ie1Xjq3f85p2RqLCSV74dnMnXgeaHOcp9
6Z6av4XYI5W6w761hYU3nVPv2Lde9O2oL04Zv9NpgFqoc28d9bvhNEzN743K
Zxx5XwSayUXkf5zTm5V3OqcmQ27nHBFryH2zX9QA+SIOzg9nM4cYlz1pnzXX
u7e0zvWEeGKuN7SvIH8hEN/VKc0w45JbnlPz1A15Zi72gHyxD6UOH3Fx1l8G
wkZbqeNu6w1if7ZwMGYvMPZ46iu1tRRVf964N5n1+U7jzMv5nolpxh9kpUve
Owt7VUrv8CpshUrFYv0xV3seeSJJ/c8wxjtfcdivHw7bK8SZDPSfUAtdFX2d
ZRxn7Fve1cu6+AavO64X4X/A038G392ngeomz13/6aHbfRed7lkX/zeoIWK4
jXcjn5TO/wJpDgw+
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlDtMk2EUhg9VaCk1lPZv+1NcGMSFxFucjG4mBhMvgLYxRqOb8VriJLpg
FOIlgMbNqJOLNwQF0QmIOjhAN92ctBQvEYiuPm9Oh5N+3/+ey/u955y2nzzf
fS5iZnuwtdhYaNaQNdvHZTBudi9jdrPV7GKT2XTUrBDwmzbb2cAvFrSZPQIf
Abd1Zq+In8DmuQ4Rf4f4uxnPN77GfR9iw00ecwVslXq5RrN6iFxPmVWInyP2
PfYB7CP2i9oT5DxGvePYDNhwnVkc/3c1Pm+xSWLTLWa7Od/GZ4XYw9y/kft3
zKyP+4682a6EWS/35HqzF8R0wy/SbLYR7CA+F8A68HlCraPU6AGf5H1VuJdT
fq7Af4jaZfI/oF4WK4B9In6Z+Ktx59aYcq3E8Rm1nmMDcefUhZ5HwG9wHwPv
xL8Y+FsGwNvB2uCwwH0V/argS2k/V9DjJbk2pF1b6Z1P+jed1YNt5MqAn4g4
x3n03triWkjTKu9dzpl9Rf8vvHcQ3yl0uB/198wRv5lv0/VozbcDvC3T5r1W
fIFc3XwrxbyHP/H9kXZuiukFH6HGaMI1fQ23qdB7I82/c77GG2fj3lPlUg+k
vXL2Ur8ncC2k3ybxCZyb3rQ/65zE5TPz0A/WhwaHiC3jcyrjPZK/3lQC25L1
3jcn3HeGmMaox2yHbwy8M+Y9vBR4jHzVA+1GNOu90kwvMg+FpM+iYv6i5b+a
nuI3G/hMa5ZVo4O39Sed61n8F6i3mHfttRPzaP8447um/hVTPqOaTdW71eo9
Fqad1KyrR+qNZl69/JPz2uIgbS8HroU01uyXsr5r2oEz6H86dH1VczTvHMVN
PStSa6nVZ32kznfhTej/DdqJp6HPvGZdM61ZHq/Nn2Z6FS4rOd9d8amCLYae
SzvexXlv6LXF4T9upKdR
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.6228499759495244, 0.6224087888345237, 0.9108444231830874], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VEtIlGEUvaaN45Q2M//883Bs4cICC9JpE5SYYGVJ5DOjVc2mFuFgi4Je
S1MUehEELXpKSQvp4aMXDAhBUBYE0aZFkkVmVNKiJ3RO52txmft/c79zzz33
3q8ym2vrXmBmfbBC2K+k2U/Yi5DZCs8sEDXL44/2mFkHbCxgVo3fuxGzc/Cz
YbOH+L5WgDPEd8F/DZx53GkI6nxdQPeIU4czKzF7iqSDyNELy+PoBu4e8c0y
iCuM6Jv+JPA8fPcjfnIh7hSZPYF/GHmXlZtVLTZrjZsth/8e5wn4SViuSNYY
0N19AcXOwH8GvCnk6gOf3Ti3oPjmgoq7jHsj0CBcIX7Mydh8gXRpCSlmuNjs
OuyW6X/6jKkHzp2QeDY4bNabTptVQs+SUuFOFKv2+YTZLPhPQ5el4F4BW497
LfjdBuuG3w7OzfDfAucN4l6xFuDWeNKINVJX5mOucfAfhQ0gzxnUetpXvp34
3QPrx92b+L8cOo5A14sps1hamE2e+sre8Q77QJ4p5H/sSdNMmdnqMmlOv7ZM
2n5GLV8S4sj+nPrfI/AoKFUtXxOqedrVwVjy8QM6o0/NqCM1/A1uf2A/FkmX
ejdXrI09Zu214NYTlv57UV+vJ7wgzjZ46tH4ErPj8IfgrwhpLm+7OSU2MecS
mg/28x5+jyH+QKHwVsaESc0GU9KNPWcs8acius/vjNOG55wx1sNaviOmMam8
nBHONM+rPPWDmAPAPusrhlrXuZ5S74zTmUbNOQ/VId1j/H3gRx3nTje71KTW
3f3HB/zL48Ll3rTFxXUC9zYlpcv5gGolzijOL/na3yu+4hhD/O1uzllfNiCe
7DP7zVliv7lX3Jc53PvkKe4BehEJa67YwwH3DjQ6DPa0BnqvionLZvBqSqpW
1tTp8l5IaXaZaw2wd4WlKXc663T/hvqO+tpX7vSOqPbvA2p/HtG8kR93n3oe
jOodIS/uOueT+GsRvx/nJ4B9En5HRH2odzvO+aEe7De5cV+3up1tBvctSc0d
eRCDs9MTFy6/Z/D/O1geMX5a+0/9yKHVvTmt7v0hH74j1IH9JXdyYa6XvmrI
ufhupz97POzeq4/QatYTV2o/6PTnWzuWlP58+9l79p0z0hURBvXNuz5s9PQ2
MS93hfPHmeEuHoqJ21XwGYI9Qu6/OZ/sLw==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtoU2EQhaetpmmNmuQmNzeJChEL0graunGhaKFqtfiuD3Rju9GFtOjC
RY3gygct+EIQXIhK8bEp+KgPFAqC4MIiCOLGhWKrRosKgvUFfodxMfD/d+Y/
c2bmzC11927uqTazDmwK1pkyGy+aLZ5hNoajEpjNT+KIm3XFzP7mzdr5drXK
7Oc0s0Jo1sobqzNbRswNfD+4DxHbgq3kPgZeM3g3CWuLzNZgjfVmt2rMUmC/
zJr1knwEf4bYEKuazjv8ZXwPZ7pPeCfAW5Qxu8D5IP4N5C+D0UvuHuwA9xbs
Wq3ZSe7rOK/HLoP/ToUS28j787wf5DoH35+819Iad+4N5Bia6jU8InY2MSvi
XvNauCeIWYqviRrmEdue8Vi9ORY4R3ET/iTnQ+RsSJjti3lt2aJzVY3qnTgo
t3q4h3r3YlnVCsauNLjkPEquAWwQXwm814Q/ocY+/EvI8Zjct/EXQ/+m83Iw
m9UvcaL2Z9g4WBM5sNRvYlZzn0z5LFTTA2LvB44/wrfOjH/TeRj/AL3qz3u8
6lUvtmTcp/c7yH0WjvdqnXN75D1SrGa+G+5jkecWZ3EZBeN4jXOK4J/H2mKu
iXTgM1Mv1dMjvJ1Ljm2834qNkiuedG0JY4H4pjxWPXzO+UPB7FXC+ykt/Abj
Rb1r4hfnMrax3r+V0p5D2HXM5xz4HZHPWjU/hU9X0rUujl/hvzPr2G+Y32ly
nSr47mjm0kZNyrUrjSzkXkk6N2lY3C7mXeviqF37TI7umO/cYbD3h67l6/R0
IvCZapaK0a5uwv++2nd2O7m+hx6rb7NC14S0IA1fAu9K1melmd6JXGPSVj9W
gc/Hgtfzts578yXntalH6v2nwP8FmoFq+ZbzWNU0TPzdyLGkOWm16b9+pCn9
C1YFvov6J/TRj2LRez1S5bvbHPpZO3wm6/8U9U+a0q5rp7RL2vl/DbimgA==

              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.6429066742565758, 0.6546343221782915, 0.9113016541290131], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVEtIVGEY/fIxLwadmevM3BkrECTs3TKdnlRqUUG6qxbWRrMyDYweQ7sW
LnLXooQgSiUyWmibykWQQZupzIqgyZUTQVJEgfagzun8i5/73f9+//nOd77z
37qjp9p6yszsIlY51i/fbH/K7ETI7EvaLI/3t0vMBvGei5q149mNdTmo/W94
jmKVcHigwqwROSPY70M8U2lWAfBWz+xPRu/1UZ3n2T2o8ySt96aozhJjLGL2
PWB2EvHNpNkNrCnUGAkKl5jFrNn7rM6S63GH+bfarKHG7BZ66QT3LqytwLsX
EVfyHMX3j3Gz64gvAPthtbA3Y782oZxWxPXgfbZMeHlPmOvBeUNKOlAP6kI+
5ErOw+BwukLvjLnX4+JFcNmbEtduxMewtoHXUFB6Ep/fqBF7aQOXGV8Y/Ygv
ecI6j7g2pTqrsVcXM5uAtiE8C+hnHlifELfgWwe4TQPjpS9dWY+5j9HjK1+4
xHwAPdagz6sBcagJww84uwL5K7EK4FbE3oeweql0s2CfvVi5kHSbDSuPdW+j
fsnpTE6zbp+YDRFpS08UyqThFXB+56ku/UB9uU8P/nQ+/Ay+C75yfgC72RcW
vdPk8hPwRS+wqqLCbnT705XiQQ5H8P2ZJ69yn3vEnANeR0w9TsEXmZh6b3ce
pT/fQON4TH7aFFLvxKGXi86Tc7Vmuzxhc1ZL3bwK4Dyflo4LiFt86UuvtfrS
JOd6YU3qwZ7JbR3msxbrGma0HHjb4+qPz2xK8Tiw7kQ0my0h6ct4Fc4ZOA/h
fTfq7PQ1B/p+3OV3oa/OpPy4iD52uJyD2PualkcPI74LXQaDmjO9Q+3GqnX3
6GFikQO9sdGTX9k7/yv73L/l/z0Myhvlcd039s77RE6cVwa5fkoaL8Nzska+
40wWspoL/cX7TJzn4PvCV23WYy45HALnvoz4H0BuG9b9gDzO+VDbftZPyMvU
KeFJK86W/w7yfQ3scy6n0XmLc2/2NG9i/c5oNvw2Bc2eYk2GhcNzzGcdYvH9
TEL3kDo+gn75pHQhd86PWvJ+Dyd1b+mdki//kB+x+8uFNeDwJzx5l/oQt+ju
bJXzFPP+Aff/04A=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtk0tIlGEUho8z6YyDNLf/n/mdahFUlBW1DLpCpXaFdJct1E1mZRewq9uC
FrVrUUKLUiMUWmRQmIsWBm3GEtFNl1VGUBRRoF2g5/TO4sD5vnO+95z3vOdb
3nmqpSdmZruwRdj6wCybN+uvNrtdY/YrMvuNnUiazVaRhz+fNVuTMluFzXDO
lcwW15ldA+B0xmxDwaw1qbtXxMvYSWLD5PeBvZYat8AeoF4rWI3cTVEvqDXb
yNtLodmNpDB2E0uC+SWhnJoc77kbxX8MRhfYHdjzuNl28P/Wm/3BypCaJmcS
/M9Fs7dgv8eu8rYL/P6E6p8hdyQt/zp3H8nfSn9z4N3B5uG2UBLXMjmDvB0I
Vc97mKX2ZCRuj7AX1DpIvJf4OPUu5jSj/7PBpvHPcdeD/4P+mzg/yQvbZ1qE
/3niD/E3w3+YWJyeLsTUTzkUJ+fiM5nn/QJ2nNw4OXeJj1bwJog3EotV3q8G
8yWxempsIX+oSr0e8Xkn1LNrNYWNpKTZG7i3hZpNN2/G0uLsXIe460Sb5rxm
vY34FfxP3LXHxLEvFAfv/X5CWpTp512tNFkXSBOfv+9EA/6hQNr6fHOcx7La
Rd/J8UA1vVZ1QrO7F4qrc26mtz2R9HLOR4kdrvTvmmd5+yytXr5j7Zw3gXGW
XgeZxxJm3xtIK98Bn92DSv+OV2J2HRnNzndsJ7V+ZjVbr+nazYTCdg19ly+H
mpXv9Df4fy3qL3lPS8FbVhAX72FfQX/C/4Jrup/zSjRYUafza+q15LQ7rtGB
gt54zP9oVNAO+e64xk/Bagg0O9e3CS43M9oV18j/+kRRu+o5XmtvQVp7zTnq
fYg0G9fsWCSNXJsdKf297ki+/4l/pi+dYA==
              "],
              VertexColors->None], 
             PolygonBox[{{5202, 5046, 5045, 4614, 1717}, {4991, 5166, 1690, 
              4539, 4990}, {5046, 5538, 1989, 5537, 5045}, {5163, 4991, 4990, 
              4538, 1689}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.6629633725636271, 0.6868598555220593, 0.9117588850749389], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVUtIVVEU3U+f+nz2er/rffdpUJYNKvoNGmQ2in4ikVo2KIKiQRmZQUU/
iKAGRVGTCpz1ESpKNErDhP4NIvL5oSbNUqSgD/RBM6q1WCcaHM459+yz9tpr
731uxdbd9c15ZtaKkY8xI202HaO9wKwFo6HQrBsGd2NmIyVmIYxFOF+fMHuI
876wWQTzaswfJ5lVRc3eAqdgilld0mwY9jFgXMTYABwPcz2wToZwF/Y5jL1w
vNIz+wH79kJhVGMcg/1CfHtRKg5dRdjjvAP2z3zs4+I4Okl8+f0OuF2G/RLY
jeG8ErgXwOdKxuwasC4Vmw1Ede8SOG8JzLYH8tkG/Dz46YT9S9xvwzgFntXA
uA6bq9gvBuafyWa3YN8NvFmeYqvIQjeMfMTbgbNaYD/A3UPgssB32LBfESgG
6pIslg6DBVozRmpKDGpaWfRfY2pGv7Svgs91CXFjTojF7zwPufXXQvFdhbun
woqLfNrBzeLizznk1o0p+PXFm4MxED+M79sC5YO5oa7My2zovCPQuu2fH9iv
hf21QPup0CMPmDVh5XBBUjGSJ+Oh3xtRxcf6+ewr37QhL/L7GZPmEzHFRuy6
lPDfQ4MTadUUz0Zc7A04H8nKL2vwbJk0oh/6Y34/wdcXX7qTI+/QnhqEnQ5H
gT3HU/0yR/RD/HrY9gfiPA7ssTLV4G/M+Sn1ATU/HZHuu4DX7GvP+jtepF7Y
BE3WlGp/uFS1wjp5BexcIK5xcEhgtBY4DZwmHNSHHDbi3tsy9VNnVLXLXDxD
zSfT8rUfvIYC5b8HHHqLVf/E4DfikMvzjPjUgO/MMtmkML/G3duw68X5vaR6
lr1LHK55xpphjdHnXE9+Q75qi3rStsfZM57NJVqzHphj5vAm5j1ZnQ0CYwjD
LxKnxhL18nnkfWVaOcmFdc7cnoloT81b4He3r2+PwPlxRr67cK/eUx0/yUjf
VvfW9UcVb4PLN+twPjBaEorrDTS4H1f9jGM9FugtOQfdnsb1PlGngynpMYz1
O4wHWDchpp1ZxcB++ep6pi8QLjG/QdcDpXrHRrN6Q1m3ta6HyYFYrCPuGSvj
5t0J1yfsKcafc98Z/4eo9qx1xsUavlGON61c7/p3+F0WyC9tqSftl0Kn0aT4
EpO1SQ58kyLuXdoHm/KUam7C1SX7aJ6n/mHv1Hl6L465vp5wvc/eOuJsBgLV
PWue2MRhTb6C/bSE/in9rnaZI/6jlgd696+n9bbyXeXbsMSTtvwPMN/MNfGI
S26/YuLAdVOgf0CVq99O9+7x/Jd7c6hHpasx9kStLw6s0T73//oLCBIESQ==

              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtszFEUxk/bmb5mmnn9O9NOFxQLFl4LG2UlFGmCQS2IiEi8qxKEYCFh
IwQLldg1CG20KaEVJPVorSRVZSEpG7SIV+LVJoLf58zi5H/v/57Hd7/znVu7
cVeuqdDMGrEQtqnKLJIwO8LPeeVmQ+z3Jc06WXdgfRmzpWmzVpzvlpldDsxG
8L9YbPYNK8J3AzFz8f0UNetJmZ2p5FtiVsf5Is5+4N9VZDab/eq4WW01cRGz
e2GzHPlWBF5f/g+odxfbg//tMs81mZydYc95hfW8wOsr3zPy58CwJOSYf7G/
n/HYwXLHFkr63YRxFfULuE8D/scLyAe2ZdQ4Su5WYjpiZuvy+xb4eUL8AHaN
dTvxE8E+AVtKfCmYZhHfT75m4eMOf4m/iq3nfuf5Nx2sM7AKsK7hvJn6yazZ
dXL1Uv8RWPszfqaak6q9hnKLo71J74l6kasw+0qtL2mvlQDvFs42Y8Pm93vO
ej8x11h3CS/1/mS9N/I5DH89YD5X7Jimge1V3Ovrn3rblnJuxfEpYj+nvdYb
8IRYh7HjIedwP1ydxrpLvMcjYG+vMbsJ1gL8m/DdgRWS/0Sp2dTAORI3LeR/
i/8g/r8rnFPlfsMdesu9RjXY+mPee2ngWMo5ERcB8WP4fkw6/0+JWcj+e15v
8q/hbD41L+H7WHphPcC/KeRLYwvwH+eOo1GPuUXsw4zjkwYbwDNe5bk1A6r1
Agwri71mG9jfcYe3Eb+ztLSNfWPE+yluL2RcW+JYsWf5NxT2HNvxHYu51hRT
B765gWtzMfW2Untbfr40I7vAEyPmA/uT8Hkg6T1Xr6WpnwmfQc1SJ/aas51p
516cdsd8RjQb//GzXp70Wur3LHxnpl2bl0r8bagPPJfqx8ndnfDeiaPRhL8J
egvE8aC4qvJZ0UyqdzfyeNTDO+wPVnpuzdTLrL8xelueRL03c1L+NqhH0oY0
JS1JI6otDUu7wiAuhgLvpThRL0arXavqST2+7+OuFXEuLTannTtp8lDKNdmS
n4fdxK4F30DUZ1iztDdw7WimxrKuSWlLmhnO+h3+Y8f6Yv5GaX40E/8AlpPP
mw==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.6830200708706786, 0.7190853888658273, 0.9122161160208645], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVUloFFEQrWwzk5nETE9PZnoy4BZwkttc3KIoKC4gooiKpyDRiGsiCGqi
HhIPHhJcwKOaaMQVFRETwSWuRxFxObiBJi4hxgVEvbi8x/vi4dO/q/+vevXq
VfW4huYlTYVm1otVhHW11KwPazmM7zNmH7BCJWZlIbMu2Ffh0CW8T4ua3cC+
JmK2oNiss8CsIcD3QN+6YmZto3AG9mdVZpc9sw7YbqXNfuBMD2ItqjQ75cnn
p5TZPpzbX2Z2D/s87B/hOxc3e4n39zhfD3/DWLvgfz/ubsc6HjZbmjA7EwjH
Otypho+niPsO2B9mzQYQdxD+DvpmhxBrDOxjsQphr06atcIeQ34TcLcHz3nw
8w3n58BnDrE2In4TVge+z8dzED7XYl+MuCsD8fCrXP46EPc34ndiPxTWnT9h
8dCD3F/FxeVZ3EsgxgC+HwSfR8BtA/zkkd/1UeK8Hc9u+PkC+0lgnA2s+ZDw
tjjMzJ8Yt6FekxF3EtaeYvF/LCbeNqeUA7F0AUN3WrVMlIrvN4XiaNjFDZco
JnPZhHsDgeo7D/FjwFyHuIuR+2nYd0TF2c6ouC2BfQXs/dhPRT3rsKY4XMRX
gFgl8FmMVRtRjtTJP73wnbF5doo7Tw7a3Zk7YcUn5+SXPFNLzeDVQ6zyMmkg
F9FdcsHYzOVcSPxdxN3fODs2Li0XAMvSuOp/AJrq83XmQUj+6IP+Jjk85KfN
4TwaEz7yzHeeZSzq/XmVNP8IvFVkpIFt8L8oIY2x1o0R8c8+aovKzrpuKVIP
rs+oT9gj9U771PaBkHRDzbRGVYd+Ux1aXS186NTLSo+DgXgn5+yVZQn1C7W8
2JOeH8D2OKn8yGuH4/ZwRj3Md+ptY0yxTgTqPfq5Bk3dTmt29MJf3FcOnWHp
nfsZUfFNbVBr1BzzoV/e63E9nnc6JK+RcvFKjHurhJP9zSd5eA1uXqTUY5wT
E+KqxRhga3HzgfOmt0L+B1w/Us8tOPsDOJ9gfxjYj6TVh6z/d08aOIX93Qrl
W+1LHxfdDLwbVi7sfWqDGmHPUaeMwToyf+bul0qX1A/PrXP4yR0xEQ9zanT2
RFb1Y+0Yj7plj3PeFCWkf862txnHoy/eGYs9ujpQn1BP7E/qoQD20YH8vETu
45PSWB2e05I6l/U1K3l+VlJ3eJ69QG2zFtQrcZI38tca157PuSntObvZn+T8
Dni94unfQo2zz4jrhK+5xtymJ8U1eb7uNPAvDvfkhD3UnFIf9aeVA/HzH3U6
+18zXa4fW+Bze1LYyXONi8vepcbph/O4NimNUjtDvu4eg/+jnJXA3A7/uzPS
/VXYbjqdj4NtWVz93uepBuT/MzB+SWm+jkT0ZC5rwH1joD64gLvnM5ofZ7L6
V7GOM6GvhZXqa2qf2Mn5H/cvoU747xrv/l/so3rXLxsQc02lNJZ3dvI20Vdv
M2feoV5zrj4jro7UL+3M5Sv8LPGkWfZnzmmyG77vV2gG81+6tVI6GXEzjH6I
k/yTW86aGof5J+5tyGhmzYH/6irVlvzSxvlGvZ5z8/kvR2kzMA==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtszVEQxuf2Vm9776X39q//+6hotaK6ERaCVkiIlljQesTCqmg8W4kE
9VhgYaGpSqyEPlTqFa9IkKD13EiwwIqNV0TimYhYld/ks5hkzpmZb76ZOXMm
tXa0tBeY2VqkEHk53uwFMhwxi43hLjSLIlOLzeqQs1mzisDsZNRsT9zsSbnZ
jLTZV2yfY2bnsC8vM9sbl31lyqwevMEisyn4dOfNFoG3AX0jcr7C7ALyPmFW
RL4bpWaPsX/BNmjC/s65rEQ5rubMasH8hO00MQ1gN5NvX1w5zpN/LncjJg7n
iB/MmCWxrYPzmAnE5pTr7jizKmI78X+OvXis2QrOBeSrJdd+4luIX8J5E+cA
DqXU/hC82+iradybUJycy9+YerUajANx9exjTpiO5TV+Jv4KdwMJ1fAb/Cby
15P/GvwOYUulxH8rPml608tdF/poEgzq+5DVLLqYURv9/xmKWwQ+1WDVIIki
8UuQ7xR8e7Efp+Ze9PnEtEZ1V4fvTTjsiKpHXnsl+QoS6kFzWj3zXr0D7yLz
2RvIt4t665nnllC5ZyfF9RRyJCHO1eiT/BwRZipQD52b51xHLd3wGUZviKvX
k8GsSarndfi2ZmW7Dv+J6JXIGXLvKhCXPmrqKxGnVfRuZqC36zk7A3F2rt6T
65wbqflSkXreT+yJjN6GY9xDH8mI303OVeA9I/5RTPmb/C1WaBb+3h9h6ykX
H59ZB/btyJFi7dAiuFYH4lpLDbvx/ZWW7jGNoTAd61VEvehHHsbUk+lp7Zjb
fSec62mkv0Sc14DfhswDbySqXDWB3pLn3ELvN+fk6/NfSO0Lxss2nfqP5jVj
n20PNf2Ay7dQuu/cpZx2xHejn/lF0Ufz6oXzm40+J6/Zz0Jq0DtTqsV3fBX+
F+H3Hn0pH8xr7Lcz6q2/kWmhMDz2cKF22f8Q/zt8p31WQ4G4+sx8tyKh/hLf
MX8ri0P9Jf5mfHfO/o/3eQxktANeu/fsck41eS2+g7vB7yBmbFJ/wDLwh7Li
2oSU0Zu35dptr2Ey/n9KhXcM/05sB3PaVZ9nOq+avBbHfFqqN+dvzWe6HuzC
MvXO7zZk9ca9nz7DW+S7878//se0g7Ut1K77m9pJvl3l4uZ/8AN872f0V3rM
P3ZpwTw=
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7030767691777299, 0.7513109222095951, 0.9126733469667901], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VUtIVVEU3Zn5fFex957vvnt9ZmUfokE6iiyIKOlHmCAERl+LdGAkfSgf
QUUFmfSbCGlpVqYD6QMl2D+aRNmLoGENEmkQBNJ3lrVW6zQ4nHP33Wd/1l57
n/IdLXV7cszsJtZkrMo8s0GsW/h4PsXsiWfWlmtWg304ajYG+StcGMfeN8ns
a8qs0TfriphV494I/rVhVReb1cbNPuK8Fjp1OI/hPDtpNgvrgtOdyJf+vJjZ
nRJ9N+fKB+2vCM2+4+5c+F8JO+fSZucLdXd3rnT2J8xeh/puxP0mrF6czyGu
nZAPIbbbsP0ZMW2H32+wd8BXjjNx9yb+fcW519cd6jO/P/nyk4Hv65DdAT4V
RcCoSLlcCcwOw2YH5K3YL+Pbg/1EqdnqlGKhv3GH2cRU5AO9ayZ/lD1G/O3Q
q8PqhI/1nrAmzt2Iq6dE50HczThfNxD/Jfi6FJUd+qCtMuB6Ef/a8b0NeA4n
hc8y5NQD/YOwvwZxzUkr/oeQvcf5DfDMQH82zrNwfoHVFZUv7vRTC/2NvvJk
jtRjHUYhH4BPD/IGxDIKnURcNWVM5A5zPI48MvjXDZ37Ufnch/PpqOKifdpa
UCTOfIINP3T6yGNfQrwohZ8ByAfAzRHse52ceK9ymHdAZxd8lecJ072ThXk3
8i0olq9D+H/QV13JLdb8WI54ccZx40REsZ3C3SxkLfDVBF/L4ac6pTh/A7NJ
kH8pVOzkB31VujNzegedvrhwZQ9lC1XTHti8gvU8onqwh3iXe7vjQzP8LEnr
m7xuzhO33zqe8ExsiAVxIB/JFdZuOzBvwKopUB7cmQvxoG3Whb46nd91iKXe
V136Ya82Ie6ztw4k1F93YT8L+1noD0K+AfJj0KlAnPG0eNxToN4h76vyNReI
2y/o/wwVewr6W2L6zxkzFBVXmAPnDrnD+bHc4UyO0y5tsv6xmDjQD52OQNyj
jyOe+jTP03/ypyqt+Bjbj5j4QW6w9typvwM25xeLA7TDmMnhLtjuDdQHRz3h
QX32EOca68baL4T8PeSL84UT58bDpPqQPcj/izzZvxbILm2Wwe+0UPGytuQL
82WPsU9YixxgvCl09z3lyNi2wnY2lE/icaJEOd9AHqWhZjk566eU0z1g9dJX
/Tl/6YO4nca9dqz9wHcFYq5O6g1gPahLnrD+j5LKjfPylpuZrMeIm9X98Lsq
KW4wRvpknGd9xVTp+pF1oE3OtnipeoFz70+h+jfiSZe1a3azhrjQ9j/+Eee0
6kpbJ4FlZ1yzhG/Ih5TeEcZw1NVrA/xsjolb7HXmSjucB1cDzYT72B8E4iLr
tdTVlD3GOFgX8mKxexceQ/dJoJ6ucG8C+yjr+py1I4a0RxyfQvdZoLo2oj71
ofww90b3ZhGD9W7+17h/lJcX6936/y63uTdyOFBdaZ99wFnc4N6uCRcnY1vg
ZlELMGhNCv9MUu8f6zgDsUwPhT37YFcorg6h1yMxYfgXAZIr2g==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlElo1VcUxk9ipvcU+15e/m/Iqy0q6kpcSY0girFFkKAEpRYHjGJcRBQV
rM9FKyqowWlhF84x2ggKtpuAaZxwo12kCC4EB1CjC0EQUevK4ffxvcWBe+49
5zvDd86duHZz56baiOhC6pB1xYi1yC4uBxsjipmIibmIK2MiRnh/2BpxuhQx
Kn1cxGT0H/MRRzhPQnpL9pHttrERP/G2E31DU0Q30t4ScbGKd7khIiHWBKQh
DS5333GuyUbM4K2Ru4/fRNwnxh2w/wDz3yTiM3f9nLdg34ztZeWL7X5yPlWI
GOCui7c0GAexP5QYTzHTOWMKSza3sB9G/gN/KBXxiFiD+G8d4zudVbNq1d1x
9Hngbac3B7B/TH3PEsf+TH0nwXpAvYvRT6Zcy/wW16ocJnF+knFuR5Ed6BXk
LdjXayLeYX8kce+nUFOuTI1l9+4LObRh+16c4NuPpIVftK0420l9k1uMrfpG
qLWviidOXvE+Ff8zcLMGm7O8fY/+iLfb2CwB7wXxZoynLvC2834s53jneV8v
LDD/Dt+pF2vgvGOse7KH8+6SubxYT274joI3fbw507mdu6e1viuT+8us7dWf
VeRyl/d2zrOo+XCrc1IumrHlxPslcf/ERxex3pDPvnAO7fCxAOmrc4xOfN+i
nyC30VrP4kfwexo8kzpXMp5d3Z2Bv9MF86d6rtHLYeQC3GysM7d7C+ZeNlny
25wxd53gVdTPomdXNf3MubY54gf0mUhCLvm8a9sA3oqiORR3svkH7KsFz/Zg
ylwszDt3cbIUrAF8Ktj+jlRy5lxca6avkusQ0lPnnJdhfwn739LeEfXmf/R7
afdoC+/LiHG73j6tOc+YZkv8Leb918TcK+YHxUY60sbQbLzI2l7zoFn6RE9e
j/NMyfa6akrZ5zn6UMG13UC/wXlRYi5lcxz9z6x350TKuatm1aoanpD/tIx3
7QtS0+weq7eKeSFrDPnqz8iWvUPaHf1Zmn1hyFc70EHsudV93tPoWZpA/b0N
ninVVs65F6pxNrW1tZrr3ibv6oGS/zrtmP62gZx3Tf/NOXLpr+ajedFsryaH
tibP+BV8/yp5F/UH9GF7tmAu5fNt0Zwovv7IEfRtzZ5FYegvuVnw36Wd6Caf
nrxz08w36i9OPGva2ZUZc6Dei/PlnOvBm1Odz/Xo3dV+aia/Amok2+0=
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7231334674847814, 0.7835364555533629, 0.9131305779127159], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVUloVUkUvfmJ+ckz+t/LH977T0UCDfaql61xIXFaKWqE4BxnSGxsFTGi
gnacZxcJGIcgagZHBImitgouRNOYdmoTu91ot4LQggujrqKewylwUbxXVffe
Ovfcc6sqlqya+WvCzM5jFGJ4gdm8yKyy2OxaaHYDo6vUbGmJWUWZ2V8FZpFv
tjdvdnGQ2SvY52Bf4JktQoAmrE1DwM8pxEpr3oa9MvjuLDIbCd8TWbPLWN+a
xJnF8vvBU7wG+FZmzGL4/oL/B4j/Fhh2wWYlfP3YbDBiFSb13YGYI3B+NiJ4
swTWxwLrUMTqTgj7KWC+C9uawWaN2J+I2P8NM3sEmzr4j4b9a8R/i72rsK9J
6Nvl/u/DfgHOPo04h4Hlma8cD2P/OEazKY9d+J+BOL/BpjHU/Bx4GMA4CZsO
h68eZ/aAgyTivMN8BnwL3PpJT2vEMw42R+G7DvP5yG8BxhjsH8D6fowuZ9eS
FD/kl7ysKFL+Re7/aax6sBa/g9ubGe0tQg2rssL8L2LP98XVQdjtBPbpWN+M
PLaEmtP2SCB7cs2as0btWDsSqo7N+LaE4ob8klNy+AXnd2c1v4H/FM66jTC+
szPYtJbKj9x2IObxUPwOB95exzl1SB/aT/dUP3JQDQ4HSlTzf3JmL3Ka96J2
PwXieA/yPQQMl2A/i5otBxeF0vWSEmmb3Lc4Xvtw5rFQtZ0QqTfYF5We9HuL
vJWJC+ZyHTivZVTj8fh+9aVn6rcnUI2WlSgHnkWspcXiZxtstmLsLlUu3GMt
Ypy7DphPIWZTWv3AOA34X5+W/52kcDGXh57ik5+F8K2NhPdv8FEVKG4/1prK
ZVvmeona2YT1jdF3XrlGzU+C78ScsL9H7RbnpRHqqdXVa3csTMSzPRR35I3Y
G/PCT85+9oSTumTvsi86PZ3FtT9wfnekO+MTzuqJ1V+rgfdeJFysFfu50fVO
vYvDvqeuqX3qsS2QJjlf4WzY63Uur4eI/ThWvHrXM7TJIddsTrY1uCfW4Ozq
Ibi7UNNReXFblZM/fRPl0hPz2pcULmImlrOezm725U/f66H6jfVNIcb2cnFB
7u87/m/BZkMknVAT5Jt99wT2j2Llz7N5Fm3WIkYP7M940k+7p5z47XD/f8bq
T/YY8+EeMTBP3jvUPLWww9Ui4bik76dIOqAG2ly9yCPvtr60eGMfDHK9wHrz
Tqf+p0ADt0NhZf99CNSDH4DnZaAemor10ym9CVfwXZ5VH7bi+yAljVek9fbw
neD9PSGtu6Pf15vBu2gOcM6NpDPy99FxOBDr7aEG2x1HzJ3aI2fUA+979gf7
ke/P8Egxed/zDqLO5wDP7KzqTBzMgznW+nonGIeYGIO1roNe6vN6e3hfkxfy
MyKjXmJdyQv3qRlq4X+nB3Lc6bDxLslHOusNOBsWiWu+G9WuB37Mq7fZ1xbo
DqL987TuQfYK9djpNMk740KxcpgE28kZzb8B710VkA==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlElsVXUUxs9roZc+at57fe/eNwCLEhNZoSunBYGCK4gMoQkyi5JgjAyC
SkKiFmUQhE0h1DAFaJkCMSEFoQKJC4NVqqBGUTdiNDHBFRV2EH5fPhYnufd/
xu87Q8eKNfNWN0XESmQUMi2LmF2KKLZGvDbG39PLETdG++1eLeI+8jq6n3MR
ufaIB42Ic+gvN0f8iu0tZE5LxBHi3UT3I/J1W8QKfC5WIpYVI17kuymJWIf/
d8Trz0fMfSJiMvm+wCaXOMYH1YhnUuog9lbyJ8R+v+rv3eTM8B0h3puAyBFj
L/r/H/9vw2cDvneJ+SS6o8S8jv16cp7k/wQygO4i8QaIdwFJwb8G/RF0z41x
rV9Sz6jENaf4/4PPaPSBJOjPFyJ60f+HrExdg3LrTViexn5uizFNQvcKMZ7N
m7PF+HeNM/aj8FngfwS+/oKvq6RYgu185Arxnsenl1r3IS+Dbx/1dvO9CZmD
fluruZqCvJOYsx6wdGbu5Q/4ryX/LPSb0S9H/0adflL/HnJ1jY2YSq3byd9K
vUPkGE/uCTVjFacbVT/+OxP7LES3I3XvhWkE/07eXsD/c95mYL8HTIFvF3KD
2KvqzqWZqGB7u+TeKUc3usGCsatfg/D3Fv5jsW3m/w54Thc8W1sY2AXknol0
J65pP1z0l4ytucXYJ5Kzo80crC46pmK18TaM7R18/m52z9T74dS91Ay8hG1f
2VjOEO9V6lteN9+avw3tninN0qm8uTiIfJWYk17iT6jYVv2YnnnHVIt2SL37
CNne6pgHquZY3B7grZPvh+D9N/EODtWMQbX3ka8ItkLDXIijb9CvpaZV/M9G
f5V4V6reVXFUR9+omdt3mzxby9SvvGfsKbBNqntW1LO9+B4vuRbNW556TvB/
kO8e6jkM1kOpfTXPm6veUeEVpqU1z7x0ynGO7xkVc6kZuYb+25qxqOZPU++s
6htAPiPXx1X3Uvz8AXe/ZZ7NB/A3iO5S1bujHdYt6Hm8f5pX9X5q5tujGXgb
bi5VfHuOk3Mo9Q3Q7otj3SLZSKeb9EvRO6N50s4No1/K27Gcb4h2+/fMtWjH
L1c8E5oF3YzrBWMWVs2EuBOn4lIcHkO/rmx82knN4qaae6eZ7MP+p4a5Vg07
6M0ndc/jWd6G0X3fcDzNjGY1Su6tbHT7Osq21Q1cQu1Z5tsmDH+WvIPaPd2c
98reWe2qbkA/37tS90I7uQjbpnbPtjCNq9lGOt1EYfmw6lskTI8AkU3cYQ==

              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7431901657918327, 0.8157619888971308, 0.9135878088586415], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNll1olmUYx2+n+3jfvW973+199zzbOpg6mijO6qROEue2oghR+4ACZwTh
VPyESPJbSzcPKptpH0bkzAMJojICLWgllpGRHaSpZFo5xIO0URSE9v/tf0cd
3DzXcz/39fW//td1PxMfXzF/eUUI4bzWeK3uUgjvNYTwleRdBb1LPlkZwic5
P58dF0K19rYkIdwjxVbJV4shvC3lqfr+XFUICyTPqwmhRjpHdP6GVkZyv54T
ayx/JPl6OYT5Be8flt/lBe/zPRvP7MtaB3lA9mfK9hrFNqL95+Vrod4vN4bQ
qTi268zpTAgXakPYJrlC3+fo+zadH9T3UdlvrbLei5XW3aLzWzPO5Q7FcKfW
m/q+ry6E37UGpfukct1TtC2eVQ2WX5K8LrH+gHTGRX+7i9ZB/jhnH8SwSrbT
+hCO69x0rYNVxm2tzm5OfI6YB6KdA5WO5VXJV/Ws1PvsCSH8lQ/hT61JwuXp
ZulrrZefK7JxQr7Pyc7mlhA2aT3DOX2fLYzekO6lrJ/ktVN5HFKO70hehw2t
uTXGD8zBEL+Tq+0bXXAHt71NIeSbbOdn5fVLyefQJxZi26H6PpqG8L72hxTX
XXrvle6pjOuE/VWKYajO+GRrzR94NKT9JDVO8Gt12Vj9y0U4BjefivwkJ/SI
pzdv/sA79qbFfXDvqPY7tjqqzPNbZXujsOurtn/sYZ/3rlrzDd1pcf835Tpa
sq06YVZotj/4sjdiskf2XtZ6JeN8z2aM8w+qyXmt9pvM7aGscQDvfVGmb8CR
3pkjvvza4n5qq/GT/Q/xL1wOVbhvjqfuke3y36+1WnGvlO4XqX0cVmybtb+0
1vbxj49jin1X0Tw9rmdys2vzh84vbnKfnIs5ENsR5f+u9s4ol1uEfXuD3x+R
nymSNyqeDc3mJTz4NGfswO1r7Z3Q+ixnThAj+GektzuxL77xhCcHlOMDcT58
K72TWscitz6oMA7gnom1hstVBeMM7+EqnH04dazEObPselPbWY3uDbhzXbk/
pHO36/xcPUO9c70i27dlnW+z9pu0Juh9heytT9zzmxLzGA6DQW9qHBbq+WBq
fc7BBfKid+lh5lVf7GVk8kImN2pNTuS+XzgMldyLcHBq5OGXsv15at6slN/v
Cp4fwzn3SKfyujjedsBqoOy+oqdGlXddwTMmJa8Gy/Tw0bz7/YzwmRE5tjXx
nGJGwcfpjebifWVjQX7ztUaq7ZM8+mIu7LOH/L30zmr9Ld1ryulwi+M8mnOt
mC39im2W/PbnPDPujXPjRtG1oS7YW1Hp3LhD4ApY0a/cE/TsQfHhSr3PXZRu
rXRfk979ZeMFVkHPhYl1NiS+27gLxu7AOvOzNc5geDWi2EYS98OyjHnEeWIl
J+5H6g2P4ACztCfv/u5Rrne3+J151h9nGj13OXHfTc66N9CF1+iR19K868/8
Z5YQE/HArwWRY/TzpDgfFuetw3kwaov7a7L/zRLqAY7scxd1xPuIGo5hJvmC
cCulzv/1smc68U3RbGhv8szmXm1MHVNf9Aln5hV8N8NteMTMYr9TZ68Vnet+
4dxTsm/6k36j1/D5Y9F+16a+25hDywr+V2DWwVfmGTWdqL3Wku926kAM1OIx
6e0oulbjGlxvav2E6vCCuLFI8S6SvFPykrzvycF6z4Qu3mMvkNfimNtp2TjV
YF4Pxxk3dl8Ij2+azWH6D124faTF/hZFG32x37u0183S+4BibCv7/wY99Imf
OnCfUUNm26Vm1504KuP/GHaHY79jY8b/7HTFGOD+T0Vj9Zbk/am5wDyl94h5
7P8ib5uh6Np0RM6BKbWDM8SPzBxklmATP93R15KYJzmSx3DEh/7GF9gejbHh
8x+TRWy9
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlV1o1XUYx58593K2nZ2dnXN2Xha0ZbAoHNhNXYlrW1GEpJJQ1BZdpCZm
RS+iTp2abgWV27IyInLaRSQS1Y3WRSXWatYMUnNhWeYsITIjCFL7fPueiwf+
v9/vef0+3+f5tz+4evEjsyLiCWQ2cikV8TayuipipoaL5oilhYh3OR+sjOjK
cpeO6KxGDzmWiTiBLOb7AxwN5iM2IdtQ25yIuNJkG+nORe7GVwL7jejeWBex
i1jzcxHLibUGm4daI0aIuTIZsQIZKkbk8THJ2+P1ES/y9mUp4uOGiGW8X8T3
b9zdQH6d+DiDfi8+enk7jX4P3xfQ6aK4Hu7mEWspOt31jhktEXvJaQ25XFsb
0cHb1eg/T6516Kwj1jl0ruFtgJhXsL+MDFVEtHPXhm6C+keI/wI2R9D/O+Xz
p+g3cj5FDh2NEbWcB8BmJ/UvBMsK9O8l9j0F4/FejWubRD5pcI03438E/7vQ
fxP9S9guQX8e+Z7nfS3f65DvwXobOT1JbiXOs3l/DZvbOd+Rc6w+zk/hawj/
Q9g+g36SehuRs+iPAscebBeC56JaY7KAeN9R/7+1tpHuIPVsTdpGWFbgc7jK
mA7ge0PJWN2FzRS9e4CaV5HfbdR4jPNV1DRMPrOQamx/T5kvwmSMeGvz5s5H
+PiLfK4ruhfXE6MX256s+SAOnifXrUhf0pzdie142rWqBxPYf4HsJtcPqfdr
cltAPW/Ah0P4H0P/q5J7JRthezltbgrjpzPGTFgdxd/PvBXQeavK+D4HttuL
5qb615n2TGgW3keOcD5HjOMJ92gT2O0j35k6Y3gL5+5Wc7Mb2YzulnL9wqtY
8AxpdtTTs+Q6U7K//WDQx3t/mQ/i0K2t5qi4qRmY4rwI++3k/w/nZ9OeAXFf
PdGs7QHPP6o9c5rNLvzNKfv7jO9H0RmvM4YH8Hew1bOnmRI3d2Scizj6LbFe
Jv9XE+bcJL19PWds1LMfsF1fMjc0Ey34P522bRv64+RyBplT45y68L0Mm4eT
ruHzgjkvriunH7G9WHI/5GMjsTfkjZ1qLJD7n1nvhgn85bH9Ke3eCVPN+liT
e6uZfyVvTogLquHXvHeKdskJziv4Xo70l/mVwvYANa6sdw7i8uGSuStOjzab
w+KuOPcL8U5xdyjpGo9nvHO1a7VDNRvCUNhpRnY0eydqFwqDjox3gGZfO/lo
yZwWlw8jS5q807TLtKM0Sy+hvyVhTol77VnvKnFQu+axrLHRzhmiV8M5Y/NO
pXdvW8bf6t9g0TtG+Klm7UZxTNzSjuwn3jDx5ld6J0zh/5uScxNmmtUU57oG
z2wT36uaPJu6m8F+Im2sxXFh9f+MJI3ZJG+jGXNlmvf1ee8Y9Vs1Xkh7JsU3
cXia/E62eBa1w8bB+s6cd4/6U8TXTVnvVvVPs6oY0+X4c1u8w7W7tRP1r9md
8q7UP+d+ensfcjI8g/8B2XELNw==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7631840877601517, 0.8471824400304654, 0.913783217179092], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFl2ls1UUUxQfa1/a15bWv79G3oaIGSFxDlIiJCSKLS2yxi0uiFTQRI5oa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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlXtolXUYxx93Odu57GznvOd43nPOKK2WUA2JigqqNdJ1wdPaEStInQUq
XZgUSTc3l6U5IVLLC7RCnZZ/2B9KBE0lLyUToSgJ648maWYTKp1k5GX1efru
j4f393uf2/e5/iY/tajcVWFm3VAVtDhntj5l9mbUbCn0w0SzFu5DCH2HwJTA
7Hqoh/tgpc6Phma3xvRvFuexlHh3RNCB/z3Uzvko9vOcQ2QqkX8G+XPItoaS
/QSdC/Bb8PlkLXz+7S6Y7YEWgOUl9JuyZjPyZnfFzTprzN4F72ZsPIFuR7VZ
L7zP6832IrsZ/v4iX+5ruR+rM9uKr21QE/bbquTrHmherXx2pc0OhYp1PhRp
MPsU/Q3Y2oT9SRmzydAReCXwrwTPC1n52gXVgfUm7LVjqyaBHNjO5KS7Aoyv
wX8V+gzdKDENgu8c9prRPQi+VcjPQ/4K8i3IfwSvDIYTxDtKPlaDbz46U+B/
AIYH8X039CL6vVAzvv+EX05SQ/gD5LMvK2yekyvYG8soV7OxfxzZ4aJke8C7
kvy9lVctdlTL9gCYNkbl42Z0h7gvr1RMLeTqXmhnjep/Cnw3op8EbwoKuT+f
EdZhdMr4mgm9T/03gacT2d/J73LOM5D/CfmzGdW+Cft1+P+WGBCx6fjoJjeL
Asl7j06Ff11WvbAQOgu2SYGwXxOT7ENZ5cZ13uD+B/5LScXQyn2EnMWw9U21
zh0Nqo3/e4TY/sXmLdiaCg1zjvHvIryfydcpsP4CHZ1AD6P/a0E947k5Bj/X
aHZ4PF+NBLEf3WeJsT+mHLQRz/054W8lf1+gv6+g2XO83ntrwHh1RD2YRj8I
1W+Lke9E1wLpPhdVLvpSOrvNG8jvdmI6XKuaDIB1K/Rh1ThmzrdnNIvuw3vZ
c+i5cx/vcT4EnreJp0QMVxHbZe5niO1HXL+C/5ehtgphvkhuL0HdSc3cbM5z
oEhCO+QCun9DRewfgb8e3WigWfIe814fA8O1ter5L5H9Chp0ff71Yut16LeY
ZtBnqcfzldBMJdEfLag23u9DWe0An/2lxBMndyeIqSKmGnruJgSaNc/hHPDM
hRbUKKZcqBlyW65zGt7pgmrrNX2c+2M56fqOqiPf/VBfXBgPIDs3r972GJz3
FzVdFZfMKHhXB9oNXgPvrePguxQVvg0p7eT/+wE6D39LSrP7NfaXwFuXUm18
Z/gueyctnu+0EudtGe2yaVAZ/Y5Qs+r9PJJTj3pv3lclbFugrmphbCN304vi
eU3OY2s0o1l2Gd81D6e1S33nlIjvJPpPm3Z0vlEz7bN8G/+nua2iZsV33oFQ
GB2bz8QeeHuhhrh2us/qskC7wmd2hPvHE9UL3tP/cC80yrb33BpsLSxqlr3H
+7PqWe/VffibBb519XpLvMYruB+sVy/6DrO0Zt5z42/ayZR6xmvhb9Za8vVA
Vr3kO95jTZGPZRHFnAhVU6+l1zBR0Jvgb4HPQCRQTb2WPlN3+tsXapZ9Jprz
2qG+Oz0H3eguKWg3e4/PbNCb4G+Bv2H+Fuwu6u3wN+EyttvH8fuM/gdv8RTt

              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7811963641094064, 0.8531890542388422, 0.9057137335160145], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFl31s1VcZx09fb297b3t7ey/3DeswcUalRid0zLVAjG+TAh0ON3CMUCaU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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlmdslWUYhl9pe9rSHnpmz3dOESRGjUqJg1a2xMSRUEAryFAgggoUFI3G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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7992086404586612, 0.8591956684472188, 0.8976442498529368], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmXmQ1MUVx387y+x9zO7MMMcilkq0iCyKAokCYoLEAwVDTABBS3CRIwh4
lCaABwE5PCAIgiwRD1DRuKhIIqYsr/JIwGiqPFKlGAN4YRJNvDBRNN8P32f5
R++vp/v169evv+/qPWTSrNEzU0mS9NCfSn3vyCbJU81J8qj61xaT5ITWJJlU
lyT71ZY1JElTbZLcou86tRVa87rm72pLkl7pJBlfnySXNCXJwook2anx75Q8
3hn0wzQ+QTSXNpm2nzbsqE6SE1PmBV1KNG3dkuRNtaclw9BckvyjJUnurkqS
k6tNA59nJNuF2uNWyfVyJkk2dk+SZ2uSpFptpvgfJt6NPZKkLJr1oukSv91q
qUrLiAyPiE//etP/U/052uOIWtNMKifJ+WqTtd/f0v6uk4xl8Ziqfh/RD9O6
HWrvi/480U5U69DczTX+doo+lfZ+7HW6ZP5CNHs1N0ny9lX/atE+LxnvlA4r
RHtylfWLPvrWmfZh8e/s5vn5FeYHX/T8eNr9X6l/i+hHNbq/QvxWqq3R7xul
nzu135can6L7XaX7nSrdnq1149T6q+2SDA/l/HuW+A+otw5HqH9anPF1jS1X
/7V6n53xfuLZrN+J7mas1p5S7TNxV/BAvyM0vVZja9T6iv7PkvMK6XmdzjRY
cp2Ydf+7+ST5vtqplaZFj69o7fkV/k3/Tc2nhYc50s9R2uMQtYHat0VYW5Lx
PXaI59LoIyNn6Cv+n2vtCI2XxGuLznqW9LNCPMfqHu7VeDllmTk/cj9TY0yh
c3D/TPTBy6zAWF66HK7+S5WW5QDetdeyJutqrXjWaZ/hJdPcLPrjJcdFor0u
7e9pGr9fY3M116GxbI3Xs/ZR9acIA7emrHfG0eGAtM/NXmPji2zoBXw26rsd
bKu/MeV75b7P62Zb5jzQb6oypsHzSeLdpH65wphgDfrprPMe82PfswMzZ+ge
/iVdN4vXH3W+X+ucT9XbVtvCfi8pJMnFaleKpiSdF9UWa36p9lqi9qToe2rt
C8JfEjpHti2SeZDGPmu1fBPTlgH5ub+9Vb5TaJuD/mPp8E8l6w8ZdoYc7PNG
9HuJbqTOMVjbPSjc7ZDOt6t/gfS0PuX5j7TPVaLvpXvpL5pjs7azBsler3al
5n6gs54u2Sq1rqf0s0m0l2ptWvOTpZdfiuZEzS+TzR+qfa/V+Id5y9Gu37ka
284YYW+c2rkNthl8W7vk2aK1GZ3lA/Fuh59oRmrNMRq/puh7Oi1wDZ8pkm9U
g+UHUw31xtUpWndq2XNgaJXaaNGfo9+fa/7DxPj9UWD4CPGfX/Tc25J3m2RI
1xjL2D+YHKS5i7XHcPWf09iF4rUh5fkZDab5SnKPUBsS/nJa+Ey+06N/iM41
tezf2PS0svHb1WQ72CWaN9p8fs6OTMi9K+SHjj7fzU2235Gy43eFyQUV1k17
6Adb7R3Y3iyf2NXoOfTOWvixZ1fwAecTA29jo4+NQcsaaPiyDh2iV+4AXM0Q
XtYKV5epP0pnOlNtTOBrZeAPm8JvwhvdMT9X38HS/7+l9x7a63Gd4wm1XTXG
DThmfLV0f5B+L9SZXuVexWOB+s9KTzsLHp8kmzi/1rEVu841G8+dgTPiLGP4
L+59ntYeqn2PVv8mzbdWmSfYgAYeTxcdU4mnxD1iALGpUePNal/W2s89mTfP
geJxT6Xt6960bYQ+9jAsYgf84c29DMVPtDgvII4gJ7YwW/y+J/636fc+3dtn
ageDSel5YM66AGP4CfxeS9EyIc/WRvs7YuflGhsjXrPFfwC+SOunpYwt+G2T
DB9o/xl5y4qdgxmww72dEvLcrrWjMo6/6OvjtOPpCeJ/fNH+oLLGtgs+f659
nhQGvkjbZodFPMWGOTv30ldnb1c7WTLP1veiwrcxki+2DK6HZo1tsEJe8qr2
PUtj90rvb6R9Ts6Lr8YOGUMn+Navwh4/zTonIB9YlLd+0S13xT1xR18Lh+ML
xs/XdfZ/5BUPaG11i+92oM56cNZ+b1WVeaNbdLww6/418Fb/SM33Uf9nBed8
b6m/J289/bTg/djrMvXTWfuW2TrTrJJzRvQ6OHT7ouZfUDu1yn4LDHFP3TQ2
s+C4CoYWxL08Ri5SZazu1z7DCp7r0P5bmo13fBX64Yx8v47+R3n7buyvp/gf
XbSO1uo+15Wd144UvyUt9otbG2x72H7vlPMI8pC2nHEABm7Q922t6VHlfBV8
gf8+Zftc/C14mR70xCKw0ivOBOY4F7nK3Eavx9b3pa1DsIyNEKOPE79p4vM/
9fdIl7vV3qy3voaH3Y2WLGcXLPcFop2cNa/nhJGOrPHNGmIdNnJFi20VHbMP
NgkmF1VZ9+RvrTnnmsh5hPrvZexToB8Q+UOdzjgj49yYPAD/RowmX8HnzUs5
n1ja9O380vCB5CADwj+fW+2YjV4W6H5+pTauzvkRNMj2hPTQM28doo+fFKyT
bZHrkOeMK9h3c3fkExembSMbtfcGtX2VPhvrwOfwgvHEndSId63aDyXLyhbn
stDeprOfGb6Cmgc9sucqxbVHtHZto2Wl9iC+ITN5JLFmk2jubrMPw4ZuDzua
02DZkBFfgj8F28gIH+qX1ZXGF/5tWda+Ej95m2S5Qn4kk3Ktg7zQ75TME/Ne
1xV1klgfiCUHBZ/dOssetUOr7Q/4UnvkWp3j4vMXt9gesIW94veeWk2NcZTN
GUvkTU09jCfyCGjROX6hX2AbDJ5UMA65w88CY7Pyth/ONkj9ywvmsUM8Dy9Z
d9Sp5ITkhvtF805320RJMvZote/9Xc41DznuhIJtYFLk+OiVe0cH6Ac9DJbe
rs8598bmO6NG6xU+l/sitx3S3fktedj6iPXUMdSC1C/kZczjz8k34EGMaCq6
9qbuJm/HBqg3J1NTlp2fUt9Q2+BPyLvIGzojDkBLLDhM513ebJug9lvd6LqX
emVm1DX4afwd/hn5iG3ENfJC8hhyGPKcrsiL8MH1Zfthart01HfEjGn1lnl9
5Hv4zIkNjpXwJY6CWWIEvgwfgV1ManC+xH74ceIU8Yo8c1nUmIWscQkmb5B+
Psk7b75S8lxV8t7zuZOM8xlkQjb6oyXvj9XGap//1lpWZJgYeDpgX1X2K/gc
sD8z8N+/aF+Pn6eeXh/nwr7QJ37gnpJrUfJVapEHIp9c3uYcDh91XNb5NHbN
ujWxFiz2D99DLnq/2l8TY4F4CB6oXTZFzUK+f2yrYwB3Ca/UN/lV5FicA3ry
6h6RG2KzvHNcEu8ta0MGeJAbrQ6aLp1jc9457e3aa0SrMQK/8+qcV40Pv4Wf
BL/kJWCYGnpTs8fxbdDh68hJyVORgzh4a9Qdh4v34jbrgVz1uRbTTNf473N+
RyDveuWbGKW7fa3k35wFDBIv5skO1mf8rkCNNbTVfhCb21pv7B2j+cdK/r1H
e72l9nKtZXup1vnt1Tr3fLX7qo1TsHJpxB10ij65E+4GXJGb/yfyc3JA/Dvn
JS6xhnHi3MvBf0LoBP3s0v67C977WK19QrI9XG/d8MUW8DFDMvYz4G5/g+2K
OgHdEO+oXTYHZvpIf093t8/iDaMr7hG/0FFhnzNddjCl7NqLmmhv1KAH/EuF
/QZvDN/IwjsSfh09E1fnVzm2PhD7tkddhRzEEfK1cs4x8cjurtlGhs/bEHKQ
H14W+SdvOdg3Z5yqe9ua87melz5OyNi3vaD+0Ix9If4gG29Hg/P2+/h8fMGO
etcR1KBjog4lR/57zvHiRfH5S8l0vIWQV+BbedOEZ2P42Y2hiyXC0tKicd+s
c2TafAdHZexrwAC+CrtnLTVEpmg7eldytZac0/UOvPP2VRPvkuRa5IlftTrm
NYrfoqLzO7BLnkocv1F73pSzjZyh+ZFFvwVyJ7eEL0L2tSHDp6L9JOf3MHKm
rsAGOSvvqPgQ8l94wB8cUC/Nifcl7hx53xE2s1nbB/G5tuwYTY6JTZJnUhst
jPqIujAXtSG1P3fKWx1z5OnUEXdI9g1F3/3KjGsPYmtv9fMtrp2pIYYUbXfc
KdgCV+Du48jreD8it8Tn7Iv8nlyXc5Xq7APvKznGvx+5EHkb/LeX7C/wFcfg
S4teg09dlLNfJRb1itjHm19n1FBrNH9zzjYDRuYFTt6Wbt4qu/7Gj+JPiYOD
in5X4uw7tG5bq/3tH0R7TYvrg454510R9QG6Byc7tO75rOd4O+dtnfziN5J9
XXffFXXVQy2O3b8t+cycd3nO703giBxwQ5y9Pe+6i9xqiXTzYMl6IX/kHRY/
Tl4APskrwB0+GuxhS3PDfjvzrv2IOfg37BkZFhf9TkeMmKAzji87/reXXc8Q
j7+ImgD/QOxCNuLXdVp7fdF3TU56XfTP1bpzys4RyLUHxdqjJP/2ZtsJGOcu
oKdmxk+R2xNrOT9nB4/4GjDJeTkr2LtLe23o7lyPu4QP9jI23vo5M3F1QMTW
fpLrwxb3O6L2xmfyvwfsET/Juxd5DDb5C8l+edl5+v8BL0kw5Q==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3uQ1WUZx9/d5ewuZ8+y57p7zm8RB2J0GGEREMq4SCkhroLaRTeJweUm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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.817220916807916, 0.8652022826555956, 0.8895747661898592], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHmQVNUVxp8wMz1bd09P700gaAKJMIoRqqJxQaKpILgvoAFc2LTKBDQG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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwll3lslVUQxS9deH2vpby1b6kgaASFAkqNUUEKigmKLIpxCSBSWsBEwQ1c
KAiUVXE3AQQF3IkxGnA3AlEWUYKSKIKgYqsiIBJFARfU38n5Y5I738w9M3fu
3Jn5OtdPvmpSUQhhN1QC1SZDmJsLYWQshNFQXSqE/VUhlEVD2FYawg/ZEB7k
2yvofsG3KazvhB6B7wvAP3Hk6RD2tQ3hLGheNVQIYX5FCPe3C2E8+OvAXwz2
EPgy1hFoIk78ITzWWXT2FIewFLzBrFfxbQr6d0GVp4RwBd/6I9tbGcIn4Kex
uQX7B8HLIT+KP7XtQzi3vX0fijwS9Rla8iG0QvvKQxiA/jGw+kA1rKdyvuPo
dsVee2wpMH+h+yc0BnuHOc9vKdsQdhP2n0a/CRunIq9EfyF8A3xzGXGE6sB+
CLxG8Hpg43di8wfUKYLDimcG28SnO/EpAuMzbH0KdWb9HfZ6gjcL/Z2sS8Dr
AVbPnGP7ERDDka9E/ktbn2kv97M54bv5FNqD/TPAK0P2Afa3w3fMO5bF2P8H
fg77d7F/RqljO6DasVGMdZYF2JhT5jM9zLlPx+fR+H899i9Ctoj937J/IzbO
xreOxOgOYpHFxlH0r6z2XStHuiA/gH+VyNpBA/D3YmgUeHVF3nsuOoWYMb5m
b3d8mIUvXYj3U2nnnHLtZ2xWsl6e9nop/nfD/5nwP7Bug85ssJqha8EL5FQC
rA3wS2O+0y0Jx0yx2gmdAl6KM21GvQPy9TnnsHJXe7qCfwE6m8C/LubcmY/O
mJhz6Guw9rP/Bnz/DLxWfGmB6sA6go/dcs5Z5eqxiGP/Pvo/l/oO9rB/Y8J7
5VML/HdZ4z2B/jB0V6WN1Yn76MtdNhPjy4MxZrC3Dv0R7L0Oupm7+iXhu3ia
/WPh98KfQPY4/ANgXZT0WbvqDolPDfJDYB+EnsfXbcjX4u8aqBb/e0NbsZcD
8wi65+S81p3FwJ/Aty7gXwL+AtZD8WcMfCP0QrVzUrkozAb4h5J+6xOwH0F3
NWfMFxnjbfx5p8qx0p3obf+Nz7URv/EW6HXez2u8nwnID+n8yM8A6xnuewLY
K9J+u0vwrx/Y24jZZnK1V5sQXsbfB8FfXWabg/PGFFYfdK6G3w8/BX4AfGf0
L89bppztDj8ib9k4+IXEYkHOtpUT89Kuqaqlu/HhUfw7jz2T0L2H893CeR/j
26Bi17x9addc1VrViEZkpXxrhf8P/+YkbFO2xrL/ML4P4ds3Rb6z5/ElE3dt
WIxP/cHfBH8+8qvRP429TehfE/Ubnop+Sdz1vB79ucjeVb3G3oX404gv41Ne
Kyb9wJuWMJZybBFnfQAax/kaYr7r+cgbor7zYfDb4n7bN8JvQlafce7tUH2C
vyXj3DwO/2/SOaNckQ8j2H9TyvEX5hD45oSxhL+Tu/kz6bdeg86v2Poma2z1
FNXmj/OuTarRoznbDdCb8Ae4/13sfwv7n2O/D/svQzaoYNlQcqof2Pfn3Bs6
sv9Z3S20IrgnqZcpx5Rb6mmq9f3hx8dc80/iTzTns6me3Y5vd0BN8APh38e/
ddDxiHvYGs72Vta1bDIxnwpfk3VtPg2+VvUx5dqtmqK7rU9Zpjtu5SzfZ9wr
vixxLZqZMpZqSB38duKxVbWaM+7m/OvR/yriGjKcsw8r+G1cU+FePxn5iohz
ZCSyUdBa9v+ETgr5j1nXTuHP46zt0u4No2KeJXonvdZMsRL9ueC9Ct576MwH
6xj+bCnyjNATvr7K2DUV7o09Cl6rRyoXLmD/rRHnxI/onoSfVuEef4/0idna
In/7CdmBjN/eHuIxK+cartqtnrCG8zdkXFt1p8rNKXm/BeXoXeDdXXAtbA6+
24fT7s26417gHcbei8R3danf9iaoJeI3PhqsQ9iYHfxtRco5oVzQHs1KHVLe
q5lpGedZXuXZoYoYnAn+6/C74OPYm4ju9rhzRzWjC/ifJD1LRPG5JucaoLev
nFKtz7LnuVLX/IMJ92D1Xs04x9E9Ac0s8R0swta9+Nuh3DVRs8PepLE1Q6jX
98X+h+Xu+UvQX1zlt6MceoP1joxnJ/msu1APV+/WnajWtBLPLyqcTxvRPVHt
WUox0iyjHqfepplGtfcd5VuF7zta8Ayo2U8zwWPYe7TKb19v+HfV85xrkWao
K9G/quBcbq/6mXFPVL1RDnZTLcLHaVHX1EnkcknK8dW32+Bvzfru9Wb/wr/B
Cfcq9aAGsBsLrhVx7qsC33+F31fh/H6Rs71U7d6pGr2VeB7NuJYohvcmXFNV
S9XDyjPuoeqdA6Eo67KMZxf1gA3sr407N1TD1sH3jtu23uTArGu2avW1YCbV
vzLO/SMxzwbTE5bJ5vS8fZavE9k/nViuhF9G/J+ENsCvr3Kt0DfV6qa8dRXD
OOs1SdcazUTqBcLQXvWES7B3MunapJ4zMuUz6mya0d7OewbW7PtyuXUvzXp2
0Z5JGc8seouqQV/he9uc/xUU43GqFwXfrWrahwnPUJqdlGNjkd1Y8Gyunn8f
9mbofRT7TcxOuKeolyjHZmFrJvRssWeIVSnXLMVHezQLJ/KOpWZizea94vZd
M7re5vCse7Ny6PyMc0y5pZr1Mfncq9r/ApqhNAsvT3n20kzczPlHVns20Jv8
H1Vz388=
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8352331931571707, 0.8712088968639724, 0.8815052825267816], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHuQ1mUVxx922X33wvK+777s+9t3wQmnRnElaaJEShmTEoGa1MxJlnVg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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlmlslFUUhi8MnZkyhfab6SydQoRopC0VjKiggsENAzUREI1SahSxIBFw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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8532454695064254, 0.8772155110723491, 0.873435798863704], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmH1slmcVxp+2tG9LS/u+LX2fvm8ZyB9MYRhNxoCUVRj7cHP6xwCzhDCn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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtllls1VUQxg+9QFsK9LaXe//0XxB4AAU0mlgWQfaAgPrAJqaCICgGREgs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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8712577458556803, 0.8832221252807259, 0.8653663152006265], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHuQ1WUZx192WfYcz7J7dmH3xzlL3NEGypkQRiFAnBJWFDUqxUDoAmJR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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlmlslFUUhm+HLjPMlJlpmfmYr0gBQYhEfrAIVQRMBCqrNsaALFVsATdQ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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.889270022204935, 0.8892287394891026, 0.8572968315375489], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmH2Q1VUZx8/ey2X3cvfefWF3L3vvou5itAsjaZlCiDZlOmxaYIKmM5qp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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwllnuQzWUYx1+7jnWcXXvO2bVnz+9YkltlCGnGIjUlTaJQhP5B1FghktCV
rLBuuXVBU9hc9oJuohlCJpcoRRfM6DJNTDNM02TG1B99vvP945l5399z+/6e
2/u0Gz9t+NS8EMIBqCm0vjiENqUh1DQLYWgihM6pEC7kQrixZQiNCI7OhpBf
EsKSWAj3wV+YCeHbZAiD4NXEQ1jMfSM6I/Ixho1GeAMi+IUhtIe/t1UI+6AU
/E7otEF+HzJdOfeB/15ZCO9Cc5qEsKg5fuHfALVG/g74ndBdnjRvbEEIW+Dt
wN9R7G+CX5IOoQr8R/B9Gp0K+NNTtl0Jvc25lm+bOQ+DH8v6m84XW4TwGbbb
ZezrMv94L/Zu5/97FSHHLw3F/7a0ZavxXwPWJRDioRZMSfjTy9GDfz/8fug+
wv8/Br5+2FiBr6kljs1zccdmJz53NneM6rB1kfsp7iehJuBryv8gFs5h8zD8
t/hWTezbk5PZ2NsOcxu6A+Hfg/8a8K/A9kj+aQfyP2JvKPGdwH0Guo3k+M1g
H4H7+rRt/wTdgb152PsT2UPoPA3+rehvRfYZ8NdzDq2JO74r4P+cNGZh/Rrq
j/589BeApxo6DpavoMPgmYvNas7rU66VJdAu8O6GThOrdtg7kbFN2ZJOR/59
OPyFBcbcuZVrQLkfB43A18PQwjzH+Arybbi35nyMGjqHrdewuVT5znOtVnF/
Nu6arUu5hlQ7qokF5C7O/SbOY4jHl9g6GhnrMvzv57wMnQH57oHB3E9j8wvw
PoLPafBKM64d1fgG7neDd3aBMWyFtw3aEndOHobXyL2OexXynxLvPdAZzjns
lZGbwpxzq28PcR4KDSIXH/J/a4j9nqR7KQH/TfL5O/ldmHCNlGO/A/b7Y78C
/vuc69UDcWNYzrknMlUFjsl55VM9xTlGjXbFV28wlDe1zgr8fZB0rmLYi8Mv
yLnXrhW6VhaXuRdUMzPIR1b9km8bzZBNcT/F/W/k1xC/fGRm4Wsd93r5hibl
G8PUlHtKvdSD+/aMc6pcbuPeDVsdcs5VAfY3YW9z5F4fQb76wJ+J/RPY70LM
3iA2fZHvy3kg8ZmAvSGRc3cYvLdx76H8439w3L01BwyD4u6x7vA+x3+vPH87
A78xZSzH4M/gnAHvMPD25dsUfE3PuXfqwNdIbL7GxhF4z+cbi2aEZoMwaXY0
zzmWmiGvk8902rF6AbwXwL8I6sH5eWpsHf7fiDxbNSMmIL+x2LWqmovTqy2g
I/TrC/ibj+1qaDG+ToFnJXgng2cW+rOhB4UVDHOprQeQaQ/vIP97JeYaOguW
75DJx95BYnCJ2F6GSuHvRmdV1piF9UXoF3Qbytz7o5Hvjn5H9HPYroB+TTom
isUY+KflL2WsLclZkvPIyLNIPf4K/7Y6a9u9g7H1zDhXwliJ/W7Yv6XIM/sk
vBOt7PsLZK6BtZD5uxbZ7sTnFPwmSfOUk3non0E/xv/9w/9Ugu0q8ThGLlYT
r6ISvwl6C2TjLNiug2kd5+OaOdz/4r6S+4FCv20lGdeu3rgf4M2Dbkp45lVm
3cPqXX27K+ueVa8qx0sy7kn1omb6WOKRh8x58FzDXrVmbeS3WTO3JfKFGfdm
l7hjfSt4Wxc55p9w/whqB7793Pdy/jjn80EogW6LjGe7ZqB6cyr8GUXu0f/A
ujZyr15HZlrONa7alkwDur8lnTvNtCvlntma1YrhDnQv8v8v4387Nhq410M3
g7d3wm//q5HfDu0Akl1abJ50NBu+jxxrzYi5+B5PPA7h/6Uiz+I5OZ81k1cV
+x/1b1Uxz6J1/NOlZp5JqvWZ5a591fyAtGtEtSGZDWm/kXobFfOenGP8z53w
s2B6BSz9Uu4FvZl3wn+t2LNHM+BR7qOynjV6M4T18ax3K2HW7NVOpV1KM1i2
x2GvIm4f/5Y558q1vi3gXxq4X27hHm5Ob3+Tce+oxzX7V0OTY34DNGsnlXo2
aubq7VYOlTu94bKlGabZJZva/bag/2jCO2AjvN7IVyJbj8wifB2PXNva8S6B
p23k3pCMeqEWGpVwTzyV9Q6i3UM2J5d6Bqj3taNp9mgn0i6kGbQzsk/5aij0
rG2b8SzVzO2U8wxQ76um1eua8Zrt6vkpWb+pekuFQbtVUca9oB1Lu6x2KO1O
2mm3RJ6pmqW1+Lta6h1Cu4N6RLW5KzIW1ejjGb8ZeivmBPf6O9DwhHv+u7Rr
TrWmmVihXSTyW66dZGLWO5p2M+noramL3At6c4a18o6p3VI7j2btE0nPVs3c
V7k3pj279ab9kfbM0qxSzrUr10b+F+3MmmVPJr0raKYN4T4x7dmuHeJ/jjqW
qQ==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.9072822985541898, 0.8952353536974794, 0.8492273478744712], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmXuw1VUVx3/33sN9cM+5j3MP93DODwELRJ7yVDGq4RmWmAmpgKWgo4LK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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3lsVVUQxk/b142+J+3re72997K4Yy1oC4JARCNoBHFHRXFBwQUQ27LE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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.9252945749034446, 0.9012419679058561, 0.8411578642113936], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmgmUlNWVx7+qrmqgq7ururu6mupCUFlEGUMDsgQP0ChmnGgQPGdUZJEE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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3mUlXMYx3937jRNM3O7977z3jvd+47EIZFMkxYNskTiOHL6g2TPkkjZ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              "],
              VertexColors->None]}]}, 
          {RGBColor[0.941176, 0.906538, 0.834043], EdgeForm[None], 
           GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmFuMldUVx/fI3A9zzpkzZy5nDlZAiBSBKZc+MFzVKmBqb1qaNAymQPVF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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwllU1sVWUQhodS20J7e8+9Pfc2p6eVIjZgMbhQXKhxocYQExMhcWOCKGCJ
C3QhEKj8BRpIQCtRS1UE2wVKwbhQQgwmUIqCJkog8lMQN/wYIhFdqJCokefN
m9xJZs58885878w3d+rCV+e9UhMRLyO1SEcpYl8SMacpYieOtWnE/izi48aI
W/ivlCP+yiMWN0csuCNiKvpMvlUKESkyztnzyGnO3wBwCfH34q/iG6uLOAB2
TXvECPFd4K8pRkwBI8HfA97DLRHTsFuxj2Pfhb6aM53hHFvRB8HvA38+3wbQ
D4E5l3o38m0R+f4Fg1+Mkm8Tvv/BGCLf9QkRk/BPTl17N/lXgNeBvwB2hfh2
9C7qbcJuRM5z9gLyJ3d5jvib6LeQg8RP59vj4J+mhhPEPoV/F7E9qe/6Gf6/
K8SDmZH/d9noN5ErYJfDucWB7q4aZoF1X+ZablDfIP7tuXPtwH8S7CfJeRZ/
Wh9xT1vEBuzVDREz4OCFKjxgr8Veg3xA7HvINeI/Iv55/CPU9Aixvfh/Jtec
xHcbB3M9+vyqfROQDeTbKA6Ib+XM55y/VLFvN+f7EudUrnUN5u4lzh+pM4fb
M/dIvVHPhql3qM3cz8LuJr695FnrR6agP8C3AfQh5IzmB/sr+Gmtt/5+7rvo
m+7aWnKs7vxTZs7FtTh6C7u7zdy8jv8XfEf49iW+ugZzcQi84wVzspv6V+Tm
9jAYMxPXoNzq+RPY49ht9Z7xZvRi6rPP0M+xzJyJK+XQLP7X4l5qJjfjm637
4LsARyn5e+BvE7U8Deabud+AZv8datqBXQXzbeKHsX/UvJeta+a2qh69H+br
D3L2Z76z7qo38zX6USSpdc4FVfdQvdPMdBJ7J/LPRL/BBxPXqNom4c9KrlG1
qSfHOPsQ9XWBfZXzw9SyjG8r0XuRbWB/k9vXS451qXeIdodyvKbelf02vqv1
btAO0e7Qjri77Deot6c3Mhm8Eb59W+uZ+gL7xapnTTtJd/+V+gZrzIF2x07N
WJN3SB9v69HMs/YYNQyB9WnZuVfhXwRWB/vo2aI5PgpWC3Yj9v01fruHE+t6
w7vA/hBpnugcM/Q2c2P/BuY5cp3NrGtGPiHXKP5TBfdAu6qz5NnWzrpOvmH8
ewq+w2jZPRD32jniVjWrVnF8EH0g9y5QD7Srxbm41s4ew38x81sWpmZ7edG7
RjO+lNzLc/dKd/iB3nyfendpR+juYxXPkjjQrr9c9mxr5+9Fb8y8W9WTN/D3
555V7Qjt2ncz73btXM32lsRvTTOu3a//CP036D9A3GwrepeLI/Vic+JdrZ7c
Bu1k6Yw=
              "],
              VertexColors->None]}]}}, {{}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4645, 5221, 5220, 6410, 4777, 6409, 4778, 6411, 4624, 
              5219, 5116, 5117, 4446, 6446, 6447, 3958, 6941, 3965, 6943, 
              6944, 6942, 4648, 5247, 5246, 6580, 6579, 5729, 4228, 5728, 
              4227, 4375, 7116, 4138, 4380, 7131, 4381, 4267, 6544, 7109, 
              7110, 4922, 5066, 4794, 7061, 7063, 7062, 4675, 5367, 5368, 
              4914, 4139, 5333, 4140, 7065, 7066, 7064, 4797, 5325, 5324, 
              5327, 5326, 4163, 6612, 6611, 4798, 7067, 6422, 5731, 7197, 
              4231, 6703, 4230, 7167, 6548, 7111, 7129, 4278, 7170, 6616, 
              6617, 4164, 5336, 6615, 6614, 6613, 4799, 6952, 6951, 3972, 
              6462, 6461, 6817, 6816, 4858, 5129, 5130, 4834, 4836, 7157, 
              4835, 5134, 5135, 4774, 6404, 4773, 7030, 5328, 4166, 6619, 
              5622, 7130, 4287, 6730, 6470, 5571, 7187, 5570, 5735, 4235, 
              7128, 5610, 5611, 7073, 4802, 6620, 6621, 4168, 5339, 4167, 
              5337, 5338, 4801, 7072, 4800, 4150, 5255, 4149, 6596, 6595, 
              4660, 4775, 7104, 3984, 5069, 3979, 6551, 6550, 6549, 4286, 
              4384, 4383, 4148, 7117, 4377, 5396, 5734, 4234, 7127, 5606, 
              6593, 6594, 5250, 5251, 4789, 7052, 7054, 7053, 4923, 6947, 
              6813, 6814, 6812, 4461, 7029, 4769, 7028, 4768, 4780, 4779, 
              7027, 4767, 7026, 4766, 5222, 5223, 4652, 6945, 4651, 6946, 
              3964, 4053, 6542, 6543, 6698, 6697, 4226, 5727, 6578, 5602, 
              7126, 4224, 5725, 5726, 5545, 7102, 3957, 4051, 7158, 7159, 
              7108, 4644, 6940, 4645}],
             RowBox[{"-", "7.2`"}]],
            Annotation[#, -7.2, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4161, 7118, 4379, 4374, 5826, 5721, 5722, 5720, 5724, 
              5723, 5761, 5718, 5719, 5376, 5377, 3956, 5825, 4372, 4445, 
              7143, 4444}],
             RowBox[{"-", "10.8`"}]],
            Annotation[#, -10.8, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4761, 5322, 5118, 3959, 5763, 5764, 5762, 5766, 5765, 
              6449, 6448, 5768, 4261, 5767, 4262, 7019, 7020, 7017, 4762, 
              7018, 5120, 5121, 4447, 5378, 5379, 5380, 4649, 6454, 6453, 
              6699, 4229, 7166, 6582, 6583, 6581, 4376, 6746, 4162, 5334, 
              5335, 4796, 6420, 4795, 6421, 4141, 5224, 6916, 6917, 6254, 
              6256, 6255, 6403, 7068, 7069, 6423, 7071, 7070, 4788, 7049, 
              4787, 7050, 4144, 7051, 4145, 6586, 6585, 4653, 5127, 5126, 
              4958, 3974, 5398, 5397, 5916, 5915, 6412, 5228, 5229, 4657, 
              6953, 7113, 7112, 4924, 4058, 3973, 7103, 4771, 4770, 7153, 
              4824, 5687, 5686, 6427, 6426, 6428, 5342, 4214, 6683, 6682, 
              4825, 7091, 4236, 4925, 4423, 7138, 4424, 4390, 6769, 4389, 
              6770, 4736, 6970, 4680, 6969, 4681, 5274, 5273, 6625, 6624, 
              5625, 4388, 6768, 6767, 6707, 4237, 6704, 6706, 6705, 4068, 
              3985, 6955, 6956, 6954, 4662, 5231, 5230, 6258, 5992, 5993, 
              5989, 5991, 5990, 4973, 3996, 5441, 4663, 5612, 5613, 4154, 
              5256, 4153, 6425, 4805, 6424, 4806, 5341, 5340, 4169, 6963, 
              6964, 6960, 6962, 6961, 5232, 4152, 7075, 4803, 7074, 4804, 
              5330, 5329, 7035, 7036, 6751, 4378, 6597, 4151, 6710, 4240, 
              6708, 4239, 6709, 4661, 5986, 4491, 5985, 4492, 5141, 5140, 
              6480, 6479, 6478, 7032, 7031, 4297, 6731, 4296, 7171, 5935, 
              4471, 6824, 5790, 5791, 5789, 5914, 5913, 6815, 5128, 5395, 
              5394}],
             RowBox[{"-", "10.8`"}]],
            Annotation[#, -10.8, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            TooltipBox[LineBox[{3955, 5072, 5075}],
             RowBox[{"-", "14.4`"}]],
            Annotation[#, -14.4, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{3990, 5142, 6845, 5987, 5988, 6733, 6732, 4305, 7033, 
              7034, 6498, 6499, 3995, 5150, 4004, 6508, 5802, 4312, 5801, 
              4313, 6055, 6056, 6054, 4524, 6957, 4664, 6598, 6599, 5234, 
              4633, 5270, 4676, 5366, 4876, 4877, 4677, 6965, 4665, 6958, 
              4666, 5236, 5235, 7043, 4782, 7042, 7045, 7044, 4525, 6868, 
              4526, 6057, 4314, 6023, 4508, 5442, 6500, 3997, 4080, 6855, 
              6856, 6021, 4507, 6022, 4306, 5994, 4494, 5426, 6489, 4867, 
              4866, 6846, 4493, 6487, 6488, 3991, 6971, 6972, 6740, 6741, 
              6739, 4368, 6742, 4187, 6645, 4413, 6785, 4637, 6784, 4412, 
              6643, 6644, 4821, 7123, 4198, 5352, 5353, 4820, 7156, 4819, 
              3986, 7097, 7093, 7094, 7092, 4826, 5359, 5358, 6686, 6685, 
              6684, 7096, 7095, 4629, 6923, 4630, 6924, 4472, 6825, 4473, 
              5936, 4288, 5919, 4466, 5399, 6463, 3975, 4059, 6818, 6819, 
              5917, 4465, 5918, 4282, 5907, 4462, 5392, 6257, 6920, 6921, 
              6919, 4627, 5226, 5227, 4656, 6949, 4655, 6950, 3968, 6547, 
              6546, 6545, 4626, 6918, 4625, 5225, 6588, 6587, 4654, 6948, 
              6276, 5897, 5899, 5898, 4269, 6723, 4268, 7168, 5884, 6805, 
              6804, 5123, 7022, 7023, 7021, 7025, 7024, 4263, 5769, 5771, 
              5770, 3960, 5119, 5122}],
             RowBox[{"-", "14.4`"}]],
            Annotation[#, -14.4, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4522, 6052, 4523, 6053, 4506, 6405, 4776, 5456, 5457, 
              5458, 5157, 4014, 6514, 5804, 4319, 5803, 4320, 6091, 6092, 
              6090, 4543, 5805, 4321, 5579, 6093, 7114, 4097, 6094, 6095, 
              5806, 4322, 5580, 6096, 7176, 7178, 7177, 4544, 6878, 4545, 
              6097, 4323, 6061, 6062, 5459, 6509, 4005, 4089, 6869, 6870, 
              6058, 6060, 6059, 4315, 6026, 6027, 5443, 6501, 3998, 4081, 
              6857, 6858, 6024, 4509, 6025, 4307, 4974, 4495, 5995, 4496, 
              5996, 6260, 6259, 6990, 4709, 6989, 6992, 6991, 4200, 5312, 
              4199, 7124, 4737, 4839, 3987, 4069, 6552, 6553, 5827, 7132, 
              4391, 6771, 4392, 6788, 6789, 6786, 4414, 6787, 5792, 5793, 
              5736, 4238, 5414, 5415, 6382, 6383, 5689, 6687, 7144, 5688, 
              6830, 6831, 6832, 6828, 6829, 6471, 3980, 4063, 6826, 6827, 
              5937, 5939, 5938, 4289, 5923, 5924, 5400, 6464, 3976, 4060, 
              6820, 6821, 5920, 5922, 5921, 4283, 5908, 4463, 5393, 6459, 
              5562, 5561, 5560, 4279, 5780, 6457, 3969, 4055, 5553, 5552, 
              4274, 5775, 6455, 5900, 5902, 5901, 4271, 6724, 4270, 7169, 
              5890, 6807, 6806, 5124, 6401, 4763, 6400, 4764, 6402, 4448, 
              5883, 4449, 5381, 6450}],
             RowBox[{"-", "18"}]],
            Annotation[#, -18, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4541, 6088, 4542, 6089, 4521, 6087, 4540, 5469, 5470, 
              4015, 5002, 4024, 5808, 5809, 5807, 4332, 5810, 6520, 6891, 
              4564, 7179, 6519, 4333, 5586, 6142, 6143, 4566, 5811, 4334, 
              5588, 6144, 6521, 7106, 6522, 6895, 6894, 6523, 5004, 4335, 
              6101, 4546, 5471, 6515, 4016, 4098, 6879, 6880, 6098, 6100, 
              6099, 4324, 6066, 4325, 5461, 5460, 4006, 4090, 6871, 6872, 
              6063, 6065, 6064, 4316, 6028, 6029, 5447, 6502, 6503, 4082, 
              4872, 4510, 5444, 5445, 5446, 6302, 6301, 6974, 4682, 6973, 
              6976, 6975, 4369, 5600, 5842, 5843, 5845, 5844, 4133, 5095, 
              5096, 4639, 6938, 4638, 5239, 4201, 6663, 6664, 6490, 6711, 
              6491, 6492, 4745, 7152, 4746, 4744, 6688, 6689, 4215, 5316, 
              4216, 5690, 5691, 6926, 6925, 6555, 6554, 7175, 4480, 6837, 
              6838, 5961, 5960, 4298, 5943, 5944, 5407, 6472, 3981, 4064, 
              6833, 5940, 5941, 4291, 5942, 4290, 5927, 5928, 5401, 6465, 
              3977, 4061, 6822, 6823, 5925, 4467, 5926, 4284, 4959, 4464, 
              5909, 5911, 5910, 5565, 5563, 4280, 5781, 6458, 5557, 5556, 
              5554, 4275, 5904, 4458, 5903, 4457, 5774, 4272, 6725, 6726, 
              5894, 5893, 6808, 4952, 5887, 5888, 5886, 4452, 5889, 4450, 
              5885, 4451, 5382, 6451}],
             RowBox[{"-", "21.6`"}]],
            Annotation[#, -21.6, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4563, 6138, 6140, 6139, 4331, 5584, 6136, 6561, 4106, 
              6137, 5003, 6892, 6893, 6141, 4565, 5019, 5018, 4032, 6532, 
              5814, 4345, 5587, 4346, 5815, 6533, 5818, 4347, 5589, 5820, 
              6562, 5819, 4033, 5021, 6149, 4568, 6148, 4569, 4886, 4107, 
              6896, 4567, 6145, 6147, 6146, 4336, 6105, 6106, 5472, 6516, 
              4017, 4099, 6881, 6882, 6102, 6104, 6103, 4326, 6069, 6070, 
              5462, 6510, 4007, 4091, 6873, 4527, 6067, 4528, 4979, 4978, 
              6031, 4511, 6030, 6033, 6032, 6262, 6303, 6304, 4810, 6441, 
              4809, 5343, 5275, 4202, 5674, 4926, 4241, 5369, 4840, 4841, 
              4074, 6773, 4394, 6772, 4395, 6774, 4393, 5626, 5627, 5628, 
              5847, 5846, 5794, 5737, 5738, 5427, 5428, 5429, 4747, 6384, 
              4748, 6385, 4481, 5417, 6481, 6482, 4070, 6839, 6840, 5962, 
              5964, 5963, 4299, 5948, 5949, 5408, 6473, 3982, 4065, 6834, 
              6835, 5945, 5947, 5946, 4292, 5929, 5930, 5405, 6466, 6467, 
              4062, 4859, 4468, 5402, 5403, 5404, 4960, 5912, 5785, 5786, 
              5564, 4281, 5782, 4460, 5779, 4277, 5555, 4276, 5776, 5906, 
              5905, 4956, 4957, 4456, 5390, 6456, 3966, 4954, 3961, 4052, 
              5548, 5546, 4264, 5891, 5892, 5383, 6452}],
             RowBox[{"-", "25.2`"}]],
            Annotation[#, -25.2, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3962, 5772, 5773, 5547, 4265, 5895, 4455, 5386, 5387, 
              4955, 4273, 5248, 4650, 4857, 4054, 3967, 5777, 5778, 5558, 
              5559, 4056, 3970, 5783, 5784, 5566, 5567, 4057, 3971, 6460, 
              4961, 4285, 5252, 4860, 5932, 4470, 5931, 4469, 4962, 4293, 
              5952, 4475, 5950, 4474, 6836, 4066, 6475, 6474, 5409, 5954, 
              5953, 4300, 5967, 4483, 5965, 4482, 6841, 4071, 6484, 6483, 
              5419, 4484, 5968, 4302, 4968, 5431, 5416, 4498, 5997, 4631, 
              6261, 4749, 7011, 4750, 7012, 4427, 5370, 5371, 5107, 7192, 
              5106, 7160, 5572, 4075, 6493, 5574, 5573, 5430, 4497, 7003, 
              4739, 7002, 4738, 6648, 6647, 6646, 6791, 6790, 7046, 6335, 
              6336, 5649, 6649, 6650, 5278, 5279, 4711, 6334, 4710, 6993, 
              4083, 3999, 4980, 4981, 5277, 5276, 4878, 6071, 4530, 6068, 
              4529, 4992, 4327, 6109, 4548, 6107, 4547, 6883, 4100, 6518, 
              6517, 5474, 4549, 6110, 4337, 6151, 6152, 6150, 4570, 6897, 
              4108, 6525, 6524, 5485, 4571, 6153, 4349, 5023, 5507, 5506, 
              4589, 6191, 4893, 5261, 4348, 5022, 6190, 6189, 4117, 6565, 
              6188, 5596, 5822, 5821, 6187, 4116, 7115, 6186, 5595, 5817, 
              5816, 6185, 4115, 4892, 4669, 5259, 4344, 5020, 4585, 6182, 
              4586, 6183, 4343, 5585, 5813, 5812, 6184}],
             RowBox[{"-", "28.8`"}]],
            Annotation[#, -28.8, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            TooltipBox[LineBox[CompressedData["
1:eJwV00lsjWEUxvFXQsKC1lhjE0QlVEtroTFcgiBolZUYosNtIm1awwLRlotb
LR0WVmK6rRh2kq5EYlgIQZW9NtGWlpgSWruK33eT/32+87znPe853zC/tGZ3
9Zjg5y8+OYRx00LYOzWECdNDGB+REUIF/9zMEM4jgcVTQiiQk0gPIX1WCJ3i
AbmfkMNLqJFLH1r7zMueHcIy5KB0RghpvH2u9+OxnCfIk5/pnHJeHFuwDVsx
qP4BehBP5T5DvvwK8TX1riOOJl45baST5UzBkLO+4CuS+mpAvnpJc+TRBjqR
V2aetrQQFspbgFq9bDT7qJzNtF4co3V0E43Lr0CDuFicpI/080WNf/Y0igv5
F+keelhugbqrsARX+NX8sfxj9DhqcBRH8FtfKbVOqHkSz82Vbc8PupO28HbQ
e3NCuI876HLuXfqBLjJ7D82ib+iwGUfQh4+IOXdIrUL36p09g65H9XUqelbq
9vKW8iqtT+BNmus8dTbw+9UcwC5rKbWKaDvtwAD6o3us/kq8tWee3G6aSbfT
drPdRgey5CbkdZq13Pll6Dd/H6byB+lL/fTQF/QzbeHXyKtGCXr4GbyJ+jjj
jHoc5ffy2/kpJMUXUIWf/OXOzox6pL/EK2h+9E7gAf+1fnLNGtNvK289bdNr
K96r/w7DarVYa8Zl1LlfKT3U02d4iir3Lsbrlv8WZdE3gFu8YrobhdiFIlRi
nbWu6LmhRHwIN3in1btJa+lra2tdN+tnDX0l/qOfdfq8pJcR+hercVHchEZ8
N2uTPd+i95lejb5j+87iP54tleY=
              "]],
             RowBox[{"-", "32.4`"}]],
            Annotation[#, -32.4, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3978, 6468, 4772, 5132, 5131, 4165, 6618, 5621, 7148, 
              4628, 6922, 6469, 5569, 7190, 5568, 5406, 5934, 5933, 4294, 
              5136, 4295, 4964, 5139, 5138, 4967, 3988, 5421, 5420, 5974, 
              5973, 4863, 5979, 5980, 5970, 5978, 5977, 4972, 3993, 5436, 
              5435, 6007, 6006, 4869, 5146, 5147, 4971, 6849, 4970, 5143, 
              4076, 3992, 5434, 4831, 5145, 5144, 5693, 5692, 4928, 4243, 
              5109, 4242, 5797, 5798, 5795, 4308, 5796, 4713, 6444, 4714, 
              5302, 4712, 5300, 5301, 4833, 5299, 5298, 6996, 4008, 6512, 
              6511, 6667, 6666, 4204, 6341, 4717, 6339, 4716, 6340, 7181, 
              4634, 6927, 6928, 6264, 6263, 5463, 4996, 4531, 5159, 4328, 
              6115, 6116, 6111, 6885, 6884, 4101, 4019, 5006, 5007, 5174, 
              5173, 4888, 6160, 6161, 6157, 6159, 6158, 5026, 4034, 5510, 
              5488, 6199, 6198, 4895, 6205, 6206, 6202, 6204, 6203, 5041, 
              4043, 5530, 5040, 4359, 5184, 4358, 6221, 6222, 6220, 6907, 
              4603, 7180, 6571, 6572, 6937, 4636, 7149, 5623, 5624, 4171, 
              5271, 5272, 4808, 7076, 4807}],
             RowBox[{"-", "36"}]],
            Annotation[#, -36, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4170, 6622, 4386, 5071, 5070, 4948, 6570, 6569, 5599, 
              4357, 6761, 6762, 6759, 4385, 6760, 4155, 6600, 7056, 7055, 
              4668, 5616, 5617, 5618, 5258, 5615, 5614}],
             RowBox[{"-", "36"}]],
            Annotation[#, -36, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4765, 5323, 4792, 7060, 4793, 7057, 7059, 7058, 5269, 
              4646, 7047, 7048, 6577, 4137, 4373, 6743, 6745, 6744, 4266, 
              5549, 5550, 5551, 3963, 5067, 5068, 4459, 6809, 6811, 6810}],
             RowBox[{"-", "36"}]],
            Annotation[#, -36, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            TooltipBox[LineBox[{4041, 5074, 5077, 4387}],
             RowBox[{"-", "39.6`"}]],
            Annotation[#, -39.6, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4479, 5137, 4067, 6477, 6476, 5413, 5959, 5958, 4301, 
              5975, 5976, 5972, 6844, 6843, 4073, 3989, 6485, 5422, 5982, 
              5981, 4303, 6008, 6009, 6004, 6848, 6847, 4077, 6495, 6494, 
              6850, 6853, 6852, 6851, 5799, 7145, 5800, 6442, 6443, 4837, 
              6445, 7101, 7100, 5375, 5372, 4513, 6035, 4832, 5374, 5373, 
              4514, 6387, 4751, 5694, 5695, 5110, 4430, 5360, 4829, 7193, 
              5108, 7194, 7161, 7163, 7162, 5575, 4084, 4001, 7105, 5576, 
              5448, 6034, 4512, 7146, 5676, 6669, 6668, 7005, 7004, 5651, 
              5650, 4415, 5848, 4783, 6342, 4718, 5652, 6651, 5285, 4685, 
              6338, 4715, 6337, 6995, 6994, 4092, 4009, 4994, 7150, 4995, 
              4552, 5169, 4884, 6122, 4555, 6112, 4554, 5008, 4338, 6156, 
              4575, 6154, 4574, 6898, 4110, 4025, 6528, 5491, 6163, 6162, 
              4350, 6200, 6201, 6197, 6903, 6902, 4118, 4035, 6534, 5511, 
              6208, 6207, 4360, 6225, 6226, 6223, 4604, 6908, 4126, 5532, 
              5531}],
             RowBox[{"-", "39.6`"}]],
            Annotation[#, -39.6, "Tooltip"]& ], 
           TagBox[
            TooltipBox[LineBox[{5076, 5073, 5389, 5388}],
             RowBox[{"-", "39.6`"}]],
            Annotation[#, -39.6, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4607, 6224, 4608, 6230, 4902, 4903, 4605, 6227, 4606, 
              6229, 4361, 6209, 4593, 5515, 6535, 6536, 4119, 4896, 4592, 
              5490, 5512, 5513, 5027, 6167, 4578, 6155, 4579, 6168, 4889, 
              5175, 5176, 5010, 5009, 4020, 4102, 6886, 6887, 6119, 4553, 
              7191, 6121, 6628, 6981, 6982, 5631, 5630, 4178, 5286, 4179, 
              5632, 6309, 6308, 4686, 7151, 5677, 6670, 7134, 7135, 6780, 
              6781, 6782, 6778, 6779, 5864, 7120, 4177, 5865, 4929, 7015, 
              4753, 7014, 4754, 5284, 5283, 5348, 5349, 4830, 6978, 6979, 
              6980, 5361, 7098, 7099, 7013, 4752, 6859, 6860, 5233, 4632, 
              5362, 5363, 4842, 4002, 4085, 6556, 6413, 6414, 7041, 7040, 
              4781, 5153, 5152, 4873, 6038, 4515, 5437, 5449, 5450, 4975, 
              6013, 4502, 6005, 4503, 4870, 4078, 6854, 4500, 6010, 4501, 
              6012, 4304, 5984, 4490, 5425, 6486, 4865, 4864, 5983, 4489, 
              5412, 5423}],
             RowBox[{"-", "43.2`"}]],
            Annotation[#, -43.2, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{3994, 5438, 5424, 6015, 6014, 4871, 6018, 6019, 6011, 
              6017, 6016, 4977, 4003, 5452, 4976, 4310, 5148, 4309, 6039, 
              6040, 6036, 6862, 6861, 4086, 6505, 6504, 6864, 6866, 6865, 
              6863, 5696, 6690, 5698, 4755, 6388, 6440, 6439, 5151, 4244, 
              4932, 4983, 4982, 6792, 4416, 5849, 5851, 5850, 4399, 5303, 
              4398, 5078, 5080, 5079, 4093, 4010, 4843, 4740, 6671, 6672, 
              4205, 5306, 4206, 6345, 6346, 6343, 4720, 6344, 6266, 6267, 
              6123, 4557, 6120, 4556, 5011, 4339, 6166, 4577, 6164, 4576, 
              6899, 4111, 4026, 6529, 5493, 6170, 6169, 4351, 5185, 4352, 
              5028, 5189, 5188, 5042, 4044, 5533, 5514, 6234, 6233, 4904, 
              6238, 6239, 6228, 6237, 6236}],
             RowBox[{"-", "46.800000000000004`"}]],
            Annotation[#, -46.800000000000004`, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4504, 5149, 4079, 6497, 6496, 5440, 4505, 6020, 4311, 
              6043, 4517, 6041, 4516, 6867, 4087, 4874, 4519, 6037, 4518, 
              6044, 4984, 5465, 5464, 5451, 6078, 6077, 6265, 6389, 6390, 
              5700, 5699, 5697, 4431, 4931, 7139, 4930, 4094, 5577, 4532, 
              6072, 6074, 6073, 6370, 4188, 6997, 4721, 6347, 4722, 5305, 
              5304, 4191, 5307, 5308, 4724, 6348, 4723, 6349, 4190, 5064, 
              4189, 5654, 5653, 4719, 5581, 4103, 4021, 5477, 5013, 4340, 
              5177, 4890, 6172, 6173, 6165, 4581, 6171, 5029, 5517, 5516, 
              5492, 4594, 4897, 4120, 6538, 6537, 5519, 4595, 6210, 4362, 
              6235, 4610, 6231, 4609, 6909, 4127, 5535, 5534}],
             RowBox[{"-", "50.4`"}]],
            Annotation[#, -50.4, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4613, 6232, 6246, 6245, 4905, 6242, 6243, 5518, 5536, 
              4045, 5043, 5190, 5030, 5031, 5187, 5186, 4353, 6174, 6175, 
              5497, 6530, 6531, 4112, 6311, 4891, 4580, 5012, 5494, 5495, 
              4687, 6310, 4692, 6314, 4693, 6315, 4370, 5601, 5852, 6576, 
              4135, 5853, 4134, 5097, 6575, 6574, 6573, 4640, 5241, 5240, 
              4207, 6673, 5678, 6712, 6713, 6714, 5739, 5478, 5873, 4434, 
              5705, 6692, 5704, 7140, 4432, 6798, 5701, 6691, 4217, 5111, 
              4218, 5702, 5703, 6930, 6931, 6929, 6558, 6557, 6874, 4533, 
              6875, 6075, 6080, 6079, 4317, 5154, 5155, 4986, 4985, 4011, 
              4987, 6047, 6048, 6042, 6050, 6049, 4875, 6045, 6046, 5439, 
              5453}],
             RowBox[{"-", "54"}]],
            Annotation[#, -54, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4046, 4128, 6910, 4611, 6240, 4612, 6244, 4363, 5191, 
              4598, 5047, 5521, 4899, 4898, 6213, 4596, 5496, 5520, 4036, 
              5032, 5180, 6313, 4688, 6312, 4689, 5179, 5178, 4180, 5633, 
              4694, 5635, 5636, 5637, 6372, 6371, 5741, 5742, 5740, 5744, 
              5743, 6415, 5331, 5332, 4815, 4917, 4181, 6433, 6434, 6432, 
              6431, 5634, 4691, 6629, 7121, 6630, 7082, 7081, 6632, 6631, 
              7173, 4433, 6799, 5870, 5872, 5871, 4401, 5578, 4400, 5829, 
              4022, 6391, 6081, 6082, 6076, 4536, 6083, 4879, 4880, 4534, 
              4989, 4535, 5161, 4318, 6051, 4520, 5455, 6506, 6507, 4088, 
              5156}],
             RowBox[{"-", "57.6`"}]],
            Annotation[#, -57.6, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4013, 5468, 5454, 6086, 6085, 4881, 5165, 5166, 4991, 
              5163, 4988, 5160, 4095, 4012, 6513, 5467, 6876, 6877, 6084, 
              5706, 6693, 5708, 6393, 6392, 4844, 5081, 4245, 4935, 4329, 
              4997, 4725, 6355, 4726, 6356, 4690, 6353, 6354, 6350, 6352, 
              6351, 6359, 4027, 5499, 5498, 5680, 5679, 4208, 6674, 6358, 
              4729, 6357, 4728, 5237, 4635, 5181, 4583, 5287, 4582, 5033, 
              4354, 6214, 4597, 6211, 6905, 6904, 4121, 4037, 5044, 5538, 
              5046, 5202, 5201, 4906, 6249, 6250, 6241, 6248, 6247}],
             RowBox[{"-", "61.2`"}]],
            Annotation[#, -61.2, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4537, 5162, 4096, 4882, 4539, 4990, 4538, 5164, 4998, 
              5480, 5479, 5466, 6128, 6127, 6268, 6394, 6395, 5710, 5709, 
              5707, 4435, 4934, 7141, 4933, 4104, 5582, 4559, 6124, 4558, 
              6125, 6374, 6373, 4192, 4920, 4730, 5659, 5660, 5661, 5309, 
              4194, 4936, 6653, 6652, 5662, 4417, 5658, 4193, 5065, 5657, 
              5656, 5655, 4727, 5590, 4113, 4028, 5500, 5035, 4355, 5192, 
              4900, 6216, 6217, 6212, 4600, 6215, 5048, 5539, 5537, 5045, 
              4614, 4907, 4129, 5541, 5540}],
             RowBox[{"-", "64.8`"}]],
            Annotation[#, -64.8, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4023, 4999, 5001, 5168, 5167, 4330, 6129, 6130, 6890, 
              6889, 4560, 6888, 6559, 6560, 6932, 6934, 6933, 5712, 5711, 
              4220, 5112, 4219, 6694, 6801, 6800, 4436, 7142, 5713, 6695, 
              5714, 4439, 5876, 5501, 5745, 6715, 5746, 5682, 5681, 4209, 
              5242, 6939, 4641, 6272, 4642, 5099, 5098, 4136, 5100, 5101, 
              4913, 4912, 5315, 4371, 6326, 6327, 6325, 4699, 6319, 7184, 
              4696, 5523, 5522, 5034, 4599, 4901, 6321, 4122, 6540, 6539, 
              5525, 6219, 6218, 4364, 5203, 5204, 5050, 5049, 5207}],
             RowBox[{"-", "68.4`"}]],
            Annotation[#, -68.4, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4617, 5051, 4618, 5208, 4908, 6251, 6252, 5524, 5542, 
              4047, 5053, 5195, 6323, 6324, 6322, 4698, 5194, 5193, 5639, 
              5638, 6320, 4697, 6328, 4700, 7083, 4817, 6634, 6635, 4184, 
              5292, 4937, 6377, 6378, 6376, 4742, 5314, 5313, 4210, 5243, 
              5244, 4741, 6375, 7007, 7006, 4182, 5084, 4183, 6633, 5640, 
              4405, 5835, 4695, 6316, 6318, 6317, 4437, 6802, 4438, 5875, 
              4402, 5583, 5831, 5830, 4029, 6396, 6131, 6132, 6126, 6135, 
              6134, 4885, 5171, 4561, 5000, 5481}],
             RowBox[{"-", "72"}]],
            Annotation[#, -72, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4048, 4130, 5205, 4615, 5058, 4616, 5054, 5543, 5052, 
              5196, 4601, 5290, 5291, 5198, 5197, 5238, 5288, 5289, 4816, 
              5354, 5355, 5351, 5350, 5357, 4211, 6675, 6676, 7090, 7089, 
              4823, 6436, 4822, 5356, 4845, 4846, 6564, 6563, 7172, 4404, 
              6783, 5832, 5834, 5833, 4403, 5086, 4918, 4919, 5103, 5102, 
              5014, 4939, 5874, 4940, 5083, 5082, 4847, 4848, 4756, 6133, 
              4757, 6397, 4562, 6398, 5482, 5483, 4105, 5170}],
             RowBox[{"-", "75.60000000000001`"}]],
            Annotation[#, -75.60000000000001, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4031, 4850, 4758, 4944, 5715, 5716, 5172, 4221, 4941, 
              5877, 4942, 5115, 5114, 4246, 4938, 5502, 6176, 4341, 5015, 
              5664, 4195, 5085, 6655, 6654, 6793, 4418, 6795, 6794, 7137, 
              7136, 6660, 5667, 6716, 6717, 4249, 5748, 4248, 5747, 6363, 
              6362, 6658, 6657, 6656, 7085, 7084, 4732, 6270, 6935, 6936, 
              6269, 5597, 5527, 5526, 5057, 5200, 5199, 4365, 5206, 4366, 
              5218, 5060, 5216, 5215}],
             RowBox[{"-", "79.2`"}]],
            Annotation[#, -79.2, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4621, 5059, 4622, 5217, 4909, 5211, 5212, 5056, 6329, 
              5055, 4038, 4123, 6998, 6999, 6360, 4731, 6361, 4704, 5296, 
              6659, 5666, 4733, 6364, 4784, 5854, 4419, 5663, 5665, 6678, 
              7125, 6679, 7009, 7008, 6680, 7147, 5684, 6900, 6901, 6177, 
              5503, 5592, 7107, 4030, 4114, 5591, 7164, 7165, 7196, 7195, 
              5113, 5319, 5318, 4440, 5879, 4442, 5878, 4441, 5016, 4342, 
              4945, 4247, 5317, 4849}],
             RowBox[{"-", "82.8`"}]],
            Annotation[#, -82.8, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4049, 4131, 5209, 4619, 5294, 4620, 6330, 6331, 6986, 
              6985, 4701, 6332, 4706, 5646, 6642, 5297, 4705, 6333, 4707, 
              6987, 6988, 6380, 6379, 7010, 5839, 7186, 5840, 5838, 4410, 
              5841, 6677, 5683, 4425, 5866, 4250, 5749, 6720, 6719, 6718, 
              5717, 6696, 4223, 5320, 5321, 4760, 6399, 4759, 7016, 4222, 
              5089, 6179, 6180, 6178, 4584, 6181, 5593, 5594, 4943}],
             RowBox[{"-", "86.4`"}]],
            Annotation[#, -86.4, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4040, 5087, 4406, 5182, 4407, 5881, 5882, 5880, 6803, 
              4443, 7174, 6638, 6639, 6983, 6984, 6637, 7122, 6636, 4411, 
              5644, 6640, 6641, 5091, 4186, 5643, 5642, 5837, 5836, 6271, 
              5753, 5754, 5750, 5752, 5751, 6381, 4743, 5648, 5647, 5645, 
              4708, 5641, 4185, 5213, 5214, 4703, 5210, 4702, 5293, 4950}],
             RowBox[{"-", "90"}]],
            Annotation[#, -90, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4259, 5295, 4260, 4951, 5062, 5061, 5859, 4422, 5858, 
              5861, 5860, 4213, 4949, 4212, 6681, 5685, 4426, 4851, 4039, 
              4124, 5093, 5094, 5090, 4408, 5104, 4409, 5856, 7185, 5857, 
              5855, 4421, 6796, 5036, 5037, 4947, 4252, 5088, 4852}],
             RowBox[{"-", "93.60000000000001`"}]],
            Annotation[#, -93.60000000000001, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4251, 4946, 5528, 6406, 4356, 5038, 5670, 4196, 5092, 
              5669, 5668, 6365, 4420, 5105, 6662, 5673, 6721, 6722, 4258, 
              5760, 4257, 5759, 5868, 5867, 4197, 5671, 5063, 4367, 5311, 
              4910, 4856, 4855}],
             RowBox[{"-", "97.2`"}]],
            Annotation[#, -97.2, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4050, 4132, 7000, 7001, 6366, 4734, 6368, 4673, 5266, 
              6661, 5672, 4735, 6369, 4785, 5245, 4643, 5310, 7087, 7088, 
              6435, 7183, 4818, 7086, 6906, 7154, 7155, 7037, 7038, 7039, 
              6407, 6408, 5598, 6541, 6567, 6568, 4125, 6566}],
             RowBox[{"-", "100.8`"}]],
            Annotation[#, -100.8, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4790, 6367, 4791, 6419, 4674, 6293, 6294, 6292, 6296, 
              6295, 6966, 6288, 7189, 6289, 6287, 6291, 7188, 6290, 4602, 
              6253, 4623, 5529, 5544}],
             RowBox[{"-", "104.4`"}]],
            Annotation[#, -104.4, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4160, 6609, 6610, 6968, 6967, 4678, 5265, 5264, 4172, 
              5267, 5268, 4679, 6297, 6299, 6298, 6300}],
             RowBox[{"-", "108"}]],
            Annotation[#, -108, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4173, 6623, 6766, 6765, 4911, 6914, 6915, 6911, 6913, 
              6912}],
             RowBox[{"-", "111.60000000000001`"}]],
            Annotation[#, -111.60000000000001`, "Tooltip"]& ]}, {}, {}}}],
       AspectRatio->1,
       BaseStyle->{FontFamily -> "Times", FontSize -> 12},
       DisplayFunction:>Identity,
       Frame->True,
       PlotLabel->FormBox[
        "\"\[Mu]=8 \[Alpha]=0.28 \[Epsilon]=10 PIN=0.10\"", TraditionalForm],
       PlotRange->{{0, 50}, {0, 50}},
       PlotRangeClipping->True,
       PlotRangePadding->{
         Scaled[0.02], 
         Scaled[0.02]}], {192., -576.}, ImageScaled[{0.5, 0.5}], {360., 360.}]
      , InsetBox[
      GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyUvXd8VcXzPxx6VXoJBEgIAQIBkpBCCEnm5tbQDEWQKiBIR3qXIjX0LiBK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        "], {{
          {RGBColor[0.293416, 0.0574044, 0.529412], EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{2266, 677, 642}, {5919, 4389, 642}, {961, 642, 
              225}, {5922, 5923, 2267}, {961, 2267, 642}, {2620, 2267, 961}, {
              642, 677, 225}, {2266, 3065, 677}, {4389, 5918, 642}, {5921, 
              5920, 2266}},
              VertexColors->None], 
             PolygonBox[{{5920, 4204, 3065, 2266}, {6330, 5922, 2267, 2620}, {
              5923, 5919, 642, 2267}, {5918, 5921, 2266, 642}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.3176422608430269, 0.09952732494083806, 0.5634649403385614], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{6758, 6759, 3064}, {2264, 2060, 354}, {5919, 5923, 
              2265}, {2264, 3184, 353}, {4204, 5920, 354}, {4194, 5297, 
              302}, {3184, 2265, 353}, {5723, 6747, 2060}, {7062, 6758, 
              3064}, {5730, 5723, 2060}, {6760, 7062, 3064}, {2060, 2264, 
              353}, {5297, 5319, 302}, {2062, 2265, 358}, {2062, 358, 303}, {
              354, 2060, 302}, {2265, 2062, 353}, {5360, 4196, 303}, {4195, 
              5360, 303}, {5724, 5732, 2062}, {5725, 5724, 2062}, {5922, 6330,
               358}, {5921, 5918, 2264}},
              VertexColors->None], 
             PolygonBox[{{6759, 5729, 353, 3064}, {5731, 6760, 3064, 353}, {
              4389, 5919, 2265, 3184}, {6747, 4194, 302, 2060}, {5732, 5731, 
              353, 2062}, {4196, 5725, 2062, 303}, {5729, 5730, 2060, 353}, {
              5923, 5922, 358, 2265}, {5918, 4389, 3184, 2264}, {5920, 5921, 
              2264, 354}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.3448768776551417, 0.14688096846025891`, 0.6017464889215516], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{5730, 5729, 2068}, {3402, 2676, 1049}, {2668, 3400, 
              2068}, {3399, 3400, 1047}, {3403, 3662, 1059}, {3401, 3403, 
              2069}, {6458, 4852, 1241}, {2666, 2668, 1059}, {2670, 2063, 
              1049}, {3406, 3661, 2676}, {1757, 2061, 1241}, {2667, 1709, 
              1047}, {1724, 1709, 960}, {6379, 6378, 2667}, {3050, 3399, 
              1047}, {3661, 3406, 1059}, {3404, 2670, 1049}, {6381, 6382, 
              2671}, {2671, 3402, 1049}, {5615, 5641, 960}, {1757, 2671, 
              1049}, {3400, 2668, 1047}, {3662, 3403, 3401}, {2668, 2068, 
              1059}, {3401, 3404, 1049}, {3661, 3662, 3401}, {3662, 3661, 
              1059}, {6387, 6386, 2676}, {2670, 3404, 2069}, {6506, 4726, 
              1757}, {4670, 5615, 960}, {4655, 6329, 1709}, {4194, 6747, 
              3050}, {5725, 4196, 2063}, {3406, 2666, 1059}, {3404, 3401, 
              2069}, {3661, 3401, 2676}, {4851, 6458, 1241}, {2063, 1757, 
              1049}, {2668, 2666, 1047}, {5731, 5732, 2069}, {1709, 3050, 
              1047}, {2676, 3401, 1049}, {5729, 6759, 2068}, {3400, 3399, 
              2068}, {6760, 5731, 2069}, {6376, 6377, 2666}},
              VertexColors->None], 
             PolygonBox[{{7065, 6376, 2666, 3406}, {6375, 6379, 2667, 1047}, {
              4852, 6506, 1757, 1241}, {6382, 6380, 3402, 2671}, {6386, 7065, 
              3406, 2676}, {6329, 4670, 960, 1709}, {4726, 6381, 2671, 
              1757}, {6758, 7062, 3403, 1059}, {5723, 5730, 2068, 3399}, {
              6759, 6758, 1059, 2068}, {5360, 4195, 2061, 1757}, {6377, 6375, 
              1047, 2666}, {5297, 4194, 3050, 1709}, {7062, 6760, 2069, 
              3403}, {6380, 6387, 2676, 3402}, {4196, 5360, 1757, 2063}, {
              6747, 5723, 3399, 3050}, {6378, 4655, 1709, 2667}, {5732, 5724, 
              2670, 2069}, {5724, 5725, 2063, 2670}, {5319, 5297, 1709, 
              1724}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.3721114944672565, 0.19423461197967976`, 0.6400280375045418], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{2978, 3012, 210}, {5772, 4783, 1133}, {6459, 4840, 
              1229}, {6380, 6382, 2669}, {4655, 6378, 942}, {6387, 6380, 
              2669}, {6711, 5700, 3011}, {6379, 6375, 2665}, {2040, 2978, 
              1974}, {2809, 2744, 1048}, {6375, 6377, 2665}, {6376, 7065, 
              234}, {4688, 5406, 998}, {4670, 6329, 2619}, {4091, 4176, 
              224}, {5358, 6783, 1756}, {2745, 1756, 1048}, {7157, 4873, 
              1273}, {5701, 5702, 2040}, {6519, 4827, 2784}, {3831, 2784, 
              942}, {6518, 4874, 1273}, {6506, 4852, 2809}, {6381, 4726, 
              1048}, {1756, 1133, 1048}, {3012, 2978, 2040}, {2744, 2745, 
              1048}, {1990, 2978, 210}, {7156, 7158, 3503}, {1133, 1229, 
              234}, {3503, 3832, 3502}, {2978, 1990, 1974}, {3831, 3832, 
              1273}, {1133, 2669, 1048}, {7065, 6386, 234}, {5702, 6679, 
              2040}, {3441, 2744, 224}, {2784, 1974, 942}, {3503, 3502, 
              998}, {1974, 2619, 942}, {3502, 3831, 942}, {2665, 3502, 942}, {
              5359, 5358, 1756}, {3832, 3503, 1273}, {2669, 1133, 234}, {1229,
               998, 234}, {6782, 5772, 1133}, {4840, 5337, 1229}, {998, 2665, 
              234}, {3832, 3831, 3502}, {5406, 4875, 998}, {4873, 6518, 
              1273}, {3502, 2665, 998}, {7100, 4781, 3441}, {2744, 2808, 
              224}, {2784, 3831, 1273}},
              VertexColors->None], 
             PolygonBox[{{7158, 7157, 1273, 3503}, {6386, 6387, 2669, 234}, {
              4782, 5359, 1756, 2745}, {6329, 4655, 942, 2619}, {5615, 4670, 
              2619, 1974}, {6457, 4782, 2745, 2744}, {4726, 6506, 2809, 
              1048}, {6783, 6782, 1133, 1756}, {4781, 6457, 2744, 3441}, {
              4852, 6458, 2744, 2809}, {6382, 6381, 1048, 2669}, {6377, 6376, 
              234, 2665}, {6679, 6712, 3012, 2040}, {4168, 6711, 3011, 210}, {
              4783, 6459, 1229, 1133}, {4827, 5616, 1974, 2784}, {5641, 5615, 
              1974, 1990}, {5616, 5701, 2040, 1974}, {5337, 4688, 998, 
              1229}, {6712, 4168, 210, 3012}, {6378, 6379, 2665, 942}, {6458, 
              4851, 2808, 2744}, {4875, 7156, 3503, 998}, {4874, 6519, 2784, 
              1273}, {4176, 7100, 3441, 224}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.39934611127937153`, 0.241588255499101, 0.6783095860875324], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{1274, 1131, 421}, {1793, 1738, 1131}, {6459, 4783, 
              2746}, {2115, 3096, 423}, {2187, 3156, 2115}, {6836, 5773, 
              2115}, {4782, 6457, 1132}, {5701, 5616, 1209}, {5047, 4282, 
              1507}, {2186, 2187, 2114}, {2187, 2115, 2114}, {2323, 2186, 
              422}, {4934, 4935, 1402}, {2783, 1973, 1209}, {6457, 4781, 
              1132}, {2114, 3095, 422}, {7100, 4176, 1467}, {2825, 1274, 
              421}, {1131, 1738, 423}, {1467, 1794, 1132}, {1591, 2783, 
              1209}, {2825, 3504, 1274}, {1132, 1604, 422}, {4688, 5337, 
              1738}, {2039, 2977, 1973}, {4168, 6712, 2977}, {7157, 7158, 
              3504}, {1794, 1604, 1132}, {6782, 6783, 3095}, {3096, 1633, 
              423}, {5616, 4827, 1209}, {1274, 1793, 1131}, {5770, 5145, 
              1591}, {2039, 1973, 1507}, {4287, 6837, 3156}, {4388, 5170, 
              1604}, {1738, 2798, 423}, {5145, 4378, 1591}, {2746, 2114, 
              423}, {5359, 4782, 1132}, {2186, 2114, 422}, {2824, 2825, 
              421}, {6519, 4874, 2826}, {2824, 1591, 1209}, {7156, 4875, 
              1274}, {5850, 4286, 2186}, {1591, 2824, 421}, {1633, 1131, 
              423}, {2826, 2824, 1209}, {2743, 2111, 1131}, {5768, 5769, 
              2111}, {2114, 2115, 423}, {1131, 2111, 421}, {5187, 4232, 
              421}, {5204, 4443, 1633}, {2798, 2746, 423}, {6712, 6679, 
              2977}, {5170, 5771, 1604}, {5982, 4445, 2323}, {6711, 4168, 
              2977}, {1633, 2743, 1131}},
              VertexColors->None], 
             PolygonBox[{{4874, 6518, 2824, 2826}, {4995, 5407, 1794, 1467}, {
              4827, 6519, 2826, 1209}, {4840, 6459, 2746, 2798}, {6518, 4873, 
              2825, 2824}, {4286, 5851, 2187, 2186}, {5702, 5701, 1209, 
              1973}, {6784, 5204, 1633, 3096}, {5851, 4287, 3156, 2187}, {
              4176, 4091, 1402, 1467}, {5614, 5047, 1507, 1973}, {5772, 6782, 
              3095, 2114}, {6783, 5358, 422, 3095}, {4781, 7100, 1467, 
              1132}, {4233, 5982, 2323, 422}, {5407, 4388, 1604, 1794}, {4875,
               5406, 1793, 1274}, {4378, 6489, 2783, 1591}, {4443, 6456, 2743,
               1633}, {5406, 4688, 1738, 1793}, {6456, 5768, 2111, 2743}, {
              4935, 4995, 1467, 1402}, {6489, 5614, 1973, 2783}, {5771, 4233, 
              422, 1604}, {5769, 5187, 421, 2111}, {5773, 6784, 3096, 2115}, {
              5358, 5359, 1132, 422}, {6679, 5702, 1973, 2977}, {6837, 6836, 
              2115, 3156}, {5700, 6711, 2977, 2039}, {4232, 5770, 1591, 
              421}, {4783, 5772, 2114, 2746}, {4873, 7157, 3504, 2825}, {4445,
               5850, 2186, 2323}, {5337, 4840, 2798, 1738}, {7158, 7156, 1274,
               3504}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.42658072809148634`, 0.28894189901852185`, 0.7165911346705226], 
           EdgeForm[None], 
           GraphicsGroupBox[{
             PolygonBox[{{5407, 4995, 641}, {3506, 1284, 522}, {5982, 4233, 
              2113}, {4995, 4935, 641}, {5770, 4232, 2112}, {1603, 1635, 
              721}, {5207, 5208, 1635}, {2821, 2819, 721}, {4233, 5771, 
              2113}, {5049, 5185, 1508}, {1623, 719, 697}, {2975, 2976, 
              209}, {7272, 4896, 3505}, {5206, 5205, 1634}, {730, 3506, 
              522}, {5189, 5009, 697}, {5184, 4932, 517}, {5048, 5987, 
              1508}, {720, 1590, 630}, {6521, 4250, 447}, {6836, 6837, 
              3155}, {6514, 6516, 2819}, {3505, 2976, 1284}, {2976, 3698, 
              209}, {722, 2975, 209}, {7162, 7160, 3507}, {2976, 2975, 
              1284}, {6710, 5954, 2302}, {730, 2113, 721}, {7242, 7243, 
              3698}, {5047, 5614, 630}, {2975, 3155, 522}, {3505, 3699, 
              2976}, {4388, 5407, 641}, {3507, 3506, 447}, {2819, 730, 721}, {
              3699, 3698, 2976}, {719, 2302, 697}, {1590, 1508, 630}, {1603, 
              721, 641}, {5614, 6489, 630}, {4167, 6709, 209}, {3506, 730, 
              447}, {3507, 3509, 1284}, {5987, 5049, 1508}, {2112, 1623, 
              720}, {6489, 4378, 630}, {4288, 5382, 523}, {4879, 7163, 
              3509}, {5850, 4445, 730}, {1634, 1590, 720}, {2302, 719, 209}, {
              4287, 5851, 522}, {5851, 4286, 522}, {4250, 7161, 447}, {3506, 
              3507, 1284}, {6784, 5773, 722}, {719, 722, 209}, {5768, 6456, 
              719}, {7159, 7272, 3505}, {1284, 2975, 522}, {1515, 1603, 
              641}, {6515, 6521, 447}, {6456, 4443, 719}, {3155, 2975, 722}, {
              5056, 5055, 1515}, {720, 1623, 697}, {1515, 641, 523}, {5204, 
              6784, 722}, {5955, 4431, 697}, {730, 2819, 447}, {5187, 5769, 
              1623}, {630, 1508, 517}, {4431, 5189, 697}, {5954, 5956, 
              2302}, {5383, 5056, 1515}, {1635, 2821, 721}},
              VertexColors->None], 
             PolygonBox[{{5169, 5207, 1635, 1603}, {7160, 4879, 3509, 3507}, {
              4443, 5204, 722, 719}, {4878, 7159, 3505, 1284}, {5055, 5169, 
              1603, 1515}, {5144, 5048, 1508, 1590}, {7243, 4167, 209, 
              3698}, {4232, 5187, 1623, 2112}, {5771, 5170, 721, 2113}, {4896,
               7244, 3699, 3505}, {5956, 5955, 697, 2302}, {5010, 5206, 1634, 
              720}, {5382, 5383, 1515, 523}, {6516, 6515, 447, 2819}, {4282, 
              5047, 630, 517}, {4445, 5982, 2113, 730}, {4286, 5850, 730, 
              522}, {7161, 7162, 3507, 447}, {5769, 5768, 719, 1623}, {4935, 
              4934, 523, 641}, {5009, 5010, 720, 697}, {5208, 4869, 2821, 
              1635}, {5185, 5184, 517, 1508}, {5145, 5770, 2112, 720}, {7244, 
              7242, 3698, 3699}, {6709, 6710, 2302, 209}, {5773, 6836, 3155, 
              722}, {5205, 5144, 1590, 1634}, {4869, 6514, 2819, 2821}, {5170,
               4388, 641, 721}, {7163, 4878, 1284, 3509}, {6837, 4287, 522, 
              3155}, {4378, 5145, 720, 630}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.4538153449036013, 0.3362955425379431, 0.7548726832535132], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1lLluU2EQhSfgJV5isB3bXNsS0IaKsNRJiIQcEAikxHaLKICCXWwFL0CA
BEJBCy0dFFGEQESh4Bl4AxoEEgI7SOF8zLgY3bn/Pf/MmTMz9+CFa+ev7jKz
K7LdskHJrD9u9lUvmbRZT5YS4EHd7FzZ7L38k3vNTrXNNgtm61WzfZNmj8bM
XgvzStbT/ft6PhTukPx78u/KpuR3hT0sWxb+V8ZsWvHPpszWimafCp73rfxm
ziyR/dWdO8KsK++M7s3KFnUvq7N03C3qXqrg7+8m/Bvcj8YT/i8i/ob85YpZ
OVEduvu7ZVaqOraSc75g4NMNvyPc95Rz3iPsn5bH/a9FymvPRH6w1NkJ/JsJ
/8b7TeVMVz0WWDjjUyf1UuOS6nsmzJa+Z2u6K/uhep/kXRewfb0vZTw/+n3O
e41F9aQg+6KcP+FY9fzk7E16rqHOBy3X6XjT7FjTdVmRJlvCLKCnchZq3p/T
indA94+knRM5R/VmC64//aEn8H+pns8rzocx70MxdMEygSffZvR6TtgbTa8H
7XluRF296C/ztRgzRq3MJ/Veb/pcwolYnJP3sub0YsM1Ze6qifeEnmUjJvXl
215jvua8uVtLXAt0+Cbs87z79IB3/G3tyLDkmtwS/nbivRzEOdzQhtjkAsc3
OFaEvVT2fsJ3EHjmiDNqfKr8rcRnhnjkYz5XVO+qbEpzsDPuM41WxOEM/gtt
7xk5Z6XBTMN3CP3pPzGZD3RE47ryNBKvsxf7MurRWsRnD/DhD0dmEJ6dhu8S
3Nmnx+GfUX1zdZ+H/ZqdXNtzMderMdsndHe+4XUwC/AlPru43fJ6mYWd2MFh
aIBW8JwOH27s7YgnMdCBHP3YEf43H6u+j8QZRO/4T3VjZv4BdN+YhA==
              "],
              VertexColors->None], 
             PolygonBox[{{7160, 7162, 3508, 1286}, {6427, 7154, 3500, 1097}, {
              5698, 5953, 2300, 696}, {4749, 6418, 2702, 1095}, {4433, 5961, 
              1095, 700}, {5794, 5793, 2135, 2136}, {4431, 5955, 2303, 
              1622}, {6769, 6768, 700, 1622}, {5049, 5987, 2329, 696}, {5793, 
              4444, 2322, 2135}, {6517, 4871, 2823, 2822}, {6801, 5386, 446, 
              3114}, {5382, 4288, 1514, 1777}, {4456, 6918, 3251, 2702}, {
              6521, 6515, 2820, 2828}, {7179, 4792, 2757, 3529}, {7177, 7180, 
              3529, 1330}, {4250, 6521, 2828, 1287}, {6709, 4167, 3010, 
              3009}, {5956, 5954, 3221, 2303}, {5796, 6800, 3114, 2137}, {
              4444, 5753, 2091, 2322}, {5188, 5795, 2136, 446}, {5952, 4083, 
              1398, 2300}, {4871, 5388, 1781, 2823}, {6516, 6514, 1263, 
              2820}, {5316, 5315, 396, 1098}, {4896, 7272, 3835, 1331}, {6918,
               4457, 2330, 3251}, {6423, 6421, 1098, 2707}, {5054, 6422, 2707,
               1514}, {7163, 4879, 1286, 1332}, {4870, 6517, 2822, 3500}, {
              4792, 5796, 2137, 2757}, {5055, 5056, 1098, 396}, {5205, 5206, 
              1476, 395}, {5954, 6710, 3009, 3221}, {5208, 5207, 396, 1263}, {
              7161, 4250, 1287, 3508}, {5987, 5048, 395, 2329}, {5315, 5752, 
              2090, 396}, {6419, 4751, 1097, 2090}, {7243, 7242, 1331, 
              3010}, {5988, 4430, 696, 2330}, {5185, 5049, 696, 1621}, {5009, 
              5189, 1625, 1476}, {5056, 5383, 1777, 1098}, {7178, 4895, 1330, 
              1781}, {4932, 5184, 1621, 1398}, {4222, 5186, 1622, 2091}, {
              5189, 4431, 1622, 1625}, {7272, 7159, 1332, 3835}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.4810499617157162, 0.3836491860573639, 0.7931542318365034], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVM9LVGEUver0ZubNDM08Z970RiFaV1s3bVL6YYukZhEtzCJaVKQ1i8A2
LQJDwdGyhIKKtJat+gGJYYKW/VgXtYqgPyAaXQqdw7ni4uP73v3uj3PPPd/b
c/5qfaTdzEawOrCGCmarodlYyuxZbHaobHY6MDtSNbvSbdaWNbtWM2tg5fJm
t+H3ETHHse/LyNZsM3uJ+2XEN2Dbj9VCjinYF1HsR7t8FnH/CH7DuG9ifxrr
7gxqDWKVUOsY9pMlYMuq/tGqMHyB/1esy6iXhz2LtVZQHeJhrdZOs4PwXzHZ
1tweJ2ZLnWa/YJ9Gr5NYf1B3OWf2e4f8+9D3zaJwDqTUA2OZ45PnmYnMaolq
3gGW57H4IFZy8R0+e5EnBftSh2KJgz7zyP0O9m/wmUX9h6HqMK7hsfdhmwkV
uxDom2fOaD0Q59PA0AkMdz2eez/2x8DTA0568srHnTnHcZ7IqM5f+D3wuABY
piLFjuK8UlbNsIKcNdXjbBhPTvpQszeRVsgdZ8hedsF2L1JPC+jxFWLredWs
O5b3sWbPmHNlccHYzYxy8RwhTzPSfC5h/kGX7oiD/LH3n+3SE/H8w31Xon6I
abQgDeeBP92pXqkVzngjLU3znvHk6UlNHI07N8w5iLpzseqksWewPqRlm3c7
85zKKNeNWDG0E+/FkjDzjpplLWqaNcglOV51ns/6u2NfA4i7UBUfxH49EX7q
tBlqjjwPO1+34HMgkjbe4Py6KB1XEFdNpCFq820k7mZ95px1f1Vvj3h4n+ve
ftMt1xg53dIXeWUvY6nt/tnXhOt2S1f05Ruhrvl2+P0CnCSJep0ravacNXGQ
A+LPVvSuWs7rus+Jb/2zv/fDwHyi5LMP9d7ISS/sfbHmuxs8hN16q8wx5Hog
Z5POIfke8n8XOQ5cJ+SS75Z4mGvTtc0z9cP/w38RAL4V
              "],
              VertexColors->None], 
             PolygonBox[{{5316, 4752, 2708, 1721}, {5755, 5385, 1779, 2093}, {
              6419, 5751, 1096, 2705}, {6708, 5045, 1505, 3008}, {5640, 4175, 
              223, 1989}, {6423, 6422, 223, 2706}, {4748, 5961, 2307, 2701}, {
              4755, 6428, 2713, 2709}, {6427, 4751, 2705, 2712}, {7177, 4895, 
              3528, 3527}, {7179, 7180, 3527, 1158}, {6769, 5186, 397, 
              3083}, {6835, 6834, 2161, 3154}, {4432, 5960, 728, 699}, {4750, 
              6424, 2709, 1096}, {5754, 6768, 3083, 2092}, {4249, 5386, 1780, 
              2134}, {4175, 6420, 2706, 223}, {5988, 4457, 740, 2301}, {7178, 
              5387, 398, 3528}, {4433, 5754, 2092, 2307}, {6420, 4994, 958, 
              2706}, {5822, 5823, 2157, 470}, {5952, 5953, 2037, 195}, {5794, 
              5795, 1624, 729}, {5959, 5957, 699, 2305}, {5387, 5388, 1264, 
              398}, {6814, 5822, 470, 3126}, {6465, 7106, 3448, 2756}, {4749, 
              6417, 2700, 2703}, {6426, 4756, 1101, 1779}, {7106, 5958, 2305, 
              3448}, {5751, 5752, 1721, 1096}, {4222, 5753, 729, 397}, {5792, 
              5981, 2321, 728}, {5821, 6858, 3178, 2037}, {5824, 6813, 3126, 
              2161}, {4870, 7154, 2712, 1264}, {6857, 6708, 3008, 3178}, {
              6425, 6466, 2756, 3423}, {6428, 5755, 2093, 2713}, {4456, 6418, 
              2703, 740}, {6812, 4257, 469, 2157}, {5980, 4280, 2184, 2321}, {
              4280, 5849, 2185, 2184}, {5849, 4281, 3154, 2185}, {4756, 7084, 
              3423, 1101}, {5636, 5638, 1096, 958}, {5820, 5821, 2037, 469}, {
              5188, 4249, 2134, 1624}, {6801, 6800, 1158, 1780}, {6417, 4748, 
              2701, 2700}, {5698, 4430, 2301, 2037}, {4752, 6421, 2706, 
              2708}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5082845785278309, 0.43100282957678476`, 0.8314357804194936], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VM9LVGEUvU7+mBGdGd88n743s2lT1qqFEkkuMrKgoKm0RSTkFAxMNbZr
UZuSWkpQ1KaidgWiqUhmmWURFArZrm0REu3qD6hzOLfFx/ve990f5557vru1
Mn68njKzCawtWBebzS5gfcNhutPsamJWazR7G5qtYi3ibjFvdroL9rBZw12Q
NfsJ31utZu9bzD5jX24z+9UOH+x3wW8gMFtBgmzOrIr1DrY9iLWeNvue0hnv
LGN2rggc3WZ/sT/SqLiMOQzb7Yi73GA2g+9sm3Ceb1OcWkpxd7jNlcisFCtG
ARjzOWHN5bRnrt243wxUB2OnMvJlzXcR6zC+1ZLZc9j04P6Z572EfJuIMZBX
jRVgPos1gvOdqPd3TpiZ/3YgDIx3z2OS4z7H/x8TOXyMePcT3THXtNv8adee
ue4gXhIr1gLst6FPDbA5g55UO8Qn/Wfcd6ooH9qTh6rX3os8HxPx9gnfZfBV
T4tLcso97YlvHTZhVv/cs78b2P9ICRf3xDYKDI8i9Ypx2GPGXygoX817zTjs
1VHgT8N+qQlf8JYJ5UP+hvLicBC1xrH4Y637C6q3AtuxUJpl/i+Oh/uyc8X+
jJS0J5/9bkONDGbVrzFgrmB1Z4QrdGx5x0mu2PvQa6dGqFXqhH2kHznvBsbJ
QHotIudoqL49aREevpc9OO8vKTfPjvk5cdCOMflueMd/YmZd0/7lP/EPuz19
b4K/qUCaJc7L7cL/FGd9wDSP/XVwFsbieQX2ryNpeAh1rxalvVPQzkSkGmdd
P7ThDGAOxs/62yVO+u2L5cuc5JR342nxQk7YM/rRhvesk77kssft51qFke+C
GnzlOpxD3vlEHLHWYfelruo+Nw4A/8Eu9YhvbM7jvAS23lj/7GPO335TUTqj
xvYGmk2cS/3OLes9WVL/2DtqcKkoHRI7ZxKxkAfqgHnJ2Q3nrRnfxkhzkBjH
HSe5p9bJf6ZTs4l4H6C+N5F6Outvnm/2A3BdK8ie/K05h9TAhuthMtFsIo+H
wEG5Q5jIV9l18gI5v0bSCetnHOLkfK37jOX8O9GlO+riYSIe6MdZT1/ySLzk
4R9af9FH
              "],
              VertexColors->None], 
             PolygonBox[{{6857, 6859, 3179, 618}, {6835, 4281, 515, 3153}, {
              5980, 5981, 2132, 515}, {7011, 7012, 2559, 2133}, {7098, 7099, 
              2738, 208}, {7226, 7010, 2133, 3650}, {6812, 5819, 3247, 
              3125}, {5129, 5046, 618, 1576}, {4753, 5384, 1130, 1099}, {6254,
               5697, 1960, 2559}, {4780, 6455, 1228, 1130}, {5819, 5823, 2160,
               3247}, {4276, 5045, 618, 513}, {5046, 4277, 513, 618}, {5213, 
              6916, 3246, 3571}, {5214, 5218, 1645, 738}, {4667, 4994, 1466, 
              1720}, {5216, 4257, 3125, 1644}, {6453, 4753, 1099, 2739}, {
              7105, 6799, 3113, 3447}, {5792, 6797, 3111, 2132}, {6797, 5960, 
              2306, 3111}, {5971, 4442, 717, 1228}, {5820, 4258, 2159, 
              2158}, {6916, 5214, 738, 3246}, {4454, 5985, 2328, 1644}, {5765,
               6798, 3112, 717}, {4755, 6424, 2704, 1100}, {7192, 4071, 194, 
              3563}, {5637, 5162, 208, 1988}, {4791, 7105, 3447, 3112}, {6425,
               7084, 2711, 2710}, {4455, 6917, 1576, 739}, {5959, 7276, 3855, 
              2304}, {5697, 7192, 3563, 1960}, {4993, 5313, 1720, 1466}, {
              6452, 4777, 2739, 2738}, {5986, 5984, 739, 2328}, {4754, 5385, 
              1100, 1099}, {4432, 5957, 2304, 2306}, {6465, 6466, 2710, 
              233}, {6426, 4754, 1099, 2711}, {7253, 7254, 3571, 194}, {4258, 
              5216, 1644, 2159}, {7222, 6813, 741, 3642}, {5640, 5639, 959, 
              1466}, {5824, 6834, 3153, 741}, {6799, 7225, 3650, 3113}, {4669,
               5318, 1466, 959}, {7276, 5958, 233, 3855}, {4750, 5638, 1988, 
              2704}, {5217, 4454, 1644, 1645}, {6814, 7222, 3642, 2160}, {
              6859, 6858, 2158, 3179}},
              VertexColors->None], 
             PolygonBox[{{5314, 5164, 5163, 4667, 1720}, {5164, 5637, 1988, 
              5636, 5163}},
              VertexColors->None]}]}, 
          {RGBColor[0.535519195339946, 0.478356473096206, 0.8697173290024841],
            EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVE1IVGEUvZbj6Mwbm/fmx5/cSIsgwjZamE1gELQL+sOCIqdISppxE9TK
CnOgRdEmXNofURmo2Qi6KCPatUpoJyZCm4RIaVd0TudGi8v33vfuPffce899
7cXykdImMxuGbYb1NZqtwkZqzXrqzbphD2vMHmTNPkZm47ifrzM7ALuOwA7c
78rqfQLW6/cFj32M2PmtZnOwyymzckrnz7hZPpAffUZx7oM9qhGHHs+7FDMb
BOYXYFYT8qE/z/3+/BQ4TxwrwvkD/gPgWQCvSlo4/U1mF0LhMI65GfsMNXW2
yP8mzpmM2TJyFhF7BnapTu9NDYqN5c12ZlVnBhjZQJxPAfsociQaVDdP4u/I
Kob+tejBacckl/6sMMfBcRzPizXKP5dXTtaYc/w3+L4tVG/Zq4r3azoQN8Zu
B+5vPI9hkIvI9Ql2G71ewDkbaabdbWZ3W/XMPkWOv4553E+ilzinwGE6rd6/
TKnOi/BfB0595v9d5Pd70LeJSPOp+FzIrQj/vcgXwD8O/slQHA/Bdxjf2nF/
MKGTOCV/5/NuYD6H30xCNVyJSXvMmXHOQ64nxvSiZ6Vmsw3wv4e4rhbVR0zG
cL6T4NARSafU8rEWYSZT4sW+TaI3lbxqqM8JizjMn4zLn7NdS2i+nHUYSGM5
1x65XQXOtVZ9o17euWa4W9Q3c37AXSHSPt2B7628dE8dU8/EZv2jvhfsI+PI
8wT6ehw2BqzvjsmZVjGfBuCuxOTLe3KjLjhbxrInQ943Wtn3sRN4XbDuRuma
uZmXu8I78vq3w+wndUzdUCdlx+T9AvLfyEirg74/1PksNDWFOg8H6iM5Urev
0+LNd+qH39n/r1vMmts00/PQyzlYKqU7zpv3fV4/+0ZdU+vsA3eX+0aOr9La
bfI4ix0tNmln+G3JuXH+7Bd79Q1zX4PNxFVTyev6BfyTof433BXmZ0+pixXX
57rfEfc99Pgikp45d86P3D6DezWtufLf+db/n+S07D2hZku+FzHUMRBq14m9
4XMk3qpriXqsuiapndD1M5LX7v3Vd6N4kyP7GLiW/gCrY8kf
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtU8tL1FEYPdmo48zPnPc4j40EBUHSQgvLCWoo2gW9oCDKKRh6aKugJxWZ
Ay1ahsspCKkM1ExhXPTuH0goWkS1sUVBaLTpQedwXHxw7/1e55zvu12VM3uG
mgCUaSFaIwPUY8D8CuBTM9CWAsIp37/wfp3+Q3Eg0gb0hYEHCeAd30Z4no4A
lSwwQPvMolnGrEkDT2LOjbYCNZ6HGb+2BbjLmh/on0wCH+kfo7/OXndo9ZB7
9haBP3mgxPp9q4DTnc5R7D/WX+wA1jF+G+tN8+01z9207bw3aC94fk672uSY
y6zVU3StTAAMst7xAtDeDoyuNNdm3qcC939J/y5y7KJ/B/kdof9xzNqcYr1y
xjHy/WL/aNwYhU0ce3LAHOsNMr9KThfZf2POWqXYYy+1WmD97xFr+oz1ny7j
FYdext5POHeR9/X0fUubq/jcyHgmmsUW2jhjR3jv53mGNUu8b1jWY452kPh+
E8M99h6jvSW2r9TwLPncJMYrvL8i/hMhc17N+Anmj7cYU3fCGNRbb5rN37xr
aUblpGeuWavmUKc1lrY/+dZg/Z0xayVNSjwvdNh3O0od0tZUfnHcrFnRhokn
YHwt4xlqdlvZr0DfAdoo57nUau6X6E8H1kDc+xPeBWlQZeytvLX6wZwZ9n/P
nFrYOzwV84w1W+3kPuo/X7A22skw+V1gfpL1H/Ftf9EchF0YJumbyBvb7sBc
taPiIk6zCWNQb3FS79mYY4VBu7Ap59lpJ8QlnPbsxem8/kLKfyceGIt2Srsk
TJrtubx9mvFD1lsqGKtmeo3436S8C5rnAM8V2skW/9nDKWsqLfV2jLmVgrXX
H6lyH45m/bf1R/4DB+KXNw==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.5627538121520608, 0.5257101166156268, 0.9079988775854744], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVD1ok1EUvfnSpCGpNcmX1OSLgz8YkNIOooJCpdVSHIxDddFF6VSl6OKg
FlFQySK4ilVaoXZQxP5akNIqrZNiBQVtB10d7OSiqNRzeu7w+N53373nnnvu
fW9738XeC4GZDWLFsf5GZvu2mo01mX3fbPaxYjYNe2/S7C0cp7H2wOcMfOYz
OEiYbWs0m4uZTWVhL8AGn3ho1lHU2dIms3xG9kvwX8D+NX7raZiw724wm4T/
aeT6hvMhYuTNHiBnDN824NwGuaGksIjTBf/DofB2gcvjklkr8GrgfRwrgE8b
Yh/C/gyxP4DRVtD+AOxV7Oew/4ScJ8tmK4g9AVsv1lnkGcd3Z86sE/sPzdLj
PTCPwWZF6RBLKD9r4b7meWMtZrsLil1ulp0+e8HzWiQcavk7JZx4QjpSw8A1
pSYBeJ4PVftR7BdRyyr2w8AdbpC9n34J5e0Cr06sHY3qDfGZl7p0pxU7g29P
WvisabRJXLrdTh9yyDgPYow6/xH09yVqG4CtmlSvzgXKH3f+S+B4vSIsYrD/
xPmCuBdZ8STnXw3iQG1iruEgtLka6SyAfxxrEfGzWcUHrnngudLoW6asOSL/
utfF3q4X1d+4+zOnOU/uWRNrY+0L8H+FNYuaboSaD8Y+gm2kIL05UzWfgVVg
D5RUO7XmDJAXMdlv1sIY6sQ+8U5MZjW3rC8VqkbOB3MyF2d/KpSmt1BrHf4H
U+Ie8/7+i3Q/2TfeCc4HZ2DjziSFz9m8G2quLsP3T6QaGcM5Jsexku4M52I1
La7ESSD2Xlbasg76s94j4LNW1GyMA28ikh6sYcY152zSjz7EeB4Kp8d7Q59W
ny/eQb4B8/4OkOOpnHjWXEPmpcaZnHQml2Xnv7+sWWF+YtzxXlIvvlPv4P8T
ej7Ni9+K18kaef/qLcLnPa+6/2fYnuQ1G/wnDjXsQO+uRPo/tEX3n3be79GS
ejgRaibIk7HE3Njz/Sn4DEDvWk5822F7g/XVNJP9fhcYw16yj3yr2vP+diH/
WqQ6+Y6GGWGuA+9+Qb2vgMPNss7+AzXOw2g=
              "],
              VertexColors->None], 
             PolygonBox[{{6893, 5951, 2297, 3212}, {4872, 7155, 2741, 1272}, {
              5613, 5612, 941, 986}, {4654, 5296, 1600, 941}, {5277, 4609, 
              901, 1183}, {6484, 4813, 2778, 2777}, {5127, 5764, 2106, 716}, {
              5440, 4929, 1396, 1826}, {5044, 6530, 2834, 1183}, {7221, 7220, 
              718, 3238}, {6411, 4868, 2818, 2695}, {6833, 4279, 514, 3150}, {
              6483, 4149, 180, 2777}, {6309, 6338, 2628, 2603}, {4890, 6529, 
              2834, 3524}, {6777, 6776, 715, 1618}, {6301, 6310, 2603, 941}, {
              5126, 5042, 1503, 1574}, {4930, 6707, 3007, 1396}, {5767, 6832, 
              3150, 718}, {6338, 6410, 2695, 2628}, {5951, 5950, 692, 2297}, {
              4441, 6909, 3238, 715}, {6530, 4275, 512, 2834}, {5312, 5167, 
              956, 1719}, {4149, 5275, 1692, 180}, {5202, 5126, 1574, 1632}, {
              4970, 6483, 2777, 901}, {5274, 4890, 3524, 1691}, {7039, 4680, 
              986, 2630}, {5043, 4607, 1691, 1503}, {4387, 5311, 1719, 
              1600}, {4363, 5127, 716, 617}, {4278, 5847, 1128, 514}, {4231, 
              5181, 1618, 2106}, {4813, 5281, 1695, 2778}, {6529, 4891, 2835, 
              2834}, {5280, 4256, 2156, 1695}, {5039, 5040, 1281, 444}, {5818,
               5044, 1183, 2156}, {4778, 6454, 1272, 1128}, {6707, 6706, 3006,
               3007}, {4868, 6520, 2827, 2818}, {6520, 4867, 1824, 2827}, {
              5167, 5166, 222, 956}, {4274, 5041, 617, 512}, {7123, 7124, 
              2777, 180}, {4174, 4992, 956, 222}, {6531, 4812, 1183, 2835}, {
              5008, 5203, 1632, 716}, {5635, 4174, 222, 1987}, {5437, 5381, 
              444, 1824}, {4779, 6340, 2630, 2741}, {5007, 5008, 716, 692}, {
              5040, 5441, 1826, 1281}, {5631, 5633, 1600, 956}, {6308, 4653, 
              941, 1600}, {4429, 6892, 3212, 3006}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.5830600779090692, 0.5584774943404042, 0.909937336062326], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVDlsE1EQHXu9dlh8xPbasbM0FAEULgmIIkVcDkQUHBJJR0RDSQECAULi
EMhIQBrokCB2mkABEiCugKJQJIIiQgpHARQpoMYWtDTmPd4vRv/v//PnvXkz
s6uPnRw9ETezyzAP9jtp1k6YjcC8ktnFbrOJmFlfaPYnZ7YEnxZ8DsOG4bOr
B9+9Zn+7zDrwW4d1Fmsua5aFrcH37aLZLdiCb1ZDzKfYL2N/IzAL0ljhH+Hs
atWs1xc+z8lhqGD2omI257n7onyI3YJdx9s4zjqhvttJrbzfDKwNsHzK7EzZ
7EiP+DA2MYhLzvWEcnwYmT2CHc0oP+b0EuJMA295hdk1T77UiNwGgHnJ6fMZ
70bB72sgDRiX5zPAf1/S2xp47i7pbhHaDGYVfxtibML5c2hbQ9xERnmRK32o
ZyePe7yvI46P+2RG+7FQ+lLbAcQZq4ov+ZMLdeD6ze2/e8JnzD5gplYJk7Vg
3syL9XnianQIMT1o98YXF77n2zs4nyxrvxX1Hy7qbhwaP8b+Zkz9MtKtnjkH
37NlaUP+zJG47BH2Cv0bZdWJee9BzfdWpHvkq+bMl/1I3aktuS9FivMjJx2p
IfE/RYr5DPtkKD3J5VQkPsxtrEv1HAqUH3ssAf+VVfUH71lr+tTdTBCXPfWz
oL5iTlMurwZ8g5Q4sD7vkuJMLL6n7uerwmD8tcBNc0awThRUD+ayH+sBWH/K
zUcg7Hnocb+ivuW7TFZ7r6i4jDnoZo59RXzGoH5z0OAt7BXOHiBGoyLcC92q
GbWawn2TBp8Y+DRCl1NV2MRizjxjjnFwjJU0c9R41OnM3su6mpI7eTLXZll9
M+t6O+v6nzG2uzivgTNTUU02gsM09v2BfBmX/h9wXq9K6y/A3FdQrvw33A3V
K7zjSp68i1z/+MDZUVJdmT9jsx/uhZofzs5/XQPlu7NH/zj+Cxbcf4c58h9T
c3vO3PGi3tKn5f6NV4qaH/bWll7NCWfkF9aDeeVCjpHjuR55NCvS6jT6dNLV
iDVddLP3EefjkWaIM912WOScdjrHoHMHOc6nNGe++1fwP0ANWZd/zH7BKQ==

              "],
              VertexColors->None], 
             PolygonBox[{{7205, 6310, 2598, 3596}, {7119, 7120, 3333, 1089}, {
              5202, 5203, 1475, 391}, {5039, 4248, 2131, 1501}, {5978, 7206, 
              3599, 2086}, {4429, 7263, 3786, 693}, {5161, 5633, 1986, 207}, {
              4428, 5950, 2298, 1617}, {5632, 5157, 207, 1986}, {6482, 4811, 
              2776, 2775}, {5791, 5790, 443, 2088}, {6302, 6412, 2696, 
              2599}, {5422, 5423, 1809, 179}, {7263, 6706, 985, 3786}, {5421, 
              4609, 1693, 1810}, {7003, 7004, 3334, 3333}, {6301, 4653, 2602, 
              2598}, {6308, 5161, 207, 2602}, {5277, 4812, 1802, 1693}, {4679,
               5451, 1834, 1733}, {4610, 5421, 1810, 1694}, {5635, 5634, 957, 
              1465}, {4608, 5275, 1444, 900}, {6327, 4991, 1465, 2618}, {5278,
               4969, 1444, 1694}, {4665, 4992, 1465, 1718}, {5452, 4928, 1307,
               1834}, {4248, 5381, 1776, 2131}, {4970, 4610, 1694, 1444}, {
              6412, 7082, 3421, 2696}, {5007, 5376, 1773, 1475}, {4991, 5309, 
              1718, 1465}, {6309, 7205, 3596, 392}, {5276, 4835, 1810, 
              1693}, {6324, 6328, 2618, 957}, {7025, 7026, 2598, 207}, {4835, 
              5279, 1694, 1810}, {4834, 5276, 1693, 1809}, {5380, 5379, 394, 
              1093}, {4928, 4065, 179, 1307}, {4811, 5378, 1774, 2776}, {5440,
               5441, 1501, 511}, {5043, 7002, 3332, 899}, {4930, 4929, 511, 
              985}, {5376, 4428, 1617, 1773}, {7083, 5380, 1093, 3421}, {6300,
               4651, 2599, 2598}, {7002, 5042, 391, 3332}, {7121, 7122, 2775, 
              393}, {6212, 5559, 900, 2525}, {5274, 4607, 899, 1306}, {6893, 
              6892, 693, 2298}, {5437, 4867, 1262, 1776}, {6411, 6410, 392, 
              1262}, {5436, 4866, 1261, 1774}, {5375, 4746, 1089, 1261}, {
              7206, 4219, 393, 3599}, {5788, 5789, 2086, 443}, {7004, 5331, 
              1733, 3334}, {6531, 4891, 1306, 1802}, {4221, 5750, 2088, 
              394}, {4969, 6211, 2525, 1444}},
              VertexColors->None], 
             PolygonBox[{{5159, 5632, 1986, 5631, 5158}, {5310, 5159, 5158, 
              4665, 1718}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.6030838231354013, 0.5906500812527482, 0.9103938157795082], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFlUtonGUUhk/nkpnESSZtZiYzjCBdtPFG3YUIgpeKCjYmjVmIigvBRRQU
FawiUqixXkoSkwq2OjMFwY1BXUSTuGgrxcsiAS+FBFHBC4KCpWutVt+H9wtZ
fPz//33fec973nP5dz/8xMTjmYhY0spqTXVFbPREtHIRd1cj7twVsamDS8WI
a7RWdHlvKWKkL+Kgvt8v2eYn7W/q/kYtYrroPTDO74io5v3+rd6LjYg+raPy
8YP2X9Ldy9rvLkT8mWw+Ecaq1mM6+7oZ8ZXWK70RucGIwXrE2WSDT/Af1PkD
Wu2cMfu7xUNnr4n7f5VtXJ7cnxqI6Gj/oYxjayduz/crtisjyrKfSzFu6s54
0TzBvEsYQ3q/Axv5/Ebr1V6fHyiZV1XfS+KxKOxT8vOreHym+we092nRfp9R
fC/0W1P0IX44jmrvkDB/VC7e07MkrfZrv1/PstYx6dbdsI5oiG9i7hB7OWKf
7N+VbUt+f0l+r9OzVTefrDAzTeetLQ6tmvNI3PcPOnY4ESe5vrfLcS9rzcjm
t6zfF/Q+X7I+K8r77cKfUwznK84Hmo7J75d125DLR4vW+XjJ9QT+ujRe6zM2
+k2n3FJjE0Xj/7Mz4oRwT4rLG4o7P2gdV4X9kdZwj/2hI/h7Ss4ZvMnTxS5r
OCSMM1o363s0bzxyMibMf+XjSNa1U071A8fjiUdOMS42jcUZd9CBs4UtHaTl
eMNxnut1TuHEOTjES37mK87R7isUl/A+zzmes6k2yDH6w528ogd1S31MJQ0/
lq/lmr8vyM/T8vNFzuv7bmPeJ86XU1zUObx4P1lwruDAXXhwn3rfm2I5rB5Z
kbaHM+bFfbih9Y09rk/eR3p850TSFJxH5HNH0/3JLIFjK/GCHxwGCn7nflua
XT3gXNwgzvuqPr9N/m+t2446JK9oP6y6Wq7bN/cuJKxs1XkiR9OpH6mDl4V9
UJqfzrve5lKMPF8vuf4nlZd7dOd3xbUqjL8ajpnapM7Zn+wyBnnEbjbZdpSH
UzXXwmzCXE6zYjL10YfCvl68F7vMi3kBN7DX+rb5rKdewM98qhlqZyHVIXfX
0/1npfFSqv/vqtYdzem3mcR5uOA6Jzf0UKex/Y3ezIf5hI+vqLlP6BFm7XjT
+X9OGr5T91yj9mdTjOMVa4cvcjmaMJmRF1MNv6k7l3a6RohxJtmOpFygIbyn
0nzempXUGzox3zZSXNQBtszFTt19fa10rVU8347pvbdhv+Rhboun4hhLsdDr
9AZ1iN0fVdsO9RofP9QTPcMd5ix4+DokHT6oWIeM3m+S7VWFNH/z7tXTFfcP
fQEOvQnW32XPd3rn6IDrkjyjC3lCu4zw1nYZ7xZ6omr7sYrnKdz4nzDf0Wkm
1Q/6/1y2DffpdXwyK/bX/c9DT/6jzKR2+h/CHc3ZP5fOqFHyD/89FdcE3/Op
LvH1orQ80rRmT+n5ZNP/PbhPNMz/Ldm9XXOdUztoh27/Ax2KNX4=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtI1HEQx2fVXR9lu67/XV02CANTS+pSUhFJ2QuSXK1LGR2iSwVF76Ii
sJAeUJfILF1PXZLsUKkdrIgex0pBiwoqCHoSHcOwPsN3DwO/+c/Md2a+M/Ov
2r63bU+emWWQAiQvbbajzGxnxOzWdLMZKbMLcbM1GDtLzEIJfJAs+ljIrHCm
WTV6caHZ41KzfwH2crNZ6D/A+I7tGPqdMMK3VvDuJcEvMhsHb1lCmI7lMe3k
f4mcBauXHCsqzWqIr5pm9rYYDPQG6llMbAOyD9/VMeEniOkD+xAyCX4b8iKt
Gr02x2xBHybfJXI9xP6M91BO70a2VJgdJP4ivUfJN8h7Kz2NhlTzJ3LX4/+U
2p4g37z3lLhpQs/i2xfItg1iw+AVVCjXI6QZ//XIRL7ZgbC4dk6cC+d8M7a/
fOvm3UA9q+i3qVJYXuMisE/Qbw1Y6/BpI3aUns6Vaia/o8wDn8aI+J4P3kL8
f/IuR7p4Z5PiZg49Pgf7NPE1xH+mptpAM/NYxxjCPlwprk9RwxF8j6alvzdx
fRipKxLnJ2Oq0Wvzbz4br8nxfEZfE+LMuerPF5cLEqrNOa2j90biN0WEUUp/
Md/BEu3gK3IPsA/vwurZfTvRR8KK2YDeEqiXL9RXH1dP3ks/0kMv+9PaFc//
ANtIoN7cJ4t9Xlw2rzGEb35at/AhT7vRi7QWaUd8N7uIvxrRjkZT6sFr95qH
8R1CduG/G+lJagbOvWMMEDs3l8/5uYLtGj4ZbNX4LA/0zd8bI5rlx6h2x2c6
Ra6VlbqNDjDu8l4K3pLcffyhnnf43zDtz2SZdsx3y2v23b6fVG2+44PEzy4T
H35j49he4z+F7UyRbv18XLftN58FvxdpDmtnPfZ2IK4dw3c1n5hfEe3sdfAm
EsLyHjPEtqRUu3NUC5+XU7lbCWvW7dxPrFgz9/fxmG7Tv/Uzm5tpzcpzrKW2
sUD/Dr+xjrQ4dO58x1t5ZwLthu+M9/Ymodl6j/8B9BqtFA==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.6231075683617334, 0.6228226681650925, 0.9108502954966903], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVEtsTVEU3e3rR3/6bt97t/f2EbRCKyR+/Uy0ZdJQUTWTiKKIiRpiIrQ0
KYlvo0lLS6QlNCRFVRnQBGFMkTQ6kfjUyAxtYy3rJAYn59x919l7r7X3Pov2
HNrWkmpmXVgRrDMFZnNDs9PZZjviZsO+2YEMsyDPbCTdbBSg7wmzb1hDmWYl
wFThzhPYK7DfD4Tpx7/j2Ifgc3eaWR18tCNQDfBlns674K8Jqx6rHPhuYDbi
fCRpdjipmH3wdw2rFPm0wk899tep2je78zLEvQu/tzMU80em4hLP79EUszu5
ZoO5st8A/oSvvD7j/xv4uA7MLdjng9d+2Kewl8LnIP5XY29Nyk9HtrR5Bj+n
omZroFXlHLMx2D5mKWY5/HxJSIcI7qUkFb8kx2xFzn/e3E8AkxVKU+p5Fefe
ULhCxB2HvRy8HiDfPsQ7gFjdOF+OKh/muLxAeTbi3l5oOwk+PWmq26dU6fou
W7YlMbMrwK1FrIe+7hAfIs4A/OwD5gJs57EWIodB4Js9+bmdKx7UsBi2YdSl
Gr634+4fT1zO4t45rAW4uxW5NWI1ZSiv1KRivUyXTqzdvbh80Q81Kc5RP8zC
5+K47OTKGpBve0w9xP5phz1IqHaPwGXEF0fWlDFZ1xH8b0DcaeR9NKa6Uiv2
GTVgzqwfa0486zbjasf+2pmhPLnTL883wPs6Vhn+d8LeC18HI9KLPum7zsX/
l2eO4rdEVD/mSD2pJTWlJl+hQV4gDLEzDv/Y1xyylpUFmjfOGmsy5MsP+4Y9
wp4MPfUNvzmvzzOFrwS2ytdsUFPOE3W4iFpdCqX7R/D+wD4ApjbQHeIb4G9L
XBp0YV8KHTela18f6FwLbA3WT+BXwV+/6yXaloeyv81WL5I/d/YItaJO1J91
fIxatUXVI204vwr+zzz1J2Z1qFnlnQact4bqb2o2miXdppDbcEJ1YU2a3KxN
uj6mbj58N8dUP8413wPW6wXuRYp0h29Vfih7FHteKCzr+NbNVL17m9if+1xf
M8+JLL1r1Idro8thXkK9S0zFHPUKY097miVibhaIJzmyL7554vXFEzfy4j32
7hs3U2MuD/JhzxF/DNiHgXphJlNzQR1XxsSTHNMKzdILNXfH3dtNLvTFN402
1o85sseo2cW44vDNYL3JewPizHrivcm96eQ8kW/2K1RO7xGzx1eMVuwno9KA
98mB2q2Dn9+e4rLnmDdz5vs94N52zgNryToyrw73JrN/x33hnsbFn9w7Eb80
pje4Gv5rAnErce8xuacVaR44C7zf72L9BcyQ/to=
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlF1MznEUx0891dOr3p7n6f/3MK+zWMym0o3EjRH1MBeuhC3rRi5xpyLD
vEQbikLDlswWpeICF2yuydoad2hz5U6q+Zx9uzjb+f3P2/ec8z3/VcdOHmhN
N7MUkoFcCc26kJV5ZrszzWoSZk9LzYax/Y6ajQRmVSVmExGzMeyT2L/GzR5h
e4xMov8oNHuLf1WW2bOYWVgsfRDpxb8HmUhTTAP2g0Vm8+iHse8g/05kOsds
D/kfop+h/hD11ueancO3kphd+FZnm5Vhzygzq8d3gpiA9y/qtUb0rqOXWr5N
ob8nfgZ8iZjwDFOznNz38NlIv2uQFPor8PUyjC/4byd2G3I2ohz91G/A5zO2
Fup3kWsT3/qo350l2x2+9WTJp6NIGLz2B3wywdodqpZjvo6+DgyVUc18LbH9
1CvHv5Yc2dhzQmH1mS7Q2yF6qIpqRsNgfZkQlm/pmn0d7z+L/jfI95OYExFh
DIgNkWb6e+A7YJfjSWHzb6PEjiDfydVC/f3EXwDfJ95NvNPwbSbfUD68QP6h
34ppV23UKADrZXJeAv9FZEmoHhyLfztF/L4icScoUK6puLjjOb23+WLNz3tM
JbVj3+U89hfop5OKdQ5GlrJ/ZDWza8rU7K9Sb0WectxF31wqm8/4WiiOO7fd
5y/6GDXnyN3JuxHsjTFxcS94C7EXheqlkxlUYH8e026ck41JYXRsnsNvxXgP
5utmxpjleEKzdE7VEP8mpv5fEz9Lr3WBbss5NopevXhfXsO5fp55bc0W5+OB
OO7zcP74rBpDcdX79dpDpeKCY5jBngi0e78J32VnqXrxnXZjGwh0W95PH7lu
U/NIpm7iZqiaXstn3E7+DqQ+Vzf8MdDMfFb+zW9hulCz9JoVxA9gP5ohTvqt
p2Likt/8FuxPsB/PUI/z7GIhrt59BunUiiTFNefk/UCYHavfiGOdKxb3HPNy
YpfFxW3P2cbs2xP6VzgGv/0NJfoX+T/AuTgL3ndR7eQ/G5KvyA==
              "],
              VertexColors->None], 
             PolygonBox[{{4984, 5293, 1706, 5294, 4983}, {6305, 4984, 4983, 
              6306, 2601}},
              VertexColors->None]}]}, 
          {RGBColor[
           0.6431313135880655, 0.6549952550774367, 0.9113067752138725], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVE1IlFEUvdk4NqPSOPPN+H1ORorYRktQaQL/Fi3aaCUt2rQxoU1Um6BV
2N+iYsyysD/Q0IhIyymFjDBXES2C6Adx084sWrQpQ3DqHM8DF5f3fe/dd8+5
5953q3pOdB8vMLNLsI2wUAVWWH+xWXnSLAVbDZtNYZ2GZYvMclGzp7DekNmA
Z7ZYZjYIn/mI2VsEOwzbX252qEz/8bhZHn5DAGjBuhuWA9avzWZ7Ymbv8b0d
PkO+WTXiNuH8AvZHNpgFiDGB/1MFwqgEr72FZm/w3QP8ecRcKBTmKvybSuT3
Ad+tiNfui1fSEw9yOJ0wu+8rJmP0hYR1C7nVufgT4LEPcYaxfy5ldhbWvMms
BvH/hXWHOSVczAzODPeXsf8NnK/j7Cb8+gOz2574NOB7J2w8qvvUkLj5sO7z
bg55ZzzlkggrxkHHk/jMl7yoK7ltLVa+5My9dFhafIW2jTHpPAecWVevKsSu
hrWGhUvMI6F1TsQpgTYx2HPotsQeQD/kkNsUNHgGOwoem3H+NyZuDdDzXiDt
GPuli0+/VES53ID/CGyb05X6EncFsX8nxHsG96qg3ZkC7f1x+8dK5c98f2Lv
R0LaUKf2hOJPow5t4PDa5cqVubN3iUcsS5s9Tqhf7vrKLedqNuv8u9LKn7lP
h5UH+QTw9ZPSzMd5EvYCPsNYazz1cCfw34HHx6hq9SiqenVg/3xM/Mljrc6m
lT3T63qB+vOb/TXn6sX3xr5kfZlHxL2xK560oA7kvuzu1gOnEzmMmbS844s/
a3XZU736HAb9WSO+GfZnM7hPw38X4n2G7ydP72w8Ie2oG+cC+TB+Cr7fgTED
Tl6paksdmHuPe1OvECMTl5ajvvqAfnzH2aL1t/bE9UOe8wY2h7OLMc0CxjkJ
/Nq4cDgP+Ib5fonNWjEvxsi7Ws/A/0FcvcNz1oqcD2AudZdLx6Uy1XLtDL71
gXhXAD+dlEaj2K8LVMsO+K7gzkJEOXBWkYO5mUg9+eZa3HzL4t7VQG+0FnwH
AuXLHsykpCdryfnE3ngIrEb4TEalcdbp3AHfRbznsSL1FmvGnicP3uV7vxYI
j1jsOfpyLvHel6T+u3A+HFOtOTPoy2++UXJhTM4Bzjj2CGvA98Gast8HA2nF
vtgRqDd4zn5hj7GHJ91Ma/PV98yL84/YxNoCLpVJafYf0ozpBg==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlDlMVVEQhkdBNiG8d9+9j3dBjCjBBhQXApoAFhbGxA0tbGygsDFqY0vi
WrggQghGTMCAWxQV4RFRozQaY2FijIZgYScujZWFCej3OxSTnHNm5p/tn1PV
fqzt6FIz24/kItcyZgPIWL5ZJ4rtCbPjFWZhidk79H+SZs3oZwrNZpGfKbM5
3vryzFLIDXTDyKoisxP4Zzl3phyrkbd+7pmk61Zzr4jMJgOzac5/8W9Nm21D
Rog/k2P2NDSbCt2/Gf0zzlWhnyeQg2D9KQeH8xtsOoiVJsYUuR3g3h2bXUFW
LjfbsQx89Lu4fyDe/BLO1LYbGcd+hPpG0Y8lXHcI/zJixYv59pNPB+f72MyC
lcbnDrlvBO8heGPIaXynwDtCvwbBSJPbr1LvnXp6D7yPyOs8r6+EXL+B+aTQ
c7gOVk3Kc60l51f0pz7l56v4t2LfknFszeAc8VZgY+iaCsx+YPuZeF35Po8s
urqE13aJt5xyxxRWF5hznBsCn43yW0/8Puyri6mTt4vcJyL3rcS+mtwXuE/n
e02D2E4wr8PEfk9Ovdj3xB5PNX1Peo2qTTOpjX3Gqv0u+E3EXrM4zyyyE/t9
ZWaF2O7hPh95jspNMcfxHaCGzcUe8yT34cCxGgqcW1Hos9LM2sDaW+bcEOZW
dGXgzYPdkeuxGwPnknLYErqNdGPUUIOud5HP2oHz6C6E3lvxq1F8zfguiC/q
XXnkXFYP66i3PvZetuc61trAuS/Mr0nvsXrbi08L5y+lHvsl+nTkPfnfC+WI
Phu47gXyNvCeqBfi9GTkO6n8xIFb6GPeFrg/QP875RxXrtqZTOQcEndkcxn8
rthnqZ0ZEtcyPmvl+ynynVB/taNNae+B8LQTmn1uuXNLHEjgW5rxXokDG2LP
SbmMFvnuqybVoj9As+0OvRea8TpxKfDeaWduBo4hX+2Ydu82b4+KfAfPUMup
tHNhiBoqI99R7aZ2ZBP2ZxOuk8/jtO+0dlk91F/1PHQu6s9JBb6z4qtq7gn9
j1E9+vP+AU64tqo=
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.6631550588143975, 0.6871678419897809, 0.9117632549310547], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVElIlWEUvZr+Pk3yDf//3q9ueoYNCyvITaXlok3llKtcRW4LpyAqCpMm
TCmhHFJqU2QqgT7TZqJtu4yiMGhTQrkoCITURedwvsXlm+5w7rn3fumWtqbW
bDPrh6yDPArMMpATOPihWQLyNN9syTN7CNmXY1YXMfuE92dZZhcSZpVx7HHO
5JqlC8zGsa8rNtuPt07c3YGP0wm9z8fMfqXMmqEznDQbgtTDX3up2YGodAaw
7oD9c/g/hfdzOJe7mIydyZYN98Qw6QMnMIwBXxrYb0d1Xw4sVbibgN1r7Bug
v5YlPw3O9gHstiPWJN4bPb3RfwQ5z2C9BJ0a5NydIzznoefn6576S3myOYi8
dgHrFGz64G8R55O4n4X/apy7YP+tCPxh34EcNwLzcCh8ZcBS7yku8XW5WNeY
ky/OGYd2jDUAu8FQXBPbBNbL0O9A/MMx4aY/rtOQt3j/kqscWMu7ucIWwHcI
/UpP61CofTXqcwWxq6C/BeutUDVtxH1TStysFsoP++RJoJzpcxz3Qb44Yn1e
FSivOpcf8UzlKQ597gGG3b7e/4CfZInZvfVmj9Evy5CFXOVNTshNL3SvQ256
krinXmJ/MD/ywbyJnTUi93Nx2VuxOCWfzGlzXBhWsb/q8qWwxrQlB3zj3Vbs
e0P1Etdtce3bwfmZqGq4CB5+Fyk/8s2eI+YZ3P9MCOuYmyHWqw22rZB0RDWc
dHXMAs5syEWcP6AuxxLKcR7nOfA6kqN8WVfmvBKon9hLnB/yR2ycV/pgvOWI
ZpH9yXg1DsMo+J5JymcPfLxHvBueOGAtWccq1HxvSj5WoH8UOvfxVoG8ZwP1
QYObT+bbArwbQt2zro2u9juRU0Wxcj0L2zb4yi9Ur3Jl3dZgdzym+PxXvgf6
W4i91hN+4n0XF+Yi6Echs25e2ZvssVSg3mJ85sA+Yk1Xcb8GeZMne/LJvh2F
j5FQf8ZHX38Ka0U72jPuIeBKBvLPWjIm+581zCsWrxk3e9ThP8f6NTv8rB9j
9vvinfvpqHqdcWKIHw+FiXVlD5FPzlCZ893t5oD/w5GUMDEWefrnuGphz6Q0
h2nEegn5ivbvAWedSdX+R0x/PfHX4++tK9VMs0//ppRfSaC/mLFqHRfkoRfY
+yCboD8O/YmEsLIHeMd/lZwFjrcJ90/RD//mz7502K8znjAvYG5+wc+gqbci
7o9lDZbczHCOSwP3P8LHC1/27LsOx9V/dybyjg==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlE1IVWEQhse/q96ke++5516PurqG/SyswBaFpi3aRGomLWoluS0yDaKi
KCgCM6pN/hS1KRRt429pFNG2pRgUBbVJIoOCSMqkeobXxcB855t555135ju5
zq72E/lmdggrxIayZn2VZhvKzA6UmNWHZmeTZqWcW2Nm17nLZcxm8xSTDsz+
ErPE3Qi2mDK7xfkOYMcLzA5zbis3Kyk1e0D8K+JbKsx6ihTTjJ+KzB5zPx/n
G36OnAawajjvpfb3hNn9dWaT5DSlzXaC3xwTRj/xC/AZpVaO+DNJcXau1fDv
gm83PU3lq4cx7vKoeYnYYWLy8Q1rLNS3LPGfwRsrUM1dxDdgbcS2xFRrERsv
Vs311P8E32P494qUezcSd8fw3Do4LRULYyP9n6xSL55/mn5uR+Lu336Tv5JR
/7PcN5N7IS3/PVSnPD+r3p6R8zEhDV0718O1mSDnXZE0CkP17L3uwJLUKovE
dwbNv4D9FTvCuR/8SWpPYUFMMb/ASkSK9W8D+JsCcXe8LfhDkbh7vz67OepN
xdRDFqzyjLSbYAYz+NMZ7Ybr9S0hjo7lmj7Frw7Vr2O4nwvVu3/7wS7Np8TX
+VSBVYnNsY8XmWEP2pzCatBnD+fpQDvtu/wcjbai1QJ4V4h/xHmAu36shftW
rJH7mUBY3fD7iTYrldrdy+R0cK5JSivnsB8uu+HUUKgdc+zXod6C1zhHbBOY
L+Pi5Fr3cX8zpvxa7h4G4jIWF3Yv9zdiqhGgbTySXv5GionvDdRbDrxhYruy
8r3eG86DWfXib7QOvwF+y5zH4ReQm1nT23fiWqgZu5bOaRv42yvExTXtIHdf
SrkZ6q9mtJM+jxfMq537g+Warb/xkVCcnIu/qfpy7ajvpnP4h5adKWGN+v+D
3jrT2oUnpdrNVt7HnzLt3yDnzcHaW4Lf0ZQwPPd8XLPzN+FvwWfoWoWRsFwz
f3sfEpqlv0GfxSj13hZpJi3Uqk2qvte8CtZqJGyf6XJaO+Cz9zc1ltZO+S45
hv/7XFPX0v+B/wF7sLfo
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.6831788040407296, 0.719340428902125, 0.9122197346482368], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVk1oVGcUvUlmMnESmzdv3sy8mUHRGkGhC0FcaZhFQchM7KJu3AVDXBWS
tv5HJyVo/Fdo6KQZhW4sii78yY8iiFIqNi6sNY0olIL4X6hadRG1/p3D+cDF
x3vzvfvde+65595v5nb2fNlda2ZnseqwrtSbvY+Y3cGPZRmzxWmzRzGzzgT2
c2Z/w/iXlNkkVgU27YHZSs/sIc5ON+j7tRqzU6HZHJwp18pne4P8luNmvXHZ
DcH3j1hl7Hfh++ImnR2F7xGsfYi7pkHxuU87+iK2tzPMDmN/HfxMYO9TnO3F
97Xw9y1WC87tB4ZMQjYLfLNh/J6H2MeBOcC6CMylvNld2Iw3mv2DfB9jDcL3
NHJ6hveVONuO9zfAczZqVkScNhdrM/aXws9AjfJhDsTZghi5uOIS+xn4PoxY
x+DjcdZsG/YuA8sKYLqLXBbB9hyef8HmPmxS4PlkTD6I9yr2q4h3He9/wP8h
2DyCn40xnZ9wnFSQ9w2sEnAsiQsv+X/VjD1P/mfPFB7i+sIXDvoYhm0VeRdr
VZ8tLlYF3zcklcsT+PncEx5yNIxVMZ1nfGpgK3Cej35878PaAT+NyKkJq4LY
E4h7Na+c/q/Rk7FYb3JDn6eoi7zq/xYYIsi3D5jaUKuOjLTG+tMf61iP71Gs
7XHlNu1qUYhIC60RaTASkw6n4P/XlOITO+vJOpKn9XlxxX3aMpcCfPd6yiUB
PO1Z1bQf8b6LqxeohU2e/OxFrtW08t+N/fmeOKa/rc5n2fUZ7U+nlTPzfYk1
r0Gc9GDvCM7/jBpPIvcXGfFOfx1J+aQeaU8/27E3kFQNWmCzLC8+qQVqjHpg
r1AjzHlVnXS0EzbRmM4R3/J6PZkvOetzuHejdjHkPhBX3xcD5ULc1Ba5vxCT
fqkTam3c6Z914DnWgos5kbfPcupD2v2L/P7zpMMAPMexTiPfKdRqKFTN3+C5
w9cs+Qb89OSUP2tyu1mxeGZkhvr3Pnj7BGfG8Pse3htDfWM82tGeuhtz9lV8
X+irj3uAa0+oXmTtyAV54Gz7PhSfrcBcSqhuzKnf6YEcRp19KVC/sVda85o1
7KGvgX/UlxY4W0pN0uSllPqEvfE77H/z1f/lpOYR68le7M6L5zF8P55Unchz
wdWO2A84/LtgU5fVN86MXqe/H9KaHdQEOXndJF7Ive/493zNfeZ1DRzP8qSZ
d6H6k71p8F2bVT/QN3VJTfJZdHkdS6qXiI01a3P7U83SK7VK/a4IpMXXiPkq
JU0xRsFpZy7vh5zqQdxFN5M7cTYZujqmxQs5+Qp7CV/3RgvO/uRpPlCnnJWs
I+ffoJuBc2ZKz9Q152GXu9uor6UZaeYEuQk/zu9Fzg/7iX1FP39mNK+59zyj
O4BaGww16xmfep/MyJ689gfitsvF5hzoQ52X+8I1jLxuBboL+7F/M1A9N+SF
iThqsrrnOBs5P466GULOyTd9clZSo+SU9+HTZtlz9r9084faHHJ39Cp3TxBn
FdwfzOnO5iy/4rRacLOD9b3ueKMfaoo9Ry2VnB5YN/6nWJ0Qt91J6Ywae5BS
z7PfPwDSf0R2
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtI1VEQxkdNr8/0fx/eq0KYujAoCKxVSjshtU1taiVJrQLtaU+NSDPR
gkpLW7Qx6LEx7UUQtSgoFyU+oiBaaWWRRbWo1B6/6WsxcObMnG9mvpk5Sxua
NjQmm9lmZBEyHjebQPaFzGowvIianc0zK8k2e5Jm1lJk1ooU55htSzf7hu/7
iFkvb/syzP4kzBpdTzE7kml2kLe7883K8D2eRIwCsxNhs7UEa8V+HlsfMuLY
3G0JzBbAaMH2mvjJ+CchD0x489jaeX/IsZEs9O/E2IjvMPHD6AEygP8Q+iDY
/YVmFeS/lRxmyfdjXLWdyRBWFfqPdGEu5m1DRHg3M1R7Dxi16eLgAlhlMbPR
JGFe4fwmkO8xOOvG3oW0Ussi9BHwn8HXGNj93H0g1nggrjz+Au/nkbvwdSfV
7HOu2Vv0gZBq/sl5GhlEv4/sJ58d+eKqPUnc38DeHVIPGrE1IZfgswqfCWy/
E+La9TFid9LTPmJtImYq3JZzV41/Gz6Pw+LMufKcLxJvKF+99porqaWqSLl7
vFNgn0T2wFch/vXU1xCod995kw0XlXmqpTeLGQirh94773E1+mHwluRopiYC
ceT4/Ugt+k7il6arR434NhUp1lMw93JuRlaiv0KP478d/F8ZysmxV5DD7SzF
mMJex90cvbvF3UP4eRRTb+apbxn5dCWEX4ZEOEcTmiWfiQ7sVqBZPIqsh8s6
5B3X9+C0BKy2iM6p9OQ5uQ8SvzZbM9QHVnFUuZdmKlZ5WGePWYH/ukDczfh7
8LIKxZ3PRA9cLo1pturwOcf73v/8u/1lVD5u851q4u2BPM2KcxiEhem+ztE1
cr0a0W76zDZz7k+odse8HFXNXqvP1Bd68zWuWXc+vLe+886H93gX+Xfi356p
nffaJ/PFrXNQH1GPvbc+g96b5YXqhfcoVKAZ9ll1jOGYduzfboW0C6vAX52p
nZjFfzJXs+g+4zHNvL/3PygFe1qBZtv/nBixPuWKf3/vvZiLabf8zmdjOtBf
4jMyRG3X/T9K1ox57JaIdtVzqOF9LTKVohnwXfIZ8N77Tq137or0d7rPGrir
jKu3/kfNkNso+XSY/tzTxDqTUG4e8y/Ib9C6
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7032025492670618, 0.7515130158144693, 0.912676214365419], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFlkts1GUUxT+ddqYz7TivTmfa0TQqIL6ibRM31GVJrLBxoQmIGBY+MCkr
dFE0WMCO0fpAsEqEaLG2XUipixZjTBSxjRtaocTEHY8EEy2iGzQh6vl5PuPi
y//rf+7j3HPPvf/eum3HIwM3hhBO6SR0PquGsKsUwmxjCCPNIXyuc1zvV5VD
ONQRQlMqhM5sCF25EG7Rczjnd+82hPB7JYTfdEbTISwo6JN6d+SGENKK2dUW
wmXZbdW7bxV7Xr8f1N/XZfuM7vfnHPM9/b5Y0FP5HhWg24shLOk+oftHus/W
HONjYZwvOi/5Rxt870s69z6dc/IrYCPfpDBcUNxnda/nnI9c1MD9tHB+pd9f
lU+fbLZnjZNca9tD+FBxxjPmZrpqfq7lQ2jWvaCcfTeF8FSTYw7qPJRxjRta
Q9ios0U2U7LtUZw5xW9VnqG8+flb73t1n1LM11KOtz3hGMSak80xve9JmoeS
sM00+n5BeJbEeUa/JYUzoTMkv4xqGW81t1dbxE+j+SQHfPD+D2HolM06+V4R
B2X197D6/bV85xVzveyeE7ZKwbmpv1s1bhamXerDetWynHBvu2N/R9GCzliw
TuAFTsB0otm+Q4q5Wr6T8l2W37Ty7lHc1cJ+h85R2Z4VtjM6Y4q1X5hfaDae
zbHH9HdNu32wR6c80eqarLmDt22Ku5gxn9zpN71eVr0r8h2UX5ewnFeu2YR1
tDdrLcEZvz+WsE7hDt7uEd57O1z7CfEwV7NeFhXzp3ZjPaV435TNB/H2Zc0J
/IGfOp7OmhNw71QfX6+6r29V/Td3ZgIe4XCD8lwsuBb00tJhTOcT1he63yu/
28rmB13fF7UNjyNxlh9P2gd9onE0xv1o7BE2aLNYtj57mjzLxD+pdw+0eXZT
wtmkM512DJ7MF+dy1MBF2a/krF9wrS0YG3y8Ejm5S5zdrTOh/KWy55y5A+f1
lLF2ZqxTNNNbNQ4wvCxONhbdc7DDKdyCmXjgpv/MFbzNyO/HnP9eVbQWiTmt
mhaK7v8laX+p4L7/2WLd4DtQ8qzC+S+yWdE5oJoPyPedvPU2o/uRvHfBnapp
rGgMdc3Z4bx7gdauRb2h/XNt1j9zfzDtvNQyFPW8QzUO1Dz37FbssGHmyc9c
oylqpkf0Es7oHfojNjPJzvoy5f0Dr1uiDrbG/mM/UfZOIfYmPRcrnj80yOyg
+bS4WVf1LOC/EH3ZnYfi/rwqvydKniF63tjuvlMT/eEOZz+0mjd2+HDcyc0d
3t3sKfTLe+YRXVEH9v0Z6wPc7KqTBe8H5r4/7l521s0Z78mXxN+Ltf/roKfU
wozU45x8X3HN1Dui+t6o2p8Z+SLu4uFoD05255tV6/JSwfOJTgYjVnBWsuad
Xk/KfkpnXPZnleuMzlDa37HJkmuEs58r5o0ZYXegSfYNOxfMp/X8Tj7D8TvA
zkbDzDdzzlwwW6NxBtEi/IJ5t7S2O+899LCw9LeaR74V9JWeUut/M32cnaTT
HeePJ7W83+GZAdu/34ak+1VPmj808Lxw7qyZg7+q3ung64rfX/rAfJTLnhF6
C29gHo47k/gf5D0/zM4x5fy0zf3rLvqbOhd3O7MKPw8qZ2/N3z3+j/gk/i+B
9vbEmZooOS4x61FjaGC/4r1d9b6hN8wBurqinvxa8XzWlbeh3bH+AYyESfY=

              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtslHUUxa8M7UxnqMzr68x0MGgs+IBgbBM2lBUpCVRcaKILNRoWPmpS
VurC8KZKFHwQ2kYJYEBsu7C0JFqNMREV0URogRoTlwIBosXKBkjA8Ds5Lm7y
/b9777mvc+996zc80TsnInqQucjGesRYS0TX3RELmyPmtEYsQ77ORexGOmsR
6WrE0aaIb7IRHcWIY7y/bIiYSEVsLUfck4/Yw3tnI//B+wrpAet5AnyC/Rbw
2zMRR/D/vhBxpmL71fg/AP4X2L+E/WH042CPCh9dO+8/5kdcLkUUwH43HXES
vNPYv439IPh96HYgfRR1ErkX/LGqfVfg8xPfj+Ozlvca5B3eG/Av4T9EjNu8
/0MOgT1GP86S2yz1ZPHd1mTfbmp8ttEYM+RT4t3BexSfzXn3SL05QU3/4n89
73wH8e8m1lL6OU2sm/Pww3cr+g760YVNP73Zi0zcFTEA3lQScT4xtvLLkFsj
8krKM/iw6hpV20Ji/kO8C8jT6PvRv1d1jfJdgP4g+T9Yc++foV8VfHdXrVMN
vysfclzc7BwWYf8B+nFqGQZjS945K1fNcAr/WeINNjhmQm0XE9vuhy8/4/8j
70HyfwvMX8H+lPkMEHsH7yfBewy858C7hc0uYt1fdL6KuYJ3J9KP7mXm+TH4
H7V61pm0uTqOvJgxZ3/B9xQx+prNievEvpG4P+JQHn0v8UfAvkV/EnRl5AC1
vgDmD3xPg7c945x7F5AL9qfRP8KsC/i/WnWvBsBbVzbnxDXVcIVerCT+LLN9
Cv3yFu+Qdkc17AP/XMVcEt8/4z2UmF+ybwAvW3Ctwh9Bd61i7qi/B+hXW+La
VbO48zDzHMqaQ0fJ7dGia9VO7kc/Sg7rM67hT7CbsLmUNoca8G1EtmW9M0v4
fqjmXgjztxZzUlxUT3L4XgCjJ2VOT5Lbd7y7Ut7hdmwP049D4RgjZdekWo5g
fzzxDNR79UO3ZXHNu64b8z74bUXzRzPTbg2X7auefw7+CfSTWdfURn3TzOXN
nDm4tmxOiUvaUeUizolrymm45JlqluLgPOLnWv/vNf4zFc9Qs9jb5N3/u+Jv
zehs4hqUu25EL9ir8uaqbohul3ZMu6Ubplt1teJdVL9Wo9tU923VDXiN7+68
Z1Vp9u2cJLc3cu7n5Zo5I67oX5HvUuJbKs4q9tX53jXlcL7gG6PbohlpF7SD
2j3txF/kMllwfeqXdnVP1bdDPVcuN8H7Nu2cUsTfif/2rDmiW/x63bmKYxN1
3zzdOt14zWY88W3WjKaId7zg/NW/dUXfYN1e3ZiZmjkubmuGc2u+wbq9itlZ
94yVq3bqDlOC8Os=
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7232262944933938, 0.7836856027268134, 0.9131326940826011], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNVUtoVVcU3T713feejbl5v3uvL7WIWq22fppg/Y2aQROHTquB2kmsv0ps
bUBMIipVFEXTQqHSBmN8BeMP/IAiag1Y0BgHGuNQBTV+By+f1kJci3UEB4dz
7rn7u/ba+0xZtWH5+piZXcQai9WTMtsbN2sbZ1aD8xfY7+DH0zKzIZzrsTzP
7O54s//GmJUSZitx1w7dTsiXnAz1VjjdIxmz2kqzPujcDsyuVpi1Js1uYX+G
74M4+xPM7jmby2OS5fkodNdX6t/zcrMvfbMb8LWpYFYMzT6Dz9WIbU2Z4uhO
m3XlzX4Zp2/G1wV7nbCzMyu7rz4w2wefbbDza1LnOsh8CtuzJimX3Z7yOIsY
fvAUI2UOw87neeX/SWQ2O1Lek7GyWI2Q+Rh3c2BnAHoPc2a9WO2e8BjEOgSb
6xB/ayidk/g/Ly+ftMv8mfsG2NmINQqZhjLlyZz2Q296Wj6LKflnrKdyyp+5
s4a9KZ1HXV1Yk+6YbBGTJOw8QA1OIrexwCaGNQ16j1GTV77yfQCbWeTShdje
4P8BX5jQHv0yl2bEeC0tzrA+awuq0XXYeYbvGtjZCpmWSHX+CD5/hu8tMcXj
x5UzMSo4DF9C90lGNapGDf+Pi3/Mf8jxbSp87gmlcx3cyMPXT7g/A/8z4KsD
9w9pJ1CN8/C7OpR92is63BrgtzmpOC9D5lagb8bThngvwWYVcm/xlS8xOJAX
DuRmMSN+noffswXVibxn/Zuxbw/lm35vFMQh1rkjo2+er+B/L/y2JMWRkuMJ
a7IwVF1afcXB++9wd9vxKgXd8bifDpklsHezQhxvwt0p9hruk6hhapLyaUrK
z06TrxWOG/9CtjpUzOxX9gz7ZS5sdgfSWwpcmxHH74jh24R4Ro4thd4SrD7I
7ArFG3KGfRBzvdDg+EvufZMQZuQMuVMb6fwafnor1Jf1DgfiTEypT7+LiQfi
uwuZEeS4NyObjOtcQTL95epn8mQa7O+Dzoeo96xIOBLDN+9xijw8npQu78k5
/ivCfiYrDhwLxV3ylnj/5TBn/1CXHGY/0w/5T8z+cbjVQbYW6+u4eutmoP7q
QZyVvmrBnBaFyqvkZgXrsgiyX1Uoxx898ZU8n4+8TkB+PvwtQ151keY3431R
rph3ow79kXTYi+cC8Zy1oR5z2Yx9zAT5WgaZRmB43zT/N7s3gHO2KqV74sL3
gNhSb9TJHMxrlnGO/Yb9caTZyW/Ks56ceTvc3Cu52cT68v+As0mMOBPezSru
rAtnPGvDepFn7Aly9e8yzRXOTM6M++WaG7TR5+YpbdMHY8kitlxOM+A4Yj7k
i8vkcasn3UH8H8K6gO+LnvBiTdvjqjmxnQ1eLwz0xhAT2mOO5DXjaXNnvpeM
bZsnPFjrmcDmj7RmFG0sdnb4pnW5d606pZ01IkfmFKR7GPWdmFYs7D/2En2d
hl5VWnrnA9WedT+KvTMn/yPYh3PKaRjyI6FqnAD3ooywJP84v4gteU38iWEc
solQd3y/m9zM5PxgHtShbo078/0hnsT1Ed8UN4d7AvGDWHRgP5KTXAtq8Wda
9sh39jY5z/mxIK8Z4oWaQeRDBnrpnDhBLDn3Gcf3kd4q8uot/+dQBQ==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtsVWUUhfeltLfnot723nPuObcXNUYKCvFRaIxQZ0wozhiqMVEnggoa
RSUxtiVCwIgYqcYYSKy0UBKLjwGQaIxgJJhIKQN5zQSTGnzgwAg+Ev2Wy8FO
/sf+91577bX/Wx7dsGb9nIhYh83FDrUixmoRT1wfsRbbUY94HfsKpylsI+s3
ioibKhHPsl+RRyzPffdbZ0Q+P+J8lRjEusz7k90Rl7jfmURcuS7iGm9LxCjN
i3iBGPey/wOfobaIBR1gSCOq5N/Luhe7iO/+LGJzOWIrMdfivw4bKzt/TXdd
Ef3k/gv/ButNxBhh/SL+CbEnUsdT/K/BcgJbydtNYHqPXLc1I8bB8hAEbMV3
C/ZnKeJse8QFavmYHG9xN0q8aeLdj/8M/juINwWWD7CXiHcHZ0dZXy1c20ps
sOk38p1mf0PNNaq2Mawf3xXYmcQcbGO9HZsqO6a4H8T/wQ734F2wbMce5/55
bJb7ibqxduFzhdp+yY1VnN8KHwMtc6+z98l/O5j2gWUSa/L2ZfZd9ONL+rWM
+wOFa+nH+hpwyP4jYq0h/z3sj2XGepn8e4i/LXV9j9GDB4h3mvwjiTneh+87
2MayeziRGZOwjLBf3BOxpMfakWbE7XjmO3HcC57N5E8r1txA4Z6ql2fJsZP9
zeT/hrc3cn+A9ST2eZjj5YV7ol6I49+JvZr8F7j/lBzD5BvCRueag4vE7sD/
YOKaT+OfgG90njUnraWc/d1hza1K3QNxrx6l+P5UNR71cJK749RQZn2u3bNV
z6xVzZhqW1AzdtV4HO6OgmE4sT6/w7eP+76K450H64mqtS0NqFdLuV9Wcc9m
pbeqZ0Wcfyht1q0lxTiVu0fqzVDiWdzAm38qnslVrFc3rVVx8gP+s9jgHOtJ
tYzz5ky7a9KsdM63djQzh9HakZb/jt0la/WLbucSXs1ipdu90UxKO7vh9JFO
a2gm98wLvziYQm8HG9aWZvAw90dyx1IND+P7K/u3E2NcAvbF2J6SNf5a4T9L
f0WL/Ul8p3NzI/+f4epS5t5KM4t424t92+Y/Ics8M5oV9Vzav6+w9jQDwr6/
7t6qhmd4+zR2qGRO3wT7LuzVss86eVsuzI00NqN5qBnbf38a3D3Xci+l0SdZ
P9WyNqQpaf0abz4re393w3+u/lrlUK8/KTy76vkA+Pam/mtU3wzcf///fyzO
jnVbc9KaZla1L2x6tsXBMO+HG/4bxWmlxzEVSzOh2T5FjF2JOd1C7lcK8y3M
VzNrWNoV5h9z90zc643++rbU8yw9LG34z9FfI43dSe13tYxPmvgXVAfwnA==

              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.743250039719726, 0.8158581896391577, 0.9135891737997833], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFlltslFUQx09tu+wFdtvut9/usoBUEhVNwEBruERjooFteVNUGtEHE8Vy
sSUSFWoAC1gJYrgYfPFFEGMAlUgrxAe1QlIuhQQbHzDReAmFpDzKpaCJ/19m
jJucnLPnmzPzn5n/zDnNL3Y9+epdQb+aEGo1NU0K4Wh9CAf1Z0DzrHQI/Vp/
pvX+RAh7tT5UCuFwyb69qbFcZ+bVhfDNhBAWJEz+iNZzEqbnsSkh7I5DqCZD
mNGgvUII+6SvLgoh22Q6D2hvJBfCZp07It2fa8yW7lrJ5FxmYzmEcj6Eks52
aV2j9TXZbdb3TZL/VDI3paNNNi7Jl3bNNwrmxw8al4WnVfIpzbtTISyVfPPE
EBYL14D8v1IM4WrRvp2rhHBWY4t8+0Myu3X+cclH0n9F53slf1D2r2hs0vo3
2RnLmd/xJLNJHIjHbI8hGO9pMpwX0CMfXs9oLZvnNTbK7t1JixN4Zsv+rIrZ
L6dNF2fHa0wOGbBv1/epmn8WlkHp7ZTva7MhdGu0yN+tTZYD9HZIZqho/myV
/Wn6f6/WY5P1vTGEHejK2zf2z6ct3nulc6zO9tC5QXnp0egVpi639ZSwrRHe
1RXjUbP22mVzveTbpa+qMSx904XnQsG4sbBinIAPxPqU9O2R/HDRsILzfc07
85bD69LXJ4xfiLSLpaOYNzzg3u65/Cph8R+otW/gI0dgJC7ovyBf24Tnacl8
Ij1LfA0+fMbfEjabTP6y8Ixq7FKOtiuP75YsJ3DpLe09obNvyI9/IuNGRnYm
af9trRdItkXjov4/Ix3PFu3bdGKTtlye1d65oumql86elPEtqbxtk84d2lso
/enJVos12vteuGdo/y/Nt3JWf/B7ScLsEtNdznN0vON68BX/8bdcb/kldw3S
vy4y/pwUlu8azS9qEj+XenwG6+ws8exyjlGfYPlSZ3fWGy/m1lpvoF7oD5x7
L2FnsQkW7Kay1kc2C/Mx2e3X6BTuv9Uz+iI7c0n+FRWL0QlWc9XIcvqr9uc0
mF3qe4nXOH0hOdHsblXst2nE5FX/R13mT+qu0fzCPvJguJqw85x9TrG6Ext2
8gKuFsm3Zq3m4OFJceRQbLpnCl93vcUVvA/FhvmI5vHIbFMXi2WrTefXif+v
Vaxn0Lf6I+tdHyqnv+TN5mnhPFM0TmzIGT6w4feyivn+rfzoKxhnUuLIRa0/
0PqE9B2PrCZGag0L/aMtsrqipr6W7gGNldJ/QvNxjVVaT4MzJe/pWk9xbL3y
93ZkecXvw+47vO5wbhNf+gU56pOOdXm7V8hPOmv8fF6x/b3JYhvnLCb0ioLW
PTnrtWCCT8Qcv0PGcnMgsnqgFl6S/pc1xlPWP8gB8V+pvU6NO9r/USOZtRpE
D3HE1oOK1QMaxzLWC6sF63HViYaHWDEv8nW39w94C64VGcNGnzhTsW/TJnhP
rTOeEHd69V7FaU9seYdr+AHfyNs+zx126AfYWqi9bWXTR69gn95CDK8799Yr
DnGDYcE/agmd/RnLNbbBX3W+EdeC+07s4Tfx7xD2ud6jwEY9wDH40ep57Jbs
2rLlgD4w6r2AWHf7HcGdT53R28GLP2C+Kd2DJcP2iOL8aN7OvKK9FRq3nds9
jg2O4Cc+zhenFhTNZ2La4njg4KrYeMhb4FbJYoq/8Anb+Bq7TjiKL3APbmEP
H7k34OxUj88G595/58GPX/CLmsJe5GcLLotMb9LiTO6oc/IG3qGs8YtcYAPf
8CvZaH2cvsk9CS/BeVr6LpWNW9fQL35+lLH6YeYNwFuBnkx/pj/S6+mr3F2N
GeuvxHu+x5y8/pSyXnFDdltLVhPjWs8r2Tcwj3mMRvydAW/hUa9zkjfgDH8H
4iv7bX4n4yN+3S+8MzWOZqwey36/8L3f48A7hfuGu2aNcrg6No6XvHfCJe7G
qvOee32qv3nQM+QxpL6GvMZ4K/CeIibcv2nv0fAMLOikD20pWSzhOPVCHJiT
vl6u2L9QNn7PUQ+fq/Gw3+eZ7P85zHgfo7/AD7hBHunvbf6O430JZvrCKb/v
lonLH+ftLDaG/b2xSTY3ly1/q/wd0OJvVOwTA+qEnoet/Xm7e+A1b2Te0+Ql
9hpf5D0MPOudh60et/smm5/Yxx/2wfkvtL+Lig==
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlUlsVWUYhj+ore3Flrb39J5z7y0IalDUgKHVMEQXSsJt3TkQSMSFRrEM
FqJRESO1TAKSCCUYEzcCJSg1goAQF44YZUzQuEAjUQwFhcSNIAVNfF5fFl/O
///f9H7jGftE90PPDo+IXug66L58xG2liF0jIhZVR4znPA7qC7/NTiNmQa8N
i6ivi1iZcYb6qyIKuYihJGJaS8To6yMGayKmwLulNaK2IaIHnenNEZ3wx8M/
CH8QW5u4P4L+m9grNEWsyGxrEvxj8P9sjKjj/Cr8niJfKANbE3h+B9vM1FjW
3BBxJ7InR2IX+3vhH8TfsqJlj2HzFL7+gj4RH58/Ee+PUBfxtXH/Ht1f8/bX
g0wbWC6B6Tvsn4D24+s8Pqrhz+d+e8k+5WsPdBn90eRgao19vI7+jU2+F/G/
DV4t95u5b4E+B8spdD4Mv53n3Jw49gH0J+PvjpJtrya+rsw5Ev8q/vvR3wY9
CvbN5KSCbgf0Jedl5HsOsZxJnVvxVctbocGca6pYvoW/tM4xTcb+8MR4fuBt
QyFiLfJj8V2pjbiL+y789dY6x6rd1hbnSjW8jP7fmWt9gYZq5zzUZFvKXwXZ
GVA7vHHguYq9f6A38P0Fbyup1ai8eeqhfWCbyX0993nof4ato7xt4D4dfx3I
d0JHc87BWc5vt5j3AjlrJb7jI32WztPgeRLaBH8IewvwfYX7EvTHENMldC9C
++DvQ/5QOeIItLw+YiP4jsP7pcW1Uc9XmtyD6r110N2KF3op557Zif2Bgmeh
D/0isdQyD+tq3MOzsD277Nwo5zfRr6vR3wVvO/Zz2K4rGe+OatdiEvr3NLgm
q5Ad22xZ9dhz2OpsNPZCvXvpqcyxqmc0i1Mz+9NManYXotNc73j2I780b/2P
kG9D/g98zqi1T83iWeW/zjMyV/p5Y7vC2zl4v6XuBclo1u/NX9sl5Oxdzlvy
zoV2iHK/otm9pRp04C/LO9eL8beR3G3EZyf8DmgBWO9vdH6EuVL0jGu21QNf
43siMqfJzQNVrvUzmbEphxXs9+c9G93YP0b9ppU9G8Kc5o1BvtWjiv18ybVV
DhrwtTVxvH1VztX72J+Qc852Zt4x2i16O5Qak7BoxjRbVYl1Ve/d8D5O3dvt
vP2MvT3cu67N41psHUi9a8Qfhe6BxLVRjy4m/u6iZy3o8VXwtmPj4Sr32NLE
O067rbfGu+0w9l65Vr8jnL9KPS96064aIIaJOfeTcn2B+XlnhHOu2TqeehdL
5wNkd2SOVzqa1ZcT+9LMqje087TrlLOrnBcVjfUcb7vxtySxrDAKu2qk2igG
1V46klUP7E2dM+VCOZpQ9oxrtlXzqal3pnblReRPgHde5l35/8xzfg+dx+o9
ozPwPyW17IP4OFl0zmV/LhgL2o8jfRbG1Yl3qvpFO+tf7gsL3h29w5yL58ue
PeXkcezNKXq3aee2Jq6haneSNKzH1mXuZ2q8s4Wtlx47W2OM34Dt0yb3j2Lc
DLa3WtwbejvNeU3mf4t2QFLym86qmWwrRsUmH9rdfQXPknb44bL/yfoXa8fN
h/dio2MRhuXYHtPs3aD6/wfjvyZK
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7632082645080606, 0.8471905023316807, 0.9137723860176945], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFl3ts1eUZx9+2p7Q9h9P23HouLeWSAW5mcxuZk6L+sWhBHRpIbKlBLuVS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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlmds1WUUxl8KtOXeFto7e3vLKGEZZyCizKgJw6gh8EEoKEuWMjTKCggo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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7811909467542686, 0.8531872476952542, 0.9057161604862615], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmXlw1dUVx39g8gJZ30vy8pL3EhAElU0BKxZBECyitQ6ExQgKBFRGRa3U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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3ts1uUVxx8Kpe1bynvp2/fye1szHIhSkLYOlMGY6IDhZljJpKMSaKsQ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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.7991736290004766, 0.8591839930588276, 0.8976599349548287], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmWmQlcUVhr8ZmH2GO/fOvXO/uXdIiYILOxJ3o8ZIKvpDSyMgQ2TTAdSJ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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3lwlWcVxl9IcpN7Sbhrcm/uvTiDLG0IkEAHClXbWosjzHS0tQRCKhA0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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8171563112466848, 0.8651807384224012, 0.8896037094233956], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHlw1dUVxy95eQl5yUvyluS9JIzVAjNgFRX8Q7QgtOpUkJEhxNIxiChL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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtl2twlOUVxx+S7IZks0l2393sZteqDDiCAurUD4oUbBVbwY6KRB1LoGoS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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8351389934928928, 0.8711774837859746, 0.8815474838919628], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmH2Q1VUZx88uLHcvu/fuvXv3vixLKcGkGZhob5CTFaWToMgmjuWA1jDk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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlnmwzmUUx597r7u8rve97+u973bdSRcVytpKU1NJM5bsDDWWaRRTQ7iY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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8531216757391009, 0.8771742291495481, 0.8734912583605298], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmH2QlWUZxt/9OLtnv8+7Z895z+5ZpWXNbJgE/qj8yGYqVEBABcOxZRrE
DBipjE2RphhmSmeAYMsarTBQ/EJJmWYCpjAhRkHFIbOpaVICDUTKUNndNCi2
68d1N/7xzPuc572f+/u+7vs9fQu+NvurtUmS3FKTJHV6vtyYJDNbkiSnH+u1
v13PLTo/kSZJf8W/X9T5fq2qzssNSbJMZz/TfrA+ST6m9U3xeropSX6VT5Jb
9a6/JF6FJNmr8x/p7L1a361vl0ytCbqzsjNJNujwQu236P1W0S7X+kV3kvyt
7DvQXRj8LxP/S7Um6PzG3iTZJP0el953i/+wzloT814tmpnSYbPOF7TaLu59
Sut60Tym85t03q87xXjH+QnJfKbH79Azp3WH5LZ1+Df7hyVzYyYddO+2Zus2
IXxQH3b9QDqdKx+tqbNdB+ps2+Qmy27W7942vRPdQfl0vH5P1bvJtfbrnY32
7VzRvJq3jMXSYZHWLyXjv5I/VnAcVkif42XbWNHZJUXLWyV+3xKfa2p9p5K3
b3cpngcz2/JQk2XdEL4/VGcadJkW+vxQtMfE90wuSQ7XmeaIdNug+G4s2c/4
e43WUPLBb/b7pdurmd+9q/2HK7Zvls6WiucyyZonfZ9Pfe+uRuvM3d/qzh2R
h8e1f7PJ/jyq5xGtVdoXxWO27q/KOefIvUciJ19vcEyGJOte8Z+uOw9rrc/Z
J8jcEPrvkF93KueuVjyGu5JkpMv371Eu3Ks1udV5MpJz7v1R9BcUHKN9olta
b79Mk+5TW5zn28VvR7d/TxLtmPb355w36HtRrWstrxjfFHHnPbFYK73+oJgu
0fsTOjtfepd1/oAY3191Xnxfdv1YNvTp3p8brTt59VHJ+le37VwkHgvLjv0B
0UwTzRh1KHumaJ0kt0V/rnjeLJ2PifZNrY2iPyaZb2itFs9vSJ8Xi7a/TjV7
ueJ4l84P6Ow13X8/54Uu1NH2dsuA/5BkPpt3vL4tPr+TbvOky64m83hU9EM6
X6f1FZ3n5O9CZrlnRPvrkvMAv3EPPamJ043GhH3B+0bZNUkyJ2rtq3EeEW9i
vT9vfgtq7QPsww+7W+1f9i9J1oqSa5FcQRYxapS9X6w6TtQF79FhVL9H2qzH
E7r385Lzd45o/664jIn3BaIf0LNa61ycEDh2nbBrRsX+7dGzV2un9juk2x7p
drzRucyTfF6je8vbbFfS5tqgLuYXHe/14fs5jfY/svEvvr2kyefU8ibtbw7c
nig/T6+4lsCWbe2OF/4+mdrnJeVCTcG5M0N+WFk1BtAT3k7dF/4kWdeLx9yo
V3S7ITBvuMa4slN3P9NrO/DZocDDn6TOY3QH23iPjWeiTvDz692+z7tJ0nlW
xVjwIPWbWTY4Sh8YTBx/8mBv4Bm1Caahy2D4H9/s1ppf6x4zLfoa9ybH3Sl5
YwB3tsgnI93GSmoReuqR/N0VfJaKZrBknADTt4Q+9Ez6H/xnSN/ZBeMn/kJ3
coz+0trh3vX5qvECLMAm6OBDbuyMmG5WXB7Vmie+o9Jra8lxfVL7zrJ79V7x
WJRZNr5/KnzYXXHOwWtUz5GK+9h43TuvbHwjF+gZ+A1/gkNg9RPiv1l6frrF
eU8Mz9Zgs2sXPT8ifg2KzbgG59uDUYPkPvaBt8N590X4Y98Xwkb6LT2UWsGe
TXGXuvxp6tr7t3L+fa1zGixzSchlfqHf0wepb+qOur6v0ThLvbwknzzV43r8
i/idzqzfd3T23R5jFnrfF/SHZe+TPcbK/qr7Ddh9j+6uy9wjb4+ZCQwnDgdT
6z5LOn4vdT78E93llz2iv0r3ZmrVa9/X4zmGOH9I/NvF/86c40oOkYdVxaS2
4BmNfKZ+yJ8+en7B2A9mkB/kBlhP7Kgj4rg96hrb2FMP/8ca8IU4wpNYPq7Y
Hi57jmImQnfyAR9PjLpgfzB8Tq6lkW+PyMZNmWNJzjOTwQd52yKX8N1A5Pzl
oj/S4zzYF3lGjoEfewJD8NFo2wc6jobt18lXb6XGQnBxUmA+fmOuQr8G4lMx
fl6r52Vlx+jjejZmth183h158omCMRE8ZHYjrtDTW45EjdP/mOGao8aXRIwu
VqznlIwxnywY38H2fOZ+CU4wr51f8cxG/6yPc7AEDAJPqJvTRecgOQQtWA3W
fqloncAFcBb/gAGX9tru/0jWK6lnOvB6LHjW66wjc+97peA6J6YLS56z34v5
EPzFh2AEe3KG5+LY/xX+mWe7Zj1bMvf1i8nP1P5hTqU3gwnkJX7qDzxZHLNr
Vbav7XKuPid9imXrSQ3QS+ZGv6In0LOYtR9K3Q+ojfFxTm0tDby9VvkwW+uM
7jfh89TzADHZWnBcmD3fST1nMUcSY+K7TvKHCsb7wdR+wSen5O/zKqa7Qs+a
1HXGLPxA5h6FLjzpU+DoxqqxdJAZrOIZ+x/y8Vtay1uc71NS5/yVXe6p9NO3
pcM7ZePX1uhV+JBaHo65/bbMsz74Rh1QG9TC53T2Ro/pqO9tUePTM2MNdGAH
98A9+gz9Bt2YG8gzMBZs5h3xBYPboh8RQ75ZiOMt0vmxDvcyvqs2xzmzFvad
nfljdlwdfqZXk3sXFdy/+c3MdjRmeOioFeIFZoPdZ/FZur+Q+juGXg3ek2MD
MWuAIUd7/M3DrIGvyF8whJr4bKfrgnvMoMg6Vfa3BzQDMZ9RJ62Z5050eSHv
7zd0p86oRXrBNYrn1RV/izAHXRWxo59f0eVYvCv+J8v2F/KoD+wCE+mRYCZz
JbmM/l+W3E7peo7eTxXPU5l7Hjoxl1IX4FYuda6u0Pu1qWe6caJfm3kmfbls
27AL3IUf38Odep9mtom1JmbgZ5TbzxYto13v2zLHpC9qjPr6jfzx9cw+4lvn
yqqxCH0XZpYBHjZHj1ips6HU3+jMScsCo16TXoWye8LTHf7eoF6oj11RI3xD
3J35G4J44bv5MV9Rc2DFOGqx4v8Y+I6Hllm3S/eKmfPuOdXZ81oDLe6lv++2
Pdhxa9hCzPhmOxTzMrTw+R8bvEgb
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlnlslGUQxl960C4Fu9vt7rdluVoJmBAPEg8u/UOR+xIUohCDIEKCB6Ei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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8711043579853089, 0.8831709745131215, 0.865435032829097], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHmMVfUVx3/ODMw85s3Me8wwd957VBZpTSapGiWCpMGFBklbKQVFHLRN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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwllmlsVFUUxy8zLe0wM+28GTqPmamUzSUkEj8JolFBA40gQtkMi0YgmigR
2oK4QYpUI6VFSaAYl5ZqhMpSNHGnYAwFWpDFha1GMRBNQChpKfuiv5P/h5Pc
885+7vmf+/rPmlfyYsA51wBlQSczzp2C/ujl3Hvwt/Kdy0s615bjXBq+EX5L
yrnCkHNh+L59nCuCtsNPxtGD2I6ELkScmx50rtt3bmKhc7V5+MXHnTHnRiG/
hPxD7IOecyF0WnKd2w0tK3DuEfzvJv75Hs7NR1YK1SEbjP5x9DeiU4mvBmKe
I7cj+BxA7BDxziMfSD47ke2CPGzjUCv2G7Gf2xt/6HSjfwv6KeHcAvyV4K8Y
+cfIniHfdvIdHJTuXOwHhmRzjXMx+V8m/1nIO4l9d1K1fUmM5ywe9kei6tE/
yA6ik8Y2C/1jyI9Dr6D/BN+aOe8g34epdz/yIprcD+rCVyn5bMC+EZoJX8UF
XUX/Yky26/CfD59Fjm25qtF6tR37GRH17DS2S5DvhF8Hfwh+c0y9HYqP0dRy
F/z72arpalI6JvsC/TXY3obOL8imwXch60yqVxvga7DtR/+C1FNKfvcXKmfL
9VF6OAJ+W9y5MvSn52kW7ohptmwmnuSufyPGTGr/kx40kftffCtjuMqhqZzb
45LZTEbJZRPfOslvGzmNxPYSNg2cO8nnMudu6IeAvo3ivkuI14qvaT3pc0Yz
arPZhXxERndod2czOwV/F9Oq3XxEqaUF+apcYSAbWcTXrL6Lz3pqr4NeRfYa
tAf7I8jf5G46IrKNZJSr+WjG/lBMd2F3tBf9dvSj5DaOnI7CX4GvDwkjZ6n1
35Rka7LluxWdt0KKcRT+nGEqoG82+x3IW0LCwGfg9QV68BL+HyLfOvQ/8TTb
N/FXzXkcGFjK+T70ywr0zc4TqKeFfDuSuushfGtDNiOhXN6Gr4LfB38Bfij8
SvzX8q0I/5f51pEQZg2rvcnhOvJ2T9ixmq3WY55m2Wo27PUn5t6gMPi6rxjm
+96QsPUs37yAMDYdfoEvbH2P/npyXY39ENs/uboby9lytTv61hPmDGvDoRp6
k6FnzchXkY+HfRxan6UYJdzdBGit044bhO61hHaJ9XhOQjvHdk099RYiT0Pz
gvL5AXczkBx65gjDN7jLmyn1YgE61eSyArqHWKPQP4OviRnlUob8Cv6vJnRu
JOZ3cdVotdmOPIv+pIxiL0TnAPH+Tmq32c4Zg34F8kH0qqmHsFqNzvwsYXZr
Rhg27M6Oard2kV9FWDv2Qko723a1fbNdcZiZWhrWzjDsVttODQrDi+Cfx0dx
jnamzeaJhHptM9qNv03IZwfl4zpyR/3l8FOQf4P8a+tRQDN7Ki2MG7bf4E53
4aslobfAdG5iH/TVy6nwq9Gt8tXLn9GP+cKsYdV2pPViLD25PaqerINvgCZx
rsQ+F39J+H22C4hxAn4xPoaFdEdF8MMzwvZK5CPj2nm262xmu7n7i320CyqQ
b0H384x6O54e1+CrCP7XiDASwV9hRrnOcdqVJ1PafbYzbRabUsKuzaTtvs3Q
A2HtwOa4dqzt1uXID8eECcOCvYmLiFfh6W00jNpbsNAXlu1NWO4JY4Ytq7kN
3602n0HthIOcF3Nf/eCfJuZHnE/z7eWw3rBKerssrdmynV1HLRXkNDmqntZn
1BPrhX07k1IPrXfmY4SnO7a7tTevL/pR5JXZ6pFh+dOk3h7D9Fr42rRqNUw1
ob8mrt7aG/o7/m5QT0FPzViYcy9f2LF/jLXI3/G1+6ymYczSrny9tYaJMcge
8zULPwb09u0n56fCegO/4rw1X/9CNuOP+7Ix3eyQsJ/qo38h2wHlBdqptkvt
H8Nmb3RvvUU2gzXks8TXrrSe70lpBm32phGzFNvdKZ3HY/8fvbgWE1YMcwFs
b8Q0+4ahAcQ+52kXWA/s7RvrKzd7Aw+kNHM2a1aT9XpHvu7Wer7CU0yLZTP/
PyZ9dLQ=
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.8890870402315171, 0.8891677198766951, 0.8573788072976639], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1mHuQ1mUVx5/dl41d9n33fd9ddpd33xU1J42ChApbIiyw2kYwLmZYMsOt
GjQQGQlsBHEGZwC5TOMFY7nUH2GiqGnSiBEOsqDogFRcnQkoLMMJkJtmXOz7
4Xv2j2d+z+/5neec85zL95znd/Wk6WPurkwpbatIKaPnq7mUNmnkqlJ6IpvS
co2jWt9Vl9LKmH+9Ud8bUrrYLaXOOtP8rMJP9jyj+S9jDv3u7im9rdGm+Y9b
U/qRxlHtGy+BtZ9K6btSYFW1afqI5i7xn11M6bzW725OabrGZ2pSGiH6rOiX
iuao5A8qe//xQkr7RdNd89/kU/pJz5REnr4nHqdbxFt6/Epjjca90m2t9J4p
2tPiv7LavC9oPl+jQzSzKnzWVaH/OfH/uNmyt4jfX/V+tWhn6/26Gut8UDYb
XptSX73fU2Nd5mm9Mu81zr46a77I6RSPXFNKhUrbdmedZU+tsu2PV9iOPLHt
DJ1lak/LnFKf0t5eKd3ewzZiDR1elPxVktWh+RLR7tGeX+jbIo2VWv9hxjbA
Fk9p34c60+RW73lSumwU3wWyzeFSSm9J73vEb71kn6iwTXrI3tPE80Sl7Yds
bDi2h3X5jvj/TjTPakyUPYaJ9miLZd2q+Qear5Dstlrz0JbLMbIiYgb9xgaf
k+IxRXosEs0O+XqWdD0tuxRkp6LGs6LPZ7zWR7wW1PqsK5LPfXvJ7y938/nh
2Vnh82NPYmRKxMlM8b5Dduinlwe1d7be+2t+UM8DGgs0f0H8V4dtN1V7fgs2
bPE5OeOT1d7XnuzD2irL3KfvH4nPmhrbY2jQf166TFLMbw29oCU+4b0m+O8W
/bVFx8zUZscB/h4l/yzTGKmxSesva32G9nxO5703Yoz3pfo+UzyHyZ43apzS
tz+L519aTPeM7JTT+uBa0y4L+mnieV/ReUF+TpT9VhLDOuNrWccz8b4l6/g+
LT5PN5qO58Gi5+PE70DONMQR/iOWyKUJGefySNngB+J1m+YbxWdCq3nvyHgP
MThH+iwSz6LW3lI8PBzzgaK7odZ+nCG5i7Ven/E34gJfYMfxof9+6VKdcf4+
LJ6Pif6KjPHohcifBVrbUnJ8EccTco63d5QfY0uOqVsaTAfNTq1dIRv2ztiO
xDexPSjm+nyZ97msfXtUtP/QeEJ8j0juYY33ss6JgbEX3BgX2PEFPQ+J5mjW
PvxG+HFnYC/2OR3fFybbm71rq51P5BV51BG02J8zFSKPsNeHVbb/MSm7utFY
i+6cB4xCz39r/zit/1R2m6Wzf1q6vSnfDak1ppJDewvOI/YOijwHezoCf45l
zYd8HKSxrMo4ydp7sU4Mjoo4BFeOxfpL0m1t3vbEvw9G7pNH9VqfnGzLDdXG
gb5ab8j7e3+tD9BYJ/qLYnABbJF+w8XnMfH5VqVtig2HV5ofe9/Ap2CO5q2a
9xNOniqYhrX6oGlQPAyT3a7p7rVi6HOlaN8vWQbxvqIq8kn2H523H8Cw6ojX
bbJrm/x7s96/rGd/jXdDr1PhX+QPLVuH5/K2CzaB782RU1nR7NLi/CrbqT74
E+usXV/p+t1VZ/bJp68XbY9q7X295H34hHjH/ui1vWgZfboZe9oyPgP+Yg3f
LQ1cmh81G334Pjp8Ola2+mzZvl8e2A8mIwd5+G59ydhE/C2RPlc1Olaf1tq6
kmPrUelyf6P98o74/bZkLABPOCdnRN87GqzzYvG5sdHysOsXw7YLteeP1cZ2
8m1EbdTrGmM963sKrv3gxv80/0Rx8Eq14+r6OuP55mrTgzP/lT33F+1zZJDD
+BBMm1ZlXPu9vn9JOtwQPukdOpPr0JOz8O+Xt4x3u1sm9sF3U6NPgBYZ0N8v
uQvFd6B0/3XUemrKobLlIWuH6O9T7ft51LI/xRlPyn4nSs7x3Trfxxrz9O2C
zlvZbLrrND9TchyT5xOzxhnqFzriR/Jqc9iBWtEv+p+XJP9QyTHzvHTYoHm7
aDv13FZyDaCfw1fE8rY62wvMoibMKUbNEe1rgc8Dda6/l40fDZGP5PtQYfWi
ZucmuUcckuPXSv9bW41HZzS/2OT6MCFkLg2bsIb+9DnoeD78AoaRd8ODH9jF
ufvmfXZ81T9whjXemW8jVxrs+86gZx07TgzZ6HK2YNnUqUze9Qssrww8x5bM
2wLnu+VNS++RCRqe3WIOPoPNYDg9KXqj89fq3OuBUXWSeylv2sMN7qehPVB0
HGObb8rGN2mclW2H1Hl/78BG5vChj6GfId66vhHP9ELcKeiHwMXBQU9fhj3B
ljuVl2+qp23v7tjcX+/4zJbdK6P/SeXfmKLvFiMDZ6bLHu9njTX4jn4UX9EP
t8vP2binDCi7pyRH6OfoP4gJcujVuO+Aj+AwNv9EtKnVer9RND4OjzoJtlAr
wZunAnPaw87Ih6a9xnQP6Kxz6/xtsXgsafa3C9E7oyd0YBY8L0luzwbbirpK
/LH3TJ1r/CM5+4AaR56CKcQlMYntif3JUfuY45M5su3OBuMkvhocviNfoLuc
Iy2+AxATyEAWtaZTZ98rnR+qcZyvibtDU9l1ghqBD8mnrpoIP2TR3yyKHnw7
GBk9HZg9J3CbPgH7UL++Ip5bi/YnvfDW6Ifp9+lVwMx5+v6fJvsJHbELmFbR
6hpM/U3S93zBmI59TgfNP7VvV8F3zcndba/J0XsciR6M/pnemR4JG9NPYOcN
krux2T0LdKyDw9tL7ju76mt9xAbfj8ReMLozatzlnAu54FtnYAX98okGY8Hm
Xu7p6d85A/c68Jm74664J7aWXeOpGwPCtvDh/P3C75fXwuZrm+xj5HNucB3b
goVnm+2/kmh6Nfm+Tm6QU2AcfSv3Eeohaw/Fekb7LhaM9esLrhnwJaY25xwH
j+bsI2KK3pqzUMcaddbKVtOBkX2jxlFjFsZ56akfLzpWqXvc8VYEdg0J/Bof
2AlWnyr5vwA1hBq+O/45YI++YRNqD5gBBoKT4Cb5vlT6P192j4Hf0RFZxBn4
Cs2hBsfBoMC21qjXf9O+w2WftevOyd4zsvdtRd/7X2l230PPs0+yjjdZ9ovR
eyCP+9ZdPY0N9IeTcu6XwDliAb3BfPShDwHjM4H/N9HrNTq2VsV9FRteJR2u
bHEsfLXsfpR6eFZrH7W4Pzin5/eL7heoLdQYasfbov1Xk+98jwuPnytbJ/51
HAgdLun7ByX7ARwcGD0/+UpfOCtiHzzD/py/LWoQ/3v4/wIG0juSe9ATC53R
S4At4Bu4TSxTP4nnMTnfX9kD72viHj067g7IBlfa4v5CnwlPegme22LOPw5k
oxN7J4bNydlc3N/5x7Mq9AQD/hAxMCr6XPY/UvZ/Ge4r24P/uvi/dCFwfq72
Hi64PtCfgfnED3bAxsRqV68PXtHzEpfE5J2y8Z5evs91xN0P/Y+U3beBM9zH
qEPUFXrk2rL7ZO7oI+LfFLlInMIf2geiNnGf5o7NvSofWAaObS35/wt3FP79
HCzbv5xve/RnByImLv/7kpzROdtkeejJP7SRUau77gt9wtf8R6ROU6PRC5yg
7sMDH+NfdJwXNYI43h62/XbZ/Sg9FN+h40yjYx8ywKlBEW+ck5ggHuhbyT/i
nJ7yYMTz3OBTCrztiH8p0+O/Hnw40644V7f4vwGf/wMo5d0p
              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlmlslFUUhi8doC2d0vm6DTNTUTSCPwRBZdMgAiKJQAwogtFEKZoIlsXQ
QFwIhJpABY1hCWCp+kMQlc1ElN0IRQgERGVPbFXQCIEKpQWRzef15cfJ3Pud
c96z3LNMp/LJIydlhRCWQ62hKSUh1KVCeDYvhCezQ7hSGsI/0NRWIczMCWFy
MoSJ0DWU7s4N4dFMCP2hueheiIdwAN39UMdYCFVtQpgN3uqEdcvB68i5N/Jn
kH0Xnf3cJyWNVYPMuLIQXoZOtscnvu2OQngI+UVgDQVzb4cQhsOfCX8IeNPg
jy/x+U7kD3MfjHwL+F8Sw450CEewsRLssXxbC28bGJXEU54fwoqCED7H377I
8hOm48veIvA4dwdvBx93Qk3YL8B+K8UO3jbwtkLnyM2syLKy8Q3nBvSz25JL
vr3B/Q9klsPrwj0L/avgzc0zRiPnx/CpGd+eBz8b7OFFtjUKH9bAW1xoX8dC
h/B9Ad/GtAthNHQT7H/BmAPeZvBWEs9q7g9zhx2mYW8mPuTEHMNizr3JX7/2
fqOKYvso36bj8wzOleh0Vez4FGFrM3jDwPuO+wDuAzN+66HUwCPkvj90neL5
gpyexf96/O8L9mD498m/hH07xRvtSjlGxaYa2839e6gQ+VltbCuFzsFs2xzN
/ST2lhD7Rmw0o3sD/gvIDwd/ELwmvm3IcQ6Lsd2K+O5CPw+8Wnw7TjxziGcY
OgHejYx5W8E8SryPc1/G/UX4A5FfSM10jRsjhvzNjGW/hVbhzyfkGLjQC/9G
oX8B+Rrka6FP4Y0np5Xksh/8tyLnULkbyr2Bcz3UC1sLwVuAbgk+b+d8LzLn
wb8A5bV1jH+W+k30FrXE+Aq88Sm/fTU+dIBfCv2Q7ZxNQfbVpHuhifxM516R
9Nte5X4OW9fRr8aXGvTz8SUOJbJcMwfIZfIWnmr2pTL3lHpJPTk38ozQbOiJ
jc7cL6YdexUYt3MuBOMz5FdB57nPS7g2liEzBtsnCv2W78unhGtEtbEFe0fh
/ZJxbl7Hvwc4PwidijtnXfBnPjprwIvw53d4s8l3dVvXyH7iqxcmvpQhn4X8
ADD3xJzjXsj3hP4C7zS0JemeVa9OhtbBW59xr8nGFfjHItdGHBv1+N8ATSG2
PjHbejPpXMhmXYFrRrVSQU2NRLeRHCzB1tK4e28F8c5p7R48iu6eyLNNM6MB
vHl8GwLeJfSP48uJjGutEfxnkG0G7yOwPobmI/sOdFvMOt3B7829B9jdocvI
XoJWZVmnifMIbJzUzOd+CNk6MN9Gt0rzM+keVu/q2z3Y/ipyrO+1ca0/HVlX
NX8//B4Z16LeSNh/px2rbOQTa3tob7ZrbEBkH+WbfNKs3QdFMc/cn9HflHKv
dIJfjXxpxrNBMvO4z0g6N5oZv8H7NePZqTfVbA1J15Jm7D7eYzn8Efl+M51r
oZH5/nYN+Rjy23PcE5ql84tdm5qp2n0taedaO1C7Yzv0Wq53SF+wduHTEzH3
7EXtCmhszDNFs2AdPgzJ80z4GtnNSedSM7KxyD2pXpTOGXRPpzyrtPPU29qR
2o3qycMdXMOqXe2ADchOKHFvy8bOyG+qt9SOfa7IO1e7Vj7+39sJzxLhXU94
p2mXKQfarWsLvAu1YzULNaM0mzQT1WstyNfkuueqSrzjtdvHQd3Av5T07lZP
/4TuwbRnw9S4dTtj84NcY+gtfkybpzdp1u4p9W6RzCLiKcv4v4Rq/HDCO127
XDuqD/Gtp78beYuJyGyM3NPqZb3RMc4tCfeudkC8yD2q3tTOmlDsmalZqf80
Nwo8wzW7VbPV6C+MPEs0Q/LwpV3Gvaj/DJfhX0x4F32Y69m3q8C+aAZqN3W7
Nd+0o+SrZphml3zexH1QyrnTTNCseCryrNTMOIvuoYR7QTtc/22ORLal/zjq
9TvSnrXq+f8A1upwCA==
              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.9070697224777251, 0.8951644652402685, 0.8493225817662311], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmXtw1dURx383hJCQe5MbyIN7b3zNqCAgYnnJQ1SEqjNoqyIqdNCKTIsi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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwllmdslWUUxx9LaQu9t/Te29v2vi8xIUGCMhwsNYS9TJQgQ1AcBNmriEJI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              "],
              VertexColors->None]}]}, 
          {RGBColor[
           0.9250524047239332, 0.901161210603842, 0.8412663562347981], 
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmXuUVeV5xveZYS54Zs6ZOQwzzjkDaAwgF01VLmmbURK0YCRUkWuqwaJy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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwtlmlsVGUUhj+6w0zpdNpOmTtlS1BQSFCogBgFg6Ai1FgKIhFRWZSiFFGj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              "],
              VertexColors->None]}]}, 
          {RGBColor[0.941176, 0.906538, 0.834043], EdgeForm[None], 
           GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmG1slWcZx+9T+t5yzunpC55zOhiMLRqDCqPt2KJ9Q9iLaaEVYmCtL0Pi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              "],
              VertexColors->None], PolygonBox[CompressedData["
1:eJwllEtIlWEQhse85jnp0aOHzn9OWovcRC0ikCDTrCBKtLx0I4sMBTVqF2WX
TdmmtE2XZdnFImhjFJaXLhgq2U0ootIgyBa5jNoY9Qzv4oNv/pn3nfnemfmX
NB2uPTTPzOo5aZxowqwzz2wsbDbKKcBuyjfbavpWEZit4wxlmP1INesoNKuL
maVnmZ3iDOaalUbNlkPaCOY7+CnwA2nC5GNX4S/B/yIs7rKk2ZYc5VgPd0PE
7HXI7FVIsYVgxsPC7CL+aZwYfJXENxO7jHq78SXJkUnsQeyHxD/gTBJfCX8j
/MXEvAQ7wfmSabYCe5b4HXC8g693gdlpsIP4t2GPgG8G347/H74U+K9SXy/2
Ud5air8TfB0xs6nKeRPsHBrwyUbJsRK+Dxh9YC+Rr41aWpPSrtvfzP0v8QXE
X8hRLRvJ8TakmvaD7UqoljkwN/CPofcR8qdwduNfA8cmsJtTlMtr9lo9p8dm
x1SLY4ap70yetK+H/xq5egJpPQ3+InrXJFWbv6GFfI34Z0Oq6RP3MvA9vOUE
HEPw9VP/WpNm7wEVgP+6QAO1gfsfvgXcW3LU6wPYdSnquff6I5zHwuq59+pd
vt7iPfudLw7HLoXyCvVdjupeOE9v95nwWXANvLY9cB4Pq8ZW7uei6rXnbMM/
hR5daJGYb3Y2TzPvs+4zNo6vknomQurxHe83ZwZsOf43+F9xVnM/maVctwP5
POdj7v2BatuJ/SQuzV1rn9kOclVQ7wi9LMuWVs9ztSuu2faIZtJncThd2jwK
xOUapeIvJv/7TGn+DDuMPZapN40S+zlQb3wH/S3VMc2Kv6mafa0pFLf3pCSm
Hfbd9Z2oxd4X0az6Tn9jMBsj6uUk+1tO7eN8G87QG+7iO5/QbLpGReRbFGjX
esm323c5Jq18R1fRi3u8dzGUv8BMwzeTUC/8H3EfrfrimkXfKQMfwHczJIxr
nwzE7T1oz9PO+677DlXh2xKI22v02fV/hP8bfIZ/gh9YKO2vU1NDXDvou+ca
3opox323PWcR8ZMRxbrmeyPSxLXwHStO6pvf/R/xH+XNusU=
              "],
              VertexColors->None]}]}}, {{}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4289, 5644, 6485, 7248, 4093, 6486, 5219, 4459, 6919, 
              4460, 7045, 4691, 7044, 4690, 6579, 6578, 6816, 6817, 6724, 
              6723, 4181, 6872, 6841, 6842, 6840, 4295, 5015, 5014, 5174, 
              4854, 7236, 4855, 4727, 5454, 5455, 5000, 4182, 7148, 4183, 
              6725, 6726, 4198, 6749, 4197, 4395, 7197, 4186, 5838, 5839, 
              5837, 6684, 6683, 6686, 6685, 4818, 7234, 4817, 5417, 4200, 
              4698, 7228, 4697, 6592, 6591, 6590, 4484, 6930, 4940, 6933, 
              4486, 6931, 4485, 6932, 4706, 7229, 4707, 4202, 5426, 4201, 
              5003, 6603, 6604, 4029, 7246, 5670, 5498, 5715, 5714, 4191, 
              7201, 4402, 4822, 4709, 4203, 7200, 7199, 4190, 5346, 4189, 
              7198, 4708, 4821, 7188, 4034, 5175, 4028, 4919, 4317, 6855, 
              6875, 6874, 4413, 4188, 6823, 6824, 6822, 4266, 7052, 4704, 
              7051, 4705, 7053, 4483, 6588, 6589, 4024, 5230, 4021, 4099, 
              5663, 5662, 4306, 7283, 5867, 6331, 7034, 4672, 7033, 4671, 
              5321, 6732, 6731, 6730, 4694, 4916, 4013, 4096, 6682, 6681, 
              7256, 4261, 6815, 4262, 5833, 6722, 5710, 7208, 4259, 5832, 
              5642, 5643, 6564, 4092, 5012, 7247, 7274, 7273, 4689, 6719, 
              6720, 6721, 5320, 7141, 4829, 7140, 4830, 7142, 4458, 5853, 
              4289}],
             RowBox[{"-", "7.4`"}]],
            Annotation[#, -7.4, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4002, 6565, 4814, 5221, 5220, 4003, 6566, 5855, 4290, 
              5854, 4291, 7127, 7128, 7125, 4815, 7126, 5223, 4462, 6920, 
              4463, 6921, 4184, 5836, 4264, 5834, 4263, 5835, 4394, 5925, 
              4856, 7237, 4857, 4728, 5456, 5001, 5002, 4846, 6728, 6727, 
              7046, 4692, 6750, 4199, 6751, 4396, 7261, 7212, 6821, 4265, 
              6818, 6820, 6819, 4100, 4022, 7048, 7049, 7047, 4699, 5323, 
              5322, 5418, 5324, 5325, 4700, 7050, 7194, 7193, 5020, 4102, 
              4025, 7187, 4820, 4819, 7235, 4880, 5011, 5460, 4861, 5429, 
              5431, 5430, 5427, 5428, 4882, 7164, 4881, 7165, 5021, 6910, 
              4207, 6763, 6882, 6881, 4416, 6785, 4234, 6786, 4735, 7066, 
              7067, 6764, 4209, 5362, 4208, 7282, 5733, 7277, 5734, 6880, 
              6879, 6878, 6828, 4267, 6825, 6827, 6826, 4113, 4035, 7055, 
              7056, 7054, 4710, 5327, 5326, 5419, 5328, 5329, 4711, 7057, 
              7064, 7063, 4730, 4119, 4042, 6621, 6620, 6829, 4270, 7257, 
              6743, 6744, 6742, 4404, 6867, 4825, 7059, 4714, 6739, 6740, 
              6741, 5347, 6738, 6737, 6736, 4713, 4824, 7189, 4048, 5532, 
              7213, 4403, 7215, 5516, 7214, 5515, 5841, 4269, 7209, 5716, 
              6735, 6734, 7266, 4530, 6951, 6618, 6619, 5239, 6613, 6612, 
              6611, 7132, 7131, 4325, 6856, 4324, 7260, 6024, 6936, 6935, 
              5231, 5232, 4498, 6937, 6939, 6938}],
             RowBox[{"-", "11.100000000000001`"}]],
            Annotation[#, -11.100000000000001`, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4410, 5940, 5924, 5828, 5829, 5827, 5831, 5830, 5852, 
              5825, 5826, 5475, 5476, 4001, 5477, 6563, 6562}],
             RowBox[{"-", "11.100000000000001`"}]],
            Annotation[#, -11.100000000000001`, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4041, 5240, 4823, 7133, 7135, 7134, 4531, 6963, 6964, 
              6960, 6962, 6961, 7151, 6504, 6505, 6356, 7058, 4712, 6357, 
              6966, 6965, 7137, 7136, 6639, 6638, 6980, 6979, 4338, 5678, 
              6134, 7267, 7269, 7268, 4565, 6981, 4566, 6135, 4339, 5885, 
              4340, 5886, 6640, 6641, 5091, 6969, 4549, 6968, 4550, 4955, 
              4126, 6967, 4547, 6100, 4548, 6101, 4332, 5086, 4533, 6073, 
              4534, 6074, 4951, 6952, 4532, 6622, 6623, 4043, 7068, 7069, 
              6865, 6866, 6864, 4390, 6899, 4437, 6897, 4436, 6898, 4177, 
              6895, 4434, 6894, 4435, 6896, 4235, 5438, 5439, 4877, 7239, 
              4876, 4036, 7171, 7167, 7168, 7166, 4884, 7169, 4883, 5442, 
              6803, 6802, 4885, 7170, 4673, 6025, 4499, 5499, 5500, 5069, 
              4318, 6011, 4488, 5488, 6596, 6597, 4103, 4941, 4487, 6593, 
              6594, 6595, 5065, 6006, 4479, 5868, 4308, 6849, 4307, 6585, 
              7264, 4476, 6929, 6586, 5657, 5656, 5654, 4301, 5998, 5999, 
              5997, 4472, 5226, 4464, 5995, 4470, 5481, 6346, 7245, 4014, 
              6580, 4005, 6570, 6569, 6923, 6922, 4461, 7129, 7130, 6567, 
              6568, 4004, 5222, 5224}],
             RowBox[{"-", "14.8`"}]],
            Annotation[#, -14.8, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4006, 6572, 6571, 6925, 6924, 4292, 5645, 5646, 4094, 
              4007, 5225, 4015, 6581, 5482, 4471, 5996, 4465, 5228, 4474, 
              5861, 4302, 5655, 4303, 5862, 4477, 6004, 7259, 4309, 6850, 
              4310, 5869, 4480, 6007, 5066, 5067, 4490, 6012, 4489, 6934, 
              4104, 6599, 6598, 5489, 4491, 6013, 4319, 6027, 6028, 6026, 
              4500, 6940, 4108, 6606, 6605, 6941, 4501, 7223, 5797, 6804, 
              5798, 4793, 6467, 5507, 5506, 4268, 5840, 5881, 5880, 6901, 
              4438, 6900, 6903, 6902, 4418, 6883, 4417, 7216, 5943, 6688, 
              6687, 4114, 4037, 4920, 4784, 7207, 4236, 5408, 4237, 7087, 
              7088, 7085, 4757, 7086, 6333, 6334, 6076, 4536, 6075, 4535, 
              5087, 4333, 6103, 4552, 6102, 4551, 6970, 4127, 6634, 6633, 
              5533, 6105, 6104, 4341, 6137, 4568, 6136, 4567, 6982, 4135, 
              6643, 6642, 5546, 4569, 5102, 5101, 6653, 6652, 5891, 4349, 
              5889, 4348, 5890, 6651, 5888, 4347, 5679, 4346, 5887, 6650, 
              5260, 4564, 6488, 4826, 6487, 7139, 7138, 5250, 6133, 4134, 
              7251, 6132, 5677, 4337, 6977, 6978, 6974, 6976, 6975}],
             RowBox[{"-", "18.5`"}]],
            Annotation[#, -18.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4008, 6573, 5989, 5991, 7186, 5990, 4293, 5647, 5649, 
              5650, 6574, 4009, 5227, 6583, 6582, 5483, 4473, 5061, 5060, 
              6928, 6000, 6001, 6845, 6844, 4304, 5865, 4478, 6005, 6851, 
              4311, 6852, 5870, 5874, 5873, 6008, 5068, 5492, 5491, 5490, 
              4492, 4942, 4105, 6601, 6600, 5493, 4493, 6014, 4320, 6030, 
              4502, 6029, 6943, 6942, 4109, 4030, 6607, 5501, 4503, 6031, 
              4326, 6046, 4515, 6946, 4514, 7265, 6689, 6690, 7035, 4674, 
              7227, 5799, 5800, 4251, 5411, 6807, 6806, 6805, 4794, 4796, 
              7233, 4795, 6626, 6625, 6830, 6624, 6788, 6787, 4238, 5334, 
              4683, 7040, 4684, 5191, 5190, 4178, 5964, 5965, 5963, 5962, 
              5704, 4391, 7072, 7073, 7070, 4736, 7071, 6388, 6389, 5536, 
              5535, 5534, 4553, 4956, 4128, 6636, 6635, 5538, 4554, 6106, 
              4342, 6139, 4570, 6138, 6984, 6983, 4136, 4053, 6644, 5547, 
              4571, 6140, 4350, 6178, 4589, 6177, 4588, 6996, 4143, 4964, 
              4591, 6179, 4590, 6180, 5114, 4068, 5905, 5906, 5903, 4356, 
              5904, 6661, 5900, 4355, 5897, 5899, 5898, 4067, 5112, 5113, 
              4587, 6176, 4586, 6995, 5261, 6171, 6172, 6170, 6169, 5684, 
              4345, 6174, 6175, 6173, 6994, 6993}],
             RowBox[{"-", "22.200000000000003`"}]],
            Annotation[#, -22.200000000000003`, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4010, 5856, 5857, 5648, 4294, 5992, 4467, 5479, 6577, 
              5859, 4296, 5651, 5653, 4097, 4016, 5863, 5864, 5658, 5659, 
              4098, 4019, 5871, 5872, 5664, 5665, 4101, 4023, 6587, 5876, 
              4315, 5666, 5668, 6016, 4495, 6015, 4494, 5070, 4321, 6034, 
              4505, 6032, 4504, 6944, 4110, 6609, 6608, 5502, 4506, 6035, 
              4327, 6049, 4517, 6047, 4516, 6947, 4115, 4946, 4519, 6045, 
              4518, 6050, 4797, 5519, 5518, 5517, 5843, 5842, 5882, 5966, 
              4210, 5736, 5735, 4419, 6886, 4421, 6884, 4420, 6885, 4120, 
              4922, 4921, 5459, 4271, 5022, 5774, 4239, 5363, 5364, 4862, 
              7202, 4211, 6390, 6335, 6109, 7275, 6110, 6107, 4555, 6108, 
              5092, 5549, 5548, 5537, 4572, 4960, 4137, 6646, 6645, 5551, 
              4573, 6141, 4351, 6182, 4593, 6181, 4592, 6997, 4144, 6655, 
              6654, 5562, 4594, 6183, 4358, 5115, 5576, 5575, 4615, 7006, 
              6225, 5693, 4357, 5909, 6224, 6223, 4152, 6704, 6222, 5692, 
              5908, 5907, 6221, 4151, 7252, 6220, 5691, 5902, 5901, 6219, 
              4150, 6703, 6218, 5689, 4354, 5896, 4611, 6213, 4612, 6214, 
              4353, 5685, 5895, 5894, 6217}],
             RowBox[{"-", "25.900000000000002`"}]],
            Annotation[#, -25.900000000000002`, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4066, 5111, 7005, 4613, 6215, 4614, 5131, 5130, 6670, 
              6669, 5910, 4365, 5690, 4366, 6257, 6258, 6256, 4630, 5133, 
              4367, 5355, 4976, 4977, 4632, 5134, 4368, 5357, 4978, 6261, 
              4633, 6260, 6671, 5911, 4369, 5694, 5913, 6705, 5912, 4073, 
              5135, 6228, 6229, 6227, 4617, 6230, 4971, 6226, 4616, 5561, 
              5577, 5578, 5116, 5117, 4596, 5566, 6656, 4966, 4965, 6184, 
              4595, 5550, 5563, 5564, 5103, 6146, 4575, 6145, 6148, 6147, 
              4961, 5262, 5263, 5094, 7231, 5093, 4049, 4129, 7089, 4758, 
              6429, 4759, 5366, 5365, 6774, 6773, 5756, 6431, 6430, 7147, 
              6904, 6905, 6770, 6771, 6772, 4785, 7101, 7103, 7102, 4537, 
              5520, 5673, 5674, 6627, 6628, 5671, 7249, 5672, 5209, 7284, 
              4446, 5443, 4447, 7109, 7110, 7107, 4798, 7108, 5244, 7182, 
              4897, 7181, 4898, 5462, 5461, 5469, 4909, 6553, 4910, 6552, 
              4899, 6053, 4522, 6048, 4523, 6054, 4947, 4948, 4520, 6051, 
              4521, 5078, 5077, 6037, 4508, 6033, 4509, 6038, 4944, 6036, 
              4507, 5503, 5504, 4031, 5071, 6017, 5878, 5879, 5667, 4316, 
              5877, 4481, 6009, 4482, 6010, 4939, 5345, 4312, 5064, 5487, 
              4938, 4937, 5343, 4305, 5063, 5486, 5485, 6003, 6002, 4298, 
              5652, 4297, 5860, 4469, 5994, 5058, 5059, 4466, 5478, 6575, 
              6576, 5057, 5229}],
             RowBox[{"-", "29.6`"}]],
            Annotation[#, -29.6, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4628, 6216, 4629, 6255, 4364, 5348, 5349, 4975, 4158, 
              5292, 5132, 7013, 7014, 6259, 4631, 6363, 6364, 6362, 4716, 
              7060, 4084, 6366, 4718, 6365, 4719, 6367, 4717, 5356, 5720, 
              5719, 4720, 4933, 4085, 6369, 4722, 6368, 4723, 6370, 4721, 
              6372, 4725, 6371, 4724, 7061, 4086, 6373, 6374, 6266, 4634, 
              6265, 4635, 4979, 4159, 7015, 7016, 6262, 6264, 6263, 4370, 
              5282, 4371, 5136, 5595, 4074, 5137, 6236, 6237, 6235, 6239, 
              6238, 4972, 6232, 4618, 5565, 5579, 5580, 5118, 6189, 6190, 
              6188, 6192, 6191, 4967, 5267, 5268, 5105, 5104, 4054, 4138, 
              6985, 6986, 6142, 4574, 7279, 6144, 6765, 7077, 7078, 5738, 
              5737, 4213, 5367, 4214, 6766, 7080, 7079, 4737, 7270, 6789, 
              4240, 5775, 7217, 4422, 6887, 6888, 5972, 7203, 4212, 5973, 
              5023, 5432, 4863, 5453, 4864, 6523, 6524, 6522, 4886, 7174, 
              5433, 5434, 4892, 7074, 7075, 7076, 5444, 7172, 7173, 7111, 
              4799, 6953, 6954, 5330, 4675, 5449, 4923, 4924, 6693, 4121, 
              6691, 6692, 6490, 7143, 4831, 7144, 4832, 5247, 5246, 4952, 
              6078, 6079, 5521, 5522, 4044, 5084, 6060, 4524, 6052, 4525, 
              6061, 4949, 6056, 6057, 5508, 5509, 4038, 5079, 5234, 5235, 
              5072, 4323, 5233, 4322, 6018, 6019, 5497, 6602, 4026, 4106, 
              4943, 4496, 5494, 5495, 5496, 6355, 6354, 5019, 6352, 4702, 
              6351, 4703, 6353, 4701, 6503, 4850, 6502, 4849, 7150, 5017, 
              5018, 4696, 5712, 6733, 5344, 4695, 6501, 4848, 6500, 4847, 
              7149, 5016, 6348, 4693, 6347, 6350, 6349, 4475, 5484, 6584, 
              4017, 5062, 4011, 4095, 4936, 5339, 5338, 6839, 6838, 5858, 
              6926, 6927, 4468, 5993, 5480}],
             RowBox[{"-", "33.300000000000004`"}]],
            Annotation[#, -33.300000000000004`, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            TooltipBox[LineBox[CompressedData["
1:eJwV011MzlEcwPHjAldU5K2ebHmvp9RTK8JwoYvcGBvWk+kFF9iYC1e0sVHX
ZlTmdbOZl2HpqkIucEFeGiWT5cJLXXidccfnXHz7nvM7v9/5nXP+T/lN+zft
mxBCmOzPjYwQFs8IYQma0IgX00Io5XFumxPCGK+fGUINTk0P4XJmCO+zQyg3
XjRLPTZYq+BEIoRa6zU5IazMCmG7eaV4mp+I94g/5Qq1nXoXWCuWt4SXciE3
yKm33z2+jz6MmyeQhy51RfJK5Ce5lIu5Ud4wVpk/jGN9HvFb/qWuUs+l8lLW
N5p/mx3CVueqs/7duBct1g+ItfIxnJB7nF+qHcAF8zbz85yfG0KG2j7nGfBO
R73ZGWsd+CJ+ja/jChbKXYBs8cfyB+W3yL9l7SbG4j78AHdRJDeJHPF++Sfl
tsYadOt9xPu3u9tV96gWW4fmiPhtNQfd4aLzfjUe5Q7fcFs8H2dxO2dwJoqs
T+Up6NJzSL+/9vqDVj1acMI+883PqZvHZ7nPe1XzRP7gTP+ce5R/2i/EbyP+
mwviO9t3C1L2GbH/hPht9SuzlsJkebWcjrlokFuPKvkf5U+S3x9/n3EN2fIP
czMOYY/c3VgrP0tuJkbkJ5zvtPMmnekUF8ZvyBdRZnyJU9wgN406FJhneL8s
ZOKON/2hR6O90967FlX6l8dvJZbHc80TnMcjeq/hnbwLZeLLzVO8jIfE7uqx
gqvi/wOe41m8I5J6FmKHcamaSjUlXMGDYj1qe5EwzxHP5Vxeze+sN8Rzci1e
41WsEet0j17uVrvZHbbgk9hn7PVOg+74BkMYxn9ADJG4
              "]],
             RowBox[{"-", "37"}]],
            Annotation[#, -37, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4681, 6341, 4682, 6496, 6497, 5703, 6717, 6718, 6715, 
              6716, 6495, 7145, 4836, 7146, 4837, 5302, 5301, 4986, 6311, 
              4656, 5596, 5618, 5619, 5146, 5147, 4639, 5599, 6674, 6675, 
              4161, 7019, 4637, 6270, 4638, 6272, 4373, 6244, 4621, 5583, 
              6663, 6664, 4154, 7009, 4619, 6242, 4620, 5120, 5119, 6198, 
              4599, 6186, 4600, 6199, 4968, 5269, 5270, 5108, 5107, 4056, 
              4139, 7090, 7091, 6432, 4763, 6433, 4739, 5372, 6775, 5759, 
              4766, 6436, 4838, 5967, 4439, 5757, 5758, 7104, 4786, 6792, 
              6793, 5776, 7224, 4556, 6111, 6112, 5676, 7190, 4050, 4130, 
              5675, 7250, 5210, 7280, 5211, 5446, 5445, 4448, 6471, 4801, 
              6470, 4800, 7112, 5251, 5252, 4900, 6536, 6113, 6114, 5463, 
              5470, 5471, 4911, 6555, 6554, 4901, 6537, 6539, 6538, 4541, 
              5525, 6629, 6630, 4123, 6957, 4539, 6082, 4540, 6084, 4330, 
              6066, 4528, 5512, 6615, 6616, 4117, 6950, 4526, 6064, 4527, 
              5081, 5080, 6043, 4512, 6040, 4513, 6044, 4945, 5237, 5238, 
              5076, 5075, 4027, 4107, 5669, 4729, 5728, 6754}],
             RowBox[{"-", "40.7`"}]],
            Annotation[#, -40.7, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4733, 7230, 4734, 4732, 6761, 6762, 4206, 5352, 5425, 
              5353, 5354, 4731, 5361, 4859, 7238, 4860, 4858, 7152, 7153, 
              6509, 6508, 4828, 5617, 6680}],
             RowBox[{"-", "40.7`"}]],
            Annotation[#, -40.7, "Tooltip"]& ], 
           TagBox[
            
            TooltipBox[
             LineBox[{4844, 6499, 4845, 5341, 5340, 5709, 5708, 5707, 6345, 
              6344, 6498, 6510, 4205, 5004, 5458, 5457, 4843, 6507, 4853, 
              5726, 6748}],
             RowBox[{"-", "40.7`"}]],
            Annotation[#, -40.7, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4087, 4169, 5300, 5148, 5149, 5304, 5303, 4987, 6314, 
              6315, 5598, 5620, 4088, 5150, 6278, 4641, 6271, 4642, 6279, 
              4980, 6275, 4640, 5582, 5600, 4075, 5138, 6245, 6246, 6243, 
              6248, 6247, 4973, 5284, 4360, 5121, 5568, 4062, 4146, 7000, 
              7001, 6195, 4598, 6197, 6393, 7081, 4738, 5370, 5739, 4215, 
              5373, 5741, 5740, 6394, 4740, 7232, 5777, 6794, 7218, 7219, 
              6890, 6891, 4424, 6889, 5974, 5975, 5977, 5976, 5025, 5026, 
              4803, 5802, 5803, 5804, 5369, 5435, 6533, 4893, 6532, 4894, 
              5448, 5447, 4887, 7241, 4888, 4802, 7113, 7115, 7114, 4676, 
              5450, 4925, 4926, 6696, 6697, 6694, 6695, 6492, 6494, 6493, 
              4833, 6491, 5883, 5884, 5255, 4957, 6117, 6118, 5524, 5539, 
              4051, 5089, 6090, 6091, 6083, 4543, 6092, 4953, 6087, 4542, 
              5511, 5526, 5527, 5085, 6067, 6068, 6065, 6070, 6069, 4950, 
              5242, 5243, 5083, 5082, 4033, 4112, 5236}],
             RowBox[{"-", "44.400000000000006`"}]],
            Annotation[#, -44.400000000000006`, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4529, 5241, 4118, 4040, 6617, 5514, 6072, 6071, 4331, 
              6088, 6089, 6085, 6959, 6958, 4124, 4046, 6631, 5529, 6094, 
              6093, 4335, 6119, 6120, 6115, 4557, 6971, 4131, 5541, 5540, 
              6475, 4558, 6473, 6474, 6472, 4804, 6527, 6528, 6526, 4889, 
              7176, 5253, 5254, 5030, 7240, 5029, 5095, 5192, 5193, 5006, 
              5005, 5368, 4423, 5945, 5946, 5944, 6699, 6698, 4140, 4057, 
              7191, 5680, 5552, 5779, 5778, 4242, 5371, 6512, 6513, 6511, 
              4865, 6392, 6391, 6337, 6201, 7204, 6202, 6196, 4601, 6200, 
              5122, 5123, 4623, 5124, 4622, 5283, 4155, 6666, 6665, 5585, 
              6250, 6249, 4374, 6276, 6277, 6273, 7021, 7020, 4162, 4076, 
              6676, 5602, 6281, 6280, 4383, 6316, 6317, 6312, 7028, 7027, 
              4170, 5622, 5621}],
             RowBox[{"-", "48.1`"}]],
            Annotation[#, -48.1, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4658, 6313, 4659, 6322, 4988, 6319, 4657, 5601, 5623, 
              5624, 5151, 6287, 4644, 6274, 4645, 6288, 4981, 6284, 4643, 
              5584, 5603, 4077, 5139, 5288, 5289, 5125, 4362, 5285, 4361, 
              6203, 6204, 5570, 5686, 6658, 6702, 6438, 6437, 6439, 7093, 
              4767, 6440, 4768, 5760, 4224, 5171, 4225, 6442, 6443, 6441, 
              4770, 5397, 5396, 4226, 5394, 5395, 4769, 7094, 7096, 7095, 
              4223, 6460, 6151, 6152, 6150, 4578, 6153, 5681, 5682, 5027, 
              7116, 5028, 4806, 5032, 7278, 5031, 4252, 5256, 4253, 5806, 
              5805, 4805, 5553, 4058, 5096, 6126, 6127, 6116, 4560, 6128, 
              4958, 6123, 4559, 5528, 5542, 5543, 5090, 6096, 4545, 6086, 
              4546, 6097, 4954, 6095, 4544, 5513, 5530}],
             RowBox[{"-", "51.800000000000004`"}]],
            Annotation[#, -51.800000000000004`, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4089, 4171, 7029, 7030, 6318, 6321, 6320, 4384, 6289, 
              6290, 5605, 6677, 4078, 4163, 7022, 7023, 6282, 6286, 6285, 
              4375, 6251, 4625, 5589, 6667, 6668, 4156, 6399, 4974, 4624, 
              5569, 5586, 5587, 6398, 6397, 6404, 4744, 6403, 6406, 6405, 
              4392, 5410, 4996, 4997, 5197, 5196, 4179, 5194, 5195, 4686, 
              6342, 4685, 7041, 5335, 4243, 5780, 5781, 5845, 6831, 5844, 
              5571, 5572, 4808, 7117, 4809, 7118, 4807, 5807, 6809, 6810, 
              5412, 4254, 5809, 5808, 7037, 7038, 4678, 7036, 6701, 6700, 
              6987, 4579, 6988, 4580, 6155, 4343, 6129, 6130, 5545, 6637, 
              4052, 4132, 6972, 6973, 6121, 6125, 6124, 4336, 6098, 6099, 
              5531, 6632, 4047, 4125, 5249}],
             RowBox[{"-", "55.5`"}]],
            Annotation[#, -55.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4561, 5257, 4133, 4959, 4563, 6122, 4562, 6131, 5097, 
              5555, 5554, 5544, 4581, 6158, 4962, 6163, 6164, 6154, 6162, 
              6161, 6476, 4064, 5947, 5948, 5683, 4425, 5983, 4450, 6911, 
              4449, 6396, 4742, 6395, 4741, 5949, 4427, 5745, 6767, 4217, 
              5179, 4216, 5744, 4787, 5782, 5783, 5784, 5336, 4244, 5409, 
              6796, 6795, 5785, 4788, 5034, 5033, 6463, 4218, 5747, 5746, 
              4745, 6407, 5742, 5743, 5286, 5287, 4743, 6400, 6402, 6401, 
              5290, 5140, 4079, 5606, 5588, 6294, 6293, 4982, 6297, 4649, 
              6283, 4648, 6296, 5152, 5626, 5625, 5604, 4660, 4989, 4172, 
              5628, 5627}],
             RowBox[{"-", "59.2`"}]],
            Annotation[#, -59.2, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4059, 5098, 5100, 5259, 5258, 4344, 6159, 6160, 6156, 
              6990, 6989, 4141, 5557, 5556, 6540, 4582, 5264, 6811, 5811, 
              4810, 6477, 4927, 5176, 4273, 5037, 5110, 5109, 6906, 4440, 
              5968, 5970, 5969, 4426, 5398, 5399, 5178, 5177, 5180, 4147, 
              4063, 6462, 4789, 6461, 4790, 5403, 5402, 4245, 6447, 6448, 
              6445, 4772, 6446, 5332, 5333, 5291, 4627, 5374, 4626, 5141, 
              4376, 6295, 4647, 6291, 4646, 7024, 4164, 4080, 6678, 5608, 
              6299, 6298, 4385, 5305, 5306, 5154, 5153, 5307}],
             RowBox[{"-", "62.900000000000006`"}]],
            Annotation[#, -62.900000000000006`, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4663, 5155, 4664, 5308, 4990, 6325, 4661, 5607, 5629, 
              5630, 5156, 6303, 6304, 6292, 6307, 6305, 4984, 5293, 4377, 
              5143, 5590, 4070, 5696, 5695, 6444, 4771, 5761, 5762, 4228, 
              5172, 4229, 6451, 4775, 6450, 4776, 5405, 5404, 4230, 5400, 
              5401, 4774, 6449, 4773, 7097, 4227, 6464, 6206, 6207, 6205, 
              4602, 6208, 5687, 5688, 5035, 6912, 5036, 4451, 5810, 5812, 
              4255, 6478, 5273, 6542, 6543, 6541, 6545, 6544, 4902, 6556, 
              4912, 5573, 6659, 6660, 6546, 6547, 6165, 6168, 4584, 6157, 
              4585, 6166, 4963, 5266, 4583, 5099, 5558}],
             RowBox[{"-", "66.60000000000001`"}]],
            Annotation[#, -66.60000000000001, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4090, 4173, 7031, 7032, 6323, 4662, 6326, 4386, 5295, 
              4652, 5160, 5611, 4985, 6306, 4983, 5294, 4650, 5142, 5609, 
              5610, 6409, 6408, 6414, 4747, 6413, 6416, 6415, 4393, 5183, 
              4998, 4999, 5201, 5200, 4180, 5198, 5199, 4687, 6343, 7043, 
              7042, 5182, 4246, 5786, 5787, 5591, 5846, 5592, 5593, 4453, 
              6913, 6915, 6914, 4452, 6481, 5813, 5814, 5212, 5817, 5816, 
              5815, 6480, 6479, 4605, 5464, 4606, 6210, 6209, 4903, 5271, 
              4603, 5465, 4604, 5272, 6558, 6557, 4904, 6548, 5893, 5892, 
              6167, 6992, 6991, 5560, 6649, 4060, 4142, 5265}],
             RowBox[{"-", "70.3`"}]],
            Annotation[#, -70.3, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4666, 6324, 6328, 6327, 4991, 5309, 5310, 5159, 5632, 
              5157, 4081, 4165, 7025, 7026, 6300, 4651, 6302, 6412, 7082, 
              7083, 5380, 5379, 4220, 5173, 4221, 5750, 5749, 5791, 5790, 
              4247, 5788, 5789, 5748, 5978, 7206, 4219, 5979, 5038, 7121, 
              7122, 6482, 4811, 5378, 5377, 5436, 4866, 5375, 4746, 5413, 
              7175, 7119, 7120, 7003, 7004, 5331, 4679, 5451, 5452, 4928, 
              4065, 4148, 5422, 5423, 5420, 4834, 5276, 4835, 5279, 5278, 
              4969, 6211, 6212, 5559, 5574}],
             RowBox[{"-", "74"}]],
            Annotation[#, -74, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4608, 5275, 4149, 6483, 4970, 4610, 5421, 4609, 5277, 
              4812, 6531, 4891, 6529, 4890, 5274, 4607, 5043, 7002, 5042, 
              5126, 5202, 5203, 5008, 5007, 5376, 4428, 5950, 5951, 6893, 
              6892, 4429, 7263, 6706, 6707, 4930, 4929, 5440, 5441, 5040, 
              5039, 4248, 5381, 5437, 4867, 6520, 4868, 6411, 6410, 6338, 
              6309, 7205, 6310, 6301, 4653, 6308, 5161, 5633, 5631, 5158, 
              4665, 4992, 4174, 5635, 5634}],
             RowBox[{"-", "77.7`"}]],
            Annotation[#, -77.7, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4072, 7123, 7124, 6484, 4813, 5281, 5280, 4256, 5818, 
              5044, 6530, 4275, 5215, 4274, 5041, 5594, 6252, 4363, 5127, 
              5764, 4231, 5181, 6777, 6776, 6907, 4441, 6909, 6908, 7221, 
              7220, 6781, 5767, 6832, 6833, 4279, 5848, 4278, 5847, 4778, 
              6454, 6779, 6778, 4872, 7155, 4779, 6340, 7039, 4680, 6339, 
              5699, 5613, 5612, 5165, 4654, 5296, 4387, 5311, 5312, 5167, 
              5166, 5317}],
             RowBox[{"-", "81.4`"}]],
            Annotation[#, -81.4, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4668, 5168, 4669, 5318, 4993, 5313, 5314, 5164, 5637, 
              5162, 4082, 4166, 7098, 7099, 6452, 4777, 6453, 4753, 5384, 
              6780, 5766, 4780, 6455, 4839, 5971, 4442, 5763, 5765, 6798, 
              7271, 4791, 7105, 6799, 7225, 7226, 7010, 7011, 7012, 6253, 
              6254, 5697, 7192, 4071, 4157, 7253, 7254, 7195, 5213, 6916, 
              5214, 5218, 5217, 4454, 5985, 5986, 5984, 4455, 6917, 5128, 
              5129, 5046, 4277, 5414, 4931}],
             RowBox[{"-", "85.10000000000001`"}]],
            Annotation[#, -85.10000000000001, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4276, 5045, 6708, 6857, 6859, 6858, 5821, 5820, 4258, 
              5216, 4257, 6812, 5819, 5823, 5822, 6814, 7222, 6813, 5824, 
              6834, 6835, 4281, 5849, 4280, 5980, 5981, 5792, 6797, 5960, 
              4432, 5957, 5959, 7276, 5958, 7106, 6465, 6466, 6425, 7084, 
              4756, 6426, 4754, 5385, 5755, 6428, 4755, 6424, 4750, 5638, 
              5636, 5163, 4667, 4994, 6420, 4175, 5640, 5639}],
             RowBox[{"-", "88.80000000000001`"}]],
            Annotation[#, -88.80000000000001, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4083, 5952, 5953, 5698, 4430, 5988, 4457, 6918, 4456, 
              6418, 4749, 6417, 4748, 5961, 4433, 5754, 6768, 6769, 5186, 
              4222, 5753, 4444, 5793, 5794, 5795, 5188, 4249, 5386, 6801, 
              6800, 5796, 4792, 7179, 7180, 7177, 4895, 7178, 5387, 5388, 
              4871, 6517, 4870, 7154, 6427, 4751, 6419, 5751, 5752, 5315, 
              5316, 4752, 6421, 6423, 6422, 5054}],
             RowBox[{"-", "92.5`"}]],
            Annotation[#, -92.5, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4288, 5382, 5383, 5056, 5055, 5169, 5207, 5208, 4869, 
              6514, 6516, 6515, 6521, 4250, 7161, 7162, 7160, 4879, 7163, 
              4878, 7159, 7272, 4896, 7244, 7242, 7243, 4167, 6709, 6710, 
              5954, 5956, 5955, 4431, 5189, 5009, 5010, 5206, 5205, 5144, 
              5048, 5987, 5049, 5185, 5184, 4932}],
             RowBox[{"-", "96.2`"}]],
            Annotation[#, -96.2, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4282, 5047, 5614, 6489, 4378, 5145, 5770, 4232, 5187, 
              5769, 5768, 6456, 4443, 5204, 6784, 5773, 6836, 6837, 4287, 
              5851, 4286, 5850, 4445, 5982, 4233, 5771, 5170, 4388, 5407, 
              4995, 4935, 4934}],
             RowBox[{"-", "99.9`"}]],
            Annotation[#, -99.9, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4091, 4176, 7100, 4781, 6457, 4782, 5359, 5358, 6783, 
              6782, 5772, 4783, 6459, 4840, 5337, 4688, 5406, 4875, 7156, 
              7158, 7157, 4873, 6518, 4874, 6519, 4827, 5616, 5701, 5702, 
              6679, 6712, 4168, 6711, 5700}],
             RowBox[{"-", "103.60000000000001`"}]],
            Annotation[#, -103.60000000000001`, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4851, 6458, 4852, 6506, 4726, 6381, 6382, 6380, 6387, 
              6386, 7065, 6376, 6377, 6375, 6379, 6378, 4655, 6329, 4670, 
              5615, 5641}],
             RowBox[{"-", "107.30000000000001`"}]],
            Annotation[#, -107.30000000000001`, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4195, 5360, 4196, 5725, 5724, 5732, 5731, 6760, 7062, 
              6758, 6759, 5729, 5730, 5723, 6747, 4194, 5297, 5319}],
             RowBox[{"-", "111"}]],
            Annotation[#, -111, "Tooltip"]& ]}, 
          {GrayLevel[0], Opacity[0.4], 
           TagBox[
            
            TooltipBox[
             LineBox[{4204, 5920, 5921, 5918, 4389, 5919, 5923, 5922, 6330}],
             RowBox[{"-", "114.7`"}]],
            Annotation[#, -114.7, "Tooltip"]& ]}, {}, {}}}],
       AspectRatio->1,
       BaseStyle->{FontFamily -> "Times", FontSize -> 12},
       DisplayFunction:>Identity,
       Frame->True,
       PlotLabel->FormBox[
        "\"\[Mu]=7 \[Alpha]=0.32 \[Epsilon]=10 PIN=0.10\"", TraditionalForm],
       PlotRange->{{0, 50}, {0, 50}},
       PlotRangeClipping->True,
       PlotRangePadding->{
         Scaled[0.02], 
         Scaled[0.02]}], {576., -576.}, 
      ImageScaled[{0.5, 0.5}], {360., 360.}]}}, {}},
  ContentSelectable->True,
  PlotRangePadding->{6, 5}]], "Output",
 CellChangeTimes->{3.3896128960512*^9, 3.389622730865*^9, 3.3899915583656*^9},
 
 CellLabel->"Out[260]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"PIN", "\[Equal]", 
       FractionBox[
        RowBox[{"\[Alpha]", " ", "\[Mu]"}], 
        RowBox[{
         RowBox[{"\[Alpha]", " ", "\[Mu]"}], "+", 
         RowBox[{"2", " ", "\[Epsilon]"}]}]]}], "/.", "\[InvisibleSpace]", 
      RowBox[{"PIN", "\[Rule]", ".4"}]}], "/.", "\[InvisibleSpace]", 
     RowBox[{"\[Epsilon]", "\[Rule]", "10"}]}], "/.", 
    RowBox[{"\[Alpha]", "\[Rule]", "1"}]}], ",", "\[Mu]"}], "]"}]], "Input",
 CellLabel->"In[261]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"\[Mu]", "\[Rule]", "13.333333333333334`"}], "}"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.3896128962422*^9, 3.3899915586626*^9},
 CellLabel->"Out[261]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"PIN", "\[Equal]", 
       FractionBox[
        RowBox[{"\[Alpha]", " ", "\[Mu]"}], 
        RowBox[{
         RowBox[{"\[Alpha]", " ", "\[Mu]"}], "+", 
         RowBox[{"2", " ", "\[Epsilon]"}]}]]}], "/.", "\[InvisibleSpace]", 
      RowBox[{"PIN", "\[Rule]", ".4"}]}], "/.", "\[InvisibleSpace]", 
     RowBox[{"\[Epsilon]", "\[Rule]", "10"}]}], "/.", 
    RowBox[{"\[Alpha]", "\[Rule]", ".9"}]}], ",", "\[Mu]"}], "]"}]], "Input",
 CellLabel->"In[262]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"\[Mu]", "\[Rule]", "14.814814814814815`"}], "}"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.3896128962952003`*^9, 3.3899915587376003`*^9},
 CellLabel->"Out[262]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"ggg", "[", "\[Mu]_", "]"}], ":=", 
   RowBox[{"gg", "[", 
    RowBox[{"\[Mu]", ",", 
     FractionBox["2.2222222`", "\[Mu]"], ",", "10"}], "]"}]}], ";"}], "\n", 
 RowBox[{"Show", "[", 
  RowBox[{"GraphicsGrid", "[", 
   RowBox[{"{", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"gg", "[", 
       RowBox[{"13.3`", ",", "1", ",", "10"}], "]"}], ",", 
      RowBox[{"gg", "[", 
       RowBox[{"14.8`", ",", "0.9`", ",", "10"}], "]"}]}], "}"}], "}"}], 
   "]"}], "]"}]}], "Input",
 CellLabel->"In[263]:="],

Cell[BoxData[
 GraphicsBox[{{}, {InsetBox[
     GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJyMXXdgFcXzD70qECAQCC0QIBAgvSdvX0sBAiEgCAooPRRBihQFRGqAiFTp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       "], {{
         {RGBColor[0.293416, 0.0574044, 0.529412], EdgeForm[None], 
          GraphicsGroupBox[{
            PolygonBox[{{8155, 8156, 3482}, {8464, 6472, 2052}, {7892, 6556, 
             2117}, {3470, 3469, 450}, {3978, 3469, 1292}, {8139, 8138, 
             3470}, {5862, 6391, 1652}, {6408, 6407, 1999}, {4194, 4183, 
             3978}, {1491, 1314, 1292}, {4163, 4140, 450}, {6594, 5558, 
             1491}, {4139, 4163, 450}, {5327, 6593, 1292}, {5316, 7892, 
             2117}, {4144, 4185, 508}, {5391, 8413, 508}, {4143, 4146, 508}, {
             4159, 4193, 1314}, {4180, 4179, 2118}, {4177, 4155, 1999}, {5116,
              7039, 999}, {6556, 6567, 2117}, {4140, 4161, 450}, {4167, 4158, 
             1189}, {5069, 6477, 952}, {4166, 4159, 1314}, {3469, 3824, 
             1292}, {4138, 4165, 999}, {4141, 4187, 3482}, {4158, 4142, 
             1189}, {4190, 4138, 999}, {6471, 8464, 2052}, {4145, 4170, 
             1615}, {4156, 4137, 952}, {5765, 5766, 1615}, {6386, 5862, 
             1652}, {7627, 7628, 2986}, {1491, 3247, 1314}, {4150, 4195, 
             678}, {6237, 5270, 1189}, {7042, 5116, 999}, {4748, 5947, 678}, {
             5558, 4534, 1491}, {6477, 5071, 952}, {4195, 4149, 678}, {5859, 
             5069, 952}, {5937, 5391, 508}, {8318, 8317, 3653}, {4535, 5937, 
             508}, {4153, 4150, 678}, {5328, 8478, 3824}, {4171, 4176, 
             1652}, {4137, 4154, 952}, {4148, 4171, 1652}, {4185, 4143, 
             508}, {5861, 6386, 1652}, {4750, 8209, 3546}, {5947, 5516, 
             678}, {3247, 3246, 1314}, {6762, 5269, 1189}, {5969, 5968, 
             1732}, {5270, 6762, 1189}},
             VertexColors->None], 
            PolygonBox[{{6390, 8577, 4147, 4174}, {8415, 8579, 4186, 4192}, {
             6566, 8139, 3470, 450}, {6569, 5316, 2117, 2130}, {6567, 6566, 
             450, 2117}, {6472, 5317, 2118, 2052}, {7039, 8318, 3653, 999}, {
             8137, 5328, 3824, 3469}, {7834, 4294, 4136, 4182}, {4149, 4189, 
             3546, 678}, {8478, 5327, 1292, 3824}, {4749, 8580, 4188, 4151}, {
             4164, 4166, 1314, 3246}, {4193, 4160, 1292, 1314}, {4191, 4190, 
             999, 3653}, {8209, 4748, 678, 3546}, {4162, 4180, 2118, 2130}, {
             7628, 7042, 999, 2986}, {8138, 8137, 3469, 3470}, {5968, 5765, 
             1615, 1732}, {6407, 6471, 2052, 1999}, {7909, 7627, 2986, 
             3246}, {6593, 6594, 1491, 1292}, {4161, 4178, 2117, 450}, {5317, 
             6569, 2130, 2118}, {4183, 4184, 3469, 3978}, {7910, 7909, 3246, 
             3247}, {4179, 4157, 2052, 2118}, {4160, 4194, 3978, 1292}, {4181,
              4164, 3246, 2986}, {4165, 4181, 2986, 999}, {4184, 4139, 450, 
             3469}, {4534, 7910, 3247, 1491}, {4170, 4152, 1732, 1615}, {4178,
              4162, 2130, 2117}, {4157, 4177, 1999, 2052}},
             VertexColors->None], 
            PolygonBox[{{6236, 6237, 1189, 4142, 4172}, {5766, 5560, 4169, 
             4145, 1615}, {5516, 5518, 4168, 4153, 678}, {8156, 6236, 4172, 
             4141, 3482}, {8579, 8155, 3482, 4187, 4186}, {5269, 7834, 4182, 
             4167, 1189}, {6391, 6390, 4174, 4148, 1652}, {8413, 8415, 4192, 
             4144, 508}, {5560, 4535, 508, 4146, 4169}, {8580, 4750, 3546, 
             4189, 4188}, {6347, 6408, 1999, 4155, 4173}, {5518, 5969, 1732, 
             4152, 4168}, {8578, 5861, 1652, 4176, 4175}, {8577, 5859, 952, 
             4154, 4147}, {5071, 6347, 4173, 4156, 952}, {8317, 4749, 4151, 
             4191, 3653}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.3133100181646372, 0.09199472491024652, 0.5573754491696493], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVMtLVHEYPT10Rh0H7zg6TlOOt/oLgoIe0CqUIBtdtSiCXtB70dpKKxeh
Qlj0QKGFJVZEQagEBUlPR7BFi56kla1dFdGmczq/xcf97u93vvOd73FvvO9k
54mlAA7QltFqK4CdSaCGz2MR8KQJuLYE2Ex/kf57Yt5mgTdFY1vzQCJvTIGY
DmIeEVNK+n7vcqAvA0QtwL1KoKcAzKwCJquAJOPKxN9NAAPEnI+BUYqZbAB6
G4F3jB+g30d7WOH7/owxF3hfygFXmbcjZ5wwDSmgOuiXyZeGbc3AdurO8L5M
7bfzrvMS+TYx/gG1fahznapRXDoTf23gEs8gOXZR8wj9LnIM1gOnqP8Fz9v4
vsjzGd7vKLiu/9xFc/2g5qcriKcfk+9WNfCX/PNpYC5tbVPkeEbbT21l9mmh
2Zjd5PxcdK4bPLvCfAdT7kdXi3vyjRzXU9aguJ8htp1ajrOu3irnUC7Nq8Q8
H7PWcpj3Qytd7zhxYzXUQT+bsj7FiVs5suE5H/wh5hmmHUo5dqTaerqZ9yxt
osq5NX+da+az9e7PWs50De1P2jvRV+u8eld/pO0T6z4Xu+fKpxlrHq94/rro
mAX2tif27KRL+tQH7el02NXvxHTH7v+eJvdU/fzF52/aFP3T7OWZguv4Sv8L
7T65JiLvpXZyLtQufu3RaNgl1a06VeNWzr0/ci2qYUvYn6jBO6RatEeXg3+T
vd+Qc49Ut3qh2ann2VCvdme+0fq1H/rGxKtZCa9cHdTSmffea55HIs9UcdNN
jn3JnMPEtBKzPufcyiu9G3Pmaed9O3GJpLlaw3eknRqPrO0578fCzs/Sb46t
qU0zD73S/+No5H0TVt+J8I/r/L/Qv0I7uDp2ny5yn9dRw51K16Yz5dIOjYca
dV4Kev4B6vu2eQ==
             "],
             VertexColors->None], 
            PolygonBox[{{4459, 7749, 3090, 408}, {4754, 5517, 1457, 1713}, {
             7043, 8135, 3468, 2488}, {8579, 8415, 3758, 1188}, {8136, 5951, 
             449, 3468}, {4796, 6990, 2438, 2724}, {5968, 5969, 1457, 407}, {
             5945, 7041, 2487, 449}, {8577, 6390, 1986, 1651}, {8482, 8484, 
             3833, 1313}, {6989, 4460, 408, 1963}, {6556, 7892, 3225, 1291}, {
             5270, 6237, 1907, 3174}, {8209, 4750, 679, 1708}, {8464, 6471, 
             408, 3808}, {7628, 7627, 507, 2488}, {8414, 5393, 1188, 3758}, {
             8317, 8318, 2486, 679}, {7040, 7038, 2486, 2487}, {8483, 4797, 
             2725, 3833}, {6593, 5327, 2128, 2147}, {4294, 7834, 3173, 180}, {
             6472, 8464, 3808, 1264}, {5094, 5860, 1651, 1986}, {8478, 5328, 
             1293, 2128}, {4797, 7043, 2488, 2725}, {5203, 6480, 1313, 
             1119}, {7892, 5316, 1264, 3225}, {5011, 7836, 3174, 1907}, {6482,
              5318, 2119, 2058}, {4534, 5558, 1340, 507}, {5947, 4748, 1708, 
             1713}, {4461, 5941, 1708, 2438}, {5517, 4753, 1712, 1457}, {4458,
              6984, 1492, 407}, {6567, 6556, 1291, 2127}, {7039, 5116, 2985, 
             2486}, {5558, 6594, 2147, 1340}, {5268, 6525, 1907, 1188}, {5516,
              5947, 1713, 1457}, {5942, 4754, 1713, 1708}, {8112, 4457, 407, 
             1712}, {5766, 5765, 407, 1492}, {7835, 4282, 180, 3173}, {5559, 
             4746, 1706, 1492}, {6407, 6408, 1963, 408}, {6387, 5863, 5862, 
             6386}, {5937, 4535, 1492, 1706}, {5116, 7042, 2488, 2985}, {8156,
              8155, 1188, 1907}, {5198, 6348, 1963, 2055}, {5195, 6481, 2058, 
             3093}, {8138, 8139, 2127, 1293}, {5071, 6477, 2055, 1963}, {7038,
              4796, 2724, 2486}, {5194, 7752, 3092, 3090}, {6557, 5203, 1119, 
             2119}, {7752, 5195, 3093, 3092}},
             VertexColors->None], 
            PolygonBox[{{6763, 7835, 3173, 5269, 6762}, {5863, 5094, 1986, 
             6391, 5862}, {6384, 6387, 6386, 5861, 216}, {5392, 8414, 3758, 
             8413, 5391}, {7837, 6763, 6762, 5270, 3174}, {5070, 6478, 2055, 
             6477, 5069}, {6346, 5070, 5069, 5859, 1651}, {5938, 5392, 5391, 
             5937, 1706}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.34003484107780463`, 0.13846197451688821`, 0.5949404207563667], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVE1sjFEUPdpqO02npt905pvpTDpGrVhIJIgEZUGbaTtpx0pqw0JCaVdi
gQkpoRWm1W78zIJEQjcVIWSUmUhQIlaGpAuxaEisiETtOMd9i5v3vvvuvefc
vy99cDQ3UgPgEKWWcjoOzLQB6xuBQd57eR+qAzb5wLFW4NoK4DDP+yngeBB4
3gSUKfcY5Lv7ls1cPfAoAHygfncHMEyfc/xebgH+UK7TJhQBNjLugZXA1zDw
LQoskESQuJMecJe+R+nXnwDO0zfP8wylSn2pyfD3kdsz2namgY/U9/I9kzCs
veReZKxAo2HqFC6IOUOfWdpPErdAm0KtcX1Yb/pYM20pP+vsrlN1EMbvFrO5
xBgDMeAO7/3EKvO+nzZPYvat+1PabPEttvAVRxzOkuNW3+o0QZvHEYuzlnms
SVudemhzhPmPBQxz2dWth/XMuL6oxuKs+Ius33y71bDC8wUlyPdSCLhBny/E
38OzmxJmTu+Z+1tynW0AssSa96y2qp3eFb9KXTHJPFjvX7T9QanQfoS8/kaN
zy2+3XZ9L9H+YtK+H9RbfzQbE9RdSJrtILlnyMEnxljC+qe6qjbSKeZA3Pqn
mil/4as+4hVz3DSbqsV/njUWWxyycctNeak/4575XuW52Tc+3fStJIzfduq2
+cb3JH1PUdYR9x19s66Peu/qMJs31GfiFj/Pvk27WVIflIfumi+96V52tRBu
jr5XyG20wWZ8PGlzvoHxX4ZsF9Sr4Xbr1wnGWApbT5eitie6X45YPoqp3DVH
irPDt5nTPLxOWX3FSb3Nu91ppe/n1YYlnKKbjS76LqQsj4Jne6iYnbT95Nl+
36TtdNj2Zc7NnWymPOMkPppTzatwd9F+J2VVs2Fq54Xb12Z7otpq/6ZcTPWw
6mJq/3y3e3rvi5jNK/rm4qZXbRbdf0MY+i+op/8Aq4a84Q==
             "],
             VertexColors->None], 
            PolygonBox[{{6474, 5196, 1116, 1962}, {5559, 6984, 2435, 677}, {
             4461, 6990, 725, 409}, {5943, 4751, 680, 724}, {6557, 5318, 1265,
              2057}, {7305, 7306, 2721, 680}, {5556, 8227, 3562, 409}, {6482, 
             6481, 1115, 1265}, {8483, 8484, 3831, 726}, {5202, 7893, 3226, 
             3096}, {4293, 6523, 2090, 1377}, {5863, 6387, 1984, 968}, {5338, 
             6480, 2057, 3832}, {7743, 5191, 1113, 1714}, {4824, 5949, 1714, 
             2751}, {5267, 5393, 1377, 2090}, {7104, 7745, 3087, 1113}, {7040,
              7041, 1711, 725}, {5194, 7749, 2436, 1115}, {5887, 8481, 3830, 
             410}, {8482, 5338, 3832, 3831}, {6526, 5008, 911, 2091}, {5946, 
             4753, 681, 1113}, {6985, 4821, 2747, 2435}, {6989, 6348, 1116, 
             2437}, {5452, 6761, 2272, 911}, {7744, 4820, 2746, 3087}, {7303, 
             4794, 2722, 2721}, {4823, 7329, 2750, 3562}, {4458, 7324, 2746, 
             2435}, {6847, 5858, 5857, 6846}, {7325, 4747, 677, 2747}, {4755, 
             5950, 724, 682}, {5011, 6525, 2091, 911}, {7754, 4462, 410, 
             3095}, {7329, 4824, 2751, 2750}, {7751, 5202, 3096, 3091}, {5198,
              6478, 1962, 1116}, {7328, 5941, 409, 2750}, {7893, 5201, 3095, 
             3226}, {7324, 4457, 3445, 2746}, {7304, 5556, 409, 2722}, {5938, 
             4746, 677, 1377}, {6986, 7750, 3091, 2436}, {8136, 8135, 726, 
             1716}, {4820, 6985, 2435, 2746}, {8112, 5946, 1113, 3445}, {8481,
              5888, 682, 3830}, {4493, 5951, 1716, 2485}, {4745, 5935, 1377, 
             677}, {6385, 5495, 968, 1984}, {6335, 6988, 2437, 1116}, {7837, 
             7836, 911, 2272}, {5942, 7328, 2750, 681}, {5945, 4493, 2485, 
             1711}},
             VertexColors->None], 
            PolygonBox[{{5858, 6345, 1962, 6346, 5857}, {6988, 6334, 6333, 
             4460, 2437}, {4412, 6385, 1984, 6384, 4411}, {4380, 6226, 1903, 
             6225, 4379}, {6233, 6526, 2091, 5268, 6232}, {6850, 4412, 4411, 
             4313, 2317}, {6334, 6987, 2436, 4459, 6333}, {6844, 6847, 6846, 
             5860, 968}, {6761, 4380, 4379, 4282, 2272}, {6524, 6233, 6232, 
             5267, 2090}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.3667596639909718, 0.1849292241235296, 0.6325053923430838], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVM9vTFEUPm311TTVdJ6ZeTNvxquJP0CEhIX6EaJmOjLMglTDQqKJTcVC
SDosbKp+tWLVBSJkQqQ2g4VoSxMRIhE/qmqBdimpsLBBwvflu4ubd+4993zf
Od859+UPHa0MNJrZYawmrHHP7DnWPOxTSbNdgdntZrO9abMe7EdgF7Jm732z
bwicj5lNtMqexJ0SfLUWswtxs0HYdfjfIOYd/L8b4M8AB8tbatYOvGs5nTcm
zKq4/71JeF9iwlyIieMD7ixi/9XZJxH7B3w1YN/zxEOOuZTZ6w6zV8h/ADkU
I7Mh+LpQx0dfGNfBOZkV1g74i7D3L9F+bJnwWcdUWrWc8YRxAHdKwOmKS58b
wJl1mHOdZp86FX8J9e1D7DHE3kHsZ9Rap8ahWX+ofS/81YzuvIC9kFLOZejw
NyWuPpwXnOYhOC9nhEOdGhLSqoJvN2o4iPsTOB+LSzti7E4Ih3EXXWxPVn1i
XTX4R3G+AfncbJWO1PABMPoz6gux7nrqBbHIQ62oB22evVxutj0S5hrEPkyr
lp/4boyrRyVwFYHZDK5p5PA0q96S822LaqH+i25OioFmgufs8/GkcIg3Hcpe
tdLsflLzdQ69GM6pjlHkc9ppO4W7j7HaYtL+h9N/K/J9ktVsVYCfi0tn8o27
+a97mn3azIs8zG0mku6sf3YF9AtUx3nMwtWc9OIMUgvqcCIpLZhzAXG3QmnI
uslFznze7F9Kszbj641Rc77FustnEDU961Afqccvh7kt0tywF5zNQqD7nHfO
KHNfjbNhX/PIWo+EqpfveEskrhHY6wPltAnfK77jhkbtbXoX1LXXzXY3ODdH
8j3C3aGc5ofYJTe37M/ZnLQrI7bsq8ZqUv8X6muB6uc5547/BepMnXYmpNWe
hHSn5nwT/B8Rfy1i1wXS/D8XxsNm
             "],
             VertexColors->None], 
            PolygonBox[{{5965, 5964, 225, 1448}, {5944, 4492, 2483, 1710}, {
             7035, 5557, 723, 2483}, {5858, 6847, 2316, 204}, {6326, 6473, 
             2053, 204}, {7743, 5948, 470, 3086}, {6845, 5849, 204, 2316}, {
             7104, 5191, 3086, 2529}, {4756, 5888, 1666, 1717}, {5940, 5005, 
             1902, 1707}, {5887, 4462, 2439, 1666}, {7103, 4822, 2749, 
             2529}, {4745, 6224, 1902, 1705}, {6475, 5066, 1958, 2054}, {4823,
              8227, 1489, 749}, {5952, 5500, 1448, 1717}, {6231, 5286, 2104, 
             1905}, {6541, 6542, 1905, 179}, {6543, 5008, 1905, 2104}, {4293, 
             5935, 1705, 179}, {5943, 4795, 1735, 1709}, {4687, 5952, 1717, 
             1666}, {5502, 4756, 1717, 1448}, {5200, 6479, 1666, 1117}, {7105,
              7103, 2529, 2530}, {7754, 5201, 1118, 2439}, {7325, 4821, 746, 
             1707}, {7751, 7750, 1114, 1118}, {5557, 4533, 506, 723}, {7305, 
             4752, 1710, 2723}, {6524, 6523, 179, 1905}, {5936, 6757, 2270, 
             1705}, {5948, 5949, 749, 470}, {5192, 7747, 1117, 1114}, {7748, 
             5193, 1114, 1958}, {7102, 8153, 3481, 2528}, {4505, 5939, 1707, 
             2527}, {6413, 4751, 1709, 2002}, {6986, 6987, 1958, 1114}, {6224,
              4747, 1707, 1902}, {4755, 5502, 1448, 1715}, {5961, 5962, 1006, 
             225}, {5005, 5936, 1705, 1902}, {8154, 5988, 470, 3481}, {5093, 
             5851, 5850, 5092}, {7303, 7306, 2723, 723}, {6335, 5197, 2054, 
             1958}, {4822, 7101, 2527, 2749}, {6474, 6345, 204, 2053}, {7744, 
             7745, 2529, 746}, {5988, 7106, 2530, 470}, {5121, 7639, 1710, 
             1006}, {4752, 6413, 2002, 1710}, {7304, 4794, 723, 1489}, {6896, 
             4378, 179, 2270}, {7326, 7327, 2528, 506}, {4795, 5950, 1715, 
             1735}},
             VertexColors->None], 
            PolygonBox[{{6473, 6329, 6328, 5196, 2053}, {5010, 6235, 1423, 
             5452, 5009}, {5451, 5007, 5006, 6226, 1423}, {6389, 5093, 5092, 
             5495, 1446}, {6544, 5010, 5009, 6543, 2104}, {6849, 5496, 1446, 
             6850, 6848}, {5851, 6845, 2316, 6844, 5850}, {6329, 6476, 2054, 
             5197, 6328}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.39348448690413923`, 0.2313964737301713, 0.6700703639298012], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVM1L1FEUvWo5n8XMb76cGT//gCBIMGtRUJBT5kQFLSJQSSPIvqwIctGq
TWpSLWpTEJXtcwRJHSc0skVQoWEFrSIijTZtLLRz5rzF5f3ee/eee+657/6a
us8dPltpZn2wKlg+ZTaSMCvicKHa7CfWH7AWv1krrLTB7E3M7EDabBwBN7B/
v9FspsJsP2JzsCLiOnGW9et+LGBWCAlzMGq2PSX/XNxsb71ZNGw2gJy7cd4D
nwJ8CgnhPoXPsRr4A6cN66W4MIfAoRn+Q+CwmjSbazS7jryv4fMF+wv4Xnbc
Fyp0/zcpn16spzJmw4idzJp1IOaMD/k8sxMpxfTgfqvDH/WkC/nfYu0JxfYh
ZidqfAk+d6DH3bT28/B5npQ+ReCXsuKyDRj3o+KzKyxdiNmJuA5PtbfjewJ8
HgN7LCQfakU9rmWlyRNwG4Wt+8WB+clzn6f6Wbs1oDZoO4A8S+AyldH5Gr67
gTNTqbPPTivWTwxyY5+bA8Kl75Va+U8G1Ve+iW+o8Q9ylXzq90qValnzq5+s
vRP13nP1srd5V2MGZ78zynE6Ib3KWgFnR0BYU8i76Em3q/ApwX+e/NDH2Yh4
fKwzq4hLpyXUO50VN2p2c5N0Y8+70uo7e/vBU3/LvY2p5n7odDmunlCvFvfO
w4gLwt759O4u1uvt9SC2F7YIzAnc52rkfwTrUdh5n/Tsc5ryfr1ePu2IOwgr
IfYRanobkWat7h3RZw61vGpQDdT+mSf9XwCjBvstQXHpd7PwD/irsFnky4PP
IZz7iQc9TqY0x5xFvh36l+N9minqXXL9o/GbucrzWq2eUpPpzfJfgf6/GrVn
DurKnlKnkNOKOT81KC/17nIzxX4W3SwcB9/bMfFgLxfdv6ZcX5M4ci4HY9KS
M5FLay64jrsZIcZIWjjL7p/FXKNx9YZ6Eq/OE+YwzobS6vF3xH5N6l3tgZ5t
iIm4/rNm1vsQfB7U6u3/B+Xyz+k=
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtk81L1GEQx2d/be7qmviy7q67umt/QKeE3oigIjXNjYiCTirsQtC7BEEK
nbqUYi+HuhSEZfeyoDZ1xYI6FAkWVtCpQ5TRoVOJ9hm+exiYeWaeme/3O8+z
cfD0oVOBme3DwtitBrNSxuxExGwxZLY7a9aNJWrN6rHxZrMh4l34M4FyfxNm
l6pUU2wxm6LHwnqzTNTsbb16+N1V4p3kepNmZeq3V5sdwB+h52ygs4k0PeNm
DZX6YbAExFMxs8fU76XXmyazJ+tUs8LsfFyzHUMPvY6kzM6Af4yaP/idjWZn
yZU5u0i/q2CYDQnjO/AtJZRfY94A+PuxQqB+n3LksR/Ei8S3ufuCHj8D6XOn
1ayOmf30KtXQG2x5Zp6MSMMSeJfbzabr4ALG82AdyYiLazjOrAeczSL+uYi4
7Kno5Zx6yPdiExFxLnN3LiMsjqEPbqsJ6eeYP+N/axJX59SHvrXcf0+8zP2H
jTpz3+/EyPVUYq9x7Dc42xEVhyHwLLaL32UwtnH/WEpYN9WIu+/cd+01N/Gv
Y3Nh9eiktiulu675KLlr2DZyW7Cn5I43i5tz/MWsZ1np4Zr9I3+B/Gvy8+S/
wm8lJd/PtoL9Yxt6xtTfe6fZUUe1ZvjbGm6WNs73VU6YvPbKBu3iZU6+72Sa
+G6rduk7d20mG/XWXaMO4s1J7X6U+At47qf1dlzvIn4hrZxr4FpMomE5rPhg
XG/W31Y0Kq7Ps9LSOQ/Se4F53wPNHyDur8zzs6NgH2uRdl7/O62Z3ts5F5Pq
4Xc/MKMrrj/lf8nf1GHuFxP6G/5H5nn/haRqfX/dSf1R/5szVfp7+5Py/Q+G
4vqTrpVr/ohe9yr9fYd5sOdbxM01X4Ob5fT2XYP/FFKq0w==
             "],
             VertexColors->None], 
            PolygonBox[{{6394, 5494, 5493, 6395, 1988}, {5494, 6388, 1985, 
             6389, 5493}, {5450, 6227, 1904, 6230, 5449}, {6234, 5450, 5449, 
             6235, 1906}, {4402, 6327, 1956, 6326, 4401}, {6823, 4402, 4401, 
             4303, 2303}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.42020930981730653`, 0.2778637233368128, 0.7076353355165185], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVE1IlFEUvZmOM2bqfI7mp9OoELQI+nEZ1CIwzcigdmEyGtbYop+ZFCSC
WkUmUuqiNiWETAYWWBHpJOLY1CLKjFZBLSyCoIJoUS6sczgPXFze+953f849
97xX33n60Kk8M+uDrYVZrdm/mFmu2CxbZXaiwmy0wOxrxGxbvdkO7Lsqza6U
m/UjoHWD2V3PbC5g1oz9GPYz2M8jPlBqdh7rMvz3Ieesad3j9r3IvYwacQDI
ImYWNod9PuIKSlVrGb6fWbtAZwF33oK4PuBZwnccdTtgi4hfiQkrfXJ1Zi9g
IyVmwyVak/lmmxAbBc4m1PpVLT/+88NmD4BpEtiOIk9HjdmtkFkK+2s4f4yc
u9DP7mL1ft83++KwHcPaFlP+R8jzEP4LOE+XKIY5iZ28EFs2T/8n1uhsyfU1
jZrXkesO8jQDZxMsUWj2FnhvxtRjI872cjY4v5yvlXhyOHuFuYzjexH+neDk
HfxrPPXMftnPpYBwcsbd8G1A3Y8h5Wbd59DAhRrlJgcv6xQXDa7WJMYDVfI/
jLWlQvmJN1crzFswn/5y1fhdpXlz1kPgbQS2E/l6ouKLXDQD5zenB+Y74jCn
kOcZ/ONB8U1NkPOFgPojt9RC0umhyVM95jmIdQAYBsDPsO++sf+D9S8sC2zf
0V8mJh66wNl8mXJyzgnH7RT+/6iTnslNT0Q8UBNZp9sp5MvAxgrF2cWweJtx
/znrcpx7sA/rzQaLzG6HVrUwGpJPwOmftfh/oEg+SXB1z9O+F72ei+rfdEz4
6M97mHW1EtD2U/TyKSgtHHd6aACudLXu9XtgSUWk0wz6uhGWFqhJ5iMn46h5
Jqoe9iPHOl93tT2oGXIWJyv0XjBnFjU3h3XHhyLSNHXCnskX83TD/4nzT0XV
D3tJOH2R86uY24SveWV86YAa4Nvzs1o+1Fa7w5COSFvERO1sdHecb9LZqObA
t+d1mbBRa42edLKCN6rV090gPvZHDrej5tZ6vUmsxfoNblYzzod9kFP2Ekf9
Dlgl/NrA1RtPOp2sFC+sW+zr3eSsaINuvqzNuKTTFe8buUh74p2Y2Cf7Iyf/
AbZj2Xw=
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlD1MVFEQhUdkYcEVZFnYfYArVBZGUUsT7RDEiInRDsgCAbRR+bOxUCsT
JMYIhTZKJIiYoIlooaIhsKJWImDlb6KtmliqqN94XjHZmXdnzpw5c+/WtJ84
dDzHzBqxXKy3xuxP2mxHxOxzsdnBpNlE3Gw2z2weu4V/qsrsYqHZHPE28iv5
Vg9ILvlDpWZNKbMxwIbWmhUGZrFAuVNrzNYF+ua+Y54tMVstNzuHf62AX+I+
8IfAv058qUwcvPcDONXRawH8yXyzDD37ye3F5nNU0wP3AIx7zLI7Zvas2uwn
+LPEw0Vmt6kfIH8xoprnnHckwIDvCOdbmSfFeaRY86yC15iSFlnwsuT/qFCu
492FXzP8FvPUs5f8fvD2xNR/Y1yYjuUaZTeYvQFjokj57ZXoHc7v83YQN4D3
lHiUeJn6lhB/BXuJvsfoOQr/8XxpPUPNhwJp3pcQB+/tPXyWx8SnY5qph3gS
zLkc9XCuDWAczRfnb3D7iu2MqGZhExpi53OV0wGX19QvhXzuw30MPW5g0/jN
9GpJa5eu6SB8Z9h3Jgpv6qfLNKPP5jsY4Ww40F3ZFdXdck1cC79jB+B2oVTn
Pt97etyk1zj2Dv8htRMJ1bZiTXDL8K08Kk33pZTTGvb/f9dKhO/nTzjbn9K8
zvFXue6M79b5D4Ddl9bdcw1m8A09FmKqaUtqR66Fa9JF3JnUbpfztLuluHzf
4au4NFwJ4ytwySRV6zs8ApduOHRhh91Hr0cp7dp33haIo3PzGV9wVh/XW/A3
cRL9SuH/dr12/AX+W2o0ey31U9TfCeS7pn/T0sjPfaa9+J3gD3LWzbfNJbrz
roe/IdfK30Q2nP8q/uWEduMzfKJ2lZzf2Ef87xV6k34fHfMMeN30r4pK03i1
vrnvnP2sLqXenlML9+3h+/H/pH9Fj7NP
             "],
             VertexColors->None], 
            PolygonBox[{{5068, 5853, 5852, 6344, 1961}, {6822, 5482, 1440, 
             6823, 6821}, {4636, 6229, 6228, 6227, 583}, {6393, 5856, 5855, 
             5095, 1987}, {6819, 5810, 192, 4302, 6818}, {5735, 6760, 2271, 
             6759, 5734}, {6397, 4680, 608, 6394, 6396}, {6332, 5812, 5811, 
             5065, 1957}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.44693413273047383`, 0.3243309729434544, 0.7452003071032357], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVVtoznEYfhk22//7tn37vu07Gd+FFIuNXHA1NVoOF4SroWxYzjZupJxa
seVGsU1Jbcj5LJKVOaScXchKJOZQbCvciMnz9Lxy8ev/+7//9/C8z3v4Z5Zv
WLB+qJkdwsnCyc43uxOYleWaPRhltitp9jXH7FzC7CzOYtzfFZl9iZt1ZZvN
xHMV5KeH6NsayNLDzVYUm83Ct8oRZj+iZr9Kzaohvwfd73hfCd0KxJqM0xHo
nAyZ/YSf9rBZG854yKpLzGpTZt0A2Qnbx7CdC9sj8Ht4pOIOItaOQrMzuG/H
MzJGcvpPFZi1IJdNiD+AXKbAZl0a+hH5aMR9p9uegm4B7sdMOdEX/cxGHoOw
X43ceqAzDfKX0KkFlvuIdQBYJ2TMevE+FDHfQfc9zplAfPBJTpjXQZzNw8x2
w+dV4K4DhiqXH88z+41vra7T6jzwfhN4msF7S5Zw70VOXeDkIWTX4eca5EsR
80FUPlZBtggxZkC3Pql33sljk9sedI4Zdxnw7cnV9zXgZG1aejV4Ho1InzEZ
m5zQ7oBjZg3JdeMwcfMhLn6ewe5izOwzbI8Dw2vHGUA3y/sshOcHnDycXr+T
53/vzJf1Jj72RjV8DsJ/A3DcR3+uiIof2oXcth/PgXzVnbKcAsmZb1X4v36v
x20Bh/PiqlWF25bjNCD3zWn52YfTly8M/MYY7Bnas5fZ2+y1Po/LOcrxHNk7
c+LCEHac1F8CeU2puGTvM0/6Zz+yp+n/OfKtLRGHdci1IyGeP8Hfm2LxkChU
f7O36yGbn1StL0K+DLZPOHeBeCeeAa8XcbIOgePphN0V78mQc0P56pjqx9ox
1zyvS4XzRDz0EXZ99l+589MdaC+w/0OuQyzf4O8uZmcqMP8C/9vGqDaMfzMu
DGWYqY9RYXwNvbpAPFzHPngRlc+w46fPvrB45C7Zj+/tpbK5C3+3cU5kK/d+
x82eYr+T8zrETUa0r8oRd1JGNeQ+SMW1E1KQvSpVvsQ+rlD42Y9vvRY5joU8
70hqh3J/FkF/Y1ozxP2XP1pcciYeFYnbsexr3Bdma4/cCsRzLK1aso4h1OJ2
gbDXoLbnY5KTj60JccIeIVbipI9u3z/cl5U+v+xT9jR7gNh/+kxxRtp9L9wA
z62F4v5CTDgYi/34NKJa9BRrBxE/OWXtGZezUOG7PQXdiRn9U7h3x5Zoj7R5
LM7jH/DakBF33Hv0w/7hTmryvXSpWD3K3tgCLKciyoHzHXLek/CRyKgXuP/S
Ec1Cn/8LiGd6RPuXfk7ivgm+mn2++Z39wH/O5Zjicna7Eppf1q/Z9yd37fKE
sP4F1+4Lzw==
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtVEtIlFEY/dTMcRpf/8zozDhO/pAEJRVF2ygokbCFEbUpLRtLtLLUAqOF
BEEqLQpCW0RQGWn2LvKRkJnapteqRY+FaUaUUbTIlOgcziw+/nv/+93znfM9
rltVv/VQspntgC2AzcbMmsNm7SlmT3xme/LNduHfMqw7M80+B8zmsR/BPiXL
LOzi32Kz7gyz75m6Owrb5xHGJ/j2ASOaatbrk+8VnJd5dCcL+9/ALE3VnfFC
szHYtUVm54G3G3fjuWbrF5pdTDebhO/fmLglI/7zQnEktw7YELAbwWkSZ7Vp
ZgeiZjthQxB5yos17u4NCJuaqCUP58VeaVqag2+22TdwWYt4VcCLw78RyZmH
vXKAkWf2Eny/APMhuLVE5B8Hx5WIPQ1/D+KnweqBXRdV7NNeafsVETY1NuHs
GO7MwLcd5w1+sxpgbgDWNvC/GzQbgL4P6YpXlKecMBeX4NMH32chs+vwfZQi
LUUh3aWmn4g1VqBYjNkBfZ3MgU+YW+B7NqB9He4swT7f0V1iHAFel6P8teHf
02xpopZK5OAB8I5ifwb72SSzEviOA2MMtgnrr/iW4tsArkPAq4GeQfwbgO3H
+irODkalhTEYqxQah5O1Lwf/HsTsMuXkAri/cZSLODh/hP4Jv7AZo4RxYTeS
1DM32ct+aduOGv0AXiv25cjHGugfRC7PBYTFfDiojx/W6hVGP2LXRBSbHNgb
93JVa9a8GncrYuqNJljUUQ6ZO95pyVGPsjd7gTeLsz+wtya+EfhXFIgP+bI2
ZSHxZU1Ym8FE/bmfAZc7QfUf+VNLdWI+iHE/qJqwFswhe/EfMEd96kliT8Q0
i4zxHuuIq9m9jH8nwHW8QFo24t/xsN4EzhtnuA3xlruaPc73HM6KXb0FnOEV
WK9y1fvsocOobbej3mFOObtT4DTsE+Zt8A3nSAs1rXPFiVxWZyk31Mxz6mvA
eTPMky1NJyPqcfY268Fen/OLO3ueb8HjxHvGN+EFYmcE1bt8gyrRa68T/cT4
U8jPdEi1YY1GsK4NarY4Y5zVxqhqyZntYW9HNbucAXLbHNLbQ47vYtJELXxD
qP1WWL3IHPwHUSbVgg==
             "],
             VertexColors->None], 
            PolygonBox[{{6338, 5480, 5479, 6339, 1960}, {6343, 5848, 951, 
             5068, 6342}, {6379, 4678, 4677, 6378, 1981}, {6331, 4666, 1636, 
             6332, 6330}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.47365895564364124`, 0.3707982225500961, 0.7827652786899532], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVEtIVVEU3alPzd/L955P3/N6zYGDtEGDJhUETQz6QJ+Bk9QyA6H8kCJY
QeTAFOdZg8hQM4KiD1gqlJqfIsoioywxMbJBEAQWhA1ai3XCweGe315nrbX3
vkXVDYfqE8zsIkYiRm+G2Y8ss5SAWZ1n1hcya08zG8d+VarZFuzXRMyqY2a5
WA/nm93LMZvBfnEe9jzdeQzQEsSV4duLu8uFZsnYb8D5+AbE4K0A1hUBzdOA
MVag9bV0s7lMsybEz0TNBuNmUyAXRWwEoxT7b33ggEc3eCUFxOVwklkI52FP
b9d7WnM+BIwRjPT1Zo1hs1147ywwR8C5FvPyhDUs4pQH5cGTdZp3ZIp/J+du
TT/oy+6APNubqpgFePIZIzUgP/768uQpPCgoAgZiPbxbBx4HUuAnNB7Nl2fH
wPF4XJh8i3eJU5KLOXj24E4r/PwDzE3uvSTHcwdydSdPWoaA9xBjHnof4fvF
13waHp/baDYAj4eBMxKTjss47842mwXOFO78BIdl+FOJeTykPFL3/lTl7S64
/Ihrn3vMG/fJhz6Q0yhiL4SUz1vI+UHcf4GzSexPYPSn64x5Zg3shDeffNUg
89+cprNpaCoDTmMytADjfVTzdndOT+hfca485F6z229DXIWn9YeosBibCf4v
I+J8H/PBfNWwBw6vIvKc3jPP9HbeFz9yY87fubwzB7NuvoK4YKHOMvHNKtQZ
6+q6q6054Hz0tT7pqR+onW8u/K8ZYF7NUB1eQX76Y+LQBZ43QtLIWiQ38j8C
jFOedD/Ilh5qaYPW26ixDtRYLbSfDqvmideVLczN0LTqS9MSzhei6jXqXAzq
HdZCm6sH8ip13BbD6k/eZ48sup5phY4VnC0lajDH1Eg/O13vPEP+n2PcxFkL
7v9y99mL466GF7HfF1N9EZuciM/a2eqrflh3jCM+a+S7y+83fL9ijED7KL5j
GAOYVyLuREQa2CvbQnqLPbocUZ+yN4hNjU050kmN7LkzMeW8Jq7+Ye/8Dksz
eVyCr/E89dFEoXqeuvflqce4z75hX7F34kWqLWqbjKm3iU+P2G/MbwPw90SV
O/pQHlzLP+fM42pYfcsc0S/GspYY1+jyvuSLBzm8hu4q9MubZP2bY0XC3B7S
P5Ge0AN6QUzms8VpPB/S/6jH/RM63Vv0dMD5xn9ArvsP/AMbdOtV
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlE9IVAEQxr9SV9vS1HV13+6+tT14KDt06FKHqEN1iKA/hy5pZBsElUZ2
qiAziMRLp6Ig+qNlFyGJTBPKxC1SIqOkrKjYwg5Bl/5cvPQbZg8Db97MN/PN
fPNetq1jV/tiSZuxUuxcQlpfK+3h5YVyqT+QonGppUzaXiF1ETuals5HpRvk
XK6RHqWkV8TfLpKaGjzHYo+xICslsb5lUoScFrBnivET1BhOSiPYsYj0rES6
Q6ydnFVRx0/HpPf1Hh+Az+kV0gLv5skdWCoNwW0GzOuIc3hH7njCcw3TQ7w3
7rWs54uQWVOOvwv+FvPl6qQGZtuGXeS5spFZ4LsSfyfcDlHzFPnT+CGzpLEK
evVUSrPkj1Dv0xJpEMwmeu1Ieq5hknDL1Xv/LVg3sanQexuHDeDfZNgj2N0I
8JTYP+YrlPjOq5m3u9Z3NUe/I+zmMDaEVp28W5vxGoa9Tc0quO+nx9Yyn8Fm
zYces37DcDlLvAt7YM+13sNqXydnI/n11G+OOt8F6v+h/iy1JrBe9J5s9Nmf
oHcs7RjLNc0+U/MH+x8rdz1H8fP4k9gIz63cx/0a18o0+wL2a8pvy3Y6T6/V
We+1Bv8lfl/gt1OGTeG3wqmZ+DVsOVx+1/m8xjFLn46Y797wE9XoGXes7ewb
2ELGb9U0G6b3pRr3jdM8+M64c7cZ/lJrPPTdmCZx5r0X93ltR9Y7H/itGIck
+NHiPVjNn/iFmNeze7xK7pXAd1dK/veM35zh7X4O8vw89Nu2GfvRZ2/atbZv
7iP5Kfbzq8q/qYcpr2FY63ky8JscLN7vAXTOYfuK/AxrN2+3bjU+ZJyTcblp
GoOdy/iz3eQ6+ncn/F9g/4TjzNKU8N3ajtsC18S0sG9oLHANLG49/wMs/Kfg

             "],
             VertexColors->None], 
            PolygonBox[{{7188, 4530, 4529, 7189, 2603}, {5037, 5814, 5813, 
             6275, 1928}, {4410, 6374, 1979, 6373, 4409}, {4530, 5982, 1745, 
             5983, 4529}, {6843, 4410, 4409, 4312, 2315}, {6785, 7624, 2984, 
             7626, 6784}, {4369, 6172, 1872, 6171, 4368}, {6337, 5817, 5816, 
             5067, 1959}, {6341, 4667, 599, 6338, 6340}, {6736, 4369, 4368, 
             4266, 2251}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.5003837785568085, 0.4172654721567375, 0.8203302502766703], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVEtIlFEY/czXOI6DM87jd5wZG2xRi2hT0QN80Kg7C1dBFOSjNqH0AN1I
jRCSQgWFWVFpWUhgPgJNhFBLKKI2LaKH2SKCNrbqsewczl1c/vt/93uc73zn
3kxbd2vXJjMbwMrHKg6YfQ2aFeDbW2XWgzVTYuYvNCvBOlxgVhsx25mGP3ya
cD5VbXYBPrtg251W7DjWr6DijifMgvD54jNrgH8dVh/88/woiPU3z+xtofbM
z/2osy8jbgVroMzsXQxn5WZv4HoHOafw315k1l1hlvHMWov1XazUvh/7vWH5
dMC/HWvNJ4x1EeHsx3n5ZrMxkHATcbewDvj0/ZTWnjHEPo66jcD+OqV+t8XN
JoHtAfK3hxCTNLsGzAvAtYzanbBfRp0o9mPoaw/z4b8Edathq4+aDaLXLsQ9
Ao4B9LwB+2nYH8L+ErmHkHcQvSczZqmMOO0BtlPo+SB6/I6c88B0v0D8kXdy
mAO+u0Xqq8WnPji/WeSLoN6/PNlGArLvgH3CE7cfkXMFODeQ5wW+t1Ev61Nu
v9PACfj3wv868mRxPhnV3HzwyXe1mLvY5WcPxMH4p4g9FheGVU98kSv20xZQ
L5xtztPshuE/hx4/o6c52CfR+xjEOs0eS82eI89Z1P/hiTfmoS85Z23iIeZm
5DgZEjeNwNxUqbMF2BOe4i6GNVfOlPdgPSge2Av54n5/WLiJOYh8ZZ60MQze
bqTVA+MYQ/9n0N0clh91z0U1Y+Jk/jV3184j59GkZsY5kkvqf5Z9lGo/E5Pu
yckSNPgbPIzirAZ4t8QVG4hKc+zdj/14pWI5B5+bRRL1QyFpjziJg+e0HUrI
XhuVRrmnXql3cr4E24eYdEJdX3G1XgFPLiEf6ni6UHfhMXzm3XzXXC1yMoha
W+OaXY2neqy1mtIdYz1iaXB2csR46u2eXzOktnlXfjo+ec43iT5luDtXI9IS
NdfiNMl3hZySkz/gbzklfr+h5nqV3qR8V4s42RM1x74myjVvzpo5+CYRfwxz
q3dvWh/meAT/Z/y6H50h4eH9eOLuCHXLHoh/e1xcMA/jGcd7Qey828RPjWad
hoeQ41JU/O5D3GJCvVP7/OeeemyOCSffbtZjfr5zubDq8t7MOjs1NO34fJ/W
+04OWIP3gHMccZqmTvgG872gBvjmdYTkQw4qkuqD95tvDXv5D1ym4gs=
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtU01LlGEUveNo86k04zjOzDszNtQignZFRQsLbYQCBVdBtGjU2oR9QJmB
1Agl6aJFYURgMzWFRJoWNFkQjloUkQUtom/7A7Xpa9k5nFlceN73uffcc889
TyZ3pLu/xsy6ELWIey1mB0Nml11mZ31mU41mwzGznlVmRbdZOWG2HDV7hVw/
7k84Zo/CZtf9ZjP4bkB9PeIGwD57zTalzZZSwnMHzR4i/34TsHD/Cfkvcfci
JWz2ywF/crVZyVTfGjHbAgwPamsRAdQGq/X/gLkvqX88HweHVuAPhXUe8ql2
BzC8QWEsgtsSYrQe/OswF/odSKjXF8RdzLsVNb6gOCUw+3hIXLs8Zo3oF0Zc
cgk/lREmsX42mHVAmzWo6fZohmlwmwOfAvi8Rs76ZrNT4DiL2qfAOATstfg3
AX3z+PcVd8vgWwpSYLPv4PMWmG8QKzjvR+9h4BVrVEPs09TUpx6/kXPS0S4K
AbMFcNvYrF4jmHkQd3+QUwyo5jnOs1Htt+ARVqVFubw/jPtx6JELav4NwDoT
Vu8S4lrcLAu+e7GPduh3EeeBuHK5o2+OZuIsLsSFJmGwdqZOXH+hx0RAnNfh
7nxY9+wxivxz+L5Zzac2yYy0pkb5mHbGXXGGXui509FuuKMKvOUHxl+XZmav
wape7ElvPkvJe/RoBPpGk6qnp8aAdwvzdHrlGe6+HzHiF35nQp6lf+j5afDZ
Dr59VT33QNtj6HkUsTuqt9LhaNd8M4+B3ZuQ9+gXenUX/rm98iy1oAacnZq0
4T7riC89OuDoDVI7emoS/e/E1J9vtILzfExn/uvDPJvTekuc90lcHNibnr2N
fmPVfpzxPTkn9NaJ8RG1HxA/MGsbOC6Az7u03jb/lcFlztH+qQe1uxpXLjXk
238Qkrb0RzkqD1Ir8tsWlob85j7prfa43g49NgVv9oSkPT1xJa0dsBc9mkXu
YkRepIZ54M2n9Bb4Zkq4X3HkRXriP/C/uos=
             "],
             VertexColors->None], 
            PolygonBox[{{6842, 5492, 1445, 6843, 6841}, {6176, 5707, 5706, 
             4975, 1874}, {6377, 5089, 5088, 5491, 1445}, {4977, 6179, 1418, 
             5441, 4976}, {5440, 4974, 4973, 6172, 1418}, {5840, 6839, 2314, 
             6838, 5839}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.5271086014699758, 0.46373272176337915`, 0.8578952218633876], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VE1IlGEQHv/WVVdzd/12XdfWXYwS0iIi6hJqmYZBYj90CDpYZpQZkX9Z
WaFmdemUHcwkLZKwpND8KYiKig6JdugeHaRDeK1DP8/T83UY3vebd+aZmWfm
m0TDqT0tyWbWCUmB1ETNZgJmi1Am+cw8kIsZZvPQPcXbEvRvg2ZjIbPJFNm2
RWXfHzE7gbezKbKddO3b88225ZntTzXLcswOFpodSjPbAfsK6PdBf99jVo9z
FPYPcP+E8x6kChjbo8qhNGw2jXgL0F8iJvz7082qgTGIe7LX7HkcuPg+AKyd
yDEAu40e+RCXmMyJcW4mAT8if/qO5Zo1FJh98Orc7d5Z2+1M2fcifh/yH4Fu
FvfHjnLNiOiN+okMxWLMMuT8ukjfaT5xytz6YHvdUR5bYmbjQdXyJws1ZoqL
AeibXT5peyUg+5+o62pAtTzD2Yt8huBTTa5cnkvhWwb5DbxrOYpNnol9Jlv4
5H5zTPoq4EyBqxaPek4f6o/5zX4g3kCSej5YoL7Tr9zt3XKq8FnXYdg/KlIM
cu8PiP+TCbNaSN0Kszj0NSHpNwBnLi5eGNPjcsT6zjmqkbPBOurdOLSjTTG4
bfKrL+Tvcr44JE+rwuKqFRhzyPk9cp7GuRBSjUfhN4E8W7M1vx3uDI941D/2
jrO81RH/nIVbMc3DMO53Ib9wn4Lf2rByzEH8JvjUIoc2zEM7ZI1X/LBuctiC
9wpgdgGzE74lYcXkfA25M8b7v1w4/8i3rkD2s3HNN3nmjDcndP+fN+1foaau
qGbSh3zOg4th3Ncj93UxzVd3QG/UPwH+u6B6Wgm/I+ClO0P/a1e+Zukr3j+H
xOGukPrH3s2HxCn5PB7U/0+uVuI9BqlP10wtupwzzrS7NyrztBeI3+hop1C/
DN13yEv4JsDNTK52xRe8f4P+BfuLee8pFE+cB/LLOXwDzOqIvpNd3sm5z9He
IQ77QD15uwAeyqPiYRO4afRrzxQjzmnE24tYH2EziFgziNUT0H9Onscd7cQl
d5+w/6PuziL3jNsB2xvIqcSrnnBf8O0h8Mcgd9K1z1aHNW+c9373vx5x9wdj
sR+Mzzz/Aqx/y2k=
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtk01IlHEQxh+3stRdanfV3XXlXbeiDK2IKLrE5kcaRoh90CHo0AdGmBEZ
WpQdzIQO3TLIVLAi+tDtYGkRRHWIIEM7dO9Up7zWoY/fMHsY+M87M88888y8
2WNn93eHJOWwpdjlmJRNSEeXSeMEzsWlvZXS1mLpwHJpbpX0Gb8b/88KqYt4
b0qq5X1xifQjKTXHPP6a/MmMtAG8DsDPR6SNgVQXuP+3TOqkfk8Bv438rqx0
BmtfKR0m5za5w4Fjf6THCL3el0uLxEL4N6Jggv+Q+nyJtJ3cbdhV3sVhqSkt
bcEvCvu3m9SGK3y+9dT/hG8P/gfw39D/VY30ssZ7W821aukJ8VtF0mipNID/
Iubvu1gjfBrAPET+EPWXwJti5p6If2tltkjSZ7MZ18J1TcLx7qHvV+KLSe9t
HKboNYkNE/9CPFfuM9ksB8G7Q//jUddyhv47yd1X5dqYRuvAHoDfBLXz2C7i
GfA7jBvxQeqvVzt3y7FdbMb+lflO6qivD3w3D+CbQ78GrL/E72GE2v6Yv2dL
nWtzyndhnE9EnaNxs5om+O8IXBvT4xfz/sbuhzx/gfd4ld+S3Yxp/RT7HnKN
WsDenfLZbedj5D6jJo+N8u6kXyv8Fgr5feSejrsWdpNDcK1PuBbWc566b3G/
TetXm/AbstuZwD8C9yuF+exGZrn3Cym/FfM/EWss6GEzvmPXbzO+K7vBNm73
VNRnsxsO7H+odO1tB1Y7F3PtDMP+jcGY34LpP038edq1Nw1mqH0Mxhi102Dk
6dWSdi3txh4RO1nhMcvZFDhH42Y7jfJenfR/1+6vl9q+tM9qmv0HjwyNhA==

             "],
             VertexColors->None], 
            PolygonBox[{{8110, 6265, 6264, 4454, 3444}, {5490, 6376, 1980, 
             6377, 5489}, {6265, 6980, 2431, 4453, 6264}, {5439, 6173, 1873, 
             6175, 5438}, {6733, 4367, 4366, 4264, 2249}, {6381, 5490, 5489, 
             6382, 1983}, {6166, 6520, 2088, 5263, 6165}, {6817, 4400, 4399, 
             4301, 2301}, {4367, 6158, 1866, 6157, 4366}, {5808, 6272, 1927, 
             6273, 5807}, {4400, 6313, 1950, 6312, 4399}, {6178, 5439, 5438, 
             6179, 1875}, {6811, 6814, 6813, 5809, 948}, {6518, 6166, 6165, 
             5262, 2087}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.5538334243831433, 0.5101999713700209, 0.8954601934501051], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVU1IlFEUvVn+jDnq/I+OfjVCbbIsa5Mlktov6aJF26IoKiqh7EeJDC1T
ylUEQWYjRRaVRFhpCtFGWhgFmlkYuYqColVSUNQ5nActLvO+9+6799xz7n2T
3NWw7XCamXXB5sLq88zO55r5c8ya4mb9C8zOZpr5sN4bMjsBp0cJswHYe5/Z
nojZHZxdTzcbj5pNY92QYVYbNPuM9Q4E7ywwa4eVZ+F8odnlgNmUT/eWB3S3
vcjsHKwn2+woYjfC+uFz22/216c1872Djc9B3BytZ7HuQ97n+WbPgL8RebrC
Zql5Zis9s0HkS4NvFXCWIF8V8Ewj9s5C3e0F1teo7yvqGoLvU1gH6h9LV176
7MXdh65G4uD+K8S5hPgXwsLyHeeTUdXeg9hXYb9R70fs1wTFQwD1xfBdj3yH
kmYHYWPguxJx1sJyECcPfNThbme6dLjgtJhA7BnH7Z/5qu8TYo7i+xZsxnSv
MqK75Pdahnwaca8P2rUibzf2NmdKR2ry2JMu5CYQFD9D2PuCWGWIUZtQH7AH
iK8F97cj9z3kuRsRT3V56hvi5C+/6UP89W5NTS57ws3fi06jgrh4ISfHgGcQ
Ot4wnbEG4v+F2n9GlZv90lsoLaj5ooB0T8bMTgVV3xnEfBDV/knEjOK7DvFX
I96w42oW/uvhU47vCtQRiIu3UmBbElOPUfPdEeVij6Qy1Cfkj9ySQ/J3MywO
j8D3m+t5Ym31qfaXwFWdUC3kiP21bZ7OyRF92MtD2eo3at7hdCfGikLhfItZ
nIJdwUysQN6ahOI0Ya8ZtsqvOta6Hmj3q1/Zq9eBYbJY9Z9Bj7fANuWq/4iB
WFjfqOMnBf+N4GEkQzqfLZLWrD/lfB7Dp61I8duwXhzTmYGPN8XyJS/knpqe
hs/VInHXhJjNsD7E3BATPmJjn3FeWdcAanmBvr2BvXJP85zm5p4cUCP2PmMQ
D+eSGInnCeqrDCsO35IPif+zyzeFPJdRa0/zlBvX7JE38tLrtGbsEVdvCXBm
x4W/FrErPGm3Br+bk+r7YXeP/nw7m93sUP+xoHpgHe5Woy8tS3vLPO0Ty1KH
pxa6e3Hpzjd1wr0txEVdWeM+4N2SlHbEyP5mXr7Tg576k71/vEAaFMf1DnLG
94T0ptOH7wKxcF7YazVuLjiDxEKe78L3Pt9A+O7H3RDWW7HOB/4fMcXk2018
xHYgpNljXv4PcDY4F+xxakT+Oat8Y9gb/O12M1WKeOeD0pb77Gv6/AN1Yff/

             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlF1o1XUYxx+nezm2lfvvbOccz3Z2JpgXzYmzm1YWuVVbNAUvvPFCMUTJ
1AvnZDM03HITu4sgyLbFqilFRE3dUupKL8S3knzDUIiCQOnKYaDg58v3XDzw
+/2et+/zfZ7n17Jl9/pdZRGxEVmA/JiPuJxEzC6MuJWK6ChErEaqqyM2YNDZ
EHFiUcSFqoj2iog30hFnihFHn40YQd5uidiJXHwu4hD+g9mINdjMw364klic
XyVeDfHWE2+UXP8T8yCxjiNt6FYgT56J+Adgafy/xmcV/r34D2OfykVc5fwl
9g/Q/4fs5rwJ+zXoPuT+K7VElX1z3NdWOkY/59fT1h3ibUcduHPWyeZ3sEws
jhhHfuP8M7WtLJSwUt8x3qcaHP8xPu+3mAPVrpqvo/uLmL9UGpO4mS3aVxzV
gj8BwzvoXwPvFLadOfMjPP8Sf6A54sUaeIC/zdw/If8m+JqbF9FDvq60e9FL
vmWZiCO1EYdrzMcMtrMF++6bb+5Vs2IrR4ZzRz0+5a73BPnbsS+rdn+WEquv
ZK8aZ8B+uuj6ZfMB+KvR96A/Rr5z+G8l3jjxxnm7j+5ag2sXB/e43826N3or
cG7KunbNzw/Yji02l8rXyfx15Y1FOffXe0Y0G1PkOIlvV+J4k9gcpb6f8p5V
cTTDbLZkjG2SeO/i/23W2ITxEfm213o2NXN/4vs3fB+Hv+9TPt/Jm2u9fd4Y
cSnxLH6B/cGiORfX3fTzo0bPpO5j7Ewv57aM8Sr+IPkneDtbwv88utPcz3Of
qPCufYfNxXLv3BDxDjc61hVy3gTPHubjGng+A08/570516Z9GEnMkWJ/Q7xR
dEeQ9irbaLbU4+FS/5fT69aC42nHtqPrgZNuZFtpF96rc2zthHL3IRML7PMQ
/HMZ9049fKXgmlWr/og5evkm+jPcX4LPTnTTWfdKPXsrYxvpxIn+gpcLnmX9
CX80sb9Fc3uf+V2I79Y6z7L4/5h+30F/M2VMmsUDiXWayU/p7ZKM77IZbDYn
4kI7VYf9osTYtYMvYHuSft9OuT71pjVjLtWjYXpxKnEv1KPV9f4ztDvaoevg
jbSxqmb9TUOJe6s/Sru5J+/Z0o5qVm80u5ea2XVw0dVgPvWnTuf9B0snTF+l
/QeqH+qpZmug1G/N2FMnSc8i
             "],
             VertexColors->None], 
            PolygonBox[{{5801, 6812, 2300, 6811, 5800}, {5705, 4969, 1868, 
             6162, 5704}, {6816, 5478, 1439, 6817, 6815}, {5436, 4968, 4967, 
             6158, 1417}, {6317, 5801, 5800, 5061, 1951}, {5064, 5842, 5841, 
             6325, 1955}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.5761267123156394, 0.5473375050612588, 0.9097792766818537], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VUtsjGEUvbTMmFY7bf+ZTmeYGgvFQkKQQSuiqq1HggUSr2oQFlrUoh1a
Eo9QEgupiDTaNKQqqZRpaSXVJYnYVKikFRIRhJXEYyXOcT6Lm//7vrmPc899
TKK2fkvdZDNrg2RA+qNmz8Nmh6ea3Q6aTc8z68K7P2K2r8Bsrc9sImb2sdjs
zzSzcb++T/Ft9cx2FJrdzDR7NMtsEHI+x2w23trgZ3SSWXCG2dyE2byAWQ/8
lReZdSBwB2zqc2XbgthDALUPsgF4VgHPwqmyy82X7S+8/QxLNwLfW+NmfcBZ
glinEHd4krB/igq/AdvrmWbpLLPv0FntfJ6G7iu8P8D7uUx9BxE3K1vYDd8W
+LkbgsBnSbZiEtsC+NmYr/NanH8i1tcM2dKOeFpzdP+HDfjnQxpxH4jrzjNj
pl3ch0HlQ/0mfOuh1z8FvuH/ByTpE+fZ05XXS+TxDbV5jXM74vfgvhf4DgBv
J+4DwNOAel3MEydnAvLHWO9g9x4y6FN+J/OUIznbnJD/NOozBTpPwM0e+BlA
3dtR77Fp6gXWdAUwrowrzybIMHIehn4NznuzVVM/fH8D/msm7MyBPTMEu0FI
Cnojzp6xfuPtg+feQuKaPF9H3LMB9ceVkDhiPqPI+1mB8j0C21qcK8Bpd1i8
kBPWl2/kgXywV3geR34T0H+IfOpge8yTbm+OcmCOzJm1Zdxc6Ach66GfRP3v
Os6r8T0IP40ZqvM8V980dO9DuqA/Bv7SMfV7J2xTIflkDozJuhz1hIN3ckkf
5HMT/J1IaE7+9yvx70fMAPxX+uT7kif/rE+lqxH7ojcinLTn76xvGd7XR4WZ
3LAm9HkC/OyMie85fumzjmXAtRwSAKYi+LvtCdt96PbF1BfEyxkddrNMH8yx
Fb10CbLE+WMPEENzWH3Pnm8BllOcI7/mqbJQvC+Ka19wVwTBW0FE87sN78sc
Hu4JzjbfRzIVh720HRzfcbYXED+FGiz2a4e9jWl/3XD1Zf/3AEt5WL3K/dTt
bNnjZZ76nPqcJdbrUIFqTw6X+hWX8Re7LzFU+fQb7y/C2rPcsbsRf1dM/rh3
+yJ6Zz7cHeTkODA0FKkOHn6/5TjfklBPsJ5r8FadkM1T4LkX1ixwB4y688Ui
5U9cTcj9S1R4+uGzPF9xyVNLWPkm8fsyyGPoT2BPXvY0E8Typlh4OLfMjX4q
wH2z2yFV+L2xWLPOeW13PUCOeaYOd2dpXHOdSuj/gz3GnmKezJH90u96hruE
ManDXcV49DOGeiXztAM5f6Uh1YKxyCtrtA55fI6qv8kt9zjjXkX8mqh68S+u
yQRw
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtIVGEUx4+VzThjNuOM86yrE2S0643PTZZatknCwF5qJrQoexj4QAsq
ITNalBERGVYE9sB8NQYmrWzTpiiDrBZtCtv2WEW/w7mLwz3f953zP//zuqmm
1trji0RkD7IEWeeIdEdERtAlW2Q8LjKGvORxs1fkY77IHDLIuZDz5bDINXw+
ee2uH9u+uNluQV4FRB6C17hUZIFzI/Z/OPdwXgP+vqjIdfwb0O/gX8FbSULk
BbY3oPA5KdKGzHL+lyUyrL4JFwuMDyuJmyuSJokxv8gp8E+Atx39InjP0X8W
iEzniHQQ4we+U9x1os/wXhSEe0xk0gMW8ebR30VM17sKsL9yl0Y/Rrwq+N7l
bop4C4tFysAqR3zgNXM3AN488Z4Q722GyHfiVVOv7GWWwz18D5HPbXK54BMJ
gF1OvHawajzGdQK89mzj3E8+A259FLMnYj3S3gzi0wD+fNJqoz24gn1b3HrT
iE1ZnkgwZtgaY4j4V7kbzxQ5T/yWkEgW75W8VyP38Z90rFbT1KeJ950J4641
LUb3Bq032qN2cntAj4fEchyBi9fFm4PTa/zHY1a7CeJvBHuDY9w1hxBvR0PG
bT02XSl6k7JZaF1uvT+NTGcYh29h66nym0F2Y1ubMlvlUMLbNmz6cqwndZz3
uv3RmS5Xf2rcy7ufu3SBcVIuepcJn7+OYevMFzs2szqrilFP/58lLTft71b6
UQf/Io/x3wGXUsewlUMn9enIt1nb5LVaH8G+ymM1Wot9YAVfn/XcH7Md0t3R
fvg4r4paLPVpDplNtet/Fv7nkFG/1SjtWE/1XWf8PfshYdsNtdmftBnU2btJ
DkFix2PWa+XwlHzGOA95rF86C73u/mvNdRZ1xnS2dCZ19yqjtgu6g7rL9S5f
fR8FqyXPsBRzhPNsyM46E1/Q30RM1x2boFY9zNdwhu1IDPtS/EuQKHoz38d8
HyGH0QuxPYPehqxG7+SuI892UXf0F1wCudYb7dFvYnUFLXf9B9Uwz7sS9q4z
eCBpO6K7cSvLcr8Ut95pDbrxPZi0N+XYSr4nw/av0Z36D0AxvCE=
             "],
             VertexColors->None], 
            PolygonBox[{{4623, 6168, 6167, 7185, 2601}, {4950, 5702, 5701, 
             6134, 1848}, {6321, 5476, 5475, 6322, 1954}, {5845, 5086, 1976, 
             6368, 5844}, {5702, 6712, 2234, 6714, 5701}, {6261, 5464, 927, 
             5031, 6260}, {6783, 4390, 4389, 4289, 2283}, {5435, 6159, 1867, 
             6161, 5434}, {5084, 5845, 5844, 6364, 1974}, {6316, 4664, 1633, 
             6317, 6315}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.5957756410640719, 0.5789078661874125, 0.9102272117377903], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VUtIVVEU3Wrqs/f0ffS9d30vr72BUYQSSWY/B2XRd5Cg2VeyoJqkRUVk
2SAIiiCICCLFrIzSKKMf2GckFUgESU36TCIIKioobVSt1ToNDvfcc/dZe+21
PzfT2tawM9vMerBysD5PNrvnm1XlmtUlzGpSZjPyzDo9szVxszM49yNm5ViV
BWYjeO8oNHsAkDW414RVETIbjJo1JM1+ZpkdLzU7gbUPDtrDuvssT6sM+w+4
2wfsgxm9jwNjDOsRcCoCZhFnz2fU7fcDfy74XQH+F3D8jDUTfDalzQ7h27kC
+aE/8lycb9YYEIftvviR20jM7HJcHF4C702x2TBshrIVG232IvYpUcVOTN9x
OAWcPE/YK+F/BdZT+BgHzuYS+J5g9g0afMeqgc2npDSlnj+wr4bfo8D8BZxc
4NTDZgm+N4HDjhzlgGd7sb+F79fxrRV3K/FsdDbD0HVmRjbkeDglng04W51R
/POh41ysPbC5CX3Wwfdz4FwD5lZwaEKs/cA7AtzzOcoftSR/PqeEtO8Edr+z
mYd7Hu7X4W479u+xvwS/u0vFiXyWw761WH6p/UPgTM9Vbobg/z7uXoT+G9Kq
n3bosLtEfGtj4kduRZPMwlhTJ0IHYL6C/gFgBFF3VXg+BvZoSPVSCfylsFmW
0vmfIhR1WOcRYEzL6P0HfP30xekaY/Kk7Sjqvw0c+nCvoQwaYtUHzbrAx4Nu
d7PFm0/WHnWdF9L+BuI4hrvd0Oo0sNc7nbs8+SA+e+t9iXJbE1LcV7Ok3864
NGxJSxd+e43z2qjOO/G8U65+O4A4OjLqiQ6cjyWEc8dXDxP/JPAuxMSXOScn
8mFPsX5Z87vApc1XDGddP7P2FwAv4qlWmedwTLmmnswD94zzucO5hZzfxlof
UM3yyVzPAHYnsFax3/F85/qrF3iLkvLFM/Yez6OuZ8mvOa7eYz1n4ex3kfLI
3FA7xnsY91pSwn9LbIezDX53+LJjvrNcDfQBfz80rC5UfbB+WJ/d4DPJ5Zf9
tzCh/uqB/9kx1SHzQTzOBObqnav5FyFxo+ZB9lSx5gz7+2tSPV7viSt5sm6r
Xe2yD4ZcPYU99RW15b1Zbj5wdhKXmKxBcqX9E/gZxHlvvmqfeWGMjU572jMX
3LMfK13PMO6CuHJA3+Rf5WyYU86zf3eDsqU+nHP5btbRftT1GuuFulCTpdD1
Y0pxBVCTc1LizxrcmFYd9iU0azhDBhLqDfYF5+UcX3OGWr5w+Pz/zHczk/H9
cTUQwL3BtPhOi0lfaptO6t/Afvxf66zP2pRmK3PKeX89rXPOvAHHZ0ux5hd1
aAaXtb5mH3PE3qBWOfh+IKra43+gzP0LDiVU6+TAmXrTza5q9GioXPHwfxiI
SytqQVzm8S+eeAB9
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlE1olWcQhSfVJPfqTXKv8d6b3CSfdWFrLEpRa6OxLmwDqVJo4k9SYluI
BXVRk4r/JnEhFJouCi6EUkULhqYVEiVJF+nPKqgQilAFF7XdiLRYNw1Ru9I+
h9PFwPt+78yZmTNnvuW9fZ0HX4iIHmwhdiCJ2I89LovYkIn4mHNrIeLwgohN
3L/LRwzlIk5VRcyWc+etZUnEbkAu4XOvNuK3uojLvM1wH8g5Rr5fpslRH7EA
+xb8fhJOcv4ce4X3nlTEP8WI1WAeIHZDZcQJ4r8g/gfwR4m5BPbFOufrrYh4
Qn3VjRErF0X8TH3DSyNOLo/IZiNWgJfFtwbbgv+r+G8quCf1IrwZ7hOliFTa
9fZSfz/vmzPueU8DveZc749glIM1Ue9a36K+OfI9TZz7NnaF+CF8zpWbj0rO
d/DZhX8b/hvJ1VJyf5/iM7Us4sMGY4vTMeq5AcbXle7vJLlvvxgxUu0aTtHb
u1gu6x7fhK/z8P898dPYR8SmyHkBbo+S4xC5D/7fr3qqAm+EuHXkWoxt5K0L
W5HxPE5T21DJtauHQXKNwH8TMQm2nbc3CubyJj0dI1cz+d/hfoaax+hlvMH5
H1DPWuLXYb+CfwTMrcTOFT1bcfCIcypnLtZXurdOrL/GPTby3lD0rNRjX2JN
6N6acW+fLPVZPXY0RbRT43XNZ3HEcfhtx9ak3XMz2KuwMvADu0qt17BbFa75
M7DOJp6V7pllnrFmqx7eS6xhcfUS94fU9nfRfEhjU+Tuz1v/aXLWwM1K8j2v
9vvbvF/mfXXaNUob82D+lLFGXsd/pGD+pfFR5tmd9yzE7/vU+gH2Vdo7od0Z
rvestUPjzONKwdqRhlrg6m7WtaimSfL/UrDWpfkU2Om8taOdbyt6Z7Wr02Bs
xvebrGetHv8A//daxwvvPvX/m3gX1nDfAf97az1r1fM48Ted1eMAXMwusTak
KXEhjUlb4mQXsfNF60H6qGk0x+JOOy4us42enTh9RD/diWexNu3Zvow9q/aM
Ozj/WTK2NKWzdkDa17e/uG8redelEcVOJe5FGLs51+WMrR0ZzJkjcaMZPIGf
pwXrQfvZU/Q/SP8eaaor7x3R7DTDoYJnqrt25jXed9Za++q5j9h9if+92nn9
ezubrGXVeE3/nzrPV/r4D7gwvsg=
             "],
             VertexColors->None], 
            PolygonBox[{{5836, 6319, 1953, 6320, 5835}, {4279, 6893, 2354, 
             4376, 4278}, {4263, 5697, 5696, 6732, 2248}, {6751, 4279, 4278, 
             5757, 1613}, {4310, 5836, 5835, 6836, 2312}, {5697, 6128, 1846, 
             6130, 5696}},
             VertexColors->None]}]}, 
         {RGBColor[0.6154245698125042, 0.610478227313566, 0.9106751467937269],
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtlEtsTWEUhTfVe1tttb3a3rq3vdWkoSgSNDfxGCAaHRgoUuqZCopoMUBQ
A29TURpvoQzqMZBqIykdaKIqEkEHQsUjkXjF29S3ss9g55zz//tfa++1139K
65tqGoea2X4ihUgvMasm5qWanQqxPtysk4S1I80WFZtVZZjtKzRr5/tw2Kws
zdfi5M+PmS0kZxb5c8g5Ss56clrBaSEugd9bYHaavM18V3FmXZpzbSwy21Tk
fOMjZo2cXwDOs9Fmu3OpJd2slv3zEa/nBd/tmc7blm/WXGr2kPdrOWYz4WiB
a3LIc1XPG2p5xHovTe4Bu5k4RG234mY3iQHwTsDzKmr2dIjZVPCmEMOz0QT8
NGKF8Hm2k9+X6rVkFDnHyXTXS+8/8lxH9bWf/VTO1PH+kl7PodVbajiURT2s
HRhmdgO863Hv6XjC7BjnV9HbdNZ+jfR8af8n4Vp30uPqmOP1hhznCbz3yLlf
7BzCkjbqZRx17gz0HJfpnNuIq+jRCs4aMH7T9xK4kmiSRc2ZxFhmcYtzj8l7
hJ7/4P+Z53OTR+6M8h674azPc8wN5DTlOYdmWxbM9w34E8AZE3JNtScc4WWW
+HsDZzcSSWpMAXsY0R/22espfPGUJ/z9a8y1kA76Vo60+g7X7MADKwJt5L0t
nJte4L74RE42PU7lTHfI85TzlfVvRGXY5xYKZieMuiBHeicDHeXTsRHXVh7v
Cnx+lrW5Uf/+HHU+ccmnV/Ldq+9ZS424JsKuSPezXfTdSaxMc+0Hclz/OeRe
K/S7I3/Is5rvVvA6Yu7tM+z3MMdWuLbjn46I1/4O3T7CVwHPa/YHyesMux/k
RfUyDYxKogve5WhVl/B98fQFXhK/7rBqkD/Uh+7XtlF+N5RXzn4SnAbqKYZz
V8T/ATdzPU85F8C5SJymhjg5RVG/P/3k7kWjJ7wvhf9LzGcq70gvafWBc++I
u5x9Tl/hQve6/gfyWzzQsjWYxWP0m5TrGraptoh7vgbfzgj88DvhWDp7G84H
QY+aVXXM+9Q8NMOK4N7Kl/LdRHJ/BLW1F/idEZew5TnhL0bzynzX5Ai19OD5
g1m+p3+H/hvy+7KE+3gENSbjrq38plzNMQrPjoh7TXe6Nt+9J7/r3yfPy3Pd
Qe9H6CmjxHUJwzsY9ZwReP9vwnvR3dMd1bvugf4p4tK/4XKBa/UfaCbavw==

             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtI1HEQx6fUXXXXbDd3//suIVLDChJbkLqYSR06ZIRpJVFUWpDloaSi
wDLrGpliSkQklGlBmGtgRSiURhCRh8iiB3XpiT2ufYbZw8DMfx6/73xn5l+4
q7nm4FwR2YxkIp6YyDy/yBo+jiB/EyL/kOoskVLEHRJxIS/miESx+wMiU8Q/
d4k8xR4JU2uBSJlbZEe2yHdHpDgo0pghUs63QZ9InWPxT4i/g91I/aRXZBK7
GH0bUoJ9GEA3qXWd/J3Et5P/htzBqMirHMOwISKyPmJ4tV4teE6C7yyxG/l2
opD8gPn0zS/kp8KGTfsZp/YMb4wQfwB/NrE1xEygb8cfQu+lvxR2J/IwLlLB
++/pp88jUoX/KP4ufKPIOfq5EbBe2vNEGsD2IG6xr3mzFX/UsfhL9OBZKPKr
QGQd8YvxrwTvOPj2oOfm4yP2J9LJbJaQMxQ1zpXrafKb4GofcjrTOCzH98Mx
rpXztehXwTfmsh7qiN2aMPxTxOxn3rUxm3Vbrsgt7Q3/gNc41t6aYuZTzMfA
nxG23C78R/zGwWian8/0G8fuRi/NsV5awtab9lSfMI6UG53xMPlt1K/X+YDh
FLNb6rfZ6QyL0Cv5tjc9z3zdz5jNTndsC7NbFjRuktiugGHQt7WmF37LCo1L
3WHlIidgs1VOtNeOAsOnPa+m1iryU9nG+Xl8t6PGdYPXZtsB/jN5NuNH1G+J
Gnb9lkTPA19Rru1kFu/NOoZNMfb5bac60/vSiu2kbeWsmfdeLjJudaa6i3oj
+pbu5Fvsd47VUnxJ8q/AT4/bbqQH7M/mEy+2z734BoJmdxNzDf14yG5J470x
u0m9D8W8HP1yyGI1Zwa90m+19Eb09i8kjAu9v03sdg2ywmX9zjKPsbjdru7Y
NFjuRixf68Uc40B7132qCNpN6C1ojXvsyocCm6/ehO7ObmpWe2xG+u8Y8tn+
6z2XU9vtMy50Zn/A9jth/ybN+QjeQ8xsAiz3wT/AfC76DLvWmOT9TyHzaUwV
ekl6/3TnvlL/W8RuV/t5DJb+9P9IOf8P6hS8vw==
             "],
             VertexColors->None], 
            PolygonBox[{{5428, 6125, 1845, 6127, 5427}, {5695, 6708, 2232, 
             6710, 5694}, {6131, 5428, 5427, 6132, 1847}, {5760, 6750, 2266, 
             6751, 5759}, {5487, 6365, 1975, 6366, 5486}, {5056, 5805, 5804, 
             6300, 1944}, {5000, 5760, 5759, 6212, 1894}, {6269, 5764, 5763, 
             5034, 1925}, {5764, 6271, 1926, 5035, 5763}, {6370, 5487, 5486, 
             6371, 1978}, {5805, 5058, 1946, 6304, 5804}, {4947, 5695, 5694, 
             6123, 1843}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.6350734985609366, 0.6420485884397198, 0.9111230818496634], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVElIlWEUvfVy6DnQG/TX5/M9Q6wUkooGsyCK1kUGlRVmbiKznkRkNGCD
qa0qiAa1Fg1Cpi0CLajUIjKqRYG1aZCkjBYF5iasRed0Pmjx8U33nnvvuef7
ZtYkKvZMNbMmDB/GN89sXpbZySSzdSGzz9ynmH3BPIYxH+vuTLNX6WYd08yW
55nVBMwOTTerht+VHAz4XvLrrB3gQ/ArzjYrSgYe7p9jXZcsjNcOZzLN7A/G
gylm42GztLjueoD/Hv5zEfcD5rUhracFzZ5iXwb8i1GzbcjhNHwbEHMnYv/E
uhOxnsD+Muw7gH0Qd2dw3o35RIbZfawzgZMG/1nwmZMuO9ocx1kv7tr9wuQ9
8Vfh7ANwqxC3DHN1QPZlyHNxnmzLMQ/kK147bEoD4mQffNdFlV8P7ppQ9zh8
98J+MK6zXy7PU2hGI3ybwNUacJUAt6Mu7gjq/ohRiroeIs5ESHlviaGGmLhc
CPsJx89hYLSEhHkU61a33h5QDcy/KiAeWeNN+LYG1IuFEbMV8Nnhky5GPWnj
HO5vIe9h5PoJMbvz1Md2VxvrWoaz7rjqYf9LItJALezrw6qfPefMWMybPWA+
GchlieOzBnGvO111YT/iU+/GgJecpT0H+8rzR+QD8e74VMfmmPDrc82aEbcw
FXFw/8NTPqxvfUg1sr4FTv+FOK9w5+w5e0D+2fPegv9aYmzWyxz/vQH49sAm
PSodHgMnu51+hmHzzOm/H9wMxKVFP+bUuN4E38x53Ldh9pDPgaDO6sFHAuPW
dPU2Kaj+3kasSs/FAvZxp5l+aKMqol7vRC4NQWEmXD5c840yFvHZn9nunXZi
fhwSh/fA21f39ovAZXlYOm0Jq/fs+xvUtTIofl7OADawfoPn1fDb7+IyPvdc
f/ekUeafgvwXRcT5DcRMAKsRsTZFpUvy1uYX1+T5KvyuYQwlicu3OeJzuEB3
PM/3pA/2piSot0ROcnEe8VTzBuBXRsUF+383V70mv11OSwWwbQ5oTZ0Smzlw
Zk5cX0CcF6h5MlW4Ccct/6qzMf1X1PnSdGEQm/8jOSd2zBM+e8X3yX5tRF61
LrddwKiLyZ896XHa5p90zOmKuHx31PlQtnRGTkbRoz7UtRW59YGfYvc/8F85
4jgh3qDrNXVE7VJL76jxmP578tjl/vOWgP4IxvoL4dPuag==
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlElszXEQx4f29aku6ev2f33v9ak0eCQaBK3qBScXoRJ7iF7E1pYIohKx
pZxUIujiXUol1TqgpbHVRRNxIMHFFnscSMpFysHnm+nhn8z8ZuY7853lP7Wh
qb5xopmt5svkC5eYNZeZVU4y+zXBbEfCbDtfB067J5sNYPsSmM0Nm23Cpzpu
ll9oNh3bgWyzLRGzmrjLZ4hP4tuMfg39Lvpn9Ez8a8CbA8ZB5HLe3meYdfK2
BL0nanYJ22b0hcTu4237OP4g+ZdPMcvL8/z1RWbfiT8ZMtsGxjzqX8lbVdjf
FsTM5sfcJr0duQGfNPJf4mPEHgK/CfzzWWZf0StL3V8c18N9XcK5iH8ucl7C
a1F/9hMbBF67elQNfor4aWANgf8G2zl68gL/2eANU1tn1PPfIMcwXFqx3wOr
H58++PbH3beLgVwG6zD+LXzdyJPw7ylw/s9yzbLRR4vNnue6/o584YhzVQ9S
SfhiT2EbyzE7BsYVajiCbQU1VuH7tMB7oZouUn9Pqdc2xtspYl9XUFe+11OH
vhPMReD9Ay8E/0+Bz1IzDdHbY/Snk16NwilH/Io9VjX+Jnct/A7BtYv4x8SO
BN4vxb+C58uoz34Xbx+QR0pdHgr77MVZXNWDmUmvSbWIXwN8ygKfZQs5PhL/
OerxwuwC60LMuYmjenO/3GtRj3qp7W3g/dfM2sCfPJ5PPWgEa1ah9049/Inv
j8BrF4cUtmX4LOWbgbwq4TelXdEO7wH/AXjHqb0P/IfIFYHfht668b1FXAe+
IyGXFSNfve2inh18JzJ9Bo3YbuLTjq0XnzTy2oTv6lHl416uRzy/bjon4T6y
6Wab4f+iwnMJcwDb3rjvombYFPccwtYN3wbvTpnfhm4kq8Rnrlnrhp/Q3/dF
Piv1fAZ6usBvQf3R7uofotvV/L7B/WOx68JM07dHRT4b7aNuc804H93oVfW2
0Gepf0QtsYvH9+kPM3zILB/wnc7wt1awpiXdpjf1qi7uu6GeDcB9sMJ3pw0O
Z5O+89pX7cAG9I1Jt+tG+5jX1oj7Cv8/ZUS9Yw==
             "],
             VertexColors->None], 
            PolygonBox[{{6211, 5756, 905, 5000, 6210}, {5254, 5544, 5543, 
             6513, 2083}, {5060, 5830, 5829, 6311, 1949}, {6268, 5761, 1614, 
             6269, 6267}, {5830, 6809, 2299, 6810, 5829}, {5544, 7082, 2516, 
             7083, 5543}, {5676, 4945, 1840, 6115, 5675}, {4299, 5796, 5795, 
             6808, 2298}, {6255, 4648, 4647, 6254, 1919}, {4944, 5676, 5675, 
             6111, 1838}},
             VertexColors->None]}]}, 
         {RGBColor[0.6547224273093691, 0.6736189495658736, 0.9115710169056], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFldtvVGUUxbcFp3S0tcx0rh1nBqkKiQlJjViupvEVCTRiNF4ITfQBzLSI
GrE1ikHxxUSMCGMviVAqGB6wtT6iBFOiwBMUTYyJpNXEP8AnNfG3WCfx4cv5
zj77stb69rfPqsGhgUZLRDRZy1gn0hE/tUdMYvy3NWK0LeLjOyJKhYjBlRFH
2a/KR9zkfXUq4leeX3ZHjC2P+JP9QDail7gt2LZ2O355JmKJbz349xC78V7v
j5cjxnl/gf2b+JTxmaDuw2X7yedMLuL9LudvlCKGWPetiPiwGjFN/kX8v8J/
htVKrYFKxEFyjWP/rpMcYL4QxrssY8znsDXxm78z4tO0n+J1Tz3iUMUaTBL7
IBiOEHs/tY7A60V4bab+FtY/7B8F12zdOi0UrIvy991tm3Q7Tdy72CcQ91Xq
rwdzE2yfpZxjG7zm0n6Ky9vUn8HvOLYv8D3DWsL+SDFiA/W24zdFviud5nWF
b/vS5vsksbsqfv8FXB/g/zxYBtGwiT6zYPgx47zK2Q+Pw+DbA4792L7m27UW
n88Dif6v8zxYNv7rcDyQMf5Wcu6k1rNodxn7PKtKnmNpayo9r7b4XfvD1Nou
DcnTTBm3ML/RZv7iLs7HEi6bwb4W3XfAd7RozuI7zb4fDHvJcR5cF/Pe/8Vz
JOO+PY3PKdY4eH4D1y3WOp0dHGczxjWVNTfx+vsun5dqpeiNG/h/Qq3XMuYs
vg/hu4OYl/Bfg32I/NuoeylvHMIgLCeL3q/BtpZ1nl69RI5NxFao8UTWWkiH
p9HvGVYDzk9VfAfU/9JUPa6el1Zn26xXA/xD3cZ/lmet4D4aZl8t2Hdn1liF
U725tWRuVWy9Zdv7utxP6qUDPCeSO7g+43fte9B+Q9G9od4URmF7r4N+aLNW
+9tt1/e9+L9cde9fJ+5ncMwQ+0PGd1XnK+0bCX6dueIVm65FfITPc8ReJG6S
+Gar+2VdzlqNpTyXpEMfOXavdO8strvfZdd3+akH9lXdQ8Kpd/W17qNmRr5g
PNJO9cXr8aLP9faZUvdk3vdF2rxVtiY6h7Gkd8VBeqtXNTPeSebGZfxvkWsh
jOdEgu021hb3ku6eaupeqKd6cz6XpQSTvule6gyk/wL5brJmk/t1Nbk7dXh8
WzMmLXEU110597d6ewTsI3nz0gyYT3htQoeNyTz5hj6Zq/9/troP0q2L/FOd
7sff2c+VfC9Wk+OPZP5rdi4lXHT/OurG8HnOc1bfNYeOVj2Lpsm3ouY8+m9o
vqvP++H3WMkzVpxfaTc29YTi1APScjHpgekO95rOVLiUVzkP5T0vNCs0z07l
rMto3mcsHfaA61zNNXQ3lEd8L9Q8B+Uvnb7PWqsbcBrucs3hkmefNPkP5NkJ
YA==
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlM1LlHEQx8eXXNvSdNd91t1nWbe0Qi+Cga6lhnd7UbKMykqwg4Zrb5Bo
r5Aey6jU1CBTKeiQG3a0wjB6O6UHiQ6lRX9AnSroM80efvCbZ2a+35nvzO/Z
2JZo6koXkQOcTE5pWCQtX2SA+5NskeaISBNnjKBOr0gF/m2ckiyRExkil/GF
YyIrOSLD+Buxz/pERog/tEbkXZ7IEPGtxM+Jxe6LGJbmbHVEysLGpZwtQZE3
5K+Q30BBB4ndzxnH7iKnito+FookPSJLxN+IinwC88h6kcOcHrB6OQ3wjVNf
d4FIBzFxfNO5ItuxN2O3k78b/B64LoE/5LUeS8BvDliu1lvB/Ss1lXus31y4
NsSs13vEnwSrk7M30zjG6Oeln/491vMI2G/hWCV2AT1WwWr0G5ZqWEZuFTX9
Jf73OpFESGSZmFHwNqFJK/W0RKz3U+g1SPwe8tszrOcm7sVwFoNVAcYE9yn0
GeP+lBgHrHOpeaimRdg1cPzBf5ec2pBprlrrN8X+nOJXjknm1xWyWnSepfA/
o/9rudZzlNh+any0Fg7VHO5KX6p38F6BN823jiyrZxl73knZ+BP4pvymt87s
Ir3OkH+Hfm97bRfn4Yh4bD+OYcfJOY7dA2fSZzkaqzOcYF6LMZv1zTSRDPw/
gqaNarTIfSlo91u6j9xDnME027F64pcKrTatUWexK6W3ajIcsZnoLGbhyy4S
8RbZ7ileOfzXo2ZrztF8y9HY89S7CPZrx3ZXdzgZtp1W24P/oYuWrs1Gd+Z/
r/hP51jPqs39gO2SavQc7jnOB2JHyXkQMA20d/32GN8O13ZdMa7CfcWx3VbN
K/HFXatNd6SaXawPGZfmVMNf7ZrW+k1nHQsal868DbsG//t00/AM9RUEbVcU
bwtctegX9di+94PfHTK/6qlaZeWbdqrZBfR4kWf16Q71kl9faO9V96MOrp+O
9dcHfx1Yta7d9Q3NxmxHtF7d0Uly2wK2e/omvlBb1LH3p296Av+83/w67wHu
fY5xKWcC7G7Xdlt7XCA/02e9aE/fsb8FrX7dKa3tl2P/Bq1RtdM3rm9ba9pJ
vfEC01bx/wGX2bzk
             "],
             VertexColors->None], 
            PolygonBox[{{5473, 6301, 1945, 6302, 5472}, {6307, 5473, 5472, 
             6308, 1948}, {5674, 6696, 2224, 6698, 5673}, {6120, 5425, 5424, 
             6121, 1842}, {5080, 5833, 5832, 6356, 1970}, {5425, 6112, 1839, 
             6114, 5424}, {5833, 5081, 1971, 6358, 5832}, {4925, 5674, 5673, 
             6087, 1822}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.6743713560578014, 0.7051893106920271, 0.9120189519615366], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtlV1o1XUYx5/NdY7HzaVnO+e/t3M2raZbL8uyUtwM77KUQJyFN1ISRczZ
CxIiZ5tbu8koqAw2iWyzNyxMsYIgehldKEFzU3LzororIYgoopvs8/X7v3j4
/87ze57v833efmfV4/t3DFZHxCSyBJlpiJhqijiWjXg0iehrjvg+E7HYEbGA
nKyPONQScRD5bWnEjejqkcvLI97Arrc2Yi9AC+CcK0Z8y3lNLuL1jPUbWxHw
t9dEnALj0oqIDPft2N7C79XYDTZGPFGOWF8X8S62U9ztQb+hhC9yBzH+RUbr
jXMf9t1ln7fAdyO/r8H/AWy3lszp/Zzx/qiCU+J4itULzzrOu+D2NVwqcLhK
Xmv5dqN/BH2Z7y+J7cfBnwJ/3VLHF4+HibsJ3Z9Fn3+HyyL2t8Fhd53rN4n+
bjDvQj7Ht7Is4mnkJ2peydhvL41YhM9X4MwQ9yO4f4wcIcYYcY8TowffeXQX
kdPoO4l1tsM8hpdZdxXfsZSbarI2696o/rLRveJO32Bs9egIOKPgrwL/Ghx+
KNnuOFzebHL9fyWnJXnqDc8D6D+lXqfAmcP2AvJJrfugHl/vy8qIMy2Os6Mt
or/NOcuukvL4IOfefFHl7/lqn08TZ6jNv28v0HPkRXAyYL5a9rmKcxf4hznv
I+ZAq/Eu5Fx31fy1Jucg/prB67MI50vkchHpyjqGYms2XhI2dZiocX7JSufY
RfzvEsfSvHS2eGbeLjqG8IfhexDeA+TWiO3zecfUbjyL7hnkhTpzUC/6qHNn
OmMfijsyz90/te6favg3938VPSM7wW/pcO9mS+6TajiQzpLqtgUuh4rmLL/N
rfZVzDuXO/eeWuehuEexvwee/dXmKFvN4W7mbVve517O7eQzC/5M4rrL/wD3
xcSztabsWdTMqZ4DKZ8TxH+wwfmONbq+qq1m/EqD51w5jbQ5rwkwn+Q8hP+V
xLukPapG/03i2VMe2j/lch79BPafYT+X81d97KU+m0rOVXOgHRR/vUmXGzzz
z5H7e5wPZ43Xk2L20+ubyfEoOZ4l/nSj+zZT9Lsm34fyrpHqo3y0N8JfgON+
9LdmzUd30u8Cc2fBmE/B9xj+I3BdV/BeKa/tDfaXr7joKz56czXTykM7+krZ
8dbDfxSfPVnP1nD6nuhuMo07gM0+ZCjrfTqTd3/eKXjHdN4Gxo/E7cZmHPyX
y/ZXH75sdy/eAuOxguekj55ubvab9XPiHMR/EN2JFeast1PzrRk7l/d/iXpQ
U/AcqC4L4lbw+6Ac/st6bvU/UErfZ9nNpbupXdJbpllSvppX1f/evPmJ20i6
Z9JrNyppTbRXwlFNlMd0aq+3pCd9WzbA6/5m85hv8vslbtqtoXSex9HNJj6v
xvemsvf1f5EbBfE=
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlV1o1mUYxu/2rrm5Wu7d+7W9+09Xos6vWcxsbVqeaWaBOgVPhEQQndMS
iVqa0+3I6mQZTYmam19o+JEUBEE6PFAEpxObemAd5lEEIp2kv4vrPbjh+T/3
fV/31/Xc/+YPdq7pKYuILUg58kVTxP0ZEWdqIo5wkUpH3M1HzJ0SsQTDO5wr
aiP6n49o4e6zhoj12YjDfD+qjMijaytEbEJ3Drxx7HcUI05VRfz6XER7JmJx
Sf8U+QN9WdrYirGb8wR3czgPVkT8mIBNjCupiEPVEaPTIjJ56wbBX4z9R+D1
Yd8FxjDn73Pgoz/K3W/TIwZqHXvfVHIh17OJsUbI+S+w/sw7/sv4rG2MGErb
dhvykNh7wFj0YsRsaujAtwWMPnxbwdiI/RHsP8d2F9JM/2Yj/6E7SA8XYruq
zr1Sz27h/y32e7E9X23sgfqIVysdowu8dcjDcPw72I8jj6j/AvZnqWUYzOvk
+w81jVDv8YLrVf1biJ3KWtf2QsQ36A4XrJONdOXIULltlhK7ExnnfAP9l/hf
S/u8Ef/tzK4nY6yTVa51InEuqvkdaruEz8foN5PjiaIx5HsL+x/wnaS/86i/
lRpr4FYvGB/iO0m9x9AfrLdOPRgFr6vEp334POD7XsGzGgO/H/tD4DdXuoaE
WX8HJ36p9PxuktvtxL1Vz9aB1V0wljAvalb0t3uqe9iObiZ4T7BfDd5KfEeL
rqWTu3vkO1jrWvQmPhUfcub238TMUtuKxLZfE38ZvkuR98Hay/dQo2emXv9M
zJmcf5/mt6J8RjLmmLilHrZwnoNsSJlj6tXjnLHUM3FHHBU3xKH36E8//VsE
3nxq3I+ur9FcFIf+xXcVPpvLnNPRtDkr/X7kJLleTzu32+QwnXpOczdRZU5M
gj9R6r/e4KmifWQrm9fItSnn3msGw9TzIO9clFMH/mN517I+ZS7IRjpx4irf
nXXWyeYTcllTqk8zugj+hQbPXjmc1znnXCqI3100R8VN7RjtinMN1slmGb35
iRkeqHH9vfj2Nnh2muFl4l/JO3/tAPVydZ1zU465vDmt+Wu+4oJmrrPuLoHd
kTG23vx9dAuyfuvqwU7weuq9q4Sp3bYnbX/tOM3qpRl+C5pJa9YznV/aF7vp
/etpv1/tuHa41paYHwurvUvfIP7/U7xTX2nyDtLuEadV+5KMua0eHKP+sTrv
Ru2DWXy/mRhLmAfQvV3v3Sy8tzgvL7hXiqHYnYl3n+zfJbetjeaSOPYVsQcy
/nfojT4DsSTgaw==
             "],
             VertexColors->None], 
            PolygonBox[{{4939, 5666, 5665, 6101, 1833}, {6253, 5751, 5750, 
             5028, 1918}, {5751, 6778, 2281, 6777, 5750}, {6464, 4997, 4996, 
             6465, 2047}, {4997, 7714, 3063, 7716, 4996}, {5826, 6305, 1947, 
             6306, 5825}, {5669, 7480, 2862, 7482, 5668}, {5790, 6780, 2282, 
             6781, 5789}, {4253, 5669, 5668, 6707, 2231}, {5666, 4940, 1834, 
             6102, 5665}, {4309, 5826, 5825, 6832, 2310}, {5030, 5790, 5789, 
             6259, 1921}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.6940202848062339, 0.7367596718181809, 0.9124668870174731], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFlVtM1mUcxx85s14dvPDywovvPxAPq1WbS8yEHFtNFFkHFVALca42VwLV
ZrbqIlBqreZKwpmYSna4MbxQsRVdRGtlXQQd5mnLddFtB9taddXn6/fZunj2
//1/z+/w/R2fxp2DGwcKQgjnOYWc7/IhfMuZvCmE7lQIX5eEcLiIi3QIn2RD
WInwRG0IMzUhXID9BvR0VQijpSGsRmca2Z0YWsr/rcXo8N+MnVXcDUP3VYfw
YBLCAHQrel9gM4/saHkIL+Drcfhnue/g7h50Di4I4Uyx5VeBawl+X8LvIr4b
kHkUX7Pwf+CcRn5Jqf3sgH8ZbFeROQfvTMqYhOf2TAhXYiwD4HmiPoQh+A1l
IXwT4+2t43BXBa8LPNuhM9CN0E2cv/D1FjnZtBC8BdaXb/mVjvB+MC+EdnCt
5bQgvwb549C9yA8W2Z5k2sD5LPF04HsfePugX8bOCHRbnemOBuLnv5p8tC9w
rNJtxd4Kzm38H8b+XN54ZF8y5wqMZTLKby021oPkMEMORiupN/yxcuOWzDZi
Gk9bdwxshzhjpbY3GOP6Hj9Hokwr92vJ6YFi86Qv/gQyh3Ih/FPm2laUub4r
kM9WWl75E25hHkP2w7zl1xHrH4lrtufmEK7l3Ft7yU1/rXMle8/HnlEO7o55
WIbs6xXg5f5dartePU18y+EfhT8l+3nHr9iVH/WYMBSDazzj/OzGz2boTvin
kO/PuHbSbyxx703QF5ew8zE21mWcC8X1JN+nOC+W2rZ6UvbVs3dkrPtmufOp
XLXD/ynn/jlGjO/zfwx6G/ifqYzzkfOd+OqL4Sr3xlLyt6vWOBdDr4l0b4n9
S74bjF+mPc+q1V0R5wz8zzj1yH3FfU/WM/A0981p21iUuBflaxz/R2rM38D9
9RrXoAD6YtZ50Z6Yl/Z87eH+uZzrJYzCJ2xvU4d8nOV6vgtrrKtayt4J+HO1
nhnlfGvKuFSvn8n578n/+0Vf5f9v8vQevpcjX0iMjyXu956Uv6ppZ5V3h/Lf
T25ns55b6f1baV31mXaP9lhz3nbFn8u6L9WTu+jbzrT7WvLafcIyFHePfGlW
1Y+KUdhny41fc6950xy1EMtZZns/stegfwPzefBOcz5KOTbt2h8rvG/lT32j
ntGdZBT7neBcGft/KmUbyn8TuV/MmSzzDhiIfSi9qai7nphO1TlexaFdqViE
5deIRzURRtXlEXDen7hWbWnXSTXah413qq2rPtM+1S59Fdn98LekPCsnqu33
dM4zr3n/JWe7sqn4mlKejc/JzUyD3wHZ1Juk3CoW7UHlcBP2NyfWuZr1TMp+
F3ujm5OZH8JJ8peqNP4bdYi1UJ1FK3bNqd4qzWpRxj2kvtH7sDu+EdqXlxus
uxGfO6qNUzHIj2qkft8b9/myxG+G3oshZF+B7sLHvdTqvrzfxS31nn/1zZ/c
X09cf70Zq2MfjuD3U2IZxsfxGr/BF2JfaTaUZ+GWPdm5AsbXEt/pPBxlttc4
F8qD3rS+OvfjSJ33rHpZ795Did8D7d2T1eYr9z0x/6pld7R5482db/+3kLOW
vOdFb7XmXP0/go0DiXF0Z+IuK3TulQ/F+EDifaddd5H4LnHOYvs/AGoohQ==

             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlVlslWUQhgc4LIZi2r/nnJ7T9j8tWwkETIgWKwVi4gKUxoWlgKSWEPfQ
oiaoUS+kpRiXkFisEWtAqXqFcEELROECl6By0coS0hq3C+/UKCQmeqPP69uL
Sb75Z76Zd96Z+f7ZO3at75ocEY8jGWRuKWIO8vKUiL9mRtRzfjAf8V5Yv8a5
ORux98aIF6bxsSKipT4ii74PdSAX0VYV8Q22PvQl2N8oRByYHvE1+ivEW1sZ
sYP4m8oivsR3IXeWAiLF51n8N+AzF9ubN0QMEbuh5PzKuZHzemRXxj6DYBlC
Xwm2M3zbTLwPayJGuXsBDLuJ9wj2RnxPTI0YI944shV9ZFLEfdg+R+8jdhf3
z3F/chIxm7s14OksRnyVOFb9jIhniHcdDl5EP0COj9OIo0g7+P/GvlH1I4eo
r5Vv75YTJ+94J7GvxXdN6twj3P8ILp6Gn/ZpvjOv2pyKy2P4X8b3EjJMrOPU
OA6+38B8EvzL+NYBvm3U20+s3AxjE4fiThhfxbcHjjbj34b8gX4dOcN5EXwM
cndP1r0Qnue43w6GMWK1cL+KWMPl1vfj343va9zfgv/b8FWL//JKc6UaHwDP
usSzUc6dZWBvTN3fxeC/Cv5W/B+aYv8i95dWmxt9uwnuluRcm/QWfD+oMTb1
+NHaiIPEH8LeTf7fwRKJ/U+B6Va47Kn0/Arv/dg7su5tJTkeRu9C34O+Ffv+
kmtSLdvKPGsLSp51zdzdYL8LWY7ejPyJ7eesuWug5p1g60TmT3fM77F9W2Xb
CeL9hP7rRL9OIyn1zq92b9XjayX7yKaenK1zjaqtexaxyH0ROTbTNU9KzJm4
Uc2vc783a+ya6eexNSSePc3TUfqxOjV27VxH3jut+sTRWbgayHv++sH8Wb13
RruinfgBrFMrvMvb8R+ltt4KY1PNh4n9furZ1w7szXrmNGuaEc1eJmeuNYNN
5Ls59SzozdCuDk70VztbRuxfyPlExvz/g/5Fld8GzVAjtfUXjFWYbyNWU2ou
FfNf4o+m3pXVxG/mvGIin+5rV9qLngXtzDixbk+MfZiYTxXMobh7CX2V7AXv
gjjt5Lyz4LO+jaTOqVzqkXr9Y9azqJ5rV78D/ycZ6wPkeqzWu6md1m7tzjuW
dmy7drHkXdYb11bnN1RvZ26WZ1EzptnSTN6JvqXGvWlC7wPb4bz99ea+w/nJ
gmsRX1fId7za/ZLPEWL1Fv12firO0XuKPgvzW/huqnPu/3uMvibn3RLee8B6
b8lvoXrWiW2sYC7FaTfc3FKwv+atNfEbobdBHJxP/MbqbdU/Q2/1Halr0Zv9
H/wX5Zg=
             "],
             VertexColors->None], 
            PolygonBox[{{5824, 6362, 1973, 5083, 5823}, {5052, 5793, 5792, 
             6290, 1939}, {6360, 5824, 5823, 5082, 1972}, {4922, 5664, 5663, 
             6082, 1819}, {5645, 6695, 2223, 6694, 5644}, {5793, 5053, 1941, 
             6294, 5792}, {5664, 6691, 2222, 6693, 5663}, {6084, 5645, 5644, 
             4924, 1821}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.7136692135546663, 0.7683300329443346, 0.9129148220734098], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1lVtslUUUhaf21NPSI+k57bn0FP6eImk1pA+EIFYjIbGKl8QLtmhioOWS
RoQKJl7QEIrxEn2xaSVUpGpA20SjRhJ8sLRArEafjBdMjIkFTXwxRrk8SRH9
Vtb4MJn559+z9t5rr9nTtnnnuseuCiEcY1Qz/lkYwqeZEK6vCeHnphBuWxxC
B+uOcgjXFEIoXR1Chf9tjB2pEJbVhfBFbQhzHH6oJYRu7EfYe46z7/A9hs21
nK1wtrfaeLdHzHbsuupDWEkAfdjfnxh3TSmE1Yyt6RAaiyEsZAzj9xb2bmZs
YX8cu+WMSyljlRmbqh3TPGOEXL6us/9fwW8ixr2sBxln8PVH4hy3p4wxWRXC
hUY4KDmXsVwIA4tC2LPA+WhWnJfhZwq/3Zx9mPz6mu3vHJi/J97/nvWOZsd0
AIwxxjHOz9Z4Vjzf4Oc0/j4ml5HIqWJXPEnGMayEpxsYnXDUhW0KDnvYLzNX
xCnnfgLnFN/bWOdqjSWc1oxxhJetdY5af5JxDorzJTC3cPZlMDeAtzuue7Dv
SDv+WfL4i3y2stfFegXrv/l3grOvY38U+7XE+EGzzwlbGhK30pHW8tkGfn2M
/1nmx4n7bmIN8PwnvC2Bl+1gV+Vdp3Xs9Wifs+up/6G8+Xsev6s4208MTxSc
g2KWvm5qtMauNMAzZ14EvheMfnCXgtOMTVXOvA2lrAnpYUmW/MCcwX6QPL7M
uUYj7HXy7038HmTdl/f6MDjTqh2+ZgrmQjy8hs1+DWK7k71XEvN4L/M9iWtS
wv79rHORbsWZeP4qZ9/yK35VM9XrB3C+A/No2nyfjL7uqhAsOPtqrNnzjdbM
RjCmW53bcNSE9DAcdaF78Ru1yGR9d6WLoah/zeJb66moE9VR8SgW4ev/ULxH
Gnsj5nF8PtLib3Etu8F4FwejjXT6Y8laFY/iSRxO1jle4R7CZiZy+zk1vJR1
zznC3jbO7kk75uUxDt3xnozvuWbxrfWtaHIAzieqjKeaCXOCGB8s2kbxSnOK
eRO2mxPjSrfSrDTZS14Hyub8DP/PMt5T7ylbT9JSM7XrZ72a9ZPE+1nFfk/h
5ySjnHafUx9RDxmt81o2V7B/K+/9D8H8iFFTZ/6PZ9zTarCZK7tf7oKDNwrW
6mzsoTpbIa6+GMNB9J/KmnPdg3TBd2FDs3uWaqKYpB3dl6exvVhw3XeRS7bo
PjKXuJdJn9La0rLP6B1Qbf5/C2SrXMS9uJG9eka61X2jBfvxBseguHJFx/Zq
g++q9vNgFxiTsYeNRn50pxfFt+M6eDpRdAzTzKv4V8Jne849Rf1E/VT6Oh3c
46Yih3qLJuJ7NMnZTs7WY/soey9krUH1V/Up9ahvi37/dEffZb0M+wXYr0BX
Nzba5ghnR7PW4H7dzQbXRfX8N+u6iNensuZWvWR9q7Uv7pTr7mCOuuP79Qs+
zyeufXsbNeB7nnv4QOI6CUd+1LPk6yL75xLnuZbYpxf7PZb2pUVxuCbnd0K8
6C3dmXM/0PugHqB7rX4zHt9T9ezLsW/fkffboB47i67vS1zz+fhfcQ+A/Uzk
X+/q2Sb3jY0F61KaPExcbzP21fv/hfj+/gdauiAs
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlW1olmUUx8/yedor0nM/z57du7fde2Yxi/CDSNqgF6FoVtCLba4X3aYO
yWZpEGEgZVRgX6qZOUwD0xwkEgr1oTY1GlGfIqO3T7OCvhSU2qcaVL8//+fD
Bdd1n3P9z//8zznX3bdl5/qnroqISVaBdb4j4vr2iL3FiKwx4s6eiDtYJxoi
ppojfssj+vsiFpdiw+fbcsS5asT2qyNO47+9K+IJ1gy+LwB4II14C/vpJeyx
j4H9Zur9O9z/IItYXQKH2EXu/MP+BBxasadgPo5vfxJxL/s9fPsG20U4vN8W
sZzzUbidBf8M+HtbwcK/E//b8B/H/jTnXayzYYwNvRHDvfZX/Mc6I74H7wh4
paaIy+QzwLe/ubuAz77cMRVrCJ/j10SMVo3/Of5/Yr9YibgBrI+x38d5I+ec
fW+bcz9QdXxpsCW3RtJmJfbLnNcRczO2WXEg1iPY96PFFDGGsD/EmkH/Zdib
wfq0w7UZXmLum3NjKYcH2I8Rv499jfUg502cdxT87SfOC6ysaA33l5yjcvuK
mKfI/e5yPV/yq1Gf18j5FfjX4LPI/j3it2DfHa7d16lrrxrOUY8rZWsnfbbC
91bsW7Hv41sDtZnlftpovN3Y12KfqNu7OXdVHU/5nQfrUOpaqWZ/5NZcWi9r
iXgDPssz94r4vM3dw/V+k+Zz3J+q39e3Vmxr4LiCXhnivJF8dyXWOoHvJbB/
rrgWn7CW4t8O/kyTY1Qza6BY+naUWh3rsna/MEirKu4p9ZJ66Ap4f+XGUo80
Vl1D5aYcD4L3bo97V/l9VouYr7l2mjnVbhN4iwXXcJzz+ty9cB3rR3L7IbX2
msExfEfxebLgO3P0+izrefYVOI0TezBzbdTDg+jzcJdnewAObSX3uOw38m01
3Fb1WCtptq074l98biLXPei/omyNpI1mdg3nrOraqsYNzPsh8KbCHC7AdW1i
rmewH8G2MrO/9HgULvfnnp3Jgt+W78r21Z2b2d9T8/skTi/DZbTTuenN2Zm4
pqrl6/iPgDedmKtynCD+ttRvgfrjVWK9iF7TBfe8ZvGC5r3omfyCuxs4zxdd
37vQYrDHsyEfzepIXT/N7Dl8B8rmJo6Hyb8Ix/5mY+ht+DWztnojni05Z+Wq
GV/IXBPVQncmyWWeNdFkPqrtcO7aq8YjxPsyMTfl8Du2Qrdn4yP1CHjXZu43
8ZFtrj5/8tnR6RjC1ptwC9qsa/csamY1q5HYV3dux/5M1XbNbCO9danif4E4
niSfg8T4sMX5PofvsbLfbmm+UDFHcdM/ZAz7S2W/1erHhHjlDtdOPfMfeGnV
WOq56W7PnPJVjP8BaIbfrQ==
             "],
             VertexColors->None], 
            PolygonBox[{{7479, 4601, 4600, 7477, 2861}, {4601, 6054, 1803, 
             6053, 4600}, {6203, 5447, 5446, 6204, 1891}, {4645, 6207, 1892, 
             6206, 4644}, {6354, 4671, 4670, 6353, 1969}, {5447, 6198, 1888, 
             6199, 5446}, {4671, 7584, 2944, 7583, 4670}, {6249, 4645, 4644, 
             6248, 1916}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.7333181423030987, 0.7999003940704882, 0.9133627571293463], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VVtsVFUU3e1M53aYDnZe995hOncKCRQfRLSlBInpRzW8/EFMCFDUtoQI
VMFoBA0JmGjslxKEGRFIoDxCC6gJKBIqYPTLxFcDLR8G/dOoJGKiFkjQtbKO
HyfnnHv22Xvttdc+d3rf5idfqDezjzBiGHczZutbzKKE2bGk2Y242fk6syq+
jeDsBtZ/5sxGy2b7U2YHCmZTWs2+bNB8sKB1JaGZ9tcjs58wTjSZdYZmi2Dz
OILWsmYrS2ZnsF4J27k4/x32f+TNvIrZt9gXs7pD+9/g4zrOZsO2p2i2GWd7
8b0wzWyqb7YMCdwTmDVj7EL8LoweT3dXN5r5nu7uAP4HQ50Ry0Ln/2yTYnbD
5u1ms1b4PYV77YizETZPweYa8E5gTIKbUcSJt8jnMsxPYPRPMfshprkK+05w
9gH8DMHmDOw7EDtHHIh7Cfu5sHkaubyREc/jyHEN9ovB+8c4nwf7POzvAI8P
P9vhZ0tW+TN3cnY0Kd5YK9aMd2fAf1tZ9sRBPmg/BOyHML6GXbenGr4UUw7k
ZwXWnyLu+UB7+t4Zl/86cNyZk00trTy+wff2knhnjMdQt26MBpzPBM4N4G0J
vq/HvA7jNU8cvBXpfg73shi7YZOEfw/Dx3oF8B+G3X7E2hWqBuR/Lc57c8K8
CHw862v9c6Pi0J4aTAfSH2vK2jM/5kzcxEx8ibRqzRxfT4v/GfD5Drg+jTsX
kMdoRWe9iNPnS1Mf4s65hHTblhLH1F4NON/DqJK3QDXjWVcoLoiP/LUVxCG1
Q00T27VWcUdch1C3Ltg/B/tJ+NqSVm03gOe+SDUehI+ejLCsxXyyIrsKcI8g
52pc/LLnmC85OOu0zTvETj1cRE5XgOky8AzDz9WSas16UMfUzk703ESrfG4s
CQcxdGRVG9blM1+4ifllXzkz3z1Y78VYg/Ur4PZis2r1AGImsF8aU87DU+Wf
vEaO2zeR6w7EXg7cV3PCSpyzppv9Gikf+sg7P6uAKxPo7il8u4JY8aT4OOE4
6Y3UY+yRfswDeembvBED+RmDj8FIvB3H+hZ4CYBpGuLPd/rnflZKdWcM5k1s
7PPBlNbU6SNF6ZM93Z5X3GPgaQJnn8ek8Rd96fYL2GwqCssCzDdx9hVqeDlQ
b7AvuKcm6Ic1mplUnZpgU1/Qe/s38B5xbwtzanA6H0NO32NUwcUm6tTV5cdI
OTNfvmvvp+Tztidu2BfkYH5ZeXcgj3WRzoeA8VWH/1H47Hc98kkgHMRwEvcG
CuLmu1BvItesCTXKuhAT/VHnky4n6nAf7C8hRs3Tv2KV+19sRX23YfzSqDu8
S5zsm7FQ9RjPiWvyPOl8kzfqcU+o3Gfj/N6yarkQnMdDvYv349t9ZfX3Puio
rqB/1ALqELkVPWlkd0a8/eWpr9hT5O///wDfRJ7xO/f0wbyou/GS+GXMehc3
BLZ/M/p/snbUAvl5CLYPl9SPfL9rLXrP3sW9ZwrqQfriW0YeGuHnXCDNsG4p
X7V7Pi/d05bvH98FaoM6IjZi5L90Tk7aZu8xDnU1DD8jZembOh1wWqXW/nG1
/g9sUx1M
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlVuIlWUUhtfsOewZR2P26f/3/rezt1PkWBaNaVRKBA3RwW7KICitHEvM
sTSMogg06OBdobYHmwgSixytICUTx9Goq6CTqeOVdVdE0QGioYvqeXn3xQff
96/1vetd77fW+ofGtt7zRC4ixlldrKtLEe+mEWlPxOrOiAH2y8oRf+UjXuPb
qYGIvUnEA+x/6o14qBKxrxoxgf1egCaxf1SLuB9bz4KIfnyHsoj3e415EazF
QxFH50cs6Y7YuJC7xYgj3G1gv57406lta4h/CfsSaze2Fj7T4E0Qr0W8TXzL
NyN+A3MU/6/BnCH+dnzuwPYjMc9jPyIM/Cfg8wZ3VxTNVZwL2LZxfj1nDtfW
I5bXnavidZDfOPgjYN9HjOHBiKvgeCfcnu+2bRP+B/rscxn2pazhfuu3ing3
sW4G71bwWnB7JrN20vBKfC+tGEt33q6bs7h+CWYO20uNiLvB/rUjYmvRmoib
OBfZdxUiLu+zRj/j+3vZWkjj8+AdxP4Ld3fywCOcz+DTEh7nswPWVFp2gTG7
KOICq5v4Lewr4b4+Mf8x9FoO9o287xfsb8d+Av6recMN89AW/ovILUv8dofJ
cYrYVc7NHvO/C99ldfPXnbNo+Q0x9pP/ae68Cp95iWtFNfMs+43YnyPeJPaV
xF7FGuW8nfOMagWNRsCr8e1v4p1Mvde3c8S6wJrrswbT8L0t891J+K7F/xb8
jxHvQzj8AZ/OqvGV427s++t+i690n7d5kPjH2b+ARlPg7cnMfRd4W7B9h0ZT
6NvsdW2caNpXNXIG7HMl+0vzRzmvqTg35ThGvjuwr81b88ekTcl21fRs3ZpL
a+V0HO4dbb0T7uTZH+VbmX3S4178JLVNPXmM/ceptZH9LWL/W3Dvfd5tLdLE
Z2nyIrbNxHynzzmr1z6rOTf13KGma0q1tI0cE7SoZK591bS0/LNkraXpaNMz
QrNBNXYdtt7EXMT5BrDXFYw1x3kL/A4OWlvVyDXgVYvuJWm0gbsvl1yb0ku1
swT/xf2uoQXkMz917Sm/J7E/nXk2SNP/CtZANtXoBPXZWmh/vbl6a9+Ae189
NkOsk+1+0QwSl3WJ60mcvqW3VqDRP3n32Clin06dnzTP1ZzjXLu+NpPPeNWz
QDNpL/s9Ve/1Tbk91Z5nynEp5wOptdUMGoZ7M3OvqQbGiL++4d7Um6k2D2fu
bdWcZqN6WljCvGLQM0vvJc12lD2DlLtmzgea3axPO62//gW7Ku4V/RNyVeek
XPS+D4Nfa7+PNHkELo+XPXukyQ+cv2+4l94D/xX2O8ueNZrRqr032++hGpwl
30ODji0O6q2LDd9Vj/0Pmvzgrg==
             "],
             VertexColors->None], 
            PolygonBox[{{6296, 5785, 5784, 5054, 1942}, {5637, 6690, 2221, 
             6689, 5636}, {6776, 4387, 4386, 4287, 2280}, {6687, 4351, 4350, 
             4238, 2220}, {4351, 6069, 1812, 6068, 4350}, {5785, 6298, 1943, 
             5055, 5784}, {6077, 5637, 5636, 4921, 1817}, {4387, 6245, 1914, 
             6244, 4386}},
             VertexColors->None]}]}, 
         {RGBColor[0.752967071051531, 0.8314707551966418, 0.9138106921852829],
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVlloXVUU3Xltk7wpffN9070vTYitraB5SaokFKGimDonIg4pYjFgq01p
alNBk/iR4NhqmlcbY1sRI7ZNUXGMOCCCYL8c0ib9FL/7U9BE0KJrsQ74sTnn
nbvP3muvs84+b8OufX1DITP7DrYG9mnCLJc0+wDz4bDZAdirdWY71prd3KD5
cM5soGh2e73Zgmf2ccFsAv7vps1GfLMTCHQI39/IaX5H2ewrrC9gby1i9uF6
sxkk7Vmv+XnM/0bs7Q3Ks4LxMsbbYBHEeA2YDiF+OTDbmpFPZ8msCzYCfJ0p
s9k8MAHPcYxfY8+P8O9tNtsETKkms8+A81/EGV9nttqgfUeApwk4fVgfcI7B
9wHsncR8C/C2wlqi4uEKxl7kfRQ+Vz1xMYm1l2CLyNWfNRtE7hriPwg7Aiw1
1BWDb9zTb8a71sVsQM7uCvhp1L7WpPaewvcavv3VKNz0Ze294HA2odjjwLDL
4RyB7/6czmIlLE7J82XEyyHWZmAI4fsaWKZefPc4/o/Dng7Lfxs4PAn/Aay9
HFVtnB8My+dwnfirFMVhFRj+Qc33AcM01s76wuwhz3bU+wr8PV91skZaW1R4
3veVj/HJ47DTWDVm1hETz+cdPmKjBunD9bkm+fHseI5V538Aeb9NSLffA19t
rTC/ifUZp43HMO/OyycObdwLTpfiZpego4sl8bcTvN1UEJ6ujDRHvTEP8zFv
H+b9MdVec3qlVqvwv7Egv3cQbxy/j2H9EnItw84C+/NYeyHQ/r2YPx6o5hfB
ZXtGcWpO+8R/a1a65Fn3uLtCThpRS1daumUNvFc800fA9wC+bUO9U1g7GlGc
pYowEc90RN/I7yZfd4wau0g9wue3kO4fR55jB9aqpf+1MOVinsjqvOmzE2M+
pbw/oaafYcdQVwp40rA5nP/n0MUXsCz4ydZrjXl/B/7BiGpbDeuMyTP11ZEW
z+SY8YjrSZzhHtgo1jrB2S+BNHAYawvwfx3rdZj7gXrKJMZQVndnHvUup6Vx
+lEX7BWfeOKI/LAHPFQU5+sw/8YT3h9Q34zjh/2A58y85IO8kE/6riblzzMk
XvITBOod1Psi6r0QV39gDQ/Xq08acv2Jvel69dS3EuqrHG+paD4Nn6M57eE9
3BKVBlqKwsq8/fC9H5aJCyvzEcMTUfUY5roBdcxXdLeoHZ4B+W9Nqb+znwRF
9Q72DWoz4ammk9h7qiSdEsuoLzyLwNmVVD+cw9ppX32Eurg+L208izv3UUoa
JAeLDuPBDWb3lKWDsYxyMP4o1p4rC+MOjHtT4q0d3+8si/czyPNUVv2E/a3m
tPQH8EwB827T70Gn+buwb39K86GU4jLm3WXhoN8V1DWc0p1iz2B/Yd/4NRB3
5K0Fea9x/Xw3zmsPLGhUH1l178tp8DERqCftaxYv5OQM1pdK8mOOFefPHspz
oyb5Zi6XdN9591ecL3XMPNQq7zdj8z7ynHi3qIc64P3Sk5ZawM1m9xawt4Vc
f5vNCh/3cy2U13oP5t0F9yYjxkBS+nkbWLYWdBevg5Y3+urp54p6A4idPY69
jT2SbzL7L+9YO3yrvvTKN2RjVvo950n3xMk38JmiNPqeezN4dhMJnSXnbb7e
NtYyjvmYL11fxfd5T3eEPaPNvU28r+xr1Bj/F1xIiNsoziLWLA1Ss0PN0u1/
SJ0z8Q==
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEls1WUUxW8HO70W2zf39b3XWlPAQkLoZIIxJk6xzrYuHMoC1KgIJaAW
Ipa6aJ1RhGIJKRgDRKFESZyKCxcmJu4cwJalce1CFrZGQ+LvcLr4kv/3v/c7
995zz703bN05NFoZEXs51ZzrcxG9bREv10ccqIjYkI+YSEVMVUXcWhNxthRx
hjPCo3/qIg5i70tyx3aoNuI4thMl2+QzUoj4sjniM7Dvwac55xjCvpuAlZmI
Ru7vYXvsuohSOaI/HXEvttvBW26JOIm9nu8MPp+C/clK/KlExAC+Zd7cVus3
p7EtkO9bCdtfLUa0dkT80RQx3RBxiVzOFZzfj+S0jfx7yf8JsMfB2EDtBfCu
8HY3HDzaHrG+xb5pML7j/VjBtaimX/D9lfMivoPEbyLX49Q0XemapvEdbvdb
xXyWeDcm/X4f8aqyEd0p15fm3zzfM1nzeZB/7xOvs+Da9xAvS305zixY3cTY
i20PZ5b+TMPfUfBn8s5XGHPkc46T4nuSf+O8PZmyv2q+ifvakrE7qbmat7e0
Rvy3wudafDtLtiVXRSzBxdfgZVbyfZ3arzYbe6gx4nNqO0b935PLIfj+DT7P
8GYZfpY4a7ANgJnj/QQ+34BVR71DVcb8kO8X8u6FcjyPvQf/R/g+Qj5/g/Vt
zu9V0yL4l9uMPY+m1ijXjLnpSriXPWhkudY97Qfrq5xjK2YJ7soF16L7UMY9
Um/EZxpbTcF8n65zbxvQ08Um97gPvC9yzk05PkW8HcTrhYseziy5JfC/hP/v
+G/Bf3PWsyR9b+V7S9Za0oxdwH44695onqR9ca78pJEL3Odz5lv1P4S+H+DM
gP0MfP/F2yuco9IffOygH4NFz5byfaXoHqk3H3D+pXeDHe6tNHA+aR/ZDjd4
dh4sGlsztBt7E/8WmhxDs3Egb61e0xyxK/KuTTVuanXNuktTi7x9DX6G4eYs
MR8Ge7zo2RRmD3xt5LxU7x30Rtkci1u9eVr3tGftGsf0e1XBvVtHf7u5r1vR
qzRwM/H7wBurtwYGuP8ExhHeLtV61sS5uNa89LeZQ3GnNye4f9Rm32li3sks
X+5w7tL7LvjYVzRX6sHHYA1nrB1p6GrOO0W7RDvqZ8XG/89q5zDJfX/a2JqZ
O9odQ9jqcVfGM6a7OBiB7zcz3p3vVBjrh6S1JcxGchtNutfSXGXeM61Zfhf/
/SVrXPk9l7CWZzPGkqa1C7XTxYc0OEcum1v8VjtuglynytaTND8G3kTJWJrX
UfB2dnj3XpRGC9aYtHWK+VlNrI0l90ozqN1Xk/V+k763Y5/D/nbCO16ztz7l
WdYMLqa844UvH71NFowtjGPEep4el+uc47ZW7xDlqn+7su6ReqMdrF2inJSL
dsr98Hlf0btEGlygd9up+dQq77AsXNyV8axqZleTS1fJWpPmHieXJwvWvny0
6yqynlXhb4LPSeUYrkfY2hnaFYrxP6CQ8iI=
             "],
             VertexColors->None]}]}, 
         {RGBColor[0.7715982075129616, 0.849988324765867, 0.9100136990700004],
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVmtMlmUYvgER+D5OHx/fB98JPghcUMZgamarrc1W5CqSw1aYG2jhYYG0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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlVtsVVUQhqc9YmsPp5z7OT3d+1xqMfpATCtFRDExgWghapXLg6BIqVZq
LJZEpFQKKvHygIiK0QJtEIMoykVNxGr0RYUIiloJPpn4aMR7eTGt+I1/HyZZ
a69Z//zzz6zZlc51d/ZWm9lW7BKsq8HsPiwyzeyDKrOhRrPdWPkys9Psw6JZ
Vd5sBN+/o2aPVMxuC8x+ZP/LDLPpZbMo9iX+Z2NmqdAsSJnNj5g9CmYbWLOx
xQTbdKnZqqTZvyWzk/jeUWe2M2c2HpfvBOdzA/n4WQfWx/pWvnWxvh87lTbL
w2H5NHHu5Gwp+EdJqp/zs2C/wv4EfF4mRh/7N4nxJHxnsT8Iv9exT+D3NPnk
ya+EXWD9B/YQ8RaB2QnWaqxILplQ/G7kzl0Fs2VZ6XGu1uw7cv+QGFvB3l9v
Nsn6Bu4v4e4JvoX412Ov4XsIOwyXFjA7wBoEswft1zaIX6IGbLgfB2NLTDkt
gs/vxDvGuhMbLalmXiuP6bGux/qrFPMi5wewZ6ql8VLWS7AjxNtDvG3kejBB
7fDvmM63RmnmWnkNPyPeq1P6vQvebnIfTel+N/Upk+8X+PxUrRrMgcsI/qfx
/wrM3zLkSo615NIekZa9SfWGa9oM3lXg3cLZAPeN3N7Gf0aNNLnI/XH4XV2t
/V6wB9PieooYX3N/daBav0O90uzDlGoThd+FhDD9rmMMcrYQzO3EWkGMQ8Sa
jEt7j3mMfXNG+yT7zeC9wP2PgekGfyG1ayiqN26mhzdwvj+lszVRaTWGfrti
0szvvogdjggji//5hLgV8K/mrFhQL2zgPMpZI/h/RdVTM+FyTUpcnNNHrPeE
qp3X4Dj77XntPcb5jDR3rR3ve7gsy+ntOqcOtIpRswfQqxurIXYuFBfXzN/K
59TnvTrxn1eSj59NoPk3cDtTVG3dZxXxi+zHud+Hzw64PD+V7044R7LK0bk4
pyawH2b/FOvF1CNLvu+j/w+1qsntgWaKzxJ/44Nw2xJKW/dvw38M/wjc9nkN
yeXuss5d37Wsu7FR8Ft4f+fg3l/WW/cZFmefKEh/77mVrNdP8XEOPWm9YX+7
/uZe4nzYOUSl9xj5nskr9qfsl6NXF5w31elNXsl5e1m97PhveW0Leis+Y1YS
a0VBs8LrG6BdW1qz1Gs+mZOmrqX31z3U8rm8tPQa+2zcFlctfAatCfQGvPed
QyvrllC97jPGZ9s6+MyuV/wFBfWwY3uM4Ua9KX9L/mYfJ3ZrXrVwTeZTrzdC
zUbP4QDrb1PCcg3mpJWDc/ceSIGdLKjWrsE+8qlktPYZsDcUpmO5pq795UnV
1uPdRL89G1e/OH/vpR1T7897qodc7sX/Ce62ktMCsGfl9Zad8xB32wPNap+h
JfybsJ9jmlFHuTsQ6N9xkv0V8GkKVatmMHvL6jHPzTXztzoz1Jn3TKWsmeiz
0DGvZd61pqWn/9MeA3tzoLfl/0R/y3Mz0to18lk7lFFsn7nX4Tsv0L9tgP0I
eMMNmu0bibfR/zWh9PEeL7OuhNLGa+y9/mBZWnjPuzYTJf1rXKN/iHUkp3/J
/3rA59essP0ftx7/P7P6F/tM+A8wcRal
             "],
             VertexColors->None], 
            PolygonBox[{{6190, 5719, 5718, 4990, 1883}, {5629, 6066, 1810, 
             4915, 5628}, {6063, 5629, 5628, 4914, 1809}, {5719, 6746, 2263, 
             6745, 5718}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.7892442791228214, 0.8558728194515265, 0.9021082748020038], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1V2lsVFUUvoUu005n6ExfZ9rpvL43Ai5RKJQYihANCDGa+EMWiRABBZVF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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtllls1FUUxq/QldKh02ln2ulMZwqFyANL0CgIQUmqRt8EigqCyKIIRlAx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             "],
             VertexColors->None], 
            PolygonBox[{{8000, 4345, 4344, 4233, 3330}, {5624, 6684, 2218, 
             6683, 5623}, {4385, 8284, 3622, 8283, 4384}, {6061, 5624, 5623, 
             4913, 1808}, {5723, 6195, 1886, 4993, 5722}, {6773, 4385, 4384, 
             4285, 2278}, {6193, 5723, 5722, 4992, 1885}, {4345, 6045, 1796, 
             6044, 4344}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.8068903507326811, 0.8617573141371858, 0.8942028505340072], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmHlwnVUZxk+aPTf3Zrk3271JbuqoMywppQtLmwqydMYyDrahY8euIG1T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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3ls1VUWx2+F9rV93d7Wt/RtiLugDBARqTpuJGqMoqgoCopsToKMG5Ei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             "],
             VertexColors->None]}]}, 
         {RGBColor[
          0.8245364223425408, 0.8676418088228451, 0.8862974262660105], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFWHtw1NUVvgnksbvZZJ9Jdve3mwSwKqU8GgpFdBTCQwGRUisERCxttUIh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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3mM1OUZx9+92Nljdmdndtidmd/s7FKtFShHQdEaonK2imiprQIClSZa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             "],
             VertexColors->None]}]}, 
         {RGBColor[
          0.8421824939524005, 0.8735263035085045, 0.8783920019980138], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1mHuQ1WUZx1/2cljO2cvZ3XMWzp7fb/fgeElguGipOU2Um0mGCikXwQvq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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3uQ1WUZx9/ds9ezu7C3cw7n8ttzcDQSGVhispsT5UaiQwKaK+ANsAkW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             "],
             VertexColors->None]}]}, 
         {RGBColor[
          0.8598285655622602, 0.8794107981941638, 0.8704865777300173], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmH2Q1VUZxw+w7wvLvbv37t27e393706C71pp6gwzluZM00wjLyqwsJaQ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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3ls1GUax992KPYcmM7RTju/6TQB1BXW1UQ0MRFRE2PcRUClLegGXN1l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             "],
             VertexColors->None]}]}, 
         {RGBColor[
          0.8774746371721199, 0.8852952928798231, 0.8625811534620206], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmWmQVOUVhi/MPj0z3TPTM83MdE/3BDUaRKNRq8wPCcQqScmAJSDgEmUT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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3mQ1NURx9+eszuzLHsy7O5vjpVDMAETUaqSP1AIVZoKR4pDQE2ZyAKB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             "],
             VertexColors->None]}]}, 
         {RGBColor[0.8951207087819797, 0.8911797875654824, 0.854675729194024],
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmXuUlXW5x99h9lyZmb1n7z3sueyZvbdoiILVOiGiYEClppAmaJmuWgiK
LkgxNbFQSBdmSWfVuVVLSyNzUPGYpzqXFNAFghADDK5AD3fHUEltALlo2jnf
D9+H1R/vfn/79/7e5/f8nsv3ubyVa2++/KYhSZLs0E+17uVMklR0/TaVJP/Z
kiT/pet2Pdue9n/Gr3UkyaFskuT0wqdqk2SO7hM1//VCkrzQliS7a5Lk4XyS
7C/7/cs7k2R5T5L01SfJ2q4k+VlrkrxalSQ36/8erT2qd1c3Jskx3UU2yZeS
ZEuH/9eJ9uymJNFryRa911dMkozm7xT9+aL5OT2bXkmSSXp2VGs26P4LzTdq
zZnad5SuZ7TPY5rr1TVX67/QLd610VjxnhX9r9d734fE64O6fibePj80SSbr
+SI9f1zv9Wu/tHjdnPa617Tm7lj/c41H6/l/6LzfaTG9V5pM80BDktwoXqq0
ZrbubzT47O+JzhFdtw3xnf9jdQ3qWtmcJAOi8dkmn2+m6GQlk1zJdPu0z691
7Uwsv1u09rjoLBHPbzV4L3SAfJHtwhbzNUE6fU7vrdC1RP/bM+b/7rgWxhpo
HAg6e0VnV63pX6UDHSrZBi4XjS/q6tI739PcLuQr3h7Qs1d1rdW7vdLpH/V8
ncYXSC/LtKapynPbYn6V5FYv/W1vsbz7Wyzz97Tvu/rfWe2122M9NrdDdJu0
x07Jaqvmz0h5T+iyL3IYq/GpWruuLknObvFZ1mo8RuON1X6PvZ6QDKcOS5Ip
w2zLC0MWyPZSne/LOdm26L2vZ7eKpzF6d7Ns6kaduUMyqtGaOl2bm73f+1pz
UWIedwafyBD56ZZc0GS9Quc87TNO13Ois1Z+s002P6BFP2yOOfH8eqNt9gmd
cY94eF3Ph+jdj2pMB9t4TXr5b/lFtebvkM7313j8V81tEa8l7b1KaxoqltOX
xe+Pda5/0ZqyZPhd0TkmOqulnyNl//9KxTpDX3fp/926Rmv8ruhsLPpsi5r8
DDpb6z1mzQHR+E2PeQYzfqdrtubzKY9ZA55sS/s/NtPZ4LNM1btTdA3T/OK0
bRR97NM5BnSdq7XrBQaLJf9j2rdBPH5T44f07n1pv1Ns8Lgj4//wBX/w+e/i
fZPeeVsyeiTj93n3V5L5Y7oOi7drtG54o/faV+9nzB+QzP7WbpuaCUaJj0Ht
9XHx+47kclTrTsMvNX5H4xW6lg61j2/X2cc0GgNOl80flL7P1dwi0e+oNqZ9
WG0bRRdZ0Ti93bjar/mC9hml+5fQr9aMThlXZjTbtifonEv07OxqY/DzgcMZ
8XxAdvUpzaek01Pz9hfw8v6SMRMZIytw7wQWVttuoX2trgs1P0rn2iiab4rm
wbTjRKvoDElZHvDAeEasP0vjH9aYT2wWu8QmayT/2qLlu6XRz5h/NvbiTPXa
a7bO39/s/XgOnv9B879oNfYii//TsyvBQM19VPT/49LJMV3DRPshrS/pvMPF
wy5h/k5dz0oft6OvnP1vUGdarLPMS4xXP4IPjf+p2WMw52PoVGuuq7EOwXD0
+AnN79H8DZqvLRuXwY1ByXVOxWc6rPHT8Kd3ntbabKsxJ6vxQNgDey0TLwPa
a7743Vb2/3U6x0td3u8+dEpM1Vm+pHNc2e0YBeZCBz1yBy/OF50xwooFEZcf
DKxHvzNlC4tF63zRuU/7NIjmlXq+JG0+4GFF2jbfm9gnesP+eyXjx4uO1QW9
+5R0tFfPJmn989LvQu33U80fFG83iOYyjXt13aHxw8KDbxYcxznzQPD5z83e
FzmP0f1sXXtFZ1mjcRAMLMu+unSeqzW/SfL8TMXPsK1LG4wt2NHeWmPjPO15
S9lYf4XON11nnqG1YzU3IDnM0dp7Cs4n8LutksfLugZrHG+4M/8NnesWXS+L
l7MUH2p13hG1jk3v6T682r6LD4t0MlHnPkN07qlxnDpY4/l+7bO96P/nCcPG
dzpnulk87BPN3aL1BdG/pGS9vSl+q3KOJ3nNT8z57OiTHAqdEpemdjs2QYv3
mJ+u8z4f532yxzjC/GLt9Yp4myB+pkl+V1R8vn6tXd3mGIWfETvwtfWa29Dm
/58e6n3Aw1sLzrFYg3+DR+ihXbQKYQfEJuIlMQ6s3VI03hIbj0iWZ0beOK7O
vIMr/1O0r5PvvRk8vK65PxX9DrI+UmP+/yia29r8/6Zq8wdvNTKuiZ3mgdiC
zxFrPikZTsoZ1ydIZpNjPFI8rNDaueQUene63u2vNw7NDBxL692XdOb7tW+R
vKlonPhQPGSH2S5HS4an6/23JIebNX+pxg9VWWc8X6h3l+m9VXo2ZIh9i7iC
f/1F9vhvbcacldLVoopt/2GtHV50HsY5iF+cZVqH94D+Jj3f12H76xWPuWHe
6yrRuFrXnZLBZJ3pqpzlxP/5accsYhU5CvZ5uNGxgzxhl3jZ1WodNopGXcX5
1witP17rGI2PkafotRNnmlvwufYXjcXg8E7p9jJiVpXtET/EJsGa6bq+1mAZ
c0fO+NW5ddYFuf+erPN/fAn/R5anSK6ndjlfeKnVORN8LNe+T+la1+h8mDt5
byc5QNqy+5hi6ZMZy3y6Dr2gZLwhNh6P+DhXz19s87nGpZ2DE7vRyQ+K1suo
vDEU/JzW6dqmL2L4nMAN+Mbeyf+xsfPDR+vE+9xO6/526WRa2Zh5h3iZ0m07
RtbkseRpyBs9IPOf5Jy3kcNQnxF/icPkpMcjn7xMz2/NGScGNXewybnoyrTz
9PrINwaarHdwjVoETN4A1pFLVxmvHxUvo7S+IH7bdfXWm0ZHi+lfXPJ+jM/S
fWXR9E7IosVyfrHR+QS5xCmSdUOLfX95ymPyGvK+B6JeuEtrFkQMGpHyGVhP
zD+v5Lh/v5i9rez8mFpsU9rxjfx7k65nqpyXc8dHtqKjLueE5D6Plp3/8D65
N/svTzvHpI7LSX6Ty86jFknGY0qWL/kMOEesmZWyLbMebGqMc5EvgA3EMrBu
MOT/tmh+o+yaZEKnsR683qF950W9g/zXx38wA3njy/gtdSo56ruykZ+QG7W4
fr2i28/Keec65Dn9UUeRY/Rozexu/39W+vl90TlXXvvkIl/E//DH8brfIxr3
6tokOi8NjX00/lfNZfPO907rci2BfV4s2pd0277flU805JyXkTOAcdgb9MeH
j5NjPJC2nM6J+pea9wnxtStre9lAnll2jb1VNF/Wdav4fE1r6nPGKnK76qH2
WWL+QNRExKvbCo5ZA3o+v8V6e6vRMZvYTi27J+qx74j2KXnbGrkdfguOkbfN
aXEc4PnDZa/5QOtbc/a1K6THA53GlqGKs6fl7S/gBmcjjhCL9kY8elX8v1I0
L3OCF7CiJPsql+yT6wvOM8DCNTrLjRnXjPQ/6iIHviFyC3xqZdn1PLX87uij
wP8k4dU/liz/sdFbOLPKtch70XOgx3A47RrlUNpjej6MwZXrq1y7bY8ajYuY
SgxiH+TFXsyxhp4AtodNLwqMOhQ0VwdN7GdS2Dk4Qy78+7xt9aK0ayRywqXR
BwF/8QfW7w3coAeDLoiJ22Lfk2eA55HtrufJL7a0OF+H7rzgDd+n9qd3AL/I
A7kgH/wZvwY3wJOn0saUn0u/j/RYD+Ai69kLTKFfQu0wRWsvSTsHILclvycn
Z+0vgz777wweyNOfiPOCP+xF7bBG9xfTtqULdV+btg2jr8GQJz6a6gk/1X2E
fHJ8ylg4K3AJvqHLeJ7kfH3e+T84tzz2op5Y3+OaYqrW1HaZT3SxLvQxLmI/
ONmD7aS9LzqZFHoB84iNYDj5+ycbPE+eQb4BJhQkj2EZ/2dMLkJv4Wvt3pt9
6TtAn/p3Vtbxntjwa60fVzFdcoYL0sYVbBIZgcnIHfmjC2LslJLjLHMXxzyy
ZD19iE/oeV+n62rOypnBz76Ux+hleLOfE6dyktU7wrq10mUuY/1SG9Bbot9E
/QvmrA366IL4RMwiPm0KXdBbYz25PfF2Q2A+eLc5YlkqY52cG3LeEP4yK+hC
80SfLmyP3HNr3v9LafcQ6HEhb3wG/qmT4H129N96Q1/EbM5BHsC5qAmw21Wh
X/IT+MC+iEdL6c+0mh/scWnY5HN6d0bF/7HdR8NHjgkvV3fbBw5K5pdnXeMT
/8kJ4O3pbtc8yORt3Td2OjYNRs+DGh8sGBe4imyQETnYhSXnuDdFzkPcJRbT
PyJ3JG88Q7JZVvAZluiezjm3OifiD7UttjI2cJL5cYGTYHch5MDe58R69q8J
HlLN7mdQU1BnUGMwfkayerrsniL90X8ou14ndpG3Eb+IjegMfdFjaCsZW7FZ
9kD25FPkVeRa5LPktWAm/b+PoidJvY39YXvks7vbTB+82RFYN7zNvSdocq5J
YVfUAcQUcsITeWu1c1d8sSf8ERmQayAHbGNM1On01P9cdo+SnILcn9wJTELu
yLxS8jeDXPS3SkGTfYhnTdEDPydkjlzxW2RLjTgyvjEcjtqbWvDjonlJxTku
MQE7B08406rApVzEHmoQ7I012AbypMdBjso5VoV+mSNXJm6Sv6yIMbHxUMRK
ZAJOgYHIBrw4cRbp/bqKz44un0w556EPuqPbWE9/ry8wH77WRMx6udN5KvGE
OLk65tulrw+KtpUXYz1nIg+HT87xOfq0FWNiV7P33Rg1MT5B3kj+1xE2Q77C
mDoCH6NvSN0Hvv650xiLfsqhI2ripzK2ffoXHWEn2AM1P/Sxi0KMsQHyS3yW
Gv2+gut0chpqO+po5Ad9bL8v5Aj2ni6dvt7h3Ije/8l+DLaATVwceoXHRdH/
y8VenIUx8gdjwRJyb96dErbEuZEFZ99Y5xw3F7kBdKEJBoAF+Dj4SazCrui5
boy+K71demD4yJPSz+6sc6K7Mo4rxIw7I84R78hX8UXkfKfs4Vt5fwd4Tu/u
zzq3owfzQqv7MOzBXtgtdUpnfAt4NuM6FrslNtNHw5boXx6JHiY9ttFl13Tg
OnFqdtRfwwNvr5euJ1ase3rY/dHHZj1riQUfiK9tResbvE91GSvghXyAmvQ3
edcM+AI1Ez6A/Kmtvx319W6+Del6sNk9tnsL1iv9syPhy/300kvuBXZrr6lt
7i3Ru787vkk1RrzAf/uL7qlhj/RfTykZRy7r8rcZ6r/9EbPAOmytFPb2SNnY
AW7QVzu/2zGCs76f9XlP+iqyPai5zcXAxIxjJPGR2mBG9KtXSqeP5/095APR
PCSafdLfAPZccv+Uuha+6PPABz4KPhBvc5FL0JscWXJ9cCy+iZJj8+0FeuTe
9K/B0A8jHx5xEtMavAd1LzXWguhjD5GeqyrOhenV3hX19YIYs544SQy7JmgO
C5unL9VVNp1f0XsvO2ehFh8f9Tj2jU0Rp+l5V/Kun4hzf22OnnSn4/RNgYtD
AhuR62DWsn0ja3/AF07k3FG/kxeTi5Ov0mN4IfoM0CaHhv5FgcNgSDPfGHsc
C4m3R7PmBTrkvSd7BNyhzzeGzTF3wlczPg95Ivkl8RTbpQ/3VsgY+Q4EhlPT
4Kf0sKlvqW3rJYP/7baOWvOun1nH2VcM9fl363lT3uM9Gu/t9jO+w7THdyv2
WRJ7/Sj2vTb6OSsC//d1+7sU32Co+aq7HLOoffEhagdq2ZnxfQU74kzYUk2H
7ZU5vqUhe3CDfgAYjZ0Sk+AFH0T38+J7K9/D7+lw7MUufhm2US0bqKm4xloT
uSCxjPxzdMiKWvmxsp/hQ8iA/PnHmsvk3TsE/3Kxfk3ERP4jH77tUcvgw+R8
+ABz3435U8XX8A5jNvk4toAcvqjDfbXidTtlR6+2Ol9E12dFb4c4k4pYc7Xk
+aeM4+obul9TckwBH4+FrOhxXhp9zuv0fFbJOIW+dwaG8838K1nXqivjuyB5
zveFhbWS/9HofY6MGEnNyBrktkD7Hsj8PQdeGe+SkzAm9/6LbGCw2/GM76Ir
iuavL3CMOI5cCx3miXi7KfyFvKYSvcT5GZ+TM/4t7xqM+ov9ry2ZB775T644
B1na49qYunha1vk9ePj/jx+5Rw==
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3mU1mUVx5/ZF2aGWd5hmHnfmff3CoYilJ7DSCIQUZqxRLF5AM0jiIDs
IgElghRLhed4spN2gkA0ZgARqY6VCKHEpiBbsggMO8bqILIK1ufblz/u+T33
d/f73Oc+90kNHtt7THoIoSIthEy+YypCWFsSwqGsEA5mh9BQDoAnYWqXEcLu
WAgtK0M4mRPCw7khLIlCqIb+IvyTi0L4HfhzqRCi4hD+isLp6NuJjkbo59E3
OBnCEKAxL4T+2BtWGkIWMtsKQ9gBzIb/Y/i/gP8i/E/BO5d/V1m3gr6/OoRc
fLiBL0vR/49ECEfRkQH+H2R6xUP4IVCJ/nnENB7ZTfh3DNphdNyEv6wshHvh
/y//huHn++WOdyzxHETXNuIbzXp9fgjHWe9D5hTrKmS2s74ATynrs8hcBy8u
c24mgM/C3kygE7FdAl8N/VP4c6BfAc7A+yYxzG2CD+RrBb71qyJH2LuJz3Fy
kQDqsDcYHXuQv458JbIX0FdP7Fn4tJX8P1cQQmt8/wB7EfgM6P3RXQN/V2L9
PjbeQX4zNo5D247OC9ALa8gHtOHYz2QdobMFufwIeAZdE4B89vskMj+FNgme
72LrQeAY+oayJ7exF/nIjyZ/m8vNO4kYmhLfQfCbWd6zf2N7Dvq+yPae3oZs
CigjnuHgu4llFDpw7f97tB7ZsfA3ZPvfI/A+CpwiNw+pXvD/FWz8NsMxjoN3
Y7njOwT+c/AZQHG6a+5OZO9Ieu9Ggr8ArXm59+oa/AdL/E9r+XxHc2SAduBd
0LEV+tKEa6EAvB/2J2J/Gvraw7OcXL+N/qnk43ChY/24xHuvmM9B/xD5/chf
hr6V9UfA9Fz7eIm9/kvMuZ9NfuuRrwOmEWtH/i2Btpr8T4U2HdibcE6VS9Xk
VXx5H58WsZ+vA5fI5S7qKUG+xgWf3aPIXMz3GW7Bug8+jWB9Gp7N6N5UY1sn
gWvQG4udnwz05aL/LHs0N9c68sBPJnx22pCDTfi6GWhFrHOwdwXdi9E3Cl+P
w19Gbvajj99hL/9m4E9u5LO+kvN5hvXpyGdZMguRXQDEtdfgi+CviJzbY/Af
JtYGIB3+1fj3C2jrapy7vkXOlc6IzoZyJtu3w3Ou0D7k0Jv+CX0PvLuBa8Tz
Gfo2oWsLZygXw2XQN4D3ynMvKE3ad/WEiHUDOiPVK/+OQE8mvdaeliL7MsQD
6CrBx4EYvZj02Z6Y7rM0BZ4HCnymzmH7j+DD0mxTur8RuVZkYzL8s/g3JFjm
GdaHqh37MvRnEU8m0Jr1TuLJZd0GnzamOcYCdB+Bf2OOc9YF+pcJ10Knppxp
8JkYegz/HgT/Mfg4cj4f+dngr+L7AynThuDT91g3lru2uvFvPryb4+5t+5Dp
B70vcBn609AHsR4IFECbAv4o6x/FrXsWMJZafZeYFrE+Rv4eT/kO0d2hfyPA
m5CDNuDrm/puKYk5t7pjtsD7IXAU/fO0/6z7Ef8T0OeCnwQfUOrc/w18G/FM
Rr6e9W8KfXZVk6pFneHdkWtOtSaeWuw9W+y7Kob9p8DX488e9G8E/yb4CuhL
oVeBtwdfCD4PvDn4CfQtiNn228BD2h/wzsj3gN4dfHKxe9sF9vdn6o3Yvxdf
aoFYzDlQ7GWSgb4K/jr4j5CvjuSvE1DL+RvJHsxkb2aV+yzLRjaxX+V8dsjx
Ge3RjL0Cvy/HPe4GulrA3yHTPScZc4yKTXdCPbEuAZpmeQ9Lka/AXh2248i/
p36S8l7v4rw3r7JO6RLPlirnULlTDzkDbSpQQb21IL7TVc6hcqc7cCjrJ1Ou
Td3Rjfh/d+S7Qnfu3dhfgPxqeO/E/hv4tqzmVq/mXxv0NW3mXrQSvIx1DBiU
YZ+WF7vmVGs7gMNV3jPt1VrwreDnK93b5OMOanVAyrWrmaVnuWcgzT6aeQay
H4OAOLFsyXDvnA5cLXAPfSLpmBWr7sxa8Ovor80wTzp7/RL0zELf3xMizzya
de6jXl7QfQH+Ing9+T2qXgPUgS8GjrP+BB8vpvlfD856L3R0oT98C+hN/hpV
M4Xu+SNYf1bs2UIzUgr9G5H/lHUt+nfh27+IEVfCBGJcW+4ZSbNRAz8vlLsH
6OyrZjQ7DYc+Ptcz1PzIPUO9YiT+74y55lXrp+A/gH+n6LkbqL9V8i/unqBe
oDvrE+gHq03TjNei0j1avVkzp+56zRyaNXTn/xl7d2GjE7qf1zwF3jbm9bQi
zw4NwB8KPUPoLj5PTg7k+06+Evedp7uuWb574Sygf5F7ombhN5SjfMe3nPUy
4HK6/5Vj6/eRZ6kYPmSiLyvuWmrNniyBtx5Ih38d/B3xpVPS/Uk9a59mM+iX
oF1D/0+gTUp679RDtiQ8g2j2kM+LsVUPTCpyT5xAbnqDx8GfzXAu2lY618pJ
Z3LZE3qzIp+Bk8TeJvKsrJn0ddZ/AsYX+Q6Zi701pa41zaCHWL+Z8Kx7BR8+
B9+R8CyqGS4N+wcS9r0leLvIM4Du/i/J+V7261f0m8vZnon/Du+qhGdRzew9
Ip8B1b58VC3vj/tsqqa3J92z1atfArZzPq/GvVe6w5ZRC32qfPbVE79DLc2B
/m6ez8wI6DvIx+Es1/QofBld4VrWnvRH9uFqz8rT8Scb2Zy4e+tdaX6L6A1z
/tb82hddfSr9tnicfL1X4jeT3kqyobunb8w07bF6rXqWepV6blvi/XrkXqkZ
pzuxdUt679TDTxT7jtfdrp6i2e4KOX6tiWc89fKcUutTTz/Aui7h2tKMo1lf
bx69dXQ+9Da4UeJerTfC59WuWdWq3hDl2I4lvbdD4V+Abwsj30Wt9B5gPRG4
v8g57Y78/VW+azo38VvmLP61L/Kb5qvmfuPobaOZY3zkHqneWMu/c2Xucept
0vk06xXVrh3pqIs802qWVY2rN2YRX4t898hs1kew3yHd/9Yh+1hz75VypN4y
LrIt9Zi8mGdEzYY606/pfo98NlXzBTV+w+rtqjdRH/FGvns143elnr4N5Gb6
jJ1I+A2kt4/O7OVS17xqvY6cHSTWBSWerdVzcqBlax4MfuNqNtuRdC1rRkvF
3DPVK9WjG4nnPDCsyDPpK5FnAN39+pcR9x2g3i+fZqDv+Urvle7oTdA3xD2L
qae/hfzKyL1QPrxMftbU+C2hN5Rmy6/Q/+t0z5hnqnwmdBbuQb4lum6XvnTz
aPZJJx/bizwD5bHOT3mWFo9qs1u1a0M12lPv9VLvl+6kd7CVkfKsqplJs85b
xbalmaeCWPL0hs/zDP0q9hbV+C2gN2kDur4WuVbVg7sST5dq949HMtxb10Se
/XT+pEszvWZ56ZQv9zQzr3x6En8+iDw7auZNS/lO1F2oGI+Ri+PAmFv8Jdib
UuO3sO7wAeA/qPZbUDxzsP/LpGtBM7Bme82Amv0041/DfvvIb1G96f8Hup6P
Bg==
             "],
             VertexColors->None]}]}, 
         {RGBColor[
          0.9127667803918393, 0.8970642822511418, 0.8467703049260273], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmgmYVNWVx1/R3dDddFV1FV2N3V1NVWvCJtHIxJF9y0hYRAHDCGomCMJ8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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtmHl0lNUZxm9mEsg2Q2aYJEwyyTeDlYZAQbS2bkjASpXgAkIB9diCihUF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             "],
             VertexColors->None]}]}, 
         {RGBColor[0.941176, 0.906538, 0.834043], EdgeForm[None], 
          GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmglwldUVx78vCyEheS8vJIGXvPBeWCpoXYvErSRiQVBR60JwAVI6U62i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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwllntolXUYxx/PNueZO9ve4zY9O2c775nWnOYlyei6maWbZmYROYXSLCIq
iko0iO6hmZpOCwMFhaILs9QSJewCOTUvZdhVZ7rlnGne8q7T1ufh+8cDv+d9
7vc3M+3pe5+KmNkHQDZQFZqVps325pudiJm1psy2lpkVR83+zTPbBX4mbvYX
7zbAkmb1gVkd9NHAlgqzSeDzkJ8PbAbPwDM8W99+TZh9gY7tyHZheDG2FmLz
wQKzGDy9wf8swp7JhzLwY8j/B+9+8LPY7su3Dt4X8e8ItCeKRIuicxG6FgNT
0bcUHWuw90qJWT90T+ph9jGyHwFt4B/CXwn992LR2nPNhpabDQFeQjaEPgD6
N9BHQj8CvSFQjB5bUw50bLXj02l8OQX8TGxvoL82phz5+3WgJqZvDfDWks9v
0TcC+sNxN2S2DdoOoCe2ZoM34P9koAbe3cR4MSqZx4n1b/AeyO7Dh8O8DwHd
I/rWzHsCOq/j3Y5MI/wjsV/Je45JVxMxvB+Tzp3gPwLLwE+Az4Z3FjAW27fz
zTJmX+LTb9nS+TnxbUE+lqcaHkR2OjwdhWaX8eEHZHcAb8Ffjr+NvBcBd3ts
fHuI91RgZ5ZZBd8G856JjzWY6oW+74n3j0DvXvjTgO77M/JtJjbugDYxo3c9
PFN596FetehbzrcacvcI39bzziWGFdR+eYl64Rn8Gw39Ej73QbYI+vPw3sy3
Eeg/h8xutw9kosqZx7YRmzkRxei6Z2bE6zaO4v/JQL2yjniaeXeg/0pEMpvB
jyfV67nARd4XyGGkJ3MQU2++CvyUrR5tQ18rMCqqGp2Dfy86UsgOI8YqatGf
njxML76I/99Ba8bfQviz8CENfwg86vWOaHY9p55Ln2Gv7QpgSkw1vi2UDded
X6DaboK/KKoaX6GXboInAi0bWIPv/cFPIr8NemtcO8N3hfd4Ne9fEuoF57kB
/LL3OLJd4DeC78JGPKJvA8FrwE+ZZqiT9yDqWU1+Muj/FPpnwAvwtsBza+iN
otp2B04EqoHnPoVMZ1I59tx6jSPwdwsl6zMWwf/jyAzE/jhiPhaoh7x3/FsL
+BTwJYXK+VLyfSDUrK6mPsuxdY/LxJTDwfh6Tbl0uc+n49qRvhsvQD/Pe0So
3HrNZ+Hb7KRmYy76JhLLeOi9odfB/zKzcD4h2n34fCe2isH35GqG3Lf96KzK
ko+d1OoyMJJ6Pek9ktKM+GyMAerRnShXbjp6StdY6ONi0jmD+GawzybkaGa/
Bp8OXpCjHem7d1Og3vUd3Ip/14aKrdI0e3WhZsln0Gs7LNRu9hqvwp/VKfWm
7/SiUrMAmJylHeu765+kdrfvsM64esZ7ZajXG9lpcfWy+xjifwWKKrO1k3x3
TUlrl/gO2+e5T+lWeY6uhjeoUOx+g0bDO6RUuzAK/3p41wGotR156vUB8BzN
V89PJJ65Kd0Kn6kucj0fen1MM3rGeyPQrtxYoFn0GfHZ8Jksgd6E/834vwH/
ByLbXqTY3Ib3/iZiOBjVDHgvjErLN++Jq9LKuef6UL5uX7+03n4DfVbXpuS7
z2wVgUxG52NR7Yjn4J0O9AXfQz4KyMe8UL2cB3TFNVM+S76DfRfeEmrWfSc2
8i5EZlqBbuoq9K8GPslRDXzXJdK6Rb7zagPlxHPhPJ6reWn1tufMZyeT0K31
Gboe38fw7S78H5+nW/taiW6r39w56HoTqDPdxJWhdqDvPt8JG7G/NNCtOwBs
gLYkoVy34t/boW6Q354H4L8Q6sb6bd0KtIPPD0Vb6zrJ9aVizZLbHO67Oqnb
4T6OCjRjPls+cwfi+gfw2+89tBffW0o0u9XkexD16ZbUv9HZqHJZCmNLrnJa
w/w/m1ZtfCaWBYrJY/Ge2I78tjLp8pv5HrJfFSs2n9HzPmuhYvEb0lImm27L
a/4uut9J69/KbXpvVCf0L+M9siBUDjx2/wfzfzmvidfC/+kWIrsgrV7xnbGS
eEtD/at4T/ut9J3tu9pv5v/+U3M2
             "],
             VertexColors->None]}]}}, {{}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           TooltipBox[LineBox[CompressedData["
1:eJwl1GtMFlQcx/EDTnvRllCAIipQKYp5a4ulBqgTIt2cuRboXJq66WzTzQja
eoEmiS68ILZauPRdbt7Ny1qXFype0mqznBe8YKCmFLmFIpdmnzNffPmd8z+/
/+UcnufJXrhi9vKEEEKJP8eSQzgxKIQDg0MYkRXCSPyeHsLAzBBuZYRQnh3C
cZ5W6xP0Nk13tiwphD+t7+IO2nAP74kPcv63dSP/X/QkbacZ4hfU/kavgzgl
3qj3IevDOIojOJ0aQsuzIeSapZWOogXRq8Y2epw2xLnpLOfBjN+miKm/DVl6
DLXfR/ejSr1C/g51JtManrXYge14Uyyb73lclHeArpKzGkfsO+V14RGetq+h
67EOXyFxQAgfuXe3s7fM0+NOvShS92X3mUZLzViGt/GruX/BfL3fwbtYgLNy
fkKb/k89F8JV/ZtQpXanWKX6n8hbixosFftZ3jlU21c4rxiqP83Xd6FZMsWz
UGqGT8V28n6Nfub4ga+c/wN8odb39jvpd/Rz+pneqe63S50Uupum0UXxzeO9
kYAz2IO90YMc+SPk52CuviNpLn4U/1jNMrFTes6h+eY4aZ1txuHOk60z3b1E
nfQh3sKs/8TPEb1P+7jDOPGxSErjRyH/jPgZoSv53kc5tni3PPWbvOloJOgR
UKJOnjpv0HnOX7FOjG/kXjvMtx19/E/3yt+ibj3q0N98ae6RGt9G/a2ox2Zn
m+JnFRuwMfpjf8zQI4V/Ot1svwnPqFPL8yX9gzbQFlqmf6E5C3DZPLvk3ORv
xmOxDbQWS8xc6q4z5RTZF2Osd6gz71TrKzEfl9CjxmhvNQYvRZWX6L7t4vfi
95QuUG+gs9fVK0a33j3owiPeTkwWn4KHYlfl3JXbROfLveG8r7rX6U0044ze
p+MMPA/kjOGvNN9tPHbPfLWCnCr5BdaLaSU+xCT71/Af30SaK3cUWuX2iL0q
1ksn0PPxO0Yf2L/A8yKGxTfjHS/+r/h583TQ32i1szVYpE+zWA5Pmv0AXJOz
j2c/WjKe/Db+D6urzU8=
             "]],
            RowBox[{"-", "9.5`"}]],
           Annotation[#, -9.5, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4833, 5999, 5573, 4204, 6614, 5574, 4552, 6001, 4553, 
             7354, 7355, 7352, 4834, 7353, 5580, 8236, 4838, 7356, 4839, 7357,
              8069, 7113, 4509, 7110, 7112, 7111, 7201, 8449, 8450, 7877, 
             8554, 5151, 8348, 5152, 7684, 8323, 8322, 5299, 8437, 7875, 8324,
              5129, 8326, 8325, 8518, 8517, 8097, 8349, 5153, 8555, 8176, 
             4690, 8177, 4691, 7203, 4511, 8157, 8158, 8029, 8028, 4327, 4216,
              8523, 7983, 8352, 8351, 5154, 6529, 6530, 5329, 7849, 7848, 
             4422, 8538, 8539, 8537, 7121, 7120, 8035, 8034, 4330, 4220, 8501,
              6860, 6636, 4872, 7394, 4574, 5598, 6648, 6634, 4880, 7418, 
             5400, 6029, 4573, 5597, 4879, 6028, 8042, 8307, 5101, 8309, 8308,
              7048, 8515, 8142, 7309, 8576, 4798, 7308, 8220, 8219, 5526, 
             4335, 8041, 6863, 6992, 6991, 4463, 6483, 6484, 5205, 6592, 5204,
              6485, 8381, 8380, 8382, 7997, 7996, 7995, 4513, 8395, 8396, 
             8394, 5222, 4464, 8542, 6993, 8541, 6994, 6866, 6867, 4341, 4229,
              5616, 8519, 5617, 5160, 5411, 4348, 6680, 6679, 5650, 4604, 
             6090, 5418, 7488, 4930, 7485, 4929, 5681, 4616, 7501, 4952, 8271,
              7500, 8055, 8054, 4361, 4256, 8503, 6880, 6720, 6917, 6916, 
             4428, 8354, 8355, 8353, 8357, 8356, 5156, 8452, 8453, 8451, 5306,
              4370, 4271, 8504, 6890, 6741, 4527, 8179, 4698, 8178, 4697, 
             8422, 5279, 8421, 5278, 7661, 5139, 6920, 6921, 6423, 5136, 8343,
              5138, 8341, 5137, 8574, 8342, 8545, 4284, 6767, 4269, 6740, 
             7131, 7130, 8164, 8163, 8162, 6915, 8078, 8513, 8514, 8273, 8274,
              4979, 8272, 8010, 8011, 5710, 6717, 6716, 6715, 7499, 7498, 
             4615, 6136, 4614, 5678, 6089, 6088, 5647, 6700, 6699}],
            RowBox[{"-", "11.4`"}]],
           Annotation[#, -11.4, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4870, 6026, 5592, 6021, 6022, 5590, 4565, 6024, 4564, 
             7384, 7385, 6628, 7976, 7977, 4323, 8247, 4857, 7382, 4858, 7383,
              4561, 7377, 4850, 6626, 7969, 7970, 4319, 8240, 4848, 7375, 
             4849, 7376, 4558, 7342, 4831, 6608, 7956, 7957, 4316, 8230, 4830,
              7339, 7341, 7340, 4544, 5998, 4545, 5571, 6609, 4201, 5572, 
             6611, 6610}],
            RowBox[{"-", "11.4`"}]],
           Annotation[#, -11.4, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4200, 5570, 4199, 5568, 5569, 5997, 5996, 4543, 7336, 
             7337, 7335, 4828, 8229, 4315, 7955, 7954, 6607, 4829, 7338, 4557,
              7372, 4846, 7371, 4845, 8239, 4318, 7968, 7967, 6625, 7374, 
             7373, 4560, 6018, 6019, 5587, 5586, 6016, 5589, 6023}],
            RowBox[{"-", "13.3`"}]],
           Annotation[#, -13.3, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4255, 5680, 6135, 5708, 5709, 6185, 6184, 4626, 7851, 
             7852, 6738, 6739, 4267, 6182, 4283, 8012, 6764, 6919, 6918, 4429,
              6239, 7619, 5111, 7618, 5112, 8359, 8360, 8358, 5157, 7691, 
             8553, 5140, 8344, 7662, 7664, 7663, 5113, 6435, 6436, 5334, 5453,
              4381, 8361, 7692, 7693, 7665, 8345, 5141, 7666, 5114, 6405, 
             5288, 7869, 5289, 7870, 5015, 8280, 5016, 7527, 4638, 7510, 7511,
              6742, 6891, 8505, 4272, 4371, 8059, 8060, 8276, 8275, 4981, 
             7509, 4627, 7502, 4953, 6722, 8007, 6884, 6883, 6881, 7174, 7173,
              4243, 4354, 8268, 8269, 7486, 4928, 5652, 5651, 6055, 6056, 
             5162, 6951, 7880, 7881, 5312, 5510, 5509, 6499, 5164, 8520, 5330,
              6952, 6445, 5163, 8370, 5165, 8367, 8369, 8368, 8397, 8198, 
             8199, 8197, 4711, 8200, 4482, 8118, 4781, 8215, 4514, 8406, 5240,
              8143, 8144, 8145, 6444, 8456, 5311, 8455, 8458, 8457, 4799, 
             6949, 6948, 6946, 4708, 5374, 4230, 7800, 5241, 7799, 5242, 7801,
              4881, 6650, 7989, 4224, 4336, 7417, 7159, 7160, 6864, 7419, 
             8512, 8550, 8549, 4882, 8258, 4883, 7421, 4575, 7398, 4873, 6638,
              6861, 7984, 4221, 4331, 8036, 8037, 7395, 7397, 7396, 4569, 
             7388, 4865, 6632, 7599, 8306, 5100, 7598, 5099, 6398, 5098, 7649,
              5132, 7646, 7648, 7647, 4212, 4325, 8027, 8026, 8305, 8304, 
             5097, 7644, 7645, 7642, 5128, 7643, 7859, 8426, 8427, 7847, 7846,
              4855, 8244, 4854, 8243, 7901, 8073, 8072, 4417, 8532, 8533, 
             8531, 7361, 7358, 8238, 8237, 6006, 7839, 7840, 7838, 7842, 7841,
              4554, 6003, 6004, 5578, 5577, 4205, 5575, 6000}],
            RowBox[{"-", "13.3`"}]],
           Annotation[#, -13.3, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4198, 5567, 4197, 6606, 5566, 4542, 5995, 4541, 7331, 
             7332, 7330, 8025, 8024, 4314, 4196, 8500, 6851, 6603, 7334, 7333,
              4555, 6014, 4556, 5582, 6012, 6011, 5585, 6017}],
            RowBox[{"-", "15.200000000000001`"}]],
           Annotation[#, -15.200000000000001`, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4268, 5711, 8416, 8417, 7850, 7854, 7853, 4980, 7522, 
             7523, 7519, 7521, 7520, 8340, 7659, 7660, 7524, 8279, 5014, 7525,
              4651, 5767, 8015, 6790, 7560, 7559, 4653, 6280, 5466, 5467, 
             5041, 7561, 7563, 7562, 4654, 6406, 4655, 5770, 6791, 6792, 5771,
              7531, 5019, 7526, 5020, 5454, 4382, 8281, 5017, 7528, 5018, 
             7530, 4639, 5737, 4984, 7508, 4985, 7516, 5442, 7514, 4982, 6721,
              6743, 4273, 5712, 7503, 7176, 7177, 6882, 4617, 7175, 4932, 
             7487, 7490, 7489, 5419, 6091, 4605, 5654, 6681, 7603, 5103, 6950,
              5102, 7602, 5161, 7885, 7886, 7882, 7884, 7883, 7895, 4483, 
             8480, 5336, 8479, 5337, 6571, 6570, 5372, 5373, 8045, 8044, 8546,
              4710, 8195, 8196, 7231, 7230, 4709, 6443, 5507, 5508, 4757, 
             7164, 4579, 6947, 4580, 7165, 4515, 7310, 7311, 7049, 7050, 4494,
              7802, 7601, 7450, 7451, 6649, 7998, 7166, 4581, 6865, 7169, 
             8043, 7167, 4231, 5605, 7423, 7424, 7420, 4885, 7425, 5401, 7422,
              4884, 6637, 6651, 5599, 4576, 7399, 4875, 6642, 7985, 7986, 
             4332, 5398, 4874, 6631, 6639, 6640, 5593, 6025, 4868, 5591, 4567,
              6399, 4566, 7386, 7387, 6630, 7981, 5396, 5395, 6020, 4563, 
             7380, 7381, 8246, 8245, 4856, 7857, 7858, 7151, 7911, 7912, 8527,
              5340, 7913, 6015, 5358, 8526, 5339, 8485, 8561, 5357, 8497, 
             7974, 8516, 7975, 6627, 4207, 6619, 6618, 7360, 7359, 4837, 7843,
              7844, 6615, 7964, 4206, 5579, 6002}],
            RowBox[{"-", "15.200000000000001`"}]],
           Annotation[#, -15.200000000000001`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4840, 6007, 6005, 7966, 7965, 6617, 5272, 7845, 4842, 
             7368, 4844, 6623, 6624, 5583, 8525, 4559, 6417, 5130, 5394, 4321,
              4210, 5588, 4213, 7980, 6629, 4862, 7653, 7654, 7650, 7652, 
             7651, 5524, 8330, 8331, 7655, 7657, 7656, 4867, 7114, 4866, 7389,
              4328, 4217, 6633, 5594, 4570, 6027, 5399, 7406, 7407, 7403, 
             7405, 7404, 5602, 4226, 6654, 6641, 7430, 7429, 5402, 6031, 6032,
              5601, 5600, 6030, 4337, 4225, 7990, 6653, 4886, 7426, 4583, 
             5606, 7999, 7168, 4896, 7452, 5405, 6038, 4582, 5604, 6035, 6034,
              8047, 8046, 8310, 8312, 8311, 7052, 7051, 4495, 5971, 5972, 
             4800, 7312, 7314, 7313, 7316, 4234, 8468, 8469, 8467, 8471, 8470,
              5223, 8400, 8401, 8398, 5224, 8399, 4465, 8384, 5206, 8383, 
             5207, 6487, 6486, 4467, 6488, 6489, 5209, 7755, 5208, 7756, 4466,
              5866, 5867, 5319, 5865, 5864, 7703, 7698, 4244, 6702, 5653, 
             4931, 5420, 7700, 4355, 4245, 6703, 5682, 4618, 6138, 5430, 7504,
              4957, 7178, 4956, 5713, 4628, 7515, 4983, 7512, 8278, 8277, 
             4372, 4274, 5738, 5739, 6241, 6240, 5455, 7537, 7538, 7529, 7536,
              7535, 5773, 4297, 6796, 6795, 7568, 7567, 5468, 6283, 4656, 
             5772, 6282, 6281, 4393, 8017, 8016, 6794, 5043, 7679, 7680, 7676,
              7678, 7677, 4306, 8346, 8347, 7673, 7675, 7674, 5042, 7564, 
             8289, 8288, 5769, 6279, 4392, 5465, 5145, 6427, 4652, 5768, 5038,
              6787, 5039, 7552, 5012, 7856, 5277, 7555, 5276, 7855, 6183, 
             6766, 6765}],
            RowBox[{"-", "17.1`"}]],
           Annotation[#, -17.1, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4847, 6013, 5581, 5992, 4826, 5994, 4827, 5993, 5096, 
             5991, 8228, 4825, 7682, 6601, 6602, 5565, 6605, 6604}],
            RowBox[{"-", "17.1`"}]],
           Annotation[#, -17.1, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4208, 6620, 6616, 7363, 7362, 4841, 7366, 7367, 6622, 
             7365, 7364, 7640, 5521, 8438, 5300, 7876, 5301, 6419, 8506, 6418,
              4418, 6905, 5523, 4510, 5584, 5368, 7204, 7205, 6906, 7215, 
             8507, 4419, 8074, 7216, 7217, 7153, 7155, 7154, 5369, 7206, 7207,
              6909, 7219, 8508, 4423, 7220, 7156, 7116, 7117, 7115, 4512, 
             7658, 5133, 6910, 8075, 7402, 4877, 7400, 4876, 8255, 4333, 4222,
              7987, 6644, 7409, 7408, 4577, 7431, 7432, 7427, 4887, 8259, 
             4338, 7992, 7991, 6656, 4888, 7433, 4585, 5608, 6666, 6652, 4898,
              7453, 5406, 6042, 4584, 5607, 6037, 6036, 4342, 6665, 6664, 
             7811, 4897, 7807, 7808, 7804, 7806, 7805, 4235, 7232, 4712, 6868,
              4713, 7315, 4802, 8221, 4801, 8462, 8463, 8459, 8461, 8460, 
             6448, 7055, 7054, 7053, 5243, 7803, 4516, 7287, 4782, 7012, 8119,
              7238, 4715, 7235, 7237, 7236, 8216, 7288, 7289, 7239, 8201, 
             4716, 7240, 4681, 5868, 6958, 6449, 5166, 7699, 6953, 6954, 6092,
              6093, 5168, 7701, 5167, 7702, 6094, 5683, 5685, 4955, 5688, 
             4954, 6137, 4362, 4257, 6723, 5714, 4629, 6187, 5443, 7517, 4989,
              7513, 4988, 5740, 4640, 7534, 5022, 7532, 5021, 8282, 4383, 
             8014, 8013, 6769, 7540, 7539, 4657, 7569, 7570, 7565, 8291, 8290,
              4394, 6798, 6797, 7937, 5044, 6284, 8090, 6933, 5148, 7681, 
             6825, 6793, 7138, 7137, 7198, 7223, 4432, 6932, 6931, 7212, 7211,
              4540, 7196, 7197, 7193, 7195, 7194, 8187, 4431, 8089, 8088, 
             8181, 8180, 4538, 5818, 4539, 5563, 6928, 4430, 6428, 6429, 5147,
              7671, 5146, 7672, 4304, 7667, 4295, 6789, 6788, 7554, 7553, 
             5013, 7557, 7558, 7556, 8287, 8286}],
            RowBox[{"-", "19"}]],
           Annotation[#, -19, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{5271, 6527, 5294, 5296, 8524, 5295, 6415, 5123, 6414, 
             5293, 7683, 5149, 6936, 6937, 6938, 6430, 6935, 6934}],
            RowBox[{"-", "19"}]],
           Annotation[#, -19, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4214, 5889, 4211, 4322, 6855, 6854, 7152, 8168, 8167, 
             6939, 8094, 8093, 8184, 8183, 7214, 6904, 4689, 7202, 5367, 6416,
              4506, 5522, 6903, 6010, 4843, 7369, 7370, 6621, 4209, 6008, 
             6009}],
            RowBox[{"-", "20.900000000000002`"}]],
           Annotation[#, -20.900000000000002`, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4296, 6238, 6276, 6277, 6425, 6424, 5040, 6278, 6922, 
             5561, 4537, 6349, 4536, 7210, 4699, 6929, 6930, 8185, 8186, 8100,
              8101, 8511, 6942, 8174, 8175, 8173, 6899, 8061, 4403, 5562, 
             5892, 5893}],
            RowBox[{"-", "20.900000000000002`"}]],
           Annotation[#, -20.900000000000002`, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4434, 8098, 6940, 4702, 5397, 4329, 4218, 7209, 4693, 
             7208, 4694, 7689, 7690, 7686, 7688, 7687, 7600, 7411, 7412, 7401,
              4878, 7410, 5603, 6658, 6657, 6643, 4889, 7436, 5403, 7440, 
             4891, 7428, 4890, 7439, 5611, 4232, 6669, 6655, 7455, 7454, 5407,
              6048, 4586, 5609, 6041, 6040, 4343, 6668, 6667, 5625, 4899, 
             5622, 5621, 4236, 6682, 5244, 7058, 7060, 7061, 6039, 4496, 8146,
              7057, 7810, 7809, 5375, 5896, 4518, 5529, 5619, 5618, 5953, 
             4443, 5511, 6447, 6446, 4714, 5897, 6957, 6956, 7234, 7233, 7604,
              7123, 7124, 7122, 4517, 7775, 5225, 7013, 7014, 7015, 6451, 
             6452, 5313, 6955, 8105, 8106, 6450, 6400, 6401, 6095, 4934, 6453,
              4933, 5684, 4619, 6139, 4621, 5689, 6143, 6142, 5715, 5716, 
             4987, 5721, 4986, 6186, 4373, 4275, 6744, 5742, 4641, 6242, 5456,
              7547, 7548, 7533, 7546, 7545, 5774, 6800, 6799, 6768, 5045, 
             7573, 5469, 7577, 5047, 7566, 5046, 7578, 7576, 5354, 7952, 5366,
              7953, 5365, 6598, 6590, 6591, 5353, 7938, 5352, 7939, 6350, 
             8362, 8363, 7694, 7696, 7695, 4701, 7213, 4700, 8182, 5564, 4404,
              6900, 5072, 7587, 8301, 8300}],
            RowBox[{"-", "20.900000000000002`"}]],
           Annotation[#, -20.900000000000002`, "Tooltip"]& ], 
          TagBox[
           TooltipBox[LineBox[{4705, 5895, 5894, 6600, 6599}],
            RowBox[{"-", "20.900000000000002`"}]],
           Annotation[#, -20.900000000000002`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4305, 8023, 6824, 8423, 8425, 8424, 7586, 8294, 7670, 
             5143, 7668, 5142, 7669, 8087, 8086, 5303, 8442, 5144, 6925, 6926,
              6927, 6426, 6924, 6923}],
            RowBox[{"-", "22.8`"}]],
           Annotation[#, -22.8, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{5117, 7629, 5118, 7872, 5292, 6901, 8063, 8062, 8560, 
             5290, 8432, 5291, 7871, 8431, 7942, 5356, 7940, 5355, 7941, 4668,
              7199, 7200, 6802, 8018, 4298, 4395, 8292, 8293, 7571, 7575, 
             7574, 4658, 6243, 5024, 5777, 6772, 6773, 4385, 8284, 5457, 5023,
              5741, 6770, 6771, 5743, 6192, 6193, 5723, 6195, 4631, 6188, 
             4630, 6144, 4960, 5720, 6726, 5433, 5431, 6140, 4620, 5687, 6704,
              4247, 6877, 6876, 7609, 8315, 8314, 7610, 5320, 8466, 5321, 
             8113, 8115, 8114, 8558, 5210, 8385, 5211, 7757, 8159, 4519, 6560,
              6561, 5376, 4246, 7269, 7263, 7264, 7262, 4758, 5899, 5898, 
             4468, 6995, 5620, 4593, 7776, 6869, 6870, 5527, 8222, 5528, 4803,
              7056, 7062, 7063, 5245, 7605, 7608, 5104, 7059, 7607, 7606, 
             4911, 5627, 4912, 6058, 4594, 6043, 4595, 6061, 5624, 6684, 4237,
              5630, 6047, 4902, 5610, 4903, 5408, 8000, 4345, 6045, 4900, 
             5613, 4901, 6046, 4587, 7441, 7442, 6660, 7993, 4227, 4339, 8260,
              8261, 7434, 7438, 7437, 4578, 7413, 7414, 6646, 6862, 7988, 
             8039, 8040, 4334, 8038, 7685, 5155, 6941, 8099}],
            RowBox[{"-", "22.8`"}]],
           Annotation[#, -22.8, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{5273, 6528, 5304, 8446, 8447, 8443, 8445, 8444, 6431, 
             5124, 8433, 5297, 8064, 8066, 8065, 8552, 5126, 8321, 5127, 7641,
              5125, 7874, 5298, 7873, 8435, 8434}],
            RowBox[{"-", "22.8`"}]],
           Annotation[#, -22.8, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4435, 8102, 8191, 8190, 4703, 7224, 7226, 7225, 4438, 
             6438, 4437}],
            RowBox[{"-", "24.700000000000003`"}]],
           Annotation[#, -24.700000000000003`, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{5074, 6549, 5075, 6351, 5483, 7590, 5073, 6801, 6826, 
             6827, 5819, 7579, 5050, 7572, 5051, 7580, 5470, 6286, 6287, 5776,
              5775, 4285, 4384, 8283, 8285, 7541, 7544, 7543, 4642, 5744, 
             4992, 5722, 4993, 6194, 5444, 6189, 6190, 5719, 6746, 5717, 4258,
              4363, 7707, 5432, 4958, 5686, 6724, 6725, 7705, 7704, 7711, 
             5869, 8465, 5870, 6559, 6558, 4682, 5871, 8116, 6998, 7268, 7267,
              7630, 7265, 7266, 6996, 6997, 8120, 8559, 5226, 8402, 8121, 
             8122, 6500, 6501, 5324, 7896, 5323, 8472, 4248, 8224, 4806, 8223,
              8226, 8225, 4804, 7065, 8147, 8148, 5973, 4497, 7066, 7067, 
             6871, 8313, 8048, 8049, 6057, 5626, 6063, 5629, 6066, 6065, 5412,
              6062, 4913, 5623, 6683, 5631, 4596, 7456, 7457, 6671, 8001, 
             4233, 4344, 6044, 5612, 5614, 6050, 6049, 5409, 7460, 4905, 6659,
              6672, 6673, 5615, 7444, 4893, 7435, 4894, 7445, 5404, 7443, 
             4892, 6645, 6661}],
            RowBox[{"-", "24.700000000000003`"}]],
           Annotation[#, -24.700000000000003`, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{6550, 6437, 4436, 7227, 7228, 8189, 8188, 4704, 8298, 
             8299, 8295, 8297, 8296}],
            RowBox[{"-", "24.700000000000003`"}]],
           Annotation[#, -24.700000000000003`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4307, 4405, 8302, 8303, 7588, 7592, 7591, 4669, 7581, 
             7582, 6804, 8019, 8020, 4396, 6285, 5048, 5783, 5049, 5779, 5778,
              7549, 5027, 7542, 7551, 7550, 5458, 6247, 4643, 5746, 6748, 
             4276, 4374, 5445, 4990, 5718, 6745, 6747, 5724, 6145, 7709, 5173,
              7708, 5174, 7710, 4959, 6141, 6959, 7706, 5170, 6455, 6964, 
             6965, 5872, 5873, 4722, 8205, 7248, 7293, 7292, 8217, 7246, 8565,
              7247, 7245, 4721, 7291, 4784, 7290, 4783, 5530, 4520, 7812, 
             5246, 7069, 7070, 7071, 6454, 7887, 7888, 7317, 4805, 7064, 8149,
              7068, 8203, 8204, 4718, 7242, 4717, 7241, 4250, 6505, 6506, 
             6064, 4914, 5628, 4915, 5413, 7814, 4349, 6059, 6060, 5632, 4597,
              6074, 5414, 7473, 7474, 6670, 6685, 6686, 5634, 5635, 4906, 
             5638, 6674, 6675, 4346, 8265, 4904, 7458, 7462, 7461, 4588, 7446,
              7447, 6663, 7994, 4228, 4340, 6033}],
            RowBox[{"-", "26.6`"}]],
           Annotation[#, -26.6, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{5078, 7589, 5079, 7595, 5484, 7593, 7594, 6803, 6828, 
             4308, 5820, 6295, 6296, 5785, 6298, 4660, 6288, 4659, 6250, 6251,
              5782, 6775, 6776, 4387, 6245, 5459, 5025, 5745, 6774, 4286, 
             5747, 6202, 5729, 5730, 4991, 6191, 4632, 5725, 4961, 6458, 6459,
              6148, 6146, 6402, 6456, 4445, 8107, 6960, 7890, 7889, 5172, 
             7780, 5229, 7779, 5228, 7126, 4521, 7125, 7128, 7127, 7611, 7243,
              7244, 6961, 6962, 6963, 5903, 5904, 5169, 5512, 4444, 5954, 
             5655, 5532, 8202, 5533, 5901, 5900, 5378, 7813, 5247, 7073, 8150,
              6067, 4916, 7815, 5248, 8407, 8408, 8004, 8003, 4251, 6578, 
             6072, 6073, 5633, 4918, 5415, 6687, 4351, 6069, 4917, 5640, 6071,
              6070, 4598, 6051, 4599, 6077, 5637, 6690, 4239, 5642, 7464, 
             7465, 7459, 7467, 7466, 5410, 7463, 4908, 6662, 6676}],
            RowBox[{"-", "28.500000000000004`"}]],
           Annotation[#, -28.500000000000004`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4907, 6052, 4347, 6678, 6677, 5646, 4909, 6054, 4601, 
             7479, 5643, 6689, 5636, 4921, 6078, 5416, 6079, 6080, 5641, 5639,
              6068, 4350, 4238, 6688, 5658, 4919, 6076, 7172, 7915, 5342, 
             8486, 7914, 5360, 7943, 5359, 6595, 6507, 6508, 5331, 7902, 7904,
              7903, 6098, 7816, 4498, 7074, 7072, 7319, 7318, 4522, 5656, 
             4607, 5534, 4606, 7270, 4759, 6999, 7001, 5905, 4720, 6492, 4719,
              5902, 5907, 5906, 4356, 4249, 5377, 5227, 7016, 8123, 8124, 
             6494, 4484, 7018, 7017, 7778, 7777, 7894, 6491, 5171, 6457, 5105,
              6147, 5106, 7612, 6727, 5728, 4633, 6198, 5447, 6203, 6205, 
             5731, 4998, 5748, 6207, 4645, 6249, 5026, 5753, 6246, 6244, 4386,
              4287, 5780, 5781, 6292, 6291, 5471, 6297, 5055, 5784, 5054, 
             5821, 7584, 4671, 6354, 5077, 5822, 5076, 6352, 4406, 6830, 
             6829}],
            RowBox[{"-", "30.400000000000002`"}]],
           Annotation[#, -30.400000000000002`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4240, 6695, 5645, 6084, 4602, 6053, 4600, 7477, 7478, 
             7475, 4920, 8266, 4352, 6692, 6691, 5664, 4922, 5660, 5659, 4252,
              6705, 5657, 4609, 6075, 4608, 7918, 7919, 5343, 7916, 5344, 
             7922, 7923, 7920, 5345, 7921, 4936, 6097, 5341, 7493, 7494, 7495,
              6572, 4937, 7817, 5249, 7075, 7077, 5975, 4808, 6509, 4807, 
             5974, 5531, 4357, 6878, 4935, 7000, 7492, 7491, 7782, 4469, 8386,
              8387, 7760, 5215, 7761, 5213, 6493, 7003, 6495, 5214, 7765, 
             5216, 7762, 7764, 7763, 4471, 5875, 4470, 7002, 7759, 7758, 5212,
              6885, 4364, 4259, 5726, 7712, 5727, 4995, 6199, 5446, 6204, 
             5448, 6749, 5752, 6208, 6206, 4644, 6248, 4646, 5754, 6257, 6256,
              5786, 5787, 5053, 5793, 5052, 6289, 4397, 8022, 8021, 6806, 
             7585, 7583, 4670, 6353, 4672, 6362, 5824, 6360, 6359}],
            RowBox[{"-", "32.300000000000004`"}]],
           Annotation[#, -32.300000000000004`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{5082, 5823, 5083, 6361, 5485, 7596, 7597, 6805, 6831, 
             4309, 5826, 6305, 5791, 6290, 5792, 6294, 6293, 4661, 6258, 5030,
              5790, 6780, 5462, 5460, 6252, 6253, 5751, 6778, 5749, 4277, 
             4375, 6196, 4994, 6464, 4997, 7714, 7715, 7713, 7719, 7718, 8372,
              5179, 5874, 4683, 6490, 4684, 7272, 4763, 7258, 4762, 7635, 
             7636, 7632, 5119, 7631, 7634, 7633, 4760, 7251, 4761, 7271, 8125,
              4485, 6096, 7021, 7020, 7019, 5230, 7781, 5379, 5380, 5251, 
             7818, 5252, 7819, 5250, 7822, 5253, 7820, 8410, 8409, 4499, 5909,
              7496, 7497, 7076, 4938, 7860, 5283, 6879, 8053, 6533, 5282, 
             6531, 5280, 6104, 5281, 6107, 7917, 6106, 5421, 5422, 4939, 5666,
              4940, 6103, 4610, 6081, 6082, 5663, 6693, 5661, 4253, 5669, 
             7480, 7481, 7476, 7484, 7483, 5417, 6085, 4924, 5644, 6694}],
            RowBox[{"-", "34.2`"}]],
           Annotation[#, -34.2, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4923, 6083, 4353, 6697, 6696, 5674, 4925, 5670, 7482, 
             5668, 6707, 6706, 5662, 6113, 6112, 5425, 6120, 4611, 6102, 5665,
              6101, 6100, 4358, 7823, 5423, 4942, 6532, 4941, 6105, 6511, 
             6510, 4260, 5908, 4723, 6099, 4725, 5910, 7078, 7821, 4809, 5976,
              6461, 6460, 5919, 7081, 7080, 7079, 4810, 7321, 4524, 5537, 
             4785, 7294, 4786, 7295, 4729, 7250, 7253, 7252, 8218, 7296, 7297,
              8207, 8206, 4731, 5877, 5876, 6974, 6973, 6462, 8371, 5175, 
             7722, 6969, 6201, 7716, 4996, 6465, 5177, 6197, 5176, 6463, 6209,
              5755, 6779, 6777, 5750, 5028, 5461, 4388, 4288, 6781, 5789, 
             6259, 4662, 6301, 5473, 6307, 6309, 5794, 5059, 6306, 5825, 6832,
              5827, 5081, 5833, 5080, 6355, 4407, 6834, 6833}],
            RowBox[{"-", "36.1`"}]],
           Annotation[#, -36.1, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4254, 5671, 6698, 5673, 6087, 6086, 4612, 6116, 4945, 
             5676, 4944, 5426, 6114, 5424, 6121, 4946, 5667, 6118, 6117, 6581,
              4262, 7944, 7946, 7824, 8411, 5256, 7825, 4943, 6109, 7082, 
             5544, 5254, 6512, 5383, 7249, 4724, 6966, 6968, 4447, 7320, 4446,
              7256, 7257, 7254, 4730, 5914, 5913, 4450, 6972, 4787, 7022, 
             7023, 7024, 5918, 4486, 5920, 8127, 8126, 7025, 4788, 5539, 5538,
              8404, 5232, 7786, 7788, 7787, 5180, 7721, 8374, 8373, 4448, 
             6200, 4449, 8108, 6971, 5178, 7717, 4999, 5756, 6211, 4648, 6255,
              5029, 5762, 5761, 6268, 5796, 4299, 6807, 5788, 5057, 6302, 
             5472, 6308, 5474, 6809, 5830, 5060, 6310, 4673, 6357, 6358, 5832,
              6356, 5831, 6369}],
            RowBox[{"-", "38"}]],
           Annotation[#, -38, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{5087, 5834, 6372, 6370, 5487, 6365, 4674, 6311, 5829, 
             6810, 4300, 4398, 6299, 5056, 5805, 5058, 5798, 6808, 5795, 6267,
              6269, 5764, 6271, 4650, 6254, 4647, 6210, 5000, 5760, 6750, 
             7614, 7615, 6970, 5107, 7613, 5218, 7720, 7771, 7770, 4488, 5545,
              4487, 8128, 7026, 4789, 7298, 5381, 5382, 6886, 5917, 4728, 
             5911, 4726, 5956, 4727, 7273, 7274, 7255, 4766, 7179, 4622, 6967,
              7181, 7180, 4525, 5977, 4526, 6513, 5543, 7083, 4500, 6108, 
             7085, 7084, 7905, 5255, 7947, 5361, 7945, 8009, 6582, 5346, 6119,
              6584, 6583, 4947, 5695, 6708, 6709, 4359, 6110, 6111, 5675, 
             6115, 4613, 6131, 5428, 6125, 6126, 5672, 6711}],
            RowBox[{"-", "39.900000000000006`"}]],
           Annotation[#, -39.900000000000006`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4948, 5429, 6127, 5427, 6132, 4949, 5677, 6129, 6128, 
             5697, 4263, 5692, 6710, 5694, 6123, 6122, 7183, 7182, 7926, 7927,
              5362, 7948, 8499, 8498, 6596, 6514, 4963, 7507, 7925, 7924, 
             5332, 6151, 6150, 7826, 5257, 7087, 7086, 5542, 4811, 8572, 5541,
              8573, 5540, 6729, 6728, 7505, 7506, 5691, 7789, 5690, 4472, 
             5915, 5916, 4765, 5912, 4764, 5955, 4769, 5385, 5384, 5970, 4528,
              7784, 7785, 7783, 5231, 8403, 4474, 6403, 6404, 7769, 7768, 
             6892, 6893, 4279, 6751, 5759, 6212, 4649, 6263, 5463, 6270, 5035,
              5763, 5034, 5797, 4663, 6303, 6304, 5804, 6300, 5803, 6319, 
             5836, 4310, 6835, 5828, 5085, 6366, 5486, 6371, 5488, 6837}],
            RowBox[{"-", "41.800000000000004`"}]],
           Annotation[#, -41.800000000000004`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4311, 4408, 6363, 5084, 5845, 5086, 5837, 6836, 5835, 
             6320, 5063, 5806, 6323, 6321, 5476, 6316, 4664, 5802, 6782, 6783,
              4390, 6261, 5464, 5032, 5758, 7728, 5757, 4278, 4376, 7766, 
             7767, 7737, 5217, 5879, 5878, 4475, 7004, 4771, 7007, 8117, 4478,
              5546, 4476, 5957, 4477, 7006, 7005, 4770, 5519, 4473, 7791, 
             5233, 7790, 5234, 7792, 4962, 6730, 6888, 8008, 8056, 4261, 4365,
              6887, 8529, 8530, 8528, 8571, 8570, 5258, 5979, 7828, 5978, 
             7089, 7088, 6152, 4964, 6155, 6540, 6538, 6539, 6580, 6579, 6149,
              7928, 5347, 7863, 8428, 7861, 8487, 8490, 8489, 7184, 8488, 
             4623, 6168, 5435, 6159, 6160, 5693, 6731, 6732, 5696, 6130, 5698,
              4950, 5702, 6712, 6713, 4360, 6124}],
            RowBox[{"-", "43.7`"}]],
           Annotation[#, -43.7, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4265, 5699, 6714, 5701, 6134, 6133, 4624, 6163, 4969, 
             5705, 4968, 5436, 6161, 5434, 6167, 7185, 6169, 5285, 6164, 5284,
              7862, 6534, 6537, 6536, 8058, 8057, 6889, 7865, 7864, 4966, 
             6153, 4965, 6154, 5922, 4501, 8412, 5260, 7829, 5261, 6516, 6515,
              8152, 8151, 7091, 5259, 7827, 6752, 7132, 8165, 7133, 7028, 
             7027, 4489, 5927, 8319, 8320, 7637, 5120, 7638, 4767, 5958, 6902,
              5959, 4768, 5498, 5497, 5934, 4685, 7740, 7741, 7738, 5188, 
             8521, 7739, 7742, 5181, 8375, 8376, 7731, 7730, 5033, 6467, 5031,
              6260, 4389, 4289, 5799, 6812, 5801, 6317, 6315, 5475, 6322, 
             5478, 6816, 5842, 5064, 6324, 4675, 6367, 6368, 5844, 6364, 5843,
              6380}],
            RowBox[{"-", "45.6`"}]],
           Annotation[#, -45.6, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{5091, 5846, 6383, 6381, 5490, 6376, 4676, 6325, 5841, 
             6815, 6817, 4400, 6313, 5477, 5061, 5800, 6811, 6814, 5808, 6272,
              6466, 5185, 6262, 5186, 6266, 8110, 6265, 6980, 6979, 7729, 
             8378, 8377, 8379, 5189, 8556, 8557, 8522, 4456, 5880, 5881, 4744,
              5499, 4743, 5932, 4791, 5926, 4739, 6409, 4740, 7301, 7302, 
             7299, 4790, 7300, 5547, 5548, 4814, 7090, 7092, 7093, 5931, 7724,
              7725, 7723, 7727, 7726, 4812, 6978, 4813, 7323, 4734, 6156, 
             4732, 5921, 4280, 6517, 6518, 6166, 6520, 4970, 6535, 4971, 5437,
              6733, 4367, 6158, 4967, 5704, 6162, 4625, 6178, 5439, 6173, 
             6174, 5700, 6735}],
            RowBox[{"-", "47.5`"}]],
           Annotation[#, -47.5, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4974, 5440, 6175, 5438, 6179, 4977, 5707, 6176, 5703, 
             6157, 4366, 4264, 6754, 6734, 7831, 7830, 4972, 6519, 5263, 6165,
              5262, 5388, 5387, 7259, 4733, 6975, 6977, 4452, 7322, 4451, 
             8109, 5929, 4741, 5514, 4742, 5924, 5923, 4455, 6983, 4792, 7029,
              7031, 7032, 5928, 4490, 5933, 8130, 8129, 7034, 4793, 5550, 
             5549, 6468, 5190, 6555, 5315, 6470, 6469, 5184, 7735, 5183, 7734,
              7733, 4453, 6264, 4454, 8111, 6982, 5187, 7736, 5036, 6273, 
             5807, 6813, 5809, 5062, 5815, 6314, 6312, 4399, 4301, 5838, 6839,
              5840, 5089, 6377, 5489, 6382, 5492, 6842}],
            RowBox[{"-", "49.400000000000006`"}]],
           Annotation[#, -49.400000000000006`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{6841, 6843, 4410, 6374, 5491, 5088, 5839, 6838, 6840, 
             5847, 6336, 6337, 5817, 6341, 4667, 6318, 4665, 6274, 5037, 5814,
              6785, 7624, 7625, 6981, 7621, 7620, 7732, 5182, 7891, 5314, 
             7897, 7898, 6554, 6553, 6565, 4491, 8132, 8131, 8477, 8476, 5326,
              7134, 7135, 7136, 6564, 5386, 7617, 5110, 7033, 5108, 7030, 
             5109, 7616, 4737, 8208, 4738, 7261, 4735, 5930, 5513, 5515, 4774,
              7278, 7186, 7187, 6976, 7190, 7188, 4530, 5982, 4531, 5554, 
             7094, 4502, 6170, 7097, 7095, 7908, 5264, 7949, 7950, 6753, 6755,
              6756, 6586, 6587, 6177, 4975, 5706, 4976, 5441, 6736, 4369, 
             6172, 4973}],
            RowBox[{"-", "51.300000000000004`"}]],
           Annotation[#, -51.300000000000004`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4312, 4409, 6373, 6375, 5854, 5090, 6379, 4678, 6343, 
             5848, 5067, 5816, 6340, 6338, 5480, 6331, 4666, 6275, 5813, 6784,
              7626, 6786, 6897, 7622, 5115, 8316, 7623, 5221, 7774, 5322, 
             6498, 5220, 6562, 8475, 7899, 8510, 7900, 8474, 8473, 8166, 6563,
              5325, 5389, 4290, 7279, 7275, 7276, 7260, 4772, 7277, 4736, 
             5925, 7008, 5732, 4634, 7518, 6894, 5552, 7189, 4529, 5983, 4816,
              5553, 7098, 4503, 7832, 6220, 7906, 7907, 7096, 5333, 6522, 
             7929, 6521, 6597, 5363, 7951, 5364, 7931, 8492, 5350, 7932, 7191,
              6181, 4978, 5736, 6737, 4266, 4368, 6171}],
            RowBox[{"-", "53.2`"}]],
           Annotation[#, -53.2, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4281, 6760, 5735, 6229, 4636, 6180, 4635, 8494, 8496, 
             8495, 5351, 7934, 8562, 8569, 6218, 7933, 6219, 6589, 8563, 6588,
              5002, 8491, 5348, 7930, 5349, 6574, 6573, 5003, 6221, 5265, 
             7099, 5266, 7833, 4817, 5987, 4815, 5980, 5551, 4377, 7793, 6895,
              5001, 5733, 6214, 6213, 6503, 4479, 8210, 8211, 7280, 4777, 
             7281, 4773, 5960, 7011, 5555, 4532, 7285, 4778, 7282, 7284, 8566,
              7283, 4481, 5885, 4480, 8392, 7009, 7773, 7772, 5219, 8391, 
             8390, 6898, 8388, 4391, 4291, 5810, 6819, 5812, 6332, 6330, 5479,
              6339, 5482, 6822, 5853, 5068, 6342, 4677, 6378, 4680, 6397, 
             5856, 6393, 6392}],
            RowBox[{"-", "55.1`"}]],
           Annotation[#, -55.1, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{5095, 5855, 6396, 6394, 5494, 6388, 4679, 6344, 5852, 
             6821, 6823, 4402, 6327, 5481, 5065, 5811, 6818, 4302, 7746, 7753,
              5882, 8389, 5883, 6497, 6496, 4686, 5886, 7010, 7286, 4776, 
             5966, 6412, 6410, 6411, 5963, 4775, 5967, 5520, 7795, 5237, 7037,
              8133, 8134, 6215, 6216, 5236, 7794, 5235, 8405, 4292, 5989, 
             5990, 5986, 4818, 5981, 4819, 5985, 5984, 4504, 7100, 6222, 5004,
              6223, 6547, 6548, 6546, 5287, 6585, 6217, 7935, 7936, 7868, 
             8429, 8430, 7866, 7867, 7192, 8493, 4637, 6234, 5450, 6227, 6228,
              5734, 6759}],
            RowBox[{"-", "57.00000000000001`"}]],
           Annotation[#, -57.00000000000001, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{5007, 5451, 6230, 5449, 6235, 5010, 6544, 5286, 6231, 
             6542, 6541, 6545, 4378, 6896, 6758, 6757, 5936, 5005, 5940, 5939,
              4505, 7101, 4822, 7103, 7105, 7106, 5988, 8154, 8153, 7102, 
             7327, 7326, 5390, 6502, 4533, 5557, 7035, 4492, 5944, 7639, 5121,
              7036, 5122, 5962, 5961, 4413, 5964, 5965, 5501, 5500, 5952, 
             4687, 6479, 5200, 5884, 5199, 7747, 5192, 6820, 5193, 7748, 5066,
              6475, 6476, 6329, 6473, 6326, 4401, 4303, 5849, 6845, 5851, 
             5093, 6389, 5493, 6395, 5496, 6849}],
            RowBox[{"-", "58.900000000000006`"}]],
           Annotation[#, -58.900000000000006`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{6848, 6850, 4412, 6385, 5495, 5092, 5850, 6844, 6847, 
             5858, 6345, 6474, 5196, 6328, 5197, 6335, 6988, 6334, 6987, 6986,
              7750, 7751, 5202, 7893, 5201, 7754, 4462, 5887, 8481, 5888, 
             4756, 5502, 4755, 5950, 4795, 5943, 4751, 6413, 4752, 7305, 7306,
              7303, 4794, 7304, 5556, 8227, 4823, 7329, 4824, 5949, 5948, 
             7743, 5191, 7104, 7745, 7744, 4820, 6985, 4821, 7325, 4747, 6224,
              4745, 5935, 4293, 6523, 6524, 6233, 6526, 5008, 6543, 5009, 
             5452, 6761, 4380, 6226, 5006}],
            RowBox[{"-", "60.800000000000004`"}]],
           Annotation[#, -60.800000000000004`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4313, 4411, 6384, 6387, 5863, 5094, 5860, 6846, 5857, 
             6346, 5070, 6478, 5198, 6348, 6989, 4460, 6333, 4459, 7749, 5194,
              7752, 5195, 6481, 6482, 5318, 6557, 5203, 6480, 5338, 8482, 
             8484, 8483, 4797, 7043, 8135, 8136, 5951, 4493, 5945, 7041, 7040,
              7038, 4796, 6990, 4461, 5941, 7328, 5942, 4754, 5517, 4753, 
             5946, 8112, 4457, 7324, 4458, 6984, 5559, 4746, 5938, 5392, 8414,
              5393, 5267, 6232, 5268, 6525, 5011, 7836, 7837, 6763, 7835, 
             4282, 4379, 6225}],
            RowBox[{"-", "62.7`"}]],
           Annotation[#, -62.7, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4294, 7834, 5269, 6762, 5270, 6237, 6236, 8156, 8155, 
             8579, 8415, 8413, 5391, 5937, 4535, 5560, 5766, 5765, 5968, 5969,
              5518, 5516, 5947, 4748, 8209, 4750, 8580, 4749, 8317, 8318, 
             7039, 5116, 7042, 7628, 7627, 7909, 7910, 4534, 5558, 6594, 6593,
              5327, 8478, 5328, 8137, 8138, 8139, 6566, 6567, 6556, 7892, 
             5316, 6569, 5317, 6472, 8464, 6471, 6407, 6408, 6347, 5071, 6477,
              5069, 5859, 8577, 6390, 6391, 5862, 6386, 5861, 8578}],
            RowBox[{"-", "64.60000000000001`"}]],
           Annotation[#, -64.60000000000001, "Tooltip"]& ]}, {}, {}}}],
      AspectRatio->1,
      BaseStyle->{FontFamily -> "Times", FontSize -> 12},
      DisplayFunction:>Identity,
      Frame->True,
      PlotLabel->FormBox[
       "\"\[Mu]=13.3 \[Alpha]=1.00 \[Epsilon]=10 PIN=0.40\"", 
        TraditionalForm],
      PlotRange->{{0, 50}, {0, 50}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {192., -189.}, ImageScaled[{0.5, 0.5}], {360., 360.}],
     InsetBox[
     GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJykfXV4Vcfzd3ALxUIgEDx4gLiS3L2WBAmEIEVLoTjBKW4tGtyhQJGgwaG4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       "], {{
         {RGBColor[0.293416, 0.0574044, 0.529412], EdgeForm[None], 
          GraphicsGroupBox[{
            PolygonBox[{{8398, 8397, 3769}, {8346, 8347, 3718}, {4083, 4124, 
             3718}, {4071, 4091, 1534}, {4405, 7887, 1480}, {4126, 4087, 
             3769}, {7748, 4405, 1480}, {8345, 5576, 1534}, {7343, 5199, 
             1166}, {4070, 4121, 438}, {4121, 4109, 438}, {7350, 4735, 756}, {
             5994, 4432, 480}, {4734, 7350, 756}, {5157, 8311, 3684}, {5992, 
             5200, 1166}, {5200, 7343, 1166}, {4128, 4069, 437}, {4404, 6388, 
             438}, {4401, 8176, 437}, {4432, 5991, 480}, {4127, 4128, 437}, {
             8312, 5160, 3685}, {6388, 4403, 438}, {4735, 5995, 756}, {4090, 
             4089, 1480}, {4080, 4068, 480}, {4115, 4116, 3529}, {8176, 4402, 
             437}, {5576, 7352, 1534}, {4693, 8169, 3529}, {4068, 4072, 
             480}, {4106, 4127, 437}, {4118, 4117, 3683}, {4109, 4110, 438}, {
             8310, 5156, 3683}},
             VertexColors->None], 
            PolygonBox[{{8507, 8502, 4093, 4098}, {7750, 8499, 4082, 4100}, {
             8506, 8497, 4073, 4097}, {5367, 8516, 4125, 4088}, {6433, 8503, 
             4094, 4096}, {5199, 7748, 1480, 1166}, {8503, 5003, 4077, 
             4094}, {7814, 8500, 4085, 4105}, {8502, 6433, 4096, 4093}, {5918,
              8498, 4076, 4092}, {8508, 4228, 4067, 4103}, {5236, 8512, 4113, 
             4084}, {8512, 8513, 4114, 4113}, {5158, 8514, 4119, 4078}, {6762,
              8509, 4104, 4099}, {8509, 8508, 4103, 4104}, {7811, 4921, 4075, 
             4102}, {5991, 5992, 1166, 480}, {8068, 8510, 4107, 4108}, {8516, 
             5236, 4084, 4125}, {8311, 4401, 437, 3684}, {8497, 5918, 4092, 
             4073}, {5003, 8506, 4097, 4077}, {8511, 8071, 4112, 4111}, {8513,
              8505, 4095, 4114}, {4074, 4071, 1534, 756}, {5995, 5994, 480, 
             756}, {4403, 8312, 3685, 438}, {7352, 4734, 756, 1534}, {4081, 
             4080, 480, 1166}, {4123, 4070, 438, 3685}, {4921, 7814, 4105, 
             4075}, {8515, 5161, 4079, 4122}, {4069, 4120, 3684, 437}, {8501, 
             7804, 4101, 4086}, {8500, 6762, 4099, 4085}, {4089, 4081, 1166, 
             1480}, {8505, 7811, 4102, 4095}, {4072, 4074, 756, 480}},
             VertexColors->None], 
            PolygonBox[{{4402, 8068, 4108, 4106, 437}, {5156, 8511, 4111, 
             4118, 3683}, {8499, 8345, 1534, 4091, 4082}, {8347, 7750, 4100, 
             4083, 3718}, {7887, 8501, 4086, 4090, 1480}, {8514, 5157, 3684, 
             4120, 4119}, {7804, 8398, 3769, 4087, 4101}, {8071, 4404, 438, 
             4110, 4112}, {5160, 8515, 4122, 4123, 3685}, {8498, 8310, 3683, 
             4117, 4076}, {8169, 5367, 4088, 4115, 3529}, {8397, 5158, 4078, 
             4126, 3769}, {8510, 4693, 3529, 4116, 4107}, {5161, 8346, 3718, 
             4124, 4079}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.3117315380935727, 0.08925017435888898, 0.5551567044762832], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtU0tPk1EQPVREaaqx9TNfK7X96ivgDhSM7xdhayTQqjsxRKAGE6LoQg1C
0bhDd+hGE9c+0ER27oxQhMSF+hd05Su69RzOt5jcydwzM+eemVsauNI7mgBQ
oa2hTReBX7TyBmA0B3SlgfkGYDwCOkrAu3VAIQTa6T9oArp5/guAibXABcan
NtnvZHylAMykgCLPiNZIP7se+NMIXKQdIWZkK/CFsWXWv0nMGWKOMb6TNQeI
OUD/BO0F/d2MfYycu8Dev0l8gTZOntdyrh3xvL+Z9cizjfhTzB0kvjfurdzX
eeA57X0S2EHM36x7nc8A1S3AYsJ5rYFzP7cATwPzVs96kzFD5D4c829LmZP4
rFCDKdZ8Qz27+abTzG3lPchtLGcNZ4n5TvvK8JPAPVR/mBreyrhPjbmTtH7m
tnMO1yNjNJPO0HNZYuwh8Z84ry5yPlRyzRHeV0Nz7ef9UcY/JPy+cxnzr/Ds
yZnzS+nBu7cN1nK5YL10fzZjTJnnZNZ15th3jLxnNwI/i75T/EfWmkpPYfvi
uGZ4PJ6jdudgvD/bA+cIX2OdjrR3T9irgfH7Q89GnLUfmqXmOMT4pdC6a+f6
UsY/a/auaXbScjC0noucdz1p3fax/l2+sUxcvdk6CXOy5D1Tfe3Iq3hPdL+U
NAf1U47wE6xxp+DemrGw0nAVG+MVW90P+rfJZz5vHtJO7xF+mv6eVMyNGtyj
FhXqUCT3baG10hzm8q5zg/rvTfutl7k71Rb7Ne7U4ZJ3r4e8HkeuOyt8yX3U
cybuuyvwm/Ve/eNvRWv5KPL/kYb/ASsBl/k=
             "],
             VertexColors->None], 
            PolygonBox[{{7887, 4405, 439, 3216}, {4697, 5993, 755, 714}, {
             8178, 8177, 712, 3414}, {7807, 7812, 3157, 1199}, {7805, 5235, 
             3154, 3153}, {5160, 8312, 2468, 1125}, {7352, 5576, 440, 2763}, {
             8510, 8068, 3414, 712}, {5941, 7351, 2762, 440}, {8504, 5004, 
             5003, 8503}, {5991, 4432, 2507, 1737}, {7749, 6510, 440, 3114}, {
             4330, 6432, 1998, 216}, {7019, 7711, 3076, 2468}, {5995, 4735, 
             2764, 1739}, {7809, 7807, 1199, 3156}, {4692, 5985, 1408, 712}, {
             8347, 8346, 1125, 3114}, {4402, 8176, 3538, 3414}, {5156, 8310, 
             1983, 1122}, {4227, 7808, 3156, 1408}, {7343, 5200, 3113, 
             2758}, {7345, 7344, 439, 2758}, {7712, 6511, 1125, 3076}, {6762, 
             8500, 3158, 2263}, {7811, 8505, 1199, 3157}, {8169, 4693, 712, 
             1408}, {6433, 8502, 216, 1998}, {5574, 8370, 3737, 439}, {5576, 
             8345, 3717, 440}, {5199, 7343, 2758, 3112}, {8508, 8509, 2263, 
             180}, {5236, 8516, 1408, 1199}, {5235, 6518, 2072, 3154}, {5234, 
             7805, 3153, 3737}, {8311, 5157, 1123, 1268}, {8176, 4401, 1268, 
             3538}, {5200, 5992, 1737, 3113}, {4735, 7350, 2762, 2764}, {5989,
              4695, 713, 755}, {6514, 5154, 1122, 1983}, {4405, 7748, 3112, 
             439}, {4754, 8069, 3414, 3538}, {5917, 6400, 1983, 1681}, {8499, 
             7750, 3114, 3717}, {6511, 7749, 3114, 1125}, {7804, 8501, 3216, 
             3153}, {7350, 4734, 2763, 2762}, {7346, 7347, 2758, 713}, {8498, 
             5918, 1681, 1983}, {6390, 8070, 3415, 1122}, {7813, 6760, 2263, 
             3158}, {7886, 5280, 1268, 2072}, {5281, 8396, 3538, 1268}, {6761,
              4303, 180, 2263}, {4432, 5994, 1739, 2507}, {8071, 8511, 1122, 
             3415}, {7351, 5943, 714, 2762}, {8397, 8398, 3153, 1123}},
             VertexColors->None], 
            PolygonBox[{{8070, 6389, 6388, 4404, 3415}, {4922, 7813, 3158, 
             7814, 4921}, {6389, 7020, 2468, 4403, 6388}, {7810, 4922, 4921, 
             7811, 3157}, {5001, 8504, 8503, 6433, 1998}, {5004, 5919, 1681, 
             8506, 5003}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.3386143852090891, 0.1359921853766091, 0.5929437981960655], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFlMtrU2EQxUdjYixVYs31tkl7kyjiP6C1uHctqCBulEQrRksrVXShtmqN
+EQQcSe0VtCVtLFixbWmllKRCrqr3VbwUXwtFPwd5oKLj/sx35k5c85MUqr0
7epdbmb7OQnOk3azOmeqyWwwMrvA2d1sNpozK2bNyivMqqHZvhaz6ZTZWe7j
68y+kDxG3jinQW4jMHuXN/ubNOsE0wF+GPyfpL8vQXqX97E4d477QXCvwDyj
xtbQ8WW+e8h9yb0HzI9WsxcrvU5X2uwx/dSL5AUeu7rWucV7nt6HMmhCV437
driuJ5znUMyV5XsTzAyYCvfunHOJ53LBuayN/kPHPKW3Cc7rJtctDeq/3mF2
GMwUuaNwDcS+DePZLPdb3Au83yDnOfjfq8x+cb4vM7uDvrfUnKfmNd6D0DFX
6GELWkbQsoE6/fSxHs1HiQ2CmUy6pouR69LMxC9vi2AX0TUB5gM9vs8757bQ
/ZW3vWB2kBtQs5T1GWu+iWaP6T5E/CGYnmbXo6+4ZvB8hLfJNWZf8aiRch8e
gb0fuedHcu678NIqfWnuBWLH8l5f+XpTPCK+Oev8A/T5if4/Umc243VVU/rE
Jy7NX3PWDmxCy4HA9aqe/Ja3qnEm9DqnwJzIOr6PezHWXmPOt1u8h5PCcMK0
77c8Vu43MEucudW843l75Luxk317QPw08Wrgs1GOetnY5v10ldjLeO7iPxd5
D9oh7azmtUCfn1t9xzQT9afe5Ot8ynt4A6aa832QD+JUrvZ3Id5h7bR6lJYa
9Trj/RFnf6xdO6U3xfeiuzv8zzMdcx3PuO/yXF434rjy5bm81e/jZ/x7LAc+
A+m9FPl/h/ZF+y790l7Bq3uh5/4Dzr64xw==
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtU8tL1GEUPTU1jmExj9/0c5zfPMRFf4GTf4HboKLXziGjQTNTqkVl6RRo
LYKoRUSgPWgnNhoYrVMLKcpeu1q36CFRLQo6pzOLC9/33XPvPffc+3VWj+8e
2gigl7aJVsnSMsBECzCzGRjIA4O0J7z3EVBI8x4Cq3FgmjZeBC7Q5hm7pw0o
B0CR91ib8YeJ7Wa+ccYvEd/P+znm+MjzMu0FsSdzwI8NwFXGFOgv0R7HWIM5
j7F2JXSt+63k1Q7sSzuXcp6h7yztK/GfiP9C/wnm255w/BxrNwrAStyYO6x3
t2iu4vw5CdykxZl7gf0+oK+HMVeIHSRmjPd64LNi6sz/MmltpFElBQyxXjZh
fnvT5ihuT1nzHflPB+7tF/3DOfckrGKmWKucc2313B06p/zq+S2x16XXFsdn
6Q9C9yb8bZ4/EPO71RpeI3a1DCxuAxIxa/W9BKxttWYLETDJ/H9Y7xlzHqR/
gDNf5P05/Q/pn6WtcymW6R8lt9GM5x+S73nWGwuttWJ+steuwD5hJqjXxaa+
0nCJueci51LNMn2londD/depZV9orTSPQ+Szszlv7dg9cr+RN5/T7OEIfbtS
PmumB4jvJcdvLb6P8Iyc9dEOaNdqoXvTzkWs3Rm4vvgqdn/au6gc0kIcVFv9
addrTX301kX85Yy1/z+zDubK+qwdesVeX+etvWb2Pu8ZanaaUSMyZ3Fdob+f
8cNJ9y4N3hBbTXmWf5lzhFwbZWs3S863WH8qZV9PwrPUm87CSKtHkWcrzfS3
TuUcq5z6i9Wm3trRdep7qWS+2hHNfj4yN8XvCLyzmpX+dI18j3Z4tvoDM4F3
XP1p5/R3JtvtE0ZarOW9e6rxDycyr8o=
             "],
             VertexColors->None], 
            PolygonBox[{{6513, 6385, 6384, 5154, 2070}, {6385, 6516, 2071, 
             5155, 6384}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.3654972323246055, 0.1827341963943292, 0.6307308919158477], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVM9LVGEUveP0a2YUnJk3P9688akUtC5simpT1iaoqP9AsUBIc7JFUY2/
xlIphOwnkVBpUbiIqIUt2mvUmCUlQbUo2hky1DI6p/MWl+/77ne/e88997zX
3H7yaHeNmZ2AhWEda8wm15kNw1nKmPXBPuM8ETNbxvod/sNxs+Pwfw3hHdaG
hOI7UzpzvxLWux/YO/ClYOM4t/pmNz2z+YjZnbzZqYRiXmB/Hf45+l2zEdTY
BH/fWrNIWPsB5BhKCUM93v3NKmcxoTv6t2Dtb1IPV5JmHs4v8X4Rud+hxm/k
L3uqzbpR3FXhq6KXAmK7PHHwGra0wWwU9zfS6DdnNhYWrud54bmN+La43qax
f5gTnlXYn4hwLyHfR0912hH7wdPdFHjYiTxzqHMB/eYDDvfAf9Axuwx/V61Z
N6wTNoBehx3hKqKvn8A0g5w5xLswQ8z2pLAS5wx8ZUc5XoGPJ75y+VibHMWv
ImcRtWfXmz1rMHuK91dj8q1k5V8AxgrsS9TsAd7e94VnBPc7EF9GzHtP/JLb
XYi9lJWfMz5TJ51MN4KfRp23gquSqxm1Yt0H+4X4QUe8kJPHvnAT8xjqbERM
BTEHgPNiwANnO57UfKnN3RHpjj3H6hRzJCW+yBXxPvKFrR1vtyU1X76jVvmW
8yxlNdN+rC1BDLWWDjTM2VCn7Gu/qx6Iv6ZWPVE/t4B9sF76JRZqiXiuJaQJ
6mEIfZSCfkfj0j11Rf2So8lAS9+iiie///UakhbIBXNSsz2u6k4g/5u8tMf6
b/PCcNrVvDnTloy+AeqfvfCeddvA1WxOfBbQdyElLXH2xMGc077mRMznkedc
RtyTD+ZiHs6d/wt+j8S12RE29lENeqD+yBf9zbjPI28I50OOvgHm6gH+Yzlx
1ws8va7myLl1ZDQX/jOmGlX/E/Z3M8JawX7BU73lYLbEQ33PB7Pjv2SvLz+/
p3s55T+bFQ5i+AcGpsTs
             "],
             VertexColors->None], 
            PolygonBox[{{6011, 4331, 2364, 1753}, {7226, 7227, 1924, 1225}, {
             7850, 7846, 3177, 3182}, {6383, 5457, 940, 1977}, {8102, 6264, 
             479, 3448}, {5252, 7849, 3182, 3181}, {7708, 4979, 940, 3073}, {
             7120, 6041, 499, 2547}, {7842, 7848, 3181, 3177}, {7348, 7349, 
             2505, 548}, {5909, 5002, 1999, 1679}, {4636, 5940, 741, 645}, {
             7846, 7845, 1225, 3177}, {6468, 6013, 1006, 2031}, {6045, 4450, 
             2550, 1777}, {4302, 4208, 2261, 3339}, {7374, 4755, 777, 1407}, {
             4753, 7378, 2784, 1777}, {7124, 5575, 777, 2550}, {7375, 4753, 
             1777, 2782}, {4331, 6014, 1755, 2364}, {6757, 5986, 905, 2261}, {
             7709, 5937, 1694, 3074}, {7705, 7709, 3074, 3072}, {4637, 5942, 
             1124, 645}, {8100, 4431, 479, 2505}, {7378, 7119, 2547, 2784}, {
             7062, 5041, 2963, 2506}, {5159, 7710, 3072, 1124}, {6015, 5482, 
             741, 1755}, {4238, 4322, 4321, 4237}, {7127, 7128, 1777, 778}, {
             5482, 5481, 645, 741}, {7847, 5251, 1225, 1924}, {5575, 4489, 
             548, 777}, {6013, 4721, 740, 1006}, {6270, 6272, 905, 499}, {
             5041, 6012, 1753, 2963}, {7373, 7376, 2782, 777}, {7848, 4302, 
             3339, 3181}, {7746, 7747, 2506, 740}, {6817, 7707, 3073, 940}, {
             5987, 7123, 2549, 499}, {5990, 7602, 2962, 479}, {6016, 6469, 
             2031, 1006}, {4916, 6273, 1227, 905}, {7063, 8101, 3448, 2506}, {
             6434, 4624, 1679, 1999}, {4722, 6016, 1006, 741}, {6264, 6265, 
             1164, 479}, {5040, 7062, 2506, 2962}, {5988, 5987, 499, 905}, {
             7377, 4756, 778, 2549}, {5254, 7852, 3181, 1227}, {5429, 6278, 
             6277, 5428}, {6438, 5472, 5471, 6437}, {6526, 6525, 645, 1694}, {
             5197, 8344, 1407, 1164}, {7806, 7843, 3177, 1225}, {6515, 6516, 
             1977, 940}},
             VertexColors->None], 
            PolygonBox[{{6279, 5429, 5428, 6280, 1924}, {6820, 7706, 3072, 
             5152, 6819}, {6278, 4919, 906, 5430, 6277}, {6822, 4238, 4237, 
             5908, 1679}, {5472, 6434, 1999, 6435, 5471}, {7707, 6820, 6819, 
             5153, 3073}, {5007, 6438, 6437, 5473, 954}, {4322, 6383, 1977, 
             6382, 4321}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.39238007944012193`, 0.2294762074120493, 0.66851798563563], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1VEtIlFEUPvmYMZ2GcZxp5p/5HSvIWrTIVITa9VhERGSrWkWbIKHJapGU
NT5QW/QixUdBBGkPokXhQmnRphaVjlkQiBIEmRszaJNY1Pf13RaX//7nnvOd
73zn3Lv+WLbxZIGZNWMVYn3Fz0zA7O0qs6aQWRbrSLHZtgqzV55ZD5x2lZmd
Cmm/Cd83OL9SBHvGbGdGttsJs1tY34HVHFLMa/gsJ80OxM2eICYfM1uB/xjO
LwXNGkqE2ZE2q0XsHcSGo2a/EHMRvIarzPrTLleZ4ojZts6sHljZInGsrRBO
Ctj3UmY/gbsB9qNx4VwH5hDwl4BfDYwLyN0F/2nf7AbOgtjvgX8OPrMB1UBf
2gdhm0rrn9jpjPAbkLMLPDuAlQuqHnIgNvkS/zd8v5SbHQeHFuBkketjQLbD
SdknImbj4PkUvaiEv4f1AvtzOB+pEt4AeA5iPSs1exBWj9gr8j0dFeet4NO7
Vnlz1C8pTvRlDHUrBN6HmPp7FXlHUM8yammDb12FqwPatmM1ISYSlY7UsAbn
iynhzyNPW0Q8d4DXdqxD4PYH3A05niPniZC+tLckVQ9rSXnqMTGpR58vTcZX
68s5ZOz9sHg2g++Qp1k5CJ2WkpqlT+A4jbOpsOYmU6J54GxuLlEPHoJPZ0y1
0+8/L2o/7/oyDJ+RjPRkb++6+elBzsuesMqj4k3OddDcjypnY1zzTT7EeORw
WunridMs6p8rVX/Pw/e9078TfDaGZH+MuK6Y7s036LeINbnGbF+l2d5KzT7n
fUtGNfIu5ty8DUG/btSyUizNmtz9rUlontijuYDuOO8683MO/mmLOloTOmM/
Z1xPR8F3DP1YKFBPGMe+0MYZpJ19+BxR7dR8zvnVQ5NuN9tngN/ry8566Mc9
53XBYTL/bk8cOL+cM84Y70RfQvpQJ95b2hnHWhnbD+yzUdlu4jvpS2u+Nz9c
X6hZe1q6MY5zxlhq9jIu2xTiJnz1itxGXY0DmLH9MfWqGhyvgdu7oN6MvK/3
IZ/WO8L9XxAG1FE=
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtVDtMk2EUvaItiEiktPwtLX/roDiqYEx0EdBBjQ64OamDJJJYiyYGH21p
C+IAMVHeJiZGfMdB44AxxsVFHi3GxIRATByQBXVw0BDBc3I63OS7//2+c889
995/65l46/kSM2uFbYDtdszSMbP2SrM4PnTBT/jM5rxmC7DlqFlTldmkx2xm
s9nBkFkLbGqd7jQjNhwxe1VuNrHR7FTAbDvis6VmSSTKhM2ysHte4fcEzfbg
ThtiWdxpcs2O+/Gmwmwb7HCdWR7+BM4tm8zegdsT3PmIt4/BcW6LWarI9z34
NyOWDYhfPd4cgD+NOy9NeHfA7UdU3L+XKFcnOFwrVc4RxIdgSyWqYQhcW1xh
9SHnZdzdWQ3e6+V7gPUV+QvIfw53koifhga/UXsKmAPAuuhTri+oeRVY/1zV
ewL4HcBaqkEclsD5fq3ZYo1q+VNm9g3c+2F/cX6BmtaA76AnNbBVnPcB/4Gr
3MQL4Vzr6i0xnrrSjFq1484o3s2Gxe8n+GQQ64qpF9SQWBlo8qhYTwM4Ha2T
9r2oeQXxK0Hpz56kcE7g/t4y1XsD/v6IuLDGZEgzRXwXdy5Bi1HEVzzSeArn
QcRLgT2PNwVwy4flkx9nrRcYO8rU01HiVYsLc45Dr9aAtKFGEeBfDykXZ6wb
WJ2OtH+DmoegbRvenIUN4pxH/kJEuZmTs7/LkXbUY8ARR8Y5QzngNTrC5nz3
g0sEfhjWh/NDzMN4VLPBmTrm10xz9nKVqvUu7JdX+aaBfRucF8qVYwZ+pjiv
/HYzpJkjHjWoh1/l0y5xp5ZrNePE50x+Qr41+G8rNJMW1cyxH/zG3eBOc5ep
J3MNx8SNOY+g193AuFDcN/b6Gd7HK9TzRfQjF9Susr9jEWlCLV4jRyM0SAc1
C2nYh4B6QO3Z8wZH/wjep4bclRFHveZMcjZTxff0e/ziRC4JcOjwKSdzzXv1
7zkU0pn/IHLvCio3a8jBf+7qLfl/9mtnOdsnPartalBn1jgGLrd84sMecRfG
Xe0Od+I/sI/LoA==
             "],
             VertexColors->None], 
            PolygonBox[{{6276, 5783, 5782, 4919, 1923}, {5916, 6439, 2001, 
             5007, 5915}, {5911, 6821, 2297, 6822, 5910}, {6396, 6399, 6398, 
             4624, 638}, {6387, 4978, 4977, 5457, 1454}, {5864, 6818, 2296, 
             6817, 5863}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.41926292655563846`, 0.2762182184297696, 0.7063050793554125], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVEtIlGEUvZqjOf6JM47zOzqORotWlSiV7UqCiiDCTVSr3AU5PpIizWeF
1ialJs1aBRIUQymBoamD0qaSMFpYmyBCKtBN4KaHncP5Fpf/mzv3ce459/u2
NzY3JLPNrAe2BdZUaHYBNlBgdrXUrAvWnWdWU2w2FtP5Xdws5Zut5ZpdwfdG
udkqijQjryBg9jbL7FPCbF/ELOyZjSK+PiT/KGoswwbQrBo1L6N+J2q247we
NUvD3wPfMmqmc8w6cP6NWl158o358mdKzJbwOxfx857+Z81OxD918Z34dsBO
ejJiZ+4CznOe4rsDwr4DNhQ2e+CrZhvw/EKtaeTc8zUz5x3Dtx1xL4JmrzDH
RkQ1WjF7Etaao7iNgOrEkBtH3BB8M8A8HHX/VZqtRITlZb64Iz9fqswC+C8D
f0uh6rIm/29x56TzhzzFfowoPuRyWI/nJqdjA/qOl4nbO+j/DVqcyxXuZ0HN
H40JK3GGfenB+CcVZreQ+3OrZp4MiquL4CAFLiaC0ph+4n8elO9rtnSgftSi
DbHJsPxJh53YqMO804I40i6XsWd8nbkjN2PakyHUGY4rrg7fA7AGnGc9fU9j
pkfA+x1z9mGWCXCzktBeHka/85jjVECY+GXfpSKzg4ipAW/X0WsV/DQidxQ1
TpTpTN36Q9JuGr3qC5TPnb3kuBosE1/kijo0Oy3INbG/AT//0Meg2QxqbKLX
H9hcnvqkourFneotkj57sA+PK6T7e/iPVagvsYz6wsPZmt1u9CG+v0p3uDWi
u8SdJ8+LnvhZR/+7cWm1gPMiLLXNrNjTl/r+TQgrcR5Cr7VK4admrL0Xc50t
FUfETP2Zz1zi4U5wH1rd3lLv/cCzK6G4DHJ/OI24m74vDo9gvqMV4ncwpt7s
y5npo167E9KVfcYRW10lXMdxri2Wzj3wfS4STmq7ViZ9r+HN6C+XLhn3DpAT
cjPrztyXSVf/dlhvDe8y37BaX3NRk7qIOH/o643gved+cT+Itw99emFT+drN
GVd/BNzfj4uzkah2jhyG0Guneyum3N5w9s2Q+CJXm8D2IaI6fHdfl+jeMC/u
7i/vdNrd62zollWp2Up93XPGrEZ1T1jzP0Ui4b0=
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtI1FEUxo+PxhzN1BmdGWcci1r2MMVyW0QPWihuwna5iaAUrU2Z5uhC
a1GWTpq1bSmVBIWWhVJQYW8i2wRRUZERLYIeYr/DN4sD9/7Pvd/3ne+c+1/d
0tbUmm1mDUQucaDCbKTcrCVg9nq52XzYLLvK7E6h2SzxjfxUmdmvZWY1+WZj
EbPBUrNAjtk4AAn2Z0Jmz/L4DkYbuWN8+wTJe2IBrO/EtaDZoyyzF+C/W2V2
D+xmMHPIveTbXfbTxAJ8m8DbWiC+DyVmOyq13w9+qhhs9KZYf0XvPriGEsJ3
vmqw+4jWIrN+7kyAPZ80KwW7AL6fUbOlEt2fzBO21+hczlHH+RT3D3H/MDEV
M7tNdHN3DfgPWNegr5u7B+Ffl9QdP1sCxynyAzHV2smZYbRWh7R2/f1wj6H3
RlB4l9B/mVgIyNPcKvVgNuNHL1pOEnX50nQcvhNR8TeSP432HxXKux9e+0ai
PVceTNK7uWLV5z2sDakG176dendT/55KrfeSP0r/RtA3EZTGf/DtJL+N/GSh
uK5Uylvn7AH7POevBuXRc7AD1DDD2X7qaaa2VNzsZr486WE9VC4v/Fuc8yvh
LCIqYsIaJMaDwlwP/wx46RVmoUL12jEdy3u+Ga8uZPR6j8+x7khoFhzjbbFm
xr1xvQ/DqslrcQ/q2W9Iajbcs7+sn4blRRcef0HrQIXu+8yMobE9rNn3Hqxl
H41o9v0NvAprpr13PuMRckv0a5rcOH70xjUzPiu30DOM1iPUfj2oet6ENQN+
3z30t9EZlRZ/I7NVmkmfHfckndCb83odw8gvJfW2pjifxb6eM01BvWl/W8MZ
/33e06zbi/X2/R+wi96OxtQ773lXVJ65V17vIvvSiGrxmv6wb2TGGojfrO+X
acZdn/8DPoPfEdJbc//SEc2kz6LP/BP86Exqlt1T7+WWsHrtPf0I1mJStXgN
F8mPEo+zlK8Fry8uL/3bKPuzmf+Tc8zFxeHY/u0/L2jAYA==
             "],
             VertexColors->None], 
            PolygonBox[{{5456, 6386, 1978, 6387, 5455}, {6234, 6235, 1634, 
             4582, 6233}, {6428, 6427, 6426, 4625, 1680}, {7225, 6234, 6233, 
             4581, 2642}, {4623, 6397, 1982, 6396, 4622}, {4982, 6394, 6393, 
             5458, 941}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.446145773671155, 0.32296022944748987`, 0.7440921730751949], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVDtMk2EUvbyhiLblL+3fFkqxq48JcTFOPgZBHYxijESMcRKKURPRCNFB
HQQhKoqJEwlootHBGJEaHzEY1IgLsjiIDwZjoJMxEj3H8w03/9eb+5177jn3
a/pAx84jhWbWhShCBKNmv2Jml0rNBpJmPUGz+8iv88yu4ndnsdlh36w5ZPap
wOxH3GxTrdkhgNwNmPUhXuLcWm7WXWLWgGhH/ZmI8vkK4COXx92z9WZrEcnl
ZqPAGENcqDSrqBIO++Zwd8IX3hN8xxF7cF4En7ynHot1ZguIe8vMdoH7NPK7
kd8MvD/Ib8W5E3e6EO04vwFGshpzlJkleEZM4bwxZfYtpf7zmNtDfgYcxoE5
ivrPRcJ77QmTWNly9SKvFylxu4PzY9Skke9NIAdNc9BzBrkPvu7EkMs4/HPQ
8povXajro4i0/Y2+x9F3oky4K52e+1GzLSYNu+DJ5aT0eu9pfvLpDCNXYzZX
KN8aPXk3iFxPQvmhoOYkhwX4eDGuGWdRW5zSfKX4TvvSbxj6eoh9wJmFDg9Q
dxO/T+B8HTWNqMmj/3bwPwb8buTfJeQ5+baFxJkz0Df6OwhurU6fPnAewvkn
zq/AqwV3Rkw9z1eq73hEO8d9OxhUHWv6k9rX/ztTql3kvo3UajbOtRp8mjzh
zYHL04hwqP2ziPR/GNcb4P5zF1rC2gfu+JeQ6g25rzHNmMN3Bzy6VSaO3GX2
ypTIf/rYG5Ef1Jx5+kgv1tTrDfBOHLxu+5qRuzPm6z5rMw7nFHqdrJOGA/Cx
H9EGnh2ePKD+e1HTGtNuB6rEnZosIbcirLmGgL2lVvXkRL2oFfvdcP7yO+zO
9Ji/yTkLva/USHNqsBSSDnwTp2N6F/T8rfP9b0g7wZoGNw9n2eBrZuLzfXAP
yKMwpf3j7h1Frjksbny7Wfd+meN/CvPc04+unu+fM5EDNViflA7VUfnK2TlH
2ulPz6cS8n3U1/8auX0H5jxiEto1AaMgpTdFLSfdf9Nz5FbVaR7ucsDpnHZ+
ET8cle7s+w+X6snZ
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtk01szGEQxp9Wt3Rtsd3/frW7+285+zhRFx8nXFScaCIaksappdqLEm1C
0jpY5aBF4oS2ElInUl0NDpJWoiTUxUElPg6lPYgQ4TeZHiZ533lnnnnmmXkb
jrTvbyuXtB2rwF7WSZNJ6VdEWqySRvLSeEb6uEzqXym1xKVpYvbYe5nUlZC+
p6R5rJPzm6z0LpBOrJA6sAH84+C1UqSlUnobeIy9Hcdec36F70DEczpz0kns
OfHFqDSU9RyL3Yw1cr4Dpz64BDFpfUHaYu+QXwfGVnIbc453bLm0j9q/4T/B
uQvMTfXSrbz3k1sl/eFNNf5mMT301ouV4DqH72paOhd3PebxTfF2OO69m+8J
+JdT3pv1aNw21ju2cbzNfRs9HKrw+yjnu1hDxH2rqV3HfYraF8GYhM+/uPO5
iW+R3naGUrRa6iZnN3i78q6/6ZEmtxaOWSzFeRY9xsi5jjbXsGb8bdS4F/We
huDaVCvtxQZTjr0Dq6r2Gpa7oeDaGobtQrHGe7d+H3EvUaeZ2msjvgsLxN+P
LXEi/z33GzGfyQz36azPwmbSs0b6GXjuGNKfIra74Ltmmp4Fvzfp2tsO/EWL
Usa1ME0eJp2D5X8gfoBZX8o5lu2LYT+m3sElfmfIPZ1xbOM3wdtC4FpaD+fZ
2UTatbcZ/IBLe+DaWk+znCtD37+jEdeyI+e1TNPhhM/YZms7VUZseeix1u8M
9QazjmcYX0Lfcct/gd7fmEE/s+jDvnIuYnNx3yXTYITc4az3ajvzIPAZ22xN
4yb4fA4dy2b4idwrKc81jAvgjub97xq/Z8Q+Df2vmEb/Ae4EnF8=
             "],
             VertexColors->None], 
            PolygonBox[{{4892, 5785, 5784, 6235, 1896}, {5776, 4483, 4482, 
             5777, 1632}, {6392, 4981, 4980, 5906, 1678}, {6735, 4196, 165, 
             5781, 6734}, {6420, 6423, 6422, 5912, 215}, {4945, 5866, 5865, 
             6336, 1951}, {6793, 7592, 2953, 7594, 6792}, {4483, 6039, 1774, 
             6040, 4482}, {5871, 6395, 1981, 4982, 5870}, {5914, 4997, 1995, 
             6428, 5913}},
             VertexColors->None]}]}, 
         {RGBColor[0.4730286207866714, 0.36970224046521, 0.7818792667949772], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNVE1sjFEUvfrDTKPS0a/z+TpjOrYSrGyLVSWo0opuqEwbCUanCRVWfloL
kTCpxGgkFpVIi4QFWnTYmA5RCdqwEouK/0SrGhIb5+S8JhYv/ebe884999z7
uiKV2d5ZYmYZnFKc5mVmLTh7F5lVhcx2VJrNlpn1Rc0e15j9Ljc7XW1W65sN
48LqiFlPnXD7Y2Zr8HsgbOYHZh7OM/DMJMzGkZtB3Jaa/QX/qYVmt5NmHbVm
E6iTw/c98P8C/xQ4zqNeGzCNi5VPIf4NsSl3dw6/z0JXXYn4fkQV5/1CheJf
EdsGrZehc8YzyweK70ad3mrp/wTtW6GpgHga+VFwXYHmjSH1z97XA7MB5z3w
D5EfWi7MJPJpp4339sUU3wlvOtFzHnruo9ZIjfSy/3bkb4IzB0wP+kyi7ijy
qZi8vQgNndC6FrwdVdAGf8ZCyued/93Q/gf1RsF1Dv5EnM5iWBrZI+u3+tJA
XQ9cX48CaZr3j34Rfw09DYL3CzAF8BQrxdWIe90J6eZ8ht2MqD0blX7OsN3N
8STyx3GKwDwNSxvrcI6XlgjTBM4AtTrAWQ/uTZ4wJ3BvVUR3b0Dn9UBzeIN8
fyCfOf/vbtZ3UDMHj8pQpxm9znrSM+dqk3Ozpxr85l7QI/bbBvwW1Hq3wOwF
9zMhL2PQtRI6jpZqj+kfd+NgSG+Cce7ynojmeThu9hm7kQX3MPSM4JSH1eek
242X0PA60K7UB/KaPnMHQ+4dxYFZl5BfXVHpYK1DcdUgf7Wvt0d8PzBxh2nC
3wOB9nwQ3Gdq9c05PqmQfvaRnf9Gfy2+dsnH3WNuvvRjly9PMtAzndCuduH7
VlLfY2HxEsN5cq6Mp8HzEbO5C54L8KcvJuwgdA3hNKCvNvA/R3wa8daQZsu9
/RBVD9Q8AC+v4hzB/vVG9E4404lAe0A/2R/fNvHc0aLTwxqt/70FcjBOTxtc
nLvMntnvODhfeZotfeK86c9bT/VYq+DeFHn4P+Cn52pWKkf8Pzm61DY=
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtk0tIlWEQhqfU+g0Nj+f/Pf1H89g2qFYtu+wKzCNkFzdlKJiUpRDdg667
wMwik6CFbbSCCirDvGxSuxh0g9pEi+hGQZqFq6LnZc5i4Jtv5nvnfWe+WdLY
tnHvXDOrxvKx4dhsBPtTYJYhkE6ZpfCfzDcbyDPrS5o9XGR2Ff8bdoR4R5lZ
wzyzd9gnYlZqdorzPeKnM2bN5WYzgG8uNuvifB4bKzQbxzp524XNUq+Kes+o
VQdGM29bA7NNnNuJH8rzu0H8VbHXFodbVWZ3sMsLzW5S41/CbJScEewv553k
Lk947RLwzoHVXupcxXlZwjXIn4DDLO++lzn/IfDvp82SKdeeD98fxFqTrk05
v0Kz6dB79YGcWrA/o3mCemPc9S+mZ9iVPOd7Ee29xA8QnwCvCX5Z3gyDVQy/
arC60fMbLmfRswJ+W1Me7yX/Gm8b8N/PcYwdxGvg3AT2NPGf8NufdO7iNxqZ
fUy4VvX4cCX6Q++V3uhtA/GpQvcj+GyPXK9mfpD8tlx+tshnPRQ5ljDXEN8Q
OteE5gW3upTPUjnHSl2ztI4tMHsF/lvyGwt8vprdbex14DPcTbw19l6pZ1Pg
T1e6NnHYBvaFcv876kFP7BzFTZi1zKsn91/GwZyh1iVyVgbOqYLeTOLvCTzn
AVqy6H9c6H9+IHLO4qoZfKXfXzI+y07uAurfTftfUI+aOLeVmL0JXMNTuGVz
81BP68kvC322mqHOJyL/a7o7Sq1d5T7bR+B3w+1MwrmKs7i1xN5bcRzk/Q38
9fj1WH/sdzqvC5yLdki9FactcHkRuq/90d9cm/G/qj+q3XxO/zqKfEcnOVeE
Xk93+pv7Kly7+KeJrc7tn/ZJuywN4q6d1m6dzOnTTr0MnYNqq+fS0hc7V9XQ
LmtHhSXMpbytSfvsNJ/jke+IeiXM67FzUL568B9jgrc7
             "],
             VertexColors->None], 
            PolygonBox[{{5570, 7114, 2544, 7115, 5569}, {5230, 5570, 5569, 
             6551, 2101}, {4295, 6224, 1892, 6223, 4294}, {5905, 6391, 1980, 
             6392, 5904}, {6335, 5860, 923, 4945, 6334}, {6840, 4329, 4328, 
             4248, 2309}, {6733, 4295, 4294, 4196, 2246}, {4329, 6421, 1993, 
             6420, 4328}, {6374, 4606, 4605, 6373, 1973}, {4247, 5905, 5904, 
             6838, 2308}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.4999114679021878, 0.41644425148293013`, 0.8196663605147595], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVFtLlFEU3Vo5KoM4l2++ufFp1EtB6R+QoAgys0wDhdBITWW0USqKzAdB
iB4q6IK+lF1fumcKTReIqaEBp35ATxHdiB6yC/QWreX6HjbnzJl91l577XW+
1b2j7dlyM8siViDyFWaFKrMfONy70uwXYq7MbDFl9tIxu4Pfg9gPIa4h7wLW
blf3nITZt1qz+VVmY1Gz457ZrqDZp5jZi7jZJHKu15ldxJ2fwM9Xmy1hfY3o
w9l+xCwwB7BOIe8q9h24VwLuTXD7FzL74uP8Af6WpNkEas3Um7Xg91SN2btq
YZDzOeCcT6nGlSrhPcL5c/RxNqYc3lkDjgvYH0CdxpD2Tdg/jSu/WC6+yznA
608JrxN9NyB/HufbwKUZUag0O4q+jyBag9IsG5VuRWjTipxL4Dwehr6udN6K
OtmYehzH2YaQuLG/r/XqkbwaXHGYwN3Hjjilgf0GtXqAH8O+y9Xc1uH/9rhq
sb+1QeX0o6/ZqOZaZB/+nj25nvp6iJpziBlo8wEYmQh6DOC+K97knAHfFvA7
BtO89TQz6psC3kBCteijVxXKPxHWPDiLv8AMAauEfkehSV9Svd9KQ6e0fEEs
ciEOPZVLylc74/Ii9ZxGnd9R6dMB/CFHs7oP7sM4n65RferE85PoowZ1c+C8
EXoOJjRfD2u3I/weRz7mvgO5exDPgBHHWQKxGBBuc1Bz57o9KM3b4poxNV/y
/c2c5X2FfH4QvA554k99+T7oGWpEruRJXk0+t+8x6UWtJh3NmLU+4/xjTG+h
Db1vTgh/B/7f5Gne1HimVnfz/hyYQ08d9qRbADmn/ZxCrbzIWYwAbxgRqdT7
45x4znfMd8i6yYh8QA8Q472Pk0POE8RlaNWFtRMxGJDv6Dn67TbmvOBIn33g
cKNO/fN9rHek4b20PEEdS+B7Jqz3TW8+8P3JvneH1Rfn3x6WhvzG8J1w9iOo
O+ZpTqy9jFEmL5yKyA/8HvDt8bzRlZ+Ik0nJu5wh69xNq9Z/B5vPnw==
             "],
             VertexColors->None], 
            PolygonBox[{{4996, 5470, 1459, 1994}, {6502, 7741, 3107, 1118}, {
             7058, 7059, 1729, 752}, {5979, 4688, 710, 751}, {7339, 7338, 435,
              2753}, {5556, 8366, 3734, 435}, {5230, 6550, 1404, 1195}, {4690,
              5982, 751, 711}, {8067, 4398, 434, 2464}, {6729, 4292, 164, 
             2244}, {7794, 7798, 3149, 1195}, {5976, 7790, 3145, 1115}, {7016,
              6319, 434, 2465}, {5558, 8343, 3715, 436}, {4399, 7018, 752, 
             435}, {7797, 6222, 1195, 3149}, {7800, 6728, 2244, 3151}, {5279, 
             7883, 2777, 1266}, {5363, 5971, 1723, 1404}, {6330, 7017, 2465, 
             434}, {6507, 4944, 1949, 2067}, {5143, 6500, 1266, 1115}, {6319, 
             7013, 2463, 434}, {6756, 8368, 3735, 2244}, {6231, 5417, 1436, 
             1895}, {7742, 5196, 3108, 3107}, {6755, 6756, 2244, 164}, {4944, 
             5860, 1664, 1949}, {5418, 4891, 1895, 1436}, {6229, 5745, 5744, 
             6228}, {7791, 5228, 3146, 3145}, {4749, 7011, 2462, 2777}, {7340,
              7341, 2753, 710}, {6329, 5149, 2067, 1949}, {5559, 5558, 436, 
             753}, {7792, 5143, 1115, 3146}, {6840, 6839, 952, 1459}, {7790, 
             5975, 435, 3145}, {8099, 8098, 753, 1731}, {7741, 6501, 436, 
             3107}, {5897, 6837, 6836, 5896}, {7884, 7885, 2464, 1118}, {7795,
              5229, 1195, 1404}, {6371, 5867, 203, 1971}, {7010, 5564, 433, 
             2462}, {4685, 5364, 1404, 1723}, {6226, 4890, 887, 1436}, {5000, 
             6430, 1459, 952}, {4319, 6369, 1971, 203}, {5935, 5936, 711, 
             436}, {5981, 4429, 2500, 1729}, {4429, 5983, 1731, 2500}, {7743, 
             5148, 1118, 3108}, {4397, 5504, 1115, 433}, {5227, 7791, 3145, 
             3734}, {8367, 5232, 3151, 3735}, {7703, 5147, 1118, 2463}, {5869,
              4973, 4972, 5868}, {4889, 6224, 1436, 887}, {5469, 6424, 1994, 
             1459}, {5859, 6328, 1949, 1664}, {5195, 7742, 3107, 3715}},
             VertexColors->None], 
            PolygonBox[{{6333, 7015, 2465, 7017, 6332}, {7796, 4888, 4887, 
             7797, 3149}, {4973, 5861, 1664, 6374, 4972}, {5742, 6229, 6228, 
             6227, 164}, {5566, 4685, 1723, 5972, 5565}, {5149, 6333, 6332, 
             6508, 2067}, {6425, 5897, 5896, 4996, 1994}, {7009, 5566, 5565, 
             7010, 2462}, {6837, 5895, 203, 4247, 6836}, {5745, 6232, 1895, 
             4891, 5744}, {4888, 7800, 3151, 7801, 4887}, {4970, 5869, 5868, 
             6371, 1971}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.5267943150177042, 0.46318626250065026`, 0.8574534542345418], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVE1IVGEUvZk6zph/M2/mvZnRsdn1Q9g2ol2ElUkJVlD5U6mlqVkhbhJa
BC3UoF2UGLjKSgn6pzLGgkxaVxglRJsi+8FqF53j+RaX99737j333HPv/bKH
ext68sxsELYS1h0z+5Ywm8BH2jPzM2b/is2u4JnAv36cr42b7QvMWhD4q8Bs
Ec8MLA6fDGJa8vFMml2A/+uQ2SO8//Dkk8JZA2LbcF5WJOyD8J+oNLsBy0XM
XqTN7lWDR9jsbsos7JsNFZpdxPkk+YHDUWD6wCwFRmu52QxsHPwn8sUp47gR
j/ymEf8B9hE+R4AzHEW9wGnD+xT4XAafReQbWmX2c4XZH+Rdgm3GWS/q6vKE
vQtxT1H/DDCrcPYQsc350qwzJt0G4B84rc5VqH7y2YLz9qTqjmfNYrDaMmkz
WymfB8D7BKxp8KxNCYs49cjZ5fDJ67fj1o//p2A7wfs8zgJPek7FpBe16kTO
47BokTRlHcxVB24j8MnhbD/y9iHvHmjUDYzTwKwDZoC4Q8h9Nk+93eCrvyPQ
YQ6c5yPqTzKpHm0HzpsKzcbnhOaJnKnHGVdLO/A6YPcLxIWcqPn3sHrA9yZg
XPL1To3Ihf0dRY7xsDiwD61p9eKEpz4t44FDxM0M8ciRve73pQs1GcS8vC2X
zrdQx03YTETca9xszwL7ZVq8yHfAF+c55BlDvmFweowcm4rFgf1c73pKjHdO
B/Jlz8n5WaH05kwynvzILVcobM7nRuSp8VXrq0rtAzVaKMV/WK5Is5xz88yZ
5R5Qq23I66HGZsxzCXpUklTvngSaV84qtSRHzgx73hio79Rl2mGGfO0eNRzD
8xrsL3J0pIVFHM5Ctdt37uLJjHgQr8fNEv+xds4kdSJ3asV9Hc2op9SdulCT
xZA4kEtbUjvDeN49a+KaH/KnDoztRZ4djv98Qn1lT8mDs0BNmMfLyp89vx7I
hxgLDqcPPlfdHvE+OJbWnUD9iqPScF1WfvTh2ddA57erUJ+vGrrhn46qd9w3
YlCrevBPZPQ9udrsTlz3UyPi9vqao6SnHaZW3PX37h5e8nTvULfdeB7wpdHz
tOIZu6xfSLptBcaXlLRuwv+GqGaFc0Zf4vwHuo3ddQ==
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlMtLlGEUxk+OOt4adZyZb8axUZfVIrfRrgvkBUmDCrLUUjNNjS4WQYs2
tVEX7aKLi1ZqCUFXCAy1q/QHhJFWRFFoFF120e/wzOIw551zeZ/nOef9ag8N
tQ7mmFkzlos9T5vdiJmNlJitFpotl5l9SJq9x5bw9/K7D2vJN+ugsIv8buxK
2KyNBtcyZlex4ZBZIz2S9Ipz7shVvKLCbHul2WyB2eZis6LAbJzzX84j9Pxc
Y7YtG1+KyP9Fj7kc/feCu56lhe07di5qdpAes9S+w0bAGA7Uy3s2gLU+KayO
+QRYBujXVCJ8qRQYsfM5qpkgdwprB+sMelSTO0D8er44fAXPFzAuR4THteoi
HimQZqece0a9XYOXVWavsBWwZrijgfqnYJzH6vHPgPUPv1uIL3FfB+eeuNmD
PHK482ygHI/5f63w7c3GnXOsFn2xnaVmN8npRJvDWCgkzaa4ewJbdf2KzL7B
bSUprs55E73ryqktFMdp5nNnndkT/BV6HIVbc4XmG4XjeEw74vqPwvE09cOB
ch1jL7EjaennNQfAmqHH67DuS6BNKqZd8PgjsDxMSmuvacVfH9e8usPSurNM
3FzzjwmzT9hkSDXpqDRzrRz/70phciyu6Vbu3kXODLFV+o3iL2SE3TXaH2iH
fHc852RGO+i75zsyBpd78ax+9Gsjv6lcvmt0C21bovLniqRlN3eWFkhTr71Q
rtl7j+P0Pwb/yVztTA+5ac7/ilVzv5p9Jv/HGu1MDO2HEtrddvBf5v6Xae2+
5wQZvUHX03s4FufsXB3TAvjmsvuwCL674Jms0i78zNNbGosq5jP13dodlZ6+
Y/4W+lKavc+nH6zTNap1Dr5bt6vE3fm9BetiQrPxGV0CfymYI9hF/Gp6rY1q
F3wn3sC1JalZ+8z7qW1iho1YH/6eQN8M/1Y8zheX+bR857SR3d9Qq/fmb2CQ
mh1JaeWaOZe6QLvhnP4Dz6u8Dw==
             "],
             VertexColors->None], 
            PolygonBox[{{6370, 5454, 938, 4970, 6369}, {6816, 4320, 4319, 
             4236, 2295}, {4293, 6214, 1887, 6213, 4292}, {5416, 6730, 6729, 
             6728, 886}, {5903, 6431, 1997, 5000, 5902}, {6225, 5739, 5738, 
             4890, 1893}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.5536771621332207, 0.5099282735183706, 0.8952405479543243], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNVEtsjFEUPvo2M02nnX/mn3bGVG0qiBAJGwviEdPQYMGGpq1H1KO1UETS
lnjUs54JMqK1QhqvBWphVUXbNeLZsiraRNKwQeL7fFdicXNPzj33nO9851FR
37S6McvMjuNk46wvNlsYN/s2wWw4bNZVZnYr16wK99SoWQZyf5HZB5y+HLOQ
b1ZUanYqz2xtgdk43ifjBEr1Rv1QyqzHk74G/h+Umx0ulI8hnNqQ2QDuQSfn
AkhWtjBQl/7PhvaMvyGkmxj4ftXZTKswm14h200h6Tfi3go83cjrYr7ZZeBp
9fT3ZInZaV/xjiXNjuLcCJh9hu2KqHI/D7xtCfABouaAh4aY2UrkVQofMfj6
HTQbKdB9OVsx0y7uC9iNQjeEvycQ65SLtQN4yvHfw783iLcPcXKgb4c+DX0l
9LOBYbuLFYmYrcHfXbA5CAwtYcVcBZvXxbIZmWz2BW+zJpp9zZM/xqoL6416
+mMOtH+WIx19Nhcq5jtw3on3AGJ1Q78CcR/GlBdxDadkdwfyXWCthpzJ030o
S3mwD1jravD3KS4Oo/gXx59f4OgS5LlJ8T+Auz8pDsj76aD42o34zb74+wE8
eyC/dFxeD4jPRmD7GBPOxWXiiLkcgb/2pOx6kd/ZoOznQbcceOrRA9eAoc1T
jZ6its8T6rt9uFsSsu/FPYg/o/AxH/ZVsJ8C+8XwsQinJl+HPDLuEmDYAjzN
kDt8zRJ7+ExQOMgt/1Om/8eoSWdKfVoN7A2l4nlbTHVljd5DboffVsj1McWg
f/byTdfP/cB5zlfNiTXXzc4Y3r/i9MCmHL6L3Dzem2R2OyJc5HWv47bS1Y04
yRm5J86xkPqEebK+o663WCtyzJpwnlhX1nQ6/M3w1Res4fcy+WoCfi8i/K+Q
e2NY/jpKVAPing9sBzzF4qxwPtnDtZATJeqxqylx92++eXP2nyRUM3LAGnNH
EE9hifL/u4uKtBto77nepc9cX/3EXjoP+YKvPJkbueHsL4hqHlijDDBcSSlG
JfTL4tqNNxwftN/vKR/mwj7d6fw/iKuurCl7gbNHzoO+9iZxvsX7O2dDXlhb
8nAAPXU/qj3LuyMhmXPGfc1ZG3L7mfuR/7hrWa++sPZ4xu3ocVdrzgBrTJzL
4X+dL/3mqHgnPzN97UHmxd7vSqn3H3na7/S3NK4Y9B/FDvYqtAfT0Ffh1ME+
GlEfswd+FosL5vgH/Ln1nA==
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtVEtIlGEUvfmo0dHGcf5//plxfEwbI3u4qk2LCokykwh6LEJSe5mkBRkh
qEmZRaT2gCgrdVUUpUGUtTUptW1pEeWiRY8RhB6bijqn4+LCnbn3nnvuuff7
EzWNWxtSzGwnLA32OGx2Mmb2yWd2NdWsPGLWiP+a4FcvMAuHzHoLzGqzzGrw
e9gxmywya043K4LdQGwkx2wG8T2wvFyzQ1EzB3in0CiMeGVI2H/8Zhc9s6qg
WUe22df5Zptds4/o+RDY94DXlmd2Nm52O9NsGvUHwGVTRFzIaRfqH+Sb9QDr
3Tyz4/i931Xua+CdQe0Q4t1+YUTQvwQ5vYj9ZQ3w6kPq14r/ShPghpxRiPE+
gDr0H0T8AnJHMsxaMJuD+G+/NFoHLmWwKtQfTRVWpSvuxOyAllFHszL/DnLX
uJr/CuJ9wLoG2w2t9sIeFaI3eqalSpM21N4sUIx6OglpTH8j+G3DLM+QnwS3
Wcx/HbEjsLGAdrTY1c64q15wqsMuDkbVmxxCwHMTwpqAlST0H/3xgLCWJOQT
cxn6FTjSjhq+QO+xPO0uHXg98AcwQ1O29OpGfpcnbimMwz+XK58zTmA/Xbmq
5QzbPc3EfOoZcaQhtaPmddhXLXYwit9b0HMD9HRDugXOtxS1g6ip9IkftRxH
j7eZ6vcDWN9hSfivEI+h93JP98L75Gzu3H45I2/xV1C3wX12Aa8iqFv7hn4L
oWUWMPywbPhJ5KcD7y7wh5H/FP+1B5W7w6fb4Q0Smz2oVSdyin3SrBNcm9Hj
A7jcAucAsDox30tgnQeHftSeAMfnabo53vbpuHJ541PoVQoOK2CT8Asd7Yxa
8A3ybexzNStnnnXEkdz4hlvQezV6LMoSB94m3wTfAm90KCrNWE+NV6L3qrju
i2/mMLh+juj+qcHaiN4Id8mdOoivj2hX3NllTzfB3STBsQG7veTJJ59M+DMR
aUm8dvAdmLt/avAFuyyLCb80Q7v9GdPb54774ffFVEtOY3HNTGzexDSwnjj6
dnEHfOvHPN3G/zcPPtU5wuY37X5I3wTeNr855Y5unr+pWQW4Fruaj5pUoHd9
WLfKm20D1hvYFKwVlg89/Z52yx3/A3dV0f8=
             "],
             VertexColors->None], 
            PolygonBox[{{6219, 4568, 4567, 6217, 1889}, {6192, 6193, 1616, 
             4569, 6191}, {6415, 6414, 6413, 4621, 1677}, {4620, 6380, 1976, 
             6379, 4619}, {4313, 6314, 1943, 6313, 4312}, {6791, 4223, 4222, 
             5851, 1662}},
             VertexColors->None]}]}, 
         {RGBColor[0.5761280077583746, 0.5473395864772734, 0.909779306213958],
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVEtsTVEU3T7tq9fb6n19r319fW41ERIawUANqBKJxifCwETQT9BKWhJG
EikTrz9tfYKKoE00QcOLljIwKYqkjLRqRiKppoaCYmCtrDM4Ofvuu8/ae6+9
zimtPbanaa6ZpbDmYX2Nm+2MmZ2Ec16h2Td8t2ealSfN1mLtDpvtD5nNLDC7
O9+sCN9PYL+ZY9aG78ewX8N+6pm9yzWrwd6PmEH438K/E5gbfcUwdtDFTyLP
rK+8iYhZusjsRqZiDxQoZgtiNseVvxW1NBcrdzvsRpz5Mlf57iDv3gyzw8wN
exR11eUIj3Vmo4ZFwJ9Av3V5Zg3Av2LCSoRVZ3qRWUtSNn0DYeF34mwXeQFW
G3I2BrKnsKax+oDTDn93ob47gdGV1Plu1LstKp6qkbMKdivsj8BIw+5BvZF8
s1eo7QSGsTphtgZrOEu5+11tjah5DCuNXAPg4wHWtZD4G4qLwygwT7uYf774
pf9Jgc7T/9fXvOk/BH8Z7F2oeQn2Cfw7CH8KtfSWqGZy3JZULZwvuSbPldDL
PZzpz1A/jOW8aj3NgDH3w6qfZ6mhQU/4xBjy5CvPRl2e5pVwseyXXF52fG5C
rkcuF3VADZIr1kh+mTsDsd+dbh+C1zJ8ZyH+KOyRYp0h99MJ1ZuJ/48Sin8G
/2dw14c6ej3tjHmR677B2ywwmn1pn/pij+z1Js41R1X/vpD8tGeA/SCqGngn
hhx3Z+C7jTNHPPXOnb38gW82EMc92FeVCp/6uOoJZz3nW6w7xJ4aouprDPvP
QPyex6w6kuKe2uYdoP5552pyNKNxxFb68vN+EJ/2BfB5sURx/L88kJ+1XA9U
zynkTSWFR769iDgkBrFYZzO0lkL/09Dwe9T2K5AeqLUjBdIb8W45zF7sZxFX
D/sS5tKUJ84/wD8RCDsXmlyINRqSFipi0gNnyDeLNfCd2OTeCs6B82DeCtzr
DVjrssXTD8cV34xap9XtOFfvaqOuP+UJvwPYLRHd9QXIfwZ39SVwKxDj5yvH
x8WYW1R6WRFIK9TJykK9a+RlNeJXYe1G/DT24zg7HNIMK+Oa47mk3iDqfwT5
B2LiYRn2HXHdHb7BfHsYfwUxU3l60+qdnqg99kmbuqLuBt1dm/TVJ3sc93Xn
ibk1obeA/nigd4RvxYpCfdNOOz6Jz75/B/qOwa4CTjXs56hlaVzv6n8wC/Hd

             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlElI1HEUx5+Vk8skzTQ64ziOdgoyIgoyqERpjyT00CUsHQ+2kB7qlOTY
gsuMC2aUQlQKCZHUgKXTwUuLFWmn0jpk2cXGrkULHfo83v/w4P3+b/l+3/Zf
G2mqblwmItXICuRqkUilX+RlmkjdKpHmApG+ItPHMkV2FopsR0qzRbYhLdhb
PSLBLJFx7BUBkRre39FrVopMFovcCotMkXwkR6SX3F1ekSWXyDck6hMZwn7C
bT6/0d/y7Ui6yFHiL60RcRETx3eKt5vYnHzT9VsWejb2Wbg/59sbYssCFpsg
Zwf+fX7DW3QZdrffdP12Af7nC6y2x8hF4nc49SmnA7zbwYjBrRPpComcJscI
9d5HlsD6R73naOIEmP30aiP5b5K7np4lif9MTUPkekb9TdTzNU9kAWlE/4Xt
L5J0W80HFT9sXBTzAe853oPYM7CngiLzTj7l0w6fNuR1mtWwHz7rckWOw6cW
PgP4tpDjLth1xJRj94J7drnIPeKnVzOroOVO0MMNcC/xm67fNmObxOc674kM
kQZ4pxNfRe7DLss9CEbEbRiL+Gbm2yx0JtrLVnwa3NbTrSHjpFyq6N8d9F0B
46M+ujs9ftsN3SGdXcxrb53XevxLkHxiX1HzD/T3Hqt3jPhZfYfNdoP3IXKX
hgxLfY7BvzbPeqc7XkmvUgGbnc7wJLYtQatVa/5Jrmmf5X6EfAnbTmj8sOKB
vS9gvdb8ehu6gwmnXp39E5/56sx0dk/p0bDYexT8Lmcf9VsMrt0h260F8nWg
d4bstnTG2qvdAbst7dkLel1WaLeob73FHvxHs+wmz4B3ymez1h72YouHLLf6
JKn3CvVe1pp1J+E2V2y7qnxuh+2G9XZ1xmXwHQV/APwR9iNaYDuvu64c9bbi
zCvl3NtH8kWRVIb5J+D70Ge7pTcS9dg/RGN1pnpbzc496o1NgDUPr0/IOPom
5IPHdk93sA3eFbmWWzGugd/vN+7K4Q/8Z8CbybEbqyfPXmL2IBH0BLZ3YdsV
vTH11ZnrrDWmkl0vd/Jrzf8BKVzNVg==
             "],
             VertexColors->None], 
            PolygonBox[{{5901, 4994, 1990, 6415, 5900}, {5741, 6705, 2231, 
             6707, 5740}, {4869, 5741, 5740, 6193, 1873}, {6376, 4975, 4974, 
             5893, 1675}, {6788, 5814, 176, 4221, 6787}, {6324, 6327, 6326, 
             4603, 628}, {4992, 5901, 5900, 6410, 1988}, {6215, 5731, 5730, 
             4885, 1888}, {5854, 6790, 2281, 6791, 5853}, {6318, 5816, 5815, 
             4941, 1944}, {6726, 6727, 1615, 5736, 6725}, {5858, 6378, 1975, 
             4976, 5857}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.5958931208610523, 0.5790966235320588, 0.9102298899153187], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVVlIlFEUPpnjxhgz4zibzowGPbRQ1FMLSb2ktI0U9RBiaUHSTvQSRlpE
qxVGhlRQQQVFWbY49SBhEQSVRqlRkEUv7UUbLfTQ9/nd6OFy73/Pud859/vO
uX9p7dp5azLMrAVjKMbbiNnLkFlDllmex6zVa5aL+SPGuFyzajhX+8y6ME7A
vy9o9iVhVgP7HJzpzZJPH/b6EzrfHDbbGTAbggBXYX+I8SxDuIdgH8D+CuCs
hn8FvlM5ik37e+zdga3coxgdsK8fqjnt1quAvSGs2FXZZue98n+bKzzG/Qic
TwnZejFfDAr/TJ5yYj6BArNpEWHMAgeZ+K7D2VHYG42Rwn4cuS3EWJOJHIDR
X6LvuhydY9zxiPswR/ea75GN+wejZj1R5ZwCdidiHMZ6eqHZ9pjWT0Oycd2c
NKsvUm6rse9x+TzH+oXTqN4Pfoqlz0zwMBc8pLOkTyosjf74pSv9S73SiRrN
gf1AUv5tyKHXr/3PmK8Dfx32OzBvi2l9D/f9Du4WIFbPMPDm1boM2HeDums5
fGdg3MKdx8eUN3Mui5vdjoqLx8BoCzrtwd9XfF/Cugt3rfJLt6U4Vw7cuYib
xnwF4yj43QIO/sB2FZiXsdeOcSRb/A04Pt9hfc8nHUbAnoEzs+EzAzEnIdYO
aDca8ygMn1f11RKVRtSV9jh42Ipz+WHF2gVubxSplhbD18f6wV3aEasFsTIQ
qxbrk3Fpfw25tEXE2wdgnc0T5j5o1ejXug/zN+RQmSedVwak9SJo4UWMS8hn
M/YbnD8xiEVNJ+JcU5G+qUX3MHHIHqXG1HEWcq8IqL7JNflgTVKrKXHpdT9f
uMRk/KZi5VCCHEYi/1b43MT9BhLCZsyNAcXtxP6EmHyIN/UfJvB/uTph7DMO
n31d6Xp7d0DvAvnkd9z1FO1x5xOIigvykIv1K5/WYyPKj3EHa86jnqIe7Gne
kXXKPJjDb6x/JPRmVIXUM+yX09B3Q0T+y7BfE1K9se76/Vo3IcdzhfJfAnt1
SP1Tm6+3jXmyXsgHNWIOKZcHNaE2zKEyLD3Ixx5wvL9YvA9yGdR58j3c3Wsv
7LuLxR11JRbfP/Zcj9Oa78Fy11+P/Oph6s73+FnJf3/6ssaIQQ64fwx9fxwj
6d5A1gE1qioU7+TZ5+43+Na5HHiXrLB6g33Btzzt3s90ULjErAfGmFJpydp5
E1P9sPZZ0+SK9k1R+VCHUwXS4kFU/xXmybc16XLoxt7PhN595kYNmB/fp2vu
vUo6Pei/FnivgduRrT5udDkvx/wE+xeyxcHkhM79BS+g/gA=
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtIlGEUhk8zOjphMDo6o+PMr0a1yCgMggpS26hUXiiKFoOmBVZ4CWmX
YK0qs6IoiWoTVFBo5ZgztJDICFrYBVOLwIo20kVbFJXhoudw/sUH3/m/8573
Pbe/pKVzZ4dHRPZy0jiJApFjnFimSANn2hGZ5PjTRa5k4RMUWZvPHfugV2Q8
INLOmcC3zC/SHBJpwX6KXecTeUasHDDxDPz5tppYldn4LBMJEC/OvY1v1dy/
gj+Orx9MgvgjXnuryDe8+jQSvwLMQ3xriV8XFukuNDuF3Q02mSuyJ9NyeKR6
IqbviM+4pxzLRTWsIPYU3xqWiuyA4xb8TcTw4/8B/mGw8TzT44XjB9jvqhd7
kBi/uY/Dt1v1Yi/CNwTG4zf9TdhtOdTQZ3qPovcNfI0e+1YDdl/I3krAXywS
ucCZ8Vg+NWC7wExwH/FZ7rVhe9Ocp7MtB9Wu9T4fFTnHiYG/S04B4g+RT32m
mxNcKfyv438VfT/RvyFq+Sfgf879NJxLeJsnpx7iL1DfjjSL+Rl8JzVKgu/x
WW/mHauF9miAWo3R/3V+w2fBfSJotdAajoIfBnPN5Z/Ftz9gtdX3BLXzhc1f
azgJfzxksbRmvehrR98LencHzTPF5FRs2pvT7e1M1HJXn1VwjQZsXmvh3IzW
KmpyknyK+PaY3J4U2uxpzod4G3Ntnbc57EbwN9iNOfhb0bKGmPUZNv+l3LeH
7K7f1qO/LGK2ct4D/y1iu9GfZbVe6fZLa/4lYjOgvdea3YyJ3A7a7OnMbkLv
Rse4VfNl6nmJ0+W1GczGt5x4u9Ltm+5iMmSzrvh3vC1mW6/uwzmL/TFktvaw
Cv79IZsd3dlfjs20zrLq24Ke8pjtrnIsp7/eoNVS8ztFvfvC1h+t3x+wL8kn
BT7JGYHvfch6rTP3Cu2vC6xXqnea3qVyrRe6s38dy0nfNMYnsK1B066aF3j/
51hs3bnKfOtZtTt/2rvD2DXu/yTB+9Y849eZq0drQ9h2SXuqu6H/PP3X6Y54
4Drg1kN79IBYbx3rnf4D9V/T5O6r7p/u5rYc20fd0XZ8e939UQ26O6WO/et0
h/ro/9moO7v0tAJtg2gc4JRz/w+bEMnL
             "],
             VertexColors->None], 
            PolygonBox[{{5892, 6375, 1974, 6376, 5891}, {6317, 4593, 1648, 
             6318, 6316}, {4245, 5892, 5891, 6834, 2306}, {4602, 6325, 1948, 
             6324, 4601}, {4194, 5734, 5733, 6727, 2243}, {6361, 4602, 4601, 
             6360, 1967}, {6321, 5439, 5438, 6322, 1947}, {5734, 6186, 1871, 
             6188, 5733}},
             VertexColors->None]}]}, 
         {RGBColor[0.61565823396373, 0.6108536605868442, 0.9106804736166794], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1lW9olmUUxk+bc9vbu3Lbu+f9Z89rQRQGqw8FaR9WglZOVCIpdEwb5Yyk
pQ6EWoGVgdTH/BIlQe7VDxq8+a8/hltQ2CqKjEij5ZeIoKwICqKo38V1++Hm
3Pe5z33Odc65nvNcOzZx3+NtEXGA1c4a7I94qBqxl8MblYjfeiNkMLYIfRax
H5v5JRHfsT69KuKzq5Gsadbv2B6t2P4G5I2stZ0RX/LulVrEYXyexk+1Yj8L
ykjOc+wfxqaN2FuxOcH93/h6emHEFHI1djV8vovN69ztQL8T3T19EWfQ5+g+
wW4F+o86Is4h59G3iPkPb/ZxvhMZyX9WiLiArGIzMhBxDLuT7da9xN0M+icW
E2exz2/zfrDb9x1djqG41xP3Nmzu5/xIv/NUjm9RvxbrgQURK4sRt/LmMPsh
7LfnEb9eEbGhHvFaiZqhP4mcwj7Hz8vI/VW/eZO3R4p+KwyKL2yKr/yE4amq
4ylWgXpcrDhf9aOry/5Vv0sl13AjcdeU7Uf+dne4DqP04ZeSz5djK+7qPtda
sZbie2viwPHMXBFPZvF5hvVTt7lzc9JvI9eZpN97pWupmD9W3Hv1fTrxR/gl
xSfVoZn2ul+P7bqya7AOf0t7XYMnkaN1498Ink2sYWyG6Gkr8XA5eJaxfoCH
m5HT/d6LR+9l5lI/+l7WWfTn8DlZtp9dPRGHiL8TuafhnqmewnYo4Zwjxz7q
VqBm32fOTXnVkPdm5qS4fL5iPr8K185n3h9D937CsJ06b87c1xHkV7xZs9D9
/DdxeHjAPBPHfhaPcvNDWJoJpzjeGDDP1f/pVNtB1Zy79Z2uazPh34OuSO4f
dvq99MrxUm6+iqvivnih3lXQbSuZG83L3z37HBwN1rfYP1g3z8SxFeT7Mf4f
A8uLcGw+fe9foPucNYH+CLNkPPmUb521P8X+a/YfMGeOIh/N/T2pL9IJv96L
r1vY7+gxDmFQj5W/cr+Qeaao5qPEfLbhu2H0N1XcI80t8UacUR+m6u6FuPt8
j+vwX5/niGrbTt2uw36c/XH8nWjYTjUUF4RBUmfVZ7LHuKUXRmGdwfdst/fS
S+os/a7kS/rniNVW9owS17ZU7VszZlHJObbwOVt0Hcaxv5hmkebTH7lnlHrQ
SjYzBcfRtyP+KJ7qKa4dqJlvmu+y11t9KwfT7FVs4RQ29Xwy4dwEnmLJM3J3
iiH/mhn6f2huqFcrEwbNpPHcc2mImHfVrG+C/+A1ES8wM/ZR87Pk8E3Bc1tz
RDz8i/78WbdO8344zXx9c3nq6TMDnheaFern2nL6LxRdC+Wlmk0k/oinc4mr
qs2SovGvAufy3Llqfo/0mg+riHVLzRwQT4RRuYyheyfzDLwd+2W5e343fu7I
7fd/U98PZg==
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlE1IlGEQxydzV103W3d19313a1eDoOzr0iHrEAnZx4pKFEWZmURbFJko
RGmBUYFot7xIX6QHDxVJZh8U7gaFmFAQRQZZlw5BZRbUJajfMHsYeJ7nnfnP
f/4z85Y3t2w7liMiO7Bc7FtcxHFEhrjc9IuMLxDpKBaJ4vTOJ/IwLDLLXYPO
zxWpiYhsCYqMcU9jlcRXl4gkiI1jOSGRxeBt94gcxH8sJtIdNKwvBSK1xG/N
xk9jTa7IRXwyfPPlw4n4JDmrvCL1eSK/+XZjIbkLRcJgpMh1q0xko984K5c9
YHjyDbOScyOWz/0z8XXkywREVoL/AMx27keDljvJvT8qchk7zXkIvn3EjpPj
NvirwVgFnwHyXyi0+l1qizrGr0/1w/ckeGm4naDmRvj+iRlXlxxB8BpKRUaJ
HYdPCr0OxU0L1biT78vLRSbnox/xf6m9i5zP8O32GnYveOkCyzFD7HdsZo71
S7U5Ts96faZRD76b4FTmt5hC8H1Bw1LMdcSeKjZu6uPH9wz8nntMwwa+ZWLG
T3NMUWsqZNqoRlfRqgc9J8QwD4O3nrcmj/Uk4thMKTeN30zuZMRmRfuzn/oO
hG02VD+dtQbuNV7TQ7XogPPOXNNEZ6MnW7/OSAKujwMWqxg5YIdc01Y1XkL+
avBSnGt5C1DfPtdmQTG1t1+z/LTHU/jeIaY/z+qbwLc8O79H0OMK/hNxm81Z
4j/h/zFs9asmb5nF0RLr3dMikZ/4vuGeAXsY24tvZ3aftEbN9Yi3Vq/lfMK5
LmRnfdPz+7Bx0bcKYtuo8bXXeqTc/gWtduW4CD2WOlar1jzIecCxfVW8awmR
+yXGrW0ec492yVLjrjXobjQGbHZ1R36QbyRss64Y9xLWU+3lOeKXca5wTH/d
AZ3NNXHbfe3R9YTl1FztWJrvIwmL1Zl6ib6XXNutFmyQ+W2OWj7dsV9gTaPp
ZJHV9wI+Kxz7F2g+naX6iO2yztRZsLsS1hvdedVqd9h2WzVrZTd2xcxXd2Rt
3DgqN/1nDYN91zGttN4NcKmKWu91pvvQZjhqs656fSizHmtvlaP2+pVrtWjP
/wNQ/8SC
             "],
             VertexColors->None], 
            PolygonBox[{{6189, 5409, 5408, 6190, 1872}, {5467, 6411, 1989, 
             6412, 5466}, {7587, 6261, 6260, 5030, 2950}, {6261, 6458, 2021, 
             5029, 6260}, {5409, 6183, 1870, 6185, 5408}, {5551, 5725, 1612, 
             4564, 5550}, {6417, 5467, 5466, 6418, 1992}, {5723, 5551, 5550, 
             4563, 1611}},
             VertexColors->None]}]}, 
         {RGBColor[0.6354233470664076, 0.6426106976416296, 0.91113105731804], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVEtLVAEUPr7SmcyccR53nNFRXBRWtK6g0Eqx3GRWlk8iM3uZSFAiUS0i
FF2kpYkGqWVYpoi2kDBrl62CKAqqP1DYJttE9n1+d3G45557zne+87r5J1sq
Liaa2U1IEqQj0+wjZAF6tmMWgZSmmv3MNQtDL4FekG22P2RWtM7saZ5ZI74V
p5tNBc2qYV9OMeuNmP31y9/CZml474T/WMzsCuw/kDQG/xxg9sO/GJhFkNdp
ZrPA+OUza4H/ddhmQtLL4f8V/s/gn0xeLoeqqFkrcAc8ZkPwH4T8Bs565KxG
TAdyjeaYbc8yawef74hthj4DvQKxLYjtQ+xywOwC+QG/AfZ6yLBHMgSZTjDb
l67v48nABn4ZankBDvfwvXKD2XP43Iijn3G9s77coGqsDalH7M8fYNdFhZsF
nMSw+pOJXrYh5mqquPZ7hFkKbl3wv428l8BxBPV8SVDNg5CpBPG47+o9XrNX
XvV5HP5PYno/DAwfsKqS5cPvxOH89kKOwF4DPlt9qpezGo3JrwaxIzHlY65i
1/8xbI8gC8RHzR/wnExQntqIcnV7xZW5TsF+Olf5LjuaDWtlfd1uje3Ivws+
Qa9qXYkqlv2ijdzIibi0cy/KgtqNg3h+y5Re4GjP2H/uC3eXO7MJ9nzYjycK
s8fF4fzrMzXj6qhqZr1nweUcpASc38Dnll91km9DRJzZs8KwuOUDfwn2A6ir
1dFcOdMg9NMB7U8l8sxD+kzPjY70VfBMdu+FtRKbmLy/ubhuir3nDDhr3usD
Rzd7BnmacrWn70O6Z9pnEbcYVV7iVbmYjqPbJuZbPIfArROx84iddjH/odY4
+wufPehfM/wWcV+rftXJGrvIAd9WYO9K1z0wB+/PF9ENlgBzs6P+18G2I6Ld
XgLecEBx1+CfHtBeef264bX7RdwY+4NcS+hzkqPZ8f7Kw+oF7/hQVLe8ds8p
2kPO805EM2WPd4fUZ97KgHsvRxF3LKp7m8Q/rQkc7maYpSLO4xeHXo/2m5gd
Pt0zc+yMaI+5w7yBbveuOENz7/oEfNvCiuXebXP5T0CfyBOHuZD+d/zXNQa0
K5wX58Hc7An/j+w9+XPfP7n/Q2IX+oRP7Hc+4T+MKQfxWU9fhuJfoidx9y7O
w3+LT7PgTrN3xOH8P4e0A/8B86zaIQ==
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlMlLlHEYxx/N0kzDGZ1pXmcYp1NhtFwrKG1RMqFyy6WdtJUWgkiRaIEI
pQ5lu0JlG6UVol4KWy6RnVoudaj+gKgu2aWoz5dnDg88z/t9nu+z/t6Z2/bX
7Ms0s2YkCymdYbYlbtY31exxhtkE+m+kF/sTdjV4M/YV7NEpZnlFZi2B6/UQ
NIG1IFfBLyM3E2Y3kEcZ/u1zgVk3MpFjNjDZbD2+e2NmD8Evga/BPhpxrvPY
B4k9kPDc4qsDr0UawS9iHyK2kBqmwdeebVYeNcsopBGkLOrYYXw6st1HekPc
c+nbtWKzt/iNEfILfFfSbDBldmG62XPsMvA66u0xt1fi24A9il5Oz8/h2knM
ijyzu9S0k3wxclTAXcG3+dSxNfDcypcP/hr/rjzvcRzsdeD8VeD36PUW8o2l
PMs1+xky64evB2w/+ebBNxx1Xf3eAHuK/QS5jt4G/h79HdKK3kf91RGftWbe
Rv7zgdenesPY49TTnecz3Qi2IfDdy+4Dr4/7rOVTHSYu4bWdRarIMZKuRzVm
ch8n+fYKvYtvi+GuhOM0XBH8K4n/gX2EeobBe9HXxn3Xmskp8BrsHuwP+A+k
fKaaZQ87OVFitinwe6nLN/tI7t5i35122Eq+7fgvx38ZUhrymxL/IDFzQn5D
6kccO4o8h7i184VwLwp8V8LV+/ESz6WYJeRLxXxX2tkb+JLYq7Cb2Nkf6t+d
9Ho1Y/V2jPjafK8hBHdh4LPpzPTdL6P+FznOt4fYucVejzimwvcF/qFs9/+K
nht2Xd/akn5TuiX1rLfZGPe3ojdaDtfSYr9d5Vgd9jcq/Eyu5ypL48o5Qq3D
JX4bqv8c9bUWua2d7SDf9/T+dPPq7WXCd6UeA90j7+MSeCXxk7CzYv5WNJ9N
5O4I+S3on3Kf2T9I+a70j7id8BpV2xjSj30n4brexMa43+DZdPyWwG9Yt6ob
W0AvFVHPpzdaQu7xkOfWjvQW9Yba0/+DJG/jb9hrVc054NmBz1c7quKeO4n/
kb5X9a5v0jWD2fDPijm/craQuznqt6aYwYjfvPxlF8D9L+rceiPr4u4jTDc/
BHYw5vXpTW2Oe8/qVf/k/5DgxYM=
             "],
             VertexColors->None], 
            PolygonBox[{{4861, 5706, 5705, 6170, 1863}, {5843, 6811, 2293, 
             6810, 5842}, {4865, 6179, 1431, 5407, 4864}, {5820, 4938, 1940, 
             6308, 5819}, {6176, 5697, 5696, 4863, 1866}, {4969, 5886, 5885, 
             6368, 1970}, {5706, 4862, 1865, 6174, 5705}, {6302, 6305, 6304, 
             5818, 190}, {5886, 6813, 2294, 6814, 5885}, {6359, 4966, 4965, 
             5451, 1452}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.6551884601690853, 0.6743677346964151, 0.9115816410194008], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtlVuI1VUUxpcdPWcuzDhz5pwzM57jmbHAIMnAoAetNFEqx4dSMbwMisxM
WqJJIISE6UPW+KZJ4CWwi11HCS8wOiraDXuaIUOhsCcdIVHfapqc+n18+2Hx
33vtdfnWt9be/xkbti7b8lBEHEAyyJZKxGbk/bqINeWIVcglDM6zPzMl4gY2
PcWI3qL3ufaIGuS7XMSn+H1Rse2JaRFjpYj3shF/tEXU5r0eRC7WRvxJzDv4
n6vzehTdSEPEhPSFiO+J08Z6GfnfzVvfzXodcpn1QeIcwmf15IgfsD2H/hb7
IcXLGPNSMB4Bx5fsX+L8MHHvYz/eGXGW/U3sv2b9FbK/MaIMzgryArU8wndS
S0QX6xnEeBgZqInYA5Z7Hca6oiliAfpT6Hfy3cH+Fuuu5oj+susM4iyChwXg
baxGTEXG6s3Nb23m5NHWiGLBtayFy24kR5yN5NrG2UjWdQ03mP95+M4mX2/G
uMXjMXz3YvMjNheo/TKcvNPhereTp4X4q1j317m3ssnW2O/upIj5xLwC7vlZ
6xRT/NeBt5Cw9af44u0J8i9MdQ22u2ey/4vaDnC+EvsM+rfh8BIcTlDLkpIx
K7Z6LnvZbs/Z/jnOO9uMIU/eZuSfevf6bNZ9b+G8IE7xebLG356M+3+ywzMg
XOJd2HrA1Vt1TcuxWVF2TvXt8dS7B2CrFo3trWbXI2w3iPE78q3w0dvFxN1D
6F347UZG8Z1At6nFM/Yzvh0p7wDxXi6b56ewGc8bp+b8fuL8A80L8isxf6Jf
g2kmn8/bX77yKxftqzmYW+sZOIPfg1bj6aXGfWCeA57/yNXU7pgb+LbnzZ9m
QXdAuXUHxb/uyBKwD+HzIfEvcH4euU2OF9F/k/TqgXqk+TlKfR8h+3Lup7gT
b+J1VrO5Xc/6sVbnXVn2/Mn3Nfrwb+KhAcx9Vc+g+vB61b14A79XsRnOetb6
06yqf+qteJuK77ppnhlxpvss3g6h+wVMO/D9HD6X5j0zegPUJ/VoZmOqP+P7
pLdKfB7F/uOK36PhdC79ADwcn+69ZCjZH8P2k4r3egcVR7mEUVg1P03UPQaG
hXD1NBjndTp/HzGvcXYc/UbW1xO2kZJrEH7df90l1TgZ2wwyF/su4n1Wce/E
j+68ah8v+c3VeyJuGgv2vVI11+JZvJ6Y7vqeKbrH6q/u3LMl3zu998qh+KpX
XGmtuVEM8T8Lv/qqZ+Jven41cX6v4NlVv06XPFuKL1/Nvd5w3WPxJa701UyL
T83caOqv3nXVrzdnPzPcV/Bd3wzOV8rOq/fszTRXuhPDKc4a8OxtdRz5K454
0Bumd1Y68SXehEcz0p3+Kf8DUt4Puw==
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEtIlGEUho9pM17wMuPM+M/842gGLVIKClpkZYVBYBuNjNIoRMVumgVR
FFibrFyVRFBB2MVK0yhLqNTQbthK6UIQFQRpYGTQoqywnsNx8cP3feec933P
7Z9T1VBWP0tENvAl8PWERPY6Io18dzgX5oqUZonc94jMSxPJ4X3aL1ITL7KS
tz5XpJn7aKrIeJJIv2tvev7CdxH/5UGRs/if9oqcxNbJfQLSR0mGXeKzs3LE
ZYqsgreIcwn+aQGR5JjIZsSV8/k45+OzH9vvFJG/cLeGRRYnilTD0RAVKYNj
GvyWZJECsCuxe7GfhyMIXhX3cx7DbIF/wDWt3+JEHnD+FEEnsZ95K8Nejd7e
2diIuQz+AThHyLcPnw7q04H9K/ZW6pOEzQv+Y/Qdx39+lmlQbuXsyRGp8BnX
GG9hv3EottrfkvsHanab+BvkUwN2ndbXYxquwt8eNe5BcnyP77qg+WpME7X5
EzJu1ZCAln/cX9PbQa/lVk/8iWTL8Sa2Jt7GEq1Hw2jLc6z22oMg5xpqtgh7
MW/Xib3GNwR3P/F9xLc51luN11q0YX+YbDVZg30u+F3EN6NhO/o+OqZPNW+k
3uV8mxKsp5eIrXRtPhTjGffnUeuF9nQBWAsjNns6g8eozRD27wlmjwf7hc/s
S8FLZV78MZsV5cjHtssxrjNg5oHVm2HaVKOPXNe7ZvtJzD58M4ifmonfyXlH
zGbtLv61AauR1kZn0CX/gojZ1OcI53shq43W+Af3PfR7RPXBMYX+opl5Xw3G
BfCGwfckWk41nDtzbbbauRfjO+m33DTHKNyFIavFWt7SY5azalXNW8KWk+ai
O/CEWj2N2uw51OslsbX0pJvYQ+DVam4B41INKdzTA7Z7WrPJHOuJ1lp3Xnf5
oM+wtGdv0PPLZ1iK2Z0tsizX/h0TaHyHvS7Teq8zsAL+dMdy1xqcQm/mTD11
5kpd22nd5QHwD9Or0ZDhj2Mf4x4fttnWGdfZqXBtN1SP7uqVqM2q7uwrYo9G
LFYxAnAtCVmvlHMrsdtc20Wd8Vvo78o07dqz3WBNBkyL7lCM/LMdq732oDHL
eqy91Z39D3JPyPs=
             "],
             VertexColors->None], 
            PolygonBox[{{6364, 5450, 5449, 6365, 1969}, {5450, 6358, 1966, 
             6359, 5449}, {4989, 5889, 5888, 6404, 1985}, {5405, 6171, 1864, 
             6173, 5404}, {6178, 5405, 5404, 6179, 1867}, {6700, 4283, 4282, 
             4180, 2228}, {6786, 4311, 4310, 4220, 2279}, {6258, 7694, 3065, 
             7696, 6257}, {4311, 6303, 1938, 6302, 4310}, {5704, 6689, 2221, 
             6691, 5703}, {4847, 5704, 5703, 6148, 1850}, {4283, 6160, 1858, 
             6159, 4282}, {5889, 4991, 1987, 6408, 5888}, {4219, 5811, 5810, 
             6784, 2278}, {7276, 4192, 4191, 7277, 2697}, {4192, 6545, 2098, 
             6546, 4191}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.6749535732717631, 0.7061247717512006, 0.9120322247207614], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtlVuIlWUUhpd7HB3nvLd7z79n9t5zEJIapRNl5Yx2kUFkXShINUFMViBk
aklWdJAc7KDYgYHQq7pIJGzGySGLzkJpB0ILZ1KyLgq8CNKLKC0re97e7+Lj
//7vX+td73rXWt/ft3r9ynW5iNjDqmP9VYi4oiPiHl6G2zhj/Toj4uc5ETPT
fqox4lvWhZxtvmu0c2t3xEPliH28DoLR0xWxdFbEjqaIZ2ZGnMB3nLPRdgzA
W9gb0c/qbDV2fcJ/tBLxCOutOY6vGML/Eezj4D4B5hR+x1gH8W0kbmsx4nfi
fITf9iziFDYbeP5GvF/w3cX7CfiO5Mx3OMVSXOXWz7oK7BZiDGI7zf6bsmPN
AOfJuRGTdYlri+0/INaHFXNbw/fvsZ+Y7XgTfD+e834/+0PE2t3ovXjcW4pY
mZnPu2C8U0k8wHmO2CN11kNcxVO8DufN7RzPad434nt5LSIDa7Q+Yh5nX/Dt
ds6X8HwYzHEwp8HYVO96TYL9eLtr9AZ+S/PmJn3GE7d+NLsu9YBqMJH434z9
yRR3Ed+LSasx9nfj8wD7Q9WIMz3mfZ7zzUm3M/TVMt6H2K9qICf41Fib8Xuz
w75HeC7Hfic2fZzflOyPEuds3nG/Lrs2qkuBurewzjW5T1SHf3LuX+Ug/upH
5ax8d6HVgl7rfBq8DWXHXUEdBnrdn1eDd2PR/boWnb5KWi3P20424i3+2h8l
lz+aXK/n+T6Ud73awGjq9jfFU9+oZy7C5gixdhL3NvSc1YnGYN0F3/vbjXdr
g+uh3tDsvNDkXpogj1uSPnfgu7Vm3wWc5bvNQbx3JPvt1OHizFjKdSzp/Hfm
+dRsjuG7r+YYmhn1iDT8CT7vVcx5K7Xblvn8fTgu5NsrxN0Lh20Va9Hc4Jz+
ny9stmCzu959Ld2lwVSH66faXcb3S1lvNxivUjJmc9lzqL7SfHQWjKne1Pyo
P+9E40lyG2EOH8ubhzicRIMfWK81R6xG11LRvNSDmmH14bqqc1Ne7WXno7ji
Lt0Va36DMVRn9aN0l+bqR/XxUNJZduqBp8E7lmwOVz0D6v8+zk4n+y1wHITr
l2BWiFsreU7VC/Mzx/8Ev4NV362audcbXUfdYQ9mvksGwKyWXTt9F55sdK8M
t3hOX0Sbl3r8Phfbl9FiNFxbYauOiq/7QhyG0WBt1TY5zkrpPvmXmBdY1842
90Xdjres6DmRBgfwnddl+4+r7hXhP4VuZzus+SA5jtfcl7q3dI9sTDrX0hwJ
V3jKRTPdluZ6DXj3lVz368H7s+C51qyrXxVrj+6hkvtN99yqhLsYDQZ6fF9L
A+WpHPVfWpz6/5LMc6IZke9U+q9Jf+ktPV8tu6eFv6Tkd+272BfK7h/NouZZ
86h/0qZ0v0lL3dHS5xpyuqHovOvgc6rd9p+j7fnMtdif4ipf3c3rM3P4DG0/
rfouV+8qJ+UiDTQH4nxlyf8P3QPPFvx/kk7/AXMdIok=
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEuMlFUQhWummXGme5x+TXf//WCYMREVjAsXIDDIQlgYdGNCohAQESMK
gsYAEl+BSQAhgonG4M4YiDHqMEA0IuEVefhYiAYEFNyQsCCCK+XREPyOpxc3
+euvuqdOnaq6g4tWPLG8PSIWc8ZwvihHPFqMeP+OiJc6I6aNi8gk2PLfGTFx
IOJ+TrU34kJ3xK1KxNc5/B0Rk7kzAd/UcY5VzAM1MMB8GKyvuiJO8n0tH/Eq
SbelIv4mdkMBcL5PpyMqpYjLBWMJ86++iL3ViDncXYU9hP+TxPx2YG+qR8wG
72wb97A3w2ce9jD5L5Kz2B9xN/+28d3MRBzHV+b+LO4P8a8E/j0tfw85+rDT
3Pkn4/g14G+F40LwdlPvSuwX4bcDriPYP1F7D/HXiN0Z5romby7ifBf1D3Lm
plzPOr7X1ly7+K8C7wXwtoM3Ct5U9JmTs1bS7D1yf0bNx9rM4Rzc/+S80WkN
e+H7NHgnusx5D/H7wLzdbg1G8T3F/V3kagP/NrUOls3nITQYj/05GI93WYOZ
+CKxTzF5aiu06hvGfh1ub9c8G6pXWtbA/7DDmp4n9g/OWOyPe+g98zDEeTfj
HnWiz1vM156UOW0k34IWf+E/B9ZizjD8z3Ka8LtZsRZPYv9OvlNl1z9Kvvnw
+a7Bv7Tn4xVilxR89xJ4c+nFN3XP6mXy7xwb8Tz+M/i3ZKz9OvL92OEejOD/
smifOD/G96eJe6WenSH3kqJzi4N2oyfxLGlHuvs9c5od9eMXfCcS90rxVfIt
qnrWVOMstJ9M/Hjs9SzgJL5n9vlbnA5T20HOLWJ/Tbu3R7F/S7vHK6j3Nf7t
6vaMnBywJtLiMPt3nXy5xP15NuVZ14yLn/o9hXq+z1tb+Y80vHO6rxwF7t4o
2CeMPuxi6z1QzQ2+64nvq0d6O6aXrJd29CrYP5etnTS4CLcPcp5F9f8Rcp3r
96xIg6U5ayjtNBPq7aGGa1ePfwBvRtl8lDNLbG+/Z1Ez+ib5U1Vja8a0G1ly
/NvC06zfhwYfdXrmtcvace22djoD1sSiZ1F39uI73vDb1JGlHnKP1NRo42vW
Tw1Ya8283qYree+H6r8Cfnvi3dc+NbEnla2d9kVvx4Ml5xP/ZeRazpnQbY1X
1/1G6G1Qj58h97Kc6/1/B7GbZXMRp03UdgM+qaxn5lvuNrHHZI15L/7peb9d
0mA//gN1v5V6gxWrmlWr7gjrnYp3R5j1kmsQd82DdunlindNO/Uf0qDmzw==

             "],
             VertexColors->None], 
            PolygonBox[{{4181, 5699, 5698, 6703, 2229}, {5882, 6362, 1968, 
             6363, 5881}, {4243, 5882, 5881, 6831, 2304}, {5699, 6142, 1848, 
             6144, 5698}},
             VertexColors->None]}]}, 
         {RGBColor[0.6947186863744408, 0.737881808805986, 0.912482808422122], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNlUtsVHUUxg+vlrbDOFNm5uJ0ZjpdiBYsRjBgwRpNFTesJMSgFAOEhUXt
RhRdYCyIlsRofIDGxDYqEFJ8JAgYKAIKPiKBNkpNoCSSaDQh4EaFVKL+Ps9Z
uPjn3jn3PL7zne/8p2V1z/2PTzSztzmTOKcLZleyZs/UmD3SaPZkk9k5HM5y
BurNRnmeqWKrmPWnzI6WzL7gvErsE/i2Ed9O7K682T5+X8P/5AyzVdPNdtfi
32x2jPP6NLPrUv5M8BnMmc1MzKZMMfuZ+O/qPXZ20WxpxmzvVLPnqLO20TGo
ZlfG664n5r16x1gHrnXYB7B/SOwV8LxAvirvS7DvIU+Ruq11Zs9PNrtArx3g
68LnUb6vwm8Enx2THYcwzKOXWzkD1DmL7zg5v4m6qin8Zwr+TfY5ZbNbOLMa
zH4j/3bsj+EzDm9bE8+7EA5m8fsHSP8TW5X4udH3SPTeS4+bSs67clzKep5+
cu+BrxrqT4WPq9HjXDCmKo6zE9sy+tkCnt68z1JzfJ857G3yuivp9cWC995D
/ja+bWBGn/L9QWr9BEcvgfPGFnyZUwGfr7A/QJ7fwfxlvPcHn5rFj+S8UPTf
v/L8hdPG+wLq3M7poNbXxLXH+wryHkx7Lxvh665ax/Yute7Ie7/qY2HBe5Ee
nwpNClM3sWnq9/H9zqL3cjjtOaWl5eT4GPsn5GxKuTY192G4/wAOP0s7vkNp
18xpvn9U53NdTI+LwHp8gtmB680u5nzm4mNezuvfTJ7ZnCLxZepUOIPop4O4
m4g/McExSuvCKc1+X3WuLjGTBHzLwba50Xfmv7nDw9D/cA1FL9LdaxnXmPpT
n/Ldiu182KXlvpjpUPgodhjbqYLvdQvPfxLX2364XJNzvavvi6Fp7dNA7JQ4
E3fiYR91/kh8FuWU28VnN/jXk2sM/wYwLGr2WWq+XdOcB+lJ2lct8S675n4i
63qSlnYQu5PzBrGpGZ5LebQfR7KOWXEPR86XqfN01JVtZdjfZEe2l11Tl7Ou
J2lJM9AskuirGPg3ov0aZvw5tf5OfH/Uo7jZVnbMG6jTk7hGxiKX8hyMOagX
1ZddGK7h+xdnPjknUv8yHLXX+h12X+L3WDHve6Xex7E38t6Jz1Xe07zfXev5
VE85bwNPa8V70Gw0L2G4B/t87KWYSSnmpT0R3tHAqXlID4dDV8J9hDvmWMnv
ndbE72L56F55q+h3yxY42MxZh39f4neZZjlGzc6cc6j9eCjjO7Kbvl5pdnzv
EDen6rW0J882+I4vIPbenOtI99a3Gb+7NE/pST3fQK3pFd/NtqrffeJZPan/
0eBgLO7hYTgb4QzC2y6eS5jrNt535n3HtF/KVxc5dU/1hwakqxWhyV7qbOJ0
N/h/2vGs60WcHYrd1H/g+fgfPBdYxPm/XgELOA==
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlElsjWEUhk/RAe3Ve7mDv/f21oJozbMaGlLDwo6FmEosFbXQEJKKSs1T
zCxoQg0xLSgSiZYYY2jFtEBCQkhMK7ogeN6cuzjJ//3fGd7znvc7fRbXzFze
ycyqsC5YR8ysrqfZrVyzTTlmU3uZjSo2K8o3S2HPS8xeYMf43oP/kJRZJDCb
09msrLvZld5mV7HabLM4iQfjOwjL4VzP/VtyvcEauniOxwnyUfMBuc5Q8zz1
tqfNFhSY3QhhSbOTUbM/5HrazWx82GxlkVk52G5nmf2Mm/3ChnPeSI55heQm
5ng3x/CE3E3U2E/uNfhUEH+GfF+4u9jVrD+x26g3n3p3yPeeXpqi3ltpV/8W
BtXWv5347sCq8F+InSb3D3IuA98Baoykt3eBxwb018FdOz5nM/W3Uq8hYvaM
fB85V9LvmGLnVpyMA2+a+LN53s8I7uvgb0nIa2bDbV7EZ6MZ6W4kPj3y3ecS
3BxiJnVw/YqZNPA9hx4u813NvxBY7oBpNngnE/8PPH/jXkscbsB/bcr9l+Af
wT+Iun8l/r/xvU/8RHxHc74d9p7Vq2ZygNgj2Dpim+inDC0NJ74RvmtzXUtT
wJvMd03NZ16f6XcQfDWCoRn8V4o8VjMfjf+EsM9aMbPwHxPz+pfgKE7uoSnX
njQ4gfOphM9CM6nm+z3xC3K8/l2+wwnvRT0dpvbWQtefOFWtenK87uQ1V4Fl
NdaW4/++cV9D/OpM/rmct8R9lh/ooYVzFvfluc6nuNyYcu7F6bCoYxIWcbI5
5hzpLIwH8e1gvpNyfYafwHc05W9HHN0jf0HCZyf8b8H+KOFvRz2I+xM9ffaa
wWHOVzN86lwROOfiWvXGUv9rxGcpTm+i9daka12al1ZbMvqXZsuFN3Du5V/M
d3Oh60ea3YDveuwl2BrxXwG275n3Ic701krhKzvb39wi4kdk9PGU+IfkWlro
3/qnt1yf9Fx60/2IbUPz10Oe4xxaai/xXaEdo92hN6q3qR0yAy7aYv72tAPG
czcu7fyIY+2uEuxcnuunmtpVgXMjjsrIPRALQh5Tih5bid9X4G9uQIlzLG7l
s5f4C4FrSfMQ1r5xzyXMtcxiYth3zRs4qYn7TtIuksYWg7c9g1c7Yzr30+Ku
RXF6M+07VrtVGHZz3pX2XNpp35lXn5hrTxrU7K8VuTalgf8BjtYh
             "],
             VertexColors->None], 
            PolygonBox[{{5809, 6312, 1942, 4940, 5808}, {4844, 5693, 5692, 
             6140, 1846}, {4909, 5806, 5805, 6256, 1912}, {4961, 5847, 5846, 
             6349, 1961}, {5398, 7470, 2858, 7472, 5397}, {5847, 4962, 1963, 
             6353, 5846}, {5682, 6699, 2227, 6698, 5681}, {4855, 5685, 5684, 
             6156, 1856}, {5464, 6405, 1986, 6406, 5463}, {6145, 5398, 5397, 
             6146, 1849}, {5693, 6685, 2219, 6687, 5692}, {5685, 4856, 1857, 
             6157, 5684}, {5806, 6752, 2257, 6753, 5805}, {6310, 5809, 5808, 
             4939, 1941}, {6161, 5682, 5681, 4858, 1859}, {7556, 5464, 5463, 
             7557, 2923}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.7144837994771184, 0.7696388458607714, 0.9129333921234827], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVUtsVVUU3Vja99oSePf9373vvpYBA6AtwoNoADUGqaTApBAsnwEtAiYg
DCTG0CqBV0gkYAINCkTUBi3/T4GikQCKlTAgyKDhEyUpJEQ+GhKjiExwLdYZ
nJxzz9ln73XWXnvfkW2rm1e9YGbHMcowTufMfqwxmxM1eyVt9kfcrAUHf2XM
XsT35AqzdMpsSNasGRffwTzomS3F+sNqs9eH4c4Qs6OwrU+YdeLuoG82sWA2
DzZ3PN2h/W2sV7j1N5iv4M5ZYLiGO2ewtw3rkzGzB4i9Eetzedgh9v1yszh8
jgvN9gKnj+8cxrtDzRK1ZmuxvwxY1sO+hPFlldnPiDU8K/w7MB9DrFUVOmOs
7zHGjDDLOD+1vmLQ/1jME2A/Bfab8L49w2RXQoxguNlN4B2AzxnAdgj7DWnx
xVgPwV8W32/BZrSvM+5PTJp9VKtYXwDv7778XAeOmVXiim9YjtGOdQe47Aj1
fR++53vimXzfqdR6P+L3BuALGGZViSf6oc/eCr2xEXffzOh7N/x9GipvPSnx
yzsD4Lwrq1wwb/2e/FzG3q2Y3uhhfdETJ1l8J1y+qBvmgnuteOMS3H8NGN+D
7exAdh8j1imcBbA5UiksxFgPPhpqxekB5gcxVuJsUsRse7l02DVCeSJv/fC9
O6mcfALNbsGIwO4y7r4NvvaVSb8vp3WX93jejLub8O52p5M3YDMjkA52Yu8k
1oOVyk8xqVjUYJCWDqnB2764og7olz6ZB3LPXJAD6vIG1u97yg3PrsLnYfjc
Acw/AO/ogmyJ/UhS+LbW6D0R57vL4SangeM64/wT23XgasV7e/DGvZhX5FVD
tOPMvLBez3uq2RZPGiLODl9ck2fGIK7n/DiNc/2I+LOql43wvTAmn3cxx/Ge
AaybwNn0jDg8jv1fsf4A+5Pg+1/w2wYcl7BeG5Ouotgrz0nbgXsbuboKmzb3
lsdYL3F5vIL4v2AcjEh/nGmzGnt/esJ/ChhOuNyxNtdXC3/W9Qf6p+5+Kiim
7/LHfb6v1/WEFuSqMxQ2+qJ+if8euFoE/OejwkjeiWEsOKnLq24bcTY3Kp99
sC340u60mHghJ7zfWa16bwjVT5mX7TFxSj6JveRsdgHzvpx8h776MvVMvD0u
L+Tk76i4WgMOpniqLb6v0eFpAMb6vHoD++74lHrvqGr1OWJoiqnnUuen4acv
kK4X++rp7OcJrDM1usOaXuNJaxcKqnOuk/D9JKO3UxPss9TMS/Axx/nvzIs7
8rYO/j4P9W7q5bNQnD+Fj/8y6gFjsPdbXHjZUwNffbW7Sv7Z3x7D9h+MIuw3
I84uX7xQR+wL5Id1X3K1PxU8xHC2ICLtl5xmpiHO/Jx8PolLx8TzbU7/SPa5
r+CjO1QuqWXq5rmeEbPG9fw6+N7qeCu6PsZ9/kOYF+bnUaC6Za0yZntKcVkr
A65evk6oX7NXF1PKB33uCcUdeeuDTr7LSX9FxPR99edDadU5a4QcP3M8U9v0
xz5AXb/qS5sb8qpz5ou9udtxyx7Z5PpkBzAsi6tv89/I/wFj/Q+a1SMA
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlEls1VUUxg8W6JMa5L2+qe/1X8rCIErrUAlGJIYUC0FWxUCFFfNCKRsw
RuoC+iBIgokSBEyYIrQJQwhCgWCkymAkMciCOEWSQkyUKWyQFDb6+/ze4ib3
3nPOd875zjBhyerO7iciYhlnJKclH7GmHHH0yYhfEDxbipjEeW10xJKaiMmN
Ee3I5yN7a0zEMfRn1Ud8UhvRjc60dERHIeL4aGMsGhdxJxtRHBXxKfg7ihH3
09aVzVbsb/JeDl4F/Dexbc9EnOX9Necf3gNgfIZ9G/pfgPV7U8TupyI2cdZi
uxmdP8G7hc453s8lEc/URXRi/whZNmf7Kdgvxf4U8d8kthnY/0Vuf3MGUxG/
4f+78RHfchrQn8ffJHy9gs2qkf7rB6uSsb/bvCdOIAf0a7lvezriXfiZmXHs
P4N3MW1OxaU4WwxXP8LBYfT7kF8A/zynHlmZv3tpcyRu3sPHv+SSwn8n/teT
0x9g1xPvlynn2JZ1DPKtGK/g62re2N/A9yB4zbwX1JiPPrAPFi1TDVSb0w3O
VTX6Cdnlqv0hYhjmnsLnOmz7+bvNvY2/adi+jLwDLncnjk38ZZqJj7OC9zV8
XKd2WzgPUs55NvxdKRpb7/2JMWRbweYl5G0594J6Qr3WCqdzx5i/OeTzcc5c
/Toi4ivki8nhDWo5RE1b4Hd10dwJYzpc7UuMrRqfaTCGbN/mvbTJPa7eFsau
knNWroq5xHsO8W8M53wcf13Y3x3h+o/F13DG9poR9eYu4ulIuUeH0H0dnYW1
zmcvsbTAT3lsRC8xfci7h5hX1pmzd9A/WXYu8tGLfENiLiXfwX1nYrl89nCv
rdZHGK1g70nMp3ysAe9+2b2oHjvB/QD+PqozxjLu16r9+H/OzMultHNRfT9P
PDOaFdmU4SbhdNWYE83+4bz5Vs+Oz3sG1WvquSNZ56xct4NxAz6HSq6l5vX7
tH3Kl/hdQS6VRu8WzYx6/2KTZ10z8AJYrYnxNQPq7YcF7wb5a2n0ThC2eubF
vHtW+MppOff16OxFtp8zj9o+X7Jv5VMBe1NirsWpZuGHvOdFM7ENrrYXPTv6
Gyh7J6gXtFOmI7tR3WeqeS++NjR698nnUfK52mwutDP66ceBrHtRPdON/eR6
9644nUt8zdV5/wCMYXK9k3Gu2mnaRX0556udVGAXtY+zrmxUq8cF7z7VbB26
PTlzqx07q2BOxIV2tnbp+2nnI7xXsZ+aNzeyV689rM6H+GkitqRkmeL5DwqC
43k=
             "],
             VertexColors->None], 
            PolygonBox[{{5770, 6717, 2239, 6718, 5769}, {4880, 5770, 5769, 
             6205, 1882}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.7342489125797962, 0.8013958829155567, 0.9133839758248434], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1lW1o1WUYxm/XdrbTNvVsZ+dsOy9bEOVixqK+ub44tdKKlJCiUDMJKjCK
1GpbRE6dc5NeECrcshradJFKQR+alRZkHxIqCJaURWFtNHtBqb7U7+L69+Hh
POd+7vu6r/v1f8XGR9ZsroiIg5zLOBuznHLEeGXEtoaIsWLEDAr9rRFPc36q
iegoRdQ3Reytiri9EHEr5/l0xJFcxE2NEfsAGuf9YDuyeRFLuK9I5AewvRGc
R7n3IXuK/8e47y/aRvonMhFtYF2XitjD7zBnHfcX5kccrzG3IWQLG42zHs7n
8xFvIN8C59GE8yfIZ4mlG56N+DzL223oD17u96/xNY2sKmd5P/6XYjNQab7j
OXOeBnsKTquxuTpjruIpu28SzI/xU4PtXG3EQ7wvIyd7yMkXzRFfciar0YHH
bmQT2D5b5/sd+CqDczbveHfCra/W3E5jdxF/j+H3e3Q7qx37ILpbkGeRV+O/
jxzuxPYHZCNJrtbDJ5U1/qfITnOeSJlDRY3lr8JzOfa78Dmle8J5FL+fJfq9
5OHPnGvUSw6fXBjxI/adyDejtw1OL+NnGn9zYO4mhyvhcQY/96C/A9t7welG
91zG96+4b8L+rWrrvtdi/TX1EdvJy2Fi34fOi5xhdD4A+6Oy397PuB6qhXy+
krXfFL1xtMo85xPPDnR/rfSbaiLM2ib3rvr2RNm4wlTd/i1Z7ySyU2Xbq6c2
ZN1Xs+TpL/7fAJ8sOH8nHFSf/c2u0SH6fZ36rc7xFKvM4WjKMuVcNfwubczf
wPid01XtPC1pdn525Zw73VUT9ZPqMpbgClP1PNPsGpW4Fzhd3K/ndwaua8nD
TNazIV93wfNYwb6z6PyS6Mzi/2fui+HwHHU432CO+t1U9v2akmdS/aneUTyK
pQjOhbz9dsB3UavnUzN0ssVzdB/34wX3rWZOvabeXgb+w0X3exG7VfTV4Rrv
Bs225vpSxjOg3Kbgkgbrj1rnVXUVvmZodb1naiRtPOGvwOdr8B5GNoCfB/D3
Om+N1Kih3f3yLniHWoyn+iguzfVSdB/EZjv6PdyX5/3W38YObLO/obTfNYMz
cFsE18l5ro3wxE0zurhoHe1R7SblYCWxthLnRI3trszbVjt08H/+6B4o+r/k
6mnpKMf3l53nrRnvAuGvpSefKXkGtD8WJDiKWTOv/Jxq8U7UPhSeuArzQ/Cu
ynhGRkvOnfLWB2Zvk+2FfWeT8QeSuZJ+d9KPkmmWNMOy1fyMJTN4C3irCq6P
8rg37TwvSGITz2uJtafgvF7Kea9pp8lnRYv9ag8dafWMq+7ipT6cRHYxZ33N
34ZkBjUzQ8m+1cyfq/Oufgc/d2e8UzUrXTnPQg91ybdZT1w6i7afAu+lrHf2
P3nPm3peM9TYZFt9c7Ym9dU38O3k26H4lSPlWb2r3Ki3J+jBN9v9TVM+RpKc
XMDv4xnnRrMykuRHPdKR9NjnzMTNTZ6Lb/Pea4pvLu/9qL30H5l0Ifo=
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlF1o1mUYxu/m3PauqXu/370f2zxJ0hmGnbmT/CjLjNyBNAizGVqEhh+o
uHcFaZo5pSShQstS1DmlJgodtCj1IDpIiDowo0wUcsFEIS1P9Hd1vQcPPP//
fd3Xfd0fzz217/WeNXURsYpTz3mlEHElGbGsIWJdY8TCbMTP/DvJfR3AVGfE
tdaILydGNEyO6G2LmJfiG3xdU8STpYhv2iO2tkQMJiIu4Ptjwb6bwXyH7VvO
8QeM2QLX9znbruG/nthnsb+NbXJzxCTiX4R/KvH3EPNeJWJ/xf7jD0ZcTlqz
tL4Ax3J8X+Q8RzLdcFzKo63VvuI4gv6jnbYNw/E32DHOCe7dYP4FnyHmEvQ+
Bv4cttOZiE9arGk3WrvSEZsaXaMb4Mfzxs/iexGx2sE8yv04+QwX4eZs53tk
QsRr5YhD+FfRvp38ZvC9jfM59yE0fIHvPvIZhH8f+EG+p6Wdn+L9gu120vVU
T/qz7oFqfx79R7l/hN5x8i/zPQXfBnK4Rby18F1H6/P4/0FvuvBPZNwz9UqY
Ee7H4DxUb8zXaBvl7EpYn2wb8c/UGbOB+2J83sc+hRwOgt1Ev8awv9Nsm3qq
XgpzCdvvefdjMXpmgn+E827CnB+Q35xsLXf0vYz2PjQerrfe3/CfXnF95b8I
/mlJ93I3HJ9hO1hxrsqpq+waS7tiLCD20/jsSXhmF8D3VMm+O9D4MFzP1vLR
TCwn9nttjq18i/RyNj1ZCv8QNX8I/AB6t9V7JjfC92nZXKrBmx0RByrW0jPJ
s7QT/IWJnqk097aiucR5kvvSrN/XvSbPujjkK37N8jzO5RbXMI8tXXQt9O8Y
s92D/RS+e3mfj5PLAJgl+O9Ewxsd7qnqoX8d5FIqelZV0xXMy+2c5/WE6k29
fk2ZX/YPM+YU11stnuUbJfdeM/0x9vPtfpuawf/wv5v329AbeQbsEynPhmqs
Xiys1V890W7Z2+rcVYMq+VfTfjvStJr52JXzW9Ab3II93eldJEwr2O6Cd8H/
846esYx7p/k6Sy/Ptfmtq2d13Ptr/Io3C+5b0tvofvzD908F10LxG+AabXfu
2jHz0T635NnSG7mp/VGbD3H0Jv3mND9/Yi9hG+N7ZqNrMor9Tt67RvXpg3uo
073TzOvt78g5H+2AFdhfqr0H9fgvalnOmUuc+9F6oOB9qh0hbXPBbG22RtV6
fsq7RzV/tewaqTbCHIarn3m42uQ3OKfgHaJaSsNX1OtMxrtFM/wD+IGid7di
roS7mvUu0k6rYhvOmUucp4h/uuRaKIf7ip/p0g==
             "],
             VertexColors->None], 
            PolygonBox[{{5679, 6128, 1840, 6130, 5678}, {5802, 6253, 1911, 
             6254, 5801}, {6355, 5837, 5836, 4963, 1964}, {6782, 4309, 4308, 
             4218, 2277}, {5657, 6684, 2218, 6683, 5656}, {4309, 6292, 1932, 
             6291, 4308}, {5837, 6357, 1965, 4964, 5836}, {6681, 4276, 4275, 
             4170, 2217}, {4178, 5679, 5678, 6697, 2226}, {4217, 5802, 5801, 
             6780, 2276}, {4276, 6123, 1837, 6122, 4275}, {6135, 5657, 5656, 
             4843, 1844}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.7540140256824738, 0.8331529199703421, 0.9138345595262041], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJw1lmls1FUUxZ9dpp1udKYznXamjHUpRROpfsQGo41rIH7AqhFBsWAEEtBI
tGlIq5gSbSRaiLEoGpVYZSli6gpSgkulMSkYDXGh8EUllJYtcYkxMf4O5++H
l/+b9+5y7r3n3jeXdTy6cE1BCOErViGrOB9CEetCeQh7y0L4hLUfgV6+z7D2
sn+3MYR3WJuqQpgstMxxdEcrLSeZI+w/jc6nMiF8WI18cQgzsiFUs94qDWFW
TQhP14awHhsfFVt+BN2WmO870f2X7wZ+jwkf+21F2LskhDpspNGdw103+2/Y
v4ydWfy+vMS6MeKYgY/7+N08M4TZrCuJqyMVwnLubqwI4S+wjSTscxQM09hv
Zx2P4yfytRO5h/Hdyn1nMoQ3G0LYV2a8+yLM/fUhLMXudaXG1ILPu8Gzo8L6
0v0SmS/qbasSvc3kKInu7lwIw5eGMIHP0+CayltvEXGtIm+Hkd+Pzx5kvkVv
nFi/rwuhCz8v8T2KzNvYP0ss5zL2fU8au+gPg+E0uCZZa5E5xfoYG5P4PYa/
Dez/Yf8G9p9MuubCoTwLm2JUbVTHPjAN4PMcdzuofwc4b6iwrOxKXrE1Rbm6
g7huSZo/B5C9jd93ITscs3+ddxPTTTnn+kQ8yjs5LyLG4YTrmWc/v9o13Uq8
r7L6ifEMes1Z+13N2U7k3kPmx4z1pSvOKm5xUjwVX+W3CS5kIv4ofz1Z53AM
O4vhzCD2GzirrzU3fiKOSnK6oNC8V7yKu4X7hej3Btf+avTakWng/FTSHFjO
dw2YDmMnwXkm6/PX8fUaa1OJedpaZ66OkKtYynyvR7Ym7d6pRPdk0vYVi3rs
Yt9x11th7grTRJRD9aPqqzrvIld/N3o/ip1X8NUJniHs78q6zx7E71LWvArH
3xX1XQr5C+C/JmZM86Kai8PKg2opHqifhK0G+c8Slt8cN2fUI+uUY+5WxTwn
hO9/DmqvGh3E/+d5xzMV9aH4eStxL8jZjnzKt+ou7mxJWfZA3OfC05p2b6im
f4LlaNQvMez8XuuZ8kHM3L7Yv3H3nHir+be20TPwELIDWefqWfbNNZ5Xe6j5
Dxlz8l7sv1DvnI8jt7HcPNsKrtG8+6UNnDfnHOP7fPfkzHf1RDzlvhiIMAnP
CWwfA+sVJebE+XLLXJv23BQGxfli3HwuwE9h3nJPgL0s5Tn2HPb7cs5NN5zv
Ya0od5zq0/bIr+IXzmZimZ+z3XX4KSCugyXO2Zas86Z8iEPKiWaJZIVFPNaZ
5rBsi5eyrzyKm8Ivvw9V2dcKZs4jDe7R8pRxS35J3nwVNzTvVFtx4GfyMZHx
fFfPbW9038lea+RLcWgv+5qv4oNi7yBvy9Kut2bGorz58zg2SvPGtxrby5Ku
4fO1nimaJ+q5RNrcVg/PjbvvNIPEL8X7WKX9KA/iyMboXXuAuAbx0V9lLuur
WL7Omx/C+Rt+T2Y8O8Y4P5T3rP0O/3/UugfvJB9XJdxP9+f9ZqjPFxPLEtaR
UuMW74StLWnOiW+/YPtXfs8u8ewVVuG8HZzbo3dTOVZvi8PT+BmM+lTcORvx
R7lT/LJ/PTHNZTVVOR/j0bl0hEV508yaE72D8q15oHqth4NPsVaW+3+BZpxk
hV34xOcEZ23cbSt17+rNU/92i/8Jc2ukwe+iOHI+41ks+9Psz2TsW/8RNOs1
5/UGrmzwe6C5rrdfs10xDEe5G0J290z3sN6TrmrH0sd3otq10Bwdivj5H/gc
Yj8=
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlWto1XUYx5+529l0tnO2nTPPtuMqtRk09aWLIiRFUYLKinROnUYqZKHV
EJloTEyynL4wKy2TzTRNY9HFcnbDNCONepHo7E0XL5u3xN4E0efL97z48f89
/+f+fS6/29uffWT5sIjYwCniJLIRSc7uRMSswohb6YgD0P9Bn4DflIy4nIvY
NyKiuyzia+5fcoZQ7uJfS23EMOjrwyOeLEW3LuIc+l3lfJEfh/5XOcseK45Y
2xDxeE1EH76WonN0dMQVdFIEtbUiYll9xFTof6HXY2MX9LRURD/059DTMxEz
sPkcsn0lEc83RnyH/WvEs3BkxCD3Y9UR749wzHvJ5y1i7Ca2bfhcCz2H+E6R
3zroxchfwP5s7g8g31cZ8Ts+mpDvIv92+O3V5t3POYn+TuxtydtrajCGwm4M
+fyK7hnOOGLrCNvuHhUxKWEfB8mtDXvdxHoefPrI/wnyGSyIGIAugTcfn/fh
qxCdHrDaAQafkPtF8Hube0fKWHzGWcT9ADFswveP+FyAbhtnd5FtLMw5J+Wi
+D9Adj/nUqF1VsFbTfzL0D2IzAD0CfLrLXX9Z0LXw5+Qz6cQewWcR4tc85Pw
J9YYS+nLdgs1GTvSPnYS7wvEeJp6fUq8v2D751rbXoXNzWDzJ3jdjb+J5PsX
94q0sdK/MRn3oHrvdeSPcL/KvwnwmqG3E9vGSttbD70Uf1Pwd5h+WQleVcR2
CXoS8lugv0D/Zsb0Pcin4A9BN+f5V7gn8f9Yof99T6z/JB3rHui/8fdMxlhP
JqY34bdgo6fYPabZOJrHRzMyA9l2+B/D/widm9heXcVMlDreNu7H+deRt38N
+fq8f+U4i355iLO1zDPwYsqYCsvDnALwK0nZlmz2gn1rzvOjHusj9vPYvLPU
PZnB9hnkG/P4aZZexv7RMs9UbcoxK1bhXUY/luQ828pJvfUePraMdI8dz1lH
shvpr/FJ10y0Yhb21WljLczPQRenHYtiKq/2DGt2Z2NvcqNnXLOtHlKs2bT7
TzH3Y/9sxnf92wb+Q0n376v8W55xjVSbU/zbAf9W0vhqZlfQn+savHu0g2aS
+wzOEfDajM4A+rm0e/2OUvd6MefGcOev2M+mzFMOms27ktbVjCaQHV7tXFT/
SmI5VGl77yZcm3k570bVaAqynTXejWPhJ2s8s5pVzcAN4rme8fwKw73gs6/R
fP2bg+zcnHeHdmgGf7VZ11Y99Br98SH5/VbmHXOR2Acz7nXxhaVmqjmPr2ZV
O0a7RTM7jf68N9/fwnc78fbnvJsGyW9Po3eUdpN6QrOgmkhWOlOx9069e1UY
r2lwDYT9EjBdgL0L+fnXjuxJ26f0VdM/0K/K94928ibur6S9a7RzVFvNqGZH
Nf4h7ZkSrZmtIJ7xDd7N2tHqpduyroX6R2/VEuJ7qdxv1kryWdHo2qnma+B3
Zr3rlc+hOu9w7W5hugj7i1PuNc3409h6uNK7Rzbnk9/latPab2vY952jjd1P
5X7r+ut9Vz9oN2knqT6aF71l31LDpxJ+0x6s8xupt1E77g1yn17l+JT/N6Pc
U+ol6czLekcLr9PQveTfmvVd+O7Peidpf+jNP0ZtT+Q8y3pT58LfkDbWejNV
K+WkXFQz5aodLlo5tla5BsJeb8jVOr/xetu1Y/4HKA42sA==
             "],
             VertexColors->None], 
            PolygonBox[{{5388, 7463, 2852, 7465, 5387}, {6109, 5388, 5387, 
             6110, 1827}, {7551, 5461, 5460, 7552, 2921}, {5766, 6202, 1881, 
             6203, 5765}, {4202, 5766, 5765, 6747, 2254}, {5461, 6401, 1984, 
             6402, 5460}},
             VertexColors->None]}]}, 
         {RGBColor[
          0.7726427852651448, 0.8503366636124604, 0.9095457291731025], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtVltsVFUU3TPTx9i5nVdn5nbuMNfB0mk1TVT8svgvyA9ViUlteRQThNaK
VRNpfFHQ1hhF+Wh8Ja3BYilVaH8ImvCDCrSfSnkoQRSMUdCk9dfEtVzn4+Te
OXeftdde+3Fmdd+zjw5GzWwBK4Z15Q6zy1h/RMyu+mbzGbONMHipaHZvziyI
m+1uxnvebKLW7ETJ7F1+w/5wk9mLZbMdCbP20Ow+2K/C/vaE9oaBcx2Yv2TN
qvVmT68y+zEwa4W/poLZDew/BhIRnI1h/Y0z3+H5QsVszAMefC6k5bcLz504
vx+ca5Py8T3ee2A/Bft1njiNNJpN1pjtAc/nsHbi/RL8+PC3Eb7G68yOwf8F
cDuHuH4C7gzwoylhUAfGMAubM3jfghgXcfYznL0G7h05YR4sihP5PAHN5u+U
7yfBpxur05N2/TXCWYSvh/PytRnP18rS59Ma8Z7A8zD8tMPfSExaDhfFZaBG
T+K8AT17c7LfB4yhsrRgrtaE8nnTFz9y+wrcLuD3pqhiChzOK9DsZaxdSXE8
D14/wKYDeNehVxc4bHWakGO0UTkkZhU2DYjhHZy5BexsXvjz0GM7OLyH/TPY
XwBOGmdy0CaPdRVYn8T13AWcJZ5FzB3ISRoYN3ydbYbOPtY0bH/1hcv9/eDx
dkI8WSMNOdUJ9/htCXHdQrzj0HogKrvfY4r3GDh/UZZtbaiaY731Qu+9gfL7
Dbhsw+8pvG+Cz1P0C56WkhbUoRM8P3L4CfiPA+efhHzyG/dn2RNZ+c+A594m
1ewB4N/t8ss62uPqcz32PsT5g9CqkJTW7JG34P8DcFuBDisJ+emCfQY4t33p
1gr/a3OqkWXsb3X8zxZUu4xrPWwfrEifm+DVUFDfUTviMsYTceV8BO+N+P4b
ztwD/BlwPFrSnCiAS8qd/TmrWuEZ1gj1Ye8chgYHfOF0IqY3YT9h6jv23IjL
Cd+Zl/dh82dGup1GjfwFX14DdIWv9rLqJRoKizhL8GvAfCSmPvY8xc78kwtj
bEOsrRVpyd7tcTosA3PQl/9nfP3meySlGUjN+8B/gy+enRXVDWtmDLw2ZIT/
OTAPgU9/nfaoL/fZ34Nl9fhacH4Aq+jpN3PIvLDPuEe+vaHmF/VjP1321VPs
vyuB4qnie1uoOI+jHtKBbI7iOYP1b1z1ydqccIvzj/qH4HIxq7l3CdgXscJ6
6UbNmAPOG9Yla5J1wRwzv2046+V1FzTjfRE4pYRm1aTzQ36sG8Y+6qmWqRtn
CuPkXJmLy4Z8IkXNPs69xwvqMdZeD3xOOQ5Pwc+OvLj1JVXzrFHOuVfdrJt0
uP/HWC89WA9fIkfHS9KC98xsSTklt5NJ+UpiP+bq5xx0/TbU90PArlZUB0N4
Po+1DWe6Yf867LvrFD/rkZqwBolFHHKkLXuTs3DFzdvzbmZxDtyPHm0JpUm/
22P9c5aP+tLzSEG1xbqaLokT+dRlpR11G8J+i69+asb+2VCaZwL1DPnx7plz
eeL92RvoDuVszuQ1S7fkpBe1ul1S77HvyP3rjPgTg3vMH3mwn6hnC3itwboL
cXvgeyqjWUGNOVOoM+0jbiYwlwPuHhwHlzFwWo5rVnHmsi+Iy7nH+4WzxNzZ
oKD7mzXMHJyOaw5cg899Li/0yVphnfCOIhfizKU1j1gn9Yh3c0E5Yn9TO+q2
DnweCoTLeud/IOa3mtcZ2rOuR11cjCnquHG27U6LD++xtLvLPsZsmHZ55Fnm
ir5SgeJhLP8ByEI/5A==
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlUts1GUUxS/TmU5LBzsz7cy0M51xFDrFV1BcGMCVK5WN+IgJUoPUpPKw
BhoSaUSlqECMii6QiEk1WlLbEoSNQRNjYhQMSwWEaPCBhChoQlnowsTf8czi
Jt/3/+4999zn/4Z1zzw4koiIUSSJDPdEnM9FrG6N2JyO2M/9YDFiI/c3uM9W
Iv6pR8zrjOhvj9jDfTdyel7Ex23Y1SIGyhFHOC/PRGxFt6M74lpHxLrrIpZy
XoxOH++bcPgW+G8imxL2cQTfPdWICvor+TaAbn/NuhnwtoG9D7mK/Yct5jKD
/0SnOQ0V8IOMY3sKacE2gUwQ219g9qN/1/V8WxBRhM/FfMQC4rsZ37cQ3wXu
v5Yi9qYiHgJ/Gl8XshHfYv8vPl/hbRp/54j9d2Lege0KdD5vc87uxNdt8N8J
31743s3bF1n7l458/wxGI20OH4E1hZxtN6ZiGQRjRcYxNeA7UDdX6fxZcc6V
68z8iAPk82v0d6P/CTrt+OvGx4/c30VnBF9P5p2LS8Tzd90xiHuA/xn57io6
1gcSrt2VirFVw9d7Ix5FZxyuS1RTzo2aazEDnzTnh7HfC9wc+d3WRd8Q/xOc
U/C5h9hzZXNZj04KLlfRn8TfGP6S2Kdqrs2qpHMVzbtydpRYDsHnvaQ5jHJ+
Fk5l8GbxX+B9JmtsxXwG/Ab+X6N+K/ExiO81yDjnCb5dJPbfkJ3hmk/C5XLO
/SedtfA/ST9Op8zxErrZgvvh//6Qfd750rdD+JpFvsP+VTifzLsGilc9vIx8
L6/7TTNyCvvTyK2tzncDf7cXzE3zthisgap7XxijdedIuVlLPhfxVqs6thvB
zML/PuKfajOmcvGlOKedkzOcD2edj1ravZyruzbq6a9q7iH1zi7yt6TgHSAu
4nQT/Db0+Kz8nBV/Yry/xZjPgbWM/H3DeT0YJfj0V81NHF8sOkbhaT7eAetA
j2ddOV/D24lmvjXPx+FzomYumuFj8D1PPoeT7rFO8D7NOXeqwSJ074D/xqR7
IotuZ9m+xW8K/Xu7vLvks6XoHaP8KoZz2lX4GAd7FfeF4G2vOxZhJtH/vuTc
Sf9t9I8XzVU7aDP9uAXZ12qO6t1GwfVSD6t3H6t5F6qHx3rdw+pd7cCXwH4Z
mUiaw8KSZ0yzJcwr3C+XHK/qezRnTuKiGIaoxdMlz7Z6cBLuj5OP95PeIbtK
nnnNunwc1u6oeJals4HYV5ddG83bcF/EU33e7ZqRP/D3QdG1V70eKXrm55rz
eiznGdFsaJ6ep/bbq+4t7YAS2LmiuatH53f7n6B/gXZWG1hb6u5t/SO0O8ao
11CH7X8hvkxzP2tnjvA+2OX8q2fUe0tr3rXqwVzBO118NJ8/EN8eZK7N+tfA
as1792i+XwBvR3Nf6Zt2909594J2+P6ybaQrjK1V7zDpiuN/tsbyzw==
             "],
             VertexColors->None], 
            PolygonBox[{{5654, 7459, 2850, 7461, 5653}, {5830, 6296, 1935, 
             6297, 5829}, {4232, 5830, 5829, 6807, 2290}, {4169, 5654, 5653, 
             6680, 2216}},
             VertexColors->None]}]}, 
         {RGBColor[0.790393198310673, 0.8562559533866391, 0.9015935600354302],
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtllls1FUUxi9Mp+0wLczSWdoyM63dY0AFE1NEQUOi+KAJUSLuLEYgWvbS
NhqKoGh8wpVAJIpRoRALLyqKCCXBB+KDVHBLBCmoiU8SH4zR6O/z+z/c/O+c
e+5ZvvOdc6d12ZpFfZNDCD+wYqzD6RBOFkK4C2FXSwidrMLUEBprQ/igOoRV
VSFcXw7hSkMIa+MhlGpCuBb5Qi5vZP8g+xXc3ZIPoT6LDvLLqRBGmkIYRf92
bN+aCeEQeoVSCHlWUzKEv7G/HdlozLZ/juyfws4XxRD6ORtpDuF57k+wX8B3
PnZG2e9Afhv6w8T2EHaOcu+NSSH0ksvTxH+S/Xr2n04P4RfOlmPvx7TjTGI/
werBTpFYjiDvYL8F3aXYH0enty6Ei+S2F/vLMj6T/Blst4FFgvMa8ktXQtiD
/zJ22lkV8upE/hr7UeRfc/efnLHqStrG3azHk8bmMPJJxPIvOu3E0DPdcSvm
/TXGYJ1ixt9WYlnN2oTuRtYJdAb5DrDG2P8KPke5ew/62/D7Lhj9xf48++84
q2CvKm4bW6n7U9xrQP9Y3LVRvVXrhdQuRUz96HzC+bS867281n72gO2T3J2S
Nu5TOR8vWKdN9Y3unoz7jvSFl7ikupyhFo9lnV8n+t8QXxt4rCfedayRhLFQ
vOKk6jCQdC1Ub3FE8W/Dz4qi+ae4qyJ5W8Ic0f6zuPkhbhxn/yj7Xew/Z7+U
/W728/A/PN3x6d6h6O4F6ngULLqx9wR+rsuZP/dT/xxni6jds3nHoRjmgEcb
OIxhcy2663L2ebZgzolvc8jvo7Q58AC5D2Fngjiuch7PuB/E9z9bzPkZ6MzG
X3fSPmvz9rsX3t2JnTX1+EFnMOs+momtS9Wuu3rsRfzPR74fvweajWEfca3J
GZu5yN7m7jl0GrCZaLK++l+Yj8eMi3ir2u3nfGfKvXuKOA+m7GMiY1wUmzAW
p4Szcvud2h8Dz07sd1X8O0Z+49y5Wm8/qrF8iV9D4H0Qf2MF94a4MDLZ8aqO
55GfK3gOLSD2t9B5JebcJqLchUtzrXvmdWIejHBeWWWZajSUNXbC7Sfs3VRx
DPJ9PIq/s+y6Sl+8Vm+I20vYT41mnfh6IGEd2ddeeHaCceD+bmydRv+FlHH4
v28TzunDtDkhPsh/R4tjKBHXNdE86cbGLGZdfZ15sSOah8p7c5NzF0c1b+T3
24Tnl/DQfLol5RmlPlRuXxLDc9jYXjBXBrCxD1urqj0bhqP5cI76JKLZtS9r
rsj+7MYQbmyM+pLvhmb33jvY+C1tO5INNVquegpP+e4gl66y59kf6C4peb7N
IL/WsvFTTJp/qqNkMxssP0Eec5uMp2rbV+9YB7ExUPJM7SfezTnPHvFWc0ez
Qd8p0V71idc5F8X6Uo0xXM+9DTnPhtaS30XNH71pqrdqPZb3+6S5rDmkWSOs
2vn2JF1H4S/sFf9q5kZP1u/kxYz7Wb2cynlea1brXld0V1+dq9fGo9/a7yqa
Q+LPqyXjJUyEqXiqmuodi+X9ltWA233ia7XfmWVJ87wO2aWM+/QO9N+sGEdx
SzVR/W/AdnvZePXityNnzMVl1UD7K9i4TPw9Ne4n3RX/NxDbxpL9ncX+4iiG
h8GgD/kjyN/LOx/lspPvy0XXQP2g/p6GPN3i/xCK/d6885Gd3px7XvXqx95m
1oqke1u2xSXN0U0lz1LNePWb5rzmr+IV5hfSnuma53rzu6P3Xdg1Jj0DP0bn
q6hGh6I3RrUei7isfhKPuqN+V09q/qru4tjKaM6Iq8PR262+1xxTrrPK/r+i
+moOqtc1W4Sn7kg/Qx5ZVjHpWZureN7eTB9sjTipmX0m45hPF91XwkE5y4/w
1B3ZmBdzb4mv4pv+m6jP1eN6gxWDOH8iilE2juD3+ybfeZ++Xpw2xxuIK1cy
Xv8BSldWXQ==
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtlUtslFUYho+9TduBOp12/plO+dtC6S0SQHDTaASMCeBCDSoJKKKlJpZo
W2yhl2gErUFjYgKiC40EIVraGKobtQpKK9EEVlJBcSHQelm4gpUxJvq8vrM4
yX/+7/Z+73c5S7v6tvQWhRDe4JRw7sqHcCenvziEd0pDGMiEMFsbwpPlIXzN
vTcOYSP/9mD0eDKEump0uX+FbLQihPEohA85p/H1Jj7uTocwVIce9h+VhTCI
bg6bEXTPYvNBPTacn7j/jc8EsVozjl2CzXZsL6K/ucz3gO/L+GxMhLCcfyPI
NyLvXRxCPfII7JXodCAb5z6P7vVsCHNg2cK5wncl/u8rso8Z7rNZ+9K/H9Fv
IMdGcmvmjOBvR41zXcDfELbnwLiL7xkwvo/sCPpPQ94l/Ed8F6PzGrI8NvuX
hNCFzzl8nyJGCmxpzmZ09wVj/Q15R8KYT2D7SQo88PEHPs4S61n+neZ7JzEP
oH+VfLuLXKNtDSGshYNr2Hcuwo77Kmx6wFPJfRn3o40h9MHPAf4dBt9J+D7K
9zD45sj9h7Tx3Eb8qrw5Fh/Hyo31Mjpxwjpt+JpLub438H8e22+yzncd8j78
j3CewvfHxe6VCzliJtwzr2L7dt5cykc5/j8nnweKXJ9hbB/CZqrUPrZx3x5b
X/YVyE6lzJ3wqDf3ZtxL6tE28m3lzN4SQhJ83xH725xt95HTKLFbG439f58Z
cyxu1cPfk8vFrGshTH3IzyB/DPkZ5NNgXRQZq+Tv4esgfO4vcQ8eAtsN5GPY
/4PNylrXQNyrR9rph0/xcX+Re6If2xXolIJngppX0U+dOfe/6nsHtZ0v9K9q
fB7ZbORcxrm/BTfHa9x7U8QbzngmNYuz+FiN73biL8b/7hLn8gw606XOabTO
OpKV8G85/ppj86sZUC0GuHclXZNf8P1iZGzqwa3V7kH1nmY4Ax+1nPXF7omd
YLtZ4EN4P0O/Y4l7Wxy0NFEzTrbKOU7AxyRnocw9+jP1uq75SHomiiP3qHqz
pcy7pDfj2minSHcFeNuSttEsprnnksbTDX+7csb+KPb/It+Qsj/1/J5610S1
mOSci5yDsItz7bJ1ac+ydppqdxU87RWu4RW+76VeU8ib+XeQ7x3orCx3TmNZ
z7jyn+f+RI1jyLdmJAPWqtj81YG5CFlt7G/lu4zvpXFhfyXdGwtp96P4W8/3
PVnvWmH4Eq5fTnu3KmYztbq91ruhgrMV/6159452bA/cvBSZmwHwbEB3NTb9
pe7xI+iezLu3p4g3DXcPpmwvDo5H9iHbHnz8BV8J9N8t937YC/bRGtuqJw7n
/GborXgdmxFk1U3uNem0NnlnaVepRzZVe+dr12unKfbDkbELw/Pov9Dk3GbY
AUPEHs4bi2L8CjepjGdXO/eRyG9OSSH/C3CVi821aqBdP5LxLGnnX8L+i2rv
Su3EOfTjwntxEzzt4D8ROZ442AS+qsj6eo+0OzszxqL915LxG6f4mnnVPhv7
7VB87briJvvWTNbE5lRcqqd/r3XPqddUo8HYNRJ3mlntIu0M9a/4/hPsh3Lm
Whgn0Z3Ie1eL8zX4a2nw7tTOuDXymyrsyqEick66a0fqrRmu8+7Qm9MMP8/V
e3a0gzWL1+CoPemZ1NvbGvttVQ8PxX4zFL+bf2uJvabBtZONZueVrGdFM9QB
v2OF/hxMeHY1Iy2F90oy1UDcS2cV8XbnrKuZ/w8Vnxw+
             "],
             VertexColors->None]}]}, 
         {RGBColor[
          0.8081436113562012, 0.8621752431608178, 0.8936413908977578], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFV1lslFUUvnaZ6ZR2prP8M9Npp9OFVUgpIC0Nu2HTByES6EJBxSUQoaBA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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtllls1FUUxm+nhWlLt5l2Ztppp9OyK0ggKBRZVTZ9EKIRKIQouASjFGRr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             "],
             VertexColors->None]}]}, 
         {RGBColor[
          0.8258940244017293, 0.8680945329349966, 0.8856892217600856], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtWGl0lFcZvmEyM9kms36ZTDLLlzZVwVLF1CQClaUSqJUUAQmbFIGEE1Da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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3uQ1WUZx9/dc/bKXs7Zs2fP7rnsOWuYikIZCsgloHFXbIZLgCw3gVmB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             "],
             VertexColors->None]}]}, 
         {RGBColor[
          0.8436444374472575, 0.8740138227091753, 0.8777370526224132], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHlw1dUVx38JyUvCy0vyXnhJyHu/vAQXBBeqaHGrK7h0VCQgghWwaoMI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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtmHlwldUZxk9uEm7ITUJuwk1Ccr/khqUKqDiiRREoKLh0UCQEWSrgglCW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             "],
             VertexColors->None]}]}, 
         {RGBColor[0.8613948504927855, 0.879933112483354, 0.8697848834847409],
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmHmQVNUVxt/07PN6erqne2aY5c30DBDjkhhJFMUNGQUrQYPIMkRBkwpo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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3lwldUZxk/2wL0JSW4u4Sb3u7kEoiO1dUArxaIgAdEWUJFAaAHpItJF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             "],
             VertexColors->None]}]}, 
         {RGBColor[
          0.8791452635383137, 0.8858524022575328, 0.8618327143470687], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmXlw1dUVx38JAbK9bI8YkryX92JRsSIK4oJW2WQUFFRW62gdEQWtSgcU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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtl3lwltUVxm8WErJ95MvSJF/yLbFBaYGwFAVUEmQZ2aQtQbAdbUcMiB1b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             "],
             VertexColors->None]}]}, 
         {RGBColor[
          0.8968956765838418, 0.8917716920317116, 0.8538805452093963], 
          EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmXmUVdWVxm8VVa+q3qu5eDVQ9eq9IqhpxQmZwqQ0SiKiwSKaYlSwbY1R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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtmGl01NUZxm/2ZCYZZ5JMkklmMhNwl4qyVwS0KC2LCsGigqBArdYNS6Xn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             "],
             VertexColors->None]}]}, 
         {RGBColor[0.9146460896293699, 0.8976909818058902, 0.845928376071724],
           EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFmgmUlNWVxz96Laq7qqu6upteqqlqGBFoMFEWMdKAoojxKIJmFp1sMxOZ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             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwtmHlwldUZxj+y35uF3HuTQO7C/SKCQgBFAoqyKLjgikGcWtHW1ipjqzhO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             "],
             VertexColors->None]}]}, 
         {RGBColor[0.941176, 0.906538, 0.834043], EdgeForm[None], 
          GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNmguQleV5x9/FXdlFz/n27NlF98Y5u+yNy7LAIqJOuSMpKyJqqEZNvNXa
iNBEjZdoms7EaoBEBDsdiwSTprcZHScJpoLONNEgdKbTBm8VhAg4yk0FVMQt
aPv/7fNn7Awv73Pe7/3ey3P9P8+3bTcuv3zZsJTSkjNS0r90S0VKvVlK80VP
OTelda0pTdTY9nxKPWrzRR9sTmmVxhtEH0gx1qZ+6Vkp3a52nX5fWqtn9Sl9
uyql1fr9hNoOzfnPUkr/XEhpnjYd2ZLSJM2brvFHPWex6JVq40SvUr+gKtZl
zceKKT3cGnv8QGf8oialr2j8sOb9sfp29e/zjul7z07pbrUXKlMano91Tmr8
A88Zrf5DtQHTR0x3qD+UYm3uNVz//av2Pi4GHfRejB9Vu8Tzj5nuVD+vKvbt
1++zdccRunNR755sTGlzncZF16lfK/5cr/mbdb6/FT8eF71P99qrtlPtcj2/
QPy5QuMvZ8EfePlfWu9X4uElw4Kn8O4y9Vs1/6GGlGZq/0qNrVGbpfETlTEH
3n4o/jWpLRddqzUztb/Rs7/Xu/+k9pnmtmvPdWo6YnrtjJBhQXtt0rPn1CZr
fp3u1ac5ffBKdz2o1lsR85k7XnR3LqWuXPy+SfvcqNYgeq9k8bzW6dOcneJJ
je5Td0bIm7GV1oEJolek2BP67yzfH1pe8OaCuuDPP+rszYW4y8Na/yG1K1P0
/H5Ea1yo+evLMf5uLp4VNX+gKvTsNc0Z0Ngl+bjjK+LTK5J7pvEHKuIZZ6g0
XyerVYmerb7f5xzwu8e0zxi1c/PBpyfNq0u9PuutkKxO6f6/rwobwr7QrzG2
Nfj/D7rXz9Q+1dqbNW9WdUqvW18arDPHZI8fq9WNiLUX+gxnqs3BjtFt3eP6
cjxDNsgEfv9yVEqLGkNXd2ndi6pDzvdpz3vrw3aQ/7R8vIPtrKiIux8xjSzy
yK0ibPlDy0ivpwMjQrYjRb+YhR2cI7pL+jNO/P2e6J+ID+vV3tY+jV5nhsbf
0pzdaoe0xko9v0Ly3V4V9pRvibWuFX20OfiBHp3VEvtl+A+vc0NF/IauVX+G
2kzR27X/q5ZvoSbuwPnnVIVMLhd9eSHkxO8J4kGv2gtafz32qPa+fjd5L9b8
DwmtrDPM0pxpkvkPGmKdg5p3AL3X+F9mIWPoL7T/oNocnWFPFjaLPmE337UP
XClG/khth+Y8XB0+Gv88tyr0CJ2sqYjxi0WfLIadL7d+3ug1C6bRCfgBXyaK
/oP22q0mNRiyh5vV5oneoPP8qdfEPvDB01LoCD4LPTmh34vQXfWfmu5Wf6fe
vczjgynoMfa9S0RPUv9MLnR6lWR3ofi2WLzeq2fVFbEvZxju88xN4TfgyboU
c65RO993v9ZnQ7ZLrJ/o/02+70U6z4Vq78pPPSMFHKyN+IVOw2v4zNrQxLuf
am676Mph0bflYv4S6w/nn+n98AV/UhF+gX2Jn8iCc7XqvZZc8O7Ps/D916aw
p+7cl/t3e1/uAb+4C+/w7jTHHOIccecBx8f71T+Vjzkj2Dsf+/He503yBaXg
7duyj6O1Eas5F3dBpnXmD7xljVavg14Ps26w5zDHuyO+52T7QGj8HjqALKaq
b5IMfy5dfUy/l2nfp6VUg9KT4/g+68Yo9berXWp6mdrXU9jRN22nzepvs011
qr9D7buiW/3udaL/Tfe7uxB+sMnzF2m8pH652jdEd6j/tunRppeJ/khtYUXE
a/ZaWxExdJNpZPqx53Sp/8Tnh25T/xdqS0WXvddlXv9bvgv+mnsTc9s9frvP
tsznbPc63KXLd1xq2d5j24e+27JuNH/gG/HwPtt+t/q7KkIv+jy+0rZ/VUX4
f/DJVZYX+nWtbQf7+pplV5C8atWW1IS8GD/PfLja62BTV3t8m/RkchZ37fEZ
4C33/sx33+rn0LtET1I7Xhn3vdP3wi7v8fnBCPdWRFzjjtBggLHmw0rf907z
k/Vf9ppT1Z+Xxd7M/451Zoxp1oF/TeZhQ03wdGEK3Wm2XE7JdnZIb78jP/cH
0bvVujX3qcbQae73P/Lx2zU+SuObpIe3SA9P6f33y4oh5fDv2N+AbfA3ej5W
7yxQLHtVfQXxS3ut1n8P1Ecs+KrmXlkKf1OdxXv4zLOyiCe3pcDjnzSHr4AH
3eZhp9brVrt4ROhRl++OTHrMW/SizzoDVsDfsQ6+4WRT+AfGeIaPfV3rDW+J
OA4WBm8zBxwKntxhe1ljuqkU/IJXPxEPnigH9iLefS8LTLPLcQN8ktc9tmrO
VzV/Xz6wxvPGmMx53PrMGGcGWz1n+gvt879qc2siFj2bCwyAzjznd9+ojOdg
/kd0iaWl2H+q5Ph5bWCeJ/Px7ul4ttnvwjN4CT7pqgm53Wc/DEYjv8iJN2sl
14nad4ToR0X3iX5L6/VpvtKntEFjTXp2kX5vbA1dQU/g5wH781741hz8nSN+
9OpsZXxdTbyHfm6T/A9o3Xq9t/fs8Imc5xnFzS/0/v5c8J2z/lDj/Vrj57Xx
Lj6TtRbaFpjDu8Qi9Jd4hJ1NsN1xnx25wGk7c3Ef7oINjbUd/VryfDaLeazR
6nWwJ+wKHEVu9zPndxdr7mHx+mzjLPAWeBjMAPYAJ4BD3lb7DL3N4jf0k7rj
zHI8+yONH9I6NXq/Rnx9RPztGRH8Zu1zLS/uCR/maf5ctbM0NlP9DLURNfEO
Y4JnQ35lnH3LrlzcGR7gk8bbD+8Tzz/ORc4FXjtlzEY86bDfwybItYgJja2B
BbFbco33cpF3HDCNHOn3e5z4MNo+jbzkXY8P6o6bnDPyDuPgoh2S+c5S6C4x
pN2xgxyoWBd5ELEevAhuybTOY4XgE7LbaZm+1xB5EXuBgz/NRR73nu57POf7
Dwuchi/CLw3PQodvzSIW4pfwl5ubwmfu1v6V5eDXmy0hJ/gNXiW2gFl/Uw4b
xj7/uiFwA5iB3HWGsfd00+DWp8WTZ/KBgcH9yAf9/LHs+uVyPKuWMY2XHd1R
E/nEpFLkFOAT4hvYg/XYg73W6fyzs8gX0C3ye/TteZ3rtZbQsRdEv9ESOtOf
j/XAhPgk7jPkSyvirHPsP/usM/fZptAr8hbyCvDtRuVcq+ujPjHb49z3pSxy
pFlqV+r5Bt1rhfZcXQj/gm/BTnttg6dqo0aDD8fPrnCeyxnxF5wTHPtkFpiP
mDTWeQf7cmbOPk7PC1lgf3TyXetnTznyWHJY7Bs/go2Dv0YZt6AjPCfvf8ry
ebMyMBQxAVwE3gH3gIt2a5+3HK/JVQecC4OVwaRgkon5yLXIs8An4FPwxrOy
uRnluMMr+ahJoW8PZlEPIjcb9Br4k/X5wMTgYWIq8oMnzGd9fCx5xDzrBzL6
s+qQx3yvRx3owYaoraA3a2XX00pxX/be7poY9rTDfgMsDS/gA/3T5kuvbOR3
5cDYjPEMmn2IB+x1k9ZeUR+xC96yDzY4PYt6E7Wm+Y5BxA7yqUmORzMsU3Rp
gfRlSm3kidxntnWP+/3eOkeNiloVcfeeQuBpsHRza+AJ4t9i9bfUBh7ImiNm
E6//pRC1POp4bxTD9rA77PsO2zg1AGoB1AcW605X1Id+bixEDQtf+mprvM87
5BnkG+Qdc7LA6OBzsDe4G3s/rrOMawy+UQdcb/mu0RqPFkNneN7rOfvzUTOC
P8TYPY6zYHb8NHlBj3WJessRvfdhY/jN7dazOvui2eYtfJxt21lTCByA3LfI
V/yuKWIbmHmKz48ufWB9Jq6RGxPzNmYR//BLE8WPf2/60m632d+Ta4NRwCXb
9HxrU8RX8Hy/9eG47QMeUY8hTyNHGy2ZtKn9VRY1nqmu8xBPqEUSU4jF1AHI
uai75E3jF1+yLyIWXOTcmdhLDAYTUiOsbQv+gOGZg67Cv0PNwUOwIvh8sW0f
erXt5dfGD4xt9ZxO9VvUOowvmEOc2uY5+A1qltQur0+hu+gw/niK8T+xb0B2
dmYx4hk6iP10GGPcZf5TjwGX4g9v0PyjxajPEbPR69usG8zB3vCR+Erqjb/1
OZebV8QS/DO+ET9JfoF+5bPQMWrP1FDAoqx3wLU+dBweo9u3SwcK0qfpufCZ
+FjykW49X+r7ssaA18GP7jY/6/DfWegmvhAfh3+j7kK8Jnbj8x60X2NvbON0
zRyafIQ7dfpe4ExyD/hwWxY0Nsv77PEVr13t9aktUbem1oReHC6HbmDz4xxf
wFQXWk/Q/Y2WL/zE34C7J8DLLHwX58deiMvU2qhN4B/ajD+IR9TL95kP1Kyw
H2yH2i01PnAN9Wd6YicxlDwTep7O+GIxYg6+j3EwMLlVl/EYOoVuge2QQ2ZZ
kAtjd8SnZq3TpJbLh+7car6x3k3ei1ys21gRf0AN8fSdoPH/1S2BOcAbL9oG
0SvskrkT/S50n3nCM+hj9jf4nXqdZW0xvnt8y/iQ+AtWbHPsnmW8cY59LLLo
dv+s5UIufZ7vjnx+a93AfpALcRYbZj5rUOPi7vAAmY+3vYDNwf/IHD+CDpCT
HrEPw5fBW3Qdm5mv828pRvxm/6m2a/wCZ+ixXtxl3WiV7bSQB2ntq9V/rRQ1
NWog8ANs9v0s9JX5L9t28CfUl8lPTp+Ru3FOagWN9ofo9V7r9h7T6Bv9Ho/j
G4l52BN1kf4sah/ciVyBPIFchJyEevjx5qjdk29TJ2ecnGSRzn5ZKepxHzkW
jrTc51tPwBbUGcFNPJ/rOZyfu1DLPep30QliAPcH44J3Z3s++jXbOsberMN7
mX1O3naL70PPqdl3jIy8mFrOZPPwBscO9JMYWGyJOAjuwZ457+vgplLUfanD
U4MGu3CPCbb3jxzniWXo1EbrFT6TOgK+mhwOf45fn2J7xNdusW502L/ut5/j
TnN8rw7rEDYIfgNLkYfjO+usq/go8sEHHI+6rPODOvu6usgRx7iuQXzge8yA
vx+h7+g9Po+8knz5fsdlsBpYiHng1DHWSWRDPrXHOgVuJo+g3t3n3IIcCB0m
5yDeEcc/Np/wQeQ42OJQTc+yZk3qKqwDfgMnYN9gXWpxxI+CMRdxAFmAISd5
HJ3l/h2WP/yp85rvuF7D3F2e3299ILfijsSqaY5TOz2n4DXQJb4hU4snPyWG
1GSRa5I7UIubleIb6efF8Le91n9wFPU66ngzib/yFXPLEd+G4qpjGbV5cBDr
UK/Gx2Cna8xr+ExNg1oBtbJhxuDg6rx9UYdxLLYN5kEXOm3PPZ5zve19ku8O
viQ+6khDd6q2LJADvAOr7PdZOec7HkdG+ywX9ABb/b791VvmM7b2eF18N8WO
+S7d3hZzyZWQLfkMOfKLtmve3WcZUDcirhOb6KlrENPfMU1diH6vaXwJ/hF/
srMlvsnxG0yIHx/6ficc+N8tUUuhTvaO3+WbMLwlV8UGxprPv1D++6tRzj+y
8H34otO1Wt5dqIGFtVHDYR2+MUMTR/m9wvKttQ9ZXxv1ynqvcdDzd+pQG+ri
/OBTzg324OzEEM7f49/YO9gLzE2d8InaqNUiow7HHPwGMeY9332Hnp8oxR3A
mof8HZwa3gbX8fjGUmf7RUdmuP7A8/GlmIOONlufqRdmpaj99dk34keJ3+2O
3fgcbAwfAtYEcw7VsVPEd2pB15kGIw3a70xzLAOrFYwb9/p8fHMDN6E/Q7gv
CywMpqFuRU5S4rxqhy2Tw+YzOGm0+UOtgdpTg8/UaDzMd7yibX+lYyu1wYdc
t+DZKtv1DN8LelGK+7Qbj3Xax2Ob5KK7XE9eVR95L3GEv+MgPoAt+E4Fzuj3
+BH7YTAWc+rtM8Ga2Bp1H2I4cud7+5Rc5OPER2IIep4TTr+5FM9OfydDD5fY
z+N3FhgLsNfpbxgrvSbrIUN4Tg2C2gffZIjn53utzPH0Gs/BtpFblXUJP4Zf
w6dd5XHuyHcbfAFxgTsNOjagh2ea/1ebnmK5DLfcF/j8xAP+noX7dpo+Zn5e
UIq/7+HvAvDVmxyz1phGl5FjZn3ObKfQ2Han/cBS0+gsetNt+X7UHHWHdfYJ
XY6hecv8G9a3Luv5QsfCTtNgiC7fkXH4ANZlfKT5QyyEn5ea7jIPuSM82SKj
fKkcdZr7fTb8AxgRbAg25ls3cZeYek0p/haBbxCMf5q+xMbgl3MsR+af730/
8bunv01i34vsB6ibYfe9xk7fzAJbg6vJg7cZW1KrK/4/m91rfw4+Aic12B6p
AaJjyHO6x4mjHziPGNYSdU90n7kbPJ+e3+RUk70vMYizcKZ+/y3EB66VPVeM
v5ciHqHbvI9P4ZtUp+V1q++CXyI3ujmL3Ax7nGFcR32mZD6Ax8G1+HBiNPnS
IdvpIdd3qm03nJma/GjrCbzDpvB71E7OaYn6yQnXC8CN7M85yP92tUTMYC9w
MXqDzsCDn2Zf/j0ANHzlvDPNT3xpwRgSXNBjnAA+ZS/yF/KpE/4eCo6b69oF
vGi1b+fOJfs9fo+yboAx8RknrSej7FdZP++7bPYc4gL9G57/da8JT6jRzTWG
pFb5puMC3y/K5vn/AZjVoiA=
             "],
             VertexColors->None], PolygonBox[CompressedData["
1:eJwlln1o1WUUx89dM/dy7+7uy5a7L7t3FnOKteUsZpJpakG6yqlB9UdWYlCa
UJAaFASVptbmS4TNzWH2BomVE2RZSGlZf/myrK2ZlqBzFlo2c7nNPt8duAee
53fOc16+z3O+51Y8sbzh2Rwz24rkItGI2eNZs/uLzF4KmB1OmG3j29QCs3S+
2ZgKs4GUWe11Zk+HzcoyfEN+D5qdRiLoW4rN4vi6E/27UbMw9tWc/yWETxR/
4v8c/r/Ax3x8HygzeyrP7Mgos+no9sbM+tGdxD7L2Y+xmU2SM/g2RKxd+DyL
7gxSQbz2tNkgub5CvEHOD5Hzeuzz2S8hl3N8O028ZnLay7oLH3s428a3Hmz3
4/8+8qukvnHEq0RexvYevnWgP8WZo9i+QI5hztbis94ck272jyLv5LjPFOsk
cgYsziJvUm+QHKvJpQuAv8dfEn8h/B2iniMJx1jYliMF2A5T/0xsqzhTBTZx
7Edjf5WYs1jPRPjZLXwrxL6BGPPZT1T92NeRXwP7w+B5HsM/kPHYApsdp7Yo
Z6ZjW0C8DzlbxP529leIuRosNxN/WaHZUqQKXRP4VFNfMzabWX9FvsF8v9M2
Ym3Efh+1VOP/UtLs76RjHUU/mnh5KY+lNzEsbErMFpjf4QDnF/JtLOsS7rAI
243EqCFeDJ/NJH0FfTf6HqSTdRB9B7XtYv8DsVvxd5J9LTVeo9Y7OPMbvmLk
u431f5z5FVugsUHs86mpMuwYbyf+QeqJkN809QHnF5DoWnytQTYRayDjsZXT
NvTz0Dfmus089OtK/O0q5ilqP4n0qlZ8NqL/Ev+FrKvAYwfxDrEvyfc31Ek+
D+N/HP7r8LcS3/cSYxLrFmQ1vl9HNuN7+iivfTuFzMhzDFqwbc16rsK0ifOf
lZut4O2tQhLoPgWD3pC/uQUZx0xYCfMI/lrxNxV/09gHwD8HySc/zEawVw+r
d3UHu7EvA79nwG8ueBzE6AFs0iHPuTPlHKDelw9h3xb1XtYdFKb8TektRbnf
EwnvwZHeQ6ZE/Q51d3rTx/A/h/1lzobBvB78Ehm3fcO8t1+k5n253uPbWX9C
PQPsL+d6760XpuY9+CpYvlbiWKpH1Aut+OTTSE88iO9+9gUBr0m9P7bYe0cc
UMzdHqNHnif+Q8gj2A+irwn4Ha4l1n7OzEJ3nPh16KcgzxGvU/yFbSTpZ2s5
06H3mfLeF6feim1NxrlBb/oufDXFnCsnEP82agtzfiH2kwJ+d+I0cZnuUFza
EPHeF6cuxteTGc9lMvld5OxK9CU5zlljqf/GCudOcW4uvtojfreXsJ/D2R8z
3mvi1Mnkf4EYa/Ic00VZz0m51JPDY+wvxHw9t8i55QMweTvgHHMx5jbSacZc
z74c/31Bv+OJSZ9Bmj3CeCjmnCguXES83bztz8sdO73xr9HfnXUurFG/xpzD
xF3iTM2arcX+ljVzxM1/gXl/yDk6xnoYm+Kw53gt4Zw7wrXE38n6vbhzo97U
AWwPIjuL/I2sQtde7r2mNyhu7OY++wqcI0Pk/n7c32ItMSakfGZqVmrmfIev
RvQ9uX6n6lXNOM029exE1jdnnEvE+co1GnUslLOwyyDng45hKfGOpr1XFqPf
QOGzs46NOPof8O1POjfrzWY5W1/qvSQf32K7Lu6zaleRc+VVfCw350zN+hNR
fxua+eKiUMTfljhpR9xrVq3iLHFFWdpntzijN+6cJ65TT5wAj56Uc7MwG0/8
TWnvlTHYxDSvUj5bxNEbiBVM+WzQTEpgm0z7rFaO4tou9L0Fzrk3UVs647NY
M/0ndD+nnIvV898Q762416o7XRLx/xD676D/FDdgW5ry2SDOXhFxjIWtemh+
3GewZq9mwr/E2hJ1blMOa9LO6eJyzdSr5DeY8P8SmhEfRfw/lf5L6T/ONGLv
iXmva2Yvxd+yjHOz3kgfd3cu6bNSPai3XJV1rPSm/wfDOFPQ
             "],
             VertexColors->None]}]}}, {{}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           TooltipBox[LineBox[CompressedData["
1:eJwl1Flsjlkcx/FDJkOre6n2ra3GToNYoi6GWCJRy7S4sCRjiyvDpSWZO8PE
WurGVFupPSH22JfQ2iIzaG2luJrYFSOUtubzxMX3/Z3/ev7nvM/z5MxdXLio
RQihwM+ILiEcSw/hZ/oKL5GSE0JJSghd8ZR9KhbCg6wQGulX1PM/yRbDY+R2
DqEfyvlzMVrNGIzCRb3H0o10E7aJ90dahxDSUalfFSr4BiKDrz2u8F3FDr5B
SNY/xl+eGsJN/lv4wj6VZiaznaQN7Eyz75Hzt/y9tAvfDbGsjhA7wveP2FH6
g7nqxDLZWegj9wL/I73r8Bgx/on2n5ihl3VndMKPzpKvvpre0vs2vU0z7LFE
j/NyltJ69zPLeiZq8QAn1JWZOVX8N/ZCbGjrzPxJ8pMxRHw2+62+v9J6Ooem
qekn3to+d/mq7ZnCTkWTOYeqm4zDnZwVh1Ck91G6jC7Hc30WyC1w3vfqPuAd
PuI/lOg5ONP/LWeG2aajxl7P1E7iy2MPQ7G83nq9Q69I3WU2fwd0RDd31ldO
D/v0RHdks2N4g696xjlHI21AM+LZTbTYXmn6NeubHvV17l18SeK76Xbsd85F
5t1HK9g7kSi+g8bkr2wXwipcYf9BV2CY+Rvlr7YulFNpPYWuYRfQ4fb6Zs88
mie3UG0bPafQApSLTaVbo/8QpRgu71/1zc58QF0TPUhjYtm45jkawDcQpfLK
cN0Zp7KnoUH9gfYhrHUnz53nBd6Y6y0Gi413b2vtmWCOdXQNLuu7nl6iVahE
rvy++KjmF30no8o+48SWyv1T/83sVbSYto3+N/nfzNeMC+zV8uapm49E91Eh
nkBr2HeQjwlokt8YPUdiJ6P3Sc8c8/2Erjiufyt6go6Ut819fIm+EbTW+R85
Ux0eohbn9NusV7L1X3I+yy2J3unoOeNP5N9Ek2g8XwLdyI6nRbSXvdpYx0Xv
mZm3qP0UvW98Z/XeICfO+j7uoSz6BkXvkPUZ8da0Fe6gVCxVrMb6tFhL2gLV
2Cr22nl+d09F6d+/n/8DLDbQBA==
             "]],
            RowBox[{"-", "9"}]],
           Annotation[#, -9, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           TooltipBox[LineBox[CompressedData["
1:eJwV00lsDVAUxvEj0VaQGjoYW2MlEiR0IoiFDTG2CRFjI0EQLUmtDLG0MYYa
YihlK0qKUpJaNOZY6ECrWpWQGIuIiOn3Fv/33XvOd8499773Rq0tKy7tERF9
feT0j+gzIqJtSMSpgRHNGRFNGCf+UqzF+uiAiMpBERW0Y3jEGb52epq+ou08
XSOtaVUipq4DnahNi7iWFVGDd/Yb9Gh2Xg5fEx1L69XexTr79diABbwL8UH9
YrliFKHbXOPFK8Qz7AchEzW4ZMZU/b9Zf5TvOSziB/8E/mI9i5Ak1iC3imc1
lmMlVmCeuxTwf+XvTpzN+x77zH5Z72o8VHvffi7/HHzCRTX5eGP9YGjEF7XV
7vVZvyrzvPMuf8X+JBD7jT/pETf1Oy//Vr6O/6C+p/XfT0/RA3QJ30kznMBx
7OGrVV+mLoueo8NoJR1OV8jn8k1Bif10+9fu3Yk8se20HDPEu2i2mbMwArft
S/Wpo1voA7GHiTuZYa8ZZ4tddYcqXECD+I3siOv4Zd6NvJswEmfsR9Gz9JB7
9PYuyZkRT/Wu0Wep+lQ9+yFdLkeuXrzX4IjBYrlq8jAF3826SP6JfKF9Lz1T
9C6wnpm4C+54/2VoFGvCMzTzlei3BlOdd0Xfq7hl7mp6GdN8B+XqtplhK465
SwVa1KZ7rwxkolD9P7406265UmeWYSvum+Ue0sz3nL5AC9rQijvOK1K/GNPx
V58jzjiK1+jEDp5r9DCtoYcSvw+1O2ktrcMtTPQWrc6cRNvobmduNvcu+tNs
jWab5e5Zif+NeDbd7u3KUWCfjzykiOfzPVaX630e0fk8ydZJGG09Bl+cM5kv
2dlJ+A/Qiqhm
             "]],
            RowBox[{"-", "10.8`"}]],
           Annotation[#, -10.8, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4759, 6619, 4760, 5589, 5588, 4132, 7173, 7174, 7171, 
             4494, 7172, 7944, 7168, 4493, 7165, 7167, 8417, 7166, 8418, 4131,
              7943, 8480, 5043, 8253, 5044, 7603, 8033, 7231, 4641, 7230, 
             4640, 6616, 6615, 6614, 7130, 7129, 7164, 5523, 4332, 6904, 6903,
              4639, 5517, 5516, 4250, 4129, 5340, 4643, 6906, 8010, 4335, 
             5526, 7184, 7139, 7140, 7137, 4453, 7138, 4648, 7235, 4649, 7236,
              8040, 7618, 5060, 7616, 5059, 8262, 7617, 7402, 7403, 7185, 
             8136, 4507, 7188, 4781, 7404, 8139, 4509, 8140, 8141, 7193, 7192,
              7406, 5604, 5605, 4795, 6650, 7954}],
            RowBox[{"-", "10.8`"}]],
           Annotation[#, -10.8, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           TooltipBox[LineBox[CompressedData["
1:eJwV01tMz2EYwPEns5ELc4pEpTGnRq0b5hhmyLmsrtjExibaMm7dOG+4cdxs
iMlpLjAmKuZCcxpudLCSCospNTcYPv+L7//7Hp73eZ73/e2fVVpeuCMpIvr7
KRkSsWpkRF5qRAXvxAbj9WgaGpGfHpE5PGJtWkQhinBWzITMiPEYOC5itv1B
nIx+WRFJGDYm4klKxHTjffI8HR1Rj58j1FBz1qiIAzxZjmY1WuRo4lZ+z1Vc
IL6Nz/EHXsZTxBeNjcgZph/O5XW8X40KvX1Hlz52yf0avc6Vmm/GJjSbj9Jb
KsKZY+45R94jfDRxZ/vv1G/kBt4iRz3mqTsfC6wtwkI0Oz+Tk8XOVDfL/ifu
RDtyE31itfP3MED+Xv1me78+nswT0eoteuT5gW78k6+Xt+m3j5O82Xbj4DK+
nxFx0niccyPkuan/FM4yP6yHcnfbre4u5InvsvYVB3EIe63twynj0ziDPXKc
5BM4jtvyvxHTxue5mKusX8FVHBZ/g6/jGkrsXxC3RG+l6k7z3efqaaNxp/0O
zPMGy+wXYClWYDlqxK3hf3KGd7si7rdzGfgkZya/tf6XQ94MpKPWuST7dRze
6ZLv0SPmB/rkep74FvZnyHcZ3cY9KMBLb/zMmVd8x7vdRq4zt/iX2GL9JnvH
F/bvir+Hwdby7F3EdOMcPEA1ytRerG62XhrMp3KaHr+p8V2OWn7ENfyY6/gE
V3MNL5D/IU9w5qu1deZFyE+8gT4+oxNb1fnCK9UI/s1/Et9FvQ6xVdzO7f57
bXzBvMW9Kvmj+SnnK93xIt6LacYk92nkc9b+A41EpJw=
             "]],
            RowBox[{"-", "12.6`"}]],
           Annotation[#, -12.6, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4363, 8034, 4660, 8160, 4642, 5520, 4451, 6046, 4757, 
             5522, 5521, 5587, 5518, 5519, 5099, 6475, 5338, 5339, 4638, 8182,
              4758, 8179, 8181, 8180}],
            RowBox[{"-", "12.6`"}]],
           Annotation[#, -12.6, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           TooltipBox[LineBox[CompressedData["
1:eJwV00dsjwEYx/GnNRJBbFpatLESItHaq0LdtBK0JeEi1sGOtImTEar+PZAQ
eyY1qq1EnRAdKhyIiKs9wqUkHZyoz3v45fvs93mf9//P2bhr1c60iPhLmwdH
lI+MWJ8RcR4vUDU7Rd+HRJwYG3GSto+O2EF71VfJHcEU/+2wiMwxERvGRaSp
bxsRkY7d4lvGR2yjrdTJX5UZsUfdZn4Xf1JOxAr2Z3YRfsF32RG3cbXa33gV
l5q5jP7w2+Un6vvKLtbzDd+L1eIStZXDI+rte5caqER8kLpS3KSuw27FuIS/
z/6/aKKdyr3Pa1rGXkr99SxWMwALkv30/NC7HHP5pfpy1U2z11QqFB8vPkd8
DW0zq8It54nnZUXk01fxV2bMYHcP1Sv/hN/PPVtxMv+9+Frv8QE/jtLLnkFl
lK2mItmVUvY6Tou8bzXekD/NbsFTWINX7FUlt4B/DBfiYfFDVGjWATxI88WP
ys/DShybfD+c47t24VycRbMpS26//dvN/knXaLdYgV0/6y9y82K6oLbenmvk
HsqV4CWxi7RW3XVzV6rLMatW7o3bTGDP0pOLBe7XaXZv9+tDPWKtbtKX/U9/
DzXz0/gt2AtrzOlS18jv9l4vzHxJM828aV4e5lN4dg/eU9ep/rK+dmzgt+tL
mZWhJjO5Mfu87zpd/Bw+5T/HJ/gM23CgWS2e00xNdFa8SvwMPsImfIiP8b6Z
FXZvxPLkt+o2dck356+jEipLfq/Jf8bOZ9ygA9PdLY0emFGn5w7domNuWEWV
lKLjdNb71MjdoE/6P9IUM94l30vsP2g6m0s=
             "]],
            RowBox[{"-", "14.4`"}]],
           Annotation[#, -14.4, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{5237, 6556, 5256, 6561, 8009, 8008, 5257, 8383, 5042, 
             6472, 6471, 7856, 7657, 7658, 6950, 6951, 4361, 6476, 6949, 
             6948}],
            RowBox[{"-", "14.4`"}]],
           Annotation[#, -14.4, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           TooltipBox[LineBox[{4664, 5950, 5949, 6613, 6612}],
            RowBox[{"-", "16.2`"}]],
           Annotation[#, -16.2, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4772, 6052, 5602, 4137, 7948, 6627, 7821, 7820, 4774, 
             7400, 7401, 6630, 6631, 4139, 7609, 5051, 7608, 7607, 6633, 4457,
              7234, 4647, 7233, 4646, 5528, 4458, 7622, 5065, 6918, 6919, 
             4343, 6442, 4344, 8019, 6920, 7624, 7623, 4459, 7627, 7628, 6922,
              6923, 6924, 6447, 8024, 8023, 6925, 5071, 5530, 5529, 7629, 
             7630, 6647, 6646, 6636, 4788, 7413, 5372, 7415, 4790, 7410, 4789,
              7414, 5609, 6658, 6657, 6644, 4804, 7431, 5376, 7434, 7435, 
             7426, 7433, 7432, 5626, 4160, 5621, 5623, 4803, 6076, 4524, 5620,
              7958, 7203, 4814, 7452, 5380, 6089, 4523, 5617, 4813, 6084, 
             7990, 8226, 5024, 8228, 8227, 8453, 8452, 7070, 6019, 8495, 4738,
              6539, 4737, 6018, 5536, 6869, 6868, 7727, 5182, 5616, 5615, 
             4407, 5952, 5953, 4700, 7287, 7289, 7288, 7294, 4164, 5347, 6581,
              6580, 4465, 8315, 5163, 8313, 5162, 8314, 4408, 7890, 5289, 
             7889, 5290, 7891, 5288, 7888, 6871, 6872, 4165, 5673, 7677, 5674,
              5122, 5401, 7669, 4280, 6693, 6692, 5713, 4560, 6197, 5411, 
             7482, 4876, 7220, 4875, 5752, 4572, 6241, 4897, 5756, 5755, 6245,
              5790, 4215, 6773, 6741, 7506, 7505, 5431, 7509, 4931, 7500, 
             4930, 7508, 5822, 6801, 6800, 6771, 4951, 7525, 5444, 7527, 4953,
              7523, 4952, 7526, 7655, 7654, 4239, 5371, 5096, 6945, 6946, 
             6947, 6465, 8032, 8031, 6944, 7653, 7652, 5370, 7650, 7651, 6942,
              6943, 4359, 6463, 4358, 8030, 6941, 5095, 7649, 5368, 5369, 
             4657, 7253, 4658, 7254, 4491, 6823, 4490, 7162, 5085, 7641, 7515,
              7516, 6796, 7518, 7517, 4924, 7826, 7827, 7520, 7823, 7822, 
             5789, 6767, 6766}],
            RowBox[{"-", "16.2`"}]],
           Annotation[#, -16.2, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           TooltipBox[LineBox[CompressedData["
1:eJwV0VtMz2Ecx/EnqVnOhwrlkGXDFFvOhwubHDYq1ZBTxIxYOWxm6M5lDnfm
sDmELccrYXOaVNh0srkhmbPWBbGKxrz+F+/f5/t8n8/3+32e55dSVJpbEhVC
iPfJGxrCplEh5NMiOp8W0lx6ZGQIR3EMn63fybfiyZgQalGDnqQQkpCMmMFq
k0OYOUQfGkXn0V60Pj6EF2hRk2G9Sr4AqxGvLk19On6YtZDnw6AQGvjfilux
hm8d1mKv9R78HRvCFDXN4peI1reJZqJRbbm++SNC6Hb2Zusm9JGrHu4OaJP/
6j7faLZex838nhDCdrlibMNO7ECm2ho1P9U/4tnnnPv5m9Ff7QBk8y3QKy/S
k+8sX6p1GR1Pt6pP06eIttg/ID9HnzV6PESqvTi+HPnZ8gVyueI4/i49u9GJ
P/iNUjNL8E/ce1gIu8XRdBe9NzqEE+JxiSEkeJeb3iGRlnm/jfQQLaQzeWdh
Gq6ZnUFnYDrW22/3lhvoQf4z4kXOfVivTHra+lTk7Xmb8dG8Febm4rz/csPe
TZTzX7SuwAUU8FXw5/Hl46xcl/U5+sq9t5i1GYvc/4jaOnd4ig7v0+AsMXSl
umv2a+Rr0S133ZkaxbHietrI2ynuq2cPvWG/n7eqke8tN9TZ5upRifv8D9DG
dznyj8VR4oH8gxDDH4t5vO3ydyP95Ru9WQNtolVyt9DDc5rnDG6bdUeu1Z3f
oVaP+MhbmvHc/GeY5N7baTGy7eVgGbKwHFP1O49qtY8xkX8CBuqxxP7SyFth
slmLaTrvW7NO2a/T8yRNTPEf7SXT6XxJNMveL96OyL+2F61nC27L38Eb8Ra1
r2mV+33muaL2k39ylX6xvhRZ00r6Xv4j/gNMV6uG
             "]],
            RowBox[{"-", "18"}]],
           Annotation[#, -18, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4230, 6282, 6337, 6591, 6592, 6606, 6605, 5315, 7912, 
             7913, 6938, 8463, 4355, 6338, 4356, 5579, 5580, 5311, 5578, 5577,
              5947, 5948, 4662}],
            RowBox[{"-", "19.8`"}]],
           Annotation[#, -19.8, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4374, 6485, 4375, 6961, 6960, 8454, 5035, 8241, 8007, 
             8006, 8479, 4983, 8208, 4984, 7536, 4612, 7533, 4955, 6805, 7978,
              7979, 4315, 5446, 4954, 5823, 6802, 6803, 5825, 7510, 4933, 
             7504, 7512, 7511, 5432, 6288, 4586, 5793, 6745, 4201, 4297, 5422,
              4899, 5753, 6743, 6744, 5758, 6200, 5719, 5720, 6199, 6198, 
             4561, 5715, 4854, 7675, 7676, 6973, 7866, 8059, 4386, 7867, 4385,
              5498, 5125, 6975, 6976, 4389, 7729, 5538, 8339, 8340, 8338, 
             8342, 8341, 5184, 6534, 7046, 6533, 5183, 8336, 8337, 8333, 8335,
              8334, 4387, 5955, 4388, 8060, 6974, 4673, 8056, 8438, 8057, 
             7668, 8058, 4383, 7354, 4384, 6971, 6969, 7270, 7269, 5351, 7765,
              5210, 7078, 8112, 8113, 6090, 6091, 5212, 7767, 5211, 8356, 
             4167, 5639, 6095, 4817, 5625, 4818, 5383, 4269, 6092, 6093, 5628,
              4527, 6100, 5384, 7455, 7456, 6659, 6668, 6669, 5630, 6079, 
             4808, 5611, 4809, 5378, 4266, 8194, 4806, 7437, 4807, 7441, 4519,
              7416, 4792, 6649, 6862, 8489, 6863, 7952, 8220, 5019, 8451, 
             6955, 8043, 4368, 6071, 6557}],
            RowBox[{"-", "19.8`"}]],
           Annotation[#, -19.8, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{6057, 6055, 4138, 7908, 7909, 7907, 5308, 7910, 7911, 
             6905, 4333, 6058, 4334, 5524, 5525, 5313, 5341, 4140, 5944, 6635,
              6634}],
            RowBox[{"-", "19.8`"}]],
           Annotation[#, -19.8, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4157, 7953, 6663, 6648, 7443, 7442, 5379, 7446, 7447, 
             7438, 7445, 7444, 5634, 4161, 5631, 5633, 6082, 6081, 4528, 7457,
              7458, 7453, 4819, 8195, 4270, 5385, 4821, 5629, 4820, 6099, 
             5646, 4168, 5641, 5643, 6098, 6097, 4536, 5640, 6677, 7768, 4835,
              6114, 6452, 6451, 7769, 4438, 7079, 7077, 4739, 7355, 4466, 
             7209, 4535, 6970, 4534, 7208, 4705, 7295, 4706, 7296, 4671, 6519,
              4670, 5954, 4672, 5956, 6873, 5350, 5349, 7728, 5185, 7047, 
             8093, 4423, 6521, 4424, 8094, 7714, 5166, 7713, 5165, 7572, 5025,
              6972, 5026, 7573, 6694, 5718, 4562, 6490, 5413, 6201, 4878, 
             5721, 4877, 5759, 4574, 6243, 6244, 5763, 5762, 6249, 5794, 6776,
              6775, 5792, 4932, 5433, 4307, 7975, 7974, 6778, 7514, 7513, 
             4596, 6342, 6343, 5827, 5826, 6346, 5873, 4241, 6824, 6804, 7540,
              7539, 5459, 7544, 7545, 7535, 7543, 7542}],
            RowBox[{"-", "21.6`"}]],
           Annotation[#, -21.6, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4362, 5484, 5309, 5487, 5486, 4376, 5492, 5493, 5314, 
             5491, 5490}],
            RowBox[{"-", "21.6`"}]],
           Annotation[#, -21.6, "Tooltip"]& ], 
          TagBox[
           
           TooltipBox[
            LineBox[{4372, 5485, 5312, 5489, 5488, 4377, 5496, 5497, 5316, 
             5495, 5494}],
            RowBox[{"-", "21.6`"}]],
           Annotation[#, -21.6, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4810, 6083, 4267, 7957, 7956, 6665, 4811, 7448, 4529, 
             5635, 6670, 5632, 4822, 6105, 5386, 7462, 4823, 7454, 7460, 7459,
              5654, 4169, 5648, 5650, 6102, 6101, 4538, 5647, 6678, 5642, 
             4837, 6116, 5392, 6120, 4537, 5644, 4834, 6113, 7991, 8229, 8231,
              8230, 7570, 7080, 7081, 7082, 6022, 8440, 6023, 5213, 6021, 
             6020, 5539, 4273, 6874, 4833, 7468, 4832, 7469, 7731, 7730, 4409,
              8316, 8317, 7715, 5169, 7716, 5167, 6520, 7027, 6522, 5168, 
             7720, 5170, 7717, 7719, 7718, 4410, 5923, 7026, 7025, 7024, 5164,
              6879, 4281, 4176, 5716, 7687, 5717, 5127, 5414, 7680, 4288, 
             6716, 6715, 5761, 4575, 6247, 5423, 6250, 4905, 5764, 4904, 5795,
              4587, 6289, 6290, 5799, 5798, 6296, 5830, 4232, 6806, 6777, 
             4957, 7534, 5447, 6347, 4960, 5828, 4959, 5874, 4613, 7541, 4986,
              7537, 4985, 8209, 4324, 6826, 6825}],
            RowBox[{"-", "23.400000000000002`"}]],
           Annotation[#, -23.400000000000002`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4162, 6673, 6664, 7464, 7463, 5388, 6109, 4530, 5636, 
             6104, 6103, 4271, 6672, 6671, 5658, 4824, 5655, 7461, 5653, 6680,
              6679, 5649, 6125, 6124, 5394, 6131, 4539, 5651, 4836, 6115, 
             4274, 7777, 5393, 4838, 5645, 6118, 6117, 6543, 4177, 7271, 7272,
              6875, 7084, 8441, 7083, 7773, 7772, 8357, 5214, 7086, 7087, 
             7088, 6542, 4439, 8114, 7085, 7771, 7770, 6695, 7149, 8126, 7150,
              7049, 7048, 4425, 5960, 8249, 8250, 7595, 7597, 7596, 4707, 
             5997, 6898, 5998, 4708, 5475, 5474, 5961, 4632, 7879, 7880, 7877,
              5275, 7878, 8308, 5130, 8309, 5131, 7688, 8306, 8307, 7684, 
             7685, 7681, 7683, 7682, 6202, 5766, 4202, 6746, 5760, 4902, 5424,
              4298, 6749, 6748, 5797, 4588, 6294, 5434, 6298, 6299, 5800, 
             4936, 6297, 5829, 6807, 5831, 4958, 5835, 4956, 6344, 4316, 4233,
              6808, 5876, 4614, 6401, 5461, 7551, 7553, 7538, 7550, 7549}],
            RowBox[{"-", "25.2`"}]],
           Annotation[#, -25.2, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4825, 5389, 7465, 5387, 6110, 4827, 5637, 4826, 6108, 
             5662, 4171, 6684, 5657, 6135, 4541, 6106, 4540, 6126, 6127, 5660,
              4840, 6123, 4276, 6681, 5395, 4841, 5652, 6129, 6128, 5679, 
             4178, 8360, 8361, 7778, 5218, 7780, 4839, 6544, 5216, 6119, 5215,
              5354, 5353, 7273, 4674, 6978, 6980, 7274, 6981, 7357, 7356, 
             8062, 8061, 6977, 5272, 7869, 5126, 7775, 7776, 7774, 8359, 8358,
              5540, 7324, 4391, 6987, 6985, 4677, 7327, 4726, 7325, 4725, 
             7326, 4727, 7328, 4678, 6988, 6990, 4393, 5924, 4392, 7914, 7915,
              5317, 6595, 6594, 5186, 5501, 4390, 7872, 7873, 7871, 7870, 
             6983, 5129, 7686, 4879, 6203, 5765, 6747, 5767, 4903, 5772, 5771,
              6253, 5802, 4217, 6779, 5796, 4934, 5435, 6292, 4309, 6782, 
             6781, 5834, 6301, 6300, 4597, 6345, 4598, 6357, 5837, 6355, 6354,
              5877, 6828, 6827, 5875, 4987, 6402, 5460, 7552, 5462, 6829}],
            RowBox[{"-", "27"}]],
           Annotation[#, -27, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4242, 4325, 8210, 8211, 7546, 4988, 7548, 4615, 5878, 
             4963, 5836, 4964, 6356, 5448, 6350, 6351, 5833, 5832, 4218, 4308,
              6291, 6293, 5807, 4935, 5803, 6780, 5801, 6254, 4908, 5773, 
             4578, 6248, 4576, 6204, 4880, 5770, 6717, 7584, 7585, 6982, 7581,
              8063, 7580, 7679, 5128, 7876, 5274, 7898, 5273, 6567, 5301, 
             6584, 4426, 5513, 5300, 7875, 7874, 4467, 6989, 4468, 7151, 5027,
              7275, 4676, 6984, 6986, 5957, 4675, 6489, 5499, 5500, 4710, 
             7300, 7211, 7212, 6979, 7214, 7213, 4469, 6024, 4470, 5544, 7089,
              4440, 6121, 4441, 7092, 7090, 5217, 7779, 6696, 6697, 5678, 
             6130, 5680, 4842, 5683, 6682, 4170, 4275, 6122, 5659, 5661, 6138,
              6137, 5396, 6136, 4843, 5656, 6683, 5663, 4542, 7466, 4828, 
             6675, 7959, 4163, 4272, 6107}],
            RowBox[{"-", "28.8`"}]],
           Annotation[#, -28.8, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4172, 6688, 6674, 7471, 7470, 5398, 6145, 4543, 5664, 
             6134, 6133, 4277, 6686, 6685, 5693, 4844, 5688, 5687, 4179, 6699,
              5682, 6161, 4550, 6132, 4549, 6158, 4856, 5685, 4855, 7575, 
             7576, 7091, 5028, 7578, 7574, 5219, 7094, 7093, 5543, 4740, 5542,
              7358, 5541, 6881, 6880, 7732, 4548, 5677, 7028, 4411, 5958, 
             5959, 4709, 7297, 7299, 7298, 7301, 4187, 5355, 6583, 6582, 8129,
              8128, 7721, 5172, 8318, 5171, 8487, 8077, 8078, 8076, 5293, 
             8400, 5292, 7583, 8237, 8238, 7582, 6887, 6888, 4188, 6718, 5769,
              6205, 4577, 6251, 5425, 5427, 6752, 5806, 4909, 6255, 4589, 
             6295, 4591, 6312, 5809, 6310, 6309, 5838, 5840, 4962, 5847, 4961,
              6348, 4317, 4234, 6809, 5880, 4616, 6405, 5464, 7556, 7558, 
             7547, 7555, 7554}],
            RowBox[{"-", "30.6`"}]],
           Annotation[#, -30.6, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4845, 5399, 7472, 5397, 6146, 4846, 5665, 6143, 6142, 
             5699, 4181, 5690, 6687, 5692, 6140, 6139, 4552, 5689, 6698, 5681,
              4858, 6162, 5402, 6166, 4551, 6157, 5684, 6156, 6155, 7995, 
             7994, 8234, 8236, 8235, 7577, 7095, 7096, 4442, 6025, 8442, 6026,
              4741, 7359, 7361, 7360, 7363, 4189, 8404, 8405, 7899, 7901, 
             7900, 5187, 7735, 5189, 7733, 5188, 7734, 4412, 5506, 4713, 7030,
              7031, 7032, 5999, 4414, 5545, 8080, 8079, 7033, 4714, 7029, 
             4413, 5928, 8075, 5929, 5291, 5926, 8399, 5925, 7699, 7689, 4203,
              6750, 5768, 4906, 5426, 7692, 4299, 4204, 6753, 5805, 6256, 
             4590, 6306, 5436, 6311, 4940, 5808, 4939, 5839, 4599, 6352, 6353,
              5846, 6349, 5845, 6362, 5882, 4243, 6830, 5879, 4990, 6406, 
             5463, 7557, 5465, 6832}],
            RowBox[{"-", "32.4`"}]],
           Annotation[#, -32.4, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4244, 4326, 6403, 4989, 5889, 4991, 5883, 6831, 5881, 
             6363, 4968, 5848, 6366, 6364, 5450, 6358, 4600, 5844, 6785, 6786,
              4311, 6303, 5437, 4937, 5804, 6783, 4219, 5811, 6258, 7694, 
             7695, 7693, 7698, 7697, 4907, 7691, 5135, 6751, 5134, 7690, 5140,
              5927, 5141, 6498, 4633, 5970, 5476, 5477, 4712, 6001, 6899, 
             6000, 4711, 7599, 5037, 7598, 8252, 8251, 5969, 4427, 7050, 5547,
              4473, 6585, 5356, 7364, 7365, 7099, 8115, 4443, 6027, 7102, 
             7101, 7100, 4743, 8175, 7366, 7367, 7097, 8443, 7098, 6882, 7278,
              7276, 4192, 6545, 6163, 6164, 5686, 4859, 5403, 6700, 4283, 
             6160, 4857, 5695, 4553, 6178, 5405, 6171, 6172, 5691, 6702, 6703,
              5698, 6144, 5700, 4847, 5704, 6689, 6690, 4278, 6141}],
            RowBox[{"-", "34.2`"}]],
           Annotation[#, -34.2, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4182, 5701, 6691, 5703, 6148, 6147, 4554, 6175, 4862, 
             5706, 4861, 5406, 6173, 5404, 6179, 4865, 5697, 6176, 5694, 6159,
              4282, 4180, 6720, 6701, 7787, 7786, 4860, 6547, 5222, 6165, 
             5221, 6546, 4191, 7277, 5359, 4679, 5549, 6993, 7279, 6994, 4742,
              7362, 8064, 6992, 5276, 7881, 5133, 7782, 5220, 7781, 8363, 
             8362, 5546, 7329, 7330, 7004, 7003, 7001, 4683, 7333, 7334, 7331,
              4728, 7332, 7336, 7335, 4684, 7005, 7007, 7008, 5930, 4396, 
             6492, 6493, 5137, 6995, 6996, 4395, 6252, 6999, 6998, 6262, 5139,
              6259, 7696, 6257, 5810, 6784, 5813, 4938, 5820, 6305, 6302, 
             4310, 4220, 5841, 6811, 5843, 4966, 6359, 5449, 6365, 5452, 6813,
              5886, 4969, 6367, 4617, 6407, 6408, 5888, 6404, 5887, 6416}],
            RowBox[{"-", "36"}]],
           Annotation[#, -36, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4995, 5890, 6419, 6417, 5467, 6411, 4618, 6368, 5885, 
             6814, 4235, 4318, 5451, 4965, 5842, 6810, 6812, 5849, 6320, 5818,
              6304, 5819, 6308, 6307, 4592, 5812, 4910, 6497, 4911, 7587, 
             6261, 6458, 6457, 6494, 8066, 8065, 6997, 5277, 7882, 5138, 6496,
              7053, 7051, 5191, 7739, 4474, 7006, 7159, 7158, 7586, 7280, 
             7281, 7000, 7002, 5964, 4681, 6491, 5132, 5502, 6991, 4394, 6003,
              5722, 5723, 5551, 5725, 4476, 5962, 5358, 7783, 7784, 7104, 
             8116, 8117, 6167, 7917, 6168, 5224, 8364, 5223, 8365, 7785, 6719,
              6721, 6722, 6601, 6602, 6177, 4863, 5696, 4864, 5407, 4284, 
             6169, 6170, 5705, 6174, 4555, 6189, 5409, 6183, 6184, 5702, 
             6704}],
            RowBox[{"-", "37.800000000000004`"}]],
           Annotation[#, -37.800000000000004`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4867, 5410, 6185, 5408, 6190, 4868, 5707, 6187, 6186, 
             5734, 4194, 5729, 5732, 4866, 6181, 7221, 7923, 8414, 8413, 7922,
              5333, 6608, 6607, 6548, 6549, 5320, 7916, 7919, 7918, 6207, 
             7788, 4444, 7105, 7103, 4745, 7368, 4475, 5724, 4564, 5550, 4563,
              6004, 4716, 5727, 7036, 5966, 4682, 6002, 4680, 5963, 5968, 
             5967, 4289, 4190, 5357, 4729, 7052, 7736, 7737, 5190, 7738, 8096,
              8095, 5303, 8406, 5302, 6524, 8492, 6523, 6578, 6577, 5136, 
             6495, 5029, 6260, 5030, 7588, 6754, 5817, 4593, 6317, 5439, 6321,
              6323, 5821, 4943, 5850, 6325, 4602, 6361, 4967, 5856, 5855, 
             6375, 5892, 4245, 6833, 5884, 4993, 6412, 5466, 6418, 5468, 
             6835}],
            RowBox[{"-", "39.6`"}]],
           Annotation[#, -39.6, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           TooltipBox[
            LineBox[{4246, 4327, 6409, 4992, 5901, 4994, 5894, 6834, 5891, 
             6376, 4975, 5858, 6378, 4604, 6360, 4601, 6324, 6327, 5854, 6790,
              5441, 6322, 5438, 6316, 6318, 5816, 6788, 5814, 4205, 4300, 
             6891, 5294, 7895, 5296, 7892, 7894, 7893, 4416, 7722, 7723, 7161,
              4479, 7038, 4480, 7310, 7311, 7307, 4717, 7306, 7309, 7308, 
             4715, 7035, 8081, 8082, 5965, 4415, 5726, 5728, 5192, 6889, 7902,
              4290, 5548, 6028, 4744, 6033, 4746, 7789, 5226, 7106, 5225, 
             6208, 4882, 6597, 7486, 7485, 7483, 5323, 6206, 4881, 7928, 5326,
              7927, 7930, 7929, 5325, 7925, 5324, 7926, 7924, 4565, 6180, 
             4566, 6215, 5731, 6726, 6727, 5733, 6188, 5735, 4869, 5741, 6705,
              6706, 4285, 6182}],
            RowBox[{"-", "41.4`"}]],
           Annotation[#, -41.4, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4195, 5737, 6707, 5740, 6193, 6192, 4568, 6219, 5736, 
             6725, 5730, 4885, 6216, 5415, 6220, 7223, 7222, 7832, 5246, 7833,
              8496, 7834, 7839, 8381, 7838, 5247, 7841, 5249, 7484, 5248, 
             7840, 6211, 6212, 6210, 6209, 4445, 6030, 6031, 4748, 6029, 4747,
              6032, 6036, 6035, 4206, 8407, 5304, 7903, 5305, 7904, 5193, 
             6535, 8097, 7056, 5194, 7740, 7037, 7034, 7303, 7302, 7600, 7304,
              7305, 7039, 8083, 5934, 4634, 6576, 5295, 5932, 5931, 7704, 
             7700, 4221, 6787, 5815, 4941, 5440, 6314, 4313, 4223, 6791, 5853,
              6326, 4603, 6372, 5453, 6377, 4976, 5857, 4974, 5893, 6380, 
             4620, 6414, 6415, 5900, 6410, 5899, 6429}],
            RowBox[{"-", "43.2`"}]],
           Annotation[#, -43.2, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{6732, 5739, 6225, 4569, 6191, 4567, 6217, 6218, 5743, 
             4884, 6214, 4293, 6730, 5416, 4886, 7835, 7836, 7793, 8490, 8491,
              8467, 8469, 8468, 7837, 4291, 4193, 8424, 6890, 6724, 4883, 
             5974, 5973, 7108, 7107, 7371, 4750, 7110, 7112, 7113, 6034, 4446,
              8118, 7109, 7370, 7369, 5360, 6586, 4481, 5557, 7054, 4428, 
             5980, 7601, 5038, 7055, 5039, 6006, 6005, 6901, 6900, 6902, 6007,
              6008, 5479, 5478, 5984, 4635, 6509, 5151, 5933, 5150, 7701, 
             5144, 6789, 5145, 7702, 4942, 6505, 6506, 6315, 6313, 4312, 4222,
              5851, 5852, 4971, 5454, 6370, 4320, 6816, 6815, 5898, 6381, 
             6379, 4619, 6413, 4621, 6431, 5903, 4999}],
            RowBox[{"-", "45"}]],
           Annotation[#, -45, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4998, 5902, 5000, 6430, 5469, 6424, 6425, 5897, 6837, 
             5895, 4236, 4319, 6369, 4970, 5869, 4973, 5861, 5859, 6328, 6329,
              5149, 6333, 7015, 7016, 6319, 7013, 7012, 7703, 5147, 6502, 
             7741, 6501, 4400, 5935, 5936, 4691, 5480, 4690, 5982, 4730, 5979,
              4688, 6466, 4689, 7340, 7341, 7337, 7339, 7338, 5556, 8366, 
             5227, 7791, 5228, 7792, 5143, 6500, 5278, 7111, 5279, 7883, 4749,
              7011, 7009, 5566, 4685, 5364, 6723, 8423, 7795, 5229, 7794, 
             7798, 7796, 4888, 7800, 6728, 6729, 4292, 6213, 5742, 6229, 5745,
              6232, 6231, 5417, 6226, 4890, 5738, 6731}],
            RowBox[{"-", "46.800000000000004`"}]],
           Annotation[#, -46.800000000000004`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4889, 6224, 4295, 6733, 5418, 4891, 5744, 6228, 6227, 
             6603, 4207, 6755, 6756, 8368, 8369, 8367, 5232, 7801, 4887, 7797,
              6222, 7114, 5570, 5230, 6550, 5363, 5971, 5972, 5565, 7010, 
             5564, 5774, 6010, 4397, 5504, 5142, 5976, 7790, 5975, 4399, 7018,
              4731, 7057, 7058, 7059, 5981, 4429, 5983, 8099, 8098, 7060, 
             4732, 5559, 5558, 8343, 5195, 7742, 5196, 7743, 5148, 7884, 7885,
              7014, 8067, 4398, 6503, 6461, 6330, 7017, 6332, 6508, 6507, 
             4944, 5860, 6335, 4606, 6374, 4972, 5868, 6371, 5867, 6391, 5905,
              4247, 6836, 5896, 4996, 5470, 6421, 4329, 6840, 6839}],
            RowBox[{"-", "48.6`"}]],
           Annotation[#, -48.6, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4248, 4328, 6420, 6423, 5914, 4997, 5907, 6838, 5904, 
             6392, 4981, 5871, 6395, 4608, 6373, 4605, 6334, 4945, 5866, 6793,
              7592, 7593, 6331, 5032, 7590, 6462, 5146, 6570, 6571, 5306, 
             6569, 6568, 6504, 4430, 5560, 5561, 6590, 6589, 5361, 5362, 8001,
              8000, 8166, 4687, 8167, 8168, 7284, 7283, 4686, 6499, 5503, 
             5505, 4720, 5775, 4579, 5567, 5778, 5776, 4483, 6039, 4484, 6551,
              5569, 7115, 4447, 6221, 4448, 7116, 7921, 7920, 5231, 7799, 
             6553, 5334, 6611, 5335, 6610, 5328, 6230, 5329, 6604, 4892, 5785,
              6735, 4196, 4294, 6223}],
            RowBox[{"-", "50.4`"}]],
           Annotation[#, -50.4, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4209, 5781, 6734, 5784, 6235, 6234, 7225, 7224, 7937, 
             7938, 5337, 7940, 5336, 7942, 6609, 6552, 7496, 7495, 7932, 5321,
              6269, 6268, 7802, 7803, 7118, 7117, 5568, 4751, 6040, 4482, 
             5777, 5563, 6892, 7494, 4580, 5779, 7040, 7041, 5977, 7315, 5978,
              4718, 7282, 7313, 7312, 7314, 4224, 5366, 6588, 6587, 8131, 
             8130, 7725, 5175, 7318, 5174, 7897, 5298, 7896, 5297, 7726, 5176,
              7591, 8239, 8323, 8240, 7589, 6896, 6794, 7594, 6792, 5865, 
             6336, 4607, 6386, 5456, 6394, 4982, 5870, 4980, 5906, 6397, 4623,
              6427, 6428, 5913, 6422, 5912, 6436}],
            RowBox[{"-", "52.2`"}]],
           Annotation[#, -52.2, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{6759, 5783, 6276, 4582, 6233, 4581, 7933, 7934, 7936, 
             7935, 5330, 8415, 7941, 5332, 7939, 5331, 6267, 7851, 6266, 7931,
              5327, 7497, 7499, 4915, 6598, 4914, 6554, 5233, 6271, 7121, 
             6042, 4752, 6043, 6044, 6038, 7372, 6037, 5562, 4301, 7744, 6893,
              4913, 5780, 4912, 6263, 6537, 6536, 4417, 8173, 8174, 7316, 
             4723, 7317, 4719, 6009, 7045, 5573, 4487, 7322, 4724, 7319, 7321,
              7320, 4418, 5939, 7043, 7042, 7724, 5173, 8321, 8322, 8319, 
             4314, 4225, 5862, 6818, 5864, 4978, 6387, 5455, 6393, 5458, 6821,
              5911, 6399, 6396, 4622, 6426, 4625, 6439, 5916, 5006}],
            RowBox[{"-", "54"}]],
           Annotation[#, -54, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{5005, 5915, 5007, 6438, 5472, 6434, 4624, 6398, 5910, 
             6822, 4238, 4322, 6383, 5457, 4977, 5863, 6817, 7707, 6820, 7706,
              7705, 7709, 5937, 8320, 5938, 6526, 6525, 4636, 5940, 7044, 
             7323, 4722, 6016, 6469, 6467, 6468, 6013, 4721, 6017, 5507, 7746,
              7747, 7063, 8101, 8102, 6264, 6265, 5198, 7745, 5197, 8344, 
             4226, 7374, 4755, 7373, 7376, 7375, 4753, 7378, 7119, 7120, 6041,
              4449, 6270, 6272, 4916, 6273, 5253, 7498, 5254, 7852, 5252, 
             7849, 8382, 7850, 7846, 7845, 5250, 7844, 7226, 7227, 6279, 5429,
              6278, 4919, 5782, 6758}],
            RowBox[{"-", "55.800000000000004`"}]],
           Annotation[#, -55.800000000000004`, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4918, 5430, 6277, 5428, 6280, 4920, 7847, 5251, 7806, 
             7843, 7842, 7848, 4302, 4208, 6757, 5986, 4917, 5988, 5987, 7123,
              7122, 7377, 4756, 7125, 7127, 7128, 6045, 4450, 7124, 5575, 
             4489, 6555, 4488, 7348, 7349, 7061, 8100, 4431, 5990, 7602, 5040,
              7062, 5041, 6012, 6011, 4331, 6014, 6015, 5482, 5481, 5996, 
             4637, 5942, 7021, 6512, 5159, 7710, 5152, 6819, 5153, 7708, 4979,
              6515, 6516, 6385, 6513, 6382, 4321, 4237, 5908, 5909, 5002, 
             6435, 5471, 6437, 5473, 6841}],
            RowBox[{"-", "57.6`"}]],
           Annotation[#, -57.6, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4249, 4330, 6432, 5001, 8504, 5004, 5919, 5917, 6400, 
             6514, 5154, 6384, 5155, 6390, 8070, 6389, 7020, 7019, 7711, 7712,
              6511, 7749, 6510, 4406, 5941, 7351, 5943, 4698, 5483, 4697, 
             5993, 4733, 5989, 4695, 6470, 4696, 7346, 7347, 7342, 7345, 7344,
              5574, 8370, 5234, 7805, 5235, 6518, 6517, 7886, 5280, 7126, 
             5281, 8396, 4754, 8069, 8178, 8177, 4694, 6274, 4692, 5985, 4227,
              7808, 7809, 7807, 7812, 7810, 4922, 7813, 6760, 6761, 4303, 
             6275}],
            RowBox[{"-", "59.4`"}]],
           Annotation[#, -59.4, "Tooltip"]& ]}, 
         {GrayLevel[0], Opacity[0.4], 
          TagBox[
           
           TooltipBox[
            LineBox[{4228, 8508, 8509, 6762, 8500, 7814, 4921, 7811, 8505, 
             8513, 8512, 5236, 8516, 5367, 8169, 4693, 8510, 8068, 4402, 8176,
              4401, 8311, 5157, 8514, 5158, 8397, 8398, 7804, 8501, 7887, 
             4405, 7748, 5199, 7343, 5200, 5992, 5991, 4432, 5994, 5995, 4735,
              7350, 4734, 7352, 5576, 8345, 8499, 7750, 8347, 8346, 5161, 
             8515, 5160, 8312, 4403, 6388, 4404, 8071, 8511, 5156, 8310, 8498,
              5918, 8497, 8506, 5003, 8503, 6433, 8502, 8507}],
            RowBox[{"-", "61.2`"}]],
           Annotation[#, -61.2, "Tooltip"]& ]}, {}, {}}}],
      AspectRatio->1,
      BaseStyle->{FontFamily -> "Times", FontSize -> 12},
      DisplayFunction:>Identity,
      Frame->True,
      PlotLabel->FormBox[
       "\"\[Mu]=14.8 \[Alpha]=0.90 \[Epsilon]=10 PIN=0.40\"", 
        TraditionalForm],
      PlotRange->{{0, 50}, {0, 50}},
      PlotRangeClipping->True,
      PlotRangePadding->{
        Scaled[0.02], 
        Scaled[0.02]}], {576., -189.}, 
     ImageScaled[{0.5, 0.5}], {360., 360.}]}, {}},
  ContentSelectable->True,
  PlotRangePadding->{6, 5}]], "Output",
 CellChangeTimes->{3.3896129018122*^9, 3.3896227427869997`*^9, 
  3.3899915689105997`*^9},
 CellLabel->"Out[264]="]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
ScreenStyleEnvironment->"Working",
PrintingStyleEnvironment->"Printout",
WindowSize->{1578, 1027},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
Visible->True,
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PrintingMargins"->{{54, 54}, {72, 72}}},
ShowSelection->True,
InputAliases->{"intt" -> RowBox[{"\[Integral]", 
     RowBox[{"\[SelectionPlaceholder]", 
       RowBox[{"\[DifferentialD]", "\[Placeholder]"}]}]}], "dintt" -> 
  RowBox[{
     SubsuperscriptBox[
     "\[Integral]", "\[SelectionPlaceholder]", "\[Placeholder]"], 
     RowBox[{"\[Placeholder]", 
       RowBox[{"\[DifferentialD]", "\[Placeholder]"}]}]}], "sumt" -> RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"\[SelectionPlaceholder]", "=", "\[Placeholder]"}], 
      "\[Placeholder]"], "\[Placeholder]"}], "prodt" -> RowBox[{
     UnderoverscriptBox["\[Product]", 
      RowBox[{"\[SelectionPlaceholder]", "=", "\[Placeholder]"}], 
      "\[Placeholder]"], "\[Placeholder]"}], "dt" -> RowBox[{
     SubscriptBox["\[PartialD]", "\[Placeholder]"], " ", 
     "\[SelectionPlaceholder]"}], "E" -> "\[ScriptCapitalE]", "notation" -> 
  RowBox[{"Notation", "[", 
     RowBox[{
       TagBox["\[SelectionPlaceholder]", "NotationTemplateTag"], " ", 
       "\[DoubleLongLeftRightArrow]", " ", 
       TagBox["\[Placeholder]", "NotationTemplateTag"]}], "]"}], "notation>" -> 
  RowBox[{"Notation", "[", 
     RowBox[{
       TagBox["\[SelectionPlaceholder]", "NotationTemplateTag"], " ", 
       "\[DoubleLongRightArrow]", " ", 
       TagBox["\[Placeholder]", "NotationTemplateTag"]}], "]"}], "notation<" -> 
  RowBox[{"Notation", "[", 
     RowBox[{
       TagBox["\[SelectionPlaceholder]", "NotationTemplateTag"], " ", 
       "\[DoubleLongLeftArrow]", " ", 
       TagBox["\[Placeholder]", "NotationTemplateTag"]}], "]"}], "symb" -> 
  RowBox[{"Symbolize", "[", 
     TagBox["\[SelectionPlaceholder]", "NotationTemplateTag"], "]"}], 
  "infixnotation" -> RowBox[{"InfixNotation", "[", 
     RowBox[{
       TagBox["\[SelectionPlaceholder]", "NotationTemplateTag"], ",", 
       "\[Placeholder]"}], "]"}], "addia" -> RowBox[{"AddInputAlias", "[", 
     RowBox[{"\"\[SelectionPlaceholder]\"", "\[Rule]", 
       TagBox["\[Placeholder]", "NotationTemplateTag"]}], "]"}], "pattwraper" -> 
  TagBox["\[SelectionPlaceholder]", "NotationPatternTag"], "madeboxeswraper" -> 
  TagBox["\[SelectionPlaceholder]", "NotationMadeBoxesTag"]},
Magnification->1.5,
FrontEndVersion->"6.0 for Microsoft Windows (32-bit) (April 20, 2007)",
StyleDefinitions->"EMMStyle02.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "SeqTrade1"->{
  Cell[52109, 1687, 107, 3, 65, "Subsection",
   CellTags->"SeqTrade1"]},
 "SeqTrade2"->{
  Cell[55157, 1807, 95, 1, 39, "Subsection",
   CellTags->"SeqTrade2"]},
 "SeqTrade3"->{
  Cell[89841, 2885, 90, 1, 39, "Subsection",
   CellTags->"SeqTrade3"]},
 "SeqTrade4"->{
  Cell[159970, 5035, 78, 1, 65, "Subsection",
   CellTags->"SeqTrade4"]},
 "c:132"->{
  Cell[205035, 6189, 115, 3, 51, "Section",
   CellTags->"c:132"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"SeqTrade1", 3040348, 53777},
 {"SeqTrade2", 3040438, 53780},
 {"SeqTrade3", 3040527, 53783},
 {"SeqTrade4", 3040616, 53786},
 {"c:132", 3040702, 53789}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[590, 23, 1089, 27, 430, "Title"],
Cell[CellGroupData[{
Cell[1704, 54, 410, 9, 43, "Input"],
Cell[2117, 65, 762, 16, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2916, 86, 32, 0, 92, "Section"],
Cell[2951, 88, 375, 10, 43, "Input"],
Cell[3329, 100, 265, 9, 33, "Text"],
Cell[3597, 111, 98, 3, 43, "Input"],
Cell[CellGroupData[{
Cell[3720, 118, 43, 0, 65, "Subsection"],
Cell[3766, 120, 147, 3, 43, "Input"],
Cell[3916, 125, 185, 4, 43, "Input"],
Cell[4104, 131, 366, 12, 43, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[4519, 149, 43, 0, 92, "Section"],
Cell[CellGroupData[{
Cell[4587, 153, 32, 0, 65, "Subsection"],
Cell[4622, 155, 137, 3, 32, "Text"],
Cell[CellGroupData[{
Cell[4784, 162, 981, 30, 71, "Input"],
Cell[5768, 194, 553, 17, 38, "Output"]
}, Open  ]],
Cell[6336, 214, 247, 6, 33, "Text"],
Cell[CellGroupData[{
Cell[6608, 224, 94, 2, 43, "Input"],
Cell[6705, 228, 13505, 385, 121, "Output"]
}, Open  ]],
Cell[20225, 616, 219, 6, 33, "Text"],
Cell[CellGroupData[{
Cell[20469, 626, 1106, 32, 98, "Input"],
Cell[21578, 660, 19088, 558, 513, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[40715, 1224, 47, 0, 65, "Subsection"],
Cell[40765, 1226, 92, 2, 32, "Text"],
Cell[CellGroupData[{
Cell[40882, 1232, 196, 6, 43, "Input"],
Cell[41081, 1240, 510, 16, 38, "Output"]
}, Open  ]],
Cell[41606, 1259, 33, 0, 32, "Text"],
Cell[CellGroupData[{
Cell[41664, 1263, 82, 2, 43, "Input"],
Cell[41749, 1267, 140, 3, 38, "Output"]
}, Open  ]],
Cell[41904, 1273, 80, 4, 33, "Text"],
Cell[CellGroupData[{
Cell[42009, 1281, 143, 4, 43, "Input"],
Cell[42155, 1287, 286, 8, 56, "Output"]
}, Open  ]],
Cell[42456, 1298, 135, 6, 33, "Text"],
Cell[CellGroupData[{
Cell[42616, 1308, 158, 5, 43, "Input"],
Cell[42777, 1315, 286, 9, 56, "Output"]
}, Open  ]],
Cell[43078, 1327, 219, 8, 33, "Text"],
Cell[CellGroupData[{
Cell[43322, 1339, 281, 9, 43, "Input"],
Cell[43606, 1350, 303, 8, 59, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[43946, 1363, 381, 12, 43, "Input"],
Cell[44330, 1377, 143, 3, 38, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[44510, 1385, 272, 9, 43, "Input"],
Cell[44785, 1396, 647, 22, 60, "Output"]
}, Open  ]],
Cell[45447, 1421, 724, 24, 51, "Text"],
Cell[46174, 1447, 112, 4, 33, "Text"],
Cell[CellGroupData[{
Cell[46311, 1455, 281, 9, 43, "Input"],
Cell[46595, 1466, 397, 13, 59, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[47029, 1484, 272, 9, 43, "Input"],
Cell[47304, 1495, 628, 21, 60, "Output"]
}, Open  ]],
Cell[47947, 1519, 725, 24, 51, "Text"],
Cell[CellGroupData[{
Cell[48697, 1547, 109, 3, 43, "Input"],
Cell[48809, 1552, 564, 19, 62, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[49410, 1576, 135, 4, 43, "Input"],
Cell[49548, 1582, 225, 6, 38, "Output"]
}, Open  ]],
Cell[49788, 1591, 195, 6, 33, "Text"],
Cell[CellGroupData[{
Cell[50008, 1601, 272, 9, 43, "Input"],
Cell[50283, 1612, 569, 20, 60, "Output"]
}, Open  ]],
Cell[50867, 1635, 127, 4, 33, "Text"],
Cell[CellGroupData[{
Cell[51019, 1643, 311, 9, 43, "Input"],
Cell[51333, 1654, 148, 3, 38, "Output"]
}, Open  ]],
Cell[51496, 1660, 100, 2, 32, "Text"],
Cell[CellGroupData[{
Cell[51621, 1666, 300, 10, 43, "Input"],
Cell[51924, 1678, 136, 3, 38, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[52109, 1687, 107, 3, 65, "Subsection",
 CellTags->"SeqTrade1"],
Cell[CellGroupData[{
Cell[52241, 1694, 54, 1, 43, "Input"],
Cell[52298, 1697, 647, 22, 60, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[52982, 1724, 174, 5, 43, "Input"],
Cell[53159, 1731, 312, 9, 56, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[53508, 1745, 507, 16, 43, "Input"],
Cell[54018, 1763, 370, 12, 56, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[54425, 1780, 541, 16, 43, "Input"],
Cell[54969, 1798, 139, 3, 38, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[55157, 1807, 95, 1, 39, "Subsection",
 CellTags->"SeqTrade2"],
Cell[CellGroupData[{
Cell[55277, 1812, 544, 17, 71, "Input"],
Cell[55824, 1831, 11373, 324, 103, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[67234, 2160, 1566, 46, 152, "Input"],
Cell[68803, 2208, 15478, 453, 447, "Output"]
}, Open  ]],
Cell[84296, 2664, 76, 0, 32, "Text"],
Cell[CellGroupData[{
Cell[84397, 2668, 225, 7, 43, "Input"],
Cell[84625, 2677, 144, 3, 38, "Output"]
}, Open  ]],
Cell[84784, 2683, 102, 4, 33, "Text"],
Cell[CellGroupData[{
Cell[84911, 2691, 273, 8, 43, "Input"],
Cell[85187, 2701, 337, 11, 59, "Output"]
}, Open  ]],
Cell[85539, 2715, 41, 0, 32, "Text"],
Cell[CellGroupData[{
Cell[85605, 2719, 273, 8, 43, "Input"],
Cell[85881, 2729, 301, 9, 59, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[86219, 2743, 272, 9, 43, "Input"],
Cell[86494, 2754, 490, 16, 60, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[87021, 2775, 272, 9, 43, "Input"],
Cell[87296, 2786, 380, 11, 60, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[87713, 2802, 163, 4, 43, "Input"],
Cell[87879, 2808, 426, 14, 61, "Output"]
}, Open  ]],
Cell[88320, 2825, 95, 2, 32, "Text"],
Cell[CellGroupData[{
Cell[88440, 2831, 473, 15, 43, "Input"],
Cell[88916, 2848, 148, 3, 38, "Output"]
}, Open  ]],
Cell[89079, 2854, 95, 2, 32, "Text"],
Cell[CellGroupData[{
Cell[89199, 2860, 450, 14, 43, "Input"],
Cell[89652, 2876, 140, 3, 38, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[89841, 2885, 90, 1, 39, "Subsection",
 CellTags->"SeqTrade3"],
Cell[CellGroupData[{
Cell[89956, 2890, 1068, 30, 287, "Input"],
Cell[91027, 2922, 22075, 627, 155, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[113139, 3554, 2362, 67, 179, "Input"],
Cell[115504, 3623, 30647, 882, 643, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[146188, 4510, 225, 7, 43, "Input"],
Cell[146416, 4519, 140, 3, 38, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[146593, 4527, 273, 8, 43, "Input"],
Cell[146869, 4537, 695, 24, 59, "Output"]
}, Open  ]],
Cell[147579, 4564, 273, 8, 43, "Input"],
Cell[CellGroupData[{
Cell[147877, 4576, 88, 2, 43, "Input"],
Cell[147968, 4580, 154, 3, 38, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[148159, 4588, 272, 9, 43, "Input"],
Cell[148434, 4599, 1067, 38, 60, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[149538, 4642, 273, 8, 43, "Input"],
Cell[149814, 4652, 663, 22, 59, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[150514, 4679, 273, 9, 43, "Input"],
Cell[150790, 4690, 1044, 37, 60, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[151871, 4732, 163, 4, 43, "Input"],
Cell[152037, 4738, 143, 3, 38, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[152229, 4747, 49, 0, 39, "Subsection"],
Cell[152281, 4749, 327, 16, 33, "Text"],
Cell[152611, 4767, 847, 26, 71, "Input"],
Cell[153461, 4795, 95, 2, 32, "Text"],
Cell[CellGroupData[{
Cell[153581, 4801, 205, 6, 43, "Input"],
Cell[153789, 4809, 459, 15, 59, "Output"]
}, Open  ]],
Cell[154263, 4827, 44, 0, 32, "Text"],
Cell[CellGroupData[{
Cell[154332, 4831, 209, 6, 43, "Input"],
Cell[154544, 4839, 489, 15, 38, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[155070, 4859, 144, 4, 43, "Input"],
Cell[155217, 4865, 547, 17, 59, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[155801, 4887, 504, 15, 43, "Input"],
Cell[156308, 4904, 147, 3, 38, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[156492, 4912, 504, 15, 43, "Input"],
Cell[156999, 4929, 143, 3, 38, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[157179, 4937, 229, 6, 43, "Input"],
Cell[157411, 4945, 1094, 33, 38, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[158542, 4983, 144, 4, 43, "Input"],
Cell[158689, 4989, 1232, 40, 66, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[159970, 5035, 78, 1, 65, "Subsection",
 CellTags->"SeqTrade4"],
Cell[160051, 5038, 111, 3, 32, "Text"],
Cell[CellGroupData[{
Cell[160187, 5045, 311, 9, 43, "Input"],
Cell[160501, 5056, 503, 15, 59, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[161053, 5077, 52, 0, 65, "Subsection"],
Cell[161108, 5079, 53, 0, 32, "Text"],
Cell[CellGroupData[{
Cell[161186, 5083, 321, 9, 43, "Input"],
Cell[161510, 5094, 1104, 35, 66, "Output"]
}, Open  ]],
Cell[162629, 5132, 159, 7, 33, "Text"],
Cell[CellGroupData[{
Cell[162813, 5143, 391, 12, 43, "Input"],
Cell[163207, 5157, 372, 11, 59, "Output"]
}, Open  ]],
Cell[163594, 5171, 310, 9, 43, "Input"],
Cell[CellGroupData[{
Cell[163929, 5184, 196, 6, 43, "Input"],
Cell[164128, 5192, 143, 3, 38, "Output"]
}, Open  ]],
Cell[164286, 5198, 153, 3, 32, "Text"],
Cell[CellGroupData[{
Cell[164464, 5205, 274, 9, 43, "Input"],
Cell[164741, 5216, 638, 20, 60, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[165416, 5241, 270, 9, 43, "Input"],
Cell[165689, 5252, 327, 10, 38, "Output"]
}, Open  ]],
Cell[166031, 5265, 65, 0, 32, "Text"],
Cell[CellGroupData[{
Cell[166121, 5269, 112, 3, 43, "Input"],
Cell[166236, 5274, 533, 16, 60, "Output"]
}, Open  ]],
Cell[166784, 5293, 49, 0, 32, "Text"],
Cell[CellGroupData[{
Cell[166858, 5297, 184, 6, 43, "Input"],
Cell[167045, 5305, 139, 3, 38, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[167221, 5313, 254, 8, 43, "Input"],
Cell[167478, 5323, 263, 8, 56, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[167802, 5338, 65, 0, 51, "Section"],
Cell[CellGroupData[{
Cell[167892, 5342, 181, 5, 43, "Input"],
Cell[168076, 5349, 364, 11, 40, "Output"]
}, Open  ]],
Cell[168455, 5363, 146, 4, 43, "Input"],
Cell[168604, 5369, 921, 30, 68, "Input"],
Cell[CellGroupData[{
Cell[169550, 5403, 53, 1, 43, "Input"],
Cell[169606, 5406, 1241, 42, 58, "Output"]
}, Open  ]],
Cell[170862, 5451, 889, 26, 97, "Input"],
Cell[171754, 5479, 599, 16, 43, "Input"],
Cell[CellGroupData[{
Cell[172378, 5499, 1479, 31, 71, "Input"],
Cell[173860, 5532, 6069, 105, 307, "Output"]
}, Open  ]],
Cell[179944, 5640, 1051, 28, 43, "Input"],
Cell[180998, 5670, 503, 14, 43, "Input"],
Cell[181504, 5686, 279, 8, 43, "Input"],
Cell[CellGroupData[{
Cell[181808, 5698, 759, 17, 43, "Input"],
Cell[182570, 5717, 8391, 153, 297, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[190998, 5875, 158, 5, 43, "Input"],
Cell[191159, 5882, 266, 8, 58, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[191462, 5895, 1007, 29, 71, "Input"],
Cell[192472, 5926, 972, 28, 38, "Output"]
}, Open  ]],
Cell[193459, 5957, 718, 22, 43, "Input"],
Cell[194180, 5981, 521, 16, 56, "Input"],
Cell[CellGroupData[{
Cell[194726, 6001, 301, 8, 43, "Input"],
Cell[195030, 6011, 9956, 172, 354, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[205035, 6189, 115, 3, 51, "Section",
 CellTags->"c:132"],
Cell[CellGroupData[{
Cell[205175, 6196, 218, 6, 68, "Input"],
Cell[205396, 6204, 868, 26, 56, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[206301, 6235, 586, 17, 68, "Input"],
Cell[206890, 6254, 2068, 66, 86, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[208995, 6325, 256, 7, 43, "Input"],
Cell[209254, 6334, 314, 8, 38, "Output"]
}, Open  ]],
Cell[209583, 6345, 1036, 27, 71, "Input"],
Cell[210622, 6374, 1339, 34, 98, "Input"],
Cell[CellGroupData[{
Cell[211986, 6412, 231, 6, 43, "Input"],
Cell[212220, 6420, 521497, 8579, 300, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[733754, 15004, 46, 0, 44, "Subsubsection"],
Cell[733803, 15006, 661, 14, 78, "Text"],
Cell[734467, 15022, 1940, 50, 160, "Input"],
Cell[CellGroupData[{
Cell[736432, 15076, 593, 18, 43, "Input"],
Cell[737028, 15096, 868318, 14564, 558, "Output"]
}, Open  ]],
Cell[1605361, 29663, 546, 13, 78, "Text"],
Cell[1605910, 29678, 614, 17, 67, "Input"],
Cell[CellGroupData[{
Cell[1606549, 29699, 617, 20, 101, "Input"],
Cell[1607169, 29721, 908968, 15196, 558, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2516174, 44922, 557, 15, 67, "Input"],
Cell[2516734, 44939, 213, 6, 38, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2516984, 44950, 558, 15, 67, "Input"],
Cell[2517545, 44967, 221, 6, 38, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2517803, 44978, 552, 17, 101, "Input"],
Cell[2518358, 44997, 519254, 8715, 300, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
